# umi-plugin-udesk-sdk

支持多 sdk 架构

## 更新日志

[CHANGELOG](CHANGELOG.MD)

## Install

```bash
$ yarn install
```

```bash
$ yarn build --watch
$ yarn start
```

## Usage

Configure in `.umirc.js`,

```js
export default {
    plugins: [['umi-plugin-udesk-sdk']],
};
```

## 特性

1. 支持多 sdk，真正做到按需编译，源头上提高编译和启动
1. 每个 sdk 可以按需配置所需的路由
1. 支持启动/编译单个 sdk
1. 系统默认使用 sdk 模式，app 也是运行在 sdk 模式下，可控制 sdk 的 sdkOptions 默认值，也可以控制运行时主路由的全局 sdkOptions
1. 默认会自动创建一个`main` sdk，默认包含所有路由，可以创建同名 sdk 配置，即可自己控制
1. 支持所有 umi 特性
1. 可以在 config 中使用`udesk-sdk`来进行配置（todo：更多详细配置项），自持对每个 sdk 进行独立的配置
1. 支持对单个 sdk 进行运行时配置
1. 默认支持 global-config，可以通过 Context 访问
1. 新增支持 currentScript 模式，可以更优雅的传递 class 类
1. 升级 sdk-loader 组件
1. 支持传入 routerBase 定制动态根路由，结合新版 sdk-loader 自动获取父路由地址，简化集成
1. 集成 minifest 模式，更好的加载效率（vendor 拆分，dynamic import，多 css，自动移除 css？）
1. 增加 sdk/types，自定义 SdkOptions 和 GlobalData 的类型
1. 增加 useSdkOptions/useGlobalData 两个 hooks
1. 可以支持自定义 sdk 入口文件，与 sdk 配置文件创建同名 ts 文件即可，在入口文件中可以导入 sdk 的基类

## 限制

1. 不能多实例
1. publicPath 只能在编译时决定，可以借助.env，实现多环境支持，但不支持客户端自定义。虽然 umi 中有 runtimePublicPath 支持微前端，但借助了 window 全局变量，多 sdk 会串。后面有时间在研究有没有更好的办法。
1. 有些页面 request 全局拦截器需要获取当前 routerBase，这个是由 sdkOptions 传入的，只能把每个 sdk 实例的 sdkOptions 上升为全局对象，才能在 app.ts 中获取到，但这样就不能支持多实例了。
1. 使用 history 跳转路由也是一个例子，不同的示例的 routerBase 不相同，第二个示例修改了 history 的 basename 后，第一个示例就不能正确跳转了

## todo

1. 单个 sdk 支持堵路的 umi config
1. 支持不同的 sdk 指定独立的编译目录，或支持输出多 html
1. 如何控制 layout 路由强制隐藏
1. 控制 routeBase、publicPath，以及 runtimePublicPath、exportStatic?.dynamicRoot
1. runtimePublicPath、exportStatic?.dynamicRoot 使用全局变量来控制，如何保证不串？
1. 可以修改 gitignore，在 sdk 目录下保留一些不受 git 管理的配置文件（规则？），这样每个人就可以定制一些精简版的 sdk，加速编译时间
1. 形成项目模板

## 注意

1. 不要在模板文件中从 umi 中导入东西，而是要从具体的模块中，容易导致循环引用
1. 导入 umi 类型的话，从 `umi/lib/cjs` 导入
1. 导入 umi 运行时的内容的话，从 `@umijs/runtime` 导入
1. 如果是插件开发，在 node 端的话，从 `umi`导入就可以了
1. 不要在模板中使用 @或@@ 导入东西，而是要用 placeholder 替换成相对或绝对地址

## SDK 配置

新建 `src/sdk` 目录，新建一个 `.json`文件，
格式：

```json
{
    "routes": ["**/*"]
}
```

最小配置是必须包含 `routes` 路由表，路由规则支持配置 `Route` 对象，也支持配置字符串，来从当前系统路由中进行查找。字符串支持完整路由，也支持 [glob](https://github.com/micromatch/micromatch#matching-features) 模糊路由，底层匹配库使用的是[micromatch](https://github.com/micromatch/micromatch)

## 启动一个 sdk

如果要启动 `user-admin` sdk，

```bash
yarn start user-admin
```

编译

```bash
yarn build user-admin
```

如果省略最后一个参数，则表示启动默认 sdk，即名称为 `main` 的 sdk，默认会包含所有路由。如果希望定制默认 sdk，创建同名配置文件即可

```bash
yarn start
# 等效于
yarn start main
```

```bash
yarn build
# 等效于
yarn build main
```

## Break Change

1. 去掉静态方法 destroy、reload 方法
1. 静态属性 enableSentry, enableVersionMonitoring 改成 options
1. 静态方法 currentVersion 改名 version
1. 去掉静态方法 sdkName
1. 去掉 SessionStorage 检测版本变化的机制，全改成实例级的 file monitor
1. renderTo 方法第一个 udeskEvents 参数去掉，改成 this.eventManager
1. sdk 的 onShowNewVersionTip 事件名改成 versionChanged 事件
1. options.versionMonitorIntervalSeconds 改成 options.versionMonitorInterval
1. 静态方法 getUserForSentry 改为 options.getSentryUser
1. 静态方法 getTagsContextForSentry 改为 options.getSentryTagsContext
1. 静态方法 getExtraContextForSentry 改为 options.getSentryExtraContext
1. 静态方法 getShouldSendForSentry 改为 options.getSentryShouldSend
1. 去掉 options.enableForceUpdate
1. getVersionPath 方法改成 options.versionFileUrl
1. 去掉 initGlobalNameSpace，改成 currentScript 模式
1. transitionTo 参数与 umi history 相同，与之前的规则略有不同
1. sdk 不再继承 EventedClass
1. 静态方法 changeOptions 改名 update，且 sdk 可空
1. 环境变量的 REACT_APP 前缀去掉了
1.

## LICENSE

MIT
