import React                from 'react';
import { configure, mount } from 'enzyme';
import UdeskPhoneNumber     from '@udesk-webapp/phone-number';
import Adapter              from 'enzyme-adapter-react-16';

configure({adapter : new Adapter()});

let mockCustomer = {owner_id : 5, owner_group_id : 9};

test('UdeskPhoneNumber test', () => {
    let wrapper = mount(
        <UdeskPhoneNumber number="18500643337" customer={mockCustomer} currentUser={{user_groups : [{id : 1}]}}/>
    );

    expect(wrapper.text()).toBe('185****3337');
});

test('UdeskPhoneNumber Test:User have role customer_show_cellphone_all', () => {
    let wrapper = mount(
        <UdeskPhoneNumber number="18500643337" customer={mockCustomer} currentUser={{user_groups : [{id : 1}]}}
                          roleCan={{customer_show_cellphone_all : true}}/>
    );

    expect(wrapper.text()).toBe('18500643337');
});

test('UserPhoneNumber Test:User have role customer_show_cellphone_group and owner_group match', () => {
    let wrapper = mount(
        <UdeskPhoneNumber number="18500643337" customer={mockCustomer}
                          currentUser={{user_groups : [{id : 1}, {id : 9}]}}
                          roleCan={{customer_show_cellphone_group : true}}/>
    );

    expect(wrapper.text()).toBe('18500643337');
});

test('UserPhoneNumber Test:User have role customer_show_cellphone_group and owner_group not match', () => {
    let wrapper = mount(
        <UdeskPhoneNumber number="18500643337" customer={mockCustomer} currentUser={{user_groups : [{id : 1}]}}
                          roleCan={{customer_show_cellphone_group : true}}/>
    );

    expect(wrapper.text()).toBe('185****3337');
});

test('UserPhoneNumber Test:User have role customer_show_cellphone_group and customer no have owner_group_id', () => {
    let mockCustomer = {owner_id : 5};

    let wrapper = mount(
        <UdeskPhoneNumber number="18500643337" customer={mockCustomer}
                          currentUser={{user_groups : [{id : 9}, {id : 1}]}}
                          roleCan={{customer_show_cellphone_group : true}}/>
    );

    expect(wrapper.text()).toBe('185****3337');
});

test('UserPhoneNumber Test:User have role customer_show_cellphone_personal and owner_id match', () => {
    let wrapper = mount(
        <UdeskPhoneNumber number="18500643337" customer={mockCustomer} currentUser={{user_groups : [{id : 9}], id : 5}}
                          roleCan={{customer_show_cellphone_personal : true}}/>
    );

    expect(wrapper.text()).toBe('18500643337');
});

test('UserPhoneNumber Test:User have role customer_show_cellphone_personal and owner_id not match', () => {
    let mockCustomer = {owner_id : 5};

    let wrapper = mount(
        <UdeskPhoneNumber number="18500643337" customer={mockCustomer} currentUser={{user_groups : [{id : 9}], id : 3}}
                          roleCan={{customer_show_cellphone_personal : true}}/>
    );

    expect(wrapper.text()).toBe('185****3337');
});

test('UserPhoneNumber Test:User have role customer_show_cellphone_personal and customer no have owner_id', () => {
    let mockCustomer = {};
    let wrapper = mount(
        <UdeskPhoneNumber number="18500643337" customer={mockCustomer} currentUser={{user_groups : [{id : 9}], id : 3}}
                          roleCan={{customer_show_cellphone_personal : true}}/>
    );

    expect(wrapper.text()).toBe('185****3337');
});

test('UserPhoneNumber Test:Enable click', () => {
    expect.assertions(2);

    let wrapper = mount(
        <UdeskPhoneNumber number="18500643337" customer={mockCustomer} currentUser={{user_groups : [{id : 9}], id : 3}}
                          roleCan={{customer_show_cellphone_all : true}} phoneIcon={true}
                          onPhoneClick={(number, realNumber) => expect(number).toBe('18500643337')}/>
    );

    expect(wrapper.find('a').length).toBe(1);
    wrapper.find('a').simulate('click');
});

test('UserPhoneNumber Test: isMask test', () => {
    let wrapper = mount(
        <UdeskPhoneNumber number="18500643337" customer={mockCustomer} currentUser={{user_groups : [{id : 9}], id : 3}}
                          isMask={true}/>
    );
    expect(wrapper.text()).toBe('185****3337');

    wrapper.setProps({
        isMask : false
    });
    expect(wrapper.text()).toBe('18500643337');

    wrapper.setProps({
        isMask : function() {
            return true;
        }
    });
    expect(wrapper.text()).toBe('185****3337');

    wrapper.setProps({
        isMask : function() {
            throw new Error('test error');
        }
    });
    expect(wrapper.text()).toBe('185****3337');
});

test('UserPhoneNumber Test: phoneIcon and smsIcon', () => {
    let wrapper = mount(
        <UdeskPhoneNumber number="18500643337" customer={mockCustomer} currentUser={{user_groups : [{id : 9}], id : 3}}
                          smsIcon={true} phoneIcon={true}/>
    );

    expect(wrapper.find('i').length).toBe(2);
});
