import { configure, mount } from 'enzyme';
import ColumnControl        from '../src/control';
import React                from 'react';
import Adapter              from 'enzyme-adapter-react-16';

configure({adapter : new Adapter()});

const CancelButton = ({onClick, t}) => {
    return <button className='btn btn-default' onClick={onClick}>{t('label.cancel')}</button>;
};

const ResetButton = ({onClick, t}) => {
    return <button className='btn btn-default' onClick={onClick}>{t('label.restoreDefault')}</button>;
};

const ConfirmButton = ({onClick, t}) => {
    return <button className='btn btn-primary' onClick={onClick}>{t('label.confirm')}</button>;
};

test('列控制器测试', () => {
    const component = mount(
        <ColumnControl className="test" style={{color : '#fff'}}
                       onChange={(value) => component.setProps({value})}
                       t={id => id}
                       columns={[
                           {id : 'full_name', category : 'label.person'},
                           {id : 'first_name', category : 'label.person'},
                           {id : 'last_name', category : 'label.person'},
                           {id : 'nick_name', category : 'label.person'},
                           {id : 'age', category : 'label.person'},
                           {id : 'email.name', category : 'label.email'},
                           {id : 'email.content', category : 'label.email'}
                       ]}
                       value={[]}
                       leftTitle='左侧标题'
                       rightTitle='右侧标题'
                       onReset={() => component.setProps({value : ['full_name', 'first_name', 'email.name']})}
                       ConfirmButton={ConfirmButton}
                       CancelButton={CancelButton}
                       ResetButton={ResetButton}
        />
    );

    expect(component).toMatchSnapshot();

    //测试切换category
    component.find('ul.category-tab').find('li').at(1).simulate('click');
    expect(component.find('select').at(0).text()).toBe('email.nameemail.content');
    component.setState({activeCategory : 'label.person'});

    //测试搜索功能
    component.find('input').simulate('change', {target : {value : 'name'}});
    expect(component.find('select').at(0).text()).toBe('full_name' +
                                                       'first_name' +
                                                       'last_name' +
                                                       'nick_name');

    //测试搜索后从左往右移动功能
    component.setState({leftSelectedColumnIds : ['full_name', 'last_name']});
    component.find('button').at(1).simulate('click');
    expect(component.find('select').at(0).children().length).toBe(2);
    expect(component.find('select').at(1).children().length).toBe(2);

    component.setState({keyword : ''});

    //测试从左往右移动功能
    component.find('button').at(1).simulate('click');
    expect(component.find('select').at(0).children().length).toBe(3);
    component.setState({leftSelectedColumnIds : ['nick_name', 'age']});
    component.find('button').at(1).simulate('click');
    expect(component.find('select').at(0).children().length).toBe(1);
    expect(component.find('select').at(1).children().length).toBe(4);

    //测试点击确定按钮
    expect(component.prop('value')).toEqual([]);
    component.find('.bottom-bar').find('button').at(0).simulate('click');
    expect(component.prop('value')).toEqual(['full_name', 'last_name', 'nick_name', 'age']);
    expect(component.state('value')).toEqual(['full_name', 'last_name', 'nick_name', 'age']);

    //测试从右往左移动功能
    component.find('button').at(0).simulate('click');

    expect(component.find('select').at(1).children().length).toBe(4);
    component.setState({rightSelectedColumnIds : ['full_name']});
    component.find('button').at(0).simulate('click');

    expect(component.find('select').at(1).children().length).toBe(3);
    expect(component.find('select').at(0).children().length).toBe(2);
    expect(component.state('rightSelectedColumnIds')).toEqual([]);

    //测试第一个元素上移无效
    component.setState({rightSelectedColumnIds : ['last_name']});
    component.find('button').at(2).simulate('click');
    expect(component.find('select').at(1).text()).toBe('last_namenick_nameage');

    //测试第二个元素上移到第一位
    component.setState({rightSelectedColumnIds : ['nick_name']});
    component.find('button').at(2).simulate('click');
    expect(component.find('select').at(1).text()).toBe('nick_namelast_nameage');

    //测试最后一个元素下移无效
    component.setState({rightSelectedColumnIds : ['age']});
    component.find('button').at(3).simulate('click');
    expect(component.find('select').at(1).text()).toBe('nick_namelast_nameage');

    //测试第一个元素下移到第二位
    component.setState({rightSelectedColumnIds : ['nick_name']});
    component.find('button').at(3).simulate('click');
    expect(component.find('select').at(1).text()).toBe('last_namenick_nameage');

    //当右侧没有内容时，执行上移下移操作
    component.setProps({value : []});
    expect(component.find('select').at(1).text()).toBe('');
    component.find('button').at(2).simulate('click');
    component.find('button').at(3).simulate('click');
    expect(component.find('select').at(1).text()).toBe('');

    //测试传入新的value
    component.setProps({value : ['email.name', 'email.content']});
    expect(component.find('select').at(1).text()).toBe('email.nameemail.content');

    //测试恢复默认按钮
    component.find('.bottom-bar').find('button').at(2).simulate('click');
    expect(component.find('select').at(1).text()).toBe('full_namefirst_nameemail.name');

    //测试 allowReset
    expect(component.find('.bottom-bar').find('button').length).toBe(3);
    component.setProps({allowReset : false});
    expect(component.find('.bottom-bar').find('button').length).toBe(2);

    //测试 allowSort
    expect(component.find('.sort-btn-wrapper').length).toBe(1);
    component.setProps({allowSort : false});
    expect(component.find('.sort-btn-wrapper').length).toBe(0);

    // 未开启 allowAutoSelectNext 不会自动选中下一个
    component.setState({ value: [], leftSelectedColumnIds: ['full_name'], rightSelectedColumnIds: [] });
    component.find('button').at(1).simulate('click');
    expect(component.state('value')).toEqual(['full_name']);
    expect(component.state('leftSelectedColumnIds')).toEqual([]);


    // 开启 allowAutoSelectNext 则会自动选中下一个
    component.setState({ value: [], leftSelectedColumnIds: ['full_name'], rightSelectedColumnIds: [] });
    component.setProps({ allowAutoSelectNext: true })
    component.find('button').at(1).simulate('click');
    expect(component.state('value')).toEqual(['full_name']);
    expect(component.state('leftSelectedColumnIds')).toEqual(['first_name']);
    component.find('button').at(1).simulate('click');
    component.find('button').at(1).simulate('click');
    expect(component.state('value')).toEqual(['full_name', 'first_name', 'last_name']);
    expect(component.state('leftSelectedColumnIds')).toEqual(['nick_name']);
    expect(component.state('rightSelectedColumnIds')).toEqual([]);
    component.setState({ rightSelectedColumnIds: ['first_name'] });
    component.find('button').at(0).simulate('click');
    component.find('button').at(0).simulate('click');
    expect(component.state('value')).toEqual(['full_name']);
    expect(component.state('leftSelectedColumnIds')).toEqual(['nick_name']);
    expect(component.state('rightSelectedColumnIds')).toEqual([]);
    component.find('button').at(1).simulate('click');
    expect(component.state('value')).toEqual(['full_name', 'nick_name']);
    expect(component.state('leftSelectedColumnIds')).toEqual(['age']);
    expect(component.state('rightSelectedColumnIds')).toEqual([]);

    // 未开启 allowSetoDefault 恢复默认按钮不显示
    component.setState({ value: [], leftSelectedColumnIds: [], rightSelectedColumnIds: [] });
    component.setProps({ allowAutoSelectNext: false })
    expect(component.find('a.set-to-default').length).toBe(0);

    // 开启 allowSetoDefault
    component.setProps({ allowAutoSelectNext: false, allowSetoDefault: true, setoDefaultValue: ['nick_name', 'email.content'] })
    expect(component.find('a.set-to-default').length).toBe(1);
    component.find('a.set-to-default').at(0).simulate('click')
    expect(component.state('value')).toEqual(['nick_name', 'email.content']);
    component.setProps({ allowSetoDefault: false })

    // 恢复重制对左右选中值的处理
    component.setProps({
        allowSetoDefault: true,
        setoDefaultValue: ['nick_name', 'email.content']
    })
    component.setState({
        value: ['email.content', 'age', 'email.name'],
        leftSelectedColumnIds: ['full_name', 'nick_name'],
        rightSelectedColumnIds: ['email.content', 'age']
    });
    component.find('a.set-to-default').at(0).simulate('click')
    expect(component.state('value')).toEqual(['nick_name', 'email.content']);
    expect(component.state('leftSelectedColumnIds')).toEqual(['full_name']);
    expect(component.state('rightSelectedColumnIds')).toEqual(['email.content']);
    component.setProps({ allowSetoDefault: false })
    expect(component.find('a.set-to-default').length).toBe(0);

});
