import { configure, mount } from 'enzyme';
import ColumnControl        from '../src';
import React                from 'react';
import Adapter              from 'enzyme-adapter-react-16';

configure({adapter : new Adapter()});

test('列控制器测试', () => {
    expect.assertions(5);
    const component = mount(
        <ColumnControl className="test" style={{color : '#fff'}}
                       onChange={(column) => expect(column.id).toEqual('name')}
                       t={id => id}
                       columns={[
                           {id : 'name', display : true, category : 'label.person'},
                           {id : 'age', display : true, category : 'label.person'},
                           {id : 'email.name', display : false, category : 'label.email'},
                           {id : 'email.content', display : false, category : 'label.email'}
                       ]}
        />
    );

    expect(component.find('h5').at(0).text()).toEqual('label.person');
    expect(component.find('h5').at(1).text()).toEqual('label.email');
    expect(component.find('label')).toHaveLength(4);
    expect(component.find('label').at(0).text()).toEqual('name');
    component.find('input').at(0).simulate('change');
});

test('不带category列控制测试', () => {
    const component = mount(
        <ColumnControl className="test" style={{color : '#fff'}}
                       t={id => 'column.' + id}
                       columns={[
                           {id : 'name', display : true},
                           {id : 'age', display : true},
                           {id : 'email.name', display : false},
                           {id : 'email.content', display : false}
                       ]}
        />
    );

    expect(component.find('h5')).toHaveLength(0);
});

test('搜索框测试', () => {
    const component = mount(
        <ColumnControl columns={[
            {id : 'name', display : true, name : '名称'},
            {id : 'age', display : true},
            {id : 'email.name', display : true},
            {id : 'email.content', display : false}
        ]}
                       t={id => 'column.' + id}
                       className="test"
                       showSearch={true}
        />
    );

    let searchInput = component.children().at(0).children().at(0).children().at(0);

    expect(searchInput.type()).toBe('input');

    expect(component.children().at(0).children().at(1).children().at(0).children().length).toBe(4);

    searchInput.simulate('change', {target : {value : 'name'}});
    expect(component.children().at(0).children().at(1).children().at(0).children().length).toBe(1);

    searchInput.simulate('change', {target : {value : ''}});
    expect(component.children().at(0).children().at(1).children().at(0).children().length).toBe(4);

    searchInput.simulate('change', {target : {value : 'abcdefg'}});
    expect(component.children().at(0).children().at(1).children().at(0).children().length).toBe(0);

});

test('恢复默认测试', () => {
    const component = mount(
        <ColumnControl columns={[{id : 'name', display : true, name : '名称'},
                                 {id : 'age', display : true},
                                 {id : 'email.name', display : true},
                                 {id : 'email.content', display : false}]}
                       t={id => 'column.' + id}
                       showSearch={true}
                       showRecoverDefault={true}
        />
    );

    expect(component.children().at(0).children().length).toBe(3);
    expect(component.children().at(0).children().at(1).find('a').text()).toBe('column.label.recover_default');
});
