const log = require("npmlog");
const fs = require("fs");
const path = require("path");
const ICON_FILE_PATH = require('../../utils/icon/index')['ICON_FILE_PATH'];

function createBuilder(args) {
    const opts = {
        [ICON_FILE_PATH]: {
            alias: "ifp",
            describe: "存放iconfont的文件路径",
            type:'string'
        },
        // is: {
        //     describe: "icon settings, use as default config",
        //     alias: "icon-set",
        //     type: "string",
        // },
    };
    args.options(opts);
}
function createHandler(args) {
    const configFilePath = path.resolve(__dirname,"../../config/.icon.config.json");
    const configFileTemplatePath = path.resolve(__dirname,"../../config/.icon.config.json.template");
    // log.info("udesk-helper icon 收到参数：", args);
    const packages = args.packages || {};
    let { [ICON_FILE_PATH]: iconFilePath } = packages;
    if (!iconFilePath) {
        log.error("icon命令错误:icon file path不存在！");
        process.exit(2);
    }
    const existConfigFile = fs.existsSync(configFilePath);
    if (!existConfigFile) {
        const existConfigFileTemplate = fs.existsSync(configFileTemplatePath);
        if (!existConfigFileTemplate) {
            log.error("icon 命令错误：配置文件和配置文件模板均不存在，请手动重建或联系开发者！");
        } else {
            fs.copyFileSync(configFileTemplatePath, configFilePath);
        }
    }
    // 替换配置
    let config = fs.readFileSync(configFilePath, "utf-8");
    if (!config) {
        log.error("icon 命令错误：配置文件内容为空，请添加配置内容！");
    } else {
        let parseConfig;
        try {
            parseConfig = JSON.parse(config);
        } catch (e) {
            log.error("icon命令错误：模板解析失败", e);
            exit(1);
        }
        parseConfig[ICON_FILE_PATH] = iconFilePath;
        fs.writeFileSync(configFilePath, JSON.stringify(parseConfig), "utf-8");
        log.info(`icon命令提示：修改配置成功，icon文件路径为${iconFilePath}`);
    }
}

const iconConfig = {
    cmd: "set-icon-config [packages...]",
    desc: "icon settings, use as default config",
    builder: createBuilder,
    handler: createHandler,
};

module.exports = iconConfig;
