const log = require('npmlog');
const fs = require('fs');
const ChildProcess = require('child_process');
const splitSymbol = '/';
const path = require('path');
const ejs = require('ejs');
const { getCamelCase2 } = require('../utils');

function createBuilder(args) {
    const opts = {
        lc: {
            describe: "add a new package from template",
            alias: "lerna-create",
            type: "boolean",
        },
        // preid is copied from ../version/command because a whitelist for one option isn't worth it
        // u: {
        //     describe: "update a old version to package.json",
        //     type: "boolean",
        //     alias: "update",
        //     // requiresArg: true,
        //     // defaultDescription: "alpha",
        // },
        // level: {
        //     describe: "when add a new version with beta,option value is 'major', 'minor', 'patch'",
        //     type: "string",
        //     // requiresArg: true,
        //     defaultDescription: "patch",
        // },
        // "beta": {
        //     describe: "when add a new version with beta,set the beta value",
        //     type: "string",
        //     // requiresArg: false,
        // },
        // "max-num": {
        //     describe: "Maximum number of files processed",
        //     type: "number",
        //     // requiresArg: false,
        // }
    }
    args.options(opts);
}

function copyDir(root, targetRoot, package, packageName) {
    fs.readdir(root, (error, files) => {

        if (error == null && files) {

            log.info('udesk-helper', `copyDir end:${files}`, typeof files);

            files.forEach(file => {

                let filePath = path.resolve(root, file);
                const targetFile = path.resolve(targetRoot, file);

                if (/\.ejs$/.test(file)) {

                    const targetFilePath = path.resolve(targetRoot, file.replace(/\.ejs$/, ''));

                    log.info('udesk-helper', `render ${filePath} to ${targetFilePath}`);

                    ejs.renderFile(filePath, {
                        package: package,
                        packageName: packageName,
                        desc: "", // 待增加
                        camelCasePackageName: getCamelCase2(packageName)
                    }, (err, str) => {
                        if (str) {
                            fs.writeFileSync(targetFilePath, str);
                        } else {
                            log.error('udesk-helper', err)
                        }
                    });
                } else {
                    let fileState = fs.lstatSync(filePath);

                    log.info('udesk-helper', `copy ${filePath} to ${targetFile}`);

                    // 如果是文件,直接拷贝
                    if (fileState && fileState.isFile()) {

                        fs.copyFile(filePath, targetFile, (err) => {
                            if (err == null) {
                                log.verbose('udesk-helper', `copy ${filePath} to ${targetFile} success!`);
                            }
                        });

                        // 如果是目录,准备递归
                    } else if (fileState && fileState.isDirectory()) {
                        // 判断目标目录是否存在
                        // const targetFilePath = path.resolve(targetFile, file);
                        if (!fs.existsSync(targetFile)) {
                            log.info('udesk-helper', `create ${targetFile}`)
                            fs.mkdirSync(targetFile);
                        }
                        copyDir(path.resolve(root, file), targetFile, package, packageName);
                    }
                }
            });
        } else {
            log.info('udesk-helper', 'copy template error:copyDir error')
        }
    })
}

function copyTempate(package) {
    if (!package) {
        return;
    }
    // windows 是否需要考虑斜杠和反斜杠区别
    console.log("package copyTempate", package)
    let packageArr = package.split('/');
    let currentRoot = process.cwd();
    // 这里写死,其实lerna可配,应该从packages.json文件取
    const lernaPakgeRootName = 'packages';
    let packageName = packageArr[packageArr.length - 1];
    // 包目录
    let packageRoot = path.resolve(currentRoot, lernaPakgeRootName, packageName);
    // let packageRoot = `${currentRoot}${splitSymbol}${lernaPakgeRootName}${packageName}`;
    // 模板所在目录
    let templateRoot = path.resolve(__dirname,'../package-template') ;

    if (fs.existsSync(currentRoot)) {
        // 读取模板目录下全部文件
        log.info('udesk-helper',`${templateRoot}, ${packageRoot}, ${package}, ${packageName}`)
        copyDir(templateRoot, packageRoot, package, packageName)
    } else {
        log.info('udesk-helper', 'copy template error:package is not exist');
    }
}

function createHandler(args) {
    console.log("args", args)
    // all arguments without --
    let packages = args.packages;
    // // if add new version
    // let newVersion = args.n;
    // // if update old version
    // let updateVersion = args.u;
    // // new version reg
    // let betaSymbol = args.beta;
    // // 'major', 'minor', 'patch'
    // let changeLevel = args.level || 'patch';
    // // default maximun pakges number is 30
    let maxNum = args['max-num'] || 30;

    if (!packages) {
        log.info('udesk-helper', 'packages can not be empty');
        return;
    }

    // const publishPackages = getAllPackages(packages);
    const publishPackages = packages;
    if (publishPackages.length > maxNum) {
        log.info('udesk-helper', 'The maximun number of packets that can be processed is : j% ', maxNum);
    }
    // start to publish
    console.log('process.cwd()', process.cwd())
    publishPackages.forEach(package => {
        if (package) {
            let lc = ChildProcess.spawn(`lerna create ${package}`, {
                cwd: process.cwd(),
                stdio: 'inherit',
                shell: true
            });
            // lc.stdout.on('data', (data) => {
            //     log.info('udesk-helper', data.toString('utf-8'))
            // })
            // lc.stderr.on('data', (data) => {
            //     log.info('udesk-helper stderr',data.toString('utf-8'))
            // })
            lc.on('close', (code) => {
                // log.info('udesk-helper error', `子进程退出，退出码 ${code}`)
                copyTempate(package);
            });
        }
    });
}

const lc = {
    cmd: 'lc [packages...]',
    desc: 'lerna create new package ,then copy template',
    builder: createBuilder,
    handler: createHandler
}
module.exports = lc;