module.exports = {
    'env': {
        'browser': true,
        'commonjs': true,
        'es6': true,
        'jest': true
    },
    'extends': [
        'airbnb',
        'prettier',
        'plugin:react/recommended',
        'plugin:import/typescript',
        'prettier/react'
    ],
    'parser': '@typescript-eslint/parser',
    // 'parserOptions': {
    //     'ecmaFeatures': {
    //         'jsx': true
    //     },
    //     'ecmaVersion': 2018,
    //     'sourceType': 'module'
    // },
    'plugins': [
        'react',
        'babel',
        'jest',
        '@typescript-eslint',
        'react-hooks'
    ],
    'overrides': [
        {
            files: ['*.ts', '*.tsx'],
            rules: {
                '@typescript-eslint/no-unused-vars': [2, { args: 'none' }],
                'no-unused-expressions': 'off',
                '@typescript-eslint/no-unused-expressions': 2,
                '@typescript-eslint/no-use-before-define': 'off'
            }
        },
        {
            files: ['*.md'],
            globals: {
                React: true,
                ReactDOM: true,
                mountNode: true
            },
            rules: {
                indent: 0,
                'no-console': 0,
                'no-plusplus': 0,
                'eol-last': 0,
                'no-script-url': 0,
                'prefer-rest-params': 0,
                'react/no-access-state-in-setstate': 0,
                'react/destructuring-assignment': 0,
                'react/no-multi-comp': 0,
                'jsx-a11y/href-no-hash': 0,
                'import/no-extraneous-dependencies': 0,
                'jsx-a11y/control-has-associated-label': 0
            },
        },
    ],
    'rules': {
        'react/jsx-one-expression-per-line': 0,
        'react/prop-types': 0,
        'react/forbid-prop-types': 0,
        'react/jsx-indent': 0,
        'react/jsx-wrap-multilines': ['error', { declaration: false, assignment: false }],
        'react/jsx-filename-extension': 0,
        'react/state-in-constructor': 0,
        'react/jsx-props-no-spreading': 0,
        'react/destructuring-assignment': 0, // TODO: remove later
        'react/require-default-props': 0,
        'react/sort-comp': 0,
        'react/display-name': 0,
        'react/static-property-placement': 0,
        'react/no-find-dom-node': 0,
        'react/no-unused-prop-types': 0,
        'react/default-props-match-prop-types': 0,
        'react-hooks/rules-of-hooks': 2, // Checks rules of Hooks
        'react/no-danger': 0,

        'jsx-a11y/no-static-element-interactions': 0,
        'jsx-a11y/anchor-has-content': 0,
        'jsx-a11y/click-events-have-key-events': 0,
        'jsx-a11y/anchor-is-valid': 0,
        'jsx-a11y/no-noninteractive-element-interactions': 0,
        // label-has-for has been deprecated
        // https://github.com/evcohen/eslint-plugin-jsx-a11y/blob/master/docs/rules/label-has-for.md
        'jsx-a11y/label-has-for': 0,

        'import/extensions': 0,
        'import/no-cycle': 0,
        "import/no-unresolved": [2, { "ignore": ["udesk-ui"] }],
        // 'import/no-unresolved': [2, {commonjs: true, amd: true}],
        'import/no-extraneous-dependencies': [
            'error',
            {
                devDependencies: [
                    'site/**',
                    'tests/**',
                    'scripts/**',
                    '**/*.test.js',
                    '**/__tests__/*',
                    '*.config.js',
                    '**/*.md',
                ],
            },
        ],

        // 'comma-dangle': ['error', 'always-multiline'],
        //这里是因为react报错,暂时无发解决添加,后面需要研究下为什么react会报未定义先使用的错误
        'no-use-before-define': 0

    },
    'settings': {
        'react': {
            'version': '16.14.0'
        }
    }
};