const HtmlWebpackPlugin = require('html-webpack-plugin');
const path = require('path');
const paths = require('./paths');

const imageInlineSizeLimit = parseInt(
    process.env.IMAGE_INLINE_SIZE_LIMIT || '10000'
);
module.exports = {
    mode: "development",
    entry: path.resolve(__dirname, './test.tsx'),
    output: {
        path: path.resolve(__dirname, './dist'),
        filename: 'index_bundle.js'
    },
    resolve: {
        extensions: ['.tsx', '.ts', '.js','.jsx', '.json'],
        alias: {
            "react": "react",
            "react-dom":"react-dom"
            // "@udesk-webapp/custom-field-container": path.resolve(__dirname, '../src/index.tsx')
        }
    },
    module: {
        strictExportPresence: true,
        rules: [{
            test: /\.(js|mjs|jsx|ts|tsx)$/,
            enforce: 'pre',
            include: [paths.appIndexJs],
            loader: require.resolve('eslint-loader'),
            // loader: 'babel-loader',
            // exclude: path.join(__dirname, "../node_modules")
        }, {
            oneOf: [
                {
                    test: [/\.bmp$/, /\.gif$/, /\.jpe?g$/, /\.png$/],
                    loader: require.resolve('url-loader'),
                    options: {
                        limit: imageInlineSizeLimit,
                        name: 'static/media/[name].[hash:8].[ext]'
                    }
                },
                {
                    test: /\.(js|mjs|jsx|ts|tsx)$/,
                    include: [paths.appSrc, path.resolve(__dirname, './test.tsx'),path.resolve(__dirname, "../../../node_modules/udesk-react")],
                    loader: require.resolve('babel-loader'),
                    exclude: /node_modules\/(?!udesk-react)/,
                    options: {
                        cacheDirectory: false,
                        presets: [
                            '@babel/preset-env',
                            '@babel/preset-react',
                            '@babel/preset-typescript',
                        ],
                        plugins:[
                            "@babel/plugin-proposal-class-properties",
                            'jsx-control-statements',
                            [
                                "import",
                                {
                                    "libraryName": "udesk-ui",
                                    "libraryDirectory": "es",
                                    "style": true //注意：从v4.15.0开始，值为true，再之前的版本设置为false
                                }
                            ]
                        ]
                    }
                },
                {
                    test: /\.css$/,
                    use: [{
                        loader: 'style-loader',
                    }, {
                        loader: 'css-loader'
                    }]
                },
                {
                    test: /\.scss$/,
                    use: [{
                        loader: 'style-loader',
                    }, {
                        loader: 'css-loader'
                    }, {
                        loader: 'sass-loader'
                    }]
                },
                {
                    test: /\.less$/,
                    use: [{
                        loader: 'style-loader',
                    }, {
                        loader: 'css-loader'
                    }, {
                        loader: 'less-loader',
                        options: {
                            lessOptions: {
                                javascriptEnabled: true
                            }
                        }
                    }]
                },
                {
                    loader: require.resolve('file-loader'),
                    // Exclude `js` files to keep "css" loader working as it injects
                    // its runtime that would otherwise be processed through "file" loader.
                    // Also exclude `html` and `json` extensions so they get processed
                    // by webpacks internal loaders.
                    exclude: [/\.(js|mjs|jsx|ts|tsx)$/, /\.html$/, /\.json$/],
                    options: {
                        name: 'static/media/[name].[hash:8].[ext]'
                    }
                }
            ]
        }]
    },
    plugins: [
        new HtmlWebpackPlugin({
            template: path.resolve(__dirname, 'index.html')
        })
    ],
    devServer: {
        contentBase: path.join(__dirname, "build"),
        hot: true,
        open: true,
        port: 9000,
        proxy:[{
            context:['/spa1'],
            target :'http://demo.t4.tryudesk.com',
            changeOrigin:true
        }]
    },

}
