'use strict';

const fs = require('fs');
const path = require('path');

// Make sure any symlinks in the project folder are resolved:
// https://github.com/facebookincubator/create-react-app/issues/637
const appDirectory = fs.realpathSync(process.cwd());
const resolveApp = relativePath => path.resolve(appDirectory, relativePath);

module.exports = {
    dotenv: resolveApp('.env'),
    webpackStatsFile: resolveApp('build/stats.json'),
    webpackServerDir: resolveApp('build/server'),
    appRoot: appDirectory,
    appBuild: resolveApp('dist'),
    appBuildVersion: resolveApp('./dist/VERSION'),
    appReleaseVersionBuild: resolveApp('./dist/RELEASE-VERSION'),
    appStyles: resolveApp('src/styles/index.js'),
    packageIndexJs: resolveApp('index.js'),
    appTestingSupport: resolveApp('testing-support.js'),
    appPublic: resolveApp('public'),
    appHtml: resolveApp('public/index.html'),
    appPackage: resolveApp('package.json'),
    appNodeModules: resolveApp('node_modules'),
    appSrc: resolveApp('src'),
    srcUdesk: resolveApp('src/udesk'),
    srcUdeskNpmRefs: resolveApp('src/udesk/ref-npm-modules'),
    srcUdeskEntry: resolveApp('src/udesk/index.js'),
    udeskNpmImports: resolveApp('src/udesk/ref-npm-modules/_imports.js'),
    packagesDir: resolveApp('packages'),
    packagesNodeModulesRegExp: new RegExp(resolveApp('packages') + `${path.sep}[^/]+?${path.sep}node_modules`),
    packagesSrcDirRegExps: [
        new RegExp("^" + resolveApp('packages') + `${path.sep}[^/]+?${path.sep}src`),
        new RegExp("^" + resolveApp('packages') + `${path.sep}[^/]+?${path.sep}.+?\..+?`)
    ],
    packageHealthCheckEntry: resolveApp('packages/udesk-toolkits-health-check/index.js'),
    srcPluginUtilsCryptoMd5: resolveApp('src/udesk-plugins/utils/crypto/md5.js'),
    srcPluginUtilsCryptoSha256: resolveApp('src/udesk-plugins/utils/crypto/sha-256.js'),
    srcPluginUtilsCryptoSha224: resolveApp('src/udesk-plugins/utils/crypto/sha-224.js'),
};
