var HtmlWebpackPlugin = require('html-webpack-plugin');
var path = require('path');
const postcssNormalize = require('postcss-normalize');
const webpack = require('webpack');

module.exports = {
    entry: path.resolve(__dirname, './test.tsx'),
    output: {
        path: path.resolve(__dirname, './dist'),
        filename: 'index_bundle.js',
    },
    devtool: 'cheap-module-source-map',
    devServer: {
        contentBase: path.join(__dirname, 'dist'),
        compress: true,
        port: 9000,
        proxy: [
            {
                context: ['/spa1', '/users', '/assets', '/static_cmps'],
                target: 'http://demo.t3.tryudesk.com/',
                changeOrigin: true,
            },
        ],
    },
    cache: true,
    resolve: {
        extensions: ['.js', '.json', '.ts', '.tsx', '.jsx', '.less'],
        alias: {
            '@udesk-webapp/session-assist': path.resolve(__dirname, '../src'),
        },
    },
    module: {
        rules: [
            {
                test: /\.(tsx|jsx|ts)?$/,
                exclude: /(node_modules|bower_components)/,
                use: {
                    loader: 'babel-loader',
                    options: {
                        sourceMaps: true,
                    },
                },
            },
            {
                oneOf: [
                    {
                        test: /\.(css)$/i,
                        exclude: /\.module\.css$/,
                        use: [
                            'style-loader',
                            {
                                loader: 'css-loader',
                                options: { 
                                    importLoaders: 1, 
                                    modules: {
                                        localIdentName: '[local]_[hash:base64:5]'
                                    } 
                                },
                            },
                            {
                                loader: 'postcss-loader',
                                options: {
                                    ident: 'postcss',
                                    plugins: () => [
                                        require('postcss-flexbugs-fixes'),
                                        require('postcss-preset-env')({
                                            autoprefixer: {
                                                flexbox: 'no-2009',
                                            },
                                            stage: 3,
                                        }),
                                        // Adds PostCSS Normalize as the reset css with default options,
                                        // so that it honors browserslist config in package.json
                                        // which in turn let's users customize the target behavior as per their needs.
                                        postcssNormalize(),
                                    ],
                                },
                            },
                        ],
                        sideEffects: true,
                    },
                    {
                        test: /\.(module.css)$/i,
                        use: [
                            'style-loader',
                            {
                                loader: 'css-loader',
                                options: {
                                    importLoaders: 1,
                                    modules: {
                                        localIdentName: '[local]_[hash:base64:5]',
                                    }
                                   
                                },
                            },
                            {
                                loader: 'postcss-loader',
                                options: {
                                    ident: 'postcss',
                                    plugins: () => [
                                        require('postcss-flexbugs-fixes'),
                                        require('postcss-preset-env')({
                                            autoprefixer: {
                                                flexbox: 'no-2009',
                                            },
                                            stage: 3,
                                        }),
                                        // Adds PostCSS Normalize as the reset css with default options,
                                        // so that it honors browserslist config in package.json
                                        // which in turn let's users customize the target behavior as per their needs.
                                        postcssNormalize(),
                                    ],
                                },
                            },
                        ],
                    },
                    {
                        test: /\.(scss)$/i,
                        exclude: /\.module\.(scss|sass)$/,
                        use: [
                            'style-loader',
                            {
                                loader: 'css-loader',
                                options: {
                                    importLoaders: 3,
                                    modules: {
                                        localIdentName: '[local]_[hash:base64:5]',
                                    }
                                },
                            },
                            {
                                loader: 'postcss-loader',
                                options: {
                                    ident: 'postcss',
                                    plugins: () => [
                                        require('postcss-flexbugs-fixes'),
                                        require('postcss-preset-env')({
                                            autoprefixer: {
                                                flexbox: 'no-2009',
                                            },
                                            stage: 3,
                                        }),
                                        // Adds PostCSS Normalize as the reset css with default options,
                                        // so that it honors browserslist config in package.json
                                        // which in turn let's users customize the target behavior as per their needs.
                                        postcssNormalize(),
                                    ],
                                },
                            },
                            'sass-loader',
                        ],
                    },

                    {
                        test: /\.(module.sass|module.scss)$/i,
                        use: [
                            'style-loader',
                            {
                                loader: 'css-loader',
                                options: {
                                    importLoaders: 3,
                                    modules: {
                                        localIdentName: '[local]_[hash:base64:5]',
                                    }
                                },
                            },
                            {
                                loader: 'postcss-loader',
                                options: {
                                    ident: 'postcss',
                                    plugins: () => [
                                        require('postcss-flexbugs-fixes'),
                                        require('postcss-preset-env')({
                                            autoprefixer: {
                                                flexbox: 'no-2009',
                                            },
                                            stage: 3,
                                        }),
                                        // Adds PostCSS Normalize as the reset css with default options,
                                        // so that it honors browserslist config in package.json
                                        // which in turn let's users customize the target behavior as per their needs.
                                        postcssNormalize(),
                                    ],
                                },
                            },
                            {
                                loader: 'sass-loader',
                                options: {
                                    additionalData: '$prefix:udesk-avs-web-prefix;',
                                    sourceMap: true,
                                },
                            },
                        ],
                    },
                    {
                        test: /\.less$/,
                        use: [
                            {
                                loader: 'style-loader',
                            },
                            {
                                loader: 'css-loader',
                            },
                            {
                                loader: 'less-loader',
                                
                                options: {
                                    lessOptions: {
                                        javascriptEnabled: true,
                                        modifyVars: {
                                            'ant-prefix': 'udesk-avs-web-prefix',
                                            // 这里支持其他变量 参考 https://github.com/ant-design/ant-design/blob/master/components/style/themes/default.less
                                        },
                                    },
                                },
                            },
                        ],
                    },
                    {
                        test: /\.(png|jpg|gif|svg)$/,
                        use: [
                            {
                                loader: 'file-loader',
                                options: {},
                            },
                        ],
                    },
                ],
            },
        ],
    },
    plugins: [
        new HtmlWebpackPlugin({
            template: path.resolve(__dirname, 'index.html'),
        }),
        new webpack.NormalModuleReplacementPlugin(/\/style\/index.less$/, (opt) => {
            if (!opt.rawRequest && /antd\/es\/(\w|-)*\/?style/.test(opt.context)) {
                const request = opt.request.split('!');
                if (request.length > 0) {
                    const last = request[request.length - 1];
                    if (/antd\/es\/(\w|-)+\/style/.test(last)) {
                        return;
                    }
                }
                request.pop();
                const newPath = 'udesk-ui/es/style/reset.less';
                request.push(newPath);
                opt.request = request.join('!');
            }
        }),
    ],
};
