export function highlightText(elem, keyword, options) {
  if (!keyword) return;
  var textNodes = getTextNodesIn(elem, function (n) {
    var keywordRegExp = new RegExp(keyword);
    return !/^\n\s*$/.test(n.nodeValue) && keywordRegExp.test(n.nodeValue);
  });
  var splitRegExp = new RegExp("(".concat(keyword, ")"));
  textNodes.forEach(function (node) {
    var txtArr = node.nodeValue.split(splitRegExp).filter(function (txt) {
      return txt !== '';
    });
    txtArr.forEach(function (txt) {
      var newNode;
      if (txt !== keyword) {
        newNode = document.createTextNode(txt);
      } else {
        var tagName = options && options.tagName || 'em';
        newNode = document.createElement(tagName);
        newNode.setAttribute('_udeskHighlight', '');
        if (options && options.className) {
          newNode.className = options.className;
        }
        newNode.appendChild(document.createTextNode(txt));
      }
      node.parentNode.insertBefore(newNode, node);
    });
    node.parentNode.removeChild(node);
  });
}
export function highlightTextIndex(elem, item, options) {
  if (!item) return;
  var textNodes = getTextNodesIn(elem);
  var start = item.start,
    length = item.length;
  var currentStart = 0;
  var txtArr = [];
  var strArr = [];
  for (var i = textNodes.length - 1; i >= 0; i--) {
    if (!item.start && !item.length) {
      txtArr = [{
        text: textNodes[i].nodeValue,
        flag: true
      }].filter(function (txt) {
        return txt.text !== '';
      });
    } else {
      if (length === 0) return;
      if (start >= currentStart && start < currentStart + textNodes[i].length || strArr.length > 0) {
        var currentStr = '';
        if (strArr.length === 0) {
          var startStr = textNodes[i].nodeValue.slice(0, start - currentStart);
          var endStr = textNodes[i].nodeValue.slice(start - currentStart + length);
          currentStr = textNodes[i].nodeValue.substr(start - currentStart, length);
          txtArr = [{
            text: startStr,
            flag: false
          }, {
            text: currentStr,
            flag: true
          }, {
            text: endStr,
            flag: false
          }].filter(function (txt) {
            return txt.text !== '';
          });
        } else {
          var _endStr = textNodes[i].nodeValue.slice(length);
          currentStr = textNodes[i].nodeValue.substr(0, length);
          txtArr = [{
            text: currentStr,
            flag: true
          }, {
            text: _endStr,
            flag: false
          }].filter(function (txt) {
            return txt.text !== '';
          });
        }
        strArr.push(currentStr);
        length -= currentStr.length;
      }
      currentStart += textNodes[i].length;
    }
    for (var j = 0; j < txtArr.length; j++) {
      var newNode = void 0;
      if (!txtArr[j].flag) {
        newNode = document.createTextNode(txtArr[j].text);
      } else {
        var tagName = options && options.tagName || 'em';
        newNode = document.createElement(tagName);
        newNode.setAttribute('_udeskHighlight', '');
        if (options && options.className) {
          newNode.className = options.className;
        }
        newNode.appendChild(document.createTextNode(txtArr[j].text));
      }
      textNodes[i].parentNode.insertBefore(newNode, textNodes[i]);
    }
    if (txtArr && txtArr.length > 0) {
      textNodes[i].parentNode.removeChild(textNodes[i]);
    }
  }
}
export function clearHighlight(elem) {
  var emElems = getElementsByAttributeName(elem, '_udeskHighlight');
  for (var i = emElems.length - 1; i >= 0; i--) {
    var emNode = emElems[i];
    var parentNode = emNode.parentNode;
    var newNode = document.createTextNode(emNode.textContent);
    parentNode.insertBefore(newNode, emNode);
    parentNode.removeChild(emNode);
    parentNode.normalize();
  }
}
export function getTextNodesIn(elem, opt_fnFilter) {
  var textNodes = [];
  if (elem) {
    for (var nodes = elem.childNodes, i = nodes.length; i--;) {
      var node = nodes[i],
        nodeType = node.nodeType;
      if (nodeType === 3) {
        if (!opt_fnFilter || opt_fnFilter(node, elem)) {
          textNodes.push(node);
        }
      } else if (nodeType === 1 || nodeType === 9 || nodeType === 11) {
        textNodes = textNodes.concat(getTextNodesIn(node, opt_fnFilter));
      }
    }
  }
  return textNodes;
}
function getElementsByAttributeName(elem, attributeName) {
  var matchNodes = [];
  if (elem) {
    var nodes = elem.children;
    for (var i = 0; i < nodes.length; i++) {
      var node = nodes[i];
      if (node.hasAttribute(attributeName)) {
        matchNodes.push(node);
      } else {
        matchNodes = matchNodes.concat(getElementsByAttributeName(node, attributeName));
      }
    }
  }
  return matchNodes;
}