import { childlessTags, closingTagAncestorBreakers, closingTags, voidTags } from "./tags";
export var parseDefaults = {
  voidTags: voidTags,
  closingTags: closingTags,
  childlessTags: childlessTags,
  closingTagAncestorBreakers: closingTagAncestorBreakers,
  includePositions: false
};
export function isRealNaN(x) {
  return typeof x === 'number' && isNaN(x);
}
export function arrayIncludes(array, searchElement, position) {
  var len = array.length;
  if (len === 0) return false;
  var lookupIndex = position | 0;
  var isNaNElement = isRealNaN(searchElement);
  var searchIndex = lookupIndex >= 0 ? lookupIndex : len + lookupIndex;
  while (searchIndex < len) {
    var element = array[searchIndex++];
    if (element === searchElement) return true;
    if (isNaNElement && isRealNaN(element)) return true;
  }
  return false;
}
export function formatAttributes(attributes) {
  return attributes.reduce(function (attrs, attribute) {
    var key = attribute.key,
      value = attribute.value;
    if (value === null) {
      return "".concat(attrs, " ").concat(key);
    }
    var quoteEscape = value.indexOf("'") !== -1;
    var quote = quoteEscape ? '"' : "'";
    return "".concat(attrs, " ").concat(key, "=").concat(quote).concat(value).concat(quote);
  }, '');
}
export function toHTML(tree, options) {
  return tree.map(function (node) {
    if (node.type === 'text') {
      return node.content;
    }
    if (node.type === 'comment') {
      return "<!--".concat(node.content, "-->");
    }
    var tagName = node.tagName,
      attributes = node.attributes,
      children = node.children;
    var isSelfClosing = arrayIncludes(options.voidTags, tagName.toLowerCase());
    return isSelfClosing ? "<".concat(tagName).concat(formatAttributes(attributes), ">") : "<".concat(tagName).concat(formatAttributes(attributes), ">").concat(toHTML(children, options), "</").concat(tagName, ">");
  }).join('');
}
export function stringify(ast) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : parseDefaults;
  return toHTML(ast, options);
}