import React from "react";
import { RobotQuestion } from "./question/robot-question/types";
import { IArticle, IAttachment, IExpression, ILink, IQuestion, RepositoryType, Scope } from "./types";
export declare const useRequestData: (type: RepositoryType, params: {
    query?: string | undefined;
    scope?: Scope | undefined;
    per_page?: number | undefined;
    page?: number | undefined;
    current_works?: boolean | undefined;
    catalog_id?: string | number | undefined;
    knowledge_catalog_id?: number | undefined;
}) => {
    loading: boolean;
    catalogs: never[];
    data: any;
};
/**
 * 返回一个 object，key 为 catalog的 id，值为该 catalog下的内容
 */
declare type Catalog = {
    knowledge_catalog_id: number;
} | {
    knowledge_attachment: {
        knowledge_catalog_id: number;
    };
};
export declare const useCatalogMap: (data: Catalog[]) => {
    [key: number]: any[];
};
export declare function hit(id: string | number, type: "KnowledgeQuestion" | "UsefulExpression" | "UsefulLink" | "KnowledgeAttachment"): Promise<import("axios").AxiosResponse<any>>;
export declare const RepositoryWidgetContext: React.Context<{
    allowSendQuestion?: boolean | undefined;
    onUse?: ((type: RepositoryType | "km", data: IExpression | ILink | IQuestion | RobotQuestion | IAttachment["knowledge_attachment"] | IArticle) => void) | undefined;
    showRobotQuestionTab?: boolean | undefined;
    onSend?: ((type: "attachment" | "question" | "robotQuestion", data: IAttachment["knowledge_attachment"] | IQuestion | RobotQuestion) => void) | undefined;
    kmApiBase?: string | undefined;
    onExpand?: ((keys: (number | string)[]) => void) | undefined;
    expandedKeys?: (string | number)[] | undefined;
    activeScope?: Scope | undefined;
    onScopeChange?: ((scope: Scope) => void) | undefined;
    onCatalogChange?: ((catalog?: number | string) => void) | undefined;
    onSpaceChange?: ((space?: number | string) => void) | undefined;
    activeCatalog?: number | undefined;
    activeSpace?: string | number | undefined;
    showPublicScope?: boolean | undefined;
    previewUrl: string;
    agent?: IAgent | undefined;
    customer?: Customer | undefined;
    ticket?: ITicket | undefined;
    ticketCustomFields?: CustomField[] | undefined;
    customerCustomFields?: CustomField[] | undefined;
}>;
export declare const replaceHtmlTag: (text: string) => string;
export declare const makeQuestionData: (item: {
    content: string;
    content_type: string;
    title: string;
    url?: string | undefined;
    id: number;
    isRobotQuestion?: boolean | undefined;
}, platform?: string, content?: string, url?: string) => any;
/**
 * 机器人问题转换为普通问题格式
 * @param {RobotQuestion} robotQuestion
 * @return Question
 */
export declare const robotToCommonQuestion: (robotQuestion: RobotQuestion) => IQuestion;
export declare const articleToCommonQuestion: (article: IArticle) => IQuestion;
interface TabSetting {
    activeTab?: RepositoryType | "km";
    activeScope?: Scope;
    activeSpace?: string;
    activeCatalog?: number;
}
export declare const useTabSetting: () => {
    tabSetting: TabSetting | undefined;
    saveTab: (key: any) => void;
    loading: boolean;
    saveScope: (scope: any) => void;
    saveSpace: (space: any) => void;
    saveCatalog: (catalog: any) => void;
};
/**
 * 除了 question 以外，其他的标签页如果 scope 是 robot，则返回 public
 * @param {Scope} scope
 * @returns {string | "public" | "personal"}
 */
export declare const useSafeScope: (scope?: Scope) => "public" | "personal";
/**
 * 如果 context 里的 expandedKeys 为 undefined，返回所有的 catalog，即展开所有的节点
 * @param {object} catalogMap
 */
export declare const useExpandedKeys: (catalogMap: {
    [key: string]: any;
}) => (string | number)[];
export declare const useSelectionCache: (key: number | string | undefined, { defaultTab, defaultScope, }: {
    defaultTab?: RepositoryType | "km" | undefined;
    defaultScope?: Scope | undefined;
}) => {
    activeTab: RepositoryType | "km";
    setActiveTab: (tab: RepositoryType | "km") => void;
    query: string;
    setQuery: (keyword: string) => void;
    activeScope: Scope;
    setActiveScope: (scope: Scope) => void;
    expandedKeys: (string | number)[] | undefined;
    setExpandedKeys: (keys: (number | string)[]) => void;
    scrollTop: number | undefined;
    setScrollTop: (scrollTop: number) => void;
};
export {};
