"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findItemCommon = exports.deleteTreeNode = exports.moveTreeNode = exports.findTreeNode = exports.forEachTree = void 0;
var get_1 = __importDefault(require("lodash-es/get"));
var set_1 = __importDefault(require("lodash-es/set"));
function forEachTree(treeData, callback, options) {
    if (treeData === void 0) { treeData = []; }
    var _a = (options || {}).nodeChildrenPath, nodeChildrenPath = _a === void 0 ? 'subs' : _a;
    treeData === null || treeData === void 0 ? void 0 : treeData.forEach(function (i) {
        callback(i);
        forEachTree((0, get_1.default)(i, nodeChildrenPath), callback, options);
    });
}
exports.forEachTree = forEachTree;
function findTreeNode(treeData, nodeKey, nodeChildrenPath) {
    if (!nodeKey) {
        return;
    }
    var result;
    nodeKey.split('-').reduce(function (prevValue, item) {
        result = prevValue[parseInt(item)];
        if (!result) {
            return [];
        }
        return (0, get_1.default)(result, nodeChildrenPath, []);
    }, treeData);
    return result;
}
exports.findTreeNode = findTreeNode;
/**
 * 移动树里的两个节点，会修改源数据
 * @param {Option[]} tree
 * @param {string} nodeKey
 * @param {string} targetNodeKey - 如果为空，则移动到 tree 里的 position 位置
 * @param nodeChildrenPath
 * @param {number} position
 */
function moveTreeNode(tree, nodeKey, targetNodeKey, nodeChildrenPath, position) {
    var targetNodeSubs = tree;
    if (targetNodeKey) {
        var targetNode = findTreeNode(tree, targetNodeKey, nodeChildrenPath);
        if (!targetNode) {
            return;
        }
        if (!(0, get_1.default)(targetNode, nodeChildrenPath)) {
            (0, set_1.default)(targetNode, nodeChildrenPath, []);
        }
        targetNodeSubs = (0, get_1.default)(targetNode, nodeChildrenPath);
    }
    var dragNode = deleteTreeNode(tree, nodeKey, nodeChildrenPath);
    if (!dragNode) {
        return;
    }
    if (typeof position !== 'number') {
        targetNodeSubs.push(dragNode);
    }
    else {
        if (nodeKey.substring(0, nodeKey.lastIndexOf('-')) === targetNodeKey.substring(0, targetNodeKey.lastIndexOf('-')) &&
            parseInt(nodeKey.substring(nodeKey.lastIndexOf('-'))) < position) {
            position -= 1;
        }
        targetNodeSubs.splice(position, 0, dragNode);
    }
}
exports.moveTreeNode = moveTreeNode;
function deleteTreeNode(tree, nodeKey, nodeChildrenPath) {
    var parentNodeKey = nodeKey.substring(0, nodeKey.lastIndexOf('-'));
    var _parentSubs;
    if (parentNodeKey) {
        var _parentNode = findTreeNode(tree, nodeKey.substring(0, nodeKey.lastIndexOf('-')), nodeChildrenPath);
        if (!(0, get_1.default)(_parentNode, nodeChildrenPath)) {
            return;
        }
        _parentSubs = (0, get_1.default)(_parentNode, nodeChildrenPath);
    }
    else {
        _parentSubs = tree;
    }
    var _dragNodeIndex = parseInt(nodeKey.substring(nodeKey.length - 1));
    var dragNode = _parentSubs.splice(_dragNodeIndex, 1)[0];
    return dragNode;
}
exports.deleteTreeNode = deleteTreeNode;
function findItemCommon(value, nodeKey, isFather) {
    if (isFather === void 0) { isFather = false; }
    var changeItem = value;
    nodeKey.forEach(function (item) {
        changeItem = (0, get_1.default)(changeItem, ['subs', item], changeItem[item]);
    });
    return changeItem;
}
exports.findItemCommon = findItemCommon;
