"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importStar(require("react"));
var udesk_ui_1 = require("udesk-ui");
var locale_1 = require("@udesk-webapp/locale");
var FileTextTwoTone_1 = tslib_1.__importDefault(require("@ant-design/icons/FileTextTwoTone"));
var axios_1 = tslib_1.__importDefault(require("axios"));
var dompurify_1 = tslib_1.__importDefault(require("dompurify"));
var utilities_1 = require("../utilities");
function getArticle(kmBase, params) {
    if (!kmBase) {
        return Promise.reject('km host is empty');
    }
    return axios_1.default
        .get("".concat(kmBase, "/articles/").concat(params.id), {
        params: {
            companyId: params.companyId,
            subdomain: params.subdomain,
            email: params.email,
            limitSpaceIds: params.limitSpaceIds,
            enablePrivate: params.enablePrivate,
            passport: params.passport,
        },
    })
        .then(function (res) {
        if (res.data.code === 200) {
            return res.data.data;
        }
        throw new Error(res.data);
    });
}
var useArticle = function (host, _a) {
    var companyId = _a.companyId, subdomain = _a.subdomain, email = _a.email, limitSpaceIds = _a.limitSpaceIds, enablePrivate = _a.enablePrivate, passport = _a.passport, articleId = _a.articleId;
    var _b = (0, react_1.useState)(false), loading = _b[0], setLoading = _b[1];
    var _c = (0, react_1.useState)(), article = _c[0], setArticle = _c[1];
    (0, react_1.useEffect)(function () {
        setLoading(true);
        getArticle(host, {
            companyId: companyId,
            subdomain: subdomain,
            email: email,
            limitSpaceIds: limitSpaceIds,
            enablePrivate: enablePrivate,
            passport: passport,
            id: articleId,
        })
            .then(function (res) {
            setArticle(res);
        })
            .finally(function () { return setLoading(false); });
    }, [companyId, subdomain, email, limitSpaceIds, enablePrivate, passport, articleId, host]);
    return [article, loading];
};
var PopoverContent = function (_a) {
    var data = _a.data, companyId = _a.companyId, subdomain = _a.subdomain, email = _a.email, limitSpaceIds = _a.limitSpaceIds, enablePrivate = _a.enablePrivate, passport = _a.passport;
    var kmApiBase = (0, react_1.useContext)(utilities_1.RepositoryWidgetContext).kmApiBase;
    var _b = useArticle(kmApiBase || '', {
        companyId: companyId,
        subdomain: subdomain,
        email: email,
        limitSpaceIds: limitSpaceIds,
        enablePrivate: enablePrivate,
        passport: passport,
        articleId: data.id,
    }), article = _b[0], loading = _b[1];
    if (loading) {
        return react_1.default.createElement(udesk_ui_1.Spin, null);
    }
    if (article) {
        return (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement("h4", null, article.title),
            react_1.default.createElement("div", { dangerouslySetInnerHTML: { __html: dompurify_1.default.sanitize(article.contentBody) } })));
    }
    return null;
};
var ArticleItem = function (_a) {
    var data = _a.data, props = tslib_1.__rest(_a, ["data"]);
    var t = (0, locale_1.useTranslation)().t;
    var _b = (0, react_1.useState)(false), loading = _b[0], setLoading = _b[1];
    var _c = (0, react_1.useContext)(utilities_1.RepositoryWidgetContext), kmApiBase = _c.kmApiBase, onUse = _c.onUse;
    var onUseClick = (0, react_1.useCallback)(function () {
        if (loading) {
            return;
        }
        setLoading(true);
        getArticle(kmApiBase || '', {
            companyId: props.companyId,
            subdomain: props.subdomain,
            email: props.email,
            limitSpaceIds: props.limitSpaceIds,
            enablePrivate: props.enablePrivate,
            passport: props.passport,
            id: data.id,
        })
            .then(function (res) { return onUse === null || onUse === void 0 ? void 0 : onUse('km', (0, utilities_1.articleToCommonQuestion)(res)); })
            .finally(function () { return setLoading(false); });
    }, [
        data.id,
        kmApiBase,
        loading,
        onUse,
        props.companyId,
        props.email,
        props.enablePrivate,
        props.limitSpaceIds,
        props.passport,
        props.subdomain,
    ]);
    return (react_1.default.createElement(udesk_ui_1.List.Item, { actions: [
            react_1.default.createElement(udesk_ui_1.Button, { key: 'use', type: 'link', onClick: onUseClick }, t('label.use')),
        ] },
        react_1.default.createElement(udesk_ui_1.Popover, { content: react_1.default.createElement(PopoverContent, tslib_1.__assign({ data: data }, props)), placement: 'left' },
            react_1.default.createElement("div", null,
                react_1.default.createElement(FileTextTwoTone_1.default, null),
                data.title))));
};
exports.default = ArticleItem;
