"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importStar(require("react"));
var cytoscape_1 = tslib_1.__importDefault(require("cytoscape"));
var button_1 = tslib_1.__importDefault(require("udesk-ui/es/button"));
var ajax_1 = require("../ajax");
var index_module_less_1 = tslib_1.__importDefault(require("./index.module.less"));
var stylesheets = {
    default: [{
            selector: 'node',
            style: {
                'label': 'data(name)',
                'width': 'data(width)',
                'height': 'data(width)',
                'font-size': 6,
                'overlay-opacity': 0,
                'border-width': 1,
                'border-style': 'solid',
                'border-color': '#fff',
                'opacity': 1
            }
        }, {
            selector: 'edge',
            style: {
                'curve-style': 'bezier',
                'target-arrow-shape': 'triangle',
                'width': 1,
                'arrow-scale': .2,
                'overlay-opacity': 0,
                'opacity': .5
            }
        }, {
            selector: 'node.hover',
            style: {
                'width': 'data(hoverWidth)',
                'height': 'data(hoverWidth)',
                'opacity': 1
            }
        }, {
            selector: 'edge.hover',
            style: {
                'width': 4,
                'arrow-scale': .6,
                'opacity': 1
            }
        }, {
            selector: 'node.click',
            style: {
                'width': 'data(hoverWidth)',
                'height': 'data(hoverWidth)',
                'opacity': 1
            }
        }, {
            selector: 'edge.click',
            style: {
                'width': 4,
                'arrow-scale': .6,
                'opacity': 1
            }
        }]
};
var colorsMapping = {
    default: {
        nodesColors: [
            '#D6162E',
            '#294254',
            '#46A1A9',
            '#E38163',
            '#7BCEAE',
            '#61A682',
            '#D58D16',
            '#C3A29A',
            '#6E6F74'
        ],
        nodesHighLightColors: [
            '#D6162E',
            '#294254',
            '#46A1A9',
            '#E38163',
            '#7BCEAE',
            '#61A682',
            '#D58D16',
            '#C3A29A',
            '#6E6F74'
        ],
        edgesColors: [
            '#D6162E',
            '#294254',
            '#46A1A9',
            '#E38163',
            '#7BCEAE',
            '#61A682',
            '#D58D16',
            '#C3A29A',
            '#6E6F74'
        ],
        edgesHighLightColors: [
            '#D6162E',
            '#294254',
            '#46A1A9',
            '#E38163',
            '#7BCEAE',
            '#61A682',
            '#D58D16',
            '#C3A29A',
            '#6E6F74'
        ]
    }
};
// 获取颜色配置文件
function getColorMap(theme) {
    if (colorsMapping) {
        if (theme) {
            return colorsMapping[theme];
        }
        return colorsMapping['default'];
    }
    return null;
}
// 获取颜色
function getColor(index, colorList) {
    if (typeof index === 'number' && colorList instanceof Array) {
        var count = index % colorList.length;
        return colorList[count];
    }
    return null;
}
// 获取图表基本配置
function getStylesheet(theme) {
    if (stylesheets) {
        if (theme) {
            return stylesheets[theme];
        }
        return stylesheets['default'];
    }
    return null;
}
function getPointSize(params) {
    var initialSize = params.initialSize || 1;
    var criterion = params.criterion || 1;
    var multiple = params.multiple || 1;
    var initialSizeNumber = Number(initialSize);
    var criterionNumber = Number(criterion);
    var multipleNumber = Number(multiple);
    if (Number.isNaN(initialSizeNumber) || initialSizeNumber <= 0) {
        initialSizeNumber = 1;
    }
    if (Number.isNaN(criterionNumber) || criterionNumber <= 0) {
        criterionNumber = 1;
    }
    if (Number.isNaN(multipleNumber) || multipleNumber <= 0) {
        multipleNumber = 1;
    }
    var level = Math.ceil(initialSizeNumber / criterionNumber);
    return level * multipleNumber;
}
function formatter(data) {
    var formattedData = [];
    if (data && data.nodes && data.relations) {
        // 知识点的图上大小根据知识点的连接数进行计算
        var knowledgePointWidthMap_1 = {};
        if (data.relations instanceof Array && data.relations.length > 0) {
            data.relations.forEach(function (item) {
                if (item.startNodeId in knowledgePointWidthMap_1) {
                    knowledgePointWidthMap_1[item.startNodeId]++;
                }
                else {
                    knowledgePointWidthMap_1[item.startNodeId] = 1;
                }
            });
        }
        // 后端请求数据需要进行重构，重构所需的格式:https://js.cytoscape.org/#getting-started中的elements
        if (data.nodes instanceof Array && data.nodes.length > 0) {
            data.nodes.forEach(function (item) {
                var count = knowledgePointWidthMap_1[item.id];
                formattedData.push({
                    data: {
                        id: 'knowledgeNode' + item.id,
                        name: item.name,
                        group: 'knowledgeNode' + item.id,
                        properties: item.properties,
                        width: getPointSize({
                            initialSize: count,
                            criterion: 6,
                            multiple: 8
                        }),
                        hoverWidth: getPointSize({
                            initialSize: count,
                            criterion: 6,
                            multiple: 10
                        })
                    }
                });
            });
        }
        if (data.relations instanceof Array && data.relations.length > 0) {
            data.relations.forEach(function (item) {
                formattedData.push({
                    data: {
                        id: 'relation' + item.id,
                        name: item.name,
                        properties: item.properties,
                        source: 'knowledgeNode' + item.startNodeId,
                        target: 'knowledgeNode' + item.endNodeId,
                        group: 'relation' + item.id
                    }
                });
            });
        }
    }
    if (formattedData instanceof Array && formattedData.length > 0) {
        var colorList_1 = getColorMap() || {
            nodesColors: [],
            nodesHighLightColors: [],
            edgesColors: [],
            edgesHighLightColors: []
        };
        var nodesRegExp_1 = /knowledgeNode+/;
        // const relationRegExp = /relation+/;
        var cyStylesheet_1 = [];
        var nodeColorCount_1 = 0;
        var relationColorCount_1 = 0;
        // const arrowColorCount = 0;
        var relationsTypes_1 = {};
        formattedData.forEach(function (item) {
            // 点之间不需要同一类型
            if (nodesRegExp_1.test(item.data.group)) {
                var color = getColor(nodeColorCount_1, colorList_1.nodesColors);
                var highLightColor = getColor(nodeColorCount_1, colorList_1.nodesHighLightColors);
                if (color) {
                    cyStylesheet_1.push({
                        selector: 'node[group="' + item.data.group + '"]',
                        style: {
                            'background-color': color ? color : null
                        }
                    }, {
                        selector: 'node[group="' + item.data.group + '"].click',
                        style: {
                            'background-color': highLightColor ? highLightColor : null
                        }
                    }, {
                        selector: 'node[group="' + item.data.group + '"].hover',
                        style: {
                            'background-color': highLightColor ? highLightColor : null
                        }
                    });
                    nodeColorCount_1++;
                }
            }
            else {
                // 线之间可能有同种类型，同一类型颜色相同，所以用一个对象列出已经有颜色的类型
                var color = void 0, highLightColor = void 0;
                if (Object.prototype.hasOwnProperty.call(relationsTypes_1, item.data.group)) {
                    color = relationsTypes_1[item.data.group]['color'];
                    highLightColor = relationsTypes_1[item.data.group]['highLightColor'];
                }
                else {
                    color = getColor(relationColorCount_1, colorList_1.edgesColors);
                    highLightColor = getColor(relationColorCount_1, colorList_1.edgesHighLightColors);
                    relationsTypes_1[item.data.group] = {
                        color: color,
                        highLightColor: highLightColor
                    };
                }
                // <pos>-arrow-color这里pos是一个变量，箭头颜色不应该这样直接设置，因为箭头的种类很多，
                // 应该用正则替换来做，赶任务，抽空优化
                if (color) {
                    cyStylesheet_1.push({
                        selector: 'edge[group="' + item.data.group + '"]',
                        style: {
                            'line-color': color ? color : null,
                            'target-arrow-color': color
                        }
                    }, {
                        selector: 'edge[group="' + item.data.group + '"].click',
                        style: {
                            'line-color': highLightColor ? highLightColor : null,
                            'target-arrow-color': highLightColor
                        }
                    }, {
                        selector: 'edge[group="' + item.data.group + '"].hover',
                        style: {
                            'line-color': highLightColor ? highLightColor : null,
                            'target-arrow-color': highLightColor
                        }
                    });
                    relationColorCount_1++;
                }
            }
        });
        var stylesheet = getStylesheet() || [];
        cyStylesheet_1 = stylesheet.concat(cyStylesheet_1);
        return [formattedData, cyStylesheet_1];
    }
    return [[], []];
}
var closeStyle = {
    border: '0',
    background: 'transparent',
    position: 'absolute',
    right: '0',
    top: '0',
    fontSize: '20px'
};
var relativeStyle = {
    position: 'relative'
};
var KnowledgeGraph = function (_a) {
    var knowledgeId = _a.knowledgeId, className = _a.className;
    var ref = (0, react_1.useRef)(null);
    var hoverElement = (0, react_1.useRef)(null);
    var hoverGraph = (0, react_1.useCallback)(function (cyEve) {
        var target = cyEve.target;
        var hoverEle = hoverElement.current;
        if (hoverEle) {
            if (hoverEle._private.group === 'nodes') {
                hoverEle.neighborhood().removeClass('hover');
            }
            else if (hoverEle._private.group === 'edges') {
                hoverEle.connectedNodes().removeClass('hover');
            }
            hoverEle.removeClass('hover');
        }
        if (target.length > 0) {
            // 鼠标hover高亮，给当前和相邻元素加上hover状态的class
            // 如果这些元素上有了点击状态的类名，就不要添加颜色
            if (target._private.group === 'nodes') {
                var neighborhood = cyEve.target.neighborhood();
                var targetId_1 = target._private.data.id;
                var targetNodes_1 = [];
                neighborhood.forEach(function (item) {
                    if (item._private.group === 'edges' && item._private.data.source === targetId_1) {
                        var node = item._private.data.target;
                        if (!item.hasClass('click')) {
                            item.addClass('hover');
                        }
                        if (!targetNodes_1.includes(node)) {
                            targetNodes_1.push(node);
                        }
                    }
                });
                neighborhood.forEach(function (item) {
                    if (item._private.group === 'nodes' && targetNodes_1.includes(item._private.data.id)) {
                        if (!item.hasClass('click')) {
                            item.addClass('hover');
                        }
                    }
                });
            }
            else if (target._private.group === 'edges') {
                target.connectedNodes().filter(function (ele) { return !ele.hasClass('click'); }).addClass('hover');
            }
            if (!target.hasClass('click')) {
                target.addClass('hover');
            }
            hoverElement.current = target;
        }
    }, []);
    var outOfGraph = (0, react_1.useCallback)(function (cyEve) {
        var hoverEle = hoverElement.current;
        if (hoverEle) {
            if (hoverEle._private.group === 'nodes') {
                hoverEle.neighborhood().removeClass('hover');
            }
            else if (hoverEle._private.group === 'edges') {
                hoverEle.connectedNodes().removeClass('hover');
            }
            hoverEle.removeClass('hover');
        }
    }, []);
    var clickedElementRef = (0, react_1.useRef)(null);
    var clickGraph = (0, react_1.useCallback)(function (cyEve) {
        var target = cyEve.target;
        var clickedEle = clickedElementRef.current;
        if (clickedEle) {
            if (clickedEle._private.group === 'nodes') {
                clickedEle.neighborhood().removeClass('click');
            }
            else if (clickedEle._private.group === 'edges') {
                clickedEle.connectedNodes().removeClass('click');
            }
            clickedEle.removeClass('click');
        }
        // 区分一下，如果是线条节点，用connectedNodes方法
        // 如果是知识点节点，用neighborhood
        if (target.length > 0) {
            // 点击高亮，给当前点击状态和相邻元素加上点击状态的class
            if (target._private.group === 'nodes') {
                var neighborhood = cyEve.target.neighborhood();
                var targetId_2 = target._private.data.id;
                var targetNodes_2 = [];
                neighborhood.forEach(function (item) {
                    if (item._private.group === 'edges' && item._private.data.source === targetId_2) {
                        var node = item._private.data.target;
                        item.addClass('click');
                        item.removeClass('hover');
                        if (!targetNodes_2.includes(node)) {
                            targetNodes_2.push(node);
                        }
                    }
                });
                neighborhood.forEach(function (item) {
                    if (item._private.group === 'nodes' && targetNodes_2.includes(item._private.data.id)) {
                        item.addClass('click');
                        item.removeClass('hover');
                    }
                });
            }
            else if (target._private.group === 'edges') {
                target.connectedNodes().addClass('click').removeClass('hover');
            }
            target.addClass('click');
            target.removeClass('hover');
            clickedElementRef.current = target;
            var tooltipData_1 = target._private.data.properties;
            if (typeof tooltipData_1 === 'object') {
                var tooltipDataKeys = Object.keys(tooltipData_1);
                var tooltipDataList_1 = [];
                if (tooltipDataKeys.length > 0) {
                    tooltipDataKeys.forEach(function (item) {
                        tooltipDataList_1.push({
                            key: item,
                            value: tooltipData_1[item]
                        });
                    });
                }
                tooltipData_1 = tooltipDataList_1;
            }
        }
    }, []);
    var _b = (0, react_1.useState)(), graphData = _b[0], setGraphData = _b[1];
    (0, react_1.useEffect)(function () {
        if (!graphData && !ref.current) {
            return;
        }
        var cy = (0, cytoscape_1.default)({
            container: ref.current,
            elements: graphData[0],
            layout: { name: 'cose' },
            style: graphData[1]
        });
        cy.on('vclick', clickGraph);
        cy.on('tapdragover', hoverGraph);
        cy.on('tapdragout', outOfGraph);
        return function () {
            cy.destroy();
        };
    }, [clickGraph, graphData, hoverGraph, outOfGraph]);
    var fetchData = (0, react_1.useCallback)(function () {
        (0, ajax_1.getJSON)('/im_robots/knowledge_diagram', {
            knowledgeId: knowledgeId,
            maxHops: 1
        }).then(function (resp) {
            var formattedData = formatter(resp);
            setGraphData(formattedData);
        });
    }, [knowledgeId]);
    var _c = (0, react_1.useState)(false), shown = _c[0], setShown = _c[1];
    var showGraph = (0, react_1.useCallback)(function () {
        setShown(true);
        fetchData();
    }, [fetchData]);
    var onClose = (0, react_1.useCallback)(function () { return setShown(false); }, []);
    if (shown) {
        return react_1.default.createElement("div", { style: relativeStyle, className: className },
            react_1.default.createElement("div", { ref: ref, className: index_module_less_1.default['graphContainer'] }),
            react_1.default.createElement("button", { className: index_module_less_1.default['closeButton'], onClick: onClose }, "\u00D7"));
    }
    else {
        return react_1.default.createElement(button_1.default, { onClick: showGraph, className: index_module_less_1.default['linkButton'] }, "\u67E5\u770B\u56FE\u8C31");
    }
};
exports.default = KnowledgeGraph;
