"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports["default"] = void 0;

var _inheritsLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/inheritsLoose"));

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _filter = _interopRequireDefault(require("lodash-es/filter"));

var _difference = _interopRequireDefault(require("lodash-es/difference"));

var _includes = _interopRequireDefault(require("lodash-es/includes"));

var _map = _interopRequireDefault(require("lodash-es/map"));

var _reduce = _interopRequireDefault(require("lodash-es/reduce"));

var _uniq = _interopRequireDefault(require("lodash-es/uniq"));

var _isEqual = _interopRequireDefault(require("lodash-es/isEqual"));

var _differenceWith = _interopRequireDefault(require("lodash-es/differenceWith"));

var _thead = _interopRequireDefault(require("./thead"));

/**
 * columns示例
 * ```
 * [
 * {id:'col1', display: true,  name: '列1'}
 * {id:'col2', display: true,  name: '列2'}
 * {id:'col3', display: false, name: '列3'}
 * ]
 * ```
 * formatter示例
 * ```
 * {
 *     col1: function(value){
 *         return format(value)
 *     }
 * }
 * ```
 *
 * @property {array<string>} columns - 列，字符串数组
 * @property {string} translatePrefix - 多语言前缀，该前缀与columns里的每一项组成key去查找对应的翻译内容
 * @property {array} data
 * @property {object} formatter - 格式化列的数据，是个object，key为columns的项
 */
var UdeskTable = /*#__PURE__*/function (_Component) {
  (0, _inheritsLoose2["default"])(UdeskTable, _Component);

  function UdeskTable(props) {
    var _this;

    _this = _Component.call(this, props) || this;

    _this._isAllChecked = function () {
      var checkedData = _this.state.checkedData;
      var _this$props = _this.props,
          _this$props$data = _this$props.data,
          data = _this$props$data === void 0 ? [] : _this$props$data,
          isRowChecked = _this$props.isRowChecked;
      return data && data.length && data.every(function (item) {
        return isRowChecked(item, checkedData);
      });
    };

    _this._onRowCheckboxChange = function (e, rowData) {
      var _this$state = _this.state,
          checkedData = _this$state.checkedData,
          isControlledCheckedData = _this$state.isControlledCheckedData;
      var onCheckedDataChange = _this.props.onCheckedDataChange;
      var newCheckedData = [];

      if (e.target.checked) {
        newCheckedData = [].concat(checkedData, [rowData]);
      } else {
        newCheckedData = (0, _filter["default"])(checkedData, function (i) {
          return !(0, _isEqual["default"])(i, rowData);
        });
      } //受控模式下不更改state.checkedData,有props同步


      !isControlledCheckedData && _this.setState({
        checkedData: newCheckedData
      });

      if (typeof onCheckedDataChange === 'function') {
        onCheckedDataChange(newCheckedData);
      }
    };

    _this._onHeadCheckboxChange = function (e) {
      var newCheckedData = [];
      var _this$props2 = _this.props,
          _this$props2$data = _this$props2.data,
          data = _this$props2$data === void 0 ? [] : _this$props2$data,
          onCheckedDataChange = _this$props2.onCheckedDataChange;
      var _this$state2 = _this.state,
          checkedData = _this$state2.checkedData,
          isControlledCheckedData = _this$state2.isControlledCheckedData;

      if (e.target.checked) {
        newCheckedData = (0, _uniq["default"])([].concat(checkedData, data || []), _isEqual["default"]);
      } else {
        newCheckedData = (0, _differenceWith["default"])(checkedData, data || [], _isEqual["default"]);
      } //受控模式下不更改state.checkedData,有props同步


      !isControlledCheckedData && _this.setState({
        checkedData: newCheckedData
      });

      if (typeof onCheckedDataChange === 'function') {
        onCheckedDataChange(newCheckedData);
      }
    };

    _this.defaultFormatter = {
      cellphones: function cellphones(value) {
        if (Array.isArray(value)) {
          return (0, _map["default"])(value, function (i) {
            return i.content;
          }).join(',');
        }

        return value;
      },
      level: function level(value) {
        if (typeof props.t === 'function') {
          return props.t('customer.level.' + value);
        } else {
          return value;
        }
      }
    };
    var _isControlledCheckedData = false;

    if ('checkedData' in props) {
      _isControlledCheckedData = true;
    }

    _this.state = {
      checkedData: [],
      originData: _this.props.data || [],
      isControlledCheckedData: _isControlledCheckedData
    };
    return _this;
  }

  UdeskTable.getDerivedStateFromProps = function getDerivedStateFromProps(props, state) {
    var isControlledCheckedData = state.isControlledCheckedData;
    var newState = {};

    if (props.data !== state.originData) {
      if (typeof props.onCheckedDataChange === 'function' && !isControlledCheckedData) {
        props.onCheckedDataChange([]);
      }

      newState.originData = props.data || [];
      !isControlledCheckedData && (newState.checkedData = []);
    }

    if (isControlledCheckedData && props.checkedData !== state.checkedData) {
      newState.checkedData = props.checkedData || [];
    }

    if (Object.keys(newState).length > 0) {
      return newState;
    }

    return null;
  };

  var _proto = UdeskTable.prototype;

  _proto.render = function render() {
    var _this2 = this;

    var _this$props3 = this.props,
        _this$props3$t = _this$props3.t,
        t = _this$props3$t === void 0 ? function (v) {
      return v;
    } : _this$props3$t,
        className = _this$props3.className,
        _this$props3$data = _this$props3.data,
        data = _this$props3$data === void 0 ? [] : _this$props3$data,
        _this$props3$columns = _this$props3.columns,
        columns = _this$props3$columns === void 0 ? [] : _this$props3$columns,
        _this$props3$formatte = _this$props3.formatter,
        globalFormatter = _this$props3$formatte === void 0 ? this.defaultFormatter : _this$props3$formatte,
        onRowClick = _this$props3.onRowClick,
        checkbox = _this$props3.checkbox,
        dnd = _this$props3.dnd,
        onColumnPlaceChange = _this$props3.onColumnPlaceChange,
        onSort = _this$props3.onSort,
        sortingColumns = _this$props3.sortingColumns,
        isRowChecked = _this$props3.isRowChecked;
    var checkedData = this.state.checkedData;
    return /*#__PURE__*/_react["default"].createElement("table", {
      className: className
    }, /*#__PURE__*/_react["default"].createElement(_thead["default"], {
      dnd: dnd,
      columns: columns,
      onColumnPlaceChange: onColumnPlaceChange,
      checkbox: checkbox,
      checkboxChecked: this._isAllChecked(),
      onCheckboxChange: this._onHeadCheckboxChange,
      onSort: onSort,
      sortingColumns: sortingColumns,
      t: t
    }), /*#__PURE__*/_react["default"].createElement("tbody", null, (0, _map["default"])(data, function (dataItem, idx) {
      var shouldFireOnRowClick = true;
      return /*#__PURE__*/_react["default"].createElement("tr", {
        key: idx,
        onClick: function onClick() {
          if (!shouldFireOnRowClick) {
            shouldFireOnRowClick = true;
            return;
          }

          onRowClick && onRowClick(dataItem, idx);
        }
      }, checkbox && /*#__PURE__*/_react["default"].createElement("td", {
        onClick: function onClick() {
          return shouldFireOnRowClick = false;
        }
      }, /*#__PURE__*/_react["default"].createElement("input", {
        type: "checkbox",
        checked: isRowChecked(dataItem, checkedData),
        onChange: function onChange(e) {
          return _this2._onRowCheckboxChange(e, dataItem);
        }
      })), (0, _map["default"])(columns, function (column) {
        var col = column.id,
            colFormatter = column.formatter;
        var result;
        var keyPath = col.split('.');
        var value = (0, _reduce["default"])(keyPath, function (preVal, key) {
          return preVal && preVal[key];
        }, dataItem);

        if (globalFormatter[col]) {
          result = globalFormatter[col](value, dataItem);
        } else if (colFormatter) {
          result = colFormatter(value, dataItem);
        } else {
          result = value;
        }

        if (column.cellComponent) {
          return /*#__PURE__*/_react["default"].createElement(column.cellComponent, {
            key: col,
            className: column.className,
            rowData: dataItem,
            colData: column
          }, result);
        }

        return /*#__PURE__*/_react["default"].createElement("td", {
          key: col,
          className: column.className
        }, result);
      }));
    })));
  };

  return UdeskTable;
}(_react.Component);

UdeskTable.defaultProps = {
  checkbox: false,
  allChecked: false,
  onCheckedDataChange: function onCheckedDataChange() {},
  isRowChecked: function isRowChecked(rowData, checkedData) {
    return (0, _includes["default"])(checkedData, rowData);
  }
};
UdeskTable.propTypes = {
  columns: _propTypes["default"].arrayOf(_propTypes["default"].shape({
    id: _propTypes["default"].string.isRequired,
    name: _propTypes["default"].string,
    formatter: _propTypes["default"].func
  })).isRequired,
  data: _propTypes["default"].arrayOf(_propTypes["default"].object),
  checkedData: _propTypes["default"].arrayOf(_propTypes["default"].object),
  formatter: _propTypes["default"].object,
  className: _propTypes["default"].string,
  t: _propTypes["default"].func,
  onRowClick: _propTypes["default"].func,
  checkbox: _propTypes["default"].bool,
  //是否显示checkbox
  onCheckedDataChange: _propTypes["default"].func,
  isRowChecked: _propTypes["default"].func
};
var _default = UdeskTable;
exports["default"] = _default;