"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dataURItoBlob = exports.getMonitorConfig = exports.notification = exports.getImNoteReqSwitch = exports.hasImNoteRequest = exports.getEnumText = exports.loadJS = void 0;
var i18n_1 = __importDefault(require("../i18n"));
var ajax_1 = require("./ajax");
var store_1 = require("../store");
var loadFiles = [];
var state = {};
function loadJS(url, cbFn, isNotReplace) {
    if (!url)
        return;
    switch (url) {
        case "lightbox":
            url = "libs/lightbox.js";
            break;
    }
    if (!isNotReplace) {
        if (loadFiles.includes(url)) {
            return cbFn && cbFn();
        }
        loadFiles.push(url);
    }
    getScript(url, cbFn);
}
exports.loadJS = loadJS;
function getScript(source, callback) {
    var script = document.createElement("script");
    var prior = document.getElementsByTagName("script")[0];
    script.async = true;
    script.onload = script.onreadystatechange = function (_, isAbort) {
        if (isAbort ||
            !script.readyState ||
            /loaded|complete/.test(script.readyState)) {
            script.onload = script.onreadystatechange = null;
            script = undefined;
            if (!isAbort && callback)
                setTimeout(callback, 0);
        }
    };
    script.src = source;
    prior.parentNode.insertBefore(script, prior);
}
function getEnumText(enumName, key) {
    if (!enumName || !key) {
        return "";
    }
    return i18n_1.default.t("enums.".concat(enumName, ".").concat(key));
}
exports.getEnumText = getEnumText;
function hasImNoteRequest(id) {
    var imNoteFields = (store_1.store
        .getState()
        .noteTemplateConfig.noteTemplates.find(function (item) { return item.id === id && item.category === "im"; }) || {}).fields || [];
    if (imNoteFields.length === 0) {
        return false;
    }
    var hasImNoteRequest = imNoteFields.some(function (field) {
        return field.is_required;
    });
    return hasImNoteRequest;
}
exports.hasImNoteRequest = hasImNoteRequest;
function getImNoteReqSwitch() {
    var data = state.imNotReqSwitchConfig;
    if (data) {
        return new Promise(function (resolve, reject) {
            resolve(data);
        });
    }
    return (0, ajax_1.getJSON)("/im/im_setting/agent_note_fill").then(function (res) {
        if (res === void 0) { res = {}; }
        if (res.code === 1000) {
            state.imNotReqSwitchConfig = res;
        }
        return res;
    });
}
exports.getImNoteReqSwitch = getImNoteReqSwitch;
/* 桌面通知
   title:通知标题
   msg：通知具体内容
   options:{
       click():点击桌面通知事件
   }
*/
var notificationQueues = [];
function notification(title, msg, options) {
    title = title || i18n_1.default.t("page.im.imMessages.systemNotice");
    options = options || {};
    if (!window.Notification) {
        console.log("Notification Not supported！");
        return;
    }
    if (Notification.permission === "granted") {
        Notification.requestPermission(function (permission) { });
        var instance = new Notification(title, {
            body: msg,
            icon: options.img || "./notification24.png",
        });
        options.url && window.open(options.url);
        instance.onclick = function (ev) {
            options.click && options.click(ev);
            instance.close();
        };
        options.count_down &&
            setTimeout(function () {
                instance.close();
            }, options.count_down);
        notificationQueues.push(instance);
        if (notificationQueues.length > 1) {
            var instance1 = notificationQueues.shift();
            instance1.close();
        }
    }
    else {
        try {
            return Notification.requestPermission();
        }
        catch (e) {
            console.log(e);
        }
    }
}
exports.notification = notification;
function getMonitorConfig() {
    if (state.monitorConfig) {
        return new Promise(function (resolve, reject) {
            resolve(state.monitorConfig);
        });
    }
    return (0, ajax_1.getJSON)("/im/im_monitor_configs.json").then(function (res) {
        state.monitorConfig = res;
        return res;
    });
}
exports.getMonitorConfig = getMonitorConfig;
function dataURItoBlob(dataURI) {
    // convert base64 to raw binary data held in a string
    // doesn't handle URLEncoded DataURIs - see SO answer #6850276 for code that does this
    var byteString = atob(dataURI.split(",")[1]);
    // separate out the mime component
    var mimeString = dataURI.split(",")[0].split(":")[1].split(";")[0];
    // write the bytes of the string to an ArrayBuffer
    var ab = new ArrayBuffer(byteString.length);
    var ia = new Uint8Array(ab);
    for (var i = 0; i < byteString.length; i++) {
        ia[i] = byteString.charCodeAt(i);
    }
    return new Blob([ab], { type: mimeString });
}
exports.dataURItoBlob = dataURItoBlob;
