"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.customFieldValueToText = void 0;
var find_1 = __importDefault(require("lodash-es/find"));
var isArray_1 = __importDefault(require("lodash-es/isArray"));
var isNil_1 = __importDefault(require("lodash-es/isNil"));
var isString_1 = __importDefault(require("lodash-es/isString"));
var reduce_1 = __importDefault(require("lodash-es/reduce"));
var numeral_1 = __importDefault(require("numeral"));
/**
 * 将自定字段的值转换为文字，例如多选字段'1,2'转换为'选项1,选项2'
 * @param {CustomField} field - 自定义字段
 * @param {string|string[]} value - 值
 */
function customFieldValueToText(field, value) {
    if (!value) {
        return "";
    }
    var type = field.content_type;
    var valueString = (0, isString_1.default)(value) ? value : value === null || value === void 0 ? void 0 : value.join(",");
    var valueAry = (0, isArray_1.default)(value) ? value : value === null || value === void 0 ? void 0 : value.split(",");
    var options = field.select_options;
    switch (type) {
        case "droplist":
        case "checkbox":
        case "radio":
            return options
                .filter(function (i) { return valueAry.includes("".concat(i.value)); })
                .map(function (i) { return i.title; })
                .join(",");
        case "chained_droplist": {
            if (!valueString) {
                return "";
            }
            var resultAry_1 = [];
            (0, reduce_1.default)(valueAry, function (preValue, item) {
                var _a;
                var option = (0, find_1.default)(preValue, ["value", item]);
                if (option) {
                    resultAry_1.push(option.title);
                }
                return (_a = option === null || option === void 0 ? void 0 : option.subs) !== null && _a !== void 0 ? _a : [];
            }, options);
            return resultAry_1.join("/");
        }
        case "calculate":
            switch (field.calculate_result_type) {
                case "duration":
                    return (0, isNil_1.default)(valueString)
                        ? ""
                        : (0, numeral_1.default)(valueString).format("00:00:00");
                case "percentage":
                    return (0, isNil_1.default)(valueString) ? "" : (0, numeral_1.default)(valueString).format("0.00%");
                case "numeric":
                    return (0, isNil_1.default)(valueString) ? "" : Number(valueString);
                default:
                    return valueString;
            }
        default:
            return valueString;
    }
}
exports.customFieldValueToText = customFieldValueToText;
