"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDateRanges = exports.replaceLinks = exports.canPreviewByFileName = exports.getPreviewUrl = exports.stringToHex = exports.formatProps = exports.getFieldsDefaultValue = exports.formatCustomField = exports.mapCascader = exports.isSysField = exports.getFileTypeByContentType = exports.r = exports.getId = exports.removeHTMLTag = void 0;
var isFunction_1 = __importDefault(require("lodash-es/isFunction"));
var assign_1 = __importDefault(require("lodash-es/assign"));
var includes_1 = __importDefault(require("lodash-es/includes"));
function removeHTMLTag(str) {
    if (!str) {
        return '';
    }
    str = str.replace(/<\/?[^>]*>/g, ''); //去除HTML tag
    // str = str.replace(/[ | ]*\n/g, '\n'); //去除行尾空白
    //str = str.replace(/\n[\s| | ]*\r/g,'\n'); //去除多余空行
    str = str.replace(/&nbsp;/gi, ' '); //去掉&nbsp;
    return str;
}
exports.removeHTMLTag = removeHTMLTag;
function getId() {
    return Date.parse(new Date().toString()) + Math.floor(Math.random() * 1000000);
}
exports.getId = getId;
// 动态获取字数长度限制的警告信息
function r(template) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    for (var i = 0; i < args.length; i++) {
        template = template.replace(/{[^}]+}/, args[i]);
    }
    return template;
}
exports.r = r;
function getFileTypeByContentType(contentType) {
    switch (contentType) {
        case 'link':
        case 'datetime':
        case 'text':
        case 'date':
        case 'time':
        case 'numeric':
        case 'calculate':
        case 'area_text':
        case 'number':
        case 'location':
            return 'TextField';
        case 'radio':
        case 'checkbox':
        case 'radio_droplist':
        case 'checkbox_droplist':
        case 'chained_droplist':
        case 'tag_tree':
        default:
            return 'SelectField';
    }
}
exports.getFileTypeByContentType = getFileTypeByContentType;
function textValidFn(is_required, _value, field) {
    if (_value === void 0) { _value = ''; }
    var validTexts = [];
    var setting = field.setting || {};
    // 检查必填
    if (is_required && !_value) {
        validTexts.push('该字段为必填项');
    }
    // 长度校验
    if (setting.max_length && _value.length > setting.max_length) {
        validTexts.push("\u8D85\u8FC7\u6700\u5927\u957F\u5EA6\u9650\u5236(".concat(setting.max_length, ")"));
    }
    // if (_value.length > 10) {
    //     validTexts.push(`超过最大长度限制(${10})`);
    // }
    // 正则校验
    if (setting.valid_reg) {
        var reg = new RegExp(setting.valid_reg, 'ig');
        if (!reg.test(_value)) {
            validTexts.push("\u5185\u5BB9\u683C\u5F0F\u6821\u9A8C\u5931\u8D25");
        }
    }
    if (validTexts.length > 0) {
        var errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    }
    else {
        return Promise.resolve();
    }
}
function textareaValidFn(is_required, _value, field) {
    if (_value === void 0) { _value = ''; }
    var validTexts = [];
    var setting = field.setting || {};
    // 检查必填
    if (is_required && !_value) {
        validTexts.push('该字段为必填项');
    }
    // 长度校验
    if (setting.max_length && _value.length > setting.max_length) {
        validTexts.push("\u8D85\u8FC7\u6700\u5927\u957F\u5EA6\u9650\u5236(".concat(setting.max_length, ")"));
    }
    if (validTexts.length > 0) {
        var errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    }
    else {
        return Promise.resolve();
    }
}
function linkValidFn(is_required, _value, field) {
    if (_value === void 0) { _value = ''; }
    var validTexts = [];
    // 检查必填
    if (is_required && !_value) {
        validTexts.push('该字段为必填项');
    }
    if (validTexts.length > 0) {
        var errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    }
    else {
        return Promise.resolve();
    }
}
function numberValidFn(is_required, _value, field) {
    if (_value === void 0) { _value = ''; }
    var validTexts = [];
    // 检查必填
    if (is_required && !_value) {
        validTexts.push('该字段为必填项');
    }
    if (validTexts.length > 0) {
        var errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    }
    else {
        return Promise.resolve();
    }
}
function commonValidFn(rule, _value, field) {
    if (_value === void 0) { _value = ''; }
    var validTexts = [];
    // 检查必填
    if (field.isRequired && (!_value || !_value.length)) {
        validTexts.push('该字段为必填项');
    }
    if (validTexts.length > 0) {
        var errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    }
    else {
        return Promise.resolve();
    }
}
function chainedValidFn(rule, _value, field) {
    if (_value === void 0) { _value = []; }
    var validTexts = [];
    // 检查必填
    if (field.is_required && (!_value || !_value.length)) {
        validTexts.push('该字段为必填项');
    }
    /*
     *如果开启了选择最后一级的开关
     *如果填过这个字段
     *并且此字段是可编辑的
     *进行是否填写到最后一级的判断
     */
    if (_value.length && field.full_select && !field.disabled) {
        /*
         * 最后一级的数据
         * 根据选择的数据（数组），循环查找当前对应的已经选择的数据，将该层级数据的subs赋值给lastOption
         * 直到循环完毕，lastOption为下一次应该选择的数据，如果没有数据，说明最后一级已经选完了。
         */
        var lastOption_1 = field.select_options;
        _value.forEach(function (item) {
            lastOption_1.forEach(function (_item) {
                if (_item.value === item) {
                    lastOption_1 = _item.subs || null;
                }
            });
        });
        if (lastOption_1) {
            validTexts.push('必须选择最末级');
        }
    }
    if (validTexts.length > 0) {
        var errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    }
    else {
        return Promise.resolve();
    }
}
var sysFields = [
    'name',
    'nick_name',
    'email',
    'description',
    'cellphone',
    'organizations',
    'organization',
    'tags',
    'cellphones',
    'domains',
    'org_email_suffix',
];
function isSysField(type) {
    return sysFields.includes(type);
}
exports.isSysField = isSysField;
// 遍历级联字段
function mapCascader(options) {
    var result = [];
    options.forEach(function (item) {
        var obj = {
            label: item.title,
            value: item.value,
        };
        if (item.subs && item.subs.length > 0) {
            obj['children'] = mapCascader(item.subs);
        }
        result.push(obj);
    });
    return result;
}
exports.mapCascader = mapCascader;
/**
 * @param {string} fieldType  那种字段?   customer | ticket
 * @param {string} editType   什么操作?   edit | new | show
 * @param {object} fieldsAuthority    字段全选hash表
 */
function formatCustomField(_a) {
    var fields = _a.fields, fieldsAuthority = _a.fieldsAuthority, _b = _a.trigger, trigger = _b === void 0 ? 'onChange' : _b, fieldType = _a.fieldType, editType = _a.editType;
    if (!Array.isArray(fields)) {
        return [fields];
    }
    return fields
        .map(function (field) {
        if ((0, includes_1.default)(sysFields, field.name)) {
            return __assign(__assign({}, field), { trigger: trigger });
        }
        // authKey 可能的值: editable 可编辑 readable 只读 invisible 不可见 must_fill_in 必填
        var authKey = '';
        // 处理字段权限相关
        if (fieldType && editType && fieldsAuthority) {
            switch (fieldType) {
                case 'customer':
                    authKey = fieldsAuthority[field.name] ? fieldsAuthority[field.name][editType] || '' : '';
                    break;
                case 'ticket':
                    authKey = fieldsAuthority[field.name] ? fieldsAuthority[field.name][editType] || '' : '';
                    break;
            }
        }
        // 字段不可见时返回空
        if (authKey === 'invisible') {
            return '';
        }
        var setting = field.setting || {};
        var tooltipObj = setting.tooltipObj || {};
        var explainObj = setting.explain || {};
        var fieldItem = {
            label: field.title,
            layout: field.layout || '',
            tooltip: field.tooltip || '',
            tooltipShow: !!(tooltipObj.show && tooltipObj.content),
            lines: setting.lines || '',
            isRequired: field.is_required || false,
            // isRequired: authKey === 'must_fill_in' || field.is_required || false,
            field: {
                trigger: field.content_type === 'staff' ? 'onChange' : trigger,
                id: field.name,
                type: field.content_type,
                remark: field.comment || '',
                explain: explainObj.content,
                explainShow: !!(explainObj.show || explainObj.content),
                disabled: authKey === 'readable',
                isRequired: authKey === 'must_fill_in',
                options: [],
                rowsLine: 4,
            },
        };
        switch (field.content_type) {
            case 'text':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { 
                        // defaultValue: field.default_value || '',
                        placeholder: field.palceholder || '', maxLength: field.max_length || null, validReg: field.valid_reg || null, customValidFn: (0, isFunction_1.default)(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return textValidFn(fieldItem.field.isRequired, value, field);
                            } }),
                });
                break;
            case 'area_text':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { type: 'textarea', placeholder: field.palceholder || '', 
                        // defaultValue: field.default_value || '',
                        maxLength: field.max_length || null, customValidFn: (0, isFunction_1.default)(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return textareaValidFn(fieldItem.field.isRequired, value, field);
                            } }),
                });
                break;
            case 'link':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { 
                        // defaultValue: field.default_value || '',
                        placeholder: field.palceholder || '', regLink: true, customValidFn: (0, isFunction_1.default)(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return linkValidFn(fieldItem.field.isRequired, value, field);
                            } }),
                });
                break;
            case 'date':
            case 'time':
            case 'datetime':
            case 'location':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { customValidFn: (0, isFunction_1.default)(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return commonValidFn(rule, value, fieldItem.field);
                            } }),
                });
                break;
            case 'calculate':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { customValidFn: (0, isFunction_1.default)(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return commonValidFn(rule, value, fieldItem.field);
                            } }),
                });
                break;
            case 'number':
            case 'numeric':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { 
                        // defaultValue: field.default_value || '',
                        placeholder: field.palceholder || '', integerDigits: field.integer_digits || '', decimalDigits: field.decimal_digits || '', addonBefore: field.addon_before || '', addonAfter: field.addon_after || '', prefix: field.prefix || '', suffix: field.suffix || '', customValidFn: (0, isFunction_1.default)(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return numberValidFn(fieldItem.field.isRequired, value, field);
                            } }),
                });
                break;
            case 'droplist':
            case 'droplist_multi':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { trigger: 'onChange', options: field.select_options.map(function (option) {
                            return {
                                icon: option.icon || '',
                                title: option.title,
                                bgColor: option.bg_clor || '',
                                default: option.default || false,
                                value: option.value,
                            };
                        }), customValidFn: (0, isFunction_1.default)(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return commonValidFn(rule, value, fieldItem.field);
                            } }),
                });
                break;
            case 'radio':
            case 'checkbox':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { arrangement: field.arrangement || 'auto', trigger: 'onChange', options: field.select_options.map(function (option) {
                            return {
                                icon: option.icon || '',
                                title: option.title,
                                bgColor: option.bg_clor || '',
                                default: option.default || false,
                                value: option.value,
                            };
                        }), customValidFn: (0, isFunction_1.default)(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return commonValidFn(rule, value, fieldItem.field);
                            } }),
                });
                break;
            case 'chained_droplist':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { type: 'cascade', trigger: 'onChange', arrangement: field.arrangement || 'auto', viewType: field.view_type, options: mapCascader(field.select_options), customValidFn: (0, isFunction_1.default)(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                var _a;
                                return chainedValidFn(rule, value, Object.assign(field, {
                                    disabled: (_a = fieldItem === null || fieldItem === void 0 ? void 0 : fieldItem.field) === null || _a === void 0 ? void 0 : _a.disabled,
                                }));
                            } }),
                });
                break;
        }
        fieldItem = Object.assign(fieldItem, (0, isFunction_1.default)(field.convertProps) ? field.convertProps(field) : {});
        return fieldItem;
    })
        .filter(function (i) { return i; });
}
exports.formatCustomField = formatCustomField;
function getFieldsDefaultValue(_a) {
    var _b = _a.fields, fields = _b === void 0 ? [] : _b, _c = _a.values, values = _c === void 0 ? {} : _c;
    var defaultValues = {};
    fields.forEach(function (field) {
        var setting = field.setting || {};
        var selectOption = field.select_options || [];
        var selectOptionDefault = [];
        switch (field.content_type) {
            case 'text':
            case 'area_text':
            case 'link':
            case 'date':
            case 'time':
            case 'datetime':
            case 'location':
            case 'calculate':
            case 'number':
            case 'numeric':
                if (setting.default_value) {
                    defaultValues[field.name] = setting.default_value;
                }
                else {
                    defaultValues[field.name] = '';
                }
                break;
            case 'droplist':
            case 'droplist_multi':
            case 'radio':
            case 'checkbox':
                selectOptionDefault = selectOption.filter(function (option) {
                    return option.default;
                });
                if (selectOptionDefault && selectOptionDefault.length > 0) {
                    defaultValues[field.name] = selectOptionDefault.map(function (option) { return option.value; });
                }
                else {
                    defaultValues[field.name] = '';
                }
                break;
            case 'chained_droplist':
                defaultValues[field.name] = '';
                break;
        }
    });
    return Object.assign({}, defaultValues, values);
}
exports.getFieldsDefaultValue = getFieldsDefaultValue;
function formatProps(props) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3;
    var pageProps = {};
    var page = props.page || 'im';
    var commonProps = {
        page: props.page,
        lang: props.lang,
        company: props.company,
        roleCan: props.roleCan,
        isAdmin: props.isAdmin,
        user: props.user,
        admin: props.admin,
        agents: props.agents || [],
        agentGroups: props.agentGroups,
        customerFieldsv2: props.customerFieldsv2,
        organizationFieldsv2: props.organizationFieldsv2,
        noteTemplates: props.noteTemplates || [],
        imAgentNoteTemplatesSort: props.im_agent_note_templates_sort || [],
        ticketTemplates: props.ticketTemplates || [],
        customerFieldsAuthority: ((_a = props.customFieldsAuthority) === null || _a === void 0 ? void 0 : _a.customer_fields_abilities) || {},
        ticketFieldsAbilities: ((_b = props.customFieldsAuthority) === null || _b === void 0 ? void 0 : _b.ticket_fields_abilities) || {},
        tabPlugins: props.tabPlugins,
        tabsWidth: props.tabsWidth,
        getTencentMap: props.getTencentMap,
        companyConfig: props.companyConfig,
        previewUrl: props.previewUrl,
        currentTab: props.currentTab,
        agentStatus: props.agentStatus,
        pluginFilter: props.pluginFilter || [],
        customerFields: props.customerFields,
        organizationFields: props.organizationFields,
        noteFields: props.noteFields || [],
        ticketTab: props.ticketTab,
        currentUser: props.currentUser,
        smsapp: props.smsapp,
        traceOpen: props.traceOpen,
        apiHost: props.apiHost,
        kmrobot_url: props.kmrobot_url,
        vistor_server: props.vistor_server,
        fieldHooks: props.fieldHooks,
        getCompanyCategoryConfig: props.getCompanyCategoryConfig,
        platforms: props.platforms || [],
        mchatConfig: props.mchatConfig,
        modals: props.modals,
        sdk: props === null || props === void 0 ? void 0 : props.sdk,
        token: props === null || props === void 0 ? void 0 : props.token,
        spa1Path: props === null || props === void 0 ? void 0 : props.spa1Path,
        credentials: props === null || props === void 0 ? void 0 : props.credentials,
        template: props === null || props === void 0 ? void 0 : props.template,
    };
    switch (page) {
        case 'im':
            pageProps = {
                accessInfo: (_c = props.chat) === null || _c === void 0 ? void 0 : _c.accessInfo,
                agent: ((_d = props.chat) === null || _d === void 0 ? void 0 : _d.agent) || {},
                customerId: (_e = props.chat) === null || _e === void 0 ? void 0 : _e.customer_id,
                sessionKey: (_f = props.chat) === null || _f === void 0 ? void 0 : _f.session_key,
                imSubSessionId: (_g = props.chat) === null || _g === void 0 ? void 0 : _g.im_sub_session_id,
                customer: ((_h = props.chat) === null || _h === void 0 ? void 0 : _h.customer) || {},
                organizations: ((_j = props.chat) === null || _j === void 0 ? void 0 : _j.organizations) || [],
                agentConfig: props.agentConfig,
                agentTabConfig: ((_k = props === null || props === void 0 ? void 0 : props.agentConfig) === null || _k === void 0 ? void 0 : _k.agent_tab_config) || {},
                newVisit: ((_l = props.chat) === null || _l === void 0 ? void 0 : _l.newVisit) || {},
                newTicketReact: ((_m = props.chat) === null || _m === void 0 ? void 0 : _m.newTicketReact) || {},
                newRecord: ((_o = props.chat) === null || _o === void 0 ? void 0 : _o.newRecord) || {},
                imNewPlatforms: props.imNewPlatforms,
                chat: props.chat,
                platform: ((_p = props.chat) === null || _p === void 0 ? void 0 : _p.platform) || '',
                plugTab: ((_q = props.chat) === null || _q === void 0 ? void 0 : _q.plugTab) || {},
                event: props.event || {},
                ImNoteTemplates: props.ImNoteTemplates || [],
                companyCategoryConfig: props.companyCategoryConfig,
            };
            break;
        case 'mpv':
            pageProps = {
                accessInfo: (_r = props.chat) === null || _r === void 0 ? void 0 : _r.accessInfo,
                agent: ((_s = props.chat) === null || _s === void 0 ? void 0 : _s.agent) || {},
                customerId: (_t = props.chat) === null || _t === void 0 ? void 0 : _t.customer_id,
                sessionKey: (_u = props.chat) === null || _u === void 0 ? void 0 : _u.session_key,
                imSubSessionId: (_v = props.chat) === null || _v === void 0 ? void 0 : _v.im_sub_session_id,
                customer: ((_w = props.chat) === null || _w === void 0 ? void 0 : _w.customer) || {},
                organizations: ((_x = props.chat) === null || _x === void 0 ? void 0 : _x.organizations) || [],
                agentConfig: props.agentConfig,
                agentTabConfig: ((_y = props === null || props === void 0 ? void 0 : props.agentConfig) === null || _y === void 0 ? void 0 : _y.agent_tab_config) || {},
                newVisit: ((_z = props.chat) === null || _z === void 0 ? void 0 : _z.newVisit) || {},
                newTicketReact: ((_0 = props.chat) === null || _0 === void 0 ? void 0 : _0.newTicketReact) || {},
                newRecord: ((_1 = props.chat) === null || _1 === void 0 ? void 0 : _1.newRecord) || {},
                imNewPlatforms: props.imNewPlatforms,
                chat: props.chat,
                platform: ((_2 = props.chat) === null || _2 === void 0 ? void 0 : _2.platform) || '',
                plugTab: ((_3 = props.chat) === null || _3 === void 0 ? void 0 : _3.plugTab) || {},
                event: props.event || {},
                ImNoteTemplates: props.ImNoteTemplates || [],
                companyCategoryConfig: props.companyCategoryConfig,
            };
            break;
    }
    return (0, assign_1.default)(commonProps, pageProps);
}
exports.formatProps = formatProps;
function stringToHex(str) {
    var hexResult = [];
    for (var i = 0, len = str.length; i < len; i++) {
        var hex = str.charCodeAt(i).toString(16);
        hexResult.push(hex);
    }
    return hexResult.join('');
}
exports.stringToHex = stringToHex;
function getPreviewUrl(url, options) {
    if (!url) {
        return '';
    }
    var _a = options || {}, _b = _a.needEncode, needEncode = _b === void 0 ? false : _b, previewUrl = _a.previewUrl;
    if (needEncode) {
        url = encodeURIComponent(decodeURIComponent(url));
        url = url.replace(/%2F/g, '/');
    }
    var hexUrl;
    try {
        hexUrl = stringToHex(url);
    }
    catch (e) {
        console.error('preview-url-helper', e);
        return url;
    }
    return "".concat(previewUrl, "/wopi/files/").concat(hexUrl, "/documents");
}
exports.getPreviewUrl = getPreviewUrl;
/**
 * 通过文件名判断是否可以预览
 * @param {string} fileName
 * @returns {boolean}
 */
function canPreviewByFileName(fileName) {
    return /\.(doc|docx|odt|rtf|wps|ppt|pptx|odp|dp|xls|xlsx|ods|csv|et|pdf)$/.test(fileName);
}
exports.canPreviewByFileName = canPreviewByFileName;
function replaceLinks(text) {
    return (text + '').replace(/((http|https):\/\/([\w-]+\.)+[\w-:]+(\/[\w-./?;%&=#：:]*)?)/g, '<a href="$1" target="_blank">$1</a>');
}
exports.replaceLinks = replaceLinks;
var react_1 = require("react");
var moment_1 = __importDefault(require("moment"));
var react_i18next_1 = require("react-i18next");
function useDateRanges() {
    var t = (0, react_i18next_1.useTranslation)().t;
    return (0, react_1.useMemo)(function () {
        var _a;
        return (_a = {},
            _a[t('page.im.record.today')] = [(0, moment_1.default)(), (0, moment_1.default)()],
            _a[t('page.im.record.yesterday')] = [(0, moment_1.default)().subtract(1, 'days'), (0, moment_1.default)().subtract(1, 'days')],
            _a[t('page.im.record.recentSevenDays')] = [(0, moment_1.default)().subtract(6, 'days'), (0, moment_1.default)()],
            _a[t('page.im.record.recentThirtyDays')] = [(0, moment_1.default)().subtract(29, 'days'), (0, moment_1.default)()],
            _a[t('page.im.record.currentMonth')] = [(0, moment_1.default)().startOf('month'), (0, moment_1.default)().endOf('month')],
            _a[t('page.im.record.lastMonth')] = [
                (0, moment_1.default)().subtract(1, 'month').startOf('month'),
                (0, moment_1.default)().subtract(1, 'month').endOf('month'),
            ],
            _a[t('page.im.record.currentYear')] = [(0, moment_1.default)().startOf('year').startOf('day'), (0, moment_1.default)()],
            _a[t('page.im.record.lastYear')] = [
                (0, moment_1.default)().subtract(1, 'year').startOf('year').startOf('day'),
                (0, moment_1.default)().subtract(1, 'year').endOf('year').endOf('day'),
            ],
            _a[t('page.im.record.recentOneYear')] = [(0, moment_1.default)().subtract(1, 'year'), (0, moment_1.default)()],
            _a);
    }, [t]);
}
exports.useDateRanges = useDateRanges;
