import React, { useMemo } from 'react';
import { useTranslation } from 'react-i18next';
import unformat from 'numeral';
import { getStanardFileSizeSuffixes } from './utilities';
import last from 'lodash-es/last';
var linkReg = /^<a .*>.*<\/a>?$/;
var emojiReg = /\[emoji([^\].]+)]/g;
export var KNOWLEDGE_TYPE;
(function (KNOWLEDGE_TYPE) {
    KNOWLEDGE_TYPE["ATTACHMENT"] = "attachment";
    KNOWLEDGE_TYPE["QUESTION"] = "question";
    KNOWLEDGE_TYPE["USEFUL_LINK"] = "useful_link";
    // USEFUL_EXPRESSION = 'useful_expression'
})(KNOWLEDGE_TYPE || (KNOWLEDGE_TYPE = {}));
var AddToKnowledgeMenuItem = function (_a) {
    var content = _a.content, msg = _a.msg, onClick = _a.onClick;
    var t = useTranslation().t;
    var msgType = msg.type;
    var knowledgeData = useMemo(function () {
        var _a;
        var type = msgType;
        var isText = type === 'message' || type === 'info' || false;
        var isRich = type === 'rich' || false;
        var isImage = type === 'image' || false;
        var isFile = type === 'file' || false;
        if (isText) {
            var msgContentMeta = msg.data.content;
            var isLink = linkReg.test(msgContentMeta);
            var isHasEmoji = emojiReg.test(msgContentMeta);
            var filterEmoji = msgContentMeta.replace(emojiReg, '').trim() || '';
            if (isLink) {
                return { content: content, type: KNOWLEDGE_TYPE.USEFUL_LINK };
            }
            else if (isHasEmoji && filterEmoji.length > 0) {
                return { content: filterEmoji, isRich: false, type: KNOWLEDGE_TYPE.QUESTION };
            }
            else if (!isLink && !isHasEmoji) {
                return { content: content, isRich: false, type: KNOWLEDGE_TYPE.QUESTION };
            }
        }
        else if (isRich) {
            return { content: content, isRich: true, type: KNOWLEDGE_TYPE.QUESTION };
        }
        else if (isImage) {
            var fileName = msg.filename || '';
            var sizeShow = msg.filesize || '';
            var fileSize = unformat(getStanardFileSizeSuffixes(sizeShow)).value() || '';
            var fileType = type || '';
            var remoteFileName = last(content.split('/'));
            return {
                file_source: 'qiniu/udeskim',
                file_name: fileName,
                file_size: fileSize,
                size_show: sizeShow,
                file_type: fileType,
                remote_file_name: remoteFileName,
                type: KNOWLEDGE_TYPE.ATTACHMENT
            };
        }
        else if (isFile) {
            var fileName = msg.filename || '';
            var sizeShow = msg.filesize || '';
            var fileSize = unformat(getStanardFileSizeSuffixes(sizeShow)).value() || '';
            var fileType = msg.filetype || '';
            var remoteFileName = msg.randomFileName; //TODO 每次刷新页面后，此属性为 undefined
            var content_1 = (_a = msg.data) === null || _a === void 0 ? void 0 : _a.content;
            var fileDirName = '';
            if (content_1) {
                var strArr = (content_1 + '').split('/');
                fileDirName = strArr[strArr.length - 1];
            }
            return {
                file_source: 'qiniu/udeskim',
                file_name: fileName,
                file_size: fileSize,
                size_show: sizeShow,
                file_type: fileType,
                file_dir_name: fileDirName,
                remote_file_name: remoteFileName,
                type: KNOWLEDGE_TYPE.ATTACHMENT
            };
        }
        else {
            return null;
        }
    }, [content, msg.data.content, msg.filename, msg.filesize, msg.filetype, msg.randomFileName, msgType]);
    if (!knowledgeData) {
        return null;
    }
    return React.createElement("button", { onClick: function () { return onClick === null || onClick === void 0 ? void 0 : onClick(knowledgeData); } }, t('page.im.log.addToKnowledgeBase'));
};
export default AddToKnowledgeMenuItem;
