import { __assign, __rest, __spreadArray } from "tslib";
import React, { useEffect, useState, useCallback, useContext, useMemo, useRef } from 'react';
import { Popover, TreeSelect, Spin, Empty, Button } from 'udesk-ui';
import VirtualList from 'rc-virtual-list';
import ScopeSwitch from '../scope-switch';
import { useTranslation } from '@udesk-webapp/locale';
import { createUseSelectedCatalog, hit, RepositoryWidgetContext, useRequestData, useSafeScope, } from '../utilities';
import css from '../style.module.scss';
import { canPreviewByFileName, getPreviewUrl } from '../utils';
import SearchInput from '../search-input';
import classNames from 'classnames';
import CatalogSelect from '../catalog-select';
var TreeNode = TreeSelect.TreeNode;
var ContainerHeight = 650;
var AttachmentInner = function (_a) {
    var data = _a.data, onScrollToBottom = _a.onScrollToBottom;
    var t = useTranslation().t;
    var wrapHeightRef = useRef();
    var _b = useContext(RepositoryWidgetContext), isWorkbench = _b.isWorkbench, onSend = _b.onSend, previewUrl = _b.previewUrl;
    useEffect(function () {
        var _a;
        var wrapDom = (_a = document.getElementsByClassName('repositoryTabsWrap')) === null || _a === void 0 ? void 0 : _a[0];
        if (wrapDom) { // 避免小数 出现外层滚动条
            wrapHeightRef.current = wrapDom.clientHeight ? (wrapDom.clientHeight - 173) : ContainerHeight;
        }
    }, []);
    var onClick = function (data) {
        onSend === null || onSend === void 0 ? void 0 : onSend('attachment', data);
        hit(data.id, 'KnowledgeAttachment');
    };
    var previewUrlFun = function (data) {
        window.open(getPreviewUrl(data.file_url, { previewUrl: previewUrl }));
    };
    var generateContent = function (data) {
        var content = [
            React.createElement("a", { key: '1', href: data.file_url, rel: 'noopener noreferrer', target: '_blank', className: css['attachment-link'] }, data.file_name),
            React.createElement("div", { key: '2', className: css.nodeOperations },
                canPreviewByFileName(data.file_name) ? (React.createElement("div", { className: css.lineBtns },
                    React.createElement(Button, { type: 'link', size: 'small', className: css['line-btn-item'], onClick: function () { previewUrlFun(data); } }, t('label.preview')),
                    ' ',
                    isWorkbench ? React.createElement("span", { className: css.line }) : null)) : null,
                isWorkbench ? React.createElement(Button, { type: 'link', size: 'small', className: css['line-btn-item'], onClick: function () { onClick(data); } }, t('label.send')) : null),
        ];
        if (data.file_type.startsWith('image') || /png|jpg|jpeg|gif/.test(data.file_type)) {
            return (React.createElement("div", { className: css['repository-item-wrap'] },
                React.createElement(Popover, { autoAdjustOverflow: false, content: React.createElement("img", { src: data.file_url, alt: data.file_name, style: { width: '100%' } }), placement: 'leftBottom', overlayStyle: { maxWidth: '300px', maxHeight: '300px' } },
                    React.createElement("div", { className: "".concat(css['repository-item'], " ").concat(css['attachment-item']) }, content))));
        }
        else {
            return (React.createElement("div", { className: "".concat(css['repository-item-wrap'], " ").concat(css['repository-item'], " ").concat(css['attachment-item']) }, content));
        }
    };
    var onScroll = function (e) {
        // 滚动到了底部
        if (e.currentTarget.scrollHeight - e.currentTarget.scrollTop === wrapHeightRef.current) {
            onScrollToBottom();
        }
    };
    return (React.createElement("div", { id: 'knowledgeTreeWrapper', className: classNames(css['repository-virtualList-wrapper']) },
        React.createElement(VirtualList, { data: data, height: wrapHeightRef.current, itemHeight: 45, itemKey: 'id', onScroll: onScroll }, function (item) { return generateContent(item); })));
};
export var useSelectedCatalog = createUseSelectedCatalog();
var Attachment = function (_a) {
    var _b, _c, _d, _e, _f;
    var refreshKnowledge = _a.refreshKnowledge, roleCan = _a.roleCan, onSearch = _a.onSearch, mineChats = _a.mineChats, agentConfig = _a.agentConfig, dialogueTagData = _a.dialogueTagData, stashKey = _a.stashKey, props = __rest(_a, ["refreshKnowledge", "roleCan", "onSearch", "mineChats", "agentConfig", "dialogueTagData", "stashKey"]);
    var query = props.query;
    var chat = (mineChats === null || mineChats === void 0 ? void 0 : mineChats.find(function (chat) { return chat.selected; })) || {};
    var _g = useContext(RepositoryWidgetContext), activeScope = _g.activeScope, onScopeChange = _g.onScopeChange;
    var _scope = useSafeScope(activeScope);
    var _h = useSelectedCatalog(_scope, stashKey), selectedCatalog = _h[0], setSelectedCatalog = _h[1];
    var tagsIdRef = useRef(-1);
    var _j = useState(false), tagCatalog = _j[0], setTagCatalog = _j[1];
    var _k = useState(1), page = _k[0], setPage = _k[1];
    var _l = useState([]), usefulAttachment = _l[0], setUsefulAttachment = _l[1];
    var pagingRef = useRef({ current_page: 0, total_pages: 0 });
    var t = useTranslation().t;
    var params = useMemo(function () {
        var q = query.trim();
        var catalogId = selectedCatalog;
        if (selectedCatalog === -1 || q) { // 查询是在全部分类中筛选
            catalogId = 'all';
        }
        return {
            catalog_id: catalogId,
            scope: _scope,
            per_page: 20,
            page: page,
            query: q
        };
    }, [page, _scope, query, selectedCatalog]);
    var _m = useRequestData('attachment', params, refreshKnowledge, roleCan), loading = _m.loading, setLoading = _m.setLoading, catalogs = _m.catalogs, data = _m.data, hasPermission = _m.hasPermission;
    useEffect(function () {
        setPage(1);
        setTagCatalog(false);
        setLoading(true);
    }, [_scope]);
    var tagNameArray = useMemo(function () {
        var _a, _b, _c, _d;
        if (((_a = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _a === void 0 ? void 0 : _a.enable) &&
            (dialogueTagData === null || dialogueTagData === void 0 ? void 0 : dialogueTagData.length) &&
            ((_b = chat.dialogueTag) === null || _b === void 0 ? void 0 : _b.length)) {
            if (((_c = agentConfig.session_tags_config) === null || _c === void 0 ? void 0 : _c.mode_tags) === 'first') {
                return chat.dialogueTag[0].map(function (i) { var _a; return (_a = dialogueTagData === null || dialogueTagData === void 0 ? void 0 : dialogueTagData.find(function (j) { return j.id === i; })) === null || _a === void 0 ? void 0 : _a.name; }).filter(function (i) { return !!i; });
            }
            else if (((_d = agentConfig.session_tags_config) === null || _d === void 0 ? void 0 : _d.mode_tags) === 'new') {
                return chat.dialogueTag[chat.dialogueTag.length - 1].map(function (i) { var _a; return (_a = dialogueTagData === null || dialogueTagData === void 0 ? void 0 : dialogueTagData.find(function (j) { return j.id === i; })) === null || _a === void 0 ? void 0 : _a.name; }).filter(function (i) { return !!i; });
            }
        }
        return [];
    }, [agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config, chat.dialogueTag, dialogueTagData]);
    var tagName = useMemo(function () {
        var _a, _b;
        if (((_a = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _a === void 0 ? void 0 : _a.enable)
            && ((_b = agentConfig.session_tags_config) === null || _b === void 0 ? void 0 : _b.show_mode) === 'keyword'
            && tagNameArray.length > 0) {
            return tagNameArray.join(' ');
        }
        return '';
    }, [(_b = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _b === void 0 ? void 0 : _b.enable, (_c = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _c === void 0 ? void 0 : _c.show_mode, tagNameArray]);
    var tagsName = useCallback(function (list, tagList, num) {
        list.children.forEach(function (items) {
            var _a, _b;
            if (items.name === tagList[num] && ((_a = items.children) === null || _a === void 0 ? void 0 : _a.length) && tagList[num + 1]) {
                num += 1;
                tagsName(items, tagList, num);
            }
            else if ((items.name === tagList[num] && !((_b = items.children) === null || _b === void 0 ? void 0 : _b.length)) ||
                (items.name === tagList[num] && !tagList[num + 1])) {
                tagsIdRef.current = items.id;
            }
            else if (!tagsIdRef.current) {
                tagsIdRef.current = list.id;
            }
        });
        return tagsIdRef.current;
    }, []);
    var defaultTags = useCallback(function () {
        var _a, _b, _c, _d, _e;
        var tagList = [];
        if (((_a = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _a === void 0 ? void 0 : _a.enable) &&
            ((_b = agentConfig.session_tags_config) === null || _b === void 0 ? void 0 : _b.show_mode) === 'type' &&
            !tagCatalog &&
            (dialogueTagData === null || dialogueTagData === void 0 ? void 0 : dialogueTagData.length) &&
            ((_c = chat.dialogueTag) === null || _c === void 0 ? void 0 : _c.length)) {
            if (((_d = agentConfig.session_tags_config) === null || _d === void 0 ? void 0 : _d.mode_tags) === 'first') {
                chat.dialogueTag[0].forEach(function (item) {
                    dialogueTagData.forEach(function (items) {
                        if (item === items.id) {
                            tagList.push(items.name);
                        }
                    });
                });
                var num_1 = 0;
                catalogs.forEach(function (item) {
                    var _a;
                    if (item.name === tagList[num_1] && tagList.length > 1 && ((_a = item.children) === null || _a === void 0 ? void 0 : _a.length)) {
                        num_1 += 1;
                        tagsName(item, tagList, num_1);
                    }
                    else if (item.name === tagList[tagList.length - 1]) {
                        tagsIdRef.current = item.id;
                    }
                });
            }
            else if (((_e = agentConfig.session_tags_config) === null || _e === void 0 ? void 0 : _e.mode_tags) === 'new') {
                chat.dialogueTag[chat.dialogueTag.length - 1].forEach(function (item) {
                    dialogueTagData.forEach(function (items) {
                        if (item === items.id) {
                            tagList.push(items.name);
                        }
                    });
                });
                var num_2 = 0;
                catalogs.forEach(function (item) {
                    var _a;
                    if (item.name === tagList[num_2] && tagList.length > 1 && ((_a = item.children) === null || _a === void 0 ? void 0 : _a.length)) {
                        num_2 += 1;
                        tagsName(item, tagList, num_2);
                    }
                    else if (item.name === tagList[tagList.length - 1]) {
                        tagsIdRef.current = item.id;
                    }
                });
            }
            setSelectedCatalog(tagsIdRef.current);
            return tagsIdRef.current;
        }
        else {
            return -1;
        }
    }, [(_d = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _d === void 0 ? void 0 : _d.enable, (_e = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _e === void 0 ? void 0 : _e.mode_tags, (_f = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _f === void 0 ? void 0 : _f.show_mode, catalogs, chat.dialogueTag, dialogueTagData, tagCatalog, tagsName, setSelectedCatalog]);
    useEffect(function () {
        defaultTags();
    }, [defaultTags]);
    useEffect(function () {
        if (loading)
            return;
        setPage(1);
        setUsefulAttachment([]);
    }, [params.query]);
    useEffect(function () {
        setUsefulAttachment([]);
    }, [params.catalog_id, params.scope]);
    useEffect(function () {
        var _a;
        if (data === null || data === void 0 ? void 0 : data.knowledge_attachments) {
            var attachments_1 = (_a = data === null || data === void 0 ? void 0 : data.knowledge_attachments) === null || _a === void 0 ? void 0 : _a.map(function (a) { return a.knowledge_attachment; });
            // @ts-ignore
            setUsefulAttachment(function (prevState) { return __spreadArray(__spreadArray([], prevState, true), attachments_1, true); });
            pagingRef.current = data.meta;
        }
    }, [data]);
    var pageChangeCallback = function () {
        var _a;
        if (loading)
            return;
        if (page >= ((_a = pagingRef.current) === null || _a === void 0 ? void 0 : _a.total_pages))
            return;
        setLoading(true);
        setPage(page + 1);
    };
    var changeSelectedCatalogs = useCallback(function (value, label, extra) {
        setTagCatalog(true);
        setSelectedCatalog(value);
        setLoading(true);
        setPage(1);
        tagsIdRef.current = -1;
    }, []);
    useEffect(function () {
        tagsIdRef.current = -1;
    }, [catalogs]);
    return React.createElement(React.Fragment, null,
        React.createElement(ScopeSwitch, { value: _scope, onChange: onScopeChange }),
        React.createElement(SearchInput, { onSearch: onSearch, defaultValue: tagName ? tagName : query }),
        (query && query.trim()) ? null :
            React.createElement(CatalogSelect, { data: catalogs, prependTreeNodes: [React.createElement(TreeNode, { value: -1, key: '-1', title: t('page.im.km.question.all_catalog') })], value: selectedCatalog, onChange: changeSelectedCatalogs }),
        (function () {
            if (!usefulAttachment) {
                return null;
            }
            if (!hasPermission) {
                return React.createElement("div", { className: classNames(css['no-permission']) }, "\u60A8\u6682\u65E0\u6743\u9650\u67E5\u770B\u6B64\u5185\u5BB9");
            }
            if (!loading && (usefulAttachment && !usefulAttachment.length)) {
                return React.createElement(Empty, { className: "h-full w-hull flex justify-center items-center flex-col" });
            }
            return (React.createElement(React.Fragment, null,
                React.createElement(AttachmentInner, __assign({}, props, { data: usefulAttachment, onScrollToBottom: pageChangeCallback })),
                loading ? React.createElement("div", { className: classNames(css['repository-loading']) },
                    React.createElement(Spin, { className: 'w-full text-center' })) : null));
        })());
};
export default Attachment;
