import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import PropTypes from 'prop-types';
import React, { Fragment } from 'react';
import forEach from 'lodash-es/forEach';
import includes from 'lodash-es/includes';
import isFunction from 'lodash-es/isFunction';
import join from 'lodash-es/join';
import map from 'lodash-es/map';
import split from 'lodash-es/split';
import { library } from '@fortawesome/fontawesome-svg-core';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faTimesCircle } from '@fortawesome/free-solid-svg-icons';
import { jsx as ___EmotionJSX } from "@emotion/react";
library.add(faTimesCircle);

var UdeskTimePicker = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(UdeskTimePicker, _React$Component);

  UdeskTimePicker.getDerivedStateFromProps = function getDerivedStateFromProps(props, state) {
    if (props.time !== state.beforeTime) {
      return {
        HH: split(props.time, ':', 3)[0] || '',
        mm: split(props.time, ':', 3)[1] || '',
        ss: split(props.time, ':', 3)[2] || '',
        beforeTime: props.time
      };
    }

    return null;
  };

  function UdeskTimePicker(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;

    _defineProperty(_assertThisInitialized(_this), "state", {
      HH: split(_this.props.time, ':', 3)[0] || '',
      mm: split(_this.props.time, ':', 3)[1] || '',
      ss: split(_this.props.time, ':', 3)[2] || '',
      beforeTime: _this.props.time,
      showPicker: false
    });

    _defineProperty(_assertThisInitialized(_this), "_ref", /*#__PURE__*/React.createRef());

    _this._leadingZero = _this._leadingZero.bind(_assertThisInitialized(_this));
    _this._range = _this._range.bind(_assertThisInitialized(_this));
    _this._times = _this._times.bind(_assertThisInitialized(_this));
    _this.onSelectTime = _this.onSelectTime.bind(_assertThisInitialized(_this));
    _this.showUdeskTimePicker = _this.showUdeskTimePicker.bind(_assertThisInitialized(_this));
    _this._sendOutSideClick = _this._sendOutSideClick.bind(_assertThisInitialized(_this));
    return _this;
  }

  var _proto = UdeskTimePicker.prototype;

  _proto.componentDidMount = function componentDidMount() {
    document.addEventListener('mousedown', this._sendOutSideClick);
    var time = this.props.time;
    this.setState({
      time: time
    });
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    document.removeEventListener('mousedown', this._sendOutSideClick);
  };

  _proto._sendOutSideClick = function _sendOutSideClick(e) {
    //eslint-disable-next-line no-console
    if (!contains(this._ref.current, e.target)) {
      this.setState({
        showPicker: false
      });
    }
  };

  _proto._leadingZero = function _leadingZero(num, size) {
    var str = String(num);

    while (str.length < (size || 2)) {
      str = '0' + num;
    }

    return str;
  };

  _proto._range = function _range(start, end) {
    var range = [],
        i;

    for (i = start; i < end; ++i) {
      range.push(this._leadingZero(i));
    }

    return range;
  };

  _proto._times = function _times() {
    var _this2 = this;

    var times = {};
    var format = this.props.format;
    forEach(split(format, ':', 3), function (i) {
      var values = i === 'HH' ? _this2._range(0, 24) : _this2._range(0, 60);
      times[i] = {
        values: values
      };
    });
    return times;
  };

  _proto.onSelectTime = function onSelectTime(unit, value) {
    var _this$setState;

    this.setState((_this$setState = {}, _this$setState[unit] = value, _this$setState));
  };

  _proto.showUdeskTimePicker = function showUdeskTimePicker(e) {
    if (this._doNotOpenPicker) {
      this._doNotOpenPicker = false;
      return;
    }

    this.setState({
      showPicker: true
    });
  };

  _proto._onInputChange = function _onInputChange(timeUnit, value) {
    var _this$setState2;

    this.setState((_this$setState2 = {}, _this$setState2[timeUnit] = value, _this$setState2));
  };

  _proto._fireOnChange = function _fireOnChange() {
    var onChange = this.props.onChange;

    if (!isFunction(onChange)) {
      return;
    }

    onChange(this._getFullValue());
  };

  _proto._getFullValue = function _getFullValue() {
    var _this3 = this;

    if (!this.state.HH && !this.state.mm && !this.state.ss) {
      return '';
    }

    var regex = {
      HH: /^[0-1][0-9]|2[0-3]$/,
      mm: /^[0-5][0-9]$/,
      ss: /^[0-5][0-9]$/
    };
    var valueAry = [];
    forEach(this.props.format.split(':'), function (timeUnit) {
      var value = _this3.state[timeUnit];

      if (regex[timeUnit].test(value)) {
        //满足条件直接使用
        valueAry.push(value);
      } else if (/^\d$/.test(value)) {
        var _this3$setState;

        //只输入一位数字，前面补零
        valueAry.push('0' + value);

        _this3.setState((_this3$setState = {}, _this3$setState[timeUnit] = '0' + value, _this3$setState));
      } else {
        var _this3$setState2;

        //全不满足设为00
        valueAry.push('00');

        _this3.setState((_this3$setState2 = {}, _this3$setState2[timeUnit] = '00', _this3$setState2));
      }
    });
    return join(valueAry, ':');
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps, prevState, snapshot) {
    if (!this.state.showPicker && prevState.showPicker && this._getFullValue() !== this.state.beforeTime) {
      this._fireOnChange();
    }
  };

  _proto.render = function render() {
    var _this4 = this;

    var className = this.props.className;
    var _this$state = this.state,
        showPicker = _this$state.showPicker,
        HH = _this$state.HH,
        mm = _this$state.mm,
        ss = _this$state.ss;
    var onChange = this.props.onChange;

    var times = this._times();

    return ___EmotionJSX("div", {
      className: 'udesk-time-picker ' + className,
      ref: this._ref
    }, ___EmotionJSX("div", {
      className: "udesk-time-picker-input-box",
      onClick: this.showUdeskTimePicker
    }, (HH || mm || ss) && map(times, function (v, i) {
      return ___EmotionJSX(Fragment, {
        key: i
      }, includes(['mm', 'ss'], i) ? ___EmotionJSX("span", null, ":") : void 0, ___EmotionJSX("input", {
        type: "text",
        className: "udesk-time-picker-input",
        value: _this4.state[i],
        onChange: function onChange(e) {
          return _this4._onInputChange(i, e.target.value);
        }
      }));
    }), (HH || mm || ss) && ___EmotionJSX("span", {
      className: "clear-btn",
      onClick: function onClick() {
        _this4._doNotOpenPicker = true;

        _this4.setState({
          HH: '',
          mm: '',
          ss: ''
        });

        if (isFunction(onChange) && !_this4.state.showPicker) {
          onChange('');
        }
      }
    }, ___EmotionJSX(FontAwesomeIcon, {
      icon: "times-circle"
    }))), showPicker && ___EmotionJSX("div", {
      className: "udesk-time-picker-panel"
    }, ___EmotionJSX(UlItems, {
      times: times,
      that: this
    })));
  };

  return UdeskTimePicker;
}(React.Component);

function UlItems(props) {
  return map(props.times, function (time, unit) {
    return ___EmotionJSX("div", {
      className: "udesk-time-picker-panel-select",
      key: unit
    }, ___EmotionJSX("ul", {
      className: "udesk-time-picker-panel-select-ul"
    }, ___EmotionJSX(LiItems, {
      key: unit,
      time: time,
      onClick: function onClick(selectedValue) {
        return props.that.onSelectTime(unit, selectedValue);
      }
    })));
  });
}

function LiItems(props) {
  return props.time.values.map(function (item, index) {
    return ___EmotionJSX("li", {
      className: "udesk-time-picker-panel-select-ul-li",
      key: index,
      onClick: function onClick() {
        return props.onClick(item);
      }
    }, item);
  });
}

function contains(root, n) {
  var node = n;

  while (node) {
    if (node === root) {
      return true;
    }

    node = node.parentNode;
  }

  return false;
}

UdeskTimePicker.propTypes = {
  format: PropTypes.oneOf(['HH', 'HH:mm', 'HH:mm:ss']),
  time: function time(props, propName, componentName) {
    if (!/^(([0-1][0-9]|2[0-3])(:[0-5][0-9](:[0-5][0-9])?)?)?$/.test(props[propName])) {
      return new Error('Invalid prop `' + propName + '` supplied to' + ' `' + componentName + '`. Validation failed.');
    }
  },
  onChange: PropTypes.func
};
UdeskTimePicker.defaultProps = {
  time: '',
  format: 'HH:mm',
  onChange: null
};
export default UdeskTimePicker;