import React, { useCallback, useEffect, useMemo, useState } from "react";
import DropDown, { Menu, Trigger } from "../drop-down";
import { Input, Tree, Empty } from "udesk-ui";
import apis from "../../utils/apis";
import { useSelector } from "../../store/store-hook";
import { useTranslation } from "react-i18next";
import { CareDown } from "../icon/icon";
import style from "./style.module.scss";
// const { TreeNode} = Tree;
var Search = Input.Search;
var TreeNode = function (props) {
    return (React.createElement("span", { key: props.key, title: props.title },
        props.count != null && React.createElement("b", { className: style["nodeNum"] }, props.count),
        props.title));
};
var AgentGroupSelect = function (props) {
    var t = useTranslation().t;
    var TriggerComponent = props.triggerComponent, _a = props.triggerText, triggerText = _a === void 0 ? t("agent") : _a, _b = props.placeholder, placeholder = _b === void 0 ? "" : _b, _c = props.showNum, showNum = _c === void 0 ? true : _c, _d = props.cacheEnable, cacheEnable = _d === void 0 ? true : _d, _e = props.allColleagueMonitorEnabled, allColleagueMonitorEnabled = _e === void 0 ? true : _e, onChange = props.onChange, onAgentLoad = props.onAgentLoad, _f = props.value, value = _f === void 0 ? [] : _f;
    var _g = useState(""), keyWord = _g[0], setKeyword = _g[1];
    var _h = useState([]), agents = _h[0], setAgents = _h[1];
    var _j = useState(false), dropDownVisible = _j[0], setDropDownVisible = _j[1];
    // let [checkStrictly, setCheckStrictly] = useState<boolean>(false);
    var _k = useState([]), checkedNode = _k[0], setCheckedNode = _k[1];
    var currentAgents = useSelector(function (state) { return state.agents; });
    var currentUser = useSelector(function (state) { return state.currentUser; });
    var groupAgents = useMemo(function () {
        return formGroupAgents(agents, currentAgents, currentUser === null || currentUser === void 0 ? void 0 : currentUser.user_groups, allColleagueMonitorEnabled);
    }, [
        agents,
        allColleagueMonitorEnabled,
        currentAgents,
        currentUser === null || currentUser === void 0 ? void 0 : currentUser.user_groups,
    ]);
    var filteredgroupAgents = useMemo(function () {
        var filteredAgents = agents.filter(function (agent) {
            var agentName = agent === null || agent === void 0 ? void 0 : agent.name;
            if (agentName) {
                if (keyWord) {
                    return agentName.indexOf(keyWord) !== -1;
                }
                else {
                    return true;
                }
            }
            else {
                return false;
            }
        });
        return formGroupAgents(filteredAgents, currentAgents, currentUser === null || currentUser === void 0 ? void 0 : currentUser.user_groups, allColleagueMonitorEnabled);
    }, [
        keyWord,
        agents,
        currentAgents,
        currentUser === null || currentUser === void 0 ? void 0 : currentUser.user_groups,
        allColleagueMonitorEnabled,
    ]);
    useEffect(function () {
        apis.getMonitorUserList().then(function (resp) {
            if (resp === null || resp === void 0 ? void 0 : resp.agents) {
                var respAgents = resp.agents;
                setAgents(respAgents);
            }
        });
    }, []);
    useEffect(function () {
        setCheckedNode(value);
    }, [value]);
    var onSearchChange = useCallback(function (e) {
        var value = e.target.value;
        // if (value) {
        //     setCheckStrictly(true);
        // } else {
        //     setCheckStrictly(false);
        // }
        setKeyword(value);
    }, []);
    var onVisibleChange = useCallback(function (value) {
        setDropDownVisible(value);
        if (!value) {
            onChange && onChange(checkedNode);
            setKeyword("");
        }
    }, [checkedNode, onChange]);
    var onChecked = useCallback(function (value) {
        value = updateParentValue(groupAgents, value);
        setCheckedNode(value);
    }, [groupAgents]);
    return (React.createElement(DropDown, { className: style["agent-group-select"], visible: dropDownVisible, onVisibleChange: onVisibleChange },
        React.createElement(Trigger, null,
            !!TriggerComponent && React.createElement(TriggerComponent, null),
            !TriggerComponent && (React.createElement("span", { className: style["trigger-span"] },
                triggerText,
                showNum && (agents === null || agents === void 0 ? void 0 : agents.length))),
            React.createElement(CareDown, null)),
        React.createElement(Menu, { className: style["menu"] },
            React.createElement(Menu.Item, null,
                React.createElement(Search, { value: keyWord, className: style["search"], placeholder: placeholder, onChange: onSearchChange })),
            React.createElement(Menu.Item, null,
                React.createElement("div", { className: style["content"] },
                    filteredgroupAgents.length && (React.createElement(Tree, { checkable: true, onCheck: onChecked, defaultExpandAll: true, treeData: filteredgroupAgents, checkedKeys: checkedNode })),
                    !filteredgroupAgents.length && (React.createElement(Empty, { image: Empty.PRESENTED_IMAGE_SIMPLE })))))));
};
export default AgentGroupSelect;
function formGroupAgents(agents, currentAgents, currentUserGroups, allColleagueMonitorEnabled) {
    agents = agents || [];
    return agents.reduce(function (preValue, agent) {
        var currentAgent = currentAgents.find(function (cAgent) { return agent.id == cAgent.id; });
        if (!currentAgent) {
            return preValue;
        }
        else {
            var currentAgentUserGroups_1 = currentAgent.user_groups;
            if (!allColleagueMonitorEnabled) {
                currentAgentUserGroups_1 = currentUserGroups.filter(function (user) {
                    return currentAgentUserGroups_1.some(function (group) {
                        return group.id === user.id;
                    });
                });
            }
            return addGroupAgent(preValue, currentAgentUserGroups_1, agent);
        }
    }, []);
}
function addGroupAgent(groupAgents, groups, agent) {
    if (groups) {
        groups.forEach(function (group) {
            var _a;
            var groupAgent = groupAgents.find(function (ga) { return (ga === null || ga === void 0 ? void 0 : ga.key) == (group === null || group === void 0 ? void 0 : group.id); });
            var newAgent = {
                key: "".concat(group === null || group === void 0 ? void 0 : group.id, "-").concat(agent === null || agent === void 0 ? void 0 : agent.id),
                title: (React.createElement(TreeNode, { count: agent === null || agent === void 0 ? void 0 : agent.customers_count, key: "".concat(group === null || group === void 0 ? void 0 : group.id, "-").concat(agent === null || agent === void 0 ? void 0 : agent.id), title: agent === null || agent === void 0 ? void 0 : agent.name })),
                customersCount: agent.customers_count,
            };
            if (groupAgent) {
                if (groupAgent.children) {
                    groupAgent.children.push(newAgent);
                }
                else {
                    groupAgent.children = [newAgent];
                }
                //计算总客户数
                var totalCustomerNum = (_a = groupAgent.children) === null || _a === void 0 ? void 0 : _a.reduce(function (total, agent) {
                    return (total += agent.customersCount || 0);
                }, 0);
                groupAgent.title = (React.createElement(TreeNode, { count: totalCustomerNum, key: groupAgent === null || groupAgent === void 0 ? void 0 : groupAgent.key, title: groupAgent === null || groupAgent === void 0 ? void 0 : groupAgent.name }));
            }
            else {
                groupAgents.push({
                    key: group === null || group === void 0 ? void 0 : group.id,
                    title: (React.createElement(TreeNode, { count: agent === null || agent === void 0 ? void 0 : agent.customers_count, key: group === null || group === void 0 ? void 0 : group.id, title: group === null || group === void 0 ? void 0 : group.name })),
                    name: group === null || group === void 0 ? void 0 : group.name,
                    children: [newAgent],
                });
            }
        });
    }
    return groupAgents;
}
//如果子节点没有都选中,父节点不选中
function updateParentValue(treeData, value) {
    var result = [];
    result = value.filter(function (_v) {
        if (/^\d+$/.test(_v)) {
            if (treeData) {
                var node = treeData.find(function (data) {
                    return data.key === _v;
                });
                if (node && node.children) {
                    return node.children.every(function (children) {
                        return value.indexOf(children.key) !== -1;
                    });
                }
            }
            else {
                return true;
            }
        }
        else {
            return true;
        }
    });
    return result;
}
