import React, { memo, useMemo, useState } from 'react';
import { Button, Modal, Popover } from 'udesk-ui';
import DOMPurify from 'dompurify';
import style from './index.module.less';
import classNames from 'classnames';
import { useTranslation } from 'react-i18next';
import { humanFileSize } from '@udesk-webapp/utils';
import { getFileType } from '../utilities';
var EmailType = memo(function (_a) {
    var msg = _a.msg;
    var subject = msg.subject, from = msg.from, to = msg.to, cc = msg.cc, thumbnail = msg.thumbnail, content = msg.content, data = msg.data;
    var t = useTranslation().t;
    var imgRex = /<img.*?\/?>/g;
    var thumbnailFormat = DOMPurify.sanitize(thumbnail);
    var contentFormat = DOMPurify.sanitize(content);
    var _b = useState(false), visible = _b[0], setVisible = _b[1];
    var handleUrl = function (url) {
        return url.replace(/\+/g, '%2B');
    };
    var imgHTML = useMemo(function () {
        // 获取缩略文本中第一张图片的HTML
        var matches = thumbnailFormat.match(imgRex);
        return matches && matches.length ? matches[0] : '';
    }, [thumbnailFormat]);
    var text = useMemo(function () {
        return thumbnailFormat.replace(imgRex, '').replace(/&nbsp;/gi, ' ');
    }, [thumbnailFormat]);
    var attachments = useMemo(function () {
        if (!data.attachments || !data.attachments.length)
            return null;
        return data.attachments.map(function (attach, index) {
            return React.createElement("div", { className: style['attach'], key: index },
                React.createElement("img", { className: style['file-img'], src: getFileType(".".concat(attach.suffix)) }),
                React.createElement(Popover, { content: attach.name },
                    React.createElement("a", { className: style['file-name'], href: handleUrl(attach.url), target: '_blank', rel: "noopener noreferrer", download: attach.name }, attach.name)),
                React.createElement("span", { className: style['file-size'] },
                    "(",
                    humanFileSize(attach.size),
                    ")"));
        });
    }, [data.attachments]);
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { className: style['email-wrapper'] },
            React.createElement("div", { className: style['thumbnail-content'] },
                React.createElement("div", { className: style['info'] },
                    React.createElement("h4", { className: style['ellipsis'] },
                        t('label.subject'),
                        ": ",
                        subject),
                    React.createElement("div", { className: style['ellipsis'] },
                        t('components.email_form.from'),
                        ": ",
                        from),
                    React.createElement("div", { className: style['ellipsis'] },
                        t('components.email_form.to'),
                        ": ",
                        to),
                    React.createElement("div", { className: style['ellipsis'] },
                        t('components.email_form.cc'),
                        ": ",
                        cc),
                    React.createElement("div", { className: classNames(style['text'], style['ellipsis']) }, text)),
                React.createElement("div", { className: style['image'], dangerouslySetInnerHTML: { __html: imgHTML } })),
            React.createElement("div", { className: style['attach-list'] }, attachments),
            React.createElement(Button, { className: style['detail-btn'], onClick: function () { return setVisible(true); } }, t('emailMethod.emailDetail'))),
        React.createElement(Modal, { title: t('emailMethod.emailDetail'), visible: visible, width: 1600, footer: null, onCancel: function () { return setVisible(false); }, className: style['detail-modal'] },
            React.createElement("div", { className: classNames(style['email-wrapper'], style['detail']) },
                React.createElement("h4", null,
                    t('label.subject'),
                    ": ",
                    subject),
                React.createElement("div", null,
                    t('components.email_form.from'),
                    ": ",
                    from),
                React.createElement("div", null,
                    t('components.email_form.to'),
                    ": ",
                    to),
                React.createElement("div", null,
                    t('components.email_form.cc'),
                    ": ",
                    cc),
                React.createElement("div", { className: style['content'], dangerouslySetInnerHTML: { __html: contentFormat } }),
                React.createElement("div", { className: style['attach-list'] }, attachments)))));
});
export default EmailType;
