/* eslint-disable jsx-a11y/anchor-has-content */
import React, { useCallback, useMemo } from 'react';
import { update_sdk_url } from '../utilities';
import { useTranslation } from 'react-i18next';
import style from './index.module.less';
import pdf from '../images/pdf02.png';
import word from '../images/word02.png';
import excel from '../images/excel02.png';
import ppt from '../images/ppt02.png';
import txt from '../images/txt02.png';
import unknow from '../images/unknow02.png';
var stringToHex = (function (str) {
    var hexResult = [];
    for (var i = 0; i < str.length; i++) {
        var hex = str.charCodeAt(i).toString(16);
        hexResult.push(hex);
    }
    return hexResult.join('');
});
var FileType = React.memo(function (_a) {
    var msg = _a.msg, previewURL = _a.previewURL, allowPreview = _a.allowPreview;
    //多语言配置
    var t = useTranslation().t;
    var canVideoExts = ['mp4', 'mov', 'MOV', 'm4v', 'M4V'];
    var getPreviewUrl = useCallback(function (url, encodeUrl) {
        if (!url) {
            return '';
        }
        if (encodeUrl) {
            url = encodeURIComponent(decodeURIComponent(url));
            url = url.replace(/%2F/g, '/');
        }
        var hexUrl;
        try {
            hexUrl = stringToHex(url);
        }
        catch (e) {
            console.error('preview-url-helper', e);
            return url;
        }
        return "".concat(previewURL, "/wopi/files/").concat(hexUrl, "/documents");
    }, [previewURL]);
    var _href = useMemo(function () {
        var content = msg.content, href = '', reg2 = /\.((doc)|(docx)|(odt)|(rtf)|(wps)|(ppt)|(pptx)|(odp)|(dp)|(xls)|(xlsx)|(ods)|(csv)|(et)|(pdf))$/g;
        if (content.match(reg2) && allowPreview) {
            href = getPreviewUrl(content, true);
        }
        return href;
    }, [allowPreview, getPreviewUrl, msg.content]);
    var handlerUrl = useCallback(function (url) {
        return url.replace(/\+/g, '%2B');
    }, []);
    var _type = useMemo(function () {
        var _a;
        var list = (_a = msg === null || msg === void 0 ? void 0 : msg.filename) === null || _a === void 0 ? void 0 : _a.split('.');
        var filenameType = list[list.length - 1];
        var type = '.' + (msg.fileext || filenameType);
        if (/\.(pdf)/.test(type)) {
            type = pdf;
        }
        else if (/\.(doc|docx|odt)/.test(type)) {
            type = word;
        }
        else if (/\.(xls|xlsx|csv)/.test(type)) {
            type = excel;
        }
        else if (/\.(ppt|pptx)/.test(type)) {
            type = ppt;
        }
        else if (/\.(txt)/.test(type)) {
            type = txt;
        }
        else {
            type = unknow;
        }
        return type;
    }, [msg.type]);
    return (React.createElement(React.Fragment, null,
        canVideoExts.indexOf(msg.fileext) > -1 &&
            React.createElement("video", { src: update_sdk_url(msg.content, msg), style: { maxWidth: '100%', maxHeight: '480px' }, controls: true },
                t('page.im.log.videoAddress'),
                "\uFF1A",
                msg.content),
        canVideoExts.indexOf(msg.content.fileext) === -1 &&
            React.createElement("div", { className: style['file-wrapper'] },
                React.createElement("div", null,
                    React.createElement("img", { className: style['file-img'], src: _type, alt: msg.filename })),
                React.createElement("div", { className: style['file-div'] },
                    React.createElement("a", { className: style['filename'], href: update_sdk_url(msg.is_receive ? msg.content : handlerUrl(msg.content), msg), target: '_blank', rel: "noopener noreferrer", download: msg.filename }, msg.filename),
                    React.createElement("div", { className: style['filesize'] },
                        React.createElement("div", { className: style['file-size'], style: { marginRight: '6px' } }, msg.filesize),
                        _href && React.createElement("a", { href: _href, style: { marginRight: '6px' }, target: '_blank', rel: "noopener noreferrer" }, t('page.im.log.preview')),
                        React.createElement("a", { href: update_sdk_url(msg.is_receive ? msg.content : handlerUrl(msg.content), msg), target: '_blank', rel: "noopener noreferrer", download: msg.filename }, t('page.im.log.download')))))));
});
export default FileType;
