import React, { useCallback, useEffect, useRef } from 'react';
import css from './index.module.scss';
import { useSearchList } from '../hooks/useCatalog';
import { Button, Empty, Spin } from 'udesk-ui';
import { useTranslation } from '@udesk-webapp/locale';
var ListTemplate = function (props) {
    var _a;
    var knowledgeType = props.knowledgeType, query = props.query, scope = props.scope, ItemNode = props.itemNode;
    var pageNumberRef = useRef(1);
    var _b = useSearchList(knowledgeType), searchList = _b.searchList, listData = _b.listData, listLoading = _b.listLoading, pagingRef = _b.pagingRef;
    var t = useTranslation().t;
    var listParams = useCallback(function (page) {
        var p = {
            scope: scope,
            per_page: 20,
            page: page,
            query: query,
        };
        if (knowledgeType === 'attachment') {
            p.catalog_id = 'all';
        }
        else if (knowledgeType === 'question') {
            p.current_works = true;
        }
        return p;
    }, [query, scope, knowledgeType]);
    useEffect(function () {
        pageNumberRef.current = 1;
        searchList(listParams(1));
    }, [listParams]);
    var hasMore = pageNumberRef.current < ((_a = pagingRef.current) === null || _a === void 0 ? void 0 : _a.total_pages);
    var loadMore = function () {
        pageNumberRef.current += 1;
        searchList(listParams(pageNumberRef.current));
    };
    var onScroll = function (e) {
        // 滚动到了底部
        var obj = e.currentTarget;
        if (obj.scrollTop === (obj.scrollHeight - obj.offsetHeight)) {
            if (listLoading)
                return;
            if (!hasMore)
                return;
            loadMore();
        }
    };
    var loadMoreBtn = hasMore ? React.createElement("div", { className: css['flex'] },
        React.createElement(Button, { className: css['m-auto'], type: 'link', onClick: loadMore }, t('label.load_more'))) : null;
    if (!listLoading && listData && !listData.length) {
        return React.createElement(Empty, { className: 'h-full w-hull flex justify-center items-center flex-col' });
    }
    return (React.createElement("div", { onScroll: onScroll, className: css['repository-virtualList-wrapper'] },
        listData.map(function (item, index) {
            return React.createElement(ItemNode, { item: item, key: index });
        }),
        listLoading ? (React.createElement("div", { className: css['repository-loading'] },
            React.createElement(Spin, { className: 'w-full text-center' }))) : loadMoreBtn));
};
export default React.memo(ListTemplate);
