import React, { useCallback, useContext, useMemo } from "react";
import { Button, Popover, Tree } from "udesk-ui";
import LinkOutlined from "@ant-design/icons/LinkOutlined";
import { Spin } from "udesk-ui/es";
import ScopeSwitch from "../scope-switch";
import { useTranslation } from "react-i18next";
import { hit, RepositoryWidgetContext, useCatalogMap, useExpandedKeys, useRequestData, useSafeScope, } from "../utilities";
import css from "../style.module.scss";
import { canPreviewByFileName, getPreviewUrl } from "../../../utils";
import SearchInput from "../search-input";
var TreeNodeTitle = function (_a) {
    var data = _a.data;
    var t = useTranslation().t;
    var _b = useContext(RepositoryWidgetContext), onSend = _b.onSend, previewUrl = _b.previewUrl;
    var onClick = useCallback(function () {
        onSend === null || onSend === void 0 ? void 0 : onSend("attachment", data);
        hit(data.id, "KnowledgeAttachment");
    }, [data, onSend]);
    var content = useMemo(function () { return [
        React.createElement(LinkOutlined, { key: "0" }),
        React.createElement("div", { key: "1", style: { flex: "1 1" } },
            React.createElement("a", { href: data.file_url, rel: "noopener noreferrer", target: "_blank" }, data.file_name)),
        React.createElement("div", { key: "2", className: css.nodeOperations },
            canPreviewByFileName(data.file_name) ? (React.createElement(Button, { type: "link", size: "small", target: "_blank", href: getPreviewUrl(data.file_url, { previewUrl: previewUrl }) }, t("label.preview"))) : null,
            React.createElement(Button, { type: "link", size: "small", onClick: onClick }, t("label.send"))),
    ]; }, [data.file_name, data.file_url, onClick, previewUrl, t]);
    if (data.file_type.startsWith("image") ||
        /png|jpg|jpeg|gif/.test(data.file_type)) {
        return (React.createElement(Popover, { autoAdjustOverflow: false, content: React.createElement("img", { src: data.file_url, alt: data.file_name, style: { width: "100%" } }), placement: "left", overlayStyle: { maxWidth: "300px", maxHeight: "300px" } },
            React.createElement("div", { style: { display: "flex", flex: "1 0", alignItems: "center" } }, content)));
    }
    else {
        return React.createElement(React.Fragment, null, content);
    }
};
function buildTreeData(catalogs, catalogMap, t, parentKey) {
    return catalogs.map(function (i) {
        var key = parentKey ? "".concat(parentKey, "-").concat(i.id) : i.id + "";
        return {
            key: key,
            title: i.name,
            selectable: false,
            className: css["treeNode"],
            children: (function () {
                if (i.children) {
                    return buildTreeData(i.children, catalogMap, t, key);
                }
                else {
                    return (catalogMap[i.id] || []).map(function (j) {
                        return {
                            key: "".concat(key, "-content").concat(j.knowledge_attachment.id),
                            className: css["treeNode"],
                            title: React.createElement(TreeNodeTitle, { data: j.knowledge_attachment }),
                        };
                    });
                }
            })(),
        };
    });
}
var Attachment = function (_a) {
    var _b;
    var query = _a.query, onSearch = _a.onSearch;
    var t = useTranslation().t;
    var _c = useContext(RepositoryWidgetContext), expandedKeys = _c.expandedKeys, onExpand = _c.onExpand, activeScope = _c.activeScope, onScopeChange = _c.onScopeChange;
    var _scope = useSafeScope(activeScope);
    var params = useMemo(function () {
        return {
            scope: _scope,
            per_page: 1000,
            catalog_id: "all",
            query: query,
        };
    }, [_scope, query]);
    var _d = useRequestData("attachment", params), loading = _d.loading, catalogs = _d.catalogs, data = _d.data;
    var catalogMap = useCatalogMap((_b = data === null || data === void 0 ? void 0 : data.knowledge_attachments) !== null && _b !== void 0 ? _b : []);
    var _expandedKeys = useExpandedKeys(catalogMap);
    var treeData = useMemo(function () {
        if (!data || !data.knowledge_attachments) {
            return [];
        }
        if (query) {
            return data.knowledge_attachments.map(function (_a) {
                var knowledge_attachment = _a.knowledge_attachment;
                return {
                    key: knowledge_attachment.id,
                    className: css["treeNode"],
                    title: React.createElement(TreeNodeTitle, { data: knowledge_attachment }),
                };
            });
        }
        else {
            return buildTreeData(catalogs, catalogMap, t);
        }
    }, [catalogMap, catalogs, data, query, t]);
    var mainContent = useMemo(function () {
        if (loading) {
            return React.createElement(Spin, null);
        }
        else if (treeData && treeData.length) {
            return (React.createElement(Tree, { blockNode: true, selectable: false, expandedKeys: _expandedKeys, onExpand: onExpand, showIcon: true, treeData: treeData }));
        }
        else {
            return (React.createElement("p", null, t("page.im.km.no_result", { name: "$t(label.useful_link)" })));
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [expandedKeys, loading, onExpand, t, treeData]);
    return (React.createElement(React.Fragment, null,
        React.createElement(ScopeSwitch, { value: _scope, onChange: onScopeChange }),
        React.createElement(SearchInput, { onSearch: onSearch, defaultValue: query }),
        mainContent));
};
export default Attachment;
