var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { useCallback, useEffect, useMemo, useRef, useState, } from "react";
import css from "./style.module.scss";
import Tabs from "udesk-ui/es/tabs";
import { useTranslation } from "react-i18next";
import Km from "./km";
import UsefulExpressionTabPane from "./useful_expression";
import UsefulLinkTabPane from "./useful_link";
import QuestionTabPane from "./question";
import AttachmentTabPane from "./attachment";
import { RepositoryWidgetContext, useSelectionCache, useTabSetting, } from "./utilities";
import Spin from "udesk-ui/es/spin";
import ErrorBoundary from "../simple-error-boundary";
import { csApiGetJSON } from "../../utils/csApi";
var TabPane = Tabs.TabPane;
var tabComponentMap = {
    useful_expression: UsefulExpressionTabPane,
    useful_link: UsefulLinkTabPane,
    question: QuestionTabPane,
    attachment: AttachmentTabPane,
};
var RepositoryWidget = function (_a) {
    var companyId = _a.companyId, email = _a.email, _b = _a.subdomain, subdomain = _b === void 0 ? "useful_expression" : _b, onSpaceChange = _a.onSpaceChange, onCatalogChange = _a.onCatalogChange, onUse = _a.onUse, allowSendQuestion = _a.allowSendQuestion, showRobotQuestionTab = _a.showRobotQuestionTab, onSend = _a.onSend, kmApiBase = _a.kmApiBase, selectionStateKey = _a.selectionStateKey, showPublicScope = _a.showPublicScope, previewUrl = _a.previewUrl, customer = _a.customer, ticket = _a.ticket, agent = _a.agent, ticketCustomFields = _a.ticketCustomFields, customerCustomFields = _a.customerCustomFields;
    var _c = useState(), kmSettings = _c[0], setKmSettings = _c[1];
    var t = useTranslation().t;
    var _d = useTabSetting(), saveTab = _d.saveTab, tabSetting = _d.tabSetting, loading = _d.loading, saveSpace = _d.saveSpace, saveScope = _d.saveScope, saveCatalog = _d.saveCatalog;
    var _e = useSelectionCache(selectionStateKey, useMemo(function () { return ({
        defaultTab: tabSetting === null || tabSetting === void 0 ? void 0 : tabSetting.activeTab,
        defaultScope: tabSetting === null || tabSetting === void 0 ? void 0 : tabSetting.activeScope,
    }); }, [tabSetting === null || tabSetting === void 0 ? void 0 : tabSetting.activeScope, tabSetting === null || tabSetting === void 0 ? void 0 : tabSetting.activeTab])), activeTab = _e.activeTab, setActiveTab = _e.setActiveTab, query = _e.query, setQuery = _e.setQuery, activeScope = _e.activeScope, setActiveScope = _e.setActiveScope, expandedKeys = _e.expandedKeys, setExpandedKeys = _e.setExpandedKeys;
    useEffect(function () {
        csApiGetJSON("/cs/ts/api/knowledge_manager_settings").then(function (res) { return res.data; }).then(function (res) {
            setKmSettings(res.knowledge_manager_setting);
        });
    }, []);
    var onTabChange = useCallback(function (tabKey) {
        saveTab(tabKey);
        setActiveTab(tabKey);
    }, [saveTab, setActiveTab]);
    var tabs = useMemo(function () { return ["useful_expression", "useful_link", "question", "attachment"]; }, []);
    var onScopeChange = useCallback(function (_scope) {
        saveScope(_scope);
        setActiveScope(_scope);
    }, [saveScope, setActiveScope]);
    var contextValue = useMemo(function () { return ({
        allowSendQuestion: allowSendQuestion,
        onUse: onUse,
        showRobotQuestionTab: showRobotQuestionTab,
        onSend: onSend,
        kmApiBase: kmApiBase,
        onExpand: setExpandedKeys,
        expandedKeys: expandedKeys,
        activeScope: activeScope,
        onScopeChange: onScopeChange,
        activeCatalog: tabSetting === null || tabSetting === void 0 ? void 0 : tabSetting.activeCatalog,
        onCatalogChange: saveCatalog,
        activeSpace: tabSetting === null || tabSetting === void 0 ? void 0 : tabSetting.activeSpace,
        onSpaceChange: saveSpace,
        showPublicScope: showPublicScope,
        previewUrl: previewUrl,
        customer: customer,
        ticket: ticket,
        agent: agent,
        ticketCustomFields: ticketCustomFields,
        customerCustomFields: customerCustomFields,
    }); }, [
        activeScope,
        allowSendQuestion,
        expandedKeys,
        kmApiBase,
        setExpandedKeys,
        onScopeChange,
        onSend,
        onUse,
        saveCatalog,
        saveSpace,
        showPublicScope,
        showRobotQuestionTab,
        tabSetting === null || tabSetting === void 0 ? void 0 : tabSetting.activeCatalog,
        tabSetting === null || tabSetting === void 0 ? void 0 : tabSetting.activeSpace,
        previewUrl,
        customer,
        ticket,
        agent,
        ticketCustomFields,
        customerCustomFields,
    ]);
    var ref = useRef(null);
    if (loading || !kmSettings) {
        return React.createElement(Spin, null);
    }
    return (React.createElement(ErrorBoundary, null,
        React.createElement(RepositoryWidgetContext.Provider, { value: contextValue },
            React.createElement("div", { ref: ref, className: css.container },
                React.createElement("div", { className: css.kmContent },
                    React.createElement(Tabs, { activeKey: activeTab, onChange: onTabChange, size: "small", animated: false, tabBarGutter: 0, destroyInactiveTabPane: true },
                        tabs.map(function (i) {
                            var TabComponent = tabComponentMap[i];
                            return (React.createElement(TabPane, { tab: t("page.im.km.".concat(i, ".title")), key: i },
                                React.createElement(ErrorBoundary, null,
                                    React.createElement(TabComponent, { query: query, onSearch: setQuery }))));
                        }),
                        (kmSettings === null || kmSettings === void 0 ? void 0 : kmSettings.name) && (React.createElement(TabPane, { tab: kmSettings.name, key: "km" },
                            React.createElement(ErrorBoundary, null,
                                React.createElement(Km, __assign({}, kmSettings, { companyId: companyId, email: email, subdomain: subdomain, query: query, onSearch: setQuery })))))))))));
};
export default RepositoryWidget;
