var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React, { useCallback, useContext, useEffect, useState } from "react";
import { Button, List, Popover, Spin } from "udesk-ui";
import { useTranslation } from "react-i18next";
import FileTextTwoTone from "@ant-design/icons/FileTextTwoTone";
import axios from "axios";
import DOMPurify from "dompurify";
import { articleToCommonQuestion, RepositoryWidgetContext } from "../utilities";
function getArticle(kmBase, params) {
    if (!kmBase) {
        return Promise.reject("km host is empty");
    }
    return axios
        .get("".concat(kmBase, "/articles/").concat(params.id), {
        params: {
            companyId: params.companyId,
            subdomain: params.subdomain,
            email: params.email,
            limitSpaceIds: params.limitSpaceIds,
            enablePrivate: params.enablePrivate,
            passport: params.passport,
        },
    })
        .then(function (res) {
        if (res.data.code === 200) {
            return res.data.data;
        }
        throw new Error(res.data);
    });
}
var useArticle = function (host, _a) {
    var companyId = _a.companyId, subdomain = _a.subdomain, email = _a.email, limitSpaceIds = _a.limitSpaceIds, enablePrivate = _a.enablePrivate, passport = _a.passport, articleId = _a.articleId;
    var _b = useState(false), loading = _b[0], setLoading = _b[1];
    var _c = useState(), article = _c[0], setArticle = _c[1];
    useEffect(function () {
        setLoading(true);
        getArticle(host, {
            companyId: companyId,
            subdomain: subdomain,
            email: email,
            limitSpaceIds: limitSpaceIds,
            enablePrivate: enablePrivate,
            passport: passport,
            id: articleId,
        })
            .then(function (res) {
            setArticle(res);
        })
            .finally(function () { return setLoading(false); });
    }, [
        companyId,
        subdomain,
        email,
        limitSpaceIds,
        enablePrivate,
        passport,
        articleId,
        host,
    ]);
    return [article, loading];
};
var PopoverContent = function (_a) {
    var data = _a.data, companyId = _a.companyId, subdomain = _a.subdomain, email = _a.email, limitSpaceIds = _a.limitSpaceIds, enablePrivate = _a.enablePrivate, passport = _a.passport;
    var kmApiBase = useContext(RepositoryWidgetContext).kmApiBase;
    var _b = useArticle(kmApiBase || "", {
        companyId: companyId,
        subdomain: subdomain,
        email: email,
        limitSpaceIds: limitSpaceIds,
        enablePrivate: enablePrivate,
        passport: passport,
        articleId: data.id,
    }), article = _b[0], loading = _b[1];
    if (loading) {
        return React.createElement(Spin, null);
    }
    if (article) {
        return (React.createElement(React.Fragment, null,
            React.createElement("h4", null, article.title),
            React.createElement("div", { dangerouslySetInnerHTML: {
                    __html: DOMPurify.sanitize(article.contentBody),
                } })));
    }
    return null;
};
var ArticleItem = function (_a) {
    var data = _a.data, props = __rest(_a, ["data"]);
    var t = useTranslation().t;
    var _b = useState(false), loading = _b[0], setLoading = _b[1];
    var _c = useContext(RepositoryWidgetContext), kmApiBase = _c.kmApiBase, onUse = _c.onUse;
    var onUseClick = useCallback(function () {
        if (loading) {
            return;
        }
        setLoading(true);
        getArticle(kmApiBase || "", {
            companyId: props.companyId,
            subdomain: props.subdomain,
            email: props.email,
            limitSpaceIds: props.limitSpaceIds,
            enablePrivate: props.enablePrivate,
            passport: props.passport,
            id: data.id,
        })
            .then(function (res) { return onUse === null || onUse === void 0 ? void 0 : onUse("km", articleToCommonQuestion(res)); })
            .finally(function () { return setLoading(false); });
    }, [
        data.id,
        kmApiBase,
        loading,
        onUse,
        props.companyId,
        props.email,
        props.enablePrivate,
        props.limitSpaceIds,
        props.passport,
        props.subdomain,
    ]);
    return (React.createElement(List.Item, { actions: [
            React.createElement(Button, { type: "link", onClick: onUseClick }, t("label.use")),
        ] },
        React.createElement(Popover, { content: React.createElement(PopoverContent, __assign({ data: data }, props)), placement: "left" },
            React.createElement("div", null,
                React.createElement(FileTextTwoTone, null),
                data.title))));
};
export default ArticleItem;
