var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import Viewer from "react-viewer";
import React, { useCallback, useContext, useEffect, useMemo, useRef, useState, } from "react";
import css from "../../style.module.scss";
import Button from "udesk-ui/es/button";
import { useTranslation } from "react-i18next";
import classNames from "classnames";
import createDOMPurify from "dompurify";
import useEvent from "react-use/esm/useEvent";
import { Spin } from "udesk-ui";
import Highlight from "../../hightlight";
import { RepositoryWidgetContext, makeQuestionData } from "../../utilities";
import { useSelector } from "../../../../store/store-hook";
var DOMPurify = createDOMPurify(window);
var QuestionItem = function (_a) {
    var data = _a.data, getImages = _a.getImages;
    var initData = useSelector(function (state) { return state.initData; });
    var chat = initData.chat;
    var t = useTranslation().t;
    var _b = useContext(RepositoryWidgetContext), allowSendQuestion = _b.allowSendQuestion, onUse = _b.onUse, onSend = _b.onSend;
    var _c = useState(false), showExpandButton = _c[0], setShowExpandButton = _c[1];
    var _d = useState(true), isExpand = _d[0], setIsExpand = _d[1];
    var contentStyle = useMemo(function () {
        if (isExpand) {
            return {};
        }
        else {
            return { height: "22px", overflowY: "hidden" };
        }
    }, [isExpand]);
    var contentRef = useRef(null);
    useEffect(function () {
        if (contentRef.current) {
            var imageElements = contentRef.current.querySelectorAll("img");
            var images_1 = [];
            imageElements.forEach(function (i) { return images_1.push({ src: i.src }); });
            getImages(images_1);
            if (contentRef.current.clientHeight > 40 || imageElements.length > 0) {
                setShowExpandButton(true);
                setIsExpand(false);
            }
        }
    }, [getImages]);
    var toggleExpand = useCallback(function () {
        setIsExpand(!isExpand);
    }, [isExpand]);
    var _onUse = useCallback(function () {
        onUse === null || onUse === void 0 ? void 0 : onUse("question", data);
    }, [data, onUse]);
    var onSendClick = useCallback(function () {
        var url = window.location.protocol + "//" + window.location.host + "/question/";
        var _data = makeQuestionData({
            content: data.content,
            content_type: data.content_type,
            title: data.title,
            id: parseInt(data.id),
        }, chat.platform, "", url);
        data.url = _data.url;
        onSend === null || onSend === void 0 ? void 0 : onSend("question", data);
    }, [chat, data, onSend]);
    return (React.createElement("div", { className: css["question-item"] },
        React.createElement("div", { className: classNames(css["align-justify"]) },
            React.createElement(Highlight, { className: css["word-break"], style: { color: "#177BBB" }, content: data.title }),
            React.createElement("div", { style: { flexShrink: 0 } },
                showExpandButton && (React.createElement(Button, { type: "link", size: "small", onClick: toggleExpand }, t(isExpand ? "label.collapse" : "label.expand"))),
                allowSendQuestion ? (React.createElement(Button, { type: "link", size: "small", onClick: onSendClick }, t("label.send"))) : null,
                React.createElement(Button, { type: "link", size: "small", onClick: _onUse }, t("label.use")))),
        React.createElement("div", { ref: contentRef, style: contentStyle, className: classNames(css.highlight, css["word-break"]), dangerouslySetInnerHTML: {
                __html: DOMPurify.sanitize(data.content_type === "text" ? data.content_text : data.content),
            } })));
};
var QuestionList = function (_a) {
    var data = _a.data, onScrollToBottom = _a.onScrollToBottom, loading = _a.loading;
    var _b = useState(false), imageViewerVisible = _b[0], setImageViewerVisible = _b[1];
    var _c = useState([]), images = _c[0], setImages = _c[1];
    var _d = useState(0), activeIndex = _d[0], setActiveIndex = _d[1];
    var dataWrapperRef = useRef(null);
    var onClick = useCallback(function (e) {
        if (e.target.tagName === "IMG") {
            setActiveIndex(images.findIndex(function (i) { return i.src === e.target.src; }));
            setImageViewerVisible(true);
        }
    }, [images]);
    useEvent("click", onClick, document.getElementById("react-question-common-list"));
    useEffect(function () {
        if (dataWrapperRef.current) {
            var scrollHandle_1 = function (e) {
                var element = e.currentTarget;
                if (!element) {
                    return;
                }
                if (element.scrollHeight - element.scrollTop === element.clientHeight) {
                    onScrollToBottom();
                }
            };
            dataWrapperRef.current.addEventListener("scroll", scrollHandle_1);
            return function () { var _a; return (_a = dataWrapperRef === null || dataWrapperRef === void 0 ? void 0 : dataWrapperRef.current) === null || _a === void 0 ? void 0 : _a.removeEventListener("scroll", scrollHandle_1); };
        }
    }, [onScrollToBottom]);
    var getImages = useCallback(function (i) { return setImages(function (prevState) { return __spreadArray(__spreadArray([], prevState, true), i, true); }); }, []);
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { ref: dataWrapperRef, className: css["question-list"], id: "react-question-common-list" },
            data.map(function (i) { return (React.createElement(QuestionItem, { data: i, key: i.id, getImages: getImages })); }),
            loading ? React.createElement(Spin, null) : null),
        React.createElement(Viewer, { activeIndex: activeIndex < 0 ? 0 : activeIndex, visible: imageViewerVisible, onClose: function () { return setImageViewerVisible(false); }, images: images })));
};
export default QuestionList;
