import React, { useCallback, useContext, useMemo } from "react";
import { Button, Tree } from "udesk-ui";
import Typography from "udesk-ui/es/typography";
import LinkOutlined from "@ant-design/icons/LinkOutlined";
import { Spin } from "udesk-ui/es";
import ScopeSwitch from "../scope-switch";
import { useTranslation } from "react-i18next";
import { hit, RepositoryWidgetContext, useCatalogMap, useExpandedKeys, useRequestData, useSafeScope, } from "../utilities";
import css from "../style.module.scss";
import { MiniProgram } from "../../icon/icon";
import classNames from "classnames";
import DOMPurify from "dompurify";
import FileTextTwoTone from "@ant-design/icons/FileTextTwoTone";
import SearchInput from "../../repository-widget/search-input";
var Text = Typography.Text;
function buildTreeData(catalogs, catalogMap, t, parentKey) {
    return catalogs.map(function (i) {
        var key = parentKey ? "".concat(parentKey, "-").concat(i.id) : i.id + "";
        return {
            key: key,
            title: i.name,
            selectable: false,
            className: css["treeNode"],
            children: (function () {
                if (i.children) {
                    return buildTreeData(i.children, catalogMap, t, key);
                }
                else {
                    return (catalogMap[i.id] || []).map(function (j) {
                        return {
                            key: "".concat(key, "-content").concat(j.id),
                            className: css["treeNode"],
                            originData: j,
                            title: (React.createElement(React.Fragment, null,
                                React.createElement(Text, null, j.name),
                                j.mode === "mini" ? (React.createElement("div", { className: css.nodeOperations },
                                    React.createElement(Button, { type: "link", size: "small" }, t("label.send")))) : null)),
                            icon: j.mode === "mini" ? React.createElement(MiniProgram, null) : React.createElement(LinkOutlined, null),
                        };
                    });
                }
            })(),
        };
    });
}
var UsefulLink = function (_a) {
    var _b;
    var query = _a.query, onSearch = _a.onSearch;
    var _c = useContext(RepositoryWidgetContext), onUse = _c.onUse, onExpand = _c.onExpand, expandedKeys = _c.expandedKeys, activeScope = _c.activeScope, onScopeChange = _c.onScopeChange;
    var _scope = useSafeScope(activeScope);
    var t = useTranslation().t;
    var params = useMemo(function () { return ({ scope: _scope, per_page: 1000, query: query }); }, [
        _scope,
        query,
    ]);
    var _d = useRequestData("useful_link", params), loading = _d.loading, catalogs = _d.catalogs, data = _d.data;
    var catalogMap = useCatalogMap((_b = data === null || data === void 0 ? void 0 : data.links) !== null && _b !== void 0 ? _b : []);
    var _expandedKeys = useExpandedKeys(catalogMap);
    var treeData = useMemo(function () {
        if (!query) {
            return buildTreeData(catalogs, catalogMap, t);
        }
        else {
            if (!data || !data.links) {
                return [];
            }
            return data.links.map(function (i) { return ({
                key: i.id,
                originData: i,
                className: classNames(css.noWrap, css.highlight),
                title: (React.createElement("span", { dangerouslySetInnerHTML: { __html: DOMPurify.sanitize(i.name) } })),
                icon: React.createElement(FileTextTwoTone, null),
            }); });
        }
    }, [catalogMap, catalogs, data, query, t]);
    var onClick = useCallback(function (e, treeNode) {
        var _a;
        var originData = (_a = treeNode === null || treeNode === void 0 ? void 0 : treeNode.props) === null || _a === void 0 ? void 0 : _a.originData;
        if (originData) {
            onUse === null || onUse === void 0 ? void 0 : onUse("useful_link", originData);
            hit(originData.id, "UsefulLink");
        }
    }, [onUse]);
    var mainContent = useMemo(function () {
        if (loading) {
            return React.createElement(Spin, null);
        }
        else if (treeData && treeData.length) {
            return (React.createElement(Tree, { blockNode: true, showIcon: true, expandedKeys: _expandedKeys, onExpand: onExpand, selectable: false, treeData: treeData, onClick: onClick }));
        }
        else {
            return (React.createElement("p", null, t("page.im.km.no_result", { name: "$t(label.useful_link)" })));
        }
    }, [expandedKeys, loading, onClick, onExpand, t, treeData]);
    return (React.createElement(React.Fragment, null,
        React.createElement(ScopeSwitch, { value: _scope, onChange: onScopeChange }),
        React.createElement(SearchInput, { onSearch: onSearch, defaultValue: query }),
        mainContent));
};
export default UsefulLink;
