import { __rest } from "tslib";
import React, { useCallback, useEffect, useMemo, useState } from 'react';
import DOMPurify from 'dompurify';
import { handleText } from '../utilities';
import { getChannelRobot, getRecommendQuestionList, ifEnabled } from '../robot-utils';
import { message } from 'udesk-ui';
import RobotImg from '../icon/robot-img';
import forEach from 'lodash-es/forEach';
import { useTranslation } from 'react-i18next';
import style from './index.module.less';
import RobotAutoSugItem from './robot-auto-sug-item';
import moment from 'moment';
import Search from '../icon/search';
import { removeHTMLTag } from '../utilities';
var RobotAutoSugs = React.memo(function (_a) {
    var kcsSdk = _a.kcsSdk, id = _a.id, title = _a.title, trans_text = _a.trans_text, langto = _a.langto, titleStyle = _a.titleStyle, im_sub_session_id = _a.im_sub_session_id, date = _a.date, chatPlatform = _a.chatPlatform, onSelectSuggestion = _a.onSelectSuggestion, smart_assistance_enabled = _a.smart_assistance_enabled, robotBaseURL = _a.robotBaseURL, props = __rest(_a, ["kcsSdk", "id", "title", "trans_text", "langto", "titleStyle", "im_sub_session_id", "date", "chatPlatform", "onSelectSuggestion", "smart_assistance_enabled", "robotBaseURL"]);
    var t = useTranslation().t;
    var _b = useState(false), enabled = _b[0], setEnabled = _b[1];
    var _c = useState(props.open), open = _c[0], setOpen = _c[1];
    var _d = useState(false), loadedSugData = _d[0], setLoadedSugData = _d[1];
    var _e = useState(props.twemoji), twemoji = _e[0], setTwemoji = _e[1];
    var _f = useState([]), sugsData = _f[0], setSugsData = _f[1];
    var _ifEnabled = useCallback(function () {
        return ifEnabled('smart_assistance');
    }, []);
    useEffect(function () { return setOpen(false); }, [smart_assistance_enabled]);
    useEffect(function () { return setOpen(props.open); }, [props.open]);
    var tryNoSearchResultTip = useCallback(function (data) {
        if (!data || !data.length) {
            message.error(t('page.im.log.noSearchResult'));
        }
        else {
            setOpen(true);
        }
    }, [t]);
    var loadSugData = useCallback(function (clicked) {
        setLoadedSugData(true);
        getChannelRobot(im_sub_session_id).then(function (res) {
            res = res.channel_udesk_robot || {};
            getRecommendQuestionList({
                robotId: res.km_robot_id,
                channelId: res.scene_id,
                queryContent: title,
                imSubSessionId: im_sub_session_id,
                messageId: id,
                messageCreatedAt: moment(date).format('YYYY-MM-DD HH:mm:ss')
            }, {
                robotBaseURL: robotBaseURL
            }).then(function (res) {
                var data = res.data || {};
                var sugsData = data.suggestQuestionList || [];
                var solutionResults = data.solutionResults || [];
                // 非流程
                if (!data.flowId) {
                    switch (data.msgType) {
                        case 'rich':
                        case 'text':
                            if (data.hitQuestion) {
                                sugsData.unshift({
                                    id: data.aid,
                                    content: data.hitQuestion,
                                    answer: {
                                        content: data.ansContent
                                    }
                                });
                            }
                            break;
                    }
                }
                solutionResults = forEach(solutionResults, function (item) {
                    item.answer = {
                        content: item.content
                    };
                    item.content = item.name;
                    return item;
                });
                sugsData = sugsData.concat(solutionResults);
                setSugsData(sugsData);
                clicked && tryNoSearchResultTip(sugsData);
            });
        });
    }, [date, id, im_sub_session_id, title, tryNoSearchResultTip]);
    var loadData = useCallback(function (clicked) {
        _ifEnabled().then(function (res) {
            setOpen(true);
            setEnabled(res.enabled);
            if (clicked) {
                if (res.enabled) {
                    loadSugData(clicked);
                }
            }
            else {
                ifEnabled('smart_assistance_auto_suggestion').then(function (_res) {
                    if (_res.enabled) {
                        loadSugData('');
                    }
                });
            }
        });
    }, [_ifEnabled, loadSugData]);
    useEffect(function () {
        if (open) {
            loadData('');
        }
        else {
            _ifEnabled().then(function (res) {
                setEnabled(res.enabled);
            });
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    var handleOpen = useCallback(function () {
        if (!loadedSugData) {
            loadData(true);
        }
        else {
            tryNoSearchResultTip(sugsData);
        }
    }, [loadData, loadedSugData, sugsData, tryNoSearchResultTip]);
    var text = useMemo(function () {
        return handleText(chatPlatform, {
            content: title,
            font: titleStyle,
            twemoji: twemoji
        });
    }, [chatPlatform, title, titleStyle]);
    var searchByKcs = useCallback(function () {
        kcsSdk === null || kcsSdk === void 0 ? void 0 : kcsSdk.showDrawer();
        kcsSdk === null || kcsSdk === void 0 ? void 0 : kcsSdk.searchKeyword(removeHTMLTag(text));
    }, [kcsSdk, text, removeHTMLTag]);
    return (React.createElement("div", { className: style['robot-msg-wrapper'] },
        kcsSdk ? React.createElement(Search, { onClick: function () { return searchByKcs(); }, className: style['kcs-search'] }) : null,
        React.createElement("div", { className: style['robot-msg'] },
            React.createElement("span", { className: style['robot-img-span'], dangerouslySetInnerHTML: {
                    __html: DOMPurify.sanitize(text)
                } }),
            enabled ? React.createElement(RobotImg, { className: style['robot-img'], onClick: handleOpen }) : null),
        (trans_text === null || trans_text === void 0 ? void 0 : trans_text.content) ? React.createElement("div", { className: style['trans_test'] },
            React.createElement("div", { dangerouslySetInnerHTML: { __html: DOMPurify.sanitize(trans_text === null || trans_text === void 0 ? void 0 : trans_text.content) } }),
            React.createElement("div", { className: style['trans_tip'] }, t('page.im.imMessages.AITranslateTitle'))) : null,
        sugsData.length > 0 && open &&
            React.createElement(React.Fragment, null,
                React.createElement("hr", null),
                React.createElement("ul", { style: { marginBottom: 0, textAlign: 'left' } }, sugsData.map(function (item, index) {
                    return React.createElement(RobotAutoSugItem, { key: item.id, index: index + 1, data: item, id: id, content: title, onClick: onSelectSuggestion });
                })))));
});
export default RobotAutoSugs;
