import React, { useCallback, useEffect, useLayoutEffect, useRef, useState, } from "react";
import style from "./tinymce.module.scss";
import { randomString, removeHTMLTag, uploadFile } from "../../utils/index";
import UploadModal from "../uploadModal/index";
var defaultStyle = {
    color: "rgba(0, 0, 0, 0.25)",
};
var error = {
    color: "rgba(251, 81, 39, 1)",
};
var FormTinymce = function (props) {
    var paste_data_images = props.paste_data_images;
    var toolBars = "upimg | bold italic underline | alignleft aligncenter alignright | bullist numlist outdent indent | formatselect fontsizeselect | link image |  fontselect  forecolor backcolor code ";
    var value = props.value, onChange = props.onChange, autoresize_min_height = props.autoresize_min_height, _a = props.autoresize_max_height, autoresize_max_height = _a === void 0 ? 50 : _a, _b = props.statusbar, statusbar = _b === void 0 ? false : _b, _c = props.toolbar, toolbar = _c === void 0 ? toolBars : _c, _d = props.showImg, showImg = _d === void 0 ? true : _d, _e = props.suffix, suffix = _e === void 0 ? "" : _e, maxLength = props.maxLength;
    var _f = useState(removeHTMLTag(value || "")), editorContent = _f[0], setContent = _f[1]; //更新富文本内容
    var _g = useState(false), showModal = _g[0], setShowModal = _g[1];
    var _h = useState(null), tinymceInstance = _h[0], setTinymceInstance = _h[1];
    var tinymceElementId = useRef(randomString(10));
    // 内容改变时    去html标签 计算输入的长度
    var handleEditorChange = useCallback(function () {
        var content = tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.getContent();
        setContent(removeHTMLTag(content));
        onChange === null || onChange === void 0 ? void 0 : onChange(content);
    }, [onChange, tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.getContent]);
    var hideImageUploadModal = useCallback(function () { return setShowModal(false); }, []);
    //点击弹窗的确定  将选择的图片插入并 关闭弹窗
    var addImg = useCallback(function (imgList) {
        if (!tinymceInstance) {
            return;
        }
        imgList.forEach(function (item) {
            tinymceInstance.execCommand("mceInsertContent", false, "<img src=".concat(item.src, " alt=\"\"/>"));
        });
        setShowModal(false);
    }, [tinymceInstance]);
    useLayoutEffect(function () {
        var editor;
        var win = window;
        win.tinymce.init({
            selector: "#".concat(tinymceElementId.current).concat(suffix),
            autoresize_min_height: autoresize_min_height,
            autoresize_max_height: autoresize_max_height,
            autoresize_bottom_margin: 5,
            menubar: false,
            statusbar: statusbar,
            elementpath: false,
            branding: false,
            paste_data_images: paste_data_images,
            images_upload_handler: function (blobInfo, success, failure) {
                console.log("images_upload_handler", blobInfo, new File([blobInfo.blob()], blobInfo.filename()));
                uploadFile(new File([blobInfo.blob()], blobInfo.filename()), {
                    marking: "web.mg.rich_editor_copy",
                })
                    .then(function (resp) {
                    success(resp.src);
                })
                    .catch(function (e) {
                    failure("上传图片失败");
                });
            },
            plugins: [
                "link image autoresize lists charmap preview hr ",
                "visualchars insertdatetime nonbreaking",
                "table contextmenu textcolor",
                "paste code imagetools",
            ],
            toolbar: toolbar,
            setup: function (instance) {
                editor = instance;
                instance.on("init", function () {
                    setTinymceInstance(instance);
                });
                if (showImg) {
                    instance.addButton("upimg", {
                        text: "上传图片",
                        onclick: function () {
                            setShowModal(true);
                        },
                    });
                }
            },
        });
        return function () {
            editor === null || editor === void 0 ? void 0 : editor.destroy();
        };
    }, [
        autoresize_max_height,
        autoresize_min_height,
        paste_data_images,
        showImg,
        statusbar,
        suffix,
        toolbar,
    ]);
    useEffect(function () {
        if (tinymceInstance && handleEditorChange) {
            tinymceInstance.on("change", handleEditorChange);
            return function () { return tinymceInstance.off("change", handleEditorChange); };
        }
    }, [handleEditorChange, tinymceInstance]);
    useLayoutEffect(function () {
        if (tinymceInstance && props.value !== tinymceInstance.getContent()) {
            tinymceInstance.setContent(props.value || "");
        }
    }, [props.value, tinymceInstance]);
    return (React.createElement("div", { className: style["rich-text-wrapper"] },
        React.createElement("div", { id: "".concat(tinymceElementId.current).concat(suffix) }),
        maxLength && (React.createElement("div", { className: style["editor-num"], style: editorContent.length > maxLength ? error : defaultStyle },
            editorContent.length,
            "/",
            maxLength)),
        React.createElement(UploadModal, { show: showModal, close: hideImageUploadModal, addImg: addImg })));
};
FormTinymce.defaultProps = {
    paste_data_images: false,
};
export default FormTinymce;
