var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import Tree from "udesk-ui/es/tree";
import React, { useCallback, useEffect, useMemo, useState } from "react";
import produce from "immer";
import uniq from "lodash-es/uniq";
import Node from "./node";
import FolderOutlined from "@ant-design/icons/FolderOutlined";
import css from "./style.module.scss";
import { deleteTreeNode, findItemCommon, findTreeNode, moveTreeNode } from "./utilities";
import get from 'lodash-es/get';
import set from "lodash-es/set";
function mapSelectOptions(value, _a) {
    var shouldShowNode = _a.shouldShowNode, parentKey = _a.parentKey, nodeChildrenPath = _a.nodeChildrenPath, props = __rest(_a, ["shouldShowNode", "parentKey", "nodeChildrenPath"]);
    return value.filter(shouldShowNode).map(function (i, index) {
        var key = parentKey ? "".concat(parentKey, "-").concat(index) : "".concat(index);
        return {
            key: key,
            title: React.createElement(Node, __assign({ value: i, nodeKey: key, nodeChildrenPath: nodeChildrenPath }, props)),
            icon: React.createElement(FolderOutlined, null),
            blockNode: true,
            className: css.node,
            origin: i,
            children: mapSelectOptions(get(i, nodeChildrenPath) || [], __assign({ shouldShowNode: shouldShowNode, nodeChildrenPath: nodeChildrenPath, parentKey: key }, props))
        };
    });
}
function TreeEditor(_a) {
    var onChange = _a.onChange, value = _a.value, onSelect = _a.onSelect, _b = _a.shouldShowNode, shouldShowNode = _b === void 0 ? function () { return true; } : _b, _c = _a.nodeChildrenPath, nodeChildrenPath = _c === void 0 ? 'subs' : _c, _d = _a.nodeTitlePath, nodeTitlePath = _d === void 0 ? 'title' : _d, _e = _a.nodeValuePath, nodeValuePath = _e === void 0 ? 'value' : _e, _f = _a.allowSelect, allowSelect = _f === void 0 ? function () { return true; } : _f, createNode = _a.createNode, props = __rest(_a, ["onChange", "value", "onSelect", "shouldShowNode", "nodeChildrenPath", "nodeTitlePath", "nodeValuePath", "allowSelect", "createNode"]);
    var _g = useState([]), expandedKeys = _g[0], setExpandedKeys = _g[1];
    var _h = useState([]), selectedKeys = _h[0], setSelectedKeys = _h[1];
    var onAddChild = useCallback(function (option, parents) {
        if (!onChange || !value) {
            return;
        }
        var nextValue = produce(value, function (draft) {
            var parentNode = findTreeNode(draft, parents, nodeChildrenPath);
            if (!parentNode) {
                return;
            }
            if (!get(parentNode, nodeChildrenPath)) {
                set(parentNode, nodeChildrenPath, []);
            }
            get(parentNode, nodeChildrenPath).push(createNode(parentNode));
        });
        onChange(nextValue || []);
        setExpandedKeys(function (prevState) { return uniq(__spreadArray(__spreadArray([], prevState, true), [parents], false)); });
    }, [onChange, value, nodeChildrenPath, createNode]);
    var onClickNode = useCallback(function (nodeKey, node) {
        if (allowSelect(node)) {
            setSelectedKeys([nodeKey]);
        }
    }, [allowSelect]);
    var onDeleteNode = useCallback(function (nodeKey) {
        if (!onChange || !value) {
            return;
        }
        var nodeKeyLen = nodeKey.split('-');
        var father = nodeKeyLen.length > 1 ? findItemCommon(value, nodeKeyLen.slice(0, nodeKeyLen.length - 1), true) : value;
        var deleteItem = nodeKeyLen.length > 1 ? father.subs[nodeKeyLen[nodeKeyLen.length - 1]] : father[nodeKey];
        onChange(produce(value, function (draft) {
            deleteTreeNode(draft, nodeKey, nodeChildrenPath);
        }) || [], { nodeKey: nodeKey, type: 'delete', deleteItem: deleteItem });
    }, [onChange, value, nodeChildrenPath]);
    var onNodeTitleChange = useCallback(function (title, nodeKey) {
        if (!onChange || !value) {
            return;
        }
        var newValue = produce(value, function (draft) {
            var node = findTreeNode(draft, nodeKey, nodeChildrenPath);
            if (node) {
                set(node, nodeTitlePath, title);
            }
        });
        onChange(newValue || [], { nodeKey: nodeKey, type: 'change' });
    }, [nodeChildrenPath, nodeTitlePath, onChange, value]);
    var treeData = useMemo(function () {
        if (!value) {
            return [];
        }
        return mapSelectOptions(value, __assign({ onAdd: onAddChild, onClick: onClickNode, onChange: onNodeTitleChange, onDelete: onDeleteNode, nodeChildrenPath: nodeChildrenPath, shouldShowNode: shouldShowNode, nodeTitlePath: nodeTitlePath, nodeValuePath: nodeValuePath }, props));
    }, [value, onAddChild, onClickNode, onNodeTitleChange, onDeleteNode, nodeChildrenPath, shouldShowNode, nodeTitlePath, nodeValuePath, props]);
    useEffect(function () {
        if (selectedKeys) {
            onSelect(selectedKeys[0]);
        }
    }, [selectedKeys, onSelect]);
    var onDrop = useCallback(function (_a) {
        var node = _a.node, dragNode = _a.dragNode;
        if (!onChange || !value) {
            return;
        }
        if (node.dragOver) {
            onChange(produce(value, function (draft) { return moveTreeNode(draft, dragNode.key, node.key, nodeChildrenPath); }));
        }
        else if (node.dragOverGapBottom || node.dragOverGapTop) {
            onChange(produce(value, function (draft) {
                var position = parseInt(node.key.substring(node.key.length - 1));
                if (node.dragOverGapBottom) {
                    position += 1;
                }
                moveTreeNode(draft, dragNode.key, node.key.substring(0, node.key.lastIndexOf('-')), nodeChildrenPath, position);
            }));
        }
    }, [nodeChildrenPath, onChange, value]);
    return React.createElement(Tree, { selectable: true, showIcon: true, blockNode: true, treeData: treeData, expandedKeys: expandedKeys, selectedKeys: selectedKeys, draggable: true, onDrop: onDrop, onExpand: function (keys) { return setExpandedKeys(keys); } });
}
export default TreeEditor;
export * from './utilities';
