import { __extends } from "tslib";
import React from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import { DatePicker } from 'udesk-ui';
import moment from 'moment';
import { time2Locale, time2Service } from '@udesk-webapp/utils';
var RangePicker = DatePicker.RangePicker;
var AntdRangeDatePickerComponent = /** @class */ (function (_super) {
    __extends(AntdRangeDatePickerComponent, _super);
    function AntdRangeDatePickerComponent() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        /**
         * @param date [moment, moment]
         * @param dateString [string, string]
         */
        _this.handleRangeDatePickerChange = function (date, dateString) {
            var _a = _this.props, _b = _a.attributes, attributes = _b === void 0 ? {} : _b, onChanged = _a.onChanged;
            var _c = attributes.format, format = _c === void 0 ? 'YYYY-MM-DD HH:mm' : _c, showTime = attributes.showTime, needTransfer = attributes.needTransfer;
            var internalValue = dateString;
            if (Array.isArray(internalValue)) {
                var firstDateString = dateString[0], secondDateString = dateString[1];
                if (firstDateString && secondDateString) {
                    if (showTime && needTransfer) {
                        internalValue = "".concat(time2Service(firstDateString, format), ",").concat(time2Service(secondDateString, format));
                    }
                    else {
                        internalValue = dateString.join(',');
                    }
                }
                else {
                    internalValue = undefined;
                }
            }
            else {
                internalValue = undefined;
            }
            onChanged(internalValue);
        };
        return _this;
    }
    AntdRangeDatePickerComponent.prototype.render = function () {
        var _a = this.props, disabled = _a.disabled, attributes = _a.attributes, value = _a.value;
        var classNames = attributes.classNames, placeholder = attributes.placeholder, _b = attributes.picker, picker = _b === void 0 ? 'date' : _b, _c = attributes.format, format = _c === void 0 ? 'YYYY-MM-DD HH:mm:ss' : _c, _d = attributes.showTime, showTime = _d === void 0 ? true : _d, size = attributes.size, _e = attributes.allowClear, allowClear = _e === void 0 ? true : _e, ranges = attributes.ranges, needTransfer = attributes.needTransfer;
        if (!(Array.isArray(disabled))) {
            disabled = [disabled, disabled];
        }
        var internalValue = value;
        if (typeof internalValue === 'string') {
            internalValue = internalValue.split(',') || [];
        }
        if (Array.isArray(internalValue)) {
            internalValue = internalValue.map(function (timeString) {
                if (timeString && showTime && needTransfer) {
                    var localStr = time2Locale(timeString, format);
                    return moment(localStr, format);
                }
                return moment(timeString, format);
            });
        }
        else {
            internalValue = [undefined, undefined];
        }
        return (React.createElement(RangePicker, { style: { width: '300px' }, size: size, allowClear: allowClear, className: ClassNames(classNames, 'udesk-custom-filter-antd-range-datepicker'), getPopupContainer: function (triggerNode) { return triggerNode.parentElement; }, placeholder: placeholder, disabled: disabled, picker: picker, showTime: showTime, format: format, value: internalValue, onChange: this.handleRangeDatePickerChange, ranges: ranges }));
    };
    return AntdRangeDatePickerComponent;
}(React.Component));
export default AntdRangeDatePickerComponent;
/**
 * @description 组件可能接收的prop
 * @namespace
 * @property {object} attributes - 配置valueSetting时传入，常用
 * @property {string} attributes.classNames - 自定义组件css类名
 * @property {string} attributes.placeholder - 自定义组件placeholder
 * @property {string} attributes.picker -
 * @property {string} attributes.format -
 * @property {function} onChanged - value改变唯一触发事件，其他入onCalendarChange之类可能需要在valueSetting时配置，而且是跟自定义过滤器同一上下文
 * @property {string} value - 字段value，该组件位text input,value只需考虑string,复杂选择组件需object
 *
 * @property {object} operator - condition上下文相关
 * @property {object} field - condition上下文相关
 * @property {string} name - 校验相关，错误信息提示字段名
 * @property {boolean} isValid - 校验相关 具体待补充 todo
 * @property {array} errors - 校验相关，错误信息
 * @property {object} runtimeAttributes - 运行时传入valueInput组件的属性,从custom-filter组件传入conditionValueComponentAttrs
 *
 */
AntdRangeDatePickerComponent.defaultProps = {
    value: null,
    onChanged: null,
    attributes: {},
    disabled: false,
    operator: {},
    field: {},
    name: null,
    isValid: true,
    errors: [],
    runtimeAttributes: {},
};
AntdRangeDatePickerComponent.propTypes = {
    value: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.arrayOf(PropTypes.string), // 待支持array  todo
    ]),
    onChanged: PropTypes.func,
    attributes: PropTypes.shape({
        picker: PropTypes.oneOf(['date', 'week', 'month', 'quarter', 'year']),
        format: PropTypes.oneOf([
            'YYYY-MM-DD',
            'YYYY-MM-DD HH',
            'YYYY-MM-DD HH:mm',
            'YYYY-MM-DD HH:mm:ss',
            'GGGG-[W]WW',
            'YYYY-MM',
            'YYYY-Q',
            'YYYY',
        ]),
        showTime: PropTypes.oneOfType([
            PropTypes.bool,
            PropTypes.shape({
                format: PropTypes.oneOf(['HH', 'HH:mm', 'HH:mm:ss']),
            }),
        ]),
        size: PropTypes.oneOf(['small', 'middle', 'large']),
        allowClear: PropTypes.bool,
        classNames: PropTypes.any,
        placeholder: PropTypes.any,
        needTransfer: PropTypes.bool,
    }),
    disabled: PropTypes.bool,
    operator: PropTypes.object,
    field: PropTypes.object,
    name: PropTypes.string,
    isValid: PropTypes.bool,
    errors: PropTypes.array,
    runtimeAttributes: PropTypes.object,
};
