import { __extends } from "tslib";
import React from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import { TimePicker } from 'udesk-ui';
import moment from 'moment';
import { time2Locale, time2Service } from '@udesk-webapp/utils';
var RangePicker = TimePicker.RangePicker;
var AntdRangeTimePickerComponent = /** @class */ (function (_super) {
    __extends(AntdRangeTimePickerComponent, _super);
    function AntdRangeTimePickerComponent() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        /**
         * @param time [moment, moment]
         * @param timeString [string, string]
         */
        _this.handleRangeTimePickerChange = function (time, timeString) {
            var _a = _this.props, _b = _a.attributes, attributes = _b === void 0 ? {} : _b, onChanged = _a.onChanged;
            var _c = attributes.format, format = _c === void 0 ? 'HH:mm:ss' : _c, needTransfer = attributes.needTransfer;
            var internalValue = timeString;
            if (Array.isArray(internalValue)) {
                var firstTimeString = timeString[0], secondTimeString = timeString[1];
                if (firstTimeString && secondTimeString) {
                    if (needTransfer) {
                        internalValue = "".concat(time2Service(firstTimeString, format, true), ",").concat(time2Service(secondTimeString, format, true));
                    }
                    else {
                        internalValue = timeString.join(',');
                    }
                }
                else {
                    internalValue = undefined;
                }
            }
            else {
                internalValue = undefined;
            }
            onChanged(internalValue);
        };
        return _this;
    }
    AntdRangeTimePickerComponent.prototype.render = function () {
        var _a = this.props, disabled = _a.disabled, attributes = _a.attributes, value = _a.value;
        var classNames = attributes.classNames, _b = attributes.placeholder, placeholder = _b === void 0 ? '' : _b, _c = attributes.allowClear, allowClear = _c === void 0 ? true : _c, _d = attributes.format, format = _d === void 0 ? 'HH:mm:ss' : _d, size = attributes.size, _e = attributes.separator, separator = _e === void 0 ? '~' : _e, needTransfer = attributes.needTransfer;
        if (!(Array.isArray(disabled))) {
            disabled = [disabled, disabled];
        }
        var internalValue = value;
        if (typeof internalValue === 'string') {
            internalValue = internalValue.split(',') || [];
        }
        if (Array.isArray(internalValue)) {
            internalValue = internalValue.map(function (timeString) {
                if (timeString && needTransfer) {
                    var localStr = time2Locale(timeString, format, true);
                    return moment(localStr, format);
                }
                return moment(timeString, format);
            });
        }
        else {
            internalValue = [undefined, undefined];
        }
        // size 与 className 同时使用时会导致size设置不生效(只发现RangePicker有问题)
        return (React.createElement("div", { className: ClassNames(classNames, 'udesk-custom-filter-antd-range-time-picker') },
            React.createElement(RangePicker, { style: { width: '300px' }, size: size, getPopupContainer: function (triggerNode) { return triggerNode.parentElement; }, separator: separator, placeholder: placeholder, disabled: disabled, allowClear: allowClear, format: format, value: internalValue, onChange: this.handleRangeTimePickerChange })));
    };
    return AntdRangeTimePickerComponent;
}(React.Component));
export default AntdRangeTimePickerComponent;
/**
 * @description 组件可能接收的prop
 * @namespace
 * @property {object} attributes - 配置valueSetting时传入，常用
 * @property {string} attributes.classNames - 自定义组件css类名
 * @property {string} attributes.placeholder - 自定义组件placeholder
 * @property {string} attributes.disabled - 范围选择器有两个输入框，disabled时需转换[bool,bool] 自定义过滤器目前应该只支持同时禁用两个框
 * @property {string} attributes.format -
 * @property {function} onChanged - value改变唯一触发事件，其他入onFocus之类可能需要在valueSetting时配置，而且是跟自定义过滤器同一上下文
 * @property {string} value - 字段value，该组件位text input,value只需考虑string,复杂选择组件需object
 *
 * @property {object} operator - condition上下文相关
 * @property {object} field - condition上下文相关
 * @property {string} name - 校验相关，错误信息提示字段名
 * @property {boolean} isValid - 校验相关 具体待补充 todo
 * @property {array} errors - 校验相关，错误信息
 * @property {object} runtimeAttributes - 运行时传入valueInput组件的属性,从custom-filter组件传入conditionValueComponentAttrs
 *
 */
AntdRangeTimePickerComponent.defaultProps = {
    value: null,
    onChanged: null,
    attributes: {},
    disabled: false,
    operator: {},
    field: {},
    name: null,
    isValid: true,
    errors: [],
    runtimeAttributes: {},
};
AntdRangeTimePickerComponent.propTypes = {
    value: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.arrayOf(PropTypes.string), // 待支持todo
    ]),
    onChanged: PropTypes.func,
    attributes: PropTypes.shape({
        format: PropTypes.oneOf([
            'HH',
            'HH:mm',
            'HH:mm:ss',
            'h a',
            'h:mm a',
            'h:mm:ss a',
            'h:mm:ss A',
        ]),
        placeholder: PropTypes.arrayOf(PropTypes.string),
        allowClear: PropTypes.bool,
        size: PropTypes.string,
        classNames: PropTypes.any,
        separator: PropTypes.any,
        needTransfer: PropTypes.bool,
    }),
    disabled: PropTypes.oneOfType([PropTypes.bool, PropTypes.arrayOf(PropTypes.bool)]),
    operator: PropTypes.object,
    field: PropTypes.object,
    name: PropTypes.string,
    isValid: PropTypes.bool,
    errors: PropTypes.array,
    runtimeAttributes: PropTypes.object,
};
