import { __assign } from "tslib";
// @ts-nocheck
import React, { useLayoutEffect, useRef, useEffect } from 'react';
import Viewer from 'viewerjs';
// import { downLoadFile } from '@/service';
import { ajax } from "@udesk-webapp/utils";
import { useTranslation } from '@udesk-webapp/locale';
import FileIcon from '../file-icon';
import { getPreviewUrl } from '../../utils/common';
var ImageAttachment = function (_a) {
    var url = _a.url, file_name = _a.file_name, onPreviewImage = _a.onPreviewImage, downloadFile = _a.downloadFile, id = _a.id, imgIndex = _a.imgIndex, formatMessage = _a.formatMessage;
    var type = (file_name === null || file_name === void 0 ? void 0 : file_name.split('.').pop()) || '';
    return (React.createElement("div", null,
        React.createElement(FileIcon, { type: type }),
        React.createElement("span", null, file_name),
        React.createElement("img", { src: url, style: { display: 'none' }, alt: file_name }),
        React.createElement("a", { className: "mgl-10", role: "button", "data-ignore-switch-editing": "1", onClick: function (e) {
                e.stopPropagation();
                onPreviewImage(imgIndex);
            } }, formatMessage('labels.preview')),
        React.createElement("a", { href: url, className: "mgl-5", "data-ignore-switch-editing": "1", target: "_blank", download: file_name, onClick: function (e) {
                e.stopPropagation();
                downloadFile(id);
            } }, formatMessage('labels.download'))));
};
var DocumentLikeAttachment = function (_a) {
    var url = _a.url, file_name = _a.file_name, onPreview = _a.onPreview, downloadFile = _a.downloadFile, id = _a.id, formatMessage = _a.formatMessage;
    var type = (file_name === null || file_name === void 0 ? void 0 : file_name.split('.').pop()) || '';
    return (React.createElement("div", null,
        React.createElement(FileIcon, { type: type }),
        React.createElement("span", null, file_name),
        onPreview ? (React.createElement("a", { className: "mgl-10", role: "button", "data-ignore-switch-editing": "1", onClick: function (e) {
                e.stopPropagation();
                onPreview({ url: url, file_name: file_name });
            } }, formatMessage('labels.preview'))) : null,
        React.createElement("a", { href: url, className: "mgl-5", "data-ignore-switch-editing": "1", target: "_blank", download: file_name, onClick: function (e) {
                e.stopPropagation();
                downloadFile(id);
            } }, formatMessage('labels.download'))));
};
var CommonAttachment = function (_a) {
    var url = _a.url, file_name = _a.file_name;
    var type = (file_name === null || file_name === void 0 ? void 0 : file_name.split('.').pop()) || '';
    return (React.createElement("a", { className: "link", href: url, "data-ignore-switch-editing": "1", target: "_blank" },
        React.createElement(FileIcon, { type: type }),
        file_name));
};
var AttachmentList = function (_a) {
    var attachments = _a.attachments, previewUrl = _a.previewUrl, udeskVersion = _a.udeskVersion, ticketId = _a.ticketId, downLoadLogEnable = _a.downLoadLogEnable;
    var formatMessage = useTranslation().t; // 多语言
    var ref = useRef();
    var viewer = useRef();
    useLayoutEffect(function () {
        if (!ref.current) {
            return;
        }
        viewer.current = new Viewer(ref.current);
        return function () {
            var _a;
            (_a = viewer.current) === null || _a === void 0 ? void 0 : _a.destroy();
        };
    }, []);
    useEffect(function () {
        var _a;
        (_a = viewer.current) === null || _a === void 0 ? void 0 : _a.update();
    }, [attachments]);
    var downloadFile = function (id) {
        if (!downLoadLogEnable)
            return;
        // downLoadFile({
        //     ticket_attachment_download_log: {
        //         attachment_id: id,
        //         ticket_id: ticketId,
        //     },
        // });
        ajax().postJSON('/cs/ts/api/ticket_attachment_download_logs', {
            ticket_attachment_download_log: {
                attachment_id: id,
                ticket_id: ticketId,
            }
        }).then(function (res) {
        }).catch(function (e) {
        });
    };
    var onPreview = function (_a) {
        var url = _a.url;
        if (url) {
            var u = getPreviewUrl(url, {
                previewUrl: previewUrl
            });
            window.open(u, '_blank');
        }
    };
    var imgIndex = 0;
    var imgAttachment = [], docAttachment = [], comAttachment = [];
    (attachments || []).forEach(function (i, idx) {
        if (/\.(jpg|png|gif|jpeg)$/.test(i.file_name)) {
            i.imgIndex = imgIndex++;
            imgAttachment.push(React.createElement(ImageAttachment, __assign({}, i, { onPreviewImage: function (idx) {
                    var _a;
                    (_a = viewer.current) === null || _a === void 0 ? void 0 : _a.view(idx);
                }, downloadFile: downloadFile, key: idx, formatMessage: formatMessage })));
        }
        else if (udeskVersion === 'pro' &&
            /.(doc|docx|odt|rtf|wps|ppt|pptx|odp|dp|xls|xlsx|ods|csv|et|pdf)$/.test(i.file_name)) {
            docAttachment.push(React.createElement(DocumentLikeAttachment, __assign({}, i, { onPreview: onPreview, downloadFile: downloadFile, key: idx, formatMessage: formatMessage })));
        }
        else {
            comAttachment.push(React.createElement(CommonAttachment, __assign({}, i, { key: idx })));
        }
    });
    return (React.createElement("div", null,
        React.createElement("div", { ref: ref }, imgAttachment),
        docAttachment,
        comAttachment));
};
export default AttachmentList;
