import { __assign, __rest } from "tslib";
// @ts-nocheck
import React, { useCallback, useEffect, useState, useMemo, useRef } from 'react';
import { Cascader } from 'udesk-ui';
import { getFormatOptions } from '../../utils/format';
import './index.module.scss';
import CopyText from '../copy';
// 外部不需要再处理隐藏选项，由内部统一处理，及不需要optionsMeta
// 统一逻辑：有隐藏选项返显时，value值可反显，但options依然需要隐藏
/**
 * 快速级联组件
 */
var FormCascader = React.memo(function (props) {
    var copyProps = props.copyProps, options = props.options, valueProps = props.value, fieldNames = props.fieldNames, fullSelect = props.fullSelect, readOnly = props.readOnly, disabled = props.disabled, classNames = props.classNames, _a = props.onChange, onChange = _a === void 0 ? function () { } : _a, restProps = __rest(props, ["copyProps", "options", "value", "fieldNames", "fullSelect", "readOnly", "disabled", "classNames", "onChange"]);
    var _b = useState([]), valueList = _b[0], setValueList = _b[1];
    var changeValue = useRef([]);
    useEffect(function () {
        setValueList(function () {
            if (typeof valueProps === 'string') {
                return valueProps ? valueProps.split(',') : [];
            }
            return valueProps;
        });
    }, [valueProps]);
    var _c = useMemo(function () {
        var _a = fieldNames || {}, _b = _a.label, label = _b === void 0 ? 'label' : _b, _c = _a.value, value = _c === void 0 ? 'value' : _c, _d = _a.children, children = _d === void 0 ? 'children' : _d, _e = _a.is_hide, is_hide = _e === void 0 ? 'is_hide' : _e;
        return getFormatOptions(options, { label: label, value: value, children: children, is_hide: is_hide });
    }, [options, fieldNames]), formatOptions = _c[0], formatNoHideOptions = _c[1];
    var valueObj = useMemo(function () {
        return (valueList || []).reduce(function (initValues, currentValue, index) {
            var _options = !index
                ? formatOptions
                : (initValues[index - 1] || {}).children || [];
            var keyOption = _options.find(function (item) { return currentValue === item.value; }) || {};
            initValues.push(keyOption);
            return initValues;
        }, []);
    }, [formatOptions, valueList]);
    var valueLabel = useMemo(function () {
        return valueObj.map(function (item) { return item.label; });
    }, [valueObj]);
    var showValueText = useMemo(function () {
        return valueLabel.join(' / ');
    }, [valueLabel]);
    if (readOnly) {
        return (React.createElement(CopyText, __assign({}, copyProps, { text: showValueText }),
            React.createElement(React.Fragment, null, showValueText)));
    }
    var onValueChange = useCallback(function (_value, _option) {
        // FIXME: 虽然changeOnSelect：false 可以实现最末级必填的功能，但产品需要在此时未选到最末级时给提示
        // 目前的解决方案是 changeOnSelect:true，fullSelect: true 时验证是否需要提示
        // 判断已选择的最后一级是否有children，有则加空值触发onChange后的_intervalvalidate验证
        // 并利用dropdown事件防止多次触发onChange以及后续事件导致卡顿感
        var _a;
        var newValue = _value || [];
        if (fullSelect &&
            newValue.length &&
            _option.length &&
            ((_a = _option[_option.length - 1]) === null || _a === void 0 ? void 0 : _a.children)) {
            newValue.push('');
        }
        changeValue.current = newValue;
        if (!newValue.length) {
            onChange([]);
        }
    }, []);
    var onDropdownVisibleChange = useCallback(function (visible) {
        var _a;
        !visible && ((_a = changeValue.current) === null || _a === void 0 ? void 0 : _a.length) && onChange(changeValue.current);
    }, [onChange]);
    var filter = useCallback(function (inputValue, path) {
        return path.some(function (option) { return option.label.toLowerCase().indexOf(inputValue.toLowerCase()) > -1; });
    }, []);
    return (React.createElement(CopyText, __assign({}, copyProps, { text: showValueText }),
        React.createElement(Cascader, __assign({ getPopupContainer: function (triggerNode) { return triggerNode; } }, restProps, { disabled: disabled, value: valueLabel, options: formatNoHideOptions, onChange: onValueChange, onDropdownVisibleChange: onDropdownVisibleChange, showSearch: { filter: filter }, onBlur: function () {
                var onValueBlur = props.onValueBlur, _a = props.id, id = _a === void 0 ? '' : _a;
                // 避免多次推送，开启最末级必填后最末级触发blur
                if (fullSelect && changeValue.current[changeValue.current.length - 1] === '') {
                    return;
                }
                onValueBlur === null || onValueBlur === void 0 ? void 0 : onValueBlur(id, changeValue.current || []);
            }, onClear: function () {
                // 这里模拟Blur事件用于字段推送
                var onValueBlur = props.onValueBlur, _a = props.id, id = _a === void 0 ? '' : _a;
                onValueBlur === null || onValueBlur === void 0 ? void 0 : onValueBlur(id, []);
            } }))));
});
FormCascader.defaultProps = {
    classNames: '',
    disabled: false,
    readOnly: false,
    fullSelect: false,
    changeOnSelect: true,
    value: [],
    options: [],
    onChange: function () { },
};
export default FormCascader;
