import { __assign } from "tslib";
import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { DownOutlined, UpOutlined } from '@ant-design/icons';
import { xssByRichEditorWhiteList, replaceScriptCode } from "@udesk-webapp/utils";
import CopyText from '../../copy';
import Viewer from 'react-viewer';
import './index.module.scss';
import { useTranslation } from '@udesk-webapp/locale';
function isHtml(text) {
    return /<[^>]+>/g.test(text);
}
function safeHtml(text) {
    var div = document.createElement('div');
    div.innerHTML = text;
    return (div.innerHTML || '').replace(/class\s*=\s*['"]([\w\s-])+['"]/gi, '');
}
function safeContent(text) {
    if (text && /\r\n|\n/.test(text)) {
        text = text.replace(/\r\n|\n/g, '<br>');
    }
    if (isHtml(text)) {
        return safeHtml(text);
    }
    return text;
}
var RichTextView = function (props, ref) {
    var content = props.content, _a = props.isDefaultOpen, isDefaultOpen = _a === void 0 ? false : _a, _b = props.defaultHeight, defaultHeight = _b === void 0 ? 72 : _b, copyProps = props.copyProps, _c = props.platform, platform = _c === void 0 ? '' : _c, xssWhiteList = props.xssWhiteList;
    var formatMessage = useTranslation().t; // 多语言
    var isRichTextEmpty = !content;
    var richTextContentDetailDom = null;
    var _d = useState(false), open = _d[0], setOpen = _d[1];
    var _e = useState(false), buttonsVisible = _e[0], setButtonsVisible = _e[1];
    var _f = useState(false), visible = _f[0], setVisible = _f[1];
    var _g = useState([]), viewerImages = _g[0], setViewerImages = _g[1];
    var defaultHeightStyle = useMemo(function () {
        if (!isDefaultOpen && !open) {
            return {
                maxHeight: "".concat(defaultHeight, "px"),
                height: 'auto',
            };
        }
        return {
            height: 'auto',
        };
    }, [isDefaultOpen, open, defaultHeight]);
    var onViewerClose = useCallback(function () {
        setVisible(false);
    }, []);
    var copyText = useMemo(function () {
        return content.replace(new RegExp('<[^>]+>', 'gi'), '');
    }, [content]);
    var onShrink = useCallback(function (e) {
        e.stopPropagation();
        setOpen(false);
    }, [setOpen]);
    var onUnfold = useCallback(function (e) {
        e.stopPropagation();
        setOpen(true);
    }, [setOpen]);
    var showViewer = useCallback(function (url) {
        setViewerImages([
            {
                src: url,
                alt: '',
            },
        ]);
        setVisible(true);
    }, []);
    var resizeViewHeight = function () {
        if (richTextContentDetailDom) {
            var offsetHeight = richTextContentDetailDom === null || richTextContentDetailDom === void 0 ? void 0 : richTextContentDetailDom.offsetHeight;
            var hasImg = /<img/.test(content); // 判断图片 图片加载需要时间 如果有图片直接显示‘展开’
            if ((offsetHeight > defaultHeight || hasImg) && !isDefaultOpen) {
                setButtonsVisible(true);
            }
            var images_1 = richTextContentDetailDom === null || richTextContentDetailDom === void 0 ? void 0 : richTextContentDetailDom.querySelectorAll('img');
            images_1 &&
                images_1.forEach(function (image, index) {
                    image.addEventListener('click', function (event) {
                        var imgs = [];
                        for (var i = index; i <= images_1.length; i++) {
                            var img = images_1[i];
                            if (img === null || img === void 0 ? void 0 : img.src) {
                                imgs.push({
                                    src: img.src,
                                    alt: img === null || img === void 0 ? void 0 : img.title,
                                });
                            }
                        }
                        for (var i = 0; i < index; i++) {
                            var img = images_1[i];
                            if (img === null || img === void 0 ? void 0 : img.src) {
                                imgs.push({
                                    src: img.src,
                                    alt: img === null || img === void 0 ? void 0 : img.title,
                                });
                            }
                        }
                        setViewerImages(imgs);
                        setVisible(true);
                    });
                });
        }
    };
    useEffect(function () {
        resizeViewHeight();
    }, [content]);
    var safeHtmlContent = function (content, xssWhiteList) {
        if (platform === 'email') {
            return replaceScriptCode(safeContent(content));
        }
        return xssByRichEditorWhiteList(content, {
            whiteList: xssWhiteList
        });
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(CopyText, __assign({}, copyProps, { text: copyText }),
            React.createElement("div", { className: "ticketBasicField-richTextView" },
                React.createElement("div", { style: defaultHeightStyle, className: "richTextView-content" }, isRichTextEmpty ?
                    React.createElement("span", { className: "richTextView-content-empty" }, "- -")
                    : React.createElement("div", { ref: function (ref) { return (richTextContentDetailDom = ref); }, className: "richTextView-content-wrapper" },
                        React.createElement("div", { className: "richTextView-container", dangerouslySetInnerHTML: {
                                __html: safeHtmlContent(content, xssWhiteList),
                            } }))),
                buttonsVisible &&
                    React.createElement("div", { className: "richTextView-buttons" }, open ?
                        React.createElement("a", { href: "javascript:;", onClick: onShrink, className: "spread-button m-r-sm" },
                            React.createElement("span", null, formatMessage('labels.packUp')),
                            React.createElement(UpOutlined, null))
                        : React.createElement("a", { href: "javascript:;", onClick: onUnfold, className: "spread-button m-r-sm" },
                            React.createElement("span", null, formatMessage('labels.packDown')),
                            React.createElement(DownOutlined, null))))),
        React.createElement(Viewer, { zIndex: 19999, visible: visible, onClose: onViewerClose, images: viewerImages })));
};
export default RichTextView;
