import { __assign, __rest, __spreadArray } from "tslib";
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { Select } from 'udesk-ui';
import { getFormatOptions } from '../../utils/format';
import './index.module.scss';
import CopyText from '../copy';
/**
 * 经典级联组件
 */
var FormMultipleSelect = React.memo(function (props) {
    var copyProps = props.copyProps, options = props.options, showSearch = props.showSearch, allowClear = props.allowClear, readOnly = props.readOnly, disabled = props.disabled, classNames = props.classNames, fullSelect = props.fullSelect, fieldNames = props.fieldNames, onChange = props.onChange, valueProps = props.value, placeholder = props.placeholder, _a = props.inALine, inALine = _a === void 0 ? true : _a, restProps = __rest(props, ["copyProps", "options", "showSearch", "allowClear", "readOnly", "disabled", "classNames", "fullSelect", "fieldNames", "onChange", "value", "placeholder", "inALine"]);
    var _b = useState([]), valueList = _b[0], setValueList = _b[1];
    var changeValue = useRef([]);
    var multipleSelectRef = useRef(null);
    useEffect(function () {
        setValueList(function () {
            if (typeof valueProps === 'string') {
                return valueProps ? valueProps.split(',') : [];
            }
            return valueProps || [];
        });
    }, [valueProps]);
    var _c = useMemo(function () {
        var _a = fieldNames || {}, _b = _a.label, label = _b === void 0 ? 'label' : _b, _c = _a.value, value = _c === void 0 ? 'value' : _c, _d = _a.children, children = _d === void 0 ? 'children' : _d, _e = _a.is_hide, is_hide = _e === void 0 ? 'is_hide' : _e;
        return getFormatOptions(options, { label: label, value: value, children: children, is_hide: is_hide });
    }, [options, fieldNames]), formatOptions = _c[0], formatNoHideOptions = _c[1];
    var valueObj = useMemo(function () {
        return (valueList || []).reduce(function (prev, current, index) {
            var _options = !index ? formatOptions : (prev[index - 1] || {}).children || [];
            var keyOption = _options.find(function (item) { return current === item.value; });
            keyOption && prev.push(keyOption);
            return prev;
        }, []);
    }, [formatOptions, valueList]);
    var optionsList = useMemo(function () {
        var newList = [];
        if (valueObj && valueObj.length > 0) {
            var valueNoHideObj = valueObj.reduce(function (prev, current, index) {
                var _options = index === 0 ? formatNoHideOptions : prev[index - 1];
                var keyOption = _options.find(function (item) { return current.value === item.value; });
                if (keyOption && keyOption.children) {
                    prev.push(keyOption.children);
                }
                else if (index !== valueObj.length - 1) {
                    prev.push([]);
                }
                return prev;
            }, []);
            newList = __spreadArray([formatNoHideOptions], valueNoHideObj, true);
        }
        else {
            newList.push(formatNoHideOptions);
        }
        return newList;
    }, [formatNoHideOptions, valueObj]);
    var showValueText = useMemo(function () {
        if (!valueObj.length) {
            return '- -';
        }
        return valueObj
            .map(function (item) {
            return (item === null || item === void 0 ? void 0 : item.label) || '- -';
        })
            .join(' / ');
    }, [valueObj]);
    var calcClassnames = useMemo(function () {
        if (inALine) {
            return 'form-multipleSelect-wrapper';
        }
        else {
            return 'form-multipleSelect-wrapper form-multipleSelect-wrapper-block';
        }
    }, [inALine]);
    if (readOnly) {
        return (React.createElement(CopyText, __assign({}, copyProps, { text: showValueText }),
            React.createElement("span", { className: "".concat(classNames) }, showValueText),
            ";"));
    }
    var onValueChange = useCallback(function (_value, _option, _index) {
        var _a;
        var valueAry = valueList.slice(0, _index);
        var valueObjAry = valueObj.slice(0, _index);
        if (_value) {
            valueAry.push(_value);
            valueObjAry.push(_option);
        }
        else if (valueAry.length) {
            valueAry.push('');
            valueObjAry.push({});
        }
        if (((_a = _option === null || _option === void 0 ? void 0 : _option.children) === null || _a === void 0 ? void 0 : _a.length) && fullSelect) {
            valueAry.push('');
        }
        changeValue.current = valueAry;
        onChange(valueAry, valueObjAry);
    }, [onChange, valueList]);
    var filterOption = useCallback(function (input, option) {
        var _a, _b;
        return ((_b = (_a = option === null || option === void 0 ? void 0 : option.label) === null || _a === void 0 ? void 0 : _a.toLowerCase()) === null || _b === void 0 ? void 0 : _b.indexOf(input === null || input === void 0 ? void 0 : input.toLowerCase())) >= 0;
    }, []);
    var onItemBlur = function (index) {
        // 避免多次推送，开启最末级必填后最末级触发blur
        if (fullSelect && changeValue.current[changeValue.current.length - 1] === '') {
            return;
        }
        var onValueBlur = props.onValueBlur, _a = props.id, id = _a === void 0 ? '' : _a;
        onValueBlur === null || onValueBlur === void 0 ? void 0 : onValueBlur(id, changeValue.current);
    };
    var onItemClear = function (index) {
        if (fullSelect && index !== 0) {
            return;
        }
        var onValueBlur = props.onValueBlur, _a = props.id, id = _a === void 0 ? '' : _a;
        onValueBlur === null || onValueBlur === void 0 ? void 0 : onValueBlur(id, (changeValue.current || []).slice(0, index));
    };
    return (React.createElement(CopyText, __assign({}, copyProps, { text: showValueText }),
        React.createElement("div", { className: calcClassnames, ref: multipleSelectRef }, optionsList.map(function (options, index) {
            return (React.createElement("div", { className: "form-multipleSelect-item", style: {
                    marginRight: optionsList.length > 1 && index !== optionsList.length - 1
                        ? '5px'
                        : '',
                }, key: index },
                React.createElement(Select, __assign({ getPopupContainer: function () { return multipleSelectRef.current || document.body; }, dropdownMatchSelectWidth: (options === null || options === void 0 ? void 0 : options.length) > 300 ? true : false }, restProps, { disabled: disabled, value: valueObj[index], placeholder: placeholder, allowClear: allowClear, showSearch: showSearch, options: options, filterOption: filterOption, onChange: function (value, options) {
                        return onValueChange(value, options, index);
                    }, onBlur: function () { return onItemBlur(index); }, onClear: function () { return onItemClear(index); } }))));
        }))));
});
FormMultipleSelect.defaultProps = {
    classNames: '',
    disabled: false,
    readOnly: false,
    showSearch: true,
    allowClear: true,
    placeholder: '',
    fullSelect: false,
    options: [],
    value: [],
    onChange: function () { },
};
export default FormMultipleSelect;
