import { __assign, __rest } from "tslib";
import { PlusOutlined, MinusOutlined } from '@ant-design/icons';
import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { Button, Divider, Select, Tag } from 'udesk-ui';
import { getFormatOptions, markOptionUsedByValue } from '../../utils/format';
import CopyText from '../copy';
import { useTranslation } from '@udesk-webapp/locale';
/**
 * 自定义字段 单选/多选
 * 默认labelInValue
 */
var FormSelect = function (props) {
    var _a = props.allowClear, allowClear = _a === void 0 ? true : _a, copyProps = props.copyProps, classNames = props.classNames, _b = props.showSearch, showSearch = _b === void 0 ? true : _b, value = props.value, readOnly = props.readOnly, options = props.options, fieldNames = props.fieldNames, mode = props.mode, placeholder = props.placeholder, _c = props.labelInValue, labelInValue = _c === void 0 ? true : _c, _d = props.checkAll, checkAll = _d === void 0 ? false : _d, dropdownMatchSelectWidth = props.dropdownMatchSelectWidth, restProps = __rest(props, ["allowClear", "copyProps", "classNames", "showSearch", "value", "readOnly", "options", "fieldNames", "mode", "placeholder", "labelInValue", "checkAll", "dropdownMatchSelectWidth"]);
    var t = useTranslation().t;
    var _e = useState(), curValue = _e[0], setCurValue = _e[1];
    var initValuesRef = React.useRef(value);
    useEffect(function () {
        if (mode === 'multiple' || mode === 'tags') {
            setCurValue(function () {
                if (typeof value === 'string') {
                    return value ? value.split(',') : [];
                }
                else if (typeof value === 'number') {
                    return [value];
                }
                return value || [];
            });
        }
        else {
            setCurValue(value);
        }
    }, [value, mode]);
    var _f = useMemo(function () {
        var _a = fieldNames || {}, _b = _a.label, label = _b === void 0 ? 'label' : _b, _c = _a.value, value = _c === void 0 ? 'value' : _c, _d = _a.is_hide, is_hide = _d === void 0 ? 'is_hide' : _d;
        return getFormatOptions(options, { label: label, value: value, is_hide: is_hide });
    }, [options, fieldNames]), formatOptions = _f[0], formatNoHideOptions = _f[1];
    var formatShowOptions = useMemo(function () {
        var result = [];
        var _a = fieldNames || {}, _b = _a.label, labelKey = _b === void 0 ? 'label' : _b, _c = _a.value, valueKey = _c === void 0 ? 'value' : _c, _d = _a.is_hide, is_hideKey = _d === void 0 ? 'is_hide' : _d;
        var markedOptions = markOptionUsedByValue(options, initValuesRef.current) || [];
        var _checkedList = [];
        if (!Array.isArray(value)) {
            if (typeof value === 'string') {
                _checkedList = value.split(',');
            }
            else {
                _checkedList = value ? [value] : [];
            }
        }
        else if ((value === null || value === void 0 ? void 0 : value.length) > 0 && typeof value[0] === 'object') {
            _checkedList = value.map(function (item) { return (item === null || item === void 0 ? void 0 : item[valueKey]) || ''; });
        }
        else {
            _checkedList = value || [];
        }
        markedOptions.forEach(function (item) {
            var isSelected = _checkedList != null && _checkedList.includes(item[valueKey]);
            var newItem = {
                label: item[labelKey],
                value: item[valueKey],
                disabled: (item[is_hideKey] && !isSelected)
            };
            if (!item[is_hideKey] || item.isUsed) {
                result.push(newItem);
            }
        });
        return result;
    }, [options, value, fieldNames]);
    var valueObj = useMemo(function () {
        if (mode === 'multiple' || mode === 'tags') {
            if (!curValue)
                return [];
            if ((labelInValue &&
                curValue.length > 0 &&
                (typeof curValue[0] === 'string' || typeof curValue[0] === 'number')) ||
                !labelInValue || (typeof curValue[0] === 'object' && !curValue[0].label)) {
                var _values = curValue.reduce(function (initValues, currentValue, index) {
                    var keyOption = formatOptions.find(function (item) { var _a; return (currentValue === null || currentValue === void 0 ? void 0 : currentValue.toString()) === ((_a = item.value) === null || _a === void 0 ? void 0 : _a.toString()); }) || { value: currentValue, label: currentValue };
                    initValues.push(keyOption);
                    return initValues;
                }, []);
                return _values;
            }
            return curValue;
        }
        else {
            if (!curValue)
                return '';
            if ((labelInValue && (typeof curValue === 'string' || typeof curValue === 'number')) ||
                !labelInValue || (typeof curValue === 'object' && !curValue.label)) {
                var keyOption = formatOptions.find(function (item) { var _a; return (curValue === null || curValue === void 0 ? void 0 : curValue.toString()) === ((_a = item.value) === null || _a === void 0 ? void 0 : _a.toString()); }) || { value: curValue, label: curValue };
                return keyOption;
            }
            return curValue;
        }
    }, [formatOptions, curValue, mode]);
    var showValueText = useMemo(function () {
        if (mode === 'multiple' || mode === 'tags') {
            return valueObj.map(function (item) { return item.label; }).toString();
        }
        return (valueObj === null || valueObj === void 0 ? void 0 : valueObj.label) || '';
    }, [curValue]);
    var isCheckAll = useMemo(function () { return (curValue === null || curValue === void 0 ? void 0 : curValue.length) === (formatShowOptions === null || formatShowOptions === void 0 ? void 0 : formatShowOptions.length); }, [curValue, formatShowOptions]);
    var onCheckAll = useCallback(function (e) {
        var _a, _b;
        e.preventDefault();
        if (!isCheckAll) {
            (_a = props === null || props === void 0 ? void 0 : props.onChange) === null || _a === void 0 ? void 0 : _a.call(props, formatShowOptions);
        }
        else {
            (_b = props === null || props === void 0 ? void 0 : props.onChange) === null || _b === void 0 ? void 0 : _b.call(props, []);
        }
    }, [isCheckAll, formatShowOptions]);
    if (readOnly) {
        var textRender = void 0;
        if (mode === 'multiple' || mode === 'tags') {
            textRender = valueObj.map(function (item, index) { return (React.createElement(Tag, { style: { marginRight: 3 }, key: index }, item.label)); });
        }
        else {
            textRender = React.createElement("span", { className: "".concat(classNames) }, showValueText);
        }
        return (React.createElement(CopyText, __assign({}, copyProps, { text: showValueText }),
            React.createElement(React.Fragment, null, textRender)));
    }
    var onValueChange = function (value) {
        var _a;
        if (value === undefined)
            value = '';
        (_a = props === null || props === void 0 ? void 0 : props.onChange) === null || _a === void 0 ? void 0 : _a.call(props, value);
    };
    return (React.createElement(CopyText, __assign({}, copyProps, { text: showValueText }),
        React.createElement(Select, __assign({ dropdownRender: function (menu) { return (React.createElement(React.Fragment, null,
                menu,
                checkAll && mode === 'multiple' && (React.createElement(React.Fragment, null,
                    React.createElement(Divider, { style: { margin: '8px 0' } }),
                    React.createElement(Button, { type: 'text', icon: isCheckAll ? React.createElement(MinusOutlined, null) : React.createElement(PlusOutlined, null), onClick: onCheckAll }, isCheckAll ? t('labels.noAllSelect') : t('labels.allSelect')))))); }, getPopupContainer: function (triggerNode) { return triggerNode; }, dropdownMatchSelectWidth: (formatShowOptions === null || formatShowOptions === void 0 ? void 0 : formatShowOptions.length) > 300 ? true : false }, restProps, { allowClear: allowClear, mode: mode, labelInValue: labelInValue, showSearch: showSearch, optionFilterProp: 'label', 
            // filterOption={(input, option) =>
            //   option.children.toLowerCase().indexOf(input.toLowerCase()) >= 0
            // }
            placeholder: placeholder, 
            //@ts-ignore
            value: curValue, onChange: onValueChange, onBlur: function () {
                var onValueBlur = props.onValueBlur, _a = props.id, id = _a === void 0 ? '' : _a;
                onValueBlur === null || onValueBlur === void 0 ? void 0 : onValueBlur(id, curValue);
            }, onClear: function () {
                // 这里模拟Blur事件用于字段推送
                var onValueBlur = props.onValueBlur, _a = props.id, id = _a === void 0 ? '' : _a;
                onValueBlur === null || onValueBlur === void 0 ? void 0 : onValueBlur(id, mode ? [] : '');
            }, options: formatShowOptions }))));
};
export default FormSelect;
