/* eslint-disable jsx-a11y/anchor-is-valid */
import React, { useEffect, useRef, useLayoutEffect } from 'react';
import { useTranslation } from '@udesk-webapp/locale';
import Viewer from 'viewerjs';
import FileIcon from '../file-icon';
import { getPreviewUrl } from '../../utils/common';
import { TrashIcon } from '@udesk-webapp/icon';
import { Progress } from 'udesk-ui';
// 支持预览的图片类型
var isImage = function (name) {
    if (/\.(jpg|png|gif|jpeg|bmp|tif|psd)$/.test(name)) {
        return true;
    }
    return false;
};
// 支持预览的文档类型
var isDoc = function (name) {
    if (/.(doc|docx|odt|rtf|wps|ppt|pptx|odp|dp|xls|xlsx|ods|csv|et|pdf)$/.test(name)) {
        return true;
    }
    return false;
};
var FileItem = function (props) {
    var formatMessage = useTranslation().t; // 多语言
    var file = props.file, _a = props.previewUrl, previewUrl = _a === void 0 ? '' : _a, _b = props.udeskVersion, udeskVersion = _b === void 0 ? '' : _b, _c = props.disabled, disabled = _c === void 0 ? false : _c, remove = props.remove, download = props.download;
    var name = (file === null || file === void 0 ? void 0 : file.file_name) || (file === null || file === void 0 ? void 0 : file.name) || '';
    var type = name.split('.').pop() || '';
    var ref = useRef();
    var viewer = useRef();
    useLayoutEffect(function () {
        if (!ref.current) {
            return;
        }
        viewer.current = new Viewer(ref.current);
        return function () {
            var _a;
            (_a = viewer.current) === null || _a === void 0 ? void 0 : _a.destroy();
        };
    }, []);
    useEffect(function () {
        var _a;
        (_a = viewer.current) === null || _a === void 0 ? void 0 : _a.update();
    }, [file === null || file === void 0 ? void 0 : file.url]);
    // 图片及文档预览
    var onPreview = function () {
        var _a, _b;
        var u = file === null || file === void 0 ? void 0 : file.url;
        if (/\.(jpg|png|gif|jpeg|bmp|tif|psd)$/.test(name)) {
            (_b = (_a = viewer.current).view) === null || _b === void 0 ? void 0 : _b.call(_a, 0);
            return;
        }
        else if (previewUrl) {
            u = getPreviewUrl(file === null || file === void 0 ? void 0 : file.url, {
                previewUrl: previewUrl
            });
        }
        window.open(u, '_blank');
    };
    var renderProgress = function () {
        if ((file === null || file === void 0 ? void 0 : file.status) !== 'uploading') {
            return;
        }
        return (React.createElement(Progress, { percent: (file === null || file === void 0 ? void 0 : file.percent) || 0, showInfo: false, strokeWidth: 2 }));
    };
    if (isImage(name) || (isDoc(name) && previewUrl && udeskVersion === 'pro')) {
        return (React.createElement("div", { ref: ref, className: 'form-upload-custom-file-item' },
            React.createElement(FileIcon, { type: type }),
            React.createElement("span", null, name),
            React.createElement("img", { src: file === null || file === void 0 ? void 0 : file.url, style: { display: 'none' }, alt: name }),
            React.createElement("a", { className: "mgl-10", role: "button", "data-ignore-switch-editing": "1", onClick: onPreview }, formatMessage('labels.preview')),
            React.createElement("a", { href: file === null || file === void 0 ? void 0 : file.url, className: "mgl-5", "data-ignore-switch-editing": "1", target: "_blank", download: name, rel: "noreferrer", onClick: function (e) {
                    e.stopPropagation();
                    download === null || download === void 0 ? void 0 : download(file);
                } }, formatMessage('labels.download')),
            disabled ? null : (React.createElement("span", { className: 'form-upload-icon-delete', onClick: function () { return remove === null || remove === void 0 ? void 0 : remove(file); } },
                React.createElement(TrashIcon, null))),
            React.createElement("div", null, renderProgress())));
    }
    return (React.createElement("div", { ref: ref, className: 'form-upload-custom-file-item' },
        React.createElement("a", { className: "link", href: file === null || file === void 0 ? void 0 : file.url, "data-ignore-switch-editing": "1", target: "_blank", rel: "noreferrer", onClick: function (e) {
                e.stopPropagation();
                download === null || download === void 0 ? void 0 : download(file);
            } },
            React.createElement(FileIcon, { type: type }),
            name),
        disabled ? null : (React.createElement("span", { className: 'form-upload-icon-delete', onClick: function () { return remove === null || remove === void 0 ? void 0 : remove(file); } },
            React.createElement(TrashIcon, null))),
        React.createElement("div", null, renderProgress())));
};
export default FileItem;
