import { __assign, __rest, __spreadArray } from "tslib";
// @ts-nocheck
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import uniqBy from 'lodash-es/uniqBy';
import UploadBox from '@udesk-webapp/upload-box';
import { Button, Tooltip } from 'udesk-ui';
import { replaceTemplateVar } from '@udesk-webapp/utils';
import { UploadOutlined } from '@udesk/icons';
import FileIcon from '../file-icon';
import FileItem from './FileItem';
import './index.module.scss';
import { useTranslation } from '@udesk-webapp/locale';
var iconRender = function (file) {
    var _a;
    var type = ((_a = file === null || file === void 0 ? void 0 : file.name) === null || _a === void 0 ? void 0 : _a.split('.').pop()) || '';
    return React.createElement(FileIcon, { type: type });
};
var EMPTY_ARRAY = [];
var IMAGE_SIZE_LIMIT = 2;
var AUDIO_SIZE_LIMIT = 2;
var VIDEO_SIZE_LIMIT = 10;
var FILE_SIZE_LIMIT = 20;
var FILE_NUMBER_LIMIT = 5;
var SINGLE_SIZE_LIMIT = 50;
var TOTAL_SIZE_LIMIT = 50;
var IMAGE_TYPE = ['.gif', '.jpeg', '.jpg', '.png'];
var AUDIO_TYPE = ['.mp3', '.amr'];
var VIDEO_TYPE = ['.mp4'];
var FILE_TYPE = [
    '.txt',
    '.markdown',
    '.md',
    '.pdf',
    '.doc',
    '.docx',
    '.xls',
    '.xlsx',
    '.excel',
    '.execlx',
    '.ppt',
    '.pptx',
];
var FormUpload = function (props, ref) {
    var valueProps = props.value, onChange = props.onChange, _a = props.singleLimit, singleLimit = _a === void 0 ? SINGLE_SIZE_LIMIT : _a, _b = props.totalLimit, totalLimit = _b === void 0 ? TOTAL_SIZE_LIMIT : _b, _c = props.marking, marking = _c === void 0 ? 'web.ts.create_ticket' : _c, buttonText = props.buttonText, buttonProps = props.buttonProps, uploadProps = props.uploadProps, _d = props.uploadType, uploadType = _d === void 0 ? '' : _d, _e = props.sourceChannel, sourceChannel = _e === void 0 ? '' : _e, _f = props.previewUrl, previewUrl = _f === void 0 ? '' : _f, _g = props.udeskVersion, udeskVersion = _g === void 0 ? '' : _g, weirdMode = props.weirdMode, tipRender = props.tipRender, _h = props.hoveTip, hoveTip = _h === void 0 ? false : _h, _j = props.tooltipPlacement, tooltipPlacement = _j === void 0 ? 'topLeft' : _j, singleFileOutOfSizeFun = props.singleFileOutOfSizeFun, totalFileOutOfSizeFun = props.totalFileOutOfSizeFun;
    var formatMessage = useTranslation().t; // 多语言
    var _k = uploadProps || {}, _l = _k.multiple, multiple = _l === void 0 ? true : _l, _m = _k.listType, listType = _m === void 0 ? 'text' : _m, accept = _k.accept, maxCount = _k.maxCount, restProps = __rest(_k, ["multiple", "listType", "accept", "maxCount"]);
    var uploadRef = useRef();
    var formatUpdateList = useCallback(function (valueProps) {
        var _a;
        if (!(valueProps === null || valueProps === void 0 ? void 0 : valueProps.length)) {
            return EMPTY_ARRAY;
        }
        return (_a = valueProps.map) === null || _a === void 0 ? void 0 : _a.call(valueProps, function (i) { return (__assign({ uid: i.id + '', name: i.file_name || i.name || '', status: 'done', url: i.url || '' }, i)); });
    }, []);
    var _o = useState(formatUpdateList(valueProps)), fileList = _o[0], setFileList = _o[1];
    useEffect(function () {
        // 这里仅处理需要受控新增文件的逻辑, 默认status: 'done'
        setFileList(formatUpdateList(valueProps));
    }, [valueProps, formatUpdateList]);
    // 另一种方案：这里仅处理需要受控新增文件的逻辑
    // useEffect(() => {
    //     if (valueProps && valueProps.length && valueProps.length > fileList.length) {
    //         valueProps.forEach(_file => {
    //             const isNew = fileList.some(v => v?.url === _file?.url);
    //             if (!isNew) {
    //                 addFileToList(_file);
    //             }
    //         });
    //     }
    // }, [valueProps, fileList]);
    // const addFileToList = useCallback((fileInfo) => {
    //     fileInfo && uploadRef.current.addItem(fileInfo);
    // }, [])
    var onFileChange = function (list) {
        var fileList = (list || []).reduce(function (items, currentItem) {
            var id = currentItem.id, file_name = currentItem.name, url = currentItem.url;
            if (parseInt(id)) {
                return uniqBy(__spreadArray(__spreadArray([], items, true), [{ id: id, file_name: file_name, url: url }], false), 'id');
            }
            return items;
        }, []);
        onChange === null || onChange === void 0 ? void 0 : onChange(fileList);
    };
    var makeAccept = useMemo(function () {
        if (accept) {
            return accept;
        }
        else if (uploadType === 'img') {
            return IMAGE_TYPE;
        }
        else if (sourceChannel === 'wechat') {
            return __spreadArray(__spreadArray([], AUDIO_TYPE, true), VIDEO_TYPE, true);
        }
        else if (sourceChannel === 'qywx') {
            return __spreadArray(__spreadArray(['.amr'], VIDEO_TYPE, true), FILE_TYPE, true);
        }
        return [];
    }, [sourceChannel, uploadType, accept]);
    var uploadTip = useMemo(function () {
        // 自定义提示
        if (tipRender) {
            return { __html: tipRender };
        }
        var highlightText = function (number) {
            return "<span style=\"color:#e80000\" >".concat(number, "</span>");
        };
        var tipText = '';
        var numberText = highlightText(FILE_NUMBER_LIMIT);
        var imageText = highlightText("".concat(IMAGE_SIZE_LIMIT, "M"));
        var audioText = highlightText("".concat(AUDIO_SIZE_LIMIT, "M"));
        var videoText = highlightText("".concat(VIDEO_SIZE_LIMIT, "M"));
        var fileText = highlightText("".concat(FILE_SIZE_LIMIT, "M"));
        var singleText = highlightText("".concat(singleLimit, "M"));
        var totalText = highlightText("".concat(totalLimit, "M"));
        switch (uploadType) {
            case 'img':
                // tipText = formatMessage(
                //     { id: 'components.upload.imgTip' },
                //     {
                //         number: highlightText(FILE_NUMBER_LIMIT),
                //         image: highlightText(`${IMAGE_SIZE_LIMIT}M`),
                //         total: highlightText(`${totalLimit}M`),
                //     }
                // );
                tipText = replaceTemplateVar(formatMessage('components.upload.imgTip'), numberText, imageText, totalText);
                break;
            case 'file':
                // tipText = formatMessage(
                //     { id: 'components.upload.fileTip' },
                //     {
                //         number: highlightText(FILE_NUMBER_LIMIT),
                //         audio: highlightText(`${AUDIO_SIZE_LIMIT}M`),
                //         video: highlightText(`${VIDEO_SIZE_LIMIT}M`),
                //         file: highlightText(`${FILE_SIZE_LIMIT}M`),
                //         total: highlightText(`${totalLimit}M`),
                //     }
                // );
                tipText = replaceTemplateVar(formatMessage('components.upload.fileTip'), numberText, audioText, videoText, fileText, totalText);
                break;
            default:
                // tipText = formatMessage(
                //     { id: 'components.upload.defaultTip' },
                //     {
                //         single: highlightText(`${singleLimit}M`),
                //         total: highlightText(`${totalLimit}M`),
                //     }
                // );
                tipText = replaceTemplateVar(formatMessage('components.upload.defaultTip'), singleText, totalText);
                break;
        }
        return { __html: tipText };
    }, [uploadType, singleLimit, totalLimit, formatMessage, tipRender]);
    var uploadText = useMemo(function () {
        if (buttonText) {
            return buttonText;
        }
        var text = '';
        switch (uploadType) {
            case 'img':
                // text = formatMessage({ id: 'components.upload.imgText' });
                text = formatMessage('components.upload.imgText');
                break;
            case 'file':
                // text = formatMessage({ id: 'components.upload.fileText' });
                text = formatMessage('components.upload.fileText');
                break;
            default:
                // text = formatMessage({ id: 'components.upload.defaultText' });
                text = formatMessage('components.upload.defaultText');
                break;
        }
        return text;
    }, [uploadType, buttonText, formatMessage]);
    // TODO:后面增加外面配置limit
    var fileNumberLimit = useMemo(function () {
        if (maxCount) {
            return maxCount;
        }
        else if (uploadType === 'img' || uploadType === 'file') {
            return FILE_NUMBER_LIMIT;
        }
        else {
            return 0;
        }
    }, [uploadType, maxCount]);
    var fileTypeLimit = useMemo(function () {
        if (uploadType === 'img' || uploadType === 'file') {
            return {
                imageLimit: IMAGE_SIZE_LIMIT,
                audioLimit: AUDIO_SIZE_LIMIT,
                videoLimit: VIDEO_SIZE_LIMIT,
                fileLimit: FILE_SIZE_LIMIT,
            };
        }
        return {};
    }, [uploadType]);
    // 自定义文件渲染
    var itemRender = function (originNode, file, fileList, actions) {
        var remove = actions.remove, download = actions.download;
        var _a = (uploadProps || {}).disabled, disabled = _a === void 0 ? false : _a;
        return (React.createElement(FileItem, { file: file, previewUrl: previewUrl, udeskVersion: udeskVersion, remove: remove, download: download, disabled: disabled }));
    };
    return (React.createElement(UploadBox, __assign({}, restProps, { weirdMode: weirdMode, 
        // showUploadList // 默认true
        // name={'file'}
        ref: uploadRef, accept: makeAccept, iconRender: iconRender, itemRender: itemRender, headers: {}, listType: listType, multiple: multiple, marking: marking, onChange: onFileChange, updateList: fileList, single: singleLimit * 1024 * 1024, total: totalLimit * 1024 * 1024, fileTypeLimit: fileTypeLimit, noAttachment: true, fileNumberLimit: fileNumberLimit, className: listType === 'picture' ? 'form-upload-list-inline' : '', singleFileOutOfSizeFun: singleFileOutOfSizeFun, totalFileOutOfSizeFun: totalFileOutOfSizeFun }),
        hoveTip ?
            React.createElement(Tooltip, { placement: tooltipPlacement, title: React.createElement("span", { dangerouslySetInnerHTML: uploadTip }) },
                React.createElement(Button, __assign({}, buttonProps),
                    React.createElement(UploadOutlined, null),
                    uploadText))
            :
                React.createElement(Button, __assign({}, buttonProps),
                    React.createElement(UploadOutlined, null),
                    uploadText),
        !hoveTip && React.createElement("span", { style: { color: 'rgba(0, 0, 0, 0.45)' }, dangerouslySetInnerHTML: uploadTip })));
};
function areValueEqual(prevValue, value) {
    return (prevValue === null || prevValue === void 0 ? void 0 : prevValue.length) === (value === null || value === void 0 ? void 0 : value.length) && (prevValue === null || prevValue === void 0 ? void 0 : prevValue.every(function (item) {
        return value === null || value === void 0 ? void 0 : value.find(function (_value) {
            return (_value === null || _value === void 0 ? void 0 : _value.id) === (item === null || item === void 0 ? void 0 : item.id) && (_value === null || _value === void 0 ? void 0 : _value.url) === (item === null || item === void 0 ? void 0 : item.url) && (_value === null || _value === void 0 ? void 0 : _value.file_name) === (item === null || item === void 0 ? void 0 : item.file_name);
        });
    }));
}
// value每次传递进来都是变化的，这里自定义对比方法做到更精确控制，避免死循环
export default React.memo(FormUpload, function (prevProps, props) {
    var keys = Reflect.ownKeys(prevProps);
    return keys.every(function (key) {
        if (key === 'value') {
            return areValueEqual(prevProps[key], props[key]);
        }
        else {
            // eslint-disable-next-line
            return prevProps[key] == props[key];
        }
    });
});
