import React, { useCallback, useEffect, useState } from "react";
import { Col, message, Modal, Row } from "udesk-ui";
import { useTranslation } from "react-i18next";
import CustomerSelect from "./customer-select";
import style from "./index.module.scss";
import { Change } from "../../../../components/icon/icon";
import isEmpty from "lodash-es/isEmpty";
import { postJSON } from "../../../../utils";
import { csApiPostJSON } from "../../../../utils/csApi";
var Merge = function (props) {
    var t = useTranslation().t;
    var title = props.title, concatVisible = props.concatVisible, repeatCustomer = props.repeatCustomer, cancelCombineCustomer = props.cancelCombineCustomer, imSubSessionId = props.imSubSessionId, customer = props.customer, reason = props.reason, history = props.history, mergeFn = props.mergeFn;
    var _a = useState(false), trun = _a[0], setTrun = _a[1];
    var showError = useState(false)[0];
    var _b = useState({}), mergeCustomer = _b[0], setMergeCustomer = _b[1];
    useEffect(function () {
        setMergeCustomer(repeatCustomer || null);
    }, [repeatCustomer]);
    var handleMerge = useCallback(function () {
        if (isEmpty(mergeCustomer)) {
            return message.error(t("udMerge.selectTargetCustomer"));
        }
        // 当前客户id
        var currentId = trun ? mergeCustomer.value : customer.id;
        // 目标客户id
        var mergeId = trun ? customer.id : mergeCustomer.value;
        if (currentId === mergeId) {
            return message.error(t("udMerge.sameCustomer"));
        }
        // 历史页面直接合并
        if (history) {
            csApiPostJSON("/cs/ts/api/customers/".concat(currentId, "/merge"), {
                form_id: mergeId,
            })
                .then(function (res) { return res.data; })
                .then(function (res) {
                if (res.code === 1000) {
                    message.success(t("udMerge.mergeSuccess"));
                    mergeFn && mergeFn(trun, mergeCustomer, true);
                    cancelCombineCustomer(false);
                }
                else {
                    message.error(res.code_message || t("udMerge.mergeFail"));
                }
            })
                .catch(function (e) {
                message.error(t("udMerge.mergeFail"));
            });
        }
        else {
            // 当前聊天页面, 预约合并
            postJSON("/customer_merge_reservation", {
                from_customer_id: currentId,
                to_customer_id: mergeId,
                im_sub_session_id: imSubSessionId,
                reservation_reason: reason,
            })
                .then(function (res) {
                if (res.code === 1000) {
                    message.success(t("udMerge.appointmentMergeSuccess"));
                    mergeFn && mergeFn(false, mergeCustomer, false);
                    cancelCombineCustomer(false);
                }
                else {
                    message.error(res.code_message || t("udMerge.appointmentMergeFail"));
                }
            })
                .catch(function () {
                message.error(t("udMerge.mergeFail"));
            });
        }
    }, [
        cancelCombineCustomer,
        customer.id,
        history,
        imSubSessionId,
        mergeCustomer,
        mergeFn,
        reason,
        t,
        trun,
    ]);
    // const handleMerge = () => {
    //     if (isEmpty(mergeCustomer)) {
    //         return message.error(t('udMerge.selectTargetCustomer'));
    //     }
    //     // 当前客户id
    //     let currentId = trun ? mergeCustomer.value : customer.id
    //     // 目标客户id
    //     let mergeId = trun ? customer.id : mergeCustomer.value
    //     if (currentId === mergeId) {
    //         return message.error(t('udMerge.sameCustomer'));
    //     }
    //     // 历史页面直接合并
    //     if (history) {
    //         postJSON(`/customers/${currentId}/merge`, {
    //             form_id: mergeId
    //         }).then((res: any) => {
    //             if (res.code === 1000) {
    //                 message.success(t('udMerge.mergeSuccess'));
    //                 props.mergeFn && props.mergeFn(trun, mergeCustomer, true);
    //                 cancelCombineCustomer();
    //             } else {
    //                 message.error(res.code_message || t('udMerge.mergeFail'));
    //             }
    //         }).catch((e) => {
    //             message.error(t('udMerge.mergeFail'));
    //         })
    //     } else { // 当前聊天页面, 预约合并
    //         postJSON('/customer_merge_reservation', {
    //             from_customer_id: currentId,
    //             to_customer_id: mergeId,
    //             im_sub_session_id: imSubSessionId,
    //             reservation_reason: reason,
    //         }).then((res: any) => {
    //             if (res.code === 1000) {
    //                 message.success(t('udMerge.appointmentMergeSuccess'));
    //                 props.mergeFn && props.mergeFn(false, mergeCustomer, false);
    //                 cancelCombineCustomer();
    //             } else {
    //                 message.error(res.code_message || t('udMerge.appointmentMergeFail'));
    //             }
    //         }).catch(() => {
    //             message.error(t('udMerge.mergeFail'));
    //         })
    //     }
    // }
    var handleConcatCancel = useCallback(function () {
        cancelCombineCustomer(false);
    }, [cancelCombineCustomer]);
    // const handleConcatCancel = () => {
    //     cancelCombineCustomer();
    // }
    //选择目标客户
    var selectCustomer = useCallback(function (value) {
        setMergeCustomer(value);
    }, []);
    // const selectCustomer = (value: any, option: OptionsType[number] | OptionsType) => {
    //     setMergeCustomer(value);
    // }
    // 判断是否需要显示不能合并信息
    // !! 测试关注
    // useEffect(() => {
    //     if (!isEmpty(mergeCustomer)) {
    //         if (mergeCustomer.can_merge && customer.can_merge) {
    //             setShowError(false);
    //         } else {
    //             setShowError(true);
    //         }
    //     }
    // }, [mergeCustomer.can_merge, customer.can_merge])
    //交换目标客户和原有客户 更改order
    var changeCustomer = useCallback(function () {
        setTrun(function (prev) {
            return !prev;
        });
    }, []);
    // const changeCustomer = () => {
    //     setTrun((prev) => {
    //         return !prev;
    //     });
    // }
    return (React.createElement(React.Fragment, null,
        React.createElement(Modal, { title: title, visible: concatVisible, onOk: handleMerge, okText: t("confirm"), cancelText: t("label.cancel"), onCancel: handleConcatCancel },
            React.createElement("div", { className: style["phone-concat-modal-content"] },
                React.createElement("div", null,
                    React.createElement(Row, null,
                        React.createElement(Col, { span: 8 }, t("udMerge.currentCustomer")),
                        React.createElement(Col, { span: 16 }, trun ? (React.createElement(CustomerSelect, { value: repeatCustomer, selectOnChange: selectCustomer })) : (React.createElement("span", null, customer.nick_name)))),
                    React.createElement("br", null),
                    React.createElement(Row, null,
                        React.createElement(Col, { span: 8 }, t("udMerge.targetCustomer")),
                        React.createElement(Col, { span: 16 }, trun ? (React.createElement("span", null, customer.nick_name)) : (React.createElement(CustomerSelect, { value: repeatCustomer, selectOnChange: selectCustomer }))))),
                React.createElement("div", null,
                    React.createElement(Change, { style: { marginLeft: "15px", fontSize: "50px" }, onClick: changeCustomer }))),
            React.createElement("br", null),
            showError ? (React.createElement("div", null,
                t("udMerge.typeNotMerge"),
                props.companyCategoryConfig &&
                    props.companyCategoryConfig.helpCenterDocumentHide && (
                // eslint-disable-next-line react/jsx-no-target-blank
                React.createElement("a", { href: "http://udesk.udesk.cn/hc/articles/29296", target: "_blank", style: { textDecoration: "underline" } }, "\u4E86\u89E3\u8BE6\u60C5")))) : (React.createElement(Row, null,
                React.createElement(Col, { offset: 2 },
                    React.createElement("p", null, t("udMerge.afterMergeCustomer")),
                    React.createElement("p", null, t("udMerge.mergeCustomerHelpOne")),
                    React.createElement("p", null, t("udMerge.mergeCustomerHelpTwo")),
                    React.createElement("p", null, t("udMerge.mergeCustomerHelpThree"))))))));
};
export default Merge;
