var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { useCallback, useMemo, useRef } from 'react';
import style from '../index.module.scss';
import { Button, Form, message } from 'udesk-ui';
import { useTranslation } from 'react-i18next';
import isEqual from 'lodash-es/isEqual';
import * as FieldComponent from '../../ticket/components/index';
import { formatCustomField, getFieldsDefaultValue } from '../../../utils/index';
import { csApiPostJSON } from '../../../utils/csApi';
import CustomField from '../../../components/custom-field-container';
import Qs from 'qs';
import { useSelector } from '../../../store/store-hook';
function CreateOrg(props) {
    var formRef = useRef(null);
    var t = useTranslation().t;
    var organizationFieldsV2 = useSelector(function (state) { return state.fields.organizationFieldsv2; });
    var initData = useSelector(function (state) { return state.initData; });
    var _organizationFields = useMemo(function () {
        return organizationFieldsV2
            .map(function (i) { return (__assign(__assign({}, i), { is_required: i.agent_permission === 1 })); })
            .filter(function (item) {
            if (isEqual(initData.page, 'im')) {
                return item.agent_permission && item.im_show;
            }
            else if (isEqual(initData.page, 'mpv')) {
                return item.agent_permission && item.mpv_show;
            }
            else {
                return item.agent_permission && item.cc_show;
            }
        });
    }, [organizationFieldsV2]);
    var orgFields = formatCustomField({
        fields: _organizationFields,
    });
    // 提交时触发
    var handleSubmit = useCallback(function (values) {
        var custom_fields = {};
        var sys_fields = {};
        for (var k in values) {
            if (values[k]) {
                if (k.indexOf('TextField') > -1 || k.indexOf('SelectField') > -1) {
                    custom_fields[k] = values[k];
                }
                else {
                    sys_fields[k] = values[k];
                }
            }
        }
        var params = {
            organization: __assign(__assign({}, sys_fields), { custom_fields: __assign({}, custom_fields) }),
        };
        // 开始提交请求
        csApiPostJSON('/cs/ts/api/organizations', params, {
            transformRequest: [
                function (data) {
                    return Qs.stringify(data, { arrayFormat: 'brackets' });
                },
            ],
        }).then(function (res) { return res.data; })
            .then(function (res) {
            // 创建成功后将创建的用户更新   给新增工单页面回显用
            if (res.code === 1000) {
                props.onSave({
                    label: res.organization.name,
                    value: res.organization.id,
                });
            }
            else {
                message.error(t('tip.error.saveTry'));
            }
        })
            .catch(function (err) {
            message.error(t('tip.error.saveTry'));
        });
    }, [props, t]);
    // 表单数据改变触发  开发时方便查看  提测注释掉
    var handleChangeValue = useCallback(function (changeValue, allValue) {
        // console.log(allValue)
        // setFormData(allValue);
    }, []);
    var handleInitialValue = useMemo(function () {
        var initValue = getFieldsDefaultValue({
            fields: _organizationFields,
            valuse: {},
        });
        return Object.assign({
            name: '',
            level: '',
            description: '',
            org_email_suffix: '',
            domains: '',
        }, initValue);
    }, [_organizationFields]);
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { className: style['ticket-form'] },
            React.createElement(Form, { ref: formRef, initialValues: handleInitialValue, onFinish: handleSubmit, layout: "vertical", onValuesChange: handleChangeValue },
                React.createElement(Form.Item, { name: "name", label: t('page.im.customerInfo.organization.name'), required: true, rules: [
                        {
                            required: true,
                            message: t('page.im.customerInfo.organization.validateFail'),
                        },
                    ] },
                    React.createElement(FieldComponent.text, { trigger: "onChange", isRequired: true, id: "name", type: "text" })),
                React.createElement(Form.Item, { name: "level", label: t('page.im.customerInfo.customer.level') },
                    React.createElement(FieldComponent.droplist, { id: "level", type: "droplist", trigger: "onChange", options: [
                            {
                                value: 'normal',
                                title: t('page.im.customerInfo.customer.enum.level.normal'),
                            },
                            {
                                value: 'vip',
                                title: t('page.im.customerInfo.customer.enum.level.vip'),
                            },
                        ] })),
                React.createElement(Form.Item, { name: "description", label: t('page.im.customerInfo.customer.description') },
                    React.createElement(FieldComponent.textarea, { trigger: "onChange", placeholder: t('page.im.ticket.descTip'), id: "description", type: "textarea" })),
                React.createElement(Form.Item, { name: "org_email_suffix", label: t('page.im.customerInfo.organization.org_email_suffix') },
                    React.createElement(FieldComponent.textHasPoint, { remark: t('page.im.ticket.emailSuffixHelpText') })),
                React.createElement(Form.Item, { name: "domains", label: t('page.im.customerInfo.organization.domains') },
                    React.createElement(FieldComponent.textHasPoint, { remark: t('page.im.ticket.helpBlock') })),
                orgFields.length > 1 && (React.createElement("div", { className: style['org-custom-list'] },
                    React.createElement(CustomField, { fields: orgFields }))))),
        React.createElement("div", { style: { textAlign: 'center', marginTop: '15px' } },
            React.createElement(Button, { style: { marginRight: '10px' }, onClick: props.onCancel }, t('cancel')),
            React.createElement(Button, { type: "primary", onClick: function () { return formRef.current.submit(); } }, t('confirm')))));
}
export default CreateOrg;
