import React, { useCallback, useEffect, useMemo, useState } from 'react';
import axios from 'axios';
import { buildHookRequestParamsForCustomer } from '../../../utils/field-hook';
import merge from 'lodash-es/merge';
import { Form, message } from 'udesk-ui';
import { formatCustomField, getFieldsDefaultValue, isSysField } from '../../../utils/index';
import { csApiPutJSON } from '../../../utils/csApi';
import FromList from './component/form-list';
import PlatformList from './component/platform-type';
import Qs from 'qs';
import style from './customer-form.module.scss';
import { useTranslation } from 'react-i18next';
import isEqual from 'lodash-es/isEqual';
import cloneDeep from 'lodash-es/cloneDeep';
import { useSelector } from '../../../store/store-hook';
function judgeValueChanged(changedValue, initialValue) {
    var key = Object.keys(changedValue)[0];
    if (isEqual(initialValue[key], changedValue[key])) {
        return false;
    }
    return true;
}
function getParam(changedValues) {
    var _a, _b;
    var param = {};
    var cellResult = [];
    for (var changedKey in changedValues) {
        var _isSysField = isSysField(changedKey);
        if (_isSysField) {
            switch (changedKey) {
                case 'nick_name':
                case 'email':
                case 'description':
                    merge(param, {
                        customer: (_a = {}, _a[changedKey] = changedValues[changedKey], _a),
                    });
                    break;
                case 'cellphones':
                    cellResult = changedValues[changedKey]
                        .filter(function (item) {
                        return item.id || item.content;
                    })
                        .map(function (cell) {
                        if (cell.id) {
                            if (cell.content) {
                                return cell;
                            }
                            else {
                                return Object.assign({}, cell, {
                                    content: cell.content,
                                    id: cell.id,
                                    _destroy: 1,
                                });
                            }
                        }
                        else {
                            return cell;
                        }
                    });
                    merge(param, {
                        customer: {
                            cellphones_attributes: cellResult,
                        },
                    });
                    break;
                case 'organizations':
                    merge(param, {
                        customer: {
                            organizations: changedValues[changedKey] && changedValues[changedKey].length > 0
                                ? changedValues[changedKey].map(function (item) { return item.value; }).filter(function (t) { return t; })
                                : '',
                        },
                    });
                    break;
                case 'tags':
                    merge(param, {
                        tags: changedValues[changedKey].map(function (item) { return item.label; }).join(','),
                    });
                    break;
            }
        }
        else {
            merge(param, {
                customer: {
                    custom_fields: (_b = {}, _b[changedKey] = changedValues[changedKey], _b),
                },
            });
        }
    }
    return param;
}
var CustomerInfoForm = function (_a) {
    var currentUser = _a.currentUser, smsapp = _a.smsapp, _b = _a.chat, chat = _b === void 0 ? {} : _b, company = _a.company, roleCan = _a.roleCan, customerFieldsv2 = _a.customerFieldsv2, organizationFields = _a.organizationFields, event = _a.event, companyCategoryConfig = _a.companyCategoryConfig, switchFlag = _a.switchFlag, customerFieldsAuthority = _a.customerFieldsAuthority, fieldHooks = _a.fieldHooks, isAdmin = _a.isAdmin, user = _a.user, agents = _a.agents;
    var form = Form.useForm()[0];
    var _c = useState({}), initialValue = _c[0], setInitialValue = _c[1];
    var _d = useState(), lastValues = _d[0], setLastValues = _d[1];
    var t = useTranslation().t;
    var initData = useSelector(function (state) { return state.initData; });
    var customer = chat.customer, organizations = chat.organizations, imSubSessionId = chat.im_sub_session_id, sessionKey = chat.session_key;
    var _customerFieldsv2 = useMemo(function () {
        var customerFieldsv2_bk = cloneDeep(customerFieldsv2);
        if (customerFieldsv2_bk) {
            return customerFieldsv2_bk.filter(function (field) {
                if (isEqual(initData.page, 'im')) {
                    return field.im_show;
                }
                else if (isEqual(initData.page, 'mpv')) {
                    return field.mpv_show;
                }
                else {
                    return field.cc_show;
                }
            });
        }
        else {
            return [];
        }
    }, [customerFieldsv2]);
    var customerFields = useMemo(function () {
        return formatCustomField({
            fields: _customerFieldsv2,
            trigger: 'onBlur',
            fieldsAuthority: customerFieldsAuthority,
            fieldType: 'customer',
            editType: 'edit',
        });
    }, [_customerFieldsv2, customerFieldsAuthority]);
    var getValues = useCallback(function (_customer, organizations) {
        var initValues = getFieldsDefaultValue({
            fields: _customerFieldsv2,
            values: _customer.custom_fields || {},
        }) || {};
        var _organizations = (organizations || []).map(function (org) {
            return {
                label: org.name,
                value: org.id,
            };
        });
        initValues = Object.assign({}, initValues, _customer.custom_fields, {
            nick_name: _customer.nick_name,
            cellphones: _customer.cellphones || [{ content: '' }],
            email: _customer.email,
            organizations: _organizations,
            tags: _customer.rich_tags,
            description: _customer.description,
        });
        return initValues;
    }, [_customerFieldsv2]);
    useEffect(function () {
        var values = getValues(customer, organizations);
        setInitialValue(values);
        form.setFieldsValue(values);
        setLastValues(values);
    }, [customer, form, getValues, organizations, sessionKey]);
    var availableFieldHooks = useMemo(function () {
        if (fieldHooks) {
            return fieldHooks.filter(function (i) { return i.active && i.trigger_type === 'edit' && i.field_owner === 'Customer'; });
        }
        else {
            return [];
        }
    }, [fieldHooks]);
    var submitFn = useCallback(function (changedValues) {
        var param = getParam(changedValues);
        return csApiPutJSON("/cs/ts/api/customers/".concat(customer.id), param, {
            transformRequest: [
                function (data) {
                    return Qs.stringify(data, { arrayFormat: 'brackets' });
                },
            ],
        }).then(function (res) { return res.data; })
            .then(function (res) {
            var _a;
            if (res.code === 1000) {
                var values = getValues(res.customer || {}, res.organizations);
                if (values.cellphones && values.cellphones.length) {
                    values.cellphones = [
                        {
                            content: '',
                        },
                    ];
                }
                setInitialValue(values);
                message.success(t('tip.success.save'));
                var custom_fields = Object.assign(res.customer.custom_fields, (_a = param === null || param === void 0 ? void 0 : param.customer) === null || _a === void 0 ? void 0 : _a.custom_fields);
                res.customer.custom_fields = custom_fields;
                event.updateChat(chat, {
                    organizations: res.organizations,
                    customer: res.customer,
                    nick_name: res.customer.nick_name,
                    rich_tags: res.customer.rich_tags || [],
                });
            }
            else {
                message.error(res.code_message || (res.exceptopn && res.exceptopn.message) || t('tip.error.saveTry'));
            }
        })
            .catch(function (err) {
            console.error(err);
        });
    }, [chat, customer.id, event, getValues, t]);
    var formSubmit = useCallback(function (changedValues, values) {
        var changed = judgeValueChanged(changedValues, initialValue);
        if (!changed) {
            return;
        }
        var fieldHook = availableFieldHooks.find(function (i) {
            var triggerField = i.trigger_field;
            triggerField === 'name' && (triggerField = 'nick_name');
            return triggerField && triggerField in changedValues;
        });
        if (fieldHook) {
            return axios
                .post(fieldHook.url, buildHookRequestParamsForCustomer(fieldHook, merge({}, customer, values), {
                customerCustomFields: customerFieldsv2,
                otherParams: { im_sub_session_id: imSubSessionId },
            }))
                .then(function (res) {
                if (res.data.error) {
                    message.error(res.data.error);
                }
                if (res.data.code === 1000) {
                    // form.setFieldsValue(res.data.customer);
                    return form
                        .validateFields(Object.keys(res.data.customer))
                        .then(function () {
                        submitFn(res.data.customer).finally(function () { return setLastValues(values); });
                    })
                        .catch(function () {
                        form.setFieldsValue(lastValues);
                    });
                }
                else if (!res.data.error) {
                    message.error('请求数据错误，请重试!');
                    throw new Error('请求数据错误，请重试!');
                }
                else {
                    throw new Error(res.data.error);
                }
            });
        }
        else {
            submitFn(changedValues).finally(function () { return setLastValues(values); });
        }
    }, []);
    var onValueChange = function (changedValues, values) {
        form.validateFields(Object.keys(changedValues))
            .then(function () {
            formSubmit(changedValues, values);
        })
            .catch(function (err) {
            console.log(err);
            var key = Object.keys(changedValues)[0];
            var canSubmit = true;
            err.errorFields.forEach(function (item) {
                if (item.name[0] === key) {
                    canSubmit = false;
                }
            });
            if (canSubmit) {
                formSubmit(changedValues, values);
            }
        });
    };
    return (React.createElement("div", { className: style['customer-form'], style: { marginTop: '10px' } },
        React.createElement(Form, { layout: "vertical", name: "customer-info-form", onValuesChange: onValueChange, form: form },
            React.createElement(PlatformList, { customer: customer }),
            React.createElement(FromList, { switchFlag: switchFlag, customerFields: customerFields, organizationFields: organizationFields, company: company, customer: customer, imSubSessionId: imSubSessionId, event: event, chat: chat, companyCategoryConfig: companyCategoryConfig, organizations: organizations, roleCan: roleCan, currentUser: currentUser, smsapp: smsapp, isAdmin: isAdmin, user: user, agents: agents }))));
};
export default CustomerInfoForm;
