var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React, { useCallback, useEffect, useImperativeHandle, useMemo, useRef, useState } from 'react';
import { Tabs } from 'udesk-ui';
import classnames from 'classnames';
import { useTranslation } from 'react-i18next';
import unionBy from 'lodash-es/unionBy';
import isFunction from 'lodash-es/isFunction';
import includes from 'lodash-es/includes';
import { IM_CONTACT_TAB_NAME, IM_CUSTOMER_TAB_NAME, IM_HISTORY_MESSAGE_TAB_NAME, IM_NOTE_TAB_NAME, IM_REPOSITORY_TAB_NAME, IM_RETURN_VISIT_TAB_NAME, } from '../utils';
import { csApiGetJSON } from '../utils/csApi';
import { ImContact, ImCustomer, ImCustomPlugin, ImKnowledge, ImMsgRecord, ImNote, ImTicket, ImVisit, } from './component';
import TabSettingModal from './parallel/tab-setting-modal/tab-setting-modal';
import TabParallel from './parallel/tab-parallel';
import { Setting } from '../components/icon/icon';
import ErrorBoundary from '../components/simple-error-boundary';
import { useSelector } from '../store/store-hook';
import style from './index.module.scss';
import isEqual from 'lodash-es/isEqual';
import concat from 'lodash-es/concat';
var TabPane = Tabs.TabPane;
function getTabKey(tabName, customer_id) {
    return "".concat(tabName, "-").concat(customer_id);
}
var ImPanel = React.forwardRef(function (_a, ref) {
    var _b;
    var width = _a.width, onWidthChange = _a.onWidthChange, cssMaxWidth = _a.cssMaxWidth, props = __rest(_a, ["width", "onWidthChange", "cssMaxWidth"]);
    var t = useTranslation().t;
    var _c = props.chat, chat = _c === void 0 ? {} : _c, agentConfig = props.agentConfig, currentUser = props.currentUser, noteFields = props.noteFields, event = props.event;
    var _d = useState([]), tabPlugins = _d[0], setTabPlugins = _d[1];
    var customer_id = chat.customer_id, _e = chat.plugTab, plugTab = _e === void 0 ? {} : _e;
    var containerRef = useRef(null);
    var initData = useSelector(function (state) { return state.initData; });
    var agentTabConfig = useMemo(function () {
        return (agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.agent_tab_config) || {};
    }, [agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.agent_tab_config]);
    var minWidth = useMemo(function () {
        if (agentTabConfig.parallel) {
            return 600;
        }
        else {
            return 400;
        }
    }, [agentTabConfig.parallel]);
    // const tabsWidth = 420;
    var _f = useState(), currentTab = _f[0], setCurrentTab = _f[1];
    var _g = useState(false), showModal = _g[0], setShowModal = _g[1];
    var _h = useState({
        show: false,
        ticketCover: '',
        zIndex: 0,
    }), showTicket = _h[0], setShowTicket = _h[1];
    var _j = useState({
        show: false,
        visitCover: '',
        zIndex: 0,
    }), showVisit = _j[0], setShowVisit = _j[1];
    var _k = useState({
        show: false,
        recordCover: '',
        zIndex: 0,
    }), showRecord = _k[0], setShowRecord = _k[1];
    var _l = useState(), lastCreatedTicket = _l[0], setLastCreatedTicket = _l[1];
    var tabsHash = useMemo(function () {
        if (tabPlugins && tabPlugins.length > 0) {
            var _parallel_1 = agentTabConfig.parallel;
            var order_1 = agentTabConfig.order;
            var updatePlugins_1 = [];
            var parallelPlugins_1 = {};
            var list_1 = [];
            if (order_1) {
                if (isEqual(initData.page, 'mpv')) {
                    order_1 = concat(order_1, ((initData === null || initData === void 0 ? void 0 : initData.pluginFilter) || [])
                        .filter(function (item) { return !order_1.includes("".concat(item)); })
                        .map(function (ite) { return "".concat(ite); }));
                }
                order_1.forEach(function (index) {
                    var plugin = tabPlugins.find(function (plugin) {
                        return plugin.name === index;
                    });
                    if (_parallel_1) {
                        if (_parallel_1 === index) {
                            parallelPlugins_1 = __assign({}, plugin);
                        }
                        else {
                            updatePlugins_1.push(__assign({}, plugin));
                        }
                    }
                    else {
                        updatePlugins_1.push(__assign({}, plugin));
                    }
                    list_1.push(__assign(__assign({}, plugin), { checked: _parallel_1 === index }));
                });
                updatePlugins_1 = unionBy(updatePlugins_1, tabPlugins, 'name');
                return {
                    updatePlugins: updatePlugins_1,
                    parallelPlugins: parallelPlugins_1,
                    list: list_1,
                };
            }
            else {
                tabPlugins.forEach(function (plugin) {
                    if (_parallel_1) {
                        if (_parallel_1 === plugin.name) {
                            parallelPlugins_1 = __assign({}, plugin);
                        }
                        else {
                            updatePlugins_1.push(__assign({}, plugin));
                        }
                    }
                    else {
                        updatePlugins_1.push(__assign({}, plugin));
                    }
                    list_1.push(__assign(__assign({}, plugin), { checked: _parallel_1 === plugin.name }));
                });
                return {
                    updatePlugins: updatePlugins_1,
                    parallelPlugins: parallelPlugins_1,
                    list: list_1,
                };
            }
        }
        else {
            return {
                updatePlugins: [],
                parallelPlugins: {},
                list: [],
            };
        }
    }, [tabPlugins, agentTabConfig, initData.pluginFilter]);
    var tabs = useMemo(function () { return tabsHash['updatePlugins']; }, [tabsHash]);
    var parallel = useMemo(function () { return tabsHash['parallelPlugins']; }, [tabsHash]);
    var settingList = useMemo(function () { return tabsHash['list']; }, [tabsHash]);
    var handleChange = useCallback(function (activeKey) {
        var plugTab = tabs.find(function (tab) { return getTabKey(tab.name, customer_id) === activeKey; });
        if (plugTab) {
            setCurrentTab(plugTab);
        }
        if (isFunction(event.updateChat)) {
            event.updateChat({
                chat: chat,
                updateProps: {
                    plugTab: plugTab,
                },
            });
        }
    }, [chat, customer_id, event, tabs]);
    useImperativeHandle(ref, function () { return ({
        showTicket: function (config) {
            var ticketCover = agentTabConfig.ticketCover;
            var _parallel = agentTabConfig.parallel;
            setShowTicket({
                show: config.show,
                ticketCover: _parallel ? ticketCover : 'all',
                zIndex: config.zIndex++,
            });
        },
        showVisit: function (config) {
            var visitCover = agentTabConfig.visitCover;
            var _parallel = agentTabConfig.parallel;
            setShowVisit({
                show: config.show,
                visitCover: _parallel ? visitCover : 'all',
                zIndex: config.zIndex++,
            });
        },
        showRecord: function (config) {
            var recordCover = agentTabConfig.recordCover;
            var _parallel = agentTabConfig.parallel;
            setShowRecord({
                show: config.show,
                recordCover: _parallel ? recordCover : 'all',
                zIndex: config.zIndex++,
            });
        },
        changeImPanelTab: function (id, customerId) {
            handleChange("".concat(id, "-").concat(customerId));
        },
    }); }, [
        agentTabConfig.ticketCover,
        agentTabConfig.parallel,
        agentTabConfig.visitCover,
        agentTabConfig.recordCover,
        handleChange,
    ]);
    // 获取tabs数据
    // useEffect(() => {
    //   const plugins = (initData.tabPlugins || [])
    //     .filter((plugin: any) => plugin.enabled)
    //     .map((i: any) => {
    //       return {
    //         name: i.id + "",
    //         text: i.id < 1 ? t("page.im.tabs.tagName." + i.id) : i.name,
    //         active: i.is_default,
    //         postMsgEnabled: i.post_message_enabled,
    //       };
    //     });
    //   setTabPlugins(plugins);
    // }, [t]);
    useEffect(function () {
        csApiGetJSON("/cs/ts/api/user_defined_plugins?show_page=".concat(initData.page))
            .then(function (res) { return res.data; })
            .then(function (res) {
            if (res === void 0) { res = {}; }
            if (res.code === 1000) {
                var plugins = res.plugins
                    .filter(function (plugin) {
                    return plugin.enabled &&
                        (isEqual(initData.page, 'mpv') ? includes(initData === null || initData === void 0 ? void 0 : initData.pluginFilter, plugin.id) : true);
                })
                    .map(function (i) {
                    return {
                        name: i.id + '',
                        text: i.id < 1 ? t('page.im.tabs.tagName.' + i.id) : i.name,
                        active: i.is_default,
                        postMsgEnabled: i.post_message_enabled,
                    };
                });
                setTabPlugins(plugins);
            }
        })
            .catch(function (e) {
            console.log(e);
        });
    }, [t, initData.pluginFilter]);
    useEffect(function () {
        if ((currentTab === null || currentTab === void 0 ? void 0 : currentTab.text) == '客户信息' && isFunction(event.reloadModel)) {
            event.reloadModel();
        }
    }, [currentTab]);
    var tabsWidth = useMemo(function () {
        var panelWidth = window.localStorage.getItem('stretchable-box-right-panel-width') || '370';
        var _panelWidth = parseInt(panelWidth, 10);
        if (agentTabConfig.parallel) {
            if (_panelWidth < minWidth) {
                _panelWidth = minWidth;
                window.localStorage.setItem('stretchable-box-right-panel-width', minWidth + '');
            }
        }
        else {
            if (_panelWidth > minWidth) {
                _panelWidth = minWidth;
                window.localStorage.setItem('stretchable-box-right-panel-width', minWidth + '');
            }
        }
        return _panelWidth;
    }, [agentTabConfig.parallel]);
    useEffect(function () {
        if (tabs && tabs.length > 0) {
            var activeTab = null;
            var normalTabs = tabs.filter(function (t) { return t.name !== parallel.name; });
            var pluginName_1 = plugTab === null || plugTab === void 0 ? void 0 : plugTab.name;
            // 客户身上的配置
            if (pluginName_1) {
                activeTab = normalTabs.find(function (t) { return t.name === pluginName_1; });
            }
            // 系统配置
            if (activeTab == null) {
                activeTab = normalTabs.find(function (t) { return t.active; });
            }
            // 第一个
            if (activeTab == null) {
                activeTab = normalTabs[0];
            }
            if (activeTab) {
                setCurrentTab(activeTab);
            }
        }
    }, [parallel.name, tabs, tabsWidth, plugTab.name]);
    var getComponent = useCallback(function (key) {
        if (key) {
            var plugin = tabPlugins.find(function (p) { return p.name === key; });
            switch (key) {
                case IM_CUSTOMER_TAB_NAME:
                    return (React.createElement(ImCustomer, { customer: props.customer, currentUser: props.currentUser, event: props.event, organizations: props.organization, chat: props.chat, company: props.company, imSubSessionId: props.imSubSessionId, customerId: props.customerId, companyCategoryConfig: props.companyCategoryConfig, roleCan: props.roleCan, imNewPlatforms: props.imNewPlatforms, organizationFieldsv2: props.organizationFieldsv2, showCount: props.showCount, customerFieldsAuthority: props.customerFieldsAuthority, customerFieldsv2: props.customerFieldsv2, fieldHooks: props.fieldHooks, isAdmin: props.isAdmin, organizationFields: props.organizationFields, sessionKey: props.sessionKey, smsapp: props.smsapp, switchFlag: props.switchFlag, user: props.user, agents: props.agents }));
                case IM_NOTE_TAB_NAME:
                    return (React.createElement(ImNote, { chat: props.chat, company: props.company, noteTemplates: props.noteTemplates, event: props.event }));
                case IM_CONTACT_TAB_NAME:
                    return (React.createElement(ImContact, { createdTicket: lastCreatedTicket, chat: props.chat, roleCan: props.roleCan, event: props.event, agents: props.agents }));
                case IM_REPOSITORY_TAB_NAME:
                    return (React.createElement(ImKnowledge, { event: props.event, roleCan: props.roleCan, chat: props.chat, company: props.company, user: props.user, kmrobot_url: props.kmrobot_url, previewUrl: props.previewUrl }));
                case IM_HISTORY_MESSAGE_TAB_NAME:
                    return (React.createElement(ImMsgRecord, { chat: props.chat, event: props.event, setShowRecord: props.setShowRecord }));
                case IM_RETURN_VISIT_TAB_NAME:
                    return (React.createElement(ImVisit, { content: "", showCancel: false, event: event, chat: chat, setShowVisit: setShowVisit }));
                default: {
                    return (React.createElement(ImCustomPlugin, { type: isEqual(initData.page, 'mpv') ? 'videocall_in' : 'im', currentUser: currentUser, customer: chat.customer, pluginId: plugin === null || plugin === void 0 ? void 0 : plugin.name, postMsgEnabled: plugin === null || plugin === void 0 ? void 0 : plugin.postMsgEnabled, platform: chat.platform, chat: chat, noteFields: noteFields, event: event }));
                }
            }
        }
        else {
            return '';
        }
    }, [
        tabPlugins,
        props.customer,
        props.currentUser,
        props.event,
        props.organization,
        props.chat,
        props.company,
        props.imSubSessionId,
        props.customerId,
        props.companyCategoryConfig,
        props.roleCan,
        props.imNewPlatforms,
        props.organizationFieldsv2,
        props.showCount,
        props.customerFieldsAuthority,
        props.customerFieldsv2,
        props.fieldHooks,
        props.isAdmin,
        props.organizationFields,
        props.sessionKey,
        props.smsapp,
        props.switchFlag,
        props.user,
        props.noteTemplates,
        props.agents,
        props.kmrobot_url,
        props.previewUrl,
        props.setShowRecord,
        lastCreatedTicket,
        event,
        chat,
        currentUser,
        noteFields,
    ]);
    var parallelComponent = useMemo(function () { return getComponent(parallel.name); }, [getComponent, parallel.name]);
    var handlerShowTabSettingModel = useCallback(function () { return setShowModal(true); }, []);
    var cancelTabs = useCallback(function () { return setShowModal(false); }, []);
    var onTicketCreateSuccess = useCallback(function (ticket) {
        setLastCreatedTicket(ticket);
    }, []);
    var getShowNewTab = useCallback(function (coverType) {
        return (React.createElement(React.Fragment, null,
            showTicket.ticketCover === coverType && showTicket.show && (React.createElement("div", { className: style['new-tab'], style: { zIndex: showTicket.zIndex } },
                React.createElement(ErrorBoundary, null,
                    React.createElement(ImTicket, __assign({ setShowTicket: setShowTicket, onCreateSuccess: onTicketCreateSuccess }, props))))),
            showVisit.visitCover === coverType && showVisit.show && (React.createElement("div", { className: style['new-tab'], style: { zIndex: showVisit.zIndex } },
                React.createElement(ErrorBoundary, null,
                    React.createElement(ImVisit, { content: "", showCancel: false, chat: chat, event: event, setShowVisit: setShowVisit })))),
            showRecord.recordCover === coverType && showRecord.show && (React.createElement("div", { className: style['new-tab'], style: { zIndex: showRecord.zIndex } },
                React.createElement(ErrorBoundary, null,
                    React.createElement(ImMsgRecord, __assign({ setShowRecord: setShowRecord }, props)))))));
    }, [
        chat,
        event,
        onTicketCreateSuccess,
        props,
        showRecord.recordCover,
        showRecord.show,
        showRecord.zIndex,
        showTicket.show,
        showTicket.ticketCover,
        showTicket.zIndex,
        showVisit.show,
        showVisit.visitCover,
        showVisit.zIndex,
    ]);
    var handleSwitchParallel = useCallback(function () { return console.log('x'); }, []);
    var handleHideParallel = useCallback(function () { return console.log('x'); }, []);
    var _m = useState(), _width = _m[0], _setWidth = _m[1];
    useEffect(function () {
        var _a;
        var startX = 0;
        var isMove = false;
        var maskElement = document.createElement('div');
        var containerWidth = 0;
        maskElement.style.cssText =
            'position:fixed;top:0;left:0;right:0;bottom:0;z-index:99999;background:transparent;cursor:col-resize';
        var fc = {
            mousedown: function (e) {
                var _a, _b;
                startX = e.clientX;
                isMove = true;
                document.body.appendChild(maskElement);
                containerWidth = (_b = (_a = containerRef === null || containerRef === void 0 ? void 0 : containerRef.current) === null || _a === void 0 ? void 0 : _a.offsetWidth) !== null && _b !== void 0 ? _b : 0;
            },
            mousemove: function (e) {
                if (isMove) {
                    _setWidth(containerWidth + (startX - e.clientX));
                    e.preventDefault();
                }
            },
            mouseup: function (e) {
                if (isMove) {
                    isMove = false;
                }
                maskElement.remove();
            },
        };
        (_a = document.getElementById('col-drag-line')) === null || _a === void 0 ? void 0 : _a.addEventListener('mousedown', fc.mousedown);
        document.addEventListener('mousemove', fc.mousemove);
        document.addEventListener('mouseup', fc.mouseup);
        return function () {
            var _a;
            (_a = document.getElementById('col-drag-line')) === null || _a === void 0 ? void 0 : _a.removeEventListener('mousedown', fc.mousedown);
            document.removeEventListener('mousemove', fc.mousemove);
            document.removeEventListener('mouseup', fc.mouseup);
        };
    }, []);
    useEffect(function () {
        if (_width) {
            return onWidthChange(Math.max(minWidth, _width));
        }
    }, [_width, minWidth, onWidthChange]);
    var containerStyle = useMemo(function () { return ({ width: width + 'px', maxWidth: cssMaxWidth }); }, [cssMaxWidth, width]);
    var tabBarExtraContent = useMemo(function () {
        if (!isEqual(initData.page, 'mpv')) {
            return (React.createElement("div", { className: style['tab-extra'], onClick: handlerShowTabSettingModel },
                React.createElement(Setting, null)));
        }
    }, []);
    var tabChildren = useMemo(function () {
        return tabs.map(function (tab) {
            var component = getComponent(tab.name);
            return (React.createElement(TabPane, { tab: tab.text, key: getTabKey(tab.name, customer_id) },
                React.createElement(ErrorBoundary, null, component)));
        });
    }, [customer_id, getComponent, tabs]);
    return (React.createElement("div", { className: style['im-tab-panel'], style: containerStyle, ref: containerRef },
        React.createElement("div", { className: style['drag-line'], id: "col-drag-line" }),
        React.createElement("div", { className: classnames(style['panel-container'], (_b = {},
                _b[style['has-parallel']] = !!agentTabConfig.parallel,
                _b[style['view-reverse']] = !!agentTabConfig.sortDesc,
                _b)) },
            chat.belong === 'leave' && (React.createElement("div", { style: { padding: '20px 0 ', width: '100%' } },
                React.createElement(ImCustomer, __assign({}, props)))),
            chat.belong !== 'leave' && tabs.length > 0 && (React.createElement("div", { className: style['panle-main'] },
                React.createElement(Tabs, { activeKey: getTabKey((currentTab === null || currentTab === void 0 ? void 0 : currentTab.name) || '', customer_id), tabBarGutter: 0, tabBarExtraContent: tabBarExtraContent, onChange: handleChange, className: style.tabs, destroyInactiveTabPane: true, animated: false }, tabChildren),
                React.createElement(TabSettingModal, { tabs: settingList, agentTabConfig: agentTabConfig, showModal: showModal, hideTabSetingModal: cancelTabs, event: event }),
                getShowNewTab('parallelNormal'))),
            !isEqual(initData.page, 'mpv') && agentTabConfig.parallel && React.createElement("div", { className: style['panel-line'] }),
            !isEqual(initData.page, 'mpv') && agentTabConfig.parallel && (React.createElement("div", { className: style['panel-parallel'] },
                React.createElement(TabParallel, __assign({}, parallel, { component: parallelComponent, switchParallel: handleSwitchParallel, hideParallel: handleHideParallel, agentTabConfig: agentTabConfig, event: event })),
                getShowNewTab('parallel'))),
            getShowNewTab('all'))));
});
var ImPanelMemo = React.memo(ImPanel);
export default ImPanelMemo;
