var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { useCallback, useEffect, useState } from 'react';
import { Button, Col, Form, message, Modal, Radio, Row } from 'udesk-ui';
import { csApiPostJSON } from '../../../utils/csApi';
import { DndProvider } from 'react-dnd';
import { HTML5Backend } from 'react-dnd-html5-backend';
import style from './style.module.scss';
import update from 'immutability-helper';
import { DndContainer } from './dnd-container';
import isFunction from 'lodash-es/isFunction';
import cloneDeep from 'lodash-es/cloneDeep';
import { useTranslation } from 'react-i18next';
import { useSelector } from '../../../store/store-hook';
import { isEqual } from 'lodash-es';
var TabSettingModal = function (_a) {
    var tabs = _a.tabs, _b = _a.agentTabConfig, agentTabConfig = _b === void 0 ? {} : _b, showModal = _a.showModal, hideTabSetingModal = _a.hideTabSetingModal, event = _a.event;
    var t = useTranslation().t;
    var type = 'TabSettingDrap';
    var _c = useState(false), loading = _c[0], setLoading = _c[1];
    var _d = useState({
        ticketCover: agentTabConfig.ticketCover || 'all',
        visitCover: agentTabConfig.visitCover || 'all',
        recordCover: agentTabConfig.ticketCover || 'all',
    }), cover = _d[0], setCover = _d[1];
    var _e = useState(function () {
        return (tabs || [])
            .filter(function (item) { return item.name; })
            .map(function (tab) {
            return {
                id: tab.name,
                key: tab.name,
                text: tab.text,
                checked: tab.checked,
            };
        });
    }), list = _e[0], setList = _e[1];
    var initData = useSelector(function (state) { return state.initData; });
    useEffect(function () {
        setList(resetList());
    }, [showModal, tabs]);
    var resetList = useCallback(function () {
        return (tabs || [])
            .filter(function (item) { return item.name; })
            .map(function (tab) {
            return {
                id: tab.name,
                key: tab.name,
                text: tab.text,
                checked: tab.checked,
            };
        });
    }, [tabs]);
    // 拖拽
    var moveItem = function (id, atIndex) {
        var _a = findItem(id), item = _a.item, index = _a.index;
        setList(update(list, {
            $splice: [
                [index, 1],
                [atIndex, 0, item],
            ],
        }));
    };
    // 查找对应的tab
    var findItem = function (id) {
        var item = list.filter(function (c) { return "".concat(c.id) === id; })[0];
        return {
            item: item,
            index: list.indexOf(item),
        };
    };
    // 选中平行视界
    var checkedItem = function (id) {
        setList(function (prevList) {
            var _prevList = cloneDeep(prevList);
            _prevList.map(function (item) {
                if (item.id === id) {
                    item.checked = !item.checked;
                }
                else {
                    item.checked = false;
                }
                return item;
            });
            return _prevList;
            // return produce(prevList, (draft: any) => {
            //     draft.map((item: any) => {
            //         if (item.id === id) {
            //             item.checked = !item.checked
            //         } else {
            //             item.checked = false
            //         }
            //         return item;
            //     })
            // })
        });
    };
    // 保存
    var handleSaveModalSetting = function () {
        setLoading(true);
        var _parallel = (list.find(function (item) { return item.checked; }) || {}).id || '';
        var _order = list.map(function (item) { return item.id; });
        var _data = __assign({ parallel: _parallel, order: _order, sortDesc: agentTabConfig.sortDesc }, cover);
        csApiPostJSON("/cs/im/agent/im/agent_config", {
            data: {
                agent_tab_config: _data,
            },
        }).then(function (res) { return res.data; })
            .then(function () {
            if (isFunction(event.updateAgentConfig)) {
                event.updateAgentConfig(__assign(__assign(__assign({}, agentTabConfig), { parallel: _parallel, order: _order, sortDesc: agentTabConfig.sortDesc }), cover));
            }
        })
            .catch(function (err) {
            message.error('保存失败,请重试');
        })
            .finally(function () {
            setLoading(false);
            hideTabSetingModal();
        });
    };
    // 取消
    var handleCancel = function () {
        var baseList = resetList();
        setList(baseList);
        setCover({
            ticketCover: agentTabConfig.ticketCover,
            visitCover: agentTabConfig.visitCover,
            recordCover: agentTabConfig.recordCover,
        });
        hideTabSetingModal();
    };
    var headerStyle = {
        padding: '8px 12px',
        borderBottom: '1px solid #f0f0f0',
        backgroundColor: '#fafafa',
    };
    var formItemLayout = {
        labelCol: {
            span: 4,
        },
        wrapperCol: {
            span: 20,
        },
    };
    var handleShowType = function (e) {
        var _a = e.target, name = _a.name, value = _a.value;
        // 调用agent_config接口更新数据
        setCover({
            ticketCover: name === 'ticket' ? value : cover.ticketCover,
            visitCover: name === 'visit' ? value : cover.visitCover,
            recordCover: name === 'record' ? value : cover.recordCover,
        });
    };
    return (React.createElement(Modal, { visible: showModal, title: t('label.preferences'), closable: false, destroyOnClose: true, onOk: handleSaveModalSetting, onCancel: handleCancel, wrapClassName: style['tab-setting-modal'], footer: [
            React.createElement(Button, { key: "back", onClick: handleCancel }, t('cancel')),
            React.createElement(Button, { key: "submit", type: "primary", loading: loading, onClick: handleSaveModalSetting }, t('confirm')),
        ] },
        React.createElement("div", { style: { maxHeight: '350px', overflowY: 'auto' } },
            React.createElement(DndProvider, { backend: HTML5Backend },
                React.createElement("div", { style: headerStyle },
                    React.createElement(Row, null,
                        React.createElement(Col, { span: 4 }, t('label.order')),
                        React.createElement(Col, { span: 12 }, t('label.component')),
                        React.createElement(Col, { span: 4 }, t('label.parallelHorizon')))),
                React.createElement(DndContainer, { list: list, type: type, moveItem: moveItem, findItem: findItem, checkedItem: checkedItem })),
            React.createElement("br", { style: { marginBottom: '15px' } }),
            isEqual(initData.page, 'im') && (React.createElement("div", null,
                React.createElement(Form, __assign({ name: "validate_other" }, formItemLayout),
                    React.createElement(Form.Item, { label: t('page.im.ticket.createTicket') },
                        React.createElement(Radio.Group, { name: "ticket", value: cover.ticketCover, onChange: handleShowType },
                            React.createElement(Radio, { value: 'all' }, t('label.overallCoverage')),
                            React.createElement(Radio, { value: 'parallel' }, t('label.coveringParallelHorizon')),
                            React.createElement(Radio, { value: 'parallelNormal' }, t('label.coverCommonArea')))),
                    React.createElement(Form.Item, { label: t('page.im.visit.title') },
                        React.createElement(Radio.Group, { name: "visit", value: cover.visitCover, onChange: handleShowType },
                            React.createElement(Radio, { value: 'all' }, t('label.overallCoverage')),
                            React.createElement(Radio, { value: 'parallel' }, t('label.coveringParallelHorizon')),
                            React.createElement(Radio, { value: 'parallelNormal' }, t('label.coverCommonArea')))),
                    React.createElement(Form.Item, { label: t('page.im.record.imLogRecords') },
                        React.createElement(Radio.Group, { name: "record", value: cover.recordCover, onChange: handleShowType },
                            React.createElement(Radio, { value: 'all' }, t('label.overallCoverage')),
                            React.createElement(Radio, { value: 'parallel' }, t('label.coveringParallelHorizon')),
                            React.createElement(Radio, { value: 'parallelNormal' }, t('label.coverCommonArea'))))))))));
};
export default TabSettingModal;
