/* eslint-disable no-unreachable */
import React, { useCallback, useEffect, useMemo, useState, useRef, } from "react";
import style from "../index.module.scss";
import { useTranslation } from "react-i18next";
import { Button, Form, message, Select } from "udesk-ui";
import FormTinymce from "../../../components/tinymce/tinymce";
import * as FieldComponent from "../components/index";
import { formatCustomField, r, getJSON } from "../../../utils/index";
import { csApiPostJSON } from '../../../utils/csApi';
import CustomField from "../../../components/custom-field-container";
import isEmpty from "lodash-es/isEmpty";
import moment from "moment";
import useDefault from "react-use/esm/useDefault";
var FormItem = Form.Item;
var ticketStatus = [
    { key: "open", id: 1, name: "开启" },
    { key: "solving", id: 4, name: "解决中" },
    { key: "resolved", id: 2, name: "已解决" },
    { key: "closed", id: 3, name: "已关闭" },
];
var ticketPriority = [
    { key: "urgency", id: 1, name: "紧急" },
    { key: "high", id: 2, name: "高" },
    { key: "medium", id: 3, name: "标准" },
    { key: "low", id: 4, name: "低" },
];
var templateSelectStyle = { width: "auto", minWidth: "100px" };
var cancelButtonStyle = { marginRight: "10px" };
function CreateTicket(props) {
    var currentUser = props.currentUser, _a = props.chat, chat = _a === void 0 ? {} : _a, ticketFieldsAbilities = props.ticketFieldsAbilities, company = props.company, roleCan = props.roleCan, ticketTab = props.ticketTab, admin = props.admin, event = props.event, onSaveSuccess = props.onSaveSuccess;
    var formRef = Form.useForm()[0];
    //处理工单模板选项数据
    var ticketTemplates = useMemo(function () {
        return props.ticketTemplates.map(function (item) {
            return {
                value: item.id,
                label: item.name,
            };
        });
    }, [props.ticketTemplates]);
    //是否已经成功提交表单
    var _b = useState(false), isSuccess = _b[0], setIsSuccess = _b[1];
    // 多语言
    var t = useTranslation().t;
    // 选择的模板id
    var _c = useDefault(ticketTemplates[0].value, ticketTemplates[0].value), template_id = _c[0], setTemplate_id = _c[1];
    // 自定义字段
    var _d = useState([]), customField = _d[0], setCustomField = _d[1];
    // 获取当前工单的缓存信息
    var getLocalStorage = useCallback(function () {
        var localStorageData = localStorage.getItem("".concat(props.currentUser.id, "-ticket-form-stash-at-email-im-message-").concat(chat === null || chat === void 0 ? void 0 : chat.im_sub_session_id));
        localStorageData && (localStorageData = JSON.parse(localStorageData));
        return localStorageData;
    }, [chat === null || chat === void 0 ? void 0 : chat.im_sub_session_id, props.currentUser.id]);
    var noteCustomFields = useRef();
    // 处理表单初始信息
    var formData = useMemo(function () {
        //缓存的数据
        var localStorageData = getLocalStorage();
        return {
            subject: r(t("page.im.ticket.createTicketByIM"), currentUser.nick_name),
            content: localStorageData && !isEmpty(localStorageData.createTicketFormData)
                ? localStorageData.createTicketFormData.content
                : "",
            customer_id: {
                value: chat.customer_id,
                label: chat.nick_name,
            },
            agent: [],
            follower_ids: [],
            status_id: 1,
            priority_id: 3,
            tags: [],
            attachment_ids: [],
            merchant_sid: {},
        };
    }, [
        getLocalStorage,
        chat.customer_id,
        chat.nick_name,
        t,
        currentUser.nick_name,
    ]);
    //首次进入  查看是否有缓存的模板id  如果有  将模板id换成缓存的id
    useEffect(function () {
        var _willDeleteItems = localStorage.getItem("will-delete-items");
        if (_willDeleteItems) {
            var willDeleteItems = JSON.parse(_willDeleteItems);
            var newWillDeleteItems = {};
            for (var item in willDeleteItems) {
                if (localStorage.getItem(item)) {
                    if (moment(willDeleteItems[item]).isBefore(moment())) {
                        localStorage.removeItem(item);
                    }
                    else {
                        newWillDeleteItems[item] = willDeleteItems[item];
                    }
                }
            }
            localStorage.setItem("will-delete-items", JSON.stringify(newWillDeleteItems));
        }
        var localStorageData = getLocalStorage();
        if (localStorageData) {
            setTemplate_id(localStorageData.createTicketTemplateId);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    //整个表单是否滚动   为了解决内部的下拉选择框的滚动穿透  聚焦下拉选择时  将overScroll变成hidden
    var _e = useState(true), isScroll = _e[0], setIsScroll = _e[1];
    //模板改变时   将对应模板id的模板筛选出来  渲染新的自定义表单
    useEffect(function () {
        var _a;
        var localStorageData = getLocalStorage();
        var field = formatCustomField({
            fields: ((_a = props.ticketTemplates.find(function (item) { return item.id === template_id; })) === null || _a === void 0 ? void 0 : _a.fields) || [],
            trigger: "onChange",
            fieldsAuthority: ticketFieldsAbilities[template_id],
            fieldType: "ticket",
            editType: "new",
        });
        //处理自定义字段的formData 处理好以后与系统字段合并  给formData重置初始值
        // 如果当前会话id有缓存的模板信息  直接取出  如果没有  进行以上处理
        var customFormData = {}; //自定义字段的formData
        if (localStorageData && !isEmpty(localStorageData.createTicketFormData)) {
            customFormData = localStorageData.createTicketFormData;
        }
        else {
            //循环用户自定义字段   将自定义字段对应的key 和value取出  作为自定义字段的formData
            field.forEach(function (item) {
                customFormData[item.field.id] =
                    item.field.id.split("_")[0] === "TextField" ? "" : [];
            });
        }
        // 将自定义的formData和系统字段的formdata合并   然后form重置初始值
        formRef.setFieldsValue(Object.assign(formData, customFormData));
        //保存自定义字段  渲染自定义字段的组件使用
        setCustomField(field);
        //离开新建工单页面时  缓存工单模板id和表单数据
        return function () {
            var data = {
                createTicketTemplateId: template_id,
                createTicketFormData: isSuccess ? {} : formRef.getFieldsValue(),
            };
            var stashKey = "".concat(currentUser.id, "-ticket-form-stash-at-email-im-message-").concat(chat.im_sub_session_id);
            localStorage.setItem(stashKey, JSON.stringify(data));
            //存时间
            if (!localStorage.getItem("will-delete-items")) {
                window.localStorage.setItem("will-delete-items", JSON.stringify({}));
            }
            var willDeleteItems = window.localStorage.getItem("will-delete-items");
            willDeleteItems = JSON.parse(willDeleteItems);
            willDeleteItems[stashKey] = moment().add(24, "hours").toJSON();
            localStorage.setItem("will-delete-items", JSON.stringify(willDeleteItems));
        };
    }, [
        template_id,
        isSuccess,
        getLocalStorage,
        props.ticketTemplates,
        ticketFieldsAbilities,
        currentUser.id,
        chat.im_sub_session_id,
        formRef,
        formData,
    ]);
    //点击公司  跳转到对应的公司详情
    var gotoTicket = useCallback(function (id) {
        event.transitionToRoute &&
            event.transitionToRoute({
                route: "ticket.show",
                path: id,
            });
    }, [event]);
    // 关闭工单
    var closeTicket = useCallback(function () {
        props.setShowTicket({
            show: false,
        });
        event.updateChat &&
            event.updateChat({
                chat: chat,
                updateProps: {
                    newTicketReact: {},
                },
            });
    }, [props, chat, event]);
    // 提交表单
    var handleSubmit = useCallback(function (value) {
        var params = {
            ticket: {
                assignee_id: currentUser.id,
                platform_name: "im",
                template_id: template_id,
                im_sub_session_id: chat.im_sub_session_id,
            },
            tags: "",
        };
        // 如果选了受理人 拆成客服组id客服id
        if (value.agent.length) {
            params.ticket.agent_group_id = value.agent[0];
            params.ticket.assignee_id = value.agent[1] || "";
        }
        delete value.agent;
        // 处理自定义字段   将自定义字段从formData中筛选出来
        var custom_fields = {};
        customField.forEach(function (item) {
            var key = item.field.id;
            if (value[key] !== undefined) {
                custom_fields[key] = Array.isArray(value[key])
                    ? value[key].join(",")
                    : value[key];
            }
            delete value[key];
        });
        params.ticket.custom_fields = custom_fields;
        //处理标签
        value.tags.length &&
            (params.tags = value.tags
                .map(function (item) { return item.label; })
                .join(","));
        delete value.tags;
        //处理客户id
        value.customer_id = value.customer_id.value;
        //处理附件id
        value.attachment_ids = value.attachment_ids
            ? value.attachment_ids.map(function (item) { return item.uid; })
            : [];
        //处理商户
        if (company.category === "mchat_primary")
            value.merchant_sid = value.merchant_sid.value;
        // 将剩下的字段和处理好的ticket合并
        params.ticket = Object.assign(params.ticket, value);
        csApiPostJSON("/cs/ts/api/tickets", params)
            .then(function (res) { return res.data; })
            .then(function (res) {
            var _a;
            if (res.code === 1000) {
                setIsSuccess(true);
                var success = (
                // eslint-disable-next-line jsx-a11y/anchor-is-valid
                React.createElement("span", null,
                    "\u5DE5\u5355",
                    React.createElement("a", { onClick: function () { return gotoTicket(res.ticket.id); } }, res.ticket.field_num),
                    "\u521B\u5EFA\u6210\u529F"));
                message.success(success);
                setTimeout(function () {
                    closeTicket();
                }, 1500);
                onSaveSuccess === null || onSaveSuccess === void 0 ? void 0 : onSaveSuccess(res.ticket);
            }
            else {
                message.error((res === null || res === void 0 ? void 0 : res.code_message) || ((_a = res === null || res === void 0 ? void 0 : res.exception) === null || _a === void 0 ? void 0 : _a.message) || t("saveError"));
            }
        })
            .catch(function (e) {
            message.error(e);
        });
    }, [
        closeTicket,
        customField,
        gotoTicket,
        t,
        template_id,
        chat,
        company.category,
    ]);
    // 表单数据改变触发
    function handleChangeValue(changeValue, allValue) {
        // console.log(allValue)
    }
    // 改变模板  记录模板id  改变自定义字段
    var changeTemplate = useCallback(function (value) {
        setTemplate_id(value);
    }, []);
    useEffect(function () {
        var handleFields = function () {
            customField.forEach(function (i) {
                var _a;
                var _b, _c;
                var defaults = ((_c = (_b = i === null || i === void 0 ? void 0 : i.field) === null || _b === void 0 ? void 0 : _b.options) === null || _c === void 0 ? void 0 : _c.filter(function (j) { return j.default; })) || [];
                if (!defaults.length) {
                    return;
                }
                formRef.setFieldsValue((_a = {},
                    _a[i.field.id] = defaults.map(function (j) { return j.value; }),
                    _a));
            });
            var _loop_1 = function (item) {
                var _a;
                var field = customField.find(function (_item) { return _item.field.id === item; });
                if (field) {
                    formRef.setFieldsValue((_a = {}, _a[item] = noteCustomFields.current[item], _a));
                }
            };
            for (var item in noteCustomFields.current) {
                _loop_1(item);
            }
        };
        if (noteCustomFields.current) {
            handleFields();
        }
        else {
            getJSON("/notes/detail", {
                type: "im",
                ref_id: chat.im_sub_session_id,
                template_id: "",
            }).then(function (res) {
                noteCustomFields.current = res.data.custom_fields;
                handleFields();
            });
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [customField, formRef]);
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { className: style["ticket-form"], style: { overflowY: isScroll ? "scroll" : "hidden" } },
            React.createElement(Form, { form: formRef, onFinish: handleSubmit, initialValues: formData, layout: "vertical" },
                React.createElement(Form.Item, { label: t("page.im.ticket.subject"), name: "subject" },
                    React.createElement(FieldComponent.text, { id: "subject", trigger: "onChange", type: "text" })),
                React.createElement(Form.Item, { label: t("page.im.ticket.content"), name: "content" },
                    React.createElement(FormTinymce, { autoresize_min_height: 130, autoresize_max_height: 200, statusbar: true, paste_data_images: true })),
                roleCan.ticket_attachments && (React.createElement(FormItem, { label: "", name: "attachment_ids" },
                    React.createElement(FieldComponent.upload, { ticketTab: ticketTab }))),
                React.createElement(Form.Item, { label: t("page.im.ticket.customer"), name: "customer_id" },
                    React.createElement(FieldComponent.customer, { setIsScroll: setIsScroll, ticketTab: ticketTab })),
                React.createElement(Form.Item, { label: t("page.im.ticket.receiver"), name: "agent" },
                    React.createElement(FieldComponent.receiver, null)),
                React.createElement(Form.Item, { label: t("page.im.ticket.follower"), name: "follower_ids" },
                    React.createElement(FieldComponent.follower, { admin: admin, currentUser: currentUser })),
                company.category === "mchat_primary" && (React.createElement(FieldComponent.merchant, { setIsScroll: setIsScroll })),
                React.createElement(Form.Item, { label: t("page.im.ticket.status"), name: "status_id" },
                    React.createElement(FieldComponent.selectSearch, { options: ticketStatus.map(function (item) {
                            return {
                                value: item.id,
                                title: item.name,
                            };
                        }) })),
                React.createElement(Form.Item, { label: t("page.im.ticket.priority"), name: "priority_id" },
                    React.createElement(FieldComponent.selectSearch, { options: ticketPriority.map(function (item) {
                            return {
                                value: item.id,
                                title: item.name,
                            };
                        }) })),
                React.createElement(Form.Item, { label: t("page.im.ticket.tags"), name: "tags" },
                    React.createElement(FieldComponent.tag, null)),
                React.createElement(Form.Item, { label: "" },
                    React.createElement("div", { className: style["select-template-wrapper"] },
                        React.createElement("div", { className: style["template-title"] }, t("page.im.ticket.ticketTemplate")),
                        React.createElement(Select, { showSearch: true, showArrow: true, optionFilterProp: "label", onChange: changeTemplate, style: templateSelectStyle, value: template_id, dropdownMatchSelectWidth: false, options: ticketTemplates }))),
                customField.length > 0 && React.createElement(CustomField, { fields: customField }))),
        React.createElement("div", { className: style["bottom-btn-list"] },
            React.createElement(Button, { style: cancelButtonStyle, onClick: closeTicket }, t("cancel")),
            React.createElement(Button, { type: "primary", onClick: function () { return formRef.submit(); } }, t("save")))));
}
export default CreateTicket;
