import React, { useCallback } from "react";
import style from "./style.module.scss";
import { Button, DatePicker, Form, Input, message } from "udesk-ui";
import { CuoHao, DuiHao } from "../../components/icon/icon";
import isFunction from "lodash-es/isFunction";
import moment from "moment";
import { useTranslation } from "react-i18next";
import { postJSON, YYYY_MM_DD_HH_MM_SS } from "../../utils";
var ImVisit = function (props) {
    var t = useTranslation().t;
    var formRef = Form.useForm()[0];
    var event = props.event, setShowVisit = props.setShowVisit, chat = props.chat;
    var handleSave = useCallback(function (allValue) {
        var schedule_time = allValue.schedule_time, remark = allValue.remark;
        if (!schedule_time) {
            message.warn(t("page.im.visit.choiceTime"));
            return;
        }
        if (moment(schedule_time).isBefore(moment())) {
            message.warn(t("page.im.visit.alertTimeTip"));
            return;
        }
        if (isFunction(setShowVisit)) {
            setShowVisit({
                show: false,
            });
        }
        event.updateChat &&
            event.updateChat({
                chat: chat,
                key: "newVisit",
                value: {},
            });
        postJSON("alarms", {
            alarm: {
                call_log_id: null,
                callout_task_id: null,
                callout_task_import_method: null,
                callout_task_name: null,
                callout_task_source: null,
                contact_list_id: null,
                contact_method: "im",
                customer_id: chat.customer_id,
                hide_number: false,
                im_sub_session_id: chat.im_sub_session_id,
                overdue: false,
                remark: remark || "",
                result: "unexecuted",
                schedule_time: moment(schedule_time).toDate(),
                ticket_id: null,
                user: null,
                user_id: null,
            },
        }).then(function (res) {
            var _a;
            if (res.code === "1000" || res.code === 1000) {
                message.success("保存成功");
            }
            else {
                message.warning((res === null || res === void 0 ? void 0 : res.code_message) ||
                    ((_a = res === null || res === void 0 ? void 0 : res.exception) === null || _a === void 0 ? void 0 : _a.message) ||
                    t("page.im.imMessages.voiceOvertime"));
            }
        });
    }, [chat, event, setShowVisit, t]);
    var handleClose = useCallback(function () {
        setShowVisit({
            show: false,
        });
        event.updateChat &&
            event.updateChat({
                chat: chat,
                key: "newRecord",
                value: {},
            });
    }, [chat, event, setShowVisit]);
    var save = useCallback(function () {
        formRef.submit();
    }, [formRef]);
    return (React.createElement("div", { className: style["visit-container"] },
        React.createElement("div", { className: style["visit-header"] },
            React.createElement("div", { className: style["header-name"] }, t("page.im.visit.title")),
            React.createElement("div", { className: style["header-icons"] },
                React.createElement("span", { onClick: save },
                    React.createElement(DuiHao, null)),
                React.createElement("span", { onClick: handleClose },
                    React.createElement(CuoHao, null)))),
        React.createElement("div", { className: style["visit-body"] },
            React.createElement(Form, { form: formRef, layout: "vertical", onFinish: handleSave, initialValues: {
                    schedule_time: moment(),
                } },
                React.createElement(Form.Item, { label: "\u8BA1\u5212\u65F6\u95F4", name: "schedule_time" },
                    React.createElement(DatePicker, { format: YYYY_MM_DD_HH_MM_SS, showTime: true })),
                React.createElement(Form.Item, { label: "\u5907\u6CE8", name: "remark" },
                    React.createElement(Input.TextArea, null)),
                React.createElement(Form.Item, null,
                    props.showCancel && (React.createElement(Button, { type: "primary", onClick: handleClose }, t("cancel"))),
                    React.createElement(Button, { type: "primary", htmlType: "submit", size: "small" }, t("save")))))));
};
export default ImVisit;
