import { __spreadArray } from "tslib";
import React, { useCallback, useEffect, useState } from 'react';
import { Select, Spin } from 'udesk-ui';
import { ajax } from '@udesk-webapp/utils';
import debounce from 'lodash-es/debounce';
var CompanyField = React.memo(function (props) {
    var disabled = props.disabled, valueProps = props.value, placeholder = props.placeholder, style = props.style, onChange = props.onChange, setIsScroll = props.setIsScroll;
    var _a = useState({}), value = _a[0], setValue = _a[1]; // 选择的选项
    var _b = useState(false), fetching = _b[0], setFetching = _b[1]; // 无数据时展示
    var _c = useState([]), options = _c[0], setOptions = _c[1]; // 搜索到的数据
    var _d = useState(0), lastFetchId = _d[0], setLastFetchId = _d[1];
    var _e = useState(1), page = _e[0], setPage = _e[1];
    var _f = useState(0), totalPages = _f[0], setTotalPages = _f[1];
    var _g = useState(''), searchValue = _g[0], setSearchValue = _g[1];
    useEffect(function () {
        setValue(valueProps);
    }, [valueProps]);
    // ant select onSearch 输入法时连续触发多次，
    // 防抖起基础作用，根本问题需要监听input组件onCompositonStart
    var beforeSearch = useCallback(debounce(function (_searchValue) {
        setSearchValue(_searchValue);
        setOptions([]);
        setPage(1);
        fetchCompany(_searchValue);
    }, 300), []);
    // 搜索公司
    var fetchCompany = useCallback(function (_searchValue) {
        setLastFetchId(function () { return lastFetchId + 1; });
        var fetchId = lastFetchId;
        setFetching(true);
        ajax()
            .postJSON('/cs/ts/api/organizations/search', {
            page: page,
            page_size: 20,
            query: _searchValue,
        })
            .then(function (res) {
            setPage(page + 1);
            setTotalPages(res.meta.total_pages);
            if (fetchId !== lastFetchId) {
                // for fetch callback order
                return;
            }
            var dataClone = res.organizations.map(function (item) { return ({
                label: item.name,
                value: item.id,
            }); });
            setOptions(__spreadArray(__spreadArray([], options, true), dataClone, true));
            setFetching(false);
        });
    }, [lastFetchId, page, options]);
    //
    var handleChange = useCallback(function (_value) {
        setValue(_value);
        setFetching(false);
        if (typeof onChange === 'function') {
            onChange(_value);
        }
    }, [onChange]);
    // 处理滚动请求数据
    var handleScroll = useCallback(function (e) {
        e.stopPropagation();
        if (e.target.scrollTop === e.target.scrollHeight - e.target.clientHeight) {
            if (page < totalPages) {
                fetchCompany(searchValue);
            }
        }
    }, [page, totalPages, fetchCompany, searchValue]);
    // FIXME: 此逻辑不应当在此处理，应当暴露focus和blur在外统一处理
    // 聚焦时  将外部表单取消滚动  防止滚动穿透
    var handleFocus = useCallback(function () {
        if (typeof setIsScroll === 'function') {
            setIsScroll(false);
        }
        fetchCompany(searchValue);
    }, [setIsScroll, fetchCompany, searchValue]);
    // 失焦时  将外部滚动复原
    var handleBlur = useCallback(function () {
        if (typeof setIsScroll === 'function') {
            setIsScroll(true);
        }
    }, [setIsScroll]);
    return (React.createElement(Select, { allowClear: true, showSearch: true, showArrow: true, labelInValue: true, disabled: disabled, value: value, options: options, placeholder: placeholder, filterOption: false, style: style, notFoundContent: fetching ? (React.createElement(Spin, { size: "small" })) : (React.createElement("div", { style: { display: 'flex', flexWrap: 'nowrap', padding: 8 } }, "\u6CA1\u6709\u53EF\u9009\u9879")), getPopupContainer: function (triggerNode) { return triggerNode; }, onSearch: beforeSearch, onChange: handleChange, onPopupScroll: handleScroll, onFocus: handleFocus, onBlur: handleBlur }));
});
CompanyField.defaultProps = {
    style: { width: '100%' },
    disabled: false,
    placeholder: '',
};
export default CompanyField;
