import { __rest } from "tslib";
import React, { useCallback, useEffect, useImperativeHandle, useLayoutEffect, useRef, useState } from 'react';
import css from './index.module.less';
import { removeHTMLTag } from './utils/common';
import { uploadFile } from './utils/upload';
import UploadModal from './uploadModal/index';
import Translate from './translate/index';
import { I18nextProvider } from 'react-i18next';
import newI18n, { useTranslation } from './i18n';
import tinymce from 'tinymce';
import { message } from 'udesk-ui';
var defaultStyle = {
    color: 'rgba(0, 0, 0, 0.25)'
};
var error = {
    color: 'rgba(251, 81, 39, 1)'
};
// 项目语言码转Tiny语言码
var Locale2TinyLangCodeMap = {
    'en-US': 'en_GB',
    'zh-CN': 'zh_CN',
    'zh-TW': 'zh_TW',
    'zh-HK': 'zh_HK',
    'ja-JP': 'ja',
    'ar-EG': 'ar',
    'ru-RU': 'ru',
    'th-TH': 'th_TH',
    'tr-TR': 'tr_TR',
    'vi-VN': 'vi_VN',
    'de': 'de',
    'fr': 'fr_FR',
    'pt-PT': 'pt_PT',
    'es-ES': 'es',
    'el': 'el',
    'it': 'it',
    'ko': 'ko_KR',
    'id': 'id',
};
var FormTinymce = React.forwardRef(function (_a, ref) {
    var paste_data_images = _a.paste_data_images, readonly = _a.readonly, marking = _a.marking, props = __rest(_a, ["paste_data_images", "readonly", "marking"]);
    var _b = props.showImg, showImg = _b === void 0 ? true : _b, _c = props.translate, translate = _c === void 0 ? null : _c, postRender = props.postRender;
    var toolBars = "".concat(showImg ? "upimg | " : "", " bold italic underline | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | formatselect fontsizeselect | link image |  fontselect  forecolor backcolor code ").concat(translate ? " | translate" : "");
    var value = props.value, onChange = props.onChange, autoresize_min_height = props.autoresize_min_height, _d = props.autoresize_max_height, autoresize_max_height = _d === void 0 ? 50 : _d, _e = props.statusbar, statusbar = _e === void 0 ? false : _e, _f = props.toolbar, toolbar = _f === void 0 ? toolBars : _f, maxLength = props.maxLength, backgroundColor = props.backgroundColor;
    //多语言配置
    var _g = useTranslation(), t = _g.t, i18n = _g.i18n;
    var _h = useState(removeHTMLTag(value || '')), editorContent = _h[0], setContent = _h[1]; //更新富文本内容
    var _j = useState(false), showModal = _j[0], setShowModal = _j[1];
    var _k = useState(false), showOriginal = _k[0], setShowOriginal = _k[1];
    var tinymceInstance = useRef(null);
    var targetRef = useRef(null);
    var hasManualChange = useRef(false); // 是否手动输入了文本
    var idRef = useRef("".concat(Math.floor(Math.random() * 10000)).concat(new Date().getTime()));
    var language = i18n.language;
    // 内容改变时    去html标签 计算输入的长度
    var handleEditorChange = useCallback(function () {
        var _a;
        var content = (_a = tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.current) === null || _a === void 0 ? void 0 : _a.getContent();
        setContent(removeHTMLTag(content || ''));
        onChange === null || onChange === void 0 ? void 0 : onChange(content || '');
    }, [onChange]);
    useImperativeHandle(ref, function () { return ({
        insertContent: function (content) {
            var _a, _b, _c;
            if (!tinymceInstance) {
                return;
            }
            if ((tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.current) && ((_b = (_a = tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.current) === null || _a === void 0 ? void 0 : _a.selection) === null || _b === void 0 ? void 0 : _b.getSel())) {
                tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.current.execCommand("mceInsertContent", false, content);
            }
            else {
                var curContent = ((_c = tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.current) === null || _c === void 0 ? void 0 : _c.getContent()) || '';
                setContent(curContent + content);
            }
        }
    }); });
    var hideImageUploadModal = useCallback(function () { return setShowModal(false); }, []);
    //点击弹窗的确定  将选择的图片插入并 关闭弹窗
    var addImg = useCallback(function (imgList) {
        if (!tinymceInstance) {
            return;
        }
        imgList.forEach(function (item) {
            var _a;
            (_a = tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.current) === null || _a === void 0 ? void 0 : _a.execCommand('mceInsertContent', false, "<img src=".concat(item.src, " alt=\"\"/>"));
        });
        setShowModal(false);
        hasManualChange.current = true;
    }, []);
    useLayoutEffect(function () {
        if (!targetRef.current) {
            return;
        }
        var tinymceLang = Locale2TinyLangCodeMap[language] || 'en_GB';
        var ed = new tinymce.Editor(idRef.current, {
            autoresize_min_height: autoresize_min_height,
            autoresize_max_height: autoresize_max_height,
            autoresize_bottom_margin: 5,
            menubar: false,
            language: tinymceLang,
            statusbar: statusbar,
            elementpath: false,
            branding: false,
            paste_data_images: paste_data_images,
            convert_urls: false,
            relative_urls: false,
            remove_script_host: false,
            document_base_url: '',
            extended_valid_elements: '-table[id|border=2|cellspacing|cellpadding|width=300|frame|rules|height=100|align|summary|bgcolor|background|bordercolor|style=border-color:black]',
            images_upload_handler: function (blobInfo, success, failure) {
                var _a, _b;
                var curContent = ((_a = tinymceInstance.current) === null || _a === void 0 ? void 0 : _a.getContent()) || '';
                var loadingImg = (location.origin || '') + "/entry/images/loading.svg?" + blobInfo.name();
                // const imgSrc = "data:image/png;base64," + blobInfo.base64();
                var newContent = curContent.replace(/data:image\/(png|gif|jpg|jpeg|tif|bmp);base64,[^"]*/, loadingImg);
                (_b = tinymceInstance.current) === null || _b === void 0 ? void 0 : _b.setContent(newContent || '');
                message.info(t('uploader.uploadImgIng'));
                uploadFile(new File([blobInfo.blob()], blobInfo.filename()), { marking: 'web.mg.rich_editor_copy' }).then(function (resp) {
                    var _a;
                    // success(resp.src);
                    var imgArr = (_a = tinymceInstance.current) === null || _a === void 0 ? void 0 : _a.dom.select('img');
                    imgArr === null || imgArr === void 0 ? void 0 : imgArr.forEach(function (item) {
                        var _a, _b, _c;
                        if (item.currentSrc === loadingImg) {
                            (_a = tinymceInstance.current) === null || _a === void 0 ? void 0 : _a.selection.select(item);
                            (_b = tinymceInstance.current) === null || _b === void 0 ? void 0 : _b.dom.remove(item);
                            (_c = tinymceInstance.current) === null || _c === void 0 ? void 0 : _c.insertContent("<img src=\"".concat(resp.src, "\" data-mce-src=\"").concat(resp.src, "\">"));
                        }
                    });
                    hasManualChange.current = true;
                }).catch(function (e) {
                    failure(t('uploader.uploadImgFail'));
                });
            },
            plugins: [
                "link ".concat(showImg ? "image" : '', " autoresize lists charmap preview hr "),
                'visualchars insertdatetime nonbreaking',
                'table contextmenu textcolor',
                'paste code imagetools'
            ],
            toolbar: toolbar,
            // body_class: 'panel-body',
            content_style: backgroundColor ? ".mce-content-body{background-color: transparent}" : '',
            setup: function (instance) {
                tinymceInstance.current = instance;
                if (showImg) {
                    instance.addButton('upimg', {
                        text: t('uploader.uploadImg'),
                        onclick: function () {
                            setShowModal(true);
                        }
                    });
                }
                if (translate) {
                    instance.addButton('translate', {
                        tooltip: t('statusbar.translate'),
                        icon: 'translate',
                        onclick: function () {
                            setShowOriginal(function (v) { return !v; });
                        }
                    });
                }
                instance.on('PostRender', function (e) {
                    postRender === null || postRender === void 0 ? void 0 : postRender(e);
                });
            }
        }, tinymce.EditorManager);
        ed.render();
        tinymceInstance.current = ed;
        return function () {
            ed === null || ed === void 0 ? void 0 : ed.destroy();
        };
    }, []);
    useEffect(function () {
        var _a;
        (_a = tinymceInstance.current) === null || _a === void 0 ? void 0 : _a.setMode(readonly ? 'readonly' : 'design');
    }, [readonly]);
    useEffect(function () {
        if (tinymceInstance.current && handleEditorChange) {
            tinymceInstance.current.on('change', handleEditorChange);
            return function () {
                tinymceInstance.current.off('change', handleEditorChange);
            };
        }
    }, [handleEditorChange]);
    useLayoutEffect(function () {
        var _a, _b;
        if (tinymceInstance && props.value !== ((_a = tinymceInstance.current) === null || _a === void 0 ? void 0 : _a.getContent())) {
            (_b = tinymceInstance.current) === null || _b === void 0 ? void 0 : _b.setContent(props.value || '');
        }
    }, [props.value]);
    useEffect(function () {
        if (tinymceInstance.current) {
            var handleEditorKeyup_1 = function () {
                var _a;
                var curContent = ((_a = tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.current) === null || _a === void 0 ? void 0 : _a.getContent()) || '';
                hasManualChange.current = !!curContent; // 有内容表示手动修改过  如果清空 重置为没有手动修改过
            };
            tinymceInstance.current.on('keyup', handleEditorKeyup_1);
            return function () {
                tinymceInstance.current.off('keyup', handleEditorKeyup_1);
            };
        }
    }, []);
    var onTranslate = useCallback(function (params) {
        if (translate && translate.onTranslate) {
            return translate.onTranslate(params).then(function (res) {
                var _a, _b, _c, _d;
                if (!res)
                    return true;
                var cont = formatTranslateContent(res);
                if (!hasManualChange.current) { // 如果没有手动修改过 直接用译文替换
                    (_a = tinymceInstance.current) === null || _a === void 0 ? void 0 : _a.setContent(cont);
                }
                else { // 如果手动修改过  后面插入
                    if ((tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.current) && ((_c = (_b = tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.current) === null || _b === void 0 ? void 0 : _b.selection) === null || _c === void 0 ? void 0 : _c.getSel())) {
                        tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.current.execCommand('mceInsertContent', false, cont);
                    }
                    else {
                        var curContent = ((_d = tinymceInstance === null || tinymceInstance === void 0 ? void 0 : tinymceInstance.current) === null || _d === void 0 ? void 0 : _d.getContent()) || '';
                        setContent(curContent + cont);
                    }
                }
                handleEditorChange();
                return true;
            });
        }
        return Promise.resolve(true);
    }, [translate]);
    return (React.createElement(I18nextProvider, { i18n: newI18n },
        showOriginal && React.createElement(Translate, { onTranslate: onTranslate, persistenceKey: translate === null || translate === void 0 ? void 0 : translate.persistenceKey }),
        React.createElement("div", { className: css['rich-text-wrapper'], style: { backgroundColor: backgroundColor } },
            React.createElement("textarea", { ref: targetRef, value: props.value || '', id: idRef.current }),
            maxLength && React.createElement("div", { className: 'editor-num', style: editorContent.length > maxLength ? error : defaultStyle },
                editorContent.length,
                "/",
                maxLength),
            React.createElement(UploadModal, { show: showModal, close: hideImageUploadModal, addImg: addImg, marking: marking }))));
});
// 保留原文框格式
function formatTranslateContent(res) {
    if (/\r\n|\n/.test(res)) {
        var resArr = res.split(/\r\n|\n/);
        var c_1 = '';
        resArr.forEach(function (r) {
            c_1 += "<p>".concat(r, "</p>");
        });
        return c_1;
    }
    return res;
}
FormTinymce.defaultProps = {
    paste_data_images: false
};
export default FormTinymce;
