import React, { cloneElement, useCallback, useMemo, useRef } from 'react';
import TreeSelect from "antd/es/tree-select";
import Tag from 'antd/es/tag';
import "antd/es/tree-select/style/index.less";
import "antd/es/select/style/index.less";
import 'antd/es/tag/style/index.less';
import 'antd/es/empty/style/index.less';
import get from 'lodash-es/get';
export function forEachTree(treeData, callback, options) {
    if (treeData === void 0) { treeData = []; }
    var _a = (options || {}).nodeChildrenPath, nodeChildrenPath = _a === void 0 ? 'subs' : _a;
    treeData === null || treeData === void 0 ? void 0 : treeData.forEach(function (i) {
        callback(i);
        forEachTree(get(i, nodeChildrenPath), callback, options);
    });
}
function buildTreeData(selectOptions, parentCheckable, parentKey) {
    if (!selectOptions) {
        return [];
    }
    return selectOptions.map(function (i) {
        var title = i.title;
        var isLeaf = false;
        if (i.node_type === 'leaf') {
            isLeaf = true;
        }
        else if (!i.node_type && (!i.subs || i.subs.length === 0)) {
            isLeaf = true;
        }
        if (isLeaf) {
            title = React.createElement(Tag, { color: i.color || '#00B38B' },
                i.icon_url ?
                    React.createElement("img", { src: i.icon_url, alt: '', style: { width: "1em", height: "1em", marginRight: '3px' } }) : null,
                i.title);
        }
        return {
            title: title,
            value: i.value,
            checkable: parentCheckable ? true : isLeaf,
            disableCheckbox: parentCheckable ? false : !isLeaf,
            children: buildTreeData(i.subs, parentCheckable),
            titleText: i.title
        };
    });
}
var tagRender = function (props) {
    var label = props.label, closable = props.closable, onClose = props.onClose;
    return cloneElement(label, { closable: closable, onClose: onClose });
};
var TagTree = function (_a) {
    var value = _a.value, formData = _a.formData, onChange = _a.onChange, size = _a.size, disabled = _a.disabled;
    // const treeNodes = useMemo(() => renderTreeNode(formData.select_options, formData.parent_checkable), [formData.parent_checkable, formData.select_options]);
    var treeData = useMemo(function () { return buildTreeData(formData.select_options, formData.parent_checkable); }, [formData.parent_checkable, formData.select_options]);
    var ref = useRef(null);
    var getPopupContainer = useCallback(function () { return ref.current || document.body; }, []);
    var disableStatus = typeof disabled === 'function' ? disabled() : disabled;
    var valueAry = useMemo(function () {
        if (typeof value === 'string') {
            return value.split(',');
        }
        else if (value) {
            return value;
        }
        else {
            return [];
        }
    }, [value]);
    var safeValueAry = useMemo(function () {
        if (valueAry.length === 0) {
            return [];
        }
        var result = [];
        forEachTree(formData.select_options, function (i) { return (valueAry === null || valueAry === void 0 ? void 0 : valueAry.includes(i.value)) && result.push(i.value); }, { nodeChildrenPath: 'subs' });
        return result;
    }, [formData.select_options, valueAry]);
    return (React.createElement("div", { ref: ref, style: { position: 'relative' } },
        React.createElement(TreeSelect, { showSearch: true, style: { width: '100%' }, value: safeValueAry, dropdownStyle: { maxHeight: 400, overflow: 'auto' }, placeholder: formData.input_tip, allowClear: true, treeCheckable: true, onChange: onChange, getPopupContainer: getPopupContainer, transitionName: '', treeDefaultExpandAll: formData.default_expand_all, multiple: true, tagRender: tagRender, treeData: treeData, size: size, disabled: disableStatus, treeNodeFilterProp: 'titleText' })));
};
export default TagTree;
