import { __assign, __rest, __spreadArray } from "tslib";
import React, { useCallback, useEffect, useMemo, useState, useContext } from 'react';
import { Select, Tag, Button, Input } from 'udesk-ui';
import ClassNames from 'classnames';
import { StarFilled } from '@ant-design/icons';
import { uniqBy, isNil, without } from 'lodash-es';
import { TicketContext } from '@udesk-webapp/ticket-signature/es/utilities';
import './index.module.scss';
var FollowerField = React.memo(function (props) {
    var _a;
    var valueProps = props.value, disabled = props.disabled, readOnly = props.readOnly, placeholder = props.placeholder, _b = props.onChange, onChange = _b === void 0 ? function () { } : _b, currentUser = props.currentUser, admin = props.admin, groupAgentsWithStatus = props.groupAgentsWithStatus, startClassName = props.startClassName, // ts_react中没用？其他还有在用么？
    _c = props.dropdownMatchSelectWidth, // ts_react中没用？其他还有在用么？
    dropdownMatchSelectWidth = _c === void 0 ? true : _c, restProps = __rest(props, ["value", "disabled", "readOnly", "placeholder", "onChange", "currentUser", "admin", "groupAgentsWithStatus", "startClassName", "dropdownMatchSelectWidth"]);
    var adminOption = { value: admin.id || null, label: admin.nick_name || 'Admin' };
    var currentUserOption = {
        value: currentUser.id || null,
        label: currentUser.nick_name || 'CurrentUser',
    };
    var _d = useState([]), value = _d[0], setValue = _d[1];
    var _e = useState([]), followerData = _e[0], setFollowerData = _e[1];
    var _f = useState(function () {
        return valueProps && valueProps.includes(currentUserOption.value) || false;
    }), starState = _f[0], setStarState = _f[1];
    var UNSAFE_agent = useContext(TicketContext).agent; // 客服组
    var agents = !isNil(groupAgentsWithStatus) ? groupAgentsWithStatus : (UNSAFE_agent || []);
    // 从客服组中获取客服 ？留疑为什么不直接给客服呢？
    var options = useMemo(function () {
        var list = [];
        agents.length &&
            agents.forEach(function (item) {
                var _a;
                if ((_a = item === null || item === void 0 ? void 0 : item.children) === null || _a === void 0 ? void 0 : _a.length) {
                    list = __spreadArray(__spreadArray([], list, true), item.children.map(function (_a) {
                        var label = _a.label, value = _a.value;
                        return ({ label: label, value: value });
                    }), true);
                }
            });
        // TODO: 如果客服组里拿到的客服没有当前用户呢？是否考虑隐藏星星按钮？
        currentUserOption.value && list.push(currentUserOption); // 兜底加当前用户
        list = uniqBy(list, 'value'); // 去重
        list.push(adminOption); // 加管理员
        return list;
    }, [admin, agents]);
    // 设置默认的value值 已删除的选项不显示
    useEffect(function () {
        var initValue = [];
        var initFollowerData = options.filter(function (item) {
            var isHas = valueProps && valueProps.includes(item.value);
            isHas && initValue.push(item.value);
            return isHas;
        });
        setValue(initValue);
        setFollowerData(initFollowerData);
    }, [valueProps, options]);
    // 选项变化时  根据选项中有没有包含当前客服 改变星星状态
    useEffect(function () {
        setStarState(value.includes(currentUserOption.value));
    }, [currentUserOption, value]);
    // 点击星星事件
    var onClickStar = useCallback(function () {
        var changeValues = [];
        var changeFollower = [];
        if (starState) {
            // 删除选择的当前的客服
            changeValues = without(value, currentUserOption.value);
            changeFollower = without(followerData, currentUserOption);
        }
        else {
            changeValues = __spreadArray(__spreadArray([], value, true), [currentUserOption.value], false);
            changeFollower = __spreadArray(__spreadArray([], followerData, true), [currentUserOption], false);
        }
        // 作为一个受控组件，此处set没有意义
        // setValue(changeValues);
        // setFollowerData(changeFollower);
        onChange(changeValues, changeFollower);
        // 改变选择时会触发上边的useEffect处理星星状态  此处不进行处理
    }, [currentUserOption, onChange, starState, value, followerData]);
    // 选项变化时回调
    var handleChange = useCallback(function (_value, _option) {
        setValue(_value);
        setFollowerData(_option);
        onChange(_value, _option);
    }, [onChange]);
    // 筛选规则
    var handleFilter = useCallback(function (input, option) { return option.label.toLowerCase().indexOf(input.toLowerCase()) > -1; }, []);
    var readOnlyEle = useMemo(function () {
        if (!followerData.length) {
            return React.createElement("span", null, "- -");
        }
        return (React.createElement("div", { className: "followerField-view-wrapper" }, followerData.map(function (item, index) {
            return React.createElement(Tag, { key: index, className: "custom-tag" }, item.label || '');
        })));
    }, [followerData]);
    if (readOnly) {
        return React.createElement(React.Fragment, null, readOnlyEle);
    }
    return (React.createElement(Input.Group, { compact: true, className: "followerField-wrapper" },
        React.createElement(Select, __assign({}, restProps, { disabled: disabled, showArrow: true, mode: "multiple", value: value, options: options, placeholder: placeholder, style: { width: disabled ? '100%' : 'calc(100% - 40px)' }, onChange: handleChange, filterOption: handleFilter, dropdownMatchSelectWidth: dropdownMatchSelectWidth })),
        disabled ? null : (React.createElement(Button, { className: ClassNames("star-icon", (_a = {
                    "follower-starIcon-select": starState
                },
                _a["".concat(startClassName)] = startClassName,
                _a)), icon: React.createElement(StarFilled, null), onClick: onClickStar }))));
});
FollowerField.defaultProps = {
    allowClear: true,
    disabled: false,
    readOnly: false,
    placeholder: '',
    value: [],
    onChange: function () { },
    admin: {},
    currentUser: {},
    getPopupContainer: function (triggerNode) { return triggerNode; },
};
export default FollowerField;
