import React, { useCallback, useMemo, useRef, useState } from 'react';
import { useTranslation } from 'react-i18next';
import { Checkbox } from 'udesk-ui';
import moment from 'moment';
import * as IconComponent from './icon';
import { Pencil, Resend, Unread, Read } from './icon';
import DOMPurify from 'dompurify';
import * as MessageComponent from './components';
import MessageVideo from './components/message-video';
import KnowledgeGraph from './knowledge-graph';
import css from './style.module.less';
import { EyeInvisibleOutlined } from '@ant-design/icons';
import Menu from './menu';
import classNames from "classnames";
var ImLog = function (_a) {
    var _b;
    var msg = _a.msg, onReEdit = _a.onReEdit, allowAddToCustomerInfo = _a.allowAddToCustomerInfo, allowAddToRobotKM = _a.allowAddToRobotKM, allowAddToKM = _a.allowAddToKM, allowRecall = _a.allowRecall, onSelectEmail = _a.onSelectEmail, onSelectTel = _a.onSelectTel, onRecall = _a.onRecall, onHide = _a.onHide, onRef = _a.onRef, onCheck = _a.onCheck, onUnHide = _a.onUnHide, allowVoiceToText = _a.allowVoiceToText, allowSearchImRecord = _a.allowSearchImRecord, onAddToKM = _a.onAddToKM, onSelectText = _a.onSelectText, onVoiceToText = _a.onVoiceToText, onAddToRobotKM = _a.onAddToRobotKM, allowUnHide = _a.allowUnHide, allowReedit = _a.allowReedit, msgdisplayStatus = _a.msgdisplayStatus, hideEnable = _a.hideEnable, refEnable = _a.refEnable, onReSend = _a.onReSend, agent = _a.agent, chat = _a.chat, web_message_display_status = _a.web_message_display_status, enabledSmartAssistance = _a.enabledSmartAssistance, realTimeTranslation = _a.realTimeTranslation, onSelectSuggestion = _a.onSelectSuggestion, robotBaseURL = _a.robotBaseURL, currentUser = _a.currentUser, kcsSdk = _a.kcsSdk, msgColor = _a.msgColor, msgRecvColor = _a.msgRecvColor, area = _a.area, wechatSuffix = _a.wechatSuffix, showCheckbox = _a.showCheckbox, isChecked = _a.isChecked, imLogCheckCb = _a.imLogCheckCb, openRecordCb = _a.openRecordCb, reloadImageTimes = _a.reloadImageTimes;
    //多语言
    var t = useTranslation().t;
    allowUnHide = allowUnHide === false ? false : true;
    // const { agents, chat, user, company, roleCan, currentTab,event} = udeskWebappContext
    // const chat = chatContext.chat;
    //是否是消息
    var isInfo = useMemo(function () {
        var contentType = msg.type;
        //处理isInfo
        var ifAction = function () {
            var type = contentType || 'message';
            var eventTypes = [
                'info_transfer',
                'transfer_send',
                'transfer_accept',
                'transfer_deny',
                'transfer_timeout',
                'ask_transfer_send',
                'ask_transfer_accept',
                'ask_transfer_deny',
                'ask_transfer_timeout',
                'video_call_invitation',
                'video_call_invitation_accept',
                'video_call_invitation_deny',
                'video_call_invitation_cancel',
                'video_call_invitation_hangup',
                'video_call_invitation_busy',
                'offline_reassign',
                'leave_message_assign',
                'assign_queue',
                'voice_call_invitation',
                'voice_call_invitation_accept',
                'voice_call_invitation_deny',
                'voice_call_invitation_cancel',
                'voice_call_invitation_hangup',
                'voice_call_invitation_busy',
                'info_offline_transfer'
            ];
            return msg.show_type === 'action' || eventTypes.includes(type);
        };
        var types = [
            'info',
            'info_transfer',
            'pre',
            'survey',
            'close',
            'info_appoint',
            'active_guest',
            'form',
            'form_received',
            'lock',
            'start_session',
            'vcall',
            'invitation',
            'customer_close',
            'agent_open_session',
            'suspend_expire',
            'info_offline_transfer',
            'jump_action'
        ];
        return (!types.includes(contentType) && !ifAction());
    }, [msg]);
    var eventText = useMemo(function () {
        var _a, _b;
        var event_text = msg.event_text || ((_a = msg.content) === null || _a === void 0 ? void 0 : _a.event_text);
        if (msg.timeout_freq == 1) {
            event_text = t('page.im.record.tip.noAnswer');
        }
        else if (msg.timeout_freq == 2) {
            event_text = t('page.im.record.tip.customerOvertime');
        }
        var name = (agent === null || agent === void 0 ? void 0 : agent.aliase) || (agent === null || agent === void 0 ? void 0 : agent.nick_name) || '';
        if (msg.timeout_auto_msg) {
            event_text = t('page.im.record.tip.agentOvertime', { agentName: name });
        }
        if (msg.msg_close) {
            if (msg.inviter && msg.inviter.jid) {
                name = msg.agent_nick_name || name; // 三方会话使用消息中的坐席名称
            }
            event_text = t('page.im.record.tip.agentCloseChat', { agentName: name });
        }
        if (msg.customer_close) {
            event_text = t('page.im.record.tip.customerCloseChat');
        }
        //is_active_guest:根据后台存储的消息类型为active_guest的属性
        //active_guest：xmpp收到的第一条消息的额外属性
        if (!isInfo) {
            event_text = msg.content || msg.data.content;
        }
        if (msg.type === 'jump_action') {
            event_text = msg.event_text || ((_b = msg.content) === null || _b === void 0 ? void 0 : _b.event_text);
        }
        if (msg.type === 'info_transfer') {
            event_text = msg.content || msg.data.content || t('page.im.record.tip.chatTransfer');
        }
        if (msg.type === 'suspend_expire') {
            event_text = msg.content || msg.data.content || t('page.im.record.tip.suspendExpire');
        }
        if (msg.type === 'offline_reassign' || msg.type === 'leave_message_assign' || msg.type === 'assign_queue') {
            event_text = msg.content || msg.data.content;
        }
        if (msg.type === 'info_offline_transfer') {
            event_text = msg.content || msg.data.content || t('page.im.record.tip.offlineChatTransfer', { agentName: name });
        }
        return event_text || '';
    }, [msg.event_text,
        msg.content,
        msg.timeout_freq,
        msg.timeout_auto_msg,
        msg.msg_close,
        msg.customer_close,
        msg.type,
        msg.data.content, agent === null || agent === void 0 ? void 0 : agent.aliase, agent === null || agent === void 0 ? void 0 : agent.nick_name, isInfo,
        t]);
    var timeText = useMemo(function () {
        if (eventText) {
            return moment(new Date(msg.created_at)).format('YY/MM/DD HH:mm:ss');
        }
        return '';
    }, [eventText, msg.created_at]);
    var handleMsgPre = useCallback(function () {
        var contentType = msg.type;
        var res = [];
        if (contentType === 'pre') {
            res.push(React.createElement("div", { key: 'pre' + msg.id, className: css['msg-content'], style: { opacity: 0, filter: 'opacity(0)', display: 'flex' } },
                React.createElement("div", { dangerouslySetInnerHTML: { __html: msg.content } }),
                React.createElement("div", null, "__"),
                React.createElement(Pencil, null)));
        }
        else {
            if (msg.send_status !== 'rollback') {
                if (timeText) {
                    res.push(React.createElement("div", { key: 'time-text' + msg.id, className: css['time-bar'] }, DOMPurify.sanitize(timeText)));
                }
                if (eventText) {
                    res.push(React.createElement("div", { key: 'event-text' + msg.id, className: "".concat(css['event-bar'], " ").concat((contentType === 'transfer' || contentType === 'info_transfer' || contentType === 'info_offline_transfer') ? css['red'] : '') }, DOMPurify.sanitize(eventText)));
                }
            }
        }
        return res;
    }, [msg.type, msg.id, msg.content, msg.send_status, timeText, eventText]);
    var msgType = useMemo(function () {
        var _a, _b, _c;
        var contentType = msg.type;
        try {
            if ((msg === null || msg === void 0 ? void 0 : msg.content_raw) && ((_c = (_b = (_a = JSON.parse(msg.content_raw)) === null || _a === void 0 ? void 0 : _a.robotAnswer) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.msgType) === 'db_answer') {
                contentType = 'echarts';
            }
        }
        catch (e) {
            //do nothing
        }
        return contentType;
    }, [msg.type, msg.content_raw]);
    //加载当前对应的消息类型
    var handleMessageComponent = useCallback(function () {
        var Component = MessageComponent[msgType];
        return Component ? React.createElement(Component, { msg: msg, trans_text: msg.translation, chatPlatform: chat.platform, smart_assistance_enabled: enabledSmartAssistance, real_time_translation: realTimeTranslation, onSelectSuggestion: onSelectSuggestion, robotBaseURL: robotBaseURL, kcsSdk: kcsSdk, area: area, openRecordCb: openRecordCb, reloadImageTimes: reloadImageTimes }) : null;
    }, [kcsSdk, chat.platform, enabledSmartAssistance, msg, msgType, onSelectSuggestion, realTimeTranslation]);
    var reEdit = useCallback(function () { return onReEdit === null || onReEdit === void 0 ? void 0 : onReEdit(msg.content, msg); }, [msg.content, msg, onReEdit]);
    var handleStatusIcon = useCallback(function () {
        var status = ('status_' + msg.send_status);
        var StatusComponent = IconComponent[status];
        return StatusComponent ? React.createElement(StatusComponent, { style: { fontSize: '16px' } }) : '';
    }, [msg.send_status]);
    var getMsgHeader = useMemo(function () {
        var _a, _b, _c;
        // 处理日期  如果发消息时日是当天  只显示时间，不显示日期
        var time;
        if (moment(msg.created_at).format('YYYY/MM/DD') === moment().format('YYYY/MM/DD')) {
            time = moment(msg.created_at).format('HH:mm:ss');
        }
        else {
            time = moment(msg.created_at).format('MM-DD HH:mm:ss');
        }
        if (msg.is_receive) {
            if (wechatSuffix || ((_a = msg.data) === null || _a === void 0 ? void 0 : _a.senderType)) {
                return (React.createElement(React.Fragment, null,
                    msg.sender_name || ((_b = msg.data) === null || _b === void 0 ? void 0 : _b.from),
                    React.createElement("span", { className: css['customer-wx-icon'] }, ((_c = msg.data) === null || _c === void 0 ? void 0 : _c.senderType) === 1 ? '@' + (msg.data.corpName || "企微") : '@微信'),
                    "\u00A0\u00A0",
                    time));
            }
            return msg.sender_name + ' ' + time;
        }
        else {
            return time + ' ' + msg.sender_name;
        }
    }, [msg.created_at, msg.is_receive, msg.sender_name, wechatSuffix]);
    var send_status_msg = useMemo(function () {
        if (msg.send_status === 'fail' && msg.respone) {
            return msg.respone;
        }
        else {
            var effectiveStatus = ['off_sending', 'arrive', 'off_arrive', 'fail', 'rollback'];
            if (msg.send_status && effectiveStatus.includes(msg.send_status)) {
                return t('page.im.log.' + msg.send_status);
            }
        }
        return '';
    }, [msg, t]);
    var _c = useState(''), selection = _c[0], setSelection = _c[1];
    var onMenuMouseUp = useCallback(function () {
        var _a, _b;
        var type = msg.type;
        if (window.getSelection && ['message', 'info', 'rich'].includes(type)) {
            var _selection = (_b = (_a = window.getSelection()) === null || _a === void 0 ? void 0 : _a.toString().replace(/^\s|\s$/g, '').substring(0, 20)) !== null && _b !== void 0 ? _b : '';
            setSelection(_selection);
        }
    }, [msg.type]);
    var popoverChildren = useMemo(function () {
        return React.createElement("div", { className: css['child-wrapper'], onMouseUp: onMenuMouseUp },
            React.createElement("div", { style: msg.is_receive && msgRecvColor ? msgRecvColor : msgColor || {}, className: "".concat(css['msg-content'], " ").concat(msg.log_type === 'internal' ? css['msg-internal'] : '') }, (allowUnHide && msg.message_display_status === 'hide' && !msgdisplayStatus) ? React.createElement("span", null,
                "\u3010",
                t('label.theMsgIsHidden'),
                "\u3011") : handleMessageComponent()),
            msgType === 'email' &&
                React.createElement("div", { className: classNames(css['email-send'], msg.is_receive ? css['left'] : '') }, t('emailMethod.sendFromEmail')));
    }, [handleMessageComponent, msg.log_type, msg.translation, onMenuMouseUp, msgdisplayStatus]);
    var knowledgeId = useMemo(function () {
        var _a, _b, _c;
        var content_raw = msg.content_raw;
        if (!content_raw) {
            return null;
        }
        try {
            var contentRawJson = JSON.parse(content_raw);
            return (_c = (_b = (_a = contentRawJson === null || contentRawJson === void 0 ? void 0 : contentRawJson.robotAnswer) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.knowledgeResponse) === null || _c === void 0 ? void 0 : _c.knowledgeId;
        }
        catch (e) {
            return null;
        }
    }, [msg.content_raw]);
    // 选中聊天记录事件回调
    var imLogCheckChange = useCallback(function (e) {
        e.stopPropagation();
        if (!showCheckbox || !imLogCheckCb)
            return;
        imLogCheckCb(+msg.id, !isChecked);
    }, [imLogCheckCb, isChecked, showCheckbox]);
    var wrapperRef = useRef(null);
    return (React.createElement("div", { id: "im-log-".concat(msg.id), "data-message-id": msg.msg_id, className: classNames(showCheckbox ? css['pointer'] : '', (isChecked && showCheckbox) ? css['checked'] : ''), ref: wrapperRef, onClick: imLogCheckChange },
        React.createElement("div", { className: classNames(css['msg-item'], css['msg-' + msgType], msg.is_receive ? css['msg-recv'] : '', (chat.belong === 'mine' && msg.sensitive) ? css['msg-sensitive'] : '', showCheckbox ? css['pdl-16'] : '') },
            handleMsgPre(),
            isInfo &&
                React.createElement(React.Fragment, null,
                    showCheckbox &&
                        React.createElement(Checkbox, { className: css['im-log-checkbox'], checked: isChecked }),
                    React.createElement("div", { className: css['customerName'] }, getMsgHeader),
                    ((_b = msg === null || msg === void 0 ? void 0 : msg.data) === null || _b === void 0 ? void 0 : _b.voice_url) && React.createElement(MessageVideo, { msgItem: msg }),
                    React.createElement("div", { className: "".concat(css['msg-content-wrapper'], " ").concat(css['msg-content-wrapper-' + msgType], " ").concat('msg-content-wrapper-' + msgType, " ").concat(showCheckbox ? css['msg-content-wrapper-check'] : '') },
                        allowReedit ?
                            React.createElement("a", { onClick: reEdit, className: css['re-edit-box'] }, t('page.im.log.reEdit')) : null,
                        msg.message_display_status === 'hide' ? React.createElement("span", { className: "".concat(css['eye-invisible'], " ").concat(kcsSdk && (msg.message_display_status === 'hide') && msgdisplayStatus ? css['eye-invisible-kcs'] : '') },
                            React.createElement(EyeInvisibleOutlined, null)) : null,
                        function () {
                            if (!msg.is_receive && msg.type !== 'email') {
                                if (msg.send_status && ((msg.is_wechat_fail && msg.send_status === 'fail'))) {
                                    return (React.createElement("a", { onClick: onReSend, className: "".concat(css['send-status-box'], " ").concat(css['wechat-msg'], " ").concat(css[msg.send_status]) },
                                        React.createElement(Resend, null),
                                        React.createElement("span", { className: css['send-status-box-text'] }, send_status_msg)));
                                }
                                if (!web_message_display_status || msg.send_status === 'fail' || msg.send_status === 'off_sending' || (chat.platform !== 'web' && chat.platform !== 'api') || chat.belong === 'colleague' || msg.send_status === 'rollback') {
                                    return (React.createElement("a", { onClick: onReSend, className: "".concat(css['send-status-box'], " ").concat(msg.send_status ? css[msg.send_status] : '') }, msg.send_status === 'fail' ?
                                        React.createElement(Resend, null) :
                                        React.createElement(React.Fragment, null,
                                            handleStatusIcon(),
                                            React.createElement("span", { className: css['send-status-box-text'] }, send_status_msg))));
                                }
                            }
                        }(),
                        !msg.is_receive && web_message_display_status && msg.send_status !== 'fail' && (chat.platform === 'web' || chat.platform === 'api') && chat.belong !== 'colleague' && msg.type !== 'email' &&
                            React.createElement("span", { className: "".concat(css['send-status-box'], " ").concat(css['read-status'], " ").concat((!msg.read_status || (msg.send_status === 'off_sending')) ? "".concat(css['unread'], " unread") : "".concat(css['read'], " read")) }, (!msg.read_status || (msg.send_status === 'off_sending')) ? React.createElement(Unread, null) : React.createElement(Read, null)),
                        function () {
                            if (!msg.is_receive || msg.send_status !== 'rollback')
                                return null;
                            return (React.createElement("span", { className: "".concat(css['send-status-box'], " ").concat(css['rollback']), style: { margin: kcsSdk ? '1px 0 0 26px' : '1px 0 0 6px' } },
                                handleStatusIcon(),
                                React.createElement("span", { className: css['send-status-box-text'] }, t('page.im.log.rollback'))));
                        }(),
                        React.createElement(Menu, { allowAddToKM: allowAddToKM, msgdisplayStatus: msgdisplayStatus, hideEnable: hideEnable, refEnable: refEnable, msg: msg, chatPlatform: chat.platform, currentUser: currentUser, allowRecall: allowRecall, allowAddToCustomerInfo: allowAddToCustomerInfo, allowAddToRobotKM: allowAddToRobotKM, allowSearchImRecord: allowSearchImRecord, allowVoiceToText: allowVoiceToText, allowUnHide: allowUnHide, onAddToRobotKM: onAddToRobotKM, onVoiceToText: onVoiceToText, onSelectText: onSelectText, onAddToKM: onAddToKM, onHide: onHide, onRef: onRef, onCheck: onCheck, onUnHide: onUnHide, onRecall: onRecall, onSelectEmail: onSelectEmail, onSelectTel: onSelectTel, selection: selection }, popoverChildren)),
                    knowledgeId &&
                        React.createElement("div", { className: css['flex-end'] },
                            React.createElement(KnowledgeGraph, { knowledgeId: knowledgeId }))),
            msg.send_status === 'rollback' && timeText &&
                React.createElement("div", { className: css['time-bar'] }, DOMPurify.sanitize(timeText)),
            msg.send_status === 'rollback' && eventText &&
                React.createElement("div", { className: css['event-bar'] }, DOMPurify.sanitize(eventText)))));
};
export default ImLog;
export { axiosInstance } from './ajax';
