import { __assign, __rest } from "tslib";
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { useTranslation } from '@udesk-webapp/locale';
import { Button, Form, message, Select } from 'udesk-ui';
import { ajax, formatCustomField, PATH_BASE, replaceTemplateVar } from '@udesk-webapp/utils';
import CustomField from '@udesk-webapp/custom-field-container';
import isEmpty from 'lodash-es/isEmpty';
import moment from 'moment';
import useDefault from "react-use/esm/useDefault";
import TicketSignature, { initalTicketSignature, VerifySignatureAndSaveTicketEnums } from '@udesk-webapp/ticket-signature';
import { getSignatureConfig } from "@udesk-webapp/ticket-signature/es/utilities";
import Customer from "@udesk-webapp/ticket-system-field-customer";
import Receiver from "@udesk-webapp/ticket-system-field-receiver";
import Follower from "@udesk-webapp/ticket-system-field-follower";
import Merchant from "@udesk-webapp/ticket-system-field-merchant";
import SelectSearch from "@udesk-webapp/ticket-system-field-select-search";
import Tag from "@udesk-webapp/ticket-system-field-tag";
import SubjectContentAttachments from '@udesk-webapp/ticket-system-field-subject-content-attachments';
import style from './index.module.scss';
var templateSelectStyle = { width: 'auto', minWidth: '100px' };
var cancelButtonStyle = { marginRight: '10px' };
var initialTicketSignatureData = {
    isTicketSignatureModalVisible: false,
    ticketSignature: initalTicketSignature
};
var CreateTicket = React.forwardRef(function (_a, ref) {
    var setShowTicket = _a.setShowTicket, props = __rest(_a, ["setShowTicket"]);
    var currentUser = props.currentUser, chat = props.chat, ticketFieldsAbilities = props.ticketFieldsAbilities, company = props.company, roleCan = props.roleCan, ticketTab = props.ticketTab, admin = props.admin, event = props.event, customer = props.customer, onSaveSuccess = props.onSaveSuccess;
    var formRef = Form.useForm()[0];
    var refCacheSubmitTicket = useRef({});
    var _b = useDefault(initialTicketSignatureData, initialTicketSignatureData), ticketSignatureData = _b[0], setTicketSignatureData = _b[1];
    var _c = useState(false), saveLoading = _c[0], setSaveLoading = _c[1]; //保存工单按钮loading
    // 多语言
    var t = useTranslation()[0];
    //处理工单模板选项数据
    var ticketTemplates = useMemo(function () {
        var _a;
        return (_a = props.ticketTemplates) === null || _a === void 0 ? void 0 : _a.map(function (item) {
            return {
                value: item.id,
                label: item.name
            };
        });
    }, [props.ticketTemplates]);
    //是否已经成功提交表单
    var _d = useState(false), isSuccess = _d[0], setIsSuccess = _d[1];
    // 选择的模板id
    var _e = useDefault(ticketTemplates[0].value, ticketTemplates[0].value), template_id = _e[0], setTemplate_id = _e[1];
    // 自定义字段
    var _f = useState([]), customField = _f[0], setCustomField = _f[1];
    // 获取当前工单的缓存信息
    var getLocalStorage = useCallback(function () {
        var localStorageData = localStorage.getItem("".concat(currentUser.id, "-ticket-form-stash-at-email-im-message-").concat(chat === null || chat === void 0 ? void 0 : chat.im_sub_session_id));
        localStorageData && (localStorageData = JSON.parse(localStorageData));
        return localStorageData;
    }, [chat === null || chat === void 0 ? void 0 : chat.im_sub_session_id, currentUser.id]);
    var setLocalStorage = useCallback(function (data) {
        var stashKey = "".concat(currentUser.id, "-ticket-form-stash-at-email-im-message-").concat(chat === null || chat === void 0 ? void 0 : chat.im_sub_session_id);
        localStorage.setItem(stashKey, JSON.stringify(data));
    }, [currentUser.id, chat === null || chat === void 0 ? void 0 : chat.im_sub_session_id]);
    var noteCustomFields = useRef();
    // 处理表单初始信息
    var formData = useMemo(function () {
        //缓存的数据
        var localStorageData = getLocalStorage();
        return {
            subject: replaceTemplateVar(t('page.im.ticket.createTicketByIM'), currentUser.nick_name),
            content: localStorageData && !isEmpty(localStorageData.createTicketFormData) ? localStorageData.createTicketFormData.content : '',
            customer_id: customer,
            agent: [],
            follower_ids: [],
            status_id: 1,
            priority_id: 3,
            tags: [],
            attachment_ids: [],
            merchant_sid: {}
        };
    }, [getLocalStorage, customer, t, currentUser.nick_name]);
    //首次进入  查看是否有缓存的模板id  如果有  将模板id换成缓存的id
    useEffect(function () {
        var _willDeleteItems = localStorage.getItem('will-delete-items');
        if (_willDeleteItems) {
            var willDeleteItems = JSON.parse(_willDeleteItems);
            var newWillDeleteItems = {};
            for (var item in willDeleteItems) {
                if (localStorage.getItem(item)) {
                    if (moment(willDeleteItems[item]).isBefore(moment())) {
                        localStorage.removeItem(item);
                    }
                    else {
                        newWillDeleteItems[item] = willDeleteItems[item];
                    }
                }
            }
            localStorage.setItem('will-delete-items', JSON.stringify(newWillDeleteItems));
        }
        var localStorageData = getLocalStorage();
        if (localStorageData) {
            setTemplate_id(localStorageData.createTicketTemplateId);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    //整个表单是否滚动   为了解决内部的下拉选择框的滚动穿透  聚焦下拉选择时  将overScroll变成hidden
    var _g = useState(true), isScroll = _g[0], setIsScroll = _g[1];
    //模板改变时   将对应模板id的模板筛选出来  渲染新的自定义表单
    useEffect(function () {
        var _a;
        var localStorageData = getLocalStorage();
        var field = formatCustomField({
            fields: ((_a = props.ticketTemplates.find(function (item) { return item.id === template_id; })) === null || _a === void 0 ? void 0 : _a.fields) || [],
            trigger: 'onChange',
            fieldsAuthority: ticketFieldsAbilities[template_id],
            fieldType: 'ticket',
            editType: 'new',
            showLabel: true
        });
        //处理自定义字段的formData 处理好以后与系统字段合并  给formData重置初始值
        // 如果当前会话id有缓存的模板信息  直接取出  如果没有  进行以上处理
        var customFormData = {}; //自定义字段的formData
        if (localStorageData && !isEmpty(localStorageData.createTicketFormData)) {
            customFormData = localStorageData.createTicketFormData;
        }
        else {
            //循环用户自定义字段   将自定义字段对应的key 和value取出  作为自定义字段的formData
            field.forEach(function (item) {
                customFormData[item.field.id] = item.field.id.split('_')[0] === 'TextField' ? '' : [];
            });
        }
        // 将自定义的formData和系统字段的formdata合并   然后form重置初始值
        formRef.setFieldsValue(Object.assign(formData, customFormData));
        //保存自定义字段  渲染自定义字段的组件使用
        setCustomField(field);
        //离开新建工单页面时  缓存工单模板id和表单数据
        return function () {
            var data = {
                createTicketTemplateId: template_id,
                createTicketFormData: isSuccess ? {} : formRef.getFieldsValue()
            };
            var stashKey = "".concat(currentUser.id, "-ticket-form-stash-at-email-im-message-").concat(chat === null || chat === void 0 ? void 0 : chat.im_sub_session_id);
            setLocalStorage(data);
            //存时间
            if (!localStorage.getItem('will-delete-items')) {
                window.localStorage.setItem('will-delete-items', JSON.stringify({}));
            }
            var willDeleteItems = window.localStorage.getItem('will-delete-items');
            willDeleteItems = JSON.parse(willDeleteItems);
            willDeleteItems[stashKey] = moment().add(24, 'hours').toJSON();
            localStorage.setItem('will-delete-items', JSON.stringify(willDeleteItems));
        };
    }, [template_id, isSuccess, getLocalStorage, props.ticketTemplates, ticketFieldsAbilities, currentUser.id, chat === null || chat === void 0 ? void 0 : chat.im_sub_session_id, formRef, formData]);
    //点击公司  跳转到对应的公司详情
    var gotoTicket = useCallback(function (id) {
        // TODO:这里做了个兼容,后面用gotoTicket接口
        if (typeof props.gotoTicket === 'function') {
            props.gotoTicket(id);
        }
        else if (event) {
            event.transitionToRoute && event.transitionToRoute({
                route: 'ticket.show',
                path: id
            });
        }
    }, [event]);
    // 关闭工单
    var closeTicket = useCallback(function () {
        setShowTicket({
            show: false
        });
        // TODO:这里做了个兼容,后面用closeTicket接口
        if (typeof props.closeTicket === 'function') {
            props.closeTicket(chat);
        }
        else if (event) {
            event.updateChat && event.updateChat({
                chat: chat,
                updateProps: {
                    newTicketReact: {}
                }
            });
        }
    }, [setShowTicket, chat, event, props.gotoTicket, props.closeTicket]);
    /**
     * @description 提交工单前需进行工单签名校验，把发送数据提取到外部
     */
    var postTicket = function (saveStep, signatureParams) {
        /**
         * 从缓存中取出将要提交的工单数据
         */
        var data = refCacheSubmitTicket.current;
        if (saveStep === 'verifySignatureAndSaveTicket') {
            data = __assign(__assign({}, data), signatureParams);
        }
        setSaveLoading(true);
        ajax().postJSON("/cs/ts/api/tickets", data).then(function (res) {
            var _a;
            if (res.code === 1000) {
                setIsSuccess(true);
                var success = (
                // eslint-disable-next-line jsx-a11y/anchor-is-valid
                React.createElement("span", null,
                    "\u5DE5\u5355",
                    React.createElement("a", { onClick: function () { return gotoTicket(res.ticket.id); } }, res.ticket.field_num),
                    "\u521B\u5EFA\u6210\u529F"));
                message.success(success);
                //清空缓存工单数据并关闭签名modal
                refCacheSubmitTicket.current = {};
                handleCloseTicketSignatureModal();
                setTimeout(function () {
                    setSaveLoading(false);
                    closeTicket();
                }, 1500);
                onSaveSuccess === null || onSaveSuccess === void 0 ? void 0 : onSaveSuccess(res.ticket);
            }
            else {
                setSaveLoading(false);
                message.error((res === null || res === void 0 ? void 0 : res.code_message) || ((_a = res === null || res === void 0 ? void 0 : res.exception) === null || _a === void 0 ? void 0 : _a.message) || t('saveError'));
            }
        }).catch(function (e) {
            setSaveLoading(false);
            message.error(e);
        });
    };
    // 提交表单
    var handleSubmit = useCallback(function (value) {
        var params = {
            ticket: {
                assignee_id: currentUser.id,
                platform_name: 'im',
                template_id: template_id,
                im_sub_session_id: chat === null || chat === void 0 ? void 0 : chat.im_sub_session_id
            },
            tags: ''
        };
        // 如果选了受理人 拆成客服组id客服id
        if (value.agent.length) {
            params.ticket.agent_group_id = value.agent[0];
            params.ticket.assignee_id = value.agent[1] || null;
        }
        delete value.agent;
        // 处理自定义字段   将自定义字段从formData中筛选出来
        var custom_fields = {};
        customField.forEach(function (item) {
            var key = item.field.id;
            if (value[key] !== undefined) {
                custom_fields[key] = Array.isArray(value[key]) ? value[key].join(',') : value[key];
            }
            delete value[key];
        });
        params.ticket.custom_fields = custom_fields;
        //处理标签
        value.tags.length && (params.tags = value.tags.map(function (item) { return item.label; }).join(','));
        delete value.tags;
        //处理客户id
        value.customer_id = value.customer_id.value;
        //处理附件id
        value.attachment_ids = value.attachment_ids ? value.attachment_ids.map(function (item) { return item.id; }) : [];
        //处理商户
        if (company.category === 'mchat_primary')
            value.merchant_sid = value.merchant_sid.value;
        // 将剩下的字段和处理好的ticket合并
        params.ticket = Object.assign(params.ticket, value);
        setSaveLoading(true);
        getSignatureConfig({
            mode: VerifySignatureAndSaveTicketEnums.NeedSignature,
            examine_type: 'create',
            ticket: params.ticket,
        }).then(function (res) {
            /**
             * 缓存将要提交的params,在postTicket中使用
             */
            refCacheSubmitTicket.current = params;
            if (res.signature_required) {
                var signature_required = res.signature_required, signature_require_user_email = res.signature_require_user_email, ticket_signature = res.ticket_signature;
                var _a = ticket_signature || {}, category_enabled = _a.category_enabled, category_required = _a.category_required, description_enabled = _a.description_enabled, description_required = _a.description_required, category_names = _a.category_names, name_1 = _a.name, id = _a.id;
                setTicketSignatureData({
                    isTicketSignatureModalVisible: signature_required,
                    ticketSignature: {
                        id: id,
                        name: name_1,
                        category_content: category_names,
                        category_enabled: category_enabled,
                        category_required: category_required,
                        description_enabled: description_enabled,
                        description_required: description_required,
                        signature_require_user_email: signature_require_user_email,
                    }
                });
                setSaveLoading(false);
                return;
            }
            else {
                postTicket(VerifySignatureAndSaveTicketEnums.NeedSignature);
            }
        }, function (error) {
            var _a;
            setSaveLoading(false);
            message.error((error === null || error === void 0 ? void 0 : error.code_message) || ((_a = error === null || error === void 0 ? void 0 : error.exception) === null || _a === void 0 ? void 0 : _a.message) || t('saveError'));
        });
    }, [closeTicket, customField, gotoTicket, t, template_id, chat, company.category]);
    /**
     * 提交工单签名数据并校验
     */
    var handleVerifySignatureAndSaveTicket = function (saveStep, signatureParams) {
        postTicket(saveStep, signatureParams);
    };
    /**
     * 取消签名校验
     */
    var handleCloseTicketSignatureModal = function () {
        setTicketSignatureData(initialTicketSignatureData);
    };
    // 表单数据改变触发
    function handleChangeValue(changeValue, allValue) {
        // console.log(allValue)
    }
    // 改变模板  记录模板id  改变自定义字段
    var changeTemplate = useCallback(function (value) {
        setTemplate_id(value);
    }, []);
    useEffect(function () {
        var handleFields = function () {
            customField.forEach(function (i) {
                var _a;
                var _b;
                var defaults = ((_b = i === null || i === void 0 ? void 0 : i.options) === null || _b === void 0 ? void 0 : _b.filter(function (j) { return j.default; })) || [];
                if (!defaults.length) {
                    return;
                }
                formRef.setFieldsValue((_a = {}, _a[i.field.id] = defaults.map(function (j) { return j.value; }), _a));
            });
            var _loop_1 = function (item) {
                var _a;
                var field = customField.find(function (_item) { return _item.field.id === item; });
                if (field) {
                    formRef.setFieldsValue((_a = {}, _a[item] = noteCustomFields.current[item], _a));
                }
            };
            for (var item in noteCustomFields.current) {
                _loop_1(item);
            }
        };
        if (noteCustomFields.current) {
            handleFields();
        }
        else {
            ajax().getJSON("".concat(PATH_BASE, "/notes/detail"), {
                type: 'im',
                ref_id: chat === null || chat === void 0 ? void 0 : chat.im_sub_session_id,
                template_id: ''
            }).then(function (res) {
                noteCustomFields.current = res.data.custom_fields;
                handleFields();
            });
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [customField, formRef]);
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { className: style['ticket-form'], style: { overflowY: isScroll ? 'scroll' : 'hidden' } },
            React.createElement(Form, { form: formRef, onFinish: handleSubmit, initialValues: formData, layout: 'vertical' },
                React.createElement(SubjectContentAttachments, { roleCan: roleCan, ticketTab: ticketTab }),
                React.createElement(Form.Item, { label: t('page.im.ticket.customer'), name: 'customer_id' },
                    React.createElement(Customer, { setIsScroll: setIsScroll, ticketTab: ticketTab })),
                React.createElement(Form.Item, { label: t('page.im.ticket.receiver'), name: 'agent' },
                    React.createElement(Receiver, null)),
                React.createElement(Form.Item, { label: t('page.im.ticket.follower'), name: 'follower_ids' },
                    React.createElement(Follower, { admin: admin, currentUser: currentUser })),
                (company.category === 'mchat_primary') &&
                    React.createElement(Merchant, { setIsScroll: setIsScroll }),
                React.createElement(Form.Item, { label: t('page.im.ticket.status'), name: 'status_id' },
                    React.createElement(SelectSearch, { options: ticketTab.ticketStatus.map(function (item) {
                            return {
                                value: item.id,
                                title: item.name
                            };
                        }) })),
                React.createElement(Form.Item, { label: t('page.im.ticket.priority'), name: 'priority_id' },
                    React.createElement(SelectSearch, { options: ticketTab.ticketPriority.map(function (item) {
                            return {
                                value: item.id,
                                title: item.name
                            };
                        }) })),
                React.createElement(Form.Item, { label: t('page.im.ticket.tags'), name: 'tags' },
                    React.createElement(Tag, null)),
                React.createElement(Form.Item, { label: "" },
                    React.createElement("div", { className: style['select-template-wrapper'] },
                        React.createElement("div", { className: style['template-title'] }, t('page.im.ticket.ticketTemplate')),
                        React.createElement(Select, { showSearch: true, showArrow: true, optionFilterProp: "label", onChange: changeTemplate, style: templateSelectStyle, value: template_id, dropdownMatchSelectWidth: false, options: ticketTemplates }))),
                customField.length > 0 && React.createElement(CustomField, { fields: customField }))),
        React.createElement("div", { className: style['bottom-btn-list'] },
            React.createElement(Button, { style: cancelButtonStyle, onClick: closeTicket }, t('cancel')),
            React.createElement(Button, { type: "primary", loading: saveLoading, onClick: function () { return formRef.submit(); } }, t('save'))),
        React.createElement(TicketSignature, { confirmLoading: saveLoading, isVisible: ticketSignatureData.isTicketSignatureModalVisible, ticketSignature: ticketSignatureData.ticketSignature, onVerifySignatureAndSaveTicket: handleVerifySignatureAndSaveTicket, onClose: handleCloseTicketSignatureModal })));
});
export default CreateTicket;
