import { __assign, __rest, __spreadArray } from "tslib";
// @ts-nocheck
import React, { useContext, useMemo, useEffect, useCallback } from 'react';
import { Spin, message, Button, Space } from 'udesk-ui';
import { DownOutlined, UpOutlined } from '@ant-design/icons';
import { BetaSchemaForm } from '@ant-design/pro-form';
import { useTranslation } from '@udesk-webapp/locale';
import ExpandAndLanguage from './ant-local-plugin';
import { Upload, AttachmentListView, Content, 
// ContentView,
CopyText, FormCascader, FormCheckbox, FormDatePicker, FormLocation, FormMultipleSelect, FormNumber, FormRadio, FormRange, FormRangePicker, FormSelect, FormSelectInput, FormText, FormTextArea, FormLink, FormTimePicker, FormTreeSelect, PositionEnum, } from '@udesk-webapp/custom-form-components';
import './index.module.scss';
import ProProvider from '@ant-design/pro-provider';
import { time2Locale } from '@udesk-webapp/utils';
var LAYOUT_TYPE_HORIZONTAL = 'horizontal';
var LAYOUT_TYPE_VERTICAL = 'vertical';
var LAYOUT_TYPE_INLINE = 'inline';
var InputType;
(function (InputType) {
    InputType["Content"] = "content";
    InputType["FormCascader"] = "formCascader";
    InputType["FormCheckbox"] = "formCheckbox";
    InputType["FormDatePicker"] = "formDatePicker";
    InputType["FormLocation"] = "formLocation";
    InputType["FormMultipleSelect"] = "formMultipleSelect";
    InputType["FormNumber"] = "formNumber";
    InputType["FormRadio"] = "formRadio";
    InputType["FormRange"] = "formRange";
    InputType["FormRangePicker"] = "formRangePicker";
    InputType["FormSelect"] = "formSelect";
    InputType["FormSelectInput"] = "formSelectInput";
    InputType["FormText"] = "formText";
    InputType["FormLink"] = "formLink";
    InputType["FormTextArea"] = "formTextArea";
    InputType["FormTimePicker"] = "formTimePicker";
    InputType["FormTreeSelect"] = "formTreeSelect";
    InputType["Upload"] = "upload";
})(InputType || (InputType = {}));
var CustomForm = React.memo(function (props) {
    var columns = props.columns, _a = props.valueTypes, valueTypes = _a === void 0 ? {} : _a, _b = props.layoutType, layoutType = _b === void 0 ? 'Form' : _b, _c = props.formLayout, formLayout = _c === void 0 ? LAYOUT_TYPE_VERTICAL : _c, _d = props.hiddenResetButton, hiddenResetButton = _d === void 0 ? false : _d, _e = props.hiddenSubmitButton, hiddenSubmitButton = _e === void 0 ? false : _e, modalProps = props.modalProps, _f = props.submitBtnText, submitBtnText = _f === void 0 ? '' : _f, _g = props.cancelBtnText, cancelBtnText = _g === void 0 ? '' : _g, _h = props.language, language = _h === void 0 ? '' : _h, loading = props.loading, _j = props.isModalScroll, isModalScroll = _j === void 0 ? false : _j, submitBtnType = props.submitBtnType, zIndex = props.zIndex, _k = props.maskClosable, maskClosable = _k === void 0 ? false : _k, destroyOnClose = props.destroyOnClose, visible = props.visible, afterSubmitter = props.afterSubmitter, beforeSubmitter = props.beforeSubmitter, onCancel = props.onCancel, onValuesChange = props.onValuesChange, onValuesBlur = props.onValuesBlur, submitterRender = props.submitterRender, collapseRender = props.collapseRender, restProps = __rest(props, ["columns", "valueTypes", "layoutType", "formLayout", "hiddenResetButton", "hiddenSubmitButton", "modalProps", "submitBtnText", "cancelBtnText", "language", "loading", "isModalScroll", "submitBtnType", "zIndex", "maskClosable", "destroyOnClose", "visible", "afterSubmitter", "beforeSubmitter", "onCancel", "onValuesChange", "onValuesBlur", "submitterRender", "collapseRender"]);
    var values = useContext(ProProvider);
    var _l = useTranslation(), formatMessage = _l.t, i18n = _l.i18n; // 多语言
    useEffect(function () {
        // 尝试给antD pro 插入缺失的多语言
        ExpandAndLanguage(language);
        i18n.changeLanguage(language);
    }, [language]);
    var getFormItem = useCallback(function (text, props, type) {
        var itemDom = React.createElement("div", null);
        var _a = props.fieldProps, fieldProps = _a === void 0 ? {} : _a;
        var _b = fieldProps.copyProps, copyProps = _b === void 0 ? {} : _b;
        if (layoutType === 'LightFilter' || layoutType === 'QueryFilter') {
            copyProps.switchFlag = false;
        }
        switch (type) {
            case InputType.FormText:
                itemDom = React.createElement(FormText, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            case InputType.FormLink:
                itemDom = React.createElement(FormLink, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            case InputType.FormTextArea:
                itemDom = React.createElement(FormTextArea, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            case InputType.FormNumber:
                itemDom = React.createElement(FormNumber, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            case InputType.FormSelect:
                itemDom = React.createElement(FormSelect, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            case InputType.FormSelectInput:
                itemDom = React.createElement(FormSelectInput, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            case InputType.FormMultipleSelect:
                itemDom = React.createElement(FormMultipleSelect, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            case InputType.FormCascader:
                itemDom = React.createElement(FormCascader, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            case InputType.FormLocation:
                itemDom = React.createElement(FormLocation, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            case InputType.FormCheckbox:
                itemDom = React.createElement(FormCheckbox, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            case InputType.FormRadio:
                itemDom = React.createElement(FormRadio, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            case InputType.FormRange:
                itemDom = React.createElement(FormRange, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            case InputType.FormRangePicker:
                itemDom = React.createElement(FormRangePicker, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps, language: language }));
                break;
            case InputType.FormDatePicker:
                itemDom = React.createElement(FormDatePicker, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps, language: language }));
                break;
            case InputType.FormTimePicker:
                itemDom = React.createElement(FormTimePicker, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps, language: language }));
                break;
            case InputType.FormTreeSelect:
                itemDom = React.createElement(FormTreeSelect, __assign({ onValueBlur: onValuesBlur }, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps }));
                break;
            default:
                break;
        }
        return itemDom;
    }, [onValuesBlur]);
    var getItem = useCallback(function (text, props, type) {
        var _a, _b, _c;
        var _d = props.fieldProps, fieldProps = _d === void 0 ? {} : _d;
        var _e = fieldProps.copyProps, copyProps = _e === void 0 ? {} : _e;
        if (!copyProps.placement) {
            copyProps.placement = PositionEnum.TopLeft;
        }
        var itemDom = React.createElement("div", null);
        switch (type) {
            case InputType.FormSelect:
                itemDom = React.createElement(FormSelect, __assign({}, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps, readOnly: true }));
                break;
            case InputType.FormSelectInput:
                itemDom = React.createElement(FormSelectInput, __assign({}, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps, readOnly: true }));
                break;
            case InputType.FormMultipleSelect:
                itemDom = React.createElement(FormMultipleSelect, __assign({}, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps, readOnly: true }));
                break;
            case InputType.FormCascader:
                itemDom = React.createElement(FormCascader, __assign({}, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps, readOnly: true }));
                break;
            case InputType.FormTreeSelect:
                itemDom = React.createElement(FormTreeSelect, __assign({}, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps, readOnly: true }));
                break;
            case InputType.FormCheckbox:
                itemDom = React.createElement(FormCheckbox, __assign({}, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps, readOnly: true }));
                break;
            case InputType.FormRadio:
                itemDom = React.createElement(FormRadio, __assign({}, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps, readOnly: true }));
                break;
            case InputType.FormTextArea:
                /**
                 * readonlyRenderComponent
                 * 目前只有工单详情多行文本只读模式，依然渲染textArea,其他地方依然是 <span>{text}</span>
                 */
                if (((_a = props === null || props === void 0 ? void 0 : props.fieldProps) === null || _a === void 0 ? void 0 : _a.readonlyRenderComponent) && !!text) {
                    itemDom = (React.createElement(FormTextArea, __assign({}, props === null || props === void 0 ? void 0 : props.fieldProps, { copyProps: copyProps, readOnly: true })));
                }
                else {
                    itemDom = (React.createElement("span", null, text));
                }
                break;
            case InputType.FormDatePicker:
                var showText = text;
                if (!!text && ((_b = props === null || props === void 0 ? void 0 : props.fieldProps) === null || _b === void 0 ? void 0 : _b.needTransfer) && ((_c = props === null || props === void 0 ? void 0 : props.fieldProps) === null || _c === void 0 ? void 0 : _c.showTime)) {
                    showText = time2Locale(text);
                }
                itemDom = (React.createElement(CopyText, __assign({}, copyProps, { text: showText }),
                    React.createElement("span", null, showText)));
                break;
            default:
                itemDom = (React.createElement(CopyText, __assign({}, copyProps, { text: text }), [InputType.FormLink].indexOf(type) !== -1 && text ?
                    React.createElement("a", { href: text, target: "_blank" }, text)
                    : React.createElement("span", null, text)));
                break;
        }
        return itemDom;
    }, []);
    var onFinishFailed = function (_a) {
        var _b;
        var errorFields = _a.errorFields;
        message.error(((_b = errorFields[0]) === null || _b === void 0 ? void 0 : _b.errors[0]) || '');
    };
    var valueTypeMap = useMemo(function () {
        var _a;
        // 自定义的复杂组件需要单加
        var types = (_a = {},
            _a[InputType.Content] = {
                render: function (text, props) {
                    var fieldProps = (props === null || props === void 0 ? void 0 : props.fieldProps) || {};
                    var _a = fieldProps.copyProps, copyProps = _a === void 0 ? {} : _a;
                    if (!copyProps.placement) {
                        copyProps.placement = PositionEnum.TopLeft;
                    }
                    return React.createElement(Content, __assign({}, fieldProps, { copyProps: copyProps, readOnly: true }));
                    // return <ContentView {...fieldProps} copyProps={copyProps} content={text} />
                },
                renderFormItem: function (text, props) {
                    var fieldProps = (props === null || props === void 0 ? void 0 : props.fieldProps) || {};
                    var _a = fieldProps.copyProps, copyProps = _a === void 0 ? {} : _a;
                    return React.createElement(Content, __assign({}, fieldProps, { copyProps: copyProps }));
                },
            },
            _a[InputType.Upload] = {
                render: function (attachments, props) {
                    var fieldProps = (props === null || props === void 0 ? void 0 : props.fieldProps) || {};
                    return React.createElement(AttachmentListView, __assign({}, fieldProps, { attachments: attachments }));
                },
                renderFormItem: function (text, props) {
                    var fieldProps = (props === null || props === void 0 ? void 0 : props.fieldProps) || {};
                    return React.createElement(Upload, __assign({}, fieldProps));
                },
            },
            _a);
        // 通用的组件统一返回
        [
            InputType.FormCascader,
            InputType.FormCheckbox,
            InputType.FormDatePicker,
            InputType.FormLink,
            InputType.FormLocation,
            InputType.FormMultipleSelect,
            InputType.FormNumber,
            InputType.FormRadio,
            InputType.FormRange,
            InputType.FormRangePicker,
            InputType.FormSelect,
            InputType.FormSelectInput,
            InputType.FormText,
            InputType.FormTextArea,
            InputType.FormTimePicker,
            InputType.FormTreeSelect,
        ].forEach(function (inputType) {
            var itemConfig = {
                render: function (text, props) {
                    return getItem(text, props, inputType);
                },
                renderFormItem: function (text, props) {
                    return getFormItem(text, props, inputType);
                },
            };
            types[inputType] = itemConfig;
        });
        types = __assign(__assign({}, types), valueTypes);
        return types;
    }, [columns, valueTypes, getFormItem, getItem]);
    return (React.createElement(ProProvider.Provider, { value: __assign(__assign({}, values), { valueTypeMap: valueTypeMap }) },
        React.createElement(BetaSchemaForm, __assign({}, restProps, { grid: layoutType !== 'LightFilter' && layoutType !== 'QueryFilter', layout: formLayout, layoutType: layoutType, 
            // labelCol={{ span: 4 }} // horizontal 下使用
            // wrapperCol={{ span: 14 }} // horizontal 下使用
            visible: visible, language: language, modalProps: __assign({ bodyStyle: isModalScroll ? { overflowY: 'scroll', height: 600 } : {}, width: 1000, zIndex: zIndex, destroyOnClose: destroyOnClose, maskClosable: maskClosable, modalRender: function (modal) { return (React.createElement(React.Fragment, null,
                    modal,
                    loading && isModalScroll ? (React.createElement(Spin, { spinning: loading, wrapperClassName: "customForm-model-spin-wrapper" },
                        React.createElement("div", { className: "customForm-model-spin-content" }))) : null)); } }, (modalProps || {})), submitTimeout: 10000, submitter: {
                render: function (props, doms) {
                    if (typeof submitterRender === 'function') {
                        return submitterRender(props);
                    }
                    else if (layoutType === 'LightFilter' || layoutType === 'QueryFilter') {
                        return __spreadArray([], doms, true);
                    }
                    else {
                        var btnDoms = [];
                        beforeSubmitter && btnDoms.push(beforeSubmitter(props));
                        if (!hiddenSubmitButton) {
                            btnDoms.push(React.createElement(Button, { loading: loading, type: "primary", key: "submit", onClick: function () { var _a, _b; return (_b = (_a = props.form) === null || _a === void 0 ? void 0 : _a.submit) === null || _b === void 0 ? void 0 : _b.call(_a); } }, submitBtnText || formatMessage(submitBtnType == 'save' ? 'save' : 'submit')));
                        }
                        if (!hiddenResetButton) {
                            btnDoms.push(React.createElement(Button, { key: "rest", onClick: function () {
                                    var _a;
                                    if (layoutType !== "ModalForm") {
                                        (_a = props.form) === null || _a === void 0 ? void 0 : _a.resetFields();
                                    }
                                    ;
                                    onCancel === null || onCancel === void 0 ? void 0 : onCancel();
                                } }, cancelBtnText || formatMessage('cancel')));
                        }
                        afterSubmitter && btnDoms.push(afterSubmitter(props));
                        return (React.createElement("div", { className: 'custom-form-footerToolbar' },
                            React.createElement(Space, { wrap: true }, btnDoms)));
                    }
                },
            }, columns: columns, collapseRender: function (collapsed, doms) {
                if (typeof collapseRender === 'function') {
                    return collapseRender(collapsed, doms);
                }
                else {
                    return (collapsed ? React.createElement("span", { style: { color: '#00b38b' } },
                        formatMessage('labels.packDown'),
                        " ",
                        React.createElement(DownOutlined, null)) : React.createElement("span", { style: { color: '#00b38b' } },
                        formatMessage('labels.packUp'),
                        " ",
                        React.createElement(UpOutlined, null)));
                }
            }, onFinishFailed: onFinishFailed, onValuesChange: onValuesChange }))));
});
export default CustomForm;
