import { __assign, __awaiter, __generator } from "tslib";
import React, { useEffect, useState, useCallback, useMemo, useImperativeHandle } from 'react';
import { Spin } from 'udesk-ui';
import { getTicketsList as getTicketsListInner, getCustomFields } from './service';
import TicketTable from './associated-ticket-form';
import TicketFilter from './ticket-filters/index';
import TicketFilterCondition from './ticket-filters/ticket-filter-condition';
export { TicketFilter, TicketFilterCondition, TicketTable };
export * from './ticket-filters';
var SortEnums;
(function (SortEnums) {
    SortEnums["ASCEND"] = "ascend";
    SortEnums["DESCEND"] = "descend";
})(SortEnums || (SortEnums = {}));
var ASCEND_KEY = 'asc';
var DESCEND_KEY = 'desc';
var DEFAULT_TOTAL_COUNT = 0;
var DEFAULT_CURRENT_PAGE = 1;
var DEFAULT_PAGE_SIZE = 20;
var DEFAULT_PAGE_SIZE_OPTIONS = ['5', '10', '20', '30', '50', '100'];
var DEFAULT_COLUMNS = [
    {
        id: 'field_num',
        fieldMode: 'system',
    },
    {
        id: 'subject',
        fieldMode: 'system',
        width: '200px',
    },
    {
        id: 'content',
        fieldMode: 'system',
        width: '200px',
    },
    {
        id: 'status',
        fieldMode: 'system',
        width: '100px',
    },
    {
        id: 'created_at',
        fieldMode: 'system',
        width: '170px',
        // ellipsis:true
    },
    {
        id: 'platform_zh_name',
        fieldMode: 'system',
    },
];
var TicketDetail = React.forwardRef(function (props, ref) {
    //ember接收数据
    var onRowSelectionChange = props.onRowSelectionChange, //获取单选、多选数据
    isInitDatas = props.isInitDatas, //是否初始化
    disabledLists = props.disabledLists, //需要置灰的工单
    dataSources = props.dataSources, filterLists = props.filterLists, ticketCustomFields = props.ticketCustomFields, ticketSystemFields = props.ticketSystemFields, selectedRowKeys = props.selectedRowKeys, _a = props.scrollConfig, scrollConfig = _a === void 0 ? { y: 300 } : _a, _b = props.useCustomFieldRequset, useCustomFieldRequset = _b === void 0 ? false : _b, columns = props.columns, agents = props.agents, agentsWithAdmin = props.agentsWithAdmin, allConditions = props.allConditions, anyConditions = props.anyConditions, filterMaxNumber = props.filterMaxNumber, filterText = props.filterText, _c = props.filterField, filterField = _c === void 0 ? 'activeTicketFilters' : _c, _d = props.hideFilter, hideFilter = _d === void 0 ? false : _d, _e = props.showCondition, showCondition = _e === void 0 ? true : _e, onRow = props.onRow, //行点击事件
    getTicketsList = props.getTicketsList, filters = props.filters, getFilters = props.getFilters, filterClassName = props.filterClassName, removalIconClass = props.removalIconClass, addIconClass = props.addIconClass, addIconText = props.addIconText;
    // 设置默认值，通过解构props时赋值，null的情况不会设置成功默认值
    columns = columns || DEFAULT_COLUMNS;
    var _f = useState(true), isLoading = _f[0], setIsLoading = _f[1];
    // 工单列表
    var _g = useState([]), ticketLists = _g[0], setTicketLists = _g[1];
    // 自定义字段
    var _h = useState([]), customFields = _h[0], setCustomFields = _h[1];
    // 页码页数配置
    var _j = useState({
        current: DEFAULT_CURRENT_PAGE,
        defaultPageSize: DEFAULT_PAGE_SIZE,
        pageSizeOptions: DEFAULT_PAGE_SIZE_OPTIONS,
        total: DEFAULT_TOTAL_COUNT, //总条数
    }), pagination = _j[0], setPagination = _j[1];
    var _k = useState({
        all_conditions: [],
        any_conditions: [],
        column: '',
        filter_id: '',
        group_name: '',
        order: '',
        page: DEFAULT_CURRENT_PAGE,
        page_size: DEFAULT_PAGE_SIZE,
        query: '',
        sort_by: [],
    }), listsParams = _k[0], setListsParams = _k[1];
    // TODO：默认选中工单，selectedRowKeys留给外部接口使用
    var _l = useState(selectedRowKeys || []), innerSelectedRowKeys = _l[0], setInnerSelectedRowKeys = _l[1];
    // 获取工单列表
    var getTicketsListsRequest = function (params) { return __awaiter(void 0, void 0, void 0, function () {
        var tickets, meta, list, formatedTicketLists;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!(typeof getTicketsList === 'function')) return [3 /*break*/, 2];
                    return [4 /*yield*/, getTicketsList(params)];
                case 1:
                    list = _a.sent();
                    return [3 /*break*/, 4];
                case 2: return [4 /*yield*/, getTicketsListInner(params)];
                case 3:
                    list = _a.sent();
                    _a.label = 4;
                case 4:
                    tickets = (list === null || list === void 0 ? void 0 : list.tickets) || [];
                    meta = (list === null || list === void 0 ? void 0 : list.meta) || {};
                    formatedTicketLists = tickets.map(function (item_list) {
                        return __assign({}, item_list.ticket);
                    });
                    setTicketLists(formatedTicketLists);
                    setPagination(__assign(__assign({}, pagination), { total: meta === null || meta === void 0 ? void 0 : meta.total_records, current: meta === null || meta === void 0 ? void 0 : meta.current_page }));
                    setIsLoading(false);
                    return [2 /*return*/];
            }
        });
    }); };
    // 获取自定义字段
    var getCustomFieldsRequest = function () { return __awaiter(void 0, void 0, void 0, function () {
        var customFieldsResult, customFields_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!useCustomFieldRequset) return [3 /*break*/, 2];
                    return [4 /*yield*/, getCustomFields()];
                case 1:
                    customFields_1 = (_a.sent()).custom_fields;
                    customFieldsResult = customFields_1;
                    return [3 /*break*/, 3];
                case 2:
                    customFieldsResult = props.customFields;
                    _a.label = 3;
                case 3:
                    setCustomFields(customFieldsResult || []);
                    return [2 /*return*/];
            }
        });
    }); };
    // 排序、页码更换
    var handleTableChange = function (pagination, filters, sorter) {
        // pagination 页码和每页数量信息
        // filters 筛选
        // sorter 排序所需要的信息
        var sorterBy = [];
        var sorterOrder = '';
        if (sorter.order == SortEnums.ASCEND) {
            sorterOrder = ASCEND_KEY;
        }
        else if (sorter.order == SortEnums.DESCEND) {
            sorterOrder = DESCEND_KEY;
        }
        sorter.order ? sorterBy.push([sorter.columnKey, sorterOrder]) : sorterBy;
        setListsParams(__assign(__assign({}, listsParams), { page: pagination.current, page_size: pagination.pageSize, sort_by: sorterBy }));
    };
    // 单选/多选点击配置配置
    var rowSelection = useMemo(function () {
        var prevSelectedRowKeys = innerSelectedRowKeys || [];
        return {
            onChange: function (selectedRowKeys, selectedRows) {
                // 获取单选/多选的数据
                if (typeof onRowSelectionChange === 'function') {
                    onRowSelectionChange(selectedRowKeys, selectedRows);
                }
                // TODO：支持跨页选取
                setInnerSelectedRowKeys(selectedRowKeys);
            },
            getCheckboxProps: function (record) {
                return {
                    disabled: (disabledLists || []).indexOf(record.id) !== -1,
                };
            },
            selectedRowKeys: prevSelectedRowKeys,
        };
    }, [onRowSelectionChange, innerSelectedRowKeys, disabledLists]);
    // 获取自定义字段
    useEffect(function () {
        getCustomFieldsRequest();
    }, []);
    // 初始化查询
    useEffect(function () {
        if (isInitDatas) {
            setIsLoading(true);
            getTicketsListsRequest(listsParams);
        }
    }, []);
    // 选中工单变更
    useEffect(function () {
        setInnerSelectedRowKeys(selectedRowKeys);
    }, [selectedRowKeys]);
    // 列表查询条件变化时调用查询接口
    useEffect(function () {
        setIsLoading(true);
        // initData();
        getTicketsListsRequest(listsParams);
    }, [listsParams]);
    // 点击过滤器的查询按钮
    var onSearched = useCallback(function (params) {
        if (params === void 0) { params = {}; }
        setListsParams(__assign(__assign({}, listsParams), { all_conditions: params === null || params === void 0 ? void 0 : params.allConditions, any_conditions: params === null || params === void 0 ? void 0 : params.anyConditions, filter_id: params === null || params === void 0 ? void 0 : params.filterId }));
    }, [listsParams]);
    var onFilterChang = useCallback(function (_a) {
        var filterId = _a.filterId, allConditions = _a.allConditions, anyConditions = _a.anyConditions;
        setListsParams(__assign(__assign({}, listsParams), { filter_id: filterId, all_conditions: allConditions, any_conditions: anyConditions }));
        setInnerSelectedRowKeys([]);
        setPagination(__assign(__assign({}, pagination), { current: DEFAULT_CURRENT_PAGE }));
    }, [listsParams]);
    useImperativeHandle(ref, function () { return ({
        refresh: function () {
            setIsLoading(true);
            getTicketsListsRequest(listsParams);
        },
    }); });
    return (React.createElement(React.Fragment, null,
        React.createElement(Spin, { spinning: isLoading },
            hideFilter ? null : (React.createElement("div", { className: "ticket-filter-table-filters" },
                React.createElement(TicketFilter, { dataSources: dataSources, filterLists: filterLists, allConditions: allConditions, anyConditions: anyConditions, ticketCustomFields: ticketCustomFields, onSearch: onSearched, onFilterChang: onFilterChang, filterMaxNumber: filterMaxNumber, filterText: filterText, showCondition: showCondition, filterField: filterField, filters: filters, getFilters: getFilters, filterClassName: filterClassName, removalIconClass: removalIconClass, addIconClass: addIconClass, addIconText: addIconText }))),
            React.createElement(TicketTable, { lists: ticketLists, 
                //@ts-ignore
                columns: columns, 
                //@ts-ignore
                customizeLists: customFields, 
                //@ts-ignore
                scroll: scrollConfig, 
                //@ts-ignore
                onRow: onRow, rowSelection: rowSelection, onChange: handleTableChange, selectionType: 'checkbox', pagination: pagination, isSensitiveField: false, ticketSystemFields: ticketSystemFields, agents: agents, agentsWithAdmin: agentsWithAdmin }))));
});
export default TicketDetail;
