import { __assign, __rest, __spreadArray } from "tslib";
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { Select } from 'udesk-ui';
import { getFormatOptions } from './util';
import CopyComponent from '@udesk-webapp/copy-content';
import './index.module.scss';
/**
 * 经典级联组件
 */
var MultipleSelectField = React.memo(function (props) {
    var options = props.options, showSearch = props.showSearch, allowClear = props.allowClear, readOnly = props.readOnly, disabled = props.disabled, classNames = props.classNames, fullSelect = props.fullSelect, fieldNames = props.fieldNames, onChange = props.onChange, valueProps = props.value, placeholder = props.placeholder, switchFlag = props.switchFlag, restProps = __rest(props, ["options", "showSearch", "allowClear", "readOnly", "disabled", "classNames", "fullSelect", "fieldNames", "onChange", "value", "placeholder", "switchFlag"]);
    var _a = useState([]), valueList = _a[0], setValueList = _a[1];
    var multipleSelectRef = useRef(null);
    useEffect(function () {
        setValueList(function () {
            if (typeof valueProps === 'string') {
                return valueProps ? valueProps.split(',') : [];
            }
            return valueProps || [];
        });
    }, [valueProps]);
    var _b = useMemo(function () {
        var _a = fieldNames || {}, _b = _a.label, label = _b === void 0 ? 'label' : _b, _c = _a.value, value = _c === void 0 ? 'value' : _c, _d = _a.children, children = _d === void 0 ? 'children' : _d, _e = _a.is_hide, is_hide = _e === void 0 ? 'is_hide' : _e;
        return getFormatOptions(options, { label: label, value: value, children: children, is_hide: is_hide });
    }, [options, fieldNames]), formatOptions = _b[0], formatNoHideOptions = _b[1];
    var valueObj = useMemo(function () {
        return (valueList || []).reduce(function (prev, current, index) {
            var _options = !index ? formatOptions : (prev[index - 1] || {}).children || [];
            var keyOption = _options.find(function (item) { return current === item.value; });
            keyOption && prev.push(keyOption);
            return prev;
        }, []);
    }, [formatOptions, valueList]);
    var optionsList = useMemo(function () {
        var newList = [];
        if (valueObj && valueObj.length > 0) {
            var valueNoHideObj = valueObj.reduce(function (prev, current, index) {
                var _options = index === 0 ? formatNoHideOptions : prev[index - 1];
                var keyOption = _options.find(function (item) { return current.value === item.value; });
                if (keyOption && keyOption.children) {
                    prev.push(keyOption.children);
                }
                else if (index !== valueObj.length - 1) {
                    prev.push([]);
                }
                return prev;
            }, []);
            newList = __spreadArray([formatNoHideOptions], valueNoHideObj, true);
        }
        else {
            newList.push(formatNoHideOptions);
        }
        return newList;
    }, [formatNoHideOptions, valueObj]);
    var getReadOnlyEle = useMemo(function () {
        if (!valueObj.length) {
            return React.createElement("span", null, "- -");
        }
        return valueObj.map(function (item) {
            return (item === null || item === void 0 ? void 0 : item.label) || '- -';
        }).join(' / ');
    }, [valueObj]);
    var showValueText = useMemo(function () {
        var data = valueObj.map(function (item) { return item.label || '- -'; }).join(' / ');
        return data;
    }, [valueObj]);
    if (readOnly) {
        return (React.createElement(CopyComponent, { text: showValueText, switchFlag: switchFlag, noHidden: true, type: 'primary', top: '-24px' },
            React.createElement("div", { className: "custom-field-chained-droplist-old custom-field-multiple-select ".concat(classNames) }, getReadOnlyEle)));
    }
    var handleChange = useCallback(function (_value, _option, _index) {
        var _a;
        var valueAry = valueList.slice(0, _index);
        var valueObjAry = valueObj.slice(0, _index);
        valueAry.push(_value || '');
        valueObjAry.push(_option || {});
        if (((_a = _option === null || _option === void 0 ? void 0 : _option.children) === null || _a === void 0 ? void 0 : _a.length) && fullSelect) {
            valueAry.push('');
        }
        onChange(valueAry, valueObjAry);
    }, [onChange, valueList]);
    var filterOption = useCallback(function (input, option) {
        var _a, _b;
        return ((_b = (_a = option === null || option === void 0 ? void 0 : option.label) === null || _a === void 0 ? void 0 : _a.toLowerCase()) === null || _b === void 0 ? void 0 : _b.indexOf(input === null || input === void 0 ? void 0 : input.toLowerCase())) >= 0;
    }, []);
    return (
    // 代码迁移不影响老功能暂不改名称
    //custom-field-chained-droplist-old为固定类名,需要定义样式请用css modules
    // 默认样式是组件保持一行，如果想分行显示可控制class
    React.createElement(CopyComponent, { text: showValueText, switchFlag: switchFlag, noHidden: true, type: "primary", top: "-24px" },
        React.createElement("div", { className: "custom-field-chained-droplist-old custom-field-multiple-select ".concat(classNames), ref: multipleSelectRef }, optionsList.map(function (options, index) {
            return (React.createElement("div", { className: 'custom-field-multiple-select-item', style: {
                    marginBottom: '6px',
                    marginRight: optionsList.length > 1 && index !== optionsList.length - 1 ? '5px' : ''
                }, key: index },
                React.createElement(Select, __assign({}, restProps, { disabled: disabled, value: valueObj[index], placeholder: placeholder, allowClear: allowClear, showSearch: showSearch, options: options, dropdownMatchSelectWidth: false, filterOption: filterOption, onChange: function (value, options) { return handleChange(value, options, index); }, getPopupContainer: function () { return multipleSelectRef.current || document.body; } }))));
        }))));
});
MultipleSelectField.defaultProps = {
    classNames: '',
    disabled: false,
    readOnly: false,
    showSearch: true,
    allowClear: true,
    placeholder: '',
    fullSelect: false,
    options: [],
    value: [],
    onChange: function () { },
};
export default MultipleSelectField;
