import { __assign, __rest } from "tslib";
// @ts-nocheck
import React, { useCallback, useEffect, useState, useMemo, useRef } from 'react';
import { Cascader } from 'udesk-ui';
import { getFormatOptions } from './util';
import CopyComponent from '@udesk-webapp/copy-content';
// 外部不需要再处理隐藏选项，由内部统一处理，及不需要optionsMeta
// 统一逻辑：有隐藏选项返显时，value值可反显，但options依然需要隐藏
/**
 * 快速级联组件
 * ** 注意此处需要支持antd@4.17.0 **
 */
var CascaderField = React.memo(function (props) {
    var options = props.options, valueProps = props.value, fieldNames = props.fieldNames, fullSelect = props.fullSelect, readOnly = props.readOnly, disabled = props.disabled, placeholder = props.placeholder, classNames = props.classNames, switchFlag = props.switchFlag, _a = props.onChange, onChange = _a === void 0 ? function () { } : _a, restProps = __rest(props, ["options", "value", "fieldNames", "fullSelect", "readOnly", "disabled", "placeholder", "classNames", "switchFlag", "onChange"]);
    var _b = useState([]), valueList = _b[0], setValueList = _b[1];
    var changeValue = useRef([]);
    useEffect(function () {
        var newValue = valueProps;
        if (typeof valueProps === 'string') {
            newValue = valueProps ? valueProps.split(',') : [];
        }
        setValueList(newValue);
        changeValue.current = newValue;
    }, [valueProps]);
    var _c = useMemo(function () {
        var _a = fieldNames || {}, _b = _a.label, label = _b === void 0 ? 'label' : _b, _c = _a.value, value = _c === void 0 ? 'value' : _c, _d = _a.children, children = _d === void 0 ? 'children' : _d, _e = _a.is_hide, is_hide = _e === void 0 ? 'is_hide' : _e;
        return getFormatOptions(options, { label: label, value: value, children: children, is_hide: is_hide });
    }, [options, fieldNames]), formatOptions = _c[0], formatNoHideOptions = _c[1];
    var valueObj = useMemo(function () {
        return (valueList || []).reduce(function (initValues, currentValue, index) {
            var _options = !index ? formatOptions : (initValues[index - 1] || {}).children || [];
            var keyOption = _options.find(function (item) { return currentValue === item.value; }) || {};
            initValues.push(keyOption);
            return initValues;
        }, []);
    }, [formatOptions, valueList]);
    var valueLabel = useMemo(function () {
        var values = valueObj.map(function (item) { return item.label; }) || [];
        // null 兼容placeholder显示
        return values.length !== 0 ? values : null;
    }, [valueObj]);
    var showValueText = useMemo(function () {
        return valueObj.map(function (item) { return item.label; }).join(' / ');
    }, [valueObj]);
    if (readOnly) {
        return (React.createElement(CopyComponent, { text: showValueText, switchFlag: switchFlag, noHidden: true, type: 'primary', top: '-24px' },
            React.createElement(React.Fragment, null, showValueText)));
    }
    var handleChange = useCallback(function (_value, _option) {
        var _a;
        // FIXME: 虽然changeOnSelect：false 可以实现最末级必填的功能，但产品需要在此时未选到最末级时给提示
        // 目前的解决方案是 changeOnSelect:true，fullSelect: true 时验证是否需要提示
        // 判断已选择的最后一级是否有children，有则加空值触发onChange后的_intervalvalidate验证
        // 并利用dropdown事件防止多次触发onChange以及后续事件导致卡顿感
        var newValue = _value || [];
        if (fullSelect && newValue.length && _option.length && ((_a = _option[_option.length - 1]) === null || _a === void 0 ? void 0 : _a.children)) {
            newValue.push('');
        }
        changeValue.current = newValue;
        if (!newValue.length) {
            onChange([]);
        }
    }, []);
    var handleDropdownVisibleChange = useCallback(function (visible) {
        !visible && onChange(changeValue.current || []);
    }, [onChange]);
    var filter = useCallback(function (inputValue, path) {
        return path.some(function (option) { return option.label.toLowerCase().indexOf(inputValue.toLowerCase()) > -1; });
    }, []);
    return (React.createElement(CopyComponent, { text: showValueText, switchFlag: switchFlag, noHidden: true, type: "primary", top: "-24px" },
        React.createElement(Cascader, __assign({ getPopupContainer: function (triggerNode) { return triggerNode; } }, restProps, { disabled: disabled, value: valueLabel, placeholder: placeholder, options: formatNoHideOptions, onChange: handleChange, onDropdownVisibleChange: handleDropdownVisibleChange, showSearch: { filter: filter } }))));
});
CascaderField.defaultProps = {
    classNames: '',
    disabled: false,
    readOnly: false,
    fullSelect: false,
    changeOnSelect: true,
    placeholder: '',
    value: [],
    options: [],
    onChange: function () { },
};
export default CascaderField;
