import { __spreadArray } from "tslib";
import React, { useCallback, useContext, useEffect, useState } from 'react';
import { Select, Spin, Button, Input } from 'udesk-ui';
import { ajax, PATH_BASE } from '@udesk-webapp/utils';
import { PlusOutlined, SearchOutlined } from '@ant-design/icons';
import { TicketContext } from '@udesk-webapp/ticket-signature/es/utilities';
import SearchCustomer from '@udesk-webapp/ticket-system-field-search-customer';
import debounce from 'lodash-es/debounce';
import './index.module.scss';
var Customer = React.memo(function (props) {
    var valueProps = props.value, _a = props.addCustomer, addCustomer = _a === void 0 ? true : _a, onChange = props.onChange, setIsScroll = props.setIsScroll, ticketTab = props.ticketTab, placeholder = props.placeholder, _b = props.onCreateCustomer, onCreateCustomer = _b === void 0 ? function () { } : _b, isOpenCustomerListReact = props.isOpenCustomerListReact;
    var _c = useContext(TicketContext), changePageType = _c.changePageType, companyValue = _c.organization;
    var _d = useState({}), value = _d[0], setValue = _d[1]; // 选择的选项
    var _e = useState(false), fetching = _e[0], setFetching = _e[1]; // 无数据时展示
    var _f = useState([]), options = _f[0], setOptions = _f[1]; // 搜索到的数据
    var _g = useState(0), lastFetchId = _g[0], setLastFetchId = _g[1]; //
    var _h = useState(false), showSearchModal = _h[0], setShowSearchModal = _h[1];
    var _j = useState(1), page = _j[0], setPage = _j[1];
    var _k = useState(''), searchValue = _k[0], setSearchValue = _k[1];
    var _l = useState(0), totalPages = _l[0], setTotalPages = _l[1];
    var _m = useState(JSON.parse(JSON.stringify(ticketTab))), ticketTabData = _m[0], setTicketTabData = _m[1];
    useEffect(function () {
        setValue(valueProps);
    }, [valueProps]);
    useEffect(function () {
        setOptions([]);
        setPage(1);
    }, [companyValue]);
    // ant select onSearch 输入法时连续触发多次，
    // 防抖起基础作用，根本问题需要监听input组件onCompositonStart
    var beforeSearch = debounce(function (_searchValue) {
        setSearchValue(_searchValue);
        setOptions([]);
        setPage(1);
        fetchUser(_searchValue, '');
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, 300);
    // 搜索用户
    var fetchUser = function (_searchValue, _page) {
        setLastFetchId(function () { return lastFetchId + 1; });
        var fetchId = lastFetchId;
        var companyId = companyValue && companyValue.value ? companyValue.value : '';
        setFetching(true);
        ajax().postJSON("".concat(PATH_BASE, "/customers/contain_search"), {
            page: _page ? _page : 1,
            page_size: 20,
            prefix: _searchValue,
            organization_id: companyId,
        }).then(function (res) {
            if (_page) {
                setPage(page + 1);
            }
            setTotalPages(res.meta.total_pages);
            if (fetchId !== lastFetchId) {
                // for fetch callback order
                return;
            }
            var dataClone = res.customers.map(function (item) { return ({
                label: item.nick_name,
                value: item.id,
                organization: item.organizations,
                key: item.id
            }); });
            if (_page) {
                setOptions(__spreadArray(__spreadArray([], options, true), dataClone, true));
            }
            else {
                setOptions(dataClone);
            }
            setFetching(false);
        });
    };
    //
    var handleChange = useCallback(function (_value) {
        // 通过onchange获取的数据查询options中的数据，将option传输出去
        var val = _value === null || _value === void 0 ? void 0 : _value.value;
        var optionData = options === null || options === void 0 ? void 0 : options.find(function (item) {
            return (item === null || item === void 0 ? void 0 : item.value) === val;
        });
        setValue(_value);
        setFetching(false);
        if (typeof onChange === 'function') {
            onChange(optionData);
        }
    }, [onChange, options]);
    // 点击搜索按钮  打开搜索弹窗
    var searchUser = useCallback(function () {
        // 此处留有疑问？不应该是查询id么？名字匹配会有逻辑问题
        ticketTabData === null || ticketTabData === void 0 ? void 0 : ticketTabData.customerFilters.forEach(function (item) {
            if (item.all_conditions && item.all_conditions.length) {
                var itemIndex = item.all_conditions.findIndex(function (condition) { return condition.field_name === 'organization'; });
                if (itemIndex !== -1) {
                    if (companyValue && companyValue.label) {
                        item.all_conditions[itemIndex].value = companyValue.label;
                    }
                    else {
                        item.all_conditions.splice(itemIndex, 1);
                    }
                }
                else {
                    if (companyValue && companyValue.label) {
                        item.all_conditions.push({
                            'field_name': "organization",
                            'operation': "is",
                            'value': companyValue.label,
                        });
                    }
                }
            }
            else {
                if (companyValue && companyValue.label) {
                    item.all_conditions.push({
                        'field_name': "organization",
                        'operation': "is",
                        'value': companyValue.label,
                    });
                }
            }
        });
        setTicketTabData(ticketTabData);
        setShowSearchModal(true);
    }, [ticketTabData, companyValue]);
    // 关闭搜索弹窗
    var closeSearch = useCallback(function () {
        setShowSearchModal(false);
    }, []);
    // 处理滚动事假
    var handleScroll = useCallback(function (e) {
        e.stopPropagation();
        if (e.target.scrollTop === e.target.scrollHeight - e.target.clientHeight) {
            if (page < totalPages) {
                fetchUser(searchValue, page);
            }
        }
    }, [page, totalPages, fetchUser, searchValue]);
    // FIXME: 此逻辑不应当在此处理，应当暴露focus和blur在外统一处理
    // 聚焦时  将外部表单取消滚动  防止滚动穿透
    var handleFocus = function () {
        if (typeof setIsScroll === 'function') {
            setIsScroll(false);
        }
        if (options.length === 0) {
            fetchUser(searchValue, '');
        }
    };
    // 失焦时  将外部滚动复原
    var handleBlur = function () {
        if (typeof setIsScroll === 'function') {
            setIsScroll(true);
        }
    };
    var selectCustomer = useCallback(function (_customer) {
        var _a, _b;
        var item = {
            label: _customer.label,
            value: _customer.value,
            organization: ((_b = (_a = _customer === null || _customer === void 0 ? void 0 : _customer.info) === null || _a === void 0 ? void 0 : _a.organization_ids) === null || _b === void 0 ? void 0 : _b.map(function (i) { return i.id; })) || []
        };
        setValue(item);
        setFetching(false);
        if (typeof onChange === 'function') {
            onChange(item);
        }
    }, [onChange]);
    return (React.createElement(Input.Group, { compact: true, className: "customerField-wrapper" },
        React.createElement(Select, { allowClear: true, showSearch: true, showArrow: true, labelInValue: true, style: { width: addCustomer ? 'calc(100% - 80px)' : 'calc(100% - 40px)' }, value: value, options: options, placeholder: placeholder, notFoundContent: fetching ? React.createElement(Spin, { size: "small" }) : (React.createElement("div", { style: { display: 'flex', flexWrap: 'nowrap', padding: 8 } }, "\u6CA1\u6709\u53EF\u9009\u9879")), filterOption: false, onSearch: beforeSearch, onChange: handleChange, getPopupContainer: function (triggerNode) { return triggerNode; }, onPopupScroll: handleScroll, onFocus: handleFocus, onBlur: handleBlur }),
        React.createElement(Button, { className: "search-icon", icon: React.createElement(SearchOutlined, null), onClick: searchUser }),
        addCustomer && React.createElement(Button, { className: "input-after-icon", icon: React.createElement(PlusOutlined, null), onClick: function () {
                if (isOpenCustomerListReact) {
                    onCreateCustomer();
                }
                else {
                    changePageType('customer');
                }
            } }),
        showSearchModal ?
            React.createElement(SearchCustomer, { close: closeSearch, show: showSearchModal, selectCustomer: selectCustomer, ticketTab: ticketTabData }) : null));
});
Customer.defaultProps = {
    value: '',
    // companyValue: { value: '', label: '' },
    placeholder: '',
};
export default Customer;
