import { __rest } from "tslib";
import React, { useCallback, useContext, useEffect, useState } from 'react';
import axios from 'axios';
import css from './style.module.scss';
import InfiniteScroll from 'react-infinite-scroller';
import List from 'udesk-ui/es/list';
import { Spin } from 'udesk-ui';
import ArticleItem from './article-item';
import { RepositoryWidgetContext } from '../utilities';
import { useSafeSpaceId } from './km-util';
import message from 'udesk-ui/es/message';
var ArticleList = function (_a) {
    var companyId = _a.companyId, subdomain = _a.subdomain, email = _a.email, limitSpaceIds = _a.limitSpaceIds, enablePrivate = _a.enablePrivate, passport = _a.passport, spaceId = _a.spaceId, catalogId = _a.catalogId, query = _a.query, props = __rest(_a, ["companyId", "subdomain", "email", "limitSpaceIds", "enablePrivate", "passport", "spaceId", "catalogId", "query"]);
    var _b = useState([]), data = _b[0], setData = _b[1];
    var _c = useState(1), page = _c[0], setPage = _c[1];
    var _d = useState({ pageNum: 1, pageSize: 20, total: 0 }), paging = _d[0], setPaging = _d[1];
    var _e = useState(false), loading = _e[0], setLoading = _e[1];
    var kmApiBase = useContext(RepositoryWidgetContext).kmApiBase;
    var safeSpaceId = useSafeSpaceId(spaceId);
    useEffect(function () {
        if (!passport || !spaceId) {
            return;
        }
        var params = {
            companyId: companyId,
            subdomain: subdomain,
            email: email,
            limitSpaceIds: limitSpaceIds,
            enablePrivate: enablePrivate,
            passport: passport,
            pageNum: page,
            query: query,
        };
        if (query) {
            params.spaceIds = spaceId;
            axios.post("".concat(kmApiBase, "/spaces"), params).then(function (res) {
                if (res.data.code !== 200) {
                    message.error(res.data.message);
                    return;
                }
                setData(res.data.data);
                setPaging(res.data.paging);
            });
        }
        else {
            params.spaceId = safeSpaceId;
            if (catalogId !== -1) {
                params.catalogId = catalogId;
            }
            axios.get("".concat(kmApiBase, "/articles/list"), { params: params }).then(function (res) {
                if (res.data.code === 200) {
                    setData(res.data.data);
                    setPaging(res.data.paging);
                }
                else {
                    message.error(res.data.message);
                }
            });
        }
    }, [
        catalogId,
        passport,
        spaceId,
        companyId,
        subdomain,
        email,
        limitSpaceIds,
        enablePrivate,
        page,
        query,
        kmApiBase,
        safeSpaceId,
    ]);
    var handleInfiniteOnLoad = useCallback(function () {
        if (loading) {
            return;
        }
        setLoading(true);
        setPage(function (prevState) { return prevState + 1; });
    }, [loading]);
    return (React.createElement("div", { className: css['articleList'] },
        React.createElement(InfiniteScroll, { initialLoad: false, pageStart: 0, loadMore: handleInfiniteOnLoad, hasMore: paging.pageNum * paging.pageSize < paging.total, useWindow: false },
            React.createElement(List, { dataSource: data, size: 'small', renderItem: function (item) { return (React.createElement(ArticleItem, { data: item, companyId: companyId, email: email, enablePrivate: enablePrivate, limitSpaceIds: limitSpaceIds, passport: passport, subdomain: subdomain })); } }, loading && React.createElement(Spin, { className: "w-full text-center mt-30" })))));
};
export default ArticleList;
