import { __assign } from "tslib";
import React, { useCallback, useContext, useEffect, useMemo, useState } from 'react';
import { getJSON } from '../utils';
import Spin from 'udesk-ui/es/spin';
import Space from 'udesk-ui/es/space';
import { Select, message, Empty } from 'udesk-ui';
import axios from 'axios';
import CatalogSelect from './catalog-select';
import css from './style.module.scss';
import ArticleList from './article-list';
// import { ISpace, KmSetting } from '../types';
import { RepositoryWidgetContext } from '../utilities';
import { useTranslation } from '@udesk-webapp/locale';
import SearchInput from '../search-input';
var Option = Select.Option;
var Km = function (_a) {
    var _b;
    var private_space = _a.private_space, sync = _a.sync, companyId = _a.companyId, subdomain = _a.subdomain, email = _a.email, query = _a.query, onSearch = _a.onSearch;
    var _c = useState(''), token = _c[0], setToken = _c[1];
    var _d = useState(), spacesResponse = _d[0], setSpacesResponse = _d[1];
    var _e = useContext(RepositoryWidgetContext), activeCatalog = _e.activeCatalog, onCatalogChange = _e.onCatalogChange, activeSpace = _e.activeSpace, onSpaceChange = _e.onSpaceChange, kmApiBase = _e.kmApiBase;
    var t = useTranslation().t;
    var publicParams = useMemo(function () { return ({
        companyId: companyId,
        subdomain: subdomain,
        email: email,
        limitSpaceIds: private_space.join(','),
        enablePrivate: sync,
        passport: token,
    }); }, [companyId, email, private_space, subdomain, sync, token]);
    useEffect(function () {
        getJSON("/knowledge_manager_settings/token?scope=token&_=".concat(new Date().getTime())).then(function (res) {
            setToken(res.token);
        });
    }, []);
    var getSpaces = useCallback(function () {
        if (!publicParams.passport) {
            return Promise.reject();
        }
        return axios
            .get("".concat(kmApiBase, "/spaces"), {
            params: __assign(__assign({}, publicParams), { pageSize: 100 }),
        })
            .then(function (res) {
            if (res.data.code === 200) {
                return res;
            }
            message.error(res.data.message);
            return null;
        })
            .catch(function (err) {
            var _a, _b;
            message.error(((_b = (_a = err === null || err === void 0 ? void 0 : err.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message) || '请求失败');
            return null;
        });
    }, [kmApiBase, publicParams]);
    useEffect(function () {
        getSpaces().then(function (res) {
            var _a;
            if (((_a = res === null || res === void 0 ? void 0 : res.data) === null || _a === void 0 ? void 0 : _a.code) === 200) {
                setSpacesResponse(res === null || res === void 0 ? void 0 : res.data);
            }
        });
    }, [getSpaces]);
    var allSpaces = useMemo(function () { return spacesResponse && spacesResponse.data.map(function (i) { return i.id; }).join(','); }, [spacesResponse]);
    useEffect(function () {
        var _a;
        if (query) {
            onSpaceChange === null || onSpaceChange === void 0 ? void 0 : onSpaceChange(allSpaces);
        }
        else {
            if ((_a = spacesResponse === null || spacesResponse === void 0 ? void 0 : spacesResponse.data) === null || _a === void 0 ? void 0 : _a.length) {
                onSpaceChange === null || onSpaceChange === void 0 ? void 0 : onSpaceChange(spacesResponse.data[0].id);
            }
        }
    }, [allSpaces, onSpaceChange, query, spacesResponse === null || spacesResponse === void 0 ? void 0 : spacesResponse.data]);
    var safeSpace = useMemo(function () {
        if (!query &&
            activeSpace &&
            typeof activeSpace === 'string' &&
            activeSpace.indexOf(',') > -1) {
            return activeSpace.substring(0, activeSpace.indexOf(','));
        }
        return activeSpace;
    }, [activeSpace, query]);
    if (!token) {
        return React.createElement(Spin, { className: "w-full text-center mt-30" });
    }
    if (!((_b = spacesResponse === null || spacesResponse === void 0 ? void 0 : spacesResponse.data) === null || _b === void 0 ? void 0 : _b.length)) {
        return React.createElement(Empty, { className: "h-full w-hull flex justify-center items-center flex-col" });
    }
    return (React.createElement(React.Fragment, null,
        React.createElement(SearchInput, { className: 'mt-12', onSearch: onSearch, defaultValue: query }),
        React.createElement(Space, { className: css.spaceCatalogSelectWrapper },
            React.createElement("span", null,
                t('label.space'),
                ":"),
            React.createElement(Select, { value: safeSpace, onChange: onSpaceChange, bordered: false, dropdownMatchSelectWidth: false },
                query ? (React.createElement(Option, { value: allSpaces }, t('page.im.km.km.all_space'))) : null,
                spacesResponse.data.map(function (i) { return (React.createElement(Option, { key: i.id, value: i.id + '' }, i.name)); })),
            safeSpace && !query ? (React.createElement(CatalogSelect, __assign({}, publicParams, { spaceId: safeSpace, value: activeCatalog, onChange: onCatalogChange, setDefaultValue: onCatalogChange }))) : null),
        safeSpace && activeCatalog ? (React.createElement(ArticleList, __assign({}, publicParams, { spaceId: safeSpace, catalogId: activeCatalog, query: query }))) : null));
};
export default Km;
