import AddToCustomerMenuItem from './add-to-customer-menu-item';
import SearchImRecordMenuItem from './search-im-record-menu-item';
import VoiceToTextMenuItem from './voice-to-text-menu-item';
import AddMessageToKmRobotMenuItem from './add-message-to-km-robot-menu-item';
import RecallMenuItem from './recall-menu-item';
import HideItem from './hide-item';
import RefItem from './ref-item';
import CheckItem from './check-item';
import UnHideItem from './un-hide-item';
import AddToKnowledgeMenuItem from './add-to-knowledge-menu-item';
import css from './style.module.less';
import React, { useCallback, useMemo, useState } from 'react';
import { uniq } from 'lodash-es';
import Popover from 'udesk-ui/es/popover';
var telReg = /\b(((\+\d\d)?1\d{10})|((0\d{2,3}-?)\d{7,8}))\b/g;
var emailReg = /((\w)+(\.\w+)*@(\w)+((\.\w+)+))/g;
var getPopupContainer = function (triggerNode) { return triggerNode || document.body; };
var Menu = function (_a) {
    var allowAddToCustomerInfo = _a.allowAddToCustomerInfo, allowSearchImRecord = _a.allowSearchImRecord, allowVoiceToText = _a.allowVoiceToText, allowAddToRobotKM = _a.allowAddToRobotKM, allowUnHide = _a.allowUnHide, allowAddToKM = _a.allowAddToKM, msgdisplayStatus = _a.msgdisplayStatus, hideEnable = _a.hideEnable, refEnable = _a.refEnable, allowRecall = _a.allowRecall, msg = _a.msg, chatPlatform = _a.chatPlatform, onSelectEmail = _a.onSelectEmail, onSelectTel = _a.onSelectTel, onRecall = _a.onRecall, onHide = _a.onHide, onRef = _a.onRef, onCheck = _a.onCheck, onUnHide = _a.onUnHide, onAddToKM = _a.onAddToKM, onSelectText = _a.onSelectText, onVoiceToText = _a.onVoiceToText, onAddToRobotKM = _a.onAddToRobotKM, selection = _a.selection, currentUser = _a.currentUser, children = _a.children;
    var _b = useState(false), isAddToCustomerMenuItemExpand = _b[0], setAddToCustomerMenuItemExpand = _b[1];
    var tels = useMemo(function () { return uniq(JSON.stringify(msg.content).match(telReg)); }, [msg.content]);
    var emails = useMemo(function () { return uniq(JSON.stringify(msg.content).match(emailReg)); }, [msg.content]);
    var type = useMemo(function () { return msg.type; }, [msg.type]);
    var isText = useMemo(function () { return type === 'message' || type === 'info' || false; }, [type]);
    var isRich = useMemo(function () { return type === 'rich' || false; }, [type]);
    var isImage = useMemo(function () { return type === 'image' || false; }, [type]);
    var isFile = useMemo(function () { return type === 'file' || false; }, [type]);
    var isAudio = useMemo(function () { return type === 'audio'; }, [type]);
    var isVideo = useMemo(function () { return type === 'video'; }, [type]);
    var enabledAddToCustomerInfo = useMemo(function () { return allowAddToCustomerInfo && (tels.length + emails.length) > 0; }, [allowAddToCustomerInfo, emails.length, tels.length]);
    var enabledSearchImRecord = useMemo(function () { return allowSearchImRecord && !!selection; }, [allowSearchImRecord, selection]);
    var enabledVoiceToText = useMemo(function () { return allowVoiceToText &&
        ['ios', 'android'].includes(msg.platform || '') &&
        msg.type === 'audio' &&
        msg.is_receive &&
        !msg.recognition; }, [allowVoiceToText, msg.is_receive, msg.platform, msg.recognition, msg.type]);
    var enabledAddToRobotKM = useMemo(function () { return allowAddToRobotKM && !msg.is_receive && ['message', 'rich'].includes(msg.type); }, [allowAddToRobotKM, msg.is_receive, msg.type]);
    var enabledAddToKM = useMemo(function () { return allowAddToKM &&
        (isText || isRich || isImage || isFile) &&
        !msg.is_receive &&
        !msg.auto &&
        msg.agent_id !== 'robot'; }, [allowAddToKM, isFile, isImage, isRich, isText, msg.agent_id, msg.auto, msg.is_receive]);
    var enabledHide = useMemo(function () { return hideEnable && (isText || isRich); }, [isRich, isText, hideEnable]);
    var rpaSupportType = useMemo(function () { return isText || isImage || isFile || isAudio || isVideo; }, [isText, isImage, isFile, isAudio, isVideo]);
    // 文本类型或机器人消息，非系统消息非撤回消息
    var enabledRef = useMemo(function () {
        return refEnable &&
            !msg.auto &&
            msg.status !== 'system' &&
            msg.send_status !== 'rollback' &&
            ((chatPlatform === 'web' && isText) ||
                ((chatPlatform === 'single_rpa' || chatPlatform === 'group_rpa') && rpaSupportType));
    }, [isText, refEnable, msg.auto, chatPlatform, rpaSupportType]);
    var getMenuItems = useCallback(function () {
        var checkEnableRecall = function () {
            var _a;
            /*
                allowRecall: 允许撤回（im-log组件传入参数）
                msg.type === 'uploading' 正在图片或文件正在上传
                msg.send_status === 'rollback' 已经撤回的消息
                msg.is_receive 为true是客户发送的消息
                msg.status === 'system' 系统消息，例如“欢迎语”
                msg.auto 为true是自动消息，例如客服超时回复提醒
            */
            if (!allowRecall || msg.type === 'uploading' || msg.send_status === 'rollback' || msg.is_receive || msg.status === 'system' || msg.auto) {
                return false;
            }
            var now = new Date().getTime();
            var created_at = new Date(msg.created_at).getTime();
            return !((_a = msg.content) === null || _a === void 0 ? void 0 : _a.auto) && msg.sender !== 'customer' && (now - created_at < 2000 * 60) && ((currentUser === null || currentUser === void 0 ? void 0 : currentUser.id) === msg.agent_id);
        };
        return [
            (msg.message_display_status !== 'hide' && enabledAddToCustomerInfo ? React.createElement(AddToCustomerMenuItem, { tels: tels, emails: emails, isExpand: isAddToCustomerMenuItemExpand, onSelectEmail: onSelectEmail, onSelectTel: onSelectTel, key: 'add-to-customer-menu-item' }) : null),
            (msg.message_display_status !== 'hide' && enabledSearchImRecord ?
                React.createElement(SearchImRecordMenuItem, { keyword: selection || '', onClick: onSelectText, key: 'search-im-record-menu-item' }) : null),
            (msg.message_display_status !== 'hide' && enabledVoiceToText ? React.createElement(VoiceToTextMenuItem, { onClick: onVoiceToText, msg: {
                    msg_id: msg.msg_id,
                    data: msg.data
                }, key: 'voice-to-text' }) : null),
            (msg.message_display_status !== 'hide' && enabledAddToRobotKM ?
                React.createElement(AddMessageToKmRobotMenuItem, { onClick: onAddToRobotKM, key: 'add-message-to-km-robot-menu-item', msg: { data: msg.data, type: msg.type } }) : null),
            (msg.message_display_status !== 'hide' && checkEnableRecall() ?
                React.createElement(RecallMenuItem, { onClick: onRecall, key: 'recall-menu-item' }) : null),
            (allowUnHide && enabledHide && msg.message_display_status !== 'hide' ? React.createElement(HideItem, { onClick: onHide, key: 'hide-item' }) : null),
            (allowUnHide && enabledRef && msg.message_display_status !== 'hide' ? React.createElement(RefItem, { onClick: onRef, key: 'ref-item' }) : null),
            (allowUnHide && msg.message_display_status === 'hide' && !msgdisplayStatus ? React.createElement(CheckItem, { onClick: onCheck, key: 'check-item' }) : null),
            (allowUnHide && msg.message_display_status === 'hide' ? React.createElement(UnHideItem, { onClick: onUnHide, key: 'un-hide-item' }) : null),
            (msg.message_display_status !== 'hide' && enabledAddToKM ?
                React.createElement(AddToKnowledgeMenuItem, { content: msg.data.content, onClick: onAddToKM, key: 'add-to-knowledge-menu-item', msg: {
                        type: msg.type,
                        data: msg.data,
                        is_receive: msg.is_receive,
                        filename: msg.filename,
                        filesize: msg.filesize,
                        filetype: msg.filetype,
                        randomFileName: msg.randomFileName
                    } }) : null)
        ].filter(function (i) { return i; });
    }, [emails, enabledAddToCustomerInfo, enabledAddToKM, enabledAddToRobotKM, enabledSearchImRecord, enabledVoiceToText,
        isAddToCustomerMenuItemExpand, msg.data, msg.filename, msg.filesize, msg.filetype, msg.is_receive, msg.msg_id,
        msg.randomFileName, msg.type, onAddToKM, onAddToRobotKM, onRecall, onHide, onRef, onCheck, onUnHide, onSelectEmail,
        onSelectTel, onSelectText, onVoiceToText, selection, tels]);
    var onMouseEnter = useCallback(function () { return setAddToCustomerMenuItemExpand(true); }, []);
    var _c = useState(false), visible = _c[0], setVisible = _c[1];
    var onVisibleChange = useCallback(function (v) { return setVisible(v && getMenuItems().length > 0); }, [getMenuItems]);
    return React.createElement(Popover, { onVisibleChange: onVisibleChange, visible: visible, content: function () {
            return React.createElement("div", { onMouseEnter: onMouseEnter, className: css['menus'] }, getMenuItems());
        }, getPopupContainer: getPopupContainer, overlayClassName: css['im-log'], destroyTooltipOnHide: true, placement: msg.sender === 'customer' ? 'bottomLeft' : 'bottomRight' }, children);
};
export default Menu;
