import React from 'react';
export interface IMulitlevelPickerProps<T extends Record<string, string | number>> {
    /**
     * 标题
     */
    title: React.ReactNode;
    /**
     * 数据源
     */
    dataSource?: T[];
    /**
     * 初始值
     */
    initialValue?: Partial<T>[];
    /**
     * 值
     */
    value?: Partial<T>[];
    /**
     * 数据key
     */
    dataKey: keyof T;
    /**
     * 数据展示key
     */
    displayKey: keyof T;
    /**
     * 子节点 key
     */
    childrenKey?: keyof T;
    /**
     * 是否禁用
     */
    disabled?: boolean;
    /**
     * tag中显示删除按钮
     */
    allowClear?: boolean;
    /**
     * 当多选模式下值被选择，自动清空搜索框
     */
    autoClearSearchValue?: boolean;
    /**
     * 显示可全部选择的选项
     */
    showSelectAll?: boolean;
    /**
     * 多选
     */
    multiple?: boolean;
    /**
     * 父子节点选中状态不再关联
     */
    checkStrictly?: boolean;
    /**
     * 最大选择数量
     */
    maxSelectCount?: number;
    /**
     * 可搜索
     */
    showSearch?: boolean;
    /**
     * 搜索框的值，可以通过 `onSearch` 获取用户输入
     */
    searchValue?: string;
    /**
     * 动态加载数据
     */
    loadData?: (item: T) => Promise<T[]> | T[];
    /**
     * 选泽事件
     */
    onSelect?: (item: T) => void;
    /**
     * 选中事件 触发值改变
     */
    onChange?: (value: T) => void;
    /**
     * 提交事件
     */
    onSubmit?: (value: T[]) => void;
    /**
     * 重置事件
     */
    onReset?: () => void;
    /**
     * 清除按钮点击时触发事件
     */
    onClear?: (value: Partial<T>[], item: T) => boolean;
    /**
     * 查询事件
     */
    onSearch?: (searchText: string) => void;
    /**
     * 级别改变事件
     */
    onLevelChange?: (item: T) => void;
    dataSimpleMode?: false | {
        id: string;
        pId: string;
        rootPId: string;
    };
    virtual?: boolean;
    onScroll?: (index: any) => Promise<T[]>;
    /**
     * 设置统一样式前缀
     */
    prefixCls?: string;
    /**
     * 是否把每个选项的 label 包装到 value 中，会把 value 类型从 string 变为 {value: string, label: ReactNode, halfChecked(treeCheckStrictly 时有效): string[] } 的格式
     */
    labelInValue?: boolean;
}
declare const MulitlevelPicker: <T extends {}>({ title, dataKey, displayKey, childrenKey, dataSource, initialValue, value, showSelectAll, multiple, loadData, onLevelChange, dataSimpleMode, }: IMulitlevelPickerProps<T>) => JSX.Element;
export default MulitlevelPicker;
