import { __assign, __rest, __spreadArray } from "tslib";
import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { Select } from 'udesk-ui';
import CopyComponent from '@udesk-webapp/copy-content';
import ClassNames from 'classnames';
import { createCascadeOptionsByValue, getOptionByValue } from '../util';
/**
 * 级联组件
 * @param props
 * @param props.value    选中的值
 * @param props.options  option数据
 */
var OldCascader = React.memo(function (props) {
    var options = props.options, optionsMeta = props.optionsMeta, showSearch = props.showSearch, allowClear = props.allowClear, fullSelect = props.fullSelect, onChange = props.onChange, switchFlag = props.switchFlag, valueProps = props.value, placeholder = props.placeholder, restProps = __rest(props, ["options", "optionsMeta", "showSearch", "allowClear", "fullSelect", "onChange", "switchFlag", "value", "placeholder"]);
    var _a = useState([]), optionsAry = _a[0], setOptionsAry = _a[1];
    var _b = useState([]), valueList = _b[0], setValueList = _b[1];
    var _c = useState(options), currentOptions = _c[0], setCurrentOptions = _c[1];
    useEffect(function () {
        if ((valueProps && typeof valueProps === 'string') || (Array.isArray(valueProps) && valueProps.length)) {
            var valueOption = getOptionByValue(valueProps, options);
        }
        if (Array.isArray(optionsMeta) && optionsMeta.length) {
            setCurrentOptions(createCascadeOptionsByValue(valueProps, optionsMeta));
        }
    }, [options, optionsMeta, valueProps]);
    useEffect(function () {
        setValueList(function () {
            if (!Array.isArray(valueProps)) {
                return valueProps ? valueProps.split(',') : [];
            }
            else {
                return valueProps;
            }
        });
    }, [valueProps]);
    useEffect(function () {
        var list = getOptionsAry(valueList);
        setOptionsAry(list);
    }, [currentOptions, valueList]);
    var getOptionsAry = function (lists) {
        var list = lists.reduce(function (prev, current, index) {
            var opts;
            if (!index) {
                opts = currentOptions;
            }
            else {
                opts = (prev[index - 1] || {}).children || [];
            }
            var keyOption = opts.find(function (option) {
                return current === option.value;
            });
            keyOption && prev.push({
                value: keyOption.value,
                options: opts,
                children: keyOption.children || keyOption.subs || []
            });
            return prev;
        }, []);
        if (list && list.length > 0) {
            var last = list[list.length - 1];
            if (last.children && last.children.length) {
                list.push({
                    options: last.children
                });
            }
        }
        else {
            list.push({ options: currentOptions || [] });
        }
        return list;
    };
    var handleChange = useCallback(function (item, index, _value, option) {
        var valueAry = [];
        if (index === 0) {
            if (_value != null) {
                valueAry.push(_value);
            }
        }
        else {
            for (var i = 0; i < index; i++) {
                if (valueList[i] != null) {
                    valueAry.push(valueList[i]);
                }
            }
            if (_value != null) {
                valueAry.push(_value);
            }
        }
        setValueList(valueAry);
        var changeValueAry = __spreadArray([], valueAry, true);
        var list = getOptionsAry(valueAry);
        if (list.length !== valueAry.length && fullSelect) {
            changeValueAry.push('');
        }
        onChange === null || onChange === void 0 ? void 0 : onChange(changeValueAry);
    }, [onChange, valueList]);
    var filterOption = useCallback(function (input, option) {
        var _a, _b;
        return ((_b = (_a = option === null || option === void 0 ? void 0 : option.label) === null || _a === void 0 ? void 0 : _a.toLowerCase()) === null || _b === void 0 ? void 0 : _b.indexOf(input === null || input === void 0 ? void 0 : input.toLowerCase())) >= 0;
    }, []);
    var optionsElement = useMemo(function () {
        // copy 组件应当移到外层使用不应该在此
        var optionsLength = optionsAry.length;
        return optionsAry.map(function (item, index) {
            var _a, _b;
            return React.createElement("div", { style: {
                    marginBottom: '6px'
                }, key: index },
                React.createElement(CopyComponent, { text: (_b = (_a = item === null || item === void 0 ? void 0 : item.options) === null || _a === void 0 ? void 0 : _a.find(function (i) { return i.value === item.value; })) === null || _b === void 0 ? void 0 : _b.label, switchFlag: switchFlag },
                    React.createElement(Select, __assign({}, restProps, { onChange: function (value, options) { return handleChange(item, index, value, options); }, value: item.value, placeholder: placeholder, allowClear: allowClear, showSearch: showSearch, options: item === null || item === void 0 ? void 0 : item.options, filterOption: filterOption, dropdownMatchSelectWidth: false, getPopupContainer: function (triggerNode) { return triggerNode.parentElement; } }))));
        });
    }, [allowClear, filterOption, handleChange, optionsAry, props, showSearch, switchFlag]);
    //custom-field-chained-droplist-old为固定类名,需要定义样式请用css modules
    return React.createElement("div", { className: ClassNames('custom-field-chained-droplist-old') }, optionsElement);
});
OldCascader.defaultProps = {
    disabled: false,
    showSearch: true,
    allowClear: true,
    placeholder: '',
    fullSelect: false,
    options: [],
    value: [],
    onChange: function () { },
};
export default OldCascader;
