import { __assign, __spreadArray } from "tslib";
import Viewer from 'react-viewer';
import React, { useCallback, useContext, useEffect, useMemo, useRef, useState } from 'react';
import { Button, Popover } from 'udesk-ui';
import css from '../../style.module.scss';
import { DownOutlined, RightOutlined, LinkOutlined } from '@ant-design/icons';
import { useTranslation } from '@udesk-webapp/locale';
import classNames from 'classnames';
import createDOMPurify from 'dompurify';
import useEvent from 'react-use/esm/useEvent';
import Highlight from '../../hightlight';
import { hit, makeQuestionData, RepositoryWidgetContext } from '../../utilities';
import { canPreviewByFileName, getPreviewUrl } from '../../utils';
// import { IQuestion } from '../../types';
// import { selectedChatSelector } from '@Src/store/selectors'; // 需要传进来
// import EditQuestionModal from '../edit-question-answer';
// import { useSelector } from 'react-redux';
// import { ArrowRightSOutlined, ArrowDownSOutlined } from '@udesk/icons';
var DOMPurify = createDOMPurify(window);
DOMPurify.setConfig({ ADD_ATTR: ['target'] });
DOMPurify.addHook('afterSanitizeAttributes', function (node) {
    if ('target' in node) {
        node.setAttribute('rel', 'noopener noreferrer');
    }
});
var QuestionItem = function (_a) {
    var _b, _c;
    var _d;
    var data = _a.data, agentConfig = _a.agentConfig, getImages = _a.getImages, host = _a.host, chat = _a.chat;
    // const chat = useSelector(selectedChatSelector) || ({} as IMineChat); // 需要传进来
    var t = useTranslation().t;
    // const agentConfig = useSelector((state: RootState) => state.im.config); // 需要传进来
    var _e = useContext(RepositoryWidgetContext), allowSendQuestion = _e.allowSendQuestion, allowRelateQuestion = _e.allowRelateQuestion, onUse = _e.onUse, onSend = _e.onSend, onRelate = _e.onRelate, isWorkbench = _e.isWorkbench, showQuestionModal = _e.showQuestionModal, activeScope = _e.activeScope, previewUrl = _e.previewUrl;
    var _f = useState(false), showExpandButton = _f[0], setShowExpandButton = _f[1];
    var _g = useState(true), isExpand = _g[0], setIsExpand = _g[1];
    // const [showAddModal, setShowAddModal] = useState<boolean>(false);
    var contentRef = useRef(null);
    useEffect(function () {
        if (contentRef.current) {
            var imageElements = contentRef.current.querySelectorAll('img');
            var images_1 = [];
            imageElements.forEach(function (i) { return images_1.push({ src: i.src }); });
            getImages(images_1);
            if (contentRef.current.clientHeight > 40 || imageElements.length > 0) {
                setShowExpandButton(true);
                setIsExpand(false);
            }
        }
    }, [getImages]);
    var toggleExpand = useCallback(function () {
        setIsExpand(!isExpand);
    }, [isExpand]);
    var _onUse = useCallback(function () {
        var _a;
        hit(data.id, 'KnowledgeQuestion');
        if ((_a = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.knowledge_config) === null || _a === void 0 ? void 0 : _a.question_use_edit) {
            // setShowAddModal(true);
            showQuestionModal === null || showQuestionModal === void 0 ? void 0 : showQuestionModal(data, true);
        }
        else {
            onUse === null || onUse === void 0 ? void 0 : onUse('question', data);
        }
    }, [data, onUse, showQuestionModal, (_d = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.knowledge_config) === null || _d === void 0 ? void 0 : _d.question_use_edit]);
    var onSendClick = useCallback(function () {
        hit(data.id, 'KnowledgeQuestion');
        var url = "https://".concat(host || window.location.host, "/question/");
        var _data = makeQuestionData({
            content: data.content,
            content_type: data.content_type,
            title: data.title,
            id: parseInt(data.id),
        }, chat.platform, '', url);
        onSend === null || onSend === void 0 ? void 0 : onSend('question', __assign(__assign({}, data), { url: _data.url, desc: _data.desc }));
    }, [chat, data, onSend]);
    var onRelateClick = useCallback(function () {
        onRelate === null || onRelate === void 0 ? void 0 : onRelate({
            id: data.id,
            title: data.title,
            desc: data.desc,
        });
    }, [onRelate, data]);
    var previewUrlFun = useCallback(function (url) {
        window.open(getPreviewUrl(url, { previewUrl: previewUrl }));
    }, [previewUrl]);
    var AttachmentContent = useMemo(function () {
        var attachment = data.attachment;
        if (!attachment || (typeof attachment !== 'object'))
            return null;
        var _a = attachment || {}, url = _a.url, file_name = _a.file_name, content_type = _a.content_type;
        if (!url)
            return null;
        var canPreview = canPreviewByFileName(file_name);
        var nodeItem = [
            React.createElement("div", { className: css['attachment-left'], style: { maxWidth: canPreview ? undefined : 'calc(100% - 10px)' } },
                React.createElement(LinkOutlined, { key: '0' }),
                "\u00A0\u00A0",
                React.createElement("a", { href: url, title: file_name, target: '_blank', rel: "noreferrer noopener" }, file_name)),
            canPreview ? (React.createElement(Button, { type: 'link', size: 'small', onClick: function () { return previewUrlFun(url); } }, t('label.preview'))) : null
        ];
        if (content_type.startsWith('image') || /png|jpg|jpeg|gif/.test(content_type)) {
            return (React.createElement(Popover, { autoAdjustOverflow: false, content: React.createElement("img", { src: url, alt: file_name, style: { width: '100%' } }), placement: 'left', overlayStyle: { maxWidth: '300px', maxHeight: '300px' } },
                React.createElement("div", { className: css['attachment-wrap'] }, nodeItem)));
        }
        else {
            return React.createElement("div", { className: css['attachment-wrap'] }, nodeItem);
        }
    }, [data, previewUrlFun]);
    return (React.createElement("div", { className: css['question-item'] },
        React.createElement("div", { className: css['question-item-title'] },
            React.createElement("div", { className: classNames(css['align-justify']) },
                React.createElement(Highlight, { className: css['word-break'], content: data.title })),
            React.createElement("div", { style: { overflow: 'hidden' } },
                React.createElement("div", { className: classNames(css['btns']) },
                    allowSendQuestion && isWorkbench ? (React.createElement(React.Fragment, null,
                        React.createElement(Button, { type: 'link', className: css['question-item-btn'], size: 'small', onClick: onSendClick }, t('label.send')),
                        React.createElement("span", { className: css['line'] }))) : null,
                    isWorkbench ? React.createElement(Button, { type: 'link', className: css['question-item-btn'], size: 'small', onClick: _onUse }, t('label.use')) : null,
                    allowRelateQuestion && activeScope === 'public' ? (React.createElement(React.Fragment, null,
                        isWorkbench ? React.createElement("span", { className: css['line'] }) : null,
                        React.createElement(Button, { type: 'link', className: css['question-item-btn'], size: 'small', onClick: onRelateClick }, t('label.relate')))) : null,
                    showExpandButton && (React.createElement(React.Fragment, null,
                        isWorkbench ? React.createElement("span", { className: css['line'] }) : null,
                        React.createElement(Button, { type: 'link', className: css['question-item-btn'], size: 'small', onClick: toggleExpand },
                            t(isExpand ? 'label.collapse' : 'label.expand'),
                            isExpand ? (React.createElement(DownOutlined, { className: css['arrow-svg'] })) : (React.createElement(RightOutlined, { className: css['arrow-svg'] })))))))),
        React.createElement("div", { ref: contentRef, className: classNames(css.highlight, css['word-break'], (_b = {}, _b[css.expand] = isExpand, _b), (_c = {}, _c[css['un-expand']] = !isExpand, _c)), dangerouslySetInnerHTML: {
                __html: DOMPurify.sanitize(data.content_type === 'text' ? data.content_text : data.content),
            } }),
        AttachmentContent));
};
var QuestionList = function (_a) {
    var data = _a.data, onScrollToBottom = _a.onScrollToBottom, host = _a.host, agentConfig = _a.agentConfig, chat = _a.chat;
    var _b = useState(false), imageViewerVisible = _b[0], setImageViewerVisible = _b[1];
    var _c = useState([]), images = _c[0], setImages = _c[1];
    var _d = useState(0), activeIndex = _d[0], setActiveIndex = _d[1];
    var dataWrapperRef = useRef(null);
    var onClick = useCallback(function (e) {
        if (e.target.tagName === 'IMG') {
            setActiveIndex(images.findIndex(function (i) { return i.src === e.target.src; }));
            setImageViewerVisible(true);
        }
    }, [images]);
    useEvent('click', onClick, document.getElementById('react-question-common-list'));
    useEffect(function () {
        if (dataWrapperRef.current) {
            var scrollHandle_1 = function (e) {
                var element = e.currentTarget;
                if (!element) {
                    return;
                }
                // 因为屏幕的DPI问题，会导致部分电脑scrollTop为小数，无法加载新的
                if (element.scrollHeight - element.scrollTop <= element.clientHeight + 1) {
                    onScrollToBottom();
                }
            };
            dataWrapperRef.current.addEventListener('scroll', scrollHandle_1);
            return function () { var _a; return (_a = dataWrapperRef === null || dataWrapperRef === void 0 ? void 0 : dataWrapperRef.current) === null || _a === void 0 ? void 0 : _a.removeEventListener('scroll', scrollHandle_1); };
        }
    }, [onScrollToBottom]);
    var getImages = useCallback(function (i) { return setImages(function (prevState) { return __spreadArray(__spreadArray([], prevState, true), i, true); }); }, []);
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { ref: dataWrapperRef, className: "".concat(css['question-list'], " scrollbar"), id: 'react-question-common-list' }, data.map(function (i) { return (React.createElement(QuestionItem, { data: i, key: i.id, host: host, getImages: getImages, agentConfig: agentConfig, chat: chat })); })),
        React.createElement(Viewer, { activeIndex: activeIndex < 0 ? 0 : activeIndex, visible: imageViewerVisible, onClose: function () { return setImageViewerVisible(false); }, images: images })));
};
export default QuestionList;
