import React, { useCallback, useContext, useEffect, useMemo, useState, useRef } from 'react';
import ScopeSwitch from '../scope-switch';
import CommonQuestion from './common-question/index';
import RobotQuestionComp from './robot-question';
import { fetchKnowledgeManagerRobotSettings, RepositoryWidgetContext } from '../utilities';
import SearchInput from '../search-input';
import classNames from 'classnames';
import css from '../style.module.scss';
var Question = function (_a) {
    var _b;
    var onSearch = _a.onSearch, query = _a.query, refreshKnowledge = _a.refreshKnowledge, roleCan = _a.roleCan, agentConfig = _a.agentConfig, dialogueTagData = _a.dialogueTagData, mineChats = _a.mineChats, stashKey = _a.stashKey, dialogueTagNameList = _a.dialogueTagNameList, dialogueTagMode = _a.dialogueTagMode;
    var _c = useState(), settings = _c[0], setSettings = _c[1];
    var _d = useContext(RepositoryWidgetContext), activeScope = _d.activeScope, onScopeChange = _d.onScopeChange, showRobotQuestionTab = _d.showRobotQuestionTab, host = _d.host;
    var _e = useState(query), searchValue = _e[0], setSearchValue = _e[1];
    var oldQuery = useRef("");
    // const agentConfig = useSelector((state: RootState) => state.im.config); // 需要传进来
    // const dialogueTagData = useSelector<RootState, TagItem[]>( // 需要传进来
    //     (state) => state.im.config.dialogueTag.tag
    // );
    // const mineChats = useSelector<RootState, IMineChat[]>((state) => state.mineChats); // 需要传进来
    var chat = (mineChats === null || mineChats === void 0 ? void 0 : mineChats.find(function (chat) { return chat.selected; })) || {};
    var tagName = useMemo(function () {
        var _a, _b, _c, _d, _e;
        var tagList = [];
        if (((_a = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _a === void 0 ? void 0 : _a.enable) &&
            ((_b = agentConfig.session_tags_config) === null || _b === void 0 ? void 0 : _b.show_mode) === 'keyword' &&
            ((_c = chat.dialogueTag) === null || _c === void 0 ? void 0 : _c.length)) {
            if (((_d = agentConfig.session_tags_config) === null || _d === void 0 ? void 0 : _d.mode_tags) === 'first') {
                chat.dialogueTag[0].forEach(function (item) {
                    dialogueTagData === null || dialogueTagData === void 0 ? void 0 : dialogueTagData.forEach(function (items) {
                        if (item === items.id) {
                            tagList.push(items.name);
                        }
                    });
                }); //9/10/11/12/1/2
            }
            else if (((_e = agentConfig.session_tags_config) === null || _e === void 0 ? void 0 : _e.mode_tags) === 'new') {
                chat.dialogueTag[chat.dialogueTag.length - 1].forEach(function (item) {
                    dialogueTagData === null || dialogueTagData === void 0 ? void 0 : dialogueTagData.forEach(function (items) {
                        if (item === items.id) {
                            tagList.push(items.name);
                        }
                    });
                });
            }
            return tagList.join(' ');
        }
        else {
            return false;
        }
    }, [agentConfig, chat, dialogueTagData]);
    useEffect(function () {
        fetchKnowledgeManagerRobotSettings().then(function (settings) {
            setSettings(settings);
        });
    }, []);
    useEffect(function () {
        var value = (dialogueTagNameList === null || dialogueTagNameList === void 0 ? void 0 : dialogueTagNameList.length) && dialogueTagMode === "keyword"
            ? dialogueTagNameList === null || dialogueTagNameList === void 0 ? void 0 : dialogueTagNameList.join(" ")
            : query;
        if (oldQuery.current !== value) {
            setSearchValue(value);
            oldQuery.current = value;
        }
    }, [dialogueTagMode, dialogueTagNameList, query]);
    var onSearched = useCallback(function (keyword) {
        setSearchValue(keyword);
        onSearch === null || onSearch === void 0 ? void 0 : onSearch(keyword);
    }, [onSearch]);
    //系统开通了机器人，并且showRobotQuestionTab为 true 时才会显示机器人
    var enableRobot = useMemo(function () {
        var _a;
        return showRobotQuestionTab && ((_a = settings === null || settings === void 0 ? void 0 : settings.km_join_udesk_robots) === null || _a === void 0 ? void 0 : _a.is_enabled);
    }, [(_b = settings === null || settings === void 0 ? void 0 : settings.km_join_udesk_robots) === null || _b === void 0 ? void 0 : _b.is_enabled, showRobotQuestionTab]);
    var _onScopeChange = useCallback(function (scope) {
        if (!enableRobot && scope === 'robot') {
            onScopeChange === null || onScopeChange === void 0 ? void 0 : onScopeChange('public');
        }
        else {
            onScopeChange === null || onScopeChange === void 0 ? void 0 : onScopeChange(scope);
        }
    }, [enableRobot, onScopeChange]);
    var mainContent = useMemo(function () {
        if (activeScope === 'public' || activeScope === 'personal') {
            return (React.createElement(CommonQuestion, { scope: activeScope, query: searchValue, host: host, refreshKnowledge: refreshKnowledge, roleCan: roleCan, agentConfig: agentConfig, dialogueTagData: dialogueTagData, mineChats: mineChats, stashKey: stashKey }));
        }
        else if (enableRobot) {
            return (React.createElement(RobotQuestionComp, { searchWord: searchValue, settings: settings, agentConfig: agentConfig }));
        }
        else {
            return null;
        }
    }, [activeScope, enableRobot, searchValue, settings, roleCan, refreshKnowledge]);
    var hasPermission = useMemo(function () {
        if (activeScope === 'public') {
            return !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_view_public) && !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_questions_public);
        }
        else {
            return !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_view) && !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_questions);
        }
    }, [activeScope, roleCan]);
    return (React.createElement(React.Fragment, null,
        React.createElement(ScopeSwitch, { enableRobot: enableRobot, value: activeScope, onChange: _onScopeChange }),
        hasPermission ? (React.createElement(React.Fragment, null,
            React.createElement(SearchInput, { onSearch: onSearched, defaultValue: tagName ? tagName : searchValue }),
            mainContent)) : (React.createElement("div", { className: classNames(css['no-permission']) }, "\u60A8\u6682\u65E0\u6743\u9650\u67E5\u770B\u6B64\u5185\u5BB9"))));
};
export default Question;
