import { __assign, __rest } from "tslib";
import React, { useEffect, useState, useMemo, useRef } from 'react';
import RepositoryWidget from './repository-widget';
import { BookIcon, AngleLeftIcon, AngleRightIcon } from '@udesk-webapp/icon';
import css from './style.module.scss';
var REPOSITORY_WIDTH = 480;
;
var RepositoryWidgetDrawer = function (_a) {
    var _b = _a.top, top = _b === void 0 ? '116px' : _b, // fixed布局top值
    iconTop = _a.iconTop, // fixed布局top值
    persistenceKey = _a.persistenceKey, restProps = __rest(_a, ["top", "iconTop", "persistenceKey"]);
    var _c = useState(false), open = _c[0], setOpen = _c[1];
    var _d = useState(0), resizeWidth = _d[0], setResizeWidth = _d[1];
    var handleRef = useRef(null);
    var rightPanelRef = useRef(null);
    var resizeWidthRef = useRef(0);
    var iconTopStyle = useMemo(function () {
        if (iconTop) {
            return {
                top: iconTop
            };
        }
        return {};
    }, [iconTop]);
    var OpenBar = useMemo(function () {
        return (React.createElement("div", { className: css['repository-open-bar-wrap'], onClick: function () { return setOpen(true); }, style: iconTopStyle },
            React.createElement(BookIcon, null),
            React.createElement("br", null),
            React.createElement(AngleLeftIcon, null)));
    }, []);
    var CloseBar = useMemo(function () {
        return (React.createElement("div", { className: css['repository-close-bar-wrap'], style: iconTopStyle, onClick: function () { return setOpen(false); } },
            React.createElement(BookIcon, null),
            React.createElement("br", null),
            React.createElement(AngleRightIcon, null)));
    }, [iconTopStyle]);
    var Repository = useMemo(function () {
        if (!open)
            return null;
        return (React.createElement(React.Fragment, null,
            CloseBar,
            React.createElement(RepositoryWidget, __assign({}, restProps))));
    }, [
        restProps.companyId,
        restProps.email,
        restProps.kmApiBase,
        restProps.previewUrl,
        restProps.roleCan,
        restProps.subdomain,
        restProps.agent,
        restProps.customer,
        restProps.ticket,
        restProps.customerCustomFields,
        restProps.tabs,
        restProps.organizations,
        restProps.customerFieldAbilities,
        restProps.onUse,
        restProps.onSend,
        restProps.onRelate,
        restProps.categoryValue,
        CloseBar,
        open
    ]);
    useEffect(function () {
        if (open) { // 读缓存宽度
            if (persistenceKey) {
                var localCacheParams = localStorage.getItem(persistenceKey);
                var n = localCacheParams ? parseInt(localCacheParams) : 0;
                n = n || REPOSITORY_WIDTH;
                setResizeWidth(n);
                resizeWidthRef.current = n;
            }
            else {
                resizeWidthRef.current = REPOSITORY_WIDTH;
                setResizeWidth(REPOSITORY_WIDTH);
            }
        }
        else {
            setResizeWidth(0);
        }
    }, [open]);
    useEffect(function () {
        if (!open)
            return;
        var startX = 0;
        var isMove = false;
        var startRightPanelWidth = 0;
        var maxWidth = document.body.clientWidth * 0.8;
        var maskElement = document.createElement('div');
        maskElement.style.cssText =
            'position:fixed;top:0;left:0;right:0;bottom:0;z-index:99999;background:transparent;cursor:col-resize';
        var handleRefCurrent = handleRef.current;
        var mousemove = function (e) {
            if (isMove) {
                var rightWidth = void 0;
                var newRightWidth = startRightPanelWidth + (startX - e.clientX);
                if (newRightWidth >= startRightPanelWidth) {
                    rightWidth = Math.min(newRightWidth, maxWidth);
                }
                else {
                    rightWidth = Math.max(newRightWidth, REPOSITORY_WIDTH);
                }
                if (rightPanelRef === null || rightPanelRef === void 0 ? void 0 : rightPanelRef.current)
                    rightPanelRef.current.style.width = rightWidth + 'px';
                resizeWidthRef.current = rightWidth;
                e.preventDefault();
            }
        };
        var mouseup = function (e) {
            if (isMove) {
                isMove = false;
            }
            persistenceKey && localStorage.setItem(persistenceKey, "".concat(resizeWidthRef.current));
            _destroy();
        };
        var _destroy = function () {
            document.removeEventListener('mouseup', mouseup);
            document.removeEventListener('mousemove', mousemove);
            maskElement.remove();
        };
        var mousedown = function (e) {
            var _a, _b;
            startX = e.clientX;
            isMove = true;
            document.body.appendChild(maskElement);
            startRightPanelWidth = (_b = (_a = rightPanelRef === null || rightPanelRef === void 0 ? void 0 : rightPanelRef.current) === null || _a === void 0 ? void 0 : _a.offsetWidth) !== null && _b !== void 0 ? _b : 0;
            document.addEventListener('mousemove', mousemove);
            document.addEventListener('mouseup', mouseup);
        };
        handleRefCurrent === null || handleRefCurrent === void 0 ? void 0 : handleRefCurrent.addEventListener('mousedown', mousedown);
        return function () {
            handleRefCurrent === null || handleRefCurrent === void 0 ? void 0 : handleRefCurrent.removeEventListener('mousedown', mousedown);
            _destroy();
        };
    }, [open]);
    return (React.createElement("div", { style: { width: resizeWidth + 'px', top: top }, className: css['repository-widget-drawer-wrap'], ref: rightPanelRef }, !open ? OpenBar : (React.createElement(React.Fragment, null,
        React.createElement("div", { className: css['resizable-handler'], ref: handleRef }),
        Repository))));
};
export default RepositoryWidgetDrawer;
