import { getJSON, postJSON } from './ajax';
import extend from 'lodash-es/extend';
import isObject from 'lodash-es/isObject';
import clone from 'lodash-es/clone';
import qs from 'qs';
import memoize from 'lodash-es/memoize';
var cacheData = {};
var lastGetConfigDate;
var common = {
    get: getJSON,
    post: postJSON
};
var _getJSON = function (_a) {
    var url = _a.url, _b = _a.method, method = _b === void 0 ? 'get' : _b, _c = _a.data, data = _c === void 0 ? {} : _c, cache = _a.cache, _d = _a.cacheKey, cacheKey = _d === void 0 ? '' : _d, _e = _a.successCode, successCode = _e === void 0 ? 1000 : _e, mapDefaultData = _a.mapDefaultData, needSolutionEnabled = _a.needSolutionEnabled;
    cacheKey = "".concat(url, "/").concat(cacheKey);
    if (cache && cacheData[cacheKey]) {
        return new Promise(function (resolve, reject) {
            resolve(cacheData[cacheKey]);
        });
    }
    var isSPA = url.indexOf('km.') < 0;
    if (isSPA) {
        return common[method](url, data).then(function (res) {
            if (res.code === successCode) {
                cacheData[cacheKey] = cache ? res : null;
            }
            return res;
        });
    }
    else {
        return getConfig().then(function (res) {
            var sa = res.smart_assistance || {};
            var defaultData = {};
            if (!(data === null || data === void 0 ? void 0 : data.robotId)) {
                data === null || data === void 0 ? true : delete data.robotId;
                data === null || data === void 0 ? true : delete data.channelId;
                data === null || data === void 0 ? true : delete data.channelIdList;
                if (mapDefaultData) {
                    defaultData = mapDefaultData(sa) || {};
                }
                else {
                    defaultData = {
                        robotId: sa.udesk_robot_id,
                        channelIdList: sa.scene_id ? [sa.scene_id] : [] //默认场景
                    };
                }
            }
            url = url + '?' + qs.stringify(res.auth);
            // url = url + $.param(data)
            if (method === 'get') {
                Object.keys(data).forEach(function (item) {
                    url = url + "&".concat(item, "=").concat(data[item]);
                });
            }
            data = extend(defaultData, data);
            var fetchSource = 1;
            if (sa.solution_enabled) {
                fetchSource = 2;
            }
            if (sa.solution_enabled && sa.udesk_robot_id && sa.scene_id) {
                fetchSource = 3;
            }
            if (needSolutionEnabled) {
                data['fetchSource'] = fetchSource;
            }
            return common[method](url, method === 'get' ? {} : JSON.stringify(data), {
                headers: {
                    'Content-Type': 'application/json',
                }
            }).then(function (res) {
                if (res.code === successCode) {
                    cacheData[cacheKey] = cache ? res : null;
                }
                return res;
            });
        });
    }
};
/**
 * 获取机器人配置
 */
export var getConfig = memoize(function () { return _getJSON({ url: '/knowledge_manager_settings/robot_settings' }); }, function () { return "".concat(new Date().getDate()).concat(Math.floor(new Date().getHours() / 3)); });
/**
 * 设置机器人配置
 */
export function setConfig(key, value) {
    return getConfig().then(function (res) {
        extend(res[key], value);
        return res;
    });
}
/**
 * 通过类型，判断配置开关
 * @param {string} type -
 */
export function ifEnabled(type) {
    var enableKey = 'is_enabled';
    return getConfig().then(function (cfg) {
        cfg = cfg || {};
        var enabled = cfg.is_udesk_robot_open;
        if (enabled) {
            switch (type) {
                //  case 'smart_assistance':
                //      enabled = cfg[type][enableKey];
                //      break;
                case 'smart_assistance_auto_suggestion':
                    enabled = cfg.is_auto_suggest_open && cfg[type][enableKey];
                    break;
                //  case 'km_join_udesk_robots':
                //      enabled = cfg[type][enableKey];
                //      break;
                default:
                    if (isObject(cfg[type])) {
                        enabled = cfg[type][enableKey];
                    }
                    break;
            }
        }
        return {
            enabled: enabled
        };
    });
}
/**
 * 获取机器人列表
 * @param {Array} [cfg.visibleIds] - 需要显示机器人ids
 */
export function getList(cfg, cache) {
    if (cfg === void 0) { cfg = {}; }
    if (cache === void 0) { cache = true; }
    return _getJSON({
        url: '/cs/im/agent/udesk_robots',
        data: {},
        cache: cache
    }).then(function (res) {
        if (cfg.visibleIds) {
            res = clone(res);
            // res.data = filter(res.udesk_robots, item => _.contains(cfg.visibleIds, item.km_robot_id));
        }
        return res;
    });
}
/**
 * 获取机器人分类列表
 * @param {number} cfg.robotId - 机器人id
 */
export function getCategoryList(cfg, cache) {
    if (cfg === void 0) { cfg = { robotBaseURL: '' }; }
    if (cache === void 0) { cache = true; }
    return _getJSON({
        url: cfg.robotBaseURL + '/v1/categories',
        data: cfg,
        cache: cache,
        cacheKey: "".concat(cfg.robotId),
        successCode: 200
    });
}
/**
 * 获取机器人问题列表
 * @param {number} cfg.robotId - 机器人id
 * @param {number[]} cfg.channelIdList - 场景id列表
 * @param {number} [cfg.categoryId] - 分类id
 * @param {string} [cfg.searchWord] - 关键字
 * @param {number} cfg.[pageNum=1]
 * @param {number} cfg.[pageSize=20]
 */
export function getQuestionList(cfg) {
    return _getJSON({
        method: 'post',
        url: cfg.robotBaseURL + '/v1/questions/search',
        data: extend({
            pageNum: 1,
            pageSize: 20
        }, cfg),
        successCode: 200
    });
}
/**
 * 获取机器人推荐问题列表
 * @param {number} cfg.robotId - 机器人id
 * @param {number} cfg.channelId - 场景id
 * @param {string} cfg.queryContent - 关键字
 */
export function getRecommendQuestionList(cfg, cacheCfg) {
    if (cacheCfg === void 0) { cacheCfg = { robotBaseURL: '' }; }
    return _getJSON({
        method: 'post',
        url: cacheCfg.robotBaseURL.replace('/api', '') + '/coach/assist/messages',
        data: cfg,
        cache: cacheCfg.enabled,
        cacheKey: cacheCfg.key,
        successCode: 200,
        needSolutionEnabled: true,
        mapDefaultData: function (config) {
            return {
                robotId: config.udesk_robot_id,
                channelId: config.scene_id //默认场景
            };
        }
    });
}
/**
 * 把im工作台消息存进对应的km机器人知识库
 * @param {number} [robotId] - 机器人id
 * @param {number} [categoryId] 分类id 默认 未分类
 * @param {string} [content] - 问题标题
 * @param {string} [type] [问题类型] 默认选1
 * @param {Object} [answer] - 答案
 * @param {stirng} [answer.content]  答案内容
 * @param {number} [answer.type] 答案类型  1: text   2: rich
 * @param {number[]} channelIdList - 场景id列表
 * @param {number} [suggestType] - 推荐类型 默认1
 */
export function saveQuestionToKm(cfg, cacheCfg) {
    if (cacheCfg === void 0) { cacheCfg = { robotBaseURL: '' }; }
    return _getJSON({
        method: 'post',
        url: cfg.robotBaseURL + '/v1/questions',
        data: cfg,
        cache: cacheCfg.enabled,
        cacheKey: cacheCfg.key,
        successCode: 200
    });
}
/**
 * 获取会话渠道robot配置，渠道：网页插件/微信/sdk/小程序
 * @returns
 * @param {number} id
 * @param {string} name
 * @param {number} km_robot_id
 * @param {number} scene_id
 */
export function getChannelRobot(im_sub_session_id) {
    return _getJSON({
        url: '/channel_udesk_robot',
        data: {
            im_sub_session_id: im_sub_session_id
        },
        cache: true,
        cacheKey: im_sub_session_id
    });
}
