import nodeComponentSettings from './node-components-setting';
import normalNodeConfig from './normal-node-config';
var NORMAL_FIELDS_TABLE_VALIDATOR_INDEX = 0;
var NORMAL_FIELDS_TABLE_CONVERTER_INDEX = 1;
var NORMAL_FIELDS_TABLE_START_INDEX = Math.max(NORMAL_FIELDS_TABLE_VALIDATOR_INDEX, NORMAL_FIELDS_TABLE_CONVERTER_INDEX) + 1;
var NORMAL_FIELDS_AVOID_EVALUATION_FIELDS = ['dataSource'];
var VALUE_SETTINGS_TABLE_VALIDATOR_INDEX = 0;
var VALUE_SETTINGS_TABLE_CONVERTER_INDEX = 1;
var VALUE_SETTINGS_TABLE_START_INDEX = Math.max(VALUE_SETTINGS_TABLE_VALIDATOR_INDEX, VALUE_SETTINGS_TABLE_CONVERTER_INDEX) + 1;
var VALUE_SETTINGS_AVOID_EVALUATION_FIELDS = ['component'];
export function getNodeComponentSettings(type, category, locale) {
  var nodeComponentSettingsByType = nodeComponentSettings[type];
  var nodeComponentSetting = nodeComponentSettingsByType[category];
  return buildComponentSettings(nodeComponentSetting(locale));
}
export function getNodeFieldsConfig(type, category, blackList, extraList, locale) {
  var nodeFields = normalNodeConfig[type];
  var nodeField = nodeFields[category];
  /* eslint-disable no-console */
  return buildNormalFields(nodeField(blackList, extraList, locale));
}
function buildNormalFields(configTable) {
  var fields = [];
  if (configTable.length > 0) {
    var validators = configTable[NORMAL_FIELDS_TABLE_VALIDATOR_INDEX];
    var converters = configTable[NORMAL_FIELDS_TABLE_CONVERTER_INDEX];
    for (var i = NORMAL_FIELDS_TABLE_START_INDEX; i < configTable.length; i++) {
      var config = configTable[i];
      for (var fieldName in validators) {
        if (Object.prototype.hasOwnProperty.call(validators, fieldName)) {
          var validator = validators[fieldName];
          var value = config[fieldName];
          if (typeof validator === 'function') {
            var result = validator.call(config, value);
            if (result !== true) {
              throw new Error("\nWrong configuration in 'custom-filters/demo/config/fields.js'.\nInner error message is '".concat(result, "'.\nSource row is at #").concat(i - NORMAL_FIELDS_TABLE_START_INDEX + 1, ", field is '").concat(fieldName, "'.").trim());
            }
          }
        }
      }
      var field = Object.assign({}, config);
      for (var _fieldName in converters) {
        if (Object.prototype.hasOwnProperty.call(converters, _fieldName)) {
          var converter = converters[_fieldName];
          var _value = config[_fieldName];
          if (typeof converter === 'function') {
            _value = converter.call(config, _value);
          }
          field[_fieldName] = _value;
        }
      }
      for (var _fieldName2 in field) {
        if (Object.prototype.hasOwnProperty.call(field, _fieldName2)) {
          var _value2 = field[_fieldName2];
          if (typeof _value2 === 'function' && NORMAL_FIELDS_AVOID_EVALUATION_FIELDS.indexOf(_fieldName2) === -1) {
            field[_fieldName2] = _value2.call(field, _fieldName2);
          }
        }
      }
      fields.push(field);
    }
  }
  return fields;
}
function buildComponentSettings(configTable) {
  var settingsArray = [];
  if (configTable.length > 0) {
    var validators = configTable[VALUE_SETTINGS_TABLE_VALIDATOR_INDEX];
    var converters = configTable[VALUE_SETTINGS_TABLE_CONVERTER_INDEX];
    for (var i = VALUE_SETTINGS_TABLE_START_INDEX; i < configTable.length; i++) {
      var config = configTable[i];
      for (var fieldName in validators) {
        if (Object.prototype.hasOwnProperty.call(validators, fieldName)) {
          var validator = validators[fieldName];
          var value = config[fieldName];
          if (typeof validator === 'function') {
            var result = validator.call(config, value);
            if (result !== true) {
              throw new Error("\nWrong configuration in 'field-value-settings.js'.\nInner error message is '".concat(result, "'.\nSource row is at #").concat(i - VALUE_SETTINGS_TABLE_START_INDEX + 1, ", field is '").concat(fieldName, "'.").trim());
            }
          }
        }
      }
      var settings = Object.assign({}, config);
      for (var _fieldName3 in converters) {
        if (Object.prototype.hasOwnProperty.call(converters, _fieldName3)) {
          var converter = converters[_fieldName3];
          var _value3 = config[_fieldName3];
          if (typeof converter === 'function') {
            _value3 = converter.call(config, _value3);
          }
          settings[_fieldName3] = _value3;
        }
      }
      for (var _fieldName4 in settings) {
        if (Object.prototype.hasOwnProperty.call(settings, _fieldName4)) {
          var _value4 = settings[_fieldName4];
          if (typeof _value4 === 'function' && VALUE_SETTINGS_AVOID_EVALUATION_FIELDS.indexOf(_fieldName4) === -1) {
            settings[_fieldName4] = _value4.call(settings, _fieldName4);
          }
        }
      }
      settingsArray.push(settings);
    }
  }
  return settingsArray;
}
export default {
  getNodeComponentSettings: getNodeComponentSettings
};