import validators from '../../validators';
import converters from '../../converters';
import getExtraField from './extra-field';
import { getConfigRuleTypes, getManyAnswerChannelTypes } from '../../../../../constant';
function getFieldsTable(blackList, extraList, locale) {
  var lang = locale.lang;
  var configRuleTypes = getConfigRuleTypes(locale);
  var manyAnswerChannelTypes = getManyAnswerChannelTypes(locale);
  var fieldsList = [{
    name: validators.validateString,
    text: validators.validateString,
    inputType: validators.validateInputTypes,
    operators: validators.validateOperators.bind(null, locale),
    dataSource: null,
    component: validators.validateObject,
    attributes: validators.validateObject
  }, {
    name: null,
    text: null,
    inputType: converters.convertInputType,
    operators: converters.convertOperators.bind(null, locale),
    dataSource: null,
    component: null,
    attributes: null
  }, {
    name: configRuleTypes.platform.id,
    text: configRuleTypes.platform.name,
    inputType: 'dropDown',
    operators: ['equal', 'notEqual', 'contain_any', 'not_contain_any'],
    attributes: {
      placeholder: lang.pleaseSelect,
      triggerClass: 'form-control input-sm',
      nameField: 'name',
      valueField: 'id',
      searchField: 'name'
    },
    dataSource: manyAnswerChannelTypes
  }, {
    name: configRuleTypes.src.id,
    text: configRuleTypes.src.name,
    inputType: 'text',
    operators: ['equal', 'notEqual', 'contains', 'not_contains'],
    attributes: {
      classNames: 'form-control input-sm'
    }
  }, {
    name: configRuleTypes.srcUrl.id,
    text: configRuleTypes.srcUrl.name,
    inputType: 'text',
    operators: ['equal', 'notEqual', 'contains', 'not_contains'],
    attributes: {
      classNames: 'form-control input-sm'
    }
  }, {
    name: configRuleTypes.webPluginName.id,
    text: configRuleTypes.webPluginName.name,
    inputType: 'text',
    operators: ['equal', 'notEqual', 'contains', 'not_contains'],
    attributes: {
      classNames: 'form-control input-sm'
    }
  }, {
    name: configRuleTypes.loginUrl.id,
    text: configRuleTypes.loginUrl.name,
    inputType: 'text',
    operators: ['equal', 'notEqual', 'contains', 'not_contains'],
    attributes: {
      classNames: 'form-control input-sm'
    }
  }, {
    name: configRuleTypes.sessionUrl.id,
    text: configRuleTypes.sessionUrl.name,
    inputType: 'text',
    operators: ['equal', 'notEqual', 'contains', 'not_contains'],
    attributes: {
      classNames: 'form-control input-sm'
    }
  }, {
    name: configRuleTypes.dialogueDesc.id,
    text: configRuleTypes.dialogueDesc.name,
    inputType: 'text',
    operators: ['equal', 'notEqual', 'contains', 'not_contains', 'exist', 'not_exist'],
    attributes: {
      classNames: 'form-control input-sm'
    }
  }, {
    name: configRuleTypes.tags.id,
    text: configRuleTypes.tags.name,
    inputType: 'text',
    operators: ['equal', 'notEqual', 'any_equal', 'any_not_equal', 'contain_all', 'not_contain_all', 'exist', 'not_exist'],
    attributes: {
      classNames: 'form-control input-sm'
    }
  },
  // 产品先让去掉   多个机器人无法获取
  // {
  //   name: configRuleTypes.currentTime.id,
  //   text: configRuleTypes.currentTime.name,
  //   inputType: 'autoComplete',
  //   operators: ['belong', 'not_belong'],
  //   attributes: {
  //     nameField: 'name',
  //     valueField: 'id',
  //     searchField: 'name',
  //     queryParamName: 'keyword',
  //     allowClear: false,
  //     enableEmptySearch: true,
  //     url: 'https://km.s4.udesk.cn/backend/responseTimes',
  //     url: Udesk.business.apiPath.concatApiPath(`responseTimes`, sdkOptions),
  //   },
  // },
  {
    name: configRuleTypes.orderReply.id,
    text: configRuleTypes.orderReply.name,
    inputType: 'text',
    operators: []
  }
  // {
  //   name: 'queryContent',
  //   text: lang.questionContent,
  //   inputType: 'text',
  //   operators: ['contains', 'containsSensitive'],
  //   attributes: { placeholder: '' },
  // },
  ];

  if (blackList && Array.isArray(blackList) && blackList.length) {
    fieldsList = fieldsList.filter(function (el) {
      return !blackList.includes(el.name);
    });
  }
  var extraFieldMap = getExtraField(locale);
  if (extraList && Array.isArray(extraList) && extraList.length && extraFieldMap.size) {
    extraList.forEach(function (el) {
      if (extraFieldMap.has(el)) {
        fieldsList.push(extraFieldMap.get(el));
      }
    });
  }
  return fieldsList;
}
export default getFieldsTable;