import { __assign } from "tslib";
import { ajax } from '@udesk-webapp/utils';
export { formatCondition, createCustomerName, unFormatCondition };
// 缓存查询到的客户和公司
var customerMap = {};
var organizationMap = {};
// TODO:进一步抽象
/**
 * 这个方法为业务方提供转换conditions内部的客户和公司的数据格式，也可以业务方通过自己的方法转换好后，传递进来
 *
 * @param {(ConditionInterface[] | undefined)} originalConditions 需要转换的conditions
 * @param {*} conditionFieldField condition的字段属性名
 * @param {*} conditionValueField condition的value属性名
 * @return {*} promise 因为有查询方法，所有返回promise
 */
function formatCondition(originalConditions, conditionFieldField, conditionValueField) {
    if (!originalConditions) {
        return Promise.resolve([]);
    }
    // 查找客户，这里考虑下怎么通用
    var customerIds = getArrayValues(originalConditions, 'customer_id', conditionFieldField, conditionValueField);
    var organizationIds = getArrayValues(originalConditions, 'organization', conditionFieldField, conditionValueField);
    // tag id和tag列表类型不一致，转类型
    // let newConditions = originalConditions?.map((condition) => {
    //     if (condition?.[conditionFieldField] === 'tags') {
    //         let id = condition[conditionValueField];
    //         return {
    //             ...condition,
    //             [conditionValueField]: id ? Number(id) : id,
    //         };
    //     } else {
    //         return condition;
    //     }
    // });
    if ((customerIds === null || customerIds === void 0 ? void 0 : customerIds.length) || (organizationIds === null || organizationIds === void 0 ? void 0 : organizationIds.length)) {
        // let cachedCustomerIds = customerIds?.filter((id) => customerMap[`customer_id_${id}`]);
        var loadCustomerIds_1 = customerIds === null || customerIds === void 0 ? void 0 : customerIds.filter(function (id) { return !customerMap["customer_id_".concat(id)]; });
        // let cachedOrganizationIds = organizationIds?.filter(
        //     (id) => organizationMap[`organization_${id}`]
        // );
        var loadOrganizationIds_1 = organizationIds === null || organizationIds === void 0 ? void 0 : organizationIds.filter(function (id) { return !organizationMap["organization_".concat(id)]; });
        return new Promise(function (resolve, reject) {
            Promise.allSettled([
                getCustomersData(loadCustomerIds_1),
                getOrganizationsData(loadOrganizationIds_1),
            ]).then(function (_a) {
                var customers = _a[0], organizations = _a[1];
                createNewCondition(originalConditions, customerMap, 'customer_id', createCustomerName, conditionFieldField, conditionValueField);
                createNewCondition(originalConditions, organizationMap, 'organization', 'name', conditionFieldField, conditionValueField);
                resolve(originalConditions);
            });
        });
    }
    else {
        return Promise.resolve(originalConditions);
        // setInnerConditions(newConditions);
    }
}
function getArrayValues(arr, id, fieldName, fieldValueName) {
    if (!arr) {
        return;
    }
    var result = [];
    arr.forEach(function (item) {
        if ((item === null || item === void 0 ? void 0 : item[fieldName]) === id &&
            (typeof (item === null || item === void 0 ? void 0 : item[fieldValueName]) === 'string' ||
                typeof (item === null || item === void 0 ? void 0 : item[fieldValueName]) === 'number')) {
            result = result.concat(item === null || item === void 0 ? void 0 : item[fieldValueName]);
        }
    });
    return result;
}
function createCustomerName(customer) {
    var mark = (customer === null || customer === void 0 ? void 0 : customer.email) ? customer === null || customer === void 0 ? void 0 : customer.email : customer === null || customer === void 0 ? void 0 : customer.description;
    if (mark) {
        mark = "(".concat(mark, ")");
    }
    else {
        mark = '';
    }
    return "".concat(customer === null || customer === void 0 ? void 0 : customer.nick_name).concat(mark);
}
function createNewCondition(newConditions, cacheMap, id, labelField, conditionFieldField, conditionValueField) {
    // 不更改conditions的引用
    return newConditions === null || newConditions === void 0 ? void 0 : newConditions.forEach(function (condition) {
        if (condition[conditionFieldField] === id) {
            var value = condition[conditionValueField];
            // 从缓存中找到对应的值
            var cacheData = cacheMap["".concat(id, "_").concat(value)];
            if (cacheData) {
                condition[conditionValueField] = {
                    value: (cacheData === null || cacheData === void 0 ? void 0 : cacheData.id) || null,
                    label: typeof labelField === 'function'
                        ? labelField(cacheData)
                        : cacheData === null || cacheData === void 0 ? void 0 : cacheData[labelField],
                };
                // condition[conditionValueField].value=customer?.id || null;
                // condition[conditionValueField].label=typeof labelField === 'function'
                // ? labelField(customer)
                // : customer?.[labelField];
            }
        }
    });
}
function getCustomer(id) {
    return new Promise(function (resolve, reject) {
        ajax()
            // 这里因为hotfix，没有将前缀提取到utils
            // TODO：提取到utils
            .getJSON("/cs/ts/api/customers/".concat(id))
            .then(function (resp) {
            resolve(resp === null || resp === void 0 ? void 0 : resp.customer);
        }, function (reason) {
            reject(reason);
        });
    });
}
function getOrganization(id) {
    return new Promise(function (resolve, reject) {
        ajax()
            .getJSON("/cs/ts/api/organizations/".concat(id))
            .then(function (resp) {
            resolve(resp === null || resp === void 0 ? void 0 : resp.organization);
        }, function (reason) {
            reject(reason);
        });
    });
}
function getCustomers(ids) {
    if (!ids) {
        return [];
    }
    return ids.map(function (id) {
        return getCustomer(id);
    });
}
function getOrganizations(ids) {
    if (!ids) {
        return [];
    }
    return ids.map(function (id) {
        return getOrganization(id);
    });
}
function getCacheData(cachedCustomerIds, originalConditions, conditionFieldField, conditionValueField) {
    if (cachedCustomerIds === null || cachedCustomerIds === void 0 ? void 0 : cachedCustomerIds.length) {
        var newConditions = createNewCondition(originalConditions, customerMap, 'customer_id', createCustomerName, conditionFieldField, conditionValueField);
        Promise.resolve(newConditions);
    }
    else {
        Promise.resolve();
    }
}
// 用customerMap作为媒介，里面缓存所有查询过的客户，避免多次查询
function getCustomersData(loadCustomerIds) {
    if (loadCustomerIds === null || loadCustomerIds === void 0 ? void 0 : loadCustomerIds.length) {
        return new Promise(function (resolve, reject) {
            Promise.allSettled(getCustomers(loadCustomerIds)).then(function (results) {
                // @ts-ignore
                results === null || results === void 0 ? void 0 : results.forEach(function (result) {
                    if (result.status === 'fulfilled') {
                        var customer = result.value;
                        // 查到的客户缓存
                        customerMap["customer_id_".concat(customer === null || customer === void 0 ? void 0 : customer.id)] = customer;
                    }
                });
                resolve(customerMap);
                // 替换客户
                // let newConditions = createNewCondition(
                //     originalConditions,
                //     customerMap,
                //     'customer_id',
                //     createCustomerName,
                //     conditionFieldField,
                //     conditionValueField
                // );
                // resolve(newConditions);
            });
        });
    }
    else {
        Promise.resolve(customerMap);
    }
}
function getOrganizationsData(loadOrganizationIds) {
    if (loadOrganizationIds === null || loadOrganizationIds === void 0 ? void 0 : loadOrganizationIds.length) {
        return new Promise(function (resolve, reject) {
            Promise.allSettled(getOrganizations(loadOrganizationIds)).then(function (results) {
                // @ts-ignore
                results === null || results === void 0 ? void 0 : results.forEach(function (result) {
                    if (result.status === 'fulfilled') {
                        var organization = result.value;
                        // 缓存查到的公司
                        organizationMap["organization_".concat(organization === null || organization === void 0 ? void 0 : organization.id)] = organization;
                    }
                });
                resolve(organizationMap);
                // 替换客户
                // let newConditions = createNewCondition(
                //     originalConditions,
                //     organizationMap,
                //     'customer_id',
                //     createCustomerName,
                //     conditionFieldField,
                //     conditionValueField
                // );
                // resolve(newConditions);
            });
        });
    }
    else {
        Promise.resolve(organizationMap);
    }
}
// function getCustomersData(cachedCustomerIds,loadCustomerIds,originalConditions,conditionFieldField,conditionValueField){
//     return new Promise((resolve,reject)=>{
//         Promise.all([requestCustomer(loadCustomerIds,originalConditions,conditionFieldField, conditionValueField),getCacheData(cachedCustomerIds,originalConditions,conditionFieldField,conditionValueField)])
//         if (loadCustomerIds?.length) {
//             return new Promise((resolve, reject) => {
//                 Promise.allSettled(getCustomers(loadCustomerIds)).then((results) => {
//                     // @ts-ignore
//                     results?.forEach((result) => {
//                         if (result.status === 'fulfilled') {
//                             let customer = result.value;
//                             // 查到的客户缓存
//                             customerMap[`customer_id_${customer?.id}`] = customer;
//                         }
//                     });
//                     // 替换客户
//                     let newConditions = createNewCondition(
//                         originalConditions,
//                         customerMap,
//                         'customer_id',
//                         createCustomerName,
//                         conditionFieldField,
//                         conditionValueField
//                         // 'nick_name'
//                     );
//                     resolve(newConditions)
//                     // setInnerConditions(newConditions);
//                 });
//             });
//         } else if (cachedCustomerIds?.length) {
//             let newConditions = createNewCondition(
//                 originalConditions,
//                 customerMap,
//                 'customer_id',
//                 createCustomerName,
//                 conditionFieldField,conditionValueField
//                 // 'nick_name'
//             );
//             resolve(newConditions)
//             // setInnerConditions(newConditions);
//         }
//     })
// }
/**
 *  将conditions内的客户和公司转换回id的形式，业务方如果有需要可以调用
 * 不更改conditions引用，直接更改的conditions
 * 更改conditions引用会导致整个组件重新渲染，性能低
 *
 * @param {(ConditionInterface[] | undefined)} originalConditions
 * @param {*} conditionFieldField
 * @param {*} conditionValueField
 * @return {*}
 */
var unFormatCondition = function (originalConditions, conditionFieldField, conditionValueField, copy) {
    if (copy === void 0) { copy = false; }
    if (!originalConditions) {
        return originalConditions;
    }
    if (copy) {
        return originalConditions.map(function (condition) {
            var _a;
            var _b;
            if (((condition === null || condition === void 0 ? void 0 : condition[conditionFieldField]) === 'customer_id' ||
                (condition === null || condition === void 0 ? void 0 : condition[conditionFieldField]) === 'organization') &&
                typeof (condition === null || condition === void 0 ? void 0 : condition[conditionValueField]) === 'object') {
                return __assign(__assign({}, condition), (_a = {}, _a[conditionValueField] = ((_b = condition === null || condition === void 0 ? void 0 : condition[conditionValueField]) === null || _b === void 0 ? void 0 : _b[conditionValueField]) || null, _a));
            }
            else {
                return condition;
            }
        });
    }
    else {
        originalConditions.forEach(function (condition) {
            var _a;
            if (((condition === null || condition === void 0 ? void 0 : condition[conditionFieldField]) === 'customer_id' ||
                (condition === null || condition === void 0 ? void 0 : condition[conditionFieldField]) === 'organization') &&
                typeof (condition === null || condition === void 0 ? void 0 : condition[conditionValueField]) === 'object') {
                condition[conditionValueField] =
                    ((_a = condition === null || condition === void 0 ? void 0 : condition[conditionValueField]) === null || _a === void 0 ? void 0 : _a[conditionValueField]) || null;
                // return {
                //     ...condition,
                //     [conditionValueField]:
                //         condition?.[conditionValueField]?.[conditionValueField] || null,
                // };
            }
            // else {
            //     return condition;
            // }
        });
        return originalConditions;
    }
};
