import React, { useEffect, useState } from 'react';
import { Select, Input, Button, message } from 'udesk-ui';
import { useTranslation } from 'react-i18next';
import { SwapOutlined } from '@ant-design/icons';
import dayjs from 'dayjs';
import { originalOptions, translateOptions, originalMap } from './enums';
import css from './index.module.less';
var TextArea = Input.TextArea;
var TRANSLATE_CACHE = 'translateConfig';
// 原文组件
var Translate = function (props) {
    var onTranslate = props.onTranslate, _a = props.persistenceKey, persistenceKey = _a === void 0 ? '' : _a;
    var t = useTranslation().t;
    var _b = useState(originalMap.ZhCn), originalValue = _b[0], setOriginalValue = _b[1]; // 原文语言
    var _c = useState(originalMap.EnUs), translateValue = _c[0], setTranslateValue = _c[1]; // 译文语言
    var _d = useState(''), originalText = _d[0], setOriginalText = _d[1]; // 原文
    var _e = useState(false), loading = _e[0], setLoading = _e[1];
    useEffect(function () {
        var translateConfig = JSON.parse(window.localStorage.getItem(TRANSLATE_CACHE) || '{}');
        if (translateConfig.from) {
            setOriginalValue(translateConfig.from);
        }
        if (translateConfig.to) {
            setTranslateValue(translateConfig.to);
        }
    }, []);
    useEffect(function () {
        if (persistenceKey) {
            var originalCache = JSON.parse(window.localStorage.getItem(persistenceKey) || '{}');
            // 缓存有效期1天 超出清空
            if (originalCache.cacheTime && dayjs(originalCache.cacheTime).diff(dayjs(), 'day') >= 1) {
                window.localStorage.removeItem(persistenceKey);
                originalCache = {};
            }
            if (originalCache.text) {
                setOriginalText(originalCache.text);
            }
        }
    }, []);
    // 译文下拉change
    var handleTranslateChange = function (v) {
        setTranslateValue(v);
        // 设置缓存
        window.localStorage.setItem(TRANSLATE_CACHE, JSON.stringify({
            from: originalValue,
            to: v
        }));
    };
    // 原文变化
    var handleOriginChange = function (e) {
        var v = e.target.value;
        setOriginalText(v);
        // 设置缓存
        if (persistenceKey) {
            var data = {
                cacheTime: new Date().getTime(),
                text: v
            };
            window.localStorage.setItem(persistenceKey, JSON.stringify(data));
        }
    };
    // 点击翻译
    var onHandleTranslate = function () {
        if (loading)
            return;
        if (!originalText) {
            return message.error(t('originalPlaceholder'));
        }
        var o = originalValue;
        if (/[\u4e00-\u9fa5]/.test(originalText)) { // 有一个中文就算中文
            if (originalValue !== originalMap.ZhCn) {
                o = originalMap.ZhCn;
                setOriginalValue(o);
            }
        }
        else { // auto
            if (originalValue !== originalMap.Auto) {
                o = originalMap.Auto;
                setOriginalValue(o);
            }
        }
        if (onTranslate) {
            setLoading(true);
            onTranslate({
                from: o,
                to: translateValue,
                original: originalText
            }).then(function () {
                setLoading(false);
            });
        }
        // 设置缓存
        if (persistenceKey) {
            var data = {
                cacheTime: new Date().getTime(),
                text: originalText
            };
            window.localStorage.setItem(persistenceKey, JSON.stringify(data));
        }
        window.localStorage.setItem(TRANSLATE_CACHE, JSON.stringify({
            from: o,
            to: translateValue
        }));
    };
    return (React.createElement("div", { className: css['original-wrap'] },
        React.createElement("div", { className: css['original-toolbar-wrap'] },
            React.createElement(Select, { disabled: true, style: { width: 120 }, value: originalValue, options: originalOptions, onChange: function (v) { return setOriginalValue(v); } }),
            React.createElement(SwapOutlined, { className: css['original-toolbar-icon'] }),
            React.createElement(Select, { style: { width: 120 }, value: translateValue, options: translateOptions, onChange: handleTranslateChange, className: css['original-toolbar-translate'] })),
        React.createElement("div", { className: css['original-content-wrap'] },
            React.createElement(TextArea, { value: originalText, onChange: handleOriginChange, bordered: false, autoSize: { minRows: 6 }, placeholder: t('originalPlaceholder') })),
        React.createElement(Button, { type: 'primary', onClick: onHandleTranslate, loading: loading }, t('statusbar.translate'))));
};
export default React.memo(Translate);
