import { __assign, __rest, __spreadArray } from "tslib";
import React, { useCallback, useContext, useEffect, useMemo, useRef, useState } from 'react';
import { Empty, Popover, Spin, TreeSelect } from 'udesk-ui';
import VirtualList from 'rc-virtual-list';
import ScopeSwitch from '../scope-switch';
import DOMPurify from 'dompurify';
import { useTranslation } from '@udesk-webapp/locale';
import { createUseSelectedCatalog, hit, RepositoryWidgetContext, useRequestData, useSafeScope, } from '../utilities';
import css from '../style.module.scss';
import classNames from 'classnames';
import SearchInput from '../search-input';
import { replacePlaceholder } from '../utils/repository-widget';
import CatalogSelect from '../catalog-select';
function replaceWrapAndSpace(str) {
    if (str) {
        str = str.replace(/[\n\r]/g, '<br>').replace(/[\n]/g, '<br>').replace(/[\s]/g, '&nbsp;');
    }
    else {
        str = '';
    }
    return str;
}
var TreeNode = TreeSelect.TreeNode;
var ContainerHeight = 650;
var UsefulExpressionInner = function (_a) {
    var customerFieldAbilities = _a.customerFieldAbilities, data = _a.data, query = _a.query, onScrollToBottom = _a.onScrollToBottom;
    var _b = useContext(RepositoryWidgetContext), onUse = _b.onUse, isWorkbench = _b.isWorkbench, agent = _b.agent, customer = _b.customer, organizations = _b.organizations, ticket = _b.ticket, ticketCustomFields = _b.ticketCustomFields, customerCustomFields = _b.customerCustomFields;
    var t = useTranslation().t;
    var wrapHeightRef = useRef();
    useEffect(function () {
        var _a;
        var wrapDom = (_a = document.getElementsByClassName('repositoryTabsWrap')) === null || _a === void 0 ? void 0 : _a[0];
        if (wrapDom) { // 避免小数 出现外层滚动条
            wrapHeightRef.current = wrapDom.clientHeight ? (wrapDom.clientHeight - 173) : ContainerHeight;
        }
    }, []);
    var onClickUseful = function (item) {
        if (!isWorkbench)
            return;
        if (item) {
            replacePlaceholder(item.content, {
                customer: customer,
                organizations: organizations,
                agent: agent,
                ticket: ticket,
                customerCustomFields: customerCustomFields,
                ticketCustomFields: ticketCustomFields,
                t: t,
                customerFieldAbilities: customerFieldAbilities,
            }).then(function (content) {
                onUse === null || onUse === void 0 ? void 0 : onUse('useful_expression', __assign(__assign({}, item), { content: content }));
            });
            hit(item.id, 'UsefulExpression');
        }
    };
    var generateContent = function (item) {
        if (!item)
            return null;
        var safeContent = DOMPurify.sanitize(item.content);
        return (React.createElement("div", { className: "".concat(css['repository-item-wrap'], " ").concat(query ? css['highlight'] : '') },
            React.createElement(Popover, { overlayClassName: css.expressionPopover, placement: 'left', destroyTooltipOnHide: true, content: React.createElement("div", { className: css['useful-popover-content'], dangerouslySetInnerHTML: { __html: replaceWrapAndSpace(safeContent) } }) },
                React.createElement("p", { onClick: function () { return onClickUseful(item); }, className: "".concat(css['repository-item'], " ").concat(css['expression-item']), dangerouslySetInnerHTML: { __html: safeContent } }))));
    };
    var onScroll = function (e) {
        // 滚动到了底部
        if (e.currentTarget.scrollHeight - e.currentTarget.scrollTop === wrapHeightRef.current) {
            onScrollToBottom();
        }
    };
    return (React.createElement("div", { id: 'knowledgeTreeWrapper', className: classNames(css['repository-virtualList-wrapper']) },
        React.createElement(VirtualList, { data: data, height: wrapHeightRef.current, itemHeight: 43, itemKey: 'id', onScroll: onScroll }, function (item) { return generateContent(item); })));
};
export var useSelectedCatalog = createUseSelectedCatalog();
var UsefulExpression = function (_a) {
    var _b, _c, _d, _e, _f;
    var roleCan = _a.roleCan, refreshKnowledge = _a.refreshKnowledge, onSearch = _a.onSearch, mineChats = _a.mineChats, agentConfig = _a.agentConfig, dialogueTagData = _a.dialogueTagData, _g = _a.stashKey, stashKey = _g === void 0 ? 'default' : _g, props = __rest(_a, ["roleCan", "refreshKnowledge", "onSearch", "mineChats", "agentConfig", "dialogueTagData", "stashKey"]);
    var query = props.query;
    var chat = (mineChats === null || mineChats === void 0 ? void 0 : mineChats.find(function (chat) { return chat.selected; })) || {};
    var _h = useContext(RepositoryWidgetContext), activeScope = _h.activeScope, onScopeChange = _h.onScopeChange;
    var _scope = useSafeScope(activeScope);
    var tagsIdRef = useRef(-1);
    var _j = useState(false), tagCatalog = _j[0], setTagCatalog = _j[1];
    var _k = useState(1), page = _k[0], setPage = _k[1];
    var _l = useState([]), usefulExpression = _l[0], setUsefulExpression = _l[1];
    var pagingRef = useRef({ current_page: 0, total_pages: 0 });
    var t = useTranslation().t;
    var _m = useSelectedCatalog(_scope, stashKey), selectedCatalog = _m[0], setSelectedCatalog = _m[1];
    var params = useMemo(function () {
        var q = query.trim();
        var catalogId = selectedCatalog || -1;
        if (catalogId === -1 || q) { // 查询是在全部分类中筛选
            catalogId = undefined;
        }
        return {
            knowledge_catalog_id: catalogId,
            scope: _scope,
            per_page: 20,
            page: page,
            current_works: true,
            query: q
        };
    }, [page, _scope, query, selectedCatalog]);
    var _o = useRequestData('useful_expression', params, refreshKnowledge, roleCan), loading = _o.loading, setLoading = _o.setLoading, catalogs = _o.catalogs, data = _o.data, hasPermission = _o.hasPermission;
    useEffect(function () {
        setPage(1);
        setTagCatalog(false);
        setLoading(true);
    }, [_scope]);
    var tagNameArray = useMemo(function () {
        var _a, _b, _c, _d;
        if (((_a = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _a === void 0 ? void 0 : _a.enable) &&
            (dialogueTagData === null || dialogueTagData === void 0 ? void 0 : dialogueTagData.length) &&
            ((_b = chat.dialogueTag) === null || _b === void 0 ? void 0 : _b.length)) {
            if (((_c = agentConfig.session_tags_config) === null || _c === void 0 ? void 0 : _c.mode_tags) === 'first') {
                return chat.dialogueTag[0].map(function (i) { var _a; return (_a = dialogueTagData === null || dialogueTagData === void 0 ? void 0 : dialogueTagData.find(function (j) { return j.id === i; })) === null || _a === void 0 ? void 0 : _a.name; }).filter(function (i) { return !!i; });
            }
            else if (((_d = agentConfig.session_tags_config) === null || _d === void 0 ? void 0 : _d.mode_tags) === 'new') {
                return chat.dialogueTag[chat.dialogueTag.length - 1].map(function (i) { var _a; return (_a = dialogueTagData === null || dialogueTagData === void 0 ? void 0 : dialogueTagData.find(function (j) { return j.id === i; })) === null || _a === void 0 ? void 0 : _a.name; }).filter(function (i) { return !!i; });
            }
        }
        return [];
    }, [agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config, chat.dialogueTag, dialogueTagData]);
    var tagName = useMemo(function () {
        var _a, _b;
        if (((_a = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _a === void 0 ? void 0 : _a.enable)
            && ((_b = agentConfig.session_tags_config) === null || _b === void 0 ? void 0 : _b.show_mode) === 'keyword'
            && tagNameArray.length > 0) {
            return tagNameArray.join(' ');
        }
        return '';
    }, [(_b = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _b === void 0 ? void 0 : _b.enable, (_c = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _c === void 0 ? void 0 : _c.show_mode, tagNameArray]);
    var tagsName = useCallback(function (list, tagList, num) {
        // let tagsId: number | undefined;
        list.children.forEach(function (items) {
            var _a, _b;
            if (items.name === tagList[num] && ((_a = items.children) === null || _a === void 0 ? void 0 : _a.length) && tagList[num + 1]) {
                num += 1;
                tagsName(items, tagList, num);
            }
            else if ((items.name === tagList[num] && !((_b = items.children) === null || _b === void 0 ? void 0 : _b.length)) ||
                (items.name === tagList[num] && !tagList[num + 1])) {
                // tagsId = items.id;
                tagsIdRef.current = items.id;
            }
            else if (!tagsIdRef.current) {
                // tagsId = list.id;
                tagsIdRef.current = list.id;
            }
        });
        return tagsIdRef.current;
    }, []);
    var defaultTags = useCallback(function () {
        var _a, _b, _c, _d, _e;
        var tagList = [];
        if (((_a = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _a === void 0 ? void 0 : _a.enable) &&
            ((_b = agentConfig.session_tags_config) === null || _b === void 0 ? void 0 : _b.show_mode) === 'type' &&
            !tagCatalog &&
            (dialogueTagData === null || dialogueTagData === void 0 ? void 0 : dialogueTagData.length) &&
            ((_c = chat.dialogueTag) === null || _c === void 0 ? void 0 : _c.length)) {
            if (((_d = agentConfig.session_tags_config) === null || _d === void 0 ? void 0 : _d.mode_tags) === 'first') {
                chat.dialogueTag[0].forEach(function (item) {
                    dialogueTagData.forEach(function (items) {
                        if (item === items.id) {
                            tagList.push(items.name);
                        }
                    });
                });
                var num_1 = 0;
                catalogs.forEach(function (item) {
                    var _a;
                    if (item.name === tagList[num_1] && tagList.length > 1 && ((_a = item.children) === null || _a === void 0 ? void 0 : _a.length)) {
                        num_1 += 1;
                        // tagsId = tagsName(item, tagList, num);
                        tagsName(item, tagList, num_1);
                    }
                    else if (item.name === tagList[tagList.length - 1]) {
                        // tagsId = item.id;
                        tagsIdRef.current = item.id;
                    }
                });
            }
            else if (((_e = agentConfig.session_tags_config) === null || _e === void 0 ? void 0 : _e.mode_tags) === 'new') {
                chat.dialogueTag[chat.dialogueTag.length - 1].forEach(function (item) {
                    dialogueTagData.forEach(function (items) {
                        if (item === items.id) {
                            tagList.push(items.name);
                        }
                    });
                });
                var num_2 = 0;
                catalogs.forEach(function (item) {
                    var _a;
                    if (item.name === tagList[num_2] && tagList.length > 1 && ((_a = item.children) === null || _a === void 0 ? void 0 : _a.length)) {
                        num_2 += 1;
                        // tagsId = tagsName(item, tagList, num, tagsId);
                        tagsName(item, tagList, num_2);
                    }
                    else if (item.name === tagList[tagList.length - 1]) {
                        // tagsId = item.id;
                        tagsIdRef.current = item.id;
                    }
                });
            }
            setSelectedCatalog(tagsIdRef.current);
            return tagsIdRef.current;
        }
        else {
            return -1;
        }
    }, [(_d = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _d === void 0 ? void 0 : _d.enable, (_e = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _e === void 0 ? void 0 : _e.mode_tags, (_f = agentConfig === null || agentConfig === void 0 ? void 0 : agentConfig.session_tags_config) === null || _f === void 0 ? void 0 : _f.show_mode, catalogs, chat.dialogueTag, dialogueTagData, tagCatalog, tagsName, setSelectedCatalog]);
    useEffect(function () {
        defaultTags();
    }, [defaultTags]);
    useEffect(function () {
        if (loading)
            return;
        setPage(1);
        setUsefulExpression([]);
    }, [params.query]);
    useEffect(function () {
        setUsefulExpression([]);
    }, [params.knowledge_catalog_id, params.scope]);
    useEffect(function () {
        if (data === null || data === void 0 ? void 0 : data.expressions) {
            // @ts-ignore
            setUsefulExpression(function (prevState) { return __spreadArray(__spreadArray([], prevState, true), data.expressions, true); });
            pagingRef.current = data.meta;
        }
    }, [data]);
    var pageChangeCallback = function () {
        var _a;
        if (loading)
            return;
        if (page >= ((_a = pagingRef.current) === null || _a === void 0 ? void 0 : _a.total_pages))
            return;
        setLoading(true);
        setPage(page + 1);
    };
    var changeSelectedCatalogs = useCallback(function (value, label, extra) {
        setTagCatalog(true);
        setSelectedCatalog(value);
        setLoading(true);
        setPage(1);
        tagsIdRef.current = -1;
    }, []);
    useEffect(function () {
        tagsIdRef.current = -1;
    }, [catalogs]);
    return React.createElement(React.Fragment, null,
        React.createElement(ScopeSwitch, { value: _scope, onChange: onScopeChange }),
        React.createElement(SearchInput, { onSearch: onSearch, defaultValue: tagName ? tagName : query }),
        (query && query.trim()) ? null :
            React.createElement(CatalogSelect, { data: catalogs, prependTreeNodes: [React.createElement(TreeNode, { value: -1, key: '-1', title: t('page.im.km.question.all_catalog') })], value: selectedCatalog, onChange: changeSelectedCatalogs }),
        (function () {
            if (!usefulExpression) {
                return null;
            }
            if (!hasPermission) {
                return React.createElement("div", { className: classNames(css['no-permission']) }, "\u60A8\u6682\u65E0\u6743\u9650\u67E5\u770B\u6B64\u5185\u5BB9");
            }
            if (!loading && (usefulExpression && !usefulExpression.length)) {
                return React.createElement(Empty, { className: "h-full w-hull flex justify-center items-center flex-col" });
            }
            return (React.createElement(React.Fragment, null,
                React.createElement(UsefulExpressionInner, __assign({}, props, { data: usefulExpression, onScrollToBottom: pageChangeCallback })),
                loading ? React.createElement("div", { className: classNames(css['repository-loading']) },
                    React.createElement(Spin, { className: 'w-full text-center' })) : null));
        })());
};
export default UsefulExpression;
