import React, { useMemo } from 'react';
import { uniq } from 'lodash-es';
import PlusOutlined from '@ant-design/icons/PlusOutlined';
import { useTranslation } from 'react-i18next';
import DOMPurify from 'dompurify';
import emoji from './emojiData';
import { memoize, each } from 'lodash';
import pdf from './images/pdf02.png';
import word from './images/word02.png';
import excel from './images/excel02.png';
import ppt from './images/ppt02.png';
import txt from './images/txt02.png';
import unknown from './images/unknow02.png';
var telReg = /\b(((\+\d\d)?1\d{10})|((0\d{2,3}-?)\d{7,8}))\b/g;
var emailReg = /((\w)+(\.\w+)*@(\w)+((\.\w+)+))/g;
export var useAddToCustomerMenuItem = function (allowAddToCustomerInfo, content) {
    var t = useTranslation().t;
    var tels = useMemo(function () { return uniq(JSON.stringify(content).match(telReg)); }, [content]);
    var emails = useMemo(function () { return uniq(JSON.stringify(content).match(emailReg)); }, [content]);
    var addToCustomerInfoMenu = useMemo(function () {
        if (allowAddToCustomerInfo && (tels.length + emails.length) > 1) {
            return React.createElement("div", { title: t('page.im.log.addToCustomerInfo') },
                tels.map(function (i) { return React.createElement("div", { key: i },
                    React.createElement(PlusOutlined, null),
                    i); }),
                emails.map(function (i) { return React.createElement("div", { key: i },
                    React.createElement(PlusOutlined, null),
                    i); }));
        }
        else if (allowAddToCustomerInfo && (tels.length + emails.length) === 1) {
            var contact = tels[0] || emails[0];
            return React.createElement("div", null, t('page.im.log.addToCustomerInfo'));
        }
        return null;
    }, [allowAddToCustomerInfo, emails, t, tels]);
    return [addToCustomerInfoMenu];
};
export function update_sdk_url(url, obj) {
    obj = obj || {};
    var is_receive = obj.is_receive, platform = obj.platform;
    var result = url;
    return result;
}
function getUrlBacket(url) {
    if (!url)
        return;
    var qiniuReg = /qn-im\.udesk\.cn|qn-public\.udesk\.cn|qn-private\.udesk\.cn/i;
    var qiniuRegNew = /qbox\.me/i;
    var aliReg = /aliyuncs\.com/i;
    var aliRegNew = /kefutoutiao\.com/i;
    var minioReg = /47\.94\.16\.154:9000/i; // 暂定,本地化需修改这个值
    if (qiniuReg.test(url) || qiniuRegNew.test(url)) {
        return 'qiniu';
    }
    else if (aliReg.test(url) || aliRegNew.test(url)) {
        return 'ali';
    }
    else if (minioReg.test(url)) {
        return 'minio';
    }
    else {
        return null;
    }
}
export function imageThumbnail(url, policy, fileSize) {
    if (policy === void 0) { policy = ''; }
    if (!url) {
        return '';
    }
    if (!policy) {
        policy = getUrlBacket(url);
    }
    if (policy === 'qiniu') {
        url = (url.split('?')[0] + '?imageMogr2/auto-orient/thumbnail/300x300>');
    }
    else if (policy === 'ali') {
        if (url.indexOf('Signature') === -1 || Number.parseInt(fileSize) >= 50) {
            /* gif图片不加后缀(图片添加旋转纠正参数后不动了)    大于20M图片不加后缀(加后缀IM工作台无法显示) */
            if (/\.gif$/.test(url)) {
                url = url.split('?')[0];
            }
            else {
                url = url.split('?')[0] + '?x-oss-process=image/auto-orient,0/resize,l_300';
            }
        }
        // eslint-disable-next-line no-empty
    }
    else if (policy === 'minio') {
    }
    return url;
}
export function getTencentMap(params) {
    var imgUrl = 'http://apis.map.qq.com/ws/staticmap/v2/', mapUrl = 'http://apis.map.qq.com/uri/v1/marker', key = 'NFFBZ-QGA3P-66OD5-LDYQH-GHTEV-CWBUV';
    try {
        var locationData = params.split(';') || ['0', '0', '', ''];
        var local = locationData[0] + ',' + locationData[1];
        var zoom = locationData[2] || 16;
        var localName = locationData[3] || '位置';
        // '&labels=anchor:3|offset:0_-50|bgcolor:yellow|' + localName + '|' + local +
        return {
            img_src: imgUrl + '?key=' + key + '&size=200*150&center=' + local + '&zoom=' + zoom + '&markers=' + local,
            map_src: mapUrl + '?marker=coord:' + local + ';title:位置信息;addr:' + localName
        };
    }
    catch (e) {
        return {};
    }
}
export function escapeXSS(str, canHtml) {
    str = str + '';
    if (str) {
        if ((str.search('/:') === -1) && !canHtml) {
            str = str.replace(/</g, '&lt;').replace(/>/g, '&gt;');
        }
        return str.replace(/\n/g, '<br>').replace(/%26/g, '&').replace(/%25/g, '%'); //%26为api类型特殊用途;
    }
    return '';
}
export function replaceEmotions(text) {
    return (text + '').replace(/\[[e|E]moji([^\].]+)\]/g, '<span class="emoji emoji$1"></span>');
}
export function replaceWechatEmotions(text, twemoji) {
    if (twemoji === void 0) { twemoji = false; }
    var emojiArr = emoji.getWechatData();
    text = (text + '').replace(/&gt;/g, '>').replace(/&lt;/g, '<').replace(/&#39;/g, '\'').replace(/\/::$/, '/:: ').replace(/&amp;/g, '&');
    each(emojiArr, function (item) {
        var reg = new RegExp(item.regcode, 'g');
        if (twemoji) {
            text = text.replace(reg, '<span class="wechat-emoji twemoji">' + item.twemoji + '</span>');
        }
        else {
            if (text.includes('richText')) {
                text = text.replace(reg, '<span class=\\"wechat-emoji ' + item.clz + '\\"></span>');
            }
            else {
                text = text.replace(reg, '<span class="wechat-emoji ' + item.clz + '"></span>');
            }
        }
    });
    return text;
}
export function replaceWordEmotions(text, platform) {
    var isWechatEmoji = platform === 'wechat' || platform === 'api' || platform === 'weixin_mini' || platform === 'qywx' || platform === 'web' || platform === 'weixin_kf';
    var emojiArr = isWechatEmoji ? emoji.getWechatData() : emoji.getSdkData();
    if (platform === 'line') {
        emojiArr = emoji.getWebData();
    }
    else if (platform === 'qywx') {
        emojiArr = emoji.getQywxData();
    }
    else if (platform === 'weixin_kf') {
        emojiArr = emoji.getWeixin_kfData();
    }
    each(emojiArr, function (item) {
        var reg = new RegExp(item.regtext, 'g');
        text = text.replace(reg, "<span class=\"".concat(isWechatEmoji ? 'wechat-' : '', "emoji ") + item.clz + '"></span>');
    });
    return text;
}
//富文本样式
export function handleText(chatPlatform, msg) {
    //处理content
    var telReg = /\b(((\+\d\d)?1\d{10})|((0\d{2,3}-?)\d{7,8}))\b/g;
    var emailReg = /((\w)+(\.\w+)*@(\w)+((\.\w+)+))/g;
    var content = escapeXSS(msg.content, false);
    var reg = /((http|https):\/\/([\w-]+\.)+[\w-:]+(\/[\w-./?;%+&=#：]*)?)/;
    var isLink = reg.test(content);
    var twemoji = msg.twemoji;
    content = (content + '').replace(/((http|https):\/\/([\w-]+\.)+[\w-:]+(\/[\w-./?;%+&=#：:]*)?)/g, '<a href="$1" target="_blank">$1</a>');
    if (content.search('/:') > -1) {
        content = replaceWechatEmotions(content, twemoji);
    }
    else if ((content || '').indexOf('[') > -1 && (content || '').indexOf('[emoji') < 0) {
        content = replaceWordEmotions(content, chatPlatform);
    }
    else {
        content = replaceEmotions(content);
    }
    if (!isLink) {
        content = content.replace(telReg, '<a>$1</a>').replace(emailReg, '<a>$1</a>');
    }
    // 如果是链接  不进行xss处理  否则target会被过滤掉
    if (isLink) {
        return "<div style=".concat(msg.font || '', ">").concat(content, "</div>");
    }
    else {
        return "<div style=".concat(msg.font || '', ">").concat(DOMPurify.sanitize(content), "</div>");
    }
}
export function removeHTMLTag(str) {
    if (!str) {
        return '';
    }
    str = str.replace(/<\/?[^>]*>/g, ''); //去除HTML tag
    // str = str.replace(/[ | ]*\n/g, '\n'); //去除行尾空白
    //str = str.replace(/\n[\s| | ]*\r/g,'\n'); //去除多余空行
    str = str.replace(/&nbsp;/ig, ' '); //去掉&nbsp;
    return str;
}
export function getStanardFileSizeSuffixes(sizeStr) {
    if (!sizeStr) {
        return '';
    }
    if (typeof sizeStr === 'number') {
        sizeStr = sizeStr + 'B';
    }
    var reg = /[a-zA-Z]{1,2}/i;
    var suffixes = {
        B: 'B',
        K: 'KB',
        M: 'MB',
        G: 'GB',
        T: 'TB',
        KB: 'KB',
        MB: 'MB',
        GB: 'GB',
        TB: 'TB'
    };
    var newStr = sizeStr.replace(reg, function (match) {
        return suffixes[match.toUpperCase()] || match.toUpperCase();
    });
    return newStr;
}
export var loadStyle = memoize(function (href) {
    if (!href) {
        return;
    }
    var head = document.head;
    var link = document.createElement("link");
    link.type = 'text/css';
    link.rel = 'stylesheet';
    link.href = href;
    head.appendChild(link);
});
export var loadScript = memoize(function (src) {
    return new Promise(function (resolve, reject) {
        if (!src) {
            reject(new Error('src is required'));
            return;
        }
        var script = document.createElement('script');
        script.src = src;
        script.async = true;
        script.addEventListener('load', function () {
            resolve(script);
        });
        script.addEventListener('error', function (error) {
            var _a;
            script.remove();
            reject(error);
            (_a = loadStyle === null || loadStyle === void 0 ? void 0 : loadStyle.cache) === null || _a === void 0 ? void 0 : _a.delete(src);
        });
        document.body.appendChild(script);
    });
});
export var getFileType = function (type) {
    if (typeof type !== 'string')
        return unknown;
    if (/\.(pdf)/.test(type)) {
        type = pdf;
    }
    else if (/\.(doc|docx|odt)/.test(type)) {
        type = word;
    }
    else if (/\.(xls|xlsx|csv)/.test(type)) {
        type = excel;
    }
    else if (/\.(ppt|pptx)/.test(type)) {
        type = ppt;
    }
    else if (/\.(txt)/.test(type)) {
        type = txt;
    }
    else {
        type = unknown;
    }
    return type;
};
