import { __assign, __spreadArray } from "tslib";
import React, { useCallback, useContext, useEffect, useMemo, useReducer, useRef, useState } from 'react';
import { getJSON, post, postJSON } from './utils';
import { csApiGetJSON, csApiRequest } from './utils/csApi';
import qs from 'qs';
import weixinDefaultPng from './weixin_default.png';
import useDebounce from 'react-use/esm/useDebounce';
import { isEqual, memoize } from 'lodash';
var fetchKnowledge = memoize(function (requestOptions) { return csApiRequest(__assign({}, requestOptions)).then(function (res) { return res.data; }); }, function (requestOptions) { return "".concat(requestOptions.url).concat(requestOptions.method).concat(requestOptions.method === 'get' ? qs.stringify(requestOptions.params) : requestOptions.data); });
var fetchKnowledgeCatalogs = memoize(function (params) { return csApiGetJSON('/cs/ts/api/knowledge_catalogs', {
    params: params,
}).then(function (res) { return res.data; }); }, function (params) { return params.knowledge_type_name + params.scope + params.query; });
export var IM_REPOSITORY_ROBOT_KM = -21; //对接机器人的知识库
export var IM_REPOSITORY_ATTACHMENT = -20; //附件库
export var IM_REPOSITORY_EXPRESSION = -17; //常用语
export var IM_REPOSITORY_LINK = -18; //常用链接
export var IM_REPOSITORY_QUESTION = -19; //问题库
export var useRequestData = function (type, params, refreshKnowledge, roleCan) {
    // const refreshKnowledge = useSelector((state: RootState) => state.modals.refreshKnowledge);
    // const roleCan = useSelector<RootState, IRoleCan>(state => state.roleCan);
    var scope = params.scope;
    var _a = useState([]), catalogs = _a[0], setCatalogs = _a[1];
    var _b = useState(true), loading = _b[0], setLoading = _b[1];
    var _c = useState(), data = _c[0], setData = _c[1];
    var _d = useState(true), hasPermission = _d[0], setHasPermission = _d[1];
    useEffect(function () {
        setHasPermission(function (pre) {
            if (type === 'useful_expression') {
                if (scope === 'public') {
                    return !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_view_public) && !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_useful_expressions_public);
                }
                else {
                    return !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_view) && !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_useful_expressions);
                }
            }
            else if (type === 'useful_link') {
                if (scope === 'public') {
                    return !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_view_public) && !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_useful_links_public);
                }
                else {
                    return !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_view) && !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_useful_links);
                }
            }
            else if (type === 'attachment') {
                if (scope === 'public') {
                    return !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_view_public) && !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_attachments_public);
                }
                else {
                    return !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_view) && !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_attachments);
                }
            }
            else if (type === 'question') {
                if (scope === 'public') {
                    return !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_view_public) && !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_questions_public);
                }
                else {
                    return !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_view) && !!(roleCan === null || roleCan === void 0 ? void 0 : roleCan.knowledge_questions);
                }
            }
            else {
                return true;
            }
        });
    }, [type, scope]);
    var dataUrl = (function () {
        switch (type) {
            case 'useful_expression':
            case 'useful_link':
                return "/cs/ts/api/".concat(type, "s/search");
            case 'question':
                return '/cs/ts/api/knowledge_questions/search';
            case 'attachment':
                return '/spa1/knowledge_attachments';
        }
    })();
    var requestOptions = useMemo(function () {
        var method = (function () {
            switch (type) {
                case 'useful_expression':
                case 'useful_link':
                case 'question':
                    return 'post';
                case 'attachment':
                    return 'get';
            }
        })();
        var res = {
            url: dataUrl,
            method: method
        };
        if (method === 'post') {
            res.headers = { 'content-type': 'application/x-www-form-urlencoded' };
            res.data = qs.stringify(params);
        }
        else {
            res.params = params;
        }
        return res;
    }, [dataUrl, params]);
    useDebounce(function () {
        if (!hasPermission)
            return;
        setLoading(true);
        Promise.all([
            fetchKnowledge(requestOptions),
            fetchKnowledgeCatalogs({
                knowledge_type_name: type,
                scope: scope,
                query: params.query,
            }).then(function (res) { return res.catalogs; }),
        ])
            .then(function (_a) {
            var data = _a[0], catalogs = _a[1];
            setData(data);
            setCatalogs(catalogs);
        })
            .finally(function () { return setLoading(false); });
    }, 50, [dataUrl, params, scope, type, requestOptions, refreshKnowledge, hasPermission]);
    return { loading: loading, catalogs: catalogs, data: data, hasPermission: hasPermission, setLoading: setLoading };
};
export var useCatalogMap = function (data) {
    return useMemo(function () {
        var _catalogMap = {};
        if (data) {
            data.forEach(function (i) {
                var knowledge_catalog_id;
                if ('knowledge_attachment' in i) {
                    knowledge_catalog_id = i.knowledge_attachment.knowledge_catalog_id;
                }
                else {
                    knowledge_catalog_id = i.knowledge_catalog_id;
                }
                if (!_catalogMap[knowledge_catalog_id]) {
                    _catalogMap[knowledge_catalog_id] = [];
                }
                _catalogMap[knowledge_catalog_id].push(i);
            });
        }
        return _catalogMap;
    }, [data]);
};
export function hit(id, type) {
    return postJSON('/ts/api/knowledges/hit', { id: id + '', type: type }, {
        baseURL: '/cs'
    });
}
export var RepositoryWidgetContext = React.createContext({
    previewUrl: '',
    isWorkbench: false
});
export var replaceHtmlTag = function (text) {
    var node = document.createElement('div');
    node.innerHTML = text.replace(/<br \/>/g, '\n');
    return node.innerText;
};
export var makeQuestionData = function (item, platform, content, url) {
    var data;
    var qContent = item.content;
    var content_type = item.content_type;
    if (content_type === 'html' || content_type === 'Html') {
        var pic_url = weixinDefaultPng;
        var pic_title = void 0;
        if (qContent.indexOf('<img') > -1) {
            var reg = /<img[^>]+src=['"]([^'"]+)['"]+/gi;
            var reg2 = /<img[^>]+alt=['"]([^'"]+)['"]+/gi;
            pic_url = (reg.exec(qContent) || [])[1];
            pic_title = reg2.exec(qContent);
            if (pic_title && pic_title[1]) {
                pic_title = pic_title[1];
            }
        }
        data = {
            content: qContent,
            title: item.title,
            desc: replaceHtmlTag(qContent),
            pic_title: pic_title || '',
            pic_url: pic_url,
            url: item.isRobotQuestion ? item.url : (url ? url + item.id : ''),
            platform: platform,
            type: 'html',
            isRobotQuestion: item.isRobotQuestion || false
        };
    }
    else {
        var reg = /<[^>]+>/g;
        if (reg.test(qContent)) {
            qContent = replaceHtmlTag(qContent);
        }
        data = {
            content: qContent,
            title: item.title,
            type: 'text'
        };
    }
    return data;
};
/**
 * 机器人问题转换为普通问题格式
 * @param {RobotQuestion} robotQuestion
 * @return Question
 */
export var robotToCommonQuestion = function (robotQuestion) {
    var _a, _b, _c, _d;
    return Object.assign({
        id: robotQuestion.id + '',
        title: robotQuestion.content,
        content: (_b = (_a = robotQuestion === null || robotQuestion === void 0 ? void 0 : robotQuestion.answer) === null || _a === void 0 ? void 0 : _a.content) !== null && _b !== void 0 ? _b : robotQuestion.content,
        content_text: (_d = (_c = robotQuestion === null || robotQuestion === void 0 ? void 0 : robotQuestion.answer) === null || _c === void 0 ? void 0 : _c.content) !== null && _d !== void 0 ? _d : robotQuestion.content,
        content_type: 'html',
        updated_at: robotQuestion.updateTime
    }, makeQuestionData(__assign(__assign({ content_type: 'html' }, robotQuestion), { title: robotQuestion.content, content: robotQuestion.answer ? robotQuestion.answer.content : robotQuestion.content, url: robotQuestion.wechatUrl, isRobotQuestion: true })));
};
export var articleToCommonQuestion = function (article) {
    return Object.assign({
        id: article.id + '',
        title: article.title,
        content: article.contentBody,
        content_text: article.contentBody,
        content_type: 'html',
        updated_at: article.updateTime
    }, makeQuestionData(__assign(__assign({ content_type: 'html' }, article), { content: article.contentBody })));
};
export var useTabSetting = function () {
    var _a = useState(), tabSetting = _a[0], setTabSetting = _a[1];
    var _b = useState(true), loading = _b[0], setLoading = _b[1];
    var url = '/user_profiles/knowledge_tabs';
    var csUrl = '/cs/ts/api/user_profiles/knowledge_tabs';
    var saveTab = useCallback(function (key) {
        if (key === 'km') {
            setTabSetting(function (prevState) {
                var _a, _b;
                post(url, { 'knowledge_tabs[im]': "".concat(key, "_").concat((_a = prevState === null || prevState === void 0 ? void 0 : prevState.activeSpace) !== null && _a !== void 0 ? _a : '', "_").concat((_b = prevState === null || prevState === void 0 ? void 0 : prevState.activeCatalog) !== null && _b !== void 0 ? _b : '') }).then(function () { return null; });
                return ({
                    activeSpace: prevState === null || prevState === void 0 ? void 0 : prevState.activeSpace,
                    activeCatalog: prevState === null || prevState === void 0 ? void 0 : prevState.activeCatalog,
                    activeTab: 'km'
                });
            });
        }
        else {
            setTabSetting(function (prevState) {
                var _a, _b;
                post(url, { 'knowledge_tabs[im]': "".concat(key, "_").concat((_a = prevState === null || prevState === void 0 ? void 0 : prevState.activeScope) !== null && _a !== void 0 ? _a : '') }).then(function () { return null; });
                return (__assign(__assign({}, prevState), { activeScope: (_b = prevState === null || prevState === void 0 ? void 0 : prevState.activeScope) !== null && _b !== void 0 ? _b : 'public', activeTab: key }));
            });
        }
    }, []);
    var saveScope = useCallback(function (scope) {
        setTabSetting(function (prevState) {
            var _a, _b;
            post(url, { 'knowledge_tabs[im]': "".concat((_a = prevState === null || prevState === void 0 ? void 0 : prevState.activeTab) !== null && _a !== void 0 ? _a : 'useful_expression', "_").concat(scope) }).then(function () { return null; });
            return (__assign(__assign({}, prevState), { activeTab: (_b = prevState === null || prevState === void 0 ? void 0 : prevState.activeTab) !== null && _b !== void 0 ? _b : 'useful_expression', activeScope: scope }));
        });
    }, []);
    var saveSpace = useCallback(function (space) {
        setTabSetting(function (prevState) {
            var _a, _b;
            if (typeof space === 'string' && space.indexOf(',') === -1) {
                post(url, { 'knowledge_tabs[im]': "km_".concat(space, "_").concat((_a = prevState === null || prevState === void 0 ? void 0 : prevState.activeCatalog) !== null && _a !== void 0 ? _a : '') }).then(function () { return null; });
            }
            return (__assign(__assign({}, prevState), { activeTab: 'km', activeCatalog: (_b = prevState === null || prevState === void 0 ? void 0 : prevState.activeCatalog) !== null && _b !== void 0 ? _b : -1, activeSpace: space + '' }));
        });
    }, []);
    var saveCatalog = useCallback(function (catalog) {
        setTabSetting(function (prevState) {
            var _a, _b;
            post(url, { 'knowledge_tabs[im]': "km_".concat((_a = prevState === null || prevState === void 0 ? void 0 : prevState.activeSpace) !== null && _a !== void 0 ? _a : '', "_").concat(catalog) }).then(function () { return null; });
            return (__assign(__assign({}, prevState), { activeTab: 'km', activeSpace: (_b = prevState === null || prevState === void 0 ? void 0 : prevState.activeSpace) !== null && _b !== void 0 ? _b : '', activeCatalog: catalog }));
        });
    }, []);
    useEffect(function () {
        csApiGetJSON(csUrl).then(function (res) { return res.data; }).then(function (res) {
            var _a;
            if ((_a = res === null || res === void 0 ? void 0 : res.knowledge_tabs) === null || _a === void 0 ? void 0 : _a.im) {
                if (res === null || res === void 0 ? void 0 : res.knowledge_tabs.im.startsWith('km')) {
                    setTabSetting(function (prevState) { return ({
                        activeTab: 'km',
                        activeScope: prevState === null || prevState === void 0 ? void 0 : prevState.activeScope,
                        activeSpace: res.knowledge_tabs.im.split('_')[1],
                        activeCatalog: parseInt(res.knowledge_tabs.im.split('_')[2]) || -1
                    }); });
                }
                else {
                    setTabSetting(function (prevState) {
                        var matchResult = res === null || res === void 0 ? void 0 : res.knowledge_tabs.im.match(/^(useful_expression|useful_link|useful|link|question|attachment)_(public|robot|personal)?/);
                        var result = {};
                        if (matchResult && matchResult.length > 1) {
                            var key = matchResult[1];
                            if (key === 'useful') {
                                key = 'useful_expression';
                            }
                            else if (key === 'link') {
                                key = 'useful_link';
                            }
                            result.activeTab = key;
                        }
                        if (matchResult && matchResult.length > 2) {
                            result.activeScope = matchResult[2];
                        }
                        return result;
                    });
                }
            }
        }).finally(function () { return setLoading(false); });
    }, []);
    return { tabSetting: tabSetting, saveTab: saveTab, loading: loading, saveScope: saveScope, saveSpace: saveSpace, saveCatalog: saveCatalog };
};
/**
 * 除了 question 以外，其他的标签页如果 scope 是 robot，则返回 public
 * @param {Scope} scope
 * @returns {string | "public" | "personal"}
 */
export var useSafeScope = function (scope) {
    var result = scope;
    if (!result || result === 'robot') {
        result = 'public';
    }
    return result;
};
/**
 * 如果 context 里的 expandedKeys 为 undefined，返回所有的 catalog，即展开所有的节点
 * @param {object} catalogMap
 */
export var useExpandedKeys = function (catalogMap) {
    var expandedKeys = useContext(RepositoryWidgetContext).expandedKeys;
    return useMemo(function () {
        if (!expandedKeys) {
            return Object.keys(catalogMap);
        }
        else {
            return expandedKeys;
        }
    }, [catalogMap, expandedKeys]);
};
function foreachCatalogs(_catalogs, cb, parents) {
    if (parents === void 0) { parents = []; }
    if (!_catalogs) {
        return;
    }
    _catalogs.forEach(function (i) {
        cb === null || cb === void 0 ? void 0 : cb(i, parents);
        foreachCatalogs(i.children, cb, __spreadArray(__spreadArray([], parents, true), [i], false));
    });
}
export var getDefaultExpandedKeys = function (catalogs) {
    var r = [];
    foreachCatalogs(catalogs, function (catalog, parents) {
        r.push(__spreadArray(__spreadArray([], parents.map(function (i) { return i.id; }), true), [catalog.id], false).join('-'));
    });
    return r;
};
export var getExpandedKeysByTags = function (catalogs, tags) {
    var _tagIds = [];
    var _tags = __spreadArray([], tags, true);
    foreachCatalogs(catalogs, function (catalog, parents) {
        if (catalog.name === _tags[0] && isEqual(parents.map(function (i) { return i.id; }), _tagIds)) {
            _tagIds.push(catalog.id);
            _tags.shift();
        }
    });
    return _tagIds.reduce(function (previousValue, currentValue, currentIndex, array) {
        previousValue.push(array.slice(0, currentIndex + 1).join('-'));
        return previousValue;
    }, []);
};
var SELECTION_CACHE = {};
export var useSelectionCache = function (key, _a) {
    if (key === void 0) { key = 'selection_key'; }
    var defaultTab = _a.defaultTab, defaultScope = _a.defaultScope;
    var keyCache = (function () {
        if (!SELECTION_CACHE[key]) {
            SELECTION_CACHE[key] = {};
        }
        return SELECTION_CACHE[key];
    })();
    var _b = useState('useful_expression'), activeTab = _b[0], setActiveTab = _b[1];
    //默认 tab 更新以后
    useEffect(function () {
        if (defaultTab && !keyCache.activeTab) {
            keyCache.activeTab = defaultTab;
            setActiveTab(keyCache.activeTab || 'useful_expression');
        }
    }, [defaultTab, keyCache]);
    useEffect(function () {
        setActiveTab(keyCache.activeTab || 'useful_expression');
        setQuery(keyCache.query || '');
    }, [keyCache]);
    var tabCache = useMemo(function () {
        if (!keyCache[activeTab]) {
            keyCache[activeTab] = {
                public: {},
                personal: {},
                robot: {}
            };
        }
        return keyCache[activeTab];
    }, [keyCache, activeTab]);
    var _c = useState('public'), activeScope = _c[0], setActiveScope = _c[1];
    useEffect(function () {
        if (defaultScope && !tabCache.activeScope) {
            tabCache.activeScope = defaultScope;
            setActiveScope(defaultScope);
        }
    }, [defaultScope, tabCache]);
    useEffect(function () {
        setActiveScope(tabCache.activeScope || 'public');
    }, [tabCache]);
    var scopeCache = useMemo(function () {
        if (!tabCache[activeScope]) {
            tabCache[activeScope] = {};
        }
        return tabCache[activeScope];
    }, [activeScope, tabCache]);
    var _d = useState(''), query = _d[0], setQuery = _d[1];
    var _e = useState(), scrollTop = _e[0], setScrollTop = _e[1];
    useEffect(function () {
        setScrollTop(scopeCache.scrollTop);
    }, [scopeCache]);
    return {
        activeTab: activeTab,
        setActiveTab: useCallback(function (tab) {
            keyCache.activeTab = tab;
            setActiveTab(tab);
        }, [keyCache]),
        query: query,
        setQuery: useCallback(function (keyword) {
            keyCache.query = keyword;
            setQuery(keyword);
        }, [keyCache]),
        activeScope: activeScope,
        setActiveScope: useCallback(function (scope) {
            tabCache.activeScope = scope;
            setActiveScope(scope);
        }, [tabCache]),
        scrollTop: scrollTop,
        setScrollTop: useCallback(function (scrollTop) {
            scopeCache.scrollTop = scrollTop;
            setScrollTop(scrollTop);
        }, [scopeCache])
    };
};
export var fetchKnowledgeManagerRobotSettings = function () {
    return getJSON('/knowledge_manager_settings/robot_settings');
};
export var createUseSelectedCatalog = function () {
    var initialState = {
        'default': {
            public: -1,
            personal: -1
        }
    };
    var reducer = function (state, action) {
        var _a, _b;
        var _c;
        if (action.type === 'SET') {
            if (action.payload.value === ((_c = initialState[action.payload.stashKey]) === null || _c === void 0 ? void 0 : _c[action.payload.scope])) {
                return initialState;
            }
            initialState = __assign(__assign({}, initialState), (_a = {}, _a[action.payload.stashKey] = __assign(__assign({}, initialState[action.payload.stashKey]), (_b = {}, _b[action.payload.scope] = action.payload.value, _b)), _a));
        }
        else if (action.type === 'RESET_DEFAULT') {
            initialState = __assign(__assign({}, initialState), { 'default': { public: -1, personal: -1 } });
        }
        return initialState;
    };
    return function (scope, stashKey) {
        var _a, _b;
        if (stashKey === void 0) { stashKey = 'default'; }
        var _c = useReducer(reducer, initialState), selectedCatalog = _c[0], dispatch = _c[1];
        var setSelectedCatalogRef = useRef(function (v) { return v; });
        useEffect(function () {
            setSelectedCatalogRef.current = function (id) {
                dispatch({
                    type: 'SET',
                    payload: {
                        stashKey: stashKey,
                        scope: scope,
                        value: id
                    }
                });
            };
        }, [scope, stashKey]);
        var isDefaultStashKey = stashKey === 'default';
        useEffect(function () {
            if (isDefaultStashKey) {
                return function () {
                    dispatch({ type: 'RESET_DEFAULT' });
                };
            }
        }, [scope, isDefaultStashKey]);
        return [
            (_b = (_a = selectedCatalog === null || selectedCatalog === void 0 ? void 0 : selectedCatalog[stashKey]) === null || _a === void 0 ? void 0 : _a[scope]) !== null && _b !== void 0 ? _b : -1,
            useCallback(function (id) {
                setSelectedCatalogRef.current(id);
            }, [])
        ];
    };
};
