import find from "lodash-es/find";
import isArray from "lodash-es/isArray";
import isNil from "lodash-es/isNil";
import isString from "lodash-es/isString";
import reduce from "lodash-es/reduce";
import numeral from "numeral";
/**
 * 将自定字段的值转换为文字，例如多选字段'1,2'转换为'选项1,选项2'
 * @param {CustomField} field - 自定义字段
 * @param {string|string[]} value - 值
 */
export function customFieldValueToText(field, value) {
    if (!value) {
        return "";
    }
    var type = field.content_type;
    var valueString = isString(value) ? value : value === null || value === void 0 ? void 0 : value.join(",");
    var valueAry = isArray(value) ? value : value === null || value === void 0 ? void 0 : value.split(",");
    var options = field.select_options;
    switch (type) {
        case "droplist":
        case "checkbox":
        case "radio":
            return options
                .filter(function (i) { return valueAry.includes("".concat(i.value)); })
                .map(function (i) { return i.title; })
                .join(",");
        case "chained_droplist": {
            if (!valueString) {
                return "";
            }
            var resultAry_1 = [];
            reduce(valueAry, function (preValue, item) {
                var _a;
                var option = find(preValue, ["value", item]);
                if (option) {
                    resultAry_1.push(option.title);
                }
                return (_a = option === null || option === void 0 ? void 0 : option.subs) !== null && _a !== void 0 ? _a : [];
            }, options);
            return resultAry_1.join("/");
        }
        case "calculate":
            switch (field.calculate_result_type) {
                case "duration":
                    return isNil(valueString)
                        ? ""
                        : numeral(valueString).format("00:00:00");
                case "percentage":
                    return isNil(valueString) ? "" : numeral(valueString).format("0.00%");
                case "numeric":
                    return isNil(valueString) ? "" : Number(valueString);
                default:
                    return valueString;
            }
        default:
            return valueString;
    }
}
