import { saveCustomer } from "./customer-utils";
interface ITicketFieldHookResponse {
    code?: number;
    error?: string;
    ticket?: {
        subject?: string;
        content?: string;
        customer_id?: number;
        priority_id?: number;
        status_id?: number;
        tags?: string;
        agent_group_id?: number;
        assignee_id?: number;
        [customFieldName: string]: any;
    };
}
interface ICustomerFieldHookResponse {
    code?: number;
    error?: string;
    customer?: {
        nick_name?: string;
        cellphones_attributes?: string[];
        email?: string;
        other_emails?: string[];
        description?: string;
        organization_id?: string;
        tags?: string;
        lang?: string;
        owner_group_id?: number;
        owner_id?: number;
        level?: "vip" | "normal";
        [customFieldName: string]: any;
    };
}
export declare function isSelectFieldByFieldName(fieldName: string): boolean;
export declare function executeImFieldHook(fieldHook: IFieldHook, chat: IMineChat, customer: Customer, options: {
    customerCustomFields: CustomField[];
} & Parameters<typeof saveCustomer>[1]): Promise<any>;
/**
 * 创建请求field hook的参数
 * @param {object}  fieldHook
 * @param {string}  fieldHook.url - field hook的url
 * @param {array}   fieldHook.target_fields - 要传给field hook的字段
 * @param {key}     fieldHook.key - 用于field hook验证
 * @param {object}  targetObject - 客户或者工单
 * @param {object}  otherParams - 其他的参数，呼叫中心弹屏时是电话号码，im进线是微信号、微博号等等
 * @param {string}  otherParams.phoneNumber - 电话号码，field hook的type是cc时有效
 * @param {string}  otherParams.web_token
 * @param {string}  otherParams.weixin_openid
 * @param {string}  otherParams.weibo_openid
 * @param {string}  otherParams.sdk_token
 * @param {string}  otherParams.im_sub_session_id
 * @return {object}
 */
export declare function buildHookRequestParamsForCustomer(fieldHook: IFieldHook, targetObject: Customer, options: {
    customerCustomFields: CustomField[];
    otherParams: {
        [key: string]: any;
    };
}): {
    [x: string]: {
        [key: string]: any;
    };
} & {
    [key: string]: any;
};
/**
 * field hook返回的数据无法直接传到后端修改,需要做一步转换
 * 1、如果没有custom_fields字段，则加一个空的custom_fields字段
 * 2、把cellphones_attributes的元素用content包装一下
 * 3、把自定义字段全部移动到custom_fields属性里
 * @param {object} changedData - field hook返回的客户参数
 */
export declare function convertFieldHookResponse(changedData: ICustomerFieldHookResponse["customer"] | ITicketFieldHookResponse["ticket"]): Customer;
export {};
