import { getJSON, postJSON, putJSON } from './ajax';
import { message } from 'udesk-ui';
var getToken = function (fileName, marking) {
    fileName = fileName || 'image.jpg';
    marking = marking || 'web.cc.qiniu';
    return new Promise(function (resolve, reject) {
        getJSON('/file_upload_policy', {
            marking: marking,
            key: fileName
        }).then(function (res) {
            if (res.code === 1000) {
                resolve(res);
            }
            else {
                message.error('上传时获取token失败');
                reject(res);
            }
        });
    });
};
var sendImgByBase64 = function (cfg) {
    cfg = cfg || {};
    var config = cfg.config || {};
    switch (config.storage_policy) {
        case 'ali':
            sendAliBase64(cfg);
            break;
        case 'minio':
            sendMinioBase64(cfg);
            break;
    }
};
function getExt(urlData) {
    var arr = urlData.split(',');
    var mime = arr[0].match(/:(.*?);/)[1];
    return mime.replace("image/", "");
}
export function dataUrlToFile(dataUrl, filename) {
    var _a;
    var arr = dataUrl.split(',');
    var mime = (_a = (arr[0] || '').match(/:(.*?);/)) === null || _a === void 0 ? void 0 : _a[1];
    var bytes = window.atob(arr[1]);
    var ab = new ArrayBuffer(bytes.length);
    var u8arr = new Uint8Array(ab);
    for (var i = 0; i < bytes.length; i++) {
        u8arr[i] = bytes.charCodeAt(i);
    }
    return new File([u8arr], filename, { type: mime });
}
function addToken(fileObj, cfg) {
    var config = cfg.config || {};
    var formObj = new FormData();
    formObj.append('name', fileObj.name);
    formObj.append('type', fileObj.type);
    formObj.append('lastModifiedDate', fileObj.lastModifiedDate);
    formObj.append('size', fileObj.size);
    formObj.append('policy', config.policy);
    formObj.append('OSSAccessKeyId', config.accessid);
    formObj.append('Signature', config.signature);
    formObj.append('key', cfg.key);
    formObj.append('file', fileObj);
    return formObj;
}
function sendAliBase64(cfg) {
    var config = cfg.config || {};
    var ext = getExt(cfg._data);
    var imageName = 'image_' + (+new Date()) + '_' + Math.random().toString(32).slice(-5) + '.' + ext;
    cfg.random_filename = imageName;
    cfg.key = config.dir + imageName;
    cfg.url = "".concat(config.host, "/").concat(cfg.key);
    var fileObj = dataUrlToFile(cfg._data, imageName);
    var formDateObj = addToken(fileObj, cfg);
    postJSON(config.host, formDateObj).then(function (res) {
        if (res.status === 204) {
            cfg.success && cfg.success({ src: cfg.url });
        }
        else {
            cfg.error && cfg.error();
            !cfg.noTip && message.error("上传失败，请重新上传");
        }
    });
}
function sendMinioBase64(cfg) {
    var config = cfg.config || {};
    var fields = config.fields || {};
    putJSON(config.host, cfg.data, {
        headers: {
            "Content-Type": "image/jpg",
            'X-Udesk-Minio-policy': fields.policy,
            'X-Udesk-Minio-x-amz-algorithm': fields['x-amz-algorithm'],
            'X-Udesk-Minio-x-amz-credential': fields['x-amz-credential'],
            'X-Udesk-Minio-x-amz-date': fields['x-amz-date'],
            'X-Udesk-Minio-x-amz-signature': fields['x-amz-signature'],
            'X-Udesk-Minio-key': fields.key,
            'X-Amz-Meta-Content-Encoding': 'base64,='
        }
    }).then(function (res) {
        if (res.code === 1000 || res.code === '1000') {
            cfg.success && cfg.success({ src: config.host + '/' + config.file_path });
        }
        else {
            cfg.error && cfg.error();
            !cfg.noTip && message.error("上传失败，请重新上传");
        }
    });
}
export default {
    getToken: getToken,
    sendImgByBase64: sendImgByBase64,
    dataUrlToFile: dataUrlToFile
};
