import { __assign, __awaiter, __generator, __rest } from "tslib";
import isFunction from 'lodash-es/isFunction';
import assign from 'lodash-es/assign';
import includes from 'lodash-es/includes';
// import i18n from 'src/i18n';
import isUndefined from 'lodash-es/isUndefined';
import isNull from 'lodash-es/isNull';
// import * as imLogUtils from '@udesk-webapp/im-log/es/utilities';
import Compressor from 'compressorjs';
import { IM_CONTACT_TAB_NAME, IM_CREATE_TICKET_TAB_NAME, IM_CUSTOM_PLUGIN_TAB_NAME, IM_CUSTOMER_TAB_NAME, IM_REPOSITORY_TAB_NAME, IM_RETURN_VISIT_TAB_NAME, } from './global-const';
import { getJSON } from './ajax';
var prefix = 'UDESK_WEBAPP_';
export function getId() {
    return Date.parse(new Date().toString()) + Math.floor(Math.random() * 1000000);
}
// 动态获取字数长度限制的警告信息
export function r(template) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    for (var i = 0; i < args.length; i++) {
        template = template.replace(/{[^}]+}/, args[i]);
    }
    return template;
}
export function getFileTypeByContentType(contentType) {
    switch (contentType) {
        case 'link':
        case 'datetime':
        case 'text':
        case 'date':
        case 'time':
        case 'numeric':
        case 'calculate':
        case 'area_text':
        case 'number':
        case 'location':
            return 'TextField';
        case 'radio':
        case 'checkbox':
        case 'radio_droplist':
        case 'checkbox_droplist':
        case 'chained_droplist':
        case 'tag_tree':
        default:
            return 'SelectField';
    }
}
function textValidFn(is_required, _value, field) {
    if (_value === void 0) { _value = ''; }
    var validTexts = [];
    var setting = field.setting || {};
    // 检查必填
    if (is_required && !_value) {
        validTexts.push('该字段为必填项');
    }
    // 长度校验
    if (setting.max_length && _value.length > setting.max_length) {
        validTexts.push("\u8D85\u8FC7\u6700\u5927\u957F\u5EA6\u9650\u5236(".concat(setting.max_length, ")"));
    }
    // if (_value.length > 10) {
    //     validTexts.push(`超过最大长度限制(${10})`);
    // }
    // 正则校验
    if (setting.valid_reg) {
        var reg = new RegExp(setting.valid_reg, 'ig');
        if (!reg.test(_value)) {
            validTexts.push('内容格式校验失败');
        }
    }
    if (validTexts.length > 0) {
        var errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    }
    else {
        return Promise.resolve();
    }
}
function textareaValidFn(is_required, _value, field) {
    if (_value === void 0) { _value = ''; }
    var validTexts = [];
    var setting = field.setting || {};
    // 检查必填
    if (is_required && !_value) {
        validTexts.push('该字段为必填项');
    }
    // 长度校验
    if (setting.max_length && _value.length > setting.max_length) {
        validTexts.push("\u8D85\u8FC7\u6700\u5927\u957F\u5EA6\u9650\u5236(".concat(setting.max_length, ")"));
    }
    if (validTexts.length > 0) {
        var errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    }
    else {
        return Promise.resolve();
    }
}
function linkValidFn(is_required, _value, field) {
    if (_value === void 0) { _value = ''; }
    var validTexts = [];
    // 检查必填
    if (is_required && !_value) {
        validTexts.push('该字段为必填项');
    }
    if (validTexts.length > 0) {
        var errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    }
    else {
        return Promise.resolve();
    }
}
function numberValidFn(is_required, _value, field) {
    if (_value === void 0) { _value = ''; }
    var validTexts = [];
    // 检查必填
    if (is_required && !_value) {
        validTexts.push('该字段为必填项');
    }
    if (validTexts.length > 0) {
        var errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    }
    else {
        return Promise.resolve();
    }
}
function commonValidFn(rule, _value, field) {
    if (_value === void 0) { _value = ''; }
    var validTexts = [];
    // 检查必填
    if (field.isRequired && (!_value || !_value.length)) {
        validTexts.push('该字段为必填项');
    }
    if (validTexts.length > 0) {
        var errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    }
    else {
        return Promise.resolve();
    }
}
/**
 * @description 加入了级联字段选项隐藏逻辑
 * @param rule
 * @param _value
 * @param field
 * @returns
 */
function chainedValidFn(rule, _value, field) {
    if (_value === void 0) { _value = []; }
    var validTexts = [];
    // 检查必填
    if (field.is_required && (!_value || !_value.length)) {
        validTexts.push('该字段为必填项');
    }
    /*
     *如果开启了选择最后一级的开关
     *如果填过这个字段
     *并且此字段是可编辑的
     *进行是否填写到最后一级的判断
     */
    if (typeof _value === 'string') {
        _value = _value.split(',');
    }
    if (_value.length && field.full_select && !field.disabled) {
        /*
         * 最后一级的数据
         * 根据选择的数据（数组），循环查找当前对应的已经选择的数据，将该层级数据的subs赋值给lastOption
         * 直到循环完毕，lastOption为下一次应该选择的数据，如果没有数据，说明最后一级已经选完了。
         */
        var arrValue = _value;
        if (typeof _value === 'string') {
            arrValue = _value.split(',');
        }
        var lastOption_1 = field.options;
        var isHideOptionMode_1 = false;
        arrValue === null || arrValue === void 0 ? void 0 : arrValue.forEach(function (item) {
            lastOption_1 === null || lastOption_1 === void 0 ? void 0 : lastOption_1.forEach(function (_item) {
                if (_item.value === item) {
                    lastOption_1 = _item.subs || _item.children || null;
                    item.is_hide && (isHideOptionMode_1 = true);
                }
            });
        });
        if (lastOption_1 && !isHideOptionMode_1) {
            if (Array.isArray(lastOption_1) && lastOption_1.length) {
                lastOption_1 = lastOption_1.filter(function (i) { return !i.is_hide; });
            }
            lastOption_1 && validTexts.push('必须选择最末级');
        }
    }
    if (validTexts.length > 0) {
        var errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    }
    else {
        return Promise.resolve();
    }
}
var sysFields = [
    'name',
    'nick_name',
    'email',
    'description',
    'cellphone',
    'organizations',
    'organization',
    'tags',
    'cellphones',
    'domains',
    'org_email_suffix',
    'vip_level',
];
export function isSysField(type) {
    return sysFields.includes(type);
}
// 遍历级联字段
export function mapCascader(options) {
    var result = [];
    if (!options) {
        return result;
    }
    options.forEach(function (item) {
        var obj = {
            label: item.title,
            value: item.value,
            is_hide: item.is_hide,
        };
        if (item.subs && item.subs.length > 0) {
            obj['children'] = mapCascader(item.subs);
        }
        result.push(obj);
    });
    return result;
}
export function handleCascaderHideOption(option) {
    if (option.is_hide) {
        return;
    }
    var newOption = {
        label: option.title,
        value: option.value,
        is_hide: option.is_hide,
    };
    if (Array.isArray(option.subs) && option.subs.length) {
        var subsFormat_1 = [];
        option.subs.forEach(function (item) {
            var itemFormat = handleCascaderHideOption(item);
            itemFormat && subsFormat_1.push(itemFormat);
        });
        newOption.children = subsFormat_1.length ? subsFormat_1 : null;
    }
    return newOption;
}
/**
 * @description 将级联字段隐藏的选项删除，需要返显的场景，动态添加已选择的隐藏选项
 * @param options
 * @returns
 */
export function createCacaderOptions(options) {
    var newOptions = [];
    if (Array.isArray(options) && options.length) {
        options.forEach(function (item) {
            var filterOption = handleCascaderHideOption(item);
            filterOption && newOptions.push(filterOption);
        });
    }
    return newOptions;
}
/**
 * @param {string} fieldType  那种字段?   customer | ticket
 * @param {string} editType   什么操作?   edit | new | show
 * @param {object} fieldsAuthority    字段全选hash表
 */
export function formatCustomField(_a) {
    var fields = _a.fields, fieldsAuthority = _a.fieldsAuthority, _b = _a.trigger, trigger = _b === void 0 ? 'onChange' : _b, fieldType = _a.fieldType, editType = _a.editType;
    if (!Array.isArray(fields)) {
        return [fields];
    }
    return fields
        .map(function (_a) {
        var _b;
        var sensitive_mark_placeholder = _a.sensitive_mark_placeholder, sensitive_mark_position = _a.sensitive_mark_position, sensitive_mark_size = _a.sensitive_mark_size, sensitive_mode = _a.sensitive_mode, field = __rest(_a, ["sensitive_mark_placeholder", "sensitive_mark_position", "sensitive_mark_size", "sensitive_mode"]);
        if (includes(sysFields, field.name)) {
            var authKey_1 = (_b = fieldsAuthority === null || fieldsAuthority === void 0 ? void 0 : fieldsAuthority[field.name === 'name' ? 'nick_name' : field.name]) === null || _b === void 0 ? void 0 : _b[editType];
            return __assign(__assign({}, field), { trigger: trigger, field: {
                    disabled: authKey_1 === 'readable' || authKey_1 === 'sensitive',
                    isRequired: authKey_1 === 'must_fill_in',
                    authority: authKey_1,
                    sensitive_mark_placeholder: sensitive_mark_placeholder,
                    sensitive_mark_position: sensitive_mark_position,
                    sensitive_mark_size: sensitive_mark_size,
                    sensitive_mode: sensitive_mode,
                } });
        }
        // authKey 可能的值: editable 可编辑 readable 只读 invisible 不可见 must_fill_in 必填
        var authKey = '';
        // 处理字段权限相关
        if (fieldType && editType && fieldsAuthority) {
            switch (fieldType) {
                case 'customer':
                case 'ticket':
                    authKey = fieldsAuthority[field.name]
                        ? fieldsAuthority[field.name][editType] || ''
                        : '';
                    break;
            }
        }
        // 字段不可见时返回空
        if (authKey === 'invisible') {
            return '';
        }
        var setting = field.setting || {};
        var tooltipObj = setting.tooltipObj || {};
        var explainObj = setting.explain || {};
        var fieldItem = {
            label: field.title,
            layout: field.layout || '',
            tooltip: field.tooltip || '',
            tooltipShow: !!(tooltipObj.show && tooltipObj.content),
            lines: setting.lines || '',
            isRequired: field.is_required || false,
            // isRequired: authKey === 'must_fill_in' || field.is_required || false,
            type: field.content_type,
            showLabel: field.showLabel,
            field: {
                trigger: trigger,
                id: field.name,
                type: field.content_type,
                remark: field.comment || '',
                explain: explainObj.content,
                explainShow: !!(explainObj.show || explainObj.content),
                disabled: authKey === 'readable' || authKey === 'sensitive',
                isRequired: authKey === 'must_fill_in',
                options: [],
                rowsLine: 4,
                authority: authKey,
                sensitive_mark_placeholder: sensitive_mark_placeholder,
                sensitive_mark_position: sensitive_mark_position,
                sensitive_mark_size: sensitive_mark_size,
                sensitive_mode: sensitive_mode,
                tooltip: field.tooltip,
            },
        };
        var cascaderOption;
        var cascaderOptionMeta;
        switch (field.content_type) {
            case 'text':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { 
                        // defaultValue: field.default_value || '',
                        placeholder: field.palceholder || '', maxLength: field.max_length || null, validReg: field.valid_reg || null, customValidFn: isFunction(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return textValidFn(fieldItem.field.isRequired, value, field);
                            } }),
                });
                break;
            case 'area_text':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { type: 'textarea', placeholder: field.palceholder || '', 
                        // defaultValue: field.default_value || '',
                        maxLength: field.max_length || null, customValidFn: isFunction(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return textareaValidFn(fieldItem.field.isRequired, value, field);
                            } }),
                });
                break;
            case 'link':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { 
                        // defaultValue: field.default_value || '',
                        placeholder: field.palceholder || '', regLink: true, customValidFn: isFunction(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return linkValidFn(fieldItem.field.isRequired, value, field);
                            } }),
                });
                break;
            case 'date':
            case 'time':
            case 'datetime':
            case 'location':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { customValidFn: isFunction(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return commonValidFn(rule, value, fieldItem.field);
                            } }),
                });
                break;
            case 'calculate':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { customValidFn: isFunction(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return commonValidFn(rule, value, fieldItem.field);
                            } }),
                });
                break;
            case 'number':
            case 'numeric':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { 
                        // defaultValue: field.default_value || '',
                        placeholder: field.palceholder || '', integerDigits: field.integer_digits || '', decimalDigits: field.decimal_digits || '', addonBefore: field.addon_before || '', addonAfter: field.addon_after || '', prefix: field.prefix || '', suffix: field.suffix || '', customValidFn: isFunction(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return numberValidFn(fieldItem.field.isRequired, value, field);
                            } }),
                });
                break;
            case 'droplist':
            case 'droplist_multi':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { trigger: 'onChange', options: field.select_options.map(function (option) {
                            return {
                                icon: option.icon || '',
                                title: option.title,
                                bgColor: option.bg_clor || '',
                                default: option.default || false,
                                value: option.value,
                                is_hide: option.is_hide,
                            };
                        }), customValidFn: isFunction(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return commonValidFn(rule, value, fieldItem.field);
                            } }),
                });
                break;
            case 'radio':
            case 'checkbox':
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { arrangement: field.arrangement || 'auto', trigger: 'onChange', options: field.select_options.map(function (option) {
                            return {
                                icon: option.icon || '',
                                title: option.title,
                                bgColor: option.bg_clor || '',
                                default: option.default || false,
                                value: option.value,
                                is_hide: option.is_hide,
                            };
                        }), customValidFn: isFunction(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                return commonValidFn(rule, value, fieldItem.field);
                            } }),
                });
                break;
            case 'chained_droplist':
                cascaderOption = createCacaderOptions(field.select_options);
                // 做个降级处理，调用此方法的场景不都有select_options_meta
                cascaderOptionMeta = mapCascader(field.select_options_meta || field.select_options);
                fieldItem = Object.assign(fieldItem, {
                    field: __assign(__assign({}, fieldItem.field), { type: 'cascade', trigger: 'onChange', arrangement: field.arrangement || 'auto', viewType: field.view_type, options: cascaderOption, optionsMeta: cascaderOptionMeta, customValidFn: isFunction(field.customValidFn)
                            ? field.customValidFn
                            : function (rule, value) {
                                var _a;
                                return chainedValidFn(rule, value, Object.assign(field, {
                                    disabled: (_a = fieldItem === null || fieldItem === void 0 ? void 0 : fieldItem.field) === null || _a === void 0 ? void 0 : _a.disabled,
                                    options: cascaderOption,
                                    optionsMeta: cascaderOptionMeta,
                                }));
                            } }),
                });
                break;
            case 'staff':
                fieldItem = __assign(__assign({}, fieldItem), { field: __assign(__assign({}, fieldItem.field), { trigger: 'onChange' }) });
        }
        fieldItem = Object.assign(fieldItem, isFunction(field.convertProps) ? field.convertProps(field) : {});
        return fieldItem;
    })
        .filter(function (i) { return i; });
}
export function getFieldsDefaultValue(_a) {
    var _b = _a.fields, fields = _b === void 0 ? [] : _b, _c = _a.values, values = _c === void 0 ? {} : _c;
    var defaultValues = {};
    fields.forEach(function (field) {
        var setting = field.setting || {};
        var selectOption = field.select_options || [];
        var selectOptionDefault = [];
        switch (field.content_type) {
            case 'text':
            case 'area_text':
            case 'link':
            case 'date':
            case 'time':
            case 'datetime':
            case 'location':
            case 'calculate':
            case 'number':
            case 'numeric':
                if (setting.default_value) {
                    defaultValues[field.name] = setting.default_value;
                }
                else {
                    defaultValues[field.name] = '';
                }
                break;
            case 'droplist':
            case 'droplist_multi':
            case 'radio':
            case 'checkbox':
                selectOptionDefault = selectOption.filter(function (option) {
                    return option.default;
                });
                if (selectOptionDefault && selectOptionDefault.length > 0) {
                    defaultValues[field.name] = selectOptionDefault.map(function (option) { return option.value; });
                }
                else {
                    defaultValues[field.name] = '';
                }
                break;
            case 'chained_droplist':
                defaultValues[field.name] = '';
                break;
        }
    });
    return Object.assign({}, defaultValues, values);
}
export function formatProps(props) {
    var pageProps = {};
    var page = props.page || 'im';
    var commonProps = {
        company: props.company,
        smsapp: props.smsapp,
        agents: props.agents || [],
        isAdmin: props.isAdmin,
        page: props.page,
        agentGroups: props.agentGroups,
        ticketTab: props.ticketTab,
        customUiParams: props.customUiParams,
        ticketTemplates: props.ticketTemplates || [],
        kmrobot_url: props.kmrobot_url,
        previewUrl: props.previewUrl,
        noteTemplates: props.noteTemplates || [],
        user: props.user,
        admin: props.admin,
        roleCan: props.roleCan,
        noteFields: props.noteFields || [],
        getTencentMap: props.getTencentMap,
        imAgentNoteTemplatesSort: props.im_agent_note_templates_sort || [],
        getCompanyCategoryConfig: props.getCompanyCategoryConfig,
        wfmServer: props.wfmServer,
        companyConfig: props.companyConfig,
        platforms: props.platforms || [],
        mchatConfig: props.mchatConfig,
        features: props.features || [
            IM_RETURN_VISIT_TAB_NAME,
            IM_CUSTOMER_TAB_NAME,
            IM_CONTACT_TAB_NAME,
            IM_REPOSITORY_TAB_NAME,
            IM_CREATE_TICKET_TAB_NAME,
            IM_CUSTOM_PLUGIN_TAB_NAME,
        ],
        getTicketPanel: props.getTicketPanel,
        getCustomerPanel: props.getCustomerPanel,
        tinymceOptions: props.tinymceOptions,
        host: props.host,
    };
    switch (page) {
        case 'im':
            pageProps = {
                // cellphones:props.chat?.cellphones,
                imNewPlatforms: props.imNewPlatforms,
                event: props.event || {},
                companyCategoryConfig: props.companyCategoryConfig,
                fieldHooks: props.fieldHooks,
            };
            break;
        case 'imc':
            break;
    }
    return assign(commonProps, pageProps);
}
export function stringToHex(str) {
    var hexResult = [];
    for (var i = 0, len = str.length; i < len; i++) {
        var hex = str.charCodeAt(i).toString(16);
        hexResult.push(hex);
    }
    return hexResult.join('');
}
// 需要
export function getPreviewUrl(url, options) {
    if (!url) {
        return '';
    }
    var _a = options || {}, _b = _a.needEncode, needEncode = _b === void 0 ? false : _b, previewUrl = _a.previewUrl;
    if (needEncode) {
        url = encodeURIComponent(decodeURIComponent(url));
        url = url.replace(/%2F/g, '/');
    }
    var hexUrl;
    try {
        hexUrl = stringToHex(url);
    }
    catch (e) {
        console.error('preview-url-helper', e);
        return url;
    }
    return "".concat(previewUrl, "/wopi/files/").concat(hexUrl, "/documents");
}
// 需要
/**
 * 通过文件名判断是否可以预览
 * @param {string} fileName
 * @returns {boolean}
 */
export function canPreviewByFileName(fileName) {
    return /\.(doc|docx|odt|rtf|wps|ppt|pptx|odp|dp|xls|xlsx|ods|csv|et|pdf)$/.test(fileName);
}
export function createMedia(type, cfg) {
    if (!cfg && !cfg.src)
        return;
    type = type || 'audio';
    var el = document.createElement(type);
    el.src = cfg.src;
    el.onended = function () {
        // el.playing = false;
        cfg.onended && cfg.onended();
    };
    el.onloadedmetadata = function () {
        cfg.onloaded && cfg.onloaded(Math.floor(el.duration || 0));
    };
    return {
        el: el,
        play: function () {
            el.play();
            el.playing = true;
        },
        pause: function () {
            el.pause();
            el.playing = false;
        },
        replay: function () {
            el.currentTime = 0;
            el.play();
            el.playing = true;
        },
        destroy: function () {
            el.removeAttribute('src');
        },
    };
}
export function createAudio(cfg) {
    return createMedia('audio', cfg);
}
export function replaceLinks(text) {
    return (text + '').replace(/((http|https):\/\/([\w-]+\.)+[\w-:]+(\/[\w-./?;%&=+#：:]*)?)/g, '<a href="$1" target="_blank">$1</a>');
}
export function removeStorage(name, type) {
    if (type === void 0) { type = 'session'; }
    var _name = prefix + name;
    switch (type) {
        case 'session':
            window.sessionStorage.removeItem(_name);
            break;
        case 'local':
            window.localStorage.removeItem(_name);
            break;
    }
}
export function getStorage(name, type, json) {
    if (type === void 0) { type = 'session'; }
    if (json === void 0) { json = false; }
    //json 数组或json
    var _name = prefix + name;
    var key;
    var storage;
    var data;
    switch (type) {
        case 'session':
        case 'local':
            key = type + 'Storage';
            storage = window[key];
            data = storage.getItem(_name);
            if (json && data) {
                try {
                    data = JSON.parse(data);
                }
                catch (e) {
                    console.error('getData error!');
                }
            }
            return data;
        default:
            return window.sessionStorage[_name] || window.localStorage[_name];
    }
}
export function setStorage(name, val, type, json) {
    if (type === void 0) { type = 'session'; }
    if (json === void 0) { json = false; }
    var cacheKey = prefix + name;
    if (type + 'Storage' in window) {
        if (isUndefined(val) || isNull(val)) {
            window[type + 'Storage'].removeItem(cacheKey);
        }
        else {
            if (json) {
                try {
                    val = JSON.stringify(val);
                }
                catch (e) {
                    console.error('setStorage error!');
                }
            }
            window[type + 'Storage'][cacheKey] = val;
        }
    }
}
export function getByteByStr(val) {
    var _byte = 0;
    if (typeof val === 'string') {
        for (var i = 0, l = val.length; i < l; i++) {
            // eslint-disable-next-line no-control-regex
            _byte += /[^\x00-\xff]/gi.test(val[i]) ? 2 : 1;
        }
    }
    return _byte;
}
// export function htmlReplace(content: string, tagNames: string) {
//     if (!content) {
//         return '';
//     }
//     // interface IRplTxtObj {
//     //     a: string;
//     //     img: string;
//     // }
//     tagNames.split(',').forEach((name: string) => {
//         let rpl_txt_obj: any = {
//                 a: i18n.t('link'),
//                 img: i18n.t('img'),
//             },
//             reg = new RegExp(`(<${name}.+</${name}>)|(<${name}[^>]+>)`, 'g');
//         content = (content + '').replace(reg, `【${rpl_txt_obj[name]}】`);
//     });
//     return content;
// }
export function compareStr(val1, val2, flag) {
    //'3.3.5' > '3.3.4' is true
    var arr1 = ('' + val1).split('.'), arr2 = ('' + val2).split('.'), newVal1 = '', newVal2 = '';
    var max = Math.max(arr1.length, arr2.length);
    var toFixed = function (str) {
        var newStr = str;
        for (var i = str.length; i <= 3; i++) {
            newStr = '0' + newStr;
        }
        return newStr;
    };
    var supplement = function (arr) {
        for (var i = arr.length; i < max; i++) {
            arr.push('0');
        }
        return arr;
    };
    arr1 = supplement(arr1);
    arr2 = supplement(arr2);
    newVal1 = arr1
        .map(function (val) {
        return toFixed(val);
    })
        .join('.');
    newVal2 = arr2
        .map(function (val) {
        return toFixed(val);
    })
        .join('.');
    switch (flag) {
        case '>':
            return newVal1 > newVal2;
        case '<':
            return newVal1 < newVal2;
        default:
            //">="
            return newVal1 >= newVal2;
    }
}
export function humanizeDuration(seconds) {
    if (!seconds)
        return '0s';
    var result = seconds + 's';
    if (seconds > 24 * 60 * 60) {
        result = seconds / (24 * 60 * 60) + 'd';
    }
    else if (seconds > 60 * 60) {
        result = seconds / (60 * 60) + 'h';
    }
    else if (seconds > 60) {
        result = seconds / 60 + 'm';
    }
    return result.replace(/^(\d+)(\.\d)?\d*(\w)$/, '$1$2$3');
}
export function randomString(wordCount) {
    if (!wordCount) {
        wordCount = Math.ceil(Math.random() * 10);
    }
    var words = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_';
    var res = '';
    for (var i = 0; i < wordCount; i++) {
        res = res + words.charAt(Math.floor(Math.random() * words.length));
    }
    return res;
}
var _id = 0;
export function udeskId() {
    return 'udesk_' + _id + 1;
}
export function guid(prefix, suffix) {
    return udeskId() + (prefix || '') + +new Date() + (suffix ? '*' + suffix : '');
}
export function css2obj(css) {
    if (!css) {
        return {};
    }
    return css
        .split(';')
        .map(function (i) { return i.trim(); })
        .filter(function (i) { return i && i.indexOf(':') > -1; })
        .map(function (i) { return i.split(':'); })
        .filter(function (i) { return i[0].trim() && i[1].trim(); })
        .map(function (i) {
        var _a;
        return (_a = {}, _a[i[0]] = i[1], _a);
    })
        .reduce(function (p, i) { return (__assign(__assign({}, p), i)); }, {});
}
export function checkFileIsImage(imageFile) {
    return new Promise(function (resolve) {
        var url = window.URL || window.webkitURL;
        var image = new Image();
        image.onload = function () {
            resolve(true);
        };
        image.onerror = function () {
            resolve(false);
        };
        image.src = url.createObjectURL(imageFile);
    });
}
export function compressImage(imageFile, _a) {
    var success = _a.success, error = _a.error, options = __rest(_a, ["success", "error"]);
    return __awaiter(this, void 0, void 0, function () {
        var isImageFile;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, checkFileIsImage(imageFile)];
                case 1:
                    isImageFile = _b.sent();
                    if (!isImageFile) {
                        return [2 /*return*/, Promise.reject('file is not image')];
                    }
                    return [2 /*return*/, new Promise(function (resolve, reject) {
                            new Compressor(imageFile, __assign(__assign({}, options), { success: function (result) {
                                    var mimeTypeExtensionMap = {
                                        'image/jpeg': '.jpeg',
                                        'image/png': '.png',
                                    };
                                    var fileName = imageFile.name;
                                    if (options.mimeType) {
                                        fileName = imageFile.name.replace(/\.[^.]+?$/, mimeTypeExtensionMap[options.mimeType]);
                                    }
                                    var file = new File([result], fileName);
                                    if ('uid' in imageFile) {
                                        file.uid = imageFile.uid;
                                    }
                                    try {
                                        success === null || success === void 0 ? void 0 : success(file);
                                    }
                                    finally {
                                        resolve(file);
                                    }
                                }, error: function (err) {
                                    try {
                                        error === null || error === void 0 ? void 0 : error(err);
                                    }
                                    finally {
                                        reject(err);
                                    }
                                } }));
                        })];
            }
        });
    });
}
/**
 + *@method 获取对象对应key的value值
 + */
export var getAssociativeObjValue = function (obj, key) { return obj[key]; };
/**
 * 相当于当前页面的心跳，告诉后端我打开了客服系统
 */
export var syncSysStatus = function (current_page) {
    if (current_page === void 0) { current_page = window.location.href; }
    var params = {
        time: +new Date(),
        current_page: current_page,
    };
    getJSON('/agents/sys_status', params)
        .catch(function (res) {
        console.log('get /agent/sys_status failed');
    })
        .finally(function () {
        setTimeout(function () {
            syncSysStatus(current_page);
        }, 10 * 60 * 1000);
    });
};
