const {utils} = require('umi');

module.exports = function(api) {
  api.describe({
    config: {
      schema(Joi) {
        return Joi.object({
          implementation: Joi.any(),
          sassOptions: Joi.object(),
          additionalData: Joi.alternatives(Joi.string(), Joi.func()),
          sourceMap: Joi.boolean(),
          webpackImporter: Joi.boolean(),
          warnRuleAsWarning: Joi.boolean(),
        });
      },
    },
  });
  api.chainWebpack((memo, { createCSSRule }) => {
    createCSSRule({
      lang: 'sass',
      type: 'csr',
      test: /(?<!\.module)\.(sass|scss)(\?.*)?$/,
      loader: 'sass-loader',
      options: utils.deepmerge(
        {
          implementation: require('sass'),
        },
        api.config.sass || {},
      )
    });
    memo.module.rule('sass').oneOfs.delete('css-modules');
    createCSSRule({
      lang: 'sass-module',
      type: 'csr',
      test: /\.module\.(sass|scss)(\?.*)?$/,
      loader: 'sass-loader',
      options: utils.deepmerge(
        {
          implementation: require('sass'),
        },
        api.config.sass || {},
      )
    });
    memo.module.rule('sass-module').oneOfs.delete('css');
    memo.module.rule('sass-module').oneOf('css-modules').resourceQuery(undefined);

    return memo;
  });
}
