import { __assign } from "tslib";
import React, { useMemo, useState } from 'react';
import { Table } from 'udesk-ui';
import { useTranslation } from '@udesk-webapp/locale';
// import { useIntl } from 'umi';
import { 
// @ts-ignore
systemField, getCustomFieldValue, sensitiveField, removeHtmlErrorAttr, replaceScriptCode, RemoveHtmlTag, xssByWhitelist, xssByEmailWhitelist, } from '@udesk-webapp/utils';
var columns_map = [
    // 系统字段
    { id: 'subject', text: 'components.ticketFilterTable.associatedTicketForm.subject' },
    { id: 'content', text: 'components.ticketFilterTable.associatedTicketForm.content' },
    { id: 'status', text: 'components.ticketFilterTable.associatedTicketForm.status' },
    { id: 'priority', text: 'components.ticketFilterTable.associatedTicketForm.priority' },
    { id: 'platform_zh_name', text: 'components.ticketFilterTable.associatedTicketForm.forward' },
    {
        id: 'agent_group_name',
        text: 'components.ticketFilterTable.associatedTicketForm.customerOwnerGroupName',
    },
    {
        id: 'assignee_name',
        text: 'components.ticketFilterTable.associatedTicketForm.customerOwnerName',
    },
    { id: 'created_at', text: 'components.ticketFilterTable.associatedTicketForm.createdAt' },
    { id: 'field_num', text: 'components.ticketFilterTable.associatedTicketForm.fieldNum' },
];
var associatedTicketForm = function (props) {
    var t = useTranslation().t;
    var lists = props.lists, columns = props.columns, scroll = props.scroll, onRow = props.onRow, rowSelection = props.rowSelection, onChange = props.onChange, customizeLists = props.customizeLists, selectionType = props.selectionType, pagination = props.pagination, _a = props.isSensitiveField, isSensitiveField = _a === void 0 ? true : _a, ticketSystemFields = props.ticketSystemFields, agents = props.agents;
    var _b = useState([]), ticketsLists = _b[0], setticketsLists = _b[1];
    var _c = useState([]), columnsLists = _c[0], setcolumnsLists = _c[1];
    // 系统模板
    var systemRender = function (text, record, index, id) {
        // TODO:看下是否需要undefined类型定义
        var pStr = text;
        // 案例状态
        switch (text) {
            case '开启':
                pStr = (React.createElement("span", { dangerouslySetInnerHTML: {
                        __html: "<span class=\"table-pop-open\">" + t('components.ticketFilterTable.associatedTicketForm.status.open') + "</span>",
                    } }));
                break;
            case '解决中':
                pStr = (React.createElement("span", { dangerouslySetInnerHTML: {
                        __html: "<span class=\"table-pop-solving\">" + t('components.ticketFilterTable.associatedTicketForm.status.solving') + "</span>",
                    } }));
                break;
            case '已解决':
                // pStr = '<i class="icon icon-arrow-up text-info"></i>';
                pStr = (React.createElement("span", { dangerouslySetInnerHTML: {
                        __html: "<span class=\"table-pop-resolved\">" + t('components.ticketFilterTable.associatedTicketForm.status.resolved') + "</span>",
                    } }));
                break;
            case '已关闭':
                pStr = (React.createElement("span", { dangerouslySetInnerHTML: {
                        __html: "<span class=\"table-pop-closed\">" + t('components.ticketFilterTable.associatedTicketForm.status.closed') + "</span>",
                    } }));
                break;
        }
        // 紧急度
        switch (text) {
            case '高':
                pStr = (React.createElement("span", { dangerouslySetInnerHTML: {
                        __html: "<i class=\"icon icon-arrow-up text-danger\"></i>" + t('components.ticketFilterTable.associatedTicketForm.priority.high'),
                    } }));
                break;
            case '低':
                pStr = (React.createElement("span", { dangerouslySetInnerHTML: {
                        __html: "<i class=\"icon icon-arrow-down text-info\"></i>" + t('components.ticketFilterTable.associatedTicketForm.priority.low'),
                    } }));
                break;
            case '标准':
                // pStr = '<i class="icon icon-arrow-up text-info"></i>';
                pStr = (React.createElement("span", { dangerouslySetInnerHTML: {
                        __html: "<i class=\"icon icon-arrow-up text-info\"></i>" + t('components.ticketFilterTable.associatedTicketForm.priority.normal'),
                    } }));
                break;
            case '紧急':
                pStr = (React.createElement("span", { dangerouslySetInnerHTML: {
                        __html: "<i class=\"icon icon-emergent text-danger\"></i>" + t('components.ticketFilterTable.associatedTicketForm.priority.urgent'),
                    } }));
                break;
        }
        // 常规
        switch (id) {
            // 客户姓名脱敏
            case 'customer_name':
                if (record.customer_name) {
                    var customer_name_desensitization = record.customer_name;
                    var params = {
                        name: 'name',
                    };
                    if (isSensitiveField) {
                        return ((pStr = sensitiveField(params)(customer_name_desensitization)) || '');
                    }
                    else {
                        return (pStr = customer_name_desensitization);
                    }
                }
                break;
            // 描述
            case 'content':
                if (record.content) {
                    var attachments = record.attachments || [];
                    var platform = record.platform_name;
                    var result = '';
                    var value = record.content;
                    if (platform === 'email') {
                        value = removeHtmlErrorAttr(value);
                        result = attachments.length
                            ? "\uD83D\uDCCE " + xssByEmailWhitelist(value)
                            : xssByEmailWhitelist(value);
                        result = RemoveHtmlTag(replaceScriptCode(result));
                    }
                    else {
                        result = attachments.length
                            ? "\uD83D\uDCCE " + xssByWhitelist(value)
                            : xssByWhitelist(value);
                        result = RemoveHtmlTag(replaceScriptCode(result));
                    }
                    pStr = (React.createElement("div", { dangerouslySetInnerHTML: {
                            __html: result,
                        } }));
                }
                break;
        }
        // 时间
        switch (id) {
            // 创建时间
            case 'created_at':
                if (record.created_at) {
                    var d = new Date(record.created_at);
                    return (pStr =
                        d.getFullYear() +
                            '-' +
                            (d.getMonth() + 1) +
                            '-' +
                            d.getDate() +
                            ' ' +
                            d.getHours() +
                            ':' +
                            d.getMinutes() +
                            ':' +
                            d.getSeconds());
                }
                break;
        }
        return pStr;
    };
    // 自定义模板
    var customizeRender = function (text, record, index, customize) {
        // record列表数据
        var renderList = getCustomFieldValue(customize, record.custom_fields, agents);
        return renderList;
    };
    useMemo(function () {
        //列表数据
        // lists.forEach((item) => {
        //     item.key = item.id;
        // });
        setticketsLists(lists.map(function (item) { return item; }));
    }, [lists]);
    useMemo(function () {
        //表头数据
        var columns_lists = [];
        columns.forEach(function (item_columns) {
            if (item_columns.fieldMode == 'system') {
                columns_map.forEach(function (item_map) {
                    if (item_map.id == item_columns.id) {
                        columns_lists.push({
                            title: systemField ? systemField(item_map.id, ticketSystemFields) : '',
                            dataIndex: item_map.id,
                            key: item_map.id,
                            width: item_columns.width || '150px',
                            sorter: item_columns.sorter,
                            ellipsis: item_columns.ellipsis,
                            render: function (text, record, index) {
                                return systemRender(text, record, index, item_map.id);
                            },
                        });
                    }
                });
            }
            else {
                customizeLists.forEach(function (item_map) {
                    if (item_map.name == item_columns.id) {
                        columns_lists.push({
                            title: item_map.title,
                            dataIndex: item_map.id,
                            key: item_map.id,
                            width: item_columns.width || '150px',
                            sorter: item_columns.sorter,
                            ellipsis: item_columns.ellipsis,
                            render: function (text, record, index) {
                                return customizeRender(text, record, index, item_map);
                            },
                        });
                    }
                });
            }
        });
        setcolumnsLists(columns_lists);
    }, [columns, customizeLists]);
    return (React.createElement(React.Fragment, null,
        React.createElement("div", null,
            React.createElement(Table
            // TODO:类型重新定义
            // @ts-ignore
            , { 
                // TODO:类型重新定义
                // @ts-ignore
                columns: columnsLists, dataSource: ticketsLists, scrollToFirstRowOnChange: true, scroll: scroll, rowSelection: __assign({ 
                    //表格行切换
                    type: selectionType || 'checkbox' }, rowSelection), onRow: onRow, 
                // TODO:类型重新定义
                // @ts-ignore
                onChange: onChange, pagination: pagination }))));
};
export default associatedTicketForm;
