const udeskReactPackage = require("../package.json");
const fs = require('fs');
const path = require('path');
const NODE_MODULES_PATH = "node_modules";
const BUILD_CLASS_OPTIONS_SYMBOL = Symbol("build-class-options");
let DEFAULT_BUILD_CLASS_OPTIONS = {
    environment: "development",
    buildPath: null,
    projectPath: null
};

class BuildClass {
    constructor(options) {
        this[BUILD_CLASS_OPTIONS_SYMBOL] = Object.assign({}, DEFAULT_BUILD_CLASS_OPTIONS, options || {});
    }

    preBuild() {

    }

    postBuild() {

    }

    parseWebpackConfig(webpackConfig) {
        let udeskReactDependencyKeys = Object.keys(udeskReactPackage.dependencies);
        let projectPath = this[BUILD_CLASS_OPTIONS_SYMBOL].projectPath;
        if (projectPath == null) {
            projectPath = fs.realpathSync(process.cwd());
        }
        let packageFile = path.join(projectPath, "package.json");

        if (fs.existsSync(packageFile)) {
            let alias = {};
            let projectPackage = require(packageFile);
            let projectPackageDependKeys = Object.keys(projectPackage.dependencies || {}).concat(Object.keys(projectPackage.devDependencies || {}));
            let projectPackageExcludeDependKeys = projectPackage.excludeDependencies || {};
            let aliasExclude = projectPackageExcludeDependKeys.aliasExclude || [];
            for (let udeskReactDependencyKey of udeskReactDependencyKeys) {
                if (projectPackageDependKeys.some(projectPackageDependKey => projectPackageDependKey === udeskReactDependencyKey)) {
                    if (!alias.hasOwnProperty(udeskReactDependencyKey) && !aliasExclude.includes(udeskReactDependencyKey)) {
                        alias[udeskReactDependencyKey] = path.join(fs.realpathSync(process.cwd()), `${NODE_MODULES_PATH}/${udeskReactDependencyKey}`);
                    }
                }
            }
            if (webpackConfig == null) {
                webpackConfig = {};
            }
            if (webpackConfig.resolve == null) {
                webpackConfig.resolve = {};
            }
            webpackConfig.resolve.alias = Object.assign({}, alias, webpackConfig.resolve.alias);

            // 在业务项目的webpack配置中，可以通过添加空的alias来实现禁用alias的效果
            Object.keys(webpackConfig.resolve.alias).forEach(k => {
                if (!webpackConfig.resolve.alias[k]) {
                    delete webpackConfig.resolve.alias[k];
                }
            });
        }
        return webpackConfig;
    }
}

module.exports = BuildClass;