const fs = require("fs");
const path = require("path");
const environmentMappings = require("./mappings");
const DEFAULT_ENVIRONMENT = "development";
const DEFAULT_OUTPUT_PATH = "/dist";
const BASE_CONFIG_FILE = "env/.env.json";


/**
 * 拷贝 globalSettings
 * @deprecated 已废弃，请使用 ~/lib/build/global-settings
 *
 * @param {*} [{
 *   outputPath = DEFAULT_OUTPUT_PATH,
 *   environment = DEFAULT_ENVIRONMENT,
 *   copyTo,
 *   log = true,
 * }={}]
 */
function copyEnvironment({
  outputPath = DEFAULT_OUTPUT_PATH,
  environment = DEFAULT_ENVIRONMENT,
  copyTo,
  log = true,
} = {}) {
  if (outputPath == null || outputPath === "") {
    outputPath = DEFAULT_OUTPUT_PATH;
  }
  if (environmentMappings.hasOwnProperty(environment)) {
    let baseData = getEnvironmentConfig(BASE_CONFIG_FILE);
    let relatedEnvs = Object.keys(environmentMappings[environment]);
    for (let i = 0; i < relatedEnvs.length; i++) {
      let env = relatedEnvs[i];
      let setting = environmentMappings[environment][env];
      let outputFile = path.join(outputPath, setting.outputPath);
      let configData = JSON.stringify(
        Object.assign({}, baseData, getEnvironmentConfig(setting.path)),
        null,
        4
      );
      if (log) {
        // eslint-disable-next-line no-console
        console.log(`Writing ${outputFile} for ${environment} mode`);
      }
      writeFile(outputFile, configData);

      if (env === environment && copyTo) {
        if (log) {
          // eslint-disable-next-line no-console
          console.log(`Copying ${env} config to '${copyTo}'`);
        }
        writeFile(copyTo, configData);
      }
    }
  }
}

function writeFile(path, data) {
  let dirs = path.split("/").slice(0, -1);
  if (dirs.length > 0) {
    dirs.forEach(function (folderPath, index) {
      if (folderPath) {
        let parentDir = dirs.slice(0, index + 1).join("/");
        if (!fs.existsSync(parentDir)) {
          fs.mkdirSync(parentDir);
        }
      }
    });
  }

  if (fs.existsSync(path)) {
    let oldContent = fs.readFileSync(path).toString();
    if (oldContent !== data) {
      fs.writeFileSync(path, data);
    }
  }
}

function getEnvironmentConfig(environmentConfigPath) {
  let publicConfig = {};
  let currentEnvironmentConfigPath = path.join(
    fs.realpathSync(process.cwd()),
    environmentConfigPath
  );
  if (fs.existsSync(currentEnvironmentConfigPath)) {
    let environmentConfig = require(currentEnvironmentConfigPath);
    if (environmentConfig != null) {
      publicConfig = environmentConfig;
    }

    let publicConfigKeys = Object.keys(publicConfig);
    for (var i = 0; i < publicConfigKeys.length; i++) {
      let publicConfigKey = publicConfigKeys[i];
      let singlePublicConfig = publicConfig[publicConfigKey];
      if (typeof singlePublicConfig === "object") {
        throw new Error(
          `Any item in the config can't be an object in ${environmentConfigPath}.`
        );
      }
    }
  }
  return publicConfig;
}

module.exports = copyEnvironment;
