const shell = require('shelljs');

const NOT_STAGED_MESSAGES = [
    "nothing to commit",
    "无文件要提交"
];
const PROVISIONAL_COMMIT_TIP = "因为发版公共项目需要切换到master分支，但当前分支仍有未提交的修改，自动提交并切换到master分支！";
const PROVISIONAL_COMMIT_MESSAGE = "因发版需切换至master，临时提交当前分支，如果发现异常请撤销。";

function isTargetBranch(branch = "master") {
    const gitBranchMessages = shell.exec("git symbolic-ref --short -q HEAD", { silent: true });
    const branchName = gitBranchMessages != null ? gitBranchMessages.stdout : "";
    return branchName === branch;
}

function hasUnstagedChanges() {
    const gitStatusMessages = shell.exec("git status", { silent: true });
    const statusMessage = gitStatusMessages != null ? gitStatusMessages.stdout : "";
    return !NOT_STAGED_MESSAGES.some(message => statusMessage.indexOf(message) !== -1);
}

function checkoutToBranch(branch = "master", project = "") {
    if (hasUnstagedChanges()) {
        shell.echo("-e", "\033[33;1m[Project: " + project + "]\033[31m" + PROVISIONAL_COMMIT_TIP + "\033[0m");
        shell.exec(`git add .`);
        shell.exec(`git commit -am ${PROVISIONAL_COMMIT_MESSAGE}`);
    }
    if (!isTargetBranch(branch)) {
        shell.exec(`git checkout ${branch}`, { silent: true });
    }
    shell.exec(`git reset --hard origin/${branch}`, { silent: true });
    shell.exec("git pull --rebase", { silent: true });
}

module.exports = {
    isTargetBranch,
    hasUnstagedChanges,
    checkoutToBranch
};