const path = require('path');
const fs = require('fs-extra');
const BASE_FILE = '.env';
const EXTENSIONS = ['.json', '.js'];

exports.compile = function ({
    inputDir = './env',
    outputDir = './dist/.config',
    outputFile = './dist/config/main.json',
    deployEnv = 'development',
    workspace = './',
}) {
    if (workspace) {
        inputDir = path.join(workspace, inputDir);
        outputDir = path.join(workspace, outputDir);
        outputFile = path.join(workspace, outputFile);
    }
    // 基础配置，所有配置文件的基类
    let baseConfig = loadFile(path.join(inputDir, BASE_FILE), EXTENSIONS) || {};

    // 编译其他配置文件
    let files = fs
        .readdirSync(inputDir)
        .filter((p) => EXTENSIONS.includes(path.extname(p))) // 只处理js，json文件
        .filter((p) => !EXTENSIONS.map((ext) => BASE_FILE + ext).includes(p)); // 不包含基础配置文件
    for (let i = 0; i < files.length; i++) {
        const file = files[i];
        let data = loadFile(path.join(inputDir, file));
        // 与基础配置合并
        data = Object.assign({}, baseConfig, data);
        let content = JSON.stringify(data, null, 4);
        writeFile(path.join(outputDir, file.replace(/\.js$/, ".json")), content);
        // 把当前编译环境的配置文件，输出到主文件的位置
        if (file === `.env.${deployEnv}.json` && outputFile) {
            writeFile(outputFile, content);
        }
    }
};

function loadFile(filepath, extensions) {
    let tryFiles = [];
    if (extensions) {
        tryFiles = extensions.map((ext) => filepath + ext);
    } else {
        tryFiles = [filepath];
    }
    for (let i = 0; i < tryFiles.length; i++) {
        const file = tryFiles[i];
        if (file.match(/\.(js|json)/i)) {
            return require(path.resolve(file));
        }
    }
    return undefined;
}

function writeFile(path, content) {
    // 对比文件是否有变化
    if (fs.existsSync(path)) {
        let oldContent = fs.readFileSync(path).toString();
        if (content === oldContent) {
            return;
        }
    }
    fs.writeFileSync(path, content);
}
