"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatInput = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const antd_1 = require("antd");
const coral_system_1 = require("coral-system");
const icons_1 = require("@ant-design/icons");
const chatBoxStyle = (0, coral_system_1.css) `
  position: relative;
  .ant-btn {
    position: absolute;
    bottom: 6px;
    right: 6px;
  }
  textarea {
    border: 0;
  }
`;
function ChatInput({ title = '智能编程助手', showCloseIcon = true, placeholder = '请简单描述你要执行的动作', onGenerate, onApply, onClose, }) {
    const [text, setText] = (0, react_1.useState)('');
    const [preview, setPreview] = (0, react_1.useState)('');
    const [loading, setLoading] = (0, react_1.useState)(false);
    return (react_1.default.createElement(coral_system_1.Box, { border: "solid", borderColor: "line2", borderRadius: "m", position: "relative" },
        react_1.default.createElement(coral_system_1.Box, { display: "flex", justifyContent: "space-between", alignItems: "center", bg: "fill2", px: "m", py: "s", borderBottom: "solid", borderBottomColor: "line2" },
            react_1.default.createElement(coral_system_1.Box, { fontSize: "14px" },
                react_1.default.createElement(icons_1.RobotOutlined, null),
                react_1.default.createElement(coral_system_1.Text, { ml: "m", userSelect: "none" }, title)),
            showCloseIcon && react_1.default.createElement(icons_1.CloseOutlined, { onClick: onClose })),
        react_1.default.createElement(coral_system_1.Box, { css: chatBoxStyle },
            react_1.default.createElement(antd_1.Input.TextArea, { rows: 3, value: text, onChange: (e) => setText(e.target.value), placeholder: placeholder }),
            react_1.default.createElement(antd_1.Button, { disabled: !text, size: "small", loading: loading, onClick: () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    setLoading(true);
                    const ret = yield onGenerate(text);
                    setLoading(false);
                    ret && setPreview(ret);
                }) }, "\u751F\u6210")),
        react_1.default.createElement(coral_system_1.Box, { minHeight: 48, background: "fill2", p: "m", fontSize: "12px", css: chatBoxStyle },
            react_1.default.createElement(coral_system_1.Text, { as: "code" },
                "\u7ED3\u679C\uFF1A",
                preview || '暂无'),
            react_1.default.createElement(antd_1.Button, { size: "small", disabled: !preview, onClick: () => onApply(preview) }, "\u5E94\u7528"))));
}
exports.ChatInput = ChatInput;
