import { IFrameProtocol } from './iframe-protocol';
import { IManagerState, IModuleError, ManagerStatus, UnsubscribeFunction } from '../types';
interface IManagerOptions {
    /**
     * Location of the bundler.
     */
    bundlerURL?: string;
    /**
     * Width of iframe.
     */
    width?: string;
    /**
     * Height of iframe.
     */
    height?: string;
    /**
     * If we should skip the third step: evaluation.
     */
    skipEval?: boolean;
    externalResources?: string[];
    /**
     * You can pass a custom file resolver that is responsible for resolving files.
     * We will use this to get all files from the file system.
     */
    fileResolver?: {
        isFile: (path: string) => Promise<boolean>;
        readFile: (path: string) => Promise<string>;
    };
}
interface IFile {
    code: string;
}
interface IFiles {
    [path: string]: IFile;
}
interface IDependencies {
    [depName: string]: string;
}
interface ISandboxInfo {
    files: IFiles;
    dependencies?: IDependencies;
    entry?: string;
    /**
     * What template we use, if not defined we infer the template from the dependencies or files.
     *
     * @type {string}
     */
    template?: string;
    showOpenInCodeSandbox?: boolean;
    /**
     * Only use unpkg for fetching the dependencies, no preprocessing. It's slower, but doesn't talk
     * to AWS.
     */
    disableDependencyPreprocessing?: boolean;
}
export default class PreviewManager {
    selector: string | undefined;
    element: Element;
    iframe: HTMLIFrameElement;
    iframeProtocol: IFrameProtocol;
    options: IManagerOptions;
    id: string | null;
    listener?: Function;
    bundlerURL: string;
    externalResources: string[];
    managerState: IManagerState | undefined;
    errors: IModuleError[];
    status: ManagerStatus;
    sandboxInfo: ISandboxInfo;
    unsubscribeGlobalListener: UnsubscribeFunction;
    unsubscribeChannelListener: UnsubscribeFunction;
    constructor(selector: string | HTMLIFrameElement, sandboxInfo: ISandboxInfo, options?: IManagerOptions);
    updateOptions(options: IManagerOptions): void;
    updatePreview(sandboxInfo?: ISandboxInfo): void;
    cleanup(): void;
    private dispatch;
    private getFiles;
    private initializeElement;
}
export {};
