"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollapsePanel = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const coral_system_1 = require("coral-system");
const icons_1 = require("@ant-design/icons");
const tango_helpers_1 = require("@music163/tango-helpers");
const headerStyle = (0, coral_system_1.css) `
  user-select: none;

  .anticon-up {
    margin-right: 8px;
    color: var(--tango-colors-text2);
    transition: transform 0.2s ease 0s;
  }
`;
function CollapsePanel(props) {
    const { title, extra, children, collapsed: collapsedProp, defaultCollapsed = false, onCollapse, stickyHeader, showBottomBorder = true, headerProps, bodyProps } = props, rest = tslib_1.__rest(props, ["title", "extra", "children", "collapsed", "defaultCollapsed", "onCollapse", "stickyHeader", "showBottomBorder", "headerProps", "bodyProps"]);
    const [collapsed, setCollapsed] = (0, tango_helpers_1.useControllableState)({
        value: collapsedProp,
        defaultValue: defaultCollapsed,
        onChange: onCollapse,
    });
    const iconStyle = {
        transform: collapsed ? undefined : 'rotate(-180deg)',
    };
    const stickHeaderProps = stickyHeader
        ? {
            position: 'sticky',
            top: '0px',
            zIndex: 1,
            bg: 'white',
        }
        : {};
    const CollapsePanelBorderStyle = showBottomBorder
        ? {
            borderBottom: 'solid',
            borderBottomColor: 'line2',
        }
        : {};
    return (react_1.default.createElement(coral_system_1.Box, Object.assign({ className: "CollapsePanel" }, CollapsePanelBorderStyle, rest),
        react_1.default.createElement(coral_system_1.Box, Object.assign({ display: "flex", alignItems: "center", justifyContent: "space-between", className: "CollapsePanelHeader", onClick: () => setCollapsed(!collapsed), p: "m" }, stickHeaderProps, headerProps, { css: headerStyle }),
            react_1.default.createElement(coral_system_1.Box, { display: "flex", alignItems: "center", fontSize: "14px", fontWeight: "500" },
                react_1.default.createElement(icons_1.UpOutlined, { style: iconStyle }),
                title),
            react_1.default.createElement(coral_system_1.Box, null, extra)),
        react_1.default.createElement(coral_system_1.Box, Object.assign({ className: "CollapsePanelBody", display: collapsed ? 'none' : 'block' }, bodyProps), children)));
}
exports.CollapsePanel = CollapsePanel;
