"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InputKV = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const antd_1 = require("antd");
const coral_system_1 = require("coral-system");
const icons_1 = require("@ant-design/icons");
const rowCss = (0, coral_system_1.css) `
  display: flex;
  align-items: center;
  margin-bottom: 12px;

  .ant-input-affix-wrapper:first-child {
    margin-right: 12px;
  }

  .anticon-delete {
    margin-left: 8px;
    color: #666;

    &:hover {
      color: #1890ff;
    }
  }
`;
const btnCss = (0, coral_system_1.css) `
  display: inline-block;
  cursor: pointer;
  color: #1890ff;
  max-width: 80px;
  line-height: 1;

  .btn-span {
    margin-left: 6px;
  }
`;
const tipCss = (0, coral_system_1.css) `
  font-size: 12px;
  color: red;
  margin-bottom: 8px;
`;
const OneInputKV = (props) => {
    const { keyInputProps, valueInputProps, format = 'simple', onChange, defaultValue, value, } = props;
    const [data, setData] = (0, react_1.useState)(formatValue({
        data: defaultValue || value,
        format: 'original',
    }));
    const [validate, setValidate] = (0, react_1.useState)(false);
    const counter = (0, react_1.useRef)(0);
    (0, react_1.useEffect)(() => {
        const newData = format === 'simple' ? data : formatValue({ data: value, format: 'original' });
        // compare
        const flag = compareValue(data, newData);
        // 当 value 值为真，compare 后不等时
        value && !flag && setData(newData);
    }, [value]);
    // 点击添加的回调
    const handleAdd = () => {
        // 提前校验上一个是否有 key
        const last = data[data.length - 1];
        setValidate(true);
        if (data.length > 0 && !last.key) {
            return;
        }
        counter.current++;
        const newData = data.concat([{ index: counter.current }]);
        _updateData(newData);
    };
    // 点击删除的回调
    const handleDelete = (index, item) => {
        const newData = [...data];
        // delete
        newData.splice(index, 1);
        _updateData(newData);
        !newData.length && setValidate(false);
    };
    const handleInput = (name, index, val) => {
        const newData = [...data];
        if (!newData[index])
            return;
        // 更新值
        newData[index][name] = val;
        _updateData(newData);
    };
    const _updateData = (values) => {
        const omit = (_a) => {
            var { index } = _a, rest = tslib_1.__rest(_a, ["index"]);
            return rest;
        };
        let nextValue = formatValue({ data: values, format });
        Array.isArray(nextValue) && (nextValue = nextValue.map((item) => omit(item)));
        setData(values);
        onChange === null || onChange === void 0 ? void 0 : onChange(nextValue);
    };
    // render
    const renderRow = () => data === null || data === void 0 ? void 0 : data.map((item, index) => (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(coral_system_1.Box, { key: item.index, css: rowCss },
            react_1.default.createElement(antd_1.Input, Object.assign({ allowClear: true, placeholder: "\u8BF7\u8F93\u5165 Key" }, keyInputProps, { value: item.key, onChange: (ev) => {
                    var _a;
                    handleInput('key', index, (_a = ev.target) === null || _a === void 0 ? void 0 : _a.value);
                } })),
            react_1.default.createElement(antd_1.Input, Object.assign({ allowClear: true, placeholder: "\u8BF7\u8F93\u5165 Value" }, valueInputProps, { value: item.value, onChange: (ev) => {
                    handleInput('value', index, ev.target.value);
                } })),
            react_1.default.createElement(icons_1.DeleteOutlined, { onClick: () => {
                    handleDelete(index, item);
                } })),
        !item.key && validate && react_1.default.createElement(coral_system_1.Box, { css: tipCss }, "\u63D0\u793A\uFF1A\u8BF7\u5148\u8F93\u5165 key "))));
    const rows = renderRow();
    return (react_1.default.createElement(coral_system_1.Box, null,
        rows,
        react_1.default.createElement(coral_system_1.Box, { onClick: handleAdd, css: btnCss },
            react_1.default.createElement(icons_1.PlusOutlined, null),
            " \u6DFB\u52A0\u952E\u503C")));
};
exports.InputKV = (0, react_1.memo)(OneInputKV, (prev, next) => prev.value === next.value);
const formatData = (data) => {
    const result = {};
    // 遍历数据
    data === null || data === void 0 ? void 0 : data.forEach((item) => {
        item.key && !(item.key in result) && (result[item.key] = item.value);
    });
    return result;
};
// 格式化value
const formatValue = ({ format, data }) => {
    if (format === 'original') {
        if (!data)
            return [{ index: 0 }];
        const list = Array.isArray(data)
            ? data
            : Object.keys(data).map((key) => ({ key, value: data[key] }));
        return list.map((item, idx) => (Object.assign(Object.assign({}, item), { index: idx - list.length - 1 })));
    }
    else {
        if (!data)
            return {};
        return Array.isArray(data) ? formatData(data) : data;
    }
};
const compareValue = (origin, data) => {
    const resolve = (list) => {
        let keyStr = '';
        let valueStr = '';
        list.forEach((item) => {
            const { key, value } = item;
            keyStr += key || '$';
            valueStr += value || '$';
        });
        return { keyStr, valueStr };
    };
    const oldRes = resolve(origin);
    const newRes = resolve(data);
    return oldRes.keyStr === newRes.keyStr && oldRes.valueStr === newRes.valueStr;
};
