"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Menu = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const styled_components_1 = tslib_1.__importDefault(require("styled-components"));
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const Text = styled_components_1.default.div `
  background-color: #8fa0f0;
  color: white;
  padding: 2px 4px;
`;
const List = styled_components_1.default.ul `
  list-style: none;
  padding: 0;
  background-color: white;
  border: 1px solid #d9d9d9;
  margin: 0 4px;
  position: absolute;
  width: calc(100% - 8px);
  max-width: 300px;

  > li {
    .group-title {
      color: #666;
      margin: 8px 0 4px;
      padding: 0 4px;
    }

    .action-item {
      line-height: 22px;
      padding: 0 4px;
      user-select: none;

      &:hover {
        cursor: pointer;
      }

      &.active {
        background-color: #8fa0f0;
        color: white;
      }
    }
  }
`;
let timer;
const Menu = (props) => {
    const { text = '输入 {} 进入指令选择界面', showDefaultText = true, items = [], value, onClick, onUpdate, top, } = props;
    const handleClick = (ev, item) => {
        ev.stopPropagation();
        (item === null || item === void 0 ? void 0 : item.key.indexOf('.')) > -1 && onUpdate(item === null || item === void 0 ? void 0 : item.key);
        timer && clearTimeout(timer);
        timer = setTimeout(() => {
            onClick === null || onClick === void 0 ? void 0 : onClick(item === null || item === void 0 ? void 0 : item.key);
        }, 0);
    };
    const renderItems = () => {
        return items.map((item, idx) => {
            const { label, title, key } = item;
            const newClass = (0, classnames_1.default)('action-item', { active: value === key });
            return (react_1.default.createElement("li", { key: idx },
                title && react_1.default.createElement("div", { className: "group-title" }, title),
                react_1.default.createElement("div", { className: newClass, onClick: (ev) => handleClick(ev, item) }, label)));
        });
    };
    const children = showDefaultText ? react_1.default.createElement(Text, null, text) : renderItems();
    return react_1.default.createElement(List, { style: { top } }, children);
};
exports.Menu = Menu;
