"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VariableTree = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const tango_helpers_1 = require("@music163/tango-helpers");
const coral_system_1 = require("coral-system");
const antd_1 = require("antd");
const icons_1 = require("@ant-design/icons");
const tango_ui_1 = require("@music163/tango-ui");
const add_store_1 = require("./add-store");
const add_service_1 = require("./add-service");
const value_detail_1 = require("./value-detail");
const value_preview_1 = require("./value-preview");
const service_preview_1 = require("./service-preview");
const varTreeStyle = (0, coral_system_1.css) `
  overflow: auto;
  position: relative;

  .ant-tree {
    font-family: Consolas, Menlo, Courier, monospace;
  }

  .ant-tree-node-content-wrapper.ant-tree-node-content-wrapper-normal {
    width: calc(100% - 50px);
  }

  .ant-tree .ant-tree-treenode {
    padding: 0;
  }
  .ant-tree.ant-tree-directory .ant-tree-treenode::before {
    bottom: 0;
  }

  .ant-tree-indent-unit {
    width: 12px;
  }
`;
function VariableTree(props) {
    const { dataSource = [], serviceModules = [], appContext = {}, defaultValueDetailMode, onSelect = tango_helpers_1.noop, onAddStoreVariable = tango_helpers_1.noop, onAddStore = tango_helpers_1.noop, onAddService = tango_helpers_1.noop, onUpdateStoreVariable = tango_helpers_1.noop, onUpdateService = tango_helpers_1.noop, onRemoveStoreVariable, onRemoveService, onCopy = tango_helpers_1.noop, onView = tango_helpers_1.noop, getServiceData, getServiceNames, getStoreNames, getPreviewValue = tango_helpers_1.noop, showViewButton, renderHeaderExtra, renderDetail: renderDetailProp } = props, rest = tslib_1.__rest(props, ["dataSource", "serviceModules", "appContext", "defaultValueDetailMode", "onSelect", "onAddStoreVariable", "onAddStore", "onAddService", "onUpdateStoreVariable", "onUpdateService", "onRemoveStoreVariable", "onRemoveService", "onCopy", "onView", "getServiceData", "getServiceNames", "getStoreNames", "getPreviewValue", "showViewButton", "renderHeaderExtra", "renderDetail"]);
    const [keyword, setKeyword] = (0, react_1.useState)('');
    const [activeNode, setActiveNode] = (0, react_1.useState)();
    const [mode, setMode] = (0, react_1.useState)();
    const clear = (0, react_1.useCallback)(() => {
        setActiveNode(null);
        setMode(null);
    }, []);
    const selectNode = (0, react_1.useCallback)((node, callback) => {
        if ((0, tango_helpers_1.isStoreVariablePath)(node.key)) {
            setMode('storeVariableDetail');
        }
        else if ((0, tango_helpers_1.isServiceVariablePath)(node.key)) {
            setMode('serviceDetail');
        }
        else {
            setMode('detail');
        }
        setActiveNode(node);
        callback === null || callback === void 0 ? void 0 : callback(node);
    }, []);
    const treeData = (0, react_1.useMemo)(() => {
        const pattern = new RegExp(keyword, 'ig');
        return keyword
            ? (0, tango_helpers_1.filterTreeData)(dataSource, (leaf) => pattern.test(leaf.title), 'children', true)
            : dataSource;
    }, [keyword, dataSource]);
    const state = { activeNode, mode, setMode, clear };
    return (react_1.default.createElement(coral_system_1.Box, Object.assign({ display: "flex", columnGap: "l", className: "VariableTree", css: varTreeStyle }, rest),
        react_1.default.createElement(coral_system_1.Box, { className: "VariableList", width: "40%" }, renderHeaderExtra === null || renderHeaderExtra === void 0 ? void 0 :
            renderHeaderExtra(props, state),
            react_1.default.createElement(coral_system_1.Box, { mb: "m", position: "sticky", top: "0", bg: "white", zIndex: 2 },
                react_1.default.createElement(tango_ui_1.Search, { placeholder: "\u8BF7\u8F93\u5165\u53D8\u91CF\u540D", onChange: (val) => setKeyword(val === null || val === void 0 ? void 0 : val.trim()) })),
            react_1.default.createElement(antd_1.Tree, { blockNode: true, showIcon: false, defaultExpandAll: true, treeData: treeData, onSelect: (keys, detail) => {
                    selectNode(detail.node, onSelect);
                }, titleRender: (node) => {
                    var _a;
                    const isLeaf = !node.children;
                    if (isLeaf) {
                        const showView = (_a = node.showViewButton) !== null && _a !== void 0 ? _a : showViewButton;
                        let nodePath = node.key;
                        if (/^(stores|services)\./.test(nodePath)) {
                            nodePath = `tango.${node.key.replaceAll('.', '?.').replace('?.', '.')}`;
                        }
                        return (react_1.default.createElement(coral_system_1.Box, { display: "flex", justifyContent: "space-between", alignItems: "center" },
                            react_1.default.createElement(coral_system_1.Text, { flex: "1", truncated: true },
                                node.title,
                                node.type === 'function' && (react_1.default.createElement(coral_system_1.Text, { color: "text3", ml: "m" },
                                    react_1.default.createElement(icons_1.FunctionOutlined, null)))),
                            react_1.default.createElement(coral_system_1.Box, { flex: "0 0 72px", textAlign: "right" },
                                node.showRemoveButton && (react_1.default.createElement(antd_1.Popconfirm, { title: `确认删除吗 ${node.title}？该操作会导致引用此模型的代码报错，请谨慎操作！`, onConfirm: (e) => {
                                        e.stopPropagation();
                                        if ((0, tango_helpers_1.isStoreVariablePath)(node.key)) {
                                            onRemoveStoreVariable(node.key);
                                        }
                                        else {
                                            onRemoveService(node.key);
                                        }
                                    } },
                                    react_1.default.createElement(antd_1.Button, { type: "text", size: "small", icon: react_1.default.createElement(icons_1.DeleteOutlined, null), onClick: (e) => {
                                            e.stopPropagation();
                                        } }))),
                                react_1.default.createElement(tango_ui_1.CopyClipboard, { text: nodePath }, ({ copied, onClick }) => {
                                    const label = copied ? '已复制' : '复制变量路径';
                                    return (react_1.default.createElement(antd_1.Tooltip, { title: label },
                                        react_1.default.createElement(antd_1.Button, { type: "text", size: "small", icon: react_1.default.createElement(icons_1.CopyOutlined, null), onClick: (e) => {
                                                e.stopPropagation();
                                                onClick();
                                                onCopy(node);
                                            } })));
                                }),
                                showView && (react_1.default.createElement(antd_1.Tooltip, { title: "\u67E5\u770B\u53D8\u91CF\u8BE6\u60C5" },
                                    react_1.default.createElement(antd_1.Button, { type: "text", size: "small", icon: react_1.default.createElement(icons_1.EyeOutlined, null), onClick: (e) => {
                                            e.stopPropagation();
                                            selectNode(node, onView);
                                        } }))))));
                    }
                    return (react_1.default.createElement(coral_system_1.Box, { display: "flex", alignItems: "center", justifyContent: "space-between" },
                        react_1.default.createElement(coral_system_1.Text, { mr: "m" }, node.title),
                        react_1.default.createElement(coral_system_1.Box, null,
                            /^\$?stores$/.test(node.key) && (react_1.default.createElement(antd_1.Tooltip, { title: "\u65B0\u5EFA\u6570\u636E\u6A21\u578B" },
                                react_1.default.createElement(antd_1.Button, { type: "text", size: "small", icon: react_1.default.createElement(icons_1.PlusOutlined, null), onClick: (e) => {
                                        e.stopPropagation();
                                        setMode('addStore');
                                    } }))),
                            node.showAddButton && /^stores\.[a-zA-Z0-9]+$/.test(node.key) && (react_1.default.createElement(antd_1.Tooltip, { title: `向 ${node.title} 中添加变量` },
                                react_1.default.createElement(antd_1.Button, { type: "text", size: "small", icon: react_1.default.createElement(icons_1.PlusOutlined, null), onClick: (e) => {
                                        e.stopPropagation();
                                        setActiveNode(node);
                                        setMode('addVariable');
                                    } }))),
                            /^services(\.[a-zA-Z0-9]+)?$/.test(node.key) && (react_1.default.createElement(antd_1.Tooltip, { title: `向 ${node.title} 中添加服务函数` },
                                react_1.default.createElement(antd_1.Button, { type: "text", size: "small", icon: react_1.default.createElement(icons_1.PlusOutlined, null), onClick: (e) => {
                                        e.stopPropagation();
                                        setActiveNode(node);
                                        setMode('addService');
                                    } }))))));
                } })),
        react_1.default.createElement(coral_system_1.Box, { className: "VariableDetail", flex: "1", position: "sticky", top: "0", overflow: "auto" }, (function renderDetail() {
            var _a;
            const customRenderer = renderDetailProp === null || renderDetailProp === void 0 ? void 0 : renderDetailProp(props, state);
            if (customRenderer) {
                return customRenderer;
            }
            if (mode === 'detail') {
                return react_1.default.createElement(value_detail_1.NodeCommonDetail, { data: activeNode });
            }
            if (mode === 'storeVariableDetail') {
                return (react_1.default.createElement(value_detail_1.ValueDetail, { key: activeNode.key, defaultMode: defaultValueDetailMode }, (previewMode) => previewMode === 'runtime' ? (react_1.default.createElement(value_preview_1.ValuePreview, { value: getPreviewValue(activeNode), onCopy: (valuePath) => {
                        return ['tango', activeNode.key.replaceAll('.', '?.'), valuePath].join('.');
                    } })) : (react_1.default.createElement(value_detail_1.ValueDefine, { data: activeNode, onSave: onUpdateStoreVariable }))));
            }
            if (mode === 'serviceDetail') {
                return (react_1.default.createElement(react_1.default.Fragment, null,
                    react_1.default.createElement(tango_ui_1.Panel, { shape: "solid", title: "\u670D\u52A1\u51FD\u6570\u914D\u7F6E" },
                        react_1.default.createElement(add_service_1.AddServiceForm, { key: activeNode.key, serviceModules: serviceModules, serviceNames: (function () {
                                const { moduleName } = (0, tango_helpers_1.parseServiceVariablePath)(activeNode.key);
                                return (getServiceNames === null || getServiceNames === void 0 ? void 0 : getServiceNames(moduleName)) || [];
                            })(), initialValues: Object.assign({}, getServiceData === null || getServiceData === void 0 ? void 0 : getServiceData(activeNode.key)), onCancel: clear, onSubmit: (values) => {
                                onUpdateService(values);
                            } })),
                    react_1.default.createElement(tango_ui_1.Panel, { shape: "solid", title: "\u670D\u52A1\u51FD\u6570\u9884\u89C8", mt: "l" },
                        react_1.default.createElement(service_preview_1.ServicePreview, { key: activeNode.key, appContext: appContext, functionKey: activeNode.key }))));
            }
            if (mode === 'addVariable') {
                return (react_1.default.createElement(add_store_1.AddStoreVariableForm, { parentNode: activeNode, onSubmit: (storeName, data) => {
                        onAddStoreVariable(storeName, data);
                        clear();
                    }, onCancel: () => {
                        clear();
                    } }));
            }
            if (mode === 'addStore') {
                return (react_1.default.createElement(add_store_1.AddStoreForm, { storeNames: (getStoreNames === null || getStoreNames === void 0 ? void 0 : getStoreNames()) || [], onSubmit: ({ name }) => {
                        onAddStore(name);
                        clear();
                    }, onCancel: () => {
                        clear();
                    } }));
            }
            if (mode === 'addService') {
                return (react_1.default.createElement(tango_ui_1.Panel, { shape: "solid", title: "\u521B\u5EFA\u670D\u52A1\u51FD\u6570" },
                    react_1.default.createElement(add_service_1.AddServiceForm, { key: activeNode.key, serviceModules: serviceModules, serviceNames: ((_a = activeNode.children) === null || _a === void 0 ? void 0 : _a.map((item) => item.title)) || [], initialValues: {
                            moduleName: activeNode.title,
                        }, onCancel: () => {
                            clear();
                        }, onSubmit: (values) => {
                            onAddService(values);
                            clear();
                        } })));
            }
        })())));
}
exports.VariableTree = VariableTree;
