"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigItem = exports.ConfigGroup = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const coral_system_1 = require("coral-system");
const antd_1 = require("antd");
const select_list_1 = require("./select-list");
/**
 * 配置组
 */
function ConfigGroup({ title, children, }) {
    return (react_1.default.createElement(coral_system_1.Box, { className: "SettingGroup", mb: "l" },
        react_1.default.createElement(coral_system_1.Box, { className: "SettingGroupTitle", fontSize: "14px", fontWeight: "bold", borderBottom: "solid", borderBottomColor: "#f0f0f0", px: "l", py: "s" }, title),
        react_1.default.createElement(coral_system_1.Box, { className: "SettingGroupChildren", px: "l" }, children)));
}
exports.ConfigGroup = ConfigGroup;
const builtinConfigSetterMap = {
    switch: (_a) => {
        var { value, onChange } = _a, rest = tslib_1.__rest(_a, ["value", "onChange"]);
        return react_1.default.createElement(antd_1.Switch, Object.assign({}, rest, { checked: value, onChange: onChange }));
    },
    input: (_a) => {
        var { value, onChange } = _a, rest = tslib_1.__rest(_a, ["value", "onChange"]);
        return react_1.default.createElement(antd_1.Input, Object.assign({}, rest, { value: value, onBlur: (e) => onChange(e.target.value) }));
    },
    select: antd_1.Select,
    selectList: select_list_1.SelectList,
};
/**
 * 配置项
 */
function ConfigItem({ title, description, children: childrenProp, layoutDirection = 'row', component, componentProps = {}, onChange, value, }) {
    let children = childrenProp;
    const Setter = builtinConfigSetterMap[component];
    if (Setter) {
        children = react_1.default.createElement(Setter, Object.assign({}, componentProps, { value: value, onChange: onChange }));
    }
    const { rootStyle, leftStyle, rightStyle } = useConfigItemStyle(layoutDirection);
    return (react_1.default.createElement(coral_system_1.Box, Object.assign({ className: "SettingItem", py: "m", borderBottom: "solid", borderBottomColor: "#f0f0f0" }, rootStyle),
        react_1.default.createElement(coral_system_1.Box, Object.assign({ className: "SettingItemLeft" }, leftStyle),
            react_1.default.createElement(coral_system_1.Box, { className: "SettingItemTitle", fontSize: "14px", fontWeight: "bold", mb: "s" }, title),
            react_1.default.createElement(coral_system_1.Box, { className: "SettingItemDescription", fontSize: "12px", color: "text.note" }, description)),
        react_1.default.createElement(coral_system_1.Box, Object.assign({ className: "SettingItemRight" }, rightStyle), children)));
}
exports.ConfigItem = ConfigItem;
function useConfigItemStyle(layoutDirection) {
    let rootStyle;
    let leftStyle;
    let rightStyle;
    if (layoutDirection === 'row') {
        rootStyle = {
            display: 'flex',
            alignItems: 'center',
        };
        leftStyle = {
            width: '60%',
        };
        rightStyle = {
            width: '40%',
            display: 'flex',
            justifyContent: 'flex-end',
        };
    }
    else if (layoutDirection === 'column') {
        rootStyle = {
            display: 'flex',
            flexDirection: 'column',
        };
    }
    return {
        rootStyle,
        leftStyle,
        rightStyle,
    };
}
