"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeEditor = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
const tango_ui_1 = require("@music163/tango-ui");
const tango_context_1 = require("@music163/tango-context");
const tango_core_1 = require("@music163/tango-core");
const antd_1 = require("antd");
const ideConfig = {
    // disableFileOps: {
    //   add: true,
    //   delete: true,
    // },
    // disableFolderOps: {
    //   add: true,
    //   delete: true,
    //   rename: true,
    // },
    disablePrettier: true,
    disableEslint: true,
    saveWhenBlur: true,
    // disableSetting: true,
};
exports.CodeEditor = (0, tango_context_1.observer)((_a) => {
    var { autoRemoveUnusedImports = true } = _a, rest = tslib_1.__rest(_a, ["autoRemoveUnusedImports"]);
    const editorRef = (0, react_1.useRef)(null);
    const workspace = (0, tango_context_1.useWorkspace)();
    const files = workspace.listFiles();
    const activeFile = workspace.activeFile;
    let loc; // 记录视图代码的选中位置
    const selectNode = workspace.selectSource.firstNode;
    if (selectNode && activeFile === workspace.activeViewFile) {
        loc = selectNode.loc;
    }
    (0, react_1.useEffect)(() => {
        var _a;
        (_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.refresh(files, activeFile, loc);
    }, [files, activeFile, loc]);
    const fileSave = (0, react_1.useCallback)((path, value) => {
        if (!isJsFile(path)) {
            // 非 js 文件直接保存
            workspace.updateFile(path, value, autoRemoveUnusedImports);
            return;
        }
        // js 文件需要先检查语法，只有语法正确才会保存
        if ((0, tango_core_1.isValidCode)(value)) {
            workspace.updateFile(path, value, autoRemoveUnusedImports);
        }
        else {
            antd_1.Modal.confirm({
                title: '检测到代码中存在语法错误，暂时无法将代码同步给设计器，是否回退到安全代码？',
                onOk: () => {
                    var _a;
                    (_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.refresh(files, activeFile);
                },
                onCancel: () => { },
            });
        }
    }, [workspace, autoRemoveUnusedImports, activeFile, files]);
    const handleFileChange = (0, react_1.useCallback)((type, info) => {
        switch (type) {
            case 'addFile':
                workspace.addFile(info.path, info.value);
                break;
            case 'deleteFile':
            case 'deleteFolder':
                workspace.removeFile(info.path);
                break;
            case 'renameFile':
                workspace.renameFile(info.path, info.newpath);
                workspace.setActiveFile(info.newpath);
                break;
            case 'renameFolder':
                workspace.renameFolder(info.path, info.newpath);
                break;
            case 'addFolder':
            default:
                break;
        }
    }, [workspace]);
    const handlePathChange = (0, react_1.useCallback)((path) => {
        workspace.setActiveFile(path);
    }, [workspace]);
    return (react_1.default.createElement(coral_system_1.Box, { display: "flex", flexDirection: "row", height: "100%", bg: "white" },
        react_1.default.createElement(tango_ui_1.MultiEditor, Object.assign({ ref: editorRef, options: {
                fontSize: 14,
                automaticLayout: true,
            }, ideConfig: ideConfig, onFileSave: fileSave, onPathChange: handlePathChange, onFileChange: handleFileChange, defaultPath: activeFile, defaultTheme: "GithubLightDefault", defaultFiles: files }, rest))));
});
function isJsFile(path) {
    return /.jsx?$/.test(path);
}
