import React from 'react';
import { ComponentPropValidate, IComponentProp } from '@music163/tango-helpers';
import { InputProps } from 'antd';
import { ISetterOnChangeCallbackDetail } from './types';
export interface FormItemProps extends IComponentProp {
    /**
     * 无样式模式，仅返回 setter 组件
     */
    noStyle?: boolean;
    /**
     * 表单项右侧自定义区域
     */
    extra?: React.ReactNode;
    /**
     * 表单项底部自定义区域
     */
    footer?: React.ReactNode;
}
export interface FormItemComponentProps<T = any> {
    value?: T;
    onChange: (value: T, detail?: ISetterOnChangeCallbackDetail) => void;
    readOnly?: boolean;
    disabled?: boolean;
    status?: InputProps['status'];
    [prop: string]: any;
}
export interface IFormItemCreateOptions {
    /**
     * 设置器调用名
     */
    name: string;
    /**
     * 设置器别名列表，支持多个名字
     */
    alias?: string[];
    /**
     * 渲染设置器使用的组件
     */
    component?: React.ComponentType<FormItemComponentProps>;
    /**
     * 自定义渲染器 render 函数，render 优先级高于 component
     */
    render?: (args: FormItemComponentProps) => React.ReactElement;
    /**
     * 是否禁用变量设置器
     */
    disableVariableSetter?: boolean;
    /**
     * 默认的表单值校验逻辑
     */
    validate?: ComponentPropValidate;
}
export declare function createFormItem(options: IFormItemCreateOptions): {
    ({ name, title, tip, placeholder, docs, autoCompleteOptions, setter, setterProps, defaultValue, options: setterOptions, disableVariableSetter: disableVariableSetterProp, getVisible: getVisibleProp, getSetterProps: getSetterPropsProp, deprecated, extra, footer, noStyle, validate, }: FormItemProps): React.JSX.Element;
    displayName: string;
    config: IFormItemCreateOptions;
} & {
    displayName: string;
};
/**
 * Setter 注册
 * @param config 注册选项
 */
export declare function register(config: IFormItemCreateOptions): void;
export declare function SettingFormItem(props: FormItemProps): React.JSX.Element;
