"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SettingFormItem = exports.register = exports.createFormItem = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const mobx_1 = require("mobx");
const mobx_react_lite_1 = require("mobx-react-lite");
const tango_helpers_1 = require("@music163/tango-helpers");
const tango_core_1 = require("@music163/tango-core");
const tango_ui_1 = require("@music163/tango-ui");
const context_1 = require("./context");
const form_ui_1 = require("./form-ui");
const coral_system_1 = require("coral-system");
const defaultGetSetterProps = () => ({});
const defaultGetVisible = () => true;
function createFormItem(options) {
    var _a;
    const renderSetter = (_a = options.render) !== null && _a !== void 0 ? _a : ((props) => react_1.default.createElement(options.component, props));
    function FormItem({ name, title, tip, placeholder, docs, autoCompleteOptions, setter, setterProps, defaultValue, options: setterOptions, disableVariableSetter: disableVariableSetterProp = options.disableVariableSetter, getVisible: getVisibleProp, getSetterProps: getSetterPropsProp, deprecated, extra, footer, noStyle, validate, }) {
        var _a, _b, _c;
        const { disableSwitchExpressionSetter, showItemSubtitle } = (0, context_1.useFormVariable)();
        const model = (0, context_1.useFormModel)();
        const field = model.getField(name);
        const value = (0, mobx_1.toJS)((_a = field.value) !== null && _a !== void 0 ? _a : defaultValue);
        const disableVariableSetter = disableSwitchExpressionSetter !== null && disableSwitchExpressionSetter !== void 0 ? disableSwitchExpressionSetter : disableVariableSetterProp; // Form 的设置优先
        const [isVariable, { toggle: toggleIsVariable }] = (0, tango_helpers_1.useBoolean)(() => !disableVariableSetter && (0, tango_core_1.isWrappedByExpressionContainer)(value));
        const setterName = isVariable ? 'expressionSetter' : setter;
        field.setConfig({
            validate: validate || options.validate,
        });
        const baseComponentProps = (0, tango_helpers_1.clone)({
            value,
            defaultValue,
            onChange: field.handleChange,
            status: field.error ? 'error' : undefined,
            placeholder,
            options: setterOptions,
        }, false);
        let expProps = {};
        // FIXME: 重新考虑这段代码的位置，外置这个逻辑
        if (['expressionSetter', 'expSetter', 'actionSetter', 'eventSetter'].includes(setter) ||
            isVariable) {
            expProps = {
                modalTitle: title,
                modalTip: tip,
                autoCompleteOptions,
            };
        }
        const getSetterProps = getSetterPropsProp || defaultGetSetterProps;
        // 从注册表中获取 expSetter
        const ExpressionSetter = (_c = (_b = REGISTERED_FORM_ITEM_MAP['expressionSetter']) === null || _b === void 0 ? void 0 : _b.config) === null || _c === void 0 ? void 0 : _c.component;
        const setterNode = isVariable ? (react_1.default.createElement(ExpressionSetter, Object.assign({}, expProps, baseComponentProps))) : (renderSetter(Object.assign(Object.assign(Object.assign(Object.assign({}, expProps), baseComponentProps), setterProps), getSetterProps(model))));
        const getVisible = getVisibleProp || defaultGetVisible;
        if (noStyle) {
            // 无样式模式
            return getVisible(model) ? setterNode : react_1.default.createElement("div", { "data-setter": setterName, "data-field": name });
        }
        return (react_1.default.createElement(form_ui_1.FormControl, { visible: getVisible(model), label: title, note: showItemSubtitle ? name : null, tip: tip, docs: docs, deprecated: deprecated, error: field.error, extra: react_1.default.createElement(coral_system_1.Box, null,
                extra,
                !disableVariableSetter ? (react_1.default.createElement(tango_ui_1.ToggleButton, { borderRadius: "s", size: "s", shape: "text", type: "primary", tooltip: isVariable ? '关闭 JS 表达式' : '使用 JS 表达式', tooltipPlacement: "left", selected: isVariable, onClick: () => toggleIsVariable() },
                    react_1.default.createElement(tango_ui_1.CodeOutlined, null))) : null), footer: footer, "data-setter": setterName, "data-field": name }, setterNode));
    }
    FormItem.displayName = `FormItem_${options.name}`;
    FormItem.config = options;
    return (0, mobx_react_lite_1.observer)(FormItem);
}
exports.createFormItem = createFormItem;
// 已注册的 setter 查找表
const REGISTERED_FORM_ITEM_MAP = {};
/**
 * Setter 注册
 * @param config 注册选项
 */
function register(config) {
    var _a;
    const names = [config.name, ...((_a = config.alias) !== null && _a !== void 0 ? _a : [])];
    names.forEach((name) => {
        REGISTERED_FORM_ITEM_MAP[name] = createFormItem(config);
    });
}
exports.register = register;
function SettingFormItem(props) {
    const { setter } = props;
    const Comp = REGISTERED_FORM_ITEM_MAP[setter];
    if (Comp == null) {
        const Fallback = REGISTERED_FORM_ITEM_MAP.expressionSetter;
        return (react_1.default.createElement(Fallback, Object.assign({}, props, { footer: react_1.default.createElement(coral_system_1.Text, { color: "red", mt: "m" },
                props.setter,
                " is invalid") })));
    }
    return react_1.default.createElement(Comp, props);
}
exports.SettingFormItem = SettingFormItem;
