import { ISetterOnChangeCallbackDetail } from './types';
export interface IFormModel {
    values: any;
    getValue: (name: string, defaultValue?: any) => any;
    setValue: (name: string, value: any) => void;
    getField: (name: string) => Field;
    getSubModel: (name: string) => SubModel;
    onChange: (name: string, value: any, field?: Field) => void;
}
export interface FormModelOptionsType {
    onChange?: IFormModel['onChange'];
}
export declare class FormModel implements IFormModel {
    state: {};
    _values: any;
    _fieldMap: Map<string, Field>;
    _subModelMap: {
        [key: string]: SubModel;
    };
    onChange: (name: string, value: any, field?: Field) => void;
    get values(): any;
    set values(nextValues: any);
    constructor(initValues: any, options?: FormModelOptionsType);
    getValues(): any;
    getValue(name: string, defaultValue?: any): any;
    setValue(name: string, value: any): void;
    getField(name: string): Field;
    getSubModel(name: string): SubModel;
}
declare class SubModel implements IFormModel {
    parent: IFormModel;
    name: string;
    _fieldMap: Map<string, Field>;
    _subModelMap: {
        [key: string]: SubModel;
    };
    get values(): any;
    set values(nextValues: any);
    constructor(parent: IFormModel, name: string);
    getValue(name: string, defaultValue?: any): any;
    setValue(name: string, value: any): void;
    getField(name: string): Field;
    getSubModel(name: string): SubModel;
    getPath(name: string): string;
    onChange(name: string, values: any, field: Field): void;
}
interface FieldCreateOptionsType {
    parent: IFormModel;
    name: string;
}
interface FieldConfigType {
    validate?: (value: any, field: Field, trigger: string) => string | Promise<any>;
}
interface FieldStateType {
    error?: any;
    validating?: boolean;
    cancelValidation?: () => void;
    [key: string]: any;
}
type FieldValidateTriggerType = '*' | 'blur' | 'change';
export declare class Field {
    readonly parent: IFormModel;
    readonly name: string;
    readonly id: string;
    readonly _get: () => any;
    readonly _set: (v: any) => void;
    config: FieldConfigType;
    state: FieldStateType;
    detail: ISetterOnChangeCallbackDetail;
    get value(): any;
    set value(value: any);
    get error(): any;
    constructor(options: FieldCreateOptionsType);
    validate: (trigger?: FieldValidateTriggerType) => Promise<any>;
    handleBlur: () => Promise<any>;
    handleChange: (nextValue: any, valueDetail: any) => Promise<any>;
    setConfig(config: FieldConfigType): void;
}
export {};
