"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FormHeader = exports.FieldSet = exports.FormControlGroup = exports.FormControl = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
const antd_1 = require("antd");
const tango_ui_1 = require("@music163/tango-ui");
const tango_helpers_1 = require("@music163/tango-helpers");
const icons_1 = require("@ant-design/icons");
function FormControl(_a) {
    var { visible, label, note, tip, docs, extra, deprecated, footer, error, children } = _a, rest = tslib_1.__rest(_a, ["visible", "label", "note", "tip", "docs", "extra", "deprecated", "footer", "error", "children"]);
    return (react_1.default.createElement(coral_system_1.Box, Object.assign({ className: "FormControl", display: visible ? 'block' : 'none' }, rest),
        react_1.default.createElement(coral_system_1.Box, { display: "flex", justifyContent: "space-between", alignItems: "center", mb: "s" },
            react_1.default.createElement(FormLabel, { label: label, note: note, tip: tip, docs: docs, deprecated: deprecated }),
            extra),
        react_1.default.createElement(coral_system_1.Box, null, children),
        footer,
        !!error && (react_1.default.createElement(coral_system_1.Box, { mt: "m", color: "red" }, error))));
}
exports.FormControl = FormControl;
const formControlGroupContentStyle = (0, coral_system_1.css) `
  > .FormControlGroup {
    border-style: solid;
    border-width: 1px;
  }
`;
function FormControlGroup({ label, note, tip, docs, error, extra, children, onCheck, checked, }) {
    // 默认折叠
    const [collapsed, setCollapsed] = (0, react_1.useState)(true);
    return (react_1.default.createElement(tango_ui_1.CollapsePanel, { collapsed: collapsed, onCollapse: setCollapsed, className: "FormControlGroup", stickyHeader: true, title: react_1.default.createElement(coral_system_1.Box, { display: "flex", columnGap: "s" },
            react_1.default.createElement(antd_1.Checkbox, { checked: checked, onChange: (e) => {
                    e.stopPropagation();
                    const nextChecked = e.target.checked;
                    setCollapsed(!nextChecked);
                    onCheck === null || onCheck === void 0 ? void 0 : onCheck(nextChecked);
                } }),
            react_1.default.createElement(FormLabel, { label: label, note: note, tip: tip, docs: docs })), extra: react_1.default.createElement(coral_system_1.Box, { flex: "1", textAlign: "right" }, extra), border: "solid", borderColor: "line.normal", headerProps: {
            bg: 'fill1',
            zIndex: 2,
        } },
        react_1.default.createElement(coral_system_1.Box, { p: "m", display: "flex", flexDirection: "column", rowGap: "l", css: formControlGroupContentStyle }, children),
        react_1.default.createElement(coral_system_1.Box, { color: "red" }, error)));
}
exports.FormControlGroup = FormControlGroup;
const labelStyle = (0, coral_system_1.css) `
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
  white-space: nowrap;
  max-width: 160px;
  vertical-align: bottom;
  user-select: none;

  &.hasHelp {
    border-bottom: 1px dashed var(--tango-colors-text-normal);
  }
`;
const tipStyle = (0, coral_system_1.css) `
  a {
    text-decoration: underline;
  }

  a:link {
    color: #fff;
  }
`;
function FormLabel(_a) {
    var { label, note, tip, docs, deprecated } = _a, rest = tslib_1.__rest(_a, ["label", "note", "tip", "docs", "deprecated"]);
    let help;
    if (deprecated || docs) {
        help = (react_1.default.createElement(coral_system_1.Box, null,
            react_1.default.createElement(coral_system_1.Box, { css: tipStyle },
                tip,
                docs ? (react_1.default.createElement(coral_system_1.Link, { href: docs, isExternal: true, ml: "m" }, "\u67E5\u770B\u5C5E\u6027\u6587\u6863")) : null),
            deprecated ? (react_1.default.createElement(coral_system_1.Box, { color: "#faad14" },
                react_1.default.createElement(icons_1.WarningOutlined, null),
                " \u5E9F\u5F03\u63D0\u793A\uFF1A",
                (0, tango_helpers_1.isString)(deprecated) ? deprecated : '该属性已废弃，请谨慎使用。')) : null));
    }
    else {
        help = tip;
    }
    let labelNode = (react_1.default.createElement(coral_system_1.Box, { as: "span", display: "inline-block", color: "text2", className: help ? 'hasHelp' : '', css: labelStyle, title: (0, tango_helpers_1.isString)(label) ? label : undefined },
        deprecated ? react_1.default.createElement(icons_1.WarningOutlined, { style: { color: '#faad14', marginRight: 4 } }) : null,
        label));
    if (help) {
        labelNode = react_1.default.createElement(antd_1.Tooltip, { title: help }, labelNode);
    }
    return (react_1.default.createElement(coral_system_1.Box, Object.assign({ fontSize: "14px" }, rest),
        labelNode,
        note && (react_1.default.createElement(coral_system_1.Box, { as: "i", display: "inline-block", fontSize: "12px", color: "text3", ml: "m" }, note))));
}
function FieldSet({ title, extra, children }) {
    return (react_1.default.createElement(tango_ui_1.CollapsePanel, { title: title, extra: extra },
        react_1.default.createElement(coral_system_1.Box, { px: "m" }, children)));
}
exports.FieldSet = FieldSet;
function FormHeader({ title, extra, subTitle }) {
    return (react_1.default.createElement(coral_system_1.Box, { className: "FormHeader" },
        react_1.default.createElement(coral_system_1.Box, { display: "flex", alignItems: "center" },
            react_1.default.createElement(coral_system_1.Box, { flex: "1", display: "flex", alignItems: "center" },
                react_1.default.createElement(coral_system_1.Box, { fontSize: "16px", fontWeight: "500", mr: "s" }, title),
                subTitle && react_1.default.createElement(coral_system_1.Box, null, subTitle)),
            extra && react_1.default.createElement(coral_system_1.Box, null, extra))));
}
exports.FormHeader = FormHeader;
