"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SettingForm = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
const antd_1 = require("antd");
const tango_helpers_1 = require("@music163/tango-helpers");
const tango_ui_1 = require("@music163/tango-ui");
const form_item_1 = require("./form-item");
const context_1 = require("./context");
const form_model_1 = require("./form-model");
const form_object_1 = require("./form-object");
const helpers_1 = require("./helpers");
const setter_1 = require("./setter");
const form_ui_1 = require("./form-ui");
const icons_1 = require("@ant-design/icons");
(0, setter_1.registerBuiltinSetters)();
function normalizeComponentProps(props, groupOptions) {
    const groups = groupOptions.reduce((prev, cur) => {
        prev[cur.value] = [];
        return prev;
    }, {});
    props.forEach((prop) => {
        const group = groups[prop.group] ? prop.group : 'basic';
        groups[group].push(prop);
    });
    return groups;
}
function filterComponentProps(props, keyword) {
    if (!keyword) {
        return props;
    }
    const pattern = new RegExp(keyword, 'ig');
    return (0, tango_helpers_1.filterTreeData)(props, (prop) => pattern.test(prop.title) || pattern.test(prop.name), 'props');
}
const formStyle = (0, coral_system_1.css) `
  position: relative;
  height: 100%;
  display: flex;
  flex-direction: column;
  > .SettingFormMain > :first-child:is(.FormControl) {
    margin-top: var(--tango-space-m);
  }

  > .SettingFormMain > .FormControl {
    margin-left: var(--tango-space-m);
    margin-right: var(--tango-space-m);
  }

  > .SettingFormMain > .FormObject {
    margin-left: var(--tango-space-m);
    margin-right: var(--tango-space-m);
  }
`;
const internalGroups = [
    { label: '基本', value: 'basic' },
    // { label: '事件', value: 'event' },
    { label: '样式', value: 'style' },
    { label: '高级', value: 'advanced' },
];
function SettingForm({ prototype, disableSwitchExpressionSetter, model: modelProp, defaultValue, onChange = tango_helpers_1.noop, showIdentifier = false, showSearch = true, showGroups = true, showItemSubtitle = true, renderItemExtra, groupOptions = internalGroups, }) {
    const [keyword, setKeyword] = (0, react_1.useState)('');
    const [tabKey, setTabKey] = (0, react_1.useState)('basic');
    const [model, setModel] = (0, react_1.useState)(modelProp !== null && modelProp !== void 0 ? modelProp : new form_model_1.FormModel(defaultValue, { onChange }));
    const { props: componentProps = [] } = prototype;
    const componentPropGroups = (0, react_1.useMemo)(() => {
        return normalizeComponentProps(componentProps, groupOptions);
    }, [componentProps, groupOptions]);
    const filterProps = (0, react_1.useMemo)(() => {
        const computedProps = (0, tango_helpers_1.mapTreeData)(componentProps, (node) => {
            const { getProp } = node, rest = tslib_1.__rest(node, ["getProp"]);
            return Object.assign(Object.assign({}, rest), getProp === null || getProp === void 0 ? void 0 : getProp(model));
        });
        // 没有进行搜索的情况
        if (!keyword) {
            if (showGroups) {
                return (componentPropGroups[tabKey].sort((a, b) => {
                    // TIP: 将废弃属性往后移
                    if (!a.deprecated && b.deprecated) {
                        return -1;
                    }
                    return 1;
                }) || []);
            }
            else {
                return computedProps;
            }
        }
        // 用户进行搜索的情况
        return filterComponentProps(computedProps, keyword);
    }, [keyword, componentProps, tabKey, model, showGroups, componentPropGroups]);
    const renderProps = (0, react_1.useCallback)((list) => list.map((item) => {
        const { getProp } = item, rest = tslib_1.__rest(item, ["getProp"]);
        const childProp = Object.assign(Object.assign({}, rest), getProp === null || getProp === void 0 ? void 0 : getProp(model));
        if ((0, helpers_1.isValidNestProps)(childProp.props)) {
            return react_1.default.createElement(form_object_1.SettingFormObject, Object.assign({ key: item.name }, childProp));
        }
        else {
            return (react_1.default.createElement(form_item_1.SettingFormItem, Object.assign({ key: item.name, extra: renderItemExtra === null || renderItemExtra === void 0 ? void 0 : renderItemExtra(childProp) }, childProp)));
        }
    }), [model, renderItemExtra]);
    (0, react_1.useEffect)(() => {
        // 当外部传入的 model 变化时，同步新的 model
        if (modelProp && modelProp !== model) {
            setModel(modelProp);
        }
    }, [modelProp]);
    return (react_1.default.createElement(context_1.FormVariableProvider, { value: { disableSwitchExpressionSetter, showItemSubtitle } },
        react_1.default.createElement(context_1.FormModelProvider, { value: model },
            react_1.default.createElement(coral_system_1.Box, { className: "SettingForm", mb: "xl", css: formStyle },
                react_1.default.createElement(coral_system_1.Box, { className: "SettingFormHeader", bg: "white" },
                    react_1.default.createElement(coral_system_1.Box, { px: "l", py: "m" },
                        showIdentifier && (react_1.default.createElement(form_ui_1.FormHeader, { title: prototype.title, subTitle: react_1.default.createElement(form_item_1.SettingFormItem, { noStyle: true, setter: "idSetter", name: showIdentifier.identifierKey }), extra: (prototype === null || prototype === void 0 ? void 0 : prototype.docs) ? (react_1.default.createElement(tango_ui_1.Action, { icon: react_1.default.createElement(icons_1.QuestionCircleOutlined, null), tooltip: "\u67E5\u770B\u7EC4\u4EF6\u6587\u6863", href: prototype.docs })) : null })),
                        showSearch && (react_1.default.createElement(coral_system_1.Box, { className: "SettingFormSearch", mt: "m" },
                            react_1.default.createElement(tango_ui_1.Search, { placeholder: "\u641C\u7D22\u5C5E\u6027", onChange: (val) => {
                                    setKeyword(val === null || val === void 0 ? void 0 : val.trim());
                                } })))),
                    !keyword && showGroups && (react_1.default.createElement(coral_system_1.Box, { className: "SettingFormNav" },
                        react_1.default.createElement(tango_ui_1.Tabs, { activeKey: tabKey, onChange: setTabKey }, groupOptions.map((group) => (react_1.default.createElement(tango_ui_1.Tabs.TabPane, { key: group.value, tab: group.label, disabled: !componentPropGroups[group.value].length }))))))),
                react_1.default.createElement(coral_system_1.Box, { className: "SettingFormMain", display: "flex", flexDirection: "column", paddingBottom: "m", rowGap: "m", overflow: "auto" },
                    renderProps(filterProps),
                    filterProps.length === 0 && (react_1.default.createElement(antd_1.Empty, { image: antd_1.Empty.PRESENTED_IMAGE_SIMPLE, description: "\u672A\u627E\u5230\u914D\u7F6E\u9879" })))))));
}
exports.SettingForm = SettingForm;
SettingForm.register = form_item_1.register;
