"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isWrappedByExpressionContainer = exports.isTangoVariable = exports.isDefineStore = exports.isDefineService = void 0;
const ast_1 = require("./ast");
const defineServiceHandlerNames = ['defineServices', 'createServices'];
const sfHandlerPattern = new RegExp(`^(${defineServiceHandlerNames.join('|')})$`);
/**
 * 判断给定的函数名是否是 defineServices
 * @param name
 * @returns
 */
function isDefineService(name) {
    return sfHandlerPattern.test(name);
}
exports.isDefineService = isDefineService;
const defineStoreHandlerName = 'defineStore';
/**
 * 判断给定的函数名是否是 defineStore
 * @param name
 * @returns
 */
function isDefineStore(name) {
    return defineStoreHandlerName === name;
}
exports.isDefineStore = isDefineStore;
/**
 * 是否是 tango 的变量引用
 * @example tango.stores.app.name
 * @example tango.stores?.app?.name
 *
 * @param name
 * @returns
 */
function isTangoVariable(name) {
    return /^tango\??\.(stores|services)\??\./.test(name) && name.split('.').length > 2;
}
exports.isTangoVariable = isTangoVariable;
const templatePattern = /^{(.+)}$/s;
/**
 * 判断给定字符串是否被表达式容器`{expCode}`包裹
 * @param code
 * @deprecated 新版改为 {{code}} 作为容器，使用 isWrappedCode 代替
 */
function isWrappedByExpressionContainer(code, isStrict = true) {
    if (isStrict && (0, ast_1.isValidExpressionCode)(code)) {
        return false;
    }
    return templatePattern.test(code);
}
exports.isWrappedByExpressionContainer = isWrappedByExpressionContainer;
