"use strict";
/**
 * 访问，遍历，修改 ast tree
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.traverseFile = exports.traverseComponentsEntryFile = exports.traverseViewFile = exports.cloneJSXElement = exports.removeUnusedImportSpecifiers = exports.traverseServiceFile = exports.updateBaseConfigToServiceFile = exports.updateServiceConfigToServiceFile = exports.serviceConfig2Node = exports.deleteServiceConfigFromServiceFile = exports.traverseStoreFile = exports.updateStoreState = exports.removeStoreState = exports.addStoreState = exports.traverseStoreEntryFile = exports.removeStoreToEntryFile = exports.addStoreToEntryFile = exports.traverseRouteFile = exports.removeRouteFromRouteFile = exports.updateRouteToRouteFile = exports.addRouteToRouteFile = exports.updateClassProperty = exports.addClassProperty = exports.insertImportSpecifiers = exports.updateImportDeclaration = exports.updateImportDeclarationLegacy = exports.addImportDeclaration = exports.addImportDeclarationLegacy = exports.makeImportDeclaration = exports.makeImportDeclarationLegacy = exports.updateJSXAttributes = exports.updateJSXElementAttribute = exports.replaceRootJSXElementChildren = exports.removeJSXElement = exports.replaceJSXElement = exports.insertSiblingBeforeJSXElement = exports.insertSiblingAfterJSXElement = exports.appendChildToJSXElement = exports.isJSXElementById = exports.queryXFormItemFields = exports.getJSXElementChildrenNames = exports.getJSXElementAttributes = exports.traverseExpressionNode = void 0;
const tslib_1 = require("tslib");
const t = tslib_1.__importStar(require("@babel/types"));
const traverse_1 = tslib_1.__importDefault(require("@babel/traverse"));
const tango_helpers_1 = require("@music163/tango-helpers");
const generate_1 = require("./generate");
const parse_1 = require("./parse");
const string_1 = require("../string");
const assert_1 = require("../assert");
/**
 * 将 Node 节点构造为一棵树，并执行树的遍历操作，此操作会修改原始的节点
 * @param node
 * @param options
 * @returns 返回完整的 File AST
 */
function traverseExpressionNode(node, options) {
    const file = t.file(t.program([t.blockStatement([t.expressionStatement(node)])]));
    (0, traverse_1.default)(file, options);
    return file;
}
exports.traverseExpressionNode = traverseExpressionNode;
/**
 * 遍历 jsxElement 的 attributes 集合
 * @param node
 * @param visitCallback
 */
function visitJSXElementAttributes(node, visitCallback) {
    var _a;
    node.openingElement.attributes.forEach((attrNode) => {
        if ((0, tango_helpers_1.isFunction)(visitCallback) && attrNode.type === 'JSXAttribute') {
            const name = (0, generate_1.keyNode2value)(attrNode.name);
            const value = (0, generate_1.jsxAttributeValueNode2value)(attrNode.value);
            visitCallback(name, value, attrNode);
        }
    });
    // TIP: 如果 children 节点为简单数据类型，则也作为属性值
    if (node.children.length === 1) {
        const onlyChild = node.children[0];
        let nodeValue;
        if (t.isJSXText(onlyChild)) {
            nodeValue = (_a = onlyChild.value) === null || _a === void 0 ? void 0 : _a.trim();
        }
        else if (t.isJSXExpressionContainer(onlyChild)) {
            nodeValue = (0, generate_1.jsxAttributeValueNode2value)(onlyChild);
        }
        if (nodeValue) {
            visitCallback('children', nodeValue, node.children);
        }
    }
}
/**
 * 删除 jsxElement 的目标属性
 * @param node
 * @param attrName
 */
function removeJSXElementAttributeByName(node, attrName) {
    node.openingElement.attributes = node.openingElement.attributes.filter((attrNode) => {
        if (t.isJSXAttribute(attrNode)) {
            const name = (0, generate_1.keyNode2value)(attrNode.name);
            return name !== attrName;
        }
        return true;
    });
}
/**
 * 获取 jsx 节点的属性集
 * @param node
 * @returns
 */
function getJSXElementAttributes(node) {
    const ret = {};
    visitJSXElementAttributes(node, (name, value) => {
        ret[name] = value;
    });
    return ret;
}
exports.getJSXElementAttributes = getJSXElementAttributes;
/**
 * 获取 jsx 结点的子元素名字
 * @param node
 * @returns
 */
function getJSXElementChildrenNames(node) {
    const names = new Set();
    traverseExpressionNode(node, {
        JSXElement(path) {
            const name = getJSXElementName(path.node);
            // invalid name
            if (!name) {
                return;
            }
            // html tags, e.g. div, span...
            if (name.toLowerCase() === name) {
                return;
            }
            const parts = name.split('.');
            // 嵌套类型 Button.Group，只取父级
            if (parts.length) {
                names.add(parts[0]);
            }
        },
    });
    return Array.from(names);
}
exports.getJSXElementChildrenNames = getJSXElementChildrenNames;
/**
 * 匹配名字叫 targetName 的组件
 * @param ast
 * @param targetName
 * @param callback
 */
function visitJSXElementByName(ast, targetName, callback) {
    const visitors = {
        JSXElement(path) {
            if (getJSXElementName(path.node) === targetName) {
                callback(path.node);
            }
        },
    };
    switch (ast.type) {
        case 'File':
            (0, traverse_1.default)(ast, visitors);
            break;
        case 'JSXElement':
            traverseExpressionNode(ast, visitors);
    }
}
/**
 * 查询所有XFormItem的字段名
 * @param ast 当前文件AST
 * @returns 所有 XFormItem 的字段名
 */
function queryXFormItemFields(ast) {
    const fields = [];
    visitJSXElementByName(ast, 'XFormItem', (node) => {
        const attrs = getJSXElementAttributes(node);
        if (attrs.name && !fields.includes(attrs.name)) {
            fields.push(attrs.name);
        }
    });
    return fields;
}
exports.queryXFormItemFields = queryXFormItemFields;
/**
 * 根据 dnd id 判断是否是目标的 JSXElement
 * @param node
 * @param id
 */
function isJSXElementById(node, jsxElementNodeId) {
    let isTargetJSXElement = false;
    visitJSXElementAttributes(node, (name, value) => {
        if (name === tango_helpers_1.SLOT.dnd && value === jsxElementNodeId) {
            isTargetJSXElement = true;
        }
    });
    return isTargetJSXElement;
}
exports.isJSXElementById = isJSXElementById;
/**
 * 在目标 jsx 元素中添加子元素
 * @param ast
 * @param targetJSXElementNodeId
 * @param newNode
 * @returns
 */
function appendChildToJSXElement(ast, targetJSXElementNodeId, newNode, position) {
    (0, traverse_1.default)(ast, {
        JSXElement(path) {
            if (isJSXElementById(path.node, targetJSXElementNodeId)) {
                // 强制变为非闭合标签 & 避免children添加失败
                if (path.node.openingElement.selfClosing) {
                    path.node.openingElement.selfClosing = false;
                    path.node.closingElement = t.jSXClosingElement(path.node.openingElement.name);
                }
                if (position === 'last') {
                    path.node.children.push(newNode);
                }
                else {
                    path.node.children.unshift(newNode);
                }
                path.stop();
            }
        },
    });
    return ast;
}
exports.appendChildToJSXElement = appendChildToJSXElement;
/**
 * 在目标 jsx 元素后添加兄弟节点
 * @param ast
 * @param targetJSXElementNodeId
 * @param newNode
 */
function insertSiblingAfterJSXElement(ast, targetJSXElementNodeId, newNode) {
    (0, traverse_1.default)(ast, {
        JSXElement(path) {
            if (isJSXElementById(path.node, targetJSXElementNodeId)) {
                if (path.parentPath.isJSXElement()) {
                    path.insertAfter(newNode);
                }
                else if (path.parentPath.isJSXExpressionContainer()) {
                    const fragmentNode = t.jsxFragment(t.jsxOpeningFragment(), t.jsxClosingFragment(), [
                        t.cloneDeepWithoutLoc(path.node),
                        newNode,
                    ]);
                    path.replaceWith(fragmentNode);
                }
                path.stop();
            }
        },
    });
    return ast;
}
exports.insertSiblingAfterJSXElement = insertSiblingAfterJSXElement;
/**
 * 在目标 jsx 元素前添加兄弟节点
 * @param ast
 * @param targetJSXElementNodeId
 * @param newNode
 * @returns
 */
function insertSiblingBeforeJSXElement(ast, targetJSXElementNodeId, newNode) {
    (0, traverse_1.default)(ast, {
        JSXElement(path) {
            if (isJSXElementById(path.node, targetJSXElementNodeId)) {
                path.insertBefore(newNode);
                path.stop();
            }
        },
    });
    return ast;
}
exports.insertSiblingBeforeJSXElement = insertSiblingBeforeJSXElement;
/**
 * 替换目标 jsx 元素为新的元素
 * @param ast
 * @param targetJSXElementNodeId
 * @param newNode
 * @returns
 */
function replaceJSXElement(ast, targetJSXElementNodeId, newNode) {
    (0, traverse_1.default)(ast, {
        JSXElement(path) {
            if (isJSXElementById(path.node, targetJSXElementNodeId)) {
                path.replaceWith(newNode);
                path.stop();
            }
        },
    });
    return ast;
}
exports.replaceJSXElement = replaceJSXElement;
/**
 * 删除目标 jsx 元素
 * @param ast
 * @param targetJSXElementNodeId
 */
function removeJSXElement(ast, targetJSXElementNodeId) {
    (0, traverse_1.default)(ast, {
        JSXElement(path) {
            if (isJSXElementById(path.node, targetJSXElementNodeId)) {
                const { parentPath } = path;
                const ancestorPath = path.parentPath.parentPath;
                if (t.isJSXElement(parentPath.node)) {
                    path.remove();
                }
                else if (t.isJSXAttribute(ancestorPath.node)) {
                    // 如果祖先节点在 jsxAttribute 上，则删除该属性
                    ancestorPath.remove();
                }
                else {
                    tango_helpers_1.logger.error('removeJSXElement failed', path.node);
                }
                path.stop();
            }
        },
    });
    return ast;
}
exports.removeJSXElement = removeJSXElement;
/**
 * 替换跟结点的子元素
 * @example 例如替换 `return (<Box {...props}>empty block</Box>)` 的子元素
 * @param ast
 * @param nodes
 */
function replaceRootJSXElementChildren(ast, nodes) {
    (0, traverse_1.default)(ast, {
        JSXElement(path) {
            path.node.children = nodes;
            path.stop();
        },
    });
    return ast;
}
exports.replaceRootJSXElementChildren = replaceRootJSXElementChildren;
/**
 * 新增 JSXElement 的属性节点
 * @param node
 * @param name
 * @param value
 */
function addJSXElementAttribute(node, name, value) {
    if (name === 'children' && node.children) {
        // jsx children 的情况
        node.children = (0, parse_1.value2jsxChildrenValueNode)(value);
    }
    else {
        // basic attributes
        const jsxAttributeNode = t.jsxAttribute(t.jsxIdentifier(name), (0, parse_1.value2jsxAttributeValueNode)(value));
        node.openingElement.attributes.push(jsxAttributeNode);
    }
    return node;
}
function updateJSXElementAttribute(node, attrName, attrValue) {
    const isUndefinedValue = attrValue === undefined;
    let isExist = false;
    if (isUndefinedValue) {
        // 清空该属性
        removeJSXElementAttributeByName(node, attrName);
    }
    else {
        // 更新该属性
        visitJSXElementAttributes(node, (name, prevValue, jsxAttributeNode) => {
            if (name === attrName) {
                isExist = true;
                // @ts-ignore
                if (t.isJSXAttribute(jsxAttributeNode)) {
                    jsxAttributeNode.value = (0, parse_1.value2jsxAttributeValueNode)(attrValue);
                }
                else if (name === 'children') {
                    node.children = (0, parse_1.value2jsxChildrenValueNode)(attrValue);
                }
            }
        });
    }
    if (!isExist && !isUndefinedValue) {
        // 添加该属性
        addJSXElementAttribute(node, attrName, attrValue);
    }
    return node;
}
exports.updateJSXElementAttribute = updateJSXElementAttribute;
function updateJSXAttributes(ast, nodeId, config) {
    (0, traverse_1.default)(ast, {
        JSXElement(path) {
            if (isJSXElementById(path.node, nodeId)) {
                Object.keys(config).forEach((attrName) => {
                    updateJSXElementAttribute(path.node, attrName, config[attrName]);
                });
                path.stop();
            }
        },
    });
    return ast;
}
exports.updateJSXAttributes = updateJSXAttributes;
/**
 * 从 ast 中找到最后一次 import 的位置的序号
 */
function getLastImportDeclarationIndex(node) {
    let lastImportIndex = 0;
    node.body.forEach((item, index) => {
        if (item.type === 'ImportDeclaration') {
            lastImportIndex = index + 1;
        }
    });
    return lastImportIndex;
}
/**
 * 从 ast 中找到最后一次 export 的位置的序号
 */
function getLastExportDeclarationIndex(node) {
    let lastExportIndex = 0;
    node.body.forEach((item, index) => {
        if (item.type === 'ExportNamedDeclaration') {
            lastExportIndex = index + 1;
        }
    });
    return lastExportIndex;
}
/**
 * 生成新的 ImportDeclaration
 * @deprecated 使用 makeImportDeclaration2 代替
 * @param importedModule
 * @param importedSourcePath
 * @returns
 */
function makeImportDeclarationLegacy(importedModule) {
    var _a;
    const specifierNodes = ((_a = importedModule.specifiers) === null || _a === void 0 ? void 0 : _a.map((localName) => t.importSpecifier(t.identifier(localName), t.identifier(localName)))) || [];
    let defaultSpecifierNode;
    if (importedModule.defaultSpecifier) {
        defaultSpecifierNode = t.importDefaultSpecifier(t.identifier(importedModule.defaultSpecifier));
    }
    const newSpecifierNodes = [defaultSpecifierNode, ...specifierNodes].filter((spec) => !!spec);
    return t.importDeclaration(newSpecifierNodes, t.stringLiteral(importedModule.sourcePath));
}
exports.makeImportDeclarationLegacy = makeImportDeclarationLegacy;
function specifierData2node(data) {
    switch (data.type) {
        case 'ImportDefaultSpecifier':
            return t.importDefaultSpecifier(t.identifier(data.localName));
        case 'ImportSpecifier':
            return t.importSpecifier(t.identifier(data.importedName || data.localName), t.identifier(data.localName));
        case 'ImportNamespaceSpecifier':
            return t.importNamespaceSpecifier(t.identifier(data.localName));
        default:
            return;
    }
}
function specifierDataList2nodes(specifiers) {
    return specifiers.map((item) => specifierData2node(item)).filter((item) => !!item);
}
function makeImportDeclaration(source, specifiers) {
    const specifierNodes = specifierDataList2nodes(specifiers);
    return t.importDeclaration(specifierNodes, t.stringLiteral(source));
}
exports.makeImportDeclaration = makeImportDeclaration;
/**
 * @deprecated 使用 parseImportDeclaration 代替
 * @param node
 * @returns
 */
function getImportDeclarationData(node) {
    const sourcePath = (0, generate_1.node2value)(node.source);
    let defaultSpecifier;
    const specifiers = [];
    node.specifiers.forEach((specifier) => {
        if (specifier.type === 'ImportDefaultSpecifier') {
            defaultSpecifier = (0, generate_1.keyNode2value)(specifier.local);
        }
        else if (specifier.type === 'ImportSpecifier') {
            specifiers.push((0, generate_1.keyNode2value)(specifier.local));
        }
    });
    return {
        defaultSpecifier,
        specifiers,
        sourcePath,
    };
}
/**
 * 添加新的导入语句
 * @deprecated 使用 addImportDeclaration2 代替
 * @param ast
 * @param importedModule
 * @param importedSourcePath
 * @returns
 */
function addImportDeclarationLegacy(ast, importedModule) {
    (0, traverse_1.default)(ast, {
        Program(path) {
            const lastIndex = getLastImportDeclarationIndex(path.node);
            const newImportDeclaration = makeImportDeclarationLegacy(importedModule);
            path.node.body.splice(lastIndex, 0, newImportDeclaration);
            path.stop();
        },
    });
    return ast;
}
exports.addImportDeclarationLegacy = addImportDeclarationLegacy;
function addImportDeclaration(ast, source, specifiers) {
    (0, traverse_1.default)(ast, {
        Program(path) {
            const lastIndex = getLastImportDeclarationIndex(path.node);
            const newImportDeclaration = makeImportDeclaration(source, specifiers);
            path.node.body.splice(lastIndex, 0, newImportDeclaration);
            path.stop();
        },
    });
    return ast;
}
exports.addImportDeclaration = addImportDeclaration;
/**
 * 更新已有的导入语句
 * @deprecated 使用 updateImportDeclaration2 代替
 * @param ast
 * @param importedModule
 * @param importedSourcePath
 * @returns
 */
function updateImportDeclarationLegacy(ast, importedModule) {
    (0, traverse_1.default)(ast, {
        ImportDeclaration(path) {
            const currentSourcePath = (0, generate_1.node2value)(path.node.source);
            if (currentSourcePath === importedModule.sourcePath) {
                const newImportDeclaration = makeImportDeclarationLegacy(importedModule);
                path.replaceWith(newImportDeclaration);
                path.stop(); // 只修改匹配到的第一条
            }
        },
    });
    return ast;
}
exports.updateImportDeclarationLegacy = updateImportDeclarationLegacy;
function updateImportDeclaration(ast, source, specifiers) {
    (0, traverse_1.default)(ast, {
        ImportDeclaration(path) {
            const currentSourcePath = (0, generate_1.node2value)(path.node.source);
            if (currentSourcePath === source) {
                const newImportDeclaration = makeImportDeclaration(source, specifiers);
                path.replaceWith(newImportDeclaration);
                path.stop(); // 只修改匹配到的第一条
            }
        },
    });
    return ast;
}
exports.updateImportDeclaration = updateImportDeclaration;
/**
 * 再已有的导入语句中添加新的导入符号
 * @param ast
 * @param source
 * @param newSpecifiers
 * @returns
 */
function insertImportSpecifiers(ast, source, newSpecifiers) {
    (0, traverse_1.default)(ast, {
        ImportDeclaration(path) {
            const currentSourcePath = (0, generate_1.node2value)(path.node.source);
            if (currentSourcePath === source) {
                const nodes = specifierDataList2nodes(newSpecifiers);
                path.node.specifiers.push(...nodes);
                // 只在匹配到的第一个导入声明语句添加即可，不再重复执行
                path.stop();
            }
        },
    });
    return ast;
}
exports.insertImportSpecifiers = insertImportSpecifiers;
/**
 * 新增类属性
 * @param ast
 * @param classPropertyNode
 * @returns
 */
function addClassProperty(ast, classPropertyNode) {
    (0, traverse_1.default)(ast, {
        ClassBody(path) {
            path.node.body.push(classPropertyNode);
            path.stop();
        },
    });
    return ast;
}
exports.addClassProperty = addClassProperty;
/**
 * 更新类属性
 * @param ast
 * @param targetPropertyName
 * @param propertyValueNode
 * @returns
 */
function updateClassProperty(ast, targetPropertyName, propertyValueNode) {
    (0, traverse_1.default)(ast, {
        ClassProperty(path) {
            const name = (0, generate_1.keyNode2value)(path.node.key);
            if (name === targetPropertyName) {
                path.node.value = propertyValueNode;
                path.stop();
            }
        },
    });
    return ast;
}
exports.updateClassProperty = updateClassProperty;
/**
 * 在 routes.js 中添加新的路由规则
 * @param ast
 * @param pageName
 * @returns
 */
function addRouteToRouteFile(ast, routePath, importFilePath) {
    if (/.jsx?$/.test(importFilePath)) {
        importFilePath = importFilePath.split('.')[0];
    }
    const component = (0, tango_helpers_1.upperCamelCase)(routePath.split('/').join('-'));
    (0, traverse_1.default)(ast, {
        Program(path) {
            const lastImportIndex = getLastImportDeclarationIndex(path.node);
            path.node.body.splice(lastImportIndex, 0, t.importDeclaration([t.importDefaultSpecifier(t.identifier(component))], t.stringLiteral(importFilePath.replace('/src', '.'))));
        },
        ArrayExpression(path) {
            const newNode = t.objectExpression([
                t.objectProperty(t.identifier('path'), t.stringLiteral(routePath)),
                t.objectProperty(t.identifier('component'), t.identifier(component)),
                t.objectProperty(t.identifier('exact'), t.booleanLiteral(true)),
            ]);
            path.node.elements.push(newNode);
        },
    });
    return ast;
}
exports.addRouteToRouteFile = addRouteToRouteFile;
/**
 * 更新页面路由
 * @param ast
 * @param oldRoutePath
 * @param newRoutePath
 * @returns
 */
function updateRouteToRouteFile(ast, oldRoutePath, newRoutePath) {
    (0, traverse_1.default)(ast, {
        ObjectExpression(path) {
            path.node.properties.forEach((prop) => {
                if (t.isObjectProperty(prop) &&
                    (0, generate_1.node2value)(prop.key, false) === 'path' &&
                    (0, generate_1.node2value)(prop.value, false) === oldRoutePath) {
                    prop.value = t.stringLiteral(newRoutePath);
                }
            });
        },
    });
    return ast;
}
exports.updateRouteToRouteFile = updateRouteToRouteFile;
function removeRouteFromRouteFile(ast, routePath, importPath) {
    (0, traverse_1.default)(ast, {
        ArrayExpression(path) {
            path.node.elements = path.node.elements.filter((element) => {
                if (element.type === 'ObjectExpression') {
                    const properties = (0, generate_1.node2value)(element);
                    if (properties.path === routePath) {
                        return false;
                    }
                }
                return true;
            });
        },
        ImportDeclaration(path) {
            const sourceValue = (0, generate_1.node2value)(path.node.source);
            if (sourceValue === importPath) {
                path.remove();
            }
        },
    });
    return ast;
}
exports.removeRouteFromRouteFile = removeRouteFromRouteFile;
function traverseRouteFile(ast) {
    const routes = [];
    const importMap = {};
    (0, traverse_1.default)(ast, {
        ImportDeclaration(path) {
            const { defaultSpecifier, sourcePath } = getImportDeclarationData(path.node);
            if (defaultSpecifier) {
                importMap[defaultSpecifier] = sourcePath;
            }
        },
        ObjectExpression(path) {
            const { node } = path;
            const route = (0, generate_1.node2value)(node, false);
            routes.push(route);
        },
    });
    routes.forEach((item) => {
        if (item.component && importMap[item.component]) {
            item.importPath = importMap[item.component];
        }
    });
    return routes;
}
exports.traverseRouteFile = traverseRouteFile;
/**
 * 在 stores/index.js 中添加新的store
 * @param ast
 * @param storeName
 * @returns
 */
function addStoreToEntryFile(ast, storeName) {
    const filepath = `./${storeName}`;
    const component = storeName;
    (0, traverse_1.default)(ast, {
        Program(path) {
            const lastExportIndex = getLastExportDeclarationIndex(path.node);
            path.node.body.splice(lastExportIndex, 0, t.exportNamedDeclaration(null, [t.exportSpecifier(t.identifier('default'), t.identifier(component))], t.stringLiteral(filepath)));
        },
    });
    return ast;
}
exports.addStoreToEntryFile = addStoreToEntryFile;
/**
 * 在 stores/index.js 中移除一行导出语句
 * @example export { default as app } from "./app";
 * @param ast
 * @param storeName
 * @returns
 */
function removeStoreToEntryFile(ast, storeName) {
    (0, traverse_1.default)(ast, {
        ExportNamedDeclaration(path) {
            var _a;
            const name = (0, generate_1.keyNode2value)((_a = path.node.specifiers) === null || _a === void 0 ? void 0 : _a[0].exported);
            if (name === storeName) {
                path.remove();
                path.stop();
            }
        },
    });
    return ast;
}
exports.removeStoreToEntryFile = removeStoreToEntryFile;
function traverseStoreEntryFile(ast) {
    const stores = [];
    (0, traverse_1.default)(ast, {
        ExportSpecifier(path) {
            const name = (0, generate_1.keyNode2value)(path.node.exported);
            stores.push(name);
        },
    });
    return stores;
}
exports.traverseStoreEntryFile = traverseStoreEntryFile;
function addStoreState(ast, stateName, initValue) {
    (0, traverse_1.default)(ast, {
        CallExpression(path) {
            const { node } = path;
            const calleeName = (0, generate_1.keyNode2value)(node.callee);
            if ((0, assert_1.isDefineStore)(calleeName) &&
                node.arguments.length &&
                t.isObjectExpression(node.arguments[0])) {
                node.arguments[0].properties.push(t.objectProperty(t.identifier(stateName), (0, parse_1.code2expression)(initValue)));
                path.stop();
            }
        },
    });
    return ast;
}
exports.addStoreState = addStoreState;
function removeStoreState(ast, stateName) {
    (0, traverse_1.default)(ast, {
        CallExpression(path) {
            const { node } = path;
            const calleeName = (0, generate_1.keyNode2value)(node.callee);
            if ((0, assert_1.isDefineStore)(calleeName) &&
                node.arguments.length &&
                t.isObjectExpression(node.arguments[0])) {
                let { properties } = node.arguments[0];
                properties = properties.filter((prop) => {
                    if (prop.type === 'SpreadElement') {
                        return true;
                    }
                    const propName = (0, generate_1.keyNode2value)(prop.key);
                    return propName !== stateName;
                });
                node.arguments[0].properties = properties;
            }
        },
    });
    return ast;
}
exports.removeStoreState = removeStoreState;
function updateStoreState(ast, stateName, code) {
    (0, traverse_1.default)(ast, {
        CallExpression(path) {
            const { node } = path;
            const calleeName = (0, generate_1.keyNode2value)(node.callee);
            if ((0, assert_1.isDefineStore)(calleeName) &&
                node.arguments.length &&
                t.isObjectExpression(node.arguments[0])) {
                const { properties } = node.arguments[0];
                for (const prop of properties) {
                    if (prop.type === 'SpreadElement') {
                        continue;
                    }
                    const propName = (0, generate_1.keyNode2value)(prop.key);
                    if (propName === stateName) {
                        // TIP: 仅支持对象属性，方法必须需要写为箭头函数
                        if (prop.type === 'ObjectProperty') {
                            prop.value = (0, parse_1.code2expression)(code);
                            break;
                        }
                    }
                }
            }
        },
    });
    return ast;
}
exports.updateStoreState = updateStoreState;
function traverseStoreFile(ast) {
    let namespace;
    const actions = [];
    const states = [];
    (0, traverse_1.default)(ast, {
        CallExpression(path) {
            const { node } = path;
            const calleeName = (0, generate_1.keyNode2value)(node.callee);
            if ((0, assert_1.isDefineStore)(calleeName) &&
                node.arguments.length &&
                t.isObjectExpression(node.arguments[0])) {
                node.arguments[0].properties.forEach((item) => {
                    switch (item.type) {
                        case 'ObjectMethod': {
                            actions.push({
                                name: (0, generate_1.keyNode2value)(item.key),
                                type: 'method',
                            });
                            break;
                        }
                        case 'ObjectProperty': {
                            const name = (0, generate_1.keyNode2value)(item.key);
                            if (t.isArrowFunctionExpression(item.value)) {
                                actions.push({
                                    name,
                                    type: 'method',
                                    code: (0, generate_1.node2code)(item.value),
                                });
                            }
                            else if (t.isFunctionExpression(item.value)) {
                                actions.push({
                                    name,
                                    type: 'method',
                                    code: (0, generate_1.node2code)(item.value),
                                });
                            }
                            else {
                                states.push({
                                    name,
                                    type: 'property',
                                    code: (0, generate_1.node2code)(item.value),
                                });
                            }
                            break;
                        }
                        default:
                    }
                });
                if (node.arguments[1]) {
                    namespace = (0, generate_1.node2value)(node.arguments[1], false);
                }
                path.stop();
            }
        },
    });
    return {
        namespace,
        states,
        actions,
    };
}
exports.traverseStoreFile = traverseStoreFile;
function deleteServiceConfigFromServiceFile(ast, serviceFunctionName) {
    (0, traverse_1.default)(ast, {
        CallExpression(path) {
            const calleeName = (0, generate_1.keyNode2value)(path.node.callee);
            if ((0, assert_1.isDefineService)(calleeName)) {
                if (path.node.arguments.length) {
                    const configNode = path.node.arguments[0];
                    if (t.isObjectExpression(configNode)) {
                        const index = configNode.properties.findIndex((property) => {
                            if (t.isObjectProperty(property)) {
                                return (0, generate_1.keyNode2value)(property.key) === serviceFunctionName;
                            }
                            return false;
                        });
                        configNode.properties.splice(index, 1);
                    }
                }
                path.stop();
            }
        },
    });
    return ast;
}
exports.deleteServiceConfigFromServiceFile = deleteServiceConfigFromServiceFile;
/**
 * 将服务函数的配置参数转为 ast 节点
 * @param payload
 * @returns
 */
function serviceConfig2Node(payload) {
    return (0, parse_1.object2node)(payload, (value, key) => {
        if (key === 'formatter' && value) {
            return (0, parse_1.code2expression)(value);
        }
        return (0, parse_1.value2node)(value);
    });
}
exports.serviceConfig2Node = serviceConfig2Node;
function updateServiceConfigToServiceFile(ast, config) {
    (0, traverse_1.default)(ast, {
        CallExpression(path) {
            const calleeName = (0, generate_1.keyNode2value)(path.node.callee);
            if ((0, assert_1.isDefineService)(calleeName) && path.node.arguments.length) {
                const configNode = path.node.arguments[0];
                if (t.isObjectExpression(configNode)) {
                    const newPropertiesNodeMap = Object.keys(config).reduce((properties, key) => {
                        const serviceConfig = config[key];
                        const property = t.objectProperty(t.identifier(key), serviceConfig2Node(serviceConfig));
                        properties[key] = property;
                        return properties;
                    }, {});
                    Object.keys(newPropertiesNodeMap).forEach((nodeKey) => {
                        // 如果已存在, 找到原来的 propertyNode 进行替换
                        const targetIndex = configNode.properties.findIndex((propNode) => {
                            if (t.isObjectProperty(propNode)) {
                                return (0, generate_1.keyNode2value)(propNode.key) === nodeKey;
                            }
                            return false;
                        });
                        if (targetIndex !== -1) {
                            configNode.properties[targetIndex] = newPropertiesNodeMap[nodeKey];
                        }
                        else {
                            // 不存在，直接塞到最后边
                            configNode.properties.push(newPropertiesNodeMap[nodeKey]);
                        }
                    });
                }
                path.stop();
            }
        },
    });
    return ast;
}
exports.updateServiceConfigToServiceFile = updateServiceConfigToServiceFile;
function updateBaseConfigToServiceFile(ast, config) {
    (0, traverse_1.default)(ast, {
        CallExpression(path) {
            const calleeName = (0, generate_1.keyNode2value)(path.node.callee);
            if ((0, assert_1.isDefineService)(calleeName)) {
                switch (path.node.arguments.length) {
                    case 1:
                        // 没有 baseConfig，直接创建新的参数即可
                        path.node.arguments.push((0, parse_1.value2node)(config));
                        break;
                    case 2: {
                        // 已存在 baseConfig，需要进行参数的合并
                        const baseConfigNode = path.node.arguments[1];
                        if (t.isObjectExpression(baseConfigNode)) {
                            const baseConfig = (0, generate_1.node2value)(baseConfigNode);
                            path.node.arguments[1] = (0, parse_1.value2node)(Object.assign(Object.assign({}, baseConfig), config));
                        }
                        break;
                    }
                    default:
                        break;
                }
                path.stop();
            }
        },
    });
    return ast;
}
exports.updateBaseConfigToServiceFile = updateBaseConfigToServiceFile;
function traverseServiceFile(ast) {
    let services;
    const baseConfig = {
        encryptFetch: false,
    };
    const imports = {};
    (0, traverse_1.default)(ast, {
        ImportDeclaration(path) {
            const { source, specifiers } = parseImportDeclaration(path.node);
            imports[source] = specifiers;
        },
        CallExpression(path) {
            const calleeName = (0, generate_1.keyNode2value)(path.node.callee);
            if ((0, assert_1.isDefineService)(calleeName)) {
                if (path.node.arguments.length) {
                    services = (0, generate_1.node2value)(path.node.arguments[0], false);
                    if (path.node.arguments[1]) {
                        const config = (0, generate_1.node2value)(path.node.arguments[1]);
                        baseConfig.encryptFetch = !!config.encryptFetch;
                        baseConfig.namespace = config.namespace || 'index';
                    }
                }
                path.stop();
            }
        },
    });
    return {
        imports,
        services,
        baseConfig,
    };
}
exports.traverseServiceFile = traverseServiceFile;
function getJSXElementName(node) {
    return (0, generate_1.keyNode2value)(node.openingElement.name);
}
function createJSXElementAttributesFilter(filter) {
    return (node) => {
        const attributes = node.openingElement.attributes.filter((attrNode) => {
            if (t.isJSXAttribute(attrNode)) {
                const attrName = (0, generate_1.keyNode2value)(attrNode.name);
                return filter(attrName, attrNode);
            }
            return true;
        });
        node.openingElement.attributes = attributes;
        return node;
    };
}
/**
 * 清楚 JSXElement 的追踪属性
 * @param node
 * @returns
 */
const removeTrackingAttributes = createJSXElementAttributesFilter((attrName) => {
    return attrName !== tango_helpers_1.SLOT.dnd;
});
/**
 * 从文件中移除所有 JSXElement 的追踪属性
 * @param ast
 * @returns
 */
function clearTrackingData(ast) {
    (0, traverse_1.default)(ast, {
        JSXElement(path) {
            path.node = removeTrackingAttributes(path.node);
        },
    });
    return ast;
}
/**
 * 从 JSXElement 中移除追踪属性
 * @param node
 * @returns
 */
function clearJSXElementTrackingData(node, overrideProps) {
    traverseExpressionNode(node, {
        JSXElement(path) {
            const newNode = createJSXElementAttributesFilter((attrName) => {
                if (attrName === tango_helpers_1.SLOT.dnd) {
                    return false;
                }
                if (overrideProps && attrName in overrideProps) {
                    return false;
                }
                return true;
            })(path.node);
            if (overrideProps) {
                const overrideAttributes = (0, parse_1.makeJSXAttributes)(overrideProps);
                newNode.openingElement.attributes = [
                    ...overrideAttributes,
                    ...path.node.openingElement.attributes,
                ];
            }
            path.node = newNode;
        },
    });
    return node;
}
/**
 * 清除文件中的无效导入代码
 * @param ast
 * @returns
 */
function removeUnusedImportSpecifiers(ast) {
    (0, traverse_1.default)(ast, {
        ImportDeclaration(path) {
            const sourceValue = path.node.source.value;
            // e.g. import 'style.less'; import './index.css'
            if (!path.node.specifiers.length && /\.(css|less|scss|js)$/.test(sourceValue)) {
                return;
            }
            // e.g. import {} from 'pkg';
            if (!path.node.specifiers.length) {
                path.remove();
                return;
            }
            const specifiers = path.node.specifiers.filter((specifier) => {
                const name = (0, generate_1.keyNode2value)(specifier.local);
                const binding = path.scope.getBinding(name);
                return binding === null || binding === void 0 ? void 0 : binding.referenced;
            });
            if (!specifiers.length) {
                path.remove();
            }
            else if (specifiers.length !== path.node.specifiers.length) {
                path.node.specifiers = specifiers;
            }
        },
    });
    return ast;
}
exports.removeUnusedImportSpecifiers = removeUnusedImportSpecifiers;
/**
 * 克隆一个全新的 JSXElement 节点
 * @param node
 * @returns
 */
function cloneJSXElement(node, overrideProps) {
    let cloned = t.cloneNode(node, true, true);
    cloned = clearJSXElementTrackingData(cloned, overrideProps);
    return cloned;
}
exports.cloneJSXElement = cloneJSXElement;
function traverseViewFile(ast, idGenerator) {
    const imports = {};
    const importedModules = {};
    const nodes = [];
    const cloneAst = t.cloneNode(ast, true, true);
    const cleanAst = clearTrackingData(cloneAst);
    const variables = []; // 使用的 tango 变量
    (0, traverse_1.default)(ast, {
        ImportDeclaration(path) {
            const { source, specifiers } = parseImportDeclaration(path.node);
            if (imports[source]) {
                // 存在重复的导入语句，合并导入符号列表
                imports[source] = imports[source].concat(specifiers);
            }
            else {
                imports[source] = specifiers;
            }
            // FIXME: 下面的逻辑兼容旧的逻辑，后续需要移除掉
            const declarationData = specifiers.reduce((prev, cur) => {
                switch (cur.type) {
                    case 'ImportDefaultSpecifier':
                        prev.defaultSpecifier = cur.localName;
                        break;
                    case 'ImportSpecifier':
                        prev.specifiers.push(cur.localName);
                        break;
                    case 'ImportNamespaceSpecifier':
                        prev.specifiers.push(cur.localName);
                        break;
                    default:
                        break;
                }
                return prev;
            }, {
                defaultSpecifier: undefined,
                specifiers: [],
                sourcePath: source,
            });
            const exist = importedModules[source];
            if (!exist) {
                importedModules[source] = declarationData;
            }
            else {
                importedModules[source] = Array.isArray(exist)
                    ? exist.concat([declarationData])
                    : [exist, declarationData];
            }
        },
        MemberExpression(path) {
            const variable = (0, generate_1.node2value)(path.node, false);
            const parentNode = path.parentPath.node;
            if (!t.isMemberExpression(parentNode) &&
                (0, assert_1.isTangoVariable)(variable) &&
                !variables.includes(variable)) {
                variables.push(variable);
            }
        },
        OptionalMemberExpression(path) {
            const variable = (0, generate_1.node2value)(path.node, false);
            if ((0, assert_1.isTangoVariable)(variable) && !variables.includes(variable)) {
                variables.push(variable);
            }
        },
        JSXElement(path) {
            const attributes = getJSXElementAttributes(path.node);
            // 获取组件的追踪属性
            const trackDnd = attributes[tango_helpers_1.SLOT.dnd];
            // 用户代码中的 id 标记
            const codeId = attributes.tid;
            let { component, id } = (0, tango_helpers_1.parseDndId)(trackDnd);
            component = component || getJSXElementName(path.node);
            if (!(0, string_1.isValidComponentName)(component)) {
                return;
            }
            // 如果没有 ID，生成组件的追踪 ID
            if (!trackDnd) {
                id = idGenerator.generateId(component, codeId).fullId;
            }
            // 在组件属性中添加追踪标记
            if (!attributes[tango_helpers_1.SLOT.dnd]) {
                path.node.openingElement.attributes.unshift((0, parse_1.makeJSXAttribute)(tango_helpers_1.SLOT.dnd, id));
            }
            // parentId 用于追溯上下游关系
            let parentId;
            let parentCodeId;
            const parentNode = path.findParent((p) => p.isJSXElement());
            if (t.isJSXElement(parentNode === null || parentNode === void 0 ? void 0 : parentNode.node)) {
                const parentAttributes = getJSXElementAttributes(parentNode.node);
                parentId = parentAttributes[tango_helpers_1.SLOT.dnd];
                parentCodeId = parentAttributes.tid;
            }
            nodes.push({
                id,
                codeId,
                parentId,
                parentCodeId,
                component,
                rawNode: path.node,
            });
        },
    });
    return {
        ast,
        cleanAst,
        nodes,
        imports,
        importedModules,
        variables,
    };
}
exports.traverseViewFile = traverseViewFile;
function traverseComponentsEntryFile(ast, baseDir) {
    const exportMap = {};
    (0, traverse_1.default)(ast, {
        ExportNamedDeclaration(path) {
            const node = path.node;
            let source = (0, generate_1.node2value)(node.source);
            if (baseDir) {
                // fix relative source path
                source = (0, string_1.getFullPath)(baseDir, source);
            }
            node.specifiers.forEach((specifier) => {
                if (t.isExportSpecifier(specifier)) {
                    const name = (0, generate_1.keyNode2value)(specifier.exported);
                    if (name) {
                        exportMap[name] = {
                            source,
                            exportedName: name,
                        };
                    }
                }
            });
        },
    });
    return { ast, exportMap };
}
exports.traverseComponentsEntryFile = traverseComponentsEntryFile;
/**
 * 解析导入语句
 */
function parseImportDeclaration(node) {
    const source = (0, generate_1.node2value)(node.source);
    const specifiers = [];
    node.specifiers.forEach((specifierNode) => {
        const data = {
            localName: (0, generate_1.keyNode2value)(specifierNode.local),
            type: specifierNode.type,
        };
        if (specifierNode.type === 'ImportSpecifier') {
            data.importedName = (0, generate_1.keyNode2value)(specifierNode.imported);
        }
        specifiers.push(data);
    });
    return {
        source,
        specifiers,
    };
}
/**
 * 基本的文件解析过程
 * @param ast
 */
function traverseFile(ast) {
    const imports = {};
    (0, traverse_1.default)(ast, {
        ImportDeclaration(path) {
            const { source, specifiers } = parseImportDeclaration(path.node);
            imports[source] = specifiers;
        },
    });
    return {
        imports,
    };
}
exports.traverseFile = traverseFile;
