"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.code2value = exports.value2expressionCode = exports.value2code = void 0;
const tango_helpers_1 = require("@music163/tango-helpers");
const ast_1 = require("./ast");
/**
 * js value 转为代码字符串
 * @example 1 => 1
 * @example hello => "hello"
 * @example { foo: bar } => {{ foo: bar }}
 * @example [1,2,3] => {[1,2,3]}
 *
 * @param val js value
 * @returns 表达式代码
 */
function value2code(val) {
    if (val === undefined) {
        return '';
    }
    if (val === null) {
        return 'null';
    }
    let ret;
    switch (typeof val) {
        case 'string': {
            if ((0, tango_helpers_1.isWrappedCode)(val)) {
                ret = (0, tango_helpers_1.getCodeOfWrappedCode)(val);
            }
            else {
                ret = `"${val}"`;
            }
            break;
        }
        case 'boolean':
        case 'function':
        case 'number':
            ret = String(val);
            break;
        default: {
            // other cases, including array, object, null, undefined
            const node = (0, ast_1.value2node)(val);
            ret = (0, ast_1.expression2code)(node);
            break;
        }
    }
    return ret;
}
exports.value2code = value2code;
exports.value2expressionCode = value2code;
/**
 * 代码字符串转为具体的 js value
 * @example `() => {}` 返回 undefined
 *
 * @param rawCode 代码字符串
 * @returns 返回解析后的 js value，包括：string, number, boolean, simpleObject, simpleArray
 */
function code2value(rawCode) {
    const node = (0, ast_1.code2expression)(rawCode);
    const value = (0, ast_1.node2value)(node);
    if ((0, tango_helpers_1.isWrappedCode)(value)) {
        // 能转的就转，转不能的就返回空
        return;
    }
    return value;
}
exports.code2value = code2value;
