"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.distanceToRect = exports.getElementData = exports.getElementCSSDisplay = exports.getRelativePoint = exports.getElementBoundingData = exports.setElementStyle = exports.getDragGhostElement = exports.DRAG_GHOST_ID = exports.getElement = exports.buildQueryBySlotId = void 0;
const tslib_1 = require("tslib");
const cash_dom_1 = tslib_1.__importDefault(require("cash-dom"));
const tango_helpers_1 = require("@music163/tango-helpers");
function buildQueryBySlotId(id) {
    return `[${tango_helpers_1.SLOT.dnd}="${id}"]`;
}
exports.buildQueryBySlotId = buildQueryBySlotId;
function getElement(selector) {
    return (0, cash_dom_1.default)(selector).get(0);
}
exports.getElement = getElement;
exports.DRAG_GHOST_ID = 'dragGhost';
const getDragGhostElement = () => getElement(`#${exports.DRAG_GHOST_ID}`);
exports.getDragGhostElement = getDragGhostElement;
function setElementStyle(selector, style) {
    return (0, cash_dom_1.default)(selector).css(style);
}
exports.setElementStyle = setElementStyle;
/**
 * 获取元素的外框数据
 * @param element
 * @returns
 */
function getElementBoundingData(element, relativeContainer) {
    const rect = element.getBoundingClientRect();
    const { x, y } = getRelativePoint({ x: rect.left, y: rect.top }, relativeContainer);
    return {
        width: element.offsetWidth,
        height: element.offsetHeight,
        top: y,
        left: x,
    };
}
exports.getElementBoundingData = getElementBoundingData;
function getRelativePoint(point, relativeContainer) {
    let x = point.x;
    let y = point.y;
    if (relativeContainer) {
        const containerRect = relativeContainer.getBoundingClientRect();
        x -= containerRect.left;
        y -= containerRect.top;
    }
    return { x, y };
}
exports.getRelativePoint = getRelativePoint;
/**
 * 获取元素的 css display 值
 * @param element
 * @returns
 */
function getElementCSSDisplay(element) {
    return (0, cash_dom_1.default)(element).css('display');
}
exports.getElementCSSDisplay = getElementCSSDisplay;
/**
 * 获取元素的数据
 * @param element
 * @returns
 */
function getElementData(element, relativeContainer) {
    const bounding = getElementBoundingData(element, relativeContainer);
    const data = (0, tango_helpers_1.getElementDataProps)(element);
    const dnd = (0, tango_helpers_1.parseDndId)(data.dnd);
    const display = getElementCSSDisplay(element);
    return {
        id: dnd.id,
        codeId: dnd.codeId,
        name: dnd.component || element.tagName.toLowerCase(),
        filename: dnd.filename,
        bounding,
        element,
        display,
    };
}
exports.getElementData = getElementData;
/**
 * 给定点的位置到某个元素外框的距离
 * @param point
 * @param rect
 * @returns
 */
function distanceToRect(point, rect) {
    let minX = Math.min(Math.abs(point.clientX - rect.left), Math.abs(point.clientX - rect.right));
    let minY = Math.min(Math.abs(point.clientY - rect.top), Math.abs(point.clientY - rect.bottom));
    if (point.clientX >= rect.left && point.clientX <= rect.right) {
        minX = 0;
    }
    if (point.clientY >= rect.top && point.clientY <= rect.bottom) {
        minY = 0;
    }
    return Math.sqrt(Math.pow(minX, 2) + Math.pow(minY, 2));
}
exports.distanceToRect = distanceToRect;
