"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IdGenerator = void 0;
const tango_helpers_1 = require("@music163/tango-helpers");
/**
 * ID 生成器
 */
class IdGenerator {
    constructor(options) {
        /**
         * 记录组件 ID 记录
         */
        this.map = new Map();
        this.prefix = (options === null || options === void 0 ? void 0 : options.prefix) ? encodeURIComponent(options.prefix) : undefined;
    }
    /**
     * 更新组件记录
     * @param component
     */
    setItem(component, id) {
        if (this.map.has(component)) {
            const record = this.map.get(component);
            if (id && !record.includes(id)) {
                record.push(id);
            }
            this.map.set(component, record);
        }
        else {
            const value = id ? [id] : [];
            this.map.set(component, value);
        }
    }
    /**
     * 获取组件 ID
     * @param component 组件名, 如 Button, DatePicker
     * @param codeId 用户自定义的 ID
     * @returns
     */
    generateId(component, codeId) {
        var _a;
        // FIXME: 使用 size 这里可能存在冲突的风险
        const size = ((_a = this.map.get(component)) === null || _a === void 0 ? void 0 : _a.length) + 1 || 1;
        const id = codeId || `${(0, tango_helpers_1.camelCase)(component)}${size}`;
        this.setItem(component, id);
        let fullId = `${component}:${id}`;
        if (this.prefix) {
            fullId = `${this.prefix}:${fullId}`;
        }
        return { id, fullId };
    }
}
exports.IdGenerator = IdGenerator;
