"use strict";
/* eslint-disable no-console */
Object.defineProperty(exports, "__esModule", { value: true });
exports.invariant = exports.warning = exports.logger = void 0;
exports.logger = {
    group(title, body) {
        console.groupCollapsed(title);
        console.log(body);
        console.groupEnd();
    },
    log(msg) {
        console.log('[Log]: %s', msg);
    },
    warn(msg) {
        warning(false, msg);
    },
    error(...args) {
        console.error('[Error]: ', ...args);
    },
};
/**
 * 如果条件为 false 则打印警告日志
 * @example warning(truthyValue, 'This should not log a warning');
 * @example warning(falsyValue, 'This should log a warning');
 * @see https://github.com/alexreardon/tiny-warning
 *
 * @param condition 打印警告日志的条件
 * @param message 警告日志
 */
function warning(condition, message) {
    if (condition) {
        return;
    }
    console.warn(`Warning: ${message}`);
}
exports.warning = warning;
/**
 * 如果条件为 false 则抛出错误
 * @see https://github.com/alexreardon/tiny-invariant
 * @param condition 条件
 * @param message 错误消息
 */
function invariant(condition, message) {
    if (condition) {
        return;
    }
    const text = typeof message === 'function' ? message() : message;
    throw new Error(`Invariant failed: ${text}`);
}
exports.invariant = invariant;
