"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pick = exports.omit = exports.clone = exports.setValue = exports.getValue = exports.merge = void 0;
const tslib_1 = require("tslib");
const lodash_get_1 = tslib_1.__importDefault(require("lodash.get"));
const lodash_set_1 = tslib_1.__importDefault(require("lodash.set"));
/**
 * 合并 target 和 source 对象，source 对象的优先级更高，如存在重名，覆盖 target 中的 key
 * @param target target object
 * @param source source object
 * @return new merged object, target will not be modified
 */
function merge(target, source) {
    return Object.assign(Object.assign({}, (target || {})), (source || {}));
}
exports.merge = merge;
/**
 * 从目标上下文中根据 keyPath 获取对应的值
 * @param context the object to query
 * @param keyPath the path of the property to get
 * @returns
 */
function getValue(context, keyPath) {
    keyPath = keyPath.replaceAll('?.', '.');
    return (0, lodash_get_1.default)(context, keyPath);
}
exports.getValue = getValue;
/**
 * Sets the value at path of object. If a portion of path doesn't exist, it's created. Arrays are created for missing index properties while objects are created for all other missing properties
 * @param context
 * @param keyPath
 * @param value
 * @returns
 */
function setValue(context, keyPath, value) {
    return (0, lodash_set_1.default)(context, keyPath, value);
}
exports.setValue = setValue;
/**
 * 浅拷贝目标对象，并清除对象上的 undefined value
 * @param obj
 */
function clone(obj, withUndefined = true) {
    if (!obj || typeof obj !== 'object') {
        return obj;
    }
    const target = {};
    for (const key in obj) {
        if (withUndefined) {
            target[key] = obj[key];
        }
        else if (obj[key] !== undefined) {
            target[key] = obj[key];
        }
    }
    return target;
}
exports.clone = clone;
function omit(obj, keys) {
    const target = clone(obj);
    for (const key of keys) {
        delete target[key];
    }
    return target;
}
exports.omit = omit;
function pick(obj, keys) {
    const target = {};
    for (const key of keys) {
        target[key] = obj[key];
    }
    return target;
}
exports.pick = pick;
