"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.prototype2jsxElement = exports.prototype2code = exports.propDataToKeyValueString = exports.getImportDeclarationPayloadByPrototype = exports.prototype2importDeclarationData = void 0;
const tango_helpers_1 = require("@music163/tango-helpers");
const string_1 = require("./string");
const ast_1 = require("./ast");
const assert_1 = require("./assert");
const code_helpers_1 = require("./code-helpers");
function prototype2importDeclarationData(prototype, relativeFilepath) {
    let source = prototype.package;
    if (relativeFilepath && (0, string_1.isFilepath)(source)) {
        source = (0, string_1.getRelativePath)(relativeFilepath, source);
    }
    if (source.endsWith('.js')) {
        source = source.slice(0, -3);
    }
    const specifiers = [];
    if (prototype.exportType === 'defaultExport') {
        specifiers.push({
            localName: prototype.name,
            type: 'ImportDefaultSpecifier',
        });
    }
    else {
        [prototype.name, ...(prototype.relatedImports || [])].forEach((item) => {
            specifiers.push({
                localName: item,
                type: 'ImportSpecifier',
            });
        });
    }
    return {
        source,
        specifiers,
    };
}
exports.prototype2importDeclarationData = prototype2importDeclarationData;
/**
 * 根据组件的 prototype 生成 ImportDeclarationPayload
 * @deprecated
 */
function getImportDeclarationPayloadByPrototype(prototype, relativeFilepath) {
    let defaultSpecifier;
    let specifiers;
    if (prototype.exportType === 'defaultExport') {
        defaultSpecifier = prototype.name;
        specifiers = prototype.relatedImports || [];
    }
    else {
        specifiers = [...(prototype.relatedImports || [])];
        if (prototype.type !== 'snippet') {
            specifiers.push(prototype.name);
        }
    }
    let sourcePath = prototype.package;
    if (relativeFilepath && (0, string_1.isFilepath)(sourcePath)) {
        sourcePath = (0, string_1.getRelativePath)(relativeFilepath, sourcePath);
    }
    if (sourcePath.endsWith('.js')) {
        sourcePath = sourcePath.slice(0, -3);
    }
    return {
        defaultSpecifier,
        specifiers,
        sourcePath,
    };
}
exports.getImportDeclarationPayloadByPrototype = getImportDeclarationPayloadByPrototype;
/**
 * 基于 key-value 生成 prop={value} 字符串
 * @example { name: 'foo', initValue: false } >> name={false}
 * @example { name: 'foo', initValue: 1 } >> name={1}
 * @example { name: 'foo', initValue: () => {} } >> name={()=>{}}
 * @example { name: 'foo', initValue: { foo: 'bar' } } >> name={{ foo: 'bar' }}
 * @example { name: 'foo', initValue: [{ foo: 'bar' }] } >> name={[{ foo: 'bar' }]}
 * @example { name: 'foo', initValue: 'bar' } >> name="bar"
 * @example { name: 'foo', initValue: '{() => {}}' } >> name={()=>{}}
 * @example { name: 'foo', initValue: '{{() => {}}}' } >> name={() => {}}
 * @example { name: 'foo', initValue: '{bar}' } >> name={bar}
 * @returns
 */
function propDataToKeyValueString(item, generateValue) {
    const key = item.name;
    let value = item.initValue;
    if (!value && item.autoInitValue) {
        value = (generateValue === null || generateValue === void 0 ? void 0 : generateValue(3)) || (0, tango_helpers_1.uuid)(key, 3);
    }
    if (value === undefined) {
        return;
    }
    switch (typeof value) {
        case 'number':
        case 'boolean':
        case 'function': {
            value = (0, tango_helpers_1.wrapCodeWithJSXExpressionContainer)(String(value));
            break;
        }
        case 'object': {
            try {
                value = (0, tango_helpers_1.wrapCodeWithJSXExpressionContainer)((0, code_helpers_1.value2code)(value));
            }
            catch (err) {
                tango_helpers_1.logger.error(err);
            }
            break;
        }
        case 'string': {
            if ((0, tango_helpers_1.isWrappedCode)(value)) {
                const innerCode = (0, tango_helpers_1.getCodeOfWrappedCode)(value);
                value = (0, tango_helpers_1.wrapCodeWithJSXExpressionContainer)(innerCode);
            }
            else if ((0, assert_1.isWrappedByExpressionContainer)(value)) {
                // TIP: 兼容旧版逻辑，如果是变量字符串，无需处理
            }
            else {
                value = `"${value}"`;
            }
            break;
        }
        default:
            break;
    }
    return `${key}=${value}`;
}
exports.propDataToKeyValueString = propDataToKeyValueString;
/**
 * Button prototype -> <Button>hello</Button>
 *
 * @param prototype
 * @param extraProps
 */
function prototype2code(prototype, extraProps) {
    let code;
    switch (prototype.type) {
        case 'snippet':
            code = prototype.initChildren || prototype.defaultChildren;
            break;
        default: {
            const propList = extraProps
                ? Object.keys(extraProps).map((key) => ({
                    name: key,
                    initValue: extraProps[key],
                }))
                : [];
            // merge extraProps to props
            const props = [...(prototype.props || []), ...propList];
            const keys = props.reduce((acc, item) => {
                const pair = propDataToKeyValueString(item, (fractionDigits) => (0, tango_helpers_1.uuid)(prototype.name, fractionDigits));
                return pair ? ` ${acc} ${pair}` : acc;
            }, '') || '';
            if (prototype.hasChildren) {
                code = `<${prototype.name} ${keys}>${prototype.initChildren || prototype.defaultChildren || ''}</${prototype.name}>`;
            }
            else {
                code = `<${prototype.name} ${keys.trim()} />`;
            }
            break;
        }
    }
    return code;
}
exports.prototype2code = prototype2code;
/**
 * 基于 prototype 信息生成 t.JSXElement
 * @example ButtonPrototype -> <Button>hello</Button> -> t.JSXElement
 *
 * @param prototype 组件的配置信息
 * @param props 额外的属性集
 */
function prototype2jsxElement(prototype, props) {
    const code = prototype2code(prototype, props);
    return (0, ast_1.code2expression)(code);
}
exports.prototype2jsxElement = prototype2jsxElement;
