import React from 'react';
export { default as hoistNonReactStatics } from 'hoist-non-react-statics';
export interface CreateContextOptions {
    /**
     * If `true`, React will throw if context is `null` or `undefined`
     * In some cases, you might want to support nested context, so you can set it to `false`
     */
    strict?: boolean;
    /**
     * Error message to throw if the context is `undefined`
     */
    errorMessage?: string;
    /**
     * The display name of the context
     */
    name?: string;
}
type CreateContextReturn<T> = [React.Provider<T>, () => T, React.Context<T>];
/**
 * 创建一个命名的 Context 附赠 Provider, Hook
 * @param options
 */
export declare function createContext<ContextType>(options?: CreateContextOptions): CreateContextReturn<ContextType>;
/**
 * 获取组件的 displayName
 * @param Component
 * @returns
 */
export declare const getDisplayName: (Component: any) => any;
/**
 * 包裹 DisplayName
 * @param Component
 * @param hocName
 * @returns
 */
export declare const wrapDisplayName: (Component: any, hocName: string) => string;
/**
 * 判断组件是否是函数组件
 * @returns 类组件和 forwardRef 返回 false，函数组件返回 true
 */
export declare const isFunctionComponent: (Comp: any) => boolean;
