"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isFunctionComponent = exports.wrapDisplayName = exports.getDisplayName = exports.createContext = exports.hoistNonReactStatics = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
var hoist_non_react_statics_1 = require("hoist-non-react-statics");
Object.defineProperty(exports, "hoistNonReactStatics", { enumerable: true, get: function () { return tslib_1.__importDefault(hoist_non_react_statics_1).default; } });
/**
 * 创建一个命名的 Context 附赠 Provider, Hook
 * @param options
 */
function createContext(options = {}) {
    const { strict = true, errorMessage = 'useContext: `context` is undefined. Seems you forgot to wrap component within the Provider', name, } = options;
    const Context = react_1.default.createContext(undefined);
    Context.displayName = name;
    function useContext() {
        const context = react_1.default.useContext(Context);
        if (!context && strict) {
            throw new Error(errorMessage);
        }
        return context;
    }
    return [Context.Provider, useContext, Context];
}
exports.createContext = createContext;
/**
 * 获取组件的 displayName
 * @param Component
 * @returns
 */
const getDisplayName = (Component) => {
    if (typeof Component === 'string') {
        return Component;
    }
    if (!Component) {
        return undefined;
    }
    return Component.displayName || Component.name || 'Component';
};
exports.getDisplayName = getDisplayName;
/**
 * 包裹 DisplayName
 * @param Component
 * @param hocName
 * @returns
 */
const wrapDisplayName = (Component, hocName) => `${hocName}(${(0, exports.getDisplayName)(Component)})`;
exports.wrapDisplayName = wrapDisplayName;
/**
 * 判断组件是否是函数组件
 * @returns 类组件和 forwardRef 返回 false，函数组件返回 true
 */
const isFunctionComponent = (Comp) => {
    // class component
    if (Comp.prototype && Comp.prototype.isReactComponent) {
        return false;
    }
    // forwardRef component
    if (typeof Comp === 'object' && Comp.render) {
        return false;
    }
    // function component
    return typeof Comp === 'function';
};
exports.isFunctionComponent = isFunctionComponent;
