"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useControllableState = exports.useControllableProp = void 0;
const react_1 = require("react");
const use_callback_ref_1 = require("./use-callback-ref");
function useControllableProp(prop, state) {
    const controlled = typeof prop !== 'undefined';
    const value = controlled ? prop : state;
    return (0, react_1.useMemo)(() => [controlled, value], [controlled, value]);
}
exports.useControllableProp = useControllableProp;
function useControllableState(props) {
    const { value: valueProp, defaultValue, onChange, shouldUpdate = (prev, next) => prev !== next, } = props;
    const onChangeProp = (0, use_callback_ref_1.useCallbackRef)(onChange);
    const shouldUpdateProp = (0, use_callback_ref_1.useCallbackRef)(shouldUpdate);
    const [uncontrolledState, setUncontrolledState] = (0, react_1.useState)(defaultValue);
    const controlled = valueProp !== undefined;
    const value = controlled ? valueProp : uncontrolledState;
    const setValue = (0, react_1.useCallback)((next) => {
        const setter = next;
        const nextValue = typeof next === 'function' ? setter(value) : next;
        if (!shouldUpdateProp(value, nextValue)) {
            return;
        }
        if (!controlled) {
            setUncontrolledState(nextValue);
        }
        onChangeProp(nextValue);
    }, [controlled, onChangeProp, value, shouldUpdateProp]);
    return [value, setValue];
}
exports.useControllableState = useControllableState;
