"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InputCode = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
const react_codemirror_1 = tslib_1.__importDefault(require("@uiw/react-codemirror"));
const lang_javascript_1 = require("@codemirror/lang-javascript");
const language_1 = require("@codemirror/language");
const lint_1 = require("@codemirror/lint");
const eslint = tslib_1.__importStar(require("eslint-linter-browserify"));
const tango_helpers_1 = require("@music163/tango-helpers");
const completePropertyAfter = ['PropertyName', '.', '?.'];
const dontCompleteIn = [
    'TemplateString',
    'LineComment',
    'BlockComment',
    'VariableDefinition',
    'PropertyDefinition',
];
const eslintConfig = {
    parserOptions: {
        ecmaVersion: 'latest',
        sourceType: 'module',
    },
    env: {
        browser: true,
    },
    rules: {},
};
function completeProperties(from, object) {
    const options = [];
    Object.entries(object).forEach(([key, val]) => {
        options.push({
            label: key,
            type: typeof val === 'function' ? 'function' : 'variable',
        });
    });
    return {
        from,
        options,
        validFor: /^[\w$]*$/,
    };
}
/**
 *
 * @param scope 预测的补全上下文
 * @param customOptions 自定义补全选项列表
 * @returns
 */
function buildAutoComplete(scope = window, customOptions = []) {
    /**
     * @see context https://codemirror.net/docs/ref/#autocomplete.CompletionContext
     */
    return function completeFromGlobalScope(context) {
        var _a;
        const nodeBefore = (0, language_1.syntaxTree)(context.state).resolveInner(context.pos, -1);
        if (completePropertyAfter.includes(nodeBefore.name) &&
            ((_a = nodeBefore.parent) === null || _a === void 0 ? void 0 : _a.name) === 'MemberExpression') {
            const object = nodeBefore.parent.getChild('Expression');
            if ((object === null || object === void 0 ? void 0 : object.name) === 'VariableName') {
                const from = /\./.test(nodeBefore.name) ? nodeBefore.to : nodeBefore.from;
                const variableName = context.state.sliceDoc(object.from, object.to);
                if (typeof scope[variableName] === 'object') {
                    return completeProperties(from, scope[variableName]);
                }
            }
            else if ((object === null || object === void 0 ? void 0 : object.name) === 'MemberExpression') {
                const from = nodeBefore.to;
                const variableName = context.state.sliceDoc(object.from, object.to);
                let result;
                if (variableName.startsWith('this.props')) {
                    result = (0, tango_helpers_1.getValue)(scope, variableName.replace('this.props', '__UNSAFE_TANGO_CURRENT_PAGE_RENDER_RUNTIME__'));
                }
                else {
                    result = (0, tango_helpers_1.getValue)(scope, variableName);
                }
                if (typeof result === 'object') {
                    return completeProperties(from, result);
                }
            }
        }
        else if (nodeBefore.name === 'VariableName') {
            return completeProperties(nodeBefore.from, scope);
        }
        else if (context.explicit && !dontCompleteIn.includes(nodeBefore.name)) {
            return completeProperties(context.pos, scope);
        }
        else if (!context.explicit && nodeBefore.name === '{') {
            return {
                from: context.pos,
                options: customOptions,
            };
        }
        return null;
    };
}
function InputCode(_a) {
    var { shape = 'solid', suffix, status, autoCompleteContext, autoCompleteOptions, showLineNumbers, showFoldGutter, enableESLint = false } = _a, rest = tslib_1.__rest(_a, ["shape", "suffix", "status", "autoCompleteContext", "autoCompleteOptions", "showLineNumbers", "showFoldGutter", "enableESLint"]);
    const globalJavaScriptCompletions = (0, react_1.useMemo)(() => {
        // 格式化补全的选项值，原始格式参考 https://codemirror.net/docs/ref/#autocomplete.Completion
        const options = autoCompleteOptions ? autoCompleteOptions.map((str) => ({ label: str })) : [];
        return lang_javascript_1.javascriptLanguage.data.of({
            autocomplete: buildAutoComplete(autoCompleteContext, options),
        });
    }, [autoCompleteContext, autoCompleteOptions]);
    const extensions = [(0, lang_javascript_1.javascript)({ jsx: true }), globalJavaScriptCompletions];
    if (enableESLint) {
        extensions.push((0, lint_1.lintGutter)(), (0, lint_1.linter)((0, lang_javascript_1.esLint)(new eslint.Linter(), eslintConfig)));
    }
    const { rootStyle, codeSetup } = useInputCode({ shape, status, showLineNumbers, showFoldGutter });
    return (react_1.default.createElement(coral_system_1.Box, Object.assign({ className: "InputCode", display: "flex", alignItems: "center", overflow: "hidden" }, rootStyle),
        react_1.default.createElement(coral_system_1.Box, { flex: "1", overflow: "auto" },
            react_1.default.createElement(react_codemirror_1.default, Object.assign({ extensions: extensions, basicSetup: codeSetup, placeholder: "//\u8F93\u5165 javascript \u4EE3\u7801" }, rest))),
        suffix));
}
exports.InputCode = InputCode;
function useInputCode({ shape, status, showLineNumbers, showFoldGutter, }) {
    let rootStyle;
    let lineNumbers;
    let foldGutter;
    if (shape === 'solid') {
        rootStyle = {
            border: 'solid',
            borderColor: status === 'error' ? 'error.60' : 'line.normal',
            borderRadius: 's',
            minHeight: 32,
        };
        lineNumbers = false;
        foldGutter = false;
    }
    return {
        rootStyle,
        codeSetup: {
            lineNumbers: showLineNumbers !== null && showLineNumbers !== void 0 ? showLineNumbers : lineNumbers,
            foldGutter: showFoldGutter !== null && showFoldGutter !== void 0 ? showFoldGutter : foldGutter,
            searchKeymap: false, // 默认关闭搜索快捷键，原因：https://github.com/uiwjs/react-codemirror/issues/280
        },
    };
}
