"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useLocaleManager = exports.LocaleManager = exports.getAntdLocalesMap = void 0;
const tslib_1 = require("tslib");
const zh_cn_1 = tslib_1.__importDefault(require("./locales/zh-cn"));
const en_us_1 = tslib_1.__importDefault(require("./locales/en-us"));
const react_1 = require("react");
const zh_CN = 'zh-CN';
const en_US = 'en-US';
const DEFAULT_LOCALE = zh_CN;
/**
 * 获取antd的语言资源映射关系
 * @param param0 antd的语言资源
 * @returns
 */
const getAntdLocalesMap = ({ zhCN, enUS }) => ({
    [zh_CN]: zhCN,
    [en_US]: enUS,
});
exports.getAntdLocalesMap = getAntdLocalesMap;
const locales = {
    [zh_CN]: zh_cn_1.default,
    [en_US]: en_us_1.default,
};
/**
 * 单例模式的多语言管理类
 */
class LocaleManager {
    static getInstance(currentLocale) {
        if (!currentLocale && !LocaleManager.instance)
            return null;
        LocaleManager.instance = LocaleManager.instance || new LocaleManager(currentLocale);
        return LocaleManager.instance;
    }
    constructor(currentLocale) {
        this.currentLocale = DEFAULT_LOCALE;
        if (LocaleManager.instance) {
            throw new Error('Error - use LocaleManager.getInstance()');
        }
        if (currentLocale) {
            this.currentLocale = currentLocale;
        }
    }
    changeLocale(currentLocale) {
        this.currentLocale = currentLocale;
    }
    getLocale() {
        return locales[this.currentLocale];
    }
}
exports.LocaleManager = LocaleManager;
const useLocaleManager = (locale = DEFAULT_LOCALE) => {
    const [localeManagerInstance] = (0, react_1.useState)(LocaleManager.getInstance(locale));
    return localeManagerInstance;
};
exports.useLocaleManager = useLocaleManager;
