"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Menu = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const coral_system_1 = require("coral-system");
const icons_1 = require("@ant-design/icons");
const tango_helpers_1 = require("@music163/tango-helpers");
const antd_1 = require("antd");
const menuStyle = (0, coral_system_1.css) `
  font-family: Consolas, Menlo, Courier, monospace;
  font-size: 14px;
`;
const MenuItem = (0, coral_system_1.coral)('div', (0, coral_system_1.css) `
    display: flex;
    justify-content: space-between;
    align-items: center;
    user-select: none;
    height: 28px;
    padding: 0 12px;
    cursor: pointer;

    button {
      display: none;
      background: transparent;
      border: 0;
      cursor: pointer;
    }

    .tango-menu-label {
      overflow: hidden;
      text-overflow: ellipsis;
      white-space: nowrap;
      max-width: 90%;
    }

    &:hover button {
      display: inline-block;
    }

    &:hover {
      background-color: var(--tango-colors-gray-10);
    }

    &.active {
      background-color: var(--tango-colors-primary-10);
    }
  `);
function Menu({ items = [], activeKey, onItemClick = tango_helpers_1.noop, onEdit = tango_helpers_1.noop, onDelete = tango_helpers_1.noop, }) {
    return (react_1.default.createElement(coral_system_1.Box, { className: "tango-menu", css: menuStyle }, items.map((item) => (react_1.default.createElement(MenuItem, { key: item.key, className: (0, classnames_1.default)({ active: activeKey === item.key }), onClick: () => onItemClick(item.key) },
        react_1.default.createElement("div", { className: "tango-menu-label" },
            react_1.default.createElement(coral_system_1.Text, { color: "text.body", title: item.label }, item.label),
            react_1.default.createElement(coral_system_1.Text, { ml: "m", color: "text.placeholder" }, item.note)),
        react_1.default.createElement("div", { className: "tango-menu-actions" },
            item.editable && (react_1.default.createElement("button", { onClick: () => onEdit(item.key) },
                react_1.default.createElement(icons_1.EditOutlined, null))),
            item.deletable && (react_1.default.createElement(antd_1.Popconfirm, { title: item.deletableConfirm || '确定执行此操作吗？', okText: "\u786E\u5B9A", cancelText: "\u53D6\u6D88", onConfirm: (e) => {
                    // 阻止冒泡，否则删除的时候会有 Bug
                    e.stopPropagation();
                    onDelete(item.key);
                } },
                react_1.default.createElement("button", null,
                    react_1.default.createElement(icons_1.DeleteOutlined, null))))))))));
}
exports.Menu = Menu;
