import { IWorkspace } from './interfaces';
export type SimulatorNameType = 'desktop' | 'phone';
export type DesignerViewType = 'design' | 'code' | 'dual';
interface ISimulatorType {
    name: SimulatorNameType;
    width: number;
    height: number;
}
interface IViewportBounding {
    width: number;
    height: number;
}
interface IDesignerOptions {
    workspace: IWorkspace;
    simulator?: SimulatorNameType | ISimulatorType;
    activeSidebarPanel?: string;
}
export declare class Designer {
    /**
     * 当前的沙箱模拟器类型
     */
    _simulator: ISimulatorType;
    /**
     * 当前的视图尺寸
     */
    _viewport: IViewportBounding;
    /**
     * 当前激活的视图
     */
    _activeView: DesignerViewType;
    /**
     * 当前选中的侧边栏面板
     */
    _activeSidebarPanel: string;
    /**
     * 是否显示智能引导
     */
    _showSmartWizard: boolean;
    /**
     * 是否显示右侧面板
     */
    _showRightPanel: boolean;
    /**
     * 是否预览模式
     */
    _isPreview: boolean;
    /**
     * 默认展开的侧边栏
     */
    defaultActiveSidebarPanel?: string;
    private readonly workspace;
    get simulator(): ISimulatorType;
    get viewport(): IViewportBounding;
    get activeView(): DesignerViewType;
    get isPreview(): boolean;
    get showSmartWizard(): boolean;
    get activeSidebarPanel(): string;
    get showRightPanel(): boolean;
    constructor(options: IDesignerOptions);
    setSimulator(value: ISimulatorType | SimulatorNameType): void;
    setViewport(value: IViewportBounding): void;
    setActiveView(view: DesignerViewType): void;
    setActiveSidebarPanel(panel: string): void;
    closeSidebarPanel(): void;
    toggleSmartWizard(value: boolean): void;
    toggleRightPanel(value?: boolean): void;
    toggleIsPreview(value: boolean): void;
}
export {};
