"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DragSource = void 0;
const mobx_1 = require("mobx");
const drop_target_1 = require("./drop-target");
/**
 * 拖拽来源类，被拖拽的物体
 */
class DragSource {
    get node() {
        var _a, _b;
        return this.workspace.getNode((_a = this.data) === null || _a === void 0 ? void 0 : _a.id, (_b = this.data) === null || _b === void 0 ? void 0 : _b.filename);
    }
    /**
     * 获取对应的 prototype
     */
    get prototype() {
        var _a;
        return this.workspace.getPrototype((_a = this.data) === null || _a === void 0 ? void 0 : _a.name);
    }
    get id() {
        var _a;
        return (_a = this.data) === null || _a === void 0 ? void 0 : _a.id;
    }
    get name() {
        var _a;
        return (_a = this.data) === null || _a === void 0 ? void 0 : _a.name;
    }
    get bounding() {
        var _a;
        return (_a = this.data) === null || _a === void 0 ? void 0 : _a.bounding;
    }
    constructor(workspace) {
        this.workspace = workspace;
        this.data = null;
        this.isDragging = false;
        this.dropTarget = new drop_target_1.DropTarget(workspace);
        (0, mobx_1.makeObservable)(this, {
            data: mobx_1.observable,
            isDragging: mobx_1.observable,
            set: mobx_1.action,
            clear: mobx_1.action,
            node: mobx_1.computed,
            prototype: mobx_1.computed,
        });
    }
    /**
     * 更新选中数据
     * @param props
     */
    set(data) {
        this.data = data;
        this.isDragging = !!data;
    }
    /**
     * 重置
     */
    clear() {
        this.data = null;
        this.isDragging = false;
        this.dropTarget.clear();
    }
    /**
     * 获取对应的 node
     * @deprecated
     */
    getNode() {
        return this.node;
    }
}
exports.DragSource = DragSource;
