import * as t from '@babel/types';
import { TangoFile } from './file';
import { IFileConfig, IImportSpecifierData, ImportDeclarationDataType } from '../types';
import { IWorkspace } from './interfaces';
/**
 * JS 模块实现规范
 * - ast 操纵类方法，统一返回 this，支持外层链式调用
 * - observable state 统一用 _foo 格式，并提供 getter 方法
 */
export declare class TangoModule extends TangoFile {
    ast: t.File;
    /**
     * 导入的依赖列表
     */
    importList: ImportDeclarationDataType;
    constructor(workspace: IWorkspace, props: IFileConfig, isSyncCode?: boolean);
    /**
     * 基于最新的 ast 进行同步
     * @param code 如果传入 code，则基于 code 进行同步
     * @param isFormatCode 是否格式化代码
     * @param refreshWorkspace 是否刷新 workspace
     */
    update(code?: string, isFormatCode?: boolean, refreshWorkspace?: boolean): void;
    addImportDeclaration(source: string, specifiers: IImportSpecifierData[]): this;
    updateImportDeclaration(source: string, specifiers: IImportSpecifierData[]): this;
    /**
     * 基于最新的 ast 进行源码同步
     */
    _syncByAst(): void;
    /**
     * 基于输入的源码进行同步
     * @param code 源码
     * @param isFormatCode 是否格式化代码
     * @returns
     */
    _syncByCode(code: string, isFormatCode?: boolean): void;
    _analysisAst(): void;
}
/**
 * 普通 JS 文件
 */
export declare class TangoJsModule extends TangoModule {
    constructor(workspace: IWorkspace, props: IFileConfig);
}
