"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TangoRouteModule = void 0;
const mobx_1 = require("mobx");
const helpers_1 = require("../helpers");
const module_1 = require("./module");
/**
 * 路由配置模块
 */
class TangoRouteModule extends module_1.TangoModule {
    get routes() {
        return (0, mobx_1.toJS)(this._routes);
    }
    constructor(workspace, props) {
        super(workspace, props, false);
        this.update(props.code, true, false);
        (0, mobx_1.makeObservable)(this, {
            _routes: mobx_1.observable,
            _code: mobx_1.observable,
            _cleanCode: mobx_1.observable,
            routes: mobx_1.computed,
            code: mobx_1.computed,
            cleanCode: mobx_1.computed,
            update: mobx_1.action,
        });
    }
    /**
     * 根据路由地址获取 route 对象
     */
    getRouteByRoutePath(route) {
        let record;
        for (const item of this.routes) {
            if (item.path === route) {
                record = item;
                break;
            }
        }
        return record;
    }
    /**
     * 添加一条新路由
     * @param name
     */
    addRoute(routePath, importFilePath) {
        this.ast = (0, helpers_1.addRouteToRouteFile)(this.ast, routePath, importFilePath);
        return this;
    }
    /**
     * 更新页面路由
     * @param oldRoutePath
     * @param newRoutePath
     * @returns
     */
    updateRoute(oldRoutePath, newRoutePath) {
        this.ast = (0, helpers_1.updateRouteToRouteFile)(this.ast, oldRoutePath, newRoutePath);
        return this;
    }
    /**
     * 删除一条路由
     * @param route 路由地址
     */
    removeRoute(route) {
        if (route === '/') {
            console.warn('index route should not be removed!');
            return;
        }
        const record = this.getRouteByRoutePath(route);
        this.ast = (0, helpers_1.removeRouteFromRouteFile)(this.ast, route, record.importPath);
        return this;
    }
    _analysisAst() {
        this._routes = (0, helpers_1.traverseRouteFile)(this.ast);
    }
}
exports.TangoRouteModule = TangoRouteModule;
