"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SelectSource = void 0;
const mobx_1 = require("mobx");
class SelectSource {
    get start() {
        return (0, mobx_1.toJS)(this._start);
    }
    get first() {
        if (this._items.length)
            return this._items[0];
        return;
    }
    get firstNode() {
        if (!this.first)
            return;
        return this.workspace.getNode(this.first.id, this.first.filename);
    }
    get size() {
        return this._items.length;
    }
    /**
     * 选中的结点数据 NodeData
     */
    get selected() {
        return (0, mobx_1.toJS)(this._items);
    }
    /**
     * 是否选中了结点
     */
    get isSelected() {
        return !!this.selected.length;
    }
    /**
     * 选中结点位于的文件
     */
    get file() {
        var _a;
        return (_a = this.firstNode) === null || _a === void 0 ? void 0 : _a.file;
    }
    /**
     * 选中的结点 Nodes
     */
    get nodes() {
        return this._items
            .map((item) => this.workspace.getNode(item.id, item.filename))
            .filter((node) => !!node);
    }
    constructor(workspace) {
        /**
         * 选中元素列表
         */
        this._items = [];
        /**
         * 用户选择的起点
         */
        this._start = {
            point: {
                x: 0,
                y: 0,
            },
            element: null,
        };
        this.workspace = workspace;
        (0, mobx_1.makeObservable)(this, {
            _items: mobx_1.observable,
            _start: mobx_1.observable,
            select: mobx_1.action,
            setStart: mobx_1.action,
            clear: mobx_1.action,
            start: mobx_1.computed,
            selected: mobx_1.computed,
            first: mobx_1.computed,
            firstNode: mobx_1.computed,
            size: mobx_1.computed,
            isSelected: mobx_1.computed,
            file: mobx_1.computed,
            nodes: mobx_1.computed,
        });
    }
    // 增加一个选中项
    add() { }
    // 移除一个选中项
    remove() { }
    select(items) {
        if (!items) {
            this._items = [];
        }
        else {
            this._items = Array.isArray(items) ? items : [items];
        }
        // 选中后清空起点位置信息
        this._start = null;
    }
    setStart(data) {
        this._start = data;
    }
    clear() {
        this._items = [];
    }
}
exports.SelectSource = SelectSource;
