"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TangoServiceModule = void 0;
const mobx_1 = require("mobx");
const tango_helpers_1 = require("@music163/tango-helpers");
const helpers_1 = require("../helpers");
const module_1 = require("./module");
/**
 * 数据服务模块
 */
class TangoServiceModule extends module_1.TangoModule {
    get serviceFunctions() {
        return (0, mobx_1.toJS)(this._serviceFunctions);
    }
    get baseConfig() {
        return (0, mobx_1.toJS)(this._baseConfig);
    }
    constructor(workspace, props) {
        super(workspace, props, false);
        this.name = (0, helpers_1.getModuleNameByFilename)(props.filename);
        this.update(props.code, true, false);
        (0, mobx_1.makeObservable)(this, {
            _serviceFunctions: mobx_1.observable,
            _baseConfig: mobx_1.observable,
            _code: mobx_1.observable,
            _cleanCode: mobx_1.observable,
            serviceFunctions: mobx_1.computed,
            baseConfig: mobx_1.computed,
            cleanCode: mobx_1.computed,
            code: mobx_1.computed,
            update: mobx_1.action,
        });
    }
    _analysisAst() {
        const { imports, services, baseConfig } = (0, helpers_1.traverseServiceFile)(this.ast);
        this.importList = imports;
        this._serviceFunctions = services;
        this._baseConfig = baseConfig;
        if (baseConfig.namespace) {
            this.name = baseConfig.namespace;
        }
    }
    addServiceFunction(name, config) {
        this.ast = (0, helpers_1.updateServiceConfigToServiceFile)(this.ast, { [name]: (0, tango_helpers_1.clone)(config, false) });
        return this;
    }
    /**
     * 批量添加服务函数
     * @param configs { [name: string]: object }
     * @returns
     */
    addServiceFunctions(configs) {
        this.ast = (0, helpers_1.updateServiceConfigToServiceFile)(this.ast, configs);
        return this;
    }
    updateServiceFunction(name, payload) {
        this.ast = (0, helpers_1.updateServiceConfigToServiceFile)(this.ast, { [name]: (0, tango_helpers_1.clone)(payload, false) });
        return this;
    }
    deleteServiceFunction(serviceFunctionName) {
        try {
            this.ast = (0, helpers_1.deleteServiceConfigFromServiceFile)(this.ast, serviceFunctionName);
        }
        catch (e) {
            tango_helpers_1.logger.error(e);
        }
        return this;
    }
    /**
     * 更新服务的基础配置
     */
    updateBaseConfig(data) {
        this.ast = (0, helpers_1.updateBaseConfigToServiceFile)(this.ast, data);
        return this;
    }
}
exports.TangoServiceModule = TangoServiceModule;
