"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TangoStoreModule = exports.TangoStoreEntryModule = void 0;
const mobx_1 = require("mobx");
const helpers_1 = require("../helpers");
const module_1 = require("./module");
/**
 * 入口配置模块
 */
class TangoStoreEntryModule extends module_1.TangoModule {
    get stores() {
        return (0, mobx_1.toJS)(this._stores);
    }
    constructor(workspace, props) {
        super(workspace, props, false);
        this._stores = [];
        this.update(props.code, true, false);
        (0, mobx_1.makeObservable)(this, {
            _stores: mobx_1.observable,
            _code: mobx_1.observable,
            _cleanCode: mobx_1.observable,
            stores: mobx_1.computed,
            code: mobx_1.computed,
            cleanCode: mobx_1.computed,
            update: mobx_1.action,
        });
    }
    _analysisAst() {
        this._stores = (0, helpers_1.traverseStoreEntryFile)(this.ast);
    }
    /**
     * 新建模型
     * @param name
     */
    addStore(name) {
        this.ast = (0, helpers_1.addStoreToEntryFile)(this.ast, name);
        return this;
    }
    /**
     * 删除模型
     * @param name
     */
    removeStore(name) {
        this.ast = (0, helpers_1.removeStoreToEntryFile)(this.ast, name);
        return this;
    }
}
exports.TangoStoreEntryModule = TangoStoreEntryModule;
/**
 * 状态模型模块
 */
class TangoStoreModule extends module_1.TangoModule {
    constructor(workspace, props) {
        super(workspace, props, false);
        this.name = (0, helpers_1.getModuleNameByFilename)(props.filename);
        this.update(props.code, true, false);
        (0, mobx_1.makeObservable)(this, {
            states: mobx_1.observable,
            actions: mobx_1.observable,
            _code: mobx_1.observable,
            _cleanCode: mobx_1.observable,
            cleanCode: mobx_1.computed,
            code: mobx_1.computed,
            update: mobx_1.action,
        });
    }
    /**
     * 添加状态属性
     * @param stateName
     * @param initValue
     */
    addState(stateName, initValue) {
        this.ast = (0, helpers_1.addStoreState)(this.ast, stateName, initValue);
        return this;
    }
    /**
     * 移除状态
     */
    removeState(stateName) {
        this.ast = (0, helpers_1.removeStoreState)(this.ast, stateName);
        return this;
    }
    /**
     * 更新状态代码
     * @param stateName 状态名
     * @param code 代码
     */
    updateState(stateName, code) {
        this.ast = (0, helpers_1.updateStoreState)(this.ast, stateName, code);
        return this;
    }
    _analysisAst() {
        const { namespace, states, actions } = (0, helpers_1.traverseStoreFile)(this.ast);
        this.namespace = namespace || this.name;
        this.states = states;
        this.actions = actions;
    }
}
exports.TangoStoreModule = TangoStoreModule;
