"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runApp = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const react_dom_1 = tslib_1.__importDefault(require("react-dom"));
const react_router_1 = require("react-router");
const react_router_config_1 = require("react-router-config");
const history_1 = require("./history");
const i18n_1 = tslib_1.__importDefault(require("./i18n"));
const react_i18next_1 = require("react-i18next");
function runApp(config) {
    if (window.__POWERED_BY_QIANKUN__ && config.boot.qiankun) {
        return runQiankunApp(config);
    }
    else {
        runReactApp(config);
    }
    return {};
}
exports.runApp = runApp;
function runReactApp(config) {
    var _a, _b, _c, _d, _e;
    let element;
    const routes = (_b = (_a = config.router) === null || _a === void 0 ? void 0 : _a.config) !== null && _b !== void 0 ? _b : config.routes;
    if (routes) {
        // react router app
        const routerType = (_d = (_c = config.router) === null || _c === void 0 ? void 0 : _c.type) !== null && _d !== void 0 ? _d : 'hash';
        const history = (0, history_1.createHistory)(routerType, routes);
        element = react_1.default.createElement(ReactRouterApp, { history: history, routes: routes, layout: (_e = config.router) === null || _e === void 0 ? void 0 : _e.layout });
    }
    else {
        // single entry app
        const SingleEntry = config.singleEntry || 'div';
        element = react_1.default.createElement(SingleEntry, null);
    }
    // eslint-disable-next-line react/no-deprecated
    react_dom_1.default.render(react_1.default.createElement(react_i18next_1.I18nextProvider, { i18n: i18n_1.default },
        react_1.default.createElement(AppContainer, { providers: config.providers }, element)), config.boot.mountElement);
}
function runQiankunApp(config) {
    // FIXME: 需要支持从外层传入 mountId
    console.log('runQiankunApp:: ', config.router.config);
    const mountId = '#root';
    return {
        bootstrap() {
            return Promise.resolve({});
        },
        mount(props) {
            var _a;
            console.log('mount run, ', props);
            const target = (_a = props.container) !== null && _a !== void 0 ? _a : document;
            runReactApp(Object.assign(Object.assign({}, config), { router: Object.assign(Object.assign({}, config.router), { config: config.router.config.map((route) => {
                        var _a;
                        return Object.assign(Object.assign({}, route), { path: (_a = route.path) === null || _a === void 0 ? void 0 : _a.replace(/^\/portals/, '/site/home') });
                    }) }), boot: Object.assign(Object.assign({}, config.boot), { mountElement: target.querySelector(mountId) }) }));
        },
        unmount(props) {
            var _a;
            console.log('unmount run, ', props);
            const target = (_a = props.container) !== null && _a !== void 0 ? _a : document;
            // eslint-disable-next-line react/no-deprecated
            react_dom_1.default.unmountComponentAtNode(target.querySelector(mountId));
        },
    };
}
function AppContainer({ children: childrenProp, providers = [] }) {
    let children = childrenProp;
    if (providers && providers.length) {
        children = providers.reduce((prev, provider) => {
            if (react_1.default.isValidElement(provider)) {
                return react_1.default.cloneElement(provider, {}, prev);
            }
            return prev;
        }, childrenProp);
    }
    return react_1.default.createElement(react_1.default.Fragment, null, children);
}
function ReactRouterApp({ history, routes = [], layout }) {
    if (!routes.find((route) => !route.path)) {
        // 如果用户没有定义 404 路由，则自动添加一个
        routes.push({
            component: NotFound,
        });
    }
    const elements = react_1.default.createElement(react_router_1.Router, { history: history }, (0, react_router_config_1.renderRoutes)(routes));
    if (!layout)
        return elements;
    return react_1.default.createElement(layout, {}, elements);
}
function NotFound() {
    return react_1.default.createElement("div", null, "not found");
}
