"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Navigator = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const coral_system_1 = require("coral-system");
const tango_ui_1 = require("@music163/tango-ui");
const tango_context_1 = require("@music163/tango-context");
const antd_1 = require("antd");
const icons_1 = require("@ant-design/icons");
const navigatorStyle = (0, coral_system_1.css) `
  display: flex;
  column-gap: 12px;

  .navigatorInput {
    flex: 1;
    border-color: transparent;
    background-color: var(--tango-colors-fill2);

    &:focus {
      background-color: #fff;
    }
  }
`;
class Navigator extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.changeRelativeUrl = (newUrl) => {
            if (newUrl !== this.state.relativeUrl) {
                this.setState({
                    relativeUrl: newUrl,
                });
            }
        };
        this.onInputChange = (e) => {
            const path = e.target.value.startsWith('/') ? e.target.value : `/${e.target.value}`;
            this.changeRelativeUrl(path);
        };
        this.onPressEnter = (e) => {
            var _a, _b;
            const newUrl = e.currentTarget.value;
            (_b = (_a = this.props).onInputEnter) === null || _b === void 0 ? void 0 : _b.call(_a, newUrl, e);
        };
        this.state = {
            relativeUrl: props.startRoute || '/',
        };
    }
    componentDidUpdate(prevProps, prevState) {
        if (this.props.startRoute &&
            prevProps.startRoute !== this.props.startRoute &&
            this.props.startRoute !== prevState.relativeUrl) {
            this.setState({
                relativeUrl: this.props.startRoute,
            });
        }
    }
    render() {
        const { disabled, onBack, onForward, onRefresh, extra } = this.props;
        return (react_1.default.createElement(coral_system_1.Box, { className: "navigator", bg: "white", px: "l", py: "m", borderBottom: "solid", borderBottomColor: "line.normal", css: navigatorStyle },
            react_1.default.createElement(coral_system_1.Box, null,
                react_1.default.createElement(tango_ui_1.Action, { size: "small", icon: react_1.default.createElement(icons_1.ArrowLeftOutlined, null), onClick: onBack, title: "\u8FD4\u56DE" }),
                react_1.default.createElement(tango_ui_1.Action, { size: "small", icon: react_1.default.createElement(icons_1.ArrowRightOutlined, null), onClick: onForward, title: "\u524D\u8FDB" }),
                react_1.default.createElement(tango_ui_1.Action, { size: "small", icon: react_1.default.createElement(icons_1.ReloadOutlined, null), onClick: onRefresh, title: "\u5237\u65B0" })),
            react_1.default.createElement(antd_1.Input, { className: "navigatorInput", size: "small", value: this.state.relativeUrl, onChange: this.onInputChange, onPressEnter: this.onPressEnter, disabled: disabled }),
            react_1.default.createElement(ViewportSwitch, null),
            extra));
    }
}
exports.Navigator = Navigator;
const ViewportSwitch = (0, tango_context_1.observer)(() => {
    const designer = (0, tango_context_1.useDesigner)();
    return (react_1.default.createElement(coral_system_1.Group, { attached: true },
        react_1.default.createElement(tango_ui_1.ToggleButton, { size: "s", selected: designer.simulator.name === 'desktop', onClick: () => {
                designer.setSimulator('desktop');
            } },
            react_1.default.createElement(icons_1.DesktopOutlined, null)),
        react_1.default.createElement(tango_ui_1.ToggleButton, { size: "s", selected: designer.simulator.name === 'phone', onClick: () => {
                designer.setSimulator('phone');
            } },
            react_1.default.createElement(icons_1.MobileOutlined, null))));
});
