"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sandbox = exports.CombinedSandbox = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
const tango_sandbox_1 = require("@music163/tango-sandbox");
const tango_helpers_1 = require("@music163/tango-helpers");
const tango_context_1 = require("@music163/tango-context");
const simulator_1 = require("../simulator");
const context_1 = require("../context");
const dnd_1 = require("../dnd");
const navigator_1 = require("./navigator");
const LANDING_PAGE_PATH = '/__background_landing_page__';
function useSandbox({ isPreview: isPreviewProp, configFormatter, filesFormatter, onViewChange, onMessage: onMessageProp, onLoad: onLoadProp, sandboxType, startRoute, injectScript, } = {}) {
    const workspace = (0, tango_context_1.useWorkspace)();
    const designer = (0, tango_context_1.useDesigner)();
    const sandboxQuery = (0, context_1.useSandboxQuery)();
    const isPreview = isPreviewProp !== null && isPreviewProp !== void 0 ? isPreviewProp : designer.isPreview;
    // 组件不一定会立即刷新，因此 isActive 需要实时获取
    const getIsActive = () => isPreview === designer.isPreview;
    const isActive = getIsActive();
    const getSandboxConfig = () => ({
        sandboxQuery,
        isPreview,
        isActive: getIsActive(),
        sandboxType,
    });
    const dndHandlers = (0, dnd_1.useDnd)({
        sandboxQuery,
        workspace,
        designer,
        onViewChange: (data) => onViewChange && onViewChange(data, getSandboxConfig()),
    });
    let files = Array.from(workspace.files.keys()).reduce((prev, filename) => {
        let code = workspace.getFile(filename).code;
        if (filename === '/tango.config.json') {
            code = mergeTangoConfigJson(code, { isPreview, injectScript, formatter: configFormatter });
        }
        prev[filename] = { code };
        return prev;
    }, {});
    files = fixSandboxFiles(files, workspace.entry);
    if (filesFormatter) {
        // 支持用户对传入的文件进行修改
        files = filesFormatter(files);
    }
    const onMessage = (data) => onMessageProp && onMessageProp(data, getSandboxConfig());
    const onLoad = () => onLoadProp && onLoadProp(getSandboxConfig());
    // 根据当前 workspace 状态与组件传入的状态是否一致，控制是否需要切换到空白路由
    const display = isActive ? 'block' : 'none';
    const routePath = isActive ? startRoute || workspace.activeRoute : LANDING_PAGE_PATH;
    const sandboxProps = isPreview
        ? {
            files,
            eventHandlers: (0, tango_helpers_1.pick)(dndHandlers, ['onTango']),
            onMessage,
            display,
            startRoute: routePath,
            onLoad,
        }
        : {
            files,
            eventHandlers: dndHandlers,
            onMessage,
            display,
            startRoute: routePath,
            onLoad,
        };
    return sandboxProps;
}
const PreviewSandbox = (0, tango_context_1.observer)((props, ref) => {
    const { onViewChange, onMessage, startRoute, isPreview = true, injectScript } = props, rest = tslib_1.__rest(props, ["onViewChange", "onMessage", "startRoute", "isPreview", "injectScript"]);
    const _a = useSandbox(Object.assign({ isPreview,
        onViewChange,
        onMessage,
        startRoute,
        injectScript, sandboxType: 'preview' }, rest)), { display } = _a, sandboxProps = tslib_1.__rest(_a, ["display"]);
    return (react_1.default.createElement(coral_system_1.Box, { display: display, width: "100%", height: "100%" },
        react_1.default.createElement(tango_sandbox_1.CodeSandbox, Object.assign({ ref: ref, iframeId: "preview-sandbox-container" }, sandboxProps, rest))));
}, {
    forwardRef: true,
});
PreviewSandbox.displayName = 'PreviewSandbox';
const DesignSandbox = (0, tango_context_1.observer)((props, ref) => {
    const { onViewChange, onMessage, startRoute, isPreview = false, injectScript } = props, rest = tslib_1.__rest(props, ["onViewChange", "onMessage", "startRoute", "isPreview", "injectScript"]);
    const workspace = (0, tango_context_1.useWorkspace)();
    const _a = useSandbox(Object.assign({ isPreview,
        onViewChange, onMessage: (data, config) => {
            if (onMessage) {
                onMessage(data, config);
            }
            const { sandboxQuery } = config;
            const sandboxRendered = ['success', 'resize'].includes(data.type);
            // 根据代码更新情况重设选中元素的外框
            if (sandboxRendered && workspace.selectSource.isSelected) {
                const items = workspace.selectSource.selected.map((item) => {
                    const element = sandboxQuery.getElementBySlotId(item.id);
                    if (element) {
                        const bounding = sandboxQuery.getElementBounding(element);
                        item.bounding = bounding;
                        item.element = element;
                    }
                    return item;
                });
                workspace.selectSource.select(items);
            }
        }, startRoute,
        injectScript, sandboxType: 'design' }, rest)), { display } = _a, sandboxProps = tslib_1.__rest(_a, ["display"]);
    return (react_1.default.createElement(coral_system_1.Box, { display: display, width: "100%", height: "100%" },
        react_1.default.createElement(tango_sandbox_1.CodeSandbox, Object.assign({ ref: ref }, sandboxProps, rest))));
}, {
    forwardRef: true,
});
exports.CombinedSandbox = (0, tango_context_1.observer)((_a, ref) => {
    var { onViewChange: onViewChangeProp } = _a, rest = tslib_1.__rest(_a, ["onViewChange"]);
    const workspace = (0, tango_context_1.useWorkspace)();
    const designer = (0, tango_context_1.useDesigner)();
    const designSandboxRef = (0, react_1.useRef)();
    const previewSandboxRef = (0, react_1.useRef)();
    const routePath = (0, react_1.useRef)();
    const activeSandbox = (0, react_1.useRef)();
    const [startRoute, setStartRoute] = (0, react_1.useState)(workspace.activeRoute);
    const onViewChange = (data, config) => {
        if (config.isActive) {
            const curPath = (data === null || data === void 0 ? void 0 : data.pathname) + (data === null || data === void 0 ? void 0 : data.search);
            const isSandboxChanged = config.sandboxType !== activeSandbox.current;
            // 沙箱从 inactive -> active，通过跳转回上一页把 landing page pop 出去
            // onViewChange 似乎被 memo 了？useCallback 不生效，使用 ref 存储当前的 sandbox 类型
            if (isSandboxChanged) {
                const curSandbox = config.isPreview
                    ? previewSandboxRef.current
                    : designSandboxRef.current;
                if (curSandbox) {
                    try {
                        // 当 sandbox 状态变为 active 时，此事件后于 isPreview 触发，sandbox 已经开始加载 startRoute
                        // iframe 从 landing page 跳转到 startRoute，因此相较于切换前，历史记录多了 landing page 和 startRoute
                        // 而在此事件中调用的 history.back() 晚于上述跳转的逻辑，所以 history.back() 实际是跳到 landing page
                        // 但是如果不 history.back()，用户主动点击返回按钮，会进入 landing page，因为沙箱始终用 push
                        //
                        // 如果只后退一次，再 replace 到当前路由，其虽然消解了 landing page 的问题，但是会多出一个重复的前进路由
                        // 因此回退两次，再 push 当前的路由，从而将之前沙箱内部的 push 销毁掉
                        curSandbox.iframe.contentWindow.history.go(-2);
                        // sandbox 切换 active 状态时，curPath 是切换时 sandbox 接受到的 startRoute prop
                        // 而当时的 startRoute 还没变更（下面的 useEffect 注定晚于 isPreview 变更，所以 startRoute 还是之前的）
                        // 因此 pop 路由时，取上一次 iframe 记录的 routePath
                        curSandbox.iframe.contentWindow.history.pushState(null, null, routePath.current);
                        curSandbox.iframe.contentWindow.dispatchEvent(new PopStateEvent('popstate'));
                    }
                    catch (err) {
                        // 跨域？
                        console.error(err);
                    }
                }
            }
            else {
                // 如上所述，每次切换预览态时，sandbox 第一次抛出的 curPath 都是上一次 sandbox 的 startRoute，不应该记录
                routePath.current = curPath;
            }
            // 切换沙箱后，需要使用之前沙箱的路由
            activeSandbox.current = config.sandboxType;
        }
        if (onViewChangeProp) {
            onViewChangeProp(data, config);
        }
    };
    (0, react_1.useEffect)(() => {
        // 主动切换 activeRoute 时，需要重置沙箱的路由
        routePath.current = workspace.activeRoute;
        setStartRoute(workspace.activeRoute);
    }, [workspace.activeRoute]);
    // startRoute 只在切换 sandbox 时才重新设置，避免沙箱自行 pushState
    (0, react_1.useEffect)(() => {
        setStartRoute(routePath.current);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [designer.isPreview]);
    (0, react_1.useImperativeHandle)(ref, () => ({
        designSandbox: designSandboxRef.current,
        previewSandbox: previewSandboxRef.current,
    }));
    return (react_1.default.createElement(coral_system_1.Box, { width: "100%", height: "100%" },
        react_1.default.createElement(DesignSandbox, Object.assign({ ref: designSandboxRef, onViewChange: onViewChange, startRoute: startRoute }, rest)),
        react_1.default.createElement(PreviewSandbox, Object.assign({ ref: previewSandboxRef, onViewChange: onViewChange, startRoute: startRoute }, rest))));
}, {
    forwardRef: true,
});
exports.Sandbox = (0, tango_context_1.observer)((_a) => {
    var _b, _c;
    var { isPreview: isPreviewProp, selectionTools, bundlerURL, mode = 'combined', navigatorExtra } = _a, props = tslib_1.__rest(_a, ["isPreview", "selectionTools", "bundlerURL", "mode", "navigatorExtra"]);
    const bundlerUrl = bundlerURL !== null && bundlerURL !== void 0 ? bundlerURL : 'https://codesandbox.fn.netease.com';
    const lastScrollTopRef = (0, react_1.useRef)();
    const sandboxRef = (0, react_1.useRef)(null);
    const combinedSandboxRef = (0, react_1.useRef)(null);
    const navigatorRef = (0, react_1.useRef)(null);
    const workspace = (0, tango_context_1.useWorkspace)();
    const designer = (0, tango_context_1.useDesigner)();
    let sandbox = sandboxRef.current;
    if (mode === 'combined') {
        sandbox = designer.isPreview
            ? (_b = combinedSandboxRef.current) === null || _b === void 0 ? void 0 : _b.previewSandbox
            : (_c = combinedSandboxRef.current) === null || _c === void 0 ? void 0 : _c.designSandbox;
    }
    const onViewChange = (data, { isActive }) => {
        if (isActive) {
            navigatorRef.current.changeRelativeUrl((data === null || data === void 0 ? void 0 : data.pathname) + (data === null || data === void 0 ? void 0 : data.search));
        }
    };
    const onMessage = (data, config = {}) => {
        const { sandboxQuery, isActive } = config;
        if (data.type === 'start' && isActive) {
            // 每次沙箱重新渲染前记住 iframe 内的 scrollTop
            lastScrollTopRef.current = sandboxQuery === null || sandboxQuery === void 0 ? void 0 : sandboxQuery.scrollTop;
        }
    };
    return (react_1.default.createElement(coral_system_1.Box, { className: "SandboxContainer", height: "100%" },
        react_1.default.createElement(navigator_1.Navigator, { ref: navigatorRef, disabled: !(sandbox === null || sandbox === void 0 ? void 0 : sandbox.iframe), startRoute: workspace.activeRoute, onInputEnter: (newUrl) => {
                if ((newUrl === null || newUrl === void 0 ? void 0 : newUrl.split('?')[0]) === workspace.activeRoute) {
                    // 只是输入了路由参数，默认为调试模式，直接更新沙箱内的路由地址
                    sandbox === null || sandbox === void 0 ? void 0 : sandbox.updateRoute(newUrl);
                }
                else {
                    // 如果是输入了其他路由，则直接跳转
                    workspace.setActiveRoute(newUrl);
                }
            }, onBack: () => {
                sandbox === null || sandbox === void 0 ? void 0 : sandbox.manager.iframeProtocol.dispatch({ type: 'urlback' });
            }, onForward: () => {
                sandbox === null || sandbox === void 0 ? void 0 : sandbox.manager.iframeProtocol.dispatch({ type: 'urlforward' });
            }, onRefresh: () => {
                if (!sandbox) {
                    return;
                }
                if (!canAccessIFrame(sandbox.iframe) ||
                    sandbox.iframe.contentWindow.origin !== sandbox.props.bundlerURL) {
                    sandbox.iframe.src = `${bundlerUrl}${workspace.activeRoute}`;
                }
                else {
                    sandbox.manager.iframeProtocol.dispatch({ type: 'refresh' });
                }
                workspace.selectSource.clear();
            }, extra: navigatorExtra }),
        react_1.default.createElement(simulator_1.Simulator, null,
            react_1.default.createElement(simulator_1.Viewport, { selectionTools: selectionTools },
                mode === 'single' && (react_1.default.createElement(DesignSandbox, Object.assign({ ref: sandboxRef, template: "create-react-app", bundlerURL: bundlerUrl, entry: workspace.entry, onViewChange: onViewChange, onMessage: onMessage, startRoute: workspace.activeRoute, isPreview: isPreviewProp !== null && isPreviewProp !== void 0 ? isPreviewProp : designer.isPreview }, props))),
                mode === 'combined' && (react_1.default.createElement(exports.CombinedSandbox, Object.assign({ ref: combinedSandboxRef, template: "create-react-app", bundlerURL: bundlerUrl, entry: workspace.entry, onViewChange: onViewChange, onMessage: onMessage }, props)))))));
});
// 兼容 tango.config.json，转成 sandbox.config.json
function fixSandboxFiles(files, entry = '/src/index.js') {
    if (files['/tango.config.json']) {
        const tangConfigJsonStr = files['/tango.config.json'].code;
        const tangConfigJson = JSON.parse(tangConfigJsonStr);
        files['/sandbox.config.json'] = {
            code: JSON.stringify(tangConfigJson.sandbox, null, 2),
        };
    }
    if (!files['/index.html']) {
        files['/index.html'] = {
            code: `
        <!DOCTYPE html>
        <html lang="en">
          <head>
            <meta charset="UTF-8" />
            <link rel="icon" href="data:image/ico;base64,aWNv">
            <meta name="viewport" content="width=device-width, initial-scale=1.0" />
            <title>Vite App</title>
          </head>
          <body>
            <div id="root" />
            <script type="module" src="${entry}"></script>
          </body>
        </html>
      `,
        };
    }
    return files;
}
function mergeTangoConfigJson(code, { isPreview, injectScript, formatter } = {}) {
    let json;
    try {
        json = JSON.parse(code);
    }
    catch (err) {
        tango_helpers_1.logger.error('Json parse failed!', err);
        return code;
    }
    const userJs = (0, tango_helpers_1.getValue)(json, 'sandbox.evaluateJavaScript') || '';
    let mergedUserJs = userJs;
    const designerScript = !isPreview ? 'window.__TANGO_DESIGNER__ = {}' : '';
    if (designerScript) {
        mergedUserJs = `${designerScript};${mergedUserJs}`;
    }
    if (injectScript) {
        mergedUserJs = `${mergedUserJs};${injectScript}`;
    }
    if (userJs !== mergedUserJs) {
        (0, tango_helpers_1.setValue)(json, 'sandbox.evaluateJavaScript', mergedUserJs);
    }
    // 合并 packages 内的信息至 sandbox
    const packages = (0, tango_helpers_1.getValue)(json, 'packages');
    const externals = (0, tango_helpers_1.getValue)(json, 'sandbox.externals') || {};
    const externalResources = (0, tango_helpers_1.getValue)(json, 'sandbox.externalResources') || [];
    const newExternalResources = (0, tango_helpers_1.getValue)(json, 'externalResources') || [];
    if (packages) {
        // 追加 umd 资源，并替换 url 中的 token，如版本号等
        const pushExternalResources = (list, tokenMap) => {
            const result = list.map((item) => item.replace(/{{(.*?)}}/g, (matched, token) => {
                return (tokenMap === null || tokenMap === void 0 ? void 0 : tokenMap[token]) || matched;
            }));
            externalResources.push(...result);
        };
        // 追加 externals
        const pushExternals = (name, library) => {
            if (library) {
                externals[name] = library;
            }
        };
        Object.keys(packages).forEach((name) => {
            const item = packages[name];
            // 如果是设计态，且拥有设计器资源，则使用设计器资源，否则使用默认资源
            if (item.designerResources && !isPreview) {
                pushExternalResources(item.designerResources, {
                    name,
                    version: item.version,
                });
                pushExternals(name, item.library);
            }
            else if (item.resources) {
                pushExternalResources(item.resources, {
                    name,
                    version: item.version,
                });
                pushExternals(name, item.library);
            }
        });
    }
    if (newExternalResources === null || newExternalResources === void 0 ? void 0 : newExternalResources.length) {
        externalResources.push(...newExternalResources);
    }
    (0, tango_helpers_1.setValue)(json, 'sandbox.externals', externals);
    (0, tango_helpers_1.setValue)(json, 'sandbox.externalResources', [...new Set(externalResources)]);
    if ((0, tango_helpers_1.isFunction)(formatter)) {
        json = formatter === null || formatter === void 0 ? void 0 : formatter(json);
    }
    return JSON.stringify(json);
}
function canAccessIFrame(iframe) {
    let html = null;
    try {
        // deal with older browsers
        const doc = iframe.contentDocument || iframe.contentWindow.document;
        html = doc.body.innerHTML;
    }
    catch (err) {
        // do nothing
    }
    return html !== null;
}
