"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerBuiltinSetters = exports.IdSetter = exports.TextSetter = exports.ExpressionSetter = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const antd_1 = require("antd");
const tango_ui_1 = require("@music163/tango-ui");
const form_item_1 = require("./form-item");
const coral_system_1 = require("coral-system");
function ExpressionSetter(_a) {
    var { value, onChange } = _a, rest = tslib_1.__rest(_a, ["value", "onChange"]);
    return react_1.default.createElement(tango_ui_1.InputCode, Object.assign({ onChange: (val) => onChange === null || onChange === void 0 ? void 0 : onChange(val), value: value || '' }, rest));
}
exports.ExpressionSetter = ExpressionSetter;
function TextSetter(_a) {
    var { onChange } = _a, rest = tslib_1.__rest(_a, ["onChange"]);
    return react_1.default.createElement(antd_1.Input, Object.assign({ onChange: (e) => onChange === null || onChange === void 0 ? void 0 : onChange(e.target.value) }, rest));
}
exports.TextSetter = TextSetter;
const idInputStyle = (0, coral_system_1.css) `
  > .ant-input-borderless {
    border: 1px solid transparent;
    padding: 4px;
  }

  > .ant-input-borderless:hover {
    background-color: var(--tango-colors-fill1);
  }
`;
const idPattern = /^[a-z]+[\w]*$/;
function IdSetter(_a) {
    var { value: valueProp, defaultValue, onChange, placeholder = '输入唯一组件ID' } = _a, rest = tslib_1.__rest(_a, ["value", "defaultValue", "onChange", "placeholder"]);
    const [value, setValue] = (0, react_1.useState)(valueProp || defaultValue);
    const [error, setError] = (0, react_1.useState)('');
    const [editable, setEditable] = (0, react_1.useState)(false);
    // value controlled
    (0, react_1.useEffect)(() => {
        setValue(valueProp);
    }, [valueProp]);
    const __props = editable
        ? {
            status: error ? 'error' : undefined,
            onBlur() {
                setEditable(false);
                if (!error) {
                    onChange === null || onChange === void 0 ? void 0 : onChange(value || undefined);
                }
            },
            onChange(e) {
                const newValue = e.target.value;
                setValue(e.target.value);
                setError(newValue && !idPattern.test(newValue) ? 'error' : '');
            },
        }
        : {
            readOnly: true,
            bordered: false,
            onClick() {
                setEditable(true);
            },
        };
    return (react_1.default.createElement(coral_system_1.Box, { css: idInputStyle },
        react_1.default.createElement(antd_1.Input, Object.assign({ placeholder: placeholder, value: value }, __props, rest))));
}
exports.IdSetter = IdSetter;
function registerBuiltinSetters() {
    // 预注册基础 Setter
    (0, form_item_1.register)({
        name: 'expressionSetter',
        component: ExpressionSetter,
        disableVariableSetter: true,
    });
    (0, form_item_1.register)({
        name: 'textSetter',
        component: TextSetter,
    });
    (0, form_item_1.register)({
        name: 'idSetter',
        component: IdSetter,
    });
}
exports.registerBuiltinSetters = registerBuiltinSetters;
