"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CssSetter = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
// @ts-ignore
const cssjson_1 = require("cssjson");
const antd_1 = require("antd");
const style_setter_1 = require("./style-setter");
const tango_helpers_1 = require("@music163/tango-helpers");
const getRawCssValue = (value) => {
    if (value && value.startsWith('{{css`')) {
        return value.split('').slice(6, -3).join('').trim();
    }
    return value;
};
const getPxValue = (value) => {
    const match = pxPattern.exec(value);
    if (match && match.length) {
        return match[1];
    }
    return '0';
};
const pxPattern = /^(\d+)px$/; // px校验
// 动态正则
const cssPattern = (string, name) => {
    // FIXME: check this pattern
    // eslint-disable-next-line no-useless-escape
    const re1 = new RegExp(`(?<=\s*?\n*?;?\s*?\n*?${name}\s*).*?(?=;)`, 'g');
    // eslint-disable-next-line no-useless-escape
    const re2 = new RegExp(`(?<=^[\s]*?${name}).*?(?=;)`, 'g');
    if (re1.test(string)) {
        return re1;
    }
    else if (re2.test(string)) {
        return re2;
    }
    else {
        return false;
    }
};
/**
 * coral-system css prop
 * @example 提供 css-in-js 代码支持，例如 css`background: red;`
 * @deprecated 使用嵌套属性代替
 */
function CssSetter(props) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
    const { value: valueProp, onChange } = props;
    const [contentValue, setContentValue] = (0, react_1.useState)(() => getRawCssValue(valueProp));
    const [display, setDisplay] = (0, react_1.useState)('');
    // 修改指定的CSS样式
    const changeStyle = (0, react_1.useCallback)((value, name) => {
        if (!contentValue) {
            setContentValue((it) => (it ? `${it}\n${name}: ${value};` : `\n${name}: ${value};`));
        }
        else if (cssPattern(contentValue, name)) {
            const str = contentValue.replace(cssPattern(contentValue, name), `:${value}`);
            setContentValue(str);
        }
        else {
            setContentValue((it) => (it ? `${it}\n${name}: ${value};` : `\n${name}: ${value};`));
        }
    }, [contentValue]);
    // 代码编辑器
    const codeEditorChange = (0, react_1.useCallback)((value) => {
        const val = getRawCssValue(value);
        setContentValue(val);
    }, []);
    (0, react_1.useEffect)(() => {
        if (contentValue || contentValue === '') {
            onChange((0, tango_helpers_1.wrapCode)(`css\`${contentValue}\``));
        }
    }, [contentValue]);
    const isFlex = ['flex', 'inline-flex'].includes(display);
    return (react_1.default.createElement(coral_system_1.Box, { bg: "fill1" },
        react_1.default.createElement(ItemGroup, { title: "\u5E03\u5C40\u65B9\u5F0F" },
            react_1.default.createElement(style_setter_1.DisplaySetter, { value: (_a = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _a === void 0 ? void 0 : _a.display, onChange: (v) => {
                    setDisplay(v);
                    changeStyle(v, 'display');
                } })),
        isFlex ? (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(ItemGroup, { title: "\u4E3B\u8F74\u65B9\u5411" },
                react_1.default.createElement(style_setter_1.FlexDirectionSetter, { value: (_b = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _b === void 0 ? void 0 : _b['flex-direction'], onChange: (v) => {
                        changeStyle(v, 'flex-direction');
                    } })),
            react_1.default.createElement(ItemGroup, { title: "\u4E3B\u8F74\u5BF9\u9F50\u65B9\u5F0F" },
                react_1.default.createElement(style_setter_1.FlexJustifyContentSetter, { value: (_c = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _c === void 0 ? void 0 : _c['justify-content'], onChange: (v) => {
                        changeStyle(v, 'justify-content');
                    } })),
            react_1.default.createElement(ItemGroup, { title: "\u8F85\u8F74\u5BF9\u9F50\u65B9\u5F0F" },
                react_1.default.createElement(style_setter_1.FlexAlignItemsSetter, { value: (_d = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _d === void 0 ? void 0 : _d['align-items'], onChange: (v) => {
                        changeStyle(v, 'align-items');
                    } })))) : null,
        react_1.default.createElement(ItemGroup, { title: "\u5916\u8FB9\u8DDD" },
            react_1.default.createElement(style_setter_1.SpacingSetter, { value: (_e = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _e === void 0 ? void 0 : _e.margin, onChange: (v) => {
                    changeStyle(v, 'margin');
                } })),
        react_1.default.createElement(ItemGroup, { title: "\u5185\u8FB9\u8DDD" },
            react_1.default.createElement(style_setter_1.SpacingSetter, { value: (_f = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _f === void 0 ? void 0 : _f.padding, onChange: (v) => {
                    changeStyle(v, 'padding');
                } })),
        react_1.default.createElement(ItemGroup, { title: "\u5BBD\u9AD8" },
            react_1.default.createElement(antd_1.Space, null,
                react_1.default.createElement(antd_1.InputNumber, { addonBefore: "\u5BBD\u5EA6", addonAfter: "px", defaultValue: getPxValue((_g = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _g === void 0 ? void 0 : _g.width), onChange: (v) => {
                        changeStyle(`${v}px`, 'width');
                    } }),
                react_1.default.createElement(antd_1.InputNumber, { addonBefore: "\u9AD8\u5EA6", addonAfter: "px", defaultValue: getPxValue((_h = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _h === void 0 ? void 0 : _h.height), onChange: (v) => {
                        changeStyle(`${v}px`, 'height');
                    } }))),
        react_1.default.createElement(ItemGroup, { title: "\u6587\u5B57\u989C\u8272" },
            react_1.default.createElement(style_setter_1.ColorSetter, { value: (_j = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _j === void 0 ? void 0 : _j.color, onChange: (v) => {
                    changeStyle(v, 'color');
                } })),
        react_1.default.createElement(ItemGroup, { title: "\u6587\u5B57\u5927\u5C0F" },
            react_1.default.createElement(antd_1.InputNumber, { addonBefore: "\u5B57\u4F53\u5927\u5C0F", addonAfter: "px", defaultValue: getPxValue((_k = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _k === void 0 ? void 0 : _k['font-size']), onChange: (v) => {
                    changeStyle(`${v}px`, 'font-size');
                } })),
        react_1.default.createElement(ItemGroup, { title: "\u80CC\u666F" },
            react_1.default.createElement(style_setter_1.BgSetter, { value: (_l = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _l === void 0 ? void 0 : _l.background, onChange: (v) => {
                    if (RegExp(/url/).test(v)) {
                        changeStyle(v, 'background-image');
                    }
                    else {
                        changeStyle(v, 'background-color');
                    }
                } })),
        react_1.default.createElement(ItemGroup, { title: "\u8FB9\u6846" },
            react_1.default.createElement(style_setter_1.BorderSetter, { value: (_m = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _m === void 0 ? void 0 : _m.border, onChange: (v) => {
                    changeStyle(v, 'border');
                } })),
        react_1.default.createElement(ItemGroup, { title: "\u900F\u660E\u5EA6" },
            react_1.default.createElement(antd_1.Slider, { max: 1, min: 0, step: 0.1, value: ((_o = (0, cssjson_1.toJSON)(contentValue).attributes) === null || _o === void 0 ? void 0 : _o.opacity) || 1, onChange: (v) => {
                    changeStyle(v, 'opacity');
                } })),
        react_1.default.createElement(ItemGroup, { title: "\u6837\u5F0F\u7F16\u5199" },
            react_1.default.createElement(style_setter_1.CssCodeSetter, { value: contentValue, onChange: codeEditorChange }))));
}
exports.CssSetter = CssSetter;
function ItemGroup({ title, children }) {
    return (react_1.default.createElement(coral_system_1.Box, { borderBottom: "solid", borderBottomColor: "gray.30", px: "m" },
        react_1.default.createElement(coral_system_1.Box, { pt: "m" }, title),
        react_1.default.createElement(coral_system_1.Box, { py: "m" }, children)));
}
