"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventSetter = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
const antd_1 = require("antd");
const tango_ui_1 = require("@music163/tango-ui");
const tango_context_1 = require("@music163/tango-context");
const tango_helpers_1 = require("@music163/tango-helpers");
const expression_setter_1 = require("./expression-setter");
const tango_core_1 = require("@music163/tango-core");
var EventAction;
(function (EventAction) {
    EventAction["NoAction"] = "noAction";
    EventAction["ConsoleLog"] = "consoleLog";
    EventAction["BindExpression"] = "bindExpression";
    EventAction["OpenModal"] = "openModal";
    EventAction["CloseModal"] = "closeModal";
    EventAction["NavigateTo"] = "navigateTo";
})(EventAction || (EventAction = {}));
const wrapperStyle = (0, coral_system_1.css) `
  .ant-select,
  .ant-input {
    width: 100%;
    margin-bottom: 8px;
  }
`;
const options = [
    { label: '无动作', value: EventAction.NoAction },
    { label: '打印事件', value: EventAction.ConsoleLog },
    { label: '绑定 JS 表达式', value: EventAction.BindExpression },
    { label: '打开页面', value: EventAction.NavigateTo },
    { label: '打开弹窗', value: EventAction.OpenModal },
    { label: '关闭弹窗', value: EventAction.CloseModal },
];
/**
 * 事件监听函数绑定器
 */
function EventSetter(props) {
    const { value, onChange, modalTitle } = props;
    const [type, setType] = (0, react_1.useState)(); // 事件类型
    const [temp, setTemp] = (0, react_1.useState)(''); // 二级暂存值
    const [expModalVisible, setExpModalVisible] = (0, react_1.useState)(false); // 弹窗是否显示
    const { actionVariables, routeOptions } = (0, tango_context_1.useWorkspaceData)();
    const workspace = (0, tango_context_1.useWorkspace)();
    const modalOptions = workspace.activeViewModule.listModals() || [];
    const code = (0, tango_core_1.value2code)(value);
    const handleChange = (0, react_1.useCallback)((nextValue, ...args) => {
        if (!nextValue) {
            onChange(undefined);
        }
        if (nextValue !== code) {
            onChange((0, tango_helpers_1.wrapCode)(nextValue), ...args);
        }
    }, [onChange, code]);
    const onAction = (key) => {
        setType(key); // 记录事件类型
        setTemp(''); // 重置二级选项值
        switch (key) {
            case EventAction.BindExpression:
                setExpModalVisible(true);
                break;
            case EventAction.ConsoleLog:
                handleChange('(...args) => console.log(...args)');
                break;
            case EventAction.NoAction:
                handleChange(undefined);
                break;
            default:
                break;
        }
    };
    const actionText = getActionText(type, temp, code);
    return (react_1.default.createElement(coral_system_1.Box, { css: wrapperStyle },
        react_1.default.createElement(tango_ui_1.ActionSelect, { options: options, onSelect: onAction, text: actionText }),
        react_1.default.createElement(expression_setter_1.ExpressionModal, { title: modalTitle, value: code, visible: expModalVisible, onCancel: () => setExpModalVisible(false), onOk: (nextValue) => {
                setExpModalVisible(false);
                handleChange(nextValue);
            }, dataSource: actionVariables }),
        type === EventAction.NavigateTo && (react_1.default.createElement(antd_1.AutoComplete, { placeholder: "\u9009\u62E9\u6216\u8F93\u5165\u9875\u9762\u8DEF\u7531", options: routeOptions, value: temp, onChange: setTemp, onBlur: () => {
                if (temp) {
                    handleChange(getExpressionValue(type, temp));
                }
            } })),
        type === EventAction.OpenModal && (react_1.default.createElement(antd_1.AutoComplete, { placeholder: "\u9009\u62E9\u6216\u8F93\u5165\u5F39\u7A97 id", options: modalOptions, value: temp, onChange: setTemp, onBlur: () => {
                if (temp) {
                    handleChange(getExpressionValue(type, temp));
                }
            } })),
        type === EventAction.CloseModal && (react_1.default.createElement(antd_1.AutoComplete, { placeholder: "\u9009\u62E9\u6216\u8F93\u5165\u5F39\u7A97 id", options: modalOptions, value: temp, onChange: setTemp, onBlur: () => {
                if (temp) {
                    handleChange(getExpressionValue(type, temp));
                }
            } }))));
}
exports.EventSetter = EventSetter;
const handlerMap = {
    [EventAction.OpenModal]: 'openModal',
    [EventAction.CloseModal]: 'closeModal',
    [EventAction.NavigateTo]: 'navigateTo',
};
function getActionText(type, temp, fallbackCode) {
    let text;
    if (handlerMap[type]) {
        text = getExpressionValue(type, temp);
    }
    else if (fallbackCode) {
        text = fallbackCode;
    }
    text = text || '请选择';
    return text;
}
function getExpressionValue(type, value = '') {
    const handler = handlerMap[type];
    if (handler) {
        return `() => tango.${handler}("${value}")`;
    }
}
