"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpressionModal = exports.ExpressionSetter = exports.jsonValueValidate = exports.expressionValueValidate = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
const antd_1 = require("antd");
const tango_core_1 = require("@music163/tango-core");
const tango_helpers_1 = require("@music163/tango-helpers");
const icons_1 = require("@ant-design/icons");
const tango_ui_1 = require("@music163/tango-ui");
const tango_context_1 = require("@music163/tango-context");
const components_1 = require("../components");
const context_1 = require("../context");
const helpers_1 = require("../helpers");
const interface_config_1 = require("../sidebar/datasource-panel/interface-config");
const expressionValueValidate = (value) => {
    if (!(0, tango_core_1.isValidExpressionCode)(value)) {
        return '表达式存在语法错误！';
    }
};
exports.expressionValueValidate = expressionValueValidate;
const jsonValueValidate = (value) => {
    try {
        JSON.parse(value);
    }
    catch (e) {
        return '不是合法的 JSON 语法！';
    }
};
exports.jsonValueValidate = jsonValueValidate;
const suffixStyle = (0, coral_system_1.css) `
  display: flex;
  align-items: center;
  .anticon-close-circle {
    color: rgba(0, 0, 0, 0.25);
    &:hover {
      color: rgba(0, 0, 0, 0.45);
    }
  }
`;
function ExpressionSetter(props) {
    const { onChange, modalTitle, modalTip, autoCompleteOptions, placeholder = '在这里输入JS代码', value: valueProp, status, allowClear = true, newStoreTemplate, showOptionsDropDown = true, } = props;
    // const codeValue = getCodeOfWrappedCode(valueProp);
    const [inputValue, setInputValue] = (0, react_1.useState)(valueProp);
    const [visible, { on, off }] = (0, tango_helpers_1.useBoolean)();
    // when receive new value, sync state
    (0, react_1.useEffect)(() => {
        setInputValue(valueProp);
    }, [valueProp]);
    const change = (0, react_1.useCallback)((code) => {
        if (code === valueProp) {
            return;
        }
        onChange(code);
    }, [valueProp, onChange]);
    const sandbox = (0, context_1.useSandboxQuery)();
    const evaluateContext = sandbox.window;
    return (react_1.default.createElement(coral_system_1.Box, { className: "ExpressionSetter" },
        react_1.default.createElement(tango_ui_1.InputCode, { placeholder: placeholder, suffix: react_1.default.createElement(coral_system_1.Box, { css: suffixStyle },
                allowClear && (react_1.default.createElement(icons_1.CloseCircleFilled, { title: "\u6E05\u7A7A", onClick: () => {
                        change('');
                    } })),
                showOptionsDropDown && (autoCompleteOptions === null || autoCompleteOptions === void 0 ? void 0 : autoCompleteOptions.length) && (react_1.default.createElement(antd_1.Dropdown, { menu: {
                        items: autoCompleteOptions.map((option) => ({
                            key: option,
                            label: (react_1.default.createElement(coral_system_1.Text, { onClick: () => {
                                    change(option);
                                } }, option)),
                        })),
                    } },
                    react_1.default.createElement(tango_ui_1.Action, { tooltip: "\u4F7F\u7528\u9884\u8BBE\u4EE3\u7801\u7247\u6BB5", icon: react_1.default.createElement(icons_1.MenuOutlined, null), size: "small" }))),
                react_1.default.createElement(tango_ui_1.Action, { tooltip: "\u6253\u5F00\u8868\u8FBE\u5F0F\u53D8\u91CF\u9009\u62E9\u9762\u677F", icon: react_1.default.createElement(icons_1.ExpandAltOutlined, null), onClick: on, size: "small" })), value: inputValue, onChange: setInputValue, onBlur: () => {
                change(inputValue);
            }, autoCompleteContext: evaluateContext, autoCompleteOptions: autoCompleteOptions, status: status, maxHeight: "200px" }),
        react_1.default.createElement(ExpressionModal, { title: modalTitle, subTitle: modalTip, placeholder: placeholder, autoCompleteOptions: autoCompleteOptions, newStoreTemplate: newStoreTemplate, visible: visible, value: inputValue, onCancel: () => off(), onOk: (value) => {
                change(value);
                off();
            } })));
}
exports.ExpressionSetter = ExpressionSetter;
function ExpressionModal({ title, subTitle, placeholder, visible, onCancel = tango_helpers_1.noop, onOk = tango_helpers_1.noop, defaultValue, value, dataSource, autoCompleteOptions, newStoreTemplate = helpers_1.CODE_TEMPLATES.newStoreTemplate, }) {
    const [exp, setExp] = (0, react_1.useState)(value !== null && value !== void 0 ? value : defaultValue);
    const [error, setError] = (0, react_1.useState)('');
    const workspace = (0, tango_context_1.useWorkspace)();
    const { expressionVariables } = (0, tango_context_1.useWorkspaceData)();
    const serviceModules = Object.keys(workspace.serviceModules).map((key) => ({
        label: key === 'index' ? '默认模块' : key,
        value: key,
    }));
    const sandbox = (0, context_1.useSandboxQuery)();
    const evaluateContext = sandbox.window;
    const handleExpInputChange = (val) => {
        setExp(val === null || val === void 0 ? void 0 : val.trim());
    };
    (0, react_1.useEffect)(() => {
        setExp(value);
    }, [value]);
    return (react_1.default.createElement(antd_1.Modal, { closable: false, destroyOnClose: true, width: "60%", open: visible, onCancel: onCancel, onOk: () => {
            onOk(exp);
        }, bodyStyle: {
            padding: 0,
        } },
        react_1.default.createElement(tango_ui_1.Panel, { title: `将 ${title} 设置为引用变量或自定义表达式`, subTitle: subTitle, shape: "solid" },
            react_1.default.createElement(tango_ui_1.InputCode, { shape: "inset", minHeight: "56px", maxHeight: "200px", value: exp, placeholder: placeholder, onChange: handleExpInputChange, onBlur: () => {
                    setError((0, exports.expressionValueValidate)(exp));
                }, autoCompleteContext: evaluateContext, autoCompleteOptions: autoCompleteOptions }),
            error ? (react_1.default.createElement(coral_system_1.Text, { color: "red", fontSize: "12px" }, "\u51FA\u9519\u4E86\uFF01\u8F93\u5165\u7684\u8868\u8FBE\u5F0F\u5B58\u5728\u8BED\u6CD5\u9519\u8BEF\uFF0C\u8BF7\u4FEE\u6539\u540E\u518D\u63D0\u4EA4\uFF01")) : (react_1.default.createElement(coral_system_1.Text, { fontSize: "12px", color: "text3" }, "\u8BF4\u660E\uFF1A\u4F60\u53EF\u4EE5\u5728\u4E0A\u9762\u7684\u4EE3\u7801\u8F93\u5165\u6846\u91CC\u8F93\u5165\u5E38\u89C4\u7684 javascript \u4EE3\u7801\uFF0C\u8FD8\u53EF\u4EE5\u76F4\u63A5\u4F7F\u7528 jsx \u4EE3\u7801\uFF0C\u4F46\u9700\u8981\u7B26\u5408\u8BE5\u5C5E\u6027\u7684\u63A5\u53D7\u503C\u5B9A\u4E49\u3002"))),
        react_1.default.createElement(tango_ui_1.Panel, { title: "\u4ECE\u53D8\u91CF\u5217\u8868\u4E2D\u9009\u4E2D", shape: "solid", borderTop: "0", overflow: "hidden", bodyProps: { overflow: 'hidden' } },
            react_1.default.createElement(components_1.VariableTree, { height: 380, showViewButton: true, dataSource: dataSource || expressionVariables, appContext: sandbox === null || sandbox === void 0 ? void 0 : sandbox.window['tango'], getStoreNames: () => Object.keys(workspace.storeModules), serviceModules: serviceModules, getServiceData: (serviceKey) => {
                    const data = workspace.getServiceFunction(serviceKey);
                    return Object.assign({ name: data.name, moduleName: data.moduleName, method: 'get' }, data.config);
                }, onSelect: (node) => {
                    if (!node.key) {
                        return;
                    }
                    if (node.key.split('.').length < 2) {
                        return;
                    }
                    let str;
                    if (/^(stores|services)\./.test(node.key)) {
                        str = `tango.${node.key.replaceAll('.', '?.')}`;
                    }
                    else {
                        str = `${node.key}`;
                    }
                    setExp(str);
                }, onAddStoreVariable: (storeName, data) => {
                    workspace.addStoreState(storeName, data.name, data.initialValue);
                }, onUpdateStoreVariable: (variableKey, code) => {
                    workspace.updateStoreVariable(variableKey, code);
                }, onAddStore: (storeName) => {
                    workspace.addStoreFile(storeName, newStoreTemplate);
                }, onRemoveStoreVariable: (variableKey) => {
                    workspace.removeStoreVariable(variableKey);
                }, onRemoveService: (serviceKey) => {
                    workspace.removeServiceFunction(serviceKey);
                }, onAddService: (data) => {
                    const _a = (0, interface_config_1.shapeServiceValues)(data), { name, moduleName } = _a, payload = tslib_1.__rest(_a, ["name", "moduleName"]);
                    workspace.addServiceFunction(name, payload, moduleName);
                }, onUpdateService: (data) => {
                    const _a = (0, interface_config_1.shapeServiceValues)(data), { name, moduleName } = _a, payload = tslib_1.__rest(_a, ["name", "moduleName"]);
                    workspace.updateServiceFunction(name, payload, moduleName);
                }, getPreviewValue: (node) => {
                    if (!node || !node.key) {
                        return;
                    }
                    if (node.type === 'function') {
                        return;
                    }
                    return (0, tango_helpers_1.getValue)(evaluateContext['tango'], node.key);
                } }))));
}
exports.ExpressionModal = ExpressionModal;
