"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListSetter = exports.AddListItemButton = exports.ListSetterItem = exports.useListSetter = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
const antd_1 = require("antd");
const icons_1 = require("@ant-design/icons");
const components_1 = require("../components");
function copyWithoutUndefined(source) {
    const ret = {};
    for (const key in source) {
        if (source[key] !== undefined) {
            ret[key] = source[key];
        }
    }
    return ret;
}
function useListSetter(props) {
    const { value: valueProp = [], onChange: onChangeProp } = props, rest = tslib_1.__rest(props, ["value", "onChange"]);
    const value = Array.isArray(valueProp) ? valueProp : [];
    const onChange = (next = []) => {
        onChangeProp && onChangeProp(next);
    };
    const getOnEdit = (index) => {
        return (data) => {
            const copy = [...value];
            copy[index] = data;
            onChange(copy);
        };
    };
    const getOnRemove = (index) => {
        return () => {
            const copy = [...value];
            copy.splice(index, 1);
            onChange(copy);
        };
    };
    const onAdd = (data) => {
        const next = [...value].concat([data]);
        onChange(next);
    };
    const onMove = (start, end) => {
        const copy = [...value];
        const temp = copy[end];
        copy[end] = copy[start];
        copy[start] = temp;
        onChange(copy);
    };
    return Object.assign({ value,
        onAdd,
        getOnEdit,
        getOnRemove,
        onMove }, rest);
}
exports.useListSetter = useListSetter;
const listItemStyle = (0, coral_system_1.css) `
  display: flex;
  justify-content: space-between;
  border: 1px solid var(--tango-colors-line-normal);
  border-radius: var(--tango-radii-m);
  line-height: 2;
  padding-left: 12px;

  &:hover {
    background-color: var(--tango-colors-gray-10);
  }
`;
const leftCss = (0, coral_system_1.css) `
  display: inline-block;

  .icon {
    margin-right: 8px;
    cursor: move;
  }
`;
function ListSetterItem(_a) {
    var { children, onRemove, index = 0, onMove } = _a, rest = tslib_1.__rest(_a, ["children", "onRemove", "index", "onMove"]);
    return (react_1.default.createElement(components_1.DragBox, { mb: "m", index: index, css: listItemStyle, onMove: onMove },
        react_1.default.createElement(coral_system_1.Box, { css: leftCss },
            react_1.default.createElement(icons_1.HolderOutlined, { className: "icon" }),
            children),
        react_1.default.createElement("div", null,
            react_1.default.createElement(EditButton, Object.assign({}, rest)),
            react_1.default.createElement(antd_1.Button, { type: "link", icon: react_1.default.createElement(icons_1.DeleteOutlined, null), onClick: onRemove }))));
}
exports.ListSetterItem = ListSetterItem;
function EditButton({ formFields, formInitialValues, onEdit }) {
    const [visible, setVisible] = (0, react_1.useState)(false);
    return (react_1.default.createElement(antd_1.Popover, { visible: visible, onVisibleChange: setVisible, trigger: "click", placement: "leftTop", title: "\u4FEE\u6539\u6570\u636E", content: react_1.default.createElement(NewOptionForm, { fields: formFields, initialValues: formInitialValues, onSubmit: (values) => {
                onEdit && onEdit(values);
                setVisible(false);
            } }) },
        react_1.default.createElement(antd_1.Button, { type: "link", icon: react_1.default.createElement(icons_1.EditOutlined, null) })));
}
function AddListItemButton({ formFields = [], formInitialValues = {}, getFormInitialValues, text = '添加一项', onAdd, }) {
    var _a;
    const [visible, setVisible] = (0, react_1.useState)(false);
    const initialValues = (_a = getFormInitialValues === null || getFormInitialValues === void 0 ? void 0 : getFormInitialValues()) !== null && _a !== void 0 ? _a : formInitialValues;
    return (react_1.default.createElement(antd_1.Popover, { destroyTooltipOnHide: true, visible: visible, onVisibleChange: setVisible, trigger: "click", placement: "leftTop", title: text, content: react_1.default.createElement(NewOptionForm, { fields: formFields, initialValues: initialValues, onSubmit: (values) => {
                onAdd && onAdd(values);
                setVisible(false);
            } }) },
        react_1.default.createElement(antd_1.Button, { block: true }, text)));
}
exports.AddListItemButton = AddListItemButton;
const layout = {
    labelCol: { span: 8 },
    wrapperCol: { span: 16 },
};
const tailLayout = {
    wrapperCol: { offset: 8, span: 16 },
};
function NewOptionForm({ fields = [], initialValues = {}, onSubmit }) {
    const [form] = antd_1.Form.useForm();
    const onFinish = (values) => {
        onSubmit && onSubmit(copyWithoutUndefined(values));
    };
    return (react_1.default.createElement(antd_1.Form, Object.assign({ form: form, onFinish: onFinish }, layout),
        fields.map((item) => (react_1.default.createElement(antd_1.Form.Item, { key: item.name, label: item.label, name: item.name, required: item.required, rules: [{ required: item.required }], valuePropName: item.valuePropName, extra: item.extra, style: {
                width: item.width,
            }, initialValue: initialValues[item.name] }, item.component || react_1.default.createElement(antd_1.Input, { placeholder: "\u8BF7\u8F93\u5165", autoComplete: "off" })))),
        react_1.default.createElement(antd_1.Form.Item, Object.assign({}, tailLayout),
            react_1.default.createElement(antd_1.Button, { type: "primary", htmlType: "submit" }, "\u63D0\u4EA4"))));
}
const defaultListItemFormFields = [
    { label: 'key', name: 'key', required: true },
];
const defaultGetKey = (item) => item.key;
const defaultRenderItem = (item) => item.key;
/**
 * 列表项设置器
 */
function ListSetter(props) {
    const { listItemFormFields = defaultListItemFormFields, newItemDefaultValues, getNewItemDefaultValues, addBtnText, getListItemKey = defaultGetKey, renderItem = defaultRenderItem } = props, rest = tslib_1.__rest(props, ["listItemFormFields", "newItemDefaultValues", "getNewItemDefaultValues", "addBtnText", "getListItemKey", "renderItem"]);
    const { value, onAdd, getOnEdit, getOnRemove, onMove } = useListSetter(rest);
    return (react_1.default.createElement(coral_system_1.Box, null,
        react_1.default.createElement(coral_system_1.Box, null, value.map((item, index) => {
            return (react_1.default.createElement(ListSetterItem, { key: getListItemKey(item), index: index, formFields: listItemFormFields, formInitialValues: item, onEdit: getOnEdit(index), onRemove: getOnRemove(index), onMove: onMove }, renderItem(item)));
        })),
        react_1.default.createElement(AddListItemButton, { text: addBtnText, formFields: listItemFormFields, formInitialValues: newItemDefaultValues, getFormInitialValues: getNewItemDefaultValues, onAdd: onAdd })));
}
exports.ListSetter = ListSetter;
