"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelSetter = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
const antd_1 = require("antd");
const tango_helpers_1 = require("@music163/tango-helpers");
const icons_1 = require("@ant-design/icons");
const tango_context_1 = require("@music163/tango-context");
const components_1 = require("../components");
const context_1 = require("../context");
const helpers_1 = require("../helpers");
function object2treeData(val, keyPath, currentDepth = 0, maxDepth = 999, getProps) {
    const title = keyPath.split('.').slice(-1)[0];
    const rest = getProps === null || getProps === void 0 ? void 0 : getProps(keyPath, val);
    if (typeof val !== 'object' || currentDepth > maxDepth) {
        return Object.assign({ title, key: keyPath }, rest);
    }
    return Object.assign({ title, key: keyPath, selectable: false, children: Object.keys(val).reduce((acc, key) => {
            acc.push(object2treeData(val[key], `${keyPath}.${key}`, currentDepth + 1, maxDepth, getProps));
            return acc;
        }, []) }, rest);
}
function traverseTreeData(val, callback) {
    if (Array.isArray(val)) {
        val.forEach((child) => traverseTreeData(child, callback));
    }
    else {
        callback(val);
    }
    if ('children' in val) {
        traverseTreeData(val.children, callback);
    }
}
function ModelSetter({ value, onChange, newStoreTemplate = helpers_1.CODE_TEMPLATES.newStoreTemplate, }) {
    var _a, _b;
    const [inputValue, setInputValue] = (0, react_1.useState)(value);
    const { modelVariables } = (0, tango_context_1.useWorkspaceData)();
    const evaluateContext = (0, context_1.useSandboxQuery)().window || {};
    const workspace = (0, tango_context_1.useWorkspace)();
    const definedVariables = (0, react_1.useMemo)(() => {
        const list = [];
        traverseTreeData(modelVariables, (node) => {
            if (node.key.split('.').length > 1) {
                list.push(node.key);
            }
        });
        return list;
    }, [modelVariables]);
    const variables = ((_a = evaluateContext['tango']) === null || _a === void 0 ? void 0 : _a.stores)
        ? [
            object2treeData((_b = evaluateContext['tango']) === null || _b === void 0 ? void 0 : _b.stores, 'stores', 0, 1, (keyPath, val) => {
                const ret = {};
                if ((0, tango_helpers_1.isFunction)(val)) {
                    ret.disabled = true;
                    ret.type = 'function';
                }
                if (!definedVariables.includes(keyPath)) {
                    ret.showAddButton = false;
                    ret.showRemoveButton = false;
                }
                else if (/^stores\.[a-zA-Z0-9]+$/.test(keyPath)) {
                    ret.showAddButton = true;
                }
                return ret;
            }),
        ]
        : modelVariables;
    (0, react_1.useEffect)(() => {
        // 同步受控值的变化
        if (value !== inputValue) {
            setInputValue(value);
        }
    }, [value]);
    const onInputChange = (0, react_1.useCallback)((e) => {
        var _a;
        setInputValue((_a = e.target) === null || _a === void 0 ? void 0 : _a.value);
    }, []);
    const onInputBlur = (0, react_1.useCallback)((e) => {
        var _a;
        const next = (_a = e.target) === null || _a === void 0 ? void 0 : _a.value;
        onChange(next || undefined);
    }, [onChange]);
    return (react_1.default.createElement(coral_system_1.Box, null,
        react_1.default.createElement(antd_1.Input, { value: inputValue, placeholder: "\u5C06\u7ED3\u679C\u540C\u6B65\u5230\u7684\u53D8\u91CF\uFF0C\u4F8B\u5982storeName.varName", allowClear: true, autoFocus: false, onChange: onInputChange, onBlur: onInputBlur, suffix: react_1.default.createElement(components_1.VariableTreeModal, { title: "\u540C\u6B65\u5230\u7684\u53D8\u91CF", trigger: react_1.default.createElement(antd_1.Tooltip, { title: "\u4ECE\u6A21\u578B\u5217\u8868\u9009\u62E9", placement: "topRight" },
                    react_1.default.createElement(icons_1.MenuOutlined, null)), dataSource: variables, onSelect: (node) => {
                    const modelPath = node.key.split('.').slice(1).join('.');
                    onChange(modelPath);
                }, onAddStoreVariable: (storeName, data) => {
                    workspace.addStoreState(storeName, data.name, data.initialValue);
                }, onUpdateStoreVariable: (variableKey, code) => {
                    workspace.updateStoreVariable(variableKey, code);
                }, onAddStore: (storeName) => {
                    workspace.addStoreFile(storeName, newStoreTemplate);
                }, onRemoveStoreVariable: (variablePath) => {
                    workspace.removeStoreVariable(variablePath);
                }, onRemoveService: (serviceKey) => {
                    workspace.removeServiceFunction(serviceKey);
                }, getStoreNames: () => Object.keys(workspace.storeModules), getPreviewValue: (node) => {
                    if (!node || !node.key) {
                        return;
                    }
                    return (0, tango_helpers_1.getValue)(evaluateContext['tango'], node.key);
                } }) })));
}
exports.ModelSetter = ModelSetter;
