"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BorderSetter = exports.SpacingSetter = exports.BgSetter = exports.ColorSetter = exports.FlexGapSetter = exports.FlexAlignItemsSetter = exports.FlexJustifyContentSetter = exports.FlexDirectionSetter = exports.DisplaySetter = exports.CssCodeSetter = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
const antd_1 = require("antd");
const react_color_1 = require("react-color");
const color_1 = tslib_1.__importDefault(require("color"));
const icons_1 = require("@ant-design/icons");
const tango_ui_1 = require("@music163/tango-ui");
const choice_setter_1 = require("./choice-setter");
function getRawCssValue(value) {
    if (value && value.startsWith('{css`')) {
        return value.split('').slice(5, -2).join('').trim();
    }
    return value;
}
// TODO: style object setter
/**
 * 不稳定，暂不推荐使用
 */
function CssCodeSetter({ value, onChange }) {
    const contentValue = getRawCssValue(value);
    return (react_1.default.createElement(antd_1.Popover, { placement: "leftBottom", title: "\u81EA\u5B9A\u4E49\u6837\u5F0F\uFF08CSS in JS\uFF09", content: react_1.default.createElement(coral_system_1.Box, { height: 480, width: 400 },
            react_1.default.createElement(tango_ui_1.SingleMonacoEditor, { language: "css", value: contentValue, onBlur: (newCode) => {
                    if (newCode != contentValue) {
                        onChange(`{css\`${newCode}\`}`, {});
                    }
                } })) },
        react_1.default.createElement(antd_1.Button, { block: true }, "\u7F16\u5199\u6837\u5F0F")));
}
exports.CssCodeSetter = CssCodeSetter;
/**
 * css display
 * @param props
 * @returns
 */
function DisplaySetter(props) {
    return (react_1.default.createElement(choice_setter_1.ChoiceSetter, Object.assign({ options: [
            { value: 'block', icon: 'icon-display-block', tip: '块级元素' },
            { value: 'inline-block', icon: 'icon-display-inline-block', tip: '行内块级元素' },
            { value: 'flex', icon: 'icon-flex', tip: 'Flex 弹性布局' },
            { value: 'grid', icon: 'icon-display-grid', tip: 'Grid 弹性网格布局' },
            { value: 'inline', icon: 'icon-display-inline', tip: '行内元素' },
            { value: 'none', icon: 'icon-display-none', tip: '不显示' },
        ], mode: "icon" }, props)));
}
exports.DisplaySetter = DisplaySetter;
function FlexDirectionSetter(props) {
    return (react_1.default.createElement(choice_setter_1.ChoiceSetter, Object.assign({ options: [
            { value: 'column', icon: 'icon-direction-column', tip: '单行排列' },
            { value: 'column-reverse', icon: 'icon-direction-column-reverse', tip: '单行逆序' },
            { value: 'row', icon: 'icon-direction-row', tip: '纵向排列' },
            { value: 'row-reverse', icon: 'icon-direction-row-reverse', tip: '纵向逆序' },
        ], mode: "icon" }, props)));
}
exports.FlexDirectionSetter = FlexDirectionSetter;
function FlexJustifyContentSetter(props) {
    return (react_1.default.createElement(choice_setter_1.ChoiceSetter, Object.assign({ options: [
            { value: 'flex-start', icon: 'icon-justify-flex-start', tip: '从行首起始位置开始排列' },
            { value: 'flex-end', icon: 'icon-justify-flex-end', tip: '从行尾位置开始排列' },
            { value: 'center', icon: 'icon-justify-center', tip: '居中排列' },
            {
                value: 'space-between',
                icon: 'icon-justify-space-between',
                tip: '均匀排列每个元素，首个元素放置于起点，末尾元素放置于终点',
            },
            {
                value: 'space-around',
                icon: 'icon-justify-space-around',
                tip: '均匀排列每个元素，每个元素周围分配相同的空间',
            },
        ], mode: "icon" }, props)));
}
exports.FlexJustifyContentSetter = FlexJustifyContentSetter;
function FlexAlignItemsSetter(props) {
    return (react_1.default.createElement(choice_setter_1.ChoiceSetter, Object.assign({ options: [
            { value: 'flex-start', icon: 'icon-align-flex-start', tip: '元素向辅轴起点对齐' },
            { value: 'flex-end', icon: 'icon-align-flex-end', tip: '元素向辅轴终点对齐' },
            { value: 'center', icon: 'icon-align-center', tip: '元素在辅轴居中' },
            {
                value: 'stretch',
                icon: 'icon-align-stretch',
                tip: '弹性项包含外边距的交叉轴尺寸被拉升至行高',
            },
            { value: 'baseline', icon: 'icon-align-baseline', tip: '所有元素向基线对齐' },
        ], mode: "icon" }, props)));
}
exports.FlexAlignItemsSetter = FlexAlignItemsSetter;
function FlexGapSetter({ value, onChange }) {
    const values = parseGapValue(value);
    const tempRef = (0, react_1.useRef)([...values]);
    const handleChange = (index) => {
        return (val) => {
            tempRef.current[index] = `${val}px`;
            onChange(tempRef.current.join(' '));
        };
    };
    return (react_1.default.createElement(coral_system_1.Grid, { columns: 2, spacing: "m" },
        react_1.default.createElement(antd_1.InputNumber, { addonBefore: "\u4E3B", addonAfter: "px", defaultValue: values[0], onChange: handleChange(0) }),
        react_1.default.createElement(antd_1.InputNumber, { addonBefore: "\u8F85", addonAfter: "px", defaultValue: values[1], onChange: handleChange(1) })));
}
exports.FlexGapSetter = FlexGapSetter;
function rgbaObjectToString(rbg) {
    return `rgba(${rbg.r}, ${rbg.g}, ${rbg.b}, ${rbg.a})`;
}
function normalizeColorObject(colorObject) {
    const { r, g, b, alpha } = colorObject;
    return {
        r,
        g,
        b,
        a: alpha,
    };
}
function ColorSetter(props) {
    const { value, onChange, placeholder = '请选择颜色' } = props;
    const color = hexColorPattern.test(value) ? value : normalizeColorObject((0, color_1.default)(value).object());
    const colorPanel = (react_1.default.createElement(react_color_1.SketchPicker, { color: color, onChangeComplete: (newColor) => {
            const nextValue = newColor.rgb.a === 1 ? newColor.hex : rgbaObjectToString(newColor.rgb);
            if (typeof onChange === 'function') {
                onChange(nextValue);
            }
        } }));
    return (react_1.default.createElement(antd_1.Dropdown, { overlay: colorPanel, trigger: ['click'] },
        react_1.default.createElement(antd_1.Input, { prefix: react_1.default.createElement(coral_system_1.Box, { bg: value, border: "solid", borderColor: "gray.40", size: "14px", borderRadius: "s" }), allowClear: true, placeholder: placeholder, value: value, onChange: (e) => {
                if (!e.target.value) {
                    // clear input
                    onChange(undefined);
                }
            } })));
}
exports.ColorSetter = ColorSetter;
const hexColorPattern = /^#[0-9a-fA-F]{3,}$/;
const imgUrlPattern = /^url\((.+)\)$/;
const getImageUrl = (url) => {
    const ret = imgUrlPattern.exec(url);
    if (ret && ret.length) {
        return ret[1];
    }
    return '';
};
function BgSetter({ value, onChange }) {
    const [mode, setMode] = (0, react_1.useState)(() => {
        if (!value) {
            return 'color';
        }
        return hexColorPattern.test(value) ? 'color' : 'image';
    });
    return (react_1.default.createElement(coral_system_1.Box, null,
        react_1.default.createElement(antd_1.Radio.Group, { defaultValue: mode, onChange: (e) => setMode(e.target.value), buttonStyle: "solid" },
            react_1.default.createElement(antd_1.Radio.Button, { value: "color" },
                react_1.default.createElement(icons_1.BgColorsOutlined, null)),
            react_1.default.createElement(antd_1.Radio.Button, { value: "image" },
                react_1.default.createElement(icons_1.FileImageOutlined, null))),
        react_1.default.createElement(coral_system_1.Box, { mt: "m" },
            mode === 'color' && react_1.default.createElement(ColorSetter, { value: value, onChange: onChange }),
            mode === 'image' && (react_1.default.createElement(antd_1.Input, { value: getImageUrl(value), onChange: (e) => {
                    const imgUrl = e.target.value;
                    onChange(imgUrl ? `url(${imgUrl})` : undefined);
                } })))));
}
exports.BgSetter = BgSetter;
function SpacingSetter({ value, onChange }) {
    const values = parseSpacingValue(value);
    const tempRef = (0, react_1.useRef)([...values]);
    const handleChange = (index) => {
        return (val) => {
            tempRef.current[index] = `${val}`;
            const str2px = tempRef.current.map((it) => {
                return `${it}px`;
            });
            onChange(str2px.join(' '));
        };
    };
    return (react_1.default.createElement(coral_system_1.Grid, { columns: 2, spacing: "m" },
        react_1.default.createElement(antd_1.InputNumber, { addonBefore: "\u4E0A", addonAfter: "px", defaultValue: values[0], onChange: handleChange(0) }),
        react_1.default.createElement(antd_1.InputNumber, { addonBefore: "\u4E0B", addonAfter: "px", defaultValue: values[2], onChange: handleChange(2) }),
        react_1.default.createElement(antd_1.InputNumber, { addonBefore: "\u53F3", addonAfter: "px", defaultValue: values[1], onChange: handleChange(1) }),
        react_1.default.createElement(antd_1.InputNumber, { addonBefore: "\u5DE6", addonAfter: "px", defaultValue: values[3], onChange: handleChange(3) })));
}
exports.SpacingSetter = SpacingSetter;
function parseSpacingValue(value) {
    if (!value || typeof value !== 'string') {
        return ['0', '0', '0', '0'];
    }
    const arr = value.split(' ');
    if (arr.length === 1) {
        return ['0', '0', '0', '0'];
    }
    if (arr.length === 2) {
        const x = getPxValue(arr[1]);
        const y = getPxValue(arr[0]);
        return [y, x, y, x];
    }
    if (arr.length === 4) {
        return arr.map(getPxValue);
    }
}
function parseGapValue(value) {
    if (!value || typeof value !== 'string') {
        return ['0', '0'];
    }
    const arr = value.split(' ');
    if (arr.length === 1) {
        return ['0', '0'];
    }
    return arr.map(getPxValue);
}
const pxPattern = /^(\d+)px$/;
function getPxValue(value) {
    const match = pxPattern.exec(value);
    if (match && match.length) {
        return match[1];
    }
    return '0';
}
function parseBorderValue(value) {
    if (!value || typeof value !== 'string') {
        return ['0', 'none'];
    }
    return value.split(' ');
}
function BorderSetter({ value, onChange }) {
    const valueParts = parseBorderValue(value);
    const [style, setStyle] = (0, react_1.useState)(() => {
        if (/^(none|solid|dashed)$/.test(valueParts[1])) {
            return valueParts[1];
        }
        return 'none';
    });
    const [width, setWidth] = (0, react_1.useState)(Number(getPxValue(valueParts[0])) || 0);
    const [color, setColor] = (0, react_1.useState)(valueParts[2] || '#000');
    const handleChange = (key, val) => {
        let ret;
        switch (key) {
            case 'style': {
                ret = [width, val, color];
                break;
            }
            case 'width': {
                ret = [`${val}px`, style, color];
                break;
            }
            case 'color': {
                ret = [width, style, val];
                break;
            }
            default:
                ret = ['none'];
                break;
        }
        if (ret[1] === 'none') {
            ret = ['none'];
        }
        onChange(ret.join(' '));
    };
    return (react_1.default.createElement(coral_system_1.Box, null,
        react_1.default.createElement(Label, { label: "\u98CE\u683C" },
            react_1.default.createElement(antd_1.Radio.Group, { defaultValue: style, onChange: (e) => {
                    setStyle(e.target.value);
                    handleChange('style', e.target.value);
                }, buttonStyle: "solid" },
                react_1.default.createElement(antd_1.Radio.Button, { value: "none" },
                    react_1.default.createElement(icons_1.EyeInvisibleOutlined, null)),
                react_1.default.createElement(antd_1.Radio.Button, { value: "solid" },
                    react_1.default.createElement(tango_ui_1.LineSolidOutlined, null)),
                react_1.default.createElement(antd_1.Radio.Button, { value: "dashed" },
                    react_1.default.createElement(tango_ui_1.LineDashedOutlined, null)))),
        style !== 'none' && (react_1.default.createElement(coral_system_1.Box, null,
            react_1.default.createElement(Label, { label: "\u5BBD\u5EA6", mt: "m" },
                react_1.default.createElement(antd_1.InputNumber, { defaultValue: width, addonAfter: "px", min: 0, onChange: (val) => {
                        setWidth(val);
                        handleChange('width', val);
                    } })),
            react_1.default.createElement(Label, { label: "\u989C\u8272", mt: "m" },
                react_1.default.createElement(ColorSetter, { value: color, onChange: (val) => {
                        setColor(val);
                        handleChange('color', val);
                    } }))))));
}
exports.BorderSetter = BorderSetter;
function Label(_a) {
    var { label, children } = _a, rest = tslib_1.__rest(_a, ["label", "children"]);
    return (react_1.default.createElement(coral_system_1.Box, Object.assign({ display: "flex", alignItems: "center" }, rest),
        react_1.default.createElement(coral_system_1.Box, { mr: "m" }, label),
        react_1.default.createElement(coral_system_1.Box, null, children)));
}
