"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SettingPanel = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const coral_system_1 = require("coral-system");
const tango_setting_form_1 = require("@music163/tango-setting-form");
const tango_ui_1 = require("@music163/tango-ui");
const tango_helpers_1 = require("@music163/tango-helpers");
const tango_context_1 = require("@music163/tango-context");
const setters_1 = require("./setters");
(0, setters_1.registerBuiltinSetters)();
const headerProps = {
    fontSize: '18px',
};
exports.SettingPanel = (0, tango_context_1.observer)((props) => {
    var _a, _b;
    const workspace = (0, tango_context_1.useWorkspace)();
    const designer = (0, tango_context_1.useDesigner)();
    if (!designer.showRightPanel || !workspace.selectSource.isSelected) {
        return react_1.default.createElement(coral_system_1.Box, { className: "SettingPanel" });
    }
    let display = 'flex';
    if (designer.activeView !== 'design' || designer.isPreview) {
        display = 'none';
    }
    const getSettingValue = (attributes) => {
        const ret = {};
        const keys = Object.keys(attributes);
        for (const key of keys) {
            if (/^data-/.test(key)) {
                continue;
            }
            ret[key] = attributes[key];
        }
        return ret;
    };
    const formValue = getSettingValue(((_a = workspace.selectSource.firstNode) === null || _a === void 0 ? void 0 : _a.props) || {});
    const formModel = new tango_setting_form_1.FormModel(formValue, {
        onChange(name, value, field) {
            var _a;
            if (!name) {
                return;
            }
            if (name === 'tid' && workspace.activeViewModule.hasNodeByCodeId(value)) {
                // 存在重复的 tid，不支持修改
                return;
            }
            let firstName = name;
            let realValue = value;
            const namePaths = name.split('.');
            if (namePaths.length > 1) {
                firstName = namePaths[0];
                realValue = (0, tango_helpers_1.clone)(formModel.getValue(firstName), false);
                if (!Object.keys(realValue).length) {
                    realValue = undefined;
                }
            }
            if (!field) {
                // 针对 Form.Object 清空的情况
                workspace.updateSelectedNodeAttributes({
                    [firstName]: realValue,
                });
            }
            else if (!field.error) {
                // 针对 Form.Item 变化的情况
                workspace.updateSelectedNodeAttributes({ [firstName]: realValue }, (_a = field.detail) === null || _a === void 0 ? void 0 : _a.relatedImports);
            }
        },
    });
    const prototype = workspace.componentPrototypes.get((_b = workspace.selectSource.first) === null || _b === void 0 ? void 0 : _b.name);
    return (react_1.default.createElement(tango_ui_1.Panel, { display: display, flexDirection: "column", width: "320px", borderLeft: "solid", borderLeftColor: "line2", bg: "white", headerProps: headerProps, bodyProps: {
            overflowY: 'hidden',
        }, className: "SettingPanel" }, prototype ? (react_1.default.createElement(tango_setting_form_1.SettingForm, Object.assign({ key: workspace.selectSource.first.id, model: formModel, prototype: prototype, showIdentifier: {
            identifierKey: 'tid',
            getIdentifier: () => {
                // 直接拿当前的虚拟 id 作为 tid
                const { codeId } = (0, tango_helpers_1.parseDndId)(workspace.selectSource.first.id);
                return codeId;
            },
        } }, props))) : (react_1.default.createElement(coral_system_1.Box, { p: "m" }, "\u6CA1\u6709\u627E\u5230\u4E0E\u8BE5\u7EC4\u4EF6\u76F8\u5173\u7684\u914D\u7F6E\u4FE1\u606F"))));
});
