"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComponentsPanel = exports.useFlatMenuData = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
const styled_components_1 = tslib_1.__importDefault(require("styled-components"));
const tango_helpers_1 = require("@music163/tango-helpers");
const tango_ui_1 = require("@music163/tango-ui");
const tango_context_1 = require("@music163/tango-context");
const icons_1 = require("@ant-design/icons");
const antd_1 = require("antd");
const helpers_1 = require("../helpers");
const localeMap = {
    common: '基础组件',
    atom: '原子组件',
    snippet: '组合',
    bizComp: '业务组件',
    localComp: '本地组件',
};
const emptyMenuData = {
    common: [],
};
function useFlatMenuData(menuData) {
    const flatList = (0, react_1.useMemo)(() => {
        const set = new Set();
        Object.values(menuData).forEach((cates) => {
            cates.forEach((cate) => {
                var _a;
                (_a = cate.items) === null || _a === void 0 ? void 0 : _a.forEach((item) => {
                    set.add(item);
                });
            });
        });
        return Array.from(set);
    }, [menuData]);
    return [
        {
            title: '搜索结果',
            items: flatList,
        },
    ];
}
exports.useFlatMenuData = useFlatMenuData;
exports.ComponentsPanel = (0, tango_context_1.observer)(({ menuData = emptyMenuData, showBizComps = true, getBizCompName = tango_helpers_1.upperCamelCase, loading = false, }) => {
    const [keyword, setKeyword] = (0, react_1.useState)('');
    const allList = useFlatMenuData(menuData);
    const workspace = (0, tango_context_1.useWorkspace)();
    const [bizCompData, localCompData] = (0, react_1.useMemo)(() => [
        [{ title: '已安装业务组件', items: workspace.bizComps.map(getBizCompName) }],
        [{ title: '本地组件', items: workspace.localComps }],
    ], [workspace.bizComps, workspace.localComps, getBizCompName]);
    const tabs = Object.keys(menuData).map((key) => ({
        key,
        label: localeMap[key],
        children: react_1.default.createElement(MaterialList, { data: menuData[key] }),
    }));
    if (showBizComps) {
        tabs.push({
            key: 'bizComp',
            label: localeMap.bizComp,
            children: react_1.default.createElement(MaterialList, { type: "bizComp", data: bizCompData }),
        });
    }
    if (localCompData.length) {
        tabs.push({
            key: 'localComps',
            label: localeMap.localComp,
            children: react_1.default.createElement(MaterialList, { data: localCompData }),
        });
    }
    const contentNode = tabs.length === 1 ? (tabs[0].children) : (react_1.default.createElement(tango_ui_1.Tabs, { centered: true, isTabBarSticky: true, tabBarStickyOffset: 48, items: tabs }));
    return (react_1.default.createElement(coral_system_1.Box, { className: "ComponentsView", overflowY: "auto" },
        react_1.default.createElement(coral_system_1.Box, { px: "l", py: "m", position: "sticky", top: "0", zIndex: 1, bg: "white" },
            react_1.default.createElement(tango_ui_1.Search, { placeholder: "\u641C\u7D22\u7269\u6599", onChange: setKeyword })),
        react_1.default.createElement(antd_1.Spin, { spinning: loading, tip: "\u6B63\u5728\u52A0\u8F7D\u7269\u6599\u5217\u8868..." }, !keyword ? contentNode : react_1.default.createElement(MaterialList, { data: allList, filterKeyword: keyword }))));
});
function MaterialList({ data, filterKeyword, type = 'common' }) {
    const workspace = (0, tango_context_1.useWorkspace)();
    return (react_1.default.createElement(coral_system_1.Box, { className: "ComponentsViewList" }, data.map((cate) => {
        let items = cate.items || [];
        if (filterKeyword) {
            items = items.filter((item) => {
                const pattern = new RegExp(filterKeyword, 'ig');
                const prototype = workspace.componentPrototypes.get(item);
                return prototype && (pattern.test(prototype.name) || pattern.test(prototype.title));
            });
        }
        const addBlock = items.length % 2 === 1;
        return (react_1.default.createElement(coral_system_1.Box, { key: cate.title },
            react_1.default.createElement(tango_ui_1.CollapsePanel, { key: cate.title, title: cate.title, borderBottom: "solid", borderColor: "line.normal" },
                !items.length && (react_1.default.createElement(antd_1.Empty, { image: antd_1.Empty.PRESENTED_IMAGE_SIMPLE, description: "\u6CA1\u6709\u5339\u914D\u5230\u4EFB\u4F55\u7EC4\u4EF6" })),
                react_1.default.createElement(coral_system_1.Grid, { columns: type === 'localComp' ? 1 : 2, spacing: "1px", bg: "background.normal", padding: "0" },
                    items.map((item) => {
                        const prototype = workspace.componentPrototypes.get(item);
                        if (!prototype) {
                            tango_helpers_1.logger.log(`<${item}> prototype not found!`);
                        }
                        return prototype ? react_1.default.createElement(MaterialGrid, { key: item, data: prototype }) : null;
                    }),
                    addBlock && react_1.default.createElement(coral_system_1.Box, { bg: "white" })))));
    })));
}
const StyledCommonGridItem = styled_components_1.default.div `
  position: relative;
  display: flex;
  flex-direction: column;
  align-items: center;
  justify-content: center;
  padding: 8px;
  cursor: move;
  text-align: center;
  color: var(--tango-colors-text-body);
  background-color: #fff;
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;

  .material-icon {
    font-size: 40px;
  }

  .info {
    position: absolute;
    right: 4px;
    top: 4px;
    cursor: pointer;
  }

  .anticon-question-circle {
    display: none;
    position: absolute;
    top: 8px;
    right: 8px;
  }

  img {
    height: 40px;
    width: 40px;
  }

  &:hover {
    box-shadow: 0 0 10px rgb(0 0 0 / 10%);

    > span {
      color: var(--tango-colors-brand);
    }

    .anticon-question-circle {
      display: inline-block;
    }
  }
`;
function MaterialGrid({ data }) {
    const workspace = (0, tango_context_1.useWorkspace)();
    const handleDragStart = (e) => {
        e.dataTransfer.effectAllowed = 'move';
        workspace.dragSource.set({
            name: data.name,
        });
        const ghost = (0, helpers_1.getDragGhostElement)();
        e.dataTransfer.setDragImage(ghost, 0, 0);
    };
    const handleDragEnd = () => {
        workspace.dragSource.clear();
    };
    const icon = data.icon || 'icon-placeholder';
    return (react_1.default.createElement(StyledCommonGridItem, { draggable: true, "data-name": data.name, onDragStart: handleDragStart, onDragEnd: handleDragEnd },
        icon.startsWith('icon-') ? (react_1.default.createElement(tango_ui_1.IconFont, { className: "material-icon", type: data.icon || 'icon-placeholder' })) : (react_1.default.createElement("img", { src: icon, alt: data.name })),
        react_1.default.createElement(coral_system_1.Text, { fontSize: "12px", lineHeight: "1.5" }, data.title),
        react_1.default.createElement(coral_system_1.Text, { fontSize: "12px", color: "gray.50" }, data.name),
        data.docs || data.help ? (react_1.default.createElement(antd_1.Popover, { placement: "right", title: data.title, content: react_1.default.createElement(CommonMaterialInfoBox, Object.assign({}, data)) },
            react_1.default.createElement(icons_1.QuestionCircleOutlined, null))) : null));
}
function CommonMaterialInfoBox({ help, docs }) {
    return (react_1.default.createElement(coral_system_1.Box, { maxWidth: 300 },
        !!help && react_1.default.createElement(coral_system_1.Box, { mb: "m" }, help),
        !!docs && (react_1.default.createElement(antd_1.Button, { size: "small", block: true, target: "_blank", href: docs }, "\u67E5\u770B\u7EC4\u4EF6\u6587\u6863"))));
}
