"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComponentsTree = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const antd_1 = require("antd");
const coral_system_1 = require("coral-system");
const tango_ui_1 = require("@music163/tango-ui");
const icons_1 = require("@ant-design/icons");
const tango_context_1 = require("@music163/tango-context");
const tango_core_1 = require("@music163/tango-core");
const tango_helpers_1 = require("@music163/tango-helpers");
const context_1 = require("../../context");
const helpers_1 = require("../../helpers");
const outlineStyle = (0, coral_system_1.css) `
  .ant-tree-title {
    color: rgb(85, 85, 85);
  }

  .ant-tree .ant-tree-treenode {
    padding: 2px;
  }

  .ant-tree-indent-unit {
    width: 12px;
  }

  .ant-tree .ant-tree-node-content-wrapper {
    white-space: nowrap;
    transition: none;
  }
  .variable-row {
    font-size: 12px;
  }
  .ant-tree-switcher-noop {
    display: none;
  }
  .material-icon {
    width: 1em;
    height: 1em;
  }
`;
const filedNames = {
    title: 'component',
    key: 'id',
    children: 'children',
};
const getNodeKeys = (data) => {
    const ids = [];
    data === null || data === void 0 ? void 0 : data.forEach((node) => {
        ids.push(node.id);
        if (node.children) {
            ids.push(...getNodeKeys(node.children));
        }
    });
    return ids;
};
const OutlineTreeNode = (0, tango_context_1.observer)(({ node, showToggleVisibleIcon, actionItems }) => {
    const workspace = (0, tango_context_1.useWorkspace)();
    const sandboxQuery = (0, context_1.useSandboxQuery)();
    const [visible, setVisible] = (0, react_1.useState)(true);
    const nodeLabel = (() => {
        const { codeId } = (0, tango_helpers_1.parseDndId)(node.id);
        return codeId;
    })();
    const componentPrototype = workspace.componentPrototypes.get(node.component);
    const icon = (componentPrototype === null || componentPrototype === void 0 ? void 0 : componentPrototype.icon) || 'icon-placeholder';
    const iconRender = icon.startsWith('icon-') ? (react_1.default.createElement(tango_ui_1.IconFont, { className: "material-icon", type: icon })) : (react_1.default.createElement("img", { className: "material-icon", src: icon, alt: componentPrototype.name }));
    const toggleVisible = (nodeId) => {
        sandboxQuery.getElement((0, helpers_1.buildQueryBySlotId)(nodeId)).style.visibility = visible
            ? 'hidden'
            : 'visible';
        setVisible(!visible);
    };
    const VisibleIcon = visible ? icons_1.EyeOutlined : icons_1.EyeInvisibleOutlined;
    return (react_1.default.createElement(antd_1.Space, null,
        iconRender,
        " ",
        nodeLabel,
        showToggleVisibleIcon && (react_1.default.createElement(VisibleIcon, { className: "material-icon", onClick: (e) => {
                e.stopPropagation();
                toggleVisible(node.id);
            } })),
        (actionItems === null || actionItems === void 0 ? void 0 : actionItems.length) > 0 && (react_1.default.createElement(antd_1.Dropdown, { menu: {
                items: actionItems.map((_) => ({
                    key: _.key,
                    label: _.item,
                })),
            }, trigger: ['click'] },
            react_1.default.createElement(icons_1.EllipsisOutlined, null)))));
});
exports.ComponentsTree = (0, tango_context_1.observer)(({ showToggleVisibleIcon = true, onSelect = tango_helpers_1.noop, actionItems }) => {
    var _a, _b;
    const workspace = (0, tango_context_1.useWorkspace)();
    const sandboxQuery = (0, context_1.useSandboxQuery)();
    const [selectedKeys, setSelectedKeys] = (0, react_1.useState)(workspace.selectSource.selected.map((item) => item.id));
    const file = workspace.activeViewModule;
    const nodesTree = ((_a = file === null || file === void 0 ? void 0 : file.nodesTree) !== null && _a !== void 0 ? _a : []);
    const [expandedKeys, setExpandedKeys] = (0, react_1.useState)(getNodeKeys(nodesTree));
    (0, react_1.useEffect)(() => {
        setSelectedKeys(workspace.selectSource.selected.map((item) => item.id));
    }, [workspace.selectSource.selected]);
    (0, react_1.useEffect)(() => {
        setExpandedKeys(getNodeKeys(nodesTree));
        // nodeTree 更新后重新计算expandKeys
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [(_b = nodesTree === null || nodesTree === void 0 ? void 0 : nodesTree[0]) === null || _b === void 0 ? void 0 : _b.id]);
    if (!nodesTree.length) {
        return (react_1.default.createElement(coral_system_1.Box, null,
            "\u6CA1\u6709\u6307\u5B9A\u6587\u4EF6\uFF0C\u6216\u6587\u4EF6",
            react_1.default.createElement("code", null, workspace.activeViewFile),
            "\u6CA1\u6709\u6709\u6548\u7684\u8282\u70B9\u6811"));
    }
    return (react_1.default.createElement(coral_system_1.Box, { css: outlineStyle },
        react_1.default.createElement(antd_1.Tree, { selectedKeys: selectedKeys, fieldNames: filedNames, treeData: nodesTree, onSelect: (keys) => {
                const slotKey = keys === null || keys === void 0 ? void 0 : keys[0];
                const data = sandboxQuery.getDraggableParentsData((0, helpers_1.buildQueryBySlotId)(slotKey), true);
                if (data && data.id) {
                    workspace.selectSource.select(data);
                }
                // export selected
                onSelect(slotKey);
                setSelectedKeys(keys);
            }, blockNode: true, expandedKeys: expandedKeys, onExpand: (keys) => setExpandedKeys(keys), draggable: true, onDragStart: (data) => {
                const prototype = workspace.componentPrototypes.get(data.node.component);
                if (!prototype) {
                    return;
                }
                const { canDrag } = prototype.rules || {};
                if (canDrag && !canDrag()) {
                    return;
                }
                workspace.dragSource.set({
                    id: data.node.id,
                    name: data.node.component,
                });
            }, onDrop: (data) => {
                const dropKey = data.node.key;
                let method;
                if (data.dropToGap) {
                    // 插入节点的后面
                    method = tango_core_1.DropMethod.InsertAfter;
                }
                else {
                    // 作为第一个子节点
                    method = tango_core_1.DropMethod.InsertFirstChild;
                }
                workspace.dragSource.dropTarget.set({
                    id: dropKey,
                }, method);
                workspace.dropNode();
            }, titleRender: (node) => (react_1.default.createElement(OutlineTreeNode, { actionItems: actionItems, showToggleVisibleIcon: showToggleVisibleIcon, node: node })) })));
});
