"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResizableBox = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const coral_system_1 = require("coral-system");
const react_resizable_1 = require("react-resizable");
const resizeHandleStyle = (axis, barStyle) => (0, coral_system_1.css) `
  position: absolute;
  z-index: 999;
  cursor: ${axis === 'x' ? 'col-resize' : 'row-resize'};
  ${barStyle &&
    Object.keys(barStyle)
        .map((key) => `${key}: ${barStyle[key]};`)
        .join('')}
  &:hover,
  &:active {
    background-color: var(--tango-colors-brand);
  }
`;
const getResizeHandles = (resizeHandlePosition, axis) => {
    const handles = [];
    switch (axis) {
        case 'x':
            handles.push(resizeHandlePosition === 'right' ? 'e' : 'w');
            break;
        case 'y':
            handles.push(resizeHandlePosition === 'bottom' ? 's' : 'n');
            break;
        default:
            break;
    }
    return handles;
};
function ResizableBox({ resizeHandlePosition = 'right', width: widthProp, height: heightProp, children, className, style, axis = 'x', }) {
    const [width, setWidth] = (0, react_1.useState)(widthProp);
    const [height, setHeight] = (0, react_1.useState)(heightProp);
    const barStyle = Object.assign({}, (axis === 'x'
        ? {
            height: '100%',
            width: '4px',
            top: '0',
            [resizeHandlePosition === 'right' ? 'right' : 'left']: '-4px',
        }
        : {
            height: '4px',
            width: '100%',
            left: '0',
            [resizeHandlePosition === 'bottom' ? 'bottom' : 'top']: '0',
        }));
    (0, react_1.useEffect)(() => {
        setWidth(widthProp);
    }, [widthProp]);
    (0, react_1.useEffect)(() => {
        setHeight(heightProp);
    }, [heightProp]);
    return (react_1.default.createElement(react_resizable_1.Resizable, { axis: axis, width: width, resizeHandles: getResizeHandles(resizeHandlePosition, axis), height: height, minConstraints: [150, 150], maxConstraints: [window.innerWidth * 0.6, window.innerHeight * 0.8], onResize: (e, { size }) => {
            if (axis === 'x') {
                setWidth(size.width);
            }
            if (axis === 'y') {
                setHeight(size.height);
            }
        }, onResizeStart: () => {
            document.body.style.pointerEvents = 'none';
            document.body.style.userSelect = 'none';
        }, onResizeStop: () => {
            document.body.style.pointerEvents = 'auto';
            document.body.style.userSelect = 'auto';
        }, handle: react_1.default.createElement(coral_system_1.Box, { className: "ResizeHandle", css: resizeHandleStyle(axis, barStyle) }) },
        react_1.default.createElement("div", { className: (0, classnames_1.default)('ResizableBox', className), style: Object.assign({ position: 'relative', width,
                height }, style) }, children)));
}
exports.ResizableBox = ResizableBox;
