"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sidebar = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const coral_system_1 = require("coral-system");
const antd_1 = require("antd");
const tango_context_1 = require("@music163/tango-context");
const resizable_box_1 = require("./resizable-box");
const widgets_1 = require("../widgets");
const sidebarStyle = (0, coral_system_1.css) `
  position: relative;
  background-color: var(--tango-colors-custom-sidebarBg);
  border-right: 1px solid var(--tango-colors-line-normal);
  box-shadow: rgb(0 0 0 / 5%) 0px 0px 18px;
  height: 100%;

  .SidebarPanelBarList {
    width: 50px;
    list-style: none;
    user-select: none;
    transition: all 0.3s ease;
  }

  .SidebarPanelBarListItem {
    &.active {
      background-color: var(--tango-colors-custom-sidebarItemActiveBg);
    }

    &:hover {
      background-color: var(--tango-colors-custom-sidebarItemHoverBg);
    }
  }
`;
function BaseSidebarPanel({ panelWidth: defaultPanelWidth = 280, footer, children }) {
    const designer = (0, tango_context_1.useDesigner)();
    const items = (0, react_1.useMemo)(() => {
        const ret = {};
        react_1.default.Children.forEach(children, (child) => {
            if (child && react_1.default.isValidElement(child)) {
                const widget = (0, widgets_1.getWidget)(['sidebar', child.key].join('.'));
                const fallbackNode = widget ? react_1.default.createElement(widget, child.props.widgetProps) : null;
                ret[child.key] = Object.assign({ children: fallbackNode }, child.props);
            }
        });
        return ret;
    }, [children]);
    return (react_1.default.createElement(coral_system_1.Box, { display: "flex", flexShrink: 0, css: sidebarStyle, className: "SidebarPanel" },
        react_1.default.createElement(coral_system_1.Box, { className: "SidebarPanelBar", display: "flex", flexDirection: "column", position: "relative" },
            react_1.default.createElement(coral_system_1.Box, { as: "ul", flex: "0", p: "0", m: "0", textAlign: "center", className: "SidebarPanelBarList" }, Object.keys(items).map((key) => {
                const item = items[key];
                const isActive = key === designer.activeSidebarPanel;
                return (react_1.default.createElement("li", { className: (0, classnames_1.default)('SidebarPanelBarListItem', { active: isActive }), key: key, title: item.title, onClick: () => {
                        designer.setActiveSidebarPanel(key);
                    } },
                    react_1.default.createElement(SidebarPanelBarItem, { icon: item.icon, label: item.label, showBadge: item.showBadge, isActive: isActive })));
            })),
            react_1.default.createElement(coral_system_1.Box, { className: "SidebarPanelBarFooter", pb: "l", flex: "1", display: "flex", flexDirection: "column", alignItems: "center", justifyContent: "flex-end" }, footer)),
        Object.keys(items).map((key) => {
            const panel = items[key];
            const floatPanelStyle = (panel === null || panel === void 0 ? void 0 : panel.isFloat)
                ? {
                    position: 'absolute',
                    left: '50px',
                    top: 0,
                    zIndex: 1000,
                    height: '100%',
                    boxShadow: 'var(--tango-shadows-lowRight)',
                }
                : {};
            const panelWidth = typeof (panel === null || panel === void 0 ? void 0 : panel.width) === 'number' ? panel === null || panel === void 0 ? void 0 : panel.width : defaultPanelWidth;
            return (react_1.default.createElement(resizable_box_1.ResizableBox, { key: panel.label, width: panelWidth, style: Object.assign({ display: designer.activeSidebarPanel === key ? 'block' : 'none' }, floatPanelStyle) },
                react_1.default.createElement(SidebarPanelExpandedContent, { isFloat: panel.isFloat, closeable: panel.isFloat, onClose: () => {
                        designer.closeSidebarPanel();
                    } }, panel.children)));
        })));
}
BaseSidebarPanel.Item = function ({ key, title, doc, isFloat, width, widgetProps, }) {
    return react_1.default.createElement(react_1.default.Fragment, null);
};
exports.Sidebar = (0, tango_context_1.observer)(BaseSidebarPanel);
function SidebarPanelBarItem({ isActive, icon: iconProp, label, showBadge, }) {
    const color = isActive ? 'brand' : 'text.body';
    let icon = (react_1.default.createElement(coral_system_1.Text, { fontSize: "24px", lineHeight: 1, color: color }, iconProp));
    if (showBadge) {
        icon = (react_1.default.createElement(antd_1.Badge, Object.assign({ size: "small" }, showBadge), icon));
    }
    return (react_1.default.createElement(coral_system_1.Box, { display: "flex", flexDirection: "column", alignItems: "center", justifyContent: "center", py: "12px" },
        icon,
        react_1.default.createElement(coral_system_1.Text, { fontSize: "12px", mt: "s", color: color }, label)));
}
const expandPanelStyle = (0, coral_system_1.css) `
  flex: 1;
  height: 100%;
  background-color: var(--tango-colors-white);
  border-left: 1px solid var(--tango-colors-line-normal);
  position: relative;
  overflow-y: auto;

  &.isFloat {
    position: absolute;
    left: 0;
    top: 0;
    z-index: 1000;
    height: 100%;
    width: 100%;
    box-shadow: var(--tango-shadows-lowRight);
  }
`;
function SidebarPanelExpandedContent(_a) {
    var { closeable, onClose, isFloat, className, children } = _a, rest = tslib_1.__rest(_a, ["closeable", "onClose", "isFloat", "className", "children"]);
    const classNames = (0, classnames_1.default)('SidebarPanelExpanded', {
        isFloat,
    }, className);
    return (react_1.default.createElement(coral_system_1.Box, Object.assign({ css: expandPanelStyle, className: classNames }, rest), children));
}
