"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InsertionPrompt = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const coral_system_1 = require("coral-system");
const tango_core_1 = require("@music163/tango-core");
const tango_context_1 = require("@music163/tango-context");
const insertionStyle = (0, coral_system_1.css) `
  display: none;
  z-index: 1000;
  position: absolute;
  pointer-events: none;
  transition: left 0.2s ease-out, top 0.2s ease-out;
`;
/**
 * 插入提示符
 */
exports.InsertionPrompt = (0, tango_context_1.observer)(() => {
    const workspace = (0, tango_context_1.useWorkspace)();
    const dragSource = workspace.dragSource;
    const dropTarget = dragSource.dropTarget;
    if (!dragSource.isDragging || !dropTarget.bounding) {
        // 没有拖拽，或没有着陆点的轮廓信息，不渲染
        return null;
    }
    let style;
    const bounding = dropTarget.bounding;
    switch (dropTarget.display) {
        case 'inline':
        case 'inline-block':
        case 'inline-flex':
        case 'inline-grid': {
            if (dropTarget.method === tango_core_1.DropMethod.InsertBefore) {
                style = {
                    display: 'block',
                    left: bounding.left,
                    top: bounding.top,
                    width: 2,
                    height: bounding.height,
                };
            }
            else if (dropTarget.method === tango_core_1.DropMethod.InsertAfter) {
                style = {
                    display: 'block',
                    left: bounding.left + bounding.width,
                    top: bounding.top,
                    width: 2,
                    height: bounding.height,
                };
            }
            break;
        }
        default: {
            if (dropTarget.method === tango_core_1.DropMethod.InsertBefore) {
                style = {
                    display: 'block',
                    left: bounding.left,
                    top: bounding.top,
                    width: bounding.width,
                    height: 2,
                };
            }
            else if (dropTarget.method === tango_core_1.DropMethod.InsertAfter) {
                style = {
                    display: 'block',
                    left: bounding.left,
                    top: bounding.top + bounding.height,
                    width: bounding.width,
                    height: 2,
                };
            }
        }
    }
    return react_1.default.createElement(coral_system_1.Box, { className: "AuxInsertion", bg: "brand", style: style, css: insertionStyle });
});
