"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DraggingMask = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const styled_components_1 = tslib_1.__importDefault(require("styled-components"));
const tango_core_1 = require("@music163/tango-core");
const tango_context_1 = require("@music163/tango-context");
const MaskWrapper = styled_components_1.default.div `
  position: absolute;
  top: 0;
  left: 0;
  pointer-events: none;

  &.dragging {
    background-color: rgba(23, 97, 144, 0.26);
  }

  &.dropping {
    background-color: rgba(38, 169, 252, 0.34);
  }
`;
const Mask = (0, tango_context_1.observer)(({ dragging, dropping, bounding }) => {
    if (!bounding) {
        return null;
    }
    const clazz = (0, classnames_1.default)({
        dragging,
        dropping,
    });
    const style = {
        width: bounding.width,
        height: bounding.height,
        transform: `translate(${bounding.left}px, ${bounding.top}px)`,
    };
    return react_1.default.createElement(MaskWrapper, { className: clazz, style: style });
});
const showDroppingMaskMethods = [tango_core_1.DropMethod.InsertChild, tango_core_1.DropMethod.ReplaceNode];
/**
 * 拖拽遮罩提示
 */
exports.DraggingMask = (0, tango_context_1.observer)(() => {
    const workspace = (0, tango_context_1.useWorkspace)();
    const dragSource = workspace.dragSource;
    const dropTarget = dragSource.dropTarget;
    const dragging = !!dragSource.id;
    const dropping = !!(dragSource.isDragging &&
        showDroppingMaskMethods.includes(dropTarget.method) &&
        dropTarget.bounding);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(Mask, { dragging: dragging, bounding: dragSource.bounding }),
        react_1.default.createElement(Mask, { dropping: dropping, bounding: dropTarget.bounding })));
});
