"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SelectionTools = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const styled_components_1 = tslib_1.__importStar(require("styled-components"));
const coral_system_1 = require("coral-system");
const antd_1 = require("antd");
const icons_1 = require("@ant-design/icons");
const tango_helpers_1 = require("@music163/tango-helpers");
const tango_context_1 = require("@music163/tango-context");
const tango_ui_1 = require("@music163/tango-ui");
const helpers_1 = require("../helpers");
const widgets_1 = require("../widgets");
const boundingOffset = 100; // 检测选中模块的工具栏是否溢出的偏移值
exports.SelectionTools = (0, tango_context_1.observer)(({ actions: actionsProp = ['viewSource', 'copyNode', 'deleteNode'] }) => {
    const workspace = (0, tango_context_1.useWorkspace)();
    const selectSource = workspace.selectSource;
    const actions = actionsProp.map((item) => {
        if ((0, tango_helpers_1.isString)(item)) {
            const widget = (0, widgets_1.getWidget)(['selectionMenu', item].join('.'));
            return widget ? react_1.default.createElement(widget, { key: item }) : null;
        }
        return item;
    });
    return (react_1.default.createElement(react_1.default.Fragment, null, selectSource.selected.map((item) => (react_1.default.createElement(SelectionBox, { key: item.id, showActions: selectSource.selected.length === 1, actions: actions, data: item })))));
});
const selectionBoxStyle = (0, styled_components_1.css) `
  display: none;
  border: 2px solid var(--tango-colors-brand);
  pointer-events: none;
  position: absolute;
  z-index: 999; /* 需要比 antd modal(1000) 的层级小 */
  left: 0;
  top: 0;
`;
const topAddSiblingBtnStyle = (0, styled_components_1.css) `
  position: absolute;
  left: 50%;
  transform: translate(-50%, -50%);
  top: 0;
  border-radius: 32px;
  z-index: 999;
  pointer-events: auto;
`;
const bottomAddSiblingBtnStyle = (0, styled_components_1.css) `
  position: absolute;
  left: 50%;
  transform: translate(-50%, 50%);
  bottom: 0;
  border-radius: 32px;
  z-index: 999;
  pointer-events: auto;
`;
function SelectionBox({ showActions, actions, data }) {
    var _a;
    const workspace = (0, tango_context_1.useWorkspace)();
    const designer = (0, tango_context_1.useDesigner)();
    if (!data.bounding) {
        return null;
    }
    if (!data.bounding.height && !data.bounding.width) {
        // 没有尺寸的元素不显示选中框
        return null;
    }
    const prototype = workspace.componentPrototypes.get(data.name);
    const isPage = (prototype === null || prototype === void 0 ? void 0 : prototype.type) === 'page';
    // 如果声明了 childrenName，提供快捷子元素创建入口
    let insertedList = [];
    if (prototype === null || prototype === void 0 ? void 0 : prototype.childrenName) {
        const names = Array.isArray(prototype === null || prototype === void 0 ? void 0 : prototype.childrenName)
            ? prototype.childrenName
            : [prototype.childrenName];
        insertedList = names.map((child) => {
            const proto = workspace.componentPrototypes.get(child);
            return {
                name: child,
                label: (proto === null || proto === void 0 ? void 0 : proto.title) || child,
                icon: proto === null || proto === void 0 ? void 0 : proto.icon,
                description: proto === null || proto === void 0 ? void 0 : proto.help,
            };
        });
    }
    let siblingList = [];
    if (prototype === null || prototype === void 0 ? void 0 : prototype.siblingNames) {
        siblingList = (_a = prototype.siblingNames) === null || _a === void 0 ? void 0 : _a.map((item) => {
            const proto = workspace.componentPrototypes.get(item);
            return {
                name: item,
                label: (proto === null || proto === void 0 ? void 0 : proto.title) || item,
                icon: proto === null || proto === void 0 ? void 0 : proto.icon,
                description: proto === null || proto === void 0 ? void 0 : proto.help,
            };
        });
    }
    let selectionHelpersAlign = 'top-right';
    if (data.bounding) {
        if (data.bounding.left + data.bounding.width + boundingOffset < designer.viewport.width) {
            // 选中模块已靠近右侧边界位置
            selectionHelpersAlign = 'top-left';
        }
        else if (data.bounding.top < 20) {
            // 选中模块位于最顶部位置
            selectionHelpersAlign = 'inner-top-right';
        }
    }
    const isFromCurrentFile = data.filename === workspace.activeViewFile;
    let style;
    if (data.bounding) {
        style = {
            display: 'block',
            width: data.bounding.width,
            height: data.bounding.height,
            transform: `translate(${data.bounding.left}px, ${data.bounding.top}px)`,
            ['--color-active']: isFromCurrentFile
                ? 'var(--tango-colors-brand)'
                : 'var(--tango-colors-gray-60)',
            ['--color-active-hover']: isFromCurrentFile
                ? 'var(--tango-colors-primary-50)'
                : 'var(--tango-colors-gray-50)',
        };
    }
    return (react_1.default.createElement(coral_system_1.Box, { className: "AuxSelectionBox", "data-selection-id": data.id, css: selectionBoxStyle, style: style },
        siblingList.length > 0 ? (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(InsertedDropdown, { title: "\u5728\u5F53\u524D\u8282\u70B9\u7684\u524D\u65B9\u6DFB\u52A0\u5144\u5F1F\u8282\u70B9", options: siblingList, onSelect: (name) => {
                    workspace.insertBeforeSelectedNode(name);
                } },
                react_1.default.createElement(antd_1.Tooltip, { title: "\u5728\u5F53\u524D\u8282\u70B9\u7684\u524D\u65B9\u6DFB\u52A0\u5144\u5F1F\u8282\u70B9" },
                    react_1.default.createElement(SelectionHelper, { icon: react_1.default.createElement(icons_1.PlusOutlined, null), css: topAddSiblingBtnStyle }))),
            react_1.default.createElement(InsertedDropdown, { title: "\u5728\u5F53\u524D\u8282\u70B9\u7684\u540E\u65B9\u6DFB\u52A0\u5144\u5F1F\u8282\u70B9", options: siblingList, onSelect: (name) => {
                    workspace.insertAfterSelectedNode(name);
                } },
                react_1.default.createElement(antd_1.Tooltip, { title: "\u5728\u5F53\u524D\u8282\u70B9\u7684\u540E\u65B9\u6DFB\u52A0\u5144\u5F1F\u8282\u70B9" },
                    react_1.default.createElement(SelectionHelper, { icon: react_1.default.createElement(icons_1.PlusOutlined, null), css: bottomAddSiblingBtnStyle }))))) : null,
        showActions && (react_1.default.createElement(SelectionHelpers, { align: selectionHelpersAlign },
            react_1.default.createElement(SelectionHelper, { icon: !isPage && (react_1.default.createElement(icons_1.HolderOutlined, { draggable: true, onDragStart: (e) => {
                        e.dataTransfer.effectAllowed = 'move';
                        workspace.dragSource.set(data);
                        const ghost = (0, helpers_1.getDragGhostElement)();
                        e.dataTransfer.setDragImage(ghost, 0, 0);
                    }, onDragEnd: () => {
                        workspace.dragSource.clear();
                    } })), label: react_1.default.createElement(react_1.default.Fragment, null,
                    react_1.default.createElement(NameSelector, { label: data.codeId || data.name, parents: data.parents, onSelect: (item) => {
                            workspace.selectSource.select(item);
                        } }),
                    !isFromCurrentFile && (react_1.default.createElement(antd_1.Tooltip, { title: `该节点来自其他文件（${data.filename}）` },
                        react_1.default.createElement(icons_1.InfoCircleOutlined, null)))) }),
            react_1.default.createElement(SelectionToolSet, null, !isPage && actions),
            insertedList.length > 0 && (react_1.default.createElement(InsertedDropdown, { options: insertedList, onSelect: (name) => {
                    workspace.insertToSelectedNode(name);
                } },
                react_1.default.createElement(antd_1.Tooltip, { title: "\u5FEB\u6377\u6DFB\u52A0\u5B50\u5143\u7D20" },
                    react_1.default.createElement(SelectionHelper, { icon: react_1.default.createElement(icons_1.PlusOutlined, null) }))))))));
}
const selectionHelpersStyle = (0, styled_components_1.css) `
  pointer-events: auto;
  white-space: nowrap;
  position: absolute;
  user-select: none;
  column-gap: 2px;

  &[data-align='top-right'] {
    bottom: 100%;
    right: -2px;
  }

  &[data-align='inner-top-right'] {
    top: 0;
    right: 4px;
  }

  &[data-align='top-left'] {
    bottom: 100%;
    left: -2px;
  }
`;
const SelectionHelpers = ({ align = 'top-right', children }) => {
    return (react_1.default.createElement(coral_system_1.Group, { display: "inline-flex", my: "s", css: selectionHelpersStyle, "data-align": align }, children));
};
const selectionHelperStyle = (0, styled_components_1.css) `
  outline: none;
  cursor: pointer;
  background: var(--color-active, var(--tango-colors-brand));

  &:hover {
    background: var(--color-active-hover, var(--tango-colors-primary-50));
  }
`;
const SelectionHelper = (_a) => {
    var { icon, label, children, css: customCss } = _a, rest = tslib_1.__rest(_a, ["icon", "label", "children", "css"]);
    return (react_1.default.createElement(coral_system_1.Button, Object.assign({ position: "relative", color: "white", px: "s", py: "0", border: "0", borderRadius: "s", fontSize: "12px", css: [selectionHelperStyle, customCss] }, rest),
        icon,
        label,
        children));
};
const SelectionToolSetStyle = (0, styled_components_1.css) `
  display: flex;
  align-items: center;
  position: relative;

  > :first-child {
    border-top-left-radius: var(--tango-radii-s);
    border-bottom-left-radius: var(--tango-radii-s);
  }

  > :last-child {
    border-top-right-radius: var(--tango-radii-s);
    border-bottom-right-radius: var(--tango-radii-s);
  }
`;
const SelectionToolSet = ({ children }) => {
    return (react_1.default.createElement(coral_system_1.Box, { className: "SelectionToolSet", borderRadius: "s", css: SelectionToolSetStyle }, children));
};
const slideDown = (0, styled_components_1.keyframes) `
  from {
    transform: translateY(-10%);
    opacity: 0;
  }

  to {
    transform: translateY(0);
    opacity: 0.8;
  }
`;
const NameSelectorWrapper = styled_components_1.default.div `
  display: inline-block;
  &:hover > ul {
    display: block;
  }

  ul {
    display: none;
    animation: ${slideDown} 0.2s;
    position: absolute;
    left: 0;
    list-style: none;
    padding: 0;
    text-align: left;
  }

  li {
    margin-top: 4px;
  }
`;
const NameSelector = ({ label, parents = [], onSelect = tango_helpers_1.noop }) => {
    return (react_1.default.createElement(NameSelectorWrapper, null,
        react_1.default.createElement(coral_system_1.Box, { as: "span", className: "SelectionName" }, label),
        react_1.default.createElement("ul", null, parents.map((parent, index) => (react_1.default.createElement(SelectionHelper, { as: "li", key: parent.id, onClick: () => onSelect(Object.assign(Object.assign({}, parent), { parents: parents.slice(index + 1) })) }, parent.name))))));
};
function InsertedDropdown(_a) {
    var { title = '为当前节点添加子元素', options = [], onSelect } = _a, props = tslib_1.__rest(_a, ["title", "options", "onSelect"]);
    return (react_1.default.createElement(antd_1.Dropdown, Object.assign({ trigger: ['click'], dropdownRender: () => {
            return (react_1.default.createElement(coral_system_1.Box, { bg: "#FFF", borderRadius: "m", boxShadow: "lowDown", border: "solid", borderColor: "line2", overflow: "hidden", width: "320px" },
                react_1.default.createElement(coral_system_1.Box, { px: "l", py: "m", color: "text2" }, title),
                react_1.default.createElement(coral_system_1.Box, { maxHeight: 360, overflowY: "auto" }, options.map((item) => (react_1.default.createElement(InsertedItem, { key: item.name, label: item.label, icon: item.icon, description: item.description || '暂无组件描述', onClick: () => onSelect === null || onSelect === void 0 ? void 0 : onSelect(item.name) }))))));
        } }, props)));
}
const insertedItemStyle = (0, styled_components_1.css) `
  cursor: pointer;
  user-select: none;

  &:hover {
    background-color: var(--tango-colors-fill2);
  }
`;
function InsertedItem(_a) {
    var { label, icon, description } = _a, rest = tslib_1.__rest(_a, ["label", "icon", "description"]);
    let iconNode;
    if (!icon) {
        iconNode = react_1.default.createElement(tango_ui_1.IconFont, { className: "material-icon", type: "icon-placeholder" });
    }
    else if (icon.startsWith('icon-')) {
        iconNode = react_1.default.createElement(tango_ui_1.IconFont, { className: "material-icon", type: icon });
    }
    else {
        iconNode = react_1.default.createElement("img", { src: icon, alt: label });
    }
    return (react_1.default.createElement(coral_system_1.Box, Object.assign({ display: "flex", columnGap: "m", px: "l", py: "m", fontSize: "12px", css: insertedItemStyle }, rest),
        react_1.default.createElement(coral_system_1.Box, { size: "35px", fontSize: "32px", background: "fill1", border: "1px solid", borderColor: "line2", display: "flex", alignItems: "center", justifyContent: "center" }, iconNode),
        react_1.default.createElement(coral_system_1.Box, null,
            react_1.default.createElement(coral_system_1.Box, { fontWeight: "500" }, label),
            react_1.default.createElement(coral_system_1.Box, { color: "text2" }, description))));
}
