"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListStore = void 0;
const helpers_1 = require("../helpers");
class ListStore {
    get nodes() {
        return this.nodeMap.values();
    }
    constructor(options) {
        this.keyProp = options.keyProp || 'value';
        this.childrenProp = options.childrenProp || 'children';
        this.nodeMap = new Map();
        this.visitNodes(options.data);
    }
    getNode(key) {
        if (!key) {
            return;
        }
        return this.nodeMap.get(key);
    }
    visitNodes(nodes) {
        if (!Array.isArray(nodes) || nodes.length === 0) {
            return;
        }
        nodes.forEach((node) => {
            const key = node[this.keyProp];
            this.nodeMap.set(key, node);
            (0, helpers_1.warning)(this.nodeMap.has(key), `duplicate value '${key}' detected! All node values should be unique!`);
            this.nodeMap.set(key, node);
            if (node[this.childrenProp]) {
                this.visitNodes(node[this.childrenProp]);
            }
        });
    }
}
exports.ListStore = ListStore;
