"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ToggleButton = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const coral_system_1 = require("coral-system");
const antd_1 = require("antd");
const icons_1 = require("@ant-design/icons");
const buttonStyle = (0, coral_system_1.css) `
  display: inline-flex;
  align-items: center;
  justify-content: space-between;
  user-select: none;
  cursor: pointer;
  color: var(--tango-colors-gray-50);
  padding: 0 8px;

  .anticon-down {
    font-size: 12px !important;
    margin-left: 4px;
  }

  &.size-s {
    height: 24px;
    font-size: 14px;
  }

  &.size-m {
    height: 32px;
    font-size: 14px;
  }

  &.size-l {
    height: 40px;
    font-size: 16px;
  }

  &.size-xl {
    height: 40px;
    font-size: 20px;
  }

  &.shape-text {
    padding: 0px 4px;
    height: auto;
    border: 0;
    border-radius: var(--tango-radii-s);
    color: var(--tango-colors-text2);
    background: transparent;

    &.selected.type-normal {
      color: var(--tango-colors-white);
    }

    &.selected.type-primary {
      color: var(--tango-colors-white);
      background-color: var(--tango-colors-brand);
    }

    &:hover {
      background-color: rgba(197, 197, 197, 0.16);
    }

    &.disabled {
      color: var(--tango-colors-gray-60);
    }
  }

  &.shape-outline {
    background-color: #fff;
    border: 1px solid var(--tango-colors-line2);
    border-radius: var(--tango-radii-s);

    &.disabled {
      color: var(--tango-colors-text4);
      background-color: var(--tango-colors-fill1);
    }

    &.selected.type-normal {
      color: var(--tango-colors-gray-100);
      background-color: var(--tango-colors-fill3);
      border-color: var(--tango-colors-line2);
    }

    &.selected.type-primary {
      color: #fff;
      background-color: var(--tango-colors-brand);
      border-color: var(--tango-colors-brand);
    }
  }

  &.shape-ghost {
    background-color: var(--tango-colors-custom-toolbarButtonBg, rgba(223, 223, 223, 0.08));
    color: var(--tango-colors-custom-toolbarButtonTextColor, #999);
    border: 0;
    border-radius: var(--tango-radii-m);

    &:hover {
      color: var(--tango-colors-custom-toolbarButtonTextColorHover, #fff);
      background-color: var(--tango-colors-custom-toolbarButtonBgHover, rgba(197, 197, 197, 0.16));
    }

    &.disabled {
      color: var(--tango-colors-custom-toolbarButtonTextColorDisabled, rgb(201, 205, 212));
      background-color: var(--tango-colors-custom-toolbarButtonBgDisabled, rgb(247, 248, 250));
    }

    &.selected {
      color: var(--tango-colors-custom-toolbarButtonTextColorActive);
      background-color: var(--tango-colors-custom-toolbarButtonBgActive);
    }
  }

  &.disabled {
    pointer-events: none;
  }
`;
/**
 * TODO: 使用 Action 重构
 */
function ToggleButton(props) {
    const { size = 'm', shape = 'outline', type = 'normal', disabled, selected, children, icon, tip, tooltip = tip, tooltipPlacement = 'bottom', items, overlay, overlayStyle, onItemClick, dropdownProps = {} } = props, rest = tslib_1.__rest(props, ["size", "shape", "type", "disabled", "selected", "children", "icon", "tip", "tooltip", "tooltipPlacement", "items", "overlay", "overlayStyle", "onItemClick", "dropdownProps"]);
    const clazz = (0, classnames_1.default)({
        [`size-${size}`]: size,
        [`shape-${shape}`]: shape,
        [`type-${type}`]: type,
        disabled,
        selected,
    });
    const showDropdown = !!items || !!overlay;
    let btn = (react_1.default.createElement(coral_system_1.Button, Object.assign({ css: buttonStyle, className: clazz, type: "button" }, rest),
        react_1.default.createElement(coral_system_1.Box, { as: "span" },
            icon && (react_1.default.createElement(coral_system_1.Box, { as: "span", className: "buttonIcon", mr: "s" }, icon)),
            children),
        showDropdown ? react_1.default.createElement(icons_1.DownOutlined, null) : null));
    if (tooltip) {
        btn = (react_1.default.createElement(antd_1.Tooltip, { title: tooltip, placement: tooltipPlacement }, btn));
    }
    if (showDropdown) {
        btn = (react_1.default.createElement(antd_1.Dropdown, Object.assign({}, dropdownProps, { overlay: overlay, overlayStyle: overlayStyle, menu: { items, onClick: onItemClick } }), btn));
    }
    return btn;
}
exports.ToggleButton = ToggleButton;
