"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RouteSwitchTool = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const antd_1 = require("antd");
const coral_system_1 = require("coral-system");
const icons_1 = require("@ant-design/icons");
const tango_helpers_1 = require("@music163/tango-helpers");
const tango_ui_1 = require("@music163/tango-ui");
const tango_context_1 = require("@music163/tango-context");
exports.RouteSwitchTool = (0, tango_context_1.observer)(() => {
    const workspace = (0, tango_context_1.useWorkspace)();
    return (react_1.default.createElement(PageSelect, { options: workspace.pages, value: workspace.activeRoute, onSelect: (path) => {
            workspace.setActiveRoute(path);
        }, onRemove: (path) => {
            workspace.removeViewModule(path);
        }, onCopy: (...payloads) => workspace.copyViewPage(...payloads), onUpdate: (...payloads) => workspace.updateRoute(...payloads) }));
});
function PageSelect({ value, onSelect, onRemove, onUpdate, onCopy, options }) {
    const [visible, setVisible] = (0, react_1.useState)(false);
    const optionMap = (0, react_1.useMemo)(() => {
        return (0, tango_helpers_1.array2object)(options, (item) => item.path);
    }, [options]);
    const handleChange = (next) => {
        if (typeof onSelect === 'function') {
            onSelect(next);
        }
        setVisible(false);
    };
    const current = optionMap[value];
    return (react_1.default.createElement(tango_ui_1.ToggleButton, { shape: "ghost", minWidth: 100, dropdownProps: {
            open: visible,
            onOpenChange: setVisible,
        }, overlay: react_1.default.createElement(PageList, { items: options, activeKey: value, onSelect: handleChange, onRemove: onRemove, onUpdate: onUpdate, onCopy: onCopy, onAction: (key) => {
                if (key !== 'delete') {
                    setVisible(false);
                }
            } }), icon: react_1.default.createElement(icons_1.FileOutlined, null) }, (current === null || current === void 0 ? void 0 : current.name) || (current === null || current === void 0 ? void 0 : current.label) || value));
}
const pageListStyle = (0, coral_system_1.css) `
  min-width: 200px;
  max-height: 400px;
  overflow-y: auto;
  padding: 8px 0;
  user-select: none;

  li {
    display: flex;
    align-items: center;
    justify-content: space-between;
    padding: 4px 12px;

    button {
      background: transparent;
      outline: none;
      border: 0;
    }

    &:hover {
      background-color: var(--tango-colors-gray-20);
    }

    &.active {
      background-color: var(--tango-colors-primary-20);
    }
  }
`;
function PageList({ activeKey, items = [], onSelect, onRemove, onUpdate, onCopy, onAction, }) {
    const [current, setCurrent] = (0, react_1.useState)(null);
    const [type, setType] = (0, react_1.useState)();
    return (react_1.default.createElement(coral_system_1.Box, { as: "ul", bg: "white", boxShadow: "lowDown", borderRadius: "m", css: pageListStyle },
        items.map((item) => (react_1.default.createElement(coral_system_1.Box, { as: "li", key: item.path, gap: "12px", onClick: () => {
                onSelect && onSelect(item.path);
            }, className: activeKey === item.path ? 'active' : undefined },
            react_1.default.createElement(icons_1.FileOutlined, null),
            react_1.default.createElement(coral_system_1.Box, { flex: "1" },
                item.name || '未命名',
                react_1.default.createElement(coral_system_1.Text, { as: "code", fontSize: "12px", ml: "m", color: "gray.50" }, item.path)),
            react_1.default.createElement(coral_system_1.Group, null,
                item.path !== '/' ? (react_1.default.createElement(antd_1.Popconfirm, { title: "\u786E\u8BA4\u5220\u9664\u6B64\u9875\u9762\u5417\uFF1F", onConfirm: (e) => {
                        e.stopPropagation();
                        onRemove && onRemove(item.path);
                    }, zIndex: 1060 },
                    react_1.default.createElement(antd_1.Button, { size: "small", type: "link", title: "\u5220\u9664\u9875\u9762", icon: react_1.default.createElement(icons_1.DeleteOutlined, null), onClick: (e) => {
                            e.stopPropagation();
                            onAction === null || onAction === void 0 ? void 0 : onAction('delete', e);
                        } }))) : null,
                react_1.default.createElement(antd_1.Button, { size: "small", type: "link", title: "\u590D\u5236\u9875\u9762", icon: react_1.default.createElement(icons_1.CopyOutlined, null), onClick: (e) => {
                        e.stopPropagation();
                        setCurrent(item);
                        setType('copy');
                        onAction === null || onAction === void 0 ? void 0 : onAction('copy', e);
                    } }),
                react_1.default.createElement(antd_1.Button, { size: "small", type: "link", title: "\u8BBE\u7F6E\u9875\u9762", icon: react_1.default.createElement(icons_1.SettingOutlined, null), onClick: (e) => {
                        e.stopPropagation();
                        setCurrent(item);
                        setType('update');
                        onAction === null || onAction === void 0 ? void 0 : onAction('update', e);
                    } }))))),
        react_1.default.createElement(CopyViewModalForm, { key: current === null || current === void 0 ? void 0 : current.path, title: type === 'update' ? '更新当前页面' : '复制当前页面', editType: type, items: items, visible: !!current, current: current, onOk: (0, tango_helpers_1.callAll)(type === 'update' ? onUpdate : onCopy, () => {
                setCurrent(null);
            }), onCancel: () => {
                setCurrent(null);
            } })));
}
function CopyViewModalForm({ title, current, editType, items = [], visible, onOk, onCancel, }) {
    const [form] = antd_1.Form.useForm();
    return (react_1.default.createElement(antd_1.Modal, { title: title, visible: visible, destroyOnClose: true, onCancel: onCancel, onOk: () => {
            form.validateFields().then((values) => {
                onOk === null || onOk === void 0 ? void 0 : onOk(current.path, values);
            });
        } },
        react_1.default.createElement(antd_1.Form, { wrapperCol: { span: 20 }, form: form, initialValues: { name: current === null || current === void 0 ? void 0 : current.name, path: current === null || current === void 0 ? void 0 : current.path } },
            react_1.default.createElement(antd_1.Form.Item, { label: "\u9875\u9762\u6807\u9898", name: "name", required: true },
                react_1.default.createElement(antd_1.Input, { placeholder: "\u8BF7\u8F93\u5165\u6807\u9898" })),
            react_1.default.createElement(antd_1.Form.Item, { label: "\u9875\u9762\u8DEF\u7531", name: "path", rules: [
                    { required: true },
                    editType !== 'update'
                        ? {
                            validator: (_, v) => {
                                return items.map((item) => item.path).includes(v)
                                    ? Promise.reject()
                                    : Promise.resolve();
                            },
                            message: '重复的页面路由',
                        }
                        : undefined,
                    {
                        pattern: /^\/([A-Za-z0-9-:]*\/?)*$/,
                        message: '非法的路由地址',
                    },
                ] },
                react_1.default.createElement(antd_1.Input, { placeholder: "\u8BF7\u8F93\u5165\u9875\u9762\u8DEF\u7531" })))));
}
