"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Toolbar = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const coral_system_1 = require("coral-system");
const widgets_1 = require("../widgets");
function Toolbar({ children }) {
    const [leftTools, centerTools, rightTools] = (0, react_1.useMemo)(() => {
        const left = [];
        const center = [];
        const right = [];
        let prevPlacement;
        react_1.default.Children.forEach(children, (child, index) => {
            var _a, _b;
            let fallbackNode;
            if (child.key) {
                const Widget = (0, widgets_1.getWidget)(['toolbar', child.key].join('.'));
                if (Widget) {
                    fallbackNode = react_1.default.createElement(Widget, child.props.widgetProps);
                }
            }
            let node = (_b = (_a = child.props.children) !== null && _a !== void 0 ? _a : fallbackNode) !== null && _b !== void 0 ? _b : null;
            if (!node) {
                node = child; // separator
            }
            const key = child.key || index;
            node = (react_1.default.createElement("div", { key: key, className: "ToolbarPanelItem", "data-key": key }, node));
            const placement = child.props.placement || prevPlacement || 'center';
            switch (placement) {
                case 'left':
                    left.push(node);
                    break;
                case 'right':
                    right.push(node);
                    break;
                case 'center':
                default:
                    center.push(node);
                    break;
            }
            prevPlacement = placement;
        });
        return [left, center, right];
    }, [children]);
    return (react_1.default.createElement(coral_system_1.Box, { className: "ToolbarPanel", display: "flex", justifyContent: "space-between", alignItems: "center" },
        react_1.default.createElement(coral_system_1.Group, { display: "flex", alignItems: "center", gap: "m" }, leftTools),
        react_1.default.createElement(coral_system_1.Group, { display: "flex", alignItems: "center", gap: "m" }, centerTools),
        react_1.default.createElement(coral_system_1.Group, { display: "flex", alignItems: "center", gap: "m" }, rightTools)));
}
exports.Toolbar = Toolbar;
function ToolbarItem({ placement, widgetProps, children }) {
    return react_1.default.createElement("div", null, children);
}
function Separator() {
    return react_1.default.createElement(coral_system_1.Box, { className: "Separator", width: 1, height: 16, bg: "colors.custom.toolbarDividerColor" });
}
Toolbar.Item = ToolbarItem;
Toolbar.Separator = Separator;
