import { __extends } from "tslib";
import React from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import { Cascader } from 'udesk-ui';
/**
 * antd风格级联组件
 */
var AntdCascaderComponent = /** @class */ (function (_super) {
    __extends(AntdCascaderComponent, _super);
    function AntdCascaderComponent() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        /**
         * @param {string[]} value
         * @param {object[]} selectedOptions
         */
        _this.handleCascadeChange = function (value, selectedOptions) {
            var onChanged = _this.props.onChanged;
            var internalValue = value;
            if (Array.isArray(internalValue)) {
                internalValue = internalValue.join(',');
            }
            onChanged(internalValue);
        };
        _this.filter = function (inputValue, path) {
            var attributes = _this.props.attributes;
            var _a = ((attributes === null || attributes === void 0 ? void 0 : attributes.fieldNames) || {}).label, label = _a === void 0 ? 'label' : _a;
            return path.some(function (option) { var _a; return ((_a = option === null || option === void 0 ? void 0 : option[label]) === null || _a === void 0 ? void 0 : _a.toLowerCase().indexOf(inputValue.toLowerCase())) > -1; });
        };
        return _this;
    }
    AntdCascaderComponent.prototype.render = function () {
        var _a = this.props, disabled = _a.disabled, attributes = _a.attributes, dataSource = _a.dataSource, value = _a.value;
        var classNames = attributes.classNames, popupClassName = attributes.popupClassName, size = attributes.size, placeholder = attributes.placeholder, allowClear = attributes.allowClear, _b = attributes.changeOnSelect, changeOnSelect = _b === void 0 ? true : _b, _c = attributes.fieldNames, fieldNames = _c === void 0 ? {
            label: 'label',
            value: 'value',
            children: 'children',
        } : _c, _d = attributes.width, width = _d === void 0 ? '300px' : _d;
        if (!(Array.isArray(dataSource))) {
            dataSource = [];
        }
        if (value === null || value === undefined) {
            value = [];
        }
        if (typeof value === 'string') {
            value = value.split(',');
        }
        return (React.createElement(Cascader, { showSearch: { filter: this.filter }, style: { width: width }, size: size, allowClear: allowClear, className: ClassNames(classNames, 'udesk-custom-filter-antd-cascade'), popupClassName: ClassNames(popupClassName, 'udesk-custom-filter-antd-cascade-dropdown'), getPopupContainer: function (triggerNode) { return triggerNode.parentElement; }, placeholder: placeholder, disabled: disabled, changeOnSelect: changeOnSelect, fieldNames: fieldNames, options: dataSource, value: value, onChange: this.handleCascadeChange }));
    };
    return AntdCascaderComponent;
}(React.Component));
export default AntdCascaderComponent;
/**
 * @description 组件可能接收的prop
 * @namespace
 * @property {object} attributes - 配置fields(优先级更高)和valueSetting时传入，常用
 * @property {string} attributes.classNames - 自定义组件css类名
 * @property {string} attributes.placeholder - 自定义组件placeholder
 * @property {string} attributes.dropdownClassName - 自定义下拉菜单的 className 属性
 * @property {boolean} attributes.allowClear - 单选支持清除
 * @property {function} onChanged - value改变唯一触发事件，其他入onFocus之类可能需要在valueSetting时配置，而且是跟自定义过滤器同一上下文
 * @property {string} value - 字段value，该组件位text input,value只需考虑string,复杂选择组件需object
 *
 * @property {object} operator - condition上下文相关
 * @property {object} field - condition上下文相关
 * @property {string} name - 校验相关，错误信息提示字段名
 * @property {boolean} isValid - 校验相关 具体待补充 todo
 * @property {array} errors - 校验相关，错误信息
 * @property {object} runtimeAttributes - 运行时传入valueInput组件的属性,从custom-filter组件传入conditionValueComponentAttrs
 *
 */
AntdCascaderComponent.defaultProps = {
    value: [],
    onChanged: null,
    attributes: {},
    dataSource: [],
    disabled: false,
    operator: {},
    field: {},
    name: null,
    isValid: true,
    errors: [],
    runtimeAttributes: {},
};
AntdCascaderComponent.propTypes = {
    value: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.arrayOf(PropTypes.string),
        PropTypes.arrayOf(PropTypes.number),
    ]),
    onChanged: PropTypes.func,
    dataSource: PropTypes.arrayOf(PropTypes.object),
    disabled: PropTypes.bool,
    attributes: PropTypes.shape({
        placeholder: PropTypes.string,
        classNames: PropTypes.string,
        popupClassName: PropTypes.string,
        size: PropTypes.oneOf(['small', 'middle', 'large']),
        allowClear: PropTypes.bool,
        changeOnSelect: PropTypes.bool,
        fieldNames: PropTypes.exact({
            label: PropTypes.string,
            value: PropTypes.string,
            children: PropTypes.string,
        }),
    }),
    operator: PropTypes.object,
    field: PropTypes.object,
    name: PropTypes.string,
    isValid: PropTypes.bool,
    errors: PropTypes.array,
    runtimeAttributes: PropTypes.object,
};
