import { __assign, __extends } from "tslib";
import React from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import DynamicComponent from 'udesk-react/src/components/dynamic-component';
/**
 * 过滤器暴露的字段不满足组件时，可以使用该组件，传入过滤器配置的数据
 * eg：受理人组件数据源字段为groupAgentsWithStatus  不支持过滤器数据源字段dataSource  需要动态传入
 * 目前支持数据源字段扩展 組件内部触发的事件扩展 有需求可自行扩展
 * 另外组件需要的属性字段需要自行扩展
 */
var AntdConfigureComponent = /** @class */ (function (_super) {
    __extends(AntdConfigureComponent, _super);
    function AntdConfigureComponent() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.handleSelectChange = function (value) {
            var onChanged = _this.props.onChanged;
            onChanged(value);
        };
        return _this;
    }
    AntdConfigureComponent.prototype.render = function () {
        var _a = this.props, value = _a.value, dataSource = _a.dataSource, attributes = _a.attributes;
        var classNames = attributes.classNames, targetComponent = attributes.targetComponent, _b = attributes.dataSourceKey, dataSourceKey = _b === void 0 ? 'dataSource' : _b, _c = attributes.eventTriggerName, eventTriggerName = _c === void 0 ? 'onChange' : _c;
        if (!Array.isArray(dataSource)) {
            dataSource = [];
        }
        if (value === null) {
            value = [];
        }
        var configure = {}; // 在这里加入自定义的配置字段
        configure[dataSourceKey] = dataSource;
        configure[eventTriggerName] = this.handleSelectChange;
        return targetComponent ? (React.createElement(DynamicComponent, __assign({ component: targetComponent, value: value, 
            // onChange={this.handleSelectChange}
            className: ClassNames(classNames, '') }, configure))) : null;
    };
    return AntdConfigureComponent;
}(React.Component));
export default AntdConfigureComponent;
/**
 * @description 组件可能接收的prop
 * @namespace
 * @property {object} attributes - 配置fields(优先级更高)和valueSetting时传入，常用
 * @property {string} attributes.classNames - 自定义组件css类名
 * @property {string} attributes.placeholder - 自定义组件placeholder
 * @property {Component} attributes.component组件
 * @property {function} onChanged - value改变唯一触发事件，其他入onFocus之类可能需要在valueSetting时配置，而且是跟自定义过滤器同一上下文
 * @property {string} value - 字段value，该组件位text input,value只需考虑string,复杂选择组件需object
 * @property {object} dataSource数据源
 */
AntdConfigureComponent.defaultProps = Object.assign({
    value: [],
    onChanged: null,
    attributes: {},
    dataSource: [],
});
AntdConfigureComponent.propTypes = {
    value: PropTypes.array,
    onChanged: PropTypes.func,
    attributes: PropTypes.object,
    dataSource: PropTypes.array,
};
