import { __extends } from "tslib";
import React from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import { Input, Space, Cascader } from 'udesk-ui';
/**
 * antd风格input+级联组件
 */
var AntdInputCascaderComponent = /** @class */ (function (_super) {
    __extends(AntdInputCascaderComponent, _super);
    function AntdInputCascaderComponent(props) {
        var _this = _super.call(this) || this;
        /**
         * @param {string[]} value
         * @param {object[]} selectedOptions
         */
        _this.handleCascadeChange = function (value, selectedOptions) {
            var onChanged = _this.props.onChanged;
            _this.dayVal = value;
            onChanged("".concat(_this.numVal, ",").concat(_this.dayVal.join(',')));
        };
        _this.handleTextInputChanged = function (e) {
            var onChanged = _this.props.onChanged;
            var internalValue = e.target.value;
            internalValue = internalValue.replace(/[^\d.]/g, ''); //清除“数字”和“.”以外的字符
            internalValue = internalValue.replace(/\.{2,}/g, '.'); //只保留第一个. 清除多余的
            internalValue = internalValue.replace('.', '$#$').replace(/\./g, '').replace('$#$', '.');
            if (internalValue.indexOf('.') < 0 && internalValue != '') {
                //以上已经过滤，此处控制的是如果没有小数点，首位不能为类似于 01、02的数
                internalValue = parseFloat(internalValue);
            }
            _this.numVal = internalValue;
            onChanged("".concat(_this.numVal, ",").concat(_this.dayVal));
        };
        _this.filter = function (inputValue, path) {
            var attributes = _this.props.attributes;
            var _a = ((attributes === null || attributes === void 0 ? void 0 : attributes.fieldNames) || {}).label, label = _a === void 0 ? 'label' : _a;
            return path.some(function (option) { var _a; return ((_a = option === null || option === void 0 ? void 0 : option[label]) === null || _a === void 0 ? void 0 : _a.toLowerCase().indexOf(inputValue.toLowerCase())) > -1; });
        };
        _this.numVal = '';
        _this.dayVal = [];
        return _this;
    }
    AntdInputCascaderComponent.prototype.render = function () {
        var _a = this.props, disabled = _a.disabled, attributes = _a.attributes, dataSource = _a.dataSource, value = _a.value;
        var classNames = attributes.classNames, popupClassName = attributes.popupClassName, size = attributes.size, placeholder = attributes.placeholder, allowClear = attributes.allowClear, _b = attributes.changeOnSelect, changeOnSelect = _b === void 0 ? true : _b, _c = attributes.fieldNames, fieldNames = _c === void 0 ? {
            label: 'label',
            value: 'value',
            children: 'children',
        } : _c;
        if (!(Array.isArray(dataSource))) {
            dataSource = [];
        }
        this.numVal = '';
        this.dayVal = [];
        if (value === null || value === undefined) {
            value = [];
        }
        if (typeof value === 'string') {
            value = value.split(',');
            this.numVal = value[0];
            this.dayVal = value.slice(1);
        }
        return (React.createElement(Space, null,
            React.createElement(Input, { style: { width: '120px' }, onChange: this.handleTextInputChanged, value: this.numVal }),
            React.createElement(Cascader, { showSearch: { filter: this.filter }, style: { width: '120px' }, size: size, allowClear: allowClear, className: ClassNames(classNames, 'udesk-custom-filter-antd-cascade'), popupClassName: ClassNames(popupClassName, 'udesk-custom-filter-antd-cascade-dropdown'), getPopupContainer: function (triggerNode) { return triggerNode.parentElement; }, placeholder: placeholder, disabled: disabled, changeOnSelect: changeOnSelect, fieldNames: fieldNames, options: dataSource, defaultValue: this.dayVal, onChange: this.handleCascadeChange })));
    };
    return AntdInputCascaderComponent;
}(React.Component));
export default AntdInputCascaderComponent;
/**
 * @description 组件可能接收的prop
 * @namespace
 * @property {object} attributes - 配置fields(优先级更高)和valueSetting时传入，常用
 * @property {string} attributes.classNames - 自定义组件css类名
 * @property {string} attributes.placeholder - 自定义组件placeholder
 * @property {string} attributes.dropdownClassName - 自定义下拉菜单的 className 属性
 * @property {boolean} attributes.allowClear - 单选支持清除
 * @property {function} onChanged - value改变唯一触发事件，其他入onFocus之类可能需要在valueSetting时配置，而且是跟自定义过滤器同一上下文
 * @property {string} value - 字段value，该组件位text input,value只需考虑string,复杂选择组件需object
 *
 * @property {object} operator - condition上下文相关
 * @property {object} field - condition上下文相关
 * @property {string} name - 校验相关，错误信息提示字段名
 * @property {boolean} isValid - 校验相关 具体待补充 todo
 * @property {array} errors - 校验相关，错误信息
 * @property {object} runtimeAttributes - 运行时传入valueInput组件的属性,从custom-filter组件传入conditionValueComponentAttrs
 *
 */
AntdInputCascaderComponent.defaultProps = {
    value: [],
    onChanged: null,
    attributes: {},
    dataSource: [],
    disabled: false,
    operator: {},
    field: {},
    name: null,
    isValid: true,
    errors: [],
    runtimeAttributes: {},
};
AntdInputCascaderComponent.propTypes = {
    value: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.arrayOf(PropTypes.string),
        PropTypes.arrayOf(PropTypes.number),
    ]),
    onChanged: PropTypes.func,
    dataSource: PropTypes.arrayOf(PropTypes.object),
    disabled: PropTypes.bool,
    attributes: PropTypes.shape({
        placeholder: PropTypes.string,
        classNames: PropTypes.string,
        popupClassName: PropTypes.string,
        size: PropTypes.oneOf(['small', 'middle', 'large']),
        allowClear: PropTypes.bool,
        changeOnSelect: PropTypes.bool,
        fieldNames: PropTypes.exact({
            label: PropTypes.string,
            value: PropTypes.string,
            children: PropTypes.string,
        }),
    }),
    operator: PropTypes.object,
    field: PropTypes.object,
    name: PropTypes.string,
    isValid: PropTypes.bool,
    errors: PropTypes.array,
    runtimeAttributes: PropTypes.object,
};
