import { __extends } from "tslib";
import React from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import { TimePicker } from 'udesk-ui';
import moment from 'moment';
import { time2Locale, time2Service } from '@udesk-webapp/utils';
var AntdTimePickerComponent = /** @class */ (function (_super) {
    __extends(AntdTimePickerComponent, _super);
    function AntdTimePickerComponent() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.handleDatePickerChange = function (time, timeString) {
            var _a = _this.props, _b = _a.attributes, attributes = _b === void 0 ? {} : _b, onChanged = _a.onChanged;
            var _c = attributes.format, format = _c === void 0 ? 'HH:mm:ss' : _c, needTransfer = attributes.needTransfer;
            var internalValue = timeString;
            if (timeString && needTransfer) {
                onChanged(time2Service(internalValue, format, true));
            }
            else {
                onChanged(internalValue);
            }
        };
        return _this;
    }
    AntdTimePickerComponent.prototype.render = function () {
        var _a = this.props, disabled = _a.disabled, attributes = _a.attributes, value = _a.value;
        var classNames = attributes.classNames, _b = attributes.placeholder, placeholder = _b === void 0 ? '' : _b, _c = attributes.allowClear, allowClear = _c === void 0 ? true : _c, _d = attributes.format, format = _d === void 0 ? 'HH:mm:ss' : _d, size = attributes.size, _e = attributes.use12Hours, use12Hours = _e === void 0 ? false : _e, needTransfer = attributes.needTransfer;
        value = value ? moment(value, format) : undefined;
        var internalValue;
        if (value) {
            if (needTransfer) {
                var localStr = time2Locale(value, format, true);
                internalValue = moment(localStr, format);
            }
            else {
                internalValue = moment(value, format);
            }
        }
        else {
            value = undefined;
        }
        return (React.createElement(TimePicker, { style: { width: '300px' }, className: ClassNames(classNames, 'udesk-custom-filter-antd-time-picker'), getPopupContainer: function (triggerNode) { return triggerNode.parentElement; }, size: size, placeholder: placeholder, disabled: disabled, allowClear: allowClear, use12Hours: use12Hours, format: format, value: value, onChange: this.handleDatePickerChange }));
    };
    return AntdTimePickerComponent;
}(React.Component));
export default AntdTimePickerComponent;
/**
 * @description 组件可能接收的prop
 * @namespace
 * @property {object} attributes - 配置valueSetting时传入，常用
 * @property {string} attributes.classNames - 自定义组件css类名
 * @property {string} attributes.placeholder - 自定义组件placeholder
 * @property {string} attributes.allowClear -
 * @property {string} attributes.format -
 * @property {function} onChanged - value改变唯一触发事件，其他入onFocus之类可能需要在valueSetting时配置，而且是跟自定义过滤器同一上下文
 * @property {string} value - 字段value，该组件位text input,value只需考虑string,复杂选择组件需object
 *
 * @property {object} operator - condition上下文相关
 * @property {object} field - condition上下文相关
 * @property {string} name - 校验相关，错误信息提示字段名
 * @property {boolean} isValid - 校验相关 具体待补充 todo
 * @property {array} errors - 校验相关，错误信息
 * @property {object} runtimeAttributes - 运行时传入valueInput组件的属性,从custom-filter组件传入conditionValueComponentAttrs
 *
 */
AntdTimePickerComponent.defaultProps = {
    value: null,
    onChanged: null,
    attributes: {},
    disabled: false,
    operator: {},
    field: {},
    name: null,
    isValid: true,
    errors: [],
    runtimeAttributes: {},
};
AntdTimePickerComponent.propTypes = {
    value: PropTypes.string,
    onChanged: PropTypes.func,
    attributes: PropTypes.shape({
        format: PropTypes.oneOf([
            'HH',
            'HH:mm',
            'HH:mm:ss',
            'h a',
            'h:mm a',
            'h:mm:ss a',
            'h:mm:ss A',
        ]),
        use12Hours: PropTypes.bool,
        allowClear: PropTypes.bool,
        size: PropTypes.string,
        classNames: PropTypes.any,
        placeholder: PropTypes.any,
        needTransfer: PropTypes.bool,
    }),
    disabled: PropTypes.bool,
    operator: PropTypes.object,
    field: PropTypes.object,
    name: PropTypes.string,
    isValid: PropTypes.bool,
    errors: PropTypes.array,
    runtimeAttributes: PropTypes.object,
};
