import { __extends } from "tslib";
import React from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import { Select, message, Tag } from 'udesk-ui';
import { ajax, CS_PATH_BASE } from '@udesk-webapp/utils';
import i18n from '@udesk-webapp/locale';
var Option = Select.Option;
var colorMap = {
    1: 'green',
    2: 'cyan',
    3: 'blue',
    4: 'geekblue',
    5: 'magenta',
    6: 'purple',
    7: 'red',
    8: 'orange',
    9: 'gold',
    10: '',
};
/**
 * @description 客服系统常见下拉选项
 * {
 *      title: '选项一',
 *      value: '1'
 * }
 */
var customerTagSelectComponent = /** @class */ (function (_super) {
    __extends(customerTagSelectComponent, _super);
    function customerTagSelectComponent(props) {
        var _this = _super.call(this, props) || this;
        _this.handleFocus = function (e) {
            e.persist();
            _this._loadData().then(function (resp) {
                var queryOptions = [];
                if (resp instanceof Array) {
                    queryOptions = resp;
                }
                _this.setState({
                    options: queryOptions,
                });
                //[{id: '红色', name: 'red'}] ||
            }, function () {
                message.error(i18n.t('tips.errors.dataLoadin16AgainLater'));
            });
        };
        _this.handleSelectChange = function (value, selectedOption) {
            var onChanged = _this.props.onChanged;
            var internalValue = value;
            if (value instanceof Array) {
                internalValue = value.join(',');
            }
            onChanged(internalValue);
        };
        /**
         * @param {string} query
         */
        _this.handleSelectSearch = function (query) {
            _this._loadData(query).then(function (resp) {
                var queryOptions = [];
                if (resp instanceof Array) {
                    queryOptions = resp;
                }
                _this.setState({
                    options: queryOptions,
                });
                //[{id: '红色', name: 'red'}] ||
            }, function () {
                message.error(i18n.t('tips.errors.dataLoadin16AgainLater'));
            });
        };
        _this._getTagsById = function (ids) {
            return new Promise(function (resolve, reject) {
                ajax().getJSON("".concat(CS_PATH_BASE, "/ts/api/tags"), {
                    tag_ids: ids,
                }).then(function (res) {
                    if (res.code === 1000) {
                        resolve(res.tags);
                    }
                })
                    .catch(function (error) {
                    reject(error);
                });
            });
        };
        _this._loadData = function (query) {
            return new Promise(function (resolve, reject) {
                ajax().getJSON("".concat(CS_PATH_BASE, "/ts/api/tags"), { query: query }).then(function (res) {
                    if (res.code === 1000) {
                        resolve(res.tags);
                    }
                })
                    .catch(function (error) {
                    reject(error);
                });
            });
        };
        _this.state = {
            options: [],
        };
        return _this;
    }
    customerTagSelectComponent.prototype.componentDidMount = function () {
        var _this = this;
        var value = this.props.value;
        if (value) {
            if (Array.isArray(value)) {
                this._loadData().then(function (resp) {
                    var queryOptions = [];
                    if (resp instanceof Array) {
                        queryOptions = resp;
                    }
                    _this.setState({
                        options: queryOptions,
                    });
                }, function () {
                    message.error(i18n.t('tips.errors.dataLoadin16AgainLater'));
                });
            }
            else {
                this._getTagsById(value).then(function (resp) {
                    var selectedTags = [];
                    if (resp instanceof Array) {
                        selectedTags = resp;
                    }
                    _this.setState({
                        options: selectedTags,
                    });
                }, function () {
                    message.error(i18n.t('tips.errors.dataLoadin16AgainLater'));
                });
            }
        }
    };
    customerTagSelectComponent.prototype.render = function () {
        var _a = this.props, disabled = _a.disabled, attributes = _a.attributes, 
        //dataSource,
        value = _a.value, dataSourceValueField = _a.dataSourceValueField, dataSourceNameField = _a.dataSourceNameField, defaultValueField = _a.defaultValueField, defaultNameField = _a.defaultNameField;
        var options = this.state.options;
        var classNames = attributes.classNames, dropdownClassName = attributes.dropdownClassName, placeholder = attributes.placeholder, _b = attributes.maxTagTextLength, maxTagTextLength = _b === void 0 ? 5 : _b, _c = attributes.width, width = _c === void 0 ? '300px' : _c;
        var valueField = defaultValueField;
        var nameField = defaultNameField;
        if (attributes[dataSourceValueField]) {
            valueField = attributes[dataSourceValueField];
        }
        if (attributes[dataSourceNameField]) {
            nameField = attributes[dataSourceNameField];
        }
        if (value === null) {
            value = [];
        }
        if (typeof value === 'string') {
            value = value ? value.split(',').map(function (id) { return parseInt(id); }) : [];
        }
        var optionsRender = options.map(function (option) {
            return (React.createElement(Option, { value: option[valueField], key: option[valueField], color: option.color },
                React.createElement("p", { color: option.color, style: { margin: '0' } }, option[nameField])));
        });
        var tagRender = function (optionProps) {
            var _a = optionProps.label, label = _a === void 0 ? {} : _a, _b = optionProps.onClose, onClose = _b === void 0 ? function () { } : _b;
            var _c = label.props || {}, children = _c.children, color = _c.color;
            var displayLabel = children;
            if (typeof maxTagTextLength === 'number') {
                if (typeof children === 'string' || typeof children === 'number') {
                    var strLabel = String(displayLabel);
                    if (strLabel.length > maxTagTextLength) {
                        displayLabel = "".concat(strLabel.slice(0, maxTagTextLength), "...");
                    }
                }
            }
            return displayLabel ? (React.createElement(Tag, { color: colorMap[color] || 'green', onClose: onClose, closable: !disabled, title: children }, displayLabel)) : null;
        };
        return (React.createElement(Select, { mode: "multiple", style: { width: width }, placeholder: placeholder, className: ClassNames(classNames, 'udesk-custom-filter-antd-multiple-select'), dropdownClassName: ClassNames(dropdownClassName, 'udesk-custom-filter-antd-multiple-select-dropdown'), getPopupContainer: function (triggerNode) { return triggerNode.parentElement; }, disabled: disabled, defaultActiveFirstOption: false, filterOption: false, value: value, tagRender: tagRender, onFocus: this.handleFocus, onSearch: this.handleSelectSearch, onChange: this.handleSelectChange }, optionsRender));
    };
    return customerTagSelectComponent;
}(React.Component));
export default customerTagSelectComponent;
var dropDownOptionDefaultPropsName = {
    dataSourceValueField: 'valueField',
    dataSourceNameField: 'nameField',
    dataSourceSearchField: 'searchField',
    defaultValueField: 'id',
    defaultNameField: 'name',
    defaultSearchField: 'text',
    defaultOutputValueMode: 'value', //'value'|'object'
};
/**
 * @description 组件可能接收的prop
 * @namespace
 * @property {object} attributes - 配置fields(优先级更高)和valueSetting时传入，常用
 * @property {string} attributes.classNames - 自定义组件css类名
 * @property {string} attributes.placeholder - 自定义组件placeholder
 * @property {string} attributes.dropdownClassName - 自定义下拉菜单的 className 属性
 * @property {boolean} attributes.allowClear - 单选支持清除
 * @property {function} onChanged - value改变唯一触发事件，其他入onFocus之类可能需要在valueSetting时配置，而且是跟自定义过滤器同一上下文
 * @property {string} value - 字段value，该组件位text input,value只需考虑string,复杂选择组件需object
 *
 * @property {object} operator - condition上下文相关
 * @property {object} field - condition上下文相关
 * @property {string} name - 校验相关，错误信息提示字段名
 * @property {boolean} isValid - 校验相关 具体待补充 todo
 * @property {array} errors - 校验相关，错误信息
 * @property {object} runtimeAttributes - 运行时传入valueInput组件的属性,从custom-filter组件传入conditionValueComponentAttrs
 *
 */
customerTagSelectComponent.defaultProps = Object.assign({
    value: [],
    onChanged: null,
    attributes: {},
    dataSource: [],
    disabled: false,
    operator: {},
    field: {},
    name: null,
    isValid: true,
    errors: [],
    runtimeAttributes: {},
}, dropDownOptionDefaultPropsName);
customerTagSelectComponent.propTypes = {
    value: PropTypes.array,
    onChanged: PropTypes.func,
    attributes: PropTypes.object,
    dataSource: PropTypes.array,
    disabled: PropTypes.bool,
    label: PropTypes.string,
    operator: PropTypes.object,
    field: PropTypes.object,
    name: PropTypes.string,
    isValid: PropTypes.bool,
    errors: PropTypes.array,
    runtimeAttributes: PropTypes.object,
    dataSourceValueField: PropTypes.string,
    dataSourceNameField: PropTypes.string,
    dataSourceSearchField: PropTypes.string,
    defaultValueField: PropTypes.string,
    defaultNameField: PropTypes.string,
    defaultSearchField: PropTypes.string,
    defaultOutputValueMode: PropTypes.string,
};
