import React from 'react';
import { Box, css, Text } from 'coral-system';
import { Button, Dropdown, Input, Menu } from 'antd';
import { DownOutlined, PlusSquareOutlined } from '@ant-design/icons';
const actionInputStyle = css `
  overflow: auto;
  font-family: Consolas, 'Liberation Mono', Menlo, Courier, monospace;
  cursor: pointer;
  user-select: none;

  display: flex;
  align-items: center;

  .action-input-text {
    flex: 1;
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap;
    color: var(--tango-colors-text3);

    &:hover {
      color: var(--tango-colors-text2);
    }
  }

  .anticon-down {
    color: var(--tango-colors-text-placeholder);
  }

  &::-webkit-scrollbar {
    display: none;
  }
`;
const inputStyle = { width: 'calc(100% - 82px)' };
export function ActionSelect({ showInput = false, defaultInputValue, text, options = [], onSelect, onInputChange, }) {
    const menu = (React.createElement(Menu, { onClick: ({ key }) => onSelect(key) }, options.map((item) => (React.createElement(Menu.Item, { key: item.value },
        React.createElement(Box, { display: "flex", alignItems: "center", minWidth: 200, columnGap: "m" },
            React.createElement(Box, { flex: "1" }, item.label),
            React.createElement(Text, { fontSize: "12px", color: "text.note" }, item.value)))))));
    if (showInput) {
        return (React.createElement(Input.Group, { compact: true },
            React.createElement(Input, { placeholder: "\u8BF7\u8F93\u5165\u6587\u672C", defaultValue: defaultInputValue, style: inputStyle, onBlur: (e) => {
                    onInputChange && onInputChange(e.target.value);
                }, allowClear: true }),
            React.createElement(Dropdown, { overlay: menu },
                React.createElement(Button, { icon: React.createElement(PlusSquareOutlined, null) }, "\u9884\u8BBE"))));
    }
    return (React.createElement(Dropdown, { overlay: menu, trigger: ['click'] },
        React.createElement(Box, { border: "solid", borderColor: "line.normal", borderRadius: "s", height: "32px", lineHeight: "32px", px: "m", mb: "m", css: actionInputStyle },
            React.createElement(Box, { className: "action-input-text", color: "text.body" }, text),
            React.createElement(DownOutlined, null))));
}
