import { __rest } from "tslib";
import { Tooltip } from 'antd';
import React from 'react';
import { css, Button, Link } from 'coral-system';
import cx from 'classnames';
const actionStyle = css `
  display: inline-flex;
  align-items: center;
  justify-content: center;
  width: 32px;
  height: 32px;
  outline: none;
  border: 0;
  border-radius: var(--tango-radii-s);
  color: var(--tango-colors-text2);
  background-color: transparent;

  &:hover {
    background-color: var(--tango-colors-fill2);
  }

  &.small {
    width: 24px;
    height: 24px;
  }

  &.outline {
    background-color: var(--tango-colors-fill1);
  }

  &.outline:hover {
    background-color: var(--tango-colors-fill3);
  }

  &.disabled {
    pointer-events: none;
    color: var(--tango-colors-text4);
  }
`;
export function Action(_a) {
    var { icon, tooltip, href, size, shape, disabled, className, children } = _a, rest = __rest(_a, ["icon", "tooltip", "href", "size", "shape", "disabled", "className", "children"]);
    const classNames = cx({
        'tango-action': true,
        'link-action': !!href,
        small: size === 'small',
        outline: shape === 'outline',
        disabled,
    }, className);
    let ret;
    if (href) {
        ret = (React.createElement(Link, Object.assign({ className: classNames, css: actionStyle, href: href, isExternal: true }, rest), icon !== null && icon !== void 0 ? icon : children));
    }
    else {
        ret = (React.createElement(Button, Object.assign({ type: "button", className: classNames, css: actionStyle }, rest), icon !== null && icon !== void 0 ? icon : children));
    }
    if (tooltip) {
        return React.createElement(Tooltip, { title: tooltip }, ret);
    }
    return ret;
}
