import { __awaiter } from "tslib";
import React, { useState } from 'react';
import { Button, Input } from 'antd';
import { Box, Text, css } from 'coral-system';
import { CloseOutlined, RobotOutlined } from '@ant-design/icons';
const chatBoxStyle = css `
  position: relative;
  .ant-btn {
    position: absolute;
    bottom: 6px;
    right: 6px;
  }
  textarea {
    border: 0;
  }
`;
export function ChatInput({ title = '智能编程助手', showCloseIcon = true, placeholder = '请简单描述你要执行的动作', onGenerate, onApply, onClose, }) {
    const [text, setText] = useState('');
    const [preview, setPreview] = useState('');
    const [loading, setLoading] = useState(false);
    return (React.createElement(Box, { border: "solid", borderColor: "line2", borderRadius: "m", position: "relative" },
        React.createElement(Box, { display: "flex", justifyContent: "space-between", alignItems: "center", bg: "fill2", px: "m", py: "s", borderBottom: "solid", borderBottomColor: "line2" },
            React.createElement(Box, { fontSize: "14px" },
                React.createElement(RobotOutlined, null),
                React.createElement(Text, { ml: "m", userSelect: "none" }, title)),
            showCloseIcon && React.createElement(CloseOutlined, { onClick: onClose })),
        React.createElement(Box, { css: chatBoxStyle },
            React.createElement(Input.TextArea, { rows: 3, value: text, onChange: (e) => setText(e.target.value), placeholder: placeholder }),
            React.createElement(Button, { disabled: !text, size: "small", loading: loading, onClick: () => __awaiter(this, void 0, void 0, function* () {
                    setLoading(true);
                    const ret = yield onGenerate(text);
                    setLoading(false);
                    ret && setPreview(ret);
                }) }, "\u751F\u6210")),
        React.createElement(Box, { minHeight: 48, background: "fill2", p: "m", fontSize: "12px", css: chatBoxStyle },
            React.createElement(Text, { as: "code" },
                "\u7ED3\u679C\uFF1A",
                preview || '暂无'),
            React.createElement(Button, { size: "small", disabled: !preview, onClick: () => onApply(preview) }, "\u5E94\u7528"))));
}
