export class IFrameProtocol {
    constructor(iframe, origin) {
        // Random number to identify this instance of the client when messages are coming from multiple iframes
        this.channelId = Math.floor(Math.random() * 1000000);
        // React to messages from the iframe owned by this instance
        this.channelListeners = {};
        this.channelListenersCount = 0;
        // React to messages from any iframe
        this.globalListeners = {};
        this.globalListenersCount = 0;
        // this.frameWindow = iframe.contentWindow;
        // 当在弹窗中使用 sandbox 组件时，iframe 存在，但 contentWindow 还不存在（弹窗还没弹出），需要保存 iframe
        this.iframe = iframe;
        this.origin = origin;
        this.globalListeners = [];
        this.channelListeners = [];
        this.eventListener = this.eventListener.bind(this);
        if (typeof window !== 'undefined') {
            window.addEventListener('message', this.eventListener);
        }
    }
    cleanup() {
        window.removeEventListener('message', this.eventListener);
        this.globalListeners = {};
        this.channelListeners = {};
        this.globalListenersCount = 0;
        this.channelListenersCount = 0;
    }
    // Sends the channelId and triggers an iframeHandshake promise to resolve,
    // so the iframe can start listening for messages (based on the id)
    register() {
        if (!this.iframe.contentWindow) {
            return;
        }
        this.iframe.contentWindow.postMessage({
            type: 'register-frame',
            origin: document.location.origin,
            id: this.channelId,
        }, this.origin);
    }
    // Messages are dispatched from the client directly to the instance iframe
    dispatch(message) {
        if (!this.iframe.contentWindow) {
            return;
        }
        this.iframe.contentWindow.postMessage(Object.assign({ $id: this.channelId, codesandbox: true, origin: document.location.origin }, message), this.origin);
    }
    // Add a listener that is called on any message coming from an iframe in the page
    // This is needed for the `initialize` message which comes without a channelId
    globalListen(listener) {
        if (typeof listener !== 'function') {
            return () => {
                return;
            };
        }
        const listenerId = this.globalListenersCount;
        this.globalListeners[listenerId] = listener;
        this.globalListenersCount++;
        return () => {
            delete this.globalListeners[listenerId];
        };
    }
    // Add a listener that is called on any message coming from an iframe with the instance channelId
    // All other messages (eg: from other iframes) are ignored
    channelListen(listener) {
        if (typeof listener !== 'function') {
            return () => {
                return;
            };
        }
        const listenerId = this.channelListenersCount;
        this.channelListeners[listenerId] = listener;
        this.channelListenersCount++;
        return () => {
            delete this.channelListeners[listenerId];
        };
    }
    // Handles message windows coming from iframes
    eventListener(evt) {
        // skip events originating from different iframes
        if (evt.source !== this.iframe.contentWindow) {
            return;
        }
        const message = evt.data;
        if (!message.codesandbox) {
            return;
        }
        Object.values(this.globalListeners).forEach((listener) => listener(message));
        if (message.$id !== this.channelId) {
            return;
        }
        Object.values(this.channelListeners).forEach((listener) => listener(message));
    }
}
