import { __rest } from "tslib";
import React from 'react';
import { Box, css } from 'coral-system';
import { UpOutlined } from '@ant-design/icons';
import { useControllableState } from '@music163/tango-helpers';
const headerStyle = css `
  user-select: none;

  .anticon-up {
    margin-right: 8px;
    color: var(--tango-colors-text2);
    transition: transform 0.2s ease 0s;
  }
`;
export function CollapsePanel(props) {
    const { title, extra, children, collapsed: collapsedProp, defaultCollapsed = false, onCollapse, stickyHeader, showBottomBorder = true, headerProps, bodyProps } = props, rest = __rest(props, ["title", "extra", "children", "collapsed", "defaultCollapsed", "onCollapse", "stickyHeader", "showBottomBorder", "headerProps", "bodyProps"]);
    const [collapsed, setCollapsed] = useControllableState({
        value: collapsedProp,
        defaultValue: defaultCollapsed,
        onChange: onCollapse,
    });
    const iconStyle = {
        transform: collapsed ? undefined : 'rotate(-180deg)',
    };
    const stickHeaderProps = stickyHeader
        ? {
            position: 'sticky',
            top: '0px',
            zIndex: 1,
            bg: 'white',
        }
        : {};
    const CollapsePanelBorderStyle = showBottomBorder
        ? {
            borderBottom: 'solid',
            borderBottomColor: 'line2',
        }
        : {};
    return (React.createElement(Box, Object.assign({ className: "CollapsePanel" }, CollapsePanelBorderStyle, rest),
        React.createElement(Box, Object.assign({ display: "flex", alignItems: "center", justifyContent: "space-between", className: "CollapsePanelHeader", onClick: () => setCollapsed(!collapsed), p: "m" }, stickHeaderProps, headerProps, { css: headerStyle }),
            React.createElement(Box, { display: "flex", alignItems: "center", fontSize: "14px", fontWeight: "500" },
                React.createElement(UpOutlined, { style: iconStyle }),
                title),
            React.createElement(Box, null, extra)),
        React.createElement(Box, Object.assign({ className: "CollapsePanelBody", display: collapsed ? 'none' : 'block' }, bodyProps), children)));
}
