import { __rest } from "tslib";
import { isValidFunctionCode, url2serviceName } from '@music163/tango-helpers';
import { InputCode } from '@music163/tango-ui';
import { Button, Dropdown, Form, Input, Select, Space } from 'antd';
import { Box } from 'coral-system';
import React, { useState } from 'react';
export function AddServiceForm(_a) {
    var { initialValues, onSubmit, onCancel, serviceModules, serviceNames } = _a, formProps = __rest(_a, ["initialValues", "onSubmit", "onCancel", "serviceModules", "serviceNames"]);
    const isModifyMode = !!(initialValues === null || initialValues === void 0 ? void 0 : initialValues['name']); // 是否为更新模式
    const [disabled, setDisabled] = useState(isModifyMode);
    const [form] = Form.useForm();
    return (React.createElement(Form, Object.assign({ form: form, labelCol: { span: 6 }, wrapperCol: { span: 17 }, colon: false, layout: "horizontal", initialValues: initialValues, onFinish: (values) => {
            setDisabled(true);
            onSubmit(values);
        } }, formProps),
        React.createElement(Form.Item, { label: "\u547D\u540D\u7A7A\u95F4", name: "moduleName", rules: [
                {
                    required: true,
                },
            ] },
            React.createElement(Select, { options: serviceModules, disabled: disabled || isModifyMode })),
        React.createElement(Form.Item, { label: "\u65B9\u6CD5\u540D", name: "name", rules: [
                { required: true },
                { pattern: /^[a-z]\w+$/, message: '请输入合法的方法名称' },
                !isModifyMode && {
                    validator(_, value) {
                        const isExist = serviceNames.includes(value);
                        return isExist
                            ? Promise.reject(new Error('重复的方法名，请换一个！'))
                            : Promise.resolve();
                    },
                },
            ], extra: !disabled && (React.createElement(Box, null,
                React.createElement(Button, { type: "link", size: "small", onClick: () => {
                        const url = form.getFieldValue('url');
                        if (!url) {
                            return;
                        }
                        const funcName = url2serviceName(url);
                        form.setFieldValue('name', funcName);
                    } }, "\u57FA\u4E8E\u63A5\u53E3\u8DEF\u5F84\u81EA\u52A8\u751F\u6210"))) },
            React.createElement(Input, { placeholder: "\u8BF7\u8F93\u5165\u6570\u636E\u670D\u52A1\u8C03\u7528\u540D\u79F0", disabled: disabled || isModifyMode })),
        React.createElement(Form.Item, { label: "\u8DEF\u5F84", name: "url", rules: [{ required: true, type: 'url' }] },
            React.createElement(Input, { placeholder: "\u8BF7\u8F93\u5165\u6570\u636E\u670D\u52A1\u8C03\u7528\u540D\u79F0", disabled: disabled || isModifyMode })),
        React.createElement(Form.Item, { label: "\u65B9\u6CD5", name: "method" },
            React.createElement(Select, { options: [
                    { label: 'GET', value: 'get' },
                    { label: 'POST', value: 'post' },
                ], placeholder: "\u8BF7\u9009\u62E9 HTTP \u8BF7\u6C42\u65B9\u6CD5", disabled: disabled })),
        React.createElement(Form.Item, { label: "\u683C\u5F0F\u5316\u54CD\u5E94", name: "formatter", tooltip: "\u63D0\u4F9B\u683C\u5F0F\u5316\u51FD\u6570\uFF0C\u5BF9\u7ED3\u679C\u8FDB\u884C\u683C\u5F0F\u5316", validateTrigger: "onBlur", rules: [
                {
                    validator(_, value) {
                        if (!value) {
                            return Promise.resolve();
                        }
                        return isValidFunctionCode(value)
                            ? Promise.resolve()
                            : Promise.reject(new Error('请提供合法的函数代码！'));
                    },
                },
            ], extra: !disabled && (React.createElement(Box, null,
                React.createElement(Dropdown, { menu: {
                        items: [
                            { label: '直接返回（默认）', key: 'res => res' },
                            { label: '返回数据部分（云音乐标准规范）', key: 'res => res.data' },
                        ],
                        onClick: ({ key }) => {
                            form.setFieldValue('formatter', key);
                        },
                    } },
                    React.createElement(Button, { type: "link", size: "small" }, "\u4F7F\u7528\u6A21\u677F\u51FD\u6570")))) },
            React.createElement(InputCode, { showLineNumbers: true, placeholder: "res => res", editable: !disabled })),
        React.createElement(Form.Item, { wrapperCol: { offset: 6, span: 18 } }, isModifyMode ? (React.createElement(Space, null, !disabled ? (React.createElement(React.Fragment, null,
            React.createElement(Button, { onClick: () => {
                    setDisabled(true);
                } }, "\u53D6\u6D88"),
            React.createElement(Button, { type: "primary", htmlType: "submit" }, "\u4FEE\u6539"))) : (React.createElement(Button, { onClick: () => {
                setDisabled(false);
            } }, "\u7F16\u8F91")))) : (React.createElement(Space, null,
            React.createElement(Button, { onClick: onCancel }, "\u53D6\u6D88"),
            React.createElement(Button, { type: "primary", htmlType: "submit" }, "\u521B\u5EFA"))))));
}
