import React from 'react';
import { Button, Form, Input, Space } from 'antd';
import { isValidExpressionCode } from '@music163/tango-core';
import { InputCode, Panel } from '@music163/tango-ui';
export function AddStoreVariableForm({ parentNode, onCancel, onSubmit, }) {
    return (React.createElement(Form, { layout: "vertical", autoComplete: "off", validateTrigger: "onBlur", onFinish: (values) => {
            if (onSubmit && parentNode) {
                const [type, storeName] = parentNode.key.split('.');
                onSubmit(storeName, values);
            }
        } },
        React.createElement(Form.Item, { label: "\u6240\u5C5E\u6A21\u578B" },
            React.createElement(Input, { value: parentNode === null || parentNode === void 0 ? void 0 : parentNode.key, disabled: true })),
        React.createElement(Form.Item, { label: "\u53D8\u91CF\u540D", name: "name", rules: [
                { required: true },
                { pattern: /^\w+$/, message: '非法的变量标识符' },
                {
                    validator(_, value) {
                        const found = parentNode.children.find((item) => item.title === value);
                        return found
                            ? Promise.reject(new Error('和已有变量名冲突，请换一个名字！'))
                            : Promise.resolve();
                    },
                },
            ] },
            React.createElement(Input, { placeholder: "\u8BF7\u8F93\u5165\u53D8\u91CF\u540D" })),
        React.createElement(Form.Item, { label: "\u521D\u503C", name: "initialValue", rules: [
                { required: true },
                {
                    validator(_, value) {
                        return value && isValidExpressionCode(value)
                            ? Promise.resolve()
                            : Promise.reject(new Error('代码存在语法错误，请输入合法的代码片段！'));
                    },
                },
            ] },
            React.createElement(InputCode, { shape: "inset", placeholder: "\u8BF7\u8F93\u5165\u521D\u503C" })),
        React.createElement(Form.Item, null,
            React.createElement(Space, null,
                React.createElement(Button, { onClick: onCancel }, "\u53D6\u6D88"),
                React.createElement(Button, { type: "primary", htmlType: "submit" }, "\u4FDD\u5B58")))));
}
export function AddStoreForm({ storeNames = [], onCancel, onSubmit }) {
    return (React.createElement(Panel, { title: "\u6DFB\u52A0\u6A21\u578B", subTitle: "\u6A21\u578B\u53EF\u4EE5\u7528\u6765\u7EC4\u7EC7\u4E00\u7EC4\u53D8\u91CF", bodyProps: { p: 'l' } },
        React.createElement(Form, { autoComplete: "off", colon: false, onFinish: (values) => {
                onSubmit && onSubmit(values);
            } },
            React.createElement(Form.Item, { label: "\u6A21\u578B\u540D\u79F0", name: "name", rules: [
                    { required: true },
                    { pattern: /^[a-z]\w+$/, message: '非法的变量标识符' },
                    {
                        validator(_, value) {
                            const found = storeNames.includes(value);
                            return found
                                ? Promise.reject(new Error('已存在该模型名，请换一个！'))
                                : Promise.resolve();
                        },
                    },
                ] },
                React.createElement(Input, { placeholder: "\u8BF7\u8F93\u5165\u6A21\u578B\u540D\u79F0" })),
            React.createElement(Form.Item, { label: " " },
                React.createElement(Space, null,
                    React.createElement(Button, { onClick: onCancel }, "\u53D6\u6D88"),
                    React.createElement(Button, { type: "primary", htmlType: "submit" }, "\u4FDD\u5B58"))))));
}
