import { __rest } from "tslib";
import React, { useCallback, useMemo, useState } from 'react';
import { filterTreeData, isServiceVariablePath, isStoreVariablePath, noop, parseServiceVariablePath, } from '@music163/tango-helpers';
import { css, Box, Text } from 'coral-system';
import { Button, Popconfirm, Tooltip, Tree } from 'antd';
import { CopyOutlined, DeleteOutlined, EyeOutlined, FunctionOutlined, PlusOutlined, } from '@ant-design/icons';
import { CopyClipboard, Panel, Search } from '@music163/tango-ui';
import { AddStoreForm, AddStoreVariableForm } from './add-store';
import { AddServiceForm } from './add-service';
import { NodeCommonDetail, ValueDefine, ValueDetail, } from './value-detail';
import { ValuePreview } from './value-preview';
import { ServicePreview } from './service-preview';
const varTreeStyle = css `
  overflow: auto;
  position: relative;

  .ant-tree {
    font-family: Consolas, Menlo, Courier, monospace;
  }

  .ant-tree-node-content-wrapper.ant-tree-node-content-wrapper-normal {
    width: calc(100% - 50px);
  }

  .ant-tree .ant-tree-treenode {
    padding: 0;
  }
  .ant-tree.ant-tree-directory .ant-tree-treenode::before {
    bottom: 0;
  }

  .ant-tree-indent-unit {
    width: 12px;
  }
`;
export function VariableTree(props) {
    const { dataSource = [], serviceModules = [], appContext = {}, defaultValueDetailMode, onSelect = noop, onAddStoreVariable = noop, onAddStore = noop, onAddService = noop, onUpdateStoreVariable = noop, onUpdateService = noop, onRemoveStoreVariable, onRemoveService, onCopy = noop, onView = noop, getServiceData, getServiceNames, getStoreNames, getPreviewValue = noop, showViewButton, renderHeaderExtra, renderDetail: renderDetailProp } = props, rest = __rest(props, ["dataSource", "serviceModules", "appContext", "defaultValueDetailMode", "onSelect", "onAddStoreVariable", "onAddStore", "onAddService", "onUpdateStoreVariable", "onUpdateService", "onRemoveStoreVariable", "onRemoveService", "onCopy", "onView", "getServiceData", "getServiceNames", "getStoreNames", "getPreviewValue", "showViewButton", "renderHeaderExtra", "renderDetail"]);
    const [keyword, setKeyword] = useState('');
    const [activeNode, setActiveNode] = useState();
    const [mode, setMode] = useState();
    const clear = useCallback(() => {
        setActiveNode(null);
        setMode(null);
    }, []);
    const selectNode = useCallback((node, callback) => {
        if (isStoreVariablePath(node.key)) {
            setMode('storeVariableDetail');
        }
        else if (isServiceVariablePath(node.key)) {
            setMode('serviceDetail');
        }
        else {
            setMode('detail');
        }
        setActiveNode(node);
        callback === null || callback === void 0 ? void 0 : callback(node);
    }, []);
    const treeData = useMemo(() => {
        const pattern = new RegExp(keyword, 'ig');
        return keyword
            ? filterTreeData(dataSource, (leaf) => pattern.test(leaf.title), 'children', true)
            : dataSource;
    }, [keyword, dataSource]);
    const state = { activeNode, mode, setMode, clear };
    return (React.createElement(Box, Object.assign({ display: "flex", columnGap: "l", className: "VariableTree", css: varTreeStyle }, rest),
        React.createElement(Box, { className: "VariableList", width: "40%" }, renderHeaderExtra === null || renderHeaderExtra === void 0 ? void 0 :
            renderHeaderExtra(props, state),
            React.createElement(Box, { mb: "m", position: "sticky", top: "0", bg: "white", zIndex: 2 },
                React.createElement(Search, { placeholder: "\u8BF7\u8F93\u5165\u53D8\u91CF\u540D", onChange: (val) => setKeyword(val === null || val === void 0 ? void 0 : val.trim()) })),
            React.createElement(Tree, { blockNode: true, showIcon: false, defaultExpandAll: true, treeData: treeData, onSelect: (keys, detail) => {
                    selectNode(detail.node, onSelect);
                }, titleRender: (node) => {
                    var _a;
                    const isLeaf = !node.children;
                    if (isLeaf) {
                        const showView = (_a = node.showViewButton) !== null && _a !== void 0 ? _a : showViewButton;
                        let nodePath = node.key;
                        if (/^(stores|services)\./.test(nodePath)) {
                            nodePath = `tango.${node.key.replaceAll('.', '?.').replace('?.', '.')}`;
                        }
                        return (React.createElement(Box, { display: "flex", justifyContent: "space-between", alignItems: "center" },
                            React.createElement(Text, { flex: "1", truncated: true },
                                node.title,
                                node.type === 'function' && (React.createElement(Text, { color: "text3", ml: "m" },
                                    React.createElement(FunctionOutlined, null)))),
                            React.createElement(Box, { flex: "0 0 72px", textAlign: "right" },
                                node.showRemoveButton && (React.createElement(Popconfirm, { title: `确认删除吗 ${node.title}？该操作会导致引用此模型的代码报错，请谨慎操作！`, onConfirm: (e) => {
                                        e.stopPropagation();
                                        if (isStoreVariablePath(node.key)) {
                                            onRemoveStoreVariable(node.key);
                                        }
                                        else {
                                            onRemoveService(node.key);
                                        }
                                    } },
                                    React.createElement(Button, { type: "text", size: "small", icon: React.createElement(DeleteOutlined, null), onClick: (e) => {
                                            e.stopPropagation();
                                        } }))),
                                React.createElement(CopyClipboard, { text: nodePath }, ({ copied, onClick }) => {
                                    const label = copied ? '已复制' : '复制变量路径';
                                    return (React.createElement(Tooltip, { title: label },
                                        React.createElement(Button, { type: "text", size: "small", icon: React.createElement(CopyOutlined, null), onClick: (e) => {
                                                e.stopPropagation();
                                                onClick();
                                                onCopy(node);
                                            } })));
                                }),
                                showView && (React.createElement(Tooltip, { title: "\u67E5\u770B\u53D8\u91CF\u8BE6\u60C5" },
                                    React.createElement(Button, { type: "text", size: "small", icon: React.createElement(EyeOutlined, null), onClick: (e) => {
                                            e.stopPropagation();
                                            selectNode(node, onView);
                                        } }))))));
                    }
                    return (React.createElement(Box, { display: "flex", alignItems: "center", justifyContent: "space-between" },
                        React.createElement(Text, { mr: "m" }, node.title),
                        React.createElement(Box, null,
                            /^\$?stores$/.test(node.key) && (React.createElement(Tooltip, { title: "\u65B0\u5EFA\u6570\u636E\u6A21\u578B" },
                                React.createElement(Button, { type: "text", size: "small", icon: React.createElement(PlusOutlined, null), onClick: (e) => {
                                        e.stopPropagation();
                                        setMode('addStore');
                                    } }))),
                            node.showAddButton && /^stores\.[a-zA-Z0-9]+$/.test(node.key) && (React.createElement(Tooltip, { title: `向 ${node.title} 中添加变量` },
                                React.createElement(Button, { type: "text", size: "small", icon: React.createElement(PlusOutlined, null), onClick: (e) => {
                                        e.stopPropagation();
                                        setActiveNode(node);
                                        setMode('addVariable');
                                    } }))),
                            /^services(\.[a-zA-Z0-9]+)?$/.test(node.key) && (React.createElement(Tooltip, { title: `向 ${node.title} 中添加服务函数` },
                                React.createElement(Button, { type: "text", size: "small", icon: React.createElement(PlusOutlined, null), onClick: (e) => {
                                        e.stopPropagation();
                                        setActiveNode(node);
                                        setMode('addService');
                                    } }))))));
                } })),
        React.createElement(Box, { className: "VariableDetail", flex: "1", position: "sticky", top: "0", overflow: "auto" }, (function renderDetail() {
            var _a;
            const customRenderer = renderDetailProp === null || renderDetailProp === void 0 ? void 0 : renderDetailProp(props, state);
            if (customRenderer) {
                return customRenderer;
            }
            if (mode === 'detail') {
                return React.createElement(NodeCommonDetail, { data: activeNode });
            }
            if (mode === 'storeVariableDetail') {
                return (React.createElement(ValueDetail, { key: activeNode.key, defaultMode: defaultValueDetailMode }, (previewMode) => previewMode === 'runtime' ? (React.createElement(ValuePreview, { value: getPreviewValue(activeNode), onCopy: (valuePath) => {
                        return ['tango', activeNode.key.replaceAll('.', '?.'), valuePath].join('.');
                    } })) : (React.createElement(ValueDefine, { data: activeNode, onSave: onUpdateStoreVariable }))));
            }
            if (mode === 'serviceDetail') {
                return (React.createElement(React.Fragment, null,
                    React.createElement(Panel, { shape: "solid", title: "\u670D\u52A1\u51FD\u6570\u914D\u7F6E" },
                        React.createElement(AddServiceForm, { key: activeNode.key, serviceModules: serviceModules, serviceNames: (function () {
                                const { moduleName } = parseServiceVariablePath(activeNode.key);
                                return (getServiceNames === null || getServiceNames === void 0 ? void 0 : getServiceNames(moduleName)) || [];
                            })(), initialValues: Object.assign({}, getServiceData === null || getServiceData === void 0 ? void 0 : getServiceData(activeNode.key)), onCancel: clear, onSubmit: (values) => {
                                onUpdateService(values);
                            } })),
                    React.createElement(Panel, { shape: "solid", title: "\u670D\u52A1\u51FD\u6570\u9884\u89C8", mt: "l" },
                        React.createElement(ServicePreview, { key: activeNode.key, appContext: appContext, functionKey: activeNode.key }))));
            }
            if (mode === 'addVariable') {
                return (React.createElement(AddStoreVariableForm, { parentNode: activeNode, onSubmit: (storeName, data) => {
                        onAddStoreVariable(storeName, data);
                        clear();
                    }, onCancel: () => {
                        clear();
                    } }));
            }
            if (mode === 'addStore') {
                return (React.createElement(AddStoreForm, { storeNames: (getStoreNames === null || getStoreNames === void 0 ? void 0 : getStoreNames()) || [], onSubmit: ({ name }) => {
                        onAddStore(name);
                        clear();
                    }, onCancel: () => {
                        clear();
                    } }));
            }
            if (mode === 'addService') {
                return (React.createElement(Panel, { shape: "solid", title: "\u521B\u5EFA\u670D\u52A1\u51FD\u6570" },
                    React.createElement(AddServiceForm, { key: activeNode.key, serviceModules: serviceModules, serviceNames: ((_a = activeNode.children) === null || _a === void 0 ? void 0 : _a.map((item) => item.title)) || [], initialValues: {
                            moduleName: activeNode.title,
                        }, onCancel: () => {
                            clear();
                        }, onSubmit: (values) => {
                            onAddService(values);
                            clear();
                        } })));
            }
        })())));
}
