import React, { useState } from 'react';
import { Box } from 'coral-system';
import { Button, Empty } from 'antd';
import { PlayCircleOutlined } from '@ant-design/icons';
import { InputCode, Panel, JsonView } from '@music163/tango-ui';
import { isNil, logger, getValue } from '@music163/tango-helpers';
import { code2value } from '@music163/tango-core';
export function ServicePreview({ appContext, functionKey }) {
    const [payload, setPayload] = useState({});
    const [result, setResult] = useState();
    const [error, setError] = useState('');
    return (React.createElement(Box, null,
        React.createElement(Panel, { title: "\u8BF7\u6C42\u53C2\u6570", bodyProps: { px: 'l' } },
            React.createElement(InputCode, { placeholder: '添加请求参数，对象格式，如 { key: value }', editable: true, showLineNumbers: true, onChange: (value) => {
                    const obj = code2value(value); // 转为 object 对象
                    setPayload(obj);
                } }),
            React.createElement(Button, { block: true, type: "primary", style: { margin: '8px 0' }, disabled: !appContext, onClick: () => {
                    if (!appContext) {
                        setError('执行上下文未准备好，请关闭面板重试');
                        return;
                    }
                    try {
                        const fn = getValue(appContext, functionKey);
                        fn(payload).then((data) => {
                            setError('');
                            setResult(data);
                        });
                    }
                    catch (err) {
                        setError('接口调用失败，请检查参数是否正确');
                        logger.error(err);
                    }
                }, icon: React.createElement(PlayCircleOutlined, null) }, "\u9884\u89C8")),
        React.createElement(Panel, { title: "\u8BF7\u6C42\u54CD\u5E94", bodyProps: { px: 'l' } }, error || (result ? React.createElement(ResponseDataPreview, { data: result }) : '点击预览按钮测试接口返回值'))));
}
function ResponseDataPreview({ data }) {
    let ret;
    if (!isNil(data)) {
        switch (typeof data) {
            case 'object':
                ret = React.createElement(JsonView, { src: data });
                break;
            default:
                ret = String(data);
                break;
        }
    }
    const initialRet = React.createElement(Empty, { image: Empty.PRESENTED_IMAGE_SIMPLE });
    return (React.createElement(Box, { className: "ResponseDataPreview", overflow: "auto", height: "auto", minHeight: 260, marginTop: 10 }, ret || initialRet));
}
