import React, { useState } from 'react';
import { isNil, noop, useBoolean } from '@music163/tango-helpers';
import { Empty, Space, Button, Radio, Alert } from 'antd';
import { Box } from 'coral-system';
import { InputCode, Panel } from '@music163/tango-ui';
import { isValidExpressionCode } from '@music163/tango-core';
const previewOptions = [
    { label: '运行时', value: 'runtime' },
    { label: '定义', value: 'define' },
];
export function ValueDetail({ defaultMode = 'runtime', children }) {
    const [mode, setMode] = useState(defaultMode);
    return (React.createElement(Panel, { title: "\u53D8\u91CF\u8BE6\u60C5", shape: "solid", extra: React.createElement(Radio.Group, { optionType: "button", buttonStyle: "solid", size: "small", value: mode, onChange: (e) => {
                setMode(e.target.value);
            }, options: previewOptions }) }, children(mode)));
}
/**
 * 变量值定义面板
 */
export function ValueDefine({ data, onSave = noop }) {
    const [value, setValue] = useState(data.raw);
    const [error, setError] = useState('');
    const [editable, { on, off }] = useBoolean();
    if (isNil(data.raw)) {
        return React.createElement(Empty, { image: Empty.PRESENTED_IMAGE_SIMPLE, description: "\u7F3A\u5931\u5B9A\u4E49\u6570\u636E" });
    }
    return (React.createElement(Box, null,
        React.createElement(InputCode, { shape: "inset", value: value, onChange: (nextValue) => setValue(nextValue), readOnly: !editable, onBlur: () => {
                if (!value) {
                    setError('输入内容不可为空！');
                }
                else if (value && !isValidExpressionCode(`foo = ${value}`)) {
                    // 这里先保证输入是一个合法的表达式
                    setError('代码格式错误，请检查代码语法！');
                }
                else {
                    setError('');
                }
            } }),
        React.createElement(Box, { color: "red", my: "m" }, error),
        React.createElement(Box, { mt: "l" }, editable ? (React.createElement(Space, null,
            React.createElement(Button, { onClick: () => {
                    off();
                    setError('');
                } }, "\u53D6\u6D88"),
            React.createElement(Button, { type: "primary", onClick: () => {
                    if (error) {
                        return;
                    }
                    onSave(data.key, value);
                    off();
                } }, "\u4FDD\u5B58"))) : (React.createElement(Button, { onClick: on }, "\u7F16\u8F91")))));
}
export function NodeCommonDetail({ data }) {
    if (!(data === null || data === void 0 ? void 0 : data.help)) {
        return React.createElement("div", null);
    }
    return React.createElement(Alert, { type: "info", message: `使用说明：${data.help}`, closable: true });
}
