import { __rest } from "tslib";
import React from 'react';
import { Box } from 'coral-system';
import { Input, Switch, Select } from 'antd';
import { SelectList } from './select-list';
/**
 * 配置组
 */
export function ConfigGroup({ title, children, }) {
    return (React.createElement(Box, { className: "SettingGroup", mb: "l" },
        React.createElement(Box, { className: "SettingGroupTitle", fontSize: "14px", fontWeight: "bold", borderBottom: "solid", borderBottomColor: "#f0f0f0", px: "l", py: "s" }, title),
        React.createElement(Box, { className: "SettingGroupChildren", px: "l" }, children)));
}
const builtinConfigSetterMap = {
    switch: (_a) => {
        var { value, onChange } = _a, rest = __rest(_a, ["value", "onChange"]);
        return React.createElement(Switch, Object.assign({}, rest, { checked: value, onChange: onChange }));
    },
    input: (_a) => {
        var { value, onChange } = _a, rest = __rest(_a, ["value", "onChange"]);
        return React.createElement(Input, Object.assign({}, rest, { value: value, onBlur: (e) => onChange(e.target.value) }));
    },
    select: Select,
    selectList: SelectList,
};
/**
 * 配置项
 */
export function ConfigItem({ title, description, children: childrenProp, layoutDirection = 'row', component, componentProps = {}, onChange, value, }) {
    let children = childrenProp;
    const Setter = builtinConfigSetterMap[component];
    if (Setter) {
        children = React.createElement(Setter, Object.assign({}, componentProps, { value: value, onChange: onChange }));
    }
    const { rootStyle, leftStyle, rightStyle } = useConfigItemStyle(layoutDirection);
    return (React.createElement(Box, Object.assign({ className: "SettingItem", py: "m", borderBottom: "solid", borderBottomColor: "#f0f0f0" }, rootStyle),
        React.createElement(Box, Object.assign({ className: "SettingItemLeft" }, leftStyle),
            React.createElement(Box, { className: "SettingItemTitle", fontSize: "14px", fontWeight: "bold", mb: "s" }, title),
            React.createElement(Box, { className: "SettingItemDescription", fontSize: "12px", color: "text.note" }, description)),
        React.createElement(Box, Object.assign({ className: "SettingItemRight" }, rightStyle), children)));
}
function useConfigItemStyle(layoutDirection) {
    let rootStyle;
    let leftStyle;
    let rightStyle;
    if (layoutDirection === 'row') {
        rootStyle = {
            display: 'flex',
            alignItems: 'center',
        };
        leftStyle = {
            width: '60%',
        };
        rightStyle = {
            width: '40%',
            display: 'flex',
            justifyContent: 'flex-end',
        };
    }
    else if (layoutDirection === 'column') {
        rootStyle = {
            display: 'flex',
            flexDirection: 'column',
        };
    }
    return {
        rootStyle,
        leftStyle,
        rightStyle,
    };
}
