import { createContext } from '@music163/tango-helpers';
const [TangoEngineProvider, useTangoEngine] = createContext({
    name: 'TangoEngineContext',
});
export { TangoEngineProvider };
export const useWorkspace = () => {
    var _a;
    return (_a = useTangoEngine()) === null || _a === void 0 ? void 0 : _a.engine.workspace;
};
export const useDesigner = () => {
    var _a;
    return (_a = useTangoEngine()) === null || _a === void 0 ? void 0 : _a.engine.designer;
};
export const useWorkspaceData = () => {
    var _a, _b, _c, _d, _e;
    const ctx = useTangoEngine();
    const workspace = useWorkspace();
    const modelVariables = []; // 绑定变量列表
    const storeActionVariables = []; // 模型中的所有 actions
    const storeVariables = []; // 模型中的所有变量
    const pageStoreVariables = []; // 页面中的组件实例变量
    const serviceVariables = []; // 服务中的所有变量
    pageStoreVariables.push({
        title: 'pageStore',
        key: 'pageStore',
    });
    Object.values(workspace.storeModules).forEach((file) => {
        const prefix = `stores.${file.name}`;
        const states = file.states.map((item) => ({
            title: item.name,
            key: `${prefix}.${item.name}`,
            raw: item.code,
            showRemoveButton: true,
        }));
        const actions = file.actions.map((item) => ({
            title: item.name,
            key: `${prefix}.${item.name}`,
            type: 'function',
            raw: item.code,
            showRemoveButton: true,
        }));
        modelVariables.push({
            title: file.name,
            key: prefix,
            selectable: false,
            children: states,
            showAddButton: true,
        });
        storeActionVariables.push({
            title: file.name,
            key: prefix,
            selectable: false,
            children: actions,
        });
        storeVariables.push({
            title: file.name,
            key: prefix,
            selectable: false,
            children: [...states, ...actions],
            showAddButton: true,
        });
    });
    Object.values(workspace.serviceModules).forEach((file) => {
        const prefix = file.name !== 'index' ? `services.${file.name}` : 'services';
        serviceVariables.push({
            title: file.name,
            key: prefix,
            selectable: false,
            showAddButton: true,
            children: Object.keys(file.serviceFunctions || {}).map((key) => ({
                title: key,
                key: [prefix, key].join('.'),
                type: 'function',
                showRemoveButton: true,
            })),
        });
    });
    // 路由选项列表
    const routeOptions = (_a = workspace.pages) === null || _a === void 0 ? void 0 : _a.map((item) => ({
        label: `${item.name}  (${item.path})`,
        value: item.path,
    }));
    let actionVariables = [
        buildVariableOptions('数据模型', '$stores', storeActionVariables),
        buildVariableOptions('服务函数', '$services', serviceVariables),
    ];
    if ((_b = ctx.config) === null || _b === void 0 ? void 0 : _b.customActionVariables) {
        actionVariables = actionVariables.concat((_c = ctx.config) === null || _c === void 0 ? void 0 : _c.customActionVariables);
    }
    let expressionVariables = [
        buildVariableOptions('当前页面组件实例', '$pageStore', pageStoreVariables),
        buildVariableOptions('数据模型', '$stores', storeVariables),
        buildVariableOptions('服务函数', '$services', serviceVariables),
    ];
    if ((_d = ctx.config) === null || _d === void 0 ? void 0 : _d.customExpressionVariables) {
        expressionVariables = expressionVariables.concat((_e = ctx.config) === null || _e === void 0 ? void 0 : _e.customExpressionVariables);
    }
    return {
        modelVariables: [buildVariableOptions('数据模型', 'stores', modelVariables)],
        actionVariables,
        storeVariables,
        serviceVariables,
        expressionVariables,
        routeOptions,
    };
};
function buildVariableOptions(title, key, children) {
    return {
        key,
        title,
        children,
    };
}
