import { __rest } from "tslib";
import React, { useRef, useEffect, useCallback } from 'react';
import { Box } from 'coral-system';
import { MultiEditor } from '@music163/tango-ui';
import { observer, useWorkspace } from '@music163/tango-context';
import { isValidCode } from '@music163/tango-core';
import { Modal } from 'antd';
const ideConfig = {
    // disableFileOps: {
    //   add: true,
    //   delete: true,
    // },
    // disableFolderOps: {
    //   add: true,
    //   delete: true,
    //   rename: true,
    // },
    disablePrettier: true,
    disableEslint: true,
    saveWhenBlur: true,
    // disableSetting: true,
};
export const CodeEditor = observer((_a) => {
    var { autoRemoveUnusedImports = true } = _a, rest = __rest(_a, ["autoRemoveUnusedImports"]);
    const editorRef = useRef(null);
    const workspace = useWorkspace();
    const files = workspace.listFiles();
    const activeFile = workspace.activeFile;
    let loc; // 记录视图代码的选中位置
    const selectNode = workspace.selectSource.firstNode;
    if (selectNode && activeFile === workspace.activeViewFile) {
        loc = selectNode.loc;
    }
    useEffect(() => {
        var _a;
        (_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.refresh(files, activeFile, loc);
    }, [files, activeFile, loc]);
    const fileSave = useCallback((path, value) => {
        if (!isJsFile(path)) {
            // 非 js 文件直接保存
            workspace.updateFile(path, value, autoRemoveUnusedImports);
            return;
        }
        // js 文件需要先检查语法，只有语法正确才会保存
        if (isValidCode(value)) {
            workspace.updateFile(path, value, autoRemoveUnusedImports);
        }
        else {
            Modal.confirm({
                title: '检测到代码中存在语法错误，暂时无法将代码同步给设计器，是否回退到安全代码？',
                onOk: () => {
                    var _a;
                    (_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.refresh(files, activeFile);
                },
                onCancel: () => { },
            });
        }
    }, [workspace, autoRemoveUnusedImports, activeFile, files]);
    const handleFileChange = useCallback((type, info) => {
        switch (type) {
            case 'addFile':
                workspace.addFile(info.path, info.value);
                break;
            case 'deleteFile':
            case 'deleteFolder':
                workspace.removeFile(info.path);
                break;
            case 'renameFile':
                workspace.renameFile(info.path, info.newpath);
                workspace.setActiveFile(info.newpath);
                break;
            case 'renameFolder':
                workspace.renameFolder(info.path, info.newpath);
                break;
            case 'addFolder':
            default:
                break;
        }
    }, [workspace]);
    const handlePathChange = useCallback((path) => {
        workspace.setActiveFile(path);
    }, [workspace]);
    return (React.createElement(Box, { display: "flex", flexDirection: "row", height: "100%", bg: "white" },
        React.createElement(MultiEditor, Object.assign({ ref: editorRef, options: {
                fontSize: 14,
                automaticLayout: true,
            }, ideConfig: ideConfig, onFileSave: fileSave, onPathChange: handlePathChange, onFileChange: handleFileChange, defaultPath: activeFile, defaultTheme: "GithubLightDefault", defaultFiles: files }, rest))));
});
function isJsFile(path) {
    return /.jsx?$/.test(path);
}
