import React from 'react';
import { Box } from 'coral-system';
import { logger } from '@music163/tango-helpers';
export class ErrorBoundary extends React.Component {
    static getDerivedStateFromError(error) {
        // Update state so the next render will show the fallback UI.
        logger.error(error);
        return { hasError: true };
    }
    constructor(props) {
        super(props);
        this.state = { hasError: false };
    }
    componentDidCatch(error, errorInfo) {
        // You can also log the error to an error reporting service
        logger.error(error, errorInfo);
    }
    render() {
        if (this.state.hasError) {
            // You can render any custom fallback UI
            return (React.createElement(Box, { as: "h1", color: "red" }, "Something went wrong."));
        }
        return this.props.children;
    }
}
