import React from 'react';
import { toJS } from 'mobx';
import { observer } from 'mobx-react-lite';
import { clone, useBoolean } from '@music163/tango-helpers';
import { isWrappedByExpressionContainer } from '@music163/tango-core';
import { ToggleButton, CodeOutlined } from '@music163/tango-ui';
import { useFormModel, useFormVariable } from './context';
import { FormControl } from './form-ui';
import { Box, Text } from 'coral-system';
const defaultGetSetterProps = () => ({});
const defaultGetVisible = () => true;
export function createFormItem(options) {
    var _a;
    const renderSetter = (_a = options.render) !== null && _a !== void 0 ? _a : ((props) => React.createElement(options.component, props));
    function FormItem({ name, title, tip, placeholder, docs, autoCompleteOptions, setter, setterProps, defaultValue, options: setterOptions, disableVariableSetter: disableVariableSetterProp = options.disableVariableSetter, getVisible: getVisibleProp, getSetterProps: getSetterPropsProp, deprecated, extra, footer, noStyle, validate, }) {
        var _a, _b, _c;
        const { disableSwitchExpressionSetter, showItemSubtitle } = useFormVariable();
        const model = useFormModel();
        const field = model.getField(name);
        const value = toJS((_a = field.value) !== null && _a !== void 0 ? _a : defaultValue);
        const disableVariableSetter = disableSwitchExpressionSetter !== null && disableSwitchExpressionSetter !== void 0 ? disableSwitchExpressionSetter : disableVariableSetterProp; // Form 的设置优先
        const [isVariable, { toggle: toggleIsVariable }] = useBoolean(() => !disableVariableSetter && isWrappedByExpressionContainer(value));
        const setterName = isVariable ? 'expressionSetter' : setter;
        field.setConfig({
            validate: validate || options.validate,
        });
        const baseComponentProps = clone({
            value,
            defaultValue,
            onChange: field.handleChange,
            status: field.error ? 'error' : undefined,
            placeholder,
            options: setterOptions,
        }, false);
        let expProps = {};
        // FIXME: 重新考虑这段代码的位置，外置这个逻辑
        if (['expressionSetter', 'expSetter', 'actionSetter', 'eventSetter'].includes(setter) ||
            isVariable) {
            expProps = {
                modalTitle: title,
                modalTip: tip,
                autoCompleteOptions,
            };
        }
        const getSetterProps = getSetterPropsProp || defaultGetSetterProps;
        // 从注册表中获取 expSetter
        const ExpressionSetter = (_c = (_b = REGISTERED_FORM_ITEM_MAP['expressionSetter']) === null || _b === void 0 ? void 0 : _b.config) === null || _c === void 0 ? void 0 : _c.component;
        const setterNode = isVariable ? (React.createElement(ExpressionSetter, Object.assign({}, expProps, baseComponentProps))) : (renderSetter(Object.assign(Object.assign(Object.assign(Object.assign({}, expProps), baseComponentProps), setterProps), getSetterProps(model))));
        const getVisible = getVisibleProp || defaultGetVisible;
        if (noStyle) {
            // 无样式模式
            return getVisible(model) ? setterNode : React.createElement("div", { "data-setter": setterName, "data-field": name });
        }
        return (React.createElement(FormControl, { visible: getVisible(model), label: title, note: showItemSubtitle ? name : null, tip: tip, docs: docs, deprecated: deprecated, error: field.error, extra: React.createElement(Box, null,
                extra,
                !disableVariableSetter ? (React.createElement(ToggleButton, { borderRadius: "s", size: "s", shape: "text", type: "primary", tooltip: isVariable ? '关闭 JS 表达式' : '使用 JS 表达式', tooltipPlacement: "left", selected: isVariable, onClick: () => toggleIsVariable() },
                    React.createElement(CodeOutlined, null))) : null), footer: footer, "data-setter": setterName, "data-field": name }, setterNode));
    }
    FormItem.displayName = `FormItem_${options.name}`;
    FormItem.config = options;
    return observer(FormItem);
}
// 已注册的 setter 查找表
const REGISTERED_FORM_ITEM_MAP = {};
/**
 * Setter 注册
 * @param config 注册选项
 */
export function register(config) {
    var _a;
    const names = [config.name, ...((_a = config.alias) !== null && _a !== void 0 ? _a : [])];
    names.forEach((name) => {
        REGISTERED_FORM_ITEM_MAP[name] = createFormItem(config);
    });
}
export function SettingFormItem(props) {
    const { setter } = props;
    const Comp = REGISTERED_FORM_ITEM_MAP[setter];
    if (Comp == null) {
        const Fallback = REGISTERED_FORM_ITEM_MAP.expressionSetter;
        return (React.createElement(Fallback, Object.assign({}, props, { footer: React.createElement(Text, { color: "red", mt: "m" },
                props.setter,
                " is invalid") })));
    }
    return React.createElement(Comp, props);
}
