import React from 'react';
import { Box } from 'coral-system';
import { observer } from 'mobx-react-lite';
import { SettingFormItem } from './form-item';
import { FormModelProvider, useFormModel } from './context';
import { FormControlGroup } from './form-ui';
import { isValidNestProps } from './helpers';
const defaultGetVisible = () => true;
/**
 * 为该组件下的 SettingFormItem 添加一个数据字段前缀
 */
export const SettingFormObject = observer(({ title, tip, docs, name, defaultValue, getVisible = defaultGetVisible, props = [], }) => {
    var _a;
    const parent = useFormModel();
    const visible = getVisible(parent);
    const subModel = parent.getSubModel(name);
    return (React.createElement(FormModelProvider, { value: subModel },
        React.createElement(Box, { className: "FormObject", display: visible ? 'block' : 'none' },
            React.createElement(FormControlGroup, { label: title, note: name, tip: tip, docs: docs, checked: (_a = !!subModel.values) !== null && _a !== void 0 ? _a : !!defaultValue, onCheck: (checked) => {
                    const nextValue = checked ? {} : undefined;
                    parent.setValue(name, nextValue);
                    parent.onChange(name, nextValue); // 非 Field 发起， 主动调一次
                } }, props.map((prop) => {
                if (isValidNestProps(prop.props)) {
                    return React.createElement(SettingFormObject, Object.assign({ key: prop.name }, prop));
                }
                return React.createElement(SettingFormItem, Object.assign({ key: prop.name }, prop));
            })))));
});
