import { __rest } from "tslib";
import React, { useState } from 'react';
import { css, Box, Link } from 'coral-system';
import { Checkbox, Tooltip } from 'antd';
import { CollapsePanel } from '@music163/tango-ui';
import { isString } from '@music163/tango-helpers';
import { WarningOutlined } from '@ant-design/icons';
export function FormControl(_a) {
    var { visible, label, note, tip, docs, extra, deprecated, footer, error, children } = _a, rest = __rest(_a, ["visible", "label", "note", "tip", "docs", "extra", "deprecated", "footer", "error", "children"]);
    return (React.createElement(Box, Object.assign({ className: "FormControl", display: visible ? 'block' : 'none' }, rest),
        React.createElement(Box, { display: "flex", justifyContent: "space-between", alignItems: "center", mb: "s" },
            React.createElement(FormLabel, { label: label, note: note, tip: tip, docs: docs, deprecated: deprecated }),
            extra),
        React.createElement(Box, null, children),
        footer,
        !!error && (React.createElement(Box, { mt: "m", color: "red" }, error))));
}
const formControlGroupContentStyle = css `
  > .FormControlGroup {
    border-style: solid;
    border-width: 1px;
  }
`;
export function FormControlGroup({ label, note, tip, docs, error, extra, children, onCheck, checked, }) {
    // 默认折叠
    const [collapsed, setCollapsed] = useState(true);
    return (React.createElement(CollapsePanel, { collapsed: collapsed, onCollapse: setCollapsed, className: "FormControlGroup", stickyHeader: true, title: React.createElement(Box, { display: "flex", columnGap: "s" },
            React.createElement(Checkbox, { checked: checked, onChange: (e) => {
                    e.stopPropagation();
                    const nextChecked = e.target.checked;
                    setCollapsed(!nextChecked);
                    onCheck === null || onCheck === void 0 ? void 0 : onCheck(nextChecked);
                } }),
            React.createElement(FormLabel, { label: label, note: note, tip: tip, docs: docs })), extra: React.createElement(Box, { flex: "1", textAlign: "right" }, extra), border: "solid", borderColor: "line.normal", headerProps: {
            bg: 'fill1',
            zIndex: 2,
        } },
        React.createElement(Box, { p: "m", display: "flex", flexDirection: "column", rowGap: "l", css: formControlGroupContentStyle }, children),
        React.createElement(Box, { color: "red" }, error)));
}
const labelStyle = css `
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
  white-space: nowrap;
  max-width: 160px;
  vertical-align: bottom;
  user-select: none;

  &.hasHelp {
    border-bottom: 1px dashed var(--tango-colors-text-normal);
  }
`;
const tipStyle = css `
  a {
    text-decoration: underline;
  }

  a:link {
    color: #fff;
  }
`;
function FormLabel(_a) {
    var { label, note, tip, docs, deprecated } = _a, rest = __rest(_a, ["label", "note", "tip", "docs", "deprecated"]);
    let help;
    if (deprecated || docs) {
        help = (React.createElement(Box, null,
            React.createElement(Box, { css: tipStyle },
                tip,
                docs ? (React.createElement(Link, { href: docs, isExternal: true, ml: "m" }, "\u67E5\u770B\u5C5E\u6027\u6587\u6863")) : null),
            deprecated ? (React.createElement(Box, { color: "#faad14" },
                React.createElement(WarningOutlined, null),
                " \u5E9F\u5F03\u63D0\u793A\uFF1A",
                isString(deprecated) ? deprecated : '该属性已废弃，请谨慎使用。')) : null));
    }
    else {
        help = tip;
    }
    let labelNode = (React.createElement(Box, { as: "span", display: "inline-block", color: "text2", className: help ? 'hasHelp' : '', css: labelStyle, title: isString(label) ? label : undefined },
        deprecated ? React.createElement(WarningOutlined, { style: { color: '#faad14', marginRight: 4 } }) : null,
        label));
    if (help) {
        labelNode = React.createElement(Tooltip, { title: help }, labelNode);
    }
    return (React.createElement(Box, Object.assign({ fontSize: "14px" }, rest),
        labelNode,
        note && (React.createElement(Box, { as: "i", display: "inline-block", fontSize: "12px", color: "text3", ml: "m" }, note))));
}
export function FieldSet({ title, extra, children }) {
    return (React.createElement(CollapsePanel, { title: title, extra: extra },
        React.createElement(Box, { px: "m" }, children)));
}
export function FormHeader({ title, extra, subTitle }) {
    return (React.createElement(Box, { className: "FormHeader" },
        React.createElement(Box, { display: "flex", alignItems: "center" },
            React.createElement(Box, { flex: "1", display: "flex", alignItems: "center" },
                React.createElement(Box, { fontSize: "16px", fontWeight: "500", mr: "s" }, title),
                subTitle && React.createElement(Box, null, subTitle)),
            extra && React.createElement(Box, null, extra))));
}
