import React from 'react';
import { IComponentPrototype, IComponentProp } from '@music163/tango-helpers';
import { FormModel, FormModelOptionsType } from './form-model';
interface IFormTabsGroupOption {
    label: string;
    value: string;
}
export interface SettingFormProps {
    /**
     * 表单状态管理模型实例
     */
    model?: FormModel;
    /**
     * 默认状态初值，如果没有传 model，则会基于 defaultValue 在表单内进行 model 实例化
     */
    defaultValue?: unknown;
    /**
     * 表单值变化时的回调，在校验执行后触发，仅在使用内置 model 时有效
     */
    onChange?: FormModelOptionsType['onChange'];
    /**
     * 组件的可配置描述
     */
    prototype?: IComponentPrototype;
    /**
     * 是否显示组件标识
     */
    showIdentifier?: false | {
        identifierKey: string;
    };
    /**
     * 是否显示搜索框
     */
    showSearch?: boolean;
    /**
     * 是否显示分组导航
     */
    showGroups?: boolean;
    /**
     * 选项分组信息，用于对表单项进行分组展示，如未提供，则使用内置的分组信息
     */
    groupOptions?: IFormTabsGroupOption[];
    /**
     * 是否显示表单项的副标题
     */
    showItemSubtitle?: boolean;
    /**
     * 自定义渲染表单项的额外内容（标签右侧）
     */
    renderItemExtra?: (props: IComponentProp) => React.ReactNode;
    /**
     * 是否允许表单项切换到表达式设置器
     */
    disableSwitchExpressionSetter?: boolean;
}
export declare function SettingForm({ prototype, disableSwitchExpressionSetter, model: modelProp, defaultValue, onChange, showIdentifier, showSearch, showGroups, showItemSubtitle, renderItemExtra, groupOptions, }: SettingFormProps): React.JSX.Element;
export declare namespace SettingForm {
    var register: typeof import("./form-item").register;
}
export {};
