import { __rest } from "tslib";
import React, { useState, useMemo, useCallback, useEffect } from 'react';
import { Box, css } from 'coral-system';
import { Empty } from 'antd';
import { noop, filterTreeData, mapTreeData, } from '@music163/tango-helpers';
import { Action, Search, Tabs } from '@music163/tango-ui';
import { SettingFormItem, register } from './form-item';
import { FormModelProvider, FormVariableProvider } from './context';
import { FormModel } from './form-model';
import { SettingFormObject } from './form-object';
import { isValidNestProps } from './helpers';
import { registerBuiltinSetters } from './setter';
import { FormHeader } from './form-ui';
import { QuestionCircleOutlined } from '@ant-design/icons';
registerBuiltinSetters();
function normalizeComponentProps(props, groupOptions) {
    const groups = groupOptions.reduce((prev, cur) => {
        prev[cur.value] = [];
        return prev;
    }, {});
    props.forEach((prop) => {
        const group = groups[prop.group] ? prop.group : 'basic';
        groups[group].push(prop);
    });
    return groups;
}
function filterComponentProps(props, keyword) {
    if (!keyword) {
        return props;
    }
    const pattern = new RegExp(keyword, 'ig');
    return filterTreeData(props, (prop) => pattern.test(prop.title) || pattern.test(prop.name), 'props');
}
const formStyle = css `
  position: relative;
  height: 100%;
  display: flex;
  flex-direction: column;
  > .SettingFormMain > :first-child:is(.FormControl) {
    margin-top: var(--tango-space-m);
  }

  > .SettingFormMain > .FormControl {
    margin-left: var(--tango-space-m);
    margin-right: var(--tango-space-m);
  }

  > .SettingFormMain > .FormObject {
    margin-left: var(--tango-space-m);
    margin-right: var(--tango-space-m);
  }
`;
const internalGroups = [
    { label: '基本', value: 'basic' },
    // { label: '事件', value: 'event' },
    { label: '样式', value: 'style' },
    { label: '高级', value: 'advanced' },
];
export function SettingForm({ prototype, disableSwitchExpressionSetter, model: modelProp, defaultValue, onChange = noop, showIdentifier = false, showSearch = true, showGroups = true, showItemSubtitle = true, renderItemExtra, groupOptions = internalGroups, }) {
    const [keyword, setKeyword] = useState('');
    const [tabKey, setTabKey] = useState('basic');
    const [model, setModel] = useState(modelProp !== null && modelProp !== void 0 ? modelProp : new FormModel(defaultValue, { onChange }));
    const { props: componentProps = [] } = prototype;
    const componentPropGroups = useMemo(() => {
        return normalizeComponentProps(componentProps, groupOptions);
    }, [componentProps, groupOptions]);
    const filterProps = useMemo(() => {
        const computedProps = mapTreeData(componentProps, (node) => {
            const { getProp } = node, rest = __rest(node, ["getProp"]);
            return Object.assign(Object.assign({}, rest), getProp === null || getProp === void 0 ? void 0 : getProp(model));
        });
        // 没有进行搜索的情况
        if (!keyword) {
            if (showGroups) {
                return (componentPropGroups[tabKey].sort((a, b) => {
                    // TIP: 将废弃属性往后移
                    if (!a.deprecated && b.deprecated) {
                        return -1;
                    }
                    return 1;
                }) || []);
            }
            else {
                return computedProps;
            }
        }
        // 用户进行搜索的情况
        return filterComponentProps(computedProps, keyword);
    }, [keyword, componentProps, tabKey, model, showGroups, componentPropGroups]);
    const renderProps = useCallback((list) => list.map((item) => {
        const { getProp } = item, rest = __rest(item, ["getProp"]);
        const childProp = Object.assign(Object.assign({}, rest), getProp === null || getProp === void 0 ? void 0 : getProp(model));
        if (isValidNestProps(childProp.props)) {
            return React.createElement(SettingFormObject, Object.assign({ key: item.name }, childProp));
        }
        else {
            return (React.createElement(SettingFormItem, Object.assign({ key: item.name, extra: renderItemExtra === null || renderItemExtra === void 0 ? void 0 : renderItemExtra(childProp) }, childProp)));
        }
    }), [model, renderItemExtra]);
    useEffect(() => {
        // 当外部传入的 model 变化时，同步新的 model
        if (modelProp && modelProp !== model) {
            setModel(modelProp);
        }
    }, [modelProp]);
    return (React.createElement(FormVariableProvider, { value: { disableSwitchExpressionSetter, showItemSubtitle } },
        React.createElement(FormModelProvider, { value: model },
            React.createElement(Box, { className: "SettingForm", mb: "xl", css: formStyle },
                React.createElement(Box, { className: "SettingFormHeader", bg: "white" },
                    React.createElement(Box, { px: "l", py: "m" },
                        showIdentifier && (React.createElement(FormHeader, { title: prototype.title, subTitle: React.createElement(SettingFormItem, { noStyle: true, setter: "idSetter", name: showIdentifier.identifierKey }), extra: (prototype === null || prototype === void 0 ? void 0 : prototype.docs) ? (React.createElement(Action, { icon: React.createElement(QuestionCircleOutlined, null), tooltip: "\u67E5\u770B\u7EC4\u4EF6\u6587\u6863", href: prototype.docs })) : null })),
                        showSearch && (React.createElement(Box, { className: "SettingFormSearch", mt: "m" },
                            React.createElement(Search, { placeholder: "\u641C\u7D22\u5C5E\u6027", onChange: (val) => {
                                    setKeyword(val === null || val === void 0 ? void 0 : val.trim());
                                } })))),
                    !keyword && showGroups && (React.createElement(Box, { className: "SettingFormNav" },
                        React.createElement(Tabs, { activeKey: tabKey, onChange: setTabKey }, groupOptions.map((group) => (React.createElement(Tabs.TabPane, { key: group.value, tab: group.label, disabled: !componentPropGroups[group.value].length }))))))),
                React.createElement(Box, { className: "SettingFormMain", display: "flex", flexDirection: "column", paddingBottom: "m", rowGap: "m", overflow: "auto" },
                    renderProps(filterProps),
                    filterProps.length === 0 && (React.createElement(Empty, { image: Empty.PRESENTED_IMAGE_SIMPLE, description: "\u672A\u627E\u5230\u914D\u7F6E\u9879" })))))));
}
SettingForm.register = register;
