import * as t from '@babel/types';
import { Dict } from '@music163/tango-helpers';
/**
 * 检测代码是否是合法的代码
 * @param code
 * @returns true 为合法代码，false 为非法代码
 */
export declare function isValidCode(code: string): boolean;
/**
 * 检测代码是否是合法的表达式代码
 * 表达式是一组代码的集合，它返回一个值；每一个合法的表达式都能计算成某个值
 * 只要你输入这段代码，可以形成一个值，就算是表达式
 * @example x = 1; // 1
 * @example 1=1
 * @example 'hello'
 * @example { foo: 'bar' }
 * @param code
 * @returns
 */
export declare function isValidExpressionCode(code: string): boolean;
/**
 * 将源代码解析为一棵完整的 ast 树 t.File
 * @param code
 * @returns
 */
export declare function code2ast(code: string): t.File;
/**
 * 将代码片段解析为 ast 节点
 * @example <Button>hello</Button>
 * @example { foo: 'foo' }
 * @example [{ foo: 'bar' }]
 * @example () => {}
 * @param code 输入字符串
 * @returns
 */
export declare function code2expression(code: string): import("@babel/parser").ParseResult<t.Expression>;
/**
 * 表达式代码片段转为 ast 树
 * @param code
 * @returns File
 */
export declare function expressionCode2ast(code: string): t.File;
/**
 * 将 js 值解析为 t.Node
 * @param value
 * @returns
 */
export declare function value2node(value: any): t.NullLiteral | t.Identifier | t.NumericLiteral | t.StringLiteral | t.BooleanLiteral | t.Expression;
/**
 * 将 js 普通对象解析为 t.Node
 */
export declare function object2node(obj: object, getValueNode?: (value: any, key?: string) => t.Expression): t.Expression;
export declare function code2jsxAttributeValueNode(code: string): t.JSXExpressionContainer;
export declare function value2jsxAttributeValueNode(value: any): t.JSXExpressionContainer | t.StringLiteral;
export declare function value2jsxChildrenValueNode(value: any): (t.JSXElement | t.JSXExpressionContainer | t.JSXFragment | t.JSXSpreadChild | t.JSXText)[];
/**
 * 给定具体的 value 值，生成 JSXAttribute
 * @param name 属性名
 * @param value 属性值代码
 * @returns
 */
export declare function makeJSXAttribute(name: string, value: any): t.JSXAttribute;
export declare function makeJSXAttributes(props: Dict): t.JSXAttribute[];
/**
 * 给定具体的 value 代码，生成 JSXAttribute
 * @param name 属性名
 * @param valueCode 属性值代码
 * @returns
 */
export declare function makeJSXAttributeByCode(name: string, valueCode: string): t.JSXAttribute;
/**
 * 生成 JSXElement
 * @param name
 * @param attributes
 * @param children
 * @param selfClosing
 * @returns
 */
export declare function makeJSXElement(name: string, attributes: t.JSXAttribute[], children: t.JSXElement['children'], selfClosing: boolean): t.JSXElement;
