/// <reference types="react" />
import { TangoEventName } from './enums';
type EventListenerCallbackType = (event: any) => void;
export declare const events: {
    /**
     * 绑定事件监听器
     * @param node HTML 元素
     * @param eventName 事件名
     * @param callback 监听器函数
     * @param useCapture 是否开启事件捕获
     * @returns { off } off 用于快速取消事件监听
     */
    on(node: HTMLElement | Document, eventName: string, callback: EventListenerCallbackType, useCapture?: boolean): {
        off: () => void;
    };
    /**
     * 取消事件监听器
     * @param node HTML 元素
     * @param eventName 事件名
     * @param callback 事件监听器
     * @param useCapture 是否开启事件捕获
     */
    off(node: HTMLElement | Document, eventName: string, callback: EventListenerCallbackType, useCapture?: boolean): void;
};
/**
 * 触发 tangoEvent
 * @param element 发起事件的元素
 * @param eventName 事件名
 * @param eventPayload 携带的参数
 */
export declare function dispatchTangoEvent<E extends EventTarget, P = any>(element: E, eventName: TangoEventName, eventPayload?: P): void;
/**
 * 获取按键的快捷信息
 */
export declare function getHotkey(e: React.KeyboardEvent): string;
export {};
