import { FileType } from './../types';
/**
 * 推断JS模块类型
 */
export declare function inferFileType(filename: string): FileType;
/**
 * 判断组件名是否合法
 * @example Button -> valid
 * @example isStrict ? div -> invalid : div -> valid
 * @param name
 * @param isStrict 是否严格模式（严格模式不匹配原生标签）
 * @returns
 */
export declare function isValidComponentName(name: string, isStrict?: boolean): boolean;
/**
 * 从 filename 中解析获得 moduleName
 * @example /stores/user.js -> user
 * @example /services/foo-bar.js -> fooBar
 * @param filename
 */
export declare function getModuleNameByFilename(filename: string): string;
/**
 * 基于路由名生成文件路径
 * @param routePath 路由地址
 * @param baseDir base dir
 * @param ext 后缀名
 * @returns
 */
export declare function getFilepath(routePath: string, baseDir: string, ext?: string): string;
export declare function getPrivilegeCode(appName: string, routePath: string): string;
/**
 * 格式化代码
 * @param code original source code
 * @param parser prettier parser, see https://prettier.io/docs/en/options.html#parser
 * @returns the formatted code
 */
export declare function formatCode(code: string, parser?: string): any;
/**
 * 是否匹配路由
 * @example isPathnameMatchRoute('/user/123', '/users/:id') -> true
 */
export declare function isPathnameMatchRoute(pathname: string, route: string): boolean;
/**
 * 根据文件路径计算区块的名字
 * @param filename 文件路径
 * @return 返回计算后的区块名（大驼峰）
 */
export declare function getBlockNameByFilename(filename: string): string;
/**
 * 合并两个路径
 * @param root
 * @param filename
 * @returns
 */
export declare function getFullPath(root: string, filename: string): string;
/**
 * 计算 targetFile 在 sourceFile 中的相对引用路径
 * @param sourceFile
 * @param targetFile
 * @returns
 */
export declare function getRelativePath(sourceFile: string, targetFile: string): string;
/**
 * 判断给定字符串是否是文件路径
 * @example ./pages/index.js -- yes
 * @example ../pages/index.js -- yes
 * @example ../components -- yes
 * @example /src/pages/index.js -- yes
 * @example @music163/tango-designer -- no
 * @param str
 */
export declare function isFilepath(str: string): boolean;
