import { __rest } from "tslib";
import React, { forwardRef } from 'react';
import hoistNonReactStatic from 'hoist-non-react-statics';
import { SLOT, getDisplayName, wrapDisplayName, isFunctionComponent } from '../helpers';
const renderEmpty = () => null;
export function withDnd(options) {
    return function hoc(BaseComponent) {
        var _a, _b;
        const isFC = isFunctionComponent(BaseComponent);
        const displayName = options.name || getDisplayName(BaseComponent);
        const draggable = (_a = options.draggable) !== null && _a !== void 0 ? _a : true;
        const hasWrapper = (_b = options.hasWrapper) !== null && _b !== void 0 ? _b : true;
        const display = options.display || 'block';
        const override = options.overrideProps || {};
        const renderFooter = options.renderFooter || renderEmpty;
        const Component = forwardRef((props, refProp) => {
            const dndProps = {
                [SLOT.id]: props.tid, // id 作为唯一标记
                [SLOT.dnd]: props[SLOT.dnd], // dnd 作为追踪标记
                draggable: draggable ? true : undefined,
            };
            const propsCopy = Object.assign({}, props);
            delete propsCopy[SLOT.dnd];
            delete propsCopy.tid;
            const ref = isFC ? undefined : refProp;
            if (!hasWrapper) {
                return React.createElement(BaseComponent, Object.assign({ ref: ref }, dndProps, override, propsCopy));
            }
            return (React.createElement(DndBox, Object.assign({ name: displayName, display: display, style: options.wrapperStyle }, dndProps),
                React.createElement(BaseComponent, Object.assign({ ref: ref }, override, propsCopy)),
                renderFooter(props)));
        });
        hoistNonReactStatic(Component, BaseComponent);
        Component.displayName = wrapDisplayName(displayName, 'withDnd');
        return Component;
    };
}
function DndBox(_a) {
    var { name, display, children, style: styleProp } = _a, rest = __rest(_a, ["name", "display", "children", "style"]);
    const style = Object.assign({ display, minHeight: 4 }, styleProp);
    return (React.createElement("div", Object.assign({ className: `${name}-designer tango-dndBox`, style: style }, rest), children));
}
