import { useCallback, useMemo, useState } from 'react';
import { useCallbackRef } from './use-callback-ref';
export function useControllableProp(prop, state) {
    const controlled = typeof prop !== 'undefined';
    const value = controlled ? prop : state;
    return useMemo(() => [controlled, value], [controlled, value]);
}
export function useControllableState(props) {
    const { value: valueProp, defaultValue, onChange, shouldUpdate = (prev, next) => prev !== next, } = props;
    const onChangeProp = useCallbackRef(onChange);
    const shouldUpdateProp = useCallbackRef(shouldUpdate);
    const [uncontrolledState, setUncontrolledState] = useState(defaultValue);
    const controlled = valueProp !== undefined;
    const value = controlled ? valueProp : uncontrolledState;
    const setValue = useCallback((next) => {
        const setter = next;
        const nextValue = typeof next === 'function' ? setter(value) : next;
        if (!shouldUpdateProp(value, nextValue)) {
            return;
        }
        if (!controlled) {
            setUncontrolledState(nextValue);
        }
        onChangeProp(nextValue);
    }, [controlled, onChangeProp, value, shouldUpdateProp]);
    return [value, setValue];
}
