import React, { useState } from 'react';
import { Box } from 'coral-system';
import { Button, Input, Tooltip } from 'antd';
import { DeleteOutlined, PlusOutlined } from '@ant-design/icons';
export function InputList({ value = [], onChange }) {
    const [isAdding, setIsAdding] = useState(false);
    return (React.createElement(Box, null,
        !value.length && !isAdding && React.createElement(Box, { color: "text.note" }, "\u7A7A"),
        React.createElement(Box, { display: "flex", flexDirection: "column", gap: "m" },
            value.map((item, index) => (React.createElement(InputListItem, { key: index, value: item, onBlur: (newItem) => {
                    if (newItem === item)
                        return;
                    const newValue = [...value];
                    newValue[index] = newItem;
                    onChange === null || onChange === void 0 ? void 0 : onChange(newValue);
                }, onRemove: () => {
                    const newValue = [...value];
                    newValue.splice(index, 1);
                    onChange === null || onChange === void 0 ? void 0 : onChange(newValue);
                } }))),
            isAdding && (React.createElement(InputListItem, { onBlur: (val) => {
                    if (!val)
                        return;
                    onChange === null || onChange === void 0 ? void 0 : onChange(value.concat([val]));
                    setIsAdding(false);
                }, onRemove: () => {
                    setIsAdding(false);
                } }))),
        React.createElement(Box, { mt: "m" },
            React.createElement(Button, { icon: React.createElement(PlusOutlined, null), onClick: () => setIsAdding(true) }, "\u65B0\u589E"))));
}
function InputListItem({ value: valueProp, onChange, onBlur, onRemove, }) {
    const [value, setValue] = useState(valueProp);
    return (React.createElement(Box, { display: "flex", gap: "m" },
        React.createElement(Input, { placeholder: "\u8BF7\u8F93\u5165", value: value, onChange: (e) => {
                setValue(e.target.value);
                onChange === null || onChange === void 0 ? void 0 : onChange(e.target.value);
            }, onBlur: () => onBlur === null || onBlur === void 0 ? void 0 : onBlur(value) }),
        React.createElement(Tooltip, { title: "\u5220\u9664" },
            React.createElement(Button, { type: "text", icon: React.createElement(DeleteOutlined, null), onClick: onRemove }))));
}
