import zhLocale from './locales/zh-cn';
import enLocale from './locales/en-us';
import { useState } from 'react';
const zh_CN = 'zh-CN';
const en_US = 'en-US';
const DEFAULT_LOCALE = zh_CN;
/**
 * 获取antd的语言资源映射关系
 * @param param0 antd的语言资源
 * @returns
 */
export const getAntdLocalesMap = ({ zhCN, enUS }) => ({
    [zh_CN]: zhCN,
    [en_US]: enUS,
});
const locales = {
    [zh_CN]: zhLocale,
    [en_US]: enLocale,
};
/**
 * 单例模式的多语言管理类
 */
export class LocaleManager {
    static getInstance(currentLocale) {
        if (!currentLocale && !LocaleManager.instance)
            return null;
        LocaleManager.instance = LocaleManager.instance || new LocaleManager(currentLocale);
        return LocaleManager.instance;
    }
    constructor(currentLocale) {
        this.currentLocale = DEFAULT_LOCALE;
        if (LocaleManager.instance) {
            throw new Error('Error - use LocaleManager.getInstance()');
        }
        if (currentLocale) {
            this.currentLocale = currentLocale;
        }
    }
    changeLocale(currentLocale) {
        this.currentLocale = currentLocale;
    }
    getLocale() {
        return locales[this.currentLocale];
    }
}
export const useLocaleManager = (locale = DEFAULT_LOCALE) => {
    const [localeManagerInstance] = useState(LocaleManager.getInstance(locale));
    return localeManagerInstance;
};
