import React from 'react';
import cx from 'classnames';
import { css, Box, coral, Text } from 'coral-system';
import { EditOutlined, DeleteOutlined } from '@ant-design/icons';
import { noop } from '@music163/tango-helpers';
import { Popconfirm } from 'antd';
const menuStyle = css `
  font-family: Consolas, Menlo, Courier, monospace;
  font-size: 14px;
`;
const MenuItem = coral('div', css `
    display: flex;
    justify-content: space-between;
    align-items: center;
    user-select: none;
    height: 28px;
    padding: 0 12px;
    cursor: pointer;

    button {
      display: none;
      background: transparent;
      border: 0;
      cursor: pointer;
    }

    .tango-menu-label {
      overflow: hidden;
      text-overflow: ellipsis;
      white-space: nowrap;
      max-width: 90%;
    }

    &:hover button {
      display: inline-block;
    }

    &:hover {
      background-color: var(--tango-colors-gray-10);
    }

    &.active {
      background-color: var(--tango-colors-primary-10);
    }
  `);
export function Menu({ items = [], activeKey, onItemClick = noop, onEdit = noop, onDelete = noop, }) {
    return (React.createElement(Box, { className: "tango-menu", css: menuStyle }, items.map((item) => (React.createElement(MenuItem, { key: item.key, className: cx({ active: activeKey === item.key }), onClick: () => onItemClick(item.key) },
        React.createElement("div", { className: "tango-menu-label" },
            React.createElement(Text, { color: "text.body", title: item.label }, item.label),
            React.createElement(Text, { ml: "m", color: "text.placeholder" }, item.note)),
        React.createElement("div", { className: "tango-menu-actions" },
            item.editable && (React.createElement("button", { onClick: () => onEdit(item.key) },
                React.createElement(EditOutlined, null))),
            item.deletable && (React.createElement(Popconfirm, { title: item.deletableConfirm || '确定执行此操作吗？', okText: "\u786E\u5B9A", cancelText: "\u53D6\u6D88", onConfirm: (e) => {
                    // 阻止冒泡，否则删除的时候会有 Bug
                    e.stopPropagation();
                    onDelete(item.key);
                } },
                React.createElement("button", null,
                    React.createElement(DeleteOutlined, null))))))))));
}
