import type { FileType, IFileConfig } from '../types';
import { IWorkspace } from './interfaces';
/**
 * 普通文件，不进行 AST 解析
 */
export declare class TangoFile {
    readonly workspace: IWorkspace;
    /**
     * 文件名
     */
    readonly filename: string;
    /**
     * 文件类型
     */
    readonly type: FileType;
    /**
     * 最近修改的时间戳
     */
    lastModified: number;
    _code: string;
    _cleanCode: string;
    get code(): string;
    get cleanCode(): string;
    constructor(workspace: IWorkspace, props: IFileConfig, isSyncCode?: boolean);
    /**
     * 更新文件内容
     */
    update(code?: string): void;
}
export declare class TangoLessFile extends TangoFile {
    constructor(workspace: IWorkspace, props: IFileConfig);
}
export declare class TangoJsonFile extends TangoFile {
    _object: {};
    /**
     * @deprecated 使用 file.json 代替
     */
    get object(): {};
    get json(): {};
    constructor(workspace: IWorkspace, props: IFileConfig);
    update(code?: string): void;
    /**
     * 根据路径取值
     * @param valuePath
     * @returns
     */
    getValue(valuePath: string): any;
    /**
     * 根据路径设置值
     * @param valuePath
     * @param visitor
     */
    setValue(valuePath: string, visitor: (targetValue: any) => any): this;
    /**
     * 根据路径删除值
     * @param valuePath
     * @param visitor
     */
    deleteValue(valuePath: string): this;
}
