import { IWorkspace } from './interfaces';
export declare enum HistoryMessage {
    InitView = "initView",
    AddFile = "addFile",
    RemoveFile = "removeFile",
    UpdateDependency = "updateDependency",
    RemoveDependency = "removeDependency",
    RemoveNode = "removeNode",
    ReplaceNode = "replaceNode",
    CloneNode = "cloneNode",
    InsertNode = "insertNode",
    InsertBeforeNode = "insertBeforeNode",
    InsertAfterNode = "insertAfterNode",
    DropNode = "dropNode",
    UpdateAttribute = "updateAttribute",
    UpdateCode = "updateCode"
}
type HistoryRecordData = {
    [filename: string]: string;
};
interface HistoryRecord {
    time: number;
    message: HistoryMessage;
    data: HistoryRecordData;
}
type PushDataType = Pick<HistoryRecord, 'message' | 'data'>;
/**
 * 工作区的历史记录记录
 */
export declare class TangoHistory {
    _records: HistoryRecord[];
    _index: number;
    _maxSize: number;
    private readonly workspace;
    get index(): number;
    get length(): number;
    get list(): HistoryRecord[];
    get couldBack(): boolean;
    get couldForward(): boolean;
    constructor(workspace: IWorkspace);
    _sync(data: HistoryRecordData): void;
    /**
     * 上一步
     */
    back(): void;
    /**
     * 下一步
     */
    forward(): void;
    /**
     * 通过相对位置从历史记录加载记录
     */
    go(index: number): void;
    /**
     * push 数据进入历史记录堆栈
     */
    push(data: PushDataType): void;
}
export {};
