import { action, computed, makeObservable, observable, toJS } from 'mobx';
import { clone, logger } from '@music163/tango-helpers';
import { traverseServiceFile, updateServiceConfigToServiceFile, getModuleNameByFilename, deleteServiceConfigFromServiceFile, updateBaseConfigToServiceFile, } from '../helpers';
import { TangoModule } from './module';
/**
 * 数据服务模块
 */
export class TangoServiceModule extends TangoModule {
    get serviceFunctions() {
        return toJS(this._serviceFunctions);
    }
    get baseConfig() {
        return toJS(this._baseConfig);
    }
    constructor(workspace, props) {
        super(workspace, props, false);
        this.name = getModuleNameByFilename(props.filename);
        this.update(props.code, true, false);
        makeObservable(this, {
            _serviceFunctions: observable,
            _baseConfig: observable,
            _code: observable,
            _cleanCode: observable,
            serviceFunctions: computed,
            baseConfig: computed,
            cleanCode: computed,
            code: computed,
            update: action,
        });
    }
    _analysisAst() {
        const { imports, services, baseConfig } = traverseServiceFile(this.ast);
        this.importList = imports;
        this._serviceFunctions = services;
        this._baseConfig = baseConfig;
        if (baseConfig.namespace) {
            this.name = baseConfig.namespace;
        }
    }
    addServiceFunction(name, config) {
        this.ast = updateServiceConfigToServiceFile(this.ast, { [name]: clone(config, false) });
        return this;
    }
    /**
     * 批量添加服务函数
     * @param configs { [name: string]: object }
     * @returns
     */
    addServiceFunctions(configs) {
        this.ast = updateServiceConfigToServiceFile(this.ast, configs);
        return this;
    }
    updateServiceFunction(name, payload) {
        this.ast = updateServiceConfigToServiceFile(this.ast, { [name]: clone(payload, false) });
        return this;
    }
    deleteServiceFunction(serviceFunctionName) {
        try {
            this.ast = deleteServiceConfigFromServiceFile(this.ast, serviceFunctionName);
        }
        catch (e) {
            logger.error(e);
        }
        return this;
    }
    /**
     * 更新服务的基础配置
     */
    updateBaseConfig(data) {
        this.ast = updateBaseConfigToServiceFile(this.ast, data);
        return this;
    }
}
