import * as t from '@babel/types';
import { Dict } from '@music163/tango-helpers';
import { IdGenerator } from '../helpers';
import { TangoNode } from './node';
import { IFileConfig, ITangoViewNodeData, IImportDeclarationPayload, InsertChildPositionType, IImportSpecifierSourceData, IImportSpecifierData } from '../types';
import { IViewFile, IWorkspace } from './interfaces';
import { TangoModule } from './module';
/**
 * 视图模块
 */
export declare class TangoViewModule extends TangoModule implements IViewFile {
    _nodesTree: ITangoViewNodeData[];
    /**
     * 通过导入组件名查找组件来自的包
     */
    importMap: Dict<IImportSpecifierSourceData>;
    /**
     * 视图中依赖的 tango 变量，仅 stores 和 services
     */
    variables: string[];
    /**
     * ID 生成器
     */
    idGenerator: IdGenerator;
    /**
     * codeId 列表
     */
    private _codeIdList;
    /**
     * 节点列表 <id, Node>
     */
    private _nodes;
    /**
     * 导入的模块
     * @deprecated
     */
    private _importedModules;
    get nodes(): Map<string, TangoNode>;
    get nodesTree(): ITangoViewNodeData<t.JSXElement>[];
    get tree(): t.File;
    constructor(workspace: IWorkspace, props: IFileConfig);
    _syncByAst(): void;
    _analysisAst(): void;
    hasNodeByCodeId(codeId: string): boolean;
    /**
     * 依赖列表
     */
    listImportSources(): string[];
    /**
     * 弹窗列表
     */
    listModals(): {
        label: string;
        value: string;
    }[];
    /**
     * 表单列表
     */
    listForms(): Record<string, string[]>;
    /**
     * 添加导入符号
     * @param source
     * @param newSpecifiers
     * @returns
     */
    addImportSpecifiers(source: string, newSpecifiers: IImportSpecifierData[]): this;
    /**
     * 更新导入的变量
     * @deprecated 使用 updateImportDeclaration 代替
     */
    updateImportSpecifiersLegacy(importDeclaration: IImportDeclarationPayload): this;
    /**
     * 清除无效的导入声明
     */
    removeUnusedImportSpecifiers(): this;
    getNode(nodeId: string): TangoNode;
    /**
     * 删除节点
     * @param nodeId
     */
    removeNode(nodeId: string): this;
    /**
     * 更新节点的属性
     * @deprecated 使用 updateNodeAttributes 代替
     */
    updateNodeAttribute(nodeId: string, attrName: string, attrValue?: any, relatedImports?: string[]): this;
    updateNodeAttributes(nodeId: string, config: Record<string, any>, relatedImports?: string[]): this;
    /**
     * 插入子节点的最后面
     * @param targetNodeId
     * @param newNode
     * @param position
     * @returns
     */
    insertChild(targetNodeId: string, newNode: t.JSXElement, position?: InsertChildPositionType): this;
    insertAfter(targetNodeId: string, newNode: t.JSXElement): this;
    insertBefore(targetNodeId: string, newNode: t.JSXElement): this;
    /**
     * 替换目标节点为新节点
     * @param targetNodeId
     * @param newNode
     */
    replaceNode(targetNodeId: string, newNode: t.JSXElement): this;
    /**
     * 替换 jsx 跟结点的子元素
     * @deprecated 不推荐使用
     */
    replaceViewChildren(childrenNodes: t.JSXElement[], importDeclarations?: IImportDeclarationPayload[]): this;
}
