import React from 'react';
import ReactDOM from 'react-dom';
import { Router } from 'react-router';
import { renderRoutes } from 'react-router-config';
import { createHistory } from './history';
import i18n from './i18n';
import { I18nextProvider } from 'react-i18next';
export function runApp(config) {
    if (window.__POWERED_BY_QIANKUN__ && config.boot.qiankun) {
        return runQiankunApp(config);
    }
    else {
        runReactApp(config);
    }
    return {};
}
function runReactApp(config) {
    var _a, _b, _c, _d, _e;
    let element;
    const routes = (_b = (_a = config.router) === null || _a === void 0 ? void 0 : _a.config) !== null && _b !== void 0 ? _b : config.routes;
    if (routes) {
        // react router app
        const routerType = (_d = (_c = config.router) === null || _c === void 0 ? void 0 : _c.type) !== null && _d !== void 0 ? _d : 'hash';
        const history = createHistory(routerType, routes);
        element = React.createElement(ReactRouterApp, { history: history, routes: routes, layout: (_e = config.router) === null || _e === void 0 ? void 0 : _e.layout });
    }
    else {
        // single entry app
        const SingleEntry = config.singleEntry || 'div';
        element = React.createElement(SingleEntry, null);
    }
    // eslint-disable-next-line react/no-deprecated
    ReactDOM.render(React.createElement(I18nextProvider, { i18n: i18n },
        React.createElement(AppContainer, { providers: config.providers }, element)), config.boot.mountElement);
}
function runQiankunApp(config) {
    // FIXME: 需要支持从外层传入 mountId
    console.log('runQiankunApp:: ', config.router.config);
    const mountId = '#root';
    return {
        bootstrap() {
            return Promise.resolve({});
        },
        mount(props) {
            var _a;
            console.log('mount run, ', props);
            const target = (_a = props.container) !== null && _a !== void 0 ? _a : document;
            runReactApp(Object.assign(Object.assign({}, config), { router: Object.assign(Object.assign({}, config.router), { config: config.router.config.map((route) => {
                        var _a;
                        return Object.assign(Object.assign({}, route), { path: (_a = route.path) === null || _a === void 0 ? void 0 : _a.replace(/^\/portals/, '/site/home') });
                    }) }), boot: Object.assign(Object.assign({}, config.boot), { mountElement: target.querySelector(mountId) }) }));
        },
        unmount(props) {
            var _a;
            console.log('unmount run, ', props);
            const target = (_a = props.container) !== null && _a !== void 0 ? _a : document;
            // eslint-disable-next-line react/no-deprecated
            ReactDOM.unmountComponentAtNode(target.querySelector(mountId));
        },
    };
}
function AppContainer({ children: childrenProp, providers = [] }) {
    let children = childrenProp;
    if (providers && providers.length) {
        children = providers.reduce((prev, provider) => {
            if (React.isValidElement(provider)) {
                return React.cloneElement(provider, {}, prev);
            }
            return prev;
        }, childrenProp);
    }
    return React.createElement(React.Fragment, null, children);
}
function ReactRouterApp({ history, routes = [], layout }) {
    if (!routes.find((route) => !route.path)) {
        // 如果用户没有定义 404 路由，则自动添加一个
        routes.push({
            component: NotFound,
        });
    }
    const elements = React.createElement(Router, { history: history }, renderRoutes(routes));
    if (!layout)
        return elements;
    return React.createElement(layout, {}, elements);
}
function NotFound() {
    return React.createElement("div", null, "not found");
}
