import React from 'react';
import { Box, css, Group } from 'coral-system';
import { Action, ToggleButton } from '@music163/tango-ui';
import { observer, useDesigner } from '@music163/tango-context';
import { Input } from 'antd';
import { DesktopOutlined, MobileOutlined, ReloadOutlined, ArrowLeftOutlined, ArrowRightOutlined, } from '@ant-design/icons';
const navigatorStyle = css `
  display: flex;
  column-gap: 12px;

  .navigatorInput {
    flex: 1;
    border-color: transparent;
    background-color: var(--tango-colors-fill2);

    &:focus {
      background-color: #fff;
    }
  }
`;
export class Navigator extends React.Component {
    constructor(props) {
        super(props);
        this.changeRelativeUrl = (newUrl) => {
            if (newUrl !== this.state.relativeUrl) {
                this.setState({
                    relativeUrl: newUrl,
                });
            }
        };
        this.onInputChange = (e) => {
            const path = e.target.value.startsWith('/') ? e.target.value : `/${e.target.value}`;
            this.changeRelativeUrl(path);
        };
        this.onPressEnter = (e) => {
            var _a, _b;
            const newUrl = e.currentTarget.value;
            (_b = (_a = this.props).onInputEnter) === null || _b === void 0 ? void 0 : _b.call(_a, newUrl, e);
        };
        this.state = {
            relativeUrl: props.startRoute || '/',
        };
    }
    componentDidUpdate(prevProps, prevState) {
        if (this.props.startRoute &&
            prevProps.startRoute !== this.props.startRoute &&
            this.props.startRoute !== prevState.relativeUrl) {
            this.setState({
                relativeUrl: this.props.startRoute,
            });
        }
    }
    render() {
        const { disabled, onBack, onForward, onRefresh, extra } = this.props;
        return (React.createElement(Box, { className: "navigator", bg: "white", px: "l", py: "m", borderBottom: "solid", borderBottomColor: "line.normal", css: navigatorStyle },
            React.createElement(Box, null,
                React.createElement(Action, { size: "small", icon: React.createElement(ArrowLeftOutlined, null), onClick: onBack, title: "\u8FD4\u56DE" }),
                React.createElement(Action, { size: "small", icon: React.createElement(ArrowRightOutlined, null), onClick: onForward, title: "\u524D\u8FDB" }),
                React.createElement(Action, { size: "small", icon: React.createElement(ReloadOutlined, null), onClick: onRefresh, title: "\u5237\u65B0" })),
            React.createElement(Input, { className: "navigatorInput", size: "small", value: this.state.relativeUrl, onChange: this.onInputChange, onPressEnter: this.onPressEnter, disabled: disabled }),
            React.createElement(ViewportSwitch, null),
            extra));
    }
}
const ViewportSwitch = observer(() => {
    const designer = useDesigner();
    return (React.createElement(Group, { attached: true },
        React.createElement(ToggleButton, { size: "s", selected: designer.simulator.name === 'desktop', onClick: () => {
                designer.setSimulator('desktop');
            } },
            React.createElement(DesktopOutlined, null)),
        React.createElement(ToggleButton, { size: "s", selected: designer.simulator.name === 'phone', onClick: () => {
                designer.setSimulator('phone');
            } },
            React.createElement(MobileOutlined, null))));
});
