import React, { useState } from 'react';
import { Box } from 'coral-system';
import { Button, Select, Tooltip } from 'antd';
import { DeleteOutlined, PlusOutlined } from '@ant-design/icons';
export function SelectList({ value = [], onChange, options = [], isUniqueValue = false, }) {
    const [isAdding, setIsAdding] = useState(false);
    if (isUniqueValue) {
        options = options.filter((option) => value.indexOf(option.value) < 0);
    }
    return (React.createElement(Box, null,
        !value.length && !isAdding && React.createElement(Box, { color: "text.note" }, "\u7A7A"),
        React.createElement(Box, { display: "flex", flexDirection: "column", gap: "m" },
            value.map((item, index) => (React.createElement(SelectListItem, { key: index, options: options, value: item, onChange: (newItem) => {
                    if (newItem === item)
                        return;
                    const newValue = [...value];
                    newValue[index] = newItem;
                    onChange === null || onChange === void 0 ? void 0 : onChange(newValue);
                }, onRemove: () => {
                    const newValue = [...value];
                    newValue.splice(index, 1);
                    onChange === null || onChange === void 0 ? void 0 : onChange(newValue);
                } }))),
            isAdding && (React.createElement(SelectListItem, { options: options, onChange: (val) => {
                    if (!val)
                        return;
                    onChange === null || onChange === void 0 ? void 0 : onChange(value.concat([val]));
                    setIsAdding(false);
                }, onRemove: () => {
                    setIsAdding(false);
                } }))),
        React.createElement(Box, { mt: "m" },
            React.createElement(Button, { icon: React.createElement(PlusOutlined, null), onClick: () => setIsAdding(true) }, "\u65B0\u589E"))));
}
const selectStyle = {
    flex: 1,
};
function SelectListItem({ value, onChange, onRemove, options, }) {
    return (React.createElement(Box, { display: "flex", gap: "m" },
        React.createElement(Select, { placeholder: "\u8BF7\u9009\u62E9", options: options, value: value, onChange: (val) => {
                onChange === null || onChange === void 0 ? void 0 : onChange(val);
            }, style: selectStyle }),
        React.createElement(Tooltip, { title: "\u5220\u9664" },
            React.createElement(Button, { type: "text", icon: React.createElement(DeleteOutlined, null), onClick: onRemove }))));
}
