import { __rest } from "tslib";
import React, { useEffect, useState } from 'react';
import { Input } from 'antd';
import { InputCode } from '@music163/tango-ui';
import { register } from './form-item';
import { Box, css } from 'coral-system';
export function ExpressionSetter(_a) {
    var { value, onChange } = _a, rest = __rest(_a, ["value", "onChange"]);
    return React.createElement(InputCode, Object.assign({ onChange: (val) => onChange === null || onChange === void 0 ? void 0 : onChange(val), value: value || '' }, rest));
}
export function TextSetter(_a) {
    var { onChange } = _a, rest = __rest(_a, ["onChange"]);
    return React.createElement(Input, Object.assign({ onChange: (e) => onChange === null || onChange === void 0 ? void 0 : onChange(e.target.value) }, rest));
}
const idInputStyle = css `
  > .ant-input-borderless {
    border: 1px solid transparent;
    padding: 4px;
  }

  > .ant-input-borderless:hover {
    background-color: var(--tango-colors-fill1);
  }
`;
const idPattern = /^[a-z]+[\w]*$/;
export function IdSetter(_a) {
    var { value: valueProp, defaultValue, onChange, placeholder = '输入唯一组件ID' } = _a, rest = __rest(_a, ["value", "defaultValue", "onChange", "placeholder"]);
    const [value, setValue] = useState(valueProp || defaultValue);
    const [error, setError] = useState('');
    const [editable, setEditable] = useState(false);
    // value controlled
    useEffect(() => {
        setValue(valueProp);
    }, [valueProp]);
    const __props = editable
        ? {
            status: error ? 'error' : undefined,
            onBlur() {
                setEditable(false);
                if (!error) {
                    onChange === null || onChange === void 0 ? void 0 : onChange(value || undefined);
                }
            },
            onChange(e) {
                const newValue = e.target.value;
                setValue(e.target.value);
                setError(newValue && !idPattern.test(newValue) ? 'error' : '');
            },
        }
        : {
            readOnly: true,
            bordered: false,
            onClick() {
                setEditable(true);
            },
        };
    return (React.createElement(Box, { css: idInputStyle },
        React.createElement(Input, Object.assign({ placeholder: placeholder, value: value }, __props, rest))));
}
export function registerBuiltinSetters() {
    // 预注册基础 Setter
    register({
        name: 'expressionSetter',
        component: ExpressionSetter,
        disableVariableSetter: true,
    });
    register({
        name: 'textSetter',
        component: TextSetter,
    });
    register({
        name: 'idSetter',
        component: IdSetter,
    });
}
