import React, { useCallback, useEffect, useState } from 'react';
import { Box } from 'coral-system';
// @ts-ignore
import { toJSON } from 'cssjson';
import { InputNumber, Slider, Space } from 'antd';
import { BgSetter, BorderSetter, ColorSetter, CssCodeSetter, DisplaySetter, FlexAlignItemsSetter, FlexDirectionSetter, FlexJustifyContentSetter, SpacingSetter, } from './style-setter';
import { wrapCode } from '@music163/tango-helpers';
const getRawCssValue = (value) => {
    if (value && value.startsWith('{{css`')) {
        return value.split('').slice(6, -3).join('').trim();
    }
    return value;
};
const getPxValue = (value) => {
    const match = pxPattern.exec(value);
    if (match && match.length) {
        return match[1];
    }
    return '0';
};
const pxPattern = /^(\d+)px$/; // px校验
// 动态正则
const cssPattern = (string, name) => {
    // FIXME: check this pattern
    // eslint-disable-next-line no-useless-escape
    const re1 = new RegExp(`(?<=\s*?\n*?;?\s*?\n*?${name}\s*).*?(?=;)`, 'g');
    // eslint-disable-next-line no-useless-escape
    const re2 = new RegExp(`(?<=^[\s]*?${name}).*?(?=;)`, 'g');
    if (re1.test(string)) {
        return re1;
    }
    else if (re2.test(string)) {
        return re2;
    }
    else {
        return false;
    }
};
/**
 * coral-system css prop
 * @example 提供 css-in-js 代码支持，例如 css`background: red;`
 * @deprecated 使用嵌套属性代替
 */
export function CssSetter(props) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
    const { value: valueProp, onChange } = props;
    const [contentValue, setContentValue] = useState(() => getRawCssValue(valueProp));
    const [display, setDisplay] = useState('');
    // 修改指定的CSS样式
    const changeStyle = useCallback((value, name) => {
        if (!contentValue) {
            setContentValue((it) => (it ? `${it}\n${name}: ${value};` : `\n${name}: ${value};`));
        }
        else if (cssPattern(contentValue, name)) {
            const str = contentValue.replace(cssPattern(contentValue, name), `:${value}`);
            setContentValue(str);
        }
        else {
            setContentValue((it) => (it ? `${it}\n${name}: ${value};` : `\n${name}: ${value};`));
        }
    }, [contentValue]);
    // 代码编辑器
    const codeEditorChange = useCallback((value) => {
        const val = getRawCssValue(value);
        setContentValue(val);
    }, []);
    useEffect(() => {
        if (contentValue || contentValue === '') {
            onChange(wrapCode(`css\`${contentValue}\``));
        }
    }, [contentValue]);
    const isFlex = ['flex', 'inline-flex'].includes(display);
    return (React.createElement(Box, { bg: "fill1" },
        React.createElement(ItemGroup, { title: "\u5E03\u5C40\u65B9\u5F0F" },
            React.createElement(DisplaySetter, { value: (_a = toJSON(contentValue).attributes) === null || _a === void 0 ? void 0 : _a.display, onChange: (v) => {
                    setDisplay(v);
                    changeStyle(v, 'display');
                } })),
        isFlex ? (React.createElement(React.Fragment, null,
            React.createElement(ItemGroup, { title: "\u4E3B\u8F74\u65B9\u5411" },
                React.createElement(FlexDirectionSetter, { value: (_b = toJSON(contentValue).attributes) === null || _b === void 0 ? void 0 : _b['flex-direction'], onChange: (v) => {
                        changeStyle(v, 'flex-direction');
                    } })),
            React.createElement(ItemGroup, { title: "\u4E3B\u8F74\u5BF9\u9F50\u65B9\u5F0F" },
                React.createElement(FlexJustifyContentSetter, { value: (_c = toJSON(contentValue).attributes) === null || _c === void 0 ? void 0 : _c['justify-content'], onChange: (v) => {
                        changeStyle(v, 'justify-content');
                    } })),
            React.createElement(ItemGroup, { title: "\u8F85\u8F74\u5BF9\u9F50\u65B9\u5F0F" },
                React.createElement(FlexAlignItemsSetter, { value: (_d = toJSON(contentValue).attributes) === null || _d === void 0 ? void 0 : _d['align-items'], onChange: (v) => {
                        changeStyle(v, 'align-items');
                    } })))) : null,
        React.createElement(ItemGroup, { title: "\u5916\u8FB9\u8DDD" },
            React.createElement(SpacingSetter, { value: (_e = toJSON(contentValue).attributes) === null || _e === void 0 ? void 0 : _e.margin, onChange: (v) => {
                    changeStyle(v, 'margin');
                } })),
        React.createElement(ItemGroup, { title: "\u5185\u8FB9\u8DDD" },
            React.createElement(SpacingSetter, { value: (_f = toJSON(contentValue).attributes) === null || _f === void 0 ? void 0 : _f.padding, onChange: (v) => {
                    changeStyle(v, 'padding');
                } })),
        React.createElement(ItemGroup, { title: "\u5BBD\u9AD8" },
            React.createElement(Space, null,
                React.createElement(InputNumber, { addonBefore: "\u5BBD\u5EA6", addonAfter: "px", defaultValue: getPxValue((_g = toJSON(contentValue).attributes) === null || _g === void 0 ? void 0 : _g.width), onChange: (v) => {
                        changeStyle(`${v}px`, 'width');
                    } }),
                React.createElement(InputNumber, { addonBefore: "\u9AD8\u5EA6", addonAfter: "px", defaultValue: getPxValue((_h = toJSON(contentValue).attributes) === null || _h === void 0 ? void 0 : _h.height), onChange: (v) => {
                        changeStyle(`${v}px`, 'height');
                    } }))),
        React.createElement(ItemGroup, { title: "\u6587\u5B57\u989C\u8272" },
            React.createElement(ColorSetter, { value: (_j = toJSON(contentValue).attributes) === null || _j === void 0 ? void 0 : _j.color, onChange: (v) => {
                    changeStyle(v, 'color');
                } })),
        React.createElement(ItemGroup, { title: "\u6587\u5B57\u5927\u5C0F" },
            React.createElement(InputNumber, { addonBefore: "\u5B57\u4F53\u5927\u5C0F", addonAfter: "px", defaultValue: getPxValue((_k = toJSON(contentValue).attributes) === null || _k === void 0 ? void 0 : _k['font-size']), onChange: (v) => {
                    changeStyle(`${v}px`, 'font-size');
                } })),
        React.createElement(ItemGroup, { title: "\u80CC\u666F" },
            React.createElement(BgSetter, { value: (_l = toJSON(contentValue).attributes) === null || _l === void 0 ? void 0 : _l.background, onChange: (v) => {
                    if (RegExp(/url/).test(v)) {
                        changeStyle(v, 'background-image');
                    }
                    else {
                        changeStyle(v, 'background-color');
                    }
                } })),
        React.createElement(ItemGroup, { title: "\u8FB9\u6846" },
            React.createElement(BorderSetter, { value: (_m = toJSON(contentValue).attributes) === null || _m === void 0 ? void 0 : _m.border, onChange: (v) => {
                    changeStyle(v, 'border');
                } })),
        React.createElement(ItemGroup, { title: "\u900F\u660E\u5EA6" },
            React.createElement(Slider, { max: 1, min: 0, step: 0.1, value: ((_o = toJSON(contentValue).attributes) === null || _o === void 0 ? void 0 : _o.opacity) || 1, onChange: (v) => {
                    changeStyle(v, 'opacity');
                } })),
        React.createElement(ItemGroup, { title: "\u6837\u5F0F\u7F16\u5199" },
            React.createElement(CssCodeSetter, { value: contentValue, onChange: codeEditorChange }))));
}
function ItemGroup({ title, children }) {
    return (React.createElement(Box, { borderBottom: "solid", borderBottomColor: "gray.30", px: "m" },
        React.createElement(Box, { pt: "m" }, title),
        React.createElement(Box, { py: "m" }, children)));
}
