import React, { useCallback, useState } from 'react';
import { css, Box } from 'coral-system';
import { AutoComplete } from 'antd';
import { ActionSelect } from '@music163/tango-ui';
import { useWorkspace, useWorkspaceData } from '@music163/tango-context';
import { wrapCode } from '@music163/tango-helpers';
import { ExpressionModal } from './expression-setter';
import { value2code } from '@music163/tango-core';
var EventAction;
(function (EventAction) {
    EventAction["NoAction"] = "noAction";
    EventAction["ConsoleLog"] = "consoleLog";
    EventAction["BindExpression"] = "bindExpression";
    EventAction["OpenModal"] = "openModal";
    EventAction["CloseModal"] = "closeModal";
    EventAction["NavigateTo"] = "navigateTo";
})(EventAction || (EventAction = {}));
const wrapperStyle = css `
  .ant-select,
  .ant-input {
    width: 100%;
    margin-bottom: 8px;
  }
`;
const options = [
    { label: '无动作', value: EventAction.NoAction },
    { label: '打印事件', value: EventAction.ConsoleLog },
    { label: '绑定 JS 表达式', value: EventAction.BindExpression },
    { label: '打开页面', value: EventAction.NavigateTo },
    { label: '打开弹窗', value: EventAction.OpenModal },
    { label: '关闭弹窗', value: EventAction.CloseModal },
];
/**
 * 事件监听函数绑定器
 */
export function EventSetter(props) {
    const { value, onChange, modalTitle } = props;
    const [type, setType] = useState(); // 事件类型
    const [temp, setTemp] = useState(''); // 二级暂存值
    const [expModalVisible, setExpModalVisible] = useState(false); // 弹窗是否显示
    const { actionVariables, routeOptions } = useWorkspaceData();
    const workspace = useWorkspace();
    const modalOptions = workspace.activeViewModule.listModals() || [];
    const code = value2code(value);
    const handleChange = useCallback((nextValue, ...args) => {
        if (!nextValue) {
            onChange(undefined);
        }
        if (nextValue !== code) {
            onChange(wrapCode(nextValue), ...args);
        }
    }, [onChange, code]);
    const onAction = (key) => {
        setType(key); // 记录事件类型
        setTemp(''); // 重置二级选项值
        switch (key) {
            case EventAction.BindExpression:
                setExpModalVisible(true);
                break;
            case EventAction.ConsoleLog:
                handleChange('(...args) => console.log(...args)');
                break;
            case EventAction.NoAction:
                handleChange(undefined);
                break;
            default:
                break;
        }
    };
    const actionText = getActionText(type, temp, code);
    return (React.createElement(Box, { css: wrapperStyle },
        React.createElement(ActionSelect, { options: options, onSelect: onAction, text: actionText }),
        React.createElement(ExpressionModal, { title: modalTitle, value: code, visible: expModalVisible, onCancel: () => setExpModalVisible(false), onOk: (nextValue) => {
                setExpModalVisible(false);
                handleChange(nextValue);
            }, dataSource: actionVariables }),
        type === EventAction.NavigateTo && (React.createElement(AutoComplete, { placeholder: "\u9009\u62E9\u6216\u8F93\u5165\u9875\u9762\u8DEF\u7531", options: routeOptions, value: temp, onChange: setTemp, onBlur: () => {
                if (temp) {
                    handleChange(getExpressionValue(type, temp));
                }
            } })),
        type === EventAction.OpenModal && (React.createElement(AutoComplete, { placeholder: "\u9009\u62E9\u6216\u8F93\u5165\u5F39\u7A97 id", options: modalOptions, value: temp, onChange: setTemp, onBlur: () => {
                if (temp) {
                    handleChange(getExpressionValue(type, temp));
                }
            } })),
        type === EventAction.CloseModal && (React.createElement(AutoComplete, { placeholder: "\u9009\u62E9\u6216\u8F93\u5165\u5F39\u7A97 id", options: modalOptions, value: temp, onChange: setTemp, onBlur: () => {
                if (temp) {
                    handleChange(getExpressionValue(type, temp));
                }
            } }))));
}
const handlerMap = {
    [EventAction.OpenModal]: 'openModal',
    [EventAction.CloseModal]: 'closeModal',
    [EventAction.NavigateTo]: 'navigateTo',
};
function getActionText(type, temp, fallbackCode) {
    let text;
    if (handlerMap[type]) {
        text = getExpressionValue(type, temp);
    }
    else if (fallbackCode) {
        text = fallbackCode;
    }
    text = text || '请选择';
    return text;
}
function getExpressionValue(type, value = '') {
    const handler = handlerMap[type];
    if (handler) {
        return `() => tango.${handler}("${value}")`;
    }
}
