import { __rest } from "tslib";
import React, { useState, useEffect, useCallback } from 'react';
import { Box, Text, css } from 'coral-system';
import { Dropdown, Modal } from 'antd';
import { isValidExpressionCode } from '@music163/tango-core';
import { noop, useBoolean, getValue } from '@music163/tango-helpers';
import { CloseCircleFilled, ExpandAltOutlined, MenuOutlined } from '@ant-design/icons';
import { Panel, InputCode, Action } from '@music163/tango-ui';
import { useWorkspace, useWorkspaceData } from '@music163/tango-context';
import { VariableTree } from '../components';
import { useSandboxQuery } from '../context';
import { CODE_TEMPLATES } from '../helpers';
import { shapeServiceValues } from '../sidebar/datasource-panel/interface-config';
export const expressionValueValidate = (value) => {
    if (!isValidExpressionCode(value)) {
        return '表达式存在语法错误！';
    }
};
export const jsonValueValidate = (value) => {
    try {
        JSON.parse(value);
    }
    catch (e) {
        return '不是合法的 JSON 语法！';
    }
};
const suffixStyle = css `
  display: flex;
  align-items: center;
  .anticon-close-circle {
    color: rgba(0, 0, 0, 0.25);
    &:hover {
      color: rgba(0, 0, 0, 0.45);
    }
  }
`;
export function ExpressionSetter(props) {
    const { onChange, modalTitle, modalTip, autoCompleteOptions, placeholder = '在这里输入JS代码', value: valueProp, status, allowClear = true, newStoreTemplate, showOptionsDropDown = true, } = props;
    // const codeValue = getCodeOfWrappedCode(valueProp);
    const [inputValue, setInputValue] = useState(valueProp);
    const [visible, { on, off }] = useBoolean();
    // when receive new value, sync state
    useEffect(() => {
        setInputValue(valueProp);
    }, [valueProp]);
    const change = useCallback((code) => {
        if (code === valueProp) {
            return;
        }
        onChange(code);
    }, [valueProp, onChange]);
    const sandbox = useSandboxQuery();
    const evaluateContext = sandbox.window;
    return (React.createElement(Box, { className: "ExpressionSetter" },
        React.createElement(InputCode, { placeholder: placeholder, suffix: React.createElement(Box, { css: suffixStyle },
                allowClear && (React.createElement(CloseCircleFilled, { title: "\u6E05\u7A7A", onClick: () => {
                        change('');
                    } })),
                showOptionsDropDown && (autoCompleteOptions === null || autoCompleteOptions === void 0 ? void 0 : autoCompleteOptions.length) && (React.createElement(Dropdown, { menu: {
                        items: autoCompleteOptions.map((option) => ({
                            key: option,
                            label: (React.createElement(Text, { onClick: () => {
                                    change(option);
                                } }, option)),
                        })),
                    } },
                    React.createElement(Action, { tooltip: "\u4F7F\u7528\u9884\u8BBE\u4EE3\u7801\u7247\u6BB5", icon: React.createElement(MenuOutlined, null), size: "small" }))),
                React.createElement(Action, { tooltip: "\u6253\u5F00\u8868\u8FBE\u5F0F\u53D8\u91CF\u9009\u62E9\u9762\u677F", icon: React.createElement(ExpandAltOutlined, null), onClick: on, size: "small" })), value: inputValue, onChange: setInputValue, onBlur: () => {
                change(inputValue);
            }, autoCompleteContext: evaluateContext, autoCompleteOptions: autoCompleteOptions, status: status, maxHeight: "200px" }),
        React.createElement(ExpressionModal, { title: modalTitle, subTitle: modalTip, placeholder: placeholder, autoCompleteOptions: autoCompleteOptions, newStoreTemplate: newStoreTemplate, visible: visible, value: inputValue, onCancel: () => off(), onOk: (value) => {
                change(value);
                off();
            } })));
}
export function ExpressionModal({ title, subTitle, placeholder, visible, onCancel = noop, onOk = noop, defaultValue, value, dataSource, autoCompleteOptions, newStoreTemplate = CODE_TEMPLATES.newStoreTemplate, }) {
    const [exp, setExp] = useState(value !== null && value !== void 0 ? value : defaultValue);
    const [error, setError] = useState('');
    const workspace = useWorkspace();
    const { expressionVariables } = useWorkspaceData();
    const serviceModules = Object.keys(workspace.serviceModules).map((key) => ({
        label: key === 'index' ? '默认模块' : key,
        value: key,
    }));
    const sandbox = useSandboxQuery();
    const evaluateContext = sandbox.window;
    const handleExpInputChange = (val) => {
        setExp(val === null || val === void 0 ? void 0 : val.trim());
    };
    useEffect(() => {
        setExp(value);
    }, [value]);
    return (React.createElement(Modal, { closable: false, destroyOnClose: true, width: "60%", open: visible, onCancel: onCancel, onOk: () => {
            onOk(exp);
        }, bodyStyle: {
            padding: 0,
        } },
        React.createElement(Panel, { title: `将 ${title} 设置为引用变量或自定义表达式`, subTitle: subTitle, shape: "solid" },
            React.createElement(InputCode, { shape: "inset", minHeight: "56px", maxHeight: "200px", value: exp, placeholder: placeholder, onChange: handleExpInputChange, onBlur: () => {
                    setError(expressionValueValidate(exp));
                }, autoCompleteContext: evaluateContext, autoCompleteOptions: autoCompleteOptions }),
            error ? (React.createElement(Text, { color: "red", fontSize: "12px" }, "\u51FA\u9519\u4E86\uFF01\u8F93\u5165\u7684\u8868\u8FBE\u5F0F\u5B58\u5728\u8BED\u6CD5\u9519\u8BEF\uFF0C\u8BF7\u4FEE\u6539\u540E\u518D\u63D0\u4EA4\uFF01")) : (React.createElement(Text, { fontSize: "12px", color: "text3" }, "\u8BF4\u660E\uFF1A\u4F60\u53EF\u4EE5\u5728\u4E0A\u9762\u7684\u4EE3\u7801\u8F93\u5165\u6846\u91CC\u8F93\u5165\u5E38\u89C4\u7684 javascript \u4EE3\u7801\uFF0C\u8FD8\u53EF\u4EE5\u76F4\u63A5\u4F7F\u7528 jsx \u4EE3\u7801\uFF0C\u4F46\u9700\u8981\u7B26\u5408\u8BE5\u5C5E\u6027\u7684\u63A5\u53D7\u503C\u5B9A\u4E49\u3002"))),
        React.createElement(Panel, { title: "\u4ECE\u53D8\u91CF\u5217\u8868\u4E2D\u9009\u4E2D", shape: "solid", borderTop: "0", overflow: "hidden", bodyProps: { overflow: 'hidden' } },
            React.createElement(VariableTree, { height: 380, showViewButton: true, dataSource: dataSource || expressionVariables, appContext: sandbox === null || sandbox === void 0 ? void 0 : sandbox.window['tango'], getStoreNames: () => Object.keys(workspace.storeModules), serviceModules: serviceModules, getServiceData: (serviceKey) => {
                    const data = workspace.getServiceFunction(serviceKey);
                    return Object.assign({ name: data.name, moduleName: data.moduleName, method: 'get' }, data.config);
                }, onSelect: (node) => {
                    if (!node.key) {
                        return;
                    }
                    if (node.key.split('.').length < 2) {
                        return;
                    }
                    let str;
                    if (/^(stores|services)\./.test(node.key)) {
                        str = `tango.${node.key.replaceAll('.', '?.')}`;
                    }
                    else {
                        str = `${node.key}`;
                    }
                    setExp(str);
                }, onAddStoreVariable: (storeName, data) => {
                    workspace.addStoreState(storeName, data.name, data.initialValue);
                }, onUpdateStoreVariable: (variableKey, code) => {
                    workspace.updateStoreVariable(variableKey, code);
                }, onAddStore: (storeName) => {
                    workspace.addStoreFile(storeName, newStoreTemplate);
                }, onRemoveStoreVariable: (variableKey) => {
                    workspace.removeStoreVariable(variableKey);
                }, onRemoveService: (serviceKey) => {
                    workspace.removeServiceFunction(serviceKey);
                }, onAddService: (data) => {
                    const _a = shapeServiceValues(data), { name, moduleName } = _a, payload = __rest(_a, ["name", "moduleName"]);
                    workspace.addServiceFunction(name, payload, moduleName);
                }, onUpdateService: (data) => {
                    const _a = shapeServiceValues(data), { name, moduleName } = _a, payload = __rest(_a, ["name", "moduleName"]);
                    workspace.updateServiceFunction(name, payload, moduleName);
                }, getPreviewValue: (node) => {
                    if (!node || !node.key) {
                        return;
                    }
                    if (node.type === 'function') {
                        return;
                    }
                    return getValue(evaluateContext['tango'], node.key);
                } }))));
}
