import { __rest } from "tslib";
import React, { useState } from 'react';
import { css, Box } from 'coral-system';
import { Form, Input, Button, Popover } from 'antd';
import { DeleteOutlined, EditOutlined, HolderOutlined } from '@ant-design/icons';
import { DragBox } from '../components';
function copyWithoutUndefined(source) {
    const ret = {};
    for (const key in source) {
        if (source[key] !== undefined) {
            ret[key] = source[key];
        }
    }
    return ret;
}
export function useListSetter(props) {
    const { value: valueProp = [], onChange: onChangeProp } = props, rest = __rest(props, ["value", "onChange"]);
    const value = Array.isArray(valueProp) ? valueProp : [];
    const onChange = (next = []) => {
        onChangeProp && onChangeProp(next);
    };
    const getOnEdit = (index) => {
        return (data) => {
            const copy = [...value];
            copy[index] = data;
            onChange(copy);
        };
    };
    const getOnRemove = (index) => {
        return () => {
            const copy = [...value];
            copy.splice(index, 1);
            onChange(copy);
        };
    };
    const onAdd = (data) => {
        const next = [...value].concat([data]);
        onChange(next);
    };
    const onMove = (start, end) => {
        const copy = [...value];
        const temp = copy[end];
        copy[end] = copy[start];
        copy[start] = temp;
        onChange(copy);
    };
    return Object.assign({ value,
        onAdd,
        getOnEdit,
        getOnRemove,
        onMove }, rest);
}
const listItemStyle = css `
  display: flex;
  justify-content: space-between;
  border: 1px solid var(--tango-colors-line-normal);
  border-radius: var(--tango-radii-m);
  line-height: 2;
  padding-left: 12px;

  &:hover {
    background-color: var(--tango-colors-gray-10);
  }
`;
const leftCss = css `
  display: inline-block;

  .icon {
    margin-right: 8px;
    cursor: move;
  }
`;
export function ListSetterItem(_a) {
    var { children, onRemove, index = 0, onMove } = _a, rest = __rest(_a, ["children", "onRemove", "index", "onMove"]);
    return (React.createElement(DragBox, { mb: "m", index: index, css: listItemStyle, onMove: onMove },
        React.createElement(Box, { css: leftCss },
            React.createElement(HolderOutlined, { className: "icon" }),
            children),
        React.createElement("div", null,
            React.createElement(EditButton, Object.assign({}, rest)),
            React.createElement(Button, { type: "link", icon: React.createElement(DeleteOutlined, null), onClick: onRemove }))));
}
function EditButton({ formFields, formInitialValues, onEdit }) {
    const [visible, setVisible] = useState(false);
    return (React.createElement(Popover, { visible: visible, onVisibleChange: setVisible, trigger: "click", placement: "leftTop", title: "\u4FEE\u6539\u6570\u636E", content: React.createElement(NewOptionForm, { fields: formFields, initialValues: formInitialValues, onSubmit: (values) => {
                onEdit && onEdit(values);
                setVisible(false);
            } }) },
        React.createElement(Button, { type: "link", icon: React.createElement(EditOutlined, null) })));
}
export function AddListItemButton({ formFields = [], formInitialValues = {}, getFormInitialValues, text = '添加一项', onAdd, }) {
    var _a;
    const [visible, setVisible] = useState(false);
    const initialValues = (_a = getFormInitialValues === null || getFormInitialValues === void 0 ? void 0 : getFormInitialValues()) !== null && _a !== void 0 ? _a : formInitialValues;
    return (React.createElement(Popover, { destroyTooltipOnHide: true, visible: visible, onVisibleChange: setVisible, trigger: "click", placement: "leftTop", title: text, content: React.createElement(NewOptionForm, { fields: formFields, initialValues: initialValues, onSubmit: (values) => {
                onAdd && onAdd(values);
                setVisible(false);
            } }) },
        React.createElement(Button, { block: true }, text)));
}
const layout = {
    labelCol: { span: 8 },
    wrapperCol: { span: 16 },
};
const tailLayout = {
    wrapperCol: { offset: 8, span: 16 },
};
function NewOptionForm({ fields = [], initialValues = {}, onSubmit }) {
    const [form] = Form.useForm();
    const onFinish = (values) => {
        onSubmit && onSubmit(copyWithoutUndefined(values));
    };
    return (React.createElement(Form, Object.assign({ form: form, onFinish: onFinish }, layout),
        fields.map((item) => (React.createElement(Form.Item, { key: item.name, label: item.label, name: item.name, required: item.required, rules: [{ required: item.required }], valuePropName: item.valuePropName, extra: item.extra, style: {
                width: item.width,
            }, initialValue: initialValues[item.name] }, item.component || React.createElement(Input, { placeholder: "\u8BF7\u8F93\u5165", autoComplete: "off" })))),
        React.createElement(Form.Item, Object.assign({}, tailLayout),
            React.createElement(Button, { type: "primary", htmlType: "submit" }, "\u63D0\u4EA4"))));
}
const defaultListItemFormFields = [
    { label: 'key', name: 'key', required: true },
];
const defaultGetKey = (item) => item.key;
const defaultRenderItem = (item) => item.key;
/**
 * 列表项设置器
 */
export function ListSetter(props) {
    const { listItemFormFields = defaultListItemFormFields, newItemDefaultValues, getNewItemDefaultValues, addBtnText, getListItemKey = defaultGetKey, renderItem = defaultRenderItem } = props, rest = __rest(props, ["listItemFormFields", "newItemDefaultValues", "getNewItemDefaultValues", "addBtnText", "getListItemKey", "renderItem"]);
    const { value, onAdd, getOnEdit, getOnRemove, onMove } = useListSetter(rest);
    return (React.createElement(Box, null,
        React.createElement(Box, null, value.map((item, index) => {
            return (React.createElement(ListSetterItem, { key: getListItemKey(item), index: index, formFields: listItemFormFields, formInitialValues: item, onEdit: getOnEdit(index), onRemove: getOnRemove(index), onMove: onMove }, renderItem(item)));
        })),
        React.createElement(AddListItemButton, { text: addBtnText, formFields: listItemFormFields, formInitialValues: newItemDefaultValues, getFormInitialValues: getNewItemDefaultValues, onAdd: onAdd })));
}
