import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { ActionSelect, InputCode } from '@music163/tango-ui';
import { Box } from 'coral-system';
const defaultOptions = [
    { label: '取消自定义', value: '' },
    { label: '自定义渲染', value: 'Box', render: '() => <Box></Box>' },
];
/**
 * Render Props Setters
 */
export function RenderSetter({ value, onChange, text = '自定义渲染为', options = defaultOptions, fallbackOption, }) {
    const [inputValue, setInputValue] = useState(value || '');
    useEffect(() => {
        setInputValue(value);
    }, [value]);
    const optionsMap = useMemo(() => {
        return options.reduce((prev, cur) => {
            prev[cur.value] = cur;
            return prev;
        }, {});
    }, [options]);
    const onSelect = useCallback((key) => {
        const option = optionsMap[key] || fallbackOption;
        if (option === null || option === void 0 ? void 0 : option.render) {
            onChange(option.render, { relatedImports: option.relatedImports });
        }
        else {
            onChange(undefined);
        }
    }, [optionsMap, fallbackOption, onChange]);
    return (React.createElement(Box, null,
        React.createElement(ActionSelect, { text: text, options: options, onSelect: onSelect }),
        inputValue && (React.createElement(InputCode, { value: inputValue, onChange: (val) => setInputValue(val), onBlur: () => onChange(inputValue) }))));
}
const getRender = (content, type) => {
    let code;
    switch (type) {
        case 'tableCell':
            code = `(value, record, index) => ${content}`;
            break;
        case 'tableExpandable':
            code = `{
        expandedRowRender: (record) =>  ${content},
        rowExpandable: (record) => true
      }`;
            break;
        default:
            code = `() => ${content}`;
            break;
    }
    return code;
};
const tableCellOptions = [
    { label: '取消自定义', value: '' },
    {
        label: '自定义区域',
        value: 'Box',
        render: getRender('<Box></Box>', 'tableCell'),
        relatedImports: ['Box'],
    },
    {
        label: '标签',
        value: 'Tag',
        render: getRender('<Tag>tag</Tag>', 'tableCell'),
        relatedImports: ['Tag'],
    },
    {
        label: '按钮',
        value: 'Button',
        render: getRender('<Button>button</Button>', 'tableCell'),
        relatedImports: ['Button'],
    },
    {
        label: '图片',
        value: 'Image',
        render: getRender('<Image width={150} src="https://picsum.photos/100" />', 'tableCell'),
        relatedImports: ['Image'],
    },
];
const tableExpandableOptions = [
    {
        label: '设置可展开行',
        value: 'Box',
        render: getRender('<Box></Box>', 'tableExpandable'),
        relatedImports: ['Box'],
    },
    { label: '取消可展开行', value: '' },
];
export function TableCellSetter(props) {
    return React.createElement(RenderSetter, Object.assign({ options: tableCellOptions }, props));
}
// FIXME: 应该直接用 props 嵌套的模式
export function TableExpandableSetter(props) {
    return React.createElement(RenderSetter, Object.assign({ options: tableExpandableOptions, text: "\u914D\u7F6E\u8868\u683C\u53EF\u5C55\u5F00\u884C" }, props));
}
