import { __rest } from "tslib";
import React, { useRef, useState } from 'react';
import { Box, Grid } from 'coral-system';
import { InputNumber, Dropdown, Input, Radio, Popover, Button } from 'antd';
import { SketchPicker } from 'react-color';
import Color from 'color';
import { BgColorsOutlined, EyeInvisibleOutlined, FileImageOutlined } from '@ant-design/icons';
import { SingleMonacoEditor, LineSolidOutlined, LineDashedOutlined } from '@music163/tango-ui';
import { ChoiceSetter } from './choice-setter';
function getRawCssValue(value) {
    if (value && value.startsWith('{css`')) {
        return value.split('').slice(5, -2).join('').trim();
    }
    return value;
}
// TODO: style object setter
/**
 * 不稳定，暂不推荐使用
 */
export function CssCodeSetter({ value, onChange }) {
    const contentValue = getRawCssValue(value);
    return (React.createElement(Popover, { placement: "leftBottom", title: "\u81EA\u5B9A\u4E49\u6837\u5F0F\uFF08CSS in JS\uFF09", content: React.createElement(Box, { height: 480, width: 400 },
            React.createElement(SingleMonacoEditor, { language: "css", value: contentValue, onBlur: (newCode) => {
                    if (newCode != contentValue) {
                        onChange(`{css\`${newCode}\`}`, {});
                    }
                } })) },
        React.createElement(Button, { block: true }, "\u7F16\u5199\u6837\u5F0F")));
}
/**
 * css display
 * @param props
 * @returns
 */
export function DisplaySetter(props) {
    return (React.createElement(ChoiceSetter, Object.assign({ options: [
            { value: 'block', icon: 'icon-display-block', tip: '块级元素' },
            { value: 'inline-block', icon: 'icon-display-inline-block', tip: '行内块级元素' },
            { value: 'flex', icon: 'icon-flex', tip: 'Flex 弹性布局' },
            { value: 'grid', icon: 'icon-display-grid', tip: 'Grid 弹性网格布局' },
            { value: 'inline', icon: 'icon-display-inline', tip: '行内元素' },
            { value: 'none', icon: 'icon-display-none', tip: '不显示' },
        ], mode: "icon" }, props)));
}
export function FlexDirectionSetter(props) {
    return (React.createElement(ChoiceSetter, Object.assign({ options: [
            { value: 'column', icon: 'icon-direction-column', tip: '单行排列' },
            { value: 'column-reverse', icon: 'icon-direction-column-reverse', tip: '单行逆序' },
            { value: 'row', icon: 'icon-direction-row', tip: '纵向排列' },
            { value: 'row-reverse', icon: 'icon-direction-row-reverse', tip: '纵向逆序' },
        ], mode: "icon" }, props)));
}
export function FlexJustifyContentSetter(props) {
    return (React.createElement(ChoiceSetter, Object.assign({ options: [
            { value: 'flex-start', icon: 'icon-justify-flex-start', tip: '从行首起始位置开始排列' },
            { value: 'flex-end', icon: 'icon-justify-flex-end', tip: '从行尾位置开始排列' },
            { value: 'center', icon: 'icon-justify-center', tip: '居中排列' },
            {
                value: 'space-between',
                icon: 'icon-justify-space-between',
                tip: '均匀排列每个元素，首个元素放置于起点，末尾元素放置于终点',
            },
            {
                value: 'space-around',
                icon: 'icon-justify-space-around',
                tip: '均匀排列每个元素，每个元素周围分配相同的空间',
            },
        ], mode: "icon" }, props)));
}
export function FlexAlignItemsSetter(props) {
    return (React.createElement(ChoiceSetter, Object.assign({ options: [
            { value: 'flex-start', icon: 'icon-align-flex-start', tip: '元素向辅轴起点对齐' },
            { value: 'flex-end', icon: 'icon-align-flex-end', tip: '元素向辅轴终点对齐' },
            { value: 'center', icon: 'icon-align-center', tip: '元素在辅轴居中' },
            {
                value: 'stretch',
                icon: 'icon-align-stretch',
                tip: '弹性项包含外边距的交叉轴尺寸被拉升至行高',
            },
            { value: 'baseline', icon: 'icon-align-baseline', tip: '所有元素向基线对齐' },
        ], mode: "icon" }, props)));
}
export function FlexGapSetter({ value, onChange }) {
    const values = parseGapValue(value);
    const tempRef = useRef([...values]);
    const handleChange = (index) => {
        return (val) => {
            tempRef.current[index] = `${val}px`;
            onChange(tempRef.current.join(' '));
        };
    };
    return (React.createElement(Grid, { columns: 2, spacing: "m" },
        React.createElement(InputNumber, { addonBefore: "\u4E3B", addonAfter: "px", defaultValue: values[0], onChange: handleChange(0) }),
        React.createElement(InputNumber, { addonBefore: "\u8F85", addonAfter: "px", defaultValue: values[1], onChange: handleChange(1) })));
}
function rgbaObjectToString(rbg) {
    return `rgba(${rbg.r}, ${rbg.g}, ${rbg.b}, ${rbg.a})`;
}
function normalizeColorObject(colorObject) {
    const { r, g, b, alpha } = colorObject;
    return {
        r,
        g,
        b,
        a: alpha,
    };
}
export function ColorSetter(props) {
    const { value, onChange, placeholder = '请选择颜色' } = props;
    const color = hexColorPattern.test(value) ? value : normalizeColorObject(Color(value).object());
    const colorPanel = (React.createElement(SketchPicker, { color: color, onChangeComplete: (newColor) => {
            const nextValue = newColor.rgb.a === 1 ? newColor.hex : rgbaObjectToString(newColor.rgb);
            if (typeof onChange === 'function') {
                onChange(nextValue);
            }
        } }));
    return (React.createElement(Dropdown, { overlay: colorPanel, trigger: ['click'] },
        React.createElement(Input, { prefix: React.createElement(Box, { bg: value, border: "solid", borderColor: "gray.40", size: "14px", borderRadius: "s" }), allowClear: true, placeholder: placeholder, value: value, onChange: (e) => {
                if (!e.target.value) {
                    // clear input
                    onChange(undefined);
                }
            } })));
}
const hexColorPattern = /^#[0-9a-fA-F]{3,}$/;
const imgUrlPattern = /^url\((.+)\)$/;
const getImageUrl = (url) => {
    const ret = imgUrlPattern.exec(url);
    if (ret && ret.length) {
        return ret[1];
    }
    return '';
};
export function BgSetter({ value, onChange }) {
    const [mode, setMode] = useState(() => {
        if (!value) {
            return 'color';
        }
        return hexColorPattern.test(value) ? 'color' : 'image';
    });
    return (React.createElement(Box, null,
        React.createElement(Radio.Group, { defaultValue: mode, onChange: (e) => setMode(e.target.value), buttonStyle: "solid" },
            React.createElement(Radio.Button, { value: "color" },
                React.createElement(BgColorsOutlined, null)),
            React.createElement(Radio.Button, { value: "image" },
                React.createElement(FileImageOutlined, null))),
        React.createElement(Box, { mt: "m" },
            mode === 'color' && React.createElement(ColorSetter, { value: value, onChange: onChange }),
            mode === 'image' && (React.createElement(Input, { value: getImageUrl(value), onChange: (e) => {
                    const imgUrl = e.target.value;
                    onChange(imgUrl ? `url(${imgUrl})` : undefined);
                } })))));
}
export function SpacingSetter({ value, onChange }) {
    const values = parseSpacingValue(value);
    const tempRef = useRef([...values]);
    const handleChange = (index) => {
        return (val) => {
            tempRef.current[index] = `${val}`;
            const str2px = tempRef.current.map((it) => {
                return `${it}px`;
            });
            onChange(str2px.join(' '));
        };
    };
    return (React.createElement(Grid, { columns: 2, spacing: "m" },
        React.createElement(InputNumber, { addonBefore: "\u4E0A", addonAfter: "px", defaultValue: values[0], onChange: handleChange(0) }),
        React.createElement(InputNumber, { addonBefore: "\u4E0B", addonAfter: "px", defaultValue: values[2], onChange: handleChange(2) }),
        React.createElement(InputNumber, { addonBefore: "\u53F3", addonAfter: "px", defaultValue: values[1], onChange: handleChange(1) }),
        React.createElement(InputNumber, { addonBefore: "\u5DE6", addonAfter: "px", defaultValue: values[3], onChange: handleChange(3) })));
}
function parseSpacingValue(value) {
    if (!value || typeof value !== 'string') {
        return ['0', '0', '0', '0'];
    }
    const arr = value.split(' ');
    if (arr.length === 1) {
        return ['0', '0', '0', '0'];
    }
    if (arr.length === 2) {
        const x = getPxValue(arr[1]);
        const y = getPxValue(arr[0]);
        return [y, x, y, x];
    }
    if (arr.length === 4) {
        return arr.map(getPxValue);
    }
}
function parseGapValue(value) {
    if (!value || typeof value !== 'string') {
        return ['0', '0'];
    }
    const arr = value.split(' ');
    if (arr.length === 1) {
        return ['0', '0'];
    }
    return arr.map(getPxValue);
}
const pxPattern = /^(\d+)px$/;
function getPxValue(value) {
    const match = pxPattern.exec(value);
    if (match && match.length) {
        return match[1];
    }
    return '0';
}
function parseBorderValue(value) {
    if (!value || typeof value !== 'string') {
        return ['0', 'none'];
    }
    return value.split(' ');
}
export function BorderSetter({ value, onChange }) {
    const valueParts = parseBorderValue(value);
    const [style, setStyle] = useState(() => {
        if (/^(none|solid|dashed)$/.test(valueParts[1])) {
            return valueParts[1];
        }
        return 'none';
    });
    const [width, setWidth] = useState(Number(getPxValue(valueParts[0])) || 0);
    const [color, setColor] = useState(valueParts[2] || '#000');
    const handleChange = (key, val) => {
        let ret;
        switch (key) {
            case 'style': {
                ret = [width, val, color];
                break;
            }
            case 'width': {
                ret = [`${val}px`, style, color];
                break;
            }
            case 'color': {
                ret = [width, style, val];
                break;
            }
            default:
                ret = ['none'];
                break;
        }
        if (ret[1] === 'none') {
            ret = ['none'];
        }
        onChange(ret.join(' '));
    };
    return (React.createElement(Box, null,
        React.createElement(Label, { label: "\u98CE\u683C" },
            React.createElement(Radio.Group, { defaultValue: style, onChange: (e) => {
                    setStyle(e.target.value);
                    handleChange('style', e.target.value);
                }, buttonStyle: "solid" },
                React.createElement(Radio.Button, { value: "none" },
                    React.createElement(EyeInvisibleOutlined, null)),
                React.createElement(Radio.Button, { value: "solid" },
                    React.createElement(LineSolidOutlined, null)),
                React.createElement(Radio.Button, { value: "dashed" },
                    React.createElement(LineDashedOutlined, null)))),
        style !== 'none' && (React.createElement(Box, null,
            React.createElement(Label, { label: "\u5BBD\u5EA6", mt: "m" },
                React.createElement(InputNumber, { defaultValue: width, addonAfter: "px", min: 0, onChange: (val) => {
                        setWidth(val);
                        handleChange('width', val);
                    } })),
            React.createElement(Label, { label: "\u989C\u8272", mt: "m" },
                React.createElement(ColorSetter, { value: color, onChange: (val) => {
                        setColor(val);
                        handleChange('color', val);
                    } }))))));
}
function Label(_a) {
    var { label, children } = _a, rest = __rest(_a, ["label", "children"]);
    return (React.createElement(Box, Object.assign({ display: "flex", alignItems: "center" }, rest),
        React.createElement(Box, { mr: "m" }, label),
        React.createElement(Box, null, children)));
}
