import { __rest } from "tslib";
import React, { useMemo } from 'react';
import { observer, useWorkspace } from '@music163/tango-context';
import { ConfigGroup, ConfigItem } from '@music163/tango-ui';
import { Box, css } from 'coral-system';
import { Input, Switch, Form, Button, message } from 'antd';
import { CloseCircleOutlined, CloseOutlined } from '@ant-design/icons';
export default observer(() => {
    var _a;
    const workspace = useWorkspace();
    return (React.createElement(ConfigGroup, { title: "\u4EE3\u7406\u8BBE\u7F6E" },
        React.createElement(ConfigItem, { title: "proxy", description: "\u8FD9\u91CC\u53EF\u4EE5\u914D\u7F6E\u63A5\u53E3\u8F6C\u53D1\u7684\u4EE3\u7406\u89C4\u5219", layoutDirection: "column" },
            React.createElement(ProxyManage, { initialValues: (_a = workspace.tangoConfigJson) === null || _a === void 0 ? void 0 : _a.getValue('proxy'), onSave: (value) => {
                    var _a;
                    (_a = workspace.tangoConfigJson) === null || _a === void 0 ? void 0 : _a.setValue('proxy', value).update();
                    message.success('proxy 规则更新成功！');
                } }))));
});
export const proxyRuleStyle = css `
  position: relative;
  border: 1px dashed #f0f0f0;
  padding-top: 24px;
  border-radius: 4px;
  margin-bottom: 8px;

  &:hover {
    border-color: var(--tango-colors-brand);
  }

  > .RuleRemoveBtn {
    position: absolute;
    top: 0;
    right: 0;
  }

  .RuleHeaderItem {
    margin-bottom: 10px;
  }
`;
export const proxyManageFormStyle = {
    labelCol: {
        span: 6,
    },
    wrapperCol: {
        span: 16,
    },
    autoComplete: 'off',
    colon: false,
};
function ProxyManage({ initialValues: initialValuesProp = {}, onSave, }) {
    const initialValues = useMemo(() => {
        const proxy = Object.keys(initialValuesProp).reduce((prev, cur) => {
            prev.push(Object.assign({ path: cur }, initialValuesProp[cur]));
            return prev;
        }, []);
        return {
            proxy,
        };
    }, [initialValuesProp]);
    return (React.createElement(Form, Object.assign({}, proxyManageFormStyle, { initialValues: initialValues, onFinish: (values) => {
            const proxy = values.proxy.reduce((prev, _a) => {
                var { path } = _a, rest = __rest(_a, ["path"]);
                prev[path] = rest;
                return prev;
            }, {});
            onSave === null || onSave === void 0 ? void 0 : onSave(proxy);
        } }),
        React.createElement(Form.List, { name: "proxy" }, (fields, { add, remove }) => {
            return (React.createElement(React.Fragment, null,
                fields.map((field) => (React.createElement(Box, { key: field.key, css: proxyRuleStyle },
                    React.createElement(Button, { className: "RuleRemoveBtn", type: "link", icon: React.createElement(CloseOutlined, null), onClick: () => remove(field.name) }),
                    React.createElement(Form.Item, Object.assign({}, field, { key: "path", label: "path", name: [field.name, 'path'], rules: [{ required: true }] }),
                        React.createElement(Input, null)),
                    React.createElement(Form.Item, Object.assign({}, field, { key: "target", label: "target", name: [field.name, 'target'], rules: [{ required: true }] }),
                        React.createElement(Input, null)),
                    React.createElement(Form.Item, Object.assign({}, field, { key: "changeOrigin", label: "changeOrigin", name: [field.name, 'changeOrigin'], valuePropName: "checked" }),
                        React.createElement(Switch, null)),
                    React.createElement(Form.Item, { label: "\u8BF7\u6C42\u5934" },
                        React.createElement(Form.List, { name: [field.name, 'headers'] }, (headerFields, { add: headerAdd, remove: headerRemove }) => {
                            return (React.createElement(React.Fragment, null,
                                headerFields.map((headerField) => (React.createElement(Form.Item, { className: "RuleHeaderItem", key: headerField.key },
                                    React.createElement(Box, { gap: "8px", display: "flex" },
                                        React.createElement(Form.Item, Object.assign({}, headerField, { noStyle: true, key: "name", label: "name", name: [headerField.name, 'name'], rules: [{ required: true }] }),
                                            React.createElement(Input, { placeholder: "Key" })),
                                        React.createElement(Form.Item, Object.assign({}, headerField, { noStyle: true, key: "value", label: "value", name: [headerField.name, 'value'], rules: [{ required: true }] }),
                                            React.createElement(Input, { placeholder: "Value" })),
                                        React.createElement(Button, { type: "link", icon: React.createElement(CloseCircleOutlined, null), onClick: () => headerRemove(headerField.name) }))))),
                                React.createElement("a", { onClick: () => headerAdd() }, "\u6DFB\u52A0\u8BF7\u6C42\u5934")));
                        }))))),
                React.createElement(Box, { mt: "m", mb: "l" },
                    React.createElement(Button, { block: true, type: "dashed", onClick: () => add() }, "\u6DFB\u52A0\u89C4\u5219"))));
        }),
        React.createElement(Form.Item, null,
            React.createElement(Button, { type: "primary", htmlType: "submit" }, "\u4FDD\u5B58"))));
}
