import { __awaiter, __rest } from "tslib";
import React, { useEffect, useMemo, useState } from 'react';
import { observer, useWorkspace } from '@music163/tango-context';
import { Button, Input, Modal, Form, List, Tooltip, Popconfirm, Radio, message, Checkbox, } from 'antd';
import { Box, Text } from 'coral-system';
import semverLt from 'semver/functions/lt';
import semverValid from 'semver/functions/valid';
import { ColorTag, ConfigGroup, ConfigItem } from '@music163/tango-ui';
import { MinusCircleOutlined, PlusOutlined, QuestionCircleOutlined } from '@ant-design/icons';
import { useBoolean } from '@music163/tango-helpers';
import { isUndefined } from 'lodash-es';
import { useSandboxQuery } from '../context';
var DependencyItemType;
(function (DependencyItemType) {
    DependencyItemType["\u57FA\u7840\u5305"] = "base";
    DependencyItemType["\u4E1A\u52A1\u7EC4\u4EF6"] = "biz";
    DependencyItemType["\u5176\u4ED6\u4F9D\u8D56"] = "other";
})(DependencyItemType || (DependencyItemType = {}));
export const DependencyPanel = observer(({ onBizDependencyAdd, onBaseDependencyAdd }) => {
    var _a;
    // baseDependencies
    const [templateBaseDependencies, setTemplateBaseDependencies] = useState();
    const workspace = useWorkspace();
    const dependencies = workspace === null || workspace === void 0 ? void 0 : workspace.listDependencies();
    const packages = (_a = workspace === null || workspace === void 0 ? void 0 : workspace.tangoConfigJson) === null || _a === void 0 ? void 0 : _a.getValue('packages');
    const packageNames = useMemo(() => Object.keys(packages || {}), [packages]);
    const bizDependencies = workspace === null || workspace === void 0 ? void 0 : workspace.bizComps;
    const baseDependencies = workspace === null || workspace === void 0 ? void 0 : workspace.baseComps;
    // 其他依赖
    const otherDeps = useMemo(() => 
    // 优先展示 tango.config.json 的依赖，以表明 umd 资源的加载顺序
    [...new Set([...packageNames, ...Object.keys(dependencies || {})])]
        .filter((key) => !(baseDependencies === null || baseDependencies === void 0 ? void 0 : baseDependencies.includes(key)) && !(bizDependencies === null || bizDependencies === void 0 ? void 0 : bizDependencies.includes(key)))
        .map((key) => {
        var _a, _b;
        return ({
            name: key,
            version: (_a = dependencies === null || dependencies === void 0 ? void 0 : dependencies[key]) !== null && _a !== void 0 ? _a : (_b = packages === null || packages === void 0 ? void 0 : packages[key]) === null || _b === void 0 ? void 0 : _b.version,
            package: packages === null || packages === void 0 ? void 0 : packages[key],
        });
    }), [dependencies, baseDependencies, bizDependencies, packages, packageNames]);
    // 业务组件依赖
    const bizDeps = useMemo(() => (bizDependencies === null || bizDependencies === void 0 ? void 0 : bizDependencies.map((name) => {
        var _a, _b;
        return ({
            name,
            version: (_a = dependencies[name]) !== null && _a !== void 0 ? _a : (_b = packages === null || packages === void 0 ? void 0 : packages[name]) === null || _b === void 0 ? void 0 : _b.version,
            package: packages === null || packages === void 0 ? void 0 : packages[name],
        });
    })) || [], [bizDependencies, dependencies, packages]);
    // 基础包
    const baseDeps = useMemo(() => (baseDependencies === null || baseDependencies === void 0 ? void 0 : baseDependencies.map((name) => {
        var _a, _b, _c;
        return ({
            name,
            version: (_a = dependencies[name]) !== null && _a !== void 0 ? _a : (_b = packages === null || packages === void 0 ? void 0 : packages[name]) === null || _b === void 0 ? void 0 : _b.version,
            description: (_c = (templateBaseDependencies || {})[name]) === null || _c === void 0 ? void 0 : _c.description,
            package: packages === null || packages === void 0 ? void 0 : packages[name],
        });
    })) || [], [baseDependencies, dependencies, templateBaseDependencies, packages]);
    return (React.createElement(Box, null,
        React.createElement(AddDependencyModal, { templateBaseDependencies: templateBaseDependencies, onBizDependencyAdd: onBizDependencyAdd, onBaseDependencyAdd: onBaseDependencyAdd }),
        React.createElement(ConfigGroup, { title: React.createElement("div", null,
                "\u57FA\u7840\u5305",
                React.createElement(Tooltip, { title: React.createElement(React.Fragment, null,
                        "- \u57FA\u7840\u5305\u4E0D\u80FD\u88AB\u5220\u9664",
                        React.createElement("br", null),
                        "- \u5347\u7EA7\u9ED8\u8BA4\u5347\u7EA7\u5230\u6700\u65B0\u7248\u672C",
                        React.createElement("br", null),
                        "- \u5982\u679C\u60F3\u65B0\u589E\u5BF9\u5E94\u6A21\u677F\u57FA\u7840\u5305\uFF0C\u8BF7\u8054\u7CFB\u7BA1\u7406\u5458") },
                    React.createElement(QuestionCircleOutlined, null))), key: DependencyItemType.基础包 },
            React.createElement(ConfigItem, { layoutDirection: "column" },
                React.createElement(DependencyList, { templateBaseDependencies: templateBaseDependencies, type: DependencyItemType.基础包, dependencies: baseDeps, onUpgrade: onBaseDependencyAdd }))),
        React.createElement(ConfigGroup, { title: "\u5176\u4ED6\u4F9D\u8D56", key: DependencyItemType.其他依赖 },
            React.createElement(ConfigItem, { layoutDirection: "column" },
                React.createElement(DependencyList, { type: DependencyItemType.其他依赖, dependencies: otherDeps })))));
});
function RenderItem({ type, record, templateBaseDependencies, onUpgrade, showTags = true, }) {
    var _a;
    const [open, { on, off }] = useBoolean(false);
    const workspace = useWorkspace();
    const basePackage = useMemo(() => {
        if (type !== DependencyItemType.基础包 || !templateBaseDependencies)
            return undefined;
        return templateBaseDependencies[record.name];
    }, [type, templateBaseDependencies, record]);
    /**
     * 处理依赖升级的逻辑
     */
    function onDepUpgrade(values) {
        var _a;
        // 根据是否有 packages 字段，判断是否已升级为新的规范
        const hasUpgraded = !!((_a = workspace.tangoConfigJson) === null || _a === void 0 ? void 0 : _a.getValue('packages'));
        const { version, umd, library, resources, designerResources } = values || {};
        let packageConfig = record === null || record === void 0 ? void 0 : record.package;
        if (umd === false && packageConfig) {
            packageConfig = Object.assign(Object.assign({}, packageConfig), { version, library: undefined, resources: undefined, designerResources: undefined });
        }
        else {
            packageConfig = Object.assign(Object.assign({}, packageConfig), { version,
                library,
                resources,
                designerResources });
        }
        switch (type) {
            case DependencyItemType.业务组件:
            case DependencyItemType.其他依赖: {
                workspace.updateDependency(record.name, version, hasUpgraded
                    ? {
                        package: packageConfig,
                    }
                    : {});
                message.success(`依赖 ${record.name} 升级成功`);
                onCloseModal();
                break;
            }
            case DependencyItemType.基础包: {
                // 处理 package.json
                workspace.updateDependency(record.name, (basePackage === null || basePackage === void 0 ? void 0 : basePackage.version) || 'latest', hasUpgraded
                    ? {
                        package: packageConfig,
                    }
                    : {});
                // 配置已经合并为新版，不需要走旧的逻辑
                if (!hasUpgraded) {
                    // 更新 tango.config.json externalResources
                    workspace.tangoConfigJson
                        .setValue('sandbox.externalResources', (targetValue) => {
                        // 需要被替换的位置
                        const replaceIndex = targetValue.findIndex((resource) => resource.includes(record.name));
                        // 需要被替换的元素，可能有多个，比如物料既有 js 又有 css
                        const replaceArr = targetValue.filter((resource) => resource.includes(record.name)) || [];
                        targetValue.splice(replaceIndex, replaceArr.length, ...basePackage.resources.map((resource) => resource.replace('{latest_version}', basePackage.version)));
                        return targetValue;
                    })
                        .update();
                }
                message.success(`依赖 ${record.name} 升级成功`);
                break;
            }
            default:
                break;
        }
        if (onUpgrade) {
            onUpgrade({
                type,
                name: record.name,
                version,
                detail: record,
            });
        }
    }
    /**
     * 处理依赖删除的逻辑
     */
    function onDepDelete() {
        switch (type) {
            case DependencyItemType.业务组件: {
                // 移除 package.json
                workspace.removeBizComp(record.name);
                message.success(`依赖 ${record.name} 移除成功`);
                break;
            }
            case DependencyItemType.其他依赖:
            case DependencyItemType.基础包: {
                workspace.removeDependency(record.name);
                message.success(`依赖 ${record.name} 移除成功`);
                break;
            }
            default:
                break;
        }
    }
    function onCloseModal() {
        off();
    }
    const baseNeedUpgrade = useMemo(() => {
        if (type !== DependencyItemType.基础包 || !basePackage || !record)
            return;
        // 非法版本号，比如 'latest'，会报错，直接默认升级走升级替换为最新版本
        if (!semverValid(record.version))
            return true;
        return semverLt(record.version, basePackage.version);
    }, [type, basePackage, record]);
    const tags = useMemo(() => {
        var _a, _b;
        const packages = (_a = workspace.tangoConfigJson) === null || _a === void 0 ? void 0 : _a.getValue('packages');
        // 旧配置格式化不展示
        if (!showTags || !packages) {
            return false;
        }
        const { name, package: packageInfo } = record;
        const { resources } = packageInfo || {};
        const dependencies = (_b = workspace.packageJson) === null || _b === void 0 ? void 0 : _b.getValue('dependencies');
        const hasUmd = !!(resources === null || resources === void 0 ? void 0 : resources.length);
        const packageJsonIncluded = !!(dependencies === null || dependencies === void 0 ? void 0 : dependencies[name]);
        const tangoConfigJsonIncluded = !!packageInfo;
        return (React.createElement(Box, { display: "inline-block" },
            hasUmd && (React.createElement(ColorTag, { key: "hasUmd", color: "green", tooltip: "\u8BE5\u4F9D\u8D56\u62E5\u6709 UMD \u8D44\u6E90\uFF0C\u53EF\u4F18\u5316\u52A0\u8F7D\u901F\u5EA6" }, "UMD")),
            packageJsonIncluded && React.createElement(ColorTag, { tooltip: "\u6B64\u4F9D\u8D56\u5728 package.json \u4E2D\u5B58\u5728" }, "package"),
            tangoConfigJsonIncluded && (React.createElement(ColorTag, { tooltip: "\u6B64\u4F9D\u8D56\u5728 tango.config.json \u4E2D\u5B58\u5728" }, "tango.config"))));
    }, [showTags, workspace, record]);
    return (React.createElement(List.Item, { actions: type !== DependencyItemType.基础包
            ? [
                React.createElement("a", { key: "upgrade", onClick: on }, "\u4FEE\u6539"),
                React.createElement(Popconfirm, { onConfirm: onDepDelete, title: `确认删除依赖 ${record.name}？` },
                    React.createElement("a", { key: "delete" }, "\u5220\u9664")),
            ]
            : baseNeedUpgrade
                ? [
                    React.createElement(Popconfirm, { title: `确认升级 ${record.name} 到 ${basePackage.version} 吗？`, onConfirm: () => onDepUpgrade({ version: basePackage.version }) },
                        React.createElement("a", { key: "upgrade" }, "\u5347\u7EA7")),
                ]
                : [
                    React.createElement(Text, { key: "latest", color: 'rgba(0, 0, 0, 0.45)' }, !isUndefined(baseNeedUpgrade) && '已是最新'),
                ] },
        React.createElement(List.Item.Meta, { title: React.createElement(Text, null,
                record.name,
                React.createElement(Text, { ml: 6, fontSize: '10px', color: 'rgba(0, 0, 0, 0.45)' }, semverValid(record.version) && record.version),
                showTags && React.createElement(Text, { ml: 6 }, tags)), description: React.createElement(Text, { dangerouslySetInnerHTML: { __html: ((_a = record === null || record === void 0 ? void 0 : record.package) === null || _a === void 0 ? void 0 : _a.description) || record.description } }) }),
        React.createElement(DependencyConfigModal, { open: open, record: record, onCancel: onCloseModal, onOk: (event, { values }) => onDepUpgrade(values) })));
}
function DependencyList({ type, dependencies, templateBaseDependencies, onUpgrade, }) {
    return (React.createElement(List, { style: { paddingTop: 0 }, size: "small", dataSource: dependencies, renderItem: (record) => (React.createElement(RenderItem, { type: type, record: record, templateBaseDependencies: templateBaseDependencies, onUpgrade: onUpgrade })) }));
}
function AddDependencyModal(_a) {
    var _b;
    var { templateBaseDependencies, onBizDependencyAdd, onBaseDependencyAdd, record } = _a, rest = __rest(_a, ["templateBaseDependencies", "onBizDependencyAdd", "onBaseDependencyAdd", "record"]);
    // 依赖类型
    const [open, { on, off }] = useBoolean();
    const workspace = useWorkspace();
    const sandbox = useSandboxQuery();
    const hasUpgraded = !!((_b = workspace.tangoConfigJson) === null || _b === void 0 ? void 0 : _b.getValue('packages'));
    const onFinishCallback = (name) => {
        name && message.success(`${name} 添加成功`);
        // TIP: 修复沙箱添加组件后 HMR 失效的 bug，添加完刷新
        sandbox.reload();
        off();
    };
    const onOk = (event, { values }) => {
        const { type, name, version, umd, library, resources, designerResources } = values;
        switch (type) {
            case DependencyItemType.其他依赖: {
                workspace.updateDependency(name, version, hasUpgraded && umd
                    ? {
                        package: Object.assign(Object.assign({ type: 'dependency' }, record === null || record === void 0 ? void 0 : record.package), { version,
                            library,
                            resources,
                            designerResources }),
                    }
                    : {});
                break;
            }
            // case DependencyItemType.业务组件: {
            //   const targetBizInfo = (bizList as any)?.find(
            //     (biz: { packageName: string }) => biz.packageName === name,
            //   );
            //   onBizDependencyAdd &&
            //     onBizDependencyAdd({
            //       ...values,
            //       detail: targetBizInfo,
            //     });
            //   break;
            // }
            case DependencyItemType.基础包: {
                const basePackage = templateBaseDependencies[name];
                // 添加 package.json
                workspace.updateDependency(name, basePackage.version || 'latest', hasUpgraded
                    ? {
                        package: basePackage.package,
                    }
                    : {});
                // 配置已经合并为新版，不需要走旧的逻辑
                if (!hasUpgraded) {
                    // 更新 tango.config.json baseDependencies
                    workspace.tangoConfigJson.setValue('baseDependencies', (targetValue) => {
                        const bases = targetValue || [];
                        bases.push(name);
                        return bases;
                    });
                    // 更新 tango.config.json externals
                    workspace.tangoConfigJson.setValue('sandbox.externals', (targetValue) => {
                        targetValue[name] = basePackage === null || basePackage === void 0 ? void 0 : basePackage.global;
                        return targetValue;
                    });
                    // 更新 tango.config.json externalResources
                    workspace.tangoConfigJson
                        .setValue('sandbox.externalResources', (targetValue) => {
                        basePackage.resources.forEach((resource) => {
                            targetValue.push(resource.replace('{latest_version}', basePackage.version));
                        });
                        return targetValue;
                    })
                        .update();
                }
                onBaseDependencyAdd && onBaseDependencyAdd(basePackage);
                break;
            }
            default:
                break;
        }
        onFinishCallback(values === null || values === void 0 ? void 0 : values.name);
    };
    function onCloseModal() {
        off();
    }
    return (React.createElement(Box, { px: "l", py: "m" },
        React.createElement(Button, { ghost: true, block: true, type: "primary", onClick: on },
            React.createElement(PlusOutlined, null),
            "\u6DFB\u52A0\u4F9D\u8D56"),
        React.createElement(DependencyConfigModal, Object.assign({ open: open, title: record ? `修改依赖：${record.name}` : '添加依赖', onOk: onOk, onCancel: onCloseModal, destroyOnClose: true }, rest))));
}
function ResourceFormList(props) {
    return (React.createElement(Form.List, Object.assign({}, props), (fields, { add, remove }, { errors }) => (React.createElement(React.Fragment, null,
        fields.map((field) => (React.createElement(Form.Item, { key: field.key },
            React.createElement(Box, { display: "flex", alignItems: "center" },
                React.createElement(Form.Item, Object.assign({ label: "\u8D44\u6E90\u5730\u5740" }, field, { rules: [{ required: true, whitespace: true, type: 'url' }], noStyle: true }),
                    React.createElement(Input, { placeholder: "\u8D44\u6E90\u5730\u5740\u4E2D\u7684\u7248\u672C\u53F7\u53EF\u7528 {{version}} \u66FF\u4EE3", style: { width: '100%' } })),
                React.createElement(Box, { width: "20px", fontSize: "18px", ml: "6px", textAlign: "center", flex: "none" },
                    React.createElement("a", { onClick: () => remove(field.name) },
                        React.createElement(MinusCircleOutlined, null))))))),
        React.createElement(Button, { type: "dashed", onClick: () => add(), block: true }, "\u6DFB\u52A0"),
        React.createElement(Form.ErrorList, { errors: errors })))));
}
function DependencyConfigModal(_a) {
    var _b;
    var { record, open, onOk, onCancel } = _a, rest = __rest(_a, ["record", "open", "onOk", "onCancel"]);
    // 依赖类型
    const [type, setType] = useState(DependencyItemType.其他依赖);
    const [form] = Form.useForm();
    const workspace = useWorkspace();
    // 根据是否有 packages 字段，判断是否已升级为新的规范
    const hasUpgraded = !!((_b = workspace.tangoConfigJson) === null || _b === void 0 ? void 0 : _b.getValue('packages'));
    // const [versionList, setVersionList] = useState<Array<{ label: string; value: string }>>([]);
    // const remoteServices = useRemoteServices();
    // useEffect(() => {
    //   remoteServices?.DependencyService?.listPackageVersions({ packageName: record.name }).then(
    //     (data: any) => setVersionList((data || []).map((v: string) => ({ label: v, value: v }))),
    //   );
    // }, [remoteServices, record]);
    useEffect(() => {
        var _a, _b;
        if (open && form) {
            form.resetFields();
            if (record) {
                setType(record === null || record === void 0 ? void 0 : record.type);
                form.setFieldsValue(Object.assign(Object.assign(Object.assign({}, record), record === null || record === void 0 ? void 0 : record.package), { umd: !!((_b = (_a = record === null || record === void 0 ? void 0 : record.package) === null || _a === void 0 ? void 0 : _a.resources) === null || _b === void 0 ? void 0 : _b.length) }));
            }
        }
    }, [record, open, form]);
    const onSubmit = (event) => {
        form.validateFields().then((values) => {
            if (onOk) {
                onOk(event, {
                    values,
                    hasUpgraded,
                });
            }
        });
    };
    const onValuesChange = ({ type: depType }) => {
        if (depType) {
            // type 变更触发
            form.resetFields();
            form.setFieldValue('type', depType);
            setType(depType);
        }
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(Modal, Object.assign({ width: 560, open: open, title: record ? `修改依赖：${record.name}` : '添加依赖', onOk: onSubmit, onCancel: onCancel, destroyOnClose: true }, rest),
            React.createElement(Form, { form: form, layout: "vertical", autoComplete: "off", initialValues: { type }, onValuesChange: onValuesChange },
                !record && (React.createElement(React.Fragment, null,
                    React.createElement(Form.Item, { label: "\u4F9D\u8D56\u7C7B\u578B", name: "type", rules: [{ required: true }] },
                        React.createElement(Radio.Group, null,
                            React.createElement(Radio.Button, { value: DependencyItemType.基础包 }, "\u57FA\u7840\u5305"),
                            React.createElement(Radio.Button, { value: DependencyItemType.其他依赖 }, "\u5176\u4ED6\u4F9D\u8D56"))),
                    React.createElement(Form.Item, { label: "npm \u5305\u540D", name: "name", rules: [{ required: true }] },
                        React.createElement(Input, { placeholder: "\u8F93\u5165\u4F9D\u8D56\u540D" })))),
                React.createElement(Form.Item, { label: "\u7248\u672C\u53F7", name: "version", rules: [{ required: true, message: '请输入依赖的版本号' }] },
                    React.createElement(Input, { placeholder: "\u8F93\u5165\u7248\u672C\u53F7" })),
                hasUpgraded && (React.createElement(Form.Item, { name: "umd", valuePropName: "checked" },
                    React.createElement(Checkbox, null, "\u4F7F\u7528\u9884\u6784\u5EFA UMD \u8D44\u6E90"))),
                React.createElement(Form.Item, { noStyle: true, shouldUpdate: true }, () => form.getFieldValue('umd') && (React.createElement(React.Fragment, null,
                    React.createElement(Form.Item, { label: "\u5168\u5C40\u53D8\u91CF\u540D", name: "library" },
                        React.createElement(Input, { placeholder: "\u8F93\u5165 UMD \u8D44\u6E90\u6302\u8F7D\u5728\u5168\u5C40\u53D8\u91CF\u4E0B\u7684\u53D8\u91CF\u540D" })),
                    React.createElement(Form.Item, { label: "\u8D44\u6E90\u5730\u5740", required: true },
                        React.createElement(ResourceFormList, { name: "resources", rules: [
                                {
                                    validator: (rule, value) => __awaiter(this, void 0, void 0, function* () {
                                        if (!(value === null || value === void 0 ? void 0 : value.length)) {
                                            throw new Error('资源地址不能为空');
                                        }
                                    }),
                                },
                            ] })),
                    form.getFieldValue('type') === DependencyItemType.基础包 && (React.createElement(Form.Item, { label: "\u8BBE\u8BA1\u5668\u8D44\u6E90\u5730\u5740" },
                        React.createElement(ResourceFormList, { name: "designerResources" }))))))))));
}
