import React, { useEffect, useReducer } from 'react';
import { observer } from '@music163/tango-context';
import { isFunction, isObject, pick } from '@music163/tango-helpers';
import { CollapsePanel, IconButton, JsonView } from '@music163/tango-ui';
import { ReloadOutlined } from '@ant-design/icons';
import { useSandboxQuery } from '../../context';
function processTango(obj = {}, index = 0) {
    const ret = {};
    Object.keys(obj).forEach((key) => {
        const val = obj[key];
        if (index > 2) {
            ret[key] = val;
        }
        else if (isFunction(val)) {
            ret[key] = '[Function]';
        }
        else if (isObject(val)) {
            ret[key] = processTango(val, index + 1);
        }
        else {
            ret[key] = val;
        }
    });
    return ret;
}
export const StateTree = observer(() => {
    const sandboxQuery = useSandboxQuery();
    const [, forceUpdate] = useReducer((x) => x + 1, 0);
    const tangoContext = pick(sandboxQuery.window['tango'] || {}, [
        'stores',
        'pageStore',
        'services',
        'config',
    ]);
    const callback = () => {
        forceUpdate();
    };
    useEffect(() => {
        // FIXME: 优化这段代码，应该监听 tangoContext 的变化，来刷新
        const settingForm = document.querySelector('.SettingPanel');
        const ob = new MutationObserver(callback);
        if (settingForm) {
            ob.observe(settingForm, {
                attributes: true,
                childList: true,
                subtree: true,
            });
        }
        return () => {
            ob.disconnect();
        };
    }, []);
    return (React.createElement(CollapsePanel, { title: "\u9875\u9762\u72B6\u6001", stickyHeader: true, maxHeight: "90%", overflowY: "auto", extra: React.createElement(IconButton, { tooltip: "\u540C\u6B65\u72B6\u6001", size: "small", icon: React.createElement(ReloadOutlined, null), onClick: (e) => {
                e.stopPropagation();
                forceUpdate();
            } }) },
        React.createElement(JsonView, { src: processTango(tangoContext), collapsed: 1, name: "tango", displayObjectSize: false, indentWidth: 2, enableCopy: true })));
});
