import React, { useEffect, useState } from 'react';
import cx from 'classnames';
import { Box, css } from 'coral-system';
import { Resizable } from 'react-resizable';
const resizeHandleStyle = (axis, barStyle) => css `
  position: absolute;
  z-index: 999;
  cursor: ${axis === 'x' ? 'col-resize' : 'row-resize'};
  ${barStyle &&
    Object.keys(barStyle)
        .map((key) => `${key}: ${barStyle[key]};`)
        .join('')}
  &:hover,
  &:active {
    background-color: var(--tango-colors-brand);
  }
`;
const getResizeHandles = (resizeHandlePosition, axis) => {
    const handles = [];
    switch (axis) {
        case 'x':
            handles.push(resizeHandlePosition === 'right' ? 'e' : 'w');
            break;
        case 'y':
            handles.push(resizeHandlePosition === 'bottom' ? 's' : 'n');
            break;
        default:
            break;
    }
    return handles;
};
export function ResizableBox({ resizeHandlePosition = 'right', width: widthProp, height: heightProp, children, className, style, axis = 'x', }) {
    const [width, setWidth] = useState(widthProp);
    const [height, setHeight] = useState(heightProp);
    const barStyle = Object.assign({}, (axis === 'x'
        ? {
            height: '100%',
            width: '4px',
            top: '0',
            [resizeHandlePosition === 'right' ? 'right' : 'left']: '-4px',
        }
        : {
            height: '4px',
            width: '100%',
            left: '0',
            [resizeHandlePosition === 'bottom' ? 'bottom' : 'top']: '0',
        }));
    useEffect(() => {
        setWidth(widthProp);
    }, [widthProp]);
    useEffect(() => {
        setHeight(heightProp);
    }, [heightProp]);
    return (React.createElement(Resizable, { axis: axis, width: width, resizeHandles: getResizeHandles(resizeHandlePosition, axis), height: height, minConstraints: [150, 150], maxConstraints: [window.innerWidth * 0.6, window.innerHeight * 0.8], onResize: (e, { size }) => {
            if (axis === 'x') {
                setWidth(size.width);
            }
            if (axis === 'y') {
                setHeight(size.height);
            }
        }, onResizeStart: () => {
            document.body.style.pointerEvents = 'none';
            document.body.style.userSelect = 'none';
        }, onResizeStop: () => {
            document.body.style.pointerEvents = 'auto';
            document.body.style.userSelect = 'auto';
        }, handle: React.createElement(Box, { className: "ResizeHandle", css: resizeHandleStyle(axis, barStyle) }) },
        React.createElement("div", { className: cx('ResizableBox', className), style: Object.assign({ position: 'relative', width,
                height }, style) }, children)));
}
