import { __rest } from "tslib";
import React, { useMemo } from 'react';
import cx from 'classnames';
import { Box, Text, css } from 'coral-system';
import { Badge } from 'antd';
import { observer, useDesigner } from '@music163/tango-context';
import { ResizableBox } from './resizable-box';
import { getWidget } from '../widgets';
const sidebarStyle = css `
  position: relative;
  background-color: var(--tango-colors-custom-sidebarBg);
  border-right: 1px solid var(--tango-colors-line-normal);
  box-shadow: rgb(0 0 0 / 5%) 0px 0px 18px;
  height: 100%;

  .SidebarPanelBarList {
    width: 50px;
    list-style: none;
    user-select: none;
    transition: all 0.3s ease;
  }

  .SidebarPanelBarListItem {
    &.active {
      background-color: var(--tango-colors-custom-sidebarItemActiveBg);
    }

    &:hover {
      background-color: var(--tango-colors-custom-sidebarItemHoverBg);
    }
  }
`;
function BaseSidebarPanel({ panelWidth: defaultPanelWidth = 280, footer, children }) {
    const designer = useDesigner();
    const items = useMemo(() => {
        const ret = {};
        React.Children.forEach(children, (child) => {
            if (child && React.isValidElement(child)) {
                const widget = getWidget(['sidebar', child.key].join('.'));
                const fallbackNode = widget ? React.createElement(widget, child.props.widgetProps) : null;
                ret[child.key] = Object.assign({ children: fallbackNode }, child.props);
            }
        });
        return ret;
    }, [children]);
    return (React.createElement(Box, { display: "flex", flexShrink: 0, css: sidebarStyle, className: "SidebarPanel" },
        React.createElement(Box, { className: "SidebarPanelBar", display: "flex", flexDirection: "column", position: "relative" },
            React.createElement(Box, { as: "ul", flex: "0", p: "0", m: "0", textAlign: "center", className: "SidebarPanelBarList" }, Object.keys(items).map((key) => {
                const item = items[key];
                const isActive = key === designer.activeSidebarPanel;
                return (React.createElement("li", { className: cx('SidebarPanelBarListItem', { active: isActive }), key: key, title: item.title, onClick: () => {
                        designer.setActiveSidebarPanel(key);
                    } },
                    React.createElement(SidebarPanelBarItem, { icon: item.icon, label: item.label, showBadge: item.showBadge, isActive: isActive })));
            })),
            React.createElement(Box, { className: "SidebarPanelBarFooter", pb: "l", flex: "1", display: "flex", flexDirection: "column", alignItems: "center", justifyContent: "flex-end" }, footer)),
        Object.keys(items).map((key) => {
            const panel = items[key];
            const floatPanelStyle = (panel === null || panel === void 0 ? void 0 : panel.isFloat)
                ? {
                    position: 'absolute',
                    left: '50px',
                    top: 0,
                    zIndex: 1000,
                    height: '100%',
                    boxShadow: 'var(--tango-shadows-lowRight)',
                }
                : {};
            const panelWidth = typeof (panel === null || panel === void 0 ? void 0 : panel.width) === 'number' ? panel === null || panel === void 0 ? void 0 : panel.width : defaultPanelWidth;
            return (React.createElement(ResizableBox, { key: panel.label, width: panelWidth, style: Object.assign({ display: designer.activeSidebarPanel === key ? 'block' : 'none' }, floatPanelStyle) },
                React.createElement(SidebarPanelExpandedContent, { isFloat: panel.isFloat, closeable: panel.isFloat, onClose: () => {
                        designer.closeSidebarPanel();
                    } }, panel.children)));
        })));
}
BaseSidebarPanel.Item = function ({ key, title, doc, isFloat, width, widgetProps, }) {
    return React.createElement(React.Fragment, null);
};
export const Sidebar = observer(BaseSidebarPanel);
function SidebarPanelBarItem({ isActive, icon: iconProp, label, showBadge, }) {
    const color = isActive ? 'brand' : 'text.body';
    let icon = (React.createElement(Text, { fontSize: "24px", lineHeight: 1, color: color }, iconProp));
    if (showBadge) {
        icon = (React.createElement(Badge, Object.assign({ size: "small" }, showBadge), icon));
    }
    return (React.createElement(Box, { display: "flex", flexDirection: "column", alignItems: "center", justifyContent: "center", py: "12px" },
        icon,
        React.createElement(Text, { fontSize: "12px", mt: "s", color: color }, label)));
}
const expandPanelStyle = css `
  flex: 1;
  height: 100%;
  background-color: var(--tango-colors-white);
  border-left: 1px solid var(--tango-colors-line-normal);
  position: relative;
  overflow-y: auto;

  &.isFloat {
    position: absolute;
    left: 0;
    top: 0;
    z-index: 1000;
    height: 100%;
    width: 100%;
    box-shadow: var(--tango-shadows-lowRight);
  }
`;
function SidebarPanelExpandedContent(_a) {
    var { closeable, onClose, isFloat, className, children } = _a, rest = __rest(_a, ["closeable", "onClose", "isFloat", "className", "children"]);
    const classNames = cx('SidebarPanelExpanded', {
        isFloat,
    }, className);
    return (React.createElement(Box, Object.assign({ css: expandPanelStyle, className: classNames }, rest), children));
}
