import { __rest } from "tslib";
import React, { useEffect, useRef } from 'react';
import cx from 'classnames';
import { Box } from 'coral-system';
import { useDesigner, useWorkspace } from '@music163/tango-context';
import { Ghost } from './ghost';
import { useSandboxQuery } from '../context';
import { SelectionTools } from './selection';
import { InsertionPrompt } from './insertion';
import { DraggingMask } from './mask';
export function Viewport(_a) {
    var { selectionTools, children, className } = _a, rest = __rest(_a, ["selectionTools", "children", "className"]);
    return (React.createElement(Box, Object.assign({ position: "relative", height: "100%", className: cx('Viewport', className) }, rest),
        React.createElement(ViewportBody, null, children),
        React.createElement("div", { className: "AuxTools" },
            React.createElement(SelectionTools, { actions: selectionTools }),
            React.createElement(InsertionPrompt, null),
            React.createElement(DraggingMask, null)),
        React.createElement(Ghost, null)));
}
function ViewportBody(_a) {
    var { children } = _a, rest = __rest(_a, ["children"]);
    const mainRef = useRef();
    const workspace = useWorkspace();
    const designer = useDesigner();
    const sandboxQuery = useSandboxQuery();
    useEffect(() => {
        const resizeObserver = new ResizeObserver((entries) => {
            if (mainRef.current) {
                designer.setViewport({
                    width: mainRef.current.offsetWidth,
                    height: mainRef.current.offsetHeight,
                });
            }
            if (entries.length && workspace.selectSource.isSelected) {
                const items = workspace.selectSource.selected.map((item) => {
                    const element = sandboxQuery.getElementBySlotId(item.id);
                    if (element) {
                        const bounding = sandboxQuery.getElementBounding(element);
                        item.bounding = bounding;
                        item.element = element;
                    }
                    return item;
                });
                workspace.selectSource.select(items);
            }
        });
        resizeObserver.observe(mainRef.current);
    }, []);
    return (React.createElement(Box, Object.assign({ ref: mainRef, position: "relative", height: "100%", className: "ViewportBody" }, rest), children));
}
