import React, { useMemo, useState } from 'react';
import { Button, Popconfirm, Modal, Form, Input } from 'antd';
import { Box, css, Group, Text } from 'coral-system';
import { FileOutlined, DeleteOutlined, CopyOutlined, SettingOutlined } from '@ant-design/icons';
import { array2object, callAll } from '@music163/tango-helpers';
import { ToggleButton } from '@music163/tango-ui';
import { observer, useWorkspace } from '@music163/tango-context';
export const RouteSwitchTool = observer(() => {
    const workspace = useWorkspace();
    return (React.createElement(PageSelect, { options: workspace.pages, value: workspace.activeRoute, onSelect: (path) => {
            workspace.setActiveRoute(path);
        }, onRemove: (path) => {
            workspace.removeViewModule(path);
        }, onCopy: (...payloads) => workspace.copyViewPage(...payloads), onUpdate: (...payloads) => workspace.updateRoute(...payloads) }));
});
function PageSelect({ value, onSelect, onRemove, onUpdate, onCopy, options }) {
    const [visible, setVisible] = useState(false);
    const optionMap = useMemo(() => {
        return array2object(options, (item) => item.path);
    }, [options]);
    const handleChange = (next) => {
        if (typeof onSelect === 'function') {
            onSelect(next);
        }
        setVisible(false);
    };
    const current = optionMap[value];
    return (React.createElement(ToggleButton, { shape: "ghost", minWidth: 100, dropdownProps: {
            open: visible,
            onOpenChange: setVisible,
        }, overlay: React.createElement(PageList, { items: options, activeKey: value, onSelect: handleChange, onRemove: onRemove, onUpdate: onUpdate, onCopy: onCopy, onAction: (key) => {
                if (key !== 'delete') {
                    setVisible(false);
                }
            } }), icon: React.createElement(FileOutlined, null) }, (current === null || current === void 0 ? void 0 : current.name) || (current === null || current === void 0 ? void 0 : current.label) || value));
}
const pageListStyle = css `
  min-width: 200px;
  max-height: 400px;
  overflow-y: auto;
  padding: 8px 0;
  user-select: none;

  li {
    display: flex;
    align-items: center;
    justify-content: space-between;
    padding: 4px 12px;

    button {
      background: transparent;
      outline: none;
      border: 0;
    }

    &:hover {
      background-color: var(--tango-colors-gray-20);
    }

    &.active {
      background-color: var(--tango-colors-primary-20);
    }
  }
`;
function PageList({ activeKey, items = [], onSelect, onRemove, onUpdate, onCopy, onAction, }) {
    const [current, setCurrent] = useState(null);
    const [type, setType] = useState();
    return (React.createElement(Box, { as: "ul", bg: "white", boxShadow: "lowDown", borderRadius: "m", css: pageListStyle },
        items.map((item) => (React.createElement(Box, { as: "li", key: item.path, gap: "12px", onClick: () => {
                onSelect && onSelect(item.path);
            }, className: activeKey === item.path ? 'active' : undefined },
            React.createElement(FileOutlined, null),
            React.createElement(Box, { flex: "1" },
                item.name || '未命名',
                React.createElement(Text, { as: "code", fontSize: "12px", ml: "m", color: "gray.50" }, item.path)),
            React.createElement(Group, null,
                item.path !== '/' ? (React.createElement(Popconfirm, { title: "\u786E\u8BA4\u5220\u9664\u6B64\u9875\u9762\u5417\uFF1F", onConfirm: (e) => {
                        e.stopPropagation();
                        onRemove && onRemove(item.path);
                    }, zIndex: 1060 },
                    React.createElement(Button, { size: "small", type: "link", title: "\u5220\u9664\u9875\u9762", icon: React.createElement(DeleteOutlined, null), onClick: (e) => {
                            e.stopPropagation();
                            onAction === null || onAction === void 0 ? void 0 : onAction('delete', e);
                        } }))) : null,
                React.createElement(Button, { size: "small", type: "link", title: "\u590D\u5236\u9875\u9762", icon: React.createElement(CopyOutlined, null), onClick: (e) => {
                        e.stopPropagation();
                        setCurrent(item);
                        setType('copy');
                        onAction === null || onAction === void 0 ? void 0 : onAction('copy', e);
                    } }),
                React.createElement(Button, { size: "small", type: "link", title: "\u8BBE\u7F6E\u9875\u9762", icon: React.createElement(SettingOutlined, null), onClick: (e) => {
                        e.stopPropagation();
                        setCurrent(item);
                        setType('update');
                        onAction === null || onAction === void 0 ? void 0 : onAction('update', e);
                    } }))))),
        React.createElement(CopyViewModalForm, { key: current === null || current === void 0 ? void 0 : current.path, title: type === 'update' ? '更新当前页面' : '复制当前页面', editType: type, items: items, visible: !!current, current: current, onOk: callAll(type === 'update' ? onUpdate : onCopy, () => {
                setCurrent(null);
            }), onCancel: () => {
                setCurrent(null);
            } })));
}
function CopyViewModalForm({ title, current, editType, items = [], visible, onOk, onCancel, }) {
    const [form] = Form.useForm();
    return (React.createElement(Modal, { title: title, visible: visible, destroyOnClose: true, onCancel: onCancel, onOk: () => {
            form.validateFields().then((values) => {
                onOk === null || onOk === void 0 ? void 0 : onOk(current.path, values);
            });
        } },
        React.createElement(Form, { wrapperCol: { span: 20 }, form: form, initialValues: { name: current === null || current === void 0 ? void 0 : current.name, path: current === null || current === void 0 ? void 0 : current.path } },
            React.createElement(Form.Item, { label: "\u9875\u9762\u6807\u9898", name: "name", required: true },
                React.createElement(Input, { placeholder: "\u8BF7\u8F93\u5165\u6807\u9898" })),
            React.createElement(Form.Item, { label: "\u9875\u9762\u8DEF\u7531", name: "path", rules: [
                    { required: true },
                    editType !== 'update'
                        ? {
                            validator: (_, v) => {
                                return items.map((item) => item.path).includes(v)
                                    ? Promise.reject()
                                    : Promise.resolve();
                            },
                            message: '重复的页面路由',
                        }
                        : undefined,
                    {
                        pattern: /^\/([A-Za-z0-9-:]*\/?)*$/,
                        message: '非法的路由地址',
                    },
                ] },
                React.createElement(Input, { placeholder: "\u8BF7\u8F93\u5165\u9875\u9762\u8DEF\u7531" })))));
}
