import React, { useMemo } from 'react';
import { Box, Group } from 'coral-system';
import { getWidget } from '../widgets';
export function Toolbar({ children }) {
    const [leftTools, centerTools, rightTools] = useMemo(() => {
        const left = [];
        const center = [];
        const right = [];
        let prevPlacement;
        React.Children.forEach(children, (child, index) => {
            var _a, _b;
            let fallbackNode;
            if (child.key) {
                const Widget = getWidget(['toolbar', child.key].join('.'));
                if (Widget) {
                    fallbackNode = React.createElement(Widget, child.props.widgetProps);
                }
            }
            let node = (_b = (_a = child.props.children) !== null && _a !== void 0 ? _a : fallbackNode) !== null && _b !== void 0 ? _b : null;
            if (!node) {
                node = child; // separator
            }
            const key = child.key || index;
            node = (React.createElement("div", { key: key, className: "ToolbarPanelItem", "data-key": key }, node));
            const placement = child.props.placement || prevPlacement || 'center';
            switch (placement) {
                case 'left':
                    left.push(node);
                    break;
                case 'right':
                    right.push(node);
                    break;
                case 'center':
                default:
                    center.push(node);
                    break;
            }
            prevPlacement = placement;
        });
        return [left, center, right];
    }, [children]);
    return (React.createElement(Box, { className: "ToolbarPanel", display: "flex", justifyContent: "space-between", alignItems: "center" },
        React.createElement(Group, { display: "flex", alignItems: "center", gap: "m" }, leftTools),
        React.createElement(Group, { display: "flex", alignItems: "center", gap: "m" }, centerTools),
        React.createElement(Group, { display: "flex", alignItems: "center", gap: "m" }, rightTools)));
}
function ToolbarItem({ placement, widgetProps, children }) {
    return React.createElement("div", null, children);
}
function Separator() {
    return React.createElement(Box, { className: "Separator", width: 1, height: 16, bg: "colors.custom.toolbarDividerColor" });
}
Toolbar.Item = ToolbarItem;
Toolbar.Separator = Separator;
