var _EventedClass_eventHandlers;
import { __classPrivateFieldGet, __classPrivateFieldSet } from "tslib";
const EVENT_MANAGER_SYMBOL = Symbol('[EventProvider]');
export default class EventedClass {
    constructor() {
        _EventedClass_eventHandlers.set(this, {});
    }
    /**
     * 把任意对象改造成 EventedClass 类型，支持注册事件，并不改变原始对象的引用
     *
     * @static
     * @template TO 对象类型
     * @param {TO} target 要改造的对象
     * @returns {TTarget<TO, TE>}
     * @memberof EventedClass<T>
     */
    static injectTo(target) {
        var _a;
        let newTarget = target;
        if (newTarget) {
            newTarget[EVENT_MANAGER_SYMBOL] = new EventedClass();
        }
        let memberDescriptors = Object.getOwnPropertyDescriptors(EventedClass.prototype);
        for (let name of Object.keys(memberDescriptors).map((s) => s)) {
            if (name === 'constructor') {
                continue;
            }
            let sourceValue = (_a = newTarget[EVENT_MANAGER_SYMBOL]) === null || _a === void 0 ? void 0 : _a[name];
            if (typeof sourceValue === 'function') {
                sourceValue = sourceValue.bind(newTarget[EVENT_MANAGER_SYMBOL]);
            }
            let result = {
                [name]: sourceValue,
            };
            //Remove duplicated keys, i.e. the method already exists on target.
            let duplicatedKeys = Object.keys(result).filter((key) => Object.prototype.hasOwnProperty.call(newTarget, key));
            duplicatedKeys.forEach((key) => {
                delete result[key];
            });
            Object.assign(newTarget, result);
        }
        return newTarget;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    on(eventName, ...onArgs) {
        let eventSource = onArgs[0];
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        let handler = onArgs[1];
        if (typeof eventName !== 'string') {
            throw new Error(`'eventName' must be string and cannot be null.`);
        }
        if (handler == null && typeof eventSource === 'function') {
            handler = eventSource;
            eventSource = undefined;
        }
        if (eventSource == null) {
            eventSource = '';
        }
        if (typeof eventSource !== 'string') {
            throw new Error(`'eventSource' must be string.`);
        }
        if (typeof handler !== 'function') {
            throw new Error(`'handler' must be function.`);
        }
        let eventNameResult = parseEventName(eventName);
        if (__classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName] == null) {
            __classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName] = {};
        }
        let eventNamespaces = __classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName];
        if (eventNamespaces[eventNameResult.namespace] == null) {
            eventNamespaces[eventNameResult.namespace] = {};
        }
        let eventSources = eventNamespaces[eventNameResult.namespace];
        if (eventSources[eventSource] == null) {
            eventSources[eventSource] = [];
        }
        let eventHandlers = eventSources[eventSource];
        eventHandlers.push(handler);
        return this;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    off(...offArgs) {
        let eventName = offArgs[0];
        let eventSource = offArgs[1];
        let handler = offArgs[2];
        if (eventName == null) {
            eventName = '';
        }
        if (typeof eventName !== 'string') {
            throw new Error(`'eventName' must be string and cannot be null.`);
        }
        if (handler == null && typeof eventSource === 'function') {
            handler = eventSource;
            eventSource = undefined;
        }
        if (eventSource == null) {
            eventSource = '';
        }
        if (typeof eventSource !== 'string') {
            throw new Error(`'eventSource' must be string.`);
        }
        if (handler != null && typeof handler !== 'function') {
            throw new Error(`'handler' must be function.`);
        }
        let eventNameResult = parseEventName(eventName);
        if (eventNameResult.eventName) {
            if (__classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName]) {
                let namespaces = [];
                if (eventNameResult.namespace) {
                    namespaces.push(eventNameResult.namespace);
                }
                else {
                    namespaces = Object.keys(__classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName]);
                }
                for (let namespace of namespaces) {
                    let eventHandlers = __classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName][namespace][eventSource];
                    if (eventHandlers != null) {
                        if (handler != null) {
                            let matchIndex = eventHandlers.indexOf(handler);
                            if (matchIndex !== -1) {
                                eventHandlers.splice(matchIndex, 1);
                            }
                        }
                        else {
                            if (eventSource == null || eventSource === '') {
                                delete __classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName][namespace];
                                if (Object.keys(__classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName])
                                    .length === 0) {
                                    delete __classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName];
                                }
                            }
                            else {
                                if (Object.prototype.hasOwnProperty.call(__classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName][namespace], eventSource)) {
                                    delete __classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName][namespace][eventSource];
                                }
                            }
                        }
                    }
                }
            }
        }
        else {
            __classPrivateFieldSet(this, _EventedClass_eventHandlers, {}, "f");
        }
        return this;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    trigger(eventName, ...triggerArgs) {
        let eventSource = triggerArgs[0];
        let args = triggerArgs[1];
        if (typeof eventName !== 'string') {
            throw new Error(`'eventName' must be string and cannot be null.`);
        }
        if (args == null && Array.isArray(eventSource)) {
            args = eventSource;
            eventSource = undefined;
        }
        if (eventSource == null) {
            eventSource = '';
        }
        if (typeof eventSource !== 'string') {
            throw new Error(`If you are passing event \`args\` when triggering an event, they must be an array! If you are using 'eventSource', it must be the second parameter (before \`args\`) and must be string.`);
        }
        if (args == null) {
            args = [];
        }
        if (!Array.isArray(args)) {
            //throw new Error(`'args' must be Array.`);
            args = [args];
        }
        let eventNameResult = parseEventName(eventName);
        if (__classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName]) {
            let namespaces = Object.keys(__classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName]);
            for (let namespace of namespaces) {
                let eventHandlers = [];
                if (eventSource.length > 0) {
                    eventHandlers =
                        __classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName][namespace][eventSource] ||
                            [];
                    eventHandlers = eventHandlers.concat(__classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName][namespace][''] || []);
                }
                else {
                    let eventSources = __classPrivateFieldGet(this, _EventedClass_eventHandlers, "f")[eventNameResult.eventName][namespace];
                    for (let key of Object.keys(eventSources)) {
                        eventHandlers.push(...eventSources[key]);
                    }
                }
                for (let handler of eventHandlers) {
                    handler(...args);
                }
            }
        }
        return this;
    }
}
_EventedClass_eventHandlers = new WeakMap();
function parseEventName(eventName) {
    let result = {
        eventName: eventName,
        namespace: '',
    };
    let namespaceIndex = eventName.indexOf('.');
    if (namespaceIndex !== -1) {
        result.eventName = eventName.substr(0, namespaceIndex);
        result.namespace = eventName.substr(namespaceIndex + 1);
    }
    return result;
}
//# sourceMappingURL=evented-class.js.map