"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
/* eslint-disable jsx-a11y/anchor-is-valid */
var react_1 = tslib_1.__importStar(require("react"));
var locale_1 = require("@udesk-webapp/locale");
var viewerjs_1 = tslib_1.__importDefault(require("viewerjs"));
var file_icon_1 = tslib_1.__importDefault(require("../file-icon"));
var common_1 = require("../../utils/common");
var icon_1 = require("@udesk-webapp/icon");
var udesk_ui_1 = require("udesk-ui");
// 支持预览的图片类型
var isImage = function (name) {
    if (/\.(jpg|png|gif|jpeg|bmp|tif|psd)$/.test(name)) {
        return true;
    }
    return false;
};
// 支持预览的文档类型
var isDoc = function (name) {
    if (/.(doc|docx|odt|rtf|wps|ppt|pptx|odp|dp|xls|xlsx|ods|csv|et|pdf)$/.test(name)) {
        return true;
    }
    return false;
};
var FileItem = function (props) {
    var formatMessage = (0, locale_1.useTranslation)().t; // 多语言
    var file = props.file, _a = props.previewUrl, previewUrl = _a === void 0 ? '' : _a, _b = props.udeskVersion, udeskVersion = _b === void 0 ? '' : _b, _c = props.disabled, disabled = _c === void 0 ? false : _c, remove = props.remove, download = props.download;
    var name = (file === null || file === void 0 ? void 0 : file.file_name) || (file === null || file === void 0 ? void 0 : file.name) || '';
    var type = name.split('.').pop() || '';
    var ref = (0, react_1.useRef)();
    var viewer = (0, react_1.useRef)();
    (0, react_1.useLayoutEffect)(function () {
        if (!ref.current) {
            return;
        }
        viewer.current = new viewerjs_1.default(ref.current);
        return function () {
            var _a;
            (_a = viewer.current) === null || _a === void 0 ? void 0 : _a.destroy();
        };
    }, []);
    (0, react_1.useEffect)(function () {
        var _a;
        (_a = viewer.current) === null || _a === void 0 ? void 0 : _a.update();
    }, [file === null || file === void 0 ? void 0 : file.url]);
    // 图片及文档预览
    var onPreview = function () {
        var _a, _b;
        var u = file === null || file === void 0 ? void 0 : file.url;
        if (/\.(jpg|png|gif|jpeg|bmp|tif|psd)$/.test(name)) {
            (_b = (_a = viewer.current).view) === null || _b === void 0 ? void 0 : _b.call(_a, 0);
            return;
        }
        else if (previewUrl) {
            u = (0, common_1.getPreviewUrl)(file === null || file === void 0 ? void 0 : file.url, {
                previewUrl: previewUrl
            });
        }
        window.open(u, '_blank');
    };
    var renderProgress = function () {
        if ((file === null || file === void 0 ? void 0 : file.status) !== 'uploading') {
            return;
        }
        return (react_1.default.createElement(udesk_ui_1.Progress, { percent: (file === null || file === void 0 ? void 0 : file.percent) || 0, showInfo: false, strokeWidth: 2 }));
    };
    if (isImage(name) || (isDoc(name) && previewUrl && udeskVersion === 'pro')) {
        return (react_1.default.createElement("div", { ref: ref, className: 'form-upload-custom-file-item' },
            react_1.default.createElement(file_icon_1.default, { type: type }),
            react_1.default.createElement("span", null, name),
            react_1.default.createElement("img", { src: file === null || file === void 0 ? void 0 : file.url, style: { display: 'none' }, alt: name }),
            react_1.default.createElement("a", { className: "mgl-10", role: "button", "data-ignore-switch-editing": "1", onClick: onPreview }, formatMessage('labels.preview')),
            react_1.default.createElement("a", { href: file === null || file === void 0 ? void 0 : file.url, className: "mgl-5", "data-ignore-switch-editing": "1", target: "_blank", download: name, rel: "noreferrer", onClick: function (e) {
                    e.stopPropagation();
                    download === null || download === void 0 ? void 0 : download(file);
                } }, formatMessage('labels.download')),
            disabled ? null : (react_1.default.createElement("span", { className: 'form-upload-icon-delete', onClick: function () { return remove === null || remove === void 0 ? void 0 : remove(file); } },
                react_1.default.createElement(icon_1.TrashIcon, null))),
            react_1.default.createElement("div", null, renderProgress())));
    }
    return (react_1.default.createElement("div", { ref: ref, className: 'form-upload-custom-file-item' },
        react_1.default.createElement("a", { className: "link", href: file === null || file === void 0 ? void 0 : file.url, "data-ignore-switch-editing": "1", target: "_blank", rel: "noreferrer", onClick: function (e) {
                e.stopPropagation();
                download === null || download === void 0 ? void 0 : download(file);
            } },
            react_1.default.createElement(file_icon_1.default, { type: type }),
            name),
        disabled ? null : (react_1.default.createElement("span", { className: 'form-upload-icon-delete', onClick: function () { return remove === null || remove === void 0 ? void 0 : remove(file); } },
            react_1.default.createElement(icon_1.TrashIcon, null))),
        react_1.default.createElement("div", null, renderProgress())));
};
exports.default = FileItem;
