require('exists-sync');
const fs = require('fs');
const path = require('path');
const getGitInfo = require('git-repo-info');

module.exports = getReleaseTag;

function getReleaseTag(linkedPackages) {
    /* Sample data
    {
        sha: '718b42687d49d9b5028969556af159eca8727a51',
        abbreviatedSha: '718b42687d',
        branch: 'feature_CASE_911',
        tag: null,
        committer: '李凤宝 <lifengbao@udesk.cn>',
        committerDate: '2017-10-18T09:38:29.000Z',
        author: '李凤宝 <lifengbao@udesk.cn>',
        authorDate: '2017-10-18T09:38:29.000Z',
        commitMessage: 'Some commit message',
        root: '/home/user/Documents/udesk_ember_cli' 
    }
    */
    let gitInfo = getGitInfo();
    let today = new Date();
    let year = today.getFullYear().toString().substr(2);
    let month = (today.getMonth() + 1).toString().padStart(2, "0");
    let day = (today.getDate()).toString().padStart(2, "0");

    if (linkedPackages == null) {
        linkedPackages = [];
        let packageFile = path.join(fs.realpathSync(process.cwd()), "package.json");

        if (fs.existsSync(packageFile)) {
            let projectPackage = require(packageFile);
            let dependKeys = Object.keys(projectPackage.dependencies || {}).concat(Object.keys(projectPackage.devDependencies || {}));
            for (let i = 0; i < dependKeys.length; i++) {
                let key = dependKeys[i];
                let value = projectPackage.dependencies[key] || projectPackage.devDependencies[key];
                if (typeof value === "string" && value.startsWith("link:")) {
                    linkedPackages.push(key);
                }
            }
        }
    }


    let depends = [];
    if (linkedPackages.length > 0) {
        for (let i = 0; i < linkedPackages.length; i++) {
            let packageItem = linkedPackages[i];
            if (packageItem) {
                if (typeof packageItem === "string") {
                    packageItem = {
                        name: packageItem,
                        short: packageItem
                    };
                }
            }
            let packageGitInfo = getGitInfo(`./node_modules/${packageItem.name}`);
            depends.push(`${packageItem.short || packageItem.name}.${packageGitInfo.abbreviatedSha}`);
        }
    }
    let dependTags = (depends.length > 0 ? `@${depends.join("@")}` : "");
    return `${year}.${month}.${day}.${gitInfo.abbreviatedSha}${dependTags}`;
}
