import { __assign, __rest } from "tslib";
import React, { useCallback, useEffect, useRef, useState } from 'react';
import { DatePicker } from 'udesk-ui';
import FieldErrAndExplain from '@udesk-webapp/custom-field-error-explain';
import Moment from 'moment';
import { useValid } from '@udesk-webapp/custom-field-input-base';
import CopyComponent from '@udesk-webapp/copy-content';
import { YYYY_MM_DD, YYYY_MM_DD_HH_MM, YYYY_MM_DD_HH_MM_SS, YYYY_M_D, YYYY_M_D_HH_MM, YYYY_M_D_HH_MM_SS } from '@udesk-webapp/utils';
import enUS from 'antd/es/date-picker/locale/en_US';
import zhCN from 'antd/es/date-picker/locale/zh_CN';
import ReactDOM from 'react-dom';
var DATE_FORMAT = [YYYY_MM_DD, YYYY_MM_DD_HH_MM, YYYY_MM_DD_HH_MM_SS, YYYY_M_D, YYYY_M_D_HH_MM, YYYY_M_D_HH_MM_SS];
var Date = function (_a) {
    var onChange = _a.onChange, onBlur = _a.onBlur, v = _a.value, isFormItem = _a.isFormItem, explainShow = _a.explainShow, explain = _a.explain, switchFlag = _a.switchFlag, customValidPromise = _a.customValidPromise, picker = _a.picker, mode = _a.mode, _b = _a.language, language = _b === void 0 ? 'zh-cn' : _b, _c = _a.blurToSave, blurToSave = _c === void 0 ? true : _c, props = __rest(_a, ["onChange", "onBlur", "value", "isFormItem", "explainShow", "explain", "switchFlag", "customValidPromise", "picker", "mode", "language", "blurToSave"]);
    var _d = useState(v), value = _d[0], setValue = _d[1];
    var _e = useState(''), errMsg = _e[0], setErrMsg = _e[1];
    var dateRef = useRef(null);
    var isDateChange = useRef(false); // 解决配置了失焦保存时，点击此刻无效的问题。
    var noHidden = true;
    var handleValid = useValid({
        customValidPromise: customValidPromise,
        onSuccess: function () {
            var changeFunction = props['onBlur'];
            if (typeof changeFunction === 'function') {
                changeFunction(value);
            }
        },
        onError: function (error) {
            var err = error && error.message;
            setErrMsg(err);
        },
    });
    useEffect(function () {
        setValue(v || null);
    }, [v]);
    // 这部分逻辑重复较多,可以考虑提取
    var handleOnchange = useCallback(function (date, dateString) {
        // 解决ember传值过来时,由于异步渲染引起的光标抖动问题,后面可以考虑下其他解决方案
        // 或者传入数据处由react处理后可以考虑去除此代码,以保证数据流向清晰
        // const value = formatValue(date, true);
        setValue(dateString);
        onChange === null || onChange === void 0 ? void 0 : onChange(dateString);
        // 值为空时不变更，可能是删除按钮，不会触发OpenChange
        isDateChange.current = !!dateString;
    }, [onChange]);
    var handleValidValue = useCallback(function (event) {
        var dateValue = event.target.value;
        handleValid({
            value: dateValue,
            changeFunction: onBlur,
            isFormItem: !!isFormItem
        });
    }, [handleValid, isFormItem, onBlur]);
    var handleOnOpenChange = useCallback(function (open) {
        var _a;
        if (!open && dateRef.current) {
            var dateDom = ReactDOM.findDOMNode(dateRef.current);
            var dateInputDom = dateDom ? (_a = dateDom === null || dateDom === void 0 ? void 0 : dateDom.getElementsByTagName('input')) === null || _a === void 0 ? void 0 : _a[0] : null;
            var inputValue = (dateInputDom === null || dateInputDom === void 0 ? void 0 : dateInputDom.value) || "";
            // 验证input元素中value有值，判断是否与当前值相等,且是个有效时间
            if (inputValue && value !== inputValue && !isDateChange.current && Moment(inputValue, DATE_FORMAT).isValid()) {
                setValue(inputValue);
                onChange === null || onChange === void 0 ? void 0 : onChange(inputValue);
            }
            isDateChange.current = false;
        }
    }, [value, onChange]);
    return (React.createElement(FieldErrAndExplain, { isFormItem: isFormItem, explainShow: explainShow, explain: explain, errMsg: errMsg },
        React.createElement(CopyComponent, { text: formatValue(value), switchFlag: switchFlag, noHidden: noHidden },
            React.createElement(DatePicker, __assign({ ref: dateRef, onChange: handleOnchange, 
                // 这里兼容失焦保存
                onOpenChange: blurToSave ? handleOnOpenChange : undefined, onBlur: handleValidValue, value: formatValue(value, false), picker: picker, locale: (language === null || language === void 0 ? void 0 : language.toLowerCase()) === 'zh-cn' ? zhCN : enUS }, props)))));
};
Date.defaultProps = {
    format: DATE_FORMAT,
    getPopupContainer: function (triggerNode) { return triggerNode; },
    style: {},
    isFormItem: true
};
export default Date;
function formatValue(value, ifFormat) {
    if (ifFormat === void 0) { ifFormat = true; }
    if (!value) {
        return value;
    }
    if (ifFormat) {
        return Moment(value).format(YYYY_MM_DD);
    }
    else {
        return Moment(value);
    }
}
