import { __assign, __rest } from "tslib";
import React, { useCallback, useEffect, useRef, useState, useMemo } from 'react';
import { DatePicker } from 'udesk-ui';
import FieldErrAndExplain from '@udesk-webapp/custom-field-error-explain';
import Moment from 'moment';
import { YYYY_MM_DD_HH_MM, YYYY_MM_DD_HH_MM_SS, YYYY_M_D_HH_MM, YYYY_M_D_HH_MM_SS } from '@udesk-webapp/utils';
import CopyComponent from '@udesk-webapp/copy-content';
import { useValid } from '@udesk-webapp/custom-field-input-base';
import enUS from 'antd/es/date-picker/locale/en_US';
import zhCN from 'antd/es/date-picker/locale/zh_CN';
import ReactDOM from 'react-dom';
import { time2Locale, time2Service } from '@udesk-webapp/utils';
var DateTime = function (props) {
    var customValidPromise = props.customValidPromise, switchFlag = props.switchFlag, isFormItem = props.isFormItem, onBlur = props.onBlur, onChange = props.onChange, v = props.value, explainShow = props.explainShow, explain = props.explain, picker = props.picker, mode = props.mode, _a = props.language, language = _a === void 0 ? 'zh-cn' : _a, _b = props.blurToSave, blurToSave = _b === void 0 ? true : _b, _c = props.showTime, showTime = _c === void 0 ? { format: 'HH:mm' } : _c, needTransfer = props.needTransfer, restProps = __rest(props, ["customValidPromise", "switchFlag", "isFormItem", "onBlur", "onChange", "value", "explainShow", "explain", "picker", "mode", "language", "blurToSave", "showTime", "needTransfer"]);
    var _d = useState(v), value = _d[0], setValue = _d[1];
    var _e = useState(''), errMsg = _e[0], setErrMsg = _e[1];
    var dateRef = useRef(null);
    var isDateChange = useRef(false); // 解决配置了失焦保存时，点击此刻无效的问题。
    var noHidden = true;
    var handleValid = useValid({
        customValidPromise: customValidPromise,
        onSuccess: function () {
            var changeFunction = onBlur;
            if (typeof changeFunction === 'function') {
                changeFunction(value);
            }
        },
        onError: function (error) {
            var err = error && error.message;
            setErrMsg(err);
        },
    });
    useEffect(function () {
        setValue(v || null);
    }, [v]);
    // 这部分逻辑重复较多,可以考虑提取
    var handleOnchange = useCallback(function (datetime, dateString) {
        // 解决ember传值过来时,由于异步渲染引起的光标抖动问题,后面可以考虑下其他解决方案
        // 或者传入数据处由react处理后可以考虑去除此代码,以保证数据流向清晰
        // const value = formatValue(datetime, true);
        // 多时区场景需要转为服务器时间
        if (dateString && needTransfer) {
            dateString = time2Service(dateString, YYYY_MM_DD_HH_MM);
        }
        setValue(dateString);
        // 值为空时不变更，可能是删除按钮，不会触发OpenChange
        isDateChange.current = !!dateString;
        if (typeof onChange === 'function') {
            onChange(dateString);
        }
    }, [onChange, needTransfer]);
    var handleValidValue = useCallback(function (event) {
        var datetimeValue = event.target.value;
        // 多时区场景需要转为服务器时间
        if (datetimeValue && needTransfer) {
            datetimeValue = time2Service(datetimeValue, YYYY_MM_DD_HH_MM);
        }
        handleValid({
            value: datetimeValue,
            changeFunction: onBlur,
            isFormItem: !!isFormItem
        });
    }, [isFormItem, onBlur, handleValid, needTransfer]);
    var handleOnOpenChange = useCallback(function (open) {
        var _a;
        if (!open && dateRef.current) {
            var dateDom = ReactDOM.findDOMNode(dateRef.current);
            var dateInputDom = dateDom ? (_a = dateDom === null || dateDom === void 0 ? void 0 : dateDom.getElementsByTagName('input')) === null || _a === void 0 ? void 0 : _a[0] : null;
            var inputValue = (dateInputDom === null || dateInputDom === void 0 ? void 0 : dateInputDom.value) || "";
            // 多时区场景需要转为服务器时间
            if (inputValue && needTransfer) {
                inputValue = time2Service(inputValue, YYYY_MM_DD_HH_MM);
            }
            // 验证input元素中value有值，判断是否与当前值相等,且是个有效时间
            if (inputValue && value !== inputValue && !isDateChange.current && Moment(inputValue).isValid()) {
                setValue(inputValue);
                onChange === null || onChange === void 0 ? void 0 : onChange(inputValue);
            }
            isDateChange.current = false;
        }
    }, [value, onChange, needTransfer]);
    // 展示时间，传入组件的value为服务器时间，onChange时也需转为服务器时间
    var showValue = useMemo(function () {
        if (!value || !needTransfer) {
            return value;
        }
        return time2Locale(value, YYYY_MM_DD_HH_MM);
    }, [value, needTransfer]);
    return (React.createElement(FieldErrAndExplain, { errMsg: errMsg, isFormItem: isFormItem, explainShow: explainShow, explain: explain },
        React.createElement(CopyComponent, { text: formatValue(showValue), switchFlag: switchFlag, noHidden: noHidden },
            React.createElement(DatePicker, __assign({ ref: dateRef, onChange: handleOnchange, 
                // 这里兼容失焦保存
                onOpenChange: blurToSave ? handleOnOpenChange : undefined, onBlur: handleValidValue, value: formatValue(showValue, false), picker: picker, showTime: showTime, locale: (language === null || language === void 0 ? void 0 : language.toLowerCase()) === 'zh-cn' ? zhCN : enUS }, restProps)))));
};
DateTime.defaultProps = {
    format: [YYYY_MM_DD_HH_MM, YYYY_MM_DD_HH_MM_SS, YYYY_M_D_HH_MM, YYYY_M_D_HH_MM_SS],
    style: {},
    isFormItem: true
};
export default DateTime;
function formatValue(value, ifFormat) {
    if (ifFormat === void 0) { ifFormat = true; }
    if (!value) {
        return value;
    }
    if (ifFormat) {
        return Moment(value).format(YYYY_MM_DD_HH_MM);
    }
    else {
        return Moment(value);
    }
}
