import { __rest } from "tslib";
import React, { cloneElement, useCallback, useMemo, useRef } from 'react';
import AntTreeSelect from "udesk-ui/es/tree-select";
import Tag from 'udesk-ui/es/tag';
var TreeNode = AntTreeSelect === null || AntTreeSelect === void 0 ? void 0 : AntTreeSelect.TreeNode;
function renderTreeNode(selectOptions, parentCheckable) {
    if (!selectOptions) {
        return null;
    }
    return selectOptions.map(function (i) {
        var title = i === null || i === void 0 ? void 0 : i.title;
        if (i.color) {
            title = React.createElement(Tag, { color: i.color }, i.title);
        }
        return (React.createElement(TreeNode, { value: i.value, title: title, key: i.value, checkable: parentCheckable ? true : (i.node_type === 'leaf') }, renderTreeNode(i.subs, parentCheckable)));
    });
}
var tagRender = function (props) {
    var label = props.label, closable = props.closable, onClose = props.onClose;
    return cloneElement(label, { closable: closable, onClose: onClose });
};
var TagTree = function (_a) {
    var value = _a.value, onChange = _a.onChange, props = __rest(_a, ["value", "onChange"]);
    var treeNodes = useMemo(function () { return renderTreeNode(props.select_options, props.parent_checkable); }, [props.parent_checkable, props.select_options]);
    var ref = useRef(null);
    var getPopupContainer = useCallback(function () { return ref.current || document.body; }, []);
    var valueAry = useMemo(function () {
        if (typeof value === 'string') {
            return value.split(',');
        }
        else if (value) {
            return value;
        }
        else {
            return [];
        }
    }, [value]);
    return (React.createElement("div", { ref: ref, style: { position: 'relative' } },
        React.createElement(AntTreeSelect, { showSearch: true, style: { width: '100%' }, value: valueAry, dropdownStyle: { maxHeight: 400, overflow: 'auto' }, placeholder: props.placeholder, allowClear: true, treeCheckable: true, onChange: onChange, getPopupContainer: getPopupContainer, transitionName: '', treeDefaultExpandAll: props.default_expand_all, tagRender: tagRender, disabled: props.disabled }, treeNodes)));
};
export default TagTree;
