import { __rest, __spreadArray } from "tslib";
/* eslint-disable jsx-a11y/anchor-is-valid */
import React, { useCallback, useEffect, useState } from 'react';
import { Input, message } from 'udesk-ui';
import FieldWarning from '@udesk-webapp/field-warning';
import { ajax, PATH_BASE } from '@udesk-webapp/utils';
import { useTranslation } from '@udesk-webapp/locale';
import { MinusOutlined, PlusOutlined } from '@ant-design/icons';
import style from './index.module.scss';
// 手机正则
var mobilePtn = /^1[34578][0-9]{9}$/;
// 座机正则
var landlinePtn = /^(0[0-9]{2,3}-)([2-9][0-9]{6,7})+(-[0-9]{1,4})?$/;
var CellphoneField = function (_a) {
    var event = _a.event, value = _a.value, onChange = _a.onChange, placeholder = _a.placeholder, props = __rest(_a, ["event", "value", "onChange", "placeholder"]);
    // 多语言
    var t = useTranslation().t;
    var list = typeof (value) === 'string' ? value.split(',') : [];
    var _b = useState(list), emailList = _b[0], setEmailList = _b[1];
    var _c = useState([]), errMessage = _c[0], SetErrMessage = _c[1];
    var _d = useState([]), repeat = _d[0], SetRepeat = _d[1];
    // 进来第一次生成对应邮箱数量的报错信息
    useEffect(function () {
        SetErrMessage(__spreadArray([], new Array(list.length), true));
        SetRepeat(__spreadArray([], new Array(list.length), true));
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    // 新加邮箱时   改变外部form数据
    var addEmail = useCallback(function () {
        // props.onChange && props.onChange(emailList.concat('').join(','))
        setEmailList(function (v) { return __spreadArray(__spreadArray([], v, true), [''], false); });
        SetErrMessage(function (v) { return __spreadArray(__spreadArray([], v, true), [''], false); });
        SetRepeat(function (v) { return __spreadArray(__spreadArray([], v, true), [{}], false); });
    }, []);
    // 删除   删除邮箱  删除报错信息
    var deleteEmail = useCallback(function (index) {
        setEmailList(function (v) { return __spreadArray(__spreadArray([], v.slice(0, index), true), v.slice(index + 1), true); });
        SetErrMessage(function (v) { return __spreadArray(__spreadArray([], v.slice(0, index), true), v.slice(index + 1), true); });
    }, []);
    // 输入值改变时  改变相对应的数据
    var emailInputChange = useCallback(function (e, index) {
        setEmailList(__spreadArray(__spreadArray(__spreadArray([], emailList.slice(0, index), true), [e.target.value], false), emailList.slice(index + 1), true));
    }, [emailList]);
    // 失焦触发  校验邮箱  更改外部form
    var emailOnBlur = useCallback(function (index) {
        var value = emailList[index];
        // 查看是否重复输入同一手机号
        var valueNum = 0;
        emailList.forEach(function (item) {
            if (item === value) {
                valueNum += 1;
            }
        });
        // 如果有重复的数据  提示重复  并且将该值置空
        if (valueNum > 1) {
            message.warning('电话信息重复，请检查');
            emailList[index] = '';
            setEmailList(__spreadArray([], emailList, true));
            return;
        }
        // 校验电话号码格式
        if (value && !mobilePtn.test(value) && !landlinePtn.test(value)) {
            errMessage[index] = '号码格式不正确';
            SetErrMessage(__spreadArray([], errMessage, true));
            return;
        }
        errMessage[index] = '';
        SetErrMessage(__spreadArray([], errMessage, true));
        // 查询是否有重复数据
        ajax().getJSON("".concat(PATH_BASE, "/customers/check_duplication?customer%5Bcellphone%5D=").concat(value)).then(function (res) {
            if (res.duplications.cellphone) {
                repeat[index] = {
                    id: res.duplications.cellphone.id,
                    name: res.duplications.cellphone.name,
                };
                SetRepeat(__spreadArray([], repeat, true));
                // 有重复   将填写的置空
                emailList[index] = '';
                setEmailList(__spreadArray([], emailList, true));
            }
            else {
                repeat[index] = {};
                SetRepeat(__spreadArray([], repeat, true));
            }
        });
    }, [emailList, errMessage, repeat]);
    useEffect(function () {
        if (typeof onChange === 'function') {
            onChange(emailList.join(','));
        }
    }, [emailList, onChange]);
    // 点击重复信息时   跳转
    var clickRepeatWarning = useCallback(function (id) {
        // TODO: 用goToCustomer替代event
        if (typeof props.goToCustomer === 'function') {
            props.goToCustomer(id);
        }
        else if (typeof event.transitionToRoute === 'function') {
            event.transitionToRoute({
                route: 'crm.customer.show',
                path: id
            });
        }
    }, [event, props.goToCustomer]);
    return (React.createElement(React.Fragment, null, emailList.map(function (item, index) { return (
    // eslint-disable-next-line react/no-array-index-key
    React.createElement("div", { key: index, className: style['email-item-wrapper'] },
        React.createElement("div", { className: style['email-item-box'] },
            React.createElement(Input, { style: { flex: 1 }, placeholder: placeholder, value: item, onChange: function (e) { return emailInputChange(e, index); }, onBlur: function () { return emailOnBlur(index); } }),
            (function icon() {
                if (index) {
                    return (React.createElement("div", { className: style['add-icon'], onClick: function () { return deleteEmail(index); } },
                        React.createElement(MinusOutlined, null),
                        React.createElement("div", null, t('delete'))));
                }
                return (React.createElement("div", { className: style['add-icon'], onClick: addEmail },
                    React.createElement(PlusOutlined, null),
                    React.createElement("div", null, t('add'))));
            })()),
        errMessage[index] && React.createElement("div", { style: { marginTop: '5px' } },
            React.createElement(FieldWarning, { warningText: errMessage[index] })),
        repeat.length > 0 && repeat[index] && repeat[index].id && React.createElement("div", { style: { marginTop: '5px' } },
            React.createElement("div", { className: style['repeat-warning'] },
                React.createElement("div", { className: style['warning-img'] }),
                React.createElement("div", null,
                    "\u6B64\u7535\u8BDD\u4E0E\u5DF2\u6709\u5BA2\u6237",
                    React.createElement("a", { onClick: function () { return clickRepeatWarning(repeat[index].id); } }, repeat[index].name),
                    "\u91CD\u590D"))))); })));
};
export default CellphoneField;
