import { __assign, __rest } from "tslib";
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { TimePicker } from 'udesk-ui';
import FieldErrAndExplain from '@udesk-webapp/custom-field-error-explain';
import Moment from 'moment';
import { TIME_FORMAT_S } from '@udesk-webapp/utils';
import CopyComponent from '@udesk-webapp/copy-content';
import { useValid } from '@udesk-webapp/custom-field-input-base';
import ReactDOM from 'react-dom';
var Time = function (_a) {
    var customValidPromise = _a.customValidPromise, switchFlag = _a.switchFlag, isFormItem = _a.isFormItem, onBlur = _a.onBlur, onChange = _a.onChange, v = _a.value, explainShow = _a.explainShow, explain = _a.explain, mode = _a.mode, _b = _a.blurToSave, blurToSave = _b === void 0 ? true : _b, props = __rest(_a, ["customValidPromise", "switchFlag", "isFormItem", "onBlur", "onChange", "value", "explainShow", "explain", "mode", "blurToSave"]);
    var _c = useState(v), value = _c[0], setValue = _c[1];
    var _d = useState(''), errMsg = _d[0], setErrMsg = _d[1];
    var dateRef = useRef(null);
    var isDateChange = useRef(false); // 解决配置了失焦保存时，点击此刻无效的问题。
    var handleValid = useValid({
        customValidPromise: customValidPromise,
        onSuccess: function () {
            var changeFunction = onBlur;
            if (typeof changeFunction === 'function') {
                changeFunction(value);
            }
        },
        onError: function (error) {
            var err = error && error.message;
            setErrMsg(err);
        },
    });
    useEffect(function () {
        setValue(v || null);
    }, [v]);
    // 这部分逻辑重复较多,可以考虑提取
    var handleOnchange = useCallback(function (time, timeString) {
        // 解决ember传值过来时,由于异步渲染引起的光标抖动问题,后面可以考虑下其他解决方案
        // 或者传入数据处由react处理后可以考虑去除此代码,以保证数据流向清晰
        // const value = formatValue(time, true);
        setValue(timeString);
        // 值为空时不变更，可能是删除按钮，不会触发OpenChange
        isDateChange.current = !!timeString;
        if (typeof onChange === 'function') {
            onChange(timeString);
        }
    }, [onChange]);
    var handleValidValue = useCallback(function (event) {
        var timeValue = event.target.value;
        // window.console.log(time,value)
        handleValid({
            value: timeValue,
            changeFunction: onBlur,
            isFormItem: isFormItem
        });
    }, [onBlur, isFormItem]);
    var handleOnOpenChange = useCallback(function (open) {
        var _a;
        if (!open && dateRef.current) {
            var dateDom = ReactDOM.findDOMNode(dateRef.current);
            var dateInputDom = dateDom ? (_a = dateDom === null || dateDom === void 0 ? void 0 : dateDom.getElementsByTagName('input')) === null || _a === void 0 ? void 0 : _a[0] : null;
            // const reTime = /^(?:(?:[0-2][0-3])|(?:[0-1][0-9])):[0-5][0-9]:[0-5][0-9]$/;
            var inputValue = (dateInputDom === null || dateInputDom === void 0 ? void 0 : dateInputDom.value) || "";
            // 验证input元素中value有值，判断是否与当前值相等,且是个有效时间
            if (inputValue && value !== inputValue && !isDateChange.current && Moment(inputValue, TIME_FORMAT_S).isValid()) {
                setValue(inputValue);
                onChange === null || onChange === void 0 ? void 0 : onChange(inputValue);
            }
            isDateChange.current = false;
        }
    }, [value, onChange]);
    var copyChildren = useMemo(function () { return (React.createElement(TimePicker, __assign({ ref: dateRef, onChange: handleOnchange, 
        // 这里兼容失焦保存
        onOpenChange: blurToSave ? handleOnOpenChange : undefined, onBlur: handleValidValue, value: formatValue(value, false) }, props))); }, [handleOnchange, handleValidValue, props, value]);
    return (React.createElement(FieldErrAndExplain, { errMsg: errMsg, isFormItem: isFormItem, explainShow: explainShow, explain: explain },
        React.createElement(CopyComponent, { text: formatValue(value), switchFlag: switchFlag }, copyChildren)));
};
Time.defaultProps = {
    format: TIME_FORMAT_S,
    isFormItem: true,
    style: {}
};
export default Time;
function formatValue(value, ifFormat) {
    if (ifFormat === void 0) { ifFormat = true; }
    if (!value) {
        return value;
    }
    if (ifFormat) {
        return Moment(value, TIME_FORMAT_S).format(TIME_FORMAT_S);
    }
    else {
        return Moment(value, TIME_FORMAT_S);
    }
}
