import { CssItem, JSItem, SdkManifest } from './types';
import UdeskSdkClass, { SdkClassConstructor } from './udesk-sdk';
/**
 * 浏览器兼容性
 * 由于用到了 URL，所以只能向下兼容到IE10
 *
 * 原则：
 * 可以使用ES6语法，但不要使用太新的ES6 API，例如Object.entries, array.reduce
 */
/**
 * 基于Umi框架的新版sdk loader
 *
 * @export
 * @class ManifestLoader
 * @template TSdk SDK类型
 */
export declare class ManifestLoader<TSdk extends UdeskSdkClass> {
    #private;
    constructor(manifestUrl: string, options?: Options);
    /**
     * ManifestLoader是否已经被销毁。
     *
     * 被销毁后的ManifestLoader不可再使用，需要再重新创建一个实例
     *
     * @readonly
     * @type {boolean}
     * @memberof ManifestLoader
     */
    get isDestroyed(): boolean;
    /**
     * 获取manifest文件的内容
     *
     * @return {*}  {Promise<SdkManifest>}
     * @memberof ManifestLoader
     */
    manifest(): Promise<SdkManifest>;
    /**
     * 获取manifest中的资源，包含js和css等
     *
     * @return {*}  {Promise<AssetResult<TSdk>[]>}
     * @memberof ManifestLoader
     */
    assets(): Promise<AssetResult<TSdk>[]>;
    /**
     * 销毁实例
     *
     * @memberof ManifestLoader
     */
    destroy(): void;
    _loadScript(jsFile: JSItem): Promise<AssetResult<TSdk>>;
    _loadCss(cssFile: CssItem): void;
}
export declare type AssetResult<TSdk extends UdeskSdkClass = UdeskSdkClass> = {
    url?: string;
    content?: string;
    isEntry: boolean;
    isPreload?: boolean;
    SdkClass?: SdkClassConstructor<TSdk>;
    ComponentSdkClass?: SdkClassConstructor<TSdk>;
};
export declare type Options = {};
