var _ManifestLoader_manifestUrl, _ManifestLoader_manifestPromise, _ManifestLoader_assetsPromise, _ManifestLoader_options, _ManifestLoader_isDestroyed;
import { __awaiter, __classPrivateFieldGet, __classPrivateFieldSet } from "tslib";
/**
 * 浏览器兼容性
 * 由于用到了 URL，所以只能向下兼容到IE10
 *
 * 原则：
 * 可以使用ES6语法，但不要使用太新的ES6 API，例如Object.entries, array.reduce
 */
/**
 * 基于Umi框架的新版sdk loader
 *
 * @export
 * @class ManifestLoader
 * @template TSdk SDK类型
 */
export class ManifestLoader {
    constructor(manifestUrl, options) {
        _ManifestLoader_manifestUrl.set(this, void 0);
        _ManifestLoader_manifestPromise.set(this, void 0);
        _ManifestLoader_assetsPromise.set(this, void 0);
        _ManifestLoader_options.set(this, void 0);
        _ManifestLoader_isDestroyed.set(this, false);
        if (!manifestUrl) {
            throw new Error(`'manifestUrl' must be an valid url.`);
        }
        __classPrivateFieldSet(this, _ManifestLoader_manifestUrl, new URL(manifestUrl, location.href).href, "f");
        __classPrivateFieldSet(this, _ManifestLoader_options, options || {}, "f");
    }
    /**
     * ManifestLoader是否已经被销毁。
     *
     * 被销毁后的ManifestLoader不可再使用，需要再重新创建一个实例
     *
     * @readonly
     * @type {boolean}
     * @memberof ManifestLoader
     */
    get isDestroyed() {
        return __classPrivateFieldGet(this, _ManifestLoader_isDestroyed, "f");
    }
    /**
     * 获取manifest文件的内容
     *
     * @return {*}  {Promise<SdkManifest>}
     * @memberof ManifestLoader
     */
    manifest() {
        if (!__classPrivateFieldGet(this, _ManifestLoader_manifestPromise, "f")) {
            __classPrivateFieldSet(this, _ManifestLoader_manifestPromise, new Promise((resolve, reject) => {
                const manifestUrl = __classPrivateFieldGet(this, _ManifestLoader_manifestUrl, "f");
                const xhr = new XMLHttpRequest();
                xhr.open('GET', manifestUrl + `?_=${Date.now()}`, true);
                xhr.onload = () => {
                    // 如果实例被销毁了，则不resolve了。虽然对于本组件来说，继续执行回调没问题，但有可能外层组件也已经销毁了，
                    // 如果事件继续往外传递的话，父组件可能会报错
                    if (__classPrivateFieldGet(this, _ManifestLoader_isDestroyed, "f")) {
                        return;
                    }
                    try {
                        resolve(JSON.parse(xhr.responseText));
                    }
                    catch (error) {
                        reject({
                            errorType: ErrorType.invalidManifest,
                            responseText: xhr.responseText,
                        });
                        return;
                    }
                };
                xhr.onerror = () => {
                    // 如果实例被销毁了，则不resolve了。虽然对于本组件来说，继续执行回调没问题，但有可能外层组件也已经销毁了，
                    // 如果事件继续往外传递的话，父组件可能会报错
                    if (__classPrivateFieldGet(this, _ManifestLoader_isDestroyed, "f")) {
                        return;
                    }
                    reject({
                        errorType: ErrorType.serverError,
                    });
                };
                xhr.ontimeout = () => {
                    // 如果实例被销毁了，则不resolve了。虽然对于本组件来说，继续执行回调没问题，但有可能外层组件也已经销毁了，
                    // 如果事件继续往外传递的话，父组件可能会报错
                    if (__classPrivateFieldGet(this, _ManifestLoader_isDestroyed, "f")) {
                        return;
                    }
                    reject({
                        errorType: ErrorType.timeout,
                    });
                };
                xhr.onabort = () => {
                    // 如果实例被销毁了，则不resolve了。虽然对于本组件来说，继续执行回调没问题，但有可能外层组件也已经销毁了，
                    // 如果事件继续往外传递的话，父组件可能会报错
                    if (__classPrivateFieldGet(this, _ManifestLoader_isDestroyed, "f")) {
                        return;
                    }
                    reject({
                        errorType: ErrorType.abort,
                    });
                };
                xhr.send();
            }), "f");
        }
        return __classPrivateFieldGet(this, _ManifestLoader_manifestPromise, "f");
    }
    /**
     * 获取manifest中的资源，包含js和css等
     *
     * @return {*}  {Promise<AssetResult<TSdk>[]>}
     * @memberof ManifestLoader
     */
    assets() {
        function getPreload(preloadJs, loadScript) {
            if (preloadJs.length) {
                return Promise.all(preloadJs === null || preloadJs === void 0 ? void 0 : preloadJs.map((jsFile) => loadScript(jsFile)));
            }
            else {
                return Promise.resolve();
            }
        }
        if (!__classPrivateFieldGet(this, _ManifestLoader_assetsPromise, "f")) {
            __classPrivateFieldSet(this, _ManifestLoader_assetsPromise, new Promise((resolve, reject) => {
                this.manifest()
                    .then((manifest) => {
                    // 仅加载css文件，不必监听onLoad事件
                    if (manifest.css) {
                        manifest.css.forEach((cssFile) => this._loadCss(cssFile));
                    }
                    // 加载js文件，并监听所有的onLoad事件
                    if (manifest.js) {
                        const preloadJs = manifest.js.filter((jsFile) => jsFile.isPreload);
                        const mainLoadJs = manifest.js.filter((jsFile) => !jsFile.isPreload);
                        getPreload(preloadJs, this._loadScript.bind(this)).then(() => {
                            Promise.all(mainLoadJs === null || mainLoadJs === void 0 ? void 0 : mainLoadJs.map((jsFile) => this._loadScript(jsFile)))
                                .then((values) => {
                                // 如果实例被销毁了，则不resolve了。虽然对于本组件来说，继续执行回调没问题，但有可能外层组件也已经销毁了，
                                // 如果事件继续往外传递的话，父组件可能会报错
                                if (__classPrivateFieldGet(this, _ManifestLoader_isDestroyed, "f")) {
                                    return;
                                }
                                resolve(values);
                            })
                                .catch((error) => {
                                // 如果实例被销毁了，则不resolve了。虽然对于本组件来说，继续执行回调没问题，但有可能外层组件也已经销毁了，
                                // 如果事件继续往外传递的话，父组件可能会报错
                                if (__classPrivateFieldGet(this, _ManifestLoader_isDestroyed, "f")) {
                                    return;
                                }
                                reject(error);
                            });
                        });
                    }
                    else {
                        return [];
                    }
                })
                    .catch((error) => {
                    // 如果实例被销毁了，则不resolve了。虽然对于本组件来说，继续执行回调没问题，但有可能外层组件也已经销毁了，
                    // 如果事件继续往外传递的话，父组件可能会报错
                    if (__classPrivateFieldGet(this, _ManifestLoader_isDestroyed, "f")) {
                        return;
                    }
                    reject(error);
                });
            }), "f");
        }
        return __classPrivateFieldGet(this, _ManifestLoader_assetsPromise, "f");
    }
    /**
     * 销毁实例
     *
     * @memberof ManifestLoader
     */
    destroy() {
        // todo: 组件销毁时把script和link移除掉，可以避免内存泄露？
        // 但移除以后就没有缓存效果了，每次加载sdk还需要重新下载资源，先暂缓吧
        // if (this.#manifest) { }
        __classPrivateFieldSet(this, _ManifestLoader_manifestPromise, undefined, "f");
        __classPrivateFieldSet(this, _ManifestLoader_assetsPromise, undefined, "f");
        __classPrivateFieldSet(this, _ManifestLoader_isDestroyed, true, "f");
    }
    _loadScript(jsFile) {
        return __awaiter(this, void 0, void 0, function* () {
            let script = document.createElement('script');
            if (jsFile.content) {
                script.innerHTML = jsFile.content;
                return {
                    content: jsFile.content,
                    isEntry: jsFile.isEntry,
                    isPreload: jsFile.isPreload,
                };
            }
            else if (jsFile.src) {
                return new Promise((resolve, reject) => {
                    var _a;
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    const url = new URL(jsFile.src, __classPrivateFieldGet(this, _ManifestLoader_manifestUrl, "f")).href;
                    // 不要尝试从现有script中获取，因为如果在系统初始化时就预加载sdk的话，
                    // 有可能在渲染sdk时，sdkjs并没有加载完成，此时获取不到sdkClass
                    const scripts = document.getElementsByTagName('script');
                    let loadedScript = null;
                    for (let i = 0; i < scripts.length; i++) {
                        if (((_a = scripts[i]) === null || _a === void 0 ? void 0 : _a.src.indexOf(url)) === 0) {
                            // 已经加载过了
                            if (scripts[i].sdkClass) {
                                // sdkClass存在
                                resolve({
                                    url: url,
                                    isEntry: jsFile.isEntry,
                                    SdkClass: scripts[i].sdkClass,
                                    ComponentSdkClass: scripts[i].componentSdkClass,
                                });
                                return;
                            }
                            else if (jsFile.isPreload) {
                                resolve({
                                    url: url,
                                    isEntry: jsFile.isEntry,
                                    SdkClass: scripts[i].sdkClass,
                                    ComponentSdkClass: scripts[i].componentSdkClass,
                                });
                            }
                            else {
                                loadedScript = scripts[i];
                            }
                        }
                    }
                    if (loadedScript) {
                        // 已经加载过了，但是sdkClass还没有生成，则复用其onload事件
                        script = loadedScript;
                    }
                    else {
                        Object.assign(script, jsFile);
                        script.src = url;
                        document.body.appendChild(script);
                    }
                    script.addEventListener('load', () => {
                        if (script.sdkClass) {
                            // 如果同步设置了sdkClass，则直接返回
                            resolve({
                                url: url,
                                isEntry: jsFile.isEntry,
                                SdkClass: script.sdkClass,
                                ComponentSdkClass: script.componentSdkClass,
                            });
                        }
                        else if (script.__mfsu) {
                            // 开启了mfsu，可能就是异步加载的，需要注册 onSdkLoaded 事件来接收sdkClass
                            script.onSdkLoaded = ({ sdkClass, componentSdkClass, }) => {
                                resolve({
                                    url: url,
                                    isEntry: jsFile.isEntry,
                                    SdkClass: sdkClass,
                                    ComponentSdkClass: componentSdkClass,
                                });
                            };
                        }
                        else {
                            // 否则就是普通的js，没有sdkClass
                            resolve({
                                url: url,
                                isEntry: jsFile.isEntry,
                                SdkClass: undefined,
                                ComponentSdkClass: undefined,
                            });
                        }
                    });
                    script.addEventListener('error', () => {
                        reject({
                            url: url,
                            isEntry: jsFile.isEntry,
                            errorType: ErrorType.serverError,
                        });
                    });
                    script.addEventListener('abort', () => {
                        reject({
                            url: url,
                            isEntry: jsFile.isEntry,
                            errorType: ErrorType.abort,
                        });
                    });
                });
            }
            else {
                return {
                    url: jsFile.src,
                    isEntry: jsFile.isEntry,
                };
            }
        });
    }
    _loadCss(cssFile) {
        if (cssFile.content) {
            const style = document.createElement('style');
            style.innerHTML = cssFile.content;
            document.head.appendChild(style);
        }
        else if (cssFile.href) {
            const url = new URL(cssFile.href, __classPrivateFieldGet(this, _ManifestLoader_manifestUrl, "f")).href;
            const link = document.createElement('link');
            const links = document.getElementsByTagName('link');
            for (let i = 0; i < links.length; i++) {
                if (links[i].href.indexOf(url) === 0) {
                    // 已经加载过了
                    return;
                }
            }
            link.rel = 'stylesheet';
            Object.assign(link, cssFile);
            link.href = url;
            document.head.appendChild(link);
        }
        else {
            // TODO :
        }
    }
}
_ManifestLoader_manifestUrl = new WeakMap(), _ManifestLoader_manifestPromise = new WeakMap(), _ManifestLoader_assetsPromise = new WeakMap(), _ManifestLoader_options = new WeakMap(), _ManifestLoader_isDestroyed = new WeakMap();
// eslint-disable-next-line no-unused-vars
var ErrorType;
(function (ErrorType) {
    // eslint-disable-next-line no-unused-vars
    ErrorType["serverError"] = "serverError";
    // eslint-disable-next-line no-unused-vars
    ErrorType["invalidManifest"] = "invalidManifest";
    // eslint-disable-next-line no-unused-vars
    ErrorType["timeout"] = "timeout";
    // eslint-disable-next-line no-unused-vars
    ErrorType["abort"] = "abort";
})(ErrorType || (ErrorType = {}));
//# sourceMappingURL=manifest-loader.js.map