var _a, _ReactSentry_initialized;
import { __classPrivateFieldGet, __classPrivateFieldSet } from "tslib";
import Raven from 'raven-js';
import { getRunTimeEnvironment } from './env';
const ENV = getRunTimeEnvironment();
const defaults = {
    dsn: '',
    debug: false,
    releaseVersion: '',
    userIdField: 'id',
    userNameField: 'name',
    userEmailField: 'email',
    ignoreClientOnlyErrors: false,
    maxResponseTextLength: 0,
    ignoreResponseCodes: [],
};
export class ReactSentry {
    static init(options) {
        if (__classPrivateFieldGet(this, _a, "f", _ReactSentry_initialized)) {
            return;
            // throw new Error("Should not call ReactSentry's init method twice.");
        }
        options = Object.assign(Object.assign({}, defaults), options);
        let { dsn } = options;
        if (!dsn) {
            throw new Error(`options.dsn 不能为空.`);
        }
        this.options = options;
        try {
            let dsn = this.options.dsn;
            if (dsn) {
                Raven.config(dsn, {
                    environment: ENV,
                    debug: this.options.debug,
                    // whitelistUrls: [new RegExp(RegExp.escape(process.env.STATIC_LINK), "i")],
                    release: this.options.releaseVersion,
                });
                Raven.install();
            }
            __classPrivateFieldSet(this, _a, true, "f", _ReactSentry_initialized);
        }
        catch (err) {
            // eslint-disable-next-line no-console
            console.warn('Error during `sentry` initialization: ' + err.message, err);
        }
    }
    static get initialized() {
        return __classPrivateFieldGet(ReactSentry, _a, "f", _ReactSentry_initialized);
    }
    static ready() {
        try {
            let user = {};
            if (this.options.getUser) {
                user = this.options.getUser() || {};
            }
            Raven.setUserContext(Object.assign({
                /* Sentry reserved names */
                id: user[this.options.userIdField],
                username: user[this.options.userNameField],
                email: user[this.options.userEmailField],
                // ip_address: ""
            }, user));
            // Set tags context
            let envTagsContext = {
                environment: ENV,
            };
            let tagsContext = {};
            if (this.options.getTagsContext && typeof this.options.getTagsContext === 'function') {
                tagsContext = this.options.getTagsContext() || {};
            }
            Raven.setTagsContext(Object.assign(envTagsContext, tagsContext, {
                hostname: window.location.hostname,
            }));
            // Set extra context
            let extraContext = {};
            if (this.options.getExtraContext) {
                extraContext = this.options.getExtraContext() || {};
            }
            Raven.setExtraContext(extraContext);
            Raven.setDataCallback(function (data) {
                // Executes before `shouldSendCallback`!
                // do something to data
                data.server_name = window.location.hostname;
                if (data.tags) {
                    data.tags['message'] = data.message;
                    data.tags['create.time'] = formatDate(new Date());
                }
                return data;
            });
            Raven.setShouldSendCallback((data) => {
                if (this.options.getShouldSendCallback &&
                    typeof this.options.getShouldSendCallback === 'function') {
                    if (!this.options.getShouldSendCallback(data)) {
                        return false;
                    }
                }
                if (data != null && data.extra != null && data.extra.__isAjaxResponse === true) {
                    let extra = data.extra;
                    delete extra.__isAjaxResponse;
                    if (this.options.ignoreClientOnlyErrors &&
                        extra.response &&
                        extra.response.visible === true) {
                        return false;
                    }
                    if (this.options.ignoreResponseCodes &&
                        this.options.ignoreResponseCodes.length > 0 &&
                        this.options.ignoreResponseCodes.includes(extra.statusCode)) {
                        return false;
                    }
                }
                return true;
            });
            this._enableGlobalErrorCatching();
        }
        catch (err) {
            this.captureException(err);
        }
    }
    static captureMessage(message, options) {
        if (__classPrivateFieldGet(this, _a, "f", _ReactSentry_initialized)) {
            Raven.captureMessage(message, options);
        }
        else {
            throw new Error(message);
        }
    }
    static _enableGlobalErrorCatching() {
        // todo: 支持 umi-request
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        if (window.jQuery) {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            //@ts-ignore
            window.jQuery(document).ajaxError((event, jqXHR, ajaxSettings, thrownError) => {
                let requestData = ajaxSettings.data;
                if (!/^GET$/i.test(ajaxSettings.type)) {
                    /* eslint-disable no-empty */
                    try {
                        requestData = JSON.parse(ajaxSettings.data);
                    }
                    catch (err) { }
                    /* eslint-enable no-empty */
                }
                let requestPath = ajaxSettings.url.replace(/\?.*/, '');
                let requestProtocol = null;
                let requestHostName = null;
                if (window.URL) {
                    let ajaxUrl = ajaxSettings.url;
                    if (!ajaxUrl) {
                        ajaxUrl = window.location.href;
                    }
                    if (!ajaxUrl.match(/http[s]?/i)) {
                        ajaxUrl = window.location.origin + ajaxUrl;
                    }
                    try {
                        let parsedUrl = new URL(ajaxUrl);
                        requestProtocol = parsedUrl.protocol;
                        requestHostName = parsedUrl.hostname;
                    }
                    catch (err) {
                        requestProtocol = null;
                        requestHostName = null;
                    }
                }
                else {
                    let match = ajaxSettings.url.match(/^((\w+?:)\/\/([^/]+))/i);
                    if (match) {
                        requestProtocol = match[2];
                        requestHostName = match[3];
                    }
                }
                let captureTags = this._getCaptureTags(jqXHR);
                let options = {
                    extra: Object.assign({}, {
                        __isAjaxResponse: true,
                        url: ajaxSettings.url,
                        method: ajaxSettings.type,
                        dataType: ajaxSettings.dataType,
                        contentType: ajaxSettings.contentType,
                        data: requestData,
                        statusCode: jqXHR.status,
                        error: thrownError || jqXHR.statusText,
                        response: '',
                        readyState: jqXHR.readyState,
                        async: ajaxSettings.async,
                        processData: ajaxSettings.processData,
                        global: ajaxSettings.global,
                        crossDomain: ajaxSettings.crossDomain,
                        headers: jqXHR.getAllResponseHeaders(),
                    }),
                    tags: Object.assign({}, captureTags, {
                        'ajax.url': requestPath,
                        'ajax.protocol': requestProtocol,
                        'ajax.hostname': requestHostName,
                        'ajax.code': jqXHR.status,
                    }),
                    fingerprint: [
                        window.location.protocol +
                            '//' +
                            window.location.host +
                            window.location.pathname,
                        requestPath,
                        ajaxSettings.type,
                        jqXHR.status,
                        captureTags['error.from'],
                        captureTags['error.client.ajax.reason'],
                        captureTags['error.server.exception.type'],
                    ],
                };
                let shrinkResponseText = (responseText) => {
                    if (this.options.maxResponseTextLength > 0) {
                        return responseText.substring(0, this.options.maxResponseTextLength);
                    }
                    else {
                        return responseText;
                    }
                };
                if (typeof ajaxSettings.dataType === 'string' &&
                    ajaxSettings.dataType.match(/^json$/i)) {
                    try {
                        options.extra.response = getResponseJson(jqXHR);
                    }
                    catch (err) {
                        options.extra.response = shrinkResponseText(jqXHR.responseText);
                    }
                }
                else {
                    options.extra.response = shrinkResponseText(jqXHR.responseText);
                }
                this.captureMessage(jqXHR.errorMsg || jqXHR._serverMsg || this._extractMessage(jqXHR), options);
            });
        }
    }
    static _extractMessage(reason) {
        const defaultMessage = 'Unhandled Promise error detected';
        /* eslint-disable indent */
        switch (typeof reason) {
            case 'string':
                return reason;
            case 'object':
                return reason.message || defaultMessage;
            default:
                return defaultMessage;
        }
        /* eslint-enable indent */
    }
    static _getCaptureTags(error) {
        let tags = {};
        if (isJqXHR(error)) {
            let jqXHR = error;
            if (jqXHR.status === 0 && jqXHR.readyState === 0) {
                tags['error.from'] = 'client';
                if (jqXHR.statusText === 'abort') {
                    tags['error.client.ajax.reason'] = 'abort';
                }
                else {
                    tags['error.client.ajax.reason'] = 'Maybe bad network';
                }
            }
            else {
                tags['error.from'] = 'server';
                let response = getResponseJson(jqXHR, false);
                if (response != null && response.visible === true) {
                    tags['error.server.exception.type'] = 'validation';
                }
                else {
                    tags['error.server.exception.type'] = 'exception';
                }
            }
        }
        else {
            tags['error.from'] = 'client';
        }
        return tags;
    }
}
_a = ReactSentry;
ReactSentry.options = defaults;
_ReactSentry_initialized = { value: false };
ReactSentry.captureException = (error, options) => {
    if (__classPrivateFieldGet(_a, _a, "f", _ReactSentry_initialized)) {
        Raven.captureException(error, options);
    }
    else {
        throw error;
    }
};
function isJqXHR(jqXHR) {
    return (typeof jqXHR.then === 'function' &&
        typeof jqXHR.success === 'function' &&
        typeof jqXHR.fail === 'function');
}
function getResponseJson(jqXHR, throwError = true) {
    if (throwError === true) {
        return jqXHR.responseJSON || JSON.parse(jqXHR.responseText);
    }
    else {
        try {
            return jqXHR.responseJSON || JSON.parse(jqXHR.responseText);
        }
        catch (err) {
            return null;
        }
    }
}
function formatDate(date) {
    let year = padNumber(date.getFullYear());
    let month = padNumber(date.getMonth() + 1);
    let day = padNumber(date.getDate());
    let hour = padNumber(date.getHours());
    let minute = padNumber(date.getMinutes());
    let second = padNumber(date.getSeconds());
    return `${year}-${month}-${day} ${hour}:${minute}:${second}`;
}
function padNumber(value) {
    if (value < 10) {
        return '0' + value;
    }
    else {
        return value + '';
    }
}
export default ReactSentry;
//# sourceMappingURL=react-sentry.js.map