import React, { useState, useEffect, useCallback } from 'react';
import Classnames from 'classnames';
import style from './style.less';
import { useTranslation } from '@udesk-webapp/locale';
import { ajax, PATH_BASE } from '@udesk-webapp/utils';
import { Spin, Select, Radio } from 'udesk-ui';
import TicketFilterConditions from './ticket-filter-conditions';
var TicketFilters = function (props) {
    var t = useTranslation().t;
    var _a = props.filterUrl, filterUrl = _a === void 0 ? PATH_BASE + "/init_data/filters" : _a, _b = props.showFilter, showFilter = _b === void 0 ? true : _b, getFilters = props.getFilters, 
    // conditions,
    filters = props.filters, _c = props.filterField, filterField = _c === void 0 ? 'activeTicketFilters' : _c, // 接口返回过滤器数据属性名称
    _d = props.filterText, // 接口返回过滤器数据属性名称
    filterText = _d === void 0 ? t('components.ticketFilterTable.ticketFilters') : _d, _e = props.showCondition, showCondition = _e === void 0 ? true : _e, allConditions = props.allConditions, anyConditions = props.anyConditions, dataSources = props.dataSources, ticketCustomFields = props.ticketCustomFields, _f = props.ticketCountField, ticketCountField = _f === void 0 ? 'ticket_count' : _f, onSearch = props.onSearch, onCancelSearch = props.onCancelSearch, onFilterChang = props.onFilterChang;
    var _g = useState([]), innerFilters = _g[0], setInnerFilters = _g[1];
    var _h = useState(false), filterLoading = _h[0], setFilterLoading = _h[1];
    var _j = useState(undefined), currentFilterId = _j[0], setCurrentFilterId = _j[1];
    var _k = useState(allConditions), innerAllConditions = _k[0], setInnerAllConditions = _k[1];
    var _l = useState(anyConditions), innerAnyConditions = _l[0], setInnerAnyConditions = _l[1];
    // 赋值innerFilters,暂没有支持filters受控，如果需要后期扩展下
    useEffect(function () {
        // 外部传递过滤器列表
        if (filters) {
            setInnerFilters(filters || []);
        }
        else {
            // 内部查询或者用外部查询方法
            setFilterLoading(true);
            if (getFilters && typeof getFilters === 'function') {
                var getFiltersPromise = getFilters();
                if (typeof getFiltersPromise.then === 'function') {
                    getFilters()
                        .then(function (resp) {
                        // resp:FilterInterface[]
                        setInnerFilters(resp);
                    })
                        .finally(function () {
                        setFilterLoading(false);
                    });
                }
            }
            else {
                ajax()
                    .getJSON(filterUrl)
                    .then(function (resp) {
                    var _a;
                    if (resp === null || resp === void 0 ? void 0 : resp[filterField]) {
                        var filters_1 = (_a = resp === null || resp === void 0 ? void 0 : resp[filterField]) === null || _a === void 0 ? void 0 : _a.map(function (filter) {
                            return {
                                title: filter === null || filter === void 0 ? void 0 : filter.name,
                                value: filter === null || filter === void 0 ? void 0 : filter.id,
                                label: (filter === null || filter === void 0 ? void 0 : filter.name) + "\uFF08" + (filter === null || filter === void 0 ? void 0 : filter[ticketCountField || 0]) + "\uFF09",
                                allConditions: filter === null || filter === void 0 ? void 0 : filter.all_conditions,
                                anyConditions: filter === null || filter === void 0 ? void 0 : filter.any_conditions,
                            };
                        });
                        setInnerFilters(filters_1 || []);
                        // TODO:设置默认选中过滤器
                    }
                    else {
                        setInnerFilters([]);
                    }
                })
                    .finally(function () {
                    setFilterLoading(false);
                });
            }
        }
    }, []);
    useEffect(function () { }, []);
    // 暂时非受控
    // const innerCurrentFilterId = useMemo(() => {
    //     return currentFilter?.id;
    // }, [currentFilter]);
    var onSearched = useCallback(function (_a) {
        var allConditions = _a.allConditions, anyConditions = _a.anyConditions;
        if (typeof onSearch === 'function') {
            onSearch({
                allConditions: allConditions,
                anyConditions: anyConditions,
                filterId: currentFilterId,
            });
        }
    }, [currentFilterId]);
    var onCancelSearched = useCallback(function () {
        if (typeof onCancelSearch === 'function') {
            onCancelSearch();
        }
    }, []);
    var filterChangedAction = useCallback(function (filterId, allConditions, anyConditions) {
        setCurrentFilterId(filterId);
        setInnerAllConditions(allConditions);
        setInnerAnyConditions(anyConditions);
        if (typeof onFilterChang === 'function') {
            onFilterChang(filterId, allConditions, anyConditions);
        }
    }, [innerFilters, onFilterChang]);
    // 过滤器变化
    var filterChanged = useCallback(function (filterId) {
        if (!filterId) {
            return;
        }
        // setCurrentFilterId(filterId);
        var allConditions = [];
        var anyConditions = [];
        innerFilters === null || innerFilters === void 0 ? void 0 : innerFilters.forEach(function (filter) {
            if (filterId && (filter === null || filter === void 0 ? void 0 : filter.value) === filterId) {
                allConditions = filter === null || filter === void 0 ? void 0 : filter.allConditions;
                anyConditions = filter === null || filter === void 0 ? void 0 : filter.anyConditions;
            }
        });
        filterChangedAction(filterId, allConditions, anyConditions);
    }, [innerFilters, filterChangedAction]);
    // 过滤器选择，过滤器个数小于4个的情况
    var filterSelect = useCallback(function (event) {
        var _a;
        filterChanged((_a = event === null || event === void 0 ? void 0 : event.target) === null || _a === void 0 ? void 0 : _a.value);
    }, [filterChanged]);
    return (React.createElement("div", { className: Classnames('ticket-filters', style['ticket-filters']) },
        React.createElement("div", { className: Classnames('ticket-filters-head', style['ticket-filters-head']) },
            React.createElement("div", { className: Classnames('ticket-filters-left', style['ticket-filters-left']) }, showFilter ? (React.createElement(React.Fragment, null,
                React.createElement("span", { className: Classnames('ticket-filters-filter-text', style['ticket-filters-filter-text']) }, filterText),
                React.createElement(Spin, { spinning: filterLoading },
                    (innerFilters === null || innerFilters === void 0 ? void 0 : innerFilters.length) <= 4 ? (React.createElement(Radio.Group
                    // @ts-ignore
                    , { 
                        // @ts-ignore
                        options: innerFilters, value: currentFilterId, optionType: "button", buttonStyle: "solid", onChange: filterSelect })) : null,
                    (innerFilters === null || innerFilters === void 0 ? void 0 : innerFilters.length) > 4 ? (React.createElement(Select, { placeholder: t('tips.label.pleaseSelect'), value: currentFilterId, 
                        // @ts-ignore
                        className: style['filter-select'], 
                        // @ts-ignore
                        options: innerFilters, onChange: filterChanged })) : null))) : null),
            React.createElement("div", { className: Classnames('ticket-filters-right', style['ticket-filters-right']) }, showCondition ? (React.createElement(TicketFilterConditions, { ticketCustomFields: ticketCustomFields, dataSources: dataSources, allConditions: innerAllConditions, anyConditions: innerAnyConditions, onSearch: onSearched, onCancelSearch: onCancelSearched })) : null))));
};
export default TicketFilters;
