import { __assign } from "tslib";
import React, { useState, useEffect, useCallback, useMemo } from 'react';
import Classnames from 'classnames';
import style from './style.less';
import i18n from '@udesk-webapp/locale';
import { ajax } from '@udesk-webapp/utils';
import { Spin, Select, Radio } from 'udesk-ui';
import TicketFilterConditions from './ticket-filter-conditions';
export * from './ticket-filter-condition-utils';
import { formatCondition, unFormatCondition } from './ticket-filter-condition-utils';
var TicketFilters = function (props) {
    var _a = props.filterUrl, filterUrl = _a === void 0 ? "/cs/ts/api/init_data/filters" : _a, _b = props.showFilter, showFilter = _b === void 0 ? true : _b, filterLists = props.filterLists, getFilters = props.getFilters, 
    // conditions,
    filters = props.filters, _c = props.filterField, filterField = _c === void 0 ? 'activeTicketFilters' : _c, // 接口返回过滤器数据属性名称
    _d = props.filterText, // 接口返回过滤器数据属性名称
    filterText = _d === void 0 ? i18n.t('components.ticketFilterTable.ticketFilters') : _d, _e = props.showCondition, showCondition = _e === void 0 ? true : _e, allConditions = props.allConditions, anyConditions = props.anyConditions, dataSources = props.dataSources, ticketCustomFields = props.ticketCustomFields, _f = props.ticketCountField, ticketCountField = _f === void 0 ? 'ticket_count' : _f, _g = props.filterMaxNumber, filterMaxNumber = _g === void 0 ? 4 : _g, systemFields = props.systemFields, _h = props.useFormatCondition, useFormatCondition = _h === void 0 ? true : _h, onSearch = props.onSearch, onCancelSearch = props.onCancelSearch, onFilterChang = props.onFilterChang, filterClassName = props.filterClassName, removalIconClass = props.removalIconClass, addIconClass = props.addIconClass, addIconText = props.addIconText;
    var _j = useState([]), innerFilters = _j[0], setInnerFilters = _j[1];
    var _k = useState(false), filterLoading = _k[0], setFilterLoading = _k[1];
    var _l = useState(undefined), currentFilterId = _l[0], setCurrentFilterId = _l[1];
    var _m = useState(allConditions), innerAllConditions = _m[0], setInnerAllConditions = _m[1];
    var _o = useState(anyConditions), innerAnyConditions = _o[0], setInnerAnyConditions = _o[1];
    // 赋值innerFilters,暂没有支持filters受控，如果需要后期扩展下
    useEffect(function () {
        // 外部传递过滤器列表
        if (filters) {
            setInnerFilters(filters || []);
        }
        else {
            // 内部查询或者用外部查询方法
            setFilterLoading(true);
            if (filterLists) {
                setInnerFilters(filterLists);
                setFilterLoading(false);
            }
            else if (getFilters && typeof getFilters === 'function') {
                var getFiltersPromise = getFilters();
                if (typeof getFiltersPromise.then === 'function') {
                    getFilters()
                        .then(function (resp) {
                        setInnerFilters(resp);
                    })
                        .finally(function () {
                        setFilterLoading(false);
                    });
                }
            }
            else {
                ajax()
                    .getJSON(filterUrl)
                    .then(function (resp) {
                    var _a;
                    if (resp === null || resp === void 0 ? void 0 : resp[filterField]) {
                        var filters_1 = (_a = resp === null || resp === void 0 ? void 0 : resp[filterField]) === null || _a === void 0 ? void 0 : _a.map(function (filter) {
                            return {
                                title: filter === null || filter === void 0 ? void 0 : filter.name,
                                value: filter === null || filter === void 0 ? void 0 : filter.id,
                                label: "".concat(filter === null || filter === void 0 ? void 0 : filter.name, "\uFF08").concat((filter === null || filter === void 0 ? void 0 : filter[ticketCountField]) || 0, "\uFF09"),
                                allConditions: filter === null || filter === void 0 ? void 0 : filter.all_conditions,
                                anyConditions: filter === null || filter === void 0 ? void 0 : filter.any_conditions,
                            };
                        });
                        setInnerFilters(filters_1 || []);
                        // TODO:设置默认选中过滤器
                    }
                    else {
                        setInnerFilters([]);
                    }
                })
                    .finally(function () {
                    setFilterLoading(false);
                });
            }
        }
    }, []);
    useEffect(function () {
        setInnerAllConditions(allConditions);
    }, [allConditions]);
    useEffect(function () {
        setInnerAnyConditions(anyConditions);
    }, [anyConditions]);
    // 暂时非受控
    // const innerCurrentFilterId = useMemo(() => {
    //     return currentFilter?.id;
    // }, [currentFilter]);
    var onSearched = useCallback(function (_a) {
        var allConditions = _a.allConditions, anyConditions = _a.anyConditions;
        if (typeof onSearch === 'function') {
            if (useFormatCondition) {
                onSearch({
                    allConditions: unFormatCondition(allConditions, 'field_name', 'value', true),
                    anyConditions: unFormatCondition(anyConditions, 'field_name', 'value', true),
                    filterId: currentFilterId,
                });
            }
            else {
                onSearch({
                    allConditions: allConditions,
                    anyConditions: anyConditions,
                    filterId: currentFilterId,
                });
            }
        }
    }, [currentFilterId, useFormatCondition]);
    var onCancelSearched = useCallback(function () {
        if (typeof onCancelSearch === 'function') {
            onCancelSearch();
        }
    }, []);
    var filterChangedAction = useCallback(function (filterId, allConditions, anyConditions) {
        setCurrentFilterId(filterId);
        if (useFormatCondition) {
            // 轉換公司和客戶的值
            formatCondition(allConditions, 'field_name', 'value').finally(function () {
                return setInnerAllConditions(allConditions);
            });
            formatCondition(anyConditions, 'field_name', 'value').finally(function () {
                return setInnerAnyConditions(anyConditions);
            });
        }
        else {
            setInnerAllConditions(allConditions);
            setInnerAnyConditions(anyConditions);
        }
        if (typeof onFilterChang === 'function') {
            onFilterChang({ filterId: filterId, allConditions: allConditions, anyConditions: anyConditions });
        }
    }, [innerFilters, useFormatCondition, onFilterChang]);
    // 过滤器变化
    var filterChanged = useCallback(function (filterId) {
        if (!filterId) {
            return;
        }
        // setCurrentFilterId(filterId);
        var allConditions = [];
        var anyConditions = [];
        innerFilters === null || innerFilters === void 0 ? void 0 : innerFilters.forEach(function (filter) {
            var _a, _b;
            if (filterId && (filter === null || filter === void 0 ? void 0 : filter.value) === filterId) {
                allConditions = (_a = filter === null || filter === void 0 ? void 0 : filter.allConditions) === null || _a === void 0 ? void 0 : _a.map(function (condition) {
                    if (!condition) {
                        return condition;
                    }
                    return __assign({}, condition);
                });
                anyConditions = (_b = filter === null || filter === void 0 ? void 0 : filter.anyConditions) === null || _b === void 0 ? void 0 : _b.map(function (condition) {
                    if (!condition) {
                        return condition;
                    }
                    return __assign({}, condition);
                });
            }
        });
        filterChangedAction(filterId, allConditions, anyConditions);
    }, [innerFilters, filterChangedAction]);
    // 过滤器选择，过滤器个数小于4个的情况
    var filterSelect = useCallback(function (event) {
        var _a;
        filterChanged((_a = event === null || event === void 0 ? void 0 : event.target) === null || _a === void 0 ? void 0 : _a.value);
    }, [filterChanged]);
    var filterOptions = useMemo(function () {
        if (innerFilters) {
            return innerFilters.map(function (filter) {
                return {
                    title: filter === null || filter === void 0 ? void 0 : filter.title,
                    value: filter === null || filter === void 0 ? void 0 : filter.value,
                    label: filter === null || filter === void 0 ? void 0 : filter.label,
                };
            });
        }
        else {
            return innerFilters;
        }
    }, [innerFilters]);
    return (React.createElement("div", { className: Classnames('ticket-filters', style['ticket-filters'], "".concat(filterClassName)) },
        React.createElement("div", { className: Classnames('ticket-filters-head', style['ticket-filters-head']) },
            React.createElement("div", { className: Classnames('ticket-filters-left', style['ticket-filters-left']) }, showFilter ? (React.createElement(React.Fragment, null,
                React.createElement("span", { className: Classnames('ticket-filters-filter-text', style['ticket-filters-filter-text']) }, filterText),
                React.createElement(Spin, { spinning: filterLoading }, (filterOptions === null || filterOptions === void 0 ? void 0 : filterOptions.length) <= filterMaxNumber ? (React.createElement(Radio.Group
                // @ts-ignore
                , { 
                    // @ts-ignore
                    options: filterOptions, value: currentFilterId, optionType: 'button', buttonStyle: 'solid', onChange: filterSelect })) : (React.createElement(Select, { placeholder: i18n.t('tips.label.pleaseSelect'), value: currentFilterId, 
                    // @ts-ignore
                    className: style['filter-select'], 
                    // @ts-ignore
                    options: filterOptions, onChange: filterChanged }))))) : null),
            React.createElement("div", { className: Classnames('ticket-filters-right', style['ticket-filters-right']) }, showCondition ? (React.createElement(TicketFilterConditions, { systemFields: systemFields, ticketCustomFields: ticketCustomFields, dataSources: dataSources, allConditions: innerAllConditions, anyConditions: innerAnyConditions, onSearch: onSearched, onCancelSearch: onCancelSearched, removalIconClass: removalIconClass, addIconClass: addIconClass, addIconText: addIconText, filterClassName: filterClassName })) : null))));
};
export default TicketFilters;
