/*
 * 工单自定义过滤器组件，`值`的选项大多通过dataSources属性传递
 * 如果发现默写`值`字段的选项不正确，可以优先考虑dataSources是否传递正确
 * 支持的配置项如PropsInterface所列
 *
 */
import { __assign } from "tslib";
import React, { useMemo, useCallback } from 'react';
import Udesk from 'udesk-react/src/udesk';
import CustomFilters from 'udesk-react/src/components/custom-filters';
import { ajax, PATH_BASE, CS_PATH_BASE, } from '@udesk-webapp/utils';
import { businessRuleOperatorEnums, inputTypeEnums } from '@udesk-webapp/utils';
import i18n from '@udesk-webapp/locale';
// import { useIntl, request } from 'umi';
import getComponentsConfigTable from './node-component-settings';
import { FieldAntdSelect, OperatorAntdSelect as OperatorAntdSelectComponent, } from '@udesk-webapp/custom-filter';
// import { OperatorAntdSelectComponent } from '@/components/custom-filter-input/operator-antd-select';
import Classname from 'classnames';
import style from './style.less';
import { createCustomerName } from '../ticket-filter-condition-utils';
export * from '../ticket-filter-condition-utils';
Udesk.init();
// 模板查询缓存
var templatePromise;
var tagPromise;
var knowledgeQuestionPromise;
var TicketFilterCondition = function (props) {
    /**
     * 系统字段的格式参考的自定义字段，现在为可配，为了未来支持自定义系统字段名称
     * 留意下自定义字段接口中的系统字段，两个应该保持一致
     * @type {*}
     *
     */
    var DEFAULT_TICKET_NAME = i18n.t('utils.ticket.ticket') + '：';
    var DEFAULT_CUSTOMER_NAME = i18n.t('utils.ticket.customer') + '：';
    var DEFAULT_SYSTEM_FIELDS = [
        {
            name: 'field_num',
            title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.fieldNum'),
            content_type: 'field_num',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'customer_id',
            title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.customer'),
            content_type: 'customer_id',
            field_owner: 'Ticket',
            comment: '',
            // id: '', // id有值么？
            // agent_permission: null,
            // customer_permission: null,
            // is_relate_duplicate: null,
            // is_search: null,
            // is_unique: null,
            // sensitive_mark_placeholder: null,
            // sensitive_mark_position: null,
            // sensitive_mark_size: null,
            // sensitive_mode: null,
            // tooltip: null,
        },
        {
            name: 'status',
            title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.status'),
            content_type: 'status',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'priority',
            content_type: 'priority',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'platform',
            content_type: 'platform',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'source',
            content_type: 'source',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'subject',
            content_type: 'subject',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'agent_group_id',
            title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.agentGroup'),
            content_type: 'agent_group_id',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'first_assignee_id',
            title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.firstAssigneeName'),
            content_type: 'first_assignee_id',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'assignee_id',
            title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.assignee'),
            content_type: 'assignee_id',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'created_at',
            title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.createdAt'),
            content_type: 'created_at',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'updated_at',
            title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.updatedAt'),
            content_type: 'updated_at',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'satisfaction',
            title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.satisfaction'),
            content_type: 'satisfaction',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'template_id',
            title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.template'),
            content_type: 'template_id',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'tags',
            content_type: 'tags',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'followers',
            content_type: 'followers',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'closed_at',
            content_type: 'closed_at',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'solving_at',
            content_type: 'solving_at',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'resolved_at',
            content_type: 'resolved_at',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'forward_email',
            content_type: 'forward_email',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'last_replier_type',
            content_type: 'last_replier_type',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'last_replied_at',
            content_type: 'last_replied_at',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'last_closed_operator_identifier',
            content_type: 'last_closed_operator_identifier',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'last_agent_replied_at',
            content_type: 'last_agent_replied_at',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'last_customer_replied_at',
            content_type: 'last_customer_replied_at',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'updator_id',
            title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.updator'),
            content_type: 'updator_id',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'creator_id',
            title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.creator'),
            content_type: 'creator_id',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'parse_content_char_split',
            content_type: 'parse_content_char_split',
            field_owner: 'Ticket',
            comment: '',
        },
        {
            name: 'replies',
            content_type: 'replies',
            field_owner: 'Ticket',
            comment: '',
        },
    ];
    var DEFAULT_CUSTOMER_SYSTEM_FIELDS = [
        // {
        //     name: 'customer_id',
        //     title: DEFAULT_TICKET_NAME + i18n.t('utils.ticket.customer'),
        //     content_type: 'customer_id',
        //     field_owner: 'Ticket',
        //     comment: '',
        // },
        {
            name: 'Customer#level',
            title: DEFAULT_CUSTOMER_NAME + i18n.t('utils.ticket.level'),
            content_type: 'Customer#level',
            field_owner: 'Customer',
            comment: '',
        },
        {
            name: 'organization',
            content_type: 'organization',
            field_owner: 'Customer',
            comment: '',
        },
        {
            name: 'after_created_hours',
            content_type: 'after_created_hours',
            field_owner: 'Customer',
            comment: '',
        },
        {
            name: 'after_solving_hours',
            content_type: 'after_solving_hours',
            field_owner: 'Customer',
            comment: '',
        },
        {
            name: 'after_resolved_hours',
            content_type: 'after_resolved_hours',
            field_owner: 'Customer',
            comment: '',
        },
        {
            name: 'after_closed_hours',
            content_type: 'after_closed_hours',
            field_owner: 'Customer',
            comment: '',
        },
        {
            name: 'after_assigned_hours',
            content_type: 'after_assigned_hours',
            field_owner: 'Customer',
            comment: '',
        },
        {
            name: 'after_updated_hours',
            content_type: 'after_updated_hours',
            field_owner: 'Customer',
            comment: '',
        },
        {
            name: 'after_assignee_updated_hours',
            content_type: 'after_assignee_updated_hours',
            field_owner: 'Customer',
            comment: '',
        },
        {
            name: 'after_creator_updated_hours',
            content_type: 'after_creator_updated_hours',
            field_owner: 'Customer',
            comment: '',
        },
    ];
    var DEFAULT_COMPONENTS_CONFIG = getComponentsConfigTable();
    // TODO:提取到public,多语言实现
    var DEFAULT_SATISFACTION = [
        {
            id: 1,
            name: '非常满意',
        },
        {
            id: 2,
            name: '满意',
        },
        {
            id: 3,
            name: '一般',
        },
        {
            id: 4,
            name: '不满意',
        },
        {
            id: 5,
            name: '非常不满意',
        },
    ];
    var DEFAULT_STATUS = [
        { key: 'open', id: 1, name: '开启' },
        { key: 'solving', id: 4, name: '解决中' },
        { key: 'resolved', id: 2, name: '已解决' },
        { key: 'closed', id: 3, name: '已关闭' },
    ];
    var DEFAULT_PRIORITY = [
        { key: 'urgency', id: 1, name: '紧急' },
        { key: 'high', id: 2, name: '高' },
        { key: 'medium', id: 3, name: '标准' },
        { key: 'low', id: 4, name: '低' },
    ];
    var DEFAULT_PLATFORMS = [
        { id: 1, key: 'email', name: '邮件' },
        { id: 2, key: 'weibo', name: '微博' },
        { id: 3, key: 'wechat', name: '微信' },
        { id: 4, key: 'im', name: '即时聊天' },
        { id: 99, key: 'mini_video', name: '视频通话小程序' },
        { id: 5, key: 'call', name: '电话' },
        { id: 6, key: 'feedback', name: '反馈标签' },
        { id: 7, key: 'hc', name: '帮助中心' },
        { id: 8, key: 'manual_input', name: '手工录入' },
        { id: 9, key: 'api', name: 'API' },
        { id: 10, key: 'ticket_plugin', name: '工单插件' },
        { id: 11, key: 'weixin_mini', name: '小程序' },
        { id: 12, key: 'qywx', name: '企业微信' },
        { id: 13, key: 'weixin_kf', name: '微信客服号' },
    ];
    var showConditionNumber = props.showConditionNumber, _a = props.conditionFieldField, conditionFieldField = _a === void 0 ? 'field_name' : _a, _b = props.conditionOperatorField, conditionOperatorField = _b === void 0 ? 'operation' : _b, _c = props.conditionValueField, conditionValueField = _c === void 0 ? 'value' : _c, autoChooseFirstField = props.autoChooseFirstField, showConditionHeaders = props.showConditionHeaders, showConditionHeadersWhenEmpty = props.showConditionHeadersWhenEmpty, conditionFieldColumnHeader = props.conditionFieldColumnHeader, conditionOperatorColumnHeader = props.conditionOperatorColumnHeader, conditionValueColumnHeader = props.conditionValueColumnHeader, _d = props.allowEmptyConditions, allowEmptyConditions = _d === void 0 ? true : _d, onConditionChanged = props.onConditionChanged, fieldsConfig = props.fieldsConfig, customFields = props.customFields, _e = props.systemFields, systemFields = _e === void 0 ? transformLocale(DEFAULT_SYSTEM_FIELDS) : _e, _f = props.customerSystemFields, customerSystemFields = _f === void 0 ? transformLocale(DEFAULT_CUSTOMER_SYSTEM_FIELDS) : _f, dataSources = props.dataSources, conditions = props.conditions, className = props.className, removalIconClass = props.removalIconClass, addIconClass = props.addIconClass, addIconText = props.addIconText;
    // const [conditionQuerying, setConditionQuerying] = useState<boolean>(false);
    // const [innerConditions, setInnerConditions] = useState<ConditionInterface[] | undefined>();
    var conditionFieldComponentAttrs = useMemo(function () {
        return {
            nameField: 'text',
            valueField: 'name',
            allowClear: false,
            showSearch: true,
            optionFilterProp: 'children',
            width: '200px',
        };
    }, []);
    var operatorComponentAttrs = useMemo(function () {
        return {
            nameField: 'name',
            valueField: 'id',
            allowClear: false,
            width: '100px',
        };
    }, []);
    // 默认datasources
    var DEFAULT_DATASOURCES = useMemo(function () {
        return {
            satisfaction: DEFAULT_SATISFACTION,
            status: DEFAULT_STATUS,
            priority: DEFAULT_PRIORITY,
            platforms: DEFAULT_PLATFORMS,
            sources: new Promise(function (resolve, reject) {
                ajax()
                    .getJSON("/cs/ts/api/tickets/get_ticket_source")
                    .then(function (resp) {
                    if (resp === null || resp === void 0 ? void 0 : resp.data) {
                        var data = resp.data;
                        var formatData = makeGroup({
                            valueField: 'source_channel',
                            nameField: 'show_content',
                            makeGroupNewArr: data,
                        });
                        resolve(formatData);
                    }
                    else {
                        resolve([]);
                    }
                }, function (reason) {
                    reject(reason);
                });
            }),
            templates: getTemplate(),
            tags: getTags(),
            knowledgeCatalog: getKnowledgeCatalog(),
            recentDate: getRecentDate(),
        };
    }, []);
    var innerDataSources = __assign(__assign({}, DEFAULT_DATASOURCES), dataSources);
    var fields = useMemo(function () {
        var formatCustomFields = customFields === null || customFields === void 0 ? void 0 : customFields.map(function (field) {
            return __assign(__assign({}, field), { title: "\u5DE5\u5355\uFF1A".concat(field === null || field === void 0 ? void 0 : field.title) });
        });
        var ticketFields = getCustomFilterFieldsConfig(fieldsConfig, systemFields, formatCustomFields, innerDataSources);
        var customerfields = getCustomFilterFieldsConfig(fieldsConfig, customerSystemFields, [], innerDataSources);
        return ticketFields.concat(customerfields);
    }, [fieldsConfig, systemFields, customFields, innerDataSources]);
    var conditionChanged = useCallback(function (conditions) {
        if (typeof onConditionChanged === 'function') {
            // onConditionChanged(unFormatCondition(conditions));
            onConditionChanged(conditions);
        }
    }, []);
    var fieldValueSettings = useMemo(function () {
        return DEFAULT_COMPONENTS_CONFIG;
    }, []);
    // 处理客户/公司等反显
    // useEffect(() => {
    //     console.time('init')
    //     formatCondition(conditions);
    //     console.timeEnd('init')
    // }, []);
    // useEffect(() => {
    //     console.time('condition change')
    //     formatCondition(conditions);
    //     console.timeEnd('condition change')
    // }, [conditions, formatCondition]);
    return (React.createElement(React.Fragment, null,
        React.createElement(CustomFilters, { theme: Classname(style['ticket-filter-condition'], className), conditions: conditions, fields: fields, fieldValueSettings: fieldValueSettings, conditionFieldComponent: FieldAntdSelect, conditionFieldComponentAttrs: conditionFieldComponentAttrs, operatorComponent: OperatorAntdSelectComponent, operatorComponentAttrs: operatorComponentAttrs, inputTypeIdField: 'key', conditionFieldField: conditionFieldField, conditionOperatorField: conditionOperatorField, conditionValueField: conditionValueField, showConditionNumber: showConditionNumber, autoChooseFirstField: autoChooseFirstField, showConditionHeaders: showConditionHeaders, showConditionHeadersWhenEmpty: showConditionHeadersWhenEmpty, conditionFieldColumnHeader: conditionFieldColumnHeader, conditionOperatorColumnHeader: conditionOperatorColumnHeader, conditionValueColumnHeader: conditionValueColumnHeader, allowEmptyConditions: allowEmptyConditions, onChanged: conditionChanged, removalIconClass: removalIconClass, addIconClass: addIconClass, addIconText: addIconText })));
};
export default TicketFilterCondition;
// 获取字段配置，系统字段和自定义字段，可以提到公共方法里面
/**
 * 根据字段配置生成自定义过滤器条件
 *
 * @param {(Partial<CustomField>[] | undefined)} fields
 * @return {*}
 */
function createFieldsCondition(fields, dataSources) {
    if (!fields) {
        return [];
    }
    return fields
        .map(function (field) {
        var name = field.name, title = field.title, content_type = field.content_type, _a = field.select_options, select_options = _a === void 0 ? [] : _a;
        var customFieldItem;
        if (content_type) {
            switch (content_type) {
                case 'text':
                case 'link':
                case 'area_text':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdText',
                        contentType: content_type,
                        operators: [
                            'is',
                            'not',
                            'contain',
                            'not_contain',
                            'start_with',
                            'end_with',
                            'blank',
                            'present',
                        ],
                        dataSource: null,
                        attributes: {},
                    };
                    break;
                case 'location':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdText',
                        contentType: content_type,
                        operators: ['is', 'not', 'contain', 'not_contain', 'blank', 'present'],
                        dataSource: null,
                        attributes: {},
                    };
                    break;
                case 'date':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdDate',
                        contentType: content_type,
                        operators: [
                            'is',
                            'not',
                            'before',
                            'after',
                            'range',
                            'not_range',
                            'blank',
                            'present',
                            'recent',
                        ],
                        dataSource: dataSources.recentDate,
                        attributes: {},
                    };
                    break;
                case 'time':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdTime',
                        contentType: content_type,
                        operators: [
                            'is',
                            'not',
                            'before',
                            'after',
                            'range',
                            'not_range',
                            'blank',
                            'present',
                        ],
                        dataSource: null,
                        attributes: {},
                    };
                    break;
                case 'datetime':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdDateTime',
                        contentType: content_type,
                        operators: [
                            'is',
                            'not',
                            'before',
                            'after',
                            'range',
                            'not_range',
                            'blank',
                            'present',
                            'recent',
                        ],
                        dataSource: dataSources.recentDate,
                        attributes: {},
                    };
                    break;
                case 'numeric':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdNumeric',
                        contentType: content_type,
                        operators: ['is', 'not', 'gt', 'lt', 'blank', 'present'],
                        dataSource: null,
                        attributes: {},
                    };
                    break;
                case 'number':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdNumber',
                        contentType: content_type,
                        operators: ['is', 'not', 'gt', 'lt', 'blank', 'present'],
                        dataSource: null,
                        attributes: {
                            type: 'number',
                            precision: 0,
                        },
                    };
                    break;
                case 'droplist':
                case 'radio':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'should', 'not_should', 'blank', 'present'],
                        dataSource: select_options,
                        attributes: {
                            nameField: 'title',
                            valueField: 'value',
                        },
                    };
                    break;
                case 'chained_droplist':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdCascade',
                        contentType: content_type,
                        operators: ['is', 'not', 'prefix', 'not_prefix', 'blank', 'present'],
                        dataSource: select_options,
                        attributes: {},
                    };
                    break;
                case 'checkbox':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdMultipleSelect',
                        contentType: content_type,
                        operators: [
                            'is',
                            'not',
                            'should',
                            'not_should',
                            'include',
                            'not_include',
                            'blank',
                            'present',
                        ],
                        dataSource: select_options,
                        attributes: {},
                    };
                    break;
                // 系统字段
                case 'customer_id':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'blank', 'present'],
                        dataSource: null,
                        attributes: {
                            onSearch: function (value) {
                                return searchCustomer({ value: value });
                            },
                            onPopupScroll: function (event, params) {
                                var target = event.target;
                                if (target.scrollTop + target.offsetHeight ===
                                    target.scrollHeight) {
                                    return searchCustomer(params);
                                }
                            },
                            labelInValue: true,
                            filterOption: false,
                            showSearch: true,
                            nameField: 'formatName',
                            valueField: 'id',
                        },
                    };
                    break;
                case 'priority':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'should', 'not_should'],
                        dataSource: dataSources.priority,
                        attributes: {
                            valueField: 'key',
                            nameField: 'name',
                        },
                    };
                    break;
                case 'Customer#level':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not'],
                        dataSource: dataSources.status,
                        attributes: {
                            valueField: 'key',
                            nameField: 'name',
                        },
                    };
                    break;
                case 'status':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdTicketStatus',
                        contentType: content_type,
                        operators: ['is', 'not', 'should', 'not_should'],
                        dataSource: dataSources.status,
                        attributes: {
                            valueField: 'key',
                            nameField: 'name',
                        },
                    };
                    break;
                // TODO:参考之前的规则优化
                case 'platform':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'should', 'not_should', 'blank', 'present'],
                        dataSource: dataSources.platforms,
                        attributes: {
                            valueField: 'key',
                            nameField: 'name',
                        },
                    };
                    break;
                // TODO:参考之前的规则优化
                case 'source':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'should', 'not_should', 'blank', 'present'],
                        dataSource: dataSources.sources,
                        attributes: {
                            valueField: 'source_channel',
                            nameField: 'show_content',
                            groupValueField: 'groupType',
                            groupNameField: 'groupTitle',
                            groupDataField: 'groupData',
                            isGroup: true,
                        },
                    };
                    break;
                case 'subject':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdText',
                        contentType: content_type,
                        operators: [
                            'is',
                            'not',
                            'prefix_start',
                            'not_prefix_start',
                            'contain',
                            'not_contain',
                            'blank',
                            'present',
                        ],
                        dataSource: null,
                        attributes: {},
                    };
                    break;
                case 'field_num':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdText',
                        contentType: content_type,
                        operators: ['is', 'not'],
                        dataSource: null,
                        attributes: {},
                    };
                    break;
                case 'agent_group_id':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'should', 'not_should', 'blank', 'present'],
                        dataSource: dataSources.agentGroups,
                        attributes: {
                            valueField: 'id',
                            nameField: 'name',
                        },
                    };
                    break;
                // 首次受理人
                case 'first_assignee_id':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'should', 'not_should', 'blank', 'present'],
                        dataSource: dataSources.agents,
                        attributes: {
                            valueField: 'id',
                            nameField: 'nick_name',
                        },
                    };
                    break;
                // 客服
                case 'assignee_id':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'should', 'not_should', 'blank', 'present'],
                        dataSource: dataSources.agents,
                        attributes: {
                            valueField: 'id',
                            nameField: 'nick_name',
                        },
                    };
                    break;
                case 'updated_at':
                case 'created_at':
                case 'closed_at':
                case 'solving_at':
                case 'resolved_at':
                case 'last_replied_at':
                case 'last_agent_replied_at':
                case 'last_customer_replied_at':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdDateTime',
                        contentType: content_type,
                        operators: [
                            'today',
                            'yesterday',
                            'current_week',
                            'current_month',
                            'is',
                            'not',
                            'before',
                            'after',
                            'range',
                            'not_range',
                            'blank',
                            'present',
                            'recent',
                        ],
                        dataSource: dataSources.recentDate,
                        attributes: {
                            needTransfer: true
                        },
                    };
                    break;
                case 'satisfaction':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'should', 'not_should', 'blank', 'present'],
                        dataSource: dataSources.satisfaction,
                        attributes: {
                            valueField: 'id',
                            nameField: 'name',
                        },
                    };
                    break;
                case 'template_id':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'should', 'not_should'],
                        dataSource: dataSources.templates,
                        attributes: {
                            valueField: 'id',
                            nameField: 'name',
                        },
                    };
                    break;
                // TODO: datasource怎么搞定？
                case 'tags':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdTagSelect',
                        contentType: content_type,
                        operators: [
                            'is',
                            'not',
                            'should',
                            'not_should',
                            'include',
                            'not_include',
                            'blank',
                            'present',
                        ],
                        dataSource: dataSources.tags,
                        attributes: {
                            showSearch: true,
                            valueField: 'id',
                            nameField: 'name',
                        },
                    };
                    break;
                case 'followers':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: [
                            'is',
                            'not',
                            'should',
                            'not_should',
                            'include',
                            'not_include',
                            'blank',
                            'present',
                        ],
                        dataSource: dataSources.agents,
                        attributes: {
                            valueField: 'id',
                            nameField: 'nick_name',
                        },
                    };
                    break;
                case 'forward_email':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'should', 'not_should', 'blank', 'present'],
                        dataSource: dataSources.forwardEmail,
                        attributes: {},
                    };
                    break;
                case 'last_replier_type':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not'],
                        dataSource: dataSources.agentWithcustomerAndAgent || dataSources.agents,
                        attributes: {
                            valueField: 'id',
                            nameField: 'nick_name',
                        },
                    };
                    break;
                case 'last_closed_operator_identifier':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'blank', 'present'],
                        dataSource: dataSources.agents,
                        attributes: {
                            valueField: 'id',
                            nameField: 'nick_name',
                        },
                    };
                    break;
                case 'updator_id':
                case 'creator_id':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'should', 'not_should', 'blank', 'present'],
                        dataSource: dataSources.agents,
                        attributes: {
                            valueField: 'id',
                            nameField: 'nick_name',
                        },
                    };
                    break;
                case 'replies':
                case 'parse_content_char_split': // 描述
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdText',
                        contentType: content_type,
                        operators: [
                            'prefix_start',
                            'not_prefix_start',
                            'contain',
                            'not_contain',
                            'blank',
                            'present',
                        ],
                        dataSource: null,
                        attributes: {},
                    };
                    break;
                // 客户字段
                case 'after_created_hours':
                case 'after_solving_hours':
                case 'after_resolved_hours':
                case 'after_closed_hours':
                case 'after_assigned_hours':
                case 'after_updated_hours':
                case 'after_assignee_updated_hours':
                case 'after_creator_updated_hours':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdText',
                        contentType: content_type,
                        operators: ['is_eq', 'gte', 'lte'],
                        dataSource: null,
                        attributes: {},
                    };
                    break;
                // 参考客户
                case 'organization':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdSelect',
                        contentType: content_type,
                        operators: ['is', 'not', 'blank', 'present'],
                        dataSource: null,
                        attributes: {
                            onSearch: function (value) {
                                return searchOrganization({ value: value });
                            },
                            onPopupScroll: function (event, params) {
                                var target = event.target;
                                if (target.scrollTop + target.offsetHeight ===
                                    target.scrollHeight) {
                                    return searchOrganization(params);
                                }
                            },
                            labelInValue: true,
                            filterOption: false,
                            showSearch: true,
                            nameField: 'formatName',
                            valueField: 'id',
                        },
                    };
                    break;
                case 'knowledge_catalog_ids':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdCascade',
                        contentType: content_type,
                        operators: ['is', 'not', 'prefix', 'not_prefix', 'blank', 'present'],
                        dataSource: dataSources.knowledgeCatalog,
                        attributes: {},
                    };
                    break;
                // 后端暂时不支持，这个id等后端支持时再定
                case 'knowledge_question':
                    customFieldItem = {
                        name: name,
                        text: title,
                        inputType: 'antdText',
                        contentType: content_type,
                        operators: [
                            'is',
                            'not',
                            'contain',
                            'not_contain',
                            'start_with',
                            'end_with',
                            'blank',
                            'present',
                        ],
                        dataSource: null,
                        attributes: {},
                    };
                    break;
                default:
                    break;
            }
        }
        return customFieldItem;
    })
        .filter(Boolean);
}
function getFilterFieldsConfig(systemFields, customFields, dataSources) {
    // 系统字段配置
    var systemFieldConfig = createFieldsCondition(systemFields, dataSources);
    // 自定义字段配置
    var customFieldConfig = createFieldsCondition(customFields, dataSources);
    // @ts-ignore
    return [].concat(systemFieldConfig, customFieldConfig);
}
/**
 * 获取自定义过滤器配置
 *
 * @param {(FieldInterface[] | undefined)} fieldsConfig 需要展示和修改的字段配置,和默认配置做合并,可以兼顾排序和新增配置，如果有需要好好设计下
 * @param {Partial<CustomField>[]} systemFields
 * @param {(CustomField[]|undefined)} customFields
 */
function getCustomFilterFieldsConfig(fieldsConfig, systemFields, customFields, dataSources) {
    var fields = getFilterFieldsConfig(systemFields, customFields, dataSources);
    fields.forEach(function (field) {
        if (field === null || field === void 0 ? void 0 : field.operators) {
            field.operators = field.operators.map(function (operator) {
                return businessRuleOperatorEnums[operator];
            });
            if (field.inputType) {
                field.inputType = inputTypeEnums[field.inputType];
            }
        }
    });
    return fields;
}
function toHump(str) {
    if (!str) {
        return '';
    }
    return str.replace(/(\_|-)([a-zA-Z\d])/g, function (match, letter, letter2, index) {
        return (letter2 && letter2.toUpperCase()) || '';
    });
}
// 获取字段配置，系统字段和自定义字段，可以提到公共方法里面 end
function transformLocale(fields) {
    if (!fields) {
        return fields;
    }
    fields.forEach(function (field) {
        if (!field.title) {
            var localName = toHump(field.name);
            // 此处翻译欠佳。后续需要整理components.ticketFilterTable.ticketFilterCondition 内的多语言
            field.title = i18n.t("components.ticketFilterTable.ticketFilterCondition.".concat(localName));
        }
    });
    return fields;
}
// 分组
function makeGroup(parameter) {
    var valueField = parameter.valueField, nameField = parameter.nameField, makeGroupNewArr = parameter.makeGroupNewArr;
    var sourceGroupArr = [];
    makeGroupNewArr.forEach(function (item) {
        var _a, _b;
        // 找出共同的分组名
        // let groupType = item[valueField].split('_')[0];
        var groupType = (_a = item[valueField].split('_')) === null || _a === void 0 ? void 0 : _a[0];
        var groupTitle = (_b = item[nameField].split(/[:-]/g)) === null || _b === void 0 ? void 0 : _b[0];
        var findSameGroupType = sourceGroupArr.find(function (sourceItem) {
            return sourceItem.groupType == groupType;
        });
        if (!findSameGroupType) {
            sourceGroupArr.push({
                groupTitle: groupTitle,
                groupType: groupType,
                groupData: [].concat(item),
            });
        }
        else {
            findSameGroupType.groupData.push(item);
        }
    });
    return sourceGroupArr;
}
function searchOrganization(_a) {
    var value = _a.value, page = _a.page;
    if (page) {
        page++;
    }
    else {
        page = 1;
    }
    return ajax()
        .postJSON("/cs/ts/api/organizations/search", {
        query: value,
        page: page,
    }, {
        timeout: 30000,
    })
        .then(function (resp) {
        var _a, _b;
        if (resp === null || resp === void 0 ? void 0 : resp.organizations) {
            // TODO: organization interface去public utils里面找，如果没用在里面定义下
            var organizations = resp === null || resp === void 0 ? void 0 : resp.organizations.map(function (organization) {
                var mark = organization === null || organization === void 0 ? void 0 : organization.description;
                if (mark) {
                    mark = "(".concat(mark, ")");
                }
                else {
                    mark = '';
                }
                return __assign(__assign({}, organization), { formatName: "".concat(organization === null || organization === void 0 ? void 0 : organization.name).concat(mark) });
            });
            return {
                data: organizations,
                params: {
                    page: (_b = (_a = resp === null || resp === void 0 ? void 0 : resp.organizations) === null || _a === void 0 ? void 0 : _a.meta) === null || _b === void 0 ? void 0 : _b.current_page,
                    query: value,
                },
            };
        }
        else {
            return [];
        }
    });
}
function searchCustomer(_a) {
    var value = _a.value, page = _a.page;
    if (page) {
        page++;
    }
    else {
        page = 1;
    }
    return ajax()
        .postJSON("".concat(PATH_BASE, "/customers/contain_search"), {
        prefix: value || '',
        page: page,
    }, {
        timeout: 30000,
    })
        .then(function (resp) {
        var _a;
        if (resp === null || resp === void 0 ? void 0 : resp.customers) {
            // TODO: organization interface去public utils里面找，如果没用在里面定义下
            var customers = resp === null || resp === void 0 ? void 0 : resp.customers.map(function (customer) {
                return __assign(__assign({}, customer), { formatName: createCustomerName(customer) });
            });
            return {
                data: customers,
                params: {
                    page: (_a = resp === null || resp === void 0 ? void 0 : resp.meta) === null || _a === void 0 ? void 0 : _a.current_page,
                    prefix: value,
                },
            };
        }
        else {
            return [];
        }
    });
}
function requestTemplate() {
    if (templatePromise) {
        return templatePromise;
    }
    else {
        templatePromise = new Promise(function (resolve, reject) {
            ajax()
                .getJSON("/cs/ts/api/ticket_templates", null, {
                timeout: 30000,
            })
                .then(function (resp) {
                if (resp === null || resp === void 0 ? void 0 : resp.items) {
                    var templates = resp.items;
                    resolve(templates);
                }
                else {
                    resolve([]);
                }
            }, function (reason) {
                reject(reason);
            });
        });
        return templatePromise;
    }
}
function requestTags() {
    if (tagPromise) {
        return tagPromise;
    }
    else {
        tagPromise = new Promise(function (resolve, reject) {
            ajax()
                .getJSON("".concat(CS_PATH_BASE, "/ts/api/tags"), null, {
                timeout: 30000,
            })
                .then(function (resp) {
                if (resp === null || resp === void 0 ? void 0 : resp.tags) {
                    var tags = resp.tags;
                    tags === null || tags === void 0 ? void 0 : tags.forEach(function (tag) {
                        if ((tag === null || tag === void 0 ? void 0 : tag.id) && typeof tag.id === 'number') {
                            tag.id = String(tag.id);
                        }
                    });
                    resolve(tags);
                }
                else {
                    resolve([]);
                }
            }, function (reason) {
                reject(reason);
            });
        });
        return tagPromise;
    }
}
function getTemplate() {
    return requestTemplate();
}
function requestKnowledgeCatalog() {
    if (knowledgeQuestionPromise) {
        return knowledgeQuestionPromise;
    }
    else {
        knowledgeQuestionPromise = new Promise(function (resolve, reject) {
            ajax()
                .getJSON("/cs/ts/api/knowledge_catalogs", {
                knowledge_type_name: 'question',
                scope: 'public',
            }, {
                timeout: 30000,
            })
                .then(function (resp) {
                if (resp === null || resp === void 0 ? void 0 : resp.catalogs) {
                    var catalogs = resp.catalogs;
                    resolve(getChainedOptions(catalogs, {
                        optionLabelField: 'name',
                        optionTitleField: 'name',
                        optionValueField: 'id',
                        optionChildrenField: 'children',
                        labelChildrenField: 'subs',
                    }));
                }
                else {
                    resolve([]);
                }
            }, function (reason) {
                reject(reason);
            });
        });
        return knowledgeQuestionPromise;
    }
}
function getTags() {
    return requestTags();
}
function getKnowledgeCatalog() {
    return requestKnowledgeCatalog();
}
function getRecentDate() {
    var all = [
        { label: '天', value: 'days' },
        { label: '小时', value: 'hours' },
        { label: '分钟', value: 'minutes' },
    ];
    // const minuteAndDay = [
    //     { label: '天', value: 'days' },
    //     { label: '分钟', value: 'minutes' },
    // ];
    // const onlyDay = [{ label: '天', value: 'days' }];
    // const options = Array.from(new Array(180), (_, index) => {
    //     let children = onlyDay;
    //     if (index >= 0 && index < 24) {
    //         children = all;
    //     } else if (index >= 24 && index < 60) {
    //         children = minuteAndDay;
    //     }
    //     return {
    //         label: (index + 1).toString(),
    //         value: (index + 1).toString(),
    //         children: children,
    //     };
    // });
    return all;
}
// 可以提为公共方法
function getChainedOptions(options, optionConfig) {
    if (optionConfig === void 0) { optionConfig = {}; }
    var result = [];
    var _a = optionConfig.optionLabelField, optionLabelField = _a === void 0 ? 'title' : _a, _b = optionConfig.optionTitleField, optionTitleField = _b === void 0 ? 'title' : _b, _c = optionConfig.optionValueField, optionValueField = _c === void 0 ? 'value' : _c, _d = optionConfig.optionIsHideField, optionIsHideField = _d === void 0 ? 'is_hide' : _d, _e = optionConfig.optionDefaultField, optionDefaultField = _e === void 0 ? 'default' : _e, _f = optionConfig.optionChildrenField, optionChildrenField = _f === void 0 ? 'subs' : _f, _g = optionConfig.labelChildrenField, labelChildrenField = _g === void 0 ? 'children' : _g;
    if (!options) {
        return result;
    }
    // TODO：有时间仔细看下value为什么需要是字符串
    result = options.map(function (option) {
        var _a;
        return _a = {
                label: option === null || option === void 0 ? void 0 : option[optionLabelField],
                title: option === null || option === void 0 ? void 0 : option[optionTitleField],
                is_hide: option === null || option === void 0 ? void 0 : option[optionIsHideField],
                value: typeof (option === null || option === void 0 ? void 0 : option[optionValueField]) === 'number'
                    ? String(option === null || option === void 0 ? void 0 : option[optionValueField])
                    : option === null || option === void 0 ? void 0 : option[optionValueField],
                default: option === null || option === void 0 ? void 0 : option[optionDefaultField]
            },
            _a[labelChildrenField] = getChainedOptions(option === null || option === void 0 ? void 0 : option[optionChildrenField], optionConfig),
            _a;
    });
    return result;
}
