import { AntdInput, AntdInputNumber, AntdRangeInputNumber, AntdSelect, AntdTimePicker, AntdRangeTimePicker, AntdDatePicker, AntdRangeDatePicker, AntdMultipleSelect, AntdCascader, AntdInputCascader, AntdTicketStatus, } from '@udesk-webapp/custom-filter';
import { businessRuleOperatorEnums, inputTypeEnums } from '@udesk-webapp/utils';
var getComponentsConfigTable = function () {
    var configs = [
        {
            inputType: 'antdText',
            operators: [
                'is',
                'not',
                'contain',
                'not_contain',
                'start_with',
                'end_with',
                'prefix_start',
                'not_prefix_start',
                'gte',
                'lte',
            ],
            component: AntdInput,
            attributes: {
                classNames: 'form-control',
            },
        },
        {
            inputType: 'antdText',
            operators: ['blank', 'present'],
            component: null,
            attributes: {},
        },
        {
            inputType: 'antdSelect',
            operators: ['is', 'not'],
            component: AntdSelect,
            attributes: {},
        },
        {
            inputType: 'antdSelect',
            operators: ['is_any', 'not_any', 'should', 'not_should', 'include', 'not_include'],
            component: AntdMultipleSelect,
            attributes: {
                stringValue: true,
            },
        },
        {
            inputType: 'antdTagSelect',
            operators: ['is', 'not', 'is_any', 'not_any', 'should', 'not_should', 'include', 'not_include'],
            component: AntdMultipleSelect,
            attributes: {
                stringValue: true,
            },
        },
        {
            inputType: 'antdSelect',
            operators: ['blank', 'present'],
            component: null,
            attributes: {},
        },
        {
            inputType: 'antdDate',
            operators: ['is', 'not', 'before', 'after'],
            component: AntdDatePicker,
            attributes: {
                picker: 'date',
                format: 'YYYY-MM-DD',
            },
        },
        {
            inputType: 'antdDate',
            operators: ['range', 'not_range'],
            component: AntdRangeDatePicker,
            attributes: {
                placeholder: ['开始时间', '结束时间'],
                picker: 'date',
                format: 'YYYY-MM-DD',
            },
        },
        {
            inputType: 'antdDate',
            operators: ['blank', 'present'],
            component: null,
            attributes: {},
        },
        {
            inputType: "antdDate",
            operators: ["recent"],
            component: AntdInputCascader,
            attributes: {
                changeOnSelect: false,
                placeholder: '请选择',
            }
        },
        {
            inputType: 'antdTime',
            operators: ['is', 'not', 'before', 'after'],
            component: AntdTimePicker,
            attributes: {
                format: 'HH:mm:ss',
            },
        },
        {
            inputType: 'antdTime',
            operators: ['range', 'not_range'],
            component: AntdRangeTimePicker,
            attributes: {
                format: 'HH:mm:ss',
            },
        },
        {
            inputType: 'antdTime',
            operators: ['blank', 'present'],
            component: null,
            attributes: {},
        },
        {
            inputType: 'antdDateTime',
            operators: ['is', 'not', 'before', 'after'],
            component: AntdDatePicker,
            attributes: {
                picker: 'date',
                format: 'YYYY-MM-DD HH:mm',
                showTime: {
                    format: 'HH:mm',
                },
            },
        },
        {
            inputType: 'antdDateTime',
            operators: ['range', 'not_range'],
            component: AntdRangeDatePicker,
            attributes: {
                placeholder: ['开始时间', '结束时间'],
                picker: 'date',
                format: 'YYYY-MM-DD HH:mm',
                showTime: {
                    format: 'HH:mm',
                },
            },
        },
        {
            inputType: 'antdDateTime',
            operators: ['blank', 'present'],
            component: null,
            attributes: {},
        },
        {
            inputType: "antdDateTime",
            operators: ["recent"],
            component: AntdInputCascader,
            attributes: {
                changeOnSelect: false,
                placeholder: '请选择',
            }
        },
        {
            inputType: 'antdNumeric',
            operators: ['is', 'not', 'gt', 'lt'],
            component: AntdInputNumber,
            attributes: {},
        },
        {
            inputType: 'antdNumeric',
            operators: ['range'],
            component: AntdRangeInputNumber,
            attributes: {},
        },
        {
            inputType: 'antdNumeric',
            operators: ['blank', 'present'],
            component: null,
            attributes: {},
        },
        {
            inputType: 'antdNumber',
            operators: ['is', 'not', 'gt', 'lt'],
            component: AntdInputNumber,
            attributes: {},
        },
        {
            inputType: 'antdNumber',
            operators: ['range'],
            component: AntdRangeInputNumber,
            attributes: {},
        },
        {
            inputType: 'antdNumber',
            operators: ['blank', 'present'],
            component: null,
            attributes: {},
        },
        {
            inputType: 'antdCascade',
            operators: ['is', 'not', 'belongs', 'not_belongs', 'prefix', 'not_prefix'],
            component: AntdCascader,
            attributes: {
                fieldNames: {
                    label: 'title',
                    value: 'value',
                    children: 'subs',
                },
            },
        },
        {
            inputType: 'antdCascade',
            operators: ['blank', 'present'],
            component: null,
            attributes: {},
        },
        {
            inputType: 'antdMultipleSelect',
            operators: ['is', 'not'],
            component: AntdSelect,
            attributes: {
                nameField: 'title',
                valueField: 'value',
                placeholder: '请选择',
            },
        },
        {
            inputType: 'antdMultipleSelect',
            operators: ['is_any', 'not_any', 'should', 'not_should', 'include', 'not_include'],
            component: AntdMultipleSelect,
            attributes: {
                stringValue: true,
                nameField: 'title',
                valueField: 'value',
                placeholder: '请选择',
            },
        },
        {
            inputType: 'antdMultipleSelect',
            operators: ['blank', 'present'],
            component: null,
            attributes: {},
        },
        {
            inputType: 'antdTicketStatus',
            operators: ['is', 'not'],
            component: AntdTicketStatus,
            attributes: {
                showSearch: true,
            },
        },
        {
            inputType: 'antdTicketStatus',
            operators: ['is_any', 'not_any', 'should', 'not_should', 'include', 'not_include'],
            component: AntdTicketStatus,
            attributes: {
                showSearch: true,
                mode: 'multiple',
                stringValue: true,
            },
        },
    ];
    configs.forEach(function (config) {
        var operators = config === null || config === void 0 ? void 0 : config.operators;
        if (operators) {
            config.operators = operators.map(function (operator) {
                return businessRuleOperatorEnums[operator];
            });
            if (config.inputType) {
                config.inputType = inputTypeEnums[config.inputType];
            }
        }
    });
    return configs;
};
export default getComponentsConfigTable;
