import React, { useCallback, useMemo, useState, useEffect } from 'react';
import i18n from '@udesk-webapp/locale';
import { Menu, Dropdown, Button } from 'udesk-ui';
import Classnames from 'classnames';
import { FilterIcon } from '@udesk-webapp/icon';
import TicketFilterCondition from '../ticket-filter-condition';
import style from './style.less';
var TicketFilterConditions = function (props) {
    var allConditions = props.allConditions, anyConditions = props.anyConditions, ticketCustomFields = props.ticketCustomFields, dataSources = props.dataSources, overlayClassName = props.overlayClassName, _a = props.searching, searching = _a === void 0 ? false : _a, systemFields = props.systemFields, onSearch = props.onSearch, onCancelSearch = props.onCancelSearch, removalIconClass = props.removalIconClass, addIconClass = props.addIconClass, addIconText = props.addIconText, filterClassName = props.filterClassName;
    var _b = useState(false), visible = _b[0], setVisible = _b[1];
    var _c = useState(allConditions), innerAllConditions = _c[0], setInnerAllConditions = _c[1];
    var _d = useState(anyConditions), innerAnyConditions = _d[0], setInnerAnyConditions = _d[1];
    useEffect(function () {
        setInnerAllConditions(allConditions);
    }, [allConditions]);
    useEffect(function () {
        setInnerAnyConditions(anyConditions);
    }, [anyConditions]);
    var onAllConditionChanged = useCallback(function (conditions) {
        setInnerAllConditions(conditions);
    }, []);
    var onAnyConditionChanged = useCallback(function (conditions) {
        setInnerAnyConditions(conditions);
    }, []);
    var handleVisibleChange = useCallback(function (flag) {
        setVisible(flag);
    }, []);
    var cancelSearch = useCallback(function () {
        if (typeof onCancelSearch === 'function') {
            onCancelSearch();
        }
        // 隐藏条件下拉
        handleVisibleChange(false);
    }, [handleVisibleChange]);
    var doSearch = useCallback(function () {
        if (typeof onSearch === 'function') {
            onSearch({
                allConditions: innerAllConditions,
                anyConditions: innerAnyConditions,
            });
            // 隐藏条件下拉
            handleVisibleChange(false);
        }
    }, [innerAllConditions, innerAnyConditions, handleVisibleChange]);
    var menu = useCallback(function () {
        return (React.createElement(Menu, { className: "".concat(filterClassName, " ticket-filter-conditions-filter-menu") },
            React.createElement(Menu.Item, { key: 'title', className: 'ticket-filter-conditions-title' }, i18n.t('components.ticketFilterTable.ticketFilterConditions.filter')),
            React.createElement(Menu.Item, { key: 'all', className: 'ticket-filter-conditions-li' },
                React.createElement("div", { className: 'ticket-filter-conditions-filter-title' }, i18n.t('components.ticketFilterTable.ticketFilterConditions.metAllConditions')),
                React.createElement(TicketFilterCondition, { systemFields: systemFields, dataSources: dataSources, className: 'ticket-filter-conditions-filter-condition', conditions: innerAllConditions, customFields: ticketCustomFields, onConditionChanged: onAllConditionChanged, removalIconClass: removalIconClass, addIconClass: addIconClass, addIconText: addIconText })),
            React.createElement(Menu.Item, { key: 'any', className: 'ticket-filter-conditions-li' },
                React.createElement("div", { className: 'ticket-filter-conditions-filter-title' }, i18n.t('components.ticketFilterTable.ticketFilterConditions.metAnyConditions')),
                React.createElement(TicketFilterCondition, { systemFields: systemFields, dataSources: dataSources, className: 'ticket-filter-conditions-filter-condition', conditions: innerAnyConditions, customFields: ticketCustomFields, onConditionChanged: onAnyConditionChanged, removalIconClass: removalIconClass, addIconClass: addIconClass, addIconText: addIconText })),
            React.createElement(Menu.Item, { key: 'btns', className: 'ticket-filter-conditions-btns' },
                React.createElement(Button, { className: 'ticket-filter-conditions-search-btn', onClick: doSearch, type: 'primary', loading: searching }, i18n.t('components.ticketFilterTable.ticketFilterConditions.search')),
                React.createElement(Button, { className: 'ticket-filter-conditions-cancel-btn', onClick: cancelSearch }, i18n.t('components.ticketFilterTable.ticketFilterConditions.cancel')))));
    }, [searching, innerAllConditions, innerAnyConditions, doSearch, cancelSearch]);
    var filterText = useMemo(function () {
        if (visible) {
            return i18n.t('components.ticketFilterTable.ticketFilterConditions.cancel');
        }
        else {
            return i18n.t('components.ticketFilterTable.ticketFilterConditions.filter');
        }
    }, [visible]);
    return (React.createElement(React.Fragment, null,
        React.createElement(Dropdown, { overlayClassName: Classnames(overlayClassName, style['overlay-class']), overlay: menu, onVisibleChange: handleVisibleChange, visible: visible, trigger: ['click'] },
            React.createElement("a", { className: style['dropdown-link'], onClick: function (e) { return e.preventDefault(); } },
                React.createElement(FilterIcon, null),
                React.createElement("span", { className: Classnames('filter-text', style['filter-text']) }, filterText)))));
};
export default TicketFilterConditions;
