"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importStar(require("react"));
var udesk_ui_1 = require("udesk-ui");
var react_i18next_1 = require("react-i18next");
var icons_1 = require("@ant-design/icons");
var dayjs_1 = tslib_1.__importDefault(require("dayjs"));
var enums_1 = require("./enums");
var index_module_less_1 = tslib_1.__importDefault(require("./index.module.less"));
var TextArea = udesk_ui_1.Input.TextArea;
var TRANSLATE_CACHE = 'translateConfig';
// 原文组件
var Translate = function (props) {
    var onTranslate = props.onTranslate, _a = props.persistenceKey, persistenceKey = _a === void 0 ? '' : _a;
    var t = (0, react_i18next_1.useTranslation)().t;
    var _b = (0, react_1.useState)(enums_1.originalMap.ZhCn), originalValue = _b[0], setOriginalValue = _b[1]; // 原文语言
    var _c = (0, react_1.useState)(enums_1.originalMap.EnUs), translateValue = _c[0], setTranslateValue = _c[1]; // 译文语言
    var _d = (0, react_1.useState)(''), originalText = _d[0], setOriginalText = _d[1]; // 原文
    var _e = (0, react_1.useState)(false), loading = _e[0], setLoading = _e[1];
    (0, react_1.useEffect)(function () {
        var translateConfig = JSON.parse(window.localStorage.getItem(TRANSLATE_CACHE) || '{}');
        if (translateConfig.from) {
            setOriginalValue(translateConfig.from);
        }
        if (translateConfig.to) {
            setTranslateValue(translateConfig.to);
        }
    }, []);
    (0, react_1.useEffect)(function () {
        if (persistenceKey) {
            var originalCache = JSON.parse(window.localStorage.getItem(persistenceKey) || '{}');
            // 缓存有效期1天 超出清空
            if (originalCache.cacheTime && (0, dayjs_1.default)(originalCache.cacheTime).diff((0, dayjs_1.default)(), 'day') >= 1) {
                window.localStorage.removeItem(persistenceKey);
                originalCache = {};
            }
            if (originalCache.text) {
                setOriginalText(originalCache.text);
            }
        }
    }, []);
    // 译文下拉change
    var handleTranslateChange = function (v) {
        setTranslateValue(v);
        // 设置缓存
        window.localStorage.setItem(TRANSLATE_CACHE, JSON.stringify({
            from: originalValue,
            to: v
        }));
    };
    // 原文变化
    var handleOriginChange = function (e) {
        var v = e.target.value;
        setOriginalText(v);
        // 设置缓存
        if (persistenceKey) {
            var data = {
                cacheTime: new Date().getTime(),
                text: v
            };
            window.localStorage.setItem(persistenceKey, JSON.stringify(data));
        }
    };
    // 点击翻译
    var onHandleTranslate = function () {
        if (loading)
            return;
        if (!originalText) {
            return udesk_ui_1.message.error(t('originalPlaceholder'));
        }
        var o = originalValue;
        if (/[\u4e00-\u9fa5]/.test(originalText)) { // 有一个中文就算中文
            if (originalValue !== enums_1.originalMap.ZhCn) {
                o = enums_1.originalMap.ZhCn;
                setOriginalValue(o);
            }
        }
        else { // auto
            if (originalValue !== enums_1.originalMap.Auto) {
                o = enums_1.originalMap.Auto;
                setOriginalValue(o);
            }
        }
        if (onTranslate) {
            setLoading(true);
            onTranslate({
                from: o,
                to: translateValue,
                original: originalText
            }).then(function () {
                setLoading(false);
            });
        }
        // 设置缓存
        if (persistenceKey) {
            var data = {
                cacheTime: new Date().getTime(),
                text: originalText
            };
            window.localStorage.setItem(persistenceKey, JSON.stringify(data));
        }
        window.localStorage.setItem(TRANSLATE_CACHE, JSON.stringify({
            from: o,
            to: translateValue
        }));
    };
    return (react_1.default.createElement("div", { className: index_module_less_1.default['original-wrap'] },
        react_1.default.createElement("div", { className: index_module_less_1.default['original-toolbar-wrap'] },
            react_1.default.createElement(udesk_ui_1.Select, { disabled: true, style: { width: 120 }, value: originalValue, options: enums_1.originalOptions, onChange: function (v) { return setOriginalValue(v); } }),
            react_1.default.createElement(icons_1.SwapOutlined, { className: index_module_less_1.default['original-toolbar-icon'] }),
            react_1.default.createElement(udesk_ui_1.Select, { style: { width: 120 }, value: translateValue, options: enums_1.translateOptions, onChange: handleTranslateChange, className: index_module_less_1.default['original-toolbar-translate'] })),
        react_1.default.createElement("div", { className: index_module_less_1.default['original-content-wrap'] },
            react_1.default.createElement(TextArea, { value: originalText, onChange: handleOriginChange, bordered: false, autoSize: { minRows: 6 }, placeholder: t('originalPlaceholder') })),
        react_1.default.createElement(udesk_ui_1.Button, { type: 'primary', onClick: onHandleTranslate, loading: loading }, t('statusbar.translate'))));
};
exports.default = react_1.default.memo(Translate);
