var _UdeskSdkClass_fileMonitor;
import { __awaiter, __classPrivateFieldGet, __classPrivateFieldSet } from "tslib";
import FileMonitorClass from './version-monitor/version-monitor-class';
import ReactSentry from './react-sentry';
import EventedClass from './evented-class';
export const DefaultOptions = {
    sdkClientPrefix: undefined,
    sdkServerPrefix: undefined,
    appClientPrefix: undefined,
    appServerPrefix: undefined,
    enableGlobalConfig: false,
    globalConfigFilePath: './global-config.json',
    manifestPath: './RELEASE-VERSION',
    enableNewVersionTip: true,
    versionMonitorInterval: 60,
    enableVersionMonitoring: true,
    enableSentry: false,
    sentryOptions: undefined,
    reloadCallback: undefined,
    routerBase: undefined,
    landingPath: undefined,
    lang: 'zh-CN',
    prefixCls: undefined,
};
export class UdeskSdkClass extends EventedClass {
    constructor(options) {
        super();
        _UdeskSdkClass_fileMonitor.set(this, void 0);
        this.options = Object.assign(Object.assign({}, UdeskSdkClass.defaultOptions), options);
        this.rootContext = {
            sdkOptions: this.options,
        };
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        let scriptUrl = this.constructor.scriptUrl;
        if (scriptUrl == null) {
            throw new Error('实例化一个SDKClass之前，需要先调用 setCurrentScript 静态方法，并且第一个参数传入 document.currentScript');
        }
        //#region 检查版本
        if (this.options.enableVersionMonitoring) {
            if (!this.options.manifestPath) {
                throw new Error('sdkOptions.versionFilePath 不能为空，如果 enableVersionMonitoring 设置为 true');
            }
            __classPrivateFieldSet(this, _UdeskSdkClass_fileMonitor, new FileMonitorClass({
                versionPath: this.options.manifestPath,
                intervalSeconds: this.options.versionMonitorInterval || DefaultOptions.versionMonitorInterval,
            }), "f");
            __classPrivateFieldGet(this, _UdeskSdkClass_fileMonitor, "f").on('versionChanged', (version, oldVersion) => {
                this.onVersionChanged(version, oldVersion);
            });
            __classPrivateFieldGet(this, _UdeskSdkClass_fileMonitor, "f").start();
        }
        //#endregion
        //#region sentry
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        if (this.options.enableSentry && !this.constructor.sentryInitialized) {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            //@ts-ignore
            this.constructor.sentryInitialized = true;
            ReactSentry.init(Object.assign(Object.assign({ dsn: '' }, this.options.sentryOptions), { getShouldSendCallback: (data) => {
                    var _a;
                    let getSentryShouldSend = (_a = this.options.sentryOptions) === null || _a === void 0 ? void 0 : _a.getShouldSendCallback;
                    if (getSentryShouldSend != null) {
                        return getSentryShouldSend(data);
                    }
                    else {
                        return isSdkRoute(window.location.pathname, this.routes());
                    }
                } }));
        }
        //#endregion
        //#region renderTo
        let renderToBak = this.renderTo.bind(this);
        this.renderTo = (container) => __awaiter(this, void 0, void 0, function* () {
            let callRenderTo = () => {
                if (this.options.enableSentry) {
                    ReactSentry.ready();
                }
                if (renderToBak) {
                    return renderToBak(container);
                }
                else {
                    throw new Error('renderTo 方法尚未被实现');
                }
            };
            if (this.options.enableGlobalConfig) {
                if (!this.options.globalConfigFilePath) {
                    throw new Error('sdkOptions.globalConfigFilePath 不能为空，如果 enableGlobalConfig 设置为 true');
                }
                let url = this.globalConfigFileUrl();
                let globalData = yield fetch(url + `?_=${Date.now()}`, {
                    method: 'GET',
                    cache: 'no-cache',
                })
                    .then((resp) => {
                    return resp.json();
                })
                    .then((data) => {
                    return data;
                })
                    .catch((error) => {
                    this.captureException(error);
                    return Promise.reject(new Error(`读取全局配置文件 '${url}' 失败！`));
                });
                this.rootContext.globalConfig = globalData;
                return callRenderTo();
            }
            else {
                return yield callRenderTo();
            }
        });
        //#endregion
    }
    // 一些hooks
    beforeRender({ container, rootContext }) {
        // 无默认实现
    }
    update(sdkOptions) {
        if (this.rootContext.onSdkOptionsChanged) {
            this.rootContext.onSdkOptionsChanged(sdkOptions);
        }
    }
    destroy() {
        if (__classPrivateFieldGet(this, _UdeskSdkClass_fileMonitor, "f") != null) {
            __classPrivateFieldGet(this, _UdeskSdkClass_fileMonitor, "f").destroy();
        }
    }
    version() {
        var _a;
        return (_a = __classPrivateFieldGet(this, _UdeskSdkClass_fileMonitor, "f")) === null || _a === void 0 ? void 0 : _a.version();
    }
    captureException(error) {
        if (this.options.enableSentry) {
            ReactSentry.captureException(error);
        }
    }
    versionFileUrl() {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        return new URL(this.options.manifestPath, this.constructor.scriptUrl).href;
    }
    globalConfigFileUrl() {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        return new URL(this.options.globalConfigFilePath, this.constructor.scriptUrl).href;
    }
    onVersionChanged(version, oldVersion) {
        // 如果子类需要接收通知，则可以重写此方法
        // 默认不做处理，但也不需要子类必须重写
        if (this.rootContext.onVersionChanged) {
            this.rootContext.onVersionChanged(version, oldVersion);
        }
    }
    static bindToCurrentScript(sdkClass, loadingScriptElement, sdkName = 'sdkClass' // componentSdkClass | sdkClass
    ) {
        if (loadingScriptElement instanceof HTMLScriptElement) {
            loadingScriptElement[sdkName] = sdkClass || this;
            this.scriptUrl = loadingScriptElement.src;
        }
        else {
            // try {
            //     //@ts-ignore
            //     this.scriptUrl = import.meta.url;
            // } catch (error) {}
            throw new Error('请传入正确的 script DOM元素');
        }
    }
}
_UdeskSdkClass_fileMonitor = new WeakMap();
UdeskSdkClass.defaultOptions = Object.assign({}, DefaultOptions);
UdeskSdkClass.scriptUrl = undefined;
function isSdkRoute(pathname, routes) {
    if (routes) {
        for (let i = 0; i < routes.length; i++) {
            const route = routes[i];
            if (route.path && pathname.includes(route.path)) {
                return true;
            }
            if (route.routes && isSdkRoute(pathname, route.routes)) {
                return true;
            }
        }
    }
    return false;
}
export default UdeskSdkClass;
//# sourceMappingURL=udesk-sdk.js.map