import { __rest } from "tslib";
import head from 'lodash-es/head';
import drop from 'lodash-es/drop';
export function getOptionByValue(value, options) {
    var valueIsArray = Array.isArray(value);
    if (!value || (valueIsArray && !value.length) || !options || !options.length) {
        return null;
    }
    if ('string' === typeof value) {
        value = value.split(',');
    }
    return value.reduce(function (prevValue, valueItem) {
        if (!prevValue || !prevValue.children) {
            return null;
        }
        return prevValue.children.find(function (i) { return (i.value + '') === (valueItem + ''); });
    }, { children: options });
}
export function handleCascadeHideOptionByValue(option, value) {
    var _a, _b;
    var resetValue;
    if (value) {
        var currentValue = head(value);
        resetValue = drop(value);
        if (parseInt(currentValue, 10) !== parseInt(option.value, 10) && option.is_hide) {
            // 设置隐藏的选项如果被选中就不隐藏
            return;
        }
    }
    var children = option.children, newOption = __rest(option, ["children"]);
    if (Array.isArray(option.children) && ((_a = option.children) === null || _a === void 0 ? void 0 : _a.length)) {
        var subsFormat_1 = [];
        (_b = option.children) === null || _b === void 0 ? void 0 : _b.forEach(function (item) {
            var itemFormat = handleCascadeHideOptionByValue(item, resetValue || value);
            itemFormat && subsFormat_1.push(itemFormat);
        });
        newOption.children = subsFormat_1.length ? subsFormat_1 : null;
    }
    return newOption;
}
export function createCascadeOptionsByValue(value, options) {
    if (!options || !Array.isArray(options) || !options.length)
        return options;
    if (typeof value === 'string') {
        value = value.split(',');
    }
    var newOptions = [];
    options.forEach(function (option) {
        var filterOption = handleCascadeHideOptionByValue(option, value);
        filterOption && newOptions.push(filterOption);
    });
    return newOptions;
}
