"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchFileUploadPolicy = exports.uploadFile = void 0;
var tslib_1 = require("tslib");
var axios_1 = tslib_1.__importDefault(require("axios"));
var ajax_1 = require("./ajax");
function uploadFile(file, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var saveAttachment, opts, fileUploadPolicyResp, _a, fileUrl;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    saveAttachment = options.saveAttachment, opts = tslib_1.__rest(options, ["saveAttachment"]);
                    return [4 /*yield*/, fetchFileUploadPolicy(file, opts)];
                case 1:
                    fileUploadPolicyResp = _b.sent();
                    _a = fileUploadPolicyResp === null || fileUploadPolicyResp === void 0 ? void 0 : fileUploadPolicyResp.storage_policy;
                    switch (_a) {
                        case "ali": return [3 /*break*/, 2];
                        case "minio": return [3 /*break*/, 4];
                        case "huawei_cloud_obs": return [3 /*break*/, 6];
                    }
                    return [3 /*break*/, 8];
                case 2: return [4 /*yield*/, uploadToOss(file, tslib_1.__assign(tslib_1.__assign({}, fileUploadPolicyResp), { aliParams: opts.aliParams }))];
                case 3:
                    _b.sent();
                    return [3 /*break*/, 8];
                case 4: return [4 /*yield*/, uploadToMinio(file, fileUploadPolicyResp)];
                case 5:
                    _b.sent();
                    return [3 /*break*/, 8];
                case 6: 
                // 暂时不考虑华为云
                return [4 /*yield*/, uploadToOss(file, tslib_1.__assign(tslib_1.__assign({}, fileUploadPolicyResp), { aliParams: opts.aliParams }))];
                case 7:
                    // 暂时不考虑华为云
                    _b.sent();
                    return [3 /*break*/, 8];
                case 8:
                    fileUrl = (fileUploadPolicyResp === null || fileUploadPolicyResp === void 0 ? void 0 : fileUploadPolicyResp.signed_url) ||
                        fileUploadPolicyResp.host + "/" + fileUploadPolicyResp.key;
                    if (!saveAttachment) {
                        return [2 /*return*/, { src: fileUrl }];
                    }
                    return [4 /*yield*/, fetchAttachment(file, {
                            storage_policy: fileUploadPolicyResp.storage_policy,
                            bucket: fileUploadPolicyResp.bucket,
                            key: fileUploadPolicyResp.key,
                        })];
                case 9: return [2 /*return*/, _b.sent()];
            }
        });
    });
}
exports.uploadFile = uploadFile;
function getFileName(file) {
    var _a = file.name.split("."), fileName = _a[0], fileExt = _a[1];
    var threeRandom = Math.random().toString(32).slice(-5);
    return fileName + "_" + new Date().getTime() + "_" + threeRandom + "." + (fileExt || "jpg");
}
function fetchFileUploadPolicy(file, option) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, key, policyResp, storage_policy;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = option.key, key = _a === void 0 ? getFileName(file) : _a;
                    return [4 /*yield*/, (0, ajax_1.getJSON)("/file_upload_policy", tslib_1.__assign(tslib_1.__assign({}, option), { key: key }))];
                case 1:
                    policyResp = _b.sent();
                    storage_policy = policyResp.storage_policy;
                    switch (storage_policy) {
                        case "qiniu":
                            key = policyResp.key || key;
                            break;
                        case "ali":
                        case "huawei_cloud_obs":
                            key = policyResp.key || policyResp.dir + key;
                            break;
                        case "minio":
                            key = policyResp.file_path;
                            break;
                    }
                    return [2 /*return*/, tslib_1.__assign(tslib_1.__assign({}, policyResp), { key: key })];
            }
        });
    });
}
exports.fetchFileUploadPolicy = fetchFileUploadPolicy;
function uploadToOss(file, options) {
    var _a, _b;
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var host, fileUploadPolicy, formData;
        return tslib_1.__generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    host = options.host, fileUploadPolicy = tslib_1.__rest(options, ["host"]);
                    formData = new FormData();
                    formData.append("policy", fileUploadPolicy.policy);
                    formData.append("OSSAccessKeyId", fileUploadPolicy.accessid);
                    formData.append("Signature", fileUploadPolicy.signature);
                    if (fileUploadPolicy["x-oss-object-acl"]) {
                        formData.append("x-oss-object-acl", fileUploadPolicy["x-oss-object-acl"]);
                    }
                    formData.append("key", fileUploadPolicy.key);
                    if ((_a = options.aliParams) === null || _a === void 0 ? void 0 : _a["Cache-Control"]) {
                        formData.append("Cache-Control", (_b = options.aliParams) === null || _b === void 0 ? void 0 : _b["Cache-Control"]);
                    }
                    formData.append("file", file);
                    return [4 /*yield*/, axios_1.default.post(host, formData, {
                            headers: { "Content-Type": "multipart/form-data", accept: "application/json" },
                        })];
                case 1: return [2 /*return*/, _c.sent()];
            }
        });
    });
}
function uploadToMinio(file, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var host, fileUploadPolicy, formData, fields;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    host = options.host, fileUploadPolicy = tslib_1.__rest(options, ["host"]);
                    formData = new FormData();
                    fields = fileUploadPolicy.fields;
                    if (fields) {
                        Object.keys(fields).forEach(function (key) {
                            formData.append(key, fields[key]);
                        });
                    }
                    formData.append("file", file);
                    return [4 /*yield*/, axios_1.default.post(host, formData, {
                            headers: { "Content-Type": "multipart/form-data", accept: "application/json" },
                        })];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
function fetchAttachment(file, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var url, params;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    url = "/attachments";
                    params = {
                        attachment: {
                            qiniu_key: options.key,
                            file_file_name: file.name,
                            file_content_type: file.type,
                            file_file_size: file.size,
                            bucket: options.bucket,
                            storage_policy: options.storage_policy,
                        },
                    };
                    return [4 /*yield*/, (0, ajax_1.postJSON)(url, params)];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
