var _VersionFileMonitorClass_intervalInstance, _VersionFileMonitorClass_oldVersion;
import { __classPrivateFieldGet, __classPrivateFieldSet } from "tslib";
import { InvalidArgumentError } from '../error';
import EventedClass from '../evented-class';
import SetIntervalClass from './set-interval-class';
const DEFAULT_OPTIONS = {
    versionPath: '',
    intervalSeconds: 60,
};
class VersionFileMonitorClass extends EventedClass {
    constructor(options) {
        super();
        _VersionFileMonitorClass_intervalInstance.set(this, new SetIntervalClass());
        _VersionFileMonitorClass_oldVersion.set(this, void 0);
        this.options = Object.assign({}, DEFAULT_OPTIONS, options);
    }
    start() {
        let intervalSeconds = this.options.intervalSeconds;
        if (intervalSeconds != null && typeof intervalSeconds !== 'number') {
            throw new InvalidArgumentError(`options.intervalSeconds`);
        }
        let versionPath = this.options.versionPath;
        if (versionPath == null || versionPath === '') {
            throw new InvalidArgumentError(`options.versionPath`);
        }
        __classPrivateFieldGet(this, _VersionFileMonitorClass_intervalInstance, "f").clear();
        __classPrivateFieldGet(this, _VersionFileMonitorClass_intervalInstance, "f").setInterval(() => {
            let resolveVersion = (versionText) => {
                let version = versionText;
                if (version) {
                    // 目前采用简单版本规则，即只判断是否变化。
                    // 未来可以做成对象，判断是否发生了回滚、是否有倒计时、是否强制刷新等规则
                    if (version !== __classPrivateFieldGet(this, _VersionFileMonitorClass_oldVersion, "f")) {
                        this.trigger('versionChanged', [version, __classPrivateFieldGet(this, _VersionFileMonitorClass_oldVersion, "f")]);
                    }
                    __classPrivateFieldSet(this, _VersionFileMonitorClass_oldVersion, version, "f");
                }
            };
            fetch(`${versionPath}?_=${Date.now()}`, {
                method: 'GET',
                cache: 'no-cache',
            })
                .then(function (response) {
                return response.text();
            })
                .then(function (versionText) {
                resolveVersion(versionText);
            });
        }, intervalSeconds);
    }
    reset() {
        this.stop();
        this.start();
    }
    stop() {
        __classPrivateFieldGet(this, _VersionFileMonitorClass_intervalInstance, "f").clear();
    }
    changeInterval(interval) {
        if (interval > 0) {
            this.options.intervalSeconds = interval;
            this.reset();
        }
    }
    destroy() {
        this.off();
        this.stop();
    }
    version() {
        return __classPrivateFieldGet(this, _VersionFileMonitorClass_oldVersion, "f");
    }
}
_VersionFileMonitorClass_intervalInstance = new WeakMap(), _VersionFileMonitorClass_oldVersion = new WeakMap();
export default VersionFileMonitorClass;
//# sourceMappingURL=version-monitor-class.js.map