import React, { useState, useEffect } from 'react';
import { Upload } from 'antd';
import { PlusOutlined } from '@ant-design/icons';
import type { RcFile, UploadFile, UploadProps } from 'antd/es/upload/interface';
import { useRemoteServices } from '../context';

const uploadButton = (
  <div>
    <PlusOutlined />
    <div style={{ marginTop: 8 }}>{'上传'}</div>
  </div>
);

export function UploadSetter({ value, onChange }: any) {
  const [fileList, setFileList] = useState<UploadFile[]>([]);
  const { uploadCustomRequest: customRequest } = useRemoteServices();

  useEffect(() => {
    let list: UploadFile[] = [];
    if (value) {
      list = [
        {
          uid: '-1',
          name: 'image.png',
          status: 'done',
          url: value,
        },
      ];
    }
    setFileList(list);
  }, [value]);

  const onUploadChange: UploadProps['onChange'] = ({ fileList: newFileList }) => {
    setFileList(newFileList);
  };

  const onUploadRemove = () => {
    onChange?.(null);
  };

  const onPreview = async (file: UploadFile) => {
    let src = file.url as string;
    if (!src) {
      src = await new Promise((resolve) => {
        const reader = new FileReader();
        reader.readAsDataURL(file.originFileObj as RcFile);
        reader.onload = () => resolve(reader.result as string);
      });
    }
    const image = new Image();
    image.src = src;
    const imgWindow = window.open(src);
    imgWindow?.document.write(image.outerHTML);
  };

  const handleCustomRequest = (props: any) => {
    if (!customRequest) {
      throw new Error(
        'UploadSetter components lack customRequest properties, please check whether Designer component with remoteServices.uploadCustomRequest properties',
      );
    }
    customRequest?.(props, (params: any, uploadData: any) => {
      onChange(`${uploadData.token.host}/${uploadData.filename}`);
    });
  };

  return (
    <Upload
      name="uploadFile"
      accept=".jpg,.jpeg,.png"
      listType="picture-card"
      customRequest={handleCustomRequest}
      fileList={fileList}
      maxCount={1}
      onChange={onUploadChange}
      onRemove={onUploadRemove}
      onPreview={onPreview}
    >
      {React.cloneElement(uploadButton, { disabled: !!fileList?.length })}
    </Upload>
  );
}
