import { createTrasformStream } from '../creator';
import template from 'lodash.template';
import { parseCaptionAndTypeFromFileName } from '../../utils';
import upperFirst from 'lodash.upperfirst';
import camelCase from 'lodash.camelcase';

export interface UseTemplatePluginOptions {
  template: string;
  mapToInterpolate: MapToInterpolate;
}

export interface UseTemplate2MarkdownOptions {
  template: string;
  mapToInterpolate: MapToInterpolate;
  theme: string;
}

export interface MapToInterpolate {
  (meta: { name: string; content: string; path: string }): object;
}

export const useTemplate = ({
  template: tplContent,
  mapToInterpolate
}: UseTemplatePluginOptions) => {
  const executor = template(tplContent);
  return createTrasformStream((content, { stem: name, path }) => {
    let index = name.indexOf('&');
    if (index >= 0) {
      name = name.substring(0, index);
    }
    return executor(mapToInterpolate({ name, content, path }))
  });

};


export const useTemplate2Markdown = ({
  template: tplContent,
  mapToInterpolate,
  theme
}: UseTemplate2MarkdownOptions) => {
  const executor = template(tplContent);
  return createTrasformStream((content, { stem: name, path }) => {
    let { basename, caption, category } = parseCaptionAndTypeFromFileName(name);
    basename = basename + (theme ? `-${theme}` : '');
    basename = upperFirst(camelCase(basename));
    name = `##### ${basename} \\[${category}\\] \\(${caption}\\) `;
    return executor(mapToInterpolate({ name, content, path }))
  });
};