const path = require('path');
const fs = require('fs');
const klawSync = require('klaw-sync');
const htmlparser2 = require('htmlparser2');
const render = require('dom-serializer').default;
const domutils = require('domutils');

const RegExp = /^([\w-]+)\&(.+)$/;

fixFilenames('./fill', '-fill');
fixFilenames('./outlined', '-outlined');
fixSvgAttrs();

function fixFilenames(dir, postfix) {
  klawSync(dir, {
    nodir: true,
    filter(item) {
      return item.path.endsWith('.svg');
    },
    traverseAll: true
  }).forEach((item) => {
    let matches = path.parse(item.path).name.match(RegExp);
    if (!matches) {
      throw new Error(item.path);
    }
    let enName = matches[1];
    let chName = matches[2];
    if (enName.toLowerCase().endsWith(postfix)) {
      fs.renameSync(
        item.path,
        path.join(
          path.dirname(item.path),
          enName.replace(postfix, '') + '&' + chName + '.svg'
        )
      );
    }
  });
}

function fixSvgAttrs() {
  let files = klawSync('./', {
    nodir: true,
    filter(item) {
      return item.path.endsWith('.svg');
    },
    traverseAll: true
  });
  files.forEach((item) => {
    let matches = path.parse(item.path).name.match(RegExp);
    let enName = matches[1];
    let code = fs.readFileSync(item.path).toString();
    let doc = htmlparser2.parseDocument(code, {
      xmlMode: true
    });

    let svgNode = domutils.findOne(
      (el) => el.type === 'tag' && el.name === 'svg',
      doc.children,
      true
    );
    if (svgNode) {
      let attributes = [];
      let attribs = {};
      let viewBox = svgNode.attributes.find((a) => a.name === 'viewBox');
      if (viewBox) {
        addAttr(attributes, attribs, 'viewBox', viewBox.value);
      }
      addAttr(attributes, attribs, 'focusable', 'false');
      addAttr(attributes, attribs, 'data-icon', enName);
      addAttr(attributes, attribs, 'width', '1em');
      addAttr(attributes, attribs, 'height', '1em');
      addAttr(attributes, attribs, 'fill', 'currentColor');
      addAttr(attributes, attribs, 'aria-hidden', 'true');

      svgNode.attributes = attributes;
      svgNode.attribs = attribs;
    }
    fs.writeFileSync(item.path, render(doc));
  });
}

function addAttr(attributes, attribs, key, data) {
  if (key && data) {
    attributes.push({ name: key, value: data });
    attribs[key] = data;
  }
}
