import { src, dest } from 'gulp';
import SVGO from 'svgo';
import { SVG2DefinitionOptions } from '../../plugins/svg2Definition';
import rename from 'gulp-rename';
import { UseTemplatePluginOptions } from '../../plugins/useTemplate';
import { svgo, svg2Demo, useTemplate2Markdown } from '../../plugins';
import concat from 'gulp-concat';
import { HelperRenderOptions } from '../../templates/helpers';
import { useRenderMarkdown } from '../../plugins';
import { IconDefinition } from '../../templates/types';

export interface GenerateSvgDemoOptions
    extends SVG2DefinitionOptions,
    UseTemplatePluginOptions {
    from: string[];
    toDir: string;
    svgoConfig: SVGO.Options;
    filename: string;
    getIconDefinitionFromSource: (raw: string) => IconDefinition;
    renderOptions?: HelperRenderOptions;
}

export const generateSvgDemo = ({
    from,
    toDir,
    svgoConfig,
    theme,
    extraNodeTransformFactories,
    stringify,
    template,
    mapToInterpolate,
    filename,
    getIconDefinitionFromSource,
    renderOptions = {}
}: GenerateSvgDemoOptions) =>
    function GenerateIcons() {
        return src(from)
            .pipe(svgo(svgoConfig))
            .pipe(
                svg2Demo({
                    theme,
                    extraNodeTransformFactories,
                    stringify
                })
            )
            .pipe(useTemplate2Markdown({ template, mapToInterpolate, theme }))
            .pipe(
                useRenderMarkdown({
                    getIconDefinitionFromSource,
                    renderOptions
                })
            )
            .pipe(concat(filename))
            .pipe(dest(toDir));
    };
