import upperFirst from 'lodash.upperfirst';
import camelCase from 'lodash.camelcase';
import { ThemeTypeUpperCase } from '../templates/types';
import { pipe } from 'ramda';

//filename样式为  Aaa&说明$类型
const TypeSpare = '$';
const CaptionSpare = '&';

export interface IdentifierMeta {
  name: string;
  themeSuffix?: ThemeTypeUpperCase;
}

export interface AllNameMeta {
  name: string;
  dirname?: string;
}

export interface CaptionTypeMeta {
  basename: string;
  caption: string;
  category: string;
}

export interface GetIdentifierType {
  (meta: IdentifierMeta): string;
}

export interface GetAllNameType {
  (meta: AllNameMeta): string;
}

export const getIdentifier: GetIdentifierType = pipe(
  ({ name, themeSuffix }: IdentifierMeta) =>
    name + (themeSuffix ? `-${themeSuffix}` : ''),
  camelCase,
  upperFirst
);

export const getAllInfoName: GetAllNameType = pipe(
  ({ name, dirname }: AllNameMeta) => {
    // 在文件名后加上所属类型名
    dirname = TypeSpare + dirname;
    if (name) {
      name = name + dirname;

    }
    return name
  }
);

export const parseCaptionAndTypeFromFileName = function (fileName: string): CaptionTypeMeta {
  // 取名称
  let index = fileName.indexOf(CaptionSpare);
  let basename = fileName.substring(0, index);
  // 取caption
  fileName = fileName.substring(index + 1);
  index = fileName.indexOf(TypeSpare);
  let caption = fileName.substring(0, index);
  //取type
  let category = fileName.substring(index + 1);

  return {
    basename,
    caption,
    category
  }
}