## 配置文件（必须）

在业务项目根目录下添加文件`locales-tran.config.js`，

```js
module.exports = {
    // mode: 'namePath', // 如需生成{'xxx.xxx.xxxx': '中文'}结构的locales文件时需传入
    localesFilePath: [
        'src/locales/zh-CN.ts', // 简单配置，默认生成中文
        {
            outputPath: 'src/locales/vi.ts',
            targetLanguage: 'vie', // 指定翻译语言
        },
        {
            outputPath: 'src/locales/jp.ts',
            targetLanguage: 'jp', // 指定翻译语言
        },
    ],
    generatePathMap: new Map([
        [/[^]*src\/components\/pages\/components\//, 'locales.components'],
        [/[^]*src\/components\/pages\//, 'locales.pages'],
        [/[^]*src\/pages\//, 'locales.pages'],
    ]),
    formatOutPutPathInJSX: (path) => {
        return `XXX.get('${path}')`;
    },
    formatLocalesKey: (key, defaultFormatFunc) => {
        return defaultFormatFunc(key);
    },
    sourceMap: true,
    // fileType: "json", // localesFilePath中文件为JSON格式时传入
    ignorePaths: [
        'src/components/pages/components/gong/supervision/index.tsx', // 忽略指定文件
        'src/components/pages/components/ignorePaths/', // 忽略某目录下所有文件
    ],
    // 使用`git diff`命令时，根据分支名自动获取`commit id`
    gitDiffBrancheName: 'master',
    autoImport: {
        source: '../locales', // import from的路径
        // importDefaultSpecifier: 'Locales', // export default 导出形式的引入 => import Locales from '../locales';
        importSpecifier: 'useIntl', // export {} 导出形式的引入 => import { useIntl } from '../locales';
    },
    showLog: false,
    showDebug: false,
};
```

### 字段说明

| 字段名称              | 说明                                                                                                                                                                                                           | 是否必传 |
| --------------------- | -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- | -------- |
| localesFilePath       | 项目中 locales 资源文件的路径，支持对象形式以指定生成语言                                                                                                                                                      | 必须     |
| generatePathMap       | 从中文所在文件生成到资源文件时对应的 key 的前缀的生成规则                                                                                                                                                      | 必须     |
| mode                  | 仅在需要生成平面结构时需传入'namePath'（例如 umi 项目），默认生成嵌套结构                                                                                                                                      | 非必须   |
| formatOutPutPathInJSX | 替换代码文件中的中文时调用，用于格式化生成的资源路径，例如在 umi 项目中需要使用 umi 提供的方法获取中文时可按实际需要编写                                                                                       | 非必须   |
| formatLocalesKey      | 在生成资源文件中的键值对时调用，用于格式化资源文件的 key。使用场景：比如生成的 key 过长，造成阅读不便时，可以使用此字段进行格式化，第二个形参是默认提供出来的格式化方法，调用后会生成类似 i18n 结构的 key 值。 | 非必须   |
| sourceMap             | 将中文以注释形式生成至页面中，以方便阅读，默认不开启                                                                                                                                                           | 非必须   |
| fileType              | 当 localesFilePath 对应的资源文件为 JSON 格式时传入"json"，否则不传                                                                                                                                            | 非必须   |
| ignorePaths           | 脚本遍历到的路径 includes 数组中的某一路径时，跳过生成                                                                                                                                                         | 非必须   |
| gitDiffBrancheName    | 使用`git diff`命令时，根据分支名自动获取`commit id`                                                                                                                                                            | 非必须   |
| autoImport            | 翻译过程中自动引入资源文件或 i18n 的配置                                                                                                                                                                       | 非必须   |
| showLog               | 是否显示 log，默认 true                                                                                                                                                                                        | 非必须   |
| showDebug             | 是否显示调试信息，默认 false                                                                                                                                                                                   | 非必须   |

## 脚本命令

1. 在业务项目的根目录下执行以下脚本：

```shell
    node ../udesk_fe/packages/locales-tran/src/index.js [代码文件路径]
```

e.g. node ../udesk_fe/packages/locales-tran/src/index.js src/pages/auth/access-denied.tsx

或

在业务项目的 package.json 中配置相应的 script 替代`node ../udesk_fe/packages/locales-tran/src/index.js`部分以简化操作。

2. 以当前中文资源文件为模板生成新的多语言资源：

```shell
    node ../udesk_fe/packages/locales-tran/src/locales-file-tran.js [中文资源文件路径] [新语言资源文件路径] [目标语言]
```

| 名称               | 代码 | 语种检测 | 名称               | 代码 | 语种检测 | 名称                 | 代码 | 语种检测 |
| ------------------ | ---- | -------- | ------------------ | ---- | -------- | -------------------- | ---- | -------- |
| 阿拉伯语           | ara  | 是       | 爱尔兰语           | gle  | 是       | 奥克语               | oci  | 是       |
| 阿尔巴尼亚语       | alb  | 是       | 阿尔及利亚阿拉伯语 | arq  | 否       | 阿肯语               | aka  | 否       |
| 阿拉贡语           | arg  | 否       | 阿姆哈拉语         | amh  | 是       | 阿萨姆语             | asm  | 是       |
| 艾马拉语           | aym  | 否       | 阿塞拜疆语         | aze  | 是       | 阿斯图里亚斯语       | ast  | 是       |
| 奥塞梯语           | oss  | 否       | 爱沙尼亚语         | est  | 是       | 奥杰布瓦语           | oji  | 否       |
| 奥里亚语           | ori  | 是       | 奥罗莫语           | orm  | 否       |
| 波兰语             | pl   | 是       | 波斯语             | per  | 是       | 布列塔尼语           | bre  | 是       |
| 巴什基尔语         | bak  | 否       | 巴斯克语           | baq  | 是       | 巴西葡萄牙语         | pot  | 否       |
| 白俄罗斯语         | bel  | 是       | 柏柏尔语           | ber  | 是       | 邦板牙语             | pam  | 否       |
| 保加利亚语         | bul  | 是       | 北方萨米语         | sme  | 否       | 北索托语             | ped  | 否       |
| 本巴语             | bem  | 否       | 比林语             | bli  | 否       | 比斯拉马语           | bis  | 否       |
| 俾路支语           | bal  | 否       | 冰岛语             | ice  | 是       | 波斯尼亚语           | bos  | 是       |
| 博杰普尔语         | bho  | 否       |
| 楚瓦什语           | chv  | 否       | 聪加语             | tso  | 否       |
| 丹麦语             | dan  | 是       | 德语               | de   | 是       | 鞑靼语               | tat  | 是       |
| 掸语               | sha  | 否       | 德顿语             | tet  | 否       | 迪维希语             | div  | 否       |
| 低地德语           | log  | 是       |
| 俄语               | ru   | 是       |
| 法语               | fra  | 是       | 菲律宾语           | fil  | 是       | 芬兰语               | fin  | 是       |
| 梵语               | san  | 否       | 弗留利语           | fri  | 否       | 富拉尼语             | ful  | 否       |
| 法罗语             | fao  | 否       |
| 盖尔语             | gla  | 否       | 刚果语             | kon  | 否       | 高地索布语           | ups  | 否       |
| 高棉语             | hkm  | 是       | 格陵兰语           | kal  | 否       | 格鲁吉亚语           | geo  | 是       |
| 古吉拉特语         | guj  | 是       | 古希腊语           | gra  | 否       | 古英语               | eno  | 否       |
| 瓜拉尼语           | grn  | 否       |
| 韩语               | kor  | 是       | 荷兰语             | nl   | 是       | 胡帕语               | hup  | 否       |
| 哈卡钦语           | hak  | 否       | 海地语             | ht   | 否       | 黑山语               | mot  | 否       |
| 豪萨语             | hau  | 否       |
| 吉尔吉斯语         | kir  | 否       | 加利西亚语         | glg  | 是       | 加拿大法语           | frn  | 否       |
| 加泰罗尼亚语       | cat  | 是       | 捷克语             | cs   | 是       |
| 卡拜尔语           | kab  | 是       | 卡纳达语           | kan  | 是       | 卡努里语             | kau  | 否       |
| 卡舒比语           | kah  | 否       | 康瓦尔语           | cor  | 否       | 科萨语               | xho  | 是       |
| 科西嘉语           | cos  | 否       | 克里克语           | cre  | 否       | 克里米亚鞑靼语       | cri  | 否       |
| 克林贡语           | kli  | 否       | 克罗地亚语         | hrv  | 是       | 克丘亚语             | que  | 否       |
| 克什米尔语         | kas  | 否       | 孔卡尼语           | kok  | 否       | 库尔德语             | kur  | 是       |
| 拉丁语             | lat  | 是       | 老挝语             | lao  | 否       | 罗马尼亚语           | rom  | 是       |
| 拉特加莱语         | lag  | 否       | 拉脱维亚语         | lav  | 是       | 林堡语               | lim  | 否       |
| 林加拉语           | lin  | 否       | 卢干达语           | lug  | 否       | 卢森堡语             | ltz  | 否       |
| 卢森尼亚语         | ruy  | 否       | 卢旺达语           | kin  | 是       | 立陶宛语             | lit  | 是       |
| 罗曼什语           | roh  | 否       | 罗姆语             | ro   | 否       | 逻辑语               | loj  | 否       |
| 马来语             | may  | 是       | 缅甸语             | bur  | 是       | 马拉地语             | mar  | 否       |
| 马拉加斯语         | mg   | 是       | 马拉雅拉姆语       | mal  | 是       | 马其顿语             | mac  | 是       |
| 马绍尔语           | mah  | 否       | 迈蒂利语           | mai  | 是       | 曼克斯语             | glv  | 否       |
| 毛里求斯克里奥尔语 | mau  | 否       | 毛利语             | mao  | 否       | 孟加拉语             | ben  | 是       |
| 马耳他语           | mlt  | 是       | 苗语               | hmn  | 否       |
| 挪威语             | nor  | 是       | 那不勒斯语         | nea  | 否       | 南恩德贝莱语         | nbl  | 否       |
| 南非荷兰语         | afr  | 是       | 南索托语           | sot  | 否       | 尼泊尔语             | nep  | 是       |
| 葡萄牙语           | pt   | 是       | 旁遮普语           | pan  | 是       | 帕皮阿门托语         | pap  | 否       |
| 普什图语           | pus  | 否       |
| 齐切瓦语           | nya  | 否       | 契维语             | twi  | 否       | 切罗基语             | chr  | 否       |
| 日语               | jp   | 是       | 瑞典语             | swe  | 是       |
| 萨丁尼亚语         | srd  | 否       | 萨摩亚语           | sm   | 否       | 塞尔维亚-克罗地亚语  | sec  | 否       |
| 塞尔维亚语         | srp  | 是       | 桑海语             | sol  | 否       | 僧伽罗语             | sin  | 是       |
| 世界语             | epo  | 是       | 书面挪威语         | nob  | 是       | 斯洛伐克语           | sk   | 是       |
| 斯洛文尼亚语       | slo  | 是       | 斯瓦希里语         | swa  | 是       | 塞尔维亚语（西里尔） | src  | 否       |
| 索马里语           | som  | 是       |
| 泰语               | th   | 是       | 土耳其语           | tr   | 是       | 塔吉克语             | tgk  | 是       |
| 泰米尔语           | tam  | 是       | 他加禄语           | tgl  | 是       | 提格利尼亚语         | tir  | 否       |
| 泰卢固语           | tel  | 是       | 突尼斯阿拉伯语     | tua  | 否       | 土库曼语             | tuk  | 否       |
| 乌克兰语           | ukr  | 是       | 瓦隆语             | wln  | 是       | 威尔士语             | wel  | 是       |
| 文达语             | ven  | 否       | 沃洛夫语           | wol  | 否       | 乌尔都语             | urd  | 是       |
| 西班牙语           | spa  | 是       | 希伯来语           | heb  | 是       | 希腊语               | el   | 是       |
| 匈牙利语           | hu   | 是       | 西弗里斯语         | fry  | 是       | 西里西亚语           | sil  | 否       |
| 希利盖农语         | hil  | 否       | 下索布语           | los  | 否       | 夏威夷语             | haw  | 否       |
| 新挪威语           | nno  | 是       | 西非书面语         | nqo  | 否       | 信德语               | snd  | 否       |
| 修纳语             | sna  | 否       | 宿务语             | ceb  | 否       | 叙利亚语             | syr  | 否       |
| 巽他语             | sun  | 否       |
| 英语               | en   | 是       | 印地语             | hi   | 是       | 印尼语               | id   | 是       |
| 意大利语           | it   | 是       | 越南语             | vie  | 是       | 意第绪语             | yid  | 否       |
| 因特语             | ina  | 否       | 亚齐语             | ach  | 否       | 印古什语             | ing  | 否       |
| 伊博语             | ibo  | 否       | 伊多语             | ido  | 否       | 约鲁巴语             | yor  | 否       |
| 亚美尼亚语         | arm  | 是       | 伊努克提图特语     | iku  | 否       | 伊朗语               | ir   | 否       |
| 中文(简体)         | zh   | 是       | 中文(繁体)         | cht  | 是       | 中文(文言文)         | wyw  | 是       |
| 中文(粤语)         | yue  | 是       | 扎扎其语           | zaz  | 否       | 中古法语             | frm  | 否       |
| 祖鲁语             | zul  | 否       | 爪哇语             | jav  | 否       |

## 命令介绍

安装：

```bash
npm install -g locales-tran
```

### 翻译本地中文文件到指定文件格式

```bash
locale-tran lft --chinese-path=[中文资源文件路径] --output-lang-path=[新语言资源文件路径] --target-lang=[目标语言] --translate-variable-template=[是否翻译变量模板]

```

> lft 是`locales-file-tran`的简写
> translate-variable-template 可以不填写

### 根据提交号或者分支名提取资源文件

```bash
locale-tran gd --commit-hash=[提交commit id]

```

> `--commit-hash`选填，如果不填写则读取配置文件中`gitDiffBrancheName`配置

### 提取指定文件或者目录资源文件

```bash
locale-tran lt ----file-path=[代码文件路径]

```

### 错误码列表

| 错误码 | 含义                                                     | 解决方案                                                                       |
| ------ | -------------------------------------------------------- | ------------------------------------------------------------------------------ |
| 0      | 成功                                                     |                                                                                |
| 52001  | 请求超时                                                 | 请重试                                                                         |
| 52002  | 服务端系统错误                                           | 请重试                                                                         |
| 52003  | 未授权用户                                               | 请检查 appid 是否正确或者服务是否开通                                          |
| 52010  | 开放设备授权容量不足                                     | 联系管理员扩增容量                                                             |
| 54000  | 必填参数为空或固定参数有误                               | 检查参数是否误传                                                               |
| 54001  | 签名错误                                                 | 请检查您的签名生成方法                                                         |
| 54003  | 访问频率受限                                             | 请降低您的调用频率                                                             |
| 54004  | 账户余额不足                                             | 请前往管理控制平台为账户充值                                                   |
| 54005  | 长 query 请求频繁                                        | 请降低长 query 的发送频率，3s 后再试                                           |
| 58000  | 客户端 IP 非法                                           | 检查个人资料里填写的 IP 地址是否正确可前往管理控制平台修改，IP 限制，IP 可留空 |
| 58001  | 译文语言方向不支持                                       | 检查译文语言是否在语言列表里                                                   |
| 69001  | 上传图片数据有误                                         | 检查图片是否有问题                                                             |
| 69002  | 图片识别超时                                             | 请重试                                                                         |
| 69003  | 内容识别失败                                             | 检查图片是否存在内容后重试                                                     |
| 69004  | 识别内容为空                                             | 检查图片是否存在内容后重试                                                     |
| 69005  | 图片大小超限（超过 4M）                                  | 请上传符合图片大小要求的图片                                                   |
| 69006  | 图片尺寸不符合标准（最短边至少 30px，最长边最大 4096px） | 请上传符合图片尺寸要求的图片                                                   |
| 69007  | 图片格式不支持（png/jpg）                                | 请上传 png 或 jpg 格式的图片                                                   |
| 69008  | 设备号为空                                               | 检查 cuid 参数                                                                 |
| 69012  | 文字贴合参数异常                                         | 请检查参数 paste，枚举示例：0-关闭文字贴合 1-返回整图贴合 2-返回块区贴合       |
