// @ts-nocheck
/**
 * 由于 Webpack5 的 module federation 特性，三方库的包都是异步加载的，
 * 本来写在webpack入口文件的代码应该是最先执行的，但由于异步加载机制的存在，
 * 需要等待三方库的包都异步加载完成后才会执行。
 *
 * 而我们最依赖的 document.currentScript 用来获取主script对象并向外传递SdkClass，
 * 此时 document.currentScript 已经不是最初的主js的script了，而是最后一个less文件！
 *
 * 所以我们需要在主sdk的入口前面再增加一个入口文件，这个文件没有依赖三方库，所以可以在
 * 第一时间获取到script对象，并借助全局对象保存起来，传递给真正的sdk入口。同时我们也在
 * currentScript上添加mfsu标记，告诉manifest-loader真正的入口Class会延迟加载，
 * manifest-loader还可以注册一个onSdkLoaded事件回调，当入口class加载后自动触发！
 */

// 添加mfsu标记，告诉manifest-loader存在异步加载
document.currentScript.__mfsu = true;
// 保存currentScript，在主sdk入口文件(./sdk-startup)中会用到
window.document["'%UNIQUE_ID%'"] = document.currentScript;
// 这里只是一个示例，实际上会在manifest-loader中注册，在主sdk入口文件(./sdk-startup)中会执行触发
document.currentScript.onSdkLoaded = function ({ sdkClass }) {};
