import { useContext, useMemo, useState } from 'react';
import { ApplyPluginsType } from '@umijs/runtime';
import type { PluginConfig, PluginState, ResolvedRuntimeSdkConfig } from './types';
import { Context, ContextType } from './root-component';
import { SdkOptions, GlobalData } from './runtime-types';
import { GetConfigFilePluginKey, PluginConfigPluginKey, PluginStatePluginKey } from './const';
import { UdeskSdkOptions } from '@udesk/sdk';
//@ts-ignore
import { plugin } from '"%TMP_PATH%"/core/plugin';

export function useSdkOptions(): SdkOptions {
    const context = useContext<ContextType<SdkOptions>>(Context as any);
    return context.sdkOptions;
}

export function useRouterBase(): string {
    const sdkOptions = useSdkOptions();
    const sdkConfig = useSdkConfig();
    const pluginConfig = usePluginConfig();
    const routerBase = useMemo<string>(() => {
        //todo: 把配置改到sdkConfig中
        return (
            sdkOptions.routerBase ||
            sdkConfig.umiConfig?.base ||
            pluginConfig.base ||
            "'%DEFAULT_ROUTER_BASE%'" ||
            '/'
        );
    }, [sdkOptions, sdkConfig, pluginConfig]);
    return routerBase;
}

export function usePublicPath() {
    const sdkConfig = useSdkConfig();
    const pluginConfig = usePluginConfig();
    const publicPath = useMemo<string>(() => {
        return sdkConfig.umiConfig?.publicPath || pluginConfig.publicPath;
    }, [sdkConfig, pluginConfig]);
    return publicPath;
}

export function useGlobalData(): GlobalData | undefined {
    const context = useContext<ContextType<UdeskSdkOptions, GlobalData>>(Context);
    return context.globalConfig;
}

export function getSdkConfig(): ResolvedRuntimeSdkConfig {
    return plugin.applyPlugins({
        key: GetConfigFilePluginKey,
        type: ApplyPluginsType.modify,
        initialValue: {},
    });
}
export function useSdkConfig() {
    return useMemo(() => getSdkConfig(), []);
}

export function getPluginConfig(): PluginConfig {
    return plugin.applyPlugins({
        key: PluginConfigPluginKey,
        type: ApplyPluginsType.modify,
        initialValue: {},
    });
}
export function usePluginConfig() {
    return useMemo(() => getPluginConfig(), []);
}

export function getPluginState(): PluginState {
    return plugin.applyPlugins({
        key: PluginStatePluginKey,
        type: ApplyPluginsType.modify,
        initialValue: {},
    });
}
export function usePluginState() {
    return useMemo(() => getPluginState(), []);
}
