import React, { FC, useEffect, useState, useCallback } from 'react';
import type { IRoute } from '@umijs/renderer-react';
import { UdeskSdkOptions, RootContext } from '@udesk/sdk';

export interface ContextType<TO extends UdeskSdkOptions = UdeskSdkOptions, TG = any> {
    sdkOptions: TO;
    globalConfig?: TG;
}
export const Context = React.createContext<ContextType>({ sdkOptions: undefined });
Context.displayName = 'RootSdkContext';

export type Props = {
    rootContext: RootContext;
    routes: IRoute[];
};

const UdeskSdkRoot: FC<Props> = props => {
    const { rootContext } = props;

    // 创建 contextValue
    const [contextValue, setContextValue] = useState<ContextType>(() => {
        return {
            sdkOptions: rootContext.sdkOptions,
            globalConfig: rootContext.globalConfig,
        };
    });

    useEffect(() => {
        // 接收 sdk 发送过来的版本变化的通知
        // rootContext.onVersionChanged = (version: string, oldVersion: string) => {
        //     setNewVersion(true);
        // };

        // 接收 SDK 的 update 方法调用的通知，内部进行重新渲染
        rootContext.onSdkOptionsChanged = (sdkOptions?: UdeskSdkOptions) => {
            // 有 sdkOptions 则更新，没有则重新渲染
            if (sdkOptions) {
                contextValue.sdkOptions = { ...contextValue.sdkOptions, ...sdkOptions };
            }
            setContextValue({ ...contextValue });
        };
        return () => {
            rootContext.onSdkOptionsChanged = () => {};
        };
    }, [rootContext]);

    // const [newVersion, setNewVersion] = useState<boolean>(false);
    // const onReload = useCallback(() => {
    //     setNewVersion(false);
    //     // UdeskCaseReactSdkClass.reload(
    //     //     () => {},
    //     //     () => {
    //     //         this.state.newVersionInstance[RELOAD_SYMBOL](false);
    //     //     }
    //     // );
    // }, []);
    return (
        // <React.StrictMode>
        <Context.Provider value={contextValue}>
            {/* <NewVersion
                    isVisible={newVersion}
                    autoReloadButtonDisabled={true}
                    onReload={onReload}
                /> */}
            {props.children}
        </Context.Provider>
        // </React.StrictMode>
    );
};
UdeskSdkRoot.displayName = 'UdeskSdkRoot';
export default UdeskSdkRoot;
