import React from 'react';
import type { History, Plugin } from 'umi/lib/cjs';
import { ApplyPluginsType } from '@umijs/runtime';
import { IRoute } from '@umijs/renderer-react';
import { GetRootContextPluginKey } from './const';
import SdkWrapper from './root-component';
import { RootContext } from '@udesk/sdk';

export function rootContainer(
    container: React.ReactNode,
    {
        routes,
        history,
        plugin,
        sdkClassUuid,
    }: { routes: IRoute[]; history: History; plugin: Plugin; sdkClassUuid: string }
) {
    const rootContexts: RootContext = plugin.applyPlugins({
        key: GetRootContextPluginKey,
        type: ApplyPluginsType.modify,
        /**
         * 如果作为一个主应用或者主sdk运行的模式下，应该在 ./umi-sdk-base 文件中注册事件时便指定了默认值，此处应该能获取到初始值
         * 但如果是一个umi sdk被一个常规class继承使用时（类似cti-phone中集成）
         */
        initialValue: {}, // 正常情况下应该有值，
    });
    const rootContext = rootContexts[sdkClassUuid];

    return React.createElement(SdkWrapper, { routes, rootContext }, container);
}
