import { useCallback, useEffect, useState, useMemo, useContext } from 'react';
import type { SdkRuntimeContext } from './types';
import type { UmiSdkOptions } from './umi-sdk-base';
import ReactComponentProxy from 'udesk-react/src/udesk/sdk/react-component-proxy';
import { Context } from './root-component';
import { GetConfigFilePluginKey } from './const';
import { plugin } from './umi-sdk-base';
import { ApplyPluginsType } from '@umijs/runtime';
/**
 * 根据Plugin编译时的一些内部状态信息，精确返回实例化SDK时的SdkOptions
 *
 * @export
 * @param {PluginConfig} pluginConfig Plugin编译时的配置对象
 * @param {PluginState} pluginState Plugin编译时的一些内部状态，可以知道当前正在编译的sdkName等信息
 * @returns {UmiSdkOptions} 实例化SDK时传入的SdkOptions
 */
export function getSdkOptions(
    initialValue,
    { sdkConfig, pluginConfig, pluginState }: SdkRuntimeContext
): UmiSdkOptions {
    return initialValue;
}

// 先放这里，后面再看放哪里合适
/**
 * 用代理包装的打包组件
 *
 * @export
 * @param {*} props
 * @return {*}
 */
export function ComponentComponent(props) {
    const sdkContext = useContext(Context);
    const sdkOptions = useMemo(
        function () {
            return sdkContext?.sdkOptions || {};
        },
        [sdkContext?.sdkOptions]
    );
    const componentName = useMemo(
        function () {
            return sdkOptions?.componentName;
        },
        [sdkOptions?.componentName]
    );
    const umiSdkOptions = plugin.applyPlugins({
        key: GetConfigFilePluginKey,
        type: ApplyPluginsType.modify,
        initialValue: {},
    });

    const rest = useMemo(
        function () {
            const {
                appClientPrefix,
                appServerPrefix,
                enableGlobalConfig,
                enableNewVersionTip,
                componentName,
                enableSentry,
                enableVersionMonitoring,
                globalConfigFilePath,
                landingPath,
                lang,
                prefixCls,
                reloadCallback,
                routerBase,
                sdkClientPrefix,
                sdkServerPrefix,
                versionMonitorInterval,
                appendRenderMode,
                getSentryTagsContext,
                getSentryUser,
                versionFilePath,
                ...rest
            } = sdkOptions;
            return rest;
        },
        [sdkOptions]
    );
    const [outerInstanceInstance, setOuterInstanceInstance] = useState<null | object>(null);

    const onReactComponentProxyCreated = useCallback(instance => {
        setOuterInstanceInstance(instance);
    }, []);
    const onReactComponentProxyDestroyed = useCallback(() => {
        setOuterInstanceInstance(null);
    }, []);

    const componentsMapping = umiSdkOptions?.routes;
    const componentConfig = componentsMapping?.find(component => component.name === componentName);
    const component = componentConfig?.component;

    useEffect(
        function () {
            outerInstanceInstance?.updateProps(rest);
        },
        [rest]
    );
    // 如果component为空是异常，这里兼容处理下
    // 最好抛出个错误
    return component ? (
        <ReactComponentProxy
            component={component}
            options={rest}
            onCreated={onReactComponentProxyCreated}
            onDestroyed={onReactComponentProxyDestroyed}
        />
    ) : null;
}
