import React, { Component, createRef, Fragment } from 'react';
import Options                                   from './Options';
import isNil                                     from 'lodash-es/isNil';
import PropTypes                                 from 'prop-types';

class CommonDropdown extends Component {
    dropdownMenuRef = createRef();

    static defaultProps = {
        search : function(keyword, {options, getOptionLabel}) {
            return options.filter(option => {
                let label = getOptionLabel(option);
                if (typeof label === 'string') {
                    return label.toLowerCase().indexOf(keyword.toLowerCase()) > -1;
                }
                return true;
            });
        }
    };

    constructor(props) {
        super(props);
        this.state = {
            keyword : ''
        };
    }

    render() {
        let {
                getOptionLabel,
                getOptionValue,
                onChange,
                options,
                value,
                t = (v) => v,
                isSearchable,
                search
            } = this.props;

        let {keyword} = this.state;

        if (isNil(isSearchable)) {
            isSearchable = options.length > 10;
        }

        if (keyword && typeof search === 'function') {
            options = search(keyword, {options, getOptionLabel}) || options;
        }

        return (
            <div className="dropdown-menu-left">
                {isSearchable &&
                 <Fragment>
                     <i className="icon icon-search"/>
                     <div className="dropdown-menu-search">
                         <input placeholder={t('label.search')}
                                onChange={(e) => this.setState({keyword : e.target.value})}
                         />
                     </div>
                 </Fragment>}
                <ul className="dropdown-menu-ul scrollbar" ref={this.dropdownMenuRef}>
                    <Options getOptionLabel={getOptionLabel}
                             getOptionValue={getOptionValue}
                             options={options}
                             value={value}
                             onChange={onChange}/>
                </ul>
            </div>
        );
    }
}

CommonDropdown.propTypes = {
    getOptionLabel : PropTypes.func,
    getOptionValue : PropTypes.func,
    value          : PropTypes.any,
    onChange       : PropTypes.func,
    options        : PropTypes.arrayOf(PropTypes.any),
    isSearchable   : PropTypes.bool,
    search         : PropTypes.func
};

export default CommonDropdown;
