import React, { Component, createRef, Fragment } from 'react';
import Options from './Options';
import forEach from 'lodash-es/forEach';
import PropTypes from 'prop-types';

class CommonDropdown extends Component {
    dropdownMenuRef = createRef();

    constructor(props) {
        super(props);
    }

    render() {
        let {
            getOptionLabel,
            getOptionValue,
            onChange,
            options,
            value,
            t = (v) => v
        } = this.props;
        return (
            <div className="dropdown-menu-left">
                {do {
                    if (options.length > 10) {
                    <Fragment>
                        <i className="icon icon-search" />
                        <div className="dropdown-menu-search">
                            <input placeholder={t('label.searchPlaceholder')}
                                onChange={(e) => this.filter(e.target.value)}
                            />
                        </div>
                    </Fragment>;
                    }
                }}
                <ul className="dropdown-menu-ul scrollbar" ref={this.dropdownMenuRef}>
                    <Options getOptionLabel={getOptionLabel}
                        getOptionValue={getOptionValue}
                        options={options}
                        value={value}
                        onChange={onChange} />
                </ul>
            </div>
        );
    }

    filter = (keyword) => {
        if (keyword) {
            forEach(this.dropdownMenuRef.current.children, (li) => {
                if (li.innerText.indexOf(keyword) === -1) {
                    li.style.display = 'none';
                } else {
                    li.removeAttribute('style');
                }
            });
            return;
        }
        forEach(this.dropdownMenuRef.current.children, li => li.removeAttribute('style'));
    };
}

CommonDropdown.propTypes = {
    getOptionLabel: PropTypes.func,
    getOptionValue: PropTypes.func,
    value: PropTypes.arrayOf(PropTypes.any),
    onChange: PropTypes.func,
    options: PropTypes.arrayOf(PropTypes.any)
};

export default CommonDropdown;
