import isArray   from 'lodash-es/isArray';
import isEmpty   from 'lodash-es/isEmpty';
import isNumber  from 'lodash-es/isNumber';
import PropTypes from 'prop-types';
import React     from 'react';

let Control = function({prefix, value = [], placeholder, t = v => v, isOpen, unitLabel, controlClassName, ...props}) {
    return (
        <div {...props} className={controlClassName} onClick={props.onClick}>
                <span>
                    {prefix} {do {
                    if (isArray(value) && !isEmpty(value)) {
                        if (isNumber(value[0]) && isNumber(value[1])) {
                            value.join('-') + unitLabel;
                        } else if (isNumber(value[0])) {
                            t('label.great_than') + value[0] + unitLabel;
                        } else if (isNumber(value[1])) {
                            t('label.less_than') + value[1] + unitLabel;
                        } else {
                            placeholder;
                        }
                    } else {
                        placeholder;
                    }
                }}
                </span>
            <i className={`icon icon-caret-${isOpen ? 'up' : 'down'}`}/>
        </div>
    );
};

Control.propTypes = {
    prefix      : PropTypes.string,
    className   : PropTypes.string,
    placeholder : PropTypes.string,
    value       : PropTypes.arrayOf(PropTypes.number),
    isOpen      : PropTypes.bool,
    t           : PropTypes.func,
    onClick     : PropTypes.func,
    unitLabel   : PropTypes.string
};

Control.defaultProps = {
    prefix      : '',
    placeholder : '',
    isOpen      : false,
    className   : 'form-control'
};

export default Control;
