import React, { Component, createRef } from 'react';
import PropTypes                       from 'prop-types';
import NumberInput                     from '@udesk/number-input';
import map                             from 'lodash-es/map';
import isNil                           from 'lodash-es/isNil';

class Dropdown extends Component {
    _ref = createRef();

    constructor(props) {
        super(props);
        this.state = {
            value : props.value || []
        };
    }

    render() {
        let {t = v => v, unitLabel, onChange, style} = this.props;
        let {value} = this.state;
        return (
            <div className="dropdown-menu scrollbar" style={style}>
                <div className="form-group">
                    <label>{t('label.great_than')}</label>
                    <div className="input-group input-group-sm">
                        <NumberInput className='form-control' value={isNil(value[0]) ? '' : value[0] + ''}
                                     ref={this._ref}
                                     onChange={v => this.setState({value : [v, value[1]]})}
                                     onEnter={() => onChange(map(value, i => i && parseFloat(i)))}/>
                        <span className="input-group-addon">{unitLabel}</span>
                    </div>
                </div>
                <div className="form-group">
                    <label>{t('label.less_than')}</label>
                    <div className="input-group input-group-sm">
                        <NumberInput className='form-control' value={isNil(value[1]) ? '' : value[1] + ''}
                                     onChange={v => this.setState({value : [value[0], v]})}
                                     onEnter={() => onChange(map(value, i => i && parseFloat(i)))}/>
                        <span className="input-group-addon">{unitLabel}</span>
                    </div>
                </div>
                <button className="btn btn-sm btn-default" type="button"
                        onClick={() => onChange(map(value, i => i && parseFloat(i)))}>
                    {t('label.confirm')}
                </button>
                <button className="btn btn-sm btn-link" type="button"
                        onClick={() => {
                            this.setState({value : []});
                            onChange([]);
                        }}>
                    {t('label.clear')}
                </button>
            </div>
        );
    }

    componentDidMount() {
        this._ref.current.focus();
    }
}

Dropdown.propTypes = {
    t            : PropTypes.func,
    value        : PropTypes.arrayOf(PropTypes.number),
    unitLabel    : PropTypes.string,
    onClearValue : PropTypes.func,
    onChange     : PropTypes.func,
    style        : PropTypes.object
};

export default Dropdown;
