import React from 'react';
// import Udesk from '../../../udesk';
import { loadUsersAsync, addUser, deleteUsers, searchUser, searchUserLatest, searchUserLeading } from '../../../models/users/action';

class UserListComponent extends React.Component {
    static SAVED_STATE_KEYS = ["state.searchName", "state.pageNum"]
    state = {
        columns: [
            {
                title: 'Id',
                dataIndex: 'id',
                visible: this.props.visibleColumnNames.length === 0 || this.props.visibleColumnNames.includes("Id"),
                width: 50
            }, {
                title: '姓名',
                dataIndex: 'name',
                visible: this.props.visibleColumnNames.length === 0 || this.props.visibleColumnNames.includes("name"),
                render: (text, record) => {
                    if (record.id === this.props.currentUserId) {
                        return <span className="current-user">[{text}]</span>;
                    }
                    else {
                        return text;
                    }
                },
                width: 150
            }, {
                title: '角色',
                dataIndex: 'role',
                visible: this.props.visibleColumnNames.length === 0 || this.props.visibleColumnNames.includes("role"),
                render: (text, record) => {
                    if (this.props.roles) {
                        return this.props.roles.filter(r => r.id === text).map(r => r.name)[0] || "未知";
                    }
                    else {
                        return "...";
                    }
                },
                width: 150
            }, {
                title: '头像',
                dataIndex: 'avatar',
                visible: this.props.visibleColumnNames.length === 0 || this.props.visibleColumnNames.includes("avatar"),
                render(text, record) {
                    return (<img src={text} alt={`${record.name}的头像`} height="50" />);
                },
                width: 150
            }, {
                title: '更新时间',
                dataIndex: 'updateDate',
                visible: this.props.visibleColumnNames.length === 0 || this.props.visibleColumnNames.includes("updateDate"),
                width: 100
            },],
        searchName: undefined,
        selectedRowKeys: []
    }

    componentDidMount() {
        if (!this.props.users) {
            this.props.dispatch(loadUsersAsync());
        }
        if (!this.props.roles) {
            this.props.getRolesAsync();
        }
    }

    actions = {
        search() {
            this.props.dispatch(searchUser({
                name: this.state.searchName
            }));
        },
        debounceSearch() {
            this.props.dispatch(searchUserLatest({
                name: this.state.searchName
            }));
        },
        throttlingSearch() {
            this.props.dispatch(searchUserLeading({
                name: this.state.searchName
            }));
        },
        onSelectChanged(selectedRowKeys, selectedRows) {
            this.setState({
                selectedRowKeys
            });
        },
        onRowClick(record, event) {
            this.props.setCurrentUserId(record.id);
            this.props.history.push(`/redux/user-detail/${record.id}`);
        },
        onColumnsVisibleChanged(columns) {
            this.props.chooseColumns(columns.map(c => c.dataIndex));
        },
        addUser() {
            this.props.dispatch(addUser({
                id: this.props.users.length + 1,
                name: "新人" + (this.props.users.filter(u => u.name.includes("新人")).length + 1),
                role: 4,
                avatar: "https://dss0.bdstatic.com/70cFuHSh_Q1YnxGkpoWK1HF6hhy/it/u=3738324299,1361227017&fm=26&gp=0.jpg",
                createDate: new Date().toISOString(),
                updateDate: new Date().toISOString(),
            }));
            if (this.props.searchedUsers) {
                this.actions.search();
            }
        },
        removeUsers() {
            this.props.dispatch(deleteUsers(this.state.selectedRowKeys));
            if (this.props.searchedUsers) {
                this.actions.search();
            }
        }
    }
}

UserListComponent.propTypes = {
};

UserListComponent.defaultProps = {
};

export default UserListComponent;