import { createAction } from '@reduxjs/toolkit';
import UserListComponent from "./component";
import { uniqueReduxKey as userModelUniqueReduxKey } from '../../../models/users';

// V2
UserListComponent.reduxReducers = {
    chooseColumns: (state, action, meta) => ({ ...state, visibleColumnNames: action.payload }),
    setCurrentUserId: (state, action, meta) => ({ ...state, currentUserId: action.payload }),
    getRolesSuccess: (state, action, meta) => ({ ...state, roles: action.payload }),
    // "static getRolesSuccess": (state, action, meta) => ({ ...state, roles: action.payload }),
};
UserListComponent.defaultState = {
    visibleColumnNames: [],
    currentUserId: undefined,
    roles: null,
};

UserListComponent.mapStateToProps = function ({ state, ownProps }) {
    return state;
};
UserListComponent.mapDispatchToProps = function ({ actions: boundActions, dispatch, ownProps }) {
    return boundActions;
};
// UserListComponent.mergeProps = function ({ stateProps, dispatchProps, ownProps }) {
//     return { ...ownProps, ...stateProps, ...dispatchProps };
// };
UserListComponent.mergeState = function (watcherStates = [/* { watcher, state} */], ownState) {
    return watcherStates.map(s => s.state).reduce((a, b) => ({ ...b, ...a }), ownState);
};
UserListComponent.mergeDispatch = function (watcherDispatches = [/* { watcher, dispatch} */], ownDispatch) {
    return watcherDispatches.map(d => d.dispatch).reduce((a, b) => ({ ...b, ...a }), ownDispatch);
};

UserListComponent.reduxActions = {
    extraActionA: createAction("action A"),
    extraActionB: createAction("action B"),
    // thunk actions
    deleteUserAsync: ({ actions }) => (dispatch, getState) => {
        setTimeout(() => {
            dispatch(actions.deleteUser());
        }, 1000);
    },
};

UserListComponent.reduxWatcher = [userModelUniqueReduxKey];
/*
   1. 组件类。监听组件类的全局级别
   2. 字符串。直接使用uniqueReduxKey监听，全局级别。
   3. { component: Class, reduxKey: "" }，支持监听组件类的全局级别，和实例级别
   4. { uniqueReduxKey: "" }，与字符串相同
   5. 可以是数组
*/

UserListComponent.saga = {
    takeEvery: {
        getRolesAsync({ actions }) {
            return {
                saga: this.getRolesPromise,
                successAction: "getRolesSuccess",
                errorAction: "getRolesError"
            };
        }
    },
    takeLatest: {},
    takeLeading: {
    },
    // * watchSaga({ actions }) {
    //     yield takeEvery("getRoles", UserDetailComponent.saga.getRoles);
    // },
    // * getRoles(action) {
    //     try {
    //         const roles = yield call(UserDetailComponent.saga.getRolesPromise, action.payload.userId);
    //         yield put({ type: "getRolesSuccess", payload: roles });
    //     }
    //     catch (error) {
    //         yield put({ type: "getRolesError", payload: error, error: true });
    //     }
    // },
    getRolesPromise(action, { state }) {
        return new Promise((resolve, reject) => {
            setTimeout(() => {
                resolve([
                    { id: 1, name: "管理员" },
                    { id: 2, name: "经理" },
                    { id: 3, name: "组长" },
                    { id: 4, name: "普通用户" },
                ]);
            }, 2000);
        });
    }
};


export default UserListComponent;



// // V1
// let actions = {
//     getUser: createAction("getUser"),
//     saveUser: createAction("saveUser"),
//     deleteUser: createAction("deleteUser"),
// };

// UserDetailComponent.reduxReducers = {
//     [actions.getUser]: (state, action, meta) => ({ ...state, user: { userId: action.payload, name: "first user" } }),
//     [actions.saveUser]: (state, action, meta) => ({ ...state, createdDate: Date.now() }),
//     [actions.deleteUser]: (state, action, meta) => ({ ...state, deletedDate: Date.now() }),
// };

// UserDetailComponent.defaultState = {
//     user: null,
//     createdDate: null,
//     deletedDate: null,
//     roles: null,
// };

// UserDetailComponent.mapStateToProps = function ({ state, ownProps }) {
//     // return state[key] || UserDetailComponent.defaultState;
//     return state;
// };

// UserDetailComponent.mapDispatchToProps = function ({ actions: boundActions, dispatch, ownProps }) {
//     // return { ...bindActionCreators(UserListActions, dispatch), ...bindActionCreators(actions, dispatch) };
//     return boundActions;
// };

// export {
//     actions
// };