import PropTypes           from 'prop-types';
import React, { Fragment } from 'react';
import forEach             from 'lodash-es/forEach';
import includes            from 'lodash-es/includes';
import isFunction          from 'lodash-es/isFunction';
import join                from 'lodash-es/join';
import map                 from 'lodash-es/map';
import split               from 'lodash-es/split';
import { library }         from '@fortawesome/fontawesome-svg-core';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faTimesCircle }   from '@fortawesome/free-solid-svg-icons';

library.add(faTimesCircle);

class UdeskTimePicker extends React.Component {
    state = {
        HH         : split(this.props.time, ':', 3)[0] || '',
        mm         : split(this.props.time, ':', 3)[1] || '',
        ss         : split(this.props.time, ':', 3)[2] || '',
        beforeTime : this.props.time, showPicker : false
    };
    _ref = React.createRef();

    static getDerivedStateFromProps(props, state) {
        if (props.time !== state.beforeTime) {
            return {
                HH         : split(props.time, ':', 3)[0] || '',
                mm         : split(props.time, ':', 3)[1] || '',
                ss         : split(props.time, ':', 3)[2] || '',
                beforeTime : props.time
            };
        }
        return null;
    }

    constructor(props) {
        super(props);

        this._leadingZero = this._leadingZero.bind(this);
        this._range = this._range.bind(this);
        this._times = this._times.bind(this);
        this.onSelectTime = this.onSelectTime.bind(this);
        this.showUdeskTimePicker = this.showUdeskTimePicker.bind(this);
        this._sendOutSideClick = this._sendOutSideClick.bind(this);
    }

    componentDidMount() {
        document.addEventListener('mousedown', this._sendOutSideClick);
        let {
                time
            } = this.props;
        this.setState({
            time : time
        });
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this._sendOutSideClick);
    }

    _sendOutSideClick(e) {
        //eslint-disable-next-line no-console
        if (!contains(this._ref.current, e.target)) {
            this.setState({
                showPicker : false
            });
        }
    }

    _leadingZero(num, size) {
        let str = String(num);
        while (str.length < (size || 2)) { str = '0' + num; }
        return str;
    }

    _range(start, end) {
        let range = [],
            i;
        for (i = start; i < end; ++i) {
            range.push(this._leadingZero(i));
        }
        return range;
    }

    _times() {
        let times = {};
        let {format} = this.props;
        forEach(split(format, ':', 3), i => {
            const values = do {
                if (i === 'HH') {
                    this._range(0, 24);
                } else {
                    this._range(0, 60);
                }
            };
            times[i] = {values};
        });
        return times;
    }

    onSelectTime(unit, value) {
        this.setState({
            [unit] : value
        });
    }

    showUdeskTimePicker(e) {
        if (this._doNotOpenPicker) {
            this._doNotOpenPicker = false;
            return;
        }
        this.setState({
            showPicker : true
        });
    }

    _onInputChange(timeUnit, value) {
        this.setState({[timeUnit] : value});
    }

    _fireOnChange() {
        let {onChange} = this.props;
        if (!isFunction(onChange)) {
            return;
        }

        onChange(this._getFullValue());
    }

    _getFullValue() {
        if (!this.state.HH && !this.state.mm && !this.state.ss) {
            return '';
        }

        let regex = {
            HH : /^[0-1][0-9]|2[0-3]$/,
            mm : /^[0-5][0-9]$/,
            ss : /^[0-5][0-9]$/
        };

        let valueAry = [];
        forEach(this.props.format.split(':'), (timeUnit) => {
            let value = this.state[timeUnit];
            if (regex[timeUnit].test(value)) { //满足条件直接使用
                valueAry.push(value);
            } else if (/^\d$/.test(value)) { //只输入一位数字，前面补零
                valueAry.push('0' + value);
                this.setState({[timeUnit] : '0' + value});
            } else { //全不满足设为00
                valueAry.push('00');
                this.setState({[timeUnit] : '00'});
            }
        });

        return join(valueAry, ':');
    }

    componentDidUpdate(prevProps, prevState, snapshot) {
        if (!this.state.showPicker && prevState.showPicker && this._getFullValue() !== this.state.beforeTime) {
            this._fireOnChange();
        }
    }

    render() {
        let {className} = this.props;

        let {
                showPicker,
                HH, mm, ss
            } = this.state;
        let {onChange} = this.props;

        let times = this._times();

        return (
            <div className={'udesk-time-picker ' + className} ref={this._ref}>
                <div className="udesk-time-picker-input-box" onClick={this.showUdeskTimePicker}>
                    {(HH || mm || ss) && map(times, (v, i) => {
                        return <Fragment key={i}>
                            {do {
                                if (includes(['mm', 'ss'], i)) {
                                    <span>:</span>;
                                }
                            }}
                            <input type="text" className="udesk-time-picker-input" value={this.state[i]}
                                   onChange={e => this._onInputChange(i, e.target.value)}/>
                        </Fragment>;
                    })}
                    {(HH || mm || ss) && <span className='clear-btn' onClick={() => {
                        this._doNotOpenPicker = true;
                        this.setState({HH : '', mm : '', ss : ''});
                        if (isFunction(onChange) && !this.state.showPicker) {
                            onChange('');
                        }
                    }}><FontAwesomeIcon icon='times-circle'/></span>}
                </div>
                {showPicker && <div className="udesk-time-picker-panel">
                    <UlItems times={times} that={this}/>
                </div>}
            </div>
        );
    }
}

function UlItems(props) {
    return map(props.times, (time, unit) => {
        return (
            <div className="udesk-time-picker-panel-select" key={unit}>
                <ul className="udesk-time-picker-panel-select-ul">
                    <LiItems key={unit} time={time}
                             onClick={(selectedValue) => props.that.onSelectTime(unit, selectedValue)}/>
                </ul>
            </div>
        );
    });
}

function LiItems(props) {
    return props.time.values.map((item, index) => {
        return (
            <li className="udesk-time-picker-panel-select-ul-li" key={index}
                onClick={() => props.onClick(item)}>
                {item}
            </li>
        );
    });
}

function contains(root, n) {
    let node = n;
    while (node) {
        if (node === root) {
            return true;
        }
        node = node.parentNode;
    }
    return false;
}

UdeskTimePicker.propTypes = {
    format   : PropTypes.oneOf(['HH', 'HH:mm', 'HH:mm:ss']),
    time     : function(props, propName, componentName) {
        if (!/^(([0-1][0-9]|2[0-3])(:[0-5][0-9](:[0-5][0-9])?)?)?$/.test(props[propName])) {
            return new Error(
                'Invalid prop `' + propName + '` supplied to' +
                ' `' + componentName + '`. Validation failed.'
            );
        }
    },
    onChange : PropTypes.func
};

UdeskTimePicker.defaultProps = {
    time     : '',
    format   : 'HH:mm',
    onChange : null
};

export default UdeskTimePicker;
