import { ErrorWarningOutlined } from '@udesk/icons';
import React, { useEffect, useMemo, useRef } from 'react';
import { Button, Icon, Modal, Dropdown, Menu } from 'udesk-ui';
import { ModalFuncProps } from 'udesk-ui/es/modal';
import { useLocaleReceiver } from '../locale-provider/LocaleReceiver';

// const defaultConfirm = {
//   icon: <ErrorWarningOutlined />,
// };

export interface ActionMenuItem {
  label: string;
  onClick: (item: any) => void;
  danger?: boolean;
  disabled?: boolean | ((item: any) => boolean);
  confirm?: Omit<ModalFuncProps, 'onOk'>;
}

const BatchedActions = (props: any) => {
  const { menu: menuProps = [], trigger, children, getPopupContainer, disabled, ...node } = props;
  const [contextLocale] = useLocaleReceiver('BatchedActions');

  const menu = useMemo(() => {
    return (
      <Menu>
        {menuProps.map((item: any) => {
          const disabled =
            typeof item.disabled === 'function' ? item.disabled(node) : item.disabled;
          const params = {
            ...node,
            key: node.itemKey,
          };
          return (
            <Menu.Item
              danger={!disabled && item.danger}
              disabled={disabled}
              key={item.label}
              onClick={
                item.danger || item.confirm
                  ? () => {
                      Modal.confirm({
                        // ...defaultConfirm,
                        ...contextLocale.lang.confirm,
                        ...(item.confirm || {}),
                        onOk: item.onClick?.bind(null, params),
                        getContainer: getPopupContainer,
                      });
                    }
                  : item.onClick?.bind(null, params)
              }
            >
              {item.label}
            </Menu.Item>
          );
        })}
      </Menu>
    );
  }, [menuProps, node]);

  return (
    <Dropdown
      disabled={disabled}
      overlay={menu}
      trigger={trigger}
      getPopupContainer={getPopupContainer}
      className="batched-actions-dropdown"
    >
      {children}
    </Dropdown>
  );
};

export default BatchedActions;
