## Demo

```tsx
/**
 * title: 基础使用
 * desc: CommonPage为标准右侧内容区域布局组件(即红框内区域) 内置了PageHeader组件，除API中展示的props外，均透传至PageHeader
 * background: 'rgb(240, 242, 245)'
 * compact: true
 */

import React from 'react';
import { CommonFilter } from 'udesk_gm_ui';

const FIELDS = [];
for (let i = 1; i < 50; i++) {
  FIELDS.push({
    type: 'InputNumber',
    name: `keyword${i}`,
    label: `关键字${i}`,
    isDefault: Boolean(i % 2) && Boolean(i % 3),
  });
}
export default () => {
  return (
    <div style={{ color: 'rgba(0, 0, 0, .85)' }}>
      <CommonFilter
        onSearch={(value) => {
          console.log('value', value);
        }}
        fields={FIELDS}
      ></CommonFilter>
    </div>
  );
};
```

## API

| 参数 | 说明 | 类型 | 默认值 | 版本 |
| --- | --- | --- | --- | --- |
| pageBodyClassName | content 区域的 class 名，必传参数 | string |  |  |
| loading | 控制 content 区域的 loading | boolean | false |  |
| ghostPage | content 区域背景颜色是否为透明色 | boolean | false |  |
| padding | content 区域的 padding 值，默认为 16px | number | 16 |  |
| footBar | 渲染固定页脚 | ReactNode |  |  |
| navigationTabs | 用于渲染 PageHeader 下方的路由导航用的 Tabs | NavigationTabItem[]（见 Interface 部分） |  |  |
| currentNavigationPath | 当前所在路由的 url 路径，一般从`history`对象获取，详细获取方法见`路由导航Demo` | string |  |  |
| onNavigationTabsChange | Tabs 变化时的回调函数 | (tab: NavigationTabItem) => void（见 Interface 部分） |  |  |

其余 api 与`Udesk-ui`的`PageHeader`组件保持一致

## 静态方法

| 参数 | 说明 | 类型 | 默认值 | 版本 |
| --- | --- | --- | --- | --- |
| PageInfoMemo | 储存缓存的数据的组件，使用此组件包裹后即可使用 CommonPage 的数据缓存能力 | ReactNode | - |  |
| usePageStateMemo | 自定义 hook，用于储存、读取缓存的数据 | () => ({pageInfo, storePageInfo, clearPageInfo}) | - |  |

## PageInfoMemo 的实例方法

| 参数          | 说明               | 类型                                        | 默认值 | 版本 |
| ------------- | ------------------ | ------------------------------------------- | ------ | ---- |
| pageInfo      | 缓存的数据         | {[key: string]: any}                        | -      |      |
| storePageInfo | 更新缓存数据的方法 | (newPageInfo: {[key: string]: any}) => void | -      |      |
| clearPageInfo | 清除缓存数据的方法 | () => void                                  | -      |      |

## Interface

```ts
interface NavigationTabItem {
  path: string;
  text: string;
  hidden?: boolean;
  pathName?: string;
}
```
