import React, { useEffect, useState } from 'react';
import { Tabs } from 'udesk-ui';
import { PageHeaderFooterProps } from '../types';

const { TabPane } = Tabs;

const PageHeaderFooter = (props: PageHeaderFooterProps) => {
  const { antdFooter, navigationTabs = [], onChange, currentNavigationPath } = props;

  const firstNavigationTab = navigationTabs.filter((t) => !t.hidden)?.[0] ?? null;

  const [activeKey, setActiveKey] = useState<string | undefined>(firstNavigationTab?.path);

  useEffect(() => {
    if (!currentNavigationPath) return;
    const activeTabItem = navigationTabs
      .filter((t) => !t.hidden)
      .find((tab) => currentNavigationPath.endsWith(tab.path));

    // TODO 是否需要处理当前所在tab被hidden的情况？
    activeTabItem && setActiveKey(activeTabItem.path);
  }, [currentNavigationPath]);

  const onTabsChange = (activeKey: string) => {
    setActiveKey(activeKey);
    const tab = navigationTabs.find((t) => t.path === activeKey);
    if (tab) {
      onChange?.(tab);
      setActiveKey(tab.path);
    }
  };

  if (!antdFooter && !navigationTabs) return null;
  return (
    <React.Fragment>
      {antdFooter}
      <Tabs activeKey={activeKey} onChange={onTabsChange}>
        {navigationTabs.map((tab, i) => {
          if (tab.hidden) return null;
          return <TabPane tab={tab.text} key={tab.path}></TabPane>;
        })}
      </Tabs>
    </React.Fragment>
  );
};

export { PageHeaderFooter };
