## Demo

```tsx
/**
 * title: 基础使用
 * desc: CommonPage为标准右侧内容区域布局组件(即红框内区域) 内置了PageHeader组件，除API中展示的props外，均透传至PageHeader
 * background: 'rgb(240, 242, 245)'
 * compact: true
 */

import React from 'react';
import { CommonPage } from 'udesk_gm_ui';
import { Layout, Menu, Breadcrumb } from 'udesk-ui';

const { SubMenu } = Menu;
const { Header, Content, Sider } = Layout;

export default () => {
  return (
    <Layout style={{ height: 400 }}>
      <Header className="header">
        <div className="logo" style={{ width: '100%', height: '100%' }}>
          <img
            style={{ width: 'auto', height: '100%' }}
            src="https://avatars.githubusercontent.com/u/12377911?v=4"
          />
        </div>
      </Header>
      <Layout>
        <Sider width={200} className="site-layout-background">
          <Menu
            mode="inline"
            defaultSelectedKeys={['1']}
            defaultOpenKeys={['sub1']}
            style={{ height: '100%', borderRight: 0 }}
          >
            <SubMenu key="sub1" title="subnav 1">
              <Menu.Item key="1">option1</Menu.Item>
              <Menu.Item key="2">option2</Menu.Item>
              <Menu.Item key="3">option3</Menu.Item>
              <Menu.Item key="4">option4</Menu.Item>
            </SubMenu>
          </Menu>
        </Sider>
        <Content className="site-layout-background" style={{ border: '1px solid red' }}>
          {/* 以上为止都是umi提供的， 只关注组件本身即可 */}
          <CommonPage title="基础使用展示" pageBodyClassName='xxx-page'>
            <div>CommonPage 高度默认取父容器height的100%</div>
            <div>如需页面内滚动，需设置外层容器固定高度，如：400px或者100vh等</div>
            <div style={{ color: 'red', fontWeight: 600 }}>
              除API中展示的props外，均透传至PageHeader
            </div>
            <div>占位123</div>
            <div>占位123</div>
            <div>占位123</div>
            <div>占位123</div>
            <div>占位123</div>
            <div>占位123</div>
            <div>占位123</div>
            <div>占位123</div>
            <div>占位123</div>
            <div>占位123</div>
            <div>占位123</div>
            <div>占位123</div>
          </CommonPage>
        </Content>
      </Layout>
    </Layout>
  );
};
```

```tsx
/**
 * title: loading
 * background: 'rgb(240, 242, 245)'
 * compact: true
 */

import React, { useState } from 'react';
import { CommonPage } from 'udesk_gm_ui';
import { Button } from 'udesk-ui';

export default () => {
  const [loading, setLoading] = useState(false);
  return (
    <div style={{ height: 400 }}>
      <CommonPage loading={loading} title="Loading效果展示" pageBodyClassName='loading-page'>
        <Button onClick={() => {
          setLoading(true);
          setTimeout(() => {
            setLoading(false);
          }, 2000);
        }}>mock fetch</Button>
      </CommonPage>
    </div>
  );
};
```

```tsx
/**
 * title: 无padding模式&透明背景
 * desc: padding字段传入数字即可自定义内容区域的padding，ghostPage字段为true则使用透明背景色
 * background: 'rgb(240, 242, 245)'
 * compact: true
 */

import React from 'react';
import { CommonPage } from 'udesk_gm_ui';
import { Card } from 'udesk-ui';

export default () => {
  return (
    <div style={{ height: 400 }}>
      <CommonPage padding={0} ghostPage={true} title="无padding模式&透明背景" pageBodyClassName='ghost-page'>
        <Card title="Card 1" bordered={false} size="small">
          <div>CommonPage 高度默认取父容器height的100%</div>
          <div>与 ghostPage 配合使用，可用于各种自定义布局，如：报表等</div>
        </Card>
        <Card title="Card 2" bordered={false} size="small" style={{ marginTop: 16 }}>
          <div>占位123</div>
          <div>占位123</div>
          <div>占位123</div>
        </Card>
      </CommonPage>
    </div>
  );
};
```

```tsx
/**
 * title: 固定底栏
 * desc: 多用于详情、表单页面
 * background: 'rgb(240, 242, 245)'
 * compact: true
 */

import React from 'react';
import { CommonPage } from 'udesk_gm_ui';
import { Button } from 'udesk-ui';

export default () => {
  return (
    <div style={{ height: 400 }}>
      <CommonPage
        title="固定底栏"
        pageBodyClassName='footer-page'
        footBar={
          <div>
            <Button type="primary">保存</Button>
          </div>
        }
      >
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
        <div>占位123</div>
      </CommonPage>
    </div>
  );
};
```

```tsx
/**
 * title: 路由导航
 * background: 'rgb(240, 242, 245)'
 * compact: true
 */

import React, { useState } from 'react';
import { CommonPage } from 'udesk_gm_ui';
import { Button } from 'udesk-ui';

export default () => {
  const [hiddenWechat, setHiddenWechat] = useState(false);

  //模拟history对象，umi项目从umi中导出，CRA项目从props.history.location获取，当路由跳转时会自动改变，不需要设值
  const [mockHistory, setMockHistory] = useState({
    location: 'aaaa/bbb/cc/wechat',
  });

  const navigationTabs = [
    {
      pathName: 'clientManage',
      path: '/ccpass',
      text: '呼叫中心',
    },
    {
      pathName: 'clientManageWechat',
      path: '/wechat',
      text: '企业微信',
      hidden: hiddenWechat,
    },
  ];

  const onNavigationTabsChange = (tab) => {
    console.log(tab);
    mockTransitionTo(tab);

    /*
     * mockTransitionTo 模拟路由跳转方法
     * CRA项目一般是Udesk.ui.routing.transitionTo，形参tab里有pathName字段，直接使用即可
     * umi项目一般是history.push，形参tab里有path字段，直接使用即可
     */
    function mockTransitionTo(tab) {
      setMockHistory({
        location: `aaaa/bbb/cc${tab.path}`,
      });
    }
  };

  return (
    <div>
      <Button style={{ marginBottom: 8 }} onClick={() => setHiddenWechat((prev) => !prev)}>
        隐藏企业微信导航标签
      </Button>
      <div style={{ height: 400 }}>
        <CommonPage
          pageBodyClassName='router-page'
          title="路由导航"
          navigationTabs={navigationTabs}
          currentNavigationPath={mockHistory.location}
          onNavigationTabsChange={onNavigationTabsChange}
          footer={<div>footer字段传入的Jsx会渲染在navigationTabs上方</div>}
        >
          <div>{`当前位于页面：${mockHistory.location}`}</div>
        </CommonPage>
      </div>
    </div>
  );
};
```

```tsx
/**
 * title: 列表页数据缓存
 * background: 'rgb(240, 242, 245)'
 * compact: true
 */

import React, { useEffect, useState, useRef } from 'react';
import { CommonPage } from 'udesk_gm_ui';
import { Button, Table } from 'udesk-ui';

const data = [];
for (let i = 0; i < 46; i++) {
  data.push({
    key: i,
    name: `Edward King ${i}`,
    age: 32,
  });
}

const ListPage = (props) => {
  const { pageInfo, storePageInfo } = CommonPage.usePageStateMemo();
  const [pagination, setPagination] = useState({
    current: 1,
    pageSize: 10,
    ...pageInfo,
  });

  useEffect(() => {
    console.log('pageInfo 数据更新::');
    console.log(pageInfo, storePageInfo);
  }, [pageInfo, storePageInfo]);

  const columns = [
    {
      title: 'Name',
      dataIndex: 'name',
    },
    {
      title: 'Age',
      dataIndex: 'age',
    },
    {
      title: 'Actions',
      dataIndex: 'actions',
      render: () => (
        <Button size="small" type="link" onClick={props.toDetail}>
          Detail
        </Button>
      ),
    },
  ];
  const onChange = (pagination) => {
    console.log(pagination);
    storePageInfo(pagination);
    setPagination(pagination);
  };
  return (
    <CommonPage title="list page" pageBodyClassName='memo-page'>
      <Table
        size="small"
        columns={columns}
        dataSource={data}
        onChange={onChange}
        pagination={pagination}
      />
    </CommonPage>
  );
};

const DetailPage = (props) => {
  return (
    <CommonPage title="detail page" onBack={props.toList}>
      <div>detail page</div>
    </CommonPage>
  );
};

const XXXPages = () => {
  const [currentPage, setCurrentPage] = useState<'list' | 'detail'>('list');

  const pageInfoMemoRef = useRef(null);

  // 页面卸载时默认会清除缓存的数据，可以通过PageInfoMemo组件实例方法clearPageInfo手动清除缓存的列表数据
  useEffect(() => {
    return () => {
      // pageInfoMemoRef.current.clearPageInfo();
    };
  }, []);

  return (
    <div>
      <Button onClick={() => pageInfoMemoRef.current.clearPageInfo()}>手动清除缓存数据</Button>
      <div style={{ height: 400 }}>
        <CommonPage.PageInfoMemo ref={pageInfoMemoRef}>
          {/* 使用三元表达式模拟路由跳转时组件的挂载与卸载 */}
          {currentPage === 'list' ? (
            <ListPage toDetail={() => setCurrentPage('detail')} />
          ) : (
            <DetailPage toList={() => setCurrentPage('list')} />
          )}
        </CommonPage.PageInfoMemo>
      </div>
    </div>
  );
};

export default () => {
  const [isMounted, setIsMounted] = useState(true);

  const destroy = () => setIsMounted(false);
  const mount = () => setIsMounted(true);

  return (
    <div>
      <Button onClick={() => (isMounted ? destroy() : mount())}>模拟页面卸载/挂载</Button>
      {isMounted ? <XXXPages /> : <div style={{ padding: 16, color: '#000' }}>Other page</div>}
    </div>
  );
};
```

## API

| 参数 | 说明 | 类型 | 默认值 | 版本 |
| --- | --- | --- | --- | --- |
| pageBodyClassName | content 区域的class名，必传参数 | string |  |  |
| loading | 控制 content 区域的loading | boolean | false |  |
| ghostPage | content 区域背景颜色是否为透明色 | boolean | false |  |
| padding | content 区域的 padding 值，默认为 16px | number | 16 |  |
| footBar | 渲染固定页脚 | ReactNode |  |  |
| navigationTabs | 用于渲染 PageHeader 下方的路由导航用的 Tabs | NavigationTabItem[]（见 Interface 部分） |  |  |
| currentNavigationPath | 当前所在路由的 url 路径，一般从`history`对象获取，详细获取方法见`路由导航Demo` | string |  |  |
| onNavigationTabsChange | Tabs 变化时的回调函数 | (tab: NavigationTabItem) => void（见 Interface 部分） |  |  |

其余 api 与`Udesk-ui`的`PageHeader`组件保持一致

## 静态方法

| 参数 | 说明 | 类型 | 默认值 | 版本 |
| --- | --- | --- | --- | --- |
| PageInfoMemo | 储存缓存的数据的组件，使用此组件包裹后即可使用 CommonPage 的数据缓存能力 | ReactNode | - |  |
| usePageStateMemo | 自定义 hook，用于储存、读取缓存的数据 | () => ({pageInfo, storePageInfo, clearPageInfo}) | - |  |

## PageInfoMemo 的实例方法

| 参数          | 说明               | 类型                                        | 默认值 | 版本 |
| ------------- | ------------------ | ------------------------------------------- | ------ | ---- |
| pageInfo      | 缓存的数据         | {[key: string]: any}                        | -      |      |
| storePageInfo | 更新缓存数据的方法 | (newPageInfo: {[key: string]: any}) => void | -      |      |
| clearPageInfo | 清除缓存数据的方法 | () => void                                  | -      |      |

## Interface

```ts
interface NavigationTabItem {
  path: string;
  text: string;
  hidden?: boolean;
  pathName?: string;
}
```
