# PageInfoMemo设计文档

## 目标

    解决列表页跳转至详情后跳回情况下，筛选条件持久化储存的问题

## 需求描述

    1. 全局Provider缓存数据
    2. 通过自定义hooks、props透传等形式 实现数据分发、存储
    3. 提供清除缓存的方式或方法
    4. 与当前url结合降低使用成本

## 功能边界

    此组件只负责数据持久化存储，不负责滚动位置等DOM信息恢复

## 业内解决方案

### 1. 模仿vue `<keep-alive>`标签
    优势：
    1. 使用简单，自动状态保存
    缺点：
    1. 某些实现会与路由耦合，需要改动`react-router`机制
    2. 某些实现比较hack，如果有bug难以处理
#### 库：
#### 1. react-activation

    缺点：
    1. 破坏 Context 栈结构，需要手动修复
    2. 耦合react-router
    3. React 合成事件冒泡失效

#### 2. react-keep-alive

    使用`React.createPortal API`实现
    缺点：
    1. React 合成事件冒泡失效

#### 3. react-fiber-keep-alive

    借助fiber实现，过于hack

#### 4. dva

    学习成本高，过多的无关代码引入，umi项目可以参考使用

### 2. 原生Context
    优势：
    1. 原生方式，比较稳定
    缺点：
    1. 需要编写大量重复逻辑

### 3. 第三方横向存储
    优势：
    1. 稳定
    缺点：
    1. 需引入第三方库

### 4. 浏览器存储


## 最终选择方案

    基于原生Context封装实现

## 组件设计思路

    1. 全局唯一Provider存储数据 -- PageInfoMemo
    2. 通过自定义hooks提供存储数、清除数据方法以及数据缓存 -- usePageStateMemo

        为保证向后兼容能力，存储时机由调用方主动选取，不封装集成自动化存储能力及时机
    3. usePageStateMemo通过proxy过滤当前路由下的数据以及条件清除缓存的方法，以简化使用


