import React, { useCallback, useEffect, useState } from 'react';
import { Input } from 'udesk-ui';
import FieldErrAndExplain from '../field-err-explain';
import isFunction from 'lodash-es/isFunction';
import CopyComponent from '../../copy';

const { TextArea } = Input;

const Area_text: React.FC<ITextareaField> = (props) => {
    const { isFormItem = true, switchFlag } = props;
    const [errMsg, setErrMsg] = useState('');
    const [value, setValue] = useState<string>('');
    useEffect(() => {
        setValue(props.value as string);
    }, [props.value]);
    // 失去焦点 校验是否提示
    const hanleValidateText = useCallback(
        (e: any) => {
            // 如果传递了自定义的校验规则
            const value = e.target.value;
            // 非form表单形式, 自己提示错误, 自己触发保存
            if (!isFormItem) {
                // 有自定义校验
                if (isFunction(props.customValidFn)) {
                    props
                        .customValidFn(null, value)
                        .then(
                            () => {
                                // 校验成功,可能触发失焦自动保存,要把方法跑出去
                                props.onValueChangedByBlur && props.onValueChangedByBlur({});
                            },
                            (err: any) => {
                                // 校验失败,提示错误信息
                                setErrMsg(err as string);
                            }
                        )
                        .catch((err: any) => {
                            // 校验失败,提示错误信息
                            setErrMsg(err as string);
                        });
                }
            } else {
                let changeFunction = props[props.trigger];
                if (changeFunction) {
                    changeFunction(value);
                }
            }
        },
        [isFormItem, props]
    );
    return (
        <FieldErrAndExplain {...props} errMsg={errMsg}>
            <CopyComponent text={value} switchFlag={switchFlag}>
                <TextArea
                    rows={props.rowsLine}
                    defaultValue={props.defaultValue}
                    placeholder={props.placeholder}
                    onBlur={hanleValidateText}
                    onChange={(e) => setValue(e.target.value)}
                    value={value}
                    disabled={props.disabled}
                />
            </CopyComponent>
        </FieldErrAndExplain>
    );
};

export default Area_text;
