import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { Select } from 'udesk-ui';
import CopyComponent from '../../../copy';

const { Option } = Select;
/**
 * 级联组件
 * @param props
 * @param props.value    选中的值
 * @param props.options  option数据
 */
const OldCascader: React.FC<any> = ({ options: _options, ...props }) => {
    const [optionsAry, setOptionsAry] = useState<any>([]);
    const [valueList, setValueList] = useState<any>([]);
    const options = useMemo(() => {
        return addBlank2Options(_options);
    }, [_options]);
    useEffect(() => {
        let _values: any = [];
        if (!Array.isArray(props.value)) {
            _values = props.value ? props.value.split(',') : [];
        } else {
            _values = props.value;
        }
        setValueList(_values);
    }, [props.value]);

    useEffect(() => {
        let _values: any = [];
        if (!Array.isArray(valueList)) {
            _values = valueList ? valueList.split(',') : [];
        } else {
            _values = valueList;
        }
        let list = _values.reduce((prev: any, current: any, index: number) => {
            let opts: any;
            if (!index) {
                opts = options;
            } else {
                opts = (prev[index - 1] || {}).children || [];
            }

            const keyOption = opts.find((option: any) => {
                return current === option.value;
            });
            keyOption &&
                prev.push({
                    value: keyOption.label,
                    options: opts,
                    children: keyOption.children || keyOption.subs || [],
                });
            return prev;
        }, []);
        if (list && list.length > 0) {
            let last = list[list.length - 1];

            if (last.children && last.children.length) {
                list.push({
                    options: last.children,
                });
            }
        } else {
            list.push({ options });
        }
        setOptionsAry(list);
    }, [options, valueList]);

    const handleChange = useCallback(
        (item: any, index: number, _value: any, option: any) => {
            let valueAry: any = [];
            if (index === 0) {
                valueAry.push(_value);
            } else {
                for (let i = 0; i < index; i++) {
                    valueAry.push(valueList[i]);
                }
                valueAry.push(_value);
            }

            setValueList(valueAry);
            props.onChange && props.onChange(valueAry);
        },
        [props, valueList]
    );

    const optionsElement = useMemo(() => {
        return optionsAry.map((item: any, index: number) => {
            return (
                <div style={{ marginBottom: '6px' }} key={index}>
                    <CopyComponent text={item.value} switchFlag={props.switchFlag}>
                        <Select
                            onChange={(value, options) => handleChange(item, index, value, options)}
                            value={item.value}
                            placeholder={'请选择'}
                            disabled={props.disabled}
                        >
                            {item.options.map((option: any) => {
                                return (
                                    <Option key={option.value} value={option.value}>
                                        {option.label}
                                    </Option>
                                );
                            })}
                        </Select>
                    </CopyComponent>
                </div>
            );
        });
    }, [handleChange, optionsAry, props.disabled, props.switchFlag]);
    return <div>{optionsElement}</div>;
};

function addBlank2Options(options: any[]) {
    options.unshift({
        label: '请选择',
        value: '',
        children: [],
    });
    options.forEach((item) => {
        if (item?.children?.length) {
            addBlank2Options(item?.children);
        }
    });
    return options;
}

export default OldCascader;
