import React, { useCallback, useEffect, useState, useMemo } from 'react';
import { DatePicker } from 'udesk-ui';
import isFunction from 'lodash-es/isFunction';
import FieldErrAndExplain from '../field-err-explain';
import moment from 'moment';
import { YYYY_MM_DD } from '../../../utils';
import CopyComponent from '../../copy';

const Date: React.FC<IDate> = (props) => {
    const { isFormItem = true, switchFlag } = props;
    const [value, setValue] = useState(props.value);
    const [errMsg, setErrMsg] = useState('');
    useEffect(() => {
        setValue(props.value || null);
    }, [props.value]);

    // 失去焦点 校验是否提示
    const hanleValidateTime = useCallback(
        (date: any, dateString: any) => {
            setValue(date);
            if (!isFormItem) {
                // 有自定义校验
                if (isFunction(props.customValidFn)) {
                    props
                        .customValidFn(null, date)
                        .then(
                            () => {
                                // 校验成功,可能触发失焦自动保存,要把方法跑出去
                                //props.onValueChangedByBlur && props.onValueChangedByBlur({});
                            },
                            (err: any) => {
                                // 校验失败,提示错误信息
                                setErrMsg(err as string);
                            }
                        )
                        .catch((err: any) => {
                            // 校验失败,提示错误信息
                            setErrMsg(err as string);
                        });
                }
            } else {
                let changeFunction = props[props.trigger];
                if (changeFunction) {
                    changeFunction(date ? moment(date).format(YYYY_MM_DD) : '');
                }
            }
        },
        [isFormItem, props]
    );
    const handleOpenChange = useCallback(() => {
        let changeFunction = props[props.trigger];
        if (!value && changeFunction) {
            changeFunction(moment().format(YYYY_MM_DD));
        }
    }, [props, value]);
    const copyText = useMemo(() => {
        if (moment(value).isValid()) {
            return moment(value).format(YYYY_MM_DD);
        } else {
            return value;
        }
    }, [value]);
    return (
        <FieldErrAndExplain {...props} errMsg={errMsg}>
            <CopyComponent text={copyText} switchFlag={switchFlag}>
                <DatePicker
                    value={value ? moment(value, YYYY_MM_DD) : null}
                    getPopupContainer={(triggerNode) => triggerNode}
                    style={{ marginBottom: '4px' }}
                    onChange={hanleValidateTime}
                    disabled={props.disabled}
                    onOpenChange={handleOpenChange}
                />
            </CopyComponent>
        </FieldErrAndExplain>
    );
};

export default Date;
