import React, { useCallback, useEffect, useState } from 'react';
import cssObj from './index.module.scss';
import { Input } from 'udesk-ui';
import { TiaoZhuan } from '../../icon/icon';
import { CloseCircleFilled } from '@ant-design/icons';
import FieldErrAndExplain from '../field-err-explain';
import isFunction from 'lodash-es/isFunction';
import CopyComponent from '../../copy';

const Link: React.FC<ILinkField> = (props: ILinkField) => {
    const { switchFlag } = props;
    const [errMsg, setErrMsg] = useState('');
    // 输入的值
    const [link, setLink] = useState<string>(props.defaultValue || '');
    //是否显示跳转按钮
    const [showIcon, setShowIcon] = useState<boolean>(false);
    useEffect(() => {
        setLink(props.value || '');
        setShowIcon(props.value ? true : false);
    }, [props.value]);

    //   输入值变化时  手动双向绑定  进行检验
    function handleOnchange(e: any) {
        const value = e.target.value;
        setLink(value);
    }

    // 点击输入框右侧图标  打开输入的链接页面
    const openWindow = useCallback(() => {
        window.open(link, '_blank');
    }, [link]);

    // 清空输入框
    const clear = useCallback(() => {
        setLink('');
        let changeFunction = props[props.trigger];
        if (changeFunction) {
            changeFunction('');
        }
    }, [props]);

    // 输入框失焦时触发  校验输入的内容
    const check = useCallback(() => {
        // 非form表单形式, 自己提示错误, 自己触发保存
        if (!props.isFormItem) {
            // 有自定义校验
            if (isFunction(props.customValidFn)) {
                props
                    .customValidFn(null, link)
                    .then(
                        () => {
                            // 校验成功,可能触发失焦自动保存,要把方法跑出去
                            props.onValueChangedByBlur && props.onValueChangedByBlur({});
                        },
                        (err: any) => {
                            // 校验失败,提示错误信息
                            setErrMsg(err as string);
                        }
                    )
                    .catch((err: any) => {
                        // 校验失败,提示错误信息
                        setErrMsg(err as string);
                    });
            }
        } else {
            let changeFunction = props[props.trigger];
            if (changeFunction) {
                changeFunction(link);
            }
        }
    }, [link, props]);

    const handleSuffix = (
        <div className={cssObj['icon-group-wrapper']}>
            <span className={cssObj['redict-icon']} onClick={openWindow}>
                <TiaoZhuan />
            </span>
            <span className={cssObj['clear-icon']} onClick={clear}>
                <CloseCircleFilled />
            </span>
        </div>
    );
    return (
        <FieldErrAndExplain {...props} errMsg={errMsg}>
            <CopyComponent text={link} switchFlag={switchFlag}>
                <Input
                    placeholder={props.placeholder}
                    value={link}
                    onChange={handleOnchange}
                    onBlur={check}
                    suffix={showIcon ? handleSuffix : ''}
                    disabled={props.disabled}
                />
            </CopyComponent>
        </FieldErrAndExplain>
    );
};

export default Link;
