import React, { useEffect, useState } from 'react';
import cssObj from './index.module.scss';
import { Input } from 'udesk-ui';
import { EnvironmentOutlined } from '@ant-design/icons';
import isFunction from 'lodash-es/isFunction';
import FieldErrAndExplain from '../field-err-explain';
import CopyComponent from '../../copy';

const { TextArea } = Input;

function Location(props: ILocation) {
    const [address, setAddress] = useState<string>(props.defaultValue || ''); //输入的地址
    const [showIcon, setShowIcon] = useState<boolean>(false); //是否显示右上角地址图标  鼠标经过输入框时显示
    const [errMsg, setErrMsg] = useState('');
    useEffect(() => {
        setAddress(props.value as string);
    }, [props.value]);

    // 点击定位图标  打开地图
    function openMap() {
        var url = 'http://apis.map.qq.com/uri/v1/search?keyword=' + (address || '') + '&referer=custom_field';
        window.open(url, '_blank');
    }

    const handleOnBlur = (e: any) => {
        const { isFormItem = true } = props;
        // 如果传递了自定义的校验规则
        const value = e.target.value;
        // 非form表单形式, 自己提示错误, 自己触发保存
        if (!isFormItem) {
            // 有自定义校验
            if (isFunction(props.customValidFn)) {
                props
                    .customValidFn(null, value)
                    .then(
                        () => {
                            // 校验成功,可能触发失焦自动保存,要把方法跑出去
                            props.onValueChangedByBlur && props.onValueChangedByBlur({});
                        },
                        (err: any) => {
                            // 校验失败,提示错误信息
                            setErrMsg(err as string);
                        }
                    )
                    .catch((err: any) => {
                        // 校验失败,提示错误信息
                        setErrMsg(err as string);
                    });
            }
        } else {
            let changeFunction = props[props.trigger];
            if (changeFunction) {
                changeFunction(value);
            }
        }
    };
    return (
        <FieldErrAndExplain {...props} errMsg={errMsg}>
            {/* // 中间 文本域   外层包了一层div  相对定位  里边坐标图标绝对定位    文本域加了右内边距  把图标空出来 */}
            <div
                className={cssObj['textarea-wrapper']}
                onMouseEnter={() => setShowIcon(true)}
                onMouseLeave={() => setShowIcon(false)}
            >
                <CopyComponent text={address} switchFlag={props.switchFlag}>
                    <TextArea
                        // 占位符
                        placeholder={props.placeholder}
                        // 行数
                        rows={props.rowsLine}
                        // 样式
                        className={cssObj['textarea-style']}
                        // 最大字数
                        // maxLength={data.input_max}
                        // 输入值
                        value={address}
                        // 输入变化回调
                        onChange={(e) => setAddress(e.target.value)}
                        onBlur={handleOnBlur}
                        disabled={props.disabled}
                    ></TextArea>
                </CopyComponent>
                {showIcon ? (
                    <div className={cssObj['address-icon']} onClick={openMap}>
                        <EnvironmentOutlined />
                    </div>
                ) : (
                    ''
                )}
            </div>
        </FieldErrAndExplain>
    );
}

export default Location;
