import React, { useCallback, useEffect, useState } from 'react';
import FieldErrAndExplain from '../field-err-explain';
import { Input } from 'udesk-ui';
import isFunction from 'lodash-es/isFunction';
import CopyComponent from '../../copy';

// 数值

const Numeric: React.FC<any> = (props) => {
    const { isFormItem = true, switchFlag } = props;
    const [value, setValue] = useState(props.value);
    const [errMsg, setErrMsg] = useState('');

    useEffect(() => {
        setValue(props.value || '');
    }, [props.value]);

    // 失去焦点 校验是否提示
    const handleBlur = useCallback(
        (e: React.FocusEvent<HTMLInputElement>) => {
            // 如果传递了自定义的校验规则
            const value = e.target.value;
            // 非form表单形式, 自己提示错误, 自己触发保存
            if (!isFormItem) {
                // 有自定义校验
                if (isFunction(props.customValidFn)) {
                    props
                        .customValidFn(null, value)
                        .then(
                            () => {
                                // 校验成功,可能触发失焦自动保存,要把方法跑出去
                                props.onValueChangedByBlur && props.onValueChangedByBlur({});
                            },
                            (err: any) => {
                                // 校验失败,提示错误信息
                                setErrMsg(err as string);
                            }
                        )
                        .catch((err: any) => {
                            // 校验失败,提示错误信息
                            setErrMsg(err as string);
                        });
                }
            } else {
                let changeFunction = props[props.trigger];
                if (changeFunction) {
                    changeFunction(value);
                }
            }
        },
        [isFormItem, props]
    );
    const changeValue = useCallback((e: any) => {
        const value = e.target.value;
        const reg = /^-?\d*(\.\d*)?$/;
        if (reg.test(value)) {
            setValue(value);
        }
    }, []);

    return (
        <FieldErrAndExplain {...props} errMsg={errMsg}>
            <CopyComponent text={value} switchFlag={switchFlag}>
                <Input
                    value={value}
                    placeholder={props.placeholder}
                    onBlur={handleBlur}
                    onChange={changeValue}
                    style={{ marginBottom: '4px' }}
                    addonBefore={
                        props.addonSetting &&
                        props.addonSetting.addonBefore.checked &&
                        props.addonSetting.addonBefore.content
                    }
                    addonAfter={
                        props.addonSetting &&
                        props.addonSetting.addonAfter.checked &&
                        props.addonSetting.addonAfter.content
                    }
                    prefix={
                        props.addonSetting && props.addonSetting.prefix.checked && props.addonSetting.prefix.content
                    }
                    suffix={
                        props.addonSetting && props.addonSetting.suffix.checked && props.addonSetting.suffix.content
                    }
                    disabled={props.disabled}
                />
            </CopyComponent>
        </FieldErrAndExplain>
    );
};

export default Numeric;
