import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { DanSuanShiXiao, DisableRadio, XuanZeDanXuan } from '../../icon/icon';
import cssObj from './index.module.scss';
import FieldErrAndExplain from '../field-err-explain';
import isFunction from 'lodash-es/isFunction';
import { isArray } from 'lodash-es';

function Filed_radio(props: IRadio) {
    const { isFormItem, trigger, customValidFn } = props;
    const { options = [], arrangement = 'continuity', disabled, canCancel = true } = props;
    const [errMsg, setErrMsg] = useState('');
    //选择的选项值
    const [selectValue, setSelectValue] = useState<string>();
    const yStyle: object = useMemo(
        () => ({
            display: 'block',
            height: '30px',
            lineHeight: '30px',
            cursor: disabled ? 'not-allowed' : 'pointer',
            verticalAlign: 'middle',
        }),
        [disabled]
    );

    const xStyle: object = useMemo(
        () => ({
            display: 'inline-block',
            marginRight: '24px',
            cursor: disabled ? 'not-allowed' : 'pointer',
            verticalAlign: 'middle',
        }),
        [disabled]
    );

    useEffect(() => {
        let value = isArray(props.value) ? props.value[0] : props.value;

        if (!value) {
            try {
                value = props.options?.filter((item) => item.default)?.[0]?.value;
            } catch (e) {
                console.log(e);
            }
        }
        setSelectValue(value);
    }, [props.value]);

    const changeFunction = useMemo(() => props[trigger], [props, trigger]);
    const changeRadio = useCallback(
        (value: string) => {
            if (disabled) return;
            // 如果不支持取消选择  并且点击的是已选择的 不做操作
            if (!canCancel && value === selectValue) {
                return;
            }
            // 如果点击的是已经选择的  取消选择
            const _value = value === selectValue ? '' : value;
            setSelectValue(_value);

            // 非form表单形式, 自己提示错误, 自己触发保存
            if (!isFormItem) {
                // 有自定义校验
                if (isFunction(customValidFn)) {
                    customValidFn(null, value)
                        .then(
                            () => {
                                // 校验成功,可能触发失焦自动保存,要把方法跑出去
                                // props.onValueChangedByBlur && props.onValueChangedByBlur({});
                            },
                            (err: any) => {
                                // 校验失败,提示错误信息
                                setErrMsg(err as string);
                            }
                        )
                        .catch((err: any) => {
                            // 校验失败,提示错误信息
                            setErrMsg(err as string);
                        });
                }
            } else {
                changeFunction?.(_value);
            }
        },
        [canCancel, changeFunction, disabled, isFormItem, customValidFn, selectValue]
    );

    return (
        <FieldErrAndExplain {...props} errMsg={errMsg}>
            <div className={cssObj['radio-select-wrapper']}>
                {options.map((item: any) => {
                    return (
                        <span
                            key={item.value}
                            style={arrangement === 'wrap' ? yStyle : xStyle}
                            onClick={() => changeRadio(item.value)}
                        >
                            <div style={{ display: 'flex', wordBreak: 'break-all' }}>
                                {disabled ? (
                                    <DisableRadio />
                                ) : item.value === selectValue ? (
                                    <XuanZeDanXuan style={{ color: '#00B38B', fontSize: '16px' }} />
                                ) : (
                                    <DanSuanShiXiao style={{ color: 'rgba(0, 0, 0, 0.15)', fontSize: '16px' }} />
                                )}
                                <span
                                    className={`${cssObj['select-item']} ${
                                        item.color ? cssObj['select-item-hascolor'] : ''
                                    }`}
                                    style={{
                                        background: item.color ? item.color : '',
                                        marginLeft: '8px',
                                    }}
                                >
                                    {item.icon ? <img alt="" src={item.icon} className={cssObj['setting-img']} /> : ''}
                                    <span style={{ color: disabled ? 'rgba(0, 0, 0, 0.25)' : '' }}>{item.title}</span>
                                </span>
                            </div>
                        </span>
                    );
                })}
            </div>
        </FieldErrAndExplain>
    );
}

export default Filed_radio;
