import React, { useCallback, useEffect, useState } from 'react';
import { Select } from 'udesk-ui';
import cssObj from './index.module.scss';
import CopyComponent from '../../copy';

const { Option } = Select;

function Filed_droplistRadio(props: any) {
    const { switchFlag = false } = props;
    const { onChange, options = [] } = props;
    //选择的选项值
    const [selectValue, setSelectValue] = useState<string>('');
    const [copyText, setCopyText] = useState<string>('');
    useEffect(() => {
        let value = Array.isArray(selectValue) ? selectValue.join(',') : selectValue;
        options.forEach((item: any) => {
            if (item.value === value) {
                setCopyText(item.title);
            }
        });
    }, [options, selectValue]);
    // 监听道选项改变 处理数据   取出其中默认选项  复制给Radio.Group的value  主要是外部新建字段  改变默认选项时触发  改变展示效果
    useEffect(() => {
        // 如果有值传进来 优先使用传进来的值  否则从选项中找道默认选项
        if (props.value) {
            setSelectValue(props.value as string);
        } else {
            //如果禁止使用默认值，直接不进行默认值处理
            if (props.notUseDefault) return;
            options.forEach((item: any) => {
                if (item.default) setSelectValue(item.value);
            });
        }
    }, [options, props.notUseDefault, props.value]);

    const selectOnChange = useCallback(
        (e: any) => {
            setSelectValue(e);
            if (onChange) onChange(e);
        },
        [onChange]
    );
    return (
        <CopyComponent text={copyText} switchFlag={switchFlag}>
            <Select
                value={selectValue}
                onChange={selectOnChange}
                style={{ width: '100%' }}
                getPopupContainer={(triggerNode) => triggerNode}
                disabled={props.disabled}
                showSearch={options.length > 8}
                filterOption={(input, option: any) => option.label.toLowerCase().includes(input.toLowerCase())}
                allowClear={true}
            >
                {options.map((item: any) => {
                    return (
                        <Option key={item.value} value={item.value} label={item.title}>
                            <span
                                className={`${cssObj['select-item']} ${
                                    item.color ? cssObj['select-item-hascolor'] : ''
                                }`}
                                style={{
                                    background: item.color ? item.color : '',
                                }}
                            >
                                {item.icon ? <img alt="" src={item.icon} className={cssObj['setting-img']} /> : ''}
                                {item.title}
                            </span>
                        </Option>
                    );
                })}
            </Select>
        </CopyComponent>
    );
}

export default Filed_droplistRadio;
