import React, { cloneElement, ReactElement, ReactNode, useCallback, useMemo, useRef } from 'react';
import AntTreeSelect, { TreeSelectProps } from "udesk-ui/es/tree-select";
import Tag from 'udesk-ui/es/tag';

const TreeNode = AntTreeSelect.TreeNode;

function renderTreeNode(selectOptions: any, parentCheckable: boolean | undefined): ReactNode {
    if (!selectOptions) {
        return null;
    }
    return selectOptions.map((i: SelectOptions) => {
        let title: ReactNode = i.title;
        if (i.color) {
            title = <Tag color={i.color}>{i.title}</Tag>
        }
        return <TreeNode value={i.value} title={title} key={i.value}
                         checkable={parentCheckable ? true : (i.node_type === 'leaf')}>
            {renderTreeNode(i.subs, parentCheckable)}
        </TreeNode>
    });
}

const tagRender: TreeSelectProps<String[]>['tagRender'] = (props) => {
    const {label, closable, onClose} = props;

    return cloneElement(label as ReactElement, {closable, onClose})
}


const TagTree: React.FC<any> = ({value, onChange, ...props}) => {
    const treeNodes = useMemo(() => renderTreeNode(props.select_options, props.parent_checkable), [props.parent_checkable, props.select_options]);
    const ref = useRef<HTMLDivElement>(null);
    const getPopupContainer = useCallback(() => ref.current || document.body, []);
    const valueAry = useMemo(() => {
        if (typeof value === 'string') {
            return value.split(',')
        } else if (value) {
            return value;
        } else {
            return [];
        }
    }, [value]);

    return (
        <div ref={ref} style={{position: 'relative'}}>
            <AntTreeSelect<string[]> showSearch
                                     style={{width: '100%'}}
                                     value={valueAry}
                                     dropdownStyle={{maxHeight: 400, overflow: 'auto'}}
                                     placeholder={props.placeholder}
                                     allowClear
                                     treeCheckable={true}
                                     onChange={onChange}
                                     getPopupContainer={getPopupContainer}
                                     transitionName=''
                                     treeDefaultExpandAll={props.default_expand_all}
                                     tagRender={tagRender}
                                     disabled={props.disabled}
            >
                {treeNodes}
            </AntTreeSelect>
        </div>
    );
}

export default TagTree;
