import React, { useCallback, useEffect, useState } from 'react';
import isFunction from 'lodash-es/isFunction';
import { Input } from 'udesk-ui';
import FieldErrAndExplain from '../field-err-explain';
import CopyComponent from '../../copy';

const Text: React.FC<any> = (props) => {
    const { isFormItem = true, switchFlag } = props;
    const [errMsg, setErrMsg] = useState('');
    const [value, setValue] = useState(props.value || '');
    useEffect(() => {
        setValue(props.value || '');
    }, [props.value]);

    // 失去焦点 校验是否提示
    const handleBlur = useCallback(
        (e: React.FocusEvent<HTMLInputElement>) => {
            // 如果传递了自定义的校验规则
            const value = e.target.value;

            // 非form表单形式, 自己提示错误, 自己触发保存
            if (!isFormItem) {
                // 有自定义校验
                if (isFunction(props.customValidFn)) {
                    props
                        .customValidFn(null, value)
                        .then(
                            () => {
                                // 校验成功,可能触发失焦自动保存,要把方法跑出去
                                props.onValueChangedByBlur && props.onValueChangedByBlur({});
                            },
                            (err: any) => {
                                // 校验失败,提示错误信息
                                setErrMsg(err as string);
                            }
                        )
                        .catch((err: any) => {
                            // 校验失败,提示错误信息
                            setErrMsg(err as string);
                        });
                }
            } else {
                let changeFunction = props[props.trigger];
                if (changeFunction) {
                    changeFunction(value);
                }
            }
        },
        [isFormItem, props]
    );

    return (
        <FieldErrAndExplain {...props} errMsg={errMsg}>
            <CopyComponent text={value} switchFlag={switchFlag}>
                <Input
                    value={value}
                    placeholder={props.placeholder}
                    onBlur={handleBlur}
                    onChange={(e) => setValue(e.target.value)}
                    disabled={props.disabled}
                />
            </CopyComponent>
        </FieldErrAndExplain>
    );
};

export default Text;
