import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { TimePicker } from 'udesk-ui';
import isFunction from 'lodash-es/isFunction';
import FieldErrAndExplain from '../field-err-explain';
import moment from 'moment';
import { TIME_FORMAT_S } from '../../../utils';
import CopyComponent from '../../copy';

function Time(props: ITime) {
    const { isFormItem = true, switchFlag } = props;
    const [value, setValue] = useState(props.value);
    const [errMsg, setErrMsg] = useState('');

    useEffect(() => {
        setValue(props.value || null);
    }, [props.value]);

    // 失去焦点 校验是否提示
    const hanleValidateTime = useCallback(
        (date: any, dateString: any) => {
            if (!isFormItem) {
                // 有自定义校验
                if (isFunction(props.customValidFn)) {
                    props
                        .customValidFn(null, date)
                        .then(
                            () => {
                                // 校验成功,可能触发失焦自动保存,要把方法跑出去
                                //props.onValueChangedByBlur && props.onValueChangedByBlur({});
                            },
                            (err: any) => {
                                // 校验失败,提示错误信息
                                setErrMsg(err as string);
                            }
                        )
                        .catch((err: any) => {
                            // 校验失败,提示错误信息
                            setErrMsg(err as string);
                        });
                }
            } else {
                let changeFunction = props[props.trigger];
                if (changeFunction) {
                    changeFunction(date ? moment(date).format(TIME_FORMAT_S) : '');
                }
            }
        },
        [isFormItem, props]
    );
    const copyChildren = useMemo(() => {
        return (
            <TimePicker
                value={value ? moment(value, TIME_FORMAT_S) : null}
                style={{ marginBottom: '4px' }}
                onChange={hanleValidateTime}
                disabled={props.disabled}
            />
        );
    }, [hanleValidateTime, props.disabled, value]);
    return (
        <FieldErrAndExplain {...props} errMsg={errMsg}>
            <CopyComponent text={value} switchFlag={switchFlag}>
                {copyChildren}
            </CopyComponent>
        </FieldErrAndExplain>
    );
}

export default Time;
