import React from 'react';
import PropTypes from 'prop-types';
import aceEditorLoaderService from '../../../services/ace-editor-loader';

class AceEditorComponent extends React.Component {
    constructor(props) {
        super(props);
        this.aceEditorRef = React.createRef();
    }
    //#region
    static defaultProps = {
        classNames: "",
        aceUrl: "",
        mode: "json", //javascript
        theme: "monokai",
        value: "",

        selectionStyle: "text", //"line"
        highlightActiveLine: true, // false
        highlightSelectedWord: true, // false
        readOnly: false,
        cursorStyle: "ace", // "slim" | "smooth" | "wide"
        // mergeUndoDeltas: false | true | "always"

        behavioursEnabled: true,
        wrapBehavioursEnabled: true,

        autoScrollEditorIntoView: true,
        // copy/cut the full line if selection is empty, defaults to false
        copyWithEmptySelection: false,
        useSoftTabs: false,
        navigateWithinSoftTabs: false,
        showGutter: true,
        hScrollBarAlwaysVisible: true,
        vScrollBarAlwaysVisible: true,
        printMarginColumn: 80,
        printMargin: true,

        fontSize: 12,

        onInitAceEnded: undefined,
        onChanged: null,
    }

    static propTypes = {
        aceUrl: PropTypes.oneOfType([
            PropTypes.string,
            PropTypes.func,
        ]),
    }
    //#endregion
    privates = {
        editorInstance: null,
        code: null,
    }
    componentDidMount() {
        let {
            aceUrl
        } = this.props;
        if (window.ace) {
            this.actions.initAceEditor();
        } else {
            if (typeof aceUrl === "function") {
                aceUrl = aceUrl.call(this);
            }
            if (aceUrl) {
                aceEditorLoaderService.setLoadUrl(aceUrl);
            }
            aceEditorLoaderService.loadAsync(() => {
                if (this.isDestroyed || this.isDestroying) {
                    return;
                }
                this.actions.initAceEditor();
            });
        }
    }

    componentDidUpdate(prevProps, prevState, snapshot) {
        let {
            value
        } = this.props;
        let code = this.privates.code;

        if (value !== code) {
            this.privates.code = value;
            let editorInstance = this.privates.editorInstance;
            if (editorInstance) {
                editorInstance.destroy();
            }
            this.actions.initAceEditor();
        }
    }

    componentWillUnmount() {
        if (this.isDestroyed) {
            return;
        }
        let editorInstance = this.privates.editorInstance;
        if (editorInstance) {
            if (editorInstance.destroy && typeof editorInstance.destroy === "function") {
                editorInstance.destroy();
            }
            if (editorInstance.container && typeof editorInstance.container.remove === "function") {
                editorInstance.container.remove();
            }
        }
        this.privates.editorInstance = null;
    }
    actions = {
        initAceEditor() {
            if (window.ace) {
                let that = this;
                let {
                    mode,
                    theme,
                    value,
                    selectionStyle,
                    highlightActiveLine,
                    highlightSelectedWord,
                    readOnly,
                    cursorStyle,
                    behavioursEnabled,
                    wrapBehavioursEnabled,
                    autoScrollEditorIntoView,
                    copyWithEmptySelection,
                    useSoftTabs,
                    navigateWithinSoftTabs,
                    showGutter,
                    hScrollBarAlwaysVisible,
                    vScrollBarAlwaysVisible,
                    fontSize,
                    printMarginColumn,
                    printMargin
                } = this.props;

                if (mode === "js") {
                    mode = "javascript";
                }

                if (!this.aceEditorRef || !this.aceEditorRef.current) {
                    return;
                }
                let node = this.aceEditorRef.current;
                let editorInstance = window.ace.edit(node, {
                    value,
                    selectionStyle,
                    highlightActiveLine,
                    highlightSelectedWord,
                    readOnly,
                    cursorStyle,
                    behavioursEnabled,
                    wrapBehavioursEnabled,
                    autoScrollEditorIntoView,
                    copyWithEmptySelection,
                    useSoftTabs,
                    navigateWithinSoftTabs,
                    fontSize,
                    showLineNumbers: true,
                    tabSize: 4,
                    newLineMode: "auto",
                    spellcheck: true,
                    enableBasicAutocompletion: false,
                    enableLiveAutocompletion: true,
                    wrap: true
                });

                editorInstance.renderer.setOptions({
                    hScrollBarAlwaysVisible,
                    vScrollBarAlwaysVisible,
                    printMarginColumn,
                    highlightGutterLine: true,
                    showInvisibles: false,
                    showGutter,

                    animatedScroll: true,
                    // showPrintMargin: boolean
                    printMargin,
                    // fadeFoldWidgets: boolean
                    // showFoldWidgets: boolean (defaults to true)
                    // showLineNumbers: boolean (defaults to true)
                    displayIndentGuides: false,
                    fontSize,
                    // fontFamily: css font-family value
                    // maxLines: 200,
                    minLines: 200,
                    // scrollPastEnd: number|boolean
                    // fixedWidthGutter: boolean (defaults to false)
                    // theme: path to a theme e.g "ace/theme/textmate"
                });

                editorInstance.setTheme(`ace/theme/${theme}`);

                editorInstance.on('change', function (e) {
                    let code = editorInstance.getValue();
                    that.privates.code = code;
                    that.trigger("onChanged", code);
                });

                editorInstance.getSession().setMode(`ace/mode/${mode}`);
                this.privates.code = value;
                this.privates.editorInstance = editorInstance;
                // this.actions.update();
                this.trigger("onInitAceEnded");
            }
        }
    }
}
export default AceEditorComponent;