import React, { useCallback, useEffect, useMemo, useState } from "react";
import DropDown, { Menu, Trigger } from "../drop-down";
import { Input, Tree, Empty } from "udesk-ui";
import apis from "../../utils/apis";
import { useSelector } from "../../store/store-hook";
import { useTranslation } from "react-i18next";
import { CareDown } from "../icon/icon";
import style from "./style.module.scss";
import { RootState } from "../../store/root-reducer";

// const { TreeNode} = Tree;
const { Search } = Input;

// let agents = [];
interface IOnCheck {
  (value: any): void;
}

interface onAgentLoad {
  (agents: any): void;
}

interface Props {
  triggerComponent?: any;
  triggerText?: string;
  placeholder?: string;
  showNum?: boolean;
  cacheEnable?: boolean;
  allColleagueMonitorEnabled?: boolean;
  value: string[];
  onChange?: IOnCheck;
  onAgentLoad?: onAgentLoad;
}

interface TreeNodeProps {
  title?: any;
  key?: any;
  count?: number;
}

const TreeNode: React.FC<TreeNodeProps> = (props: TreeNodeProps) => {
  return (
    <span key={props.key} title={props.title}>
      {props.count != null && <b className={style["nodeNum"]}>{props.count}</b>}
      {props.title}
    </span>
  );
};

const AgentGroupSelect: React.FC<Props> = (props) => {
  const { t } = useTranslation();
  let {
    triggerComponent: TriggerComponent,
    triggerText = t("agent"),
    placeholder = "",
    showNum = true,
    cacheEnable = true,
    allColleagueMonitorEnabled = true,
    onChange,
    onAgentLoad,
    value = [],
  } = props;
  let [keyWord, setKeyword] = useState<string>("");
  let [agents, setAgents] = useState<any[]>([]);
  let [dropDownVisible, setDropDownVisible] = useState<boolean>(false);
  // let [checkStrictly, setCheckStrictly] = useState<boolean>(false);
  let [checkedNode, setCheckedNode] = useState<any[]>([]);

  let currentAgents = useSelector((state: RootState) => state.agents);
  let currentUser = useSelector((state: RootState) => state.currentUser);

  const groupAgents = useMemo(() => {
    return formGroupAgents(
      agents,
      currentAgents,
      currentUser?.user_groups,
      allColleagueMonitorEnabled
    );
  }, [
    agents,
    allColleagueMonitorEnabled,
    currentAgents,
    currentUser?.user_groups,
  ]);
  const filteredgroupAgents = useMemo(() => {
    let filteredAgents = agents.filter((agent: any) => {
      let agentName = agent?.name;
      if (agentName) {
        if (keyWord) {
          return agentName.indexOf(keyWord) !== -1;
        } else {
          return true;
        }
      } else {
        return false;
      }
    });
    return formGroupAgents(
      filteredAgents,
      currentAgents,
      currentUser?.user_groups,
      allColleagueMonitorEnabled
    );
  }, [
    keyWord,
    agents,
    currentAgents,
    currentUser?.user_groups,
    allColleagueMonitorEnabled,
  ]);

  useEffect(() => {
    apis.getMonitorUserList().then((resp: any) => {
      if (resp?.agents) {
        const respAgents = resp.agents;
        setAgents(respAgents);
      }
    });
  }, []);
  useEffect(() => {
    setCheckedNode(value);
  }, [value]);

  const onSearchChange = useCallback((e) => {
    let value = e.target.value;
    // if (value) {
    //     setCheckStrictly(true);
    // } else {
    //     setCheckStrictly(false);
    // }
    setKeyword(value);
  }, []);
  let onVisibleChange = useCallback(
    (value) => {
      setDropDownVisible(value);
      if (!value) {
        onChange && onChange(checkedNode);
        setKeyword("");
      }
    },
    [checkedNode, onChange]
  );
  let onChecked = useCallback(
    (value) => {
      value = updateParentValue(groupAgents, value);
      setCheckedNode(value);
    },
    [groupAgents]
  );

  return (
    <DropDown
      className={style["agent-group-select"]}
      visible={dropDownVisible}
      onVisibleChange={onVisibleChange}
    >
      <Trigger>
        {!!TriggerComponent && <TriggerComponent />}
        {!TriggerComponent && (
          <span className={style["trigger-span"]}>
            {triggerText}
            {showNum && agents?.length}
          </span>
        )}
        <CareDown></CareDown>
      </Trigger>
      <Menu className={style["menu"]}>
        <Menu.Item>
          <Search
            value={keyWord}
            className={style["search"]}
            placeholder={placeholder}
            onChange={onSearchChange}
          />
        </Menu.Item>
        <Menu.Item>
          <div className={style["content"]}>
            {filteredgroupAgents.length && (
              <Tree
                checkable
                onCheck={onChecked}
                defaultExpandAll
                treeData={filteredgroupAgents}
                checkedKeys={checkedNode}
                // checkStrictly={checkStrictly}
              ></Tree>
            )}
            {!filteredgroupAgents.length && (
              <Empty image={Empty.PRESENTED_IMAGE_SIMPLE} />
            )}
          </div>
        </Menu.Item>
      </Menu>
    </DropDown>
  );
};
export default AgentGroupSelect;

function formGroupAgents(
  agents: any,
  currentAgents: any,
  currentUserGroups: any,
  allColleagueMonitorEnabled: boolean
) {
  agents = agents || [];
  return agents.reduce((preValue: any, agent: any) => {
    let currentAgent = currentAgents.find(
      (cAgent: any) => agent.id == cAgent.id
    );
    if (!currentAgent) {
      return preValue;
    } else {
      let currentAgentUserGroups = currentAgent.user_groups;
      if (!allColleagueMonitorEnabled) {
        currentAgentUserGroups = currentUserGroups.filter((user: any) => {
          return currentAgentUserGroups.some((group: any) => {
            return group.id === user.id;
          });
        });
      }
      return addGroupAgent(preValue, currentAgentUserGroups, agent);
    }
  }, []);
}

function addGroupAgent(groupAgents: any, groups: any, agent: any) {
  if (groups) {
    groups.forEach((group: any) => {
      let groupAgent = groupAgents.find((ga: any) => ga?.key == group?.id);
      let newAgent = {
        key: `${group?.id}-${agent?.id}`,
        title: (
          <TreeNode
            count={agent?.customers_count}
            key={`${group?.id}-${agent?.id}`}
            title={agent?.name}
          />
        ),
        customersCount: agent.customers_count,
      };
      if (groupAgent) {
        if (groupAgent.children) {
          groupAgent.children.push(newAgent);
        } else {
          groupAgent.children = [newAgent];
        }
        //计算总客户数
        let totalCustomerNum = groupAgent.children?.reduce(
          (total: number, agent: any) => {
            return (total += agent.customersCount || 0);
          },
          0
        );
        groupAgent.title = (
          <TreeNode
            count={totalCustomerNum}
            key={groupAgent?.key}
            title={groupAgent?.name}
          />
        );
      } else {
        groupAgents.push({
          key: group?.id,
          title: (
            <TreeNode
              count={agent?.customers_count}
              key={group?.id}
              title={group?.name}
            />
          ),
          name: group?.name,
          children: [newAgent],
        });
      }
    });
  }
  return groupAgents;
}

//如果子节点没有都选中,父节点不选中
function updateParentValue(treeData: any, value: string[]) {
  let result: string[] = [];
  result = value.filter((_v) => {
    if (/^\d+$/.test(_v)) {
      if (treeData) {
        let node = treeData.find((data: any) => {
          return data.key === _v;
        });
        if (node && node.children) {
          return node.children.every((children: any) => {
            return value.indexOf(children.key) !== -1;
          });
        }
      } else {
        return true;
      }
    } else {
      return true;
    }
  });
  return result;
}
