import React, { Component } from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import Udesk from '../../udesk/index';

class BootstrapDurationPicker extends Component {
    constructor(props) {
        super(props);
        this.dateDurationPickerElement = null;
        this.isInitializing = false;
        this.isInnerValueChange = false;
        this.oldValue = null;
    }

    componentDidMount() {
        let {
            locale,
            showDays,
            showSeconds
        } = this.props;
        let options = {};
        if (Udesk.ui.widgets.bootstrapDurationPicker && Udesk.ui.widgets.bootstrapDurationPicker.defaults) {
            options = Udesk.utils.object.clone(Udesk.ui.widgets.bootstrapDurationPicker.defaults);
        }
        if (locale != null) {
            Object.assign(options.translations, locale);
        }

        if (typeof showDays === "boolean") {
            options.showDays = showDays;
        }

        if (typeof showSeconds === "boolean") {
            options.showSeconds = showSeconds;
        }
        let that = this;
        options.onChanged = (value, isInitializing) => {
            if(!that.isInnerValueChange) {
                that.oldValue = value;
                that.isInitializing = isInitializing;
                that.props.onChanged(value, isInitializing);
            } else {
                that.isInnerValueChange = false;
            }
        };
        if (this.dateDurationPickerElement != null) { 
            this.oldValue = this.props.value;
            $(this.dateDurationPickerElement).durationPicker(options);
        }
    }

    componentDidUpdate(prevProps, prevState, snapshot) {
        let {
            value
        } = this.props;
        if (value !== this.oldValue && !this.isInitializing) {
            this._updateDurationPicker(value);
            this.isInnerValueChange = true;
        }
    }

    componentWillUnmount() {
        if (this.dateDurationPickerElement != null) {
            $(this.dateDurationPickerElement).data('durationPicker').destroy();
        }
    }

    _updateDurationPicker(value) { 
        if (this.dateDurationPickerElement != null) {
            if (value == null) {
                value = 0;
            }
            $(this.dateDurationPickerElement).data('durationPicker').setValue(value);
        }
    }

    render() {
        let {
            id,
            classNames,
            value,
            disabled
        } = this.props;

        return (
            <input type="text" id={id} className={ClassNames({ "udesk-react-bootstrap-duration-pick": true, [`${classNames}`]: classNames })} value={value} ref={(dateDurationPickerElement) => this.dateDurationPickerElement = dateDurationPickerElement} disabled={disabled} onChange={() => { }} />
        );
    }
}

BootstrapDurationPicker.propTypes = {
    id: PropTypes.string,
    onChanged: PropTypes.func
};

BootstrapDurationPicker.defaultProps = {
    id: "",
    value: "",
    locale: null,
    classNames: "",
    showDays: null,
    showSeconds: null,
    disabled: false,

    onChanged: () => { }
};

export default Udesk.react.udeskify(BootstrapDurationPicker);