import React, { useCallback, useMemo, useState } from 'react';
import { Button, message } from 'udesk-ui';
import style from './index.module.scss';
import { useTranslation } from 'react-i18next';
import CopyToClipboard from 'react-copy-to-clipboard';

interface CopyProps {
    text: string;
    left?: string; //可选,用于调整位置
    right?: string;
    top?: string;
    bottom?: string;
    switchFlag?: boolean;
    hidden?: boolean;
}

const CopyComponent: React.FC<CopyProps> = (props) => {
    const { t } = useTranslation();
    // 默认显示在右上角，不能默认显示在容器外面，因为外层容器上设置了overflow:hidden，这样复制按钮就显示不出来了。
    // 而设置overflow:hidden的原因是为了内容过长时，显示省略号...
    const { text, left = '0px', top = '-30px', bottom = '', switchFlag = false } = props;
    const buttonStyle: React.CSSProperties = {
        position: 'absolute',
        left,
        top,
        bottom,
        zIndex: 999,
        visibility: 'hidden',
    };
    const [copyStyle, setCopyStyle] = useState(buttonStyle);

    const showButton = useCallback(
        (e: any) => {
            setCopyStyle({
                ...copyStyle,
                visibility: text === '' || text === null || text === undefined ? 'hidden' : 'visible',
            });
        },
        [copyStyle, text]
    );

    const hideButton = useCallback(
        (e: any) => {
            setCopyStyle({ ...copyStyle, visibility: 'hidden' });
        },
        [copyStyle]
    );

    const copyText = useCallback(
        (text: string, result: boolean) => {
            if (result) {
                message.info(t('tip.copySuccess'));
            } else {
                message.error(t('tip.copySuccess'));
            }
        },
        [t]
    );

    const buttonIcon = useMemo(() => {
        return (
            <Button size={'small'} style={copyStyle}>
                {t('label.copy')}
            </Button>
        );
    }, [copyStyle, t]);
    return (
        <>
            {switchFlag && (
                <div
                    onMouseEnter={showButton}
                    onMouseLeave={hideButton}
                    className={`${style['copy-container']} ${props.hidden ? style['hidden'] : ''}`}
                >
                    <CopyToClipboard text={text} onCopy={copyText}>
                        {buttonIcon}
                    </CopyToClipboard>
                    <div className="copy-children">{props.children}</div>
                </div>
            )}
            {!switchFlag && (
                <div className={`${style['copy-container']} ${props.hidden ? style['hidden'] : ''}`}>
                    {props.children}
                </div>
            )}
        </>
    );
};
export default CopyComponent;
