import React from "react";
import fieldStyle from "./style.module.scss";
import { QuestionCircleOutlined } from "@ant-design/icons/lib/icons";
import { Tooltip, Form } from "udesk-ui";
import GetFieldComponet from "./field-type-component";
import isFunction from "lodash-es/isFunction";

type CustomFieldItemProps = {
  fieldItem: IFieldItem;
  isFormItem: boolean;
  switchFlag?: boolean;
  notUseDefault?: boolean;
  agents?: IAgent[];
};

const CustomFieldItem: React.FC<CustomFieldItemProps> = ({
  fieldItem,
  isFormItem,
  switchFlag = false,
  agents,
  ...props
}) => {
  const fromLabel = () => {
    return (
      <div className={fieldStyle["item-label"]}>
        {fieldItem.isRequired && <span style={{ color: "red" }}>*</span>}
        <span style={{ marginRight: "5px" }}>{fieldItem.label}:</span>
        {fieldItem.layout !== "horizontal" &&
          fieldItem.tooltipShow &&
          fieldItem.tooltip && (
            <Tooltip title={fieldItem.tooltip}>
              <QuestionCircleOutlined />
            </Tooltip>
          )}
      </div>
    );
  };

  return (
    <React.Fragment>
      {isFormItem ? (
        <Form.Item
          {...props}
          label={fromLabel()}
          name={fieldItem.field.id}
          trigger={fieldItem.field.trigger || "onChange"}
          rules={
            isFunction(fieldItem.field.customValidFn)
              ? [
                  {
                    validator: fieldItem.field.customValidFn,
                  },
                ]
              : undefined
          }
        >
          <GetFieldComponet
            switchFlag={switchFlag}
            {...fieldItem.field}
            isFormItem={isFormItem}
            notUseDefault={props.notUseDefault}
            agents={agents}
          />
        </Form.Item>
      ) : (
        <div
          className={`${fieldStyle["field-item"]} ${
            fieldItem.layout === "horizontal"
              ? fieldStyle["field-item-horizontal"]
              : ""
          }`}
        >
          {/* 头部 标题 + 提示问号 */}
          {fromLabel()}
          <div className={fieldStyle["item-content"]}>
            <GetFieldComponet
              switchFlag={switchFlag}
              {...fieldItem.field}
              isFormItem={false}
              agents={agents}
            />
          </div>
        </div>
      )}
    </React.Fragment>
  );
};

export default CustomFieldItem;
