import React from 'react';
import InputBaseClass from './input-base-class';
import Udesk from '../../udesk';
import Locales from "../../udesk/locales";

const default_Precision_Field = "precision";

class FloatNumber extends InputBaseClass {
    actions = {
        _getInternalValueFromValue(value) {
            let internalValue = this.actions._getBaseClassGlobalInternalValueFromValue(value);
            if (internalValue == null) {
                return "";
            }
            return internalValue;
        },
        _validate() {
            let {
                privates: { computes },
            } = this;

            let {
                internalValue,
                mergedAttributes
            } = computes;

            if (internalValue != null) {
                if (!Udesk.config.regex.floatNumberNullable.test(internalValue)) {
                    return [Locales.get("components.customFieldInputs.floatNumber.invalidInputError", this.props.language)];
                }
                let precisionField = mergedAttributes["precisionField"];
                if (precisionField == null) {
                    precisionField = default_Precision_Field;
                }
                let precision = mergedAttributes[precisionField];
                if (precision) {
                    let separateValue = internalValue.split(".");
                    if (separateValue.length === 2) {
                        if (separateValue[1].length > precision) {
                            return [Udesk.utils.string.format(Locales.get("components.customFieldInputs.floatNumber.invalidPrecisionError", this.props.language), precision)];
                        }
                    }
                    return true;
                }
            }
            return true;
        },

        _onInputValueChanged(e) {
            let internalValue = e.target.value;
            let {
                privates: { computes },
            } = this;

            let {
                mergedAttributes
            } = computes;

            const testReg = mergedAttributes.valueFormatterReg || /^\d+(\.\d{0,2})?$/;
            if (testReg.test(internalValue) || internalValue === "") {
                this.actions._onChanged(internalValue);
            }
        }
    }

    static get emberName() {
        return 'udesk/custom-field-inputs/float-number';
    }

    render() {
        let {
            privates: { computes },
        } = this;

        let {
            internalValue,
            mergedAttributes
        } = computes;
        let {
            disabled
        } = this.props;

        return (
            <input type="text" className={mergedAttributes.classNames} value={internalValue} disabled={disabled} autoFocus={mergedAttributes.autofocus} placeholder={mergedAttributes.placeholder} onInput={this.actions._onInputValueChanged} onChange={() => { }} />
        );
    }
}

export default Udesk.react.udeskify(FloatNumber);