import React from 'react';
import InputBaseClass from './input-base-class';
import Udesk from '../../udesk/index';
import Locales from "../../udesk/locales/index";

class SimplePhone extends InputBaseClass {
    actions = {
        _getInternalValueFromValue(value) {
            let internalValue = this.actions._getBaseClassGlobalInternalValueFromValue(value);
            if (internalValue == null) {
                return "";
            }
            return internalValue;
        },
        _validate() {
            let {
                privates: { computes },
            } = this;

            let {
                internalValue,
                mergedAttributes
            } = computes;

            if (internalValue != null) {
                if (mergedAttributes.maxLength !=null) {
                    if (internalValue.length > mergedAttributes.maxLength) {
                        return [Udesk.utils.string.format(Locales.get("components.customFieldInputs.simplePhone.integerMaxLengthError", this.props.langageCode), mergedAttributes.precision)];
                    }
                }
                if (!Udesk.config.regex.mobileOrTelephone.test(internalValue)) {
                    return [Locales.get("components.customFieldInputs.simplePhone.invalidInputError", this.props.language)];
                }
            }
            return true;
        },
        _onInputValueChanged(e) {
            let internalValue = e.target.value;
            if (/^[1-9]\d*$/.test(internalValue) || internalValue === "") {
                this.actions._onChanged(internalValue);
            }
        }
    }

    static get emberName() {
        return 'udesk/custom-field-inputs/simple-phone';
    }
    
    render() {
        let {
            privates: { computes },
        } = this;

        let {
            internalValue,
            mergedAttributes
        } = computes;
        let {
            disabled
        } = this.props;

        return (
            <input type="text" className={mergedAttributes.classNames} value={internalValue} disabled={disabled} placeholder={mergedAttributes.placeholder} autoFocus={mergedAttributes.autofocus} onInput={this.actions._onInputValueChanged} onChange={() => {}} />
        );
    }
}

export default Udesk.react.udeskify(SimplePhone);