import React from 'react';
import InputBaseClass from './input-base-class';
import Udesk from '../../udesk/index';
import Locales from "../../udesk/locales/index";
import ClassNames from 'classnames';

const default_Precision_Field = "precision";

class TextPercent extends InputBaseClass {
    actions = {
        _getInternalValueFromValue(value) {
            let internalValue = this.actions._getBaseClassGlobalInternalValueFromValue(value);
            if (internalValue == null) {
                return "";
            }
            return internalValue;
        },
        _validate() {
            let {
                privates: { computes },
            } = this;

            let {
                internalValue,
                mergedAttributes
            } = computes;

            if (internalValue != null) {
                if (!Udesk.config.regex.floatNumberNullable.test(internalValue)) {
                    return [Locales.get("components.customFieldInputs.textPrecent.invalidInputError")];
                }
                let precisionField = mergedAttributes["precisionField"];
                if (precisionField == null) {
                    precisionField = default_Precision_Field;
                }
                let precision = mergedAttributes[precisionField];
                if (precision) {
                    let separateValue = internalValue.split(".");
                    if (separateValue.length === 2) {
                        if (separateValue[1].length > precision) {
                            return [Udesk.utils.string.format(Locales.get("components.customFieldInputs.textPrecent.invalidPrecisionError", this.props.language), precision)];
                        }
                    }
                    return true;
                }
            }
            return true;
        },

        _onInputValueChanged(e) {
            let internalValue = e.target.value;
            if (/^\d+(\.\d*)?$/.test(internalValue) || internalValue === "") {
                this.actions._onChanged(internalValue);
            }
        }
    }

    static get emberName() {
        return 'udesk/custom-field-inputs/text-percent';
    }
    
    render() {
        let {
            privates: { computes },
        } = this;

        let {
            internalValue,
            mergedAttributes
        } = computes;
        let {
            disabled
        } = this.props;

        return (
            <div className="udesk-react-custom-field-compoennts-text-percent">
                <input type="text" className={ClassNames("text-percent-input", { [`${mergedAttributes.classNames}`]: mergedAttributes.classNames })} value={internalValue} disabled={disabled} autoFocus={mergedAttributes.autofocus} placeholder={mergedAttributes.placeholder} onInput={this.actions._onInputValueChanged} onChange={() => {}} />
            </div>
        );
    }
}

export default Udesk.react.udeskify(TextPercent);