import Udesk from '../../../../udesk';
import Locales from "../../../../udesk/locales";
import validators from './validators';
import converters from './converters';
import CFITextBox from '../../../custom-filter-inputs/text-box';
import CFIDropdown from '../../../custom-filter-inputs/drop-down';
import CFIReactSelect from '../../../custom-filter-inputs/react-select';
import CFIReactMultiSelect from '../../../custom-filter-inputs/react-multi-select';
import CFIAutoComplete from '../../../custom-filter-inputs/auto-complete';
import CFIDatePicker from '../../../custom-filter-inputs/date-picker';
import CFIDateRangePicker from '../../../custom-filter-inputs/date-range-picker';
import CFIIntegerNumber from '../../../custom-filter-inputs/integer-number';
import CFIFloatNumber from '../../../custom-filter-inputs/float-number';
import CFISimplePhone from '../../../custom-filter-inputs/simple-phone';
// import CFI from '../../../custom-filter-inputs/';
// import CFI from '../../../custom-filter-inputs';

function getComponentsConfigTable() {
    return [{
        inputType: validators.validateInputTypes,
        operators: validators.validateOperators,
        component: null,//validators.validateFunctionNullable,
        attributes: validators.validateObject
    }, {
        inputType: converters.convertInputType,
        operators: converters.convertOperators,
        component: null,
        attributes: null
    }, {
        inputType: "text",
        operators: ["is", "not", "contains", "not_contains", "set", "add_value", "remove_value"],
        component: CFITextBox,
        attributes: {
            classNames: "form-control input-sm"
        }
    }, {
        inputType: "text",
        operators: ["is_null", "is_not_null", "exact_match", "set_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "textArea",
        operators: ["is", "not", "contains", "not_contains", "set"],
        component: CFITextBox,
        attributes: {
            classNames: "form-control input-sm"
        }
    }, {
        inputType: "textArea",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "dropDown",
        operators: ["is", "not", "set"],
        component: CFIDropdown,
        attributes: {
            classNames: "form-control input-sm",
            nameField: "name",
            valueField: "id",
            placeholder: Locales.get("labels.pleaseSelect")
        }
    }, {
        inputType: "dropDown",
        operators: ["contains", "not_contains", "contains_any", "not_contains_any", "contains_all", "not_contains_all"],
        component: CFIReactMultiSelect,
        attributes: {
            classNames: "ember-power-select-multiple",
            nameField: "name",
            searchField: "name",
            valueField: "id"
        }
    }, {
        inputType: "dropDown",
        operators: ["is_null", "is_not_null", "set_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "powerSelect",
        operators: ["is", "not", "set"],
        component: CFIReactSelect,
        attributes: {
            classNames: "form-control input-sm"
        }
    }, {
        inputType: "powerSelect",
        operators: ["contains_any", "not_contains_any", "contains_all", "not_contains_all"],
        component: CFIReactMultiSelect,
        attributes: {
            classNames: "ember-power-select-multiple"
        }
    }, {
        inputType: "powerSelect",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "autoComplete",
        operators: ["is", "not", "set"],
        component: CFIAutoComplete,
        attributes: {
            classNames: ""
        }
    }, {
        inputType: "autoComplete",
        operators: ["contains_any", "not_contains_any", "contains_all", "not_contains_all"],
        component: CFIAutoComplete,
        // 配置多选字段
        attributes: {
            classNames: "udesk-auto-complete",
            isMultiSelect: true
        }
    }, {
        inputType: "autoComplete",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "checkBox",
        operators: ["contains_any", "not_contains_any", "contains_all", "not_contains_all", "set"],
        component: CFIReactMultiSelect,
        attributes: {
            classNames: "ember-power-select-multiple",
            nameField: "value",
            valueField: "key",
            searchField: "value"
        }
    }, {
        inputType: "checkBox",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "datePicker",
        operators: ["greater_than", "greater_than_eq", "less_than", "less_than_eq", "set"],
        component: CFIDatePicker,
        attributes: {
            classNames: "date-picker form-control input-sm",
            format: Udesk.config.date.momentDateTimeFormat,
            serverFormat: Udesk.config.date.momentDateTimeFormat
        }
    }, {
        inputType: "datePicker",
        operators: ["is", "not"],
        component: CFIDatePicker,
        attributes: {
            classNames: "date-picker form-control input-sm",
            format: Udesk.config.date.momentDateFormat,
            serverFormat: Udesk.config.date.momentDateFormat
        }
    }, {
        inputType: "datePicker",
        operators: ["between", "not_between"],
        component: CFIDateRangePicker,
        attributes: {
            classNames: "date-range-picker",
            format: Udesk.config.date.momentDateFormat,
            serverFormat: Udesk.config.date.momentDateFormat
        }
    }, {
        inputType: "datePicker",
        operators: ["today", "yesterday", "tomorrow", "next_seven_day", "last_seven_day", "this_week", "next_week", "last_week", "this_month", "next_month", "last_month", "this_year", "next_year", "last_year"],
        component: null,
        attributes: {}
    }, {
        inputType: "datePicker",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "dateTimePicker",
        operators: ["between", "not_between"],
        component: CFIDateRangePicker,
        attributes: {
            classNames: "date-range-picker",
            format: Udesk.config.date.momentDateTimeFormat,
            serverFormat: Udesk.config.date.momentDateTimeFormat
        }
    }, {
        inputType: "dateTimePicker",
        operators: ["greater_than", "greater_than_eq", "less_than", "less_than_eq", "set"],
        component: CFIIntegerNumber,
        attributes: {
            classNames: "date-range-picker",
            format: Udesk.config.date.momentDateTimeFormat,
            serverFormat: Udesk.config.date.momentDateTimeFormat
        }
    }, {
        inputType: "dateTimePicker",
        operators: ["is", "not"],
        component: CFIDatePicker,
        attributes: {
            classNames: "date-time-picker form-control input-sm",
            format: Udesk.config.date.momentDateTimeFormat,
            serverFormat: Udesk.config.date.momentDateTimeFormat
        }
    }, {
        inputType: "dateTimePicker",
        operators: ["today", "yesterday", "tomorrow", "next_seven_day", "last_seven_day", "this_week", "next_week", "last_week", "this_month", "next_month", "last_month", "this_year", "next_year", "last_year"],
        component: null,
        attributes: {
            classNames: "date-time-picker form-control input-sm"
        }
    }, {
        inputType: "dateTimePicker",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "dateRange",
        operators: ["between", "not_between", "set"],
        component: CFIDateRangePicker,
        attributes: {
            classNames: "date-range-picker",
            format: Udesk.config.date.momentDateTimeFormat,
            serverFormat: Udesk.config.date.momentDateTimeFormat
        }
    }, {
        inputType: "dateRange",
        operators: ["is", "not", "greater_than", "greater_than_eq", "less_than", "less_than_eq"],
        component: CFIDatePicker,
        attributes: {
            classNames: "date-range-picker",
            format: Udesk.config.date.momentDateTimeFormat,
            serverFormat: Udesk.config.date.momentDateTimeFormat
        }
    }, {
        inputType: "dateRange",
        operators: ["today", "yesterday", "tomorrow", "next_seven_day", "last_seven_day", "this_week", "next_week", "last_week", "this_month", "next_month", "last_month", "this_year", "next_year", "last_year"],
        component: null,
        attributes: {}
    }, {
        inputType: "dateRange",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "integerNumber",
        operators: ["is", "not", "greater_than", "greater_than_eq", "less_than", "less_than_eq", "set"],
        component: CFIIntegerNumber,
        attributes: {
            classNames: "form-control input-sm"
        }
    }, {
        inputType: "integerNumber",
        operators: ["between", "not_between"],
        component: CFIIntegerNumber,
        attributes: {
            isRange: true,
            classNames: "form-control input-sm input-sm-wide"
        }
    }, {
        inputType: "integerNumber",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "floatNumber",
        operators: ["is", "not", "greater_than", "greater_than_eq", "less_than", "less_than_eq", "set"],
        component: CFIFloatNumber,
        attributes: {
            classNames: "form-control input-sm"
        }
    }, {
        inputType: "floatNumber",
        operators: ["between", "not_between"],
        component: CFIFloatNumber,
        attributes: {
            isRange: true,
            classNames: "form-control input-sm input-sm-wide"
        }
    }, {
        inputType: "floatNumber",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "powerSelectMultiple",
        operators: ["contains_all", "not_contains_all", "contains_any", "set"],
        component: CFIReactMultiSelect,
        attributes: {
            classNames: "ember-power-select-multiple"
        }
    }, {
        inputType: "powerSelectMultiple",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    },
    // {
    //     inputType: "cascadeDropDown",
    //     operators: ["is", "not", "belongs", "not_belongs", "set"],
    //     component: "udesk/custom-filter-inputs/cascade-drop-down",
    //     attributes: {
    //         classNames: "form-control",
    //         nameField: "value",
    //         valuePath: "key",
    //         childNodesField: "subOptionList",
    //         allowEmpty: true
    //     }
    // },
    {
        inputType: "simplePhone",
        operators: ["is", "not", "contains", "not_contains", "set"],
        component: CFISimplePhone,
        attributes: {
            classNames: "form-control input-sm"
        }
    }, {
        inputType: "simplePhone",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "currentUser",
        operators: ["is", "not"],
        component: CFIAutoComplete,
        attributes: {
            classNames: "udesk-auto-complete",
            nameField: "realname",
            valueField: "id",
            searchField: "realname",
            placeholder: "",
            method: "get",
            optionMode: "object",
            queryParamName: "realnameOrEmailOrMobile",
            url: "/backend/users/realnames"
        }
    }, {
        inputType: "currentUser",
        operators: ["contains_any", "not_contains_any"],
        component: CFIAutoComplete,
        attributes: {
            classNames: "udesk-auto-complete",
            isMultiSelect: true,
            nameField: "realname",
            valueField: "id",
            searchField: "realname",
            placeholder: "",
            method: "get",
            optionMode: "object",
            queryParamName: "realnameOrEmailOrMobile",
            url: "/backend/users/realnames"
        }
    }, {
        inputType: "currentUser",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    }, {
        inputType: "currentOffice",
        operators: ["is", "not"],
        component: CFIReactSelect,
        attributes: {
            classNames: "form-control input-sm",
            placeholder: Locales.get("labels.pleaseSelect"),
            nameField: "intentName",
            searchField: "name",
            valueField: "id"
        }
    }, {
        inputType: "currentOffice",
        operators: ["contains_any", "not_contains_any"],
        component: CFIReactMultiSelect,
        attributes: {
            classNames: "ember-power-select-multiple",
            nameField: "intentName",
            searchField: "name",
            valueField: "id",
            placeholder: ""
        }
    }, {
        inputType: "currentOffice",
        operators: ["is_null", "is_not_null"],
        component: null,
        attributes: {}
    }];
}


export default getComponentsConfigTable;
