import DemoLocales from "../../../../app/locales";
import validators from './validators';
import converters from './converters';
import dataSources from './dataSources';

function getFieldsTable() {
    return [{
        name: validators.validateString,
        text: validators.validateString,
        inputType: validators.validateInputTypes,
        operators: validators.validateOperators,
        dataSource: null,
        attributes: validators.validateObject
    }, {
        name: null,
        text: null,
        inputType: converters.convertInputType,
        operators: converters.convertOperators,
        dataSource: null,
        attributes: null
    },

    {
        name: "name",
        text: DemoLocales.get("demo.customFilters.config.fields.name"),
        inputType: "text",
        operators: ["is", "not", "contains", "not_contains", "is_null", "is_not_null"],
        dataSource: null,
        attributes: {
            placeholder: ""
        }
    }, {
        name: "contactsName",
        text: DemoLocales.get("demo.customFilters.config.fields.contactsName"),
        inputType: "dropDown",
        operators: ["is", "not", "contains", "not_contains", "is_null", "is_not_null"],
        dataSource: dataSources.persons,
        attributes: {
            placeholder: ""
        }
    }, {
        name: "followStatus",
        text: DemoLocales.get("demo.customFilters.config.fields.followStatus"),
        inputType: "dropDown",
        operators: ["is", "not", "contains_any", "not_contains_any", "is_null", "is_not_null"],
        dataSource: dataSources.followStatus,
        attributes: {}
    }, {
        name: "mobile",
        text: DemoLocales.get("demo.customFilters.config.fields.mobile"),
        inputType: "text",
        operators: ["is", "not", "contains", "not_contains", "is_null", "is_not_null"],
        dataSource: null,
        attributes: {
            placeholder: ""
        }
    }, {
        name: "telephone",
        text: DemoLocales.get("demo.customFilters.config.fields.telephone"),
        inputType: "text",
        operators: ["is", "not", "contains", "not_contains", "is_null", "is_not_null"],
        dataSource: null,
        attributes: {
            placeholder: ""
        }
    }, {
        name: "email",
        text: DemoLocales.get("demo.customFilters.config.fields.email"),
        inputType: "text",
        operators: ["is", "not", "contains", "not_contains", "is_null", "is_not_null"],
        dataSource: null,
        attributes: {
            placeholder: ""
        }
    }, {
        name: "province",
        text: DemoLocales.get("demo.customFilters.config.fields.province"),
        inputType: "powerSelect",
        operators: ["is", "not", "contains_any", "not_contains_any", "is_null", "is_not_null"],
        dataSource: dataSources.provinces,
        attributes: {
            nameField: "name",
            searchField: "name"
        }
    }, {
        name: "address",
        text: DemoLocales.get("demo.customFilters.config.fields.address"),
        inputType: "text",
        operators: ["is", "not", "contains", "not_contains", "is_null", "is_not_null"],
        dataSource: null,
        attributes: {
            placeholder: ""
        }
    }, {
        name: "masterUserId",
        text: DemoLocales.get("demo.customFilters.config.fields.masterUserId"),
        inputType: "autoComplete",
        operators: ["is", "not", "contains_any", "not_contains_any", "is_null", "is_not_null"],
        dataSource: null,
        attributes: {
            nameField: "realname",
            valueField: "id",
            searchField: "realname",
            method: "get",
            optionMode: "object",
            queryParamName: "realnameOrEmailOrMobile",
            url: "/backend/users/realnames"
        }
    }, {
        name: "leadsHighSeaReceiveTime",
        text: DemoLocales.get("demo.customFilters.config.fields.leadsHighSeaReceiveTime"),
        inputType: "dateTimePicker",
        operators: ["is", "not", "between", "not_between", "is_null", "is_not_null", "greater_than", "greater_than_eq", "less_than", "less_than_eq", "today", "yesterday", "tomorrow", "next_seven_day", "last_seven_day", "this_week", "next_week", "last_week", "this_month", "next_month", "last_month", "this_year", "next_year", "last_year"],
        dataSource: null,
        attributes: {
            nameField: "name",
            searchField: "name",
            valueField: "id",
            placeholder: ""
        }
    }, {
        name: "leadsHighSeaReturnCount",
        text: DemoLocales.get("demo.customFilters.config.fields.leadsHighSeaReturnCount"),
        inputType: "integerNumber",
        operators: ["is", "not", "greater_than", "greater_than_eq", "less_than", "less_than_eq", "between", "is_null", "is_not_null"],
        dataSource: null,
        attributes: {
            placeholder: ""
        }
    }, {
        name: "leadsHighSeaId",
        text: DemoLocales.get("demo.customFilters.config.fields.leadsHighSeaId"),
        inputType: "dropDown",
        operators: ["is", "not", "contains_any", "not_contains_any", "is_null", "is_not_null"],
        dataSource: dataSources.leadHighSeaList,
        attributes: {}
    }, {
        name: "createUserId",
        text: DemoLocales.get("demo.customFilters.config.fields.createUserId"),
        inputType: "dropDown",
        operators: ["is", "not", "contains_any", "not_contains_any", "is_null", "is_not_null"],
        dataSource: dataSources.persons,
        attributes: {}
    }, {
        name: "lastMasterUserId",
        text: DemoLocales.get("demo.customFilters.config.fields.lastMasterUserId"),
        inputType: "dropDown",
        operators: ["is", "not", "contains_any", "not_contains_any", "is_null", "is_not_null"],
        dataSource: dataSources.persons,
        attributes: {}
    }, {
        name: "officeId",
        text: DemoLocales.get("demo.customFilters.config.fields.officeId"),
        inputType: "dropDown",
        operators: ["is", "not", "contains_any", "not_contains_any", "is_null", "is_not_null"],
        dataSource: dataSources.departments,
        attributes: {
            nameField: "intentName",
            placeholder: ""
        }
    }, {
        name: "notFollowDays",
        text: DemoLocales.get("demo.customFilters.config.fields.notFollowDays"),
        inputType: "integerNumber",
        operators: ["is", "not", "greater_than", "greater_than_eq", "less_than", "less_than_eq", "between", "is_null", "is_not_null"],
        dataSource: null,
        attributes: {
            placeholder: ""
        }
    }, {
        name: "highSeaStatus",
        text: DemoLocales.get("demo.customFilters.config.fields.highSeaStatus"),
        inputType: "dropDown",
        operators: ["is", "not", "contains_any", "not_contains_any", "is_null", "is_not_null"],
        dataSource: dataSources.leadListHighSeaStatus,
        attributes: {}
    }, {
        name: "updateUserId",
        text: DemoLocales.get("demo.customFilters.config.fields.updateUserId"),
        inputType: "dropDown",
        operators: ["is", "not", "contains_any", "not_contains_any", "is_null", "is_not_null"],
        dataSource: dataSources.persons,
        attributes: {}
    }, {
        name: "expireTime",
        text: DemoLocales.get("demo.customFilters.config.fields.expireTime"),
        inputType: "dateTimePicker",
        operators: ["is", "not", "between", "not_between", "is_null", "is_not_null", "greater_than", "greater_than_eq", "less_than", "less_than_eq", "today", "yesterday", "tomorrow", "next_seven_day", "last_seven_day", "this_week", "next_week", "last_week", "this_month", "next_month", "last_month", "this_year", "next_year", "last_year"],
        dataSource: null,
        attributes: {
            placeholder: ""
        }
    }, {
        name: "updateTime",
        text: DemoLocales.get("demo.customFilters.config.fields.updateTime"),
        inputType: "dateTimePicker",
        operators: ["is", "not", "between", "not_between", "is_null", "is_not_null", "greater_than", "greater_than_eq", "less_than", "less_than_eq", "today", "yesterday", "tomorrow", "next_seven_day", "last_seven_day", "this_week", "next_week", "last_week", "this_month", "next_month", "last_month", "this_year", "next_year", "last_year"],
        dataSource: null,
        attributes: {
            nameField: "name",
            searchField: "name",
            valueField: "id",
            placeholder: "",
            classNames: ""
        }
    }, {
        name: "createTime",
        text: DemoLocales.get("demo.customFilters.config.fields.createTime"),
        inputType: "dateTimePicker",
        operators: ["is", "not", "between", "not_between", "is_null", "is_not_null", "greater_than", "greater_than_eq", "less_than", "less_than_eq", "today", "yesterday", "tomorrow", "next_seven_day", "last_seven_day", "this_week", "next_week", "last_week", "this_month", "next_month", "last_month", "this_year", "next_year", "last_year"],
        dataSource: null,
        attributes: {
            nameField: "name",
            searchField: "name",
            valueField: "id",
            placeholder: ""
        }
    }, {
        name: "recentActivityRecordTime",
        text: DemoLocales.get("demo.customFilters.config.fields.recentActivityRecordTime"),
        inputType: "dateTimePicker",
        operators: ["is", "not", "between", "not_between", "is_null", "is_not_null", "greater_than", "greater_than_eq", "less_than", "less_than_eq", "today", "yesterday", "tomorrow", "next_seven_day", "last_seven_day", "this_week", "next_week", "last_week", "this_month", "next_month", "last_month", "this_year", "next_year", "last_year"],
        dataSource: null,
        attributes: {
            nameField: "name",
            searchField: "name",
            valueField: "id",
            placeholder: ""
        }
    }, {
        name: "remark",
        text: DemoLocales.get("demo.customFilters.config.fields.remark"),
        inputType: "text",
        operators: ["is", "not", "contains", "not_contains", "is_null", "is_not_null"],
        dataSource: null,
        attributes: {
            placeholder: ""
        }
    }, {
        name: "tags",
        text: DemoLocales.get("demo.customFilters.config.fields.tags"),
        inputType: "powerSelect",
        operators: ["contains_any", "not_contains_any", "contains_all", "not_contains_all", "is_null", "is_not_null"],
        dataSource: dataSources.leadTagList,
        attributes: {
            nameField: "name",
            searchField: "name",
            valueField: "id"
        },
    },

    ];
}

export default getFieldsTable;
