import getFieldsTable from './fields';
import getComponentsConfigTable from './components';

const NORMAL_FIELDS_TABLE_VALIDATOR_INDEX = 0;
const NORMAL_FIELDS_TABLE_CONVERTER_INDEX = 1;
const NORMAL_FIELDS_TABLE_START_INDEX = (Math.max(
    NORMAL_FIELDS_TABLE_VALIDATOR_INDEX,
    NORMAL_FIELDS_TABLE_CONVERTER_INDEX) + 1);
const NORMAL_FIELDS_AVOID_EVALUATION_FIELDS = ["dataSource"];

const VALUE_SETTINGS_TABLE_VALIDATOR_INDEX = 0;
const VALUE_SETTINGS_TABLE_CONVERTER_INDEX = 1;
const VALUE_SETTINGS_TABLE_START_INDEX = (Math.max(
    VALUE_SETTINGS_TABLE_VALIDATOR_INDEX,
    VALUE_SETTINGS_TABLE_CONVERTER_INDEX) + 1);
const VALUE_SETTINGS_AVOID_EVALUATION_FIELDS = ["component"];

export default {
    getFieldsConfig,
    getComponentSettings,
};

function getFieldsConfig() {
    let normalFields = getFieldsTable();
    return buildNormalFields(normalFields);
}

function buildNormalFields(configTable) {
    let fields = [];
    if (configTable.length > 0) {
        let validators = configTable[NORMAL_FIELDS_TABLE_VALIDATOR_INDEX];
        let converters = configTable[NORMAL_FIELDS_TABLE_CONVERTER_INDEX];
        for (let i = NORMAL_FIELDS_TABLE_START_INDEX; i < configTable.length; i++) {
            let config = configTable[i];
            for (let fieldName in validators) {
                if (Object.prototype.hasOwnProperty.call(validators,fieldName)) {
                    let validator = validators[fieldName];
                    let value = config[fieldName];

                    if (typeof validator === "function") {
                        let result = validator.call(config, value);
                        if (result !== true) {
                            throw new Error(`
Wrong configuration in 'custom-filters/demo/config/fields.js'.
Inner error message is '${result}'.
Source row is at #${i - NORMAL_FIELDS_TABLE_START_INDEX + 1}, field is '${fieldName}'.`.trim());
                        }
                    }
                }
            }

            var field = Object.assign({}, config);
            for (let fieldName in converters) {
                if (Object.prototype.hasOwnProperty.call(converters,fieldName)) {
                    let converter = converters[fieldName];
                    let value = config[fieldName];
                    if (typeof converter === "function") {
                        value = converter.call(config, value);
                    }
                    field[fieldName] = value;
                }
            }
            for (let fieldName in field) {
                if (Object.prototype.hasOwnProperty.call(field,fieldName)) {
                    let value = field[fieldName];
                    if (typeof value === "function" && NORMAL_FIELDS_AVOID_EVALUATION_FIELDS.indexOf(fieldName) === -1) {
                        field[fieldName] = value.call(field, fieldName);
                    }
                }
            }
            fields.push(field);
        }
    }
    return fields;
}

function getComponentSettings() {
    return buildComponentSettings(getComponentsConfigTable());
}

function buildComponentSettings(configTable) {
    let settingsArray = [];
    if (configTable.length > 0) {
        let validators = configTable[VALUE_SETTINGS_TABLE_VALIDATOR_INDEX];
        let converters = configTable[VALUE_SETTINGS_TABLE_CONVERTER_INDEX];
        for (let i = VALUE_SETTINGS_TABLE_START_INDEX; i < configTable.length; i++) {
            let config = configTable[i];

            for (let fieldName in validators) {
                if (Object.prototype.hasOwnProperty.call(validators,fieldName)) {
                    let validator = validators[fieldName];
                    let value = config[fieldName];

                    if (typeof validator === "function") {
                        let result = validator.call(config, value);
                        if (result !== true) {
                            throw new Error(`
Wrong configuration in 'field-value-settings.js'.
Inner error message is '${result}'.
Source row is at #${i - VALUE_SETTINGS_TABLE_START_INDEX + 1}, field is '${fieldName}'.`.trim());
                        }
                    }
                }
            }

            let settings = Object.assign({}, config);
            for (let fieldName in converters) {
                if (Object.prototype.hasOwnProperty.call(converters,fieldName)) {
                    let converter = converters[fieldName];
                    let value = config[fieldName];
                    if (typeof converter === "function") {
                        value = converter.call(config, value);
                    }
                    settings[fieldName] = value;
                }
            }

            for (let fieldName in settings) {
                if (Object.prototype.hasOwnProperty.call(settings,fieldName)) {
                    let value = settings[fieldName];
                    if (typeof value === "function" && VALUE_SETTINGS_AVOID_EVALUATION_FIELDS.indexOf(fieldName) === -1) {
                        settings[fieldName] = value.call(settings, fieldName);
                    }
                }
            }
            settingsArray.push(settings);
        }
    }
    return settingsArray;
}