import React from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import Locales from '../../udesk/locales';
import Udesk from "../../udesk/index";

class ReportShowSlider extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            currentPosition: 0,
        };
    }

    actions = {
        onPrev() {
            let {
                currentPosition
            } = this.state;
            if (currentPosition === 0) {
                return;
            }
            this.setState({
                currentPosition: currentPosition - 1
            });
        },
        onNext() {
            let {
                dataList
            } = this.props;
            let {
                currentPosition
            } = this.state;
            if (currentPosition + 1 === dataList.length) {
                return;
            }
            this.setState({
                currentPosition: currentPosition + 1
            });
        },
        showCurrentReportTip(ind, e) {
            e.target.disabled = true;
            e.stopPropagation();
            this.setState((prevState) => {
                if (prevState.currentPosition !== ind) {
                    return {
                        currentPosition: ind
                    };
                }
            });
            e.target.disabled = false;
        }
    }

    render() {
        let {
            className,
            languageCode,
            dataList,
            closeBtnText,
            showDots,
            showNextBtn,
            showPrevBtn,
            nextBtnText,
            prevBtnText
        } = this.props;
        let {
            currentPosition
        } = this.state;
        let dataListLength = (dataList && dataList.length) ? dataList.length : 0;

        if (!prevBtnText) {
            prevBtnText = Locales.get("components.customReactSlider.prev", languageCode);
        }
        if (!nextBtnText) {
            nextBtnText = Locales.get("components.customReactSlider.next", languageCode);
        }
        if (!closeBtnText) {
            closeBtnText = Locales.get("components.customReactSlider.close", languageCode);
        }

        return (
            <div className={ClassNames("custom-react-slider", { [`${className}`]: className })}>
                <If condition={dataListLength > 0}>
                    <ul className="custom-react-slider-content">
                        {dataList.map((item, index) => {
                            return (
                                <li className={ClassNames("custom-react-slider-content-item", { [`custom-react-slider-content-item-active`]: index === currentPosition })} key={`custom-react-slider-content-item-${index}`}>
                                    <div className="custom-react-slider-content-item-img-container">
                                        <img className="custom-react-slider-content-item-img" src={`${item.url}`} alt="" />
                                    </div>
                                    <If condition={item.title && item.content}>
                                        <div className={ClassNames("custom-react-slider-content-item-text", { [`custom-react-slider-content-item-text-active`]: index === currentPosition })}>
                                            <If condition={item.title}>
                                                <p className="custom-react-slider-content-item-text-title">
                                                    {item.title}
                                                </p>
                                            </If>
                                            <If condition={item.content}>
                                                <p className="custom-react-slider-content-item-text-content">
                                                    {item.content}
                                                </p>
                                            </If>
                                        </div>
                                    </If>
                                </li>
                            );
                        })}
                    </ul>
                    <div className="custom-react-slider-actions">
                        <If condition={showPrevBtn}>
                            <div className="custom-react-slider-btn custom-react-slider-btn-prev" onClick={this.actions.onPrev}>
                                {prevBtnText}
                            </div>
                        </If>
                        <If condition={showDots}>
                            <div className="custom-react-slider-dots">
                                {dataList.map((item, index) => {
                                    return (
                                        <span className={ClassNames("custom-react-slider-dots-item", { [`custom-react-slider-dots-item-active`]: currentPosition === index })} key={`custom-react-slider-dots-item-${index}`} onClick={(e) => this.actions.showCurrentReportTip(index, e)}>
                                        </span>
                                    );
                                })}
                            </div>
                        </If>
                        <If condition={showNextBtn}>
                            <div className="custom-react-slider-btn custom-react-slider-btn-next" onClick={this.actions.onNext}>
                                {nextBtnText}
                            </div>
                        </If>
                    </div>
                </If>
            </div>
        );
    }
}

ReportShowSlider.propTypes = {
    className: PropTypes.string,
    dataList: PropTypes.array,
    showDots: PropTypes.bool,
    showNextBtn: PropTypes.bool,
    showPrevBtn: PropTypes.bool,
    nextBtnText: PropTypes.string,
    prevBtnText: PropTypes.string,
    languageCode: PropTypes.string,
};

ReportShowSlider.defaultProps = {
    className: "",
    dataList: [],
    showDots: true,
    showNextBtn: true,
    showPrevBtn: true,
    nextBtnText: "",
    prevBtnText: "",
    languageCode: "",
};

export default Udesk.react.udeskify(ReportShowSlider);