import React from 'react';
import ClassNames from 'classnames';
import Udesk from '../../../udesk/index';
import PropTypes from 'prop-types';
import { findWidgets, traverseWidgets } from './business';
import defaultWidgets from './config/custom-ui-initialize-widgets-mappings';
import { get, clone } from 'udesk-web-toolkits/src/udesk/utils/object';
import WidgetRootComponent from './widget-root';
import DynamicComponent from '../../dynamic-component';

class CustomUiGridClass extends React.Component {
    static defaultProps = {
        classNames: '',
        type: 'normal',
        mode: 'design',
        theme: 'default',
        widgets: null,
        allowDrop: false,
        dropDataTransferWidgetTypeField: 'widgetType',
        dropDataTransferDataField: 'widgetDescriptor',
        gridCols: 12,
        gridRowHeight: 150,
        gridMargin: [10, 10],
        defaultAddWidgetWidth: 1,
        defaultAddWidgetMaxWidth: 12,
        defaultAddWidgetMinWidth: 1,
        defaultAddWidgetHeight: 1,
        enbaleWidgetDraggable: false,
        enableSetUpWidgetConfig: false,
        componentSettings: [],
        rootWidgetType: 'root',
        enableWidgetNameCount: false,
        settingsWidgetTypeField: 'widgetType',
        settingsWidgetTypeNameField: 'widgetTypeName',
        settingsComponentField: 'component',
        settingsConfigField: 'config',
        settingsConfigComponentField: 'configComponent',
        settingsAttributesField: 'attributes',
        settingsAddingWidgetRulesField: 'addingWidgetRules',
        getExtraWidgetInstanceStates: {},

        onWidgetAdded: null,
        onWidgetRemoved: null,
        onWidgetMoved: null,
        onActiveWidgetChanged: null,
        onWidgetConfigChanged: null,
        onChanged: null,
    };

    static propTypes = {
        classNames: PropTypes.string,
        type: PropTypes.string,
        mode: PropTypes.string,
        theme: PropTypes.string,
        widgets: PropTypes.array,
        allowDrop: PropTypes.bool,
        dropDataTransferWidgetTypeField: PropTypes.string,
        dropDataTransferDataField: PropTypes.string,
        gridCols: PropTypes.number,
        gridRowHeight: PropTypes.number,
        gridMargin: PropTypes.array,
        defaultAddWidgetWidth: PropTypes.number,
        defaultAddWidgetMaxWidth: PropTypes.number,
        defaultAddWidgetMinWidth: PropTypes.number,
        defaultAddWidgetHeight: PropTypes.number,
        enbaleWidgetDraggable: PropTypes.bool,
        enableSetUpWidgetConfig: PropTypes.bool,
        componentSettings: PropTypes.array,
        rootWidgetType: PropTypes.string,
        enableWidgetNameCount: PropTypes.bool,
        settingsWidgetTypeField: PropTypes.string,
        settingsWidgetTypeNameField: PropTypes.string,
        settingsComponentField: PropTypes.string,
        settingsConfigField: PropTypes.string,
        settingsConfigComponentField: PropTypes.string,
        settingsAttributesField: PropTypes.string,
        settingsAddingWidgetRulesField: PropTypes.string,
        getExtraWidgetInstanceStates: PropTypes.oneOfType([PropTypes.func, PropTypes.object]),

        onWidgetAdded: PropTypes.func,
        onWidgetRemoved: PropTypes.func,
        onWidgetMoved: PropTypes.func,
        onActiveWidgetChanged: PropTypes.func,
        onWidgetConfigChanged: PropTypes.func,
        onChanged: PropTypes.func,
    };

    getWidgets() {
        let widgets = this.privates.widgets;
        const getWidgetsJson = function (root, callback) {
            let resultWidgets = [];
            for (var widget of root) {
                let resultWidget = callback(widget);
                resultWidgets.push(resultWidget);
                if (
                    widget.instance.widgets &&
                    widget.instance.widgets instanceof Array &&
                    widget.instance.widgets.length > 0
                ) {
                    resultWidget.instance.widgets = getWidgetsJson(widget.instance.widgets, callback);
                }
            }
            return resultWidgets;
        };

        let resultWidgets = getWidgetsJson(widgets, function (widget) {
            let widgetJson = {
                descriptor: Udesk.utils.object.clone(widget.descriptor),
                instance: {
                    config: Udesk.utils.object.clone(widget.instance.config),
                    id: widget.instance.id,
                },
            };
            return widgetJson;
        });
        return resultWidgets;
    }

    privates = {
        widgets: [],
        activeWidgetId: null,
        customUiConfigComponent: null,
    };

    static computes = {
        classNames: ({ props, state, privates, locales }) => {
            let { classNames, type, mode, theme } = props;
            let componentClass = [];
            if (classNames != null && classNames !== '') {
                componentClass.push(classNames);
            }
            if (mode == null || mode === '') {
                mode = Udesk.enums.udeskReactCustomUiGridClassModeEnums.edit.id;
            }
            componentClass.push(`udesk-react-custom-ui-grid-mode-${mode}`);
            if (theme != null && theme !== '') {
                componentClass.push(`udesk-react-custom-ui-grid-theme-${theme}`);
            }
            if (type != null && type !== '') {
                componentClass.push(`udesk-react-custom-ui-grid-type-${type}`);
            }
            return componentClass.join(' ');
        },
        rootComponent: ({ props, state, privates, locales }) => {
            let { widgets } = privates;
            let RootComponent = WidgetRootComponent;
            if (widgets && widgets.length > 0 && widgets[0].settings.component) {
                RootComponent = widgets[0].instance.settings.component;
            }
            return RootComponent;
        },
        activeWidget: ({ props, state, privates, locales }) => {
            let { widgets } = privates;
            return findWidgets(widgets, function (widget) {
                return widget.instance.states.isActive;
            })[0];
        },
        enableSetUpWidgetConfig: ({ props, state, privates, locales }) => {
            let { enableSetUpWidgetConfig, mode } = props;
            return enableSetUpWidgetConfig && mode === Udesk.enums.udeskReactCustomUiGridModeEnums.design.id;
        },
    };

    actions = {
        onSelected(params) {
            let mode = this.props.mode;
            if (mode === Udesk.enums.udeskReactCustomUiGridModeEnums.design.id) {
                let configValidateResult = widgetConfigValidate(this);
                if (configValidateResult.isValid) {
                    let widgetId = params.id;
                    let widgets = this.privates.widgets;
                    let targetWidget = findWidgets(widgets, (item) => item.instance.id === widgetId)[0];
                    let previousWidget = findWidgets(widgets, (item) => item.instance.states.isActive)[0];
                    if (previousWidget == null || targetWidget.instance.id !== previousWidget.instance.id) {
                        traverseWidgets(widgets, function (widget) {
                            if (widget.instance.states.isActive) {
                                widget.instance.states.isActive = false;
                            } else {
                                if (widget.instance.id === widgetId) {
                                    widget.instance.states.isActive = true;
                                }
                            }
                        });
                        this.privates.widgets = widgets;
                        this.actions.update();
                        if (this.props.onActiveWidgetChanged != null) {
                            let previousWidgetId = previousWidget ? previousWidget.instance.id : null;
                            this.props.onActiveWidgetChanged(widgetId, previousWidgetId);
                        }
                    }
                } else {
                    let errorMessage = '';
                    if (configValidateResult.messages) {
                        errorMessage = configValidateResult.messages.join('!');
                    }
                    return Udesk.ui.notify.error(errorMessage);
                }
            } else {
                return false;
            }
        },
        onWidgetMoved(layout) {
            let mode = this.props.mode;
            if (mode === Udesk.enums.udeskReactCustomUiGridModeEnums.design.id) {
                let widgets = this.privates.widgets;
                if (widgets instanceof Array && widgets.length > 0) {
                    widgets[0].instance.config.layoutConfig = layout;
                    this.privates.widgets = widgets;
                    this.actions.update();
                    if (this.props.onWidgetMoved != null) {
                        this.props.onWidgetMoved(layout);
                    }
                }
            } else {
                return false;
            }
        },
        onWidgetConfigChanged(params) {
            let mode = this.props.mode;
            if (mode === Udesk.enums.udeskReactCustomUiGridModeEnums.design.id) {
                let targetWidgetId = params.id;
                let config = params.config;
                if (config == null || typeof config !== 'object') {
                    return;
                }
                let widgets = this.privates.widgets;
                traverseWidgets(widgets, function (widget) {
                    if (widget.instance.id === targetWidgetId) {
                        widget.instance.config = Object.assign(widget.instance.config, config);
                        return false;
                    }
                });
                this.privates.widgets = widgets;
                this.actions.update();
                if (this.props.onWidgetConfigChanged != null) {
                    this.props.onWidgetConfigChanged(params);
                }
            } else {
                return false;
            }
        },
        addWidget(params) {
            let mode = this.props.mode;
            if (mode === Udesk.enums.udeskReactCustomUiGridModeEnums.design.id) {
                let newWidgetDescriptors = params.descriptors;
                let targetWidgetId = params.id;
                let widgets = this.privates.widgets;
                let isAdded = false;
                let layoutConfig = widgets[0].instance.config.layoutConfig;
                if (layoutConfig == null || !(layoutConfig instanceof Array)) {
                    layoutConfig = [];
                }
                newWidgetDescriptors.forEach((newWidgetDescriptor) => {
                    let childWidget = createWidget(this, newWidgetDescriptor, targetWidgetId);
                    if (childWidget != null) {
                        isAdded = true;
                        let isDefaultSelect = false;
                        if (
                            childWidget.settings.widgetAttributes &&
                            childWidget.settings.widgetAttributes.defaultSelect
                        ) {
                            let configValidateResult = widgetConfigValidate(this);
                            if (configValidateResult.isValid) {
                                childWidget.instance.states.active = true;
                                isDefaultSelect = true;
                            } else {
                                let errorMessage = '';
                                if (configValidateResult.messages) {
                                    errorMessage = configValidateResult.messages.join('!');
                                }
                                return Udesk.ui.notify.error(errorMessage);
                            }
                        }
                        let {
                            defaultAddWidgetWidth,
                            defaultAddWidgetHeight,
                            defaultAddWidgetMaxWidth,
                            defaultAddWidgetMinWidth,
                            gridCols,
                        } = this.props;
                        if (
                            defaultAddWidgetWidth == null ||
                            typeof defaultAddWidgetWidth !== 'number' ||
                            defaultAddWidgetWidth > gridCols
                        ) {
                            defaultAddWidgetWidth = 1;
                        }
                        if (defaultAddWidgetHeight == null || typeof defaultAddWidgetHeight !== 'number') {
                            defaultAddWidgetHeight = 1;
                        }
                        if (
                            defaultAddWidgetMaxWidth == null ||
                            typeof defaultAddWidgetMaxWidth !== 'number' ||
                            defaultAddWidgetMaxWidth > gridCols
                        ) {
                            defaultAddWidgetMaxWidth = gridCols;
                        }
                        if (
                            defaultAddWidgetMinWidth == null ||
                            typeof defaultAddWidgetMinWidth !== 'number' ||
                            defaultAddWidgetMaxWidth > gridCols
                        ) {
                            defaultAddWidgetMinWidth = 1;
                        }
                        let context = this;
                        traverseWidgets(widgets, function (widget) {
                            if (isDefaultSelect) {
                                if (widget.instance.states.isActive) {
                                    widget.instance.states.isActive = false;
                                }
                            }
                            if (widget.instance.id === targetWidgetId) {
                                if (widget.instance.widgets == null || !(widget.instance.widgets instanceof Array)) {
                                    widget.instance.widgets = [];
                                }
                                widget.instance.widgets.push(childWidget);
                                let newWidgetLayout = getNewWidgetLayout(
                                    context,
                                    layoutConfig,
                                    defaultAddWidgetWidth,
                                    defaultAddWidgetHeight
                                );
                                layoutConfig = layoutConfig.concat([
                                    {
                                        i: childWidget.instance.id,
                                        x: newWidgetLayout.x,
                                        y: Infinity,
                                        w:
                                            get(childWidget.settings.attributes, 'defaultAddWidgetWidth') ||
                                            defaultAddWidgetWidth,
                                        h:
                                            get(childWidget.settings.attributes, 'defaultAddWidgetHeight') ||
                                            defaultAddWidgetHeight,
                                        minW:
                                            get(childWidget.settings.attributes, 'defaultAddWidgetMinWidth') ||
                                            defaultAddWidgetMinWidth,
                                        maxW:
                                            get(childWidget.settings.attributes, 'defaultAddWidgetMaxWidth') ||
                                            defaultAddWidgetMaxWidth,
                                    },
                                ]);
                            }
                        });
                    }
                });
                if (isAdded) {
                    widgets[0].instance.config.layoutConfig = layoutConfig;
                    this.privates.widgets = widgets;
                    this.actions.update();
                    if (this.props.onWidgetMoved != null) {
                        this.props.onWidgetAdded(params);
                    }
                }
            } else {
                return false;
            }
        },
        removeWidget(params) {
            let mode = this.props.mode;
            if (mode === Udesk.enums.udeskReactCustomUiGridModeEnums.design.id) {
                let widgetIds = params.ids;
                let widgets = this.privates.widgets;
                traverseWidgets(widgets, function (widget) {
                    if (widget.instance.widgets) {
                        widget.instance.widgets = widget.instance.widgets.filter((item) => {
                            return !widgetIds.some((widgetId) => widgetId === item.instance.id);
                        });
                    }
                });
                this.privates.widgets = widgets;
                this.actions.update();
                if (this.props.onWidgetRemoved != null) {
                    this.props.onWidgetRemoved(widgetIds);
                }
            } else {
                return false;
            }
        },
    };

    componentDidMount() {
        buildWidgets(this);
    }

    componentWillUnmount() {}

    render() {
        let { computes, widgets } = this.privates;
        let {
            mode,
            dropDataTransferWidgetTypeField,
            dropDataTransferDataField,
            gridCols,
            gridRowHeight,
            allowDrop,
            gridMargin,
            enbaleWidgetDraggable,
        } = this.props;
        let activeWidget = computes.activeWidget;
        return (
            <div className={ClassNames('udesk-react-custom-ui-grid', { [`${computes.classNames}`]: true })}>
                <div className='custom-ui-grid-panel'>
                    <For of={widgets} each='widget' index='index'>
                        <div key={`custom-ui-root-${index}`} className='custom-ui-root-wrapper'>
                            <DynamicComponent
                                component={computes.rootComponent}
                                mode={mode}
                                widget={widget}
                                allowDrop={allowDrop}
                                enbaleWidgetDraggable={enbaleWidgetDraggable}
                                gridCols={gridCols}
                                gridMargin={gridMargin}
                                gridRowHeight={gridRowHeight}
                                dropDataTransferDataField={dropDataTransferDataField}
                                dropDataTransferWidgetTypeField={dropDataTransferWidgetTypeField}
                                onSelected={this.actions.onSelected}
                                addWidget={this.actions.addWidget}
                                removeWidget={this.actions.removeWidget}
                                onWidgetMoved={this.actions.onWidgetMoved}
                                onCreated={widget.globalActions.onComponentCreated}
                                onDestroyed={widget.globalActions.onComponentDestoryed}
                            />
                        </div>
                    </For>
                </div>
                <If condition={computes.enableSetUpWidgetConfig}>
                    <div className='custom-ui-grid-config'>
                        <If condition={activeWidget != null}>
                            <DynamicComponent
                                component={activeWidget.settings.configComponent}
                                widgetId={activeWidget.instance.id}
                                widget={activeWidget}
                                onWidgetConfigChanged={this.actions.onWidgetConfigChanged}
                                onCreated={(instance) => {
                                    this.privates.customUiConfigComponent = instance;
                                }}
                                onDestroyed={() => {
                                    this.privates.customUiConfigComponent = null;
                                }}
                            />
                        </If>
                    </div>
                </If>
            </div>
        );
    }
}

function createWidget(context, descriptor, targetWidgetId) {
    if (descriptor == null || descriptor.type == null || descriptor.type === '') {
        return;
    }
    let widgetType = descriptor.type;
    if (targetWidgetId == null || targetWidgetId === '') {
        return;
    }
    let componentSetting = getComponentSettings(context, descriptor.type);
    let { widgets } = context.privates;
    let settingsAddingWidgetRulesField = context.props.settingsAddingWidgetRulesField;
    if (widgets != null) {
        let settingsAddingWidgetRules = get(componentSetting, settingsAddingWidgetRulesField);
        if (settingsAddingWidgetRules != null && settingsAddingWidgetRules.length > 0) {
            let getNextWidgetFun = getNextWidget(widgets);
            if (targetWidgetId != null) {
                for (let ruleIndex = 0; ruleIndex < settingsAddingWidgetRules.length; ruleIndex++) {
                    let rule = settingsAddingWidgetRules[ruleIndex];
                    let ruleResult = rule.call(this, {
                        descriptor: descriptor,
                        targetWidgetId: targetWidgetId,
                        getNextWidget: getNextWidgetFun(),
                    });
                    if (!ruleResult.isValid) {
                        Udesk.ui.notify.error(ruleResult.messages.join(','));
                        return;
                    }
                }
            }
        }
    }
    let {
        settingsWidgetTypeNameField,
        settingsConfigField,
        getExtraWidgetInstanceStates,
        settingsAttributesField,
        settingsComponentField,
        settingsConfigComponentField,
        enableWidgetNameCount,
    } = context.props;
    let widgetId = getWidgetIdentity(context, descriptor.type);
    let widgetTypeName = get(componentSetting, settingsWidgetTypeNameField);
    let widgetName = '';
    if (widgetTypeName != null) {
        if (enableWidgetNameCount) {
            widgetName = generateWidgetName(context, widgetTypeName);
        } else {
            widgetName = widgetTypeName;
        }
    }
    let defaultInstanceConfig = get(componentSetting, settingsConfigField) || {};
    let widget = {
        instance: {
            id: widgetId,
            config: clone(
                Object.assign({}, defaultInstanceConfig, {
                    name: widgetName,
                })
            ),
            states: {
                isActive: false,
            },
            widgets: [],
        },
        settings: Object.assign({}, componentSetting, {
            component: get(componentSetting, settingsComponentField),
            configComponent: get(componentSetting, settingsConfigComponentField),
            attributes: get(componentSetting, settingsAttributesField),
        }),
        descriptor: {
            type: widgetType,
            attributes: componentSetting[settingsAttributesField],
        },
        globalActions: {
            onComponentCreated: (instance) => {
                onComponentCreated(context, widget, instance);
            },
            onComponentDestoryed: () => {
                onComponentDestoryed(widget);
            },
            customUIProxy: {},
        },
    };
    if (getExtraWidgetInstanceStates != null) {
        if (typeof getExtraWidgetInstanceStates === 'object') {
            widget.instance.states = Object.assign(widget.instance.states, getExtraWidgetInstanceStates);
        }
        if (typeof getExtraWidgetInstanceStates === 'function') {
            widget.instance.states = Object.assign(widget.instance.states, getExtraWidgetInstanceStates(widget));
        }
    }
    return widget;
}

function buildWidgets(context) {
    let {
        widgets,
        rootWidgetType,
        settingsAttributesField,
        settingsComponentField,
        settingsConfigComponentField,
        getExtraWidgetInstanceStates,
    } = context.props;
    if (widgets == null || !(widgets instanceof Array) || widgets.length === 0) {
        widgets = clone(defaultWidgets);
        widgets[0].descriptor.type = rootWidgetType;
        widgets[0].instance.id = getWidgetIdentity(context, widgets[0].descriptor.type);
    } else {
        widgets = clone(widgets);
    }
    traverseWidgets(widgets, function (widget) {
        let widgetType = widget.descriptor.type;
        let componentSetting = getComponentSettings(context, widgetType);
        widget.settings = Object.assign({}, componentSetting, {
            component: get(componentSetting, settingsComponentField),
            configComponent: get(componentSetting, settingsConfigComponentField),
            attributes: get(componentSetting, settingsAttributesField),
        });
        widget.descriptor.attributes = componentSetting[settingsAttributesField];
        widget.instance.states = {
            isActive: false,
        };
        if (getExtraWidgetInstanceStates != null) {
            if (typeof getExtraWidgetInstanceStates === 'object') {
                widget.instance.states = Object.assign(widget.instance.states, getExtraWidgetInstanceStates);
            }
            if (typeof getExtraWidgetInstanceStates === 'function') {
                widget.instance.states = Object.assign(widget.instance.states, getExtraWidgetInstanceStates(widget));
            }
        }
        widget.globalActions = {
            onComponentCreated: (instance) => {
                onComponentCreated(context, widget, instance);
            },
            onComponentDestoryed: () => {
                onComponentDestoryed(widget);
            },
            customUIProxy: {},
        };
    });
    context.privates.widgets = widgets;
    context.actions.update();
}

function onComponentCreated(context, widget, instance) {
    instance.on('onChanged', (params) => {
        if (context.props.onChanged != null) {
            context.props.onChanged(params);
        }
        if (params.eventSource != null) {
            context.sendAction('onWidgetsChanged', params);
        }
    });
    widget.instance.states.componentElement = instance;
}
function onComponentDestoryed(widget, instance) {
    if (widget.instance.states.componentElement != null) {
        widget.instance.states.componentElement.off('onChanged');
        widget.instance.states.componentElement = null;
    }
}

function getNewWidgetLayout(context, layout, defaultAddWidgetWidth, defaultAddWidgetHeight) {
    let { gridCols } = context.props;

    return {
        x: (layout.length * defaultAddWidgetWidth) % gridCols,
        y: Infinity,
    };
}

function getComponentSettings(context, widgetType) {
    let { componentSettings, settingsWidgetTypeField } = context.props;
    if (componentSettings == null || !(componentSettings instanceof Array)) {
        componentSettings = [];
    }
    let componentSetting = componentSettings.find((item) => {
        if (item != null && typeof item === 'object') {
            if (get(item, settingsWidgetTypeField) === widgetType) {
                return true;
            }
        }
        return false;
    });
    return componentSetting || {};
}

function getWidgetIdentity(context, customUiType) {
    let { widgets } = context.privates;
    if (widgets == null || !(widgets instanceof Array)) {
        widgets = [];
    }
    let autoIncrementId = 1;
    traverseWidgets(widgets, (widget) => {
        let widgetId = widget.instance.id.split('-');
        let widgetNumId = Number(widgetId[widgetId.length - 1]);
        if (autoIncrementId <= widgetNumId) {
            autoIncrementId = widgetNumId + 1;
        }
    });
    return `${customUiType}-${autoIncrementId}`;
}

/*
 × 构建一个widget的name
 */
function generateWidgetName(context, widgetType) {
    let { widgets } = context.privates;
    let sortIndex = 1;
    let newWidgetName = widgetType + sortIndex;
    if (widgets != null) {
        while (findTargetWidgetName(widgets, newWidgetName)) {
            sortIndex++;
            newWidgetName = widgetType + sortIndex;
        }
    }
    return newWidgetName;
}

function findTargetWidgetName(widgets, widgetName) {
    let targetWidget = findWidgets(widgets, (item) => item.instance.config.name === widgetName)[0];
    if (targetWidget == null) {
        return false;
    }
    return true;
}

function widgetConfigValidate(context) {
    let configValidateResult = {
        isValid: true,
    };
    let customUiConfigComponent = context.privates.customUiConfigComponent;
    if (customUiConfigComponent != null && typeof customUiConfigComponent.validate === 'function') {
        configValidateResult = customUiConfigComponent.validate();
    }
    return configValidateResult;
}

/*
 × widget的customUIProxy子方法
 × 可以让子widget遍历所有的widget的function
 */
function getNextWidget(widgets, filterAttribute) {
    let widgetArray = [];
    traverseWidgets(widgets, function (widget) {
        widgetArray.push(widget);
    });
    return function () {
        let widgetIndex = 0;
        return function () {
            let nextWidget = widgetArray[widgetIndex];
            widgetIndex++;
            if (typeof filterAttribute === 'function') {
                return filterAttribute(nextWidget);
            } else {
                return nextWidget;
            }
        };
    };
}

export default Udesk.react.udeskify(CustomUiGridClass);
