import React from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import Udesk from '../../../udesk/index';

class WidgetWrapper extends React.Component {
    static defaultProps = {
        mode: "design",
        widget: null,
        removeIconClass: "udesk-react-iconfont icon-udesk-react-shanchu2",
        onSelected: null,
        removeWidget: null
    }

    static propTypes = {
        mode: PropTypes.string,
        widget: PropTypes.object,
        removeIconClass: PropTypes.string,
        onSelected: PropTypes.func,
        removeWidget: PropTypes.func
    }

    privates = {

    }

    static computes = {
        enableOperate: ({ props, state, privates, locales }) => {
            let {
                mode
            } = props;
            return mode === Udesk.enums.udeskReactCustomUiGridModeEnums.design.id;
        },
        enableShowMask: ({ props, state, privates, locales }) => {
            let {
                mode
            } = props;
            return mode !== Udesk.enums.udeskReactCustomUiGridModeEnums.edit.id;
        },
    }

    actions = {
        removeWidget(e) { 
            e.stopPropagation();
            let {
                widget,
                removeWidget,
                mode
            } = this.props;
            if (mode === Udesk.enums.udeskReactCustomUiGridModeEnums.design.id) { 
                if (removeWidget) {
                    removeWidget({
                        ids: [widget.instance.id]
                    });
                }
            }
        },
        selectWidget(e) {
            e.stopPropagation();
            let {
                widget,
                onSelected,
                mode
            } = this.props;
            if (mode === Udesk.enums.udeskReactCustomUiGridModeEnums.design.id) {
                if (onSelected) {
                    onSelected({
                        id: widget.instance.id
                    });
                }
            }
        }
    }

    render() { 
        let {
            computes
        } = this.privates;
        let {
            widget,
            mode,
            removeIconClass
        } = this.props;
        if (widget) {
            return (
                <div className={ClassNames("udesk-react-custom-ui-grid-widget-wrapper", { [`udesk-react-custom-ui-grid-widget-wrapper-${mode}`]: true, [`active`]: widget.instance.states.isActive })}>
                    <If condition={computes.enableShowMask}>
                        <div className="udesk-react-custom-ui-grid-widget-wrapper-mask" onClick={this.actions.selectWidget}></div>
                    </If>
                    <If condition={computes.enableOperate}>
                        <a className="widget-wrapper-delete" onClick={this.actions.removeWidget}>
                            <i className={removeIconClass}></i>
                        </a>
                    </If>
                    {this.props.children}
                </div>
            );
        } else { 
            return "";
        }
    }
}

export default Udesk.react.udeskify(WidgetWrapper);