import React, { useState, useCallback } from "react";
import { Button } from "udesk-ui";
import style from "./index.module.scss";
import DropDown, { Menu, Trigger } from "../drop-down";
import CustomerAutoComplete from "../customer-auto-complete";
import { useTranslation } from "react-i18next";
import { CareDown } from "../icon/icon";

interface IOnChange {
  (value: any): void;
}
interface IOnConfirm {
  (value: any): void;
}
interface IProps {
  onConfirm: IOnConfirm;
}

const CustomerDropDown: React.FC<any> = (props: IProps) => {
  let { onConfirm } = props;
  let { t } = useTranslation();
  let [visible, setVisible] = useState(false);
  let [selectCustomer, setSelectCustomer] = useState();

  let onChange = useCallback((value) => {
    setSelectCustomer(value);
  }, []);
  let onConfirmed = useCallback(() => {
    onConfirm && onConfirm(selectCustomer);
    setVisible(false);
  }, [onConfirm, selectCustomer]);
  let onCancel = useCallback(() => {
    setVisible(false);
  }, []);
  let onVisibleChange = useCallback((value) => {
    setVisible(value);
  }, []);

  return (
    <DropDown
      className={style["customer-drop-down"]}
      visible={visible}
      onVisibleChange={onVisibleChange}
    >
      <Trigger>
        {t("components.customerDropDown.customer")}
        <CareDown></CareDown>
      </Trigger>
      <Menu className={style["menu"]}>
        <Menu.Item>
          <div className={style["customer-select"]}>
            <CustomerAutoComplete
              value={selectCustomer}
              onChange={onChange}
            ></CustomerAutoComplete>
          </div>
          <div className={style["buttons"]}>
            <Button onClick={onConfirmed} type="primary" size="small">
              {t("confirm")}
            </Button>
            <Button onClick={onCancel} size="small">
              {t("cancel")}
            </Button>
          </div>
        </Menu.Item>
      </Menu>
    </DropDown>
  );
};

export default CustomerDropDown;
