import React from 'react';
import common from '../../udesk/index';
import Udesk from '../../udesk/index';

const DATE_TIME_PICKER_TYPES = {
    input: {
        id: "input",
        name: "input",
    },
    component: {
        id: "component",
        name: "component",
    }
};

const DATE_TIME_PICKER_FORMAT = {
    dateTimeFormat: "YYYY-MM-DD HH:ii:ss",
    dateTimeNoSecondFormat: "YYYY-MM-DD HH:ii",
    dateFormat: "YYYY-MM-DD",
    timeFormat: "HH:ii:ss",
    timeNoSecondFormat: "HH:ii"
};

class DateTimePicker extends React.Component {
    privates = {
        defaultOptions: {
            dateTimePickerTypes: DATE_TIME_PICKER_TYPES,
            dateTimePickerType: DATE_TIME_PICKER_TYPES.input.id,
            dateTimeType: "dateTime",
            startView: 'month',
            minView: 'hour',
            maxView: 'decade',
            format: 'YYYY-MM-DD hh:ii:ss',
            inputFormat: 'YYYY-MM-DD hh:mm:ss',
            serverFormat: 'YYYY-MM-DD hh:mm:ss',
        },
        dateTimePicker: null,
        dateTimePickerElement: null,
        value: getformatInitialDate(this, this.props.initialDate)
    }

    static computes = {
        start: ({ props, state, privates, locales }) => {
            let {
                value
            } = privates;
            let {
                inputFormat
            } = privates.defaultOptions;
            let momentStartDate = moment(value, common.config.date.momentDateTimeFormat);
            return momentStartDate.isValid() ? momentStartDate.format(inputFormat) : "";
        },
        glyphiconName: ["props.dateTimeType", ({ props, state, privates, locales }) => {
            let {
                dateTimeType,
                dateTimeIcon,
                dateIcon,
                timeIcon
            } = props;
            let glyphiconName = dateTimeIcon;
            if (dateTimeType === "dataTime") {
                glyphiconName = dateTimeIcon;
            } else if (dateTimeType === "date") {
                glyphiconName = dateIcon;
            } else if (dateTimeType === "time") {
                glyphiconName = timeIcon;
            } else {
                glyphiconName = dateTimeIcon;
            }
            return glyphiconName;
        }]
    }

    componentDidMount() {
        let options = this.getOptions();
        this.privates.dateTimePicker = $(this.privates.dateTimePickerElement);
        this.privates.dateTimePicker.datetimepicker(options);
        this.attachPickerEvents(this);
    }
    componentDidUpdate(prevProps, prevState, snapshot) {
        let {
            privates,
        } = this;

        if (privates.dirtyProps.initialDate) {
            let initialDateString = getformatInitialDate(this, this.props.initialDate);
            this.privates.dateTimePicker.data("datetimepicker").setInitialDate(initialDateString);
            this.privates.dateTimePicker.data("datetimepicker").update(initialDateString);

            this.privates.value = initialDateString;
            this.actions.update();
        }
    }
    componentWillUnmount() {
        let dateTimePicker = this.privates.dateTimePicker && this.privates.dateTimePicker.data('datetimepicker');
        if (dateTimePicker != null) {
            this.privates.dateTimePicker.off('show');
            this.privates.dateTimePicker.off('hide');
            if (typeof dateTimePicker.remove === "function") {
                dateTimePicker.remove();
            }
            this.privates.dateTimePicker = null;
        }
    }
    attachPickerEvents(that) {
        let {
            showHeaderDate,
            onChanged,
            serverFormat
        } = that.props;

        if (!showHeaderDate) {
            that.privates.dateTimePicker.off('show.date-time-picker-container').on('show.date-time-picker-container', function (e) {
                $(".datetimepicker").addClass("no-header-date");
            });

            that.privates.dateTimePicker.off('hide.date-time-picker-container').on('hide.date-time-picker-container', function (e) {
                $(".datetimepicker").removeClass("no-header-date");
            });
        }

        that.privates.dateTimePicker.on('changeDate.date-time-picker-container', (e) => {
            let date = e.date;
            let dateString;
            if (date != null) {
                date.setSeconds(0);
                that.privates.dateTimePicker.data("datetimepicker").setInitialDate(date);
                $(e.target).data("datetimepicker").update(date);
                dateString = moment(date).format(serverFormat);
            }
            if (typeof onChanged === "function") {
                that.props.onChanged(dateString);
            }
        });
    }
    getOptions() {
        let {
            dateTimePickerTypes,
            dateTimePickerType,
            dateTimeType,
            startView,
            minView,
            maxView,
            format,
            weekStart,
            startDate,
            endDate,
            daysOfWeekDisabled,
            autoclose,
            clearBtn,
            todayBtn,
            todayHighlight,
            keyboardNavigation,
            language,
            minuteStep,
            pickerPosition,
            viewSelect,
            showMeridian,
            zIndex,
            timezone,
            onRenderYear,
            onRenderMonth,
            onRenderDay,
            onRenderHour,
            onRenderMinute,
            fontAwesome,
            forceParse,
            bootcssVer,
            includedSeconds,
            inputFormat,
            serverFormat
        } = this.props;

        let {
            value
        } = this.privates;

        let newStartView = "month";
        let newMinView = "hour";
        let newMaxView = "decade";
        let newViewSelect = "hour";
        let newFormat = DATE_TIME_PICKER_FORMAT.dateTimeFormat;

        let newInputFormat = common.config.date.momentDateTimeFormat;
        let newServerFormat = common.config.date.momentDateTimeFormat;

        let newDateTimePickerTypes = dateTimePickerTypes || DATE_TIME_PICKER_TYPES;
        let newDateTimePickerType = dateTimePickerType || DATE_TIME_PICKER_TYPES.input.id;
        /* eslint-disable */
        switch (dateTimeType) {
            case "dateTime":
                newStartView = (startView || startView === 0) ? startView : "month";
                newMinView = (minView || minView === 0) ? minView : "hour";
                newMaxView = (maxView || maxView === 0) ? maxView : "decade";
                newViewSelect = (viewSelect || viewSelect === 0) ? viewSelect : "hour";
                if (format) {
                    newFormat = format;
                } else {
                    newFormat = includedSeconds ? DATE_TIME_PICKER_FORMAT.dateTimeFormat : DATE_TIME_PICKER_FORMAT.dateTimeNoSecondFormat;
                }
                if (inputFormat) {
                    newInputFormat = inputFormat;
                } else {
                    newInputFormat = includedSeconds ? common.config.date.momentDateTimeFormat : common.config.date.momentDateTimeNoSecondFormat;
                }
                if (serverFormat) {
                    newServerFormat = serverFormat;
                } else {
                    newInpnewServerFormatutFormat = includedSeconds ? common.config.date.momentDateTimeFormat : common.config.date.momentDateTimeNoSecondFormat;
                }
                break;
            case "date":
                newStartView = (minView || minView === 0) ? minView : "month";
                newMinView = (minView || minView === 0) ? minView : "month";
                newMaxView = (maxView || maxView === 0) ? maxView : "decade";
                if (format) {
                    newFormat = format;
                } else {
                    newFormat = DATE_TIME_PICKER_FORMAT.dateFormat;
                }
                if (inputFormat) {
                    newInputFormat = inputFormat;
                } else {
                    newInputFormat = common.config.date.momentDateFormat;
                }
                if (serverFormat) {
                    newServerFormat = serverFormat;
                } else {
                    newInpnewServerFormatutFormat = common.config.date.momentDateFormat;
                }
                break;
            case "time":
                newStartView = (minView || minView === 0) ? minView : "day";
                newMinView = (minView || minView === 0) ? minView : "hour";
                newMaxView = (maxView || maxView === 0) ? maxView : "day";
                if (format) {
                    newFormat = format;
                } else {
                    newFormat = includedSeconds ? DATE_TIME_PICKER_FORMAT.timeFormat : DATE_TIME_PICKER_FORMAT.timeNoSecondFormat;
                }
                if (inputFormat) {
                    newInputFormat = inputFormat;
                } else {
                    newInputFormat = includedSeconds ? common.config.date.momentTimeFormat : common.config.date.momentTimeNoSecondFormat;
                }
                if (serverFormat) {
                    newServerFormat = serverFormat;
                } else {
                    newServerFormat = includedSeconds ? common.config.date.momentTimeFormat : common.config.date.momentTimeNoSecondFormat;
                }
                break;
        }
        /* eslint-enable */

        this.privates.defaultOptions = {
            dateTimePickerTypes: newDateTimePickerTypes,
            dateTimePickerType: newDateTimePickerType,
            startView: newStartView,
            minView: newMinView,
            maxView: newMaxView,
            format: newFormat,
            inputFormat: newInputFormat,
            serverFormat: newServerFormat
        };

        return {
            container: "",
            dateTimePickerTypes: newDateTimePickerTypes,
            dateTimePickerType: newDateTimePickerType,
            dateTimeType,
            startView: newStartView,
            minView: newMinView,
            maxView: newMaxView,
            format: newFormat,
            initialDate: value,
            weekStart,
            startDate,
            endDate,
            daysOfWeekDisabled,
            autoclose,
            clearBtn,
            todayBtn,
            todayHighlight,
            keyboardNavigation,
            language,
            minuteStep,
            pickerPosition,
            viewSelect: newViewSelect,
            showMeridian,
            zIndex,
            timezone,
            onRenderYear,
            onRenderMonth,
            onRenderDay,
            onRenderHour,
            onRenderMinute,
            fontAwesome,
            forceParse,
            bootcssVer
        };
    }
    render() {
        let {
            classNames,
            initialDate,
            format,
            showComponentClearBtn,
        } = this.props;

        let {
            dateTimePickerType,
        } = this.privates.defaultOptions;

        let {
            privates: { computes },
        } = this;

        let start = computes.start;
        let glyphiconName = computes.glyphiconName;

        return (
            <div className={`date-time-picker-container ${classNames}`}>
                <Choose>
                    <When condition={(dateTimePickerType === DATE_TIME_PICKER_TYPES.input.id)}>
                        <input ref={(dateTimePickerElement) => this.privates.dateTimePickerElement = dateTimePickerElement} type="text" value={start} className="form-datetime input-date-time-picker" readOnly />
                    </When>
                    <When condition={(dateTimePickerType === DATE_TIME_PICKER_TYPES.component.id)}>
                        <div className="form-group">
                            <div className="input-group date form-datetime col-md-3" data-date={start} data-date-format={format} ref={(dateTimePickerElement) => this.privates.dateTimePickerElement = dateTimePickerElement}>
                                <input type="hidden" value={start} readOnly />
                                <input type="text" value={initialDate} className="form-control" readOnly />
                                <If condition={showComponentClearBtn}>
                                    <span className="input-group-addon">
                                        <span className="glyphicon glyphicon-remove"></span>
                                    </span>
                                </If>
                                <span className="input-group-addon">
                                    <span className={`glyphicon ${glyphiconName}`}></span>
                                </span>
                            </div>
                        </div>
                    </When>
                </Choose>
            </div>
        );
    }
}

DateTimePicker.defaultProps = {
    classNames: ["udesk-component-date-time-picker"],
    showComponentClearBtn: true,
    dateTimePickerTypes: null,
    dateTimePickerType: null,
    dateTimeType: "dataTime", //"date"，同时设置startView=>s2,minView=>2;"time",同时设置startVIew=>1,minVIew=>0,maxView=>1
    startView: '', //Number, String类型：日期时间选择器在打开时显示的视图，0-hour,1-day,2-month,3-year,4-decade
    minView: '', //Number, String类型：最小试图，同上
    maxView: '', //Number, String类型：最大试图，同上
    format: 'YYYY-MM-DD hh:ii:ss', //String类型：日期格式   yyyy-mm-dd hh:mm:ss
    initialDate: "", //Date,String类型：可以初始化日期查看器，默认现在
    weekStart: 0, //Number类型：一周从哪一天开始，0-6，表示周日到周六
    startDate: "", //Date类型：开始时间，在此之前的时间会被禁用
    endDate: "", //Date类型：结束时间，在此之后的时间会被禁用
    daysOfWeekDisabled: [], //String, Array类型：禁用星期几，比如禁用周日，周六‘0,6’，[0,6]
    autoclose: true, //Boolean类型：自动关闭，当选择一个日期之后是否立即关闭此日期时间选择器
    clearBtn: false, //Boolean类型：显示清除按钮
    todayBtn: false, //Boolean类型：显示今天按钮
    todayHighlight: false, //Boolean类型：突出显示当前日期
    keyboardNavigation: true, //Boolean类型：是否允许通过键盘上的箭头键导航日期
    language: 'zh-CN', //String类型：语言
    minuteStep: 5, //Number类型：用于构建小时视图的增量。每minuteStep分钟创建一个按钮。
    pickerPosition: 'bottom-right',//String类型：'bottom-right', 'bottom-left', 'top-right', 'top-left'
    viewSelect: '', //Number, String类型：使用此选项，您可以选择从中选择日期的视图 decade', 'year', 'month', 'day', 'hour')
    showMeridian: false, //Boolean类型：此选项将启用子午视图以查看日期和时段
    zIndex: undefined, //Number类型：zIndex值是根据DOM树自动计算的，我们寻求最高的值，要跳过此过程，可以手动设置值
    timezone: null, //时区
    onRenderYear: null, //渲染年时出发此事件
    onRenderMonth: null, //渲染月时出发此事件
    onRenderDay: null, //渲染日时出发此事件
    onRenderHour: null, //渲染时时出发此事件
    onRenderMinute: null, //渲染分时出发此事件
    fontAwesome: false, //Boolean类型：为真使用Font Awesome
    forceParse: true, //Boolean类型：是否在选择器关闭时强制解析不合法的输入值
    bootcssVer: 3, //bootstrap css version

    dateTimeIcon: "bi-iconfont icon-bi-calendar",
    dateIcon: "bi-iconfont icon-bi-calendar",
    timeIcon: "bi-iconfont icon-bi-time",//暂不支持
    showHeaderDate: false,
    includedSeconds: true,
    inputFormat: 'YYYY-MM-DD hh:mm:ss',
    serverFormat: 'YYYY-MM-DD hh:mm:ss',
    onChanged: null,
};

function getformatInitialDate(that, initialDate) {
    let {
        dateTimeType
    } = that.props;

    let statesInitialDate;
    if (initialDate != null && initialDate != "" && dateTimeType === "time") {
        let today = new Date();
        let todayString = moment(today, common.config.date.momentDateFormat).format(common.config.date.momentDateFormat);
        statesInitialDate = todayString + " " + initialDate;
    } else {
        statesInitialDate = initialDate;
    }

    let initialDateString;
    if (statesInitialDate) {
        let date = new Date(statesInitialDate);
        date.setSeconds(0);
        initialDateString = moment(date, common.config.date.momentDateTimeFormat).format(common.config.date.momentDateTimeFormat);
    }
    return initialDateString;
}
export default Udesk.react.udeskify(DateTimePicker);