import React from 'react';
import PropTypes from 'prop-types';
import Udesk from '../../udesk/index';
import DateTimePicker from '../date-time-picker';

class DateTimeRangePicker extends React.Component {
    //#region
    static defaultProps = {
        classNames: "date-time-range-picker",
        titleClassNames: "date-time-range-picker-title",
        classNameItems: "date-time-picker-item",
        title: "",
        showTitle: true,
        separatorText: "—",

        showComponentClearBtn: true,
        dateTimePickerTypes: null,
        dateTimePickerType: null,
        dateTimeType: "dataTime",
        startView: '',
        minView: '',
        maxView: '',
        format: 'YYYY-MM-DD hh:ii:ss',
        startInitialDate: "",
        endInitialDate: "",
        weekStart: 0,
        startDate: "",
        endDate: "",
        daysOfWeekDisabled: "",
        autoclose: true,
        clearBtn: false,
        todayBtn: false,
        todayHighlight: false,
        keyboardNavigation: true,
        language: 'zh-CN',
        minuteStep: 5,
        pickerPosition: 'bottom-right',
        viewSelect: '',
        showMeridian: false,
        zIndex: undefined,
        timezone: null,
        onRenderYear: null,
        onRenderMonth: null,
        onRenderDay: null,
        onRenderHour: null,
        onRenderMinute: null,
        fontAwesome: false,
        forceParse: true,
        bootcssVer: 3,

        dateTimeIcon: "bi-iconfont icon-bi-calendar",
        dateIcon: "bi-iconfont icon-bi-calendar",
        timeIcon: "bi-iconfont icon-bi-time",//暂不支持
        showHeaderDate: false,
        includedSeconds: true,
        inputFormat: 'YYYY-MM-DD hh:mm:ss',
        serverFormat: 'YYYY-MM-DD hh:mm:ss',
        onChanged: null,
    }
    static propTypes = {
        classNames: PropTypes.string,
        titleClassNames: PropTypes.string,
        classNameItems: PropTypes.string,
        title: PropTypes.string,
        showTitle: PropTypes.bool,
        separatorText: PropTypes.string,
        showComponentClearBtn: PropTypes.bool,
        dateTimeType: PropTypes.string,
        startView: PropTypes.string,
        minView: PropTypes.string,
        maxView: PropTypes.string,
        format: PropTypes.string,
        startInitialDate: PropTypes.string,
        endInitialDate: PropTypes.string,
        weekStart: PropTypes.number,
        startDate: PropTypes.string,
        endDate: PropTypes.string,
        daysOfWeekDisabled: PropTypes.string,
        autoclose: PropTypes.bool,
        clearBtn: PropTypes.bool,
        todayBtn: PropTypes.bool,
        todayHighlight: PropTypes.bool,
        keyboardNavigation: PropTypes.bool,
        language: PropTypes.string,
        minuteStep: PropTypes.number,
        pickerPosition: PropTypes.string,
        viewSelect: PropTypes.string,
        showMeridian: PropTypes.bool,
        zIndex: PropTypes.number,
        onRenderYear: PropTypes.func,
        onRenderMonth: PropTypes.func,
        onRenderDay: PropTypes.func,
        onRenderHour: PropTypes.func,
        onRenderMinute: PropTypes.func,
        fontAwesome: PropTypes.bool,
        forceParse: PropTypes.bool,
        bootcssVer: PropTypes.number,
        dateTimeIcon: PropTypes.string,
        dateIcon: PropTypes.string,
        timeIcon: PropTypes.string,
        showHeaderDate: PropTypes.bool,
        includedSeconds: PropTypes.bool,
        inputFormat: PropTypes.string,
        serverFormat: PropTypes.string,
        onChanged: PropTypes.func,
    }
    //#endregion

    privates = {
        title: "",
        startInitialDate: this.props.startInitialDate,
        endInitialDate: this.props.endInitialDate
    }
    init() {
        this.privates.title = this.props.title || this.locales.labels.time;
    }
    parseProps({ props, privates, isInitial }) {
        if (isInitial) {
            return;
        }
        let {
            startInitialDate,
            endInitialDate
        } = props;

        let {
            startInitialDate: startDate,
            endInitialDate: endDate
        } = privates;
        let changedDate = null;
        if (startInitialDate !== startDate) {
            changedDate = {};
            changedDate.startInitialDate = startInitialDate;
        }
        if (endInitialDate !== endDate) {
            if (changedDate == null) {
                changedDate = {};
            }
            changedDate.endInitialDate = endInitialDate;
        }
        return changedDate;
    }

    actions = {
        startDateChanged(date) {
            if (date === this.privates.startInitialDate) {
                return;
            }
            let {
                serverFormat,
            } = this.props;

            let endInitialDate = this.privates.endInitialDate;
            if (validDateTime(date, endInitialDate, serverFormat)) {
                this.privates.startInitialDate = date;
                this.trigger("onChanged", date, endInitialDate);
            } else {
                this.privates.startInitialDate = endInitialDate;
                this.privates.endInitialDate = date;
                this.trigger("onChanged", endInitialDate, date);
            }
            this.actions.update();
        },
        endDateChanged(date) {
            if (date === this.state.endInitialDate) {
                return;
            }
            let {
                serverFormat
            } = this.props;
            let startInitialDate = this.privates.startInitialDate;

            if (validDateTime(startInitialDate, date, serverFormat)) {
                this.trigger("onChanged", startInitialDate, date);
                this.privates.endInitialDate = date;
            } else {
                this.trigger("onChanged", date, startInitialDate);
                this.privates.startInitialDate = date;
                this.privates.endInitialDate = startInitialDate;
            }
            this.actions.update();
        }
    }
    render() {
        let {
            classNames,
            classNameItems,
            titleClassNames,
            showTitle,
            separatorText,
            ...other
        } = this.props;

        let {
            title,
            startInitialDate,
            endInitialDate
        } = this.privates;

        return (
            <div className={`${classNames}`}>
                <If condition={showTitle}>
                    <span className={`${titleClassNames}`}>
                        {title}
                    </span>
                </If>
                <DateTimePicker {...other} classNames={classNameItems} initialDate={startInitialDate} onChanged={this.actions.startDateChanged} />
                <span className="separator-mark">
                    {separatorText}
                </span>
                <DateTimePicker {...other} classNames={classNameItems} initialDate={endInitialDate} onChanged={this.actions.endDateChanged} />
            </div>
        );
    }
}

function validDateTime(start, end, serverFormat) {
    let startMoment = moment(start, serverFormat);
    let endMoment = moment(end, serverFormat);
    let isValidDateTime = (startMoment.isValid() && endMoment.isValid());

    if (isValidDateTime && endMoment.isAfter(startMoment, "seconds")) {
        return true;
    } else {
        return false;
    }
}

export default Udesk.react.udeskify(DateTimeRangePicker);