import React from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import Locales from '../../udesk/locales';
import Sortable from 'sortablejs';
import UdeskReact from '../../udesk/react';

class DragList extends React.Component {
    constructor(props) {
        super(props);
        let list = combiningList(this.props.list, this);
        this.state = {
            list,
            hasEditingItem: false,
        };
        this.dragListItemkey = "drag-list-item";
        this.selectedListItem = list[0];
    }

    componentDidMount() {
        let el = document.getElementById('component-udesk-react-drag-list');
        let {
            sort,
            delay,
            touchStartThreshold,
            disabled,
            animation,
            handle,
            filter,
            draggable,
            ghostClass,
            chosenClass,
            dragClass,
            dataIdAttr,
        } = this.props;

        let that = this;

        new Sortable(el, {
            sort,
            delay,
            touchStartThreshold,
            disabled,
            animation,
            handle,
            filter,
            draggable,
            ghostClass,
            chosenClass,
            dragClass,
            dataIdAttr,
            onEnd: function (evt) {
                let {
                    list
                } = that.state;
                let oldIndex = evt.oldIndex;
                let newIndex = evt.newIndex;
                let sortingItem = list.splice(oldIndex, 1);
                list.splice(newIndex, 0, ...sortingItem);
                that.setState({ list: list.concat([]) });
                that.actions.sorted(list);
            }
        });
    }

    shouldComponentUpdate(nextProps, nextState) {
        if (nextProps.list.length !== nextState.list.length) {
            let list = combiningList(nextProps.list, this);
            let hasEditingItem = list.some((listItem) => {
                return listItem.states.isEditing;
            });
            if (list.length === 1) {
                this.actions.selectedItem(list[0]);
            }
            this.setState({
                list,
                hasEditingItem
            });
        }
        if (nextProps.list !== this.props.list) {
            this.dragListItemkey = `drag-list-item-${Math.ceil(Math.random() * 1000)}`;
        }
        return true;
    }

    render() {
        let {
            listItemTextFieldName,
            title,
            dragListClassNames,
            addDragListIcon,
            editDragListIcon,
            deleteDragListIcon,
            confirmIcon,
            cancelIcon,
            draggable,
            handle,
            dragIcon,
            languageCode,
        } = this.props;
        let {
            list,
            hasEditingItem,
        } = this.state;
        return (
            <div className={`component-udesk-react-drag-list ${dragListClassNames}`}>
                <div className="drag-list-header clearfix">
                    <span className="title">
                        {title ? title : Locales.get("components.dragList.title", languageCode)}
                    </span>
                    <div className="action-list pull-right">
                        <i className={ClassNames('add', addDragListIcon, { hasEditingItem: hasEditingItem })} onClick={this.actions.addItem}></i>
                    </div>
                </div>
                <ul id="component-udesk-react-drag-list" className="drag-list-content">
                    <For each="dragListItem" index="index" of={list}>
                        <li key={`${this.dragListItemkey}-${index}`} className={ClassNames('drag-list-item', draggable.slice(1), { "is-selected": dragListItem.states.isSelected })}>
                            <Choose>
                                <When condition={dragListItem.states.isEditing}>
                                    <div className="drag-list-item-content clearfix">
                                        <input type="text" className="text-edit form-control" value={dragListItem.states.data[listItemTextFieldName]} onChange={this.actions.itemTextChange.params(dragListItem)} />
                                        <div className="action-list-editing pull-right">
                                            <i className={`action confirm ${confirmIcon}`} onClick={this.actions.confirmItemModify.params(dragListItem)}></i>
                                            <i className={`action cance ${cancelIcon}`} onClick={this.actions.canceItemModify.params(dragListItem, index)}></i>
                                        </div>
                                    </div>
                                </When>
                                <Otherwise>
                                    <div className="drag-list-item-content clearfix" onClick={this.actions.selectedItem.params(dragListItem)}>
                                        <i className={ClassNames('drag-icon', handle.slice(1), dragIcon, { hasEditingItem: hasEditingItem }, 'pull-left')}></i>
                                        <span className="text">
                                            {dragListItem.data[listItemTextFieldName]}
                                        </span>
                                        <div className={ClassNames('action-list', 'pull-right', { hasEditingItem: hasEditingItem })}>
                                            <i className={`action edit ${editDragListIcon}`} onClick={this.actions.editItem.params(dragListItem)}></i>
                                            <i className={`action delete ${deleteDragListIcon}`} onClick={this.actions.deleteItem.params(index)}></i>
                                        </div>
                                    </div>
                                </Otherwise>
                            </Choose>
                        </li>
                    </For>
                </ul>
            </div>

        );
    }

    actions = {
        addItem() {
            if (this.state.hasEditingItem) {
                return;
            }
            let {
                addItem
            } = this.props;
            if (typeof addItem === "function") {
                addItem();
            }
        },
        editItem(dragListItem, event) {
            event.stopPropagation();
            dragListItem.states.isEditing = true;
            this.setState({
                list: this.state.list,
                hasEditingItem: true,
            });
        },
        itemTextChange(dragListItem, event) {
            let {
                listItemTextFieldName,
            } = this.props;
            dragListItem.states.data[listItemTextFieldName] = event.target.value;
            this.setState({ list: this.state.list });
        },
        confirmItemModify(dragListItem) {
            let {
                listItemTextFieldName,
                confirmItemModify,
            } = this.props;
            let editedItemText = dragListItem.states.data[listItemTextFieldName];
            if (editedItemText && editedItemText.trim()) {
                dragListItem.data[listItemTextFieldName] = dragListItem.states.data[listItemTextFieldName];
                dragListItem.states.isEditing = false;
                if (typeof confirmItemModify === "function") {
                    confirmItemModify(dragListItem.data);
                }
                this.setState({
                    list: this.state.list,
                    hasEditingItem: false,
                });
            }
        },
        canceItemModify(dragListItem, index, event) {
            let {
                listItemTextFieldName,
            } = this.props;
            if (dragListItem.data[listItemTextFieldName] && dragListItem.data[listItemTextFieldName].trim()) {
                dragListItem.states.data[listItemTextFieldName] = dragListItem.data[listItemTextFieldName];
                dragListItem.states.isEditing = false;
                this.setState({
                    list: this.state.list,
                    hasEditingItem: false,
                });
            } else {
                this.deleteItem(index, event);
            }
        },
        deleteItem(index, event) {
            event.stopPropagation();
            let {
                deleteItem
            } = this.props;
            if (typeof deleteItem === "function") {
                deleteItem(index);
            }
        },
        selectedItem(dragListItem) {
            let {
                selectedItem
            } = this.props;
            let {
                list
            } = this.state;
            this.selectedListItem = dragListItem;
            list = list.map((listItem) => {
                if (listItem.states.isSelected) {
                    listItem.states.isSelected = false;
                }
                return listItem;
            });
            dragListItem.states.isSelected = true;
            this.setState({
                //list: this.state.list,
                list,
            });
            if (typeof selectedItem === "function") {
                selectedItem(dragListItem.data);
            }
        },
        sorted(list) {
            let {
                sorted
            } = this.props;
            list = list.map((listItem) => {
                return listItem.data;
            });
            if (typeof sorted === "function") {
                sorted(list);
            }
        }
    }
}

function combiningList(list, that) {
    let {
        selectedItem,
        listItemIdFieldName,
    } = that.props;

    let selectedListItem = that.selectedListItem;
    let selectedListItemIndex = -1;
    if (selectedListItem) {
        selectedListItemIndex = list.findIndex((listItem) => {
            return listItem === selectedListItem.data;
        });
    }


    if (list.length === 0) {
        if (typeof selectedItem === "function") {
            selectedItem(null);
        }
    }
    list = list.map((listItem, index) => {
        if (index === 0) {
            if (selectedListItemIndex === -1) {
                if (typeof selectedItem === "function") {
                    selectedItem(listItem);
                }
                return {
                    data: listItem,
                    states: {
                        isEditing: listItem[listItemIdFieldName] ? false : true,
                        isSelected: true,
                        data: Object.assign({}, listItem)
                    }
                };
            } else {
                return {
                    data: listItem,
                    states: {
                        isEditing: listItem[listItemIdFieldName] ? false : true,
                        isSelected: false,
                        data: Object.assign({}, listItem)
                    }
                };
            }
        }
        if (selectedListItemIndex === index) {
            if (typeof selectedItem === "function") {
                selectedItem(selectedListItem.data);
            }
            return selectedListItem;
        }
        return {
            data: listItem,
            states: {
                isEditing: listItem[listItemIdFieldName] ? false : true,
                isSelected: false,
                data: Object.assign({}, listItem)
            }
        };
    });
    return list;
}

DragList.propTypes = {
    list: PropTypes.array,
    listItemIdFieldName: PropTypes.string,
    listItemTextFieldName: PropTypes.string,
    title: PropTypes.string,
    sort: PropTypes.bool,
    disabled: PropTypes.bool,
    animation: PropTypes.number,
    handle: PropTypes.string,
    filter: PropTypes.string,
    draggable: PropTypes.string,
    ghostClass: PropTypes.string,
    chosenClass: PropTypes.string,
    dragClass: PropTypes.string,
    dataIdAttr: PropTypes.string,
    dragListClassNames: PropTypes.string,
    addDragListIcon: PropTypes.string,
    editDragListIcon: PropTypes.string,
    deleteDragListIcon: PropTypes.string,
    confirmIcon: PropTypes.string,
    cancelIcon: PropTypes.string,
    addItem: PropTypes.func,
    editItem: PropTypes.func,
    deleteItem: PropTypes.func,
    selectedItem: PropTypes.func,
    sorted: PropTypes.func,
    confirmItemModify: PropTypes.func,
    dragIcon: PropTypes.string,
};

DragList.defaultProps = {
    list: [],
    listItemIdFieldName: "id",
    listItemTextFieldName: "name",
    title: '',
    sort: true,
    delay: 0,
    touchStartThreshold: 0,
    disabled: false,
    animation: 150,
    handle: ".handle",
    filter: ".ignore-elements",
    draggable: ".draggable",
    ghostClass: "sortable-ghost",
    chosenClass: "sortable-chosen",
    dragClass: "sortable-drag",
    dataIdAttr: 'data-id',
    addDragListIcon: "udesk-qc-iconfont icon-udesk-qc-add",
    editDragListIcon: "udesk-qc-iconfont icon-udesk-qc-edit",
    deleteDragListIcon: "udesk-qc-iconfont icon-udesk-qc-delete",
    confirmIcon: "udesk-qc-iconfont icon-udesk-qc-queren",
    cancelIcon: "udesk-qc-iconfont icon-udesk-qc-quxiao",
    dragIcon: "udesk-qc-iconfont icon-udesk-qc-drag-and-drop",
};

export default UdeskReact.udeskify(DragList);