import React from 'react';
import Udesk from '../../udesk/index';
import ClassNames from 'classnames';
import PropTypes from 'prop-types';
import SortableJS from 'sortablejs';
import Locales from "../../udesk/locales";

class DragTable extends React.Component {
    static propTypes = {
        items: PropTypes.array,
        isDraggable: PropTypes.bool,
        columns: PropTypes.array,
        sortIcon: PropTypes.string,
        drapDirection: PropTypes.string,
        classNames: PropTypes.string,
        rowClick: PropTypes.func,
        onChanged: PropTypes.func,
        showNum: PropTypes.bool,
        isHeaderDragIconShow: PropTypes.bool,
        isEnableTableHeaderInfo: PropTypes.bool,
        dragIconColumnWidth: PropTypes.string
    }

    static defaultProps = {
        items: [],
        isDraggable: false,
        columns: [],
        sortIcon: "udesk-react-iconfont icon-udesk-react-tuodong",
        drapDirection: "y",
        classNames: "table",
        rowClick: null,
        onChanged: null,
        showNum: false,
        isHeaderDragIconShow: true,
        isEnableTableHeaderInfo: true,
        dragIconColumnWidth: "5%"
    }

    privates = {
        sortableInstance: null,
        dragTableElement: null,
        dragTableItemkey: 0
    }

    static computes = {
        _visibleColumns: ({ props, state, privates, locales }) => {
            let columns = props.columns;
            return columns.filter(c => c.visible !== false);
        },
    }

    componentDidMount() {
        if (this.privates.dragTableElement) {
            this.privates.sortableInstance = SortableJS.create(this.privates.dragTableElement, {
                animation: 150,
                handle: ".drag-table-handle",
                onSort: this.actions.sort
            });
        }
    }

    componentWillUnmount() {
        if (this.privates.sortableInstance) {
            this.privates.sortableInstance.destroy();
            this.privates.sortableInstance = null;
        }
    }

    actions = {
        _getColInnerStyle: function (column) {
            return {
                width: column.width
            };
        },
        _getDragIconColInnerStyle: function (width) {
            return {
                width: width
            };
        },
        _isCellClickable: function (item, column) {
            return !!(column.click || this.props.rowClick);
        },
        _reorderItems: function (groupModel, itemModels, draggedModel) {
            this.props.onChanged(groupModel);
        },
        _clickCellLink: function (item, column) {
            if (column.click) {
                column.click.apply(column.clickContext, arguments);
            } else {
                this.props.rowClick(arguments);
            }
        },
        _getColDisplayName(item, col) {
            let colDisplayName = Udesk.utils.object.get(item, col.name);
            if (col.get != null && typeof col.get === "function") {
                colDisplayName = col.get(item, col);
            }
            if (col.format) {
                colDisplayName = col.format(colDisplayName);
            }
            return colDisplayName;
        },
        sort(evt) {
            let {
                newIndex,
                oldIndex
            } = evt;
            let items = this.props.items;
            let sortingItem = items.splice(oldIndex, 1);
            items.splice(newIndex, 0, ...sortingItem);
            this.privates.dragTableItemkey = Math.ceil(Math.random() * 10000);
            this.props.onChanged(items);
        }
    }

    render() {
        let {
            privates: { computes }
        } = this;
        const {
            classNames,
            sortIcon,
            items,
            isDraggable,
            showNum,
            isHeaderDragIconShow,
            isEnableTableHeaderInfo,
            dragIconColumnWidth
        } = this.props;
        return (
            <table className={ClassNames("react-drag-table", { [`${classNames}`]: classNames })}>
                <colgroup>
                    <col className="move-icon-col" style={this.actions._getDragIconColInnerStyle(dragIconColumnWidth)}></col>
                    <For each="fullColumn" index="index" of={computes._visibleColumns}>
                        <If condition={fullColumn.width && fullColumn.width !== 0}>
                            <col key={`full-column-${index}`} className="data-col" style={this.actions._getColInnerStyle(fullColumn)}></col>
                        </If>
                        <If condition={fullColumn.width == null || fullColumn.width === 0}>
                            <col key={`full-column-${index}`} className="data-col"></col>
                        </If>
                    </For>
                    <col className="drag-info-col"></col>
                </colgroup>
                <thead>
                    <tr>
                        <th className={ClassNames({ "list-move-icon": true, "hidden": isDraggable && isEnableTableHeaderInfo })}>
                            <If condition={isEnableTableHeaderInfo}>
                                <i className={ClassNames({ [`${sortIcon}`]: sortIcon, "hidden": isDraggable || !isHeaderDragIconShow })}></i>
                            </If>
                            <If condition={!isEnableTableHeaderInfo}>
                                <i className={ClassNames({ [`${sortIcon}`]: sortIcon, "hidden": !isHeaderDragIconShow })}></i>
                                <span className={ClassNames({ "hidden": isHeaderDragIconShow })}>{Locales.get("components.dragTable.order")}</span>
                            </If>
                        </th>
                        <For each="col" index="index" of={computes._visibleColumns}>
                            <th key={`col-${index}`} className={ClassNames({ [`${col.thClass}`]: col.thClass, "hidden": isDraggable && isEnableTableHeaderInfo })}>{col.caption}</th>
                        </For>
                        <If condition={isDraggable && isEnableTableHeaderInfo}>
                            <th className="drag-header-info" colSpan={computes._visibleColumns.length + 1}>
                                <span className="drag-info-text">{Locales.get("components.dragTable.drapHeaderInfo", this.props.language)}</span>
                            </th>
                        </If>
                    </tr>
                </thead>
                <tbody ref={el => this.privates.dragTableElement = el}>
                    <For each="item" index="index" of={items}>
                        <tr key={`item-${this.privates.dragTableItemkey}-${index}`} className="drag-table-item">
                            <td className="list-move-icon">
                                <If condition={isDraggable}>
                                    <div className="drag-table-handle">
                                        <i className={ClassNames({ [`${sortIcon}`]: sortIcon })}></i>
                                    </div>
                                </If>
                                <If condition={!isDraggable}>
                                    <If condition={showNum}>
                                        <div className="drag-table-num">
                                            {++index}
                                        </div>
                                    </If>
                                </If>
                            </td>
                            <For each="col" index="colIndex" of={computes._visibleColumns}>
                                <td key={`td-col-${colIndex}`} className={col.classNames} title={col.titleVisible ? (col.get ? col.get(item, col) : Udesk.utils.object.get(item, col.name)) : ""}>
                                    <If condition={col.hasTemplate}>
                                        <div className="handle">
                                            {col.getYieldContent(item, col, index)}
                                        </div>
                                    </If>
                                    <If condition={!col.hasTemplate}>
                                        <With colDisplayName={this.actions._getColDisplayName(item, col)}>
                                            <If condition={this.actions._isCellClickable(item, col)}>
                                                <a onClick={this.actions._clickCellLink.params(item, col).stopPropagation()}>
                                                    {colDisplayName}
                                                </a>
                                            </If>
                                            <If condition={!this.actions._isCellClickable(item, col)}>
                                                {colDisplayName}
                                            </If>
                                        </With>
                                    </If>
                                </td>
                            </For>
                        </tr>
                    </For>
                </tbody>
            </table>
        );
    }
}

export default Udesk.react.udeskify(DragTable);