import React from 'react';
import PropTypes from 'prop-types';
import gaodeMapLoaderService from '../../services/gaode-map-loader';

class ReactGaodeMapComponent extends React.Component {
    //#region
    static defaultProps = {
        classNames: ["udesk-gaode-map"],
        maxHeightStyle: null,

        onMapInitComplete: null
    }

    static propTypes = {
        classNames: PropTypes.string,
        maxHeightStyle: PropTypes.object,

        onMapInitComplete: PropTypes.func,
    }
    //#endregion


    constructor(props) {
        super(props);
        this.amapContainerRef = React.createRef();
    }

    privates = {
        mapApi: null,
        AMapContainerClass: null,
    }

    getMapApi() {
        return this.privates.mapApi;
    }

    componentDidMount() {
        if (window.AMap) {
            this.actions.initAMap();
        } else {
            if (gaodeMapLoaderService && typeof gaodeMapLoaderService.loadAsync === "function") {
                gaodeMapLoaderService.loadAsync(() => {
                    this.actions.initAMap();
                });
            } else {
                throw new Error(`gaodeMapLoaderService is null in 'udesk_react/src/components/gaode-map/component.js'`);
            }
        }
    }

    componentWillUnmount() {
        let {
            mapApi
        } = this.privates;

        if (mapApi) {
            mapApi.destroy();
        }
    }

    actions = {
        initAMap() {
            let AMapContainer = null;
            let amapContainerRef = this.amapContainerRef;

            if (amapContainerRef == null || amapContainerRef.current == null) {
                AMapContainer = this.$(".amap-container")[0];
            } else {
                AMapContainer = amapContainerRef.current;
            }

            let map = new AMap.Map(AMapContainer, {
                resizeEnable: true,
            });

            map.plugin(["AMap.ToolBar"], function () {
                map.addControl(new AMap.ToolBar());
            });

            this.privates.mapApi = map;
            this.trigger("onMapInitComplete");
        }
    }
}

export default ReactGaodeMapComponent;