import React from 'react';
import PropTypes from 'prop-types';
import 'cropper/dist/cropper.css';
import './style.scss';

export const cropperDefaults = {
    imageCropperClass: "image-cropper-container",
    previewClass: "preview",
    resultClass: "result",
    cropperContainer: ".row-container .image-cropper",
    aspectRatio: 16 / 9,
    crop: null,
    viewMode: 0,
    dragMode: "crop",
    responsive: true,
    center: true,
    checkCrossOrigin: true,
    background: true,
    modal: true,
    guides: true,
    highlight: true,
    autoCrop: true,
    autoCropArea: 0.8,
    dragDrop: true,
    movable: true,
    resizable: true,
    zoomable: true,
    zoomOnWheel: true,
    zoomOnTouch: true,
    cropBoxMovable: true,
    cropBoxResizable: true,
    toggleDragModeOnDblclick: true,
    rotateable: true,
    minContainerWidth: 200,
    minContainerHeight: 100,
    minCanvasWidth: 0,
    minCanvasHeight: 0,
    minCropBoxWidth: 0,
    minCropBoxHeight: 0,
    build: null,
    built: null,
    dragStart: null,
    dragMove: null,
    dragEnd: null,
    zoomin: null,
    zoomout: null
};
export default class ImageCropperComponent extends React.Component {

    static defaultProps = {
        imageContent: "",
        showCropper: true,
        initAutoCrop: false,
        showPreviewContainer: true,
        showResultContainer: true,
        imageCropperClass: cropperDefaults.imageCropperClass,
        resultClass: cropperDefaults.resultClass,
        previewClass: cropperDefaults.previewClass,
        cropperContainer: cropperDefaults.cropperContainer,
        aspectRatio: cropperDefaults.aspectRatio,
        crop: cropperDefaults.crop,
        viewMode: cropperDefaults.viewMode,
        dragMode: cropperDefaults.dragMode,
        responsive: cropperDefaults.responsive,
        center: cropperDefaults.center,
        checkCrossOrigin: cropperDefaults.checkCrossOrigin,
        background: cropperDefaults.background,
        modal: cropperDefaults.modal,
        guides: cropperDefaults.guides,
        highlight: cropperDefaults.highlight,
        autoCrop: cropperDefaults.autoCrop,
        autoCropArea: cropperDefaults.autoCropArea,
        dragDrop: cropperDefaults.dragDrop,
        movable: cropperDefaults.movable,
        resizable: cropperDefaults.resizable,
        zoomable: cropperDefaults.zoomable,
        zoomOnWheel: cropperDefaults.zoomOnWheel,
        zoomOnTouch: cropperDefaults.zoomOnTouch,
        cropBoxMovable: cropperDefaults.cropBoxMovable,
        cropBoxResizable: cropperDefaults.cropBoxResizable,
        toggleDragModeOnDblclick: cropperDefaults.toggleDragModeOnDblclick,
        rotateable: cropperDefaults.rotateable,
        minContainerWidth: cropperDefaults.minContainerWidth,
        minContainerHeight: cropperDefaults.minContainerHeight,
        minCanvasWidth: cropperDefaults.minCanvasWidth,
        minCanvasHeight: cropperDefaults.minCanvasHeight,
        minCropBoxWidth: cropperDefaults.minCropBoxWidth,
        minCropBoxHeight: cropperDefaults.minCropBoxHeight,
        build: cropperDefaults.build,
        built: cropperDefaults.built,
        dragStart: cropperDefaults.dragStart,
        dragMove: cropperDefaults.dragMove,
        dragEnd: cropperDefaults.dragEnd,
        zoomin: cropperDefaults.zoomin,
        zoomout: cropperDefaults.zoomout,
        onChanged: null
    }; //
    //#endregion

    //#region propTypes
    static propTypes = {
        imageContent: PropTypes.string,
        showCropper: PropTypes.bool,
        initAutoCrop: PropTypes.bool,
        showPreviewContainer: PropTypes.bool,
        showResultContainer: PropTypes.bool,
        imageCropperClass: PropTypes.string,
        resultClass: PropTypes.string,
        previewClass: PropTypes.string,
        cropperContainer: PropTypes.string,
        aspectRatio: PropTypes.number,
        crop: PropTypes.any,
        viewMode: PropTypes.number,
        dragMode: PropTypes.string,
        responsive: PropTypes.bool,
        center: PropTypes.bool,
        checkCrossOrigin: PropTypes.bool,
        background: PropTypes.bool,
        modal: PropTypes.bool,
        guides: PropTypes.bool,
        highlight: PropTypes.bool,
        autoCrop: PropTypes.bool,
        autoCropArea: PropTypes.number,
        dragDrop: PropTypes.bool,
        movable: PropTypes.bool,
        resizable: PropTypes.bool,
        zoomable: PropTypes.bool,
        zoomOnWheel: PropTypes.bool,
        zoomOnTouch: PropTypes.bool,
        cropBoxMovable: PropTypes.bool,
        cropBoxResizable: PropTypes.bool,
        toggleDragModeOnDblclick: PropTypes.bool,
        rotateable: PropTypes.bool,
        minContainerWidth: PropTypes.number,
        minContainerHeight: PropTypes.number,
        minCanvasWidth: PropTypes.number,
        minCanvasHeight: PropTypes.number,
        minCropBoxWidth: PropTypes.number,
        minCropBoxHeight: PropTypes.number,
        build: PropTypes.any,
        built: PropTypes.any,
        dragStart: PropTypes.any,
        dragMove: PropTypes.any,
        dragEnd: PropTypes.any,
        zoomin: PropTypes.any,
        zoomout: PropTypes.any,
        onChanged: PropTypes.func
    };
    constructor(props) {
        super(props);

    }

    privates = {
    }

    init() {
        this.ownContainer = React.createRef();
        this.imageCropper == React.createRef();
        this.privates.previewClassKey = Math.random().toString(36).substring(2, 6);

    }

    componentDidMount() {
        this.actions.initImageCropper();
    }

    componentDidUpdate(prevProps) {
        let {
            imageContent
        } = this.props;

        /* eslint-enable eqeqeq */
        if (imageContent != null && this.privates.dirtyProps.imageContent) {
            /* eslint-enable eqeqeq */
            this.actions.destoryImageCropper();
            setTimeout(() => {
                this.actions.initImageCropper();
                if (this.props.initAutoCrop) {
                    setTimeout(() => {
                        this.actions.crop();
                    }, 0);
                }
            });
        }
    }

    componentWillUnmount() {
        this.actions.destoryImageCropper();
    }

    actions = {
        crop() {
            let {
                cropperContainer
            } = this.props;

            if (this.ownContainer && this.ownContainer.current) {
                let ownContainerElement = this.ownContainer.current;
                let cropperElment = ownContainerElement.querySelector(cropperContainer);
                let $cropperElment = $(cropperElment);
                let croppedCanvas = $cropperElment.cropper("getCroppedCanvas");

                let resultElement = ownContainerElement.querySelector("." + this.props.resultClass);
                resultElement.innerHTML = '<img src="' + croppedCanvas.toDataURL() + '" class="result-image" alt="" />';
                this.trigger("onChanged", croppedCanvas);
            }
        },

        initImageCropper() {
            let {
                props,
                privates
            } = this;

            if (this.ownContainer == null) {
                return;
            }

            let ownContainerElement = this.ownContainer.current;
            let previewElement = ownContainerElement.querySelector("." + props.previewClass);
            let $previews = $(previewElement);

            let cropperElment = ownContainerElement.querySelector(props.cropperContainer);
            let $cropperElment = $(cropperElment);
            $cropperElment.cropper({
                //设置属性值
                aspectRatio: props.aspectRatio,
                // crop: this.get("crop"),
                preview: "." + props.previewClass + "-" + privates.previewClassKey,
                viewMode: props.viewMode,
                dragMode: props.dragMode,
                responsive: props.responsive,
                center: props.center,
                checkCrossOrigin: props.checkCrossOrigin,
                toggleDragModeOnDblclick: props.toggleDragModeOnDblclick,
                background: props.background,
                modal: props.modal,
                guides: props.guides,
                highlight: props.highlight,
                autoCrop: props.autoCrop,
                autoCropArea: props.autoCropArea,
                dragDrop: props.dragDrop,
                movable: props.movable,
                resizable: props.resizable,
                zoomable: props.zoomable,
                zoomOnWheel: props.zoomOnWheel,
                zoomOnTouch: props.zoomOnTouch,
                cropBoxMovable: props.cropBoxMovable,
                cropBoxResizable: props.cropBoxResizable,
                rotateable: props.rotateable,
                minContainerWidth: props.minContainerWidth,
                minContainerHeight: props.minContainerHeight,
                minCanvasWidth: props.minCanvasWidth,
                minCanvasHeight: props.minCanvasHeight,
                minCropBoxWidth: props.minCropBoxWidth,
                minCropBoxHeight: props.minCropBoxHeight,
                build: props.build,
                built: props.built,
                dragStart: props.dragStart,
                dragMove: props.dragMove,
                dragEnd: props.dragEnd,
                zoomin: props.zoomin,
                zoomout: props.zoomout,
                //设置方法
                ready: function (e) {
                    let $clone = $(this)
                        .clone()
                        .removeClass("cropper-hidden");
                    $clone.css({
                        display: "block",
                        width: "100%",
                        minWidth: 0,
                        minHeight: 0,
                        maxWidth: "none",
                        maxHeight: "none"
                    });
                    $previews.html($clone);
                },
                cropstart: function () { },
                crop: function (e) {
                    $previews.css({
                        width: "100%",
                        overflow: "hidden"
                    });
                    let imageData = $(this).cropper("getImageData");
                    let previewAspectRatio = e.width / e.height;
                    $previews.each(function () {
                        let $preview = $(this);
                        let previewWidth = $preview.width();
                        let previewHeight = previewWidth / previewAspectRatio;
                        let imageScaledRatio = e.width / previewWidth;
                        $preview
                            .height(previewHeight)
                            .find("img")
                            .css({
                                width: imageData.naturalWidth / imageScaledRatio,
                                height: imageData.naturalHeight / imageScaledRatio,
                                marginLeft: -e.x / imageScaledRatio,
                                marginTop: -e.y / imageScaledRatio
                            });
                    });
                }
            });
        },
        destoryImageCropper() {
            if (this.ownContainer && this.ownContainer.current) {
                let ownContainerElement = this.ownContainer.current;
                let cropperElment = ownContainerElement.querySelector(this.props.cropperContainer);
                let $cropperElment = $(cropperElment);
                if ($cropperElment != null && $cropperElment.data("cropper")) {
                    $cropperElment.cropper("destroy");
                }
            }
        }
    };
}