import React from 'react';
import Udesk from '../../udesk/index';
import PropTypes from 'prop-types';
import Locales from "../../udesk/locales";
import { cropperDefaults } from "../image-cropper/component";

let cropperDefaultsForImageUpload = {
    allowMultipleCrop: false
};
export default class ImageUploadComponent extends React.Component {

    static defaultProps = {
        maxFileSize: null,
        uploadTips: null,
        buttonText: null,
        acceptFileTypes: "image/jpeg,.jpg,.jpeg,image/png,.png,image/gif,.gif",
        cropperOptions: {},
        showCropper: true,
        initAutoCrop: false,
        showPreviewContainer: true,
        showResultContainer: true,
        alertCopperModal: false,
        disabled: false,
        onChanged: null,
        onInvalidFileUploaded: null
    };

    static propTypes = {
        maxFileSize: PropTypes.any,
        uploadTips: PropTypes.any,
        buttonText: PropTypes.any,
        acceptFileTypes: PropTypes.string,
        cropperOptions: PropTypes.object,
        showCropper: PropTypes.bool,
        initAutoCrop: PropTypes.bool,
        showPreviewContainer: PropTypes.bool,
        showResultContainer: PropTypes.bool,
        alertCopperModal: PropTypes.bool,
        disabled: PropTypes.bool,
        onChanged: PropTypes.func,
        onInvalidFileUploaded: PropTypes.func
    };

    state = {
        fileName: "",
        imageContent: "",
        showCropper: false
    };

    privates = {
        imageContent: null,
    }

    privates = {
        ownContainer: React.createRef()
    }

    static computes = {
        _cropperOptions: [
            "props.cropperOptions",
            function ({ props, state, privates, locales }) {
                return $.extend(
                    true,
                    {},
                    cropperDefaults,
                    cropperDefaultsForImageUpload,
                    props.cropperOptions
                );
            }
        ],
        _uploadTips: [
            "props.uploadTips",
            function ({ props, state, privates, locales }) {
                let uploadTips = props.uploadTips;
                let fileSize = props.maxFileSize;

                if (uploadTips) {
                    return uploadTips;
                } else {
                    let uploadTipsFormat = Locales.current.components.imageUpload.uploadTipsFormat;
                    if (fileSize) {
                        return Udesk.utils.string.format(uploadTipsFormat, fileSize);
                    } else {
                        return Locales.current.components.imageUpload.uploadTips;
                    }
                }
            }
        ],
        _buttonText: [
            "props.buttonText",
            function ({ props, state, privates, locales }) {
                let buttonText = props.buttonText;

                if (buttonText) {
                    return buttonText;
                } else {
                    return Locales.current.components.imageUpload.buttonText;
                }
            }
        ]
    };

    componentDidMount() {
        let self = this;
        let imageUploadElement = $(this.privates.ownContainer.current);
        let maxFileSize = this.props.maxFileSize;
        imageUploadElement.find('input[type="file"]').bind('change', function () {
            let file = this.files[0];
            if (file) {
                let fileName = new Date().getTime() + file.name;
                self.setState({
                    fileName: fileName
                });
                let fileSize = file.size / 1024;
                if (maxFileSize != null && fileSize > maxFileSize) {
                    let onInvalidFileUploaded = self.props.onInvalidFileUploaded;
                    if (onInvalidFileUploaded != null && onInvalidFileUploaded instanceof Array) {
                        self.trigger("onInvalidFileUploaded", fileSize);
                    } else {
                        Udesk.ui.notify.error(self.locales.components.imageUpload.uploadError);
                    }
                    return;
                }

                let reader = new FileReader();
                reader.readAsDataURL(file);
                reader.onloadend = function (e) {
                    let imageContent = e.target.result;
                    self.setState({
                        imageContent: imageContent,
                        fileName: fileName,
                        showCropper: true
                    });
                };
            }
        });
    }

    componentWillUnmount() {
        let imageUploadElement = $(this.privates.ownContainer.current);
        imageUploadElement.find('input[type="file"]').unbind('change');
    }

    actions = {
        getCroppedCanvas(croppedCanvas) {
            this.props.onChanged(croppedCanvas, this.state.fileName);
            let allowMultipleCrop = this.privates.computes._cropperOptions.allowMultipleCrop;
            if (!allowMultipleCrop) {
                this.setState({
                    imageContent: null,
                    showCropper: false
                });
            }
        }
    };
}
