import React from 'react';
import PropTypes from 'prop-types';

const TIME_NUMBER_LISTS = ["00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41", "42", "43", "44", "45", "46", "47", "48", "49", "50", "51", "52", "53", "54", "55", "56", "57", "58", "59"];
const MATCH_NUMBER_REG = /\d+/g;

export default class JeTimePickerComponent extends React.Component {
    static defaultProps = {
        value: null,
        disabled: false,
        classNames: "",
        isClearBtnVisible: true,
        isOkButtonVisible: true,
        closeIconClass: "udesk-react-iconfont icon-udesk-react-close",
        isCloseIconVisible: true,
        clearBtnClass: "",
        okBtnClass: "",
        format: "HH:mm:ss",
        serverFormat: "HH:mm:ss",
        minuteStep: 1,
        showSeconds: false,
        pickerPosition: "bottom-left",
        dropDownZindex: 1090,
        placeholder: "",
        hideDisabledOptions: false,

        disabledHours: null,
        disabledMinutes: null,
        disabledSeconds: null,
        onChanged: null,
    }

    static propTypes = {
        disabled: PropTypes.bool,
        classNames: PropTypes.string,
        isDropDownVisible: PropTypes.bool,
        isClearBtnVisible: PropTypes.bool,
        isOkButtonVisible: PropTypes.bool,
        closeIconClass: PropTypes.string,
        isCloseIconVisible: PropTypes.bool,
        clearBtnClass: PropTypes.string,
        okBtnClass: PropTypes.string,
        format: PropTypes.string,
        serverFormat: PropTypes.string,
        minuteStep: PropTypes.number,
        showSeconds: PropTypes.bool,
        pickerPosition: PropTypes.string,
        dropDownZindex: PropTypes.number,
        placeholder: PropTypes.string,
        hideDisabledOptions: PropTypes.bool,
        disabledHours: PropTypes.func,
        disabledMinutes: PropTypes.func,
        disabledSeconds: PropTypes.func,
        onChanged: PropTypes.func
    }

    privates = {
        isDropDownVisible: false
    }

    static computes = {
        hours: [({ props, state, privates, locales }) => {
            return TIME_NUMBER_LISTS.slice(0, 24);
        }],
        minutes: [({ props, state, privates, locales }) => {
            let minuteStep = props.minuteStep;
            let minutes = [];

            for (let i = 0; i <= TIME_NUMBER_LISTS.length - 1; i += minuteStep) {
                minutes.push(TIME_NUMBER_LISTS[i]);
            }

            return minutes;
        }],
        seconds: [({ props, state, privates, locales }) => {
            return TIME_NUMBER_LISTS;
        }],
        internaleValue: ["props.value", ({ props, state, privates, locales }) => {
            let {
                value,
                format,
                serverFormat
            } = props;
            if (value != null && value !== "") {
                return moment(value, serverFormat).format(format);
            }
            return "";
        }],
    }

    componentDidMount() {
        let elementId = Math.ceil(Math.random() * 1000000000);
        this.privates.elementId = elementId;
        let that = this;
        $(document).off(`click.react-component-je-time-picker-${elementId}`).on(`click.react-component-je-time-picker-${elementId}`, (e) => {
            let isDropDownVisible = that.privates.isDropDownVisible;
            let clickParents = $(e.target).parents(".react-component-je-time-picker");
            if (isDropDownVisible && (clickParents.length === 0 || that.privates.timePickerElement !== clickParents[0])) {
                if (!isDropDownVisible) {
                    that.actions.saveValues();
                }
                that.actions.toggleDropDown();
            }
        });
    }

    componentWillUnmount() {
        $(document).off(`click.react-component-je-time-picker-${this.privates.elementId}`);
    }

    actions = {
        isHourDisabled(hour) {
            let {
                disabledHours
            } = this.props;

            let disabledHourList = null;
            if (disabledHours && typeof disabledHours === "function") {
                disabledHourList = disabledHours();
            }

            if (disabledHourList && disabledHourList.length > 0) {
                return disabledHourList.includes(Number(hour));
            }
            return false;
        },
        isMinuteDisabled(minute) {
            let {
                disabledMinutes
            } = this.props;
            let {
                selectedHour
            } = this.privates;

            let disabledMinuteList = null;
            if (disabledMinutes && typeof disabledMinutes === "function") {
                disabledMinuteList = disabledMinutes(selectedHour);
            }

            if (disabledMinuteList && disabledMinuteList.length > 0) {
                return disabledMinuteList.includes(Number(minute));
            }
            return false;
        },
        isSecondDisabled(second) {
            let {
                disabledSeconds
            } = this.props;

            let {
                selectedHour,
                selectedMinute
            } = this.privates;

            let disabledSecondList = null;
            if (disabledSeconds && typeof disabledSeconds === "function") {
                disabledSecondList = disabledSeconds(selectedHour, selectedMinute);
            }

            if (disabledSecondList && disabledSecondList.length > 0) {
                return disabledSecondList.includes(Number(second));
            }
            return false;
        },
        openDropDown(e) {
            let {
                format,
                serverFormat
            } = this.props;

            let {
                internaleValue
            } = this.privates.computes;

            let newValue = moment(internaleValue, serverFormat).format(format);
            let newValueLists = newValue.match(MATCH_NUMBER_REG);

            if (newValueLists && newValueLists.length > 0) {
                this.privates.selectedHour = newValueLists[0];
                this.privates.selectedMinute = newValueLists[1];
                this.privates.selectedSecond = newValueLists[2] || "00";
            }
            this.privates.isDropDownVisible = true;
            this.actions.update();
            //this.actions.place();
        },
        toggleDropDown() {
            this.privates.isDropDownVisible = !this.privates.isDropDownVisible;
            this.actions.update();
        },
        hourSelected(hour, disabled) {
            if (disabled) {
                return;
            }
            let selectedHour = this.privates.selectedHour;
            if (hour !== selectedHour) {
                this.privates.selectedHour = hour;
                this.actions.update();
            }
        },
        minuteSelected(minute, disabled) {
            if (disabled) {
                return;
            }
            let selectedMinute = this.privates.selectedMinute;
            if (minute !== selectedMinute) {
                this.privates.selectedMinute = minute;
                this.actions.update();
            }
        },
        secondSelected(second, disabled) {
            if (disabled) {
                return;
            }
            let selectedSecond = this.privates.selectedSecond;
            if (second !== selectedSecond) {
                this.privates.selectedSecond = second;
                this.actions.update();
            }
        },
        clearValues() {
            this.privates.selectedHour = "00";
            this.privates.selectedMinute = "00";
            this.privates.selectedSecond = "00";
            this.actions.update();
        },
        saveValues() {
            let {
                selectedHour,
                selectedMinute,
                selectedSecond
            } = this.privates;
            let {
                serverFormat,
                showSeconds,
                onChanged
            } = this.props;
            if (selectedHour && selectedMinute && (!showSeconds || selectedSecond)) {
                let timeSelected = selectedHour + ":" + selectedMinute + ":" + selectedSecond;
                let serverValue = moment(timeSelected, serverFormat).format(serverFormat);
                if (onChanged != null) {
                    onChanged(serverValue);
                }
                this.actions.toggleDropDown();
            }
        },
        // place() {
        //     if (this.privates.timePickerElement != null) { 
        //         let {
        //             pickerPosition,
        //             dropDownZindex
        //         } = this.props;
        //         let componentContainer = $(this.privates.timePickerElement);
        //         let componentOffset = componentContainer.offset();

        //         let dropDownContainer = componentContainer.find(".time-select-container");

        //         let left = componentOffset.left;
        //         if (pickerPosition === 'bottom-left' || pickerPosition === 'top-left') {
        //             left += componentContainer.outerWidth() - dropDownContainer.outerWidth();
        //         }

        //         let bodyWidth = document.body.clientWidth || window.innerWidth;
        //         let bodyHeight = document.body.clientHeight || window.innerHeight;

        //         if (left + dropDownContainer.outerWidth() > bodyWidth) {
        //             left = bodyWidth - dropDownContainer.outerWidth();
        //         }

        //         let top = 0;
        //         if (pickerPosition === 'top-left' || pickerPosition === 'top-right') {
        //             top = componentOffset.top - dropDownContainer.outerHeight();
        //         } else {
        //             top = componentOffset.top + componentContainer.outerHeight();
        //             if (top + dropDownContainer.outerHeight() > bodyHeight) {
        //                 top = componentOffset.top - dropDownContainer.outerHeight();
        //             }
        //         }

        //         top = top - componentOffset.top;
        //         left = left - componentOffset.left;

        //         let dropDownPosition = {
        //             top: `${top}px`,
        //             left: `${left}px`,
        //             zIndex: `${dropDownZindex}`
        //         };
        //         this.privates.dropDownPosition = dropDownPosition;
        //         this.actions.update();
        //     }
        //}
    }
}