import React from 'react';
import JeTimePickerComponent from '../component';
import Locales from '../../../udesk/locales';
import ClassNames from 'classnames';

export default class JeTimePickerTemplate extends JeTimePickerComponent {
    render() {
        let { actions,
            privates
        } = this;

        let {
            computes,
            isDropDownVisible
        } = privates;

        let {
            disabled,
            classNames,
            isCloseIconVisible,
            closeIconClass,
            showSeconds,
            isClearBtnVisible,
            clearBtnClass,
            isOkButtonVisible,
            okBtnClass,
            placeholder,
            hideDisabledOptions
        } = this.props;

        return (
            <div
                className={ClassNames('react-component-je-time-picker', { [`${classNames}`]: classNames })}
                ref={element => (this.privates.timePickerElement = element)}>
                <div className="show-result" onClick={actions.openDropDown}>
                    <input
                        className="show-result-content"
                        type="text"
                        disabled={disabled}
                        value={computes.internaleValue}
                        placeholder={placeholder}
                        readOnly={true}
                        onChange={() => { }}
                    />
                    <i className="fa fa-clock" />
                </div>
                <If condition={isDropDownVisible && privates.isDropDownVisible}>
                    <div className="time-select-container" style={privates.dropDownPosition}>
                        <div className="time-select-header">
                            {Locales.current.components.jeTimePicker.headerTitle}
                            <If condition={isCloseIconVisible}>
                                <i className={`close-btn ${closeIconClass}`} onClick={actions.toggleDropDown} />
                            </If>
                        </div>
                        <ul className="time-select-main">
                            <li className="time-select-content time-select-hour">
                                <p className="time-select-text">{Locales.current.components.jeTimePicker.hour}</p>
                                <div className="time-select-values time-select-hour-values block-scrollbar-y">
                                    <ul className="time-select-values-content">
                                        <For of={computes.hours} each="hour" index="index">
                                            <With disabled={actions.isHourDisabled(hour)}>
                                                <If condition={!disabled || !hideDisabledOptions}>
                                                    <li
                                                        key={`time-select-hour-item-${index}`}
                                                        className={ClassNames('time-select-values-item', {
                                                            [`active`]: hour === privates.selectedHour,
                                                            [`disabled`]: disabled
                                                        })}
                                                        onClick={actions.hourSelected.params(hour, disabled).stopPropagation()}>
                                                        {hour}
                                                    </li>
                                                </If>
                                            </With>
                                        </For>
                                    </ul>
                                </div>
                            </li>
                            <li className="time-select-content time-select-minute">
                                <p className="time-select-text">{Locales.current.components.jeTimePicker.minute}</p>
                                <div className="time-select-values time-select-hour-values block-scrollbar-y">
                                    <ul className="time-select-values-content">
                                        <For of={computes.minutes} each="minute" index="index">
                                            <With disabled={actions.isMinuteDisabled(minute)}>
                                                <If condition={!disabled || !hideDisabledOptions}>
                                                    <li
                                                        key={`time-select-minute-item-${index}`}
                                                        className={ClassNames('time-select-values-item', {
                                                            [`active`]: minute === privates.selectedMinute,
                                                            [`disabled`]: disabled
                                                        })}
                                                        onClick={actions.minuteSelected.params(minute, disabled).stopPropagation()}>
                                                        {minute}
                                                    </li>
                                                </If>
                                            </With>
                                        </For>
                                    </ul>
                                </div>
                            </li>
                            <If condition={showSeconds}>
                                <li className="time-select-content time-select-second">
                                    <p className="time-select-text">{Locales.current.components.jeTimePicker.second}</p>
                                    <div className="time-select-values time-select-hour-values block-scrollbar-y">
                                        <ul className="time-select-values-content">
                                            <For of={computes.seconds} each="second" index="index">
                                                <With disabled={actions.isSecondDisabled(second)}>
                                                    <If condition={!disabled || !hideDisabledOptions}>
                                                        <li
                                                            key={`time-select-second-item-${index}`}
                                                            className={ClassNames('time-select-values-item', {
                                                                [`active`]: second === privates.selectedSecond,
                                                                [`disabled`]: disabled
                                                            })}
                                                            onClick={actions.secondSelected.params(second, disabled).stopPropagation()}>
                                                            {second}
                                                        </li>
                                                    </If>
                                                </With>
                                            </For>
                                        </ul>
                                    </div>
                                </li>
                            </If>
                        </ul>
                        <div className="time-select-footer">
                            <div className="show-time-select-values">
                                <input
                                    className="show-time-select-values-item"
                                    type="text"
                                    name=""
                                    value={privates.selectedHour}
                                    onChange={() => { }}
                                />
                                <span className="show-time-select-values-text">
                                    {Locales.current.components.jeTimePicker.colon}
                                </span>
                                <input
                                    className="show-time-select-values-item"
                                    type="text"
                                    name=""
                                    value={privates.selectedMinute}
                                    onChange={() => { }}
                                />
                                <If condition={showSeconds}>
                                    <span className="show-time-select-values-text">
                                        {Locales.current.components.jeTimePicker.colon}
                                    </span>
                                    <input
                                        className="show-time-select-values-item"
                                        type="text"
                                        name=""
                                        value={privates.selectedSecond}
                                        onChange={() => { }}
                                    />
                                </If>
                            </div>
                            <div className="btn-container">
                                <If condition={isClearBtnVisible}>
                                    <button
                                        className={`clear-btn ${clearBtnClass}`}
                                        type="button"
                                        onClick={actions.clearValues}>
                                        {Locales.current.components.jeTimePicker.clearButtonText}
                                    </button>
                                </If>
                                <If condition={isOkButtonVisible}>
                                    <button
                                        className={`save-btn ${okBtnClass}`}
                                        type="button"
                                        onClick={actions.saveValues}>
                                        {Locales.current.components.jeTimePicker.okButtonText}
                                    </button>
                                </If>
                            </div>
                        </div>
                    </div>
                </If >
            </div >
        );
    }
}
