import React from 'react';
import PropTypes from 'prop-types';
import Locales from "../../udesk/locales/index";

let DEFAULT_BUTTON_CLASS = "btn btn-default";

export default class LoadingButtonComponent extends React.Component {
    //#region defaultProps
    static defaultProps = {
        classNames: "",
        disabled: false,
        defaultText: Locales.current.components.loadingButton.okButtonText,
        loadingText: Locales.current.components.loadingButton.loadingText,
        successText: Locales.current.components.loadingButton.okButtonText,
        failText: Locales.current.components.loadingButton.okButtonText,
        defaultClass: DEFAULT_BUTTON_CLASS,
        loadingClass: DEFAULT_BUTTON_CLASS,
        successClass: DEFAULT_BUTTON_CLASS,
        failClass: DEFAULT_BUTTON_CLASS,
        name: "",

        onclick: null
    }

    static propTypes = {
        classNames: PropTypes.string,
        name: PropTypes.string,
        defaultText: PropTypes.string,
        loadingText: PropTypes.string,
        successText: PropTypes.string,
        failText: PropTypes.string,
        defaultClass: PropTypes.string,
        loadingClass: PropTypes.string,
        successClass: PropTypes.string,
        failClass: PropTypes.string,
        disabled: PropTypes.bool,
        onclick: PropTypes.func
    }
    //#endregion

    constructor(props) {
        super(props);
        this.state = {
            buttonText: props.defaultText || Locales.current.components.loadingButton.okButtonText,
            buttonClass: props.defaultClass || DEFAULT_BUTTON_CLASS,
            disabled: props.disabled
        };
    }

    startLoading() {
        let {
            loadingText,
            loadingClass
        } = this.props;
        this.setState({
            buttonText: loadingText,
            buttonClass: loadingClass,
            disabled: true
        });
    }

    success() {
        let {
            successText,
            defaultText,
            successClass,
            defaultClass
        } = this.props;

        this.setState({
            buttonText: successText || defaultText,
            buttonClass: successClass || defaultClass,
            disabled: false
        });
    }

    fail() {
        let {
            failText,
            defaultText,
            failClass,
            defaultClass
        } = this.props;

        this.setState({
            buttonText: failText || defaultText,
            buttonClass: failClass || defaultClass,
            disabled: false
        });
    }

    actions = {
        onClicked() {
            this.trigger("onclick", ...arguments);
        }
    }
}
