import React from 'react';
import PropTypes from 'prop-types';
import Input from 'antd/lib/input';
import 'antd/lib/input/style/index.css';
import Checkbox from 'antd/lib/checkbox';
import 'antd/lib/checkbox/style/index.css';
import UdeskReact from '../../../udesk/react';
import { getColumnKey } from '../utils/column-helper';

import './columns-setting.scss';
import locales from '../../../udesk/locales';

class ColumnsSetting extends React.Component {
    // #region defaultProps
    static defaultProps = {
        localColumns: []
    };
    // #endregion

    // #region defaultProps
    static propTypes = {
        localColumns: PropTypes.array
    };
    // #endregion

    privates = {
        searchText: ''
    };

    // #region computes
    static computes = {
        defaultValue: [
            'props.localColumns',
            ({ props, state, privates, locales }) => {
                return (
                    props.localColumns &&
                    props.localColumns.filter(column => column.visible).map(column => getColumnKey(column))
                );
            }
        ],
        filterColumns: [
            'props.localColumns',
            'privates.searchText',
            ({ props, state, privates, locales }) => {
                return (
                    props.localColumns &&
                    props.localColumns.filter(column => {
                        if (typeof column.title === 'string') {
                            return column.title.includes(privates.searchText);
                        } else {
                            // find other property?
                            return false;
                        }
                    })
                );
            }
        ]
    };
    // #endregion

    // #region actions
    actions = {
        onCheckChanged(checkedValues) {
            const { localColumns, onColumnsVisibleChanged } = this.props;
            const changedLocalColumns = localColumns.map(column => {
                const columnKey = getColumnKey(column);
                column.visible = checkedValues.includes(columnKey);
                if ((checkedValues.length === 1 && column.visible) || column.disabled) {
                    column.disabled = true;
                } else {
                    column.disabled = false;
                }
                return column;
            });
            onColumnsVisibleChanged(changedLocalColumns);
        },
        onSearchColumn(value) {
            this.privates.searchText = value;
            this.actions.update();
            // const { localColumns } = this.props;
            // const filterColumns = localColumns.filter(column => column.title.includes(value));
        }
    };
    // #endregion

    parseProps({ props, prevProps, state, privates, isInitial }) {
        return {
            localColumns: props.localColumns,
            searchText: ''
        };
    }

    render() {
        let { lookUp } = locales.current.components.reactAntdTable;
        return (
            <div className="column-setting">
                <Input.Search placeholder={lookUp} onSearch={this.actions.onSearchColumn} style={{ width: 200 }} />
                <div className="column-setting-container">
                    <Checkbox.Group
                        defaultValue={this.privates.computes.defaultValue || []}
                        onChange={this.actions.onCheckChanged}>
                        <For of={this.privates.computes.filterColumns || []} each="column" index="index">
                            <div className="column-setting-item">
                                <Checkbox value={getColumnKey(column)} disabled={column.disabled}>
                                    {column.title}
                                </Checkbox>
                            </div>
                        </For>
                    </Checkbox.Group>
                </div>
            </div>
        );
    }
}

export default UdeskReact.udeskify(ColumnsSetting);
