import React from 'react';
import Udesk from '../../../udesk/index';
import ReactEcharts from '../index';
import { getDemoData } from './data';
import {
    getChartData
} from '../../report/report-echarts/base';

const IGNORE_CHART_TYPES = [Udesk.enums.echartTypes.figure, Udesk.enums.echartTypes.table];

class ReactEchartsDemo extends React.Component {
    //#region
    static defaultProps = {
        language: "ZH-CN",
        isReverseData: false,
    }
    static propTypes = {

    }
    //#endregion
    privates = {
        chartType: "line",
        mapUrls: {
            china: `static/vendor/echarts/maps/china.js`,
            chinaCities: `static/vendor/echarts/maps/china-cities.js`
        }
    }
    actions = {
        selectedType(e) {
            let value = e.target.value;
            let isIgnoreChart = IGNORE_CHART_TYPES.some(type => type.id === value || type.key === value);
            if (isIgnoreChart) {
                return;
            }
            this.privates.chartType = value;
            this.actions.update();
        }
    }

    render() {
        let {
            privates,
            actions
        } = this;

        let chartType = privates.chartType;

        let demoData = getDemoData(chartType);
        let modelData = getChartData(this, demoData);

        let pieLikeRaduis = chartType === "ring" ? ["40%", "80%"] : [0, "80%"];

        return (
            <div style={{ height: "500px" }}>
                <div>
                    <h3>react-chart</h3>
                    <span>
                        各个图表类型配置项具体参数配置，参考echarts版本3.x.x;现不支持4.0版本。
                    </span>
                </div>
                <select style={{ height: "30px" }} value={chartType} onChange={actions.selectedType}>
                    {(function () {
                        let echartTypes = [];
                        Udesk.enums.echartTypes.forEach((type, index) => {
                            let isIgnoreChart = IGNORE_CHART_TYPES.some(ignoreType => type.id === ignoreType.id || type.key === ignoreType.key);

                            let item = (
                                <option disabled={isIgnoreChart} key={index} value={type.key} >
                                    {type.name}
                                </option>);
                            echartTypes.push(item);
                        });
                        return echartTypes;
                    })()}
                </select>
                <ReactEcharts
                    type="line"
                    xLabelField="xLabel"
                    isRealTime={false}
                    type={chartType}
                    mapUrls={privates.mapUrls}
                    showExtraPercentInLabels={true}
                    showTooltips={true}
                    mapName={"china"}
                    mapSeriesRoam={"scale"}
                    pieLikeRaduis={pieLikeRaduis}
                    records={modelData.items}
                    seriesColumns={modelData.columns} />
            </div>
        );
    }
}

ReactEchartsDemo.propTypes = {
};

ReactEchartsDemo.defaultProps = {
};

export default Udesk.react.udeskify(ReactEchartsDemo);