(function (root, factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD. Register as an anonymous module.
        define(['exports', 'echarts'], factory);
    } else if (typeof exports === 'object' && typeof exports.nodeName !== 'string') {
        // CommonJS
        factory(exports, require('echarts'));
    } else {
        // Browser globals
        factory({}, root.echarts);
    }
}(this, function (exports, echarts) {
    var log = function (msg) {
        if (typeof console !== 'undefined') {
            //console && console.error && console.error(msg);
        }
    };
    if (!echarts) {
        log('ECharts is not Loaded');
        return;
    }
    echarts.registerTheme('theme-udesk', {
        "color": [
            "#2d87e0",
            "#2953a6",
            "#76dedc",
            "#07a69d",
            "#dbd37b",
            "#e69f00",
            "#cc3d55",
            "#583c66",
            "#736db9",
            "#7eb6e0"
        ],
        "backgroundColor": "rgba(0,0,0,0)",
        "textStyle": {},
        "title": {
            "textStyle": {
                "color": "#363f4d"
            },
            "subtextStyle": {
                "color": "#4e4f4f"
            }
        },
        "line": {
            "itemStyle": {
                "normal": {
                    "borderWidth": "2"
                }
            },
            "lineStyle": {
                "normal": {
                    "width": "2"
                }
            },
            "symbolSize": "6",
            "symbol": "emptyCircle",
            "smooth": false
        },
        "radar": {
            "itemStyle": {
                "normal": {
                    "borderWidth": "2"
                }
            },
            "lineStyle": {
                "normal": {
                    "width": "2"
                }
            },
            "symbolSize": "6",
            "symbol": "emptyCircle",
            "smooth": false
        },
        "bar": {
            "itemStyle": {
                "normal": {
                    "barBorderWidth": "0",
                    "barBorderColor": "#ffffff"
                },
                "emphasis": {
                    "barBorderWidth": "0",
                    "barBorderColor": "#ffffff"
                }
            }
        },
        "pie": {
            "itemStyle": {
                "normal": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                },
                "emphasis": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                }
            }
        },
        "scatter": {
            "itemStyle": {
                "normal": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                },
                "emphasis": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                }
            }
        },
        "boxplot": {
            "itemStyle": {
                "normal": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                },
                "emphasis": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                }
            }
        },
        "parallel": {
            "itemStyle": {
                "normal": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                },
                "emphasis": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                }
            }
        },
        "sankey": {
            "itemStyle": {
                "normal": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                },
                "emphasis": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                }
            }
        },
        "funnel": {
            "itemStyle": {
                "normal": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                },
                "emphasis": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                }
            }
        },
        "gauge": {
            "itemStyle": {
                "normal": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                },
                "emphasis": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                }
            }
        },
        "candlestick": {
            "itemStyle": {
                "normal": {
                    "color": "#edafda",
                    "color0": "transparent",
                    "borderColor": "#d680bc",
                    "borderColor0": "#2d87e0",
                    "borderWidth": "1"
                }
            }
        },
        "graph": {
            "itemStyle": {
                "normal": {
                    "borderWidth": "0",
                    "borderColor": "#ffffff"
                }
            },
            "lineStyle": {
                "normal": {
                    "width": 1,
                    "color": "#aaaaaa"
                }
            },
            "symbolSize": "6",
            "symbol": "emptyCircle",
            "smooth": false,
            "color": [
                "#2d87e0",
                "#2953a6",
                "#76dedc",
                "#07a69d",
                "#dbd37b",
                "#e69f00",
                "#cc3d55",
                "#583c66",
                "#736db9",
                "#7eb6e0"
            ],
            "label": {
                "normal": {
                    "textStyle": {
                        "color": "#363f4d"
                    }
                }
            }
        },
        "map": {
            "itemStyle": {
                "normal": {
                    "areaColor": "#f3f4f5",
                    "borderColor": "#bbbbbc",
                    "borderWidth": "0.5"
                },
                "emphasis": {
                    "areaColor": "rgba(176,226,255,1)",
                    "borderColor": "#787a7b",
                    "borderWidth": "0.5"
                }
            },
            "label": {
                "normal": {
                    "textStyle": {
                        "color": "#363f4d"
                    }
                },
                "emphasis": {
                    "textStyle": {
                        "color": "rgb(43,57,91)"
                    }
                }
            }
        },
        "geo": {
            "itemStyle": {
                "normal": {
                    "areaColor": "#f3f4f5",
                    "borderColor": "#bbbbbc",
                    "borderWidth": "0.5"
                },
                "emphasis": {
                    "areaColor": "rgba(176,226,255,1)",
                    "borderColor": "#787a7b",
                    "borderWidth": "0.5"
                }
            },
            "label": {
                "normal": {
                    "textStyle": {
                        "color": "#363f4d"
                    }
                },
                "emphasis": {
                    "textStyle": {
                        "color": "rgb(43,57,91)"
                    }
                }
            }
        },
        "categoryAxis": {
            "axisLine": {
                "show": true,
                "lineStyle": {
                    "color": "#d4d4d4"
                }
            },
            "axisTick": {
                "show": false,
                "lineStyle": {
                    "color": "#333"
                }
            },
            "axisLabel": {
                "show": true,
                "textStyle": {
                    "color": "#a5aeb8"
                }
            },
            "splitLine": {
                "show": true,
                "lineStyle": {
                    "color": [
                        "#f3f4f5"
                    ]
                }
            },
            "splitArea": {
                "show": false,
                "areaStyle": {
                    "color": [
                        "rgba(250,250,250,0.05)",
                        "rgba(200,200,200,0.02)"
                    ]
                }
            }
        },
        "valueAxis": {
            "axisLine": {
                "show": true,
                "lineStyle": {
                    "color": "#d4d4d4"
                }
            },
            "axisTick": {
                "show": false,
                "lineStyle": {
                    "color": "#333"
                }
            },
            "axisLabel": {
                "show": true,
                "textStyle": {
                    "color": "#a5aeb8"
                }
            },
            "splitLine": {
                "show": true,
                "lineStyle": {
                    "color": [
                        "#f3f4f5"
                    ]
                }
            },
            "splitArea": {
                "show": false,
                "areaStyle": {
                    "color": [
                        "rgba(250,250,250,0.05)",
                        "rgba(200,200,200,0.02)"
                    ]
                }
            }
        },
        "logAxis": {
            "axisLine": {
                "show": true,
                "lineStyle": {
                    "color": "#d4d4d4"
                }
            },
            "axisTick": {
                "show": false,
                "lineStyle": {
                    "color": "#333"
                }
            },
            "axisLabel": {
                "show": true,
                "textStyle": {
                    "color": "#a5aeb8"
                }
            },
            "splitLine": {
                "show": true,
                "lineStyle": {
                    "color": [
                        "#f3f4f5"
                    ]
                }
            },
            "splitArea": {
                "show": false,
                "areaStyle": {
                    "color": [
                        "rgba(250,250,250,0.05)",
                        "rgba(200,200,200,0.02)"
                    ]
                }
            }
        },
        "timeAxis": {
            "axisLine": {
                "show": true,
                "lineStyle": {
                    "color": "#d4d4d4"
                }
            },
            "axisTick": {
                "show": false,
                "lineStyle": {
                    "color": "#333"
                }
            },
            "axisLabel": {
                "show": true,
                "textStyle": {
                    "color": "#a5aeb8"
                }
            },
            "splitLine": {
                "show": true,
                "lineStyle": {
                    "color": [
                        "#f3f4f5"
                    ]
                }
            },
            "splitArea": {
                "show": false,
                "areaStyle": {
                    "color": [
                        "rgba(250,250,250,0.05)",
                        "rgba(200,200,200,0.02)"
                    ]
                }
            }
        },
        "toolbox": {
            "iconStyle": {
                "normal": {
                    "borderColor": "#bbbbbc"
                },
                "emphasis": {
                    "borderColor": "#363f4d"
                }
            }
        },
        "legend": {
            "textStyle": {
                "color": "#5b6875"
            }
        },
        "tooltip": {
            "axisPointer": {
                "lineStyle": {
                    "color": "#c8cbd6",
                    "width": "1"
                },
                "crossStyle": {
                    "color": "#c8cbd6",
                    "width": "1"
                }
            }
        },
        "timeline": {
            "lineStyle": {
                "color": "#787a7b",
                "width": 1
            },
            "itemStyle": {
                "normal": {
                    "color": "#787a7b",
                    "borderWidth": "1"
                },
                "emphasis": {
                    "color": "#bbbbbc"
                }
            },
            "controlStyle": {
                "normal": {
                    "color": "#787a7b",
                    "borderColor": "#787a7b",
                    "borderWidth": 0.5
                },
                "emphasis": {
                    "color": "#787a7b",
                    "borderColor": "#787a7b",
                    "borderWidth": 0.5
                }
            },
            "checkpointStyle": {
                "color": "#bbbbbc",
                "borderColor": "rgba(148,162,179,0.37)"
            },
            "label": {
                "normal": {
                    "textStyle": {
                        "color": "#bbbbbc"
                    }
                },
                "emphasis": {
                    "textStyle": {
                        "color": "#bbbbbc"
                    }
                }
            }
        },
        "visualMap": {
            "color": [
                "#cc3d55",
                "#2d87e0",
                "#c5dadd"
            ]
        },
        "dataZoom": {
            "backgroundColor": "rgba(0,0,0,0)",
            "dataBackgroundColor": "rgba(255,255,255,0.3)",
            "fillerColor": "rgba(167,183,204,0.4)",
            "handleColor": "#bbbbbc",
            "handleSize": "100%",
            "textStyle": {
                "color": "#787a7b"
            }
        },
        "markPoint": {
            "label": {
                "normal": {
                    "textStyle": {
                        "color": "#363f4d"
                    }
                },
                "emphasis": {
                    "textStyle": {
                        "color": "#363f4d"
                    }
                }
            }
        }
    });
}));
