import React from 'react';
import ReactForm from 'udesk-react/src/components/react-form';
import { Form, Input, Tooltip, Icon, Cascader, Select, Row, Col, Checkbox, Button, AutoComplete } from 'antd';

const { Option } = Select;
const AutoCompleteOption = AutoComplete.Option;

const residences = [
    {
        value: 'zhejiang',
        label: 'Zhejiang',
        children: [
            {
                value: 'hangzhou',
                label: 'Hangzhou',
                children: [
                    {
                        value: 'xihu',
                        label: 'West Lake'
                    }
                ]
            }
        ]
    },
    {
        value: 'jiangsu',
        label: 'Jiangsu',
        children: [
            {
                value: 'nanjing',
                label: 'Nanjing',
                children: [
                    {
                        value: 'zhonghuamen',
                        label: 'Zhong Hua Men'
                    }
                ]
            }
        ]
    }
];

class NewForm extends React.Component {
    state = {
        autoCompleteResult: []
    };

    constructor(props) {
        super(props);
        this.form = React.createRef();
    }

    handleSubmit = data => {
        
    };

    compareToFirstPassword = (rule, value, callback) => {
        const { form } = this.form.current.props;

        if (value && value !== form.getFieldValue('password')) {
            callback('Two passwords that you enter is inconsistent!');
        } else {
            callback();
        }
    };

    validateToNextPassword = (rule, value, callback) => {
        const { form } = this.form.current.props;

        if (value) {
            form.validateFields(['confirm'], { force: true });
        }
        callback();
    };

    handleWebsiteChange = value => {
        let autoCompleteResult;
        if (!value) {
            autoCompleteResult = [];
        } else {
            autoCompleteResult = ['.com', '.org', '.net'].map(domain => `${value}${domain}`);
        }
        this.setState({ autoCompleteResult });
    };

    render() {
        const { getFieldDecorator } = this.props.form;
        const { autoCompleteResult } = this.state;

        const prefixSelector = (
            <Select style={{ width: 70 }} defaultValue="86">
                <Option value="86">+86</Option>
                <Option value="87">+87</Option>
            </Select>
        );

        const websiteOptions = autoCompleteResult.map(website => (
            <AutoCompleteOption key={website}>{website}</AutoCompleteOption>
        ));

        return (
            <ReactForm wrappedComponentRef={this.form} onSubmit={this.handleSubmit}>
                <Input id="email" label="E-mail" rules={['email', 'required']} />

                <Input.Password
                    id="password"
                    label="Password"
                    rules={[
                        'required',
                        'string',
                        {
                            validator: this.validateToNextPassword
                        },
                        'max=14',
                        'min=6'
                    ]}
                    hasFeedback
                />

                <Input.Password
                    id="confirm"
                    label="Confirm Password"
                    rules={[
                        'required',
                        'string',
                        {
                            validator: this.compareToFirstPassword
                        },
                        'max=14',
                        'min=6'
                    ]}
                    hasFeedback
                />

                <Input id="nickname" label="Nickname" tooltip="What do you want others to call you?" />

                <Cascader
                    id="residence"
                    label="Habitual Residence"
                    options={residences}
                    rules={['required', 'array']}
                />

                <Input id="phone" label="Phone Number" addonBefore={prefixSelector} rules={['required', 'phone']} />

                <AutoComplete
                    id="website"
                    label="Website"
                    rules={['required']}
                    dataSource={websiteOptions}
                    onChange={this.handleWebsiteChange}
                    placeholder="website">
                    <Input />
                </AutoComplete>

                <Row>
                    <Col span={10}>
                        <Input
                            addonAfter={<Button>Get captcha</Button>}
                            id="captcha"
                            label="Captcha"
                            extra="We must make sure that your are a human."
                            rules={['required']}
                        />
                    </Col>
                    <Col span={10}>
                        <Input
                            addonAfter={<Button>Get captcha</Button>}
                            id="captcha1"
                            // label="Captcha"
                            extra="We must make sure that your are a human."
                            rules={['required']}
                        />
                    </Col>
                </Row>

                <Checkbox
                    id="agreement"
                    valuePropName="checked"
                    formItemLayout={{
                        wrapperCol: {
                            span: 16,
                            push: 8
                        }
                    }}>
                    I have read the <a>agreement</a>
                </Checkbox>

                <Button htmlType="submit" id="submit">
                    Register
                </Button>
            </ReactForm>
        );
    }
}
export default Form.create({ name: 'register' })(NewForm);
