import defaultConfig from '../config';
import Form from 'antd/lib/form';
import 'antd/lib/form/style/index.css';
import moment from 'moment';

let config = defaultConfig;

Form.setConfig = newConfig => {
    config = { ...defaultConfig, ...newConfig };
};

const ruleCreator = {
    required: label => ({
        required: true,
        message: config.required.replace('{label}', label)
    }),
    illegalCharacter: label => ({
        validator: (rule, value, callback) => {
            const isValid = /^[\s\u4e00-\u9fa5a-z0-9_-]{0,}$/i.test(value);
            if (isValid) {
                callback();
            } else {
                callback(config.illegalCharacter.replace('{label}', label));
            }
        }
    }),
    string: label => ({
        type: 'string',
        whitespace: true,
        message: config.formatError.replace('{label}', label)
    }),
    number: label => ({
        pattern: /^\d+$/,
        whitespace: true,
        message: config.formatError.replace('{label}', label)
    }),
    array: label => ({
        type: 'array',
        message: config.formatError.replace('{label}', label)
    }),
    email: label => ({
        type: 'email',
        whitespace: true,
        message: config.formatError.replace('{label}', label)
    }),
    max: max => ({
        max,
        message: config.max.replace('{max}', max)
    }),
    min: min => ({
        min,
        message: config.min.replace('{min}', min)
    }),
    phone: label => ({
        pattern: /^1[3456789]\d{9}$/,
        whitespace: true,
        message: config.formatError.replace('{label}', label)
    }),
    id: label => ({
        pattern: /^\d{17}(x|\d)$/i,
        whitespace: true,
        message: config.formatError.replace('{label}', label)
    }),
    date: label => ({
        validator: (rule, value, callback) => {
            if (moment(value).isValid()) {
                callback();
            } else {
                callback(config.formatError.replace('{label}', label));
            }
        }
    })
};

/**
 * according rules to validation
 */
const initRules = (label = '', id, rules) => {
    return rules.map(rule => {
        if (typeof rule !== 'string') {
            return rule;
        }
        const ruleKey = `${id}.${rule}`;
        // e.g. "required"
        if (!rule.includes('=')) {
            if (!initRules[ruleKey]) {
                initRules[ruleKey] = ruleCreator[rule](label);
            }
            return initRules[ruleKey];
        } else {
            // e.g. "max=5"
            const [key, val] = rule.split('=');
            if (!initRules[ruleKey]) {
                initRules[ruleKey] = ruleCreator[key](Number(val));
            }
            return initRules[ruleKey];
        }
    });
};

const initPlaceholder = (node, label = '') => {
    if (node && node.type) {
        switch (node.type.name) {
        case 'Input':
            return `${config.inputPlaceholder}${label}`;
        case 'Select':
        case 'PickerWrapper':
            return `${config.selectPlaceholder}${label}`;
        default:
            return label;
        }
    }
};

export default { initRules, initPlaceholder };
