import React from 'react';
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import ReactPaginate from 'react-paginate';
import locales from '../../udesk/locales/index';
import Udesk from '../../udesk/index';

class ReactPager extends React.Component {

    static computes = {
        totalPages: function ({ props, state, privates, locales }) {
            let {
                pageSize,
                total
            } = props;
            let totalPages = null;
            if (pageSize) {
                totalPages = Math.ceil(total / pageSize);
            } else {
                totalPages = Udesk.config.paging.defaultPageNumber;
            }
            return totalPages;
        },
        gotoText: ["computes.totalPages", function ({ props, state, privates }) {
            let {
                showTotalPagesInGodo,
                pageNum
            } = props;
            let {
                totalPages
            } = privates.computes;
            var gotoText = [];

            gotoText.push(Udesk.utils.string.format(locales.current.components.reactPager.currentPageNumberFormat, pageNum));
            if (showTotalPagesInGodo) {
                gotoText.push(locales.current.components.reactPager.gotoSeparator);
                gotoText.push(Udesk.utils.string.format(locales.current.components.reactPager.totalPagesFormat, totalPages));
            }
            return gotoText.join("");
        }],
        pageSizeList: function ({ props, state, privates, locales }) {
            let {
                pageSizeList
            } = props;
            if (pageSizeList instanceof Array && pageSizeList.length > 0) {
                return pageSizeList;
            }
            return [
                { value: 5, text: "5" },
                { value: 10, text: "10" },
                { value: 20, text: "20" },
                { value: 50, text: "50" },
                { value: 100, text: "100" }
            ];
        }
    }

    actions = {
        changePageSize(e) {
            var intValue = parseInt(e.target.value, 10);
            if (isNaN(intValue)) {
                return;
            }
            this.actions.pageChanged({
                selected: Udesk.config.paging.defaultPageNumber - 1
            }, intValue);
        },

        pageChanged(page, pageSize) {
            if (pageSize == null) {
                pageSize = this.props.pageSize;
            }
            if (page == null || !(typeof page === "object") || page.selected == null) {
                page = {
                    select: Udesk.config.paging.defaultPageNumber
                };
            }
            page.selected = parseInt(page.selected, 10) + 1;
            if (this.props.pageNum === page.selected && pageSize === this.props.pageSize) {
                return;
            }
            this.props.pageChanged(page.selected, pageSize);
        }
    }

    render() {
        let {
            classNames,
            enablePageSize,
            pageSize,
            showTotal,
            total,
            pageNum,
            totalPagesOnLeft,
            showTotalPages,
            showTotalPagesInGodo,
            enableGoto,
            paginationNext,
            paginationPrevious,
            seperator,
            marginPagesDisplayed,
            pageRangeDisplayed,
            languageCode
        } = this.props;
        let {
            privates: { computes },
        } = this;
        return (
            <div className={ClassNames("udesk-pager clearfix", { [`${classNames}`]: classNames !== "" })}>
                <div className="pager-left">
                    <If condition={enablePageSize}>
                        <div className="page-size-block">
                            <span className="page-size-prefix">{locales.get("components.reactPager.unitPrefix", languageCode)}</span>
                            <select className="page-size" value={pageSize} onChange={this.actions.changePageSize}>
                                {computes.pageSizeList.map((item, index) => {
                                    return (
                                        <option key={index} value={item.value}>{item.text}</option>
                                    );
                                })}
                            </select>
                            <span className="page-size-postfix">{locales.get("components.reactPager.unitPostfix", languageCode)}</span>
                        </div>
                    </If>
                    <If condition={showTotal}>
                        <div className="total-block">
                            {Udesk.utils.string.format(locales.get("components.reactPager.totalFormat", languageCode), total)}
                        </div>
                    </If>
                    <If condition={totalPagesOnLeft && showTotalPages && computes.totalPages}>
                        <div className="total-pages-block">
                            {Udesk.utils.string.format(locales.get("components.reactPager.totalPagesFormat", languageCode), computes.totalPages)}
                        </div>
                    </If>
                </div>
                <div className={ClassNames("pager-right", { "pager-right-not-pager": !enableGoto })}>
                    {!enableGoto ? (
                        <ReactPaginate
                            previousLabel={paginationPrevious}
                            nextLabel={paginationNext}
                            breakLabel={<span> {seperator} </span>}
                            pageCount={computes.totalPages}
                            initialPage={pageNum - 1}
                            forcePage={pageNum - 1}
                            marginPagesDisplayed={marginPagesDisplayed}
                            pageRangeDisplayed={pageRangeDisplayed}
                            onPageChange={this.actions.pageChanged}
                            containerClassName={"pagination"}
                            subContainerClassName={"pages pagination"}
                            activeClassName={"active"} />
                    ) : (<span className="goto-text-block">{computes.gotoText}</span>)
                    }
                    <If condition={(!showTotalPagesInGodo) && (!totalPagesOnLeft) && showTotalPages && computes.totalPages}>
                        <span className="total-pages-block">
                            {Udesk.utils.string.format(locales.get("components.reactPager.totalPagesFormat", languageCode), computes.totalPages)}
                        </span>
                    </If>
                </div>
            </div>
        );
    }
}

ReactPager.propTypes = {
    classNames: PropTypes.string,
    pageNum: PropTypes.number,
    pageSize: PropTypes.number,
    total: PropTypes.number,
    pager: PropTypes.bool,
    hide: PropTypes.bool,
    autoHide: PropTypes.bool,
    disabled: PropTypes.bool,
    urlTemplate: PropTypes.string,
    seperator: PropTypes.string,
    enablePageSize: PropTypes.bool,
    enableGoto: PropTypes.bool,
    pageSizeList: PropTypes.array,
    showTotal: PropTypes.bool,
    showTotalPages: PropTypes.bool,
    totalPagesOnLeft: PropTypes.bool,
    showTotalPagesInGodo: PropTypes.bool,
    paginationNext: PropTypes.string,
    paginationPrevious: PropTypes.string,
    marginPagesDisplayed: PropTypes.number,
    pageRangeDisplayed: PropTypes.number,
    pageChanged: PropTypes.func,
};

ReactPager.defaultProps = {
    languageCode: "",
    classNames: "",
    pageNum: Udesk.config.paging.defaultPageNumber,
    pageSize: Udesk.config.paging.defaultPageSize,
    total: 0,
    pager: false,
    hide: false,
    autoHide: false,
    disabled: false,
    urlTemplate: '#',
    seperator: '…',
    enablePageSize: true,
    enableGoto: false,
    pageSizeList: [],
    showTotal: true,
    showTotalPages: true,
    totalPagesOnLeft: true,
    showTotalPagesInGodo: false,
    paginationNext: '»',
    paginationPrevious: '«',
    marginPagesDisplayed: 2,
    pageRangeDisplayed: 5,
    pageChanged: function () {

    }
};

export default Udesk.react.udeskify(ReactPager);