import React from 'react';
import PropTypes from 'prop-types';
import Locales from "../../../udesk/locales/index";
import { get } from "udesk-web-toolkits/src/udesk/utils/object";

const DEFAULT_VALUE_FIELD = "id";
const DEFAULT_NAME_FIELD = "name";

const DEFAULT_CUSTOM_AUTO_COMPLETE_STYLES = {
    control: (provided, state) => {
        const borderColor = state.isFocused ? "rgba(16, 138, 148, 1)" : provided.borderColor;
        return {
            ...provided,
            borderColor,
            minHeight: "30px",
            backgroundColor: "#FFF",
            borderRadius: "0px"
        };
    },
    multiValueLabel: (provided, state) => ({
        ...provided,
        padding: "1px"
    }),
    clearIndicator: (provided, state) => ({
        ...provided,
        padding: "4px 8px"
    }),
    dropdownIndicator: (provided, state) => ({
        ...provided,
        padding: "4px 8px"
    }),
    valueContainer: (provided, state) => ({
        ...provided,
        padding: "0px 8px"
    }),
};

export default class ReactPowerSelectComponent extends React.Component { 
    static propTypes = {
        dataSource: PropTypes.array,
        valueField: PropTypes.string,
        nameField: PropTypes.string,
        classNames: PropTypes.string,
        isClearable: PropTypes.bool,
        disabled: PropTypes.bool,
        placeholder: PropTypes.string,
        noOptionsMessage: PropTypes.string,
        isMulti: PropTypes.bool,
        onChanged: PropTypes.func,
        returnObject: PropTypes.bool,
        autoFocus: PropTypes.bool,
        defaultMenuIsOpen: PropTypes.bool,
        components: PropTypes.object,
        filterOption: PropTypes.func,
    }

    static defaultProps = {
        value: null,
        dataSource: [],
        valueField: DEFAULT_VALUE_FIELD,
        nameField: DEFAULT_NAME_FIELD,
        classNames: "",
        customStyle: null,
        isClearable: true,
        disabled: false,
        placeholder: "",
        isMulti: false,
        noOptionsMessage: null,
        onChanged: null,
        returnObject: false,
        autoFocus: false,
        defaultMenuIsOpen: false,
        components: null,
        filterOption: undefined,
    }

    static computes = {
        value: ({ props, state, privates, locales }) => {
            let {
                dataSource,
                value,
                valueField,
                isMulti
            } = props;
            if (dataSource == null || !(dataSource instanceof Array)) {
                dataSource = [];
            }
            let currentValue = null;
            if(isMulti) {
                if (value != null) {
                    if (!(value instanceof Array)) { 
                        value = [];
                    }
                    currentValue = value.map(valueItem => dataSource.find(item => get(item, valueField) === valueItem));
                    currentValue = currentValue.filter(item => item != null);
                }
            } else {
                if (value != null) { 
                    currentValue = dataSource.find(item => get(item, valueField) === value);
                }
            }
            return currentValue;
        },
        getOptionLabel: ["props.nameField", ({ props, state, privates, locales }) => {
            let {
                nameField
            } = props;
            return (option) => {
                return option[nameField];
            };
        }],
        getOptionValue: ["props.valueField", ({ props, state, privates, locales }) => {
            let {
                valueField
            } = props;
            return (option) => {
                return option[valueField];
            };
        }],
        noOptionsMessageFunc: ["props.noOptionsMessage", ({ props, state, privates, locales }) => {
            let {
                noOptionsMessage
            } = props;
            let noOptionsMessageFunc = () => noOptionsMessage;
            if (noOptionsMessage == null) {
                noOptionsMessageFunc = () => Locales.current.components.reactPowerSelect.noOptionsMessage;
            }
            return noOptionsMessageFunc;
        }],
        classNames: ["props.classNames", ({ props, state, privates, locales }) => {
            let {
                classNames
            } = props;
            let componentClass = ["udesk-react-power-select"];
            if (classNames != null && classNames !== "") {
                componentClass.push(classNames);
            }
            return componentClass.join(" ");
        }],
        customStyle: ["props.customStyle", ({ props, state, privates, locales }) => {
            let {
                customStyle
            } = props;
            if (customStyle != null) {
                return customStyle;
            }
            return DEFAULT_CUSTOM_AUTO_COMPLETE_STYLES;
        }],
        dataSource: ["props.dataSource", ({ props, state, privates, locales }) => {
            let {
                dataSource
            } = props;
            if (dataSource == null || !(dataSource instanceof Array)) {
                dataSource = [];
            }
            return dataSource;
        }],
    }

    actions = {
        onChanged(value) { 
            let {
                onChanged,
                returnObject,
                valueField,
                isMulti
            } = this.props;
            let outputValue = null;
            if (value != null) {
                if (returnObject) {
                    outputValue = value;
                } else {
                    if (isMulti) {
                        if (!(value instanceof Array)) { 
                            value = [];
                        }
                        outputValue = value.map(item => item[valueField]);
                    } else { 
                        outputValue = value[valueField];
                    }
                }
            }
            if (onChanged) { 
                onChanged(outputValue);
            }
        }
    }
}