import React from 'react';
import DemoLocalesLocales from '../../../app/locales';
import data from './config/data';
import enums from './config/enums';

export default class ReactTableDemo extends React.Component {
    constructor(props, context) {
        super(props, context);
        DemoLocalesLocales.translate(this.state.columns, "name", "caption", DemoLocalesLocales.current.demo.reactTable.columns);
    }
    state = {
        // #region columns
        columns: [
            {
                "name": "name",
                "checked": true
            },
            {
                "name": "mobile",
                "checked": true
            },
            {
                "name": "telephone",
                "checked": true
            },
            {
                "name": "email",
                "checked": false
            },
            {
                "name": "classification",
                get: (item, column) => {
                    return enums.classifications.getName(item.classification);
                },
            },
            {
                "name": "location",
                "checked": false,
                get: function (item, column) {
                    // return Udesk.data.location.getFullName(item.province, item.city, item.district);
                    return `${item.province}-${item.city}-${item.district}`;
                },
            },
            {
                "name": "address",
                "checked": false
            },
            {
                "name": "businessCategoryId",
                "checked": false,
                get: function (item, column) {
                    return enums.customerBusinessCategories.getName(item.businessCategoryId);
                },
            },
            {
                "name": "masterUserId",
                "checked": true,
                get: function (item, column) {
                    if (!item.master) {
                        return "";
                    }
                    return item.master.realname;
                },
            },
            {
                "name": "officeId",
                "checked": false,
                get: function (item, column) {
                    if (!item.masterOffice) {
                        return "";
                    }
                    return item.masterOffice.name;
                },
            },
            {
                "name": "customerHighSeaId",
                "checked": false,
                get: function (item, column) {
                    if (!item.customerHighSea) {
                        return "";
                    }
                    return item.customerHighSea.name;
                },
            },
            {
                "name": "highSeaStatus",
                "checked": false,
                get: function (item, column) {
                    return enums.highSeaStatus.getName(item.highSeaStatus);
                },
            },
            {
                "name": "customerHighSeaReceiveTime",
                "checked": false
            },
            {
                "name": "expireTime",
                "checked": false
            },
            {
                "name": "customerHighSeaReturnCount",
                "checked": false
            },
            {
                "name": "createUserId",
                "checked": false,
                get: function (item, column) {
                    if (!item.createUser) {
                        return "";
                    }
                    return item.createUser.realname;
                },
            },
            {
                "name": "createTime",
                "checked": false
            },
            {
                "name": "recentActivityRecordTime",
                "checked": true
            },
            {
                "name": "locked",
                "checked": false,
                get: function (item, column) {
                    return enums.lockStatus.getName(item.locked);
                },
            },
            {
                "name": "updateUserId",
                "checked": false,
                get: function (item, column) {
                    if (!item.updateUser) {
                        return "";
                    }
                    return item.updateUser.realname;
                },
            },
            {
                "name": "updateTime",
                "checked": false
            },
            {
                "name": "notFollowDays",
                "checked": false
            },
            {
                "name": "lastMasterUserId",
                "checked": false,
                get: function (item, column) {
                    if (!item.lastMasterUser) {
                        return "";
                    }
                    return item.lastMasterUser.realname;
                },
            },
            {
                "name": "remark",
                "checked": false
            },
            {
                "name": "tags",
                "checked": false,
                get: function (item, column) {
                    if (item.tagList.length === 0) {
                        return "";
                    }
                    let tags = [];
                    item.tagList.forEach(tagItem => {
                        tags.push(tagItem.name);
                    });
                    return tags.join(",");
                },
            }
        ],
        // #endregion
        items: data
    }
}

ReactTableDemo.propTypes = {
};

ReactTableDemo.defaultProps = {
};
