import React from 'react';
import ClassNames from 'classnames';
import PropTypes from 'prop-types';
import Udesk from "../../udesk/index";

const TRANSFER_MODE = {
    manualTransfer: "manual",
    autoTransfer: "auto",
};

class ReactTransfer extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            leftDataList: [],
            rightDataList: [],
            isAllowedMoveToRight: false,
            isAllowedMoveToLeft: false
        };
    }

    actions = {
        selectedLeftItem(leftDataItem, child, e) {
            e.stopPropagation();

            let {
                transferMode,
                showCheckbox
            } = this.props;

            let {
                leftDataList,
                rightDataList
            } = this.state;
            e.target.disabled = true;

            if (transferMode === TRANSFER_MODE.autoTransfer) {
                if (leftDataItem.states.isGroups) {
                    if (child != null) {
                        //直接点击选取
                        let targetLeftDataGroup = leftDataList.find(item => item.id === leftDataItem.id);
                        if (targetLeftDataGroup && targetLeftDataGroup.children && targetLeftDataGroup.children.length > 0) {
                            let targetRightDataGroup = rightDataList.find(rightItem => rightItem.id === leftDataItem.id);
                            if (!targetRightDataGroup) {
                                targetRightDataGroup = $.extend(true, {}, targetLeftDataGroup);
                                targetRightDataGroup.children = [];
                                targetRightDataGroup.states.isSelected = true;
                                rightDataList.push(targetRightDataGroup);
                            }
                            targetLeftDataGroup.children.forEach(targetChild => {
                                if (targetChild.id === child.id) {
                                    targetChild.states.isSelected = true;
                                    targetRightDataGroup.children.push(targetChild);
                                }
                            });
                            let hasNoSelected = targetLeftDataGroup.children.some(targetChild => !targetChild.isSelected);
                            targetLeftDataGroup.states.isSelected = !hasNoSelected;
                        } else {
                            return;
                        }
                    } else {
                        return;
                    }
                } else {
                    //直接点击选取
                    leftDataList.forEach(targetChild => {
                        if (targetChild.id === leftDataItem.id) {
                            targetChild.states.isSelected = true;
                            rightDataList.push(targetChild);
                        }
                    });
                }
                let newSelectedItems = getSelectedItems(this, rightDataList);
                if (typeof this.props.onChanged === "function") {
                    this.props.onChanged(newSelectedItems);
                }
                this.setState({
                    leftDataList: [].concat(leftDataList),
                    rightDataList: [].concat(rightDataList)
                });
            } else {
                if (showCheckbox) {
                    let elm = e.target || e.srcElement;
                    if (elm.tagName === 'INPUT') {
                        let newLeftDataList = getManualChangedDataList(leftDataList, leftDataItem, child, showCheckbox);
                        if (newLeftDataList) {
                            let isAllowedMoveToRight = hasIsSelectingItem(newLeftDataList);
                            this.setState({
                                leftDataList: [].concat(newLeftDataList),
                                isAllowedMoveToRight
                            });
                        }
                    }
                } else {
                    let newLeftDataList = getManualChangedDataList(leftDataList, leftDataItem, child, showCheckbox);
                    if (newLeftDataList) {
                        let isAllowedMoveToRight = hasIsSelectingItem(newLeftDataList);
                        this.setState({
                            leftDataList: [].concat(newLeftDataList),
                            isAllowedMoveToRight

                        });
                    }
                }
            }
            e.target.disabled = false;
        },
        selectedRightItem(rightDataItem, child, e) {
            e.stopPropagation();

            let {
                transferMode,
                showCheckbox
            } = this.props;
            let {
                leftDataList,
                rightDataList
            } = this.state;
            e.target.disabled = true;
            if (transferMode === TRANSFER_MODE.autoTransfer) {
                if (rightDataItem.states.isGroups) {
                    if (child != null) {
                        //直接点击选取
                        let targetRightDataGroup = rightDataList.find(rightItem => rightItem.id === rightDataItem.id);
                        if (targetRightDataGroup && targetRightDataGroup.children && targetRightDataGroup.children.length > 0) {
                            targetRightDataGroup.children = targetRightDataGroup.children.filter(rightChild => rightChild.id !== child.id);
                            if (targetRightDataGroup.children && targetRightDataGroup.children.length < 1) {
                                rightDataList = rightDataList.filter(rightItem => rightItem.id !== targetRightDataGroup.id);
                            }

                            let targetLeftDataGroup = leftDataList.find(item => item.id === rightDataItem.id);
                            if (targetLeftDataGroup && targetLeftDataGroup.children && targetLeftDataGroup.children.length > 0) {
                                targetLeftDataGroup.states.isSelected = false;
                                targetLeftDataGroup.children.forEach(targetChild => {
                                    if (targetChild.id === child.id) {
                                        targetChild.states.isSelected = false;
                                    }
                                });
                            }
                        } else {
                            return;
                        }
                    } else {
                        return;
                    }
                } else {
                    //直接点击选取
                    rightDataList = rightDataList.filter(rightItem => rightItem.id !== rightDataItem.id);
                    leftDataList.forEach(targetChild => {
                        if (targetChild.id === rightDataItem.id) {
                            targetChild.states.isSelected = false;
                        }
                    });
                }

                let newSelectedItems = getSelectedItems(this, rightDataList);
                if (typeof this.props.onChanged === "function") {
                    this.props.onChanged(newSelectedItems);
                }
                this.setState({
                    leftDataList: [].concat(leftDataList),
                    rightDataList: [].concat(rightDataList)
                });
            } else {
                if (showCheckbox) {
                    let elm = e.target || e.srcElement;
                    if (elm.tagName === 'INPUT') {
                        let newRightDataList = getManualChangedDataList(rightDataList, rightDataItem, child, showCheckbox);
                        if (newRightDataList) {
                            let isAllowedMoveToLeft = hasIsSelectingItem(newRightDataList);
                            this.setState({
                                rightDataList: [].concat(newRightDataList),
                                isAllowedMoveToLeft,
                            });
                        }
                    }
                } else {
                    let newRightDataList = getManualChangedDataList(rightDataList, rightDataItem, child, showCheckbox);
                    if (newRightDataList) {
                        let isAllowedMoveToLeft = hasIsSelectingItem(newRightDataList);
                        this.setState({
                            rightDataList: [].concat(newRightDataList),
                            isAllowedMoveToLeft
                        });
                    }
                }
            }
            e.target.disabled = false;
        },
        moveToLeft(e) {
            let {
                transferMode,
            } = this.props;
            let {
                leftDataList,
                rightDataList,
                isAllowedMoveToLeft
            } = this.state;

            if (!isAllowedMoveToLeft) {
                return;
            }
            if (transferMode === TRANSFER_MODE.manualTransfer) {
                e.target.disabled = true;

                let newRightDataList = rightDataList.map(rightDataItem => {
                    if (rightDataItem.states.isGroups) {
                        let hasSelecting = rightDataItem.children.some(child => child.states.isSelecting);
                        if (hasSelecting) {
                            rightDataItem.children = rightDataItem.children.filter(child => !child.states.isSelecting);
                            if (rightDataItem.children && rightDataItem.children.length > 0) {
                                return rightDataItem;
                            } else {
                                return undefined;
                            }
                        } else {
                            return rightDataItem;
                        }
                    } else {
                        if (rightDataItem.states.isSelecting) {
                            leftDataList.forEach(leftDataItem => {
                                if (leftDataItem.id === rightDataItem.id) {
                                    leftDataItem.states.isSelected = false;
                                    rightDataItem.states.isSelecting = false;
                                }
                            });
                            return undefined;
                        } else {
                            rightDataItem.states.isSelecting = false;
                            return rightDataItem;
                        }
                    }
                }).filter(Boolean);

                let newSelectedItems = getSelectedItems(this, newRightDataList);
                if (typeof this.props.onChanged === "function") {
                    this.props.onChanged(newSelectedItems);
                }

                this.setState({
                    leftDataList: [].concat(leftDataList),
                    rightDataList: [].concat(newRightDataList),
                    isAllowedMoveToLeft: false
                });
                e.target.disabled = false;
            } else {
                return;
            }
        },
        moveToRight(e) {
            let {
                transferMode,
            } = this.props;
            let {
                leftDataList,
                rightDataList,
                isAllowedMoveToRight
            } = this.state;

            if (!isAllowedMoveToRight) {
                return;
            }
            if (transferMode === TRANSFER_MODE.manualTransfer) {
                e.target.disabled = true;

                leftDataList.forEach(leftDataItem => {
                    if (leftDataItem.states.isGroups) {
                        let hasSelecting = leftDataItem.children.some(child => child.states.isSelecting);
                        if (hasSelecting) {
                            let targetRightDataGroup = rightDataList.find(rightItem => rightItem.id === leftDataItem.id);
                            if (!targetRightDataGroup) {
                                targetRightDataGroup = $.extend(true, {}, leftDataItem);
                                targetRightDataGroup.children = [];
                                targetRightDataGroup.states.isSelecting = false;
                                targetRightDataGroup.states.isSelected = true;
                                rightDataList.push(targetRightDataGroup);
                            }

                            leftDataItem.children.forEach(child => {
                                if (child.states.isSelecting) {
                                    child.states.isSelecting = false;
                                    child.states.isSelected = true;
                                    targetRightDataGroup.children.push($.extend(true, {}, child));
                                }
                            });
                            leftDataItem.states.isSelecting = false;
                            let hasNoSelected = leftDataItem.children.some(child => !child.states.isSelected);
                            leftDataItem.states.isSelected = !hasNoSelected;
                        }
                    } else {
                        if (leftDataItem.states.isSelecting) {
                            leftDataItem.states.isSelecting = false;
                            leftDataItem.states.isSelected = true;
                            rightDataList.push($.extend(true, {}, leftDataItem));
                        }
                    }
                });
                let newSelectedItems = getSelectedItems(this, rightDataList);
                if (typeof this.props.onChanged === "function") {
                    this.props.onChanged(newSelectedItems);
                }
                this.setState({
                    leftDataList: [].concat(leftDataList),
                    rightDataList: [].concat(rightDataList),
                    isAllowedMoveToRight: false,
                });
                e.target.disabled = false;
            } else {
                return;
            }
        }
    }

    componentDidMount() {
        let {
            allDataItems,
            selectedItems
        } = this.props;
        let leftDataList = buildLeftDataList(this, allDataItems, selectedItems);
        let rightDataList = buildRightDataList(this, leftDataList, selectedItems);
        this.setState({
            leftDataList: [].concat(leftDataList),
            rightDataList: [].concat(rightDataList),
        });
    }
    componentDidUpdate(prevProps, prevState, snapshot) {
        let {
            selectedItems,
            allDataItems
        } = this.props;
        let {
            selectedItems: oldSelectedItems
        } = prevProps;

        let deleteIds = oldSelectedItems.filter(id => !selectedItems.includes(id));
        let addIds = selectedItems.filter(newId => !oldSelectedItems.includes(newId));
        if ((deleteIds && deleteIds.length > 0) || (addIds && addIds.length > 0)) {
            let leftDataList = buildLeftDataList(this, allDataItems, selectedItems);
            let rightDataList = buildRightDataList(this, leftDataList, selectedItems);
            this.setState({
                leftDataList: [].concat(leftDataList),
                rightDataList: [].concat(rightDataList),
            });
        }
    }
    render() {
        let {
            className,
            leftTitle,
            LeftTitle,
            rightTitle,
            showCheckbox,
            transferMode,
            showMoveRightButton,
            moveRightButtonText,
            moveRightButtonIcon,
            showMoveLeftButton,
            moveLeftButtonText,
            moveLeftButtonIcon,
            noSelectedText
        } = this.props;

        let {
            leftDataList,
            rightDataList,
            isAllowedMoveToRight,
            isAllowedMoveToLeft
        } = this.state;
        let isManualTransfer = transferMode === TRANSFER_MODE.manualTransfer;
        return (
            <div className={ClassNames("react-transfer", { [`${className}`]: className })}>
                <div className="react-transfer-item react-transfer-left">
                    <div className="react-transfer-item-title">
                        <If condition={typeof leftTitle === "string"}>
                            {leftTitle}
                        </If>
                        <LeftTitle />
                    </div>
                    <div className="react-transfer-item-container react-transfer-left-container scrollbar-y">
                        <ul className="react-transfer-item-content react-transfer-left-content">
                            {leftDataList.map((leftDataItem, index) => {
                                if (leftDataItem.states.isSelected) {
                                    return null;
                                } else {
                                    return (<li className={ClassNames("react-transfer-content-item react-transfer-left-content-item")} key={`react-transfer-left-content-item-${index}`} >
                                        <label className={ClassNames("react-transfer-content-item-content",
                                            { [`react-transfer-content-item-content-manual`]: isManualTransfer && (showCheckbox || !leftDataItem.states.isGroups) },
                                            { [`react-transfer-content-item-content-group`]: leftDataItem.states.isGroups }, { [`react-transfer-content-item-content-selecting`]: leftDataItem.states.isSelecting })} title={leftDataItem.name} onClick={this.actions.selectedLeftItem.params(leftDataItem, null)}>
                                            <If condition={isManualTransfer && showCheckbox} >
                                                <input className="react-transfer-content-item-checkbox" type="checkbox" checked={leftDataItem.states.isSelecting} readOnly />
                                            </If>
                                            <span className="react-transfer-content-item-text">
                                                {leftDataItem.name}
                                            </span>
                                        </label>
                                        <If condition={leftDataItem.children && leftDataItem.children.length > 0}>
                                            <ul className="react-transfer-content-item-children-content">
                                                {leftDataItem.children.map((child, childIndex) => {
                                                    if (child.states.isSelected) {
                                                        return null;
                                                    } else {
                                                        return (
                                                            <li className={ClassNames("react-transfer-content-item", { [`react-transfer-content-item-content-manual`]: isManualTransfer })} key={`react-transfer-left-content-child-item-${childIndex}`} >
                                                                <label className={ClassNames("react-transfer-content-item-content", { [`react-transfer-content-item-content-manual`]: isManualTransfer }, { [`react-transfer-content-item-content-selecting`]: child.states.isSelecting })} title={child.name} onClick={this.actions.selectedLeftItem.params(leftDataItem, child)}>
                                                                    <If condition={isManualTransfer && showCheckbox}>
                                                                        <input className="react-transfer-content-item-checkbox" type="checkbox" checked={child.states.isSelecting} readOnly />
                                                                    </If>
                                                                    <span className="react-transfer-content-item-text">
                                                                        {child.name}
                                                                    </span>
                                                                </label>
                                                            </li>
                                                        );
                                                    }
                                                }).filter(Boolean)}
                                            </ul>
                                        </If>
                                    </li>);
                                }
                            })}
                        </ul>
                    </div>
                </div>
                <div className="react-transfer-actions react-transfer-mid" >
                    <If condition={showMoveRightButton}>
                        <div className={ClassNames("react-transfer-actions-item", { [`react-transfer-actions-item-manual`]: isManualTransfer }, { [`react-transfer-actions-item-manual-selecting`]: isAllowedMoveToRight })} onClick={this.actions.moveToRight.stopPropagation()}>
                            <If condition={moveRightButtonIcon}>
                                <i className={ClassNames("react-transfer-move-btn-icon react-transfer-move-right-icon", { [`${moveRightButtonIcon}`]: moveRightButtonIcon })}></i>
                            </If>
                            <If condition={moveRightButtonText}>
                                <span className={ClassNames("react-transfer-move-btn-text react-transfer-move-right-text")}>
                                    {moveRightButtonText}
                                </span>
                            </If>
                        </div>
                    </If>
                    <If condition={showMoveLeftButton}>
                        <div className={ClassNames("react-transfer-actions-item", { [`react-transfer-actions-item-manual`]: isManualTransfer }, { [`react-transfer-actions-item-manual-selecting`]: isAllowedMoveToLeft })} onClick={this.actions.moveToLeft.stopPropagation()}>
                            <If condition={moveRightButtonIcon}>
                                <i className={ClassNames("react-transfer-move-btn-icon react-transfer-move-left-icon", { [`${moveLeftButtonIcon}`]: moveLeftButtonIcon })}></i>
                            </If>
                            <If condition={moveLeftButtonText}>
                                <span className={ClassNames("react-transfer-move-btn-text react-transfer-move-left-text")}>
                                    {moveLeftButtonText}
                                </span>
                            </If>
                        </div>
                    </If>
                </div>
                <div className="react-transfer-item  react-transfer-right" >
                    <p className="react-transfer-item-title">
                        {rightTitle}
                    </p>
                    <div className="react-transfer-item-container react-transfer-right-container scrollbar-y">
                        <Choose>
                            <When condition={rightDataList && rightDataList.length > 0}>
                                <ul className="react-transfer-item-content react-transfer-right-content">
                                    {rightDataList.map((rightDataItem, index) => {
                                        return (
                                            <If condition={rightDataItem.children && rightDataItem.children.length > 0}>
                                                <li className="react-transfer-content-item react-transfer-right-content-item" key={`react-transfer-right-content-item-${index}`}>
                                                    <label className={ClassNames("react-transfer-content-item-content", { [`react-transfer-content-item-content-manual`]: isManualTransfer && (showCheckbox || !rightDataItem.states.isGroups) }, { [`react-transfer-content-item-content-group`]: rightDataItem.states.isGroups }, { [`react-transfer-content-item-content-selecting`]: rightDataItem.states.isSelecting })} title={rightDataItem.name} onClick={this.actions.selectedRightItem.params(rightDataItem, null)}>
                                                        <If condition={isManualTransfer && showCheckbox}>
                                                            <input className="react-transfer-content-item-checkbox" type="checkbox" checked={rightDataItem.states.isSelecting} readOnly />
                                                        </If>
                                                        <span className="react-transfer-content-item-text">
                                                            {rightDataItem.name}
                                                        </span>
                                                    </label>
                                                    <ul className="react-transfer-content-item-children-content">
                                                        {rightDataItem.children.map((child, childIndex) => {
                                                            return (<li className={ClassNames("react-transfer-content-item")} key={`react-transfer-right-content-child-item-${childIndex}`} >
                                                                <label className={ClassNames("react-transfer-content-item-content", { [`react-transfer-content-item-content-manual`]: isManualTransfer }, { [`react-transfer-content-item-content-selecting`]: child.states.isSelecting })} title={child.name} onClick={this.actions.selectedRightItem.params(rightDataItem, child)}>
                                                                    <If condition={isManualTransfer && showCheckbox}>
                                                                        <input className="react-transfer-content-item-checkbox" type="checkbox" checked={child.states.isSelecting} readOnly />
                                                                    </If>
                                                                    <span className="react-transfer-content-item-text">
                                                                        {child.name}
                                                                    </span>
                                                                </label>
                                                            </li>);
                                                        }).filter(Boolean)}
                                                    </ul>
                                                </li>
                                            </If>
                                        );
                                    })}
                                </ul>
                            </When>
                            <Otherwise>
                                <div className="react-transfor-right-no-data">
                                    {noSelectedText}
                                </div>
                            </Otherwise>
                        </Choose>
                    </div>
                </div>
            </div>
        );
    }
}

ReactTransfer.propTypes = {
    className: PropTypes.string,
    transferMode: PropTypes.string,
    rightTitle: PropTypes.string,
    allDataItems: PropTypes.array,
    selectedItems: PropTypes.array,
    groupNameField: PropTypes.string.isRequired,
    groupValueField: PropTypes.string.isRequired,
    itemNameField: PropTypes.string.isRequired,
    itemValueField: PropTypes.string.isRequired,
    showCheckbox: PropTypes.bool,

    showMoveRightButton: PropTypes.bool,
    moveRightButtonText: PropTypes.string,
    moveRightButtonIcon: PropTypes.string,
    showMoveLeftButton: PropTypes.bool,
    moveLeftButtonText: PropTypes.string,
    moveLeftButtonIcon: PropTypes.string,
    noSelectedText: PropTypes.string,

    isVisible: PropTypes.bool,
    values: PropTypes.array,

    okText: PropTypes.string,
    cancelText: PropTypes.string,
};

ReactTransfer.defaultProps = {
    className: "",
    transferMode: "manual",
    leftTitle: "",
    rightTitle: "",
    allDataItems: [],
    selectedItems: [],
    groupNameField: "",
    groupValueField: "",
    itemNameField: "name",
    itemValueField: "id",

    showCheckbox: true,
    showMoveRightButton: true,
    moveRightButtonText: "",
    moveRightButtonIcon: "",
    showMoveLeftButton: true,
    moveLeftButtonText: "",
    moveLeftButtonIcon: "",
    noSelectedText: "Not Found",
    LeftTitle: "",
    onChanged: undefined
};

function buildLeftDataList(that, dataLists, values) {
    let leftDataList = [];
    if (dataLists && dataLists.length > 0) {
        let {
            groupNameField,
            groupValueField,
            itemNameField,
            itemValueField,
        } = that.props;

        if (itemValueField && itemNameField) {
            dataLists.forEach(dataItem => {
                let isSelected = false;

                if (values && values.length > 0) {
                    isSelected = values.some(value => value === dataItem[itemValueField]);
                }

                let leftDataItem = {
                    id: dataItem[itemValueField],
                    name: dataItem[itemNameField],
                    data: dataItem,
                    children: [],
                    states: {
                        isSelected,
                        isSelecting: false,
                        isGroups: false
                    }
                };

                groupValueField = (groupValueField !== "" || groupValueField != null) ? groupValueField : "";
                groupNameField = (groupNameField !== "" || groupNameField != null) ? groupNameField : "";

                if (groupValueField || groupNameField) {
                    let targetGroupData = leftDataList.find(group => (groupValueField !== "" && group.id === dataItem[groupValueField]) || (groupNameField !== "" && group.name === dataItem[groupNameField]));
                    if (targetGroupData) {
                        targetGroupData.children.push(leftDataItem);
                    } else {
                        let groupItem = {
                            id: groupValueField !== "" ? dataItem[groupValueField] : dataItem[groupNameField],
                            name: groupNameField !== "" ? dataItem[groupNameField] : dataItem[groupValueField],
                            data: dataItem,
                            children: [leftDataItem],
                            states: {
                                isSelected: false,
                                isSelecting: false,
                                isGroups: true
                            }
                        };
                        leftDataList.push(groupItem);
                    }
                } else {
                    leftDataList.push(leftDataItem);
                }
            });
        } else {
            throw new Error(`itemValueField or itemNameField is null in react-transfer`);
        }
    }

    leftDataList.forEach(item => {
        if (item.states.isGroups && item.children && item.children.length > 0) {
            let hasNoSelected = item.children.some(child => !child.states.isSelected);
            item.states.isSelected = !hasNoSelected;
        }
    });
    return leftDataList;
}

function buildRightDataList(that, leftDataList, values) {
    let copyLeftDataList = $.extend(true, [], leftDataList);
    let rightDataList = [];
    if (values && values.length > 0) {
        let {
            groupNameField,
            groupValueField
        } = that.props;
        groupValueField = (groupValueField !== "" || groupValueField != null) ? groupValueField : "";
        groupNameField = (groupNameField !== "" || groupNameField != null) ? groupNameField : "";

        copyLeftDataList.forEach(leftDataListItem => {
            if (groupValueField || groupNameField) {
                if (leftDataListItem.children && leftDataListItem.children.length > 0) {
                    let rightDataListItem = leftDataListItem.children.filter(child => values.some(value => value === child.id));
                    if (rightDataListItem && rightDataListItem.length > 0) {
                        leftDataListItem.children = rightDataListItem;
                        rightDataList.push(leftDataListItem);
                    }
                }
            } else {
                let hasSelectedItem = values.some(value => value === leftDataListItem.id);
                if (hasSelectedItem) {
                    rightDataList.push(leftDataListItem);
                }
            }
        });
    }
    return rightDataList;
}

function getManualChangedDataList(dataList, targetDataItem, child, showCheckbox) {
    if (targetDataItem.states.isGroups) {
        let targetDataGroup = dataList.find(item => item.id === targetDataItem.id);
        if (targetDataGroup && targetDataGroup.children && targetDataGroup.children.length > 0) {

            if (child != null) {
                //直接点击选取
                targetDataGroup.children.forEach(targetChild => {
                    if (targetChild.id === child.id) {
                        let oldValue = targetChild.states.isSelecting;
                        targetChild.states.isSelecting = !oldValue;
                    }
                });
                if (showCheckbox) {
                    let hasSelecting = targetDataGroup.children.some(child => child.states.isSelecting);
                    targetDataGroup.states.isSelecting = hasSelecting;
                }
            } else {
                //选的组，子全部勾选 
                if (showCheckbox) {
                    let hasNoSelecting = targetDataGroup.children.some(child => !child.states.isSelecting);
                    targetDataGroup.states.isSelecting = hasNoSelecting;
                    targetDataGroup.children.forEach(targetChild => {
                        targetChild.states.isSelecting = hasNoSelecting;
                    });
                } else {
                    return;
                }
            }
        } else {
            return;
        }
    } else {
        dataList.forEach(targetChild => {
            if (targetChild.id === targetDataItem.id) {
                let oldValue = targetChild.states.isSelecting;
                targetChild.states.isSelecting = !oldValue;
            }
        });
    }
    return dataList;
}

function hasIsSelectingItem(dataList) {
    if (dataList && dataList.length > 0) {
        for (let item of dataList) {
            if (item.states.isGroups) {
                if (item.children && item.children.length > 0) {
                    let hasChildSelecting = item.children.some(child => child.states.isSelecting);
                    if (hasChildSelecting) {
                        return true;
                    }
                }
            } else {
                if (item.states.isSelecting) {
                    return true;
                }
            }
        }
    }
    return false;
}

function getSelectedItems(that, dataList) {
    let selectedItems = [];
    if (dataList && dataList.length > 0) {
        let {
            itemValueField
        } = that.props;
        dataList.forEach(item => {
            if (item.states.isGroups) {
                if (item.children && item.children.length > 0) {
                    let childItems = item.children.map(child => child.data[itemValueField]);
                    selectedItems.push(...childItems);
                }
            } else {
                selectedItems.push(item.data[itemValueField]);
            }
        });
    }
    return selectedItems;
}

export default Udesk.react.udeskify(ReactTransfer);