import React from 'react';
import PropTypes from 'prop-types';
import { get } from "udesk-web-toolkits/src/udesk/utils/object";

export default class ReactTransversalSelectComponent extends React.Component {
    static defaultProps = {
        value: null,
        nameField: "name",
        valueField: "id",
        outputMode: "value", // 输出value的模式，value或object
        classNames: "",
        dataSource: [],
        sortable: true,
        leftSelectHeaderText: "",
        rightSelectHeaderText: "",
        toRightIconClass: "udesk-react-iconfont icon-udesk-react-icon_zhankai",
        toLeftIconClass: "udesk-react-iconfont icon-udesk-react-icon_shouqi",
        sortUpIconClass: "udesk-react-iconfont icon-udesk-react-zhengxu",
        sortDownIconClass: "udesk-react-iconfont icon-udesk-react-daoxu",
        onChanged: null,
        extraCheckbox: false, // 是否在右侧添加额外checkbox。如果还要扩展，可以支持传入一个组件，然后通过其onChange事件接受输出，合并到最终对象上
        extraCheckboxTitle: "", // 额外checkbox的title
        extraCheckboxField: "checked", // 用来存储checkbox状态的字段名
        onRef: null, // 用此方法将此组件this传递给父组件
    }

    static propTypes = {
        value: PropTypes.array,
        nameField: PropTypes.string,
        valueField: PropTypes.string,
        classNames: PropTypes.string,
        dataSource: PropTypes.array,
        sortable: PropTypes.bool,
        leftSelectHeaderText: PropTypes.string,
        rightSelectHeaderText: PropTypes.string,
        toRightIconClass: PropTypes.string,
        toLeftIconClass: PropTypes.string,
        onChanged: PropTypes.func,
        onRef: PropTypes.func,
    }

    privates = {
        leftActiveItem: null,
        rightActiveItem: null
    }

    static computesUsedActions = ["isEqual"]
    static computes = {
        leftDataSource: function ({ props, state, privates, locales, actions }) {
            let {
                dataSource,
                value,
            } = props;
            if (dataSource == null || !(dataSource instanceof Array)) {
                dataSource = [];
            }
            if (value == null || !(value instanceof Array)) {
                value = [];
            }
            return dataSource.filter(item => !value.some(v => actions.isEqual(v, item)));
        },
        rightDataSource: function ({ props, state, privates, locales, actions }) {
            let {
                dataSource,
                value,
                extraCheckbox,
                extraCheckboxField,
            } = props;
            if (dataSource == null || !(dataSource instanceof Array)) {
                dataSource = [];
            }
            if (value == null || !(value instanceof Array)) {
                value = [];
            }
            let rightDataSource = [];
            value.forEach(v => {
                let targetItem = dataSource.find(item => {
                    return actions.isEqual(v, item);
                });
                if (targetItem != null) {
                    if (extraCheckbox) {
                        targetItem[extraCheckboxField] = v[extraCheckboxField];
                    }
                    rightDataSource.push(targetItem);
                }
            });
            return rightDataSource;
        },
    }

    componentDidMount() {
        let {
            onRef
        } = this.props;
        if (onRef && onRef instanceof Function) {
            onRef(this);
        }
    }

    actions = {
        resetLeftItem() {
            this.privates.leftActiveItem = null;
            this.actions.update();
        },
        leftItemClick(item) {
            this.privates.leftActiveItem = item;
            this.actions.update();
        },
        rightItemClick(item) {
            this.privates.rightActiveItem = item;
            this.actions.update();
        },
        extraCheckboxChanged(item, index, e) {
            let {
                value,
                extraCheckboxField,
            } = this.props;
            if (value && value.length > index) {
                value[index][extraCheckboxField] = e.target.checked;
            }
            this.actions.triggerChange(value, item);
            this.actions.update();
        },
        toRight() {
            let leftActiveItem = this.privates.leftActiveItem;
            if (leftActiveItem == null) {
                return;
            }
            let {
                value,
            } = this.props;
            if (value == null || !(value instanceof Array)) {
                value = [];
            }
            value = value.concat([leftActiveItem]);
            //寻找下一个leftActiveItem
            let {
                leftDataSource
            } = this.privates.computes;
            let prevLectActiveItemIndex = leftDataSource.findIndex(item => this.actions.getItemValue(item) === this.actions.getItemValue(leftActiveItem));
            if (prevLectActiveItemIndex !== -1) {
                if (prevLectActiveItemIndex === leftDataSource.length - 1) {
                    if (leftDataSource.length === 1) {
                        this.privates.leftActiveItem = null;
                    } else {
                        this.privates.leftActiveItem = leftDataSource[0];
                    }
                } else {
                    this.privates.leftActiveItem = leftDataSource[prevLectActiveItemIndex + 1];
                }
                this.actions.update();
                this.actions.triggerChange(value, leftActiveItem);
            }
        },
        toLeft() {
            let rightActiveItem = this.privates.rightActiveItem;
            if (rightActiveItem == null) {
                return;
            }
            let {
                value,
                extraCheckbox,
                extraCheckboxField,
            } = this.props;
            if (value == null || !(value instanceof Array)) {
                value = [];
            }
            // 删除extraCheckbox状态，否则再次选择时，会恢复之前的状态
            if (extraCheckbox) {
                delete rightActiveItem[extraCheckboxField];
            }
            value = value.filter(item => !this.actions.isEqual(item, rightActiveItem));
            //寻找下一个rightActiveItem
            let {
                rightDataSource
            } = this.privates.computes;
            let prevLectActiveItemIndex = rightDataSource.findIndex(item => this.actions.getItemValue(item) === this.actions.getItemValue(rightActiveItem));
            if (prevLectActiveItemIndex !== -1) {
                if (prevLectActiveItemIndex === rightDataSource.length - 1) {
                    if (rightDataSource.length === 1) {
                        this.privates.rightActiveItem = null;
                    } else {
                        this.privates.rightActiveItem = rightDataSource[0];
                    }
                } else {
                    this.privates.rightActiveItem = rightDataSource[prevLectActiveItemIndex + 1];
                }
                this.actions.update();
                this.actions.triggerChange(value, rightActiveItem);
            }
        },
        sortUp() {
            let rightActiveItem = this.privates.rightActiveItem;
            if (rightActiveItem == null) {
                return;
            }
            let {
                value,
            } = this.props;
            if (value == null || !(value instanceof Array)) {
                value = [];
            }
            let rightActiveItemIndex = value.findIndex(item => this.actions.isEqual(item, rightActiveItem));
            if (rightActiveItemIndex !== -1 && rightActiveItemIndex !== 0) {
                value.splice(rightActiveItemIndex, 1);
                value.splice(rightActiveItemIndex - 1, 0, this.actions.getItemValue(rightActiveItem));
                this.actions.triggerChange(value, rightActiveItem);
            }
        },
        sortDown() {
            let rightActiveItem = this.privates.rightActiveItem;
            if (rightActiveItem == null) {
                return;
            }
            let {
                value,
            } = this.props;
            if (value == null || !(value instanceof Array)) {
                value = [];
            }
            let rightActiveItemIndex = value.findIndex(v => this.actions.isEqual(v, rightActiveItem));
            if (rightActiveItemIndex !== -1 && rightActiveItemIndex !== (value.length - 1)) {
                value.splice(rightActiveItemIndex, 1);
                value.splice(rightActiveItemIndex + 1, 0, this.actions.getItemValue(rightActiveItem));
                this.actions.triggerChange(value, rightActiveItem);
            }
        },
        getItemValue(item) {
            let {
                outputMode,
                valueField,
            } = this.props;
            if (outputMode === "object") {
                return item;
            }
            else {
                return get(item, valueField);
            }
        },
        isEqual(value, item) {
            let {
                outputMode,
                valueField,
            } = this.props;
            if (outputMode === "object") {
                return get(value, valueField) === get(item, valueField);
            }
            else {
                return value === get(item, valueField);
            }
        },
        triggerChange(value, triggerItem) {
            let {
                outputMode,
                valueField,
                onChanged
            } = this.props;
            if (onChanged != null) {
                if (outputMode === "object") {
                    onChanged(value, triggerItem);
                }
                else {
                    onChanged(value.map(item => get(item, valueField) || item /* item可能本身就是value了 */), get(triggerItem, valueField));
                }
            }
        },
    }
}