import React from "react";
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import Udesk from '../../../udesk/index';
import FigureGroupWidgetItem from "./item-base";
import ReactTool from '../../react-field-tool';
import Locales from "../../../udesk/locales";
import {
    BubbleLoader
} from 'react-css-loaders';

class FigureGroupWidget extends React.Component {
    privates = {
        figureGroups: []
    }
    init() {
        this.privates.figureGroups = buildFigureData(this.props);
    }
    parseProps({ props, prevProps, state, privates, isInitial }) {
        let {
            figureGroupData: oldFigureGroupData
        } = prevProps;
        let {
            figureGroupData
        } = props;
        if (!isInitial && oldFigureGroupData !== figureGroupData) {
            let figureGroups = buildFigureData(props);
            return {
                figureGroups
            };
        }
    }

    actions = {
        collapseGroupListToggle(groupItem) {
            let {
                figureGroups
            } = this.privates;
            figureGroups.forEach(item => {
                if (item.title === groupItem.title) {
                    item.states.isCollapse = !item.states.isCollapse;
                }
            });
            this.privates.figureGroups = figureGroups;
            this.actions.update();
        },
        onBodyClick(groupItem) {
            let {
                onBodyClick,
            } = this.props;
            if (typeof onBodyClick === "function") {
                onBodyClick(groupItem);
            }
        },
    }

    render() {
        let {
            classNames,
            reportTitleText,
            enableReportTitle,
            enalbeTollTip,
            tooltipText,
            isLoading,
            loadingColor,
            loadingDuration,
            loadingSize,
            loadingComponent: LoadingComponent,
            isForceRowNums,
            rowNums,
            expandText,
            collapseText,
            expandIconClass,
            collapseIconClass,
            showGroupTitle,
            CustomReactTool,
            CustomReactToolExportUrl,
            headerComponent,
            contentRowComponent,
        } = this.props;

        let contentStyle = {};
        if (isForceRowNums) {
            contentStyle = {
                width: `calc(100% / ${rowNums})`
            };
        }

        let {
            figureGroups
        } = this.privates;

        return (
            <div className={ClassNames("component-figure-group-widget", { [`${classNames}`]: classNames })}>
                <If condition={enableReportTitle}>
                    <div className="figure-group-widget-header">
                        <p className="figure-group-widget-hearder-title">
                            {reportTitleText}
                        </p>
                        <If condition={enalbeTollTip}>
                            <If condition={!CustomReactTool}>
                                <ReactTool tooltipData={tooltipText} />
                            </If>
                            <If condition={CustomReactTool}>
                                <CustomReactTool tooltipData={tooltipText} tooltipExportUrl={CustomReactToolExportUrl} />
                            </If>
                        </If>
                    </div>
                </If>
                <div className={ClassNames("figure-group-widget-content", { "figure-group-widget-content-auto": !isForceRowNums })}>
                    <Choose>
                        <When condition={isLoading}>
                            <If condition={LoadingComponent}>
                                <LoadingComponent color={loadingColor} duration={loadingDuration} size={loadingSize} />
                            </If>
                            <If condition={!LoadingComponent}>
                                <BubbleLoader color={loadingColor} duration={loadingDuration} size={loadingSize} />
                            </If>
                        </When>
                        <When condition={!isLoading}>
                            <If condition={figureGroups && figureGroups.length > 0}>
                                <For each="groupItem" index="index" of={figureGroups}>
                                    <div style={contentStyle} className="figure-group-widget-content-item-wrap" key={`figure-group-widget-content-item-wrap-${index}`}>
                                        <FigureGroupWidgetItem
                                            itemHeaderVisible={this.props.itemHeaderVisible}
                                            itemBodyVisible={this.props.itemBodyVisible}
                                            isCollapse={groupItem.states.isCollapse}
                                            collapseGroupListToggle={this.actions.collapseGroupListToggle}
                                            onBodyClick={this.actions.onBodyClick}
                                            classNames="figure-group-widget-content-item"
                                            groupItem={groupItem}
                                            showGroupTitle={showGroupTitle}
                                            expandText={expandText}
                                            collapseText={collapseText}
                                            expandIconClass={expandIconClass}
                                            collapseIconClass={collapseIconClass}
                                            headerComponent={headerComponent}
                                            contentRowComponent={contentRowComponent} />
                                    </div>
                                </For>
                            </If>
                        </When>
                    </Choose>
                </div>
            </div>
        );
    }
}

function getFigureData(figureGroupData) {
    const ORIGINAL = "_original";
    let dataLists = [];

    if (figureGroupData) {
        let indicators = [];
        if (figureGroupData.header && figureGroupData.header.indicators) {
            indicators = figureGroupData.header.indicators;
        }
        let row = [];
        if (figureGroupData.rows && figureGroupData.rows.length > 0 && indicators.length > 0) {
            row = figureGroupData.rows[0];
        }

        if (indicators.length > 0) {
            indicators.forEach(indicator => {
                let fieldValueOrigin = `${indicator.fieldKey}${ORIGINAL}`;
                let value = 0;
                if (row != null) {
                    if (row[indicator.fieldKey] != null) {
                        value = row[indicator.fieldKey];
                    } else if (row[indicator[fieldValueOrigin]] != null) {
                        value = row[indicator[fieldValueOrigin]];
                    }
                }

                let item = {
                    field: indicator,
                    name: indicator.fieldTitle,
                    value,
                };
                dataLists.push(item);
            });
        }
    }
    return dataLists;
}

function buildFigureData(props) {
    let {
        allowCollapseLength,
        figureGroupData,
        language
    } = props;

    let dataLists = getFigureData(figureGroupData);
    let groupsItems = [];
    let groupNames = [];
    if (dataLists && dataLists.length > 0) {
        dataLists.forEach(list => {
            let matchIndex = groupNames.findIndex(name => name === list.field.group);
            if (matchIndex > -1) {
                if (groupsItems[matchIndex].contentItems.length >= allowCollapseLength) {
                    groupsItems[matchIndex].moreItems.push(list);
                } else {
                    groupsItems[matchIndex].contentItems.push(list);
                }
            } else {
                groupNames.push(list.field.group);
                let groupItem = {
                    title: list.field.group,
                    headerItem: list,
                    contentItems: [],
                    moreItems: [],
                    states: {
                        isCollapse: false,
                        isShowTitle: list.field.isTitle,// show title
                    }
                };
                groupsItems.push(groupItem);
            }
        });
    }

    let tooltips = [];
    let rows = [];
    if (figureGroupData && figureGroupData.header && figureGroupData.header.tooltips) {
        tooltips = figureGroupData.header.tooltips;
        rows = figureGroupData.rows;
    }
    if (tooltips.length && groupsItems.length && rows.length) {
        groupsItems = groupsItems.map((items) => {
            let hasFieldKeyTooltip = tooltips.find((tooltip) => {
                return tooltip.group === items.headerItem.field.fieldKey;
            });
            if (hasFieldKeyTooltip) {
                let overallAverageScore = rows[0][hasFieldKeyTooltip.fieldKey];
                let aboveAverage = false;
                let compareTitle = Locales.get("components.figureGroupWidget.belowAverage", language);
                let compareValue = parseFloat(overallAverageScore) - parseFloat(items.headerItem.value);
                if (compareValue > 0) {
                    aboveAverage = true;
                    compareTitle = Locales.get("components.figureGroupWidget.aboveAverage", language);
                }
                items.headerItem.aboveAverage = aboveAverage;
                items.headerItem.tooltip = {
                    title: items.headerItem.name,
                    contrast: [{
                        title: hasFieldKeyTooltip.fieldTitle,
                        value: overallAverageScore
                    }, {
                        title: compareTitle,
                        value: Math.abs(compareValue)
                    }]
                };
            }
            return items;
        });
    }
    return groupsItems;
}

FigureGroupWidget.propTypes = {
    classNames: PropTypes.string,
    enableReportTitle: PropTypes.bool,
    reportTitleText: PropTypes.string,
    enalbeTollTip: PropTypes.bool,
    isLoading: PropTypes.bool,
    loadingColor: PropTypes.string,
    loadingSize: PropTypes.number,
    loadingDuration: PropTypes.number,
    noReportDataText: PropTypes.string,
    itemBodyVisible: PropTypes.bool,
    itemHeaderVisible: PropTypes.bool,
    expandText: PropTypes.string,
    collapseText: PropTypes.string,
    expandIconClass: PropTypes.string,
    collapseIconClass: PropTypes.string,
    onBodyClick: PropTypes.func,
};

FigureGroupWidget.defaultProps = {
    classNames: "",
    figureGroupData: {},
    enableReportTitle: false,
    reportTitleText: "",
    enalbeTollTip: false,
    tooltipText: "",
    loadingComponent: undefined,
    isLoading: false,
    loadingColor: "#BEC2C1",
    loadingSize: 12,
    loadingDuration: 1.4,
    noReportDataText: "",
    allowCollapseLength: 5,
    itemBodyVisible: true,
    itemHeaderVisible: true,
    isForceRowNums: false,
    rowNums: 5,
    expandText: "",
    collapseText: "",
    expandIconClass: "bi-iconfont icon-bi-angle-down",
    collapseIconClass: "bi-iconfont icon-bi-angle-up",
    showGroupTitle: false,
    CustomReactTool: null,
    CustomReactToolExportUrl: "",
    onBodyClick: function () { },
};

export default Udesk.react.udeskify(FigureGroupWidget);