import React from "react";
import PropTypes from 'prop-types';
import ClassNames from 'classnames';

class FigureGroupWidgetItem extends React.Component {
    constructor() {
        super(...arguments);
        this.collapseGroupListToggle = this.collapseGroupListToggle.bind(this);
        this.onBodyClick = this.onBodyClick.bind(this);
    }

    collapseGroupListToggle() {
        let {
            collapseGroupListToggle,
            groupItem
        } = this.props;
        if (typeof collapseGroupListToggle === "function") {
            collapseGroupListToggle(groupItem);
        }
    }

    onBodyClick() {
        let {
            onBodyClick,
            groupItem
        } = this.props;
        if (typeof onBodyClick === "function") {
            onBodyClick(groupItem);
        }
    }

    // 内容的行组件
    ContentRowItem = ({ className = '', itemData, groupItem }) => {
        const {
            contentRowComponent: ContentRowComponent
        } = this.props;

        if (ContentRowComponent) {
            return (
                <ContentRowComponent
                    className={className}
                    titleClassName={'item-title'}
                    valueClassName={'item-value'}
                    data={itemData}
                    group={groupItem} />
            );
        }
        return (
            <li className={className}>
                <div className="item-title" title={itemData.name}>
                    {itemData.name}
                </div>
                <div className="item-value">
                    {itemData.value}
                </div>
            </li>
        );
    };

    render() {
        let {
            classNames,
            showGroupTitle,
            groupItem,
            isCollapse,
            expandText,
            collapseText,
            expandIconClass,
            collapseIconClass,
            itemBodyVisible,
            itemHeaderVisible,
            contrastGreaterThan,
            contrastLessThan,
            headerComponent: HeaderComponent,
        } = this.props;
        if (groupItem == null || typeof groupItem !== "object") {
            groupItem = {};
        }

        // 内容的行组件
        const ContentRowItem = this.ContentRowItem;

        let groupHeaderItem = groupItem.headerItem || {};
        let groupContentItems = groupItem.contentItems || {};
        let groupMoreItems = groupItem.moreItems || {};

        return (
            <div className={ClassNames("component-figure-group-widget-item", { [`${classNames}`]: true })}>
                <If condition={showGroupTitle}>
                    <div className={ClassNames("widget-container-title", { "hidden-box-title": !groupItem.states.isShowTitle })}>
                        {groupItem.title}
                    </div>
                </If>
                <div className="widget-container-boby" onClick={this.onBodyClick}>
                    <If condition={itemHeaderVisible}>
                        <div className="widget-container-boby-header">
                            <If condition={HeaderComponent}>
                                <HeaderComponent className={"header-title"}
                                    titleClassName={"header-value"}
                                    valueClassName={"header-value"}
                                    data={groupHeaderItem}
                                    group={groupItem} 
                                    restProps={{contrastGreaterThan}}/>
                            </If>
                            <If condition={!HeaderComponent}>
                                <div className="header-title" title={groupHeaderItem.name}>
                                    {groupHeaderItem.name}
                                </div>
                                <div className="header-value">
                                    <If condition={groupHeaderItem.tooltip}>
                                        <i className={ClassNames("above-verage-icon", { [`${contrastGreaterThan}`]: groupHeaderItem.aboveAverage, [`${contrastLessThan}`]: !groupHeaderItem.aboveAverage })}></i>
                                    </If>
                                    {groupHeaderItem.value}
                                </div>
                                <If condition={groupHeaderItem.tooltip}>
                                    <div className="tooltips">
                                        <p className="title">
                                            {groupHeaderItem.tooltip.title}
                                        </p>
                                        <ul className="content-list">
                                            <For each="contrast" index="index" of={groupHeaderItem.tooltip.contrast}>
                                                <li key={index} className="contrast-content">
                                                    <span className="title">{contrast.title}</span>
                                                    <span className="value">{contrast.value}</span>
                                                </li>
                                            </For>
                                        </ul>
                                    </div>
                                </If>
                            </If>

                        </div>
                    </If>
                    <If condition={itemBodyVisible}>
                        <ul className="widget-container-boby-content scrollbar-y">
                            <If condition={groupContentItems && groupContentItems.length > 0}>
                                {groupContentItems.map((contentItem, index) => {
                                    return (
                                        <ContentRowItem
                                            className="content-item"
                                            key={`content-item-${index}`}
                                            itemData={contentItem}
                                            groupItem={groupItem} />
                                    );
                                })}
                            </If>
                            <If condition={groupMoreItems && groupMoreItems.length > 0}>
                                <If condition={isCollapse}>
                                    <li className="expand-collapse-container expand-container" key="expand">
                                        <ul className="more-content-list">
                                            {groupMoreItems.map((moreItem, index) => {
                                                return (
                                                    <ContentRowItem
                                                        className="content-item"
                                                        key={`content-more-item-${index}`}
                                                        itemData={moreItem}
                                                        groupItem={groupItem} />
                                                );
                                            })}
                                        </ul>
                                        <div className="expand-collapse-btn" onClick={this.collapseGroupListToggle}>
                                            <span className="expand-collapse-btn-text">{collapseText}</span>
                                            <i className={`${collapseIconClass}`}></i>
                                        </div>
                                    </li>
                                </If>
                                <If condition={!isCollapse}>
                                    <li className="expand-collapse-container collapse-container" key="collapse">
                                        <div className="expand-collapse-btn" onClick={this.collapseGroupListToggle}>
                                            <span className="expand-collapse-btn-text">{expandText}</span>
                                            <i className={`${expandIconClass}`}></i>
                                        </div>
                                    </li>
                                </If>
                            </If>
                        </ul>
                    </If>
                </div>
            </div>
        );
    }
}

FigureGroupWidgetItem.propTypes = {
    classNames: PropTypes.string,
    showGroupTitle: PropTypes.bool,
    itemBodyVisible: PropTypes.bool,
    itemHeaderVisible: PropTypes.bool,
    isCollapse: PropTypes.bool,
    expandIconClass: PropTypes.string,
    collapseIconClass: PropTypes.string,
    expandText: PropTypes.string,
    collapseText: PropTypes.string,
    collapseGroupListToggle: PropTypes.func,
    onBodyClick: PropTypes.func,
    contrastGreaterThan: PropTypes.string,
    contrastLessThan: PropTypes.string,
    headerComponent: PropTypes.elementType,
    contentRowComponent: PropTypes.elementType,
};

FigureGroupWidgetItem.defaultProps = {
    groupItem: {},
    itemBodyVisible: true,
    itemHeaderVisible: true,
    classNames: "",
    showGroupTitle: false,
    isCollapse: false,
    expandText: "",
    collapseText: "",
    expandIconClass: "bi-iconfont icon-bi-angle-up",
    collapseIconClass: "bi-iconfont icon-bi-angle-down",
    contrastGreaterThan: "udesk-qc-iconfont icon-udesk-qc-up-arrow",
    contrastLessThan: "udesk-qc-iconfont icon-udesk-qc-down-arrow",
    collapseGroupListToggle: function () { },
    onBodyClick: function () { },
    headerComponent: undefined,
    contentRowComponent: undefined
};

export default FigureGroupWidgetItem;