import React from 'react';
import PropTypes from 'prop-types';
import Locales from '../../../../../udesk/locales';
import Udesk from '../../../../../udesk';

let chartConfigOptions = {
    gridBottom: 30,
    gridContainLabel: true,
    gridLeft: 0,
    gridRight: 0,
    gridTop: 10,
    height: "300",
    legendLeft: "center",
    legendOrient: "horizontal",
    legendTop: "bottom",
    reverseBarSeries: true,
    showLegend: true,
    showTooltips: false,
    showSeriesLabels: false,
    tooltipAxisPointerType: "shadow",
    tooltipBackgroundColor: "#fff",
    tooltipBorderColor: "#E3E3E3",
    tooltipBorderWidth: 1,
    tooltipConfine: true,
    tooltipEnterable: true,
    tooltipPadding: 15,
    tooltipTextColor: "#333",
    tooltipTextFontSize: 12,
    tooltipTextLineHeight: 22,
    confine: true
};

class SentimentAnalysisDetailComponent extends React.Component {
    //#region 默认属性设置
    static propTypes = {
        showHeader: PropTypes.bool,
        pageTitle: PropTypes.string,
        keyword: PropTypes.string,
        startDate: PropTypes.string,
        endDate: PropTypes.string,
        robotId: PropTypes.number,
        showSourceComponent: PropTypes.bool
    }
    static defaultProps = {
        showHeader: false,
        pageTitle: '',
        keyword: '',
        startDate: '',
        endDate: '',
        robotId: null,
        selectedSource: 0,
        showSourceComponent: false
    }
    static computes = {
        params: ["privates.startDate", "privates.endDate", "privates.keyword", function ({
            props,
            privates
        }) {
            let params = {
                statAt: [privates.startDate, privates.endDate],
                keyword: privates.keyword,
                messageType: privates.selectedSource
            };
            let robotId = props.robotId;
            if (robotId) {
                params.robotId = robotId;
            }
            return params;
        }]
    }
    //#endregion

    privates = {
        startDate: this.props.startDate,
        endDate: this.props.endDate,
        keyword: this.props.keyword,
        selectedSource: this.props.selectedSource,
        chartConfigOptions,
        keywordTrendsChartConfigOptions: Object.assign({ formatTooltip: keywordTrendsTooltip }, chartConfigOptions, { showTooltips: true }),
        emotionAnalysisChartConfigOptions: Object.assign({ formatTooltip: emotionAnalysisTooltip }, chartConfigOptions, { showTooltips: true }),
        asyncModel: {
            popularWords: null
        },
        isBackWordCloud: false,
    }
    //#region 生命周期
    init() {
        let sentimentData = JSON.parse(Udesk.browser.storage.sessionStorage.getItem("sentimentData"));
        if (sentimentData) {
            this.privates.keyword = sentimentData.keyword;
            this.privates.startDate = sentimentData.startDate;
            this.privates.endDate = sentimentData.endDate;
            this.privates.selectedSource = sentimentData.selectedSource;
        } else {
            this.trigger("goSentimentAnalysis");
        }
    }
    asyncModel(changedParams) {
        let {
            languageCode,
            urls,
            robotId,
        } = this.props;
        let {
            startDate,
            endDate,
            keyword,
            selectedSource
        } = this.privates;
        return {
            popularWords: new Promise((resolve, reject) => {
                let params = {
                    statAt: [startDate, endDate],
                    keyword,
                    messageType: selectedSource
                };
                if (robotId) {
                    params.robotId = robotId;
                }
                if (changedParams) {
                    Object.assign(params, changedParams);
                }

                this.privates.modelRequest = Udesk.ajax.post(urls.popularWordsUrl, params).then((resp) => {
                    this.privates.modelRequest = null;
                    let data = [];
                    let fieldKey = resp.data.header.dimensions[0].fieldKey;
                    if (resp.data && resp.data.rows && resp.data.rows.length > 0) {
                        data = resp.data.rows.map((row) => {
                            return row[fieldKey];
                        });
                    }
                    resolve(data);
                }, (reason) => {
                    this.privates.modelRequest = null;
                    Udesk.ui.notify.error(reason.errorMsg || this.locales.components.report.pages.sentimentAnalysis.sentimentAnalysisDetail.getPopularWordsInformationFailed);
                    reject(reason);
                });
            }),
        };
    }

    componentDidMount() { }
    componentWillUnmount() {
        if (!this.privates.isBackWordCloud) {
            Udesk.browser.storage.sessionStorage.removeItem("sentimentData");
        }
    }
    actions = {
        confirmSearch(startDate, endDate, keyword, selectedSource) {
            if (keyword == null || keyword === '' || keyword.trim() === '') {
                this.privates.isBackWordCloud = true;
                Udesk.browser.storage.sessionStorage.setItem("sentimentData", JSON.stringify({
                    keyword,
                    startDate,
                    endDate,
                    selectedSource
                }));
                this.trigger("goSentimentAnalysis");
            } else {
                this.privates.startDate = startDate;
                this.privates.endDate = endDate;
                this.privates.keyword = keyword;
                this.privates.selectedSource = selectedSource;
                this.actions.reloadAsyncModel();
            }
        },
        selectPopularWords(popularWord) {
            let {
                keyword
            } = this.privates;
            if (keyword !== popularWord) {
                this.privates.keyword = popularWord;
                this.actions.update();
            }
        },
        onSeriesClicked(params) {
            if (Array.isArray(params.value)) {
                let relevantKeyword = params.value[2];
                if (relevantKeyword !== this.privates.keyword) {
                    this.privates.keyword = relevantKeyword;
                    this.actions.update();
                }
            }
        },
    }
    //#endregion
}

function keywordTrendsTooltip(params, multipleSeries, records, seriesColumns) {
    let paramsData = params;
    if (!Array.isArray(params)) {
        paramsData = [params];
    }

    let tooltipHeader = "";
    let headerName = "";
    if (seriesColumns && seriesColumns.length > 1) {
        headerName = seriesColumns[0].name;
    }
    if (headerName) {
        tooltipHeader = `<div class="echart-tooltip-header-name">${headerName}</div>`;
    }

    return tooltipHeader + "<div class='echart-tooltip-content'>" + paramsData.map(item => {
        let seriesColumnItem = seriesColumns[item.seriesIndex];
        let seriesName = item.seriesName;
        let name = item.name;

        if (seriesName === '\u0000-') {
            // Not show '-'
            seriesName = '';
        }

        let color = item.color || '#E3E3E3';
        if (color === "auto") {
            color = '#E3E3E3';
        }

        let formattedValue = 0;
        let tooltipValueHtml = "";
        if (records && records.length > 0) {
            let record = records[item.dataIndex];
            let seriesColumn = seriesColumns[item.seriesIndex];
            let fieldKey = seriesColumn.field;
            if (fieldKey && record) {
                formattedValue = record[fieldKey].value || record[fieldKey].originalValue || 0;
            }
            let tooltipValue = record[fieldKey].tootipValue;
            if (tooltipValue == null) {
                tooltipValue = [];
            }
            if (!Array.isArray(tooltipValue)) {
                tooltipValue = [tooltipValue];
            }

            if (tooltipValue && tooltipValue.length > 0) {
                tooltipValueHtml = tooltipValue.map(tooltipItem => {
                    return `<div class="echart-tooltip-content-extra-data-item"><span style="display: inline-block;width:8px;height:8px;border-radius:50%;background-color:${color};"></span>  ${tooltipItem}</div>`;
                }).join("");
            }
        }

        if (isNaN(item.value)) {
            formattedValue = 0;
        }

        let normalTooltip = `<div class="echart-tooltip-content-item"><div class="echart-tooltip-title">${Udesk.utils.encodeHTML(name)}</div><div class="echart-tooltip-value" style="background-color:${color};">${formattedValue}</div></div><div class="echart-tooltip-content-extra-data">${tooltipValueHtml}</div>`;
        return !multipleSeries ? (seriesName ? normalTooltip : `${formattedValue}`) : (seriesColumnItem.name ? Udesk.utils.encodeHTML(`${seriesColumnItem.name} : ${formattedValue}`) : `${formattedValue}`);
    }).join('') + "</div>";
}

function emotionAnalysisTooltip(params, multipleSeries, records, seriesColumns) {
    let paramsData = params;
    if (!Array.isArray(params)) {
        paramsData = [params];
    }

    let tooltipHeader = "";
    let headerName = "";
    if (seriesColumns && seriesColumns.length > 1) {
        headerName = seriesColumns[0].name;
    }
    if (headerName) {
        tooltipHeader = `<div class="echart-tooltip-header-name">${headerName}</div>`;
    }

    return tooltipHeader + "<div class='echart-tooltip-content'>" + paramsData.map(item => {
        let seriesColumnItem = seriesColumns[item.seriesIndex];
        let seriesName = item.seriesName;
        let name = item.name;

        if (seriesName === '\u0000-') {
            // Not show '-'
            seriesName = '';
        }

        let color = item.color || '#E3E3E3';
        if (color === "auto") {
            color = '#E3E3E3';
        }

        let tooltipValueHtml = "";
        if (records && records.length > 0) {
            let record = records[item.dataIndex];
            let seriesColumn = seriesColumns[item.seriesIndex];
            let fieldKey = seriesColumn.field;

            let tooltipValue = record[fieldKey].tootipValue;
            if (tooltipValue == null) {
                tooltipValue = [];
            }
            if (!Array.isArray(tooltipValue)) {
                tooltipValue = [tooltipValue];
            }

            if (tooltipValue && tooltipValue.length > 0) {
                tooltipValueHtml = tooltipValue.map(tooltipItem => {
                    return `<div class="echart-tooltip-content-extra-data-item"><span style="display: inline-block;width:8px;height:8px;border-radius:50%;background-color:${color};"></span>  ${tooltipItem}</div>`;
                }).join("");
            }
        }

        let percentValue = item.percent;
        if (isNaN(item.percent)) {
            percentValue = (0).toFixed(2);
        }

        let normalTooltip = `<div class="echart-tooltip-content-item"><div class="echart-tooltip-title">${Udesk.utils.encodeHTML(name)}</div><div class="echart-tooltip-value" style="background-color:${color};">${percentValue}%</div></div><div class="echart-tooltip-content-extra-data">${tooltipValueHtml}</div>`;
        return !multipleSeries ? (seriesName ? normalTooltip : `${percentValue}%`) : (seriesColumnItem.name ? Udesk.utils.encodeHTML(`${seriesColumnItem.name} : ${percentValue}%`) : `${percentValue}%`);
    }).join('') + "</div>";
}


export default SentimentAnalysisDetailComponent;