import React from 'react';
import ClassNames from 'classnames';
import Udesk from '../../../../../udesk';
import Locales from '../../../../../udesk/locales';
import SentimentAnalysisDetailComponent from './component';
import SentimentAnalysisHeader from '../sentiment-analysis-header';
import { ReportTable } from '../../../table-base';
import { ReportEcharts } from '../../../report-echarts';

export default class SentimentAnalysisDetailTemplate extends SentimentAnalysisDetailComponent {
    render() {
        let {
            sdkOptions,
            languageCode,
            urls,
            pageTitle,
            showHeader,
            showSourceComponent
        } = this.props;
        let {
            startDate,
            endDate,
            keyword,
            selectedSource,
            asyncModel,
            chartConfigOptions,
            keywordTrendsChartConfigOptions,
            emotionAnalysisChartConfigOptions,
            computes
        } = this.privates;

        return (
            <div className="pages-sentiment-analysis-detail">
                <SentimentAnalysisHeader confirmSearch={this.actions.confirmSearch} startDate={startDate} endDate={endDate} keyword={keyword} title={pageTitle} showHeader={showHeader} selectedSource={selectedSource} showSourceComponent={showSourceComponent} />
                <div className="popular-words">
                    <span className="title">
                        {Locales.get("components.report.pages.sentimentAnalysis.sentimentAnalysisDetail.popularWords", languageCode)}{Locales.get("labels.colon", languageCode)}
                    </span>
                    <ul className="popular-word-list">
                        <If condition={asyncModel && asyncModel.popularWords}>
                            <For each="popularWord" index="index" of={asyncModel.popularWords}>
                                <li key={index} title={popularWord} className={ClassNames('popular-word', { 'active': popularWord === keyword })} onClick={this.actions.selectPopularWords.params(popularWord)}>
                                    {popularWord}
                                </li>
                            </For>
                        </If>
                    </ul>
                </div>
                <div className="report-box">
                    <div className="report-item">
                        <ReportTable enableTableTitle={true} tableDataUrl={urls.typicalQuestionsUrl} tableDataParams={computes.params} enableReactPager={false} freezeHeader={false} />
                    </div>
                    <div className="report-item relevant-keywords">
                        <ReportEcharts chartTitle={this.locales.components.report.pages.sentimentAnalysis.sentimentAnalysisDetail.relevantKeywords} chartType={Udesk.enums.echartTypes.customScatterPie} chartDataUrl={urls.relevantKeywordsUrl} chartDataParams={computes.params} sdkOptions={sdkOptions} chartConfigOptions={chartConfigOptions} onSeriesClicked={this.actions.onSeriesClicked} />
                    </div>
                </div>
                <div className="report-box">
                    <div className="report-item">
                        <ReportEcharts chartTitle={this.locales.components.report.pages.sentimentAnalysis.sentimentAnalysisDetail.keywordTrends} chartType={Udesk.enums.echartTypes.line} chartDataUrl={urls.keywordTrendsUrl} chartDataParams={computes.params} sdkOptions={sdkOptions} chartConfigOptions={keywordTrendsChartConfigOptions} />
                    </div>
                    <div className="report-item emotion-analysis">
                        <ReportEcharts chartTitle={this.locales.components.report.pages.sentimentAnalysis.sentimentAnalysisDetail.emotionAnalysis} chartType={Udesk.enums.echartTypes.ring} chartDataUrl={urls.emotionAnalysisUrl} chartDataParams={computes.params} sdkOptions={sdkOptions} chartConfigOptions={emotionAnalysisChartConfigOptions} />
                    </div>
                </div>
            </div>
        );
    }
}
