import React from 'react';
import PropTypes from 'prop-types';
import Udesk from '../../../../../udesk';
import Locales from '../../../../../udesk/locales';

class SentimentAnalysisHeaderComponent extends React.Component {
    //#region 默认属性设置
    static defaultProps = {
        showHeader: false,
        title: '',
        startDate: '',
        endDate: '',
        keyword: '',
        selectedSource: 0,
        showSourceComponent: false
    }
    static propTypes = {
        showHeader: PropTypes.bool,
        title: PropTypes.string,
        startDate: PropTypes.string,
        endDate: PropTypes.string,
        keyword: PropTypes.string,
        confirmSearch: PropTypes.func,
        showSourceComponent: PropTypes.bool
    }
    //#endregion
    privates = {
        startDate: this.props.startDate,
        endDate: this.props.endDate,
        format: Udesk.config.date.momentDateFormat,
        serverFormat: Udesk.config.date.momentDateTimeFormat,
        keyword: this.props.keyword,
        ranges: getRanges(this.props.languageCode),
        dateLimit: {
            months: 1
        },
        selectedSource: this.props.selectedSource
    }
    actions = {
        onDateChanged(start, end) {
            this.privates.startDate = start;
            this.privates.endDate = end;
            this.actions.update();
        },
        onSelectedSource(value) {
            this.privates.selectedSource = value;
            this.actions.update();
        },
        onKeyWordChanged(e) {
            this.privates.keyword = e.target.value;
            this.actions.update();
        },
        confirmSearch() {
            let {
                startDate,
                endDate,
                keyword,
                selectedSource
            } = this.privates;

            this.trigger("confirmSearch", startDate, endDate, keyword, selectedSource);
        }
    }
    //#region 生命周期
    componentDidMount() {
        this.actions.confirmSearch();
    }
    parseProps({ props, prevProps, state, privates, isInitial }) {
        if (!isInitial) {
            return {
                keyword: props.keyword
            };
        }
    }
}

function getRanges(languageCode, format = Udesk.config.date.momentDateFormat) {
    let ranges = {
        [Locales.get("ui.widgets.bootstrapDateRangePicker.ranges.yesterday", languageCode)]: [moment().subtract(1, 'days').startOf('day').format(format), moment().subtract(1, 'days').endOf('day').format(format)],
        [Locales.get("ui.widgets.bootstrapDateRangePicker.ranges.recentlySevenDay", languageCode)]: [moment().subtract(6, 'days').startOf('day').format(format), moment().endOf('day').format(format)],
        [Locales.get("ui.widgets.bootstrapDateRangePicker.ranges.recentlyThirtyDay", languageCode)]: [moment().subtract(29, 'days').startOf('day').format(format), moment().endOf('day').format(format)],
        [Locales.get("ui.widgets.bootstrapDateRangePicker.ranges.thisMonth", languageCode)]: [moment().startOf('month').startOf('day'), moment().endOf('month').endOf('day')],
        [Locales.get("ui.widgets.bootstrapDateRangePicker.ranges.lastMonth", languageCode)]: [moment().subtract(1, 'month').startOf('month').startOf('day').format(format), moment().subtract(1, 'month').endOf('month').endOf('day').format(format)],
        [Locales.get("ui.widgets.bootstrapDateRangePicker.ranges.recentlyMonth", languageCode)]: [moment().subtract(1, 'month').format(format), moment().format(format)],
    };
    return ranges;
}

export default SentimentAnalysisHeaderComponent;