import React from 'react';
import PropTypes from 'prop-types';
import Udesk from '../../../../../udesk';

let chartConfigOptions = {
    gridBottom: 30,
    gridContainLabel: true,
    gridLeft: 0,
    gridRight: 0,
    gridTop: 10,
    height: "300",
    legendLeft: "center",
    legendOrient: "horizontal",
    legendTop: "bottom",
    reverseBarSeries: true,
    showLegend: false,
    showTooltips: false,
    showSeriesLabels: false,
    showExtraPercentInLabels: true,
    tooltipAxisPointerType: "shadow",
    tooltipBackgroundColor: "#fff",
    tooltipBorderColor: "#E3E3E3",
    tooltipBorderWidth: 1,
    tooltipConfine: true,
    tooltipEnterable: true,
    tooltipPadding: 20,
    tooltipTextColor: "#333",
    tooltipTextFontSize: 12,
    tooltipTextLineHeight: 22,
    textStyle: {
        normal: {
            color: function () {
                return 'rgb(' + [
                    Math.round(Math.random() * 160),
                    Math.round(Math.random() * 160),
                    Math.round(Math.random() * 160)
                ].join(',') + ')';
            }
        },
        emphasis: {
            shadowBlur: 10,
            shadowColor: '#333'
        }
    }
};

class SentimentAnalysisWordCloudComponent extends React.Component {
    //#region 默认属性设置
    static propTypes = {
        showHeader: PropTypes.bool,
        pageTitle: PropTypes.string,
        goSentimentAnalysisDetail: PropTypes.func,
        urls: PropTypes.object,
        robotId: PropTypes.number,
        showSourceComponent: PropTypes.bool
    }
    static defaultProps = {
        showHeader: false,
        pageTitle: '',
        goSentimentAnalysisDetail: null,
        urls: {},
        robotId: null,
        showSourceComponent: false
    }
    //#endregion
    static computes = {
        params: ["privates.startDate", "privates.endDate", "privates.selectedSource", "privates.keyword", function ({
            props,
            state,
            privates,
            locales
        }) {
            let params = {
                statAt: [privates.startDate, privates.endDate],
                keyword: privates.keyword,
                messageType: privates.selectedSource
            };
            let robotId = props.robotId;
            if (robotId) {
                params.robotId = robotId;
            }
            return params;
        }]
    }
    privates = {
        startDate: "",
        endDate: "",
        keyword: '',
        selectedSource: 0,
        chartConfigOptions,
        wholeMoodChartConfigOptions: chartConfigOptions
    }

    //#region 生命周期
    init() {
        let sentimentData = JSON.parse(Udesk.browser.storage.sessionStorage.getItem("sentimentData"));
        let defaultDate = moment().subtract(1, "days").format(Udesk.config.date.momentDateSlashFormat);
        if (sentimentData) {
            this.privates.startDate = sentimentData.startDate || defaultDate;
            this.privates.endDate = sentimentData.endDate || defaultDate;
            this.privates.selectedSource = sentimentData.selectedSource || 0;
        } else {
            this.privates.startDate = defaultDate;
            this.privates.endDate = defaultDate;
        }
    }

    componentDidMount() { }
    componentWillUnmount() { }
    //#endregion

    actions = {
        confirmSearch(startDate, endDate, keyword, selectedSource) {
            this.privates.startDate = startDate;
            this.privates.endDate = endDate;
            this.privates.selectedSource = selectedSource;

            if (keyword) {
                Udesk.browser.storage.sessionStorage.setItem("sentimentData", JSON.stringify({
                    keyword,
                    startDate,
                    endDate,
                    selectedSource
                }));

                this.trigger("goSentimentAnalysisDetail");
            } else {
                this.actions.update();
            }
        },
        onSeriesClicked(params) {
            let keyword = params.record.xLabel;
            let {
                startDate,
                endDate,
                selectedSource
            } = this.privates;
            Udesk.browser.storage.sessionStorage.setItem("sentimentData", JSON.stringify({
                keyword,
                startDate,
                endDate,
                selectedSource
            }));

            this.trigger("goSentimentAnalysisDetail");
        },
    }
}

export default SentimentAnalysisWordCloudComponent;