import React from "react";
import PropTypes from 'prop-types';
import ClassNames from 'classnames';
import ReactTool from '../../react-field-tool';
import ReactPager from '../../react-pager';
import ReactTable from '../../react-table';
import Udesk from '../../../udesk';
import Locales from '../../../udesk/locales';

const REACT_TABLE_MAX_HEIGHT = 300;
const REACT_TABLE_BOTTOM_HEIGHT = 50;


const DEFAULT_FUNCTION = function () { };

const DEFAULT_PAGING = {
    pageNum: Udesk.config.paging.defaultPageNumber,
    pageSize: Udesk.config.paging.defaultPageSize
};

class ReportTable extends React.Component {
    "[privates]"() {
        return {
            orderField: "",
            orderTypeEnum: Udesk.enums.sortDirection.none,
            pageNum: 1,
            pageSize: 20,
            model: {
                values: null,
                tableItems: [],
                tableColumns: [],
                stackedHeaderGroups: [],
                total: 0
            }
        };
    }

    // "[modelProps]"() {
    //     return ["chartType", "tableDataUrl", "tableDataParams"];
    // }
    modelProps = ["chartType", "tableDataUrl", "tableDataParams"]

    init() {
        let {
            orderTypeEnum,
            orderField,
            enableReactPager
        } = this.props;
        if (orderField) {
            this.privates.orderField = orderField;
            this.privates.orderTypeEnum = orderTypeEnum;
        }

        if (enableReactPager) {
            this.privates.pageNum = DEFAULT_PAGING.pageNum;
            this.privates.pageSize = DEFAULT_PAGING.pageSize;
        }
    }

    getRequestUrl() {
        return this.props.tableDataUrl;
    }
    getRequestParams() {
        return this.props.tableDataParams;
    }
    getRequestPromise() {
        let that = this;
        let dataUrl = this.getRequestUrl();
        let params = this.getRequestParams();
        if (dataUrl === "" || dataUrl == null) {
            throw new Error(`dataUrl is no valid.`);
        }
        if (!params) {
            throw new Error(`params is not null.`);
        }

        return new Promise(function (resolve, reject) {
            Udesk.ajax.post(dataUrl, params).then((resp) => {
                resolve(resp);
            }, (reason) => {
                Udesk.ui.notify.error(reason.errorMsg || that.locales.business.getReportDataFailed);
                reject(reason);
            });
        });
    }
    model() {
        return this.getRequestPromise();
    }
    parseModel(model) {
        if (model == null) {
            model = {};
        }

        let {
            orderField,
            orderTypeEnum
        } = this.privates;
        let {
            hasStackedHeaderGroups,
            selectedFields
        } = this.props;

        let values = model.data;
        let tableData = buildTableColumnsData(model.data, {
            orderTypeEnum,
            orderField,
            hasStackedHeaderGroups,
            selectedFields
        });

        return {
            values: values,
            info: values ? values.info : null,
            tableItems: [].concat(values.rows),
            tableColumns: tableData.columns,
            stackedHeaderGroups: tableData.stackedHeaderGroups,
            total: model.paging ? model.paging.total : 0
        };
    }
    parseProps({ props, prevProps, state, privates, isInitial }) {
        if (!isInitial) {
            if (props.enableReactPager && this.modelProps.some(key => privates.dirtyProps[key])) {
                return {
                    pageNum: DEFAULT_PAGING.pageNum,
                    pageSize: DEFAULT_PAGING.pageSize
                };
            }
        }
    }

    componentWillUnmount() {
        this.columnContainerInstance = null;
        this.reactTableInstance = null;
    }

    "[actions]"() {
        return {
            //列排序
            sortColumnClicked(column, sortDirection) {
                if (column && sortDirection) {
                    this.privates.orderField = column.name;
                    this.privates.orderTypeEnum = sortDirection;
                    this.actions.reloadModel();
                }
                this.trigger("onSort", column, sortDirection);
            },
            pageChanged(pageNum, pageSize) {
                this.privates.pageNum = pageNum;
                this.privates.pageSize = pageSize;
                this.actions.reloadModel();
            },
            rowContentClick(item) {
                this.trigger("rowContentClick", item);
            },
            rowClick() {
                this.trigger("rowClick", ...arguments);
            }
        };
    }

    render() {
        let {
            enableTableTitle,
            TableHeaderComponent,
            tableTitleText,
            enableTableToolTip,
            tableToolTipText,
            sortable,
            language,
            noReportDataText,
            tableRowClickable,
            freezeHeader,
            horizontalScrollbar,
            verticalScrollbar,
            rowNavigationMode,
            getCellNavigationLink,
            classNames,
            enableReactPager,
            freezeColumn,
            freezeColumnNumber,
            noSortingIconClass,
            sortAscIconClass,
            sortDescIconClass,
            enableChooseColumns,
            chooseColumnsIconClass,
        } = this.props;

        if (noReportDataText == null || noReportDataText === "") {
            noReportDataText = Locales.get("components.reportTable.noReportData", language);
        }

        let {
            privates,
            actions
        } = this;
        if (privates.model == null) {
            privates.model = {};
        }

        tableTitleText = tableTitleText || (privates.model && privates.model.info && privates.model.info.title);

        return (
            <div className={ClassNames("component-report-table", { [`${classNames}`]: true })}>
                <If condition={enableTableTitle}>
                    <div className="report-table-header">
                        <Choose>
                            <When condition={TableHeaderComponent}>
                                <TableHeaderComponent />
                            </When>
                            <Otherwise>
                                <p className="report-table-header-title">
                                    {tableTitleText}
                                </p>
                            </Otherwise>
                        </Choose>
                        <If condition={enableTableToolTip}>
                            <ReactTool tooltipData={tableToolTipText} />
                        </If>
                    </div>
                </If>
                <div className="report-table-content" ref={(reactTable) => this.reactTableInstance = reactTable}>
                    <Choose>
                        <When condition={(privates.model.tableColumns && privates.model.tableColumns.length > 0 && privates.model.tableItems && privates.model.tableItems.length > 0)}>
                            <With reactTableMaxHeight={computeReactTableMaxHeight(this)}>
                                <ReactTable columns={privates.model.tableColumns}
                                    items={privates.model.tableItems}
                                    freezeHeader={freezeHeader}
                                    horizontalScrollbar={horizontalScrollbar}
                                    maxHeight={reactTableMaxHeight}
                                    verticalScrollbar={verticalScrollbar}
                                    sortable={sortable}
                                    tableRowClickable={tableRowClickable}
                                    rowNavigationMode={rowNavigationMode}
                                    rowContentClick={actions.rowContentClick}
                                    getCellNavigationLink={getCellNavigationLink}
                                    onSorted={actions.sortColumnClicked}
                                    freezeColumn={freezeColumn}
                                    freezeColumnNumber={freezeColumnNumber}
                                    stackedHeaderGroups={privates.model.stackedHeaderGroups}
                                    noSortingIconClass={noSortingIconClass}
                                    sortAscIconClass={sortAscIconClass}
                                    sortDescIconClass={sortDescIconClass}
                                    enableChooseColumns={enableChooseColumns}
                                    chooseColumnsIconClass={chooseColumnsIconClass}
                                    rowClick={actions.rowClick} />
                                <If condition={enableReactPager}>
                                    <ReactPager pageSize={privates.pageSize} total={privates.model.total} pageNum={privates.pageNum} pageChanged={actions.pageChanged} />
                                </If>
                            </With>
                        </When>
                        <When condition={(privates.model.tableItems && privates.model.tableItems.length === 0)}>
                            <div className="not-has-report-data">
                                {noReportDataText}
                            </div>
                        </When>
                    </Choose>
                </div>
            </div>
        );
    }
}

function getStackedHeaderGroups(stackedHeaderGroups, fieldItem) {
    let group = fieldItem.group;
    if (group != null) {
        let targetHeaderGroupsItem = stackedHeaderGroups.find(groupItem => groupItem.text === group);
        if (targetHeaderGroupsItem != null) {
            targetHeaderGroupsItem.columns.push(fieldItem.fieldKey);
        } else {
            let stackedHeaderGroupsItem = {
                text: group,
                stackedHeaderClass: "stacked-header-cell-item",
                columns: [fieldItem.fieldKey],
                groups: []
            };
            stackedHeaderGroups.push(stackedHeaderGroupsItem);
        }
    }
    return stackedHeaderGroups;
}
//转化表格列数据
function buildTableColumnsData(reportData, options = {}) {
    let {
        orderTypeEnum,
        orderField,
        hasStackedHeaderGroups,
        selectedFields
    } = options;

    let allColumns = [];
    let stackedHeaderGroups = [];

    if (reportData) {
        let currentOrderField = undefined;
        let orderTypeEnums = undefined;
        if (orderField && orderTypeEnum) {
            currentOrderField = orderField;
            orderTypeEnums = orderTypeEnum;
        }

        if (reportData.header && reportData.header.dimensions && reportData.header.dimensions.length > 0) {
            let dimensions = reportData.header.dimensions;
            dimensions.forEach(dimensionItem => {
                let visible = true;
                let sortDirection = null;
                if (selectedFields && selectedFields.length > 0) {
                    visible = selectedFields.some(fieldKey => dimensionItem.fieldKey === fieldKey);
                }
                if (orderTypeEnums && currentOrderField && currentOrderField === dimensionItem.fieldKey) {
                    sortDirection = orderTypeEnums;
                }

                let hasIsShowPropery = Object.prototype.hasOwnProperty.call(dimensionItem,"isShow") && ("isShow" in dimensionItem);
                //isShow 为后端进行隐藏字段
                if (hasIsShowPropery) {
                    visible = dimensionItem.isShow;
                }

                let dimensionColumns = {
                    name: `${dimensionItem.fieldKey}`,
                    caption: dimensionItem.fieldTitle,
                    visible,
                    sortable: false,
                    sortDirection,
                    get: function (item, column) {
                        let fieldKey = column.name;
                        let fieldKeyOriginal = `${fieldKey}_original`;

                        let itemValue = 0;
                        if (item != null) {
                            if (item[fieldKey] != null) {
                                itemValue = item[fieldKey];
                            } else if (item[fieldKeyOriginal] != null) {
                                itemValue = item[fieldKeyOriginal];
                            }
                        }
                        if (itemValue == null) {
                            itemValue = 0;
                        }
                        return itemValue;
                    }
                };
                if (hasStackedHeaderGroups) {
                    stackedHeaderGroups = getStackedHeaderGroups(stackedHeaderGroups, dimensionItem);
                }
                allColumns.push(dimensionColumns);
            });
        }

        if (reportData.header && reportData.header.indicators && reportData.header.indicators.length > 0) {
            let indicators = reportData.header.indicators;
            indicators.forEach(item => {
                let visible = true;
                let sortDirection = null;

                if (selectedFields && selectedFields.length > 0) {
                    visible = selectedFields.some(fieldKey => item.fieldKey === fieldKey);
                }
                if (orderTypeEnums && currentOrderField && currentOrderField === item.fieldKey) {
                    sortDirection = orderTypeEnums;
                }
                let indicatorColumns = {
                    name: `${item.fieldKey}`,
                    caption: item.fieldTitle,
                    visible,
                    sortable: true,
                    sortDirection,
                    get: function (item, column) {
                        let fieldKey = column.name;
                        let fieldKeyOriginal = `${fieldKey}_original`;

                        let itemValue = 0;
                        if (item != null) {
                            if (item[fieldKey] != null) {
                                itemValue = item[fieldKey];
                            } else if (item[fieldKeyOriginal] != null) {
                                itemValue = item[fieldKeyOriginal];
                            }
                        }
                        if (itemValue == null) {
                            itemValue = 0;
                        }
                        return itemValue;
                    },
                    hasTemplate: false,
                    getYieldContent: function (name, item, index) {
                        return "";
                    }
                };
                if (hasStackedHeaderGroups) {
                    stackedHeaderGroups = getStackedHeaderGroups(stackedHeaderGroups, item);
                }
                allColumns.push(indicatorColumns);
            });
        }
    }

    return {
        columns: allColumns,
        stackedHeaderGroups,
    };
}

function computeReactTableMaxHeight(that) {
    let {
        freezeHeader
    } = that.props;
    if (freezeHeader && that.reactTableInstance) {
        let reactTableBox = that.reactTableInstance.getBoundingClientRect(); //获取其位置信息
        let top = document.documentElement.clientTop; //处理兼容IE 2px 问题
        let realTop = reactTableBox.top - top;

        let maxHeight = document.documentElement.clientHeight - (realTop + REACT_TABLE_BOTTOM_HEIGHT);
        if (maxHeight > REACT_TABLE_MAX_HEIGHT) {
            return maxHeight;
        } else {
            return REACT_TABLE_MAX_HEIGHT;
        }
    } else {
        return null;
    }
}

ReportTable.propTypes = {
    classNames: PropTypes.string,
    language: PropTypes.string,
    enableTableTitle: PropTypes.bool,
    tableTitleText: PropTypes.string,
    enableTableToolTip: PropTypes.bool,
    loadingColor: PropTypes.string,
    loadingSize: PropTypes.number,
    loadingDuration: PropTypes.number,
    noReportDataText: PropTypes.string,
    tableRowClickable: PropTypes.bool,

    //table config
    freezeHeader: PropTypes.bool,
    horizontalScrollbar: PropTypes.bool,
    verticalScrollbar: PropTypes.bool,
    sortable: PropTypes.bool,
    rowNavigationMode: PropTypes.string,
    enableReactPager: PropTypes.bool,

    //method
    saveSelectedColumn: PropTypes.func,
    onSorted: PropTypes.func,
    pageChanged: PropTypes.func,
    noSortingIconClass: PropTypes.string,
    sortAscIconClass: PropTypes.string,
    sortDescIconClass: PropTypes.string,
    enableChooseColumns: PropTypes.bool,
    chooseColumnsIconClass: PropTypes.string,
};

ReportTable.defaultProps = {
    classNames: "",
    language: "",
    reportData: {},
    resportPaging: DEFAULT_PAGING,
    enableTableTitle: false,
    tableTitleText: "",
    enableTableToolTip: false,
    tableToolTipText: null,
    isLoading: false,
    loadingColor: "#BEC2C1",
    loadingSize: 12,
    loadingDuration: 1.4,
    noReportDataText: "",

    //table config
    freezeHeader: true,
    horizontalScrollbar: false,
    verticalScrollbar: true,
    sortable: false,
    rowNavigationMode: "click",//click,link
    tableRowClickable: false,
    orderTypeEnum: Udesk.enums.sortDirection.none,
    orderField: null,
    selectedFields: [],
    enableReactPager: true,
    freezeColumn: false,
    freezeColumnNumber: 1,
    hasStackedHeaderGroups: false,
    sortAscIconClass: "udesk-react-iconfont icon-udesk-react-sort-amount-asc",
    sortDescIconClass: "udesk-react-iconfont icon-udesk-react-sort-amount-desc",
    noSortingIconClass: "udesk-react-iconfont icon-udesk-react-sort",
    enableChooseColumns: false,
    chooseColumnsIconClass: "udesk-react-iconfont icon-udesk-react-gear",
    //method
    saveSelectedColumn: DEFAULT_FUNCTION,
    rowContentClick: null,
    onSorted: DEFAULT_FUNCTION,
    pageChanged: DEFAULT_FUNCTION,
    rowClick: null,

    TableHeaderComponent: "",
    tableDataUrl: "",
    tableDataParams: null
};

export default ReportTable;