import React from 'react';
import PropTypes from 'prop-types';
import Udesk from '../../../udesk';
import ReportEchartsComponent from '../report-echarts';
import Icon from 'antd/lib/icon';
import LoadingAnimationComponent from '../../loading-animation';
import isEqual from 'lodash/fp/isEqual';
import 'antd/lib/icon/style/index.css';

/**
 * loading组件
 * @param {Object} loadingProps 包含父组件传入的配置项
 */
function LoadingAnimation(loadingProps) {
    return (
        <div className="component-model-loading-animation">
            <div className="loading-animation-container">
                <LoadingAnimationComponent delay={0} align={'center'} verticalAlign={'middle'} />
            </div>
        </div>
    );
}

class WhiteBoardComponent extends ReportEchartsComponent {
    //#region 生命周期
    static defaultProps = {
        ...ReportEchartsComponent.defaultProps,
        classNames: '',
        dateRange: {},
        ids: [],
        chartId: '',
        chartType: null,
        dataUrl: '',
        showChartTitle: true,
        chartTitle: '',
        chartConfigOptions: {},
        stacticParams: {},
        dateFieldValue: 'statAt',
        isAddTime: false,
        loadingColor: '#BEC2C1',
        loadingSize: 12,
        loadingDuration: 1.4,
        sdkOptions: {},
        showNoDataTip: true,
        toggleBtnToTableInfo: Udesk.enums.chartToTableTypes.toTable.name,
        toggleBtnToTableIcon: () => <Icon type="table" key={'icon-table'} />,
        toggleBtnToChartInfo: '图标',
        toggleBtnToChartIcon: () => <Icon type="rollback" key={'icon-rollback'} />,
        loadingComponent: LoadingAnimation,
        formatReportResponse: null, //是否需要格式化图表请求的数据
    };

    static propTypes = {
        ...ReportEchartsComponent.propTypes,
        classNames: PropTypes.string,
        dataUrl: PropTypes.string,
        chartTitle: PropTypes.string,
        dateFieldValue: PropTypes.string,
        showChartTitle: PropTypes.bool,
        dateRange: PropTypes.object,
        ids: PropTypes.array,
        chartId: PropTypes.string,
        chartType: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
        chartConfigOptions: PropTypes.object,
        stacticParams: PropTypes.object,
        isAddTime: PropTypes.bool,
        loadingColor: PropTypes.string,
        loadingSize: PropTypes.number,
        loadingDuration: PropTypes.number,
        sdkOptions: PropTypes.object,
    };

    //#endregion
    modelProps = ['dateRange', 'ids', 'stacticParams', 'chartType', 'chartDataUrl', 'chartDataParams'];

    getRequestUrl() {
        let { dataUrl, sdkOptions, chartDataUrl } = this.props;
        if (chartDataUrl) {
            return super.getRequestUrl();
        } else {
            return Udesk.business.apiPath.concatApiPath(dataUrl, sdkOptions);
        }
    }
    getRequestParams() {
        let { dateRange, ids, stacticParams, dateFieldValue, isAddTime, chartDataParams } = this.props;
        if (chartDataParams) {
            return super.getRequestParams();
        } else {
            let params = {
                [`${dateFieldValue}`]: [dateRange.start, dateRange.end],
                ids,
            };
            if (stacticParams) {
                Object.assign(params, stacticParams);
            }
            //KM use true
            if (isAddTime) {
                params[dateFieldValue] = [`${params[dateFieldValue][0]} 00:00:00`, `${params[dateFieldValue][1]} 00:00:00`];
            }
            return params;
        }
    }

    _requestPromise = undefined;
    _requestParams = {};
    _requestUrl = undefined;
    getRequestPromise() {
        let that = this;
        let dataUrl = this.callMethod('getRequestUrl');
        let params = this.callMethod('getRequestParams');
        let options = this.callMethod('getAjaxOptions');
        if (dataUrl === '' || dataUrl == null) {
            throw new Error(`dataUrl is no valid.`);
        }
        if (!params) {
            throw new Error(`params is not null.`);
        }
        // 需要从props上传入isNeedLoader的数据触发loading，
        // 增加 onWhiteBoardRequest 方法，
        // 但是如果没有参数对比,就会触发循环update，无法停止。。。。
        // 所以又引入了 lodash 的 isEqual 方法
        if (!isEqual(params, this._requestParams) || !isEqual(dataUrl, this._requestUrl)) {
            this._requestParams = params;
            this._requestUrl = dataUrl;

            this.trigger('onWhiteBoardRequest', { state: 'start' });
            this._requestPromise = new Promise(function (resolve, reject) {
                // 测试假数据 注释
                Udesk.ajax.post(dataUrl, params, options).then(
                    function (resp) {
                        let sourceResponse = {
                            values: resp.data,
                            info: resp.data && resp.data.info,
                            noConfigReport: resp.data.info ? resp.data.info.noConfig : false,
                            paging: resp.paging,
                        };
                        let targetResponse = that.props.formatReportResponse ? that.props.formatReportResponse(resp) : sourceResponse;
                        resolve(targetResponse);
                        that.trigger('onWhiteBoardRequest', { state: 'success' });
                    },
                    function (reason) {
                        Udesk.ui.notify.error(reason.errorMsg);
                        reject(reason);
                        that.trigger('onWhiteBoardRequest', { state: 'failed' });
                    }
                );
            });
        }
        return this._requestPromise;
    }
}

export default Udesk.react.udeskify(WhiteBoardComponent);
