import React, {
  useCallback,
  useContext,
  useEffect,
  useMemo,
  useState,
} from "react";
import ScopeSwitch from "../../../components/repository-widget/scope-switch";
import CommonQuestion from "./common-question";
import RobotQuestionComp from "./robot-question";
import { RepositoryWidgetContext } from "../utilities";
import { getJSON } from "../../../utils";
import { RobotSettings } from "./robot-question/types";
import SearchInput from "../../../components/repository-widget/search-input";

interface Props {
  query: string;
  onSearch: (query: string) => void;
}

const Question: React.FC<Props> = ({ onSearch, query }) => {
  const [settings, setSettings] = useState<RobotSettings>();
  const { activeScope, onScopeChange, showRobotQuestionTab } = useContext(
    RepositoryWidgetContext
  );
  useEffect(() => {
    getJSON("/knowledge_manager_settings/robot_settings").then(
      (settings: RobotSettings) => {
        setSettings(settings);
      }
    );
  }, []);

  //系统开通了机器人，并且showRobotQuestionTab为 true 时才会显示机器人
  const enableRobot = useMemo(() => {
    return showRobotQuestionTab && settings?.km_join_udesk_robots?.is_enabled;
  }, [settings?.km_join_udesk_robots?.is_enabled, showRobotQuestionTab]);

  const _onScopeChange = useCallback(
    (scope) => {
      if (!enableRobot && scope === "robot") {
        onScopeChange?.("public");
      } else {
        onScopeChange?.(scope);
      }
    },
    [enableRobot, onScopeChange]
  );

  const mainContent = useMemo(() => {
    if (activeScope === "public" || activeScope === "personal") {
      return <CommonQuestion scope={activeScope} query={query} />;
    } else if (enableRobot) {
      return <RobotQuestionComp searchWord={query} settings={settings} />;
    } else {
      return null;
    }
  }, [activeScope, enableRobot, query, settings]);

  return (
    <>
      <ScopeSwitch
        enableRobot={enableRobot}
        value={activeScope}
        onChange={_onScopeChange}
      />
      <SearchInput onSearch={onSearch} defaultValue={query} />
      {mainContent}
    </>
  );
};

export default Question;
