import React, { useCallback, useState } from "react";
import RobotSelect from "./robot-select";
import { Spin } from "udesk-ui";
import CategorySelect from "./category-select";
import Space from 'udesk-ui/es/space';
import RobotQuestionList from "./robot-question-list";
import { RobotSettings } from "./types";
import css from './style.module.scss';

interface Props {
    searchWord: string
    settings?: RobotSettings
}

const RobotQuestionComp: React.FC<Props> = ({searchWord, settings}) => {
    const [selectedRobotId, setSelectedRobotId] = useState<number>();
    const [selectedCategoryId, setSelectedCategoryId] = useState<number>();

    const selectFirstRobot = useCallback((robots) => {
        setSelectedRobotId(prevState => {
            if (!prevState) {
                return robots[0].km_robot_id;
            }
            return prevState;
        });
    }, []);

    if (!settings) {
        return <Spin/>;
    }

    return <>
        <Space className={css.conditionWrapper}>
            <RobotSelect settings={settings}
                         value={selectedRobotId}
                         onChange={setSelectedRobotId}
                         onDataLoaded={selectFirstRobot}/>
            {selectedRobotId &&
            <CategorySelect settings={settings}
                            value={selectedCategoryId}
                            onChange={setSelectedCategoryId}
                            setDefaultValue={setSelectedCategoryId}
                            robotId={selectedRobotId}/>}
        </Space>
        {selectedRobotId && selectedCategoryId &&
        <RobotQuestionList auth={settings.auth} robotId={selectedRobotId} categoryId={selectedCategoryId}
                           searchWord={searchWord}/>}
    </>;
}

export default RobotQuestionComp;
