import React, { useCallback, useContext } from "react";
import { Button, List, Popover } from "udesk-ui";
import { useTranslation } from "react-i18next";
import { RobotQuestion } from "../types";
import FileTextTwoTone from "@ant-design/icons/FileTextTwoTone";
import DOMPurify from "dompurify";
import {
  RepositoryWidgetContext,
  robotToCommonQuestion,
} from "../../../utilities";
import css from "../style.module.scss";
interface Props {
  data: RobotQuestion;
}

const RobotQuestionItem: React.FC<Props> = ({ data }) => {
  const { t } = useTranslation();
  const { onUse, allowSendQuestion, onSend } = useContext(
    RepositoryWidgetContext
  );
  const onUseClick = useCallback(() => {
    onUse?.("question", robotToCommonQuestion(data));
  }, [data, onUse]);
  const onSendClick = useCallback(() => {
    onSend?.("question", robotToCommonQuestion(data));
  }, [data, onSend]);

  return (
    <Popover
      content={
        <div
          dangerouslySetInnerHTML={{
            __html: DOMPurify.sanitize(data.answer?.content ?? ""),
          }}
        />
      }
      overlayStyle={{ maxWidth: "300px" }}
      overlayClassName={css.robotQuestionItemPopover}
      placement="left"
    >
      <List.Item className={css["align-justify"]}>
        <div>
          <FileTextTwoTone />
          {data.content}
        </div>
        <div className={css["nowrap"]}>
          {allowSendQuestion ? (
            <Button size="small" type="link" onClick={onSendClick}>
              {t("label.send")}
            </Button>
          ) : null}
          <Button size="small" type="link" onClick={onUseClick}>
            {t("label.use")}
          </Button>
        </div>
      </List.Item>
    </Popover>
  );
};

export default RobotQuestionItem;
