import React, { useEffect, useState } from "react";
import Select, { SelectProps } from "udesk-ui/es/select";
import { getJSON } from "../../../../utils";
import { RobotSettings } from "./types";
import {csApiGetJSON} from "../../../../utils/csApi";

interface Robot {
  name: string;
  scene_list: {
    robotId: number;
    name: string;
    type: number;
    id: number;
    deleteFlag: number;
  }[];
  km_robot_id: number;
}

interface RobotListResponse {
  code: number;
  udesk_robots: Robot[];
}

type Props = SelectProps<number> & {
  settings: RobotSettings;
  onDataLoaded: (robots: Robot[]) => void;
};
const RobotSelect: React.FC<Props> = ({ settings, onDataLoaded, ...props }) => {
  const [robotList, setRobotList] = useState<Robot[]>([]);
  useEffect(() => {
    csApiGetJSON('/cs/im/agent/udesk_robots').then(res=>res.data).then((list: RobotListResponse) => {
      const visibleIds = settings?.km_join_udesk_robots?.robot_ids;
      if (visibleIds) {
        const robots =
          list?.udesk_robots.filter((i) => {
            return visibleIds.includes(i.km_robot_id);
          }) ?? [];
        onDataLoaded(robots);
        setRobotList(robots);
      }
    });
  }, [onDataLoaded, settings?.km_join_udesk_robots?.robot_ids]);

  if (!robotList.length) {
    return null;
  }

  return (
    <Select bordered={false} {...props}>
      {robotList.map((i, idx) => {
        return (
          <Select.Option value={i.km_robot_id} key={i.km_robot_id}>
            {i.name}
          </Select.Option>
        );
      })}
    </Select>
  );
};

export default RobotSelect;
