import React, { useContext, useEffect, useState } from "react";
import { Radio } from "udesk-ui";
import { useTranslation } from "react-i18next";
import css from "./style.module.scss";
import { Scope } from "./types";
import { RepositoryWidgetContext } from "./utilities";

interface Props {
  value?: Scope;
  onChange?: (value: Scope) => void;
  enableRobot?: boolean;
}

const ScopeSwitch: React.FC<Props> = ({
  value = "public",
  enableRobot = false,
  onChange,
}) => {
  const { t } = useTranslation();
  const [_value, _setValue] = useState(value);
  const { showPublicScope } = useContext(RepositoryWidgetContext);
  useEffect(() => {
    _setValue(value);
  }, [value]);

  return (
    <div className={css.scopeSwitch}>
      <Radio.Group
        onChange={(e) => onChange && onChange(e.target.value)}
        value={_value}
      >
        {showPublicScope ? (
          <Radio.Button value="public">
            {t("page.im.km.scope.public")}
          </Radio.Button>
        ) : null}
        {enableRobot ? (
          <Radio.Button value="robot">
            {t("page.im.km.scope.robot")}
          </Radio.Button>
        ) : null}
        <Radio.Button value="personal">
          {t("page.im.km.scope.personal")}
        </Radio.Button>
      </Radio.Group>
    </div>
  );

  // <div className={css.scopeSwitch}>
  //     <Button size='small' className={classNames({active: _value === 'public'})}
  //             onClick={() => {
  //                 _setValue('public');
  //                 onChange?.('public');
  //             }}>{t('page.im.km.scope.public')}</Button>
  //     <Button size='small' className={classNames({active: _value === 'personal'})}
  //             onClick={() => {
  //                 _setValue('personal');
  //                 onChange?.('personal');
  //             }}>{t('page.im.km.scope.personal')}</Button>
  // </div>
};

export default ScopeSwitch;
