import React, {useCallback, useEffect, useMemo, useState} from "react";
import {Input} from "udesk-ui";
import SearchOutlined from '@ant-design/icons/SearchOutlined';
import CloseCircleOutlined from '@ant-design/icons/CloseCircleOutlined';
import useDebounce from 'react-use/esm/useDebounce';
import css from "./style.module.scss";
import {useTranslation} from "react-i18next";


interface Props {
    onSearch: (keyword: string) => void
    defaultValue?: string
}

const SearchInput: React.FC<Props> = ({onSearch, defaultValue}) => {
    const [keyword, setKeyword] = useState('');
    const onChange = useCallback(e => setKeyword(e.target.value), []);
    const {t} = useTranslation();

    useEffect(() => {
        setKeyword(defaultValue || '');
    }, [defaultValue]);

    useDebounce(() => {
        onSearch(keyword);
    }, 500, [onSearch, keyword]);

    const suffix = useMemo(() => {
        if (keyword) {
            return null;
        } else {
            return <SearchOutlined/>;
        }
    }, [keyword]);

    return <Input value={keyword} onChange={onChange} suffix={suffix}
                  allowClear={true}
                  placeholder={t('label.input_keyword')}
                  className={css['mgb-15']}/>;
};

export default SearchInput;
