export interface KmSetting {
    id: number
    name: string
    private_space: number[]
    sync: boolean
}

export interface ICatalog {
    id: number
    name: string
    parent_id: number
    sort: number
    subitem_count: number
    children: ICatalog[]
}

export interface ILink {
    id: number;
    name: string;
    url: string;
    knowledge_catalog_id: number;
    updated_at: string;
    sort: number;
    hit_count: number;
    custom_sort: string;
    custom_sort_all: string;
    mode?: 'mini' | null;
    miniprogram_appid?: string;
    miniprogram_path?: string;
    href?: string;
}

export interface IExpression {
    content: string
    custom_sort: string
    custom_sort_all: string
    hit_count: number
    id: string
    knowledge_catalog_id: number
    sort: number
    updated_at: string
}

export interface IQuestion {
    id: string;
    title: string;
    content: string;
    content_text: string;
    content_type: 'text' | 'html';
    knowledge_catalog_id?: number;
    lasts?: boolean;
    last_min?: any;
    last_max?: any;
    updated_at: string;
    similar_titles_count?: number;
    similar_titles?: any[];
    hit_count?: number;
    custom_sort?: string;
    custom_sort_all?: string;
    url?:string
}

export interface IAttachment {
    knowledge_attachment: {
        id: number;
        file_name: string;
        remote_file_name: string;
        file_type: string;
        size_show?: string;
        file_url: string;
        completed: boolean;
        knowledge_catalog_id: number;
        catalog_name: string;
        catalog_parent_name: string;
        updated_at: string;
        hit_count: number;
        custom_sort: string;
        custom_sort_all: string;
    }
}

type Scope = 'public' | 'personal' | 'robot';
type RepositoryType = 'useful_expression' | 'useful_link' | 'question' | 'attachment';

export interface IArticle {
    companyId: number;
    title: string;
    version: number;
    spaceId: number;
    catalogId: number;
    contentBody: string;
    deleteComplete: number;
    id: number;
    createUserId: number;
    createUserUsername: string;
    createTime: string;
    updateUserId: number;
    updateUserUsername: string;
    updateTime: string;
    deleteFlag: number;
    validityPeriodStartDate?: string;
    validityPeriodEndDate?: string;
}
