import React, { Component, ErrorInfo } from "react";
import i18n from "../../i18n";
interface Props {
  type?: "message";
}

interface State {
  hasError: boolean;
}

class SimpleErrorBoundary extends Component<Props, State> {
  constructor(props: Props) {
    super(props);
    this.state = {
      hasError: false,
    };
  }

  static getDerivedStateFromError(error: Error) {
    // 更新 state 使下一次渲染能够显示降级后的 UI
    return { hasError: true };
  }

  componentDidCatch(error: Error, errorInfo: ErrorInfo) {
    console.error("error", error);
  }
  render() {
    if (this.state.hasError) {
      return (
        <>
          {this.props.type === "message"
            ? i18n.t("msgError")
            : i18n.t("simpleError")}
        </>
      );
    }
    return this.props.children;
  }
}

export default SimpleErrorBoundary;
