import React, { useState, } from 'react';
import { Input, Row, Tooltip } from 'udesk-ui';

const {TextArea} = Input;

interface TextAreaProps {
    formData: {
        type: string,
        typeContent: string,
        help_tip_content: string, //tip提示内容
        help_tip_target: Array<string>,//客服 还是客户提示 还有内容是否为空
        input_tip: string,//输入提示 placeholder
        direct_tip_target: Array<string>,//直接提示 客服 还是客户提示 还有内容是否为空
        input_default: string,//输入框默认的值
        input_max: number,//最大长度
        // eslint-disable-next-line no-useless-escape
        input_regex?: string, //正则
        page_width: string,//页面宽度？
        sort_type?: string,//排序方式
        rows_line?: number,
        name: string,
        isRequired: boolean
    }
}

const CustomizeTextAreaComponent: React.FC<TextAreaProps> = (props => {
    const {formData} = props;
    const [errMessage, setErrMessage] = useState('');
    const [errMsgShowFlag, setErrMsgShowFlag] = useState(false);

    // 失去焦点 校验是否提示
    const hanleValidateText = (e: any) => {
        //校验是否是必填项
        let textValue = e.target.value;
        if (formData.isRequired) {
            if (textValue === '') {
                setErrMessage(() => '必填项必须输入值');
                setErrMsgShowFlag(() => true);
                return;
            } else {
                setErrMessage(() => '');
                setErrMsgShowFlag(() => false);
            }

        }
        //检验是否是纯文本

        //校验长度
        if (textValue.length > formData.input_max) {
            setErrMessage(() => '超过做大长度限制');
            setErrMsgShowFlag(() => true);
            return;
        } else {

            setErrMessage(() => '');
            setErrMsgShowFlag(() => false);

        }
    };
    return (
        <div className='box-shadow-bg'>
            <Row>
                {
                    formData.isRequired && <span style={{color: 'red'}}>*</span>
                }
                <span>{formData.name}</span>
                {
                    (formData.help_tip_content && formData.help_tip_content.length > 0) && (formData.help_tip_target && formData.help_tip_target.length > 0) &&

                    <Tooltip title={formData.help_tip_content}>
                        <i className='icon icon-caret-down'/>
                    </Tooltip>
                }
            </Row>
            <Row>

                <TextArea
                    rows={formData.rows_line}
                    defaultValue={formData.input_default}
                    placeholder={formData.input_tip}
                    onBlur={hanleValidateText}
                />
            </Row>
            <Row>
                {/* 初始没有校验提示 */}
                {errMsgShowFlag && <span>{errMessage}</span>}
            </Row>

        </div>
    );
})
// function CustomizeTextAreaComponent(props:object){

//     // eslint-disable-next-line react/prop-types

// }

export default CustomizeTextAreaComponent;
