import React, {useCallback, useEffect, useMemo, useRef, useState, ComponentProps, ElementRef} from "react";
import { Button } from "udesk-ui";
import PlusCircleOutlined from "@ant-design/icons/PlusCircleOutlined";
import EditOutlined from "@ant-design/icons/EditOutlined";
import MinusCircleOutlined from "@ant-design/icons/MinusCircleOutlined";
import { Input } from "udesk-ui";
import get from 'lodash-es/get';

function Node<T>({value, onAdd, nodeKey, onClick, onChange, onDelete, nodeTitlePath, allowAddChild = () => true, allowEditTitle = () => true, allowRemoveNode = () => true}: NodeProps<T>): ReturnType<React.FC> {
    const [editing, setEditing] = useState(!get(value, nodeTitlePath));
    const inputRef = useRef<ElementRef<typeof Input>>(null);
    const [title, setTitle] = useState(get(value, nodeTitlePath));

    useEffect(() => {
        if (inputRef.current && editing) {
            inputRef.current?.focus();
        }
    }, [editing]);
    const onFinishEdit = useCallback(() => {
        onChange(title, nodeKey);
        setEditing(false);
    }, [onChange, title, nodeKey]);
    const _onClick = useCallback(() => onClick(nodeKey, value), [nodeKey, value, onClick]);
    const enableAddChild = useMemo(() => allowAddChild(value), [allowAddChild, value]);
    const enableEditTitle = useMemo(() => allowEditTitle(value), [allowEditTitle, value]);
    const enableRemoveNode = useMemo(() => allowRemoveNode(value), [allowRemoveNode, value]);
    return <>
        {editing ? <Input ref={inputRef} value={title} size='small'
                          onBlur={onFinishEdit}
                          onPressEnter={onFinishEdit}
                          onChange={(e) => setTitle(e.target.value)}/> :
            <div onClick={_onClick}>{get(value, nodeTitlePath)}</div>}
        <div>
            {enableAddChild ? <Button size='small' type='link' icon={<PlusCircleOutlined/>}
                                      onClick={() => onAdd(value, nodeKey)}/> : null}
            {enableEditTitle ? <Button size='small' type='link' icon={<EditOutlined/>}
                                       onClick={() => setEditing(prevState => !prevState)}/> : null}
            {enableRemoveNode ? <Button size='small' type='link' danger icon={<MinusCircleOutlined/>}
                                        onClick={() => onDelete(nodeKey)}/> : null}
        </div>
    </>
}

export default Node;
