import get from "lodash-es/get";
import set from "lodash-es/set";

type Options = {
    nodeChildrenPath: ObjectPropertyPath, nodeValuePath: ObjectPropertyPath
}

export function forEachTree<T>(treeData: T[] = [], callback: (node: T) => void, options: Pick<Options, 'nodeChildrenPath'>) {
    const {nodeChildrenPath = 'subs'} = options || {};
    treeData?.forEach(i => {
        callback(i);
        forEachTree(get(i, nodeChildrenPath), callback, options);
    });
}

export function findTreeNode<T>(treeData: T[], nodeKey: string, nodeChildrenPath: ObjectPropertyPath): T | undefined {
    if (!nodeKey) {
        return;
    }
    let result;
    nodeKey.split('-').reduce((prevValue: T[], item: string) => {
        result = prevValue[parseInt(item)];
        if (!result) {
            return [];
        }
        return get(result, nodeChildrenPath, []);
    }, treeData);
    return result;
}

/**
 * 移动树里的两个节点，会修改源数据
 * @param {Option[]} tree
 * @param {string} nodeKey
 * @param {string} targetNodeKey - 如果为空，则移动到 tree 里的 position 位置
 * @param nodeChildrenPath
 * @param {number} position
 */
export function moveTreeNode<T extends object>(tree: T[], nodeKey: string, targetNodeKey: string, nodeChildrenPath: ObjectPropertyPath, position?: number) {
    let targetNodeSubs = tree;
    if (targetNodeKey) {
        const targetNode = findTreeNode(tree, targetNodeKey, nodeChildrenPath);
        if (!targetNode) {
            return;
        }
        if (!get(targetNode, nodeChildrenPath)) {
            set(targetNode, nodeChildrenPath, []);
        }
        targetNodeSubs = get(targetNode, nodeChildrenPath);
    }
    const dragNode = deleteTreeNode(tree, nodeKey, nodeChildrenPath);

    if (!dragNode) {
        return;
    }
    if (typeof position !== 'number') {
        targetNodeSubs.push(dragNode);
    } else {
        if (
            nodeKey.substring(0, nodeKey.lastIndexOf('-')) === targetNodeKey.substring(0, targetNodeKey.lastIndexOf('-')) &&
            parseInt(nodeKey.substring(nodeKey.lastIndexOf('-'))) < position
        ) {
            position -= 1;
        }
        targetNodeSubs.splice(position, 0, dragNode);
    }
}

export function deleteTreeNode<T>(tree: T[], nodeKey: string, nodeChildrenPath: ObjectPropertyPath) {
    const parentNodeKey = nodeKey.substring(0, nodeKey.lastIndexOf('-'));
    let _parentSubs;
    if (parentNodeKey) {
        const _parentNode = findTreeNode(tree, nodeKey.substring(0, nodeKey.lastIndexOf('-')), nodeChildrenPath);
        if (!get(_parentNode, nodeChildrenPath)) {
            return;
        }
        _parentSubs = get(_parentNode, nodeChildrenPath);
    } else {
        _parentSubs = tree;
    }
    const _dragNodeIndex = parseInt(nodeKey.substring(nodeKey.length - 1));
    const [dragNode] = _parentSubs.splice(_dragNodeIndex, 1);
    return dragNode;
}

export function findItemCommon<T>(value: T[], nodeKey: string[], isFather = false) {
    let changeItem: any = value
    nodeKey.forEach((item: any) => {
        changeItem = get(changeItem, ['subs', item], changeItem[item])
    })
    return changeItem
}
