import React from 'react';

class UdeskTimePicker extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            time: '00:00',
            showPicker: false
        };

        this._leadingZero = this._leadingZero.bind(this);
        this._range = this._range.bind(this);
        this._times = this._times.bind(this);
        this.onSelectTime = this.onSelectTime.bind(this);
        this.showUdeskTimePicker = this.showUdeskTimePicker.bind(this);
        this._sendOutSideClick = this._sendOutSideClick.bind(this);
    }

    componentDidMount() {
        $('body').on('mousedown', this._sendOutSideClick);
        let {
            time
        } = this.props;
        this.setState({
            time: time
        });
    }
    componentWillUnmount() {
        $('body').off('mousedown', this._sendOutSideClick);
    }

    // componentDidUpdate(prevProps, prevState, snapshot) {
    //     this.setState({
    //         time: this.props.time
    //     });
    // }

    _sendOutSideClick(e) {
        //eslint-disable-next-line no-console
        let $el = $('.udesk-time-picker');
        if (!contains($el[0], e.target)) {
            this.setState({
                showPicker: false
            });
        }
    }
    _leadingZero(num, size) {
        let str = String(num);
        while (str.length < (size || 2)) { str = "0" + num; }
        return str;
    }
    _range(start, end) {
        let range = [],
            i;
        for (i = start; i < end; ++i) {
            range.push(this._leadingZero(i));
        }
        return range;
    }
    _times() {
        let times = [];
        let {
            format
        } = this.props;
        if (format.indexOf('HH') > -1) {
            times.push({ title: "hours", values: this._range(0, 24) });
        }
        if (format.indexOf('mm') > -1) {
            times.push({ title: "minutes", values: this._range(0, 60) });
        }
        return times;
    }
    onSelectTime(e, unit, value) {
        let {
            time
        } = this.state;
        let timeArr = time.split(':');
        let hours = timeArr[0];
        let minutes = timeArr[1];
        if (unit === "hours") {
            hours = value;
        } else {
            minutes = value;
        }
        time = hours + ':' + minutes;
        this.setState({
            time: time
        });

        let onChanged = this.props.onChanged;
        if (onChanged == null) {
            return;
        }
        if (typeof onChanged === "function") {
            onChanged(time);
        }
    }
    showUdeskTimePicker(e) {
        this.setState({
            showPicker: true
        });
    }

    render() {
        let {
            time,
            showPicker
        } = this.state;

        let times = this._times();

        return (
            <div className="udesk-time-picker">
                <div className="udesk-time-picker-input-box" onClick={this.showUdeskTimePicker}>
                    <input type="text" className="udesk-time-picker-input" value={time} readOnly />
                </div>
                {showPicker &&
                    <div className="udesk-time-picker-panel">
                        <UlItems times={times} that={this} />
                    </div>
                }
            </div>
        );
    }
}

function UlItems(props) {
    return props.times.map((time, index) => {
        return (
            <div className="udesk-time-picker-panel-select" key={index}>
                <ul className="udesk-time-picker-panel-select-ul">
                    <LiItems key={index} time={time} that={props.that} />
                </ul>
            </div>
        );
    });
}

function LiItems(props) {
    return props.time.values.map((item, index) => {
        return (
            <li className="udesk-time-picker-panel-select-ul-li" key={index} onClick={(e) => props.that.onSelectTime(e, props.time.title, item)}>
                {item}
            </li>
        );
    });
}

function contains(root, n) {
    let node = n;
    while (node) {
        if (node === root) {
            return true;
        }
        node = node.parentNode;
    }
    return false;
}

UdeskTimePicker.propTypes = {

};

UdeskTimePicker.defaultProps = {
    time: '',
    format: "HH:mm",
    onChanged: null
};

export default UdeskTimePicker;