import React, { useCallback } from "react";
import AntdUpload, { UploadProps } from "udesk-ui/es/upload";
import { message } from "udesk-ui";
import { getJSON } from "../../utils";

type Props = UploadProps & {
  accept?: string;
  // beforeUpload?: () => boolean | PromiseLike<void>,
  marking: string;
};

interface FileUploadPolicy {
  accessid: string;
  bucket: string;
  code: number;
  dir: string;
  expire: number;
  host: string;
  policy: string;
  signature: string;
  storage_policy: string;
}

const Upload: React.FC<Props> = ({ marking, children, ...props }) => {
  const beforeUpload: UploadProps["beforeUpload"] = useCallback(
    async (file, fileList) => {
      const key = file.name;
      try {
        file.policy = await getJSON(
          `/spa1/file_upload_policy?marking=${marking}&key=${key}`
        );
        return Promise.resolve();
      } catch (error:any) {
        message.error(error.response?.data?.exception?.message);
        return Promise.reject();
      }
    },
    [marking]
  );
  const uploadData = useCallback(
    (file) => ({
      policy: file.policy.policy,
      OSSAccessKeyId: file.policy.accessid,
      Signature: file.policy.signature,
      key: file.policy.dir + file.name,
    }),
    []
  );
  const actionConfig = useCallback((file) => file.policy.host, []);
  return (
    <AntdUpload
      {...props}
      action={actionConfig}
      data={uploadData}
      beforeUpload={beforeUpload}
    >
      {children}
    </AntdUpload>
  );
};

export default Upload;
