import React, { useCallback, useEffect, useState } from "react";
import { Button, Col, Modal, Row } from "udesk-ui";
import style from "./index.module.scss";
import { useTranslation } from "react-i18next";
import { CheckOutlined, CloseCircleFilled } from "@ant-design/icons";
import UploadBox from "../upload-box";

interface ModalProps {
  show: boolean;
  close: () => void;
  addImg: (imgList: UploadImgList[]) => void;
}

function UploadModal(props: ModalProps) {
  // 上传的图片列表
  const [imageFiles, setImageFiles] = useState<
    React.ComponentProps<typeof UploadBox>["value"]
  >([]);
  const [selectedImages, setSelectedImages] = useState<
    React.ComponentProps<typeof UploadBox>["value"]
  >([]);

  useEffect(() => {
    try {
      let imgList = sessionStorage.getItem("imCreateTicketKImg");
      if (imgList) {
        const imgArray: UploadImgList[] = JSON.parse(imgList);
        setImageFiles((v = []) => [
          ...v,
          ...imgArray.map((i, idx) => ({
            url: i.src,
            uid: `${0 - idx}`,
            type: "image",
            size: 0,
            name: `image-${idx}`,
          })),
        ]);
      }
    } catch (e) {
      console.error(e);
    }
  }, []);

  useEffect(() => {
    return () => {
      if (imageFiles) {
        sessionStorage.setItem(
          "imCreateTicketKImg",
          JSON.stringify(
            imageFiles.map((i) => ({
              src: i.url,
            }))
          )
        );
      }
    };
  }, [imageFiles]);

  //多语言配置
  const { t } = useTranslation();

  // 点击上传  筛选出被选择的图片   将选择的图片 插入富文本编辑
  function handleOk() {
    // props.addImg(imgList.filter(item => item.isSelect));
    if (selectedImages) {
      props.addImg(
        selectedImages
          ?.filter((i) => i.url)
          .map((i) => ({ src: i.url!, isSelect: true }))
      );
    }
  }

  // 点击取消 关闭弹窗
  function handleCancel() {
    props.close();
  }

  // 删除图片
  function deleteImg(e: any, index: number) {
    e.stopPropagation();
    setImageFiles((v) => {
      if (!v) {
        return v;
      }
      const result = [...v];
      result.splice(index, 1);
      return result;
    });
  }

  // 选择图片  显示右下角对勾
  function selectImg(imageFile: any) {
    setSelectedImages((v = []) => {
      if (v.includes(imageFile)) {
        return v.filter((i) => i !== imageFile);
      } else {
        return [...v, imageFile];
      }
    });
  }

  const onChange: React.ComponentProps<
    typeof UploadBox
  >["onChange"] = useCallback((fileList) => {
    setImageFiles(fileList);
    setSelectedImages(fileList);
  }, []);

  // 弹窗页脚
  const footer = (
    <div className={style["modal-btn"]}>
      <UploadBox
        className={style["upload-btn"]}
        {...props}
        value={imageFiles}
        marking="web.ts.ticket_reply"
        onChange={onChange}
        noAttachment={false}
        showUploadList={false}
        accept="image/jpeg,image/jpg,image/png"
      >
        <Button>{t("uploader.uploadImg")}</Button>
      </UploadBox>

      <Button key="back" onClick={handleCancel}>
        {t("cancel")}
      </Button>
      <Button key="submit" type="primary" onClick={handleOk}>
        {t("uploader.insertImg")}
      </Button>
    </div>
  );
  return (
    <div>
      <Modal
        visible={props.show}
        title={t("uploader.uploadImg")}
        onOk={handleOk}
        onCancel={handleCancel}
        footer={footer}
        width={600}
      >
        <div className={style["modal-content"]}>
          <Row gutter={[16, 24]}>
            {imageFiles?.map((item, index) => {
              return (
                <Col span={8} key={item.url}>
                  <div
                    className={style["img-wrapper"]}
                    onClick={() => selectImg(item)}
                  >
                    {/* 图片 */}
                    <img src={item.url} alt={item.url} />
                    {/* 右上角删除 */}
                    <div
                      className={style["delete-icon"]}
                      onClick={(e) => deleteImg(e, index)}
                    >
                      <CloseCircleFilled />
                    </div>
                    {/* 右下角选中 */}
                    {selectedImages?.includes(item) && (
                      <div className={style["select-icon"]}>
                        <CheckOutlined />
                      </div>
                    )}
                  </div>
                </Col>
              );
            })}
          </Row>
        </div>
      </Modal>
    </div>
  );
}

export default UploadModal;
