import React from 'react';
import Udesk from '../../udesk';
import PropTypes from 'prop-types';
import WaveSurfer from 'wavesurfer.js';

class WaveAudioComponent extends React.Component {
    //#region 默认属性设置
    static propTypes = {
        src: PropTypes.string,
        options: PropTypes.object,
        rates: PropTypes.array
    }
    static defaultProps = {
        src: "https://wavesurfer-js.org/example/split-channels/stereo.mp3",
        options: {},
        rates: ["0.5", "1.0", "1.5", "2.0"]
    }
    //#endregion

    state = {
        isShowPlayButton: true,
        duration: "00:00:00",
        currentTime: "00:00:00",
        playbackRate: "1.0",
        isShowRateOptions: false,
        volume: 0.5,
        isShowVolumeRange: false
    }
    privates = {
        waveAudioRef: React.createRef(),
        defaultOptions: {
            waveColor: '#00b38b',
            progressColor: '#00B3C6',
            barWidth: 2
        }
    }

    actions = {
        play() {
            this.wavesurfer.play();
            this.setState({
                isShowPlayButton: false
            });
        },
        pause() {
            this.wavesurfer.pause();
            this.setState({
                isShowPlayButton: true
            });
        },
        showRateOptions() {
            let {
                isShowRateOptions
            } = this.state;
            this.setState({
                isShowRateOptions: !isShowRateOptions
            });
        },
        changePlaybackRate(rate) {
            this.setState({
                playbackRate: rate,
                isShowRateOptions: false
            });
            this.wavesurfer.setPlaybackRate(rate);
        },
        changeVolume(e) {
            let newVolume = e.target.value;
            this.setState({
                volume: newVolume
            });
            this.wavesurfer.setVolume(newVolume);
        },
        showVolumeRange() {
            let {
                isShowVolumeRange
            } = this.state;
            this.setState({
                isShowVolumeRange: !isShowVolumeRange
            });
        }
    }

    //#region 生命周期
    componentDidMount() {
        let {
            src,
            options
        } = this.props;
        let {
            duration,
            currentTime
        } = this.state;
        let {
            defaultOptions
        } = this.privates;
        src = normalizeAudioSrc(src);
        this.$el = this.privates.waveAudioRef.current;
        this.$waveAudioContainer = this.$el.querySelector('.wave-audio-container');
        let wavesurferOptions = Object.assign({ container: this.$waveAudioContainer }, defaultOptions, options);
        this.wavesurfer = WaveSurfer.create(wavesurferOptions);
        this.wavesurfer.load(src);
        this.wavesurfer.on('ready', () => {
            duration = Udesk.utils.date.formatDuration(this.props.duration!=null ? this.props.duration : this.wavesurfer.getDuration());
            this.setState({
                duration
            });
        });
        this.wavesurfer.on('seek', () => {
            currentTime = Udesk.utils.date.formatDuration(this.wavesurfer.getCurrentTime());
            this.setState({
                currentTime
            });
        });
        this.wavesurfer.on('audioprocess', () => {
            currentTime = Udesk.utils.date.formatDuration(this.wavesurfer.getCurrentTime());
            this.setState({
                currentTime
            });
        });
        this.wavesurfer.on('finish', () => {
            this.setState({
                isShowPlayButton: true
            });
        });
    }
    componentWillUnmount() {
        this.wavesurfer.pause();
        this.wavesurfer.destroy();
    }
    // componentWillReceiveProps() {
    //     this.wavesurfer.destroy();
    //     let {
    //         src
    //     } = this.props;
    //     let {
    //         duration,
    //         currentTime
    //     } = this.state;
    //     src = normalizeAudioSrc(src);
    //     this.$el = ReactDOM.findDOMNode(this);
    //     this.$waveAudioContainer = this.$el.querySelector('.wave-audio-container');
    //     this.wavesurfer = WaveSurfer.create({
    //         container: this.$waveAudioContainer,
    //         waveColor: '#00b38b',
    //         progressColor: '#00B3C6',
    //         barWidth: 2
    //     });
    //     this.wavesurfer.load(src);
    //     this.wavesurfer.on('ready', () => {
    //         duration = formatDuration(this.wavesurfer.getDuration());
    //         this.setState({
    //             duration
    //         });
    //     });
    //     this.wavesurfer.on('seek', () => {
    //         currentTime = formatDuration(this.wavesurfer.getCurrentTime());
    //         this.setState({
    //             currentTime
    //         });
    //     });
    //     this.wavesurfer.on('audioprocess', () => {
    //         currentTime = formatDuration(this.wavesurfer.getCurrentTime());
    //         this.setState({
    //             currentTime
    //         });
    //     });
    //     this.wavesurfer.on('finish', () => {
    //         this.setState({
    //             isShowPlayButton: true
    //         });
    //     });
    // }
    //#endregion
}

function normalizeAudioSrc(src) {
    if (src == null) {
        return src;
    }
    const protocolRegExp = /^http:\/\/(.+?)$/i;
    if (window.location.protocol === "https:" && protocolRegExp.test(src)) {
        src = src.replace(protocolRegExp, "https://$1");
    }
    let domainRegExp = /^(http[s]?):\/\/ccrecords\.alioss\.udesk\.cn(.+?)$/i;
    if (domainRegExp.test(src)) {
        src = src.replace(domainRegExp, "$1://pro-ccrecords.oss-cn-hangzhou.aliyuncs.com$2");
    }
    return src;
}

export default WaveAudioComponent;