import React from 'react';
import { ConfigProvider } from 'udesk-ui';
import type { UdeskConfigProviderProps, UdeskContext } from 'udesk-ui/es/config-provider';
import LocaleProvider, { ANT_MARK, Locale } from '../locale-provider';
import './style/index';

interface GmConfigProviderProps extends Omit<UdeskConfigProviderProps, 'locale'> {
  locale: Locale;
}

export type { GmConfigProviderProps, UdeskContext };

const GmConfigProvider = (props: GmConfigProviderProps) => {
  const { locale, ...rest } = props || {};
  return (
    <LocaleProvider locale={locale} _ANT_MARK__={ANT_MARK}>
      <ConfigProvider {...rest} locale={locale} />
    </LocaleProvider>
  );
};

type GmConfigProviderType = typeof GmConfigProvider & typeof ConfigProvider;

const ExportComponent = GmConfigProvider as GmConfigProviderType;

copyFromAntd(GmConfigProvider, ConfigProvider);

export default ExportComponent;

function copyFromAntd<TTarget, TSource>(
  udeskComponent: TTarget,
  antdComponent: TSource,
): TTarget & TSource {
  if (!udeskComponent || !antdComponent) {
    return udeskComponent as TTarget & TSource;
  }
  Object.keys(antdComponent).forEach((k) => {
    if (k !== 'render' && (udeskComponent as any)[k] === undefined) {
      (udeskComponent as any)[k] = (antdComponent as any)[k];
    }
  });
  return udeskComponent as TTarget & TSource;
}
