import React, { useState, useEffect } from 'react';
import './style/index.css';

import { Button, notification } from 'udesk-ui';
import EditorJS from '@editorjs/editorjs';
import { EDITOR_JS_TOOLS } from './tools';

import { getEditorJsTools } from './tools';
import { createReactEditorJS } from 'react-editor-js';
import editorHtml from 'editorjs-html';

//import editorParse from 'editorjs-parser';
const ReactEditorJS = createReactEditorJS();
const { checklist, quote, linkTool, image, header, list } = EDITOR_JS_TOOLS;

const DocumentEditor: React.FC<any> = (props, ref) => {
  const CHECK_LIST_STYLE =
    'display: inline-block; flex-shrink: 0;position: relative;width: 20px;height: 20px;margin: 5px;margin-left: -7px;margin-right: 7px;border-radius: 50%;border: 1px solid #d0d0d0;background: #fff;cursor: pointer;user-select: none;';
  const CHECKED_LIST_STYLE =
    'display: inline-block; flex-shrink: 0;position: relative;width: 20px;height: 20px;margin: 5px;margin-left: -7px;margin-right: 7px;border-radius: 50%;border: 1px solid #d0d0d0;background: #fff;cursor: pointer;user-select: none;background: #388ae5;border-color: #388ae5;';
  const YES_STYLE =
    "position: absolute;top: 6px;left: 5px;width: 9px;height: 4px;border: 2px solid #fff;border-top: none;border-right: none;background: transparent;transform: rotate(-45deg);content: '';opacity: 0;";
  const {
    customImageUpload,
    customVideoUpload,
    getEditorResult,
    style,
    transformHtml,
    htmlPosition,
    readOnlyFlag,
    getInit,
    getTitle,
    defaultTitle,
    titleStyle,
    ...node
  } = props;
  const [initInfo, setDefaultInfo] = useState();
  const [configTitleValue, setConfigTitleValue] = useState('');
  const [dataHtml, setDataHtml] = useState<any>();
  const [transformData, setTransformData] = useState({ blocks: [] });
  const editorCore = React.useRef(null);
  const handleInitialize = React.useCallback((instance) => {
    editorCore.current = instance;
  }, []);
  const handleSave = React.useCallback(async () => {
    const savedData = await (editorCore.current as any).save();
    setDefaultInfo(savedData);
  }, []);
  const checklist = ({ data }: any) => {
    return `<ol>
    ${data.items.map((it: any) => {
      return `<li><input type="checkbox" checked='${it.checked}'>${it.text}</li>`;
    })}
    </ol>`;
  };
  const linkTool = ({ data }: any) => {
    return `<a href='${data.link}'></a>`;
  };
  const video = ({ data }: any) => {
    let caption = data.caption ? data.caption : 'video';
    return `<video src="${
      data.file && data.file.url ? data.file.url : data.url
    }" alt="${caption}" controls/>`;
  };
  const configTitle = (e: any) => {
    setConfigTitleValue(e.target.value);
    getTitle(e.target.value);
  };
  useEffect(() => {
    getEditorJsTools({ customImageUpload, customVideoUpload });
    if (readOnlyFlag) {
      const editParser = editorHtml();
      //   let html = editParser.parse(transformHtml || { blocks: [] });
      let html = editParser.parse(transformHtml);
      // 兼容回显图片大小
      // let htmlResp = html.map((item: string) => {
      //   if (item.includes('img') || item.includes('video')) {
      //     item = item.slice(0, 5) + "style='width:100%'" + ' ' + item.slice(5);
      //     return item;
      //   }
      //   return item;
      // });
      //setDataHtml(htmlResp.join(''));
      //处理只读模式下的多选列表的回显
      var convertedHtml = '';
      transformHtml.blocks.forEach((block: any) => {
        switch (block.type) {
          case 'checklist':
            convertedHtml += "<ul style='list-style:none'>";
            block.data.items.forEach(function (li: any) {
              convertedHtml += `<li style="display:flex;line-height: 30px;"><span style='${
                li.checked ? CHECKED_LIST_STYLE : CHECK_LIST_STYLE
              }'><i style='${li.checked ? YES_STYLE : ''}'></i></span>${li.text}</li>`;
            });
            convertedHtml += '</ul>';
            break;
        }
      });
      //setDataHtml(transformHtml.join(''));
      let newHtml = html.map((item: any) => {
        if (!(typeof item === 'string')) {
          item = convertedHtml;
        }
        return item;
      });
      setDataHtml(newHtml.join(''));
    }
    getInit(editorCore);
    //内部保留相关的获取富文本编辑组件数据的方法
    // if (initInfo) {
    //         let results = editorHtml?.().validate(initInfo);
    //         let results2 = editorHtml?.({ checklist, linkTool, video }).parse(initInfo).join('');
    //         getEditorResult({ blockjson: results2, initInfo, title: configTitleValue });
    //     }
    //保留拓展的方法
    //  defaultValue={initInfo}
    //  value={data}
    setTimeout(() => {
      setTransformData(transformHtml);
    }, 400);
  }, [transformData]);
  return (
    <div style={{ width: '100%', height: '100%', background: '#fff', color: '#000' }}>
      <span style={{ color: '#000' }}></span>
      <div style={style}>
        {' '}
        <div style={titleStyle}>
          <input
            placeholder="请输入标题"
            className="titleheader"
            disabled={readOnlyFlag}
            style={{
              height: '40px',
              outline: 'none',
              border: 'none',
              fontSize: 30,
              width: '100%',
              background: '#fff',
            }}
            onChange={(e) => configTitle(e)}
            value={defaultTitle}
          />
        </div>
        <React.Fragment>
          {readOnlyFlag ? (
            <div dangerouslySetInnerHTML={{ __html: dataHtml }} style={htmlPosition}></div>
          ) : (
            <div>
              <ReactEditorJS
                holder="custom"
                onInitialize={handleInitialize}
                autofocus={true}
                tools={getEditorJsTools({ customImageUpload, customVideoUpload })}
                defaultValue={transformData}
                value={transformData}
                onChange={handleSave}
              ></ReactEditorJS>
            </div>
          )}
        </React.Fragment>
      </div>
    </div>
  );
};
export default DocumentEditor;
