import i18n from "../i18n";

/**
 * 创建一个 CustomerError 需要定义一个 ErrorCode，
 * 然后需要在 CustomerError 里定义对应的带 static 修饰符的 CustomerError 实例，
 * 在使用的时候就可以直接判断错误类型了，例如 e === CustomerError.CellphoneDuplicated，
 * 如果需要显示默认的报错可以直接使用message属性，例如 CustomerError.CellphoneDuplicated.message
 */

enum ErrorCode {
  CUSTOMER_CELLPHONE_DUPLICATED, //客户手机号已经被使用
  CUSTOMER_EMAIL_DUPLICATED, //客户邮箱已经被使用
}

export default class CustomerError extends Error {
  private readonly messageKey: string;

  public get message() {
    return i18n.t(this.messageKey);
  }

  constructor(type: ErrorCode) {
    super();
    this.messageKey = `tip.error.customer_error.${type}`;
  }

  static CellphoneDuplicated = new CustomerError(
    ErrorCode.CUSTOMER_CELLPHONE_DUPLICATED
  );
  static EmailDuplicated = new CustomerError(
    ErrorCode.CUSTOMER_EMAIL_DUPLICATED
  );
}
