const path = require('path');
const fs = require('fs-extra');
const readLocalesFile = require('./read-locales-file');
const writeLocalesFile = require('./write-locales-file');
const core = require('./core');
const generate = require('./generate');
const utils = require('./utils/utils');
const log = require('./log');

const cwd = process.cwd();

async function run(codeFilePath, options) {
    if (
        Array.isArray(options.ignorePaths) &&
        options.ignorePaths.some((ip) => codeFilePath.includes(ip))
    ) {
        return;
    }
    const [keyPath, reg] = utils.getKeyPath(options.generatePathMap, codeFilePath);
    if (!reg) {
        return;
    }

    const localesFilePathMap = readLocalesFile(options);
    if (options.noEmit) {
        log.showInfo('多语言检测中...');
    }
    const attributes = await core(path.resolve(cwd, codeFilePath), options.formatLocalesKey);
    if (!attributes.length) {
        log.info('✅  未检测出中文，跳过生成');
        return;
    }
    if (attributes.length && options.noEmit) {
        log.error(`提交中文件：${codeFilePath}包含中文，检测未通过`);
        process.exit(1);
    }
    const translateLocales = generate.locales(attributes);

    const code = generate.code(
        attributes,
        options.formatOutPutPathInJSX,
        codeFilePath,
        keyPath,
        reg[1],
        options.autoImport,
        options.sourceMap
    );
    let errorList = [];
    const generateLocalesPath = reg[1].split('.').concat(keyPath);
    const sleep = () => {
        return new Promise((res) => {
            setTimeout(() => {
                res();
            }, 1100);
        });
    };
    const REQUEST_COUNT = 9;
    for (const [key, obj] of Object.entries(localesFilePathMap)) {
        const keys = Object.keys(translateLocales);
        let targetLocales = {};

        if (options.targetLanguageMap[key] === 'zh') {
            targetLocales = { ...translateLocales };
        } else {
            while (keys.length) {
                let queue = keys.splice(0, REQUEST_COUNT).map((i) =>
                    core
                        .bdTran(translateLocales[i], options.targetLanguageMap[key])
                        .then((res) => ({
                            key: i,
                            value: res,
                        }))
                );

                const part = await Promise.all([...queue, sleep()]);

                part.length = part.length - 1;
                part.forEach((item) => {
                    targetLocales[item.key] = item.value;
                });
            }
        }
        let current = obj;
        if (options.mode === 'namePath') {
            Object.keys(targetLocales).forEach((key) => {
                if (!current.locales[`${generateLocalesPath.join('.')}.${key}`]) {
                    current.locales[`${generateLocalesPath.join('.')}.${key}`] = targetLocales[key];
                } else {
                    errorList.push(
                        ` key冲突，需人工处理:: ${generateLocalesPath.join('.')}.${key}: ${
                            targetLocales[key]
                        }`
                    );
                }
            });
        } else {
            generateLocalesPath.forEach((path, index) => {
                path = utils.camelCase(path);
                if (!current[path]) {
                    current[path] = {};
                }
                if (index !== generateLocalesPath.length - 1) {
                    current = current[path];
                }
            });
            current[generateLocalesPath[generateLocalesPath.length - 1]] = {
                ...targetLocales,
                ...current[generateLocalesPath[generateLocalesPath.length - 1]],
            };
        }
    }

    writeLocalesFile(options, localesFilePathMap);
    fs.writeFileSync(path.resolve(cwd, codeFilePath), code);
    if (errorList.length) {
        log.info(' ');
        log.info(' ');
        log.warn('warning: ');
        log.info(errorList.toString());
    } else {
        log.success('✅  转换完成');
    }
}

// main(process.argv[2]);

module.exports = {
    run,
};
