import React, { useCallback, useContext } from 'react';
import moment from 'moment';
import { dateFormat } from '../../utils';
import { useTranslation } from 'react-i18next';
import style from './style.module.scss';
import { Col, Row } from 'udesk-ui';
import { useSelector } from '../../store/store-hook';
import { RootState } from '../../store/root-reducer';

const CcConversation: React.FC<any> = (props: any) => {
    const { t } = useTranslation();
    const initData = useSelector((state: RootState) => state.initData);
    const roleCan = useSelector((state: RootState) => state.roleCan);
    const { feed } = props;
    const sdk = initData?.sdk;
    const showDetail = useCallback(
        (item: any) => {
            initData?.event?.showDetail(item);
        },
        [initData?.event?.showDetail]
    );

    function formatDuration(time: string) {
        let duration = moment.duration(time, 'seconds');
        let minutes: any = duration.asMinutes();
        let seconds = duration.seconds();
        if (minutes >= 1) {
            return parseInt(minutes) + t('minute') + seconds + t('second');
        } else {
            return seconds + t('second');
        }
    }
    return (
        <React.Fragment>
            <div className={style['top-title']}>
                <span className={style['bar-left']}></span>
                <span className={style['type-text']}>{t('call')}</span>
                <span className={style['data-text']}>{dateFormat(feed.created_at)}</span>
                {
                    // eslint-disable-next-line jsx-a11y/anchor-is-valid
                    !sdk && feed.type == 'CcConversation' && roleCan.im_customer_contact && (
                        <a className={style['detail-text']} onClick={() => showDetail(feed)}>
                            {t('detail')}
                        </a>
                    )
                }
            </div>

            <div className={style['im-main-content']}>
                <Row>
                    <Col span={6}>
                        <label>{t('page.im.imContact.type')}:</label>
                    </Col>
                    <Col span={17} offset={1}>
                        <span title={feed.source}>{feed.call_type}</span>
                    </Col>
                </Row>
                <Row>
                    <Col span={6}>
                        <label>{t('page.im.imContact.assigneeAgent')}:</label>
                    </Col>
                    <Col span={17} offset={1}>
                        <span title={feed.agent.name}>{feed.agent.name}</span>
                    </Col>
                </Row>
                <Row>
                    <Col span={6}>
                        <label>{t('page.im.imContact.callResult')}:</label>
                    </Col>
                    <Col span={17} offset={1}>
                        <span title={feed.result}>{feed.result}</span>
                    </Col>
                </Row>
                <Row>
                    <Col span={6}>
                        <label>{t('page.im.imContact.callDuration')}:</label>
                    </Col>
                    <Col span={17} offset={1}>
                        <span>{formatDuration(feed.talking_seconds)}</span>
                    </Col>
                </Row>
            </div>
        </React.Fragment>
    );
};

export default CcConversation;
