import React, { useCallback, useContext } from "react";
import { dateFormat } from "../../utils";
import CustomFieldsShow from "./custom-fields";
import { useTranslation } from "react-i18next";
import style from "./style.module.scss";
import { Col, Row } from "udesk-ui";
import { useSelector } from "../../store/store-hook";
import { RootState } from "../../store/root-reducer";

const ImSubSession: React.FC<any> = (props: any) => {
  const { t } = useTranslation();
  const initData = useSelector((state: RootState) => state.initData);
  const roleCan = useSelector((state: RootState) => state.roleCan);
  const imNoteFields = initData.noteFields || [];
  const { feed } = props;
  const sdk = initData?.sdk;
  const showDetail = useCallback(
    (item: any) => {
      initData?.event?.showDetail(item);
    },
    [initData?.event?.showDetail]
  );

  return (
    <React.Fragment>
      <div className={style["top-title"]}>
        <span className={style["bar-left"]}></span>
        <span className={style["type-text"]}>{t("im")}</span>
        <span className={style["data-text"]}>
          {dateFormat(feed.created_at)}
        </span>
        {
          // eslint-disable-next-line jsx-a11y/anchor-is-valid
          !sdk && roleCan.im_customer_contact && (
            <a
              className={style["detail-text"]}
              onClick={() => showDetail(feed)}
            >
              {t("detail")}
            </a>
          )
        }
      </div>
      <div className={style["im-main-content"]}>
        <Row>
          <Col span={6}>
            <label style={{ color: "rgba(0, 0, 0, 0.45)" }}>
              {t("agentAccepted")}:
            </label>
          </Col>
          <Col span={17} offset={1}>
            <span>{feed.agent.nick_name}</span>
          </Col>
        </Row>
        {feed.custom_fields
          ? imNoteFields.map((field: any) => {
              return (
                <CustomFieldsShow
                  key={field.name}
                  customField={field}
                  customer=""
                  value={feed.custom_fields[field.name]}
                  imShow={true}
                />
              );
            })
          : null}
      </div>
    </React.Fragment>
  );
};

export default ImSubSession;
