import React, { useCallback } from "react";
import { dateFormat } from "../../utils";
import { useTranslation } from "react-i18next";
import style from "./style.module.scss";
import { Col, Row } from "udesk-ui";
import DOMPurify from "dompurify";
import { ArrowDown, ArrowUp, Emergent } from "../../components/icon/icon";
import { TICKET_STATUS } from "../../common/enums/ticket";
import { useSelector } from "../../store/store-hook";
import { RootState } from "../../store/root-reducer";

const Ticket: React.FC<any> = (props: any) => {
  const { t } = useTranslation();
  const initData = useSelector((state: RootState) => state.initData);

  const roleCan = useSelector((state: RootState) => state.roleCan);

  const { feed } = props;
  const sdk = initData?.sdk;
  const showDetail = useCallback(
    (item: any) => {
      initData?.event?.showDetail(item);
    },
    [initData?.event?.showDetail]
  );

  return (
    <React.Fragment>
      <div className={style["top-title"]}>
        <span className={style["bar-left"]} />
        <span className={style["type-text"]}>{t("ticket")}</span>
        <span className={style["data-text"]}>
          {dateFormat(feed.created_at)}
        </span>
        {
          // eslint-disable-next-line jsx-a11y/anchor-is-valid
          !sdk && roleCan.im_customer_contact && (
            <a
              className={style["detail-text"]}
              onClick={() => showDetail(feed)}
            >
              {t("detail")}
            </a>
          )
        }
      </div>

      <div className={style["im-main-content"]}>
        <Row>
          <Col span={6}>
            <label style={{ color: "rgba(0, 0, 0, 0.45)" }}>
              {t("page.im.imContact.subject")}:
            </label>
          </Col>
          <Col span={17} offset={1}>
            <span title={feed.subject}>{feed.subject}</span>
          </Col>
        </Row>
        <Row>
          <Col span={6}>
            <label style={{ color: "rgba(0, 0, 0, 0.45)" }}>
              {t("page.im.imContact.desc")}:
            </label>
          </Col>
          <Col span={17} offset={1}>
            <span
              title={DOMPurify.sanitize(feed.content)}
              className="ellipsis-two-line"
            >
              <span
                dangerouslySetInnerHTML={{
                  __html: DOMPurify.sanitize(feed.content),
                }}
              />
            </span>
          </Col>
        </Row>

        <Row>
          <Col span={6}>
            <label style={{ color: "rgba(0, 0, 0, 0.45)" }}>
              {t("page.im.imContact.status")}:
            </label>
          </Col>
          <Col span={17} offset={1}>
            {feed.status_id === TICKET_STATUS.OPEN && (
              <span className={style["table-pop-open"]}>{feed.status}</span>
            )}
            {feed.status_id === TICKET_STATUS.RESOLVED && (
              <span className={style["table-pop-resolved"]}>{feed.status}</span>
            )}
            {feed.status_id === TICKET_STATUS.CLOSED && (
              <span className={style["table-pop-closed"]}>{feed.status}</span>
            )}
            {feed.status_id === TICKET_STATUS.SOLVING && (
              <span className={style["table-pop-solving"]}>{feed.status}</span>
            )}
          </Col>
        </Row>
        <Row>
          <Col span={6}>
            <label style={{ color: "rgba(0, 0, 0, 0.45)" }}>
              {t("page.im.imContact.priority")}:
            </label>
          </Col>
          <Col span={17} offset={1}>
            {feed.priority_id === 1 && (
              <span className={style["text-error"]}>
                <Emergent />
                {feed.priority}
              </span>
            )}
            {feed.priority_id === 2 && (
              <span className={style["text-danger"]}>
                <ArrowUp />
                {feed.priority}
              </span>
            )}
            {feed.priority_id === 3 && (
              <span className={style["text-info"]}>
                <ArrowUp />
                {feed.priority}
              </span>
            )}
            {feed.priority_id === 4 && (
              <span className={style["text-info"]}>
                <ArrowDown />
                {feed.priority}
              </span>
            )}
          </Col>
        </Row>

        <Row>
          <Col span={6}>
            <label style={{ color: "rgba(0, 0, 0, 0.45)" }}>
              {t("page.im.imContact.assigneeName")}:
            </label>
          </Col>
          <Col span={17} offset={1}>
            <span>
              {feed.agent.agent_group_name
                ? feed.agent.agent_group_name + " / "
                : ""}
              {feed.agent.assignee_name}
            </span>
          </Col>
        </Row>
      </div>
    </React.Fragment>
  );
};

export default Ticket;
