/* eslint-disable react/prop-types */
import React, { useState } from 'react';
import { Col, Collapse, Row } from 'udesk-ui';
import { useTranslation } from 'react-i18next';
import includes from 'lodash-es/includes';
import isEqual from 'lodash-es/isEqual';
import style from './index.module.scss';
import UpOutlined from '@ant-design/icons/UpOutlined';
import DownOutlined from '@ant-design/icons/DownOutlined';
import { RootState } from '../../store/root-reducer';
import { useSelector } from '../../store/store-hook';
const { Panel } = Collapse;

interface AccessInfoProps {
    imNewPlatforms: any;
    accessInfo?: AccessInfo;
}

const AccessInfo: React.FC<AccessInfoProps> = ({ accessInfo, imNewPlatforms }) => {
    const PLATFORM_ARR = ['wechat', 'weibo', 'ios', 'android', 'weixin_mini', 'web'];
    const initData = useSelector((state: RootState) => state.initData);
    const { t } = useTranslation();
    const [key, setKey] = useState<string | string[]>('open');
    const handleChange = (key: string | string[]) => {
        setKey(key);
    };
    return (
        <div className={style['visit-info']}>
            <Collapse
                bordered={false}
                activeKey={key}
                expandIconPosition="right"
                onChange={handleChange}
                expandIcon={({ isActive }) => (isActive ? <UpOutlined /> : <DownOutlined />)}
            >
                <Panel
                    showArrow={true}
                    header={<h3 className={style['visit-info-title']}>{t('page.im.customerInfo.device.title')}</h3>}
                    key={'open'}
                >
                    {isEqual(initData.page, 'im') && accessInfo && (
                        <div className={style['info-list']}>
                            {/* 最外层accessInfohide 判断 */}
                            {accessInfo.session_code && (
                                <Row>
                                    <Col className={style['info-title']}>
                                        {t('page.im.customerInfo.device.DialogueID')}
                                    </Col>
                                    <Col className={style['info-content']} span={16}>
                                        {accessInfo.session_code}
                                    </Col>
                                </Row>
                            )}

                            {/* 判断当前平台 是否是PLATFORM_ARR中的一个 */}
                            {accessInfo.menu_names && includes(PLATFORM_ARR, accessInfo.platform) && (
                                <Row>
                                    <Col className={style['info-title']}>{t('page.im.customerInfo.device.menu')}</Col>
                                    <Col className={style['info-content']} span={16}>
                                        {accessInfo.menu_names}
                                    </Col>
                                </Row>
                            )}
                            {/* web平台 */}
                            {(accessInfo.platform === 'web' || accessInfo.platform === 'mchat') && (
                                <>
                                    {accessInfo.accessUrl && (
                                        <Row>
                                            <Col className={style['info-title']}>
                                                {t('page.im.customerInfo.device.curUrl')}
                                            </Col>
                                            <Col className={style['info-content']} span={16}>
                                                <a
                                                    href={accessInfo.cur_url}
                                                    rel="noopener noreferrer"
                                                    target={'_blank'}
                                                    title={accessInfo.cur_url}
                                                >
                                                    <span>{accessInfo.cur_url_desc}</span>
                                                    {accessInfo.cur_url_show}
                                                </a>
                                            </Col>
                                        </Row>
                                    )}
                                    {accessInfo.accessUrl && (
                                        <Row>
                                            <Col className={style['info-title']}>
                                                {t('page.im.customerInfo.device.preUrl')}
                                            </Col>
                                            <Col className={style['info-content']} span={16}>
                                                <a
                                                    href={accessInfo.accessUrl}
                                                    rel="noopener noreferrer"
                                                    target={'_blank'}
                                                    title={accessInfo.accessUrl}
                                                >
                                                    <span>{accessInfo.accessUrl_desc}</span>
                                                    {accessInfo.accessUrl_show}
                                                </a>
                                            </Col>
                                        </Row>
                                    )}

                                    {accessInfo.src && (
                                        <Row>
                                            <Col className={style['info-title']}>
                                                {t('page.im.customerInfo.device.searchType')}
                                            </Col>
                                            <Col className={style['info-content']} span={16}>
                                                <a
                                                    href={accessInfo.src_url}
                                                    rel="noopener noreferrer"
                                                    target={'_blank'}
                                                    title={accessInfo.src_url}
                                                >
                                                    <span>{accessInfo.src}</span>
                                                    {accessInfo.src_url_show}
                                                </a>
                                            </Col>
                                        </Row>
                                    )}
                                    {accessInfo.web_plugin_name && (
                                        <Row>
                                            <Col className={style['info-title']}>
                                                {t('page.im.customerInfo.device.web_plugin')}
                                            </Col>
                                            <Col className={style['info-content']} span={16}>
                                                {accessInfo.web_plugin_name}
                                            </Col>
                                        </Row>
                                    )}
                                    {accessInfo.keyword && (
                                        <Row>
                                            <Col className={style['info-title']}>{t('label.keyword')}</Col>
                                            <Col className={style['info-content']} span={16}>
                                                {accessInfo.keyword}
                                            </Col>
                                        </Row>
                                    )}
                                    {accessInfo.sysInfo && (
                                        <Row>
                                            <Col className={style['info-title']}>
                                                {t('page.im.customerInfo.device.os')}
                                            </Col>
                                            <Col className={style['info-content']} span={16}>
                                                {accessInfo.sysInfo}
                                            </Col>
                                        </Row>
                                    )}
                                    {accessInfo.bowserInfo && (
                                        <Row>
                                            <Col className={style['info-title']}>
                                                {t('page.im.customerInfo.device.browser')}
                                            </Col>
                                            <Col className={style['info-content']} span={16}>
                                                {accessInfo.bowserInfo}
                                            </Col>
                                        </Row>
                                    )}
                                    {accessInfo.channel && (
                                        <Row>
                                            <Col className={style['info-title']}>
                                                {t('page.im.customerInfo.device.customChannel')}
                                            </Col>
                                            <Col className={style['info-content']} span={16}>
                                                {accessInfo.channel}
                                            </Col>
                                        </Row>
                                    )}
                                    {accessInfo.ip && (
                                        <Row>
                                            <Col className={style['info-title']}>
                                                {t('page.im.customerInfo.device.IP')}
                                            </Col>
                                            <Col className={style['info-content']} span={16}>
                                                {accessInfo.ip}
                                            </Col>
                                        </Row>
                                    )}
                                    {accessInfo.session_key && (
                                        <Row>
                                            <Col className={style['info-title']}>session_key</Col>
                                            <Col className={style['info-content']} span={16}>
                                                {accessInfo.session_key}
                                            </Col>
                                        </Row>
                                    )}

                                    {/* {
                                        accessInfo.day_sub_session_count &&
                                        <Row>
                                            <Col
                                                className={style['info-title']}>{t('page.im.customerInfo.device.chatTimes')}</Col>
                                            <Col className={style['info-content']}
                                                 span={16}>{accessInfo.day_sub_session_count}</Col>
                                        </Row>
                                    } */}
                                </>
                            )}
                            {/* weibo wechat weixin_mini qywx 平台 */}
                            {(accessInfo.platform === 'mchat' ||
                                accessInfo.platform === 'weibo' ||
                                accessInfo.platform === 'wechat' ||
                                accessInfo.platform === 'weixin_mini' ||
                                accessInfo.platform === 'qywx') &&
                                accessInfo.src && (
                                    <Row>
                                        <Col className={style['info-title']}>{t('label.source')}</Col>
                                        <Col className={style['info-content']} span={16}>
                                            {accessInfo.src}
                                        </Col>
                                    </Row>
                                )}
                            {/* android ios平台 */}
                            {(accessInfo.platform === 'android' || accessInfo.platform === 'ios') && (
                                <>
                                    <Row>
                                        <Col className={style['info-title']}>
                                            {t('page.im.customerInfo.device.phoneModal')}
                                        </Col>
                                        <Col className={style['info-content']} span={16}>
                                            {accessInfo.phoneModal}
                                        </Col>
                                    </Row>
                                    <Row>
                                        <Col className={style['info-title']}>
                                            {t('page.im.customerInfo.device.phoneVersion')}
                                        </Col>
                                        <Col className={style['info-content']} span={16}>
                                            {accessInfo.phoneVersion}
                                        </Col>
                                    </Row>
                                    <Row>
                                        <Col className={style['info-title']}>{t('label.source')}</Col>
                                        <Col className={style['info-content']} span={16}>
                                            {accessInfo.src}
                                        </Col>
                                    </Row>
                                    <Row>
                                        <Col className={style['info-title']}>
                                            {t('page.im.customerInfo.device.netWorkStatus')}
                                        </Col>
                                        <Col className={style['info-content']} span={16}>
                                            {accessInfo.netWorkStatus}
                                        </Col>
                                    </Row>
                                    <Row>
                                        <Col className={style['info-title']}>
                                            {t('page.im.customerInfo.device.carrier')}
                                        </Col>
                                        <Col className={style['info-content']} span={16}>
                                            {accessInfo.carrier}
                                        </Col>
                                    </Row>
                                    <Row>
                                        <Col className={style['info-title']}>
                                            {t('page.im.customerInfo.device.scaleScreen')}
                                        </Col>
                                        <Col className={style['info-content']} span={16}>
                                            {accessInfo.scaleScreen}
                                        </Col>
                                    </Row>
                                    {accessInfo.channel && (
                                        <Row>
                                            <Col className={style['info-title']}>
                                                {t('page.im.customerInfo.device.customChannel')}
                                            </Col>
                                            <Col className={style['info-content']} span={16}>
                                                {accessInfo.channel}
                                            </Col>
                                        </Row>
                                    )}
                                </>
                            )}
                            {/* 多商户 */}
                            {accessInfo.platform === 'mchat' && (
                                <>
                                    {
                                        // 设备型号
                                        accessInfo.phoneModal && (
                                            <Row>
                                                <Col className={style['info-title']}>
                                                    {t('page.im.customerInfo.device.phoneModal')}
                                                </Col>
                                                <Col className={style['info-content']} span={16}>
                                                    {accessInfo.phoneModal}
                                                </Col>
                                            </Row>
                                        )
                                    }
                                    {
                                        // 系统版本号
                                        accessInfo.phoneVersion && (
                                            <Row>
                                                <Col className={style['info-title']}>
                                                    {t('page.im.customerInfo.device.phoneVersion')}
                                                </Col>
                                                <Col className={style['info-content']} span={16}>
                                                    {accessInfo.phoneVersion}
                                                </Col>
                                            </Row>
                                        )
                                    }
                                    {
                                        // 来源
                                        accessInfo.src && (
                                            <Row>
                                                <Col className={style['info-title']}>{t('label.source')}</Col>
                                                <Col className={style['info-content']} span={16}>
                                                    {accessInfo.src}
                                                </Col>
                                            </Row>
                                        )
                                    }
                                    {
                                        // 网络状态
                                        accessInfo.networkStatus && (
                                            <Row>
                                                <Col className={style['info-title']}>
                                                    {t('page.im.customerInfo.device.networkStatus')}
                                                </Col>
                                                <Col className={style['info-content']} span={16}>
                                                    {accessInfo.networkStatus}
                                                </Col>
                                            </Row>
                                        )
                                    }
                                    {
                                        // 运营商
                                        accessInfo.carrier && (
                                            <Row>
                                                <Col className={style['info-title']}>
                                                    {t('page.im.customerInfo.device.carrier')}
                                                </Col>
                                                <Col className={style['info-content']} span={16}>
                                                    {accessInfo.carrier}
                                                </Col>
                                            </Row>
                                        )
                                    }
                                    {
                                        // 分辨率
                                        accessInfo.scaleScreen && (
                                            <Row>
                                                <Col className={style['info-title']}>
                                                    {t('page.im.customerInfo.device.scaleScreen')}
                                                </Col>
                                                <Col className={style['info-content']} span={16}>
                                                    {accessInfo.scaleScreen}
                                                </Col>
                                            </Row>
                                        )
                                    }
                                    {
                                        //自定义渠道信息
                                        accessInfo.channel && (
                                            <Row>
                                                <Col className={style['info-title']}>
                                                    {t('page.im.customerInfo.device.customChannel')}
                                                </Col>
                                                <Col className={style['info-content']} span={16}>
                                                    {accessInfo.channel}
                                                </Col>
                                            </Row>
                                        )
                                    }
                                </>
                            )}
                            {/* api 平台 */}
                            {accessInfo.platform === 'api' && (
                                <>
                                    <Row>
                                        <Col className={style['info-title']}>{t('label.source')}</Col>
                                        <Col className={style['info-content']} span={16}>
                                            {accessInfo.src}
                                        </Col>
                                    </Row>
                                    {accessInfo.channel && (
                                        <Row>
                                            <Col className={style['info-title']}>
                                                {t('page.im.customerInfo.device.customChannel')}
                                            </Col>
                                            <Col className={style['info-content']} span={16}>
                                                {accessInfo.channel}
                                            </Col>
                                        </Row>
                                    )}
                                </>
                            )}

                            {/* 新渠道 imNewPlatforms */}
                            {imNewPlatforms &&
                                imNewPlatforms.filter((item: any) => {
                                    return item.platform === 'channel_name';
                                }).length > 0 && (
                                    <Row>
                                        <Col className={style['info-title']}>{t('label.source')}</Col>
                                        <Col className={style['info-content']} span={16}>
                                            {accessInfo.src}
                                        </Col>
                                    </Row>
                                )}
                            {accessInfo.day_sub_session_count && (
                                <Row>
                                    <Col className={style['info-title']}>
                                        {t('page.im.customerInfo.device.chatTimes')}
                                    </Col>
                                    <Col className={style['info-content']} span={16}>
                                        {accessInfo.day_sub_session_count}
                                    </Col>
                                </Row>
                            )}
                            {accessInfo.queue_name && (
                                <Row style={{ minHeight: '20px' }}>
                                    <Col className={style['info-title']}>
                                        {t('page.im.customerInfo.device.queueList')}
                                    </Col>
                                    <Col className={style['info-content']} span={16}>
                                        {accessInfo.queue_name}
                                    </Col>
                                </Row>
                            )}
                        </div>
                    )}
                    {isEqual(initData.page, 'mpv') && accessInfo && (
                        <div className={style['info-list']}>
                            {accessInfo.sdkName && (
                                <Row>
                                    <Col className={style['info-title']}>
                                        {accessInfo?.sdkType}
                                        {/* {t('page.im.customerInfo.device.sdkName')} */}
                                    </Col>
                                    <Col className={style['info-content']} span={16}>
                                        {accessInfo?.sdkName}
                                    </Col>
                                </Row>
                            )}
                        </div>
                    )}
                </Panel>
            </Collapse>
        </div>
    );
};
export default AccessInfo;
