/* eslint-disable jsx-a11y/anchor-is-valid */
import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { Col, Form, message, Row, Select } from 'udesk-ui';
import { useTranslation } from 'react-i18next';
import style from './index.module.scss';
import { getJSON, postJSON, putJSON } from '../../../utils/ajax';
import { csApiGetJSON, csApiPutJSON, csApiPostJSON } from '../../../utils/csApi';
import { formatCustomField, getFieldsDefaultValue, isSysField } from '../../../utils/index';
import pick from 'lodash-es/pick';
import uniq from 'lodash-es/uniq';
import isEqual from 'lodash-es/isEqual';
import CustomField from '../../../components/custom-field-container';
import { Store } from 'udesk-ui/es/form';
import Field from './fields';
import DescField from './desc-fields';
import NameField from './name-fields';
import { RootState } from '../../../store/root-reducer';
import { useSelector } from '../../../store/store-hook';

const { Option } = Select;

interface IOrganizationArr {
    id: number;
    name: string;
}

interface IOrganizationProps {
    event: any;
    organizationFieldsv2: any;
    organizationFields: any;
    sessionKey?: string;
    imSubSessionId: number;
    customerId: number;
    organizations: any;
    chat: any;
}

interface IEmberProps {
    chat: any;
    updateProps: any;
}

const CompanyInfoComponent: React.FC<IOrganizationProps> = (props) => {
    const { event, organizationFieldsv2, sessionKey, customerId, organizations, chat } = props;
    const { t } = useTranslation();
    const initData = useSelector((state: RootState) => state.initData);
    const [origanizationField, setOriganizationField] = useState<any[]>([]);
    const [, setCopyName] = useState('');
    const [organizationArr, setOrganizationArr] = useState<IOrganizationArr[]>([]);
    const [companyId, setCompanyId] = useState<number | string>('');
    const [initialValue, setInitialValue] = useState<any>({});
    //根据 customer_id请求数据
    const [companyForm] = Form.useForm();
    // let _organizationFieldsv2: any = [];//用于设置初始化表格

    // let _organizationFieldsv2 = useMemo(() => {
    //     return (organizationFieldsv2 || []).filter((field: any) => {
    //         return field.im_show && field.name && (field.name.indexof('TextField') > -1 || field.name.indexof('SelectField') > -1)
    //     }).map((field: any) => {
    //         return {
    //             ...field,
    //             is_required: field.agent_permission === 1 ? true : false
    //         }
    //     })
    // }, [organizationFieldsv2]);

    let _organizationFields = useMemo(() => {
        return (organizationFieldsv2 || [])
            .filter((field: any) => {
                if (isEqual(initData.page, 'im')) {
                    return (
                        field.im_show &&
                        field.name &&
                        (field.name.indexOf('TextField') > -1 || field.name.indexOf('SelectField') > -1)
                    );
                } else if (isEqual(initData.page, 'mpv')) {
                    return (
                        field.mpv_show &&
                        field.name &&
                        (field.name.indexOf('TextField') > -1 || field.name.indexOf('SelectField') > -1)
                    );
                } else {
                    return (
                        field.cc_show &&
                        field.name &&
                        (field.name.indexOf('TextField') > -1 || field.name.indexOf('SelectField') > -1)
                    );
                }
            })
            .map((field: any) => {
                return {
                    ...field,
                    is_required: field.agent_permission === 1 ? true : false,
                };
            });
    }, [organizationFieldsv2]);

    useEffect(() => {
        //循环公司自定义字段
        const organFields = formatCustomField({
            fields: _organizationFields,
            trigger: 'onBlur',
        });
        //自定义字段的formItem
        setOriganizationField(organFields);
    }, [_organizationFields]);

    //根据customerId  session_key monitor _time 请求已经有的公司
    const getOrganizationArr = useCallback((params: any) => {
        let url: string = 'im/customer';
        getJSON(url, params)
            .then((res) => {
                if (res.code && res.code === 1000) {
                    setOrganizationArr(res.organizations);
                    changeCompany(res.customer.default_organization_id);
                }
            })
            .catch(() => {});
    }, []);

    useEffect(() => {
        //只加载一次  用来赋初始值
        let params = {
            id: customerId,
            session_key: sessionKey,
            monitor: true,
        };
        getOrganizationArr(params);
    }, [customerId, getOrganizationArr, sessionKey]);

    // 点击已选择的公司  进入公司详情
    const goCompany = useCallback(
        (companyId: any) => {
            // 如果没有选择  不进行操作
            // 判断点击的是否是选择的字的地方  其他地方不触发
            if (companyId) {
                props.event.transitionToRoute &&
                    props.event.transitionToRoute({
                        route: 'crm.organization.show',
                        path: companyId,
                    });
                return false;
            }
        },
        [props.event]
    );

    //封装form initValue 初始化表格的时候
    //疑问 获取初始值的值  解决  数据是通过接口拿的

    const getValues = useCallback(
        (_organization) => {
            let initValues =
                getFieldsDefaultValue({
                    fields: _organizationFields,
                    values: _organization.custom_fields || {},
                }) || {};
            initValues = Object.assign({}, initValues, {
                domains: _organization.domains,
                org_email_suffix: _organization.org_email_suffix,
                description: _organization.description,
                name: _organization.name,
            });
            return initValues;
        },
        [_organizationFields]
    );
    const loadData = useCallback(
        (id: number | string) => {
            if (!id) {
                return;
            }
            csApiGetJSON(`/cs/ts/api/organizations/${id}`)
                .then(res=>res.data)    
                .then((res) => {
                    if (res.code === 1000) {
                        let formFieldsValue = pick(
                            res.organization,
                            'name',
                            'domains',
                            'org_email_suffix',
                            'description'
                        );
                        let values = getValues(res.organization);
                        setInitialValue(values);
                        companyForm.setFieldsValue(values);
                        setCopyName(formFieldsValue.name);
                    }
                })
                .catch((err) => {
                    message.error(t('page.im.customerInfo.organization.failedToGetData'));
                });
        },
        [companyForm, getValues, t]
    );
    useEffect(() => {
        if (organizations && organizations.length > 0) {
            //如果organizations有长度  取默认的第一个值
            let getCompanyId = organizations[0]?.id;
            setCompanyId(getCompanyId);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);

    useEffect(() => {
        if (companyId) {
            loadData(companyId);
        }
    }, [companyId, loadData]);

    const companyChildrenOption =
        organizationArr &&
        organizationArr.map((item, index) => {
            return (
                <Option value={item.id} key={item.id}>
                    {item.name}
                </Option>
            );
        });
    //设置companyId 根据当前Id 判断是新增还是修改 如果value 是--
    const changeCompany = (value: any, option?: any) => {
        setCompanyId(value);
        if (value === '') {
            companyForm.setFieldsValue(
                Object.keys(initialValue).reduce((init: any, curent: string) => {
                    init[curent] = '';
                    return init;
                }, {})
            );
        }
    };
    //新建公司之后调用 putcustomers (put 方法) 刷新当前公司的下拉选项

    const putCustomer = useCallback(
        (customers: number[], id: number) => {
            const url = '/cs/ts/api/customers/' + customerId;
            csApiPutJSON(url, customers).then(res=>res.data).then((res) => {
                if (res.code && res.code === 1000) {
                    setOrganizationArr(res.organizations);
                    setCompanyId(id);
                    event.updateChat(chat, {
                        organizations: res.organizations,
                        customer: res.customer,
                    });
                    //调用Ember方法
                } else {
                    message.error(
                        res.code_message || (res.exceptopn && res.exceptopn.message) || t('tip.error.saveTry')
                    );
                }
            });
        },
        [chat, customerId, event, t]
    );

    function judgeValueChanged(changedValue: any, initialValue: any) {
        //只判断当前的修改值
        let key = Object.keys(changedValue)[0];
        return changedValue[key] !== initialValue[key];
    }

    const submitFn = useCallback(
        (params) => {
            const submitFnWay = companyId ? csApiPutJSON : csApiPostJSON;
            // let way = companyId ? 'putJSON' : 'postJSON';
            const url = companyId ? `/cs/ts/api/organizations/${companyId}` : '/cs/ts/api/organizations';
            submitFnWay(url, params)
                .then(res=>res.data)
                .then((res) => {
                    if (res.code === 1000) {
                        let values = getValues(res.organization || {});
                        companyForm.setFieldsValue(values);
                        //将返回的id 设置为select 项
                        // setCompanyId(res.organization.id);
                        let _organizations = organizationArr.map((item, index) => {
                            return item.id;
                        });

                        _organizations = [..._organizations, res.organization.id];
                        let params: any = {};
                        params.customer = {
                            organizations: uniq(_organizations),
                        };
                        putCustomer(params, res.organization.id);
                        message.info(t('tip.success.save'));
                    } else {
                        message.error(
                            res.code_message || (res.exceptopn && res.exceptopn.message) || t('tip.error.saveTry')
                        );
                    }
                })
                .catch(() => {});
        },
        [companyForm, companyId, getValues, organizationArr, putCustomer, t]
    );

    const onValueChange = (changedValues: Store, values: Store) => {
        const changedKey = Object.keys(changedValues)[0];
        const _isSysField = isSysField(changedKey);
        let params: any = _isSysField
            ? { organization: changedValues }
            : { organization: { custom_fields: changedValues } };
        params.mode = 'quick_edit';
        companyForm
            .validateFields(Object.keys(changedValues))
            .then(() => {
                submitFn(params);
            })
            .catch((err: Store) => {
                console.log(err);
                let key = Object.keys(changedValues)[0];
                let canSubmit = true;
                err.errorFields.forEach((item: any) => {
                    if (item.name[0] === key) {
                        canSubmit = false;
                    }
                });
                if (canSubmit) {
                    submitFn(params);
                }
            });
    };
    return (
        <div className={style['company-info-page']}>
            <Form form={companyForm} layout="vertical" name="companyInfo" onValuesChange={onValueChange}>
                {organizationArr.length > 0 && (
                    <Form.Item>
                        <Row align="middle">
                            <Col span={12}>
                                <Select
                                    getPopupContainer={(triggerNode) => triggerNode.parentNode}
                                    value={companyId}
                                    onChange={changeCompany}
                                >
                                    <Option value=""> -- </Option>
                                    {companyChildrenOption}
                                </Select>
                            </Col>
                            <Col style={{ marginLeft: '15px' }}>
                                <a onClick={() => goCompany(companyId)}>
                                    [{t('page.im.customerInfo.organization.companyDetail')}]
                                </a>
                            </Col>
                        </Row>
                    </Form.Item>
                )}
                <Form.Item
                    label={t('page.im.customerInfo.organization.org_email_suffix')}
                    name="org_email_suffix"
                    trigger="onBlur"
                >
                    <Field trigger={'onBlur'} />
                </Form.Item>
                <Form.Item label={t('page.im.customerInfo.organization.domains')} name="domains" trigger="onBlur">
                    <Field trigger={'onBlur'} />

                    {/* <Input name='domains' value={domainsValue} onChange={changeDomainsValue}/> */}
                </Form.Item>

                <Form.Item
                    label={t('page.im.customerInfo.organization.name')}
                    name="name"
                    rules={[
                        {
                            required: true,
                            message: t('page.im.customerInfo.organization.validateFail'),
                        },
                    ]}
                    trigger="onBlur"
                >
                    <NameField trigger={'onBlur'} />
                </Form.Item>

                <Form.Item
                    label={t('page.im.customerInfo.organization.description')}
                    name="description"
                    trigger="onBlur"
                >
                    <DescField trigger={'onBlur'} />
                    {/* <TextArea name='description' value={descriptionValue} onChange={changeDescriptionValue}  rows={2} /> */}
                </Form.Item>

                {origanizationField.length > 1 && <CustomField fields={origanizationField} />}
            </Form>
        </div>
    );
};
export default CompanyInfoComponent;
