import React, {ComponentProps, useCallback, useEffect, useMemo, useState} from "react";
import { Select } from "udesk-ui";
import { postJSON } from "../../../../utils/ajax";

const { Option } = Select;

interface IOptions {
  id: number;
  nick_name: string;

  [propName: string]: any;
}

interface ISearchParams {
  prefix: string;
  page_size: number;
  page: number;
  includes: string;
}

interface IProps {
  selectOnChange: (value?: number) => void;
}

const CustomerSelect: React.FC<any> = (props) => {
  const { selectOnChange } = props;
  const [optionsArr, setOptionsArr] = useState<IOptions[]>([]);
  const [value, setValue] = useState<any>(props.value || null);
  const [searchParams, setSearchParams] = useState<ISearchParams>({
    prefix: "",
    page_size: 20,
    page: 1,
    includes: "can_merge",
  });

  useEffect(() => {
    setValue(props.value || null);
  }, [props.value]);

  useEffect(() => {
    //请求数据
    if (searchParams.prefix) {
      let params = { ...searchParams };
      postJSON("/customers/contain_search", params).then((res) => {
        setOptionsArr((prev) => {
          return prev.concat([...res.customers]);
        });
      });
    } else {
      setOptionsArr([]);
      setSearchParams({ ...searchParams, page: 1 });
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [searchParams.prefix, searchParams.page]);

  const handleSearch = useCallback(
    (value: string) => {
      setSearchParams({ ...searchParams, prefix: value });
    },
    [searchParams]
  );
  // const handleSearch = (value: string) => {
  //     setSearchParams({ ...searchParams, prefix: value })
  // }

  const handleChange:ComponentProps<typeof Select>['onChange'] = (
    value: any,
    option
  ) => {
    setValue(value);
    selectOnChange && selectOnChange(value, option);
  };

  const handleScroll = useCallback(
    (e: any) => {
      e.persist();
      const { target } = e;
      if (target.scrollTop + target.offsetHeight === target.scrollHeight) {
        setSearchParams({ ...searchParams, page: searchParams.page + 1 });
      }
    },
    [searchParams]
  );
  // const handScroll = (e: any) => {
  //     e.persist();
  //     const { target } = e;
  //     if (target.scrollTop + target.offsetHeight === target.scrollHeight) {
  //         setSearchParams({ ...searchParams, page: searchParams.page + 1 });
  //     }
  // }

  const optionChildrens = useMemo(() => {
    return optionsArr.map((item) => {
      return (
        <Option value={item.id} key={item.id}>
          {item.nick_name}
        </Option>
      );
    });
  }, [optionsArr]);

  // const optionChildrens = optionsArr.map((item) => {
  //     return (<Option value={item.id} key={item.id}>{item.nick_name}</Option>)
  // })

  const tagRender:ComponentProps<typeof Select>['tagRender'] = (props) => {
    return <span>{props.label}</span>;
  };

  return (
    <Select
      style={{ width: 200 }}
      showSearch
      placeholder="请选择客户"
      value={value}
      labelInValue={true}
      defaultActiveFirstOption={true}
      showArrow={true}
      filterOption={false}
      onSearch={handleSearch}
      onChange={handleChange}
      onPopupScroll={handleScroll}
      notFoundContent={null}
      tagRender={tagRender}
      size="small"
    >
      {optionChildrens}
    </Select>
  );
};
export default CustomerSelect;
