import React, { useCallback, useEffect, useState } from "react";
import { Button, Input, message } from "udesk-ui";
import style from "./index.module.scss";
import { EmailWordOrder, Warning } from "../../../../components/icon/icon";
import { deleteJSON, getJSON } from "../../../../utils";
import Merge from "./merge";
import { useTranslation } from "react-i18next";
import CopyComponent from "../../../../components/copy";
import Qs from "qs";
import { useSelector } from "../../../../store/store-hook";
import { RootState } from "../../../../store/root-reducer";

interface EmailProps {
  value?: string;
  trigger?: "onChange" | "onBlur";
  onChange?: (value: string) => void;
  onBlur?: (value: string) => void;
  customer: Customer;
  imSubSessionId: any;
  event?: any;
  selfChat?: any;
  toCustomer?: any;
  companyCategoryConfig?: any;
  setToCustomer?: any;
  switchFlag?: boolean;
}

const Email: React.FC<EmailProps> = (props) => {
  const { t } = useTranslation();
  const [value, setValue] = useState(props.value || ""); // 初始值
  const [showRepeat, setShowRepeat] = useState(false); //  显示合并信息
  const [showMerge, setShowMerge] = useState(false); //  显示客户合并Modal
  const [repeatEmail, setRepeatEmail] = useState<any>({}); // 缓存合并的数据信息
  const [title, setTitle] = useState("客户合并");
  const [reservationed, setReservationed] = useState(false); // 是否已经预约
  const initData = useSelector((state: RootState) => state.initData);
  const { chat } = initData;
  // 动态设置value
  useEffect(() => {
    setValue(props.value || "");
  }, [props.value]);

  // 动态设置已预约合并信息
  useEffect(() => {
    if (
      props.toCustomer &&
      props.toCustomer.value &&
      props.toCustomer.label &&
      props.toCustomer.reservation_reason === "email"
    ) {
      setRepeatEmail(props.toCustomer);
      setReservationed(true);
    } else {
      setRepeatEmail({});
      setReservationed(false);
    }
  }, [
    props.toCustomer.value,
    props.toCustomer.label,
    props.toCustomer.reservation_reason,
    props.toCustomer,
  ]);

  // 值变化

  const handleChange = useCallback((e: React.FocusEvent<HTMLInputElement>) => {
    setValue(e.target.value);
  }, []);
  // const handleChange = (e: React.FocusEvent<HTMLInputElement>) => {
  //     setValue(e.target.value);
  // }

  // 失去焦点时,先验证是否有重复邮箱,如果有,显示重复提醒,并可以合并, 留言直接合并, 当前会话,预约合并
  const handleBlur = useCallback(
    (e: React.FocusEvent<HTMLInputElement>) => {
      let changeFunction = props[props.trigger || "onChange"];
      let _value = e.target.value;
      let customer = props.customer || {};
      let params = {
        customer: {
          id: customer.id,
          email: e.target.value,
        },
      };
      getJSON("/customers/check_duplication?" + Qs.stringify(params), {})
        .then((res: any) => {
          if (res.code !== 1000) {
            message.error(t("error.repeatEmailFail"));
          } else {
            let dup = res.duplications || {};
            // 有重复邮箱,显示预约合并按钮 ? 如果电话已经预约了 ?
            if (dup.email && dup.email.id) {
              setShowRepeat(true);
              setRepeatEmail({
                label: dup.email.name,
                value: dup.email.id,
                key: dup.email.id,
              });
            } else {
              if (!reservationed) {
                setShowRepeat(false);
                setRepeatEmail({});
              }
              if (changeFunction) {
                changeFunction(_value);
              }
            }
          }
        })
        .catch(() => {});
    },
    [props, reservationed, t]
  );

  // 跳转到客服系统客户详情页
  const transferToCustomer = useCallback(() => {
    props.event.transitionToRoute &&
      props.event.transitionToRoute({
        route: "crm.customer.show",
        path: repeatEmail.value,
      });
  }, [props.event, repeatEmail.value]);

  // 显示预约modal
  const showMergeModal = useCallback(() => {
    if (props.selfChat?.history) {
      setTitle(t("udMerge.mergeCustomer"));
    } else {
      setTitle(t("udMerge.appointmentMergeCustomer"));
    }
    setShowMerge(true);
  }, [props.selfChat?.history, t]);

  // 取消立即预约
  const cancelReservation = useCallback(() => {
    let customerId = props.customer.id;
    let imSubSessionId = props.imSubSessionId;

    deleteJSON("/customer_merge_reservation", {
      from_customer_id: customerId,
      im_sub_session_id: imSubSessionId,
    })
      .then((res: any) => {
        // 取消之后,通知父页面, 可以再次预约
        setReservationed(false);
        setShowRepeat(true);
        setRepeatEmail({});
        props.setToCustomer({});
      })
      .catch((err) => {});
  }, [props]);
  // const cancelReservation = () => {
  //     deleteJSON('/customer_merge_reservation', {
  //         from_customer_id: props.customer.id,
  //         im_sub_session_id: props.imSubSessionId
  //     }).then((res: any) => {
  //         // 取消之后,通知父页面, 可以再次预约
  //         setReservationed(false);
  //         setShowRepeat(true);
  //         setRepeatEmail({})
  //         props.setToCustomer({});
  //     }).catch(err => {
  //     })
  // }

  // 预约回调
  // isSelf: true 表示其他客户合并到了自己身上, false 表示自己合到了其他客户身上,自己有可能被删除
  const handleMerge = useCallback(
    (isSelf: boolean, repeatInfo: any, needMerge: boolean) => {
      let chat = props.selfChat || {};
      let history = chat.history || false;
      let setToCustomer = props.setToCustomer;
      let event = props.event;
      setShowRepeat(false);
      setReservationed(true);
      setRepeatEmail(repeatInfo);
      setToCustomer &&
        setToCustomer({ ...repeatInfo, reservation_reason: "email" });
      if (history && needMerge) {
        event &&
          event.updateChat &&
          event.updateChat({
            chat: props.selfChat,
            key: "isDeletedCustomer",
            value: !isSelf,
          });
      }
    },
    [props.selfChat, props.event, props.setToCustomer]
  );

  const createEmailTicket = useCallback(() => {
    let event = props.event;
    event && event.createEmailTicket(chat);
  }, [props.event, chat]);

  return (
    <div className={`${style["flex-row"]} ${style["flex-row-item-center"]}`}>
      <div className={style["email-input"]}>
        <CopyComponent text={value} switchFlag={props.switchFlag}>
          <Input value={value} onChange={handleChange} onBlur={handleBlur} />
        </CopyComponent>
        <EmailWordOrder
          title={"新建邮件工单"}
          onClick={createEmailTicket}
          style={{
            fontSize: "18px",
            color: "green",
            marginLeft: "10px",
            cursor: "pointer",
          }}
        />
      </div>

      {showRepeat && (
        <div className={style["repeat-box"]}>
          <Warning className={style["repeat-icon"]} />

          <div>
            此电话与已有客户 “{" "}
            <span style={{ color: "#00B38B" }} onClick={transferToCustomer}>
              {repeatEmail.label}{" "}
            </span>
            ” 重复，
            {props.selfChat.history ? (
              <Button onClick={showMergeModal} size="small">
                {t("udMerge.merge")}
              </Button>
            ) : (
              <React.Fragment>
                {t("udMerge.afterMergeConversation")}{" "}
                <Button onClick={showMergeModal} size="small">
                  {t("udMerge.appointmentMerge")}
                </Button>
              </React.Fragment>
            )}
          </div>
        </div>
      )}

      {reservationed && (
        <div className={style["repeat-box"]}>
          <Warning className={style["repeat-icon"]} />
          <div>
            {t("udMerge.hasAppointment")} “
            <span style={{ color: "#00B38B" }} onClick={transferToCustomer}>
              {repeatEmail.label}{" "}
            </span>
            ”
            <Button onClick={cancelReservation} size="small">
              {t("udMerge.cancelReservation")}
            </Button>
          </div>
        </div>
      )}

      <Merge
        title={title}
        concatVisible={showMerge}
        customer={props.customer}
        repeatCustomer={repeatEmail}
        imSubSessionId={props.imSubSessionId}
        cancelCombineCustomer={setShowMerge}
        mergeFn={handleMerge}
        companyCategoryConfig={props.companyCategoryConfig}
        history={props.selfChat.history}
        reason="email"
      />
    </div>
  );
};

export default Email;
