import React, { useEffect, useState, useMemo, useCallback } from "react";
import CustomFieldItem from "../../../../components/custom-field-container/field-item";
import TagField from "../../../ticket/components/tag-field";
import DescOrName from "./desc-or-name";
import CustomerInfoPhone from "./phone-formItem";
import Email from "./email";
import { Form } from "udesk-ui/es";
import { getJSON } from "../../../../utils";
import OrgSelect from "./org-select";
import { useTranslation } from "react-i18next";

interface CustomerFieldsProps {
  customerFields: any;
  customer: Customer;
  organizations: Organization[];
  organizationFields?: any;
  isFormItem?: boolean;
  event?: any;
  imSubSessionId?: any;
  chat: any;
  company: Company;
  roleCan: any;
  companyCategoryConfig: any;
  switchFlag?: boolean;
  currentUser: any;
  smsapp?: any;
  isAdmin?: any;
  user?: any;
  agents?: IAgent[];
}

const CustomerFields: React.FC<CustomerFieldsProps> = ({
  currentUser,
  smsapp,
  customerFields,
  chat,
  roleCan,
  customer,
  company,
  organizations,
  organizationFields,
  companyCategoryConfig,
  event,
  imSubSessionId,
  switchFlag,
  isFormItem = true,
  isAdmin,
  user,
  agents,
  ...props
}) => {
  const { t } = useTranslation();
  const [toCustomer, setToCustomer] = useState<any>({});

  function isCustomField(item: any) {
    let type = item.field ? item.field.id || "" : "";
    return type.indexOf("SelectField") > -1 || type.indexOf("TextField") > -1;
  }

  const validatorEmail = useCallback(
    (rule: any, values: any) => {
      let emaillReg = /^[a-zA-Z0-9.!#$%&’*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*$/;
      if (!values || emaillReg.test(values)) {
        return Promise.resolve();
      } else {
        return Promise.reject(t("error.emailError"));
      }
    },
    [t]
  );
  useEffect(() => {
    if (chat.history) {
      return;
    }
    getJSON("/customer_merge_reservation", {
      from_customer_id: customer.id,
      im_sub_session_id: imSubSessionId,
    })
      .then((res: any) => {
        if (res.code === 1000) {
          const to_customer = res.to_customer;
          if (!to_customer) {
            // 都没有预约, 则都可以显示立即预约按钮
            setToCustomer({});
          } else {
            setToCustomer({
              key: to_customer.id,
              label: to_customer.nick_name,
              value: to_customer.id,
              reservation_reason: res.reservation_reason,
            });
          }
        }
      })
      .catch((err: any) => {});
  }, [imSubSessionId, customer.id, chat.history]);

  const customerFieldsMap = useMemo(() => {
    return customerFields.map((fieldItem: IFieldItem & ISysField) => {
      // 自定义字段
      if (isCustomField(fieldItem)) {
        return (
          <CustomFieldItem
            key={fieldItem.field.id}
            switchFlag={switchFlag}
            agents={agents}
            fieldItem={fieldItem}
            isFormItem={isFormItem}
          />
        );
      }
      if (fieldItem.name === "tags") {
        return (
          <Form.Item
            key={fieldItem.name}
            label={t("label.tag")}
            name={"tags"}
            trigger="onBlur"
          >
            <TagField {...fieldItem} />
          </Form.Item>
        );
      } else if (fieldItem.name === "description") {
        return (
          <Form.Item
            key={fieldItem.name}
            label={t("label.description")}
            name="description"
            trigger="onBlur"
          >
            <DescOrName switchFlag={switchFlag} {...fieldItem} />
          </Form.Item>
        );
      } else if (fieldItem.name === "name") {
        return (
          <Form.Item
            key={fieldItem.name}
            label={t("label.name")}
            name="nick_name"
            trigger="onBlur"
            rules={[
              {
                required: true,
                message: t("error.nameEmpty"),
              },
            ]}
          >
            <DescOrName switchFlag={switchFlag} {...fieldItem} />
          </Form.Item>
        );
      } else if (fieldItem.name === "cellphone") {
        return (
          <Form.Item
            key={fieldItem.name}
            label={t("label.phone")}
            name="cellphones"
            trigger={"onBlur"}
          >
            <CustomerInfoPhone
              {...fieldItem}
              customer={customer}
              imSubSessionId={imSubSessionId}
              event={event}
              selfChat={{
                history: chat.history,
                jid: chat.jid,
                session_key: chat.session_key,
                im_sub_session_id: chat.im_sub_session_id,
              }}
              toCustomer={toCustomer}
              companyCategoryConfig={companyCategoryConfig}
              setToCustomer={(config: any) => setToCustomer(config)}
              switchFlag={switchFlag}
              currentUser={currentUser}
              smsapp={smsapp}
              roleCan={roleCan}
              isAdmin={isAdmin}
              user={user}
            />
          </Form.Item>
        );
      } else if (fieldItem.name === "organization") {
        return (
          <Form.Item
            key={fieldItem.name}
            label={t("label.company")}
            name="organizations"
            trigger={"onChange"}
          >
            {/* <FieldComponent.org placeholder='输入关键字查询公司' /> */}
            <OrgSelect
              {...fieldItem}
              trigger="onChange"
              customer={customer}
              event={event}
              organizations={organizations}
              organizationFields={organizationFields}
              canAdd={company.support_multi_organizations}
              canCreate={roleCan.add_new_organization}
              switchFlag={switchFlag}
            />
          </Form.Item>
        );
      } else if (fieldItem.name === "email") {
        return (
          <Form.Item
            key={fieldItem.name}
            label={t("label.email")}
            name="email"
            trigger="onBlur"
            rules={[{ validator: validatorEmail }]}
          >
            <Email
              {...fieldItem}
              customer={customer}
              imSubSessionId={imSubSessionId}
              event={event}
              selfChat={{
                history: chat.history,
                jid: chat.jid,
                session_key: chat.session_key,
                im_sub_session_id: chat.im_sub_session_id,
              }}
              toCustomer={toCustomer}
              companyCategoryConfig={companyCategoryConfig}
              setToCustomer={(config: any) => setToCustomer(config)}
              switchFlag={switchFlag}
            />
          </Form.Item>
        );
      } else {
        return "";
      }
    });
  }, [
    chat.history,
    chat.im_sub_session_id,
    chat.jid,
    chat.session_key,
    company.support_multi_organizations,
    companyCategoryConfig,
    currentUser,
    customer,
    customerFields,
    event,
    imSubSessionId,
    isAdmin,
    isFormItem,
    organizationFields,
    organizations,
    roleCan,
    smsapp,
    switchFlag,
    t,
    toCustomer,
    user,
    validatorEmail,
  ]);
  return (
    <React.Fragment>
      {customerFields.length > 0 && customerFieldsMap}
    </React.Fragment>
  );
};

export default CustomerFields;
