import React, { useCallback, useEffect, useState } from "react";
import { Col, message, Modal, Row } from "udesk-ui";
import { useTranslation } from "react-i18next";
import CustomerSelect from "./customer-select";
import style from "./index.module.scss";
import { Change } from "../../../../components/icon/icon";
import isEmpty from "lodash-es/isEmpty";
import { postJSON } from "../../../../utils";
import { csApiPostJSON } from "../../../../utils/csApi";

interface IProps {
  title?: string;
  concatVisible: boolean;
  cancelCombineCustomer: any;
  imSubSessionId: number;
  companyCategoryConfig: any;
  customer: Customer;
  reason: "email" | "cellphone";
  history?: boolean;
  mergeFn: any;
  repeatCustomer: any;
}

const Merge: React.FC<IProps> = (props) => {
  const { t } = useTranslation();
  const {
    title,
    concatVisible,
    repeatCustomer,
    cancelCombineCustomer,
    imSubSessionId,
    customer,
    reason,
    history,
    mergeFn,
  } = props;
  const [trun, setTrun] = useState(false);
  const [showError] = useState(false);
  const [mergeCustomer, setMergeCustomer] = useState<any>({});

  useEffect(() => {
    setMergeCustomer(repeatCustomer || null);
  }, [repeatCustomer]);

  const handleMerge = useCallback(() => {
    if (isEmpty(mergeCustomer)) {
      return message.error(t("udMerge.selectTargetCustomer"));
    }

    // 当前客户id
    let currentId = trun ? mergeCustomer.value : customer.id;
    // 目标客户id
    let mergeId = trun ? customer.id : mergeCustomer.value;

    if (currentId === mergeId) {
      return message.error(t("udMerge.sameCustomer"));
    }
    // 历史页面直接合并
    if (history) {
      csApiPostJSON(`/cs/ts/api/customers/${currentId}/merge`, {
        form_id: mergeId,
      })
        .then(res=>res.data)
        .then((res: any) => {
          if (res.code === 1000) {
            message.success(t("udMerge.mergeSuccess"));
            mergeFn && mergeFn(trun, mergeCustomer, true);
            cancelCombineCustomer(false);
          } else {
            message.error(res.code_message || t("udMerge.mergeFail"));
          }
        })
        .catch((e) => {
          message.error(t("udMerge.mergeFail"));
        });
    } else {
      // 当前聊天页面, 预约合并
      postJSON("/customer_merge_reservation", {
        from_customer_id: currentId,
        to_customer_id: mergeId,
        im_sub_session_id: imSubSessionId,
        reservation_reason: reason,
      })
        .then((res: any) => {
          if (res.code === 1000) {
            message.success(t("udMerge.appointmentMergeSuccess"));
            mergeFn && mergeFn(false, mergeCustomer, false);
            cancelCombineCustomer(false);
          } else {
            message.error(
              res.code_message || t("udMerge.appointmentMergeFail")
            );
          }
        })
        .catch(() => {
          message.error(t("udMerge.mergeFail"));
        });
    }
  }, [
    cancelCombineCustomer,
    customer.id,
    history,
    imSubSessionId,
    mergeCustomer,
    mergeFn,
    reason,
    t,
    trun,
  ]);
  // const handleMerge = () => {
  //     if (isEmpty(mergeCustomer)) {
  //         return message.error(t('udMerge.selectTargetCustomer'));
  //     }

  //     // 当前客户id
  //     let currentId = trun ? mergeCustomer.value : customer.id
  //     // 目标客户id
  //     let mergeId = trun ? customer.id : mergeCustomer.value

  //     if (currentId === mergeId) {
  //         return message.error(t('udMerge.sameCustomer'));
  //     }
  //     // 历史页面直接合并
  //     if (history) {
  //         postJSON(`/customers/${currentId}/merge`, {
  //             form_id: mergeId
  //         }).then((res: any) => {
  //             if (res.code === 1000) {
  //                 message.success(t('udMerge.mergeSuccess'));
  //                 props.mergeFn && props.mergeFn(trun, mergeCustomer, true);
  //                 cancelCombineCustomer();
  //             } else {
  //                 message.error(res.code_message || t('udMerge.mergeFail'));
  //             }
  //         }).catch((e) => {

  //             message.error(t('udMerge.mergeFail'));
  //         })
  //     } else { // 当前聊天页面, 预约合并
  //         postJSON('/customer_merge_reservation', {
  //             from_customer_id: currentId,
  //             to_customer_id: mergeId,
  //             im_sub_session_id: imSubSessionId,
  //             reservation_reason: reason,
  //         }).then((res: any) => {
  //             if (res.code === 1000) {
  //                 message.success(t('udMerge.appointmentMergeSuccess'));
  //                 props.mergeFn && props.mergeFn(false, mergeCustomer, false);
  //                 cancelCombineCustomer();
  //             } else {
  //                 message.error(res.code_message || t('udMerge.appointmentMergeFail'));
  //             }
  //         }).catch(() => {
  //             message.error(t('udMerge.mergeFail'));
  //         })
  //     }
  // }

  const handleConcatCancel = useCallback(() => {
    cancelCombineCustomer(false);
  }, [cancelCombineCustomer]);
  // const handleConcatCancel = () => {
  //     cancelCombineCustomer();
  // }

  //选择目标客户
  const selectCustomer = useCallback((value: any) => {
    setMergeCustomer(value);
  }, []);
  // const selectCustomer = (value: any, option: OptionsType[number] | OptionsType) => {
  //     setMergeCustomer(value);
  // }

  // 判断是否需要显示不能合并信息
  // !! 测试关注
  // useEffect(() => {
  //     if (!isEmpty(mergeCustomer)) {
  //         if (mergeCustomer.can_merge && customer.can_merge) {
  //             setShowError(false);
  //         } else {
  //             setShowError(true);
  //         }
  //     }
  // }, [mergeCustomer.can_merge, customer.can_merge])

  //交换目标客户和原有客户 更改order
  const changeCustomer = useCallback(() => {
    setTrun((prev) => {
      return !prev;
    });
  }, []);
  // const changeCustomer = () => {
  //     setTrun((prev) => {
  //         return !prev;
  //     });
  // }
  return (
    <>
      <Modal
        title={title}
        visible={concatVisible}
        onOk={handleMerge}
        okText={t("confirm")}
        cancelText={t("label.cancel")}
        onCancel={handleConcatCancel}
      >
        <div className={style["phone-concat-modal-content"]}>
          <div>
            <Row>
              <Col span={8}>{t("udMerge.currentCustomer")}</Col>
              <Col span={16}>
                {trun ? (
                  <CustomerSelect
                    value={repeatCustomer}
                    selectOnChange={selectCustomer}
                  />
                ) : (
                  <span>{customer.nick_name}</span>
                )}
              </Col>
            </Row>
            <br />
            <Row>
              <Col span={8}>{t("udMerge.targetCustomer")}</Col>
              <Col span={16}>
                {trun ? (
                  <span>{customer.nick_name}</span>
                ) : (
                  <CustomerSelect
                    value={repeatCustomer}
                    selectOnChange={selectCustomer}
                  />
                )}
              </Col>
            </Row>
          </div>
          <div>
            <Change
              style={{ marginLeft: "15px", fontSize: "50px" }}
              onClick={changeCustomer}
            />
          </div>
        </div>
        <br />

        {showError ? (
          <div>
            {t("udMerge.typeNotMerge")}
            {props.companyCategoryConfig &&
              props.companyCategoryConfig.helpCenterDocumentHide && (
                // eslint-disable-next-line react/jsx-no-target-blank
                <a
                  href="http://udesk.udesk.cn/hc/articles/29296"
                  target="_blank"
                  style={{ textDecoration: "underline" }}
                >
                  了解详情
                </a>
              )}
          </div>
        ) : (
          <Row>
            <Col offset={2}>
              <p>{t("udMerge.afterMergeCustomer")}</p>
              <p>{t("udMerge.mergeCustomerHelpOne")}</p>
              <p>{t("udMerge.mergeCustomerHelpTwo")}</p>
              <p>{t("udMerge.mergeCustomerHelpThree")}</p>
            </Col>
          </Row>
        )}
      </Modal>
    </>
  );
};
export default Merge;
