import React, { useCallback, useEffect, useState } from "react";
import { message, Modal, Select, Spin } from "udesk-ui";
import { useTranslation } from "react-i18next";
import { csApiPostJSON } from "../../../../utils/csApi";
import style from "./index.module.scss";
import { MinusOutlined, PlusOutlined } from "@ant-design/icons";
import CopyComponent from "../../../../components/copy";

import CreateOrg from "../create-org";
import cloneDeep from "lodash-es/cloneDeep";
import includes from "lodash-es/includes";

const { Option } = Select;

const OrgSelect: React.FC<any> = (props) => {
  const { t } = useTranslation();
  const [showModal, setShowModal] = useState(false);
  const [optionsArr, setOptionsArr] = useState<any>([]);
  const [valueList, setValueList] = useState<AntdSelectOptions[]>([]);
  const [fetching, setFetching] = useState<boolean>(false); //无数据时展示

  useEffect(() => {
    setValueList(() => {
      if (Array.isArray(props.value) && props.value.length > 0) {
        return props.value.map((item: any) => {
          return {
            label: item.name || item.label,
            value: item.id || item.value,
          };
        });
      } else {
        return [
          {
            label: "",
            value: "",
          },
        ];
      }
    });
  }, [props.value]);

  const getOrg = useCallback(
    (query: string = "") => {
      setFetching(true);
      csApiPostJSON("/cs/ts/api/organizations/search", {query: query}).then(res=>res.data)
        .then((res: any) => {
          if (res.code === 1000) {
            setOptionsArr(
              res.organizations.map((item: any) => {
                return {
                  value: item.id,
                  label: item.name,
                };
              })
            );
          } else {
            message.error(t("error.requestFailed"));
          }
        })
        .catch((err: any) => {})
        .finally(() => {
          setFetching(false);
        });
    },
    [t]
  );

  // 获取焦点时请求下数据
  const handleFocus = useCallback(() => {
    getOrg("");
  }, [getOrg]);
  // const handleFocus = () => {
  //     getOrg('');
  // }

  const handleSearch = useCallback(
    (value: string) => {
      getOrg(value);
    },
    [getOrg]
  );

  // const handleSearch = (value: string) => {
  //     getOrg(value);
  // }

  const handleChange = useCallback(
    (value: any, index: number) => {
      let list = cloneDeep(valueList);
      //改变当前选择的公司的值
      if (value === undefined) {
        list.splice(index, 1);
      } else {
        list = list.map((item: any, idx: number) => {
          if (index === idx) {
            return {
              label: value.label,
              value: value.value,
            };
          } else {
            return item;
          }
        });
      }
      // list.splice(index, 1, value || {} as AntdSelectOptions)
      // 重新赋值
      if (list && list.length > 0) {
        setValueList([...list]);
      } else {
        setValueList([
          {
            label: "",
            value: "",
          },
        ]);
      }
      setFetching(false);
      let changeFunction = props[props.trigger];
      if (changeFunction) {
        // const _value = list.map(item => item.value).filter(t => t);
        // changeFunction(_value.length ? _value : '');
        changeFunction(list);
      }
    },
    [props, valueList]
  );

  // 添加新的公司
  const addOrg = useCallback(() => {
    let canAdd = true;
    //如果有公司还没有选  禁止新加
    valueList.forEach((item) => {
      if (!item.value) {
        canAdd = false;
        return;
      }
    });
    canAdd &&
      setValueList([
        ...valueList.concat({ value: "", label: "" } as AntdSelectOptions),
      ]);
  }, [valueList]);

  // 删除公司
  const deleteOrg = useCallback(
    (index: number) => {
      let list = cloneDeep(valueList);
      list.splice(index, 1);
      setValueList(list);

      let changeFunction = props[props.trigger];
      if (changeFunction) {
        // const _value = list.map(item => item.value).filter(t => t);
        // changeFunction(_value.length ? _value : '');
        changeFunction(list);
      }
    },
    [props, valueList]
  );

  // const handleOk = (companyInfo: any) => {
  //     setValueList([...valueList.concat(companyInfo as AntdSelectOptions)])
  // }

  const changeOrgValue = useCallback(
    (info: any) => {
      let list:AntdSelectOptions[] = [];
      if (props.canAdd) {
        list = [...valueList.concat(info as AntdSelectOptions)];
      } else {
        list = [info];
      }

      setValueList(list);
      setShowModal(false);
      let changeFunction = props[props.trigger];
      if (changeFunction) {
        // const _value = list.map(item => item.value).filter(t => t);
        // changeFunction(_value.length ? _value : '');
        changeFunction(list);
      }
    },
    [props, valueList]
  );

  // 点击已选择的公司  进入公司详情
  const goCompany = useCallback(
    (e: any, item: any) => {
      // 如果没有选择  不进行操作
      // 判断点击的是否是选择的字的地方  其他地方不触发
      if (e.target.className === "#{$prefix}-select-selection-item") {
        if (item.value) {
          props.event.transitionToRoute &&
            props.event.transitionToRoute({
              route: "crm.organization.show",
              path: item.value,
            });
          return false;
        }
      }
    },
    [props.event]
  );

  return (
    <div className={style["org-selet-container"]}>
      {valueList.map((item, index) => {
        return (
          <div
            key={item.value}
            className={`${style["org-item"]} ${
              item.value ? style["link-class"] : ""
            }`}
          >
            <CopyComponent
              text={item.label}
              switchFlag={props.switchFlag}
              key={item.value}
            >
              <Select
                key={item.value}
                // onClick={(e) => goCompany(e, item)}
                showSearch={true}
                allowClear={true}
                placeholder="输入关键字查询公司"
                value={item}
                labelInValue={true}
                defaultActiveFirstOption={false}
                showArrow={true}
                filterOption={false}
                onFocus={handleFocus}
                onSearch={handleSearch}
                onChange={(item) => handleChange(item, index)}
                notFoundContent={fetching ? <Spin size="small" /> : "无数据"}
                // tagRender={tagRender}
              >
                {optionsArr.length &&
                  optionsArr.map((option: any) => {
                    let listIds = valueList.map((_l: any) => _l.value || _l);
                    if (!includes(listIds, option.value)) {
                      return (
                        <Option key={option.value} value={option.value}>
                          {option.label}
                        </Option>
                      );
                    } else {
                      return "";
                    }
                  })}
              </Select>
            </CopyComponent>
            {/* 当前公司配置可多选 */}
            {props.canAdd && (
              <React.Fragment>
                {!index ? (
                  <div onClick={addOrg} className={style["add-org"]}>
                    <PlusOutlined />
                  </div>
                ) : (
                  <div
                    onClick={() => deleteOrg(index)}
                    className={style["add-org"]}
                  >
                    <MinusOutlined />
                  </div>
                )}
              </React.Fragment>
            )}
          </div>
        );
      })}
      {/* 如果该角色有新增权限 并且 公司条数少于20  显示新建按钮 */}
      {props.canCreate && valueList.length < 20 && (
        <div
          className={style["create-org"]}
          onClick={() => setShowModal(true)}
          style={{ cursor: "pointer", display: "inline-block" }}
        >
          <PlusOutlined />
          {t("page.im.customerInfo.customer.add")}
        </div>
      )}
      <Modal
        title={t("page.im.customerInfo.customer.add")}
        visible={showModal}
        footer={null}
        forceRender={true}
        width={700}
        // onOk={changeOrgValue}
        onCancel={() => setShowModal(false)}
        bodyStyle={{ padding: "20px 0" }}
      >
        <CreateOrg
          onSave={changeOrgValue}
          organizationFields={props.organizationFields}
          onCancel={() => setShowModal(false)}
        />
      </Modal>
    </div>
  );
};
export default OrgSelect;
