import React, { ChangeEvent, useCallback, useEffect, useState } from "react";
import { Button, Input, message } from "udesk-ui";
import CopyComponent from "../../../../components/copy";
import {
  AddTask,
  DelTask,
  NewMessage,
  Phone,
  Warning,
} from "../../../../components/icon/icon";
import style from "./index.module.scss";
import Merge from "./merge";
import cloneDeep from "lodash-es/cloneDeep";
import { deleteJSON, getJSON } from "../../../../utils/ajax";
import isEqual from "lodash-es/isEqual";
import { useTranslation } from "react-i18next";
import Qs from "qs";

// interface SelfProps {
//   value?: IPhoneItem[];
//   onChange?: (value: string) => void;
//   imSubSessionId: number;
//   customerId: number;
//   switchFlag?: boolean;
// }

/*
    triggerChange(customer);
    customer:{
        cellphones_attributes:[
            {id:'',content:''}
        ]
    }
*/

interface ICustomer {
  cellphones_attributes: IPhoneItem[];
}

interface IPhoneItem {
  id?: number;
  content?: string;
  repeatPhone?: any;
  _destroy?: any;
  setToCustomer?: any;
}

const CustomerInfoPhone: React.FC<any> = (props) => {
  const { t } = useTranslation();
  //先缓存起来
  const [cellphonesArr, setCellphonesArr] = useState<IPhoneItem[]>(() => {
    return props.value
      ? props.value.length < 1
        ? [
            {
              content: "",
            },
          ]
        : cloneDeep(
            props.value.map((item: any) => {
              return {
                id: item.id,
                content: getPhoneText(item.content),
              };
            })
          )
      : [
          {
            content: "",
          },
        ];
  });
  const [reservationed, setReservationed] = useState(false); // 是否已经预约
  const [hasReservation, setHasReservation] = useState<any>({}); // 缓存已预约数据
  const [showMerge, setShowMerge] = useState(false);
  const [repeatPhone, setRepeatPhone] = useState<any>({});
  const [title, setTitle] = useState("合并客户");

  const getCellphone = useCallback((phoneNumber) => {
    try {
      if (Array.isArray(phoneNumber)) {
        return phoneNumber.map((i: any) => hidePhoneNumber(i + ""));
      } else {
        return hidePhoneNumber(phoneNumber + "");
      }
    } catch (e) {
      return Array.isArray(phoneNumber) ? [] : "";
    }
  }, []);

  const roleCanShow = useCallback(() => {
    let role_can = props.roleCan;
    let isAdmin = props.isAdmin;
    if (isAdmin || role_can.customer_show_cellphone_all) {
      return true;
    }

    if (!props.customer) {
      return false;
    }

    let user = props.user;
    let user_groups = user.user_groups || [];

    let ownerGroupId = props.customer.owner_group_id;
    if (
      role_can.customer_show_cellphone_group &&
      user_groups.find((item: any) => item.id === ownerGroupId)
    ) {
      return true;
    }

    let ownerId = props.customer.owner_id;
    if (
      role_can.customer_show_cellphone_personal &&
      ownerId &&
      parseInt(ownerId) === parseInt("" + user.id, 10)
    ) {
      return true;
    }

    return false;
  }, [props.isAdmin, props.roleCan, props.user, props.customer]);

  const roleCanEdit = useCallback(() => {
    let role_can = props.roleCan;
    let isAdmin = props.isAdmin;
    if (isAdmin || role_can.customer_edit_all) {
      return true;
    }

    if (!props.customer) {
      return false;
    }

    let user = props.user;
    let user_groups = user.user_groups || [];

    let ownerGroupId = props.customer.owner_group_id;
    if (
      role_can.customer_show_cellphone_group &&
      user_groups.find((item: any) => item.id === ownerGroupId)
    ) {
      return true;
    }

    let ownerId = props.customer.owner_id;
    if (
      role_can.customer_show_cellphone_personal &&
      ownerId &&
      parseInt(ownerId) === parseInt("" + user.id, 10)
    ) {
      return true;
    }

    return false;
  }, [props.isAdmin, props.roleCan, props.user, props.customer]);

  const getPhoneText = useCallback(
    (text) => {
      let isShow = roleCanShow();
      let hidePhone = props.customer.hide_number;
      if (isShow && !hidePhone) {
        return text;
      } else {
        return getCellphone(text);
      }
    },
    [getCellphone, props.customer.hide_number, roleCanShow]
  );

  // 监听props.value的变化,显示新的callphone列表
  useEffect(() => {
    setCellphonesArr(() => {
      return props.value
        ? props.value.filter((item: any) => item.content).length < 1
          ? [
              {
                content: "",
              },
            ]
          : cloneDeep(
              props.value.map((item: any) => {
                return {
                  id: item.id,
                  content: getPhoneText(item.content),
                  _destroy: item._destroy || false,
                };
              })
            )
        : [
            {
              content: "",
            },
          ];
    });
  }, [props.value]);

  // 动态设置已预约合并信息
  useEffect(() => {
    if (
      props.toCustomer &&
      props.toCustomer.key &&
      props.toCustomer.label &&
      props.toCustomer.reservation_reason === "cellphone"
    ) {
      setHasReservation(props.toCustomer);
      setReservationed(true);
    } else {
      setHasReservation({});
      setReservationed(false);
    }
  }, [
    props.toCustomer,
    props.toCustomer.key,
    props.toCustomer.label,
    props.toCustomer.reservation_reason,
  ]);

  // 电话数据改变
  const changePhonesAttr = useCallback(({ type, value, index }: any) => {
    setCellphonesArr((prevCellphones: IPhoneItem[]) => {
      const _prevCellphones = cloneDeep(prevCellphones);
      if (type === "add") {
        _prevCellphones.push({
          content: "",
        });
      } else if (type === "del") {
        _prevCellphones.splice(index, 1);
      } else if (type === "edit") {
        _prevCellphones.map((item: any, idx: number) => {
          if (idx === index) {
            item.content = value;
          }
          return item;
        });
      } else if (type === "repeatPhone") {
        _prevCellphones.map((item: any, idx: number) => {
          if (idx === index) {
            item["repeatPhone"] = value;
          }
          return item;
        });
      }

      return _prevCellphones;
    });
  }, []);
  // 增加
  const add = useCallback(() => {
    changePhonesAttr({
      type: "add",
    });
  }, [changePhonesAttr]);

  //删除一个
  const del = useCallback(
    (index: number, item: any) => {
      let _cellphonesArr = cellphonesArr.map((cell, idx) => {
        if (index === idx) {
          return Object.assign({}, cell, {
            _destroy: "1",
          });
        } else {
          return cell;
        }
      });
      setCellphonesArr([..._cellphonesArr]);
      if (item.content) {
        let changeFunction = props[props.trigger || "onChange"];
        if (changeFunction) {
          changeFunction(_cellphonesArr);
        }
      }
    },
    [cellphonesArr, props]
  );

  // 预约回调
  const handleMerge = useCallback(
    (isSelf: boolean, repeatInfo: any = {}, needMerge: boolean = false) => {
      setShowMerge(false);
      setReservationed(true);
      setHasReservation(repeatInfo);
      props.setToCustomer({ ...repeatInfo, reservation_reason: "cellphone" });
      if (props.selfChat.history && needMerge) {
        props.event.updateChat &&
          props.event.updateChat({
            chat: props.selfChat,
            key: "isDeletedCustomer",
            value: !isSelf,
          });
      }
    },
    [props]
  );

  // 电话输入框失焦处理
  const handleBlur = useCallback(
    (e: ChangeEvent<HTMLInputElement>, index: number) => {
      let changeFunction = props[props.trigger || "onChange"];
      let _value = e.target.value;

      // 如果值不变
      if (isEqual(props.value, cellphonesArr)) {
        return;
      }
      // 校验号码 是否已经和本客户已有的重复
      let hasNumberFlag: boolean = props.value.some((item: any) => {
        return item.content === _value;
      });

      if (hasNumberFlag) {
        return message.error("电话信息重复");
      }
      let params = {
        customer: {
          id: props.customer.id,
          cellphone: e.target.value,
        },
      };
      getJSON("/customers/check_duplication?" + Qs.stringify(params), {})
        .then((res: any) => {
          if (res.code !== 1000) {
            message.error("检测重复电话失败");
          } else {
            let dup = res.duplications || {};
            // 有重复邮箱,显示预约合并按钮 ? 如果电话已经预约了 ?
            if (dup.cellphone && dup.cellphone.id) {
              changePhonesAttr({
                type: "repeatPhone",
                key: "repeatPhone",
                value: {
                  label: dup.cellphone.name,
                  value: dup.cellphone.id,
                  key: dup.cellphone.id,
                },
                index: index,
              });
            } else {
              if (changeFunction) {
                const _cellphonesArr = cellphonesArr.map((cell: IPhoneItem) => {
                  return Object.assign(
                    {},
                    { ...cell },
                    !cell.content ? { _destroy: 1 } : {}
                  );
                });
                changeFunction(_cellphonesArr);
              }
            }
          }
        })
        .catch(() => {});
    },
    [cellphonesArr, changePhonesAttr, props]
  );

  // 实时输入
  const handleChange = useCallback(
    (e: any, index: number) => {
      let _value = e.target.value;

      changePhonesAttr({
        type: "edit",
        value: _value,
        index: index,
      });
    },
    [changePhonesAttr]
  );

  const transferToCustomer = useCallback(
    (id: number) => {
      props.event.transitionToRoute &&
        props.event.transitionToRoute({
          route: "crm.customer.show",
          path: id,
        });
    },
    [props.event]
  );

  // 显示合并modal
  const showMergeModal = useCallback(
    (repeatPhone: any) => {
      if (props.selfChat.history) {
        setTitle("客户合并");
      } else {
        setTitle("预约客户合并");
      }
      setRepeatPhone(repeatPhone);
      setShowMerge(true);
    },
    [props.selfChat.history]
  );

  // 取消立即预约
  const cancelReservation = useCallback(() => {
    deleteJSON("/customer_merge_reservation", {
      from_customer_id: props.customer.id,
      im_sub_session_id: props.imSubSessionId,
    })
      .then((res: any) => {
        // 取消之后,通知父页面, 可以再次预约
        if (res.code === 1000) {
          setReservationed(false);
          setHasReservation({});
          props.setToCustomer({});
        } else {
          message.error("取消失败,请重试");
        }
      })
      .catch((err: any) => {});
  }, [props]);

  function hidePhoneNumber(phoneNumber: any) {
    if (!phoneNumber) {
      return "";
    }
    if (phoneNumber.length > 7) {
      return phoneNumber.substr(0, 3) + "****" + phoneNumber.substr(-4, 4);
    } else {
      return "****" + phoneNumber.substr(-3, 3);
    }
  }

  const callPhone = useCallback(
    (phoneNumberObj: IPhoneItem) => {
      props.event.onMakeCall(phoneNumberObj);
    },
    [props.event]
  );
  const sendMessage = useCallback(
    (phoneNumberObj: IPhoneItem) => {
      props.event.openSms(phoneNumberObj);
    },
    [props]
  );
  return (
    <div className={style["cellphone-list"]}>
      {cellphonesArr.map((item, index) => {
        let Operator =
          index === 0 ? (
            <AddTask
              title={t("label.addPhone")}
              key={index}
              onClick={add}
              style={{
                fontSize: "18px",
                marginLeft: "10px",
                cursor: "pointer",
              }}
            />
          ) : (
            <DelTask
              key={index}
              onClick={() => del(index, item)}
              style={{
                fontSize: "18px",
                marginLeft: "10px",
                cursor: "pointer",
              }}
            />
          );
        return (
          <React.Fragment key={item.id || index}>
            {!item._destroy && (
              <div
                className={`${style["flex-row"]} ${style["flex-row-item-center"]} ${style["margin-bottom-15"]}`}
              >
                <div className={style["cell-item"]}>
                  <CopyComponent
                    text={item.content || ""}
                    switchFlag={props.switchFlag}
                  >
                    <Input
                      onBlur={(e) => handleBlur(e, index)}
                      onChange={(e) => handleChange(e, index)}
                      value={item.content}
                    />
                  </CopyComponent>
                  {props.currentUser && props.currentUser.isCall && (
                    <Phone
                      title={t("label.phone")}
                      onClick={() => callPhone(item)}
                      style={{
                        fontSize: "18px",
                        color: "green",
                        marginLeft: "10px",
                        cursor: "pointer",
                      }}
                    />
                  )}
                  {props.smsapp && props.smsapp.is_open && (
                    <NewMessage
                      title={t("label.message")}
                      onClick={() => sendMessage(item)}
                      style={{
                        fontSize: "18px",
                        color: "blue",
                        marginLeft: "10px",
                        cursor: "pointer",
                      }}
                    />
                  )}
                  {Operator}
                </div>

                {item.repeatPhone && item.repeatPhone.value && (
                  <div className={style["repeat-box"]}>
                    <Warning
                      style={{
                        fontSize: "18px",
                        marginRight: "10px",
                        cursor: "pointer",
                      }}
                      className={style["repeat-icon"]}
                    />

                    <div>
                      此电话与已有客户 “{" "}
                      <span
                        style={{ color: "#00B38B" }}
                        onClick={() =>
                          transferToCustomer(item.repeatPhone.value)
                        }
                      >
                        {item.repeatPhone.label}{" "}
                      </span>
                      ” 重复，
                      {props.selfChat.history ? (
                        <Button
                          onClick={() => showMergeModal(item.repeatPhone)}
                          size="small"
                        >
                          合并
                        </Button>
                      ) : (
                        <React.Fragment>
                          {reservationed ? (
                            "对话结束后可合并"
                          ) : (
                            <React.Fragment>
                              对话结束后可合并, 或者{" "}
                              <Button
                                onClick={() => showMergeModal(item.repeatPhone)}
                                size="small"
                              >
                                预约合并
                              </Button>
                            </React.Fragment>
                          )}
                        </React.Fragment>
                      )}
                    </div>
                  </div>
                )}
              </div>
            )}
          </React.Fragment>
        );
      })}
      {reservationed && hasReservation && hasReservation.label && (
        <div className={style["repeat-box"]}>
          <Warning className={style["repeat-icon"]} />
          <div>
            已预约对话结束后合并 “
            <span
              style={{ color: "#00B38B" }}
              onClick={() => transferToCustomer(hasReservation.value)}
            >
              {hasReservation.label}{" "}
            </span>
            ”
            <Button onClick={cancelReservation} size="small">
              取消预约
            </Button>
          </div>
        </div>
      )}

      <Merge
        title={title}
        customer={props.customer}
        concatVisible={showMerge}
        repeatCustomer={repeatPhone}
        imSubSessionId={props.imSubSessionId}
        cancelCombineCustomer={() => setShowMerge(false)}
        companyCategoryConfig={props.companyCategoryConfig}
        history={props.selfChat.history}
        reason="cellphone"
        mergeFn={handleMerge}
      />
    </div>
  );
};

export default CustomerInfoPhone;
