import React, { useCallback, useEffect, useState } from 'react';
import { Button, message, Modal, Tooltip } from 'udesk-ui';
import { useTranslation } from 'react-i18next';
import { putJSON } from '../../../utils';
import style from '../index.module.scss';
import { BlackList, ShiftOut } from '../../../components/icon/icon';

import { useDispatch } from '../../../store/store-hook';
import { useSelector } from '../../../store/store-hook';
import { RootState } from '../../../store/root-reducer';
import {csApiPutJSON} from "../../../utils/csApi";
interface DetailProps {
    customer: Customer;
    company: Company;
    event: any;
    chat: any;
}

const CustomerInfoDetail: React.FC<DetailProps> = ({ customer = {}, company, event, chat }) => {
    const dispatch = useDispatch();
    const { t } = useTranslation();
    const initData = useSelector((state: RootState) => state.initData);
    const isCustomerOwnerOn = company.is_customer_owner_on;
    const [level, setLevel] = useState<string>(customer.level || 'normal');
    const [block, setBlock] = useState(false);
    const [modal, setModal] = useState(false);
    const roleCan = useSelector((state: RootState) => state.roleCan);
    useEffect(() => {
        setLevel(customer.level || 'normal');
    }, [customer.level]);

    useEffect(() => {
        setBlock(customer.is_blocked || false);
    }, [customer.is_blocked]);

    const toggleVip = useCallback(() => {
        csApiPutJSON(`/cs/ts/api/customers/${customer.id}`, {
            customer: {
                level: level === 'vip' ? 'normal' : 'vip',
            },
        }).then(res=>res.data)
            .then((res) => {
                if (res.code === 1000) {
                    setLevel(level === 'vip' ? 'normal' : 'vip');
                    event.updateChatCustomer(res.customer);
                    message.info(t('tip.success.save'));
                } else {
                    message.error(
                        res.code_message || (res.exceptopn && res.exceptopn.message) || t('tip.error.saveTry')
                    );
                }
            })
            .catch((err) => {
                message.error(t('tip.error.saveTry'));
            });
    }, [customer.id, event, level, t]);

    const goCustomer = useCallback(() => {
        event.transitionToRoute &&
            event.transitionToRoute({
                route: 'crm.customer.show',
                path: customer.id,
            });
    }, [customer.id, event]);

    const goBlock = useCallback(() => {
        csApiPutJSON(`/cs/ts/api/customers/${customer.id}/toggle_block`)
            .then(res=>res.data)
            .then((res) => {
                if (res.code === 1000) {
                    setBlock((prev) => {
                        if (prev) {
                            message.success(t('page.im.customerInfo.cancel_blocked'));
                        } else {
                            message.success(t('page.im.customerInfo.blocked'));
                        }
                        event.updateChat &&
                            event.updateChat({
                                chat,
                                updateProps: {
                                    customer: {
                                        is_blocked: !prev,
                                    },
                                },
                            });
                        return !prev;
                    });
                    setModal(false);
                } else {
                    message.error(
                        res.code_message || (res.exceptopn && res.exceptopn.message) || t('tip.error.saveTry')
                    );
                }
            })
            .catch((err) => {
                message.error(t('tip.error.saveTry'));
            });
    }, [chat, customer.id, dispatch, t]);

    return (
        <div className={style['customer-info-header-container']}>
            <div className={style['customer-info-header']}>
                <div className={style['customer-info-header-left']}>
                    <div className={style['customer-info-uid']}>uid: {customer.uid}</div>

                    <Tooltip
                        placement="right"
                        title={
                            level === 'vip'
                                ? t('page.im.customerInfo.customer.enum.level.cancelvip')
                                : t('page.im.customerInfo.customer.enum.level.settingvip')
                        }
                    >
                        <div
                            onClick={toggleVip}
                            className={level === 'vip' ? style['vip-customer-box'] : style['common-customer-box']}
                        >
                            {level === 'vip' ? 'VIP' : t('page.im.customerInfo.customer.enum.level.normal')}
                        </div>
                    </Tooltip>
                </div>
                {!!roleCan.defriend_customer && (
                    <div className={style['customer-info-header-right']}>
                        {block && (
                            <React.Fragment>
                                <ShiftOut />
                                <span className={style['go-back-list']} onClick={goBlock}>
                                    {'移出'}
                                </span>
                            </React.Fragment>
                        )}
                        {!block && (
                            <React.Fragment>
                                <BlackList />
                                <span className={style['go-back-list']} onClick={() => setModal(true)}>
                                    {t('label.block')}
                                </span>
                            </React.Fragment>
                        )}
                        {!initData?.sdk && (<Button size={'small'} onClick={goCustomer}>
                            {t('label.detail')}
                        </Button>)}
                            
                    </div>
                )}
            </div>

            <div className={style['counts-1']}>
                <div className={style['count-item']}>
                    <span className={style['count-item-title']}>{t('label.callRecord')}</span>
                    <span className={style['count-item-count']}>{customer.calls_count || 0}</span>
                </div>
                <div className={style['count-item']}>
                    <span className={style['count-item-title']}>{t('label.imRecord')}</span>
                    <span className={style['count-item-count']}>{customer.im_sub_sessions_count || 0}</span>
                </div>
                <div className={style['count-item']}>
                    <span className={style['count-item-title']}>{t('label.ticket')}</span>
                    <span className={style['count-item-count']}>{customer.ticket_count || 0}</span>
                </div>
                <div className={style['count-item']}>
                    <span className={style['count-item-title']}>{t('label.videoRecord')}</span>
                    <span className={style['count-item-count']}>{customer.video_records_count || 0}</span>
                </div>
                {isCustomerOwnerOn && (
                    <React.Fragment>
                        <div className={style['count-item']}>
                            <span className={style['count-item-title']}>{t('label.owner')}</span>
                            <span className={style['count-item-count']}>{customer.owner_name}</span>
                        </div>
                        <div className={style['count-item']}>
                            <span className={style['count-item-title']}>{t('label.ownerGroup')}</span>
                            <span className={style['count-item-count']}>{customer.owner_group_name}</span>
                        </div>
                    </React.Fragment>
                )}
            </div>
            {/* {
                isCustomerOwnerOn &&
                <div className={style['counts-1']}>
                    <div className={style['count-item']}>
                        <span className={style['count-item-title']}>{t('label.owner')}</span>
                        <span className={style['count-item-count']}>{customer.owner_name}</span>
                    </div>
                    <div className={style['count-item']}>
                        <span className={style['count-item-title']}>{t('label.ownerGroup')}</span>
                        <span className={style['count-item-count']}>{customer.owner_group_name}</span>
                    </div>
                </div>
            } */}

            <Modal
                title="确认拉黑"
                visible={modal}
                onOk={goBlock}
                onCancel={() => setModal(false)}
                okText={t('confirm')}
                cancelText={t('cancel')}
            >
                <p>确定要拉黑这个客户吗</p>
            </Modal>
        </div>
    );
};
export default CustomerInfoDetail;
