import React, { ComponentProps, useEffect, useState } from 'react';
import CustomerInfoForm from './form';
import CompanyInfoComponent from '../company-info/company-info';
import style from '../index.module.scss';
import isEqual from 'lodash-es/isEqual';
import { csApiGetJSON } from '../../../utils/csApi';
import { useSelector } from '../../../store/store-hook';
import { RootState } from '../../../store/root-reducer';
type Props = {
    showCount: boolean;
    customer: Customer;
    // organizations: Organization[];
    // company: Company;
    // customerFieldsv2: any;
    // sessionKey: string;
    // chat: any;
    // imSubSessionId: number;
    // customerId: number;
    // event: any
    // roleCan: any
    // companyCategoryConfig: any
    // organizationFields: any
    organizationFieldsv2: any;
    // currentUser: any
    // smsapp?: any
    // fieldHooks?: any
    // user?: any
    // isAdmin?: any
    // customerFieldsAuthority?: any
} & ComponentProps<typeof CustomerInfoForm>;

const CustomerInfo: React.FC<Props> = (props) => {
    const { chat = {}, event, organizationFieldsv2, organizationFields, organizations } = props;

    const { customer, im_sub_session_id: imSubSessionId, customer_id: customerId, session_key: sessionKey } = chat;
    const initData = useSelector((state: RootState) => state.initData);
    //tab切换状态  客户信息 customer 公司信息 org
    const [type, setType] = useState<string>('');
    const [showOrg, setShowOrg] = useState<boolean>(false);
    /**
  cc_show: false
    code: 1000
    im_priority_display: false
    im_show: true
 */
    useEffect(() => {
        const URL = '/cs/ts/api/custom_fields/organization_setting';
        csApiGetJSON(URL).then(res=>res.data)
            .then(res=>res.data)
            .then((res) => {
                if (res.code === 1000) {
                    if (isEqual(initData.page, 'im')) {
                        setShowOrg(res.im_show);
                        setType(res.im_show && res.im_priority_display ? 'org' : 'customer');
                    } else if (isEqual(initData.page, 'mpv')) {
                        setShowOrg(res.mpv_show);
                        setType(res.mpv_show && res.im_priority_display ? 'org' : 'customer');
                    } else {
                        setShowOrg(res.cc_show);
                        setType(res.cc_show && res.im_priority_display ? 'org' : 'customer');
                    }
                } else {
                    setType('customer');
                }
            })
            .catch((err) => {
                setType('customer');
            });
    }, []);

    return (
        <div className={style['info-page']}>
            {customer ? (
                <React.Fragment>
                    {showOrg && (
                        <div className={style['tab-list']}>
                            <div>
                                <span
                                    className={`${style['tab-item-name']} ${
                                        type === 'customer' ? style['active'] : ''
                                    }`}
                                    onClick={() => setType('customer')}
                                >
                                    客户信息
                                </span>
                                <span className={style['filter-span-separator']}>|</span>
                            </div>
                            {showOrg && (
                                <div>
                                    <span
                                        className={`${style['tab-item-name']} ${type === 'org' ? style['active'] : ''}`}
                                        onClick={() => setType('org')}
                                    >
                                        公司信息
                                    </span>
                                </div>
                            )}
                        </div>
                    )}

                    <div className={style['tab-content']}>
                        {type === 'customer' && <CustomerInfoForm {...props} switchFlag={true} />}

                        {type === 'org' && (
                            <CompanyInfoComponent
                                event={event}
                                organizationFieldsv2={organizationFieldsv2}
                                organizationFields={organizationFields}
                                sessionKey={sessionKey}
                                customerId={customerId}
                                chat={chat}
                                imSubSessionId={imSubSessionId}
                                organizations={organizations}
                            />
                        )}
                    </div>
                </React.Fragment>
            ) : null}
        </div>
    );
};
export default CustomerInfo;
