import React, { useImperativeHandle, useMemo, useRef } from "react";
import { RootState } from "../store/root-reducer";
import { useSelector, useDispatch } from "../store/store-hook";
import { updateAgentConfig } from "../store/agent-config";
import Index from "./index";
interface Props {
  width: number;
  cssMaxWidth: string;
  ref: any;
  onWidthChange: (params: any) => void;
}

export interface ImPanelEntryRef {
  showTicket: (config: { show: boolean; zIndex: number }) => void;
  showVisit: (config: { show: boolean; zIndex: number }) => void;
  showRecord: (config: { show: boolean; zIndex: number }) => void;
  changeImPanelTab: (id: string, customerId: number) => void;
}

const ImPanelEntry = React.forwardRef<ImPanelEntryRef, Props>(
  ({ width, onWidthChange, cssMaxWidth, ...props }, ref) => {
    const agentConfig = useSelector(
      (state: RootState) => state.agentConfig || {}
    );
    const currentUser = useSelector((state: RootState) => state.currentUser);
    const fields = useSelector((state: RootState) => state.fields);
    const roleCan = useSelector((state: RootState) => state.roleCan);
    const company = useSelector((state: RootState) => state.company);
    const initData = useSelector((state: RootState) => state.initData);
    const noteTemplates = useSelector(
      (state: RootState) => state.noteTemplateConfig.noteTemplates
    );

    const dispatch = useDispatch();
    const {
      smsapp,
      agents,
      page,
      agentGroups,
      ticketTab,
      ticketTemplates,
      ticketFieldsAbilities,
      kmrobot_url,
      previewUrl,
      imNewPlatforms,
      companyCategoryConfig,
      customerFieldsAuthority,
      user,
      admin,
      isAdmin,
      fieldHooks,
      event,
      chat,
    } = initData;

    const _event: any = useMemo(() => {
      return {
        updateChat: function (chat: any, updateProps: any) {
          event.updateChat({
            chat: chat,
            updateProps: updateProps,
          });
        },
        transitionToRoute: function (pathConfig: any) {
          event.transitionToRoute(pathConfig);
        },
        createReturnVisit: function () {
          // TODO 创建回访计划
          console.log("TODO 创建回访计划");
        },
        updateAgentConfig: function (data: any) {
          dispatch(
            updateAgentConfig({
              agent_tab_config: data,
            })
          );
        },
        updateChatCustomer: function (customer: Customer) {
          event.updateChatCustomer(customer);
        },
        onMakeCall: function (phoneNumberObj: any) {
          event.onMakeCall(phoneNumberObj);
        },
        openSms: function (phoneNumberObj: any) {
          event.openSms(phoneNumberObj);
        },
        createEmailTicket: function (chat: IMineChat) {
          event.createEmailTicket(chat);
        },
        onUse: function (type: string, data: any) {
          event.onKnowledgeUseClick(type, data);
        },
        onSend: function (type: string, data: any) {
          event.onKnowledgeSendClick(type, data);
        },
        reloadModel: function () {
          event.reloadModel();
        },
      };
    }, [dispatch, event, chat?.platform]);

    const imPanelRef = useRef<any>();
    // 父组件调用的方法

    useImperativeHandle(ref, () => ({
      showTicket: function (config) {
        // 调用子组件的方法
        if (typeof imPanelRef.current?.showTicket === "function") {
          imPanelRef.current.showTicket(config);
        }
      },
      showVisit: function (config) {
        // 调用子组件的方法
        if (typeof imPanelRef.current?.showVisit === "function") {
          imPanelRef.current.showVisit(config);
        }
      },
      showRecord: function (config) {
        // 调用子组件的方法
        if (typeof imPanelRef.current?.showRecord === "function") {
          imPanelRef.current.showRecord(config);
        }
      },
      changeImPanelTab: function (id: string, customerId: number) {
        // 调用子组件的方法
        if (typeof imPanelRef.current?.changeImPanelTab === "function") {
          imPanelRef.current.changeImPanelTab(id, customerId);
        }
      },
    }));

    return (
      <Index
        customerFieldsv2={fields.customerFieldsv2}
        organizationFieldsv2={fields.organizationFieldsv2}
        organizationFields={fields.organizationFields}
        noteFields={fields.noteFields}
        currentUser={currentUser}
        roleCan={roleCan}
        company={company}
        agentConfig={agentConfig}
        noteTemplates={noteTemplates}
        //
        page={page}
        chat={chat}
        ticketTemplates={ticketTemplates}
        smsapp={smsapp}
        imNewPlatforms={imNewPlatforms}
        companyCategoryConfig={companyCategoryConfig}
        event={_event}
        customerFieldsAuthority={customerFieldsAuthority}
        user={user}
        isAdmin={isAdmin}
        fieldHooks={fieldHooks}
        agents={agents}
        kmrobot_url={kmrobot_url}
        previewUrl={previewUrl}
        ticketFieldsAbilities={ticketFieldsAbilities}
        ticketTab={ticketTab}
        agentGroups={agentGroups}
        admin={admin}
        ref={imPanelRef}
        width={width}
        onWidthChange={onWidthChange}
        cssMaxWidth={cssMaxWidth}
      />
    );
  }
);

ImPanelEntry.displayName = "ImPanelEntry";

export default ImPanelEntry;
