/* eslint-disable jsx-a11y/anchor-is-valid */
import React, { useCallback, useEffect, useState,Fragment } from "react";
import { Form, message, Select } from "udesk-ui";
import { useTranslation } from "react-i18next";
import * as FieldComponent from "../../components/_fields";
import style from "./index.module.scss";
import {
  formatCustomField,
  getFieldsDefaultValue,
  getJSON,
  postJSON,
} from "../../utils/index";
import CustomField from "../../components/custom-field-container";
import { useDispatch } from "../../store/store-hook";
import { ChatUtils } from "../../utils/im-utils";
import { RootState } from "../../store/root-reducer";
import { useSelector } from "../../store/store-hook";
import { isEqual } from "lodash-es";
const note_category: { [key: string]: string } = {
  im: "im",
  mpv: "video",
  call: "call",
};
const { Option } = Select;
type Props = {
  chat: IMineChat;
  noteTemplates: Template[];
  company: Company;
  event: any;
};
const ImNote: React.FC<Props> = (props) => {
  const dispatch = useDispatch();
  const initData = useSelector((state: RootState) => state.initData);
  const businessRender = initData?.template?.business?.render;
  const oldBusinessTemplate = initData?.template?.business?.oldBusinessTemplate;
  const [formRef] = Form.useForm();
  // 多语言
  const { t } = useTranslation();
  //ember传进来的参数
  // const udeskWebappContext: ImContextProps = useContext(UdeskContext);
  const { chat, noteTemplates, company, event } = props;
  const [, setIsSubjectEnable] = useState<boolean>(true); //是否显示主题
  const [templateId, setTemplateId] = useState<string>(""); //当前选择的模板id
  const [initFormData, setInitFormData] = useState<any>();
  //主题字段是否必填
  const [, setIsSubjectRequired] = useState<boolean>(false);
  // 自定义字段
  const [customField, setCustomField] = useState<any[]>([]);
  //此业务记录是否处于已完成状态
  const [noteComplete, setNoteComplete] = useState<boolean>(true);
  const [id, setId] = useState<number>(0);
  //处理工单模板选项数据
  const _noteTemplates: AntdSelectOptions[] = noteTemplates
    .filter((item: any) => item.category === note_category[initData.page]) //
    .map((item: any) => {
      return {
        value: item.id,
        label: item.name,
      };
    });
  // 公司选项
  const organizations =
    chat?.organizations?.map((item: { id: number; name: string }) => {
      return {
        value: item.id,
        title: item.name,
      };
    }) ?? [];
  const [allDone, setAllDone] = useState<boolean>(true);

  const handleAllReqHasValue = useCallback(
    (data, customField: IFieldItem[]) => {
      let isRequestFields = customField.filter(function (field) {
        return field.isRequired;
      });
      let allDone = isRequestFields.every(function (item, index) {
        return data[item.field.id];
      });
      setAllDone(allDone);
      event.updateChat &&
        event.updateChat({
          chat: chat,
          updateProps: {
            note_complete: allDone,
          },
        });
    },
    [chat, dispatch]
  );

  //根据模板id获取详情
  const getTemplateById = useCallback(
    (id: string) => {
      getJSON("/notes/detail", {
        type: initData.page, //,
        ref_id: chat.im_sub_session_id, //,
        template_id: id,
      }).then((res) => {
        setId(res.data.id);
        if (id) {
          event.updateChat &&
            event.updateChat({
              chat,
              updateProps: {
                agent_note_template_id: parseInt(id),
                noteHasReqField: ChatUtils.getNoteHasReqField(parseInt(id)),
              },
            });
        }

        setTemplateId(res.data.agent_note_template.id);
        if (!isEqual(initData.page, "mpv")) {
          setNoteComplete(res.data.im_sub_session.note_complete);
        }
        //控制是否显示 主题 字段
        setIsSubjectEnable(res.data.agent_note_template.is_subject_enable);
        setIsSubjectRequired(res.data.agent_note_template.is_subject_required);
        // 处理自定义字段
        let customField = formatCustomField({
          fields: res.data.agent_note_template.fields,
          trigger: "onBlur",
        });
        let customerFormData: { [index: string]: any } = {}; //自定义字段的formData
        //循环用户自定义字段
        customField.forEach((item: any) => {
          customerFormData[item.field.id] =
            item.field.id.split("_")[0] === "TextField" ? "" : [];
        });
        //将已经有数据的自定义字段和初始化完成的自定义字段合并
        customerFormData = Object.assign(
          getFieldsDefaultValue(customField),
          res.data.custom_fields
        );

        // 将基础的表单数据和自定义表单数据合并
        let formData = Object.assign(
          {
            content: res.data.content || "",
            organization_id: res.data.organization_id,
          },
          customerFormData
        );
        setInitFormData(formData);
        // 表单重置初始值
        formRef.setFieldsValue(formData);
        if (res.data.agent_note_template.is_subject_enable) {
          customField.unshift({
            field: {
              disabled: false,
              id: "content",
              isRequired: res.data.agent_note_template.is_subject_required,
              trigger: "onBlur",
              type: "text",
              customValidFn: function (rule: any, value: string) {
                let validTexts: string[] = [];
                if (
                  res.data.agent_note_template.is_subject_required &&
                  !value
                ) {
                  validTexts.push("该字段为必填项");
                }
                if (validTexts.length > 0) {
                  const errMsg = validTexts.join(",");
                  return Promise.reject(errMsg);
                } else {
                  return Promise.resolve();
                }
              },
            },
            isRequired: res.data.agent_note_template.is_subject_required,
            label: "主题",
          });
        }
        //存储自定义字段  循环用
        setCustomField(customField);
        handleAllReqHasValue(formData, customField);
      });
    },
    [chat, dispatch, formRef, handleAllReqHasValue]
  );

  useEffect(() => {
    getTemplateById("");
  }, [getTemplateById]);

  const changeTemplate = useCallback(
    (value: string) => {
      getTemplateById(value);
    },
    [getTemplateById]
  );

  const submit = useCallback(
    (data: any) => {
      postJSON(
        "/notes",
        Object.assign(
          {
            type: initData.page, //,
            ref_id: chat.im_sub_session_id, //,
            template_id: templateId,
          },
          data
        )
      )
        .then((res) => {
          //当前表单的值
          let values = formRef.getFieldsValue();
          //报错信息
          if (res.code !== 1000 && res.code !== "1000") {
            message.error(
              res?.code_message || res?.exception?.message || t("saveError")
            );
            let key = Object.keys(data.custom_fields)[0];
            values[key] = initFormData[key];
            formRef.setFieldsValue(values);
            return;
          }
          message.success("保存成功");
          // 如果当前业务记录不是完成状态  判断必填项是否完成  如果都填完  就请求一个完成记录的接口

          handleAllReqHasValue(values, customField);
          if (!noteComplete) {
            //是否必填都填写的 标识
            let allReqHasValue = true;
            //从自定义字段中筛选出必填项
            var imReqFields = customField.filter(function (field) {
              return field.isRequired;
            });
            //如果没有必填项 也认为没有填全
            if (imReqFields.length === 0) {
              allReqHasValue = false;
            }
            //查看必填项是否有值
            imReqFields.forEach(function (item) {
              if (!values[item.field.id]) {
                allReqHasValue = false;
              }
            });
            if (allReqHasValue) {
              postJSON("/im/note_complete", {
                complete: true,
                im_sub_session_id: chat.im_sub_session_id,
              }).then((res) => {
                setNoteComplete(true);
              });
            }
          }
          //将已经有数据的自定义字段和初始化完成的自定义字段合并
          let customerFormData = Object.assign(
            getFieldsDefaultValue(customField),
            res.data.custom_fields
          );
          // 将基础的表单数据和自定义表单数据合并
          let formData = Object.assign(
            {
              content: res.data.content,
              organization_id: res.data.organization_id,
            },
            customerFormData
          );
          setInitFormData(formData);
          // 表单重置初始值
          formRef.setFieldsValue(formData);
        })
        .catch((e) => {
          console.log(e);
        });
    },
    [
      chat,
      templateId,
      formRef,
      handleAllReqHasValue,
      customField,
      noteComplete,
      t,
      initFormData,
    ]
  );
  const valueChange = useCallback(
    (value: any) => {
      // 获取改变的字段的key值
      let key = Object.keys(value)[0];
      if (key.split("_")[0] === "SelectField" && Array.isArray(value[key])) {
        value[key] = value[key].join(",");
      }

      if (!value[key]) value[key] = "";
      //判断改变的字段是自定义字段还是系统字段
      let params: { [key: string]: any } = {};
      if (key === "content" || key === "organization_id") {
        params = value;
      } else {
        params = {
          custom_fields: value,
        };
      }
      // 触发form表单的校验  检查校验错误信息中是否包含本次修改数据   如果不存在  提交本条数据
      formRef
        .validateFields(Object.keys(value))
        .then((res: any) => {
          submit(params);
        })
        .catch((e: any) => {
          let canSubmit = true;
          e.errorFields.forEach((item: any) => {
            if (item.name[0] === key) {
              canSubmit = false;
            }
          });
          if (canSubmit) {
            submit(params);
          }
        });
    },
    [initFormData, formRef, submit]
  );

  return (
    <Fragment>
      { !oldBusinessTemplate ? (<div className={style["note-wrapper"]}>
      {!allDone && <div className={style["tip"]}>{t("page.im.note.tip")}</div>}
      {_noteTemplates.length ? (
        <Select
          showArrow={true} //显示下拉小箭头
          onChange={changeTemplate} //选中 option
          getPopupContainer={(triggerNode) => triggerNode} //菜单渲染父节点
          defaultValue={_noteTemplates[0].value as string}
          bordered={false}
          value={templateId}
          className={style["select-wrapper"]}
          dropdownMatchSelectWidth={false}
        >
          {_noteTemplates.map((d: AntdSelectOptions) => (
            <Option key={d.value} value={d.value}>
              {d.label}
            </Option>
          ))}
        </Select>
      ) : (
        <a className={style["select-name"]}>{_noteTemplates[0].label}</a>
      )}

      <Form
        form={formRef}
        initialValues={{
          content: "",
          organization_id: "",
        }}
        onValuesChange={(changedValues, allValues) =>
          valueChange(changedValues)
        }
        layout="vertical"
      >
        {/* 公司 */}
        {organizations.length > 1 && company.support_multi_organizations && (
          <Form.Item
            name="organization_id"
            label={t("page.im.ticket.organizations")}
          >
            <FieldComponent.droplist
              id="level"
              type="droplist"
              trigger="onChange"
              options={organizations}
            />
          </Form.Item>
        )}
        {/* 主题加入到了自定义字段里，以为要处理必填逻辑，统一用自定义字段的 */}
        {customField.length > 0 && (
          <CustomField
            fields={customField}
            switchFlag={true}
            notUseDefault={!!id}
          />
        )}
      </Form>
    </div>) : businessRender && businessRender() }
    
    </Fragment>
    
  );
};

export default ImNote;
