import React from 'react';
import style from './style.module.scss';
import { HuiFan, ZhuanYi } from '../../../components/icon/icon';
import { csApiPostJSON } from '../../../utils/csApi';
import isFunction from 'lodash-es/isFunction';
import { message } from 'udesk-ui';

interface ParallelTabProps {
    switchParallel: () => void;
    hideParallel: () => void;
    component: React.Component;
    text: string;
    agentTabConfig: AgentTabConfig;
    event: any;
}

/**
 * 平行视界
 * @param props
 */
const ParallelTab: React.FC<ParallelTabProps> = (props) => {
    // const udeskImContext: ImContextProps = useContext(UdeskContext);
    // const { event } = udeskImContext;
    const { event } = props;

    const handleSwitchParallel = () => {
        const _data: any = {
            agent_tab_config: {
                ...props.agentTabConfig,
                sortDesc: !props.agentTabConfig.sortDesc,
            },
        };
        csApiPostJSON(`/cs/im/agent/im/agent_config`, {
            data: _data,
        }).then(res=>res.data)
            .then(() => {
                if (isFunction(event.updateAgentConfig)) {
                    event.updateAgentConfig({
                        ...props.agentTabConfig,
                        sortDesc: !props.agentTabConfig.sortDesc,
                    });
                }
            })
            .catch((err) => {
                message.error('保存失败,请重试');
            });
    };

    const handleHideParallel = () => {
        const _data: any = {
            agent_tab_config: {
                ...props.agentTabConfig,
                parallel: '',
                sortDesc: false,
            },
        };
        csApiPostJSON(`/cs/im/agent/im/agent_config`, {
            data: _data,
        }).then(res=>res.data)
            .then(() => {
                if (isFunction(event.updateAgentConfig)) {
                    event.updateAgentConfig({
                        ...props.agentTabConfig,
                        parallel: '',
                        sortDesc: false,
                    });
                }
            })
            .catch(() => {
                message.error('保存失败,请重试');
            });
    };
    return (
        <div className={style['parallel-container']}>
            <div className={style['parallel-header']}>
                <div className={style['header-name']}>{props.text}</div>
                <div className={style['header-icons']}>
                    <span onClick={handleSwitchParallel}>
                        <ZhuanYi />
                    </span>
                    <span onClick={handleHideParallel}>
                        <HuiFan />
                    </span>
                </div>
            </div>
            <div className={style['parallel-body']}>{props.component}</div>
        </div>
    );
};

export default ParallelTab;
