import React from "react";
import { useDrag, useDrop } from "react-dnd";
import { Col, Row } from "udesk-ui";
import { DanSuanShiXiao, XuanZeDanXuan } from "../../../components/icon/icon";
import { MenuOutlined } from "@ant-design/icons";

const style: React.CSSProperties = {
  padding: "8px 12px",
  borderBottom: "1px solid #fafafa",
};

const handleStyle: React.CSSProperties = {
  display: "inline-block",
  cursor: "move",
};

export interface DndItemProps {
  id: string;
  key: string;
  text: string;
  type: string;
  checked: boolean;
  checkedItemToContainer: (id: string) => void;
  moveItemToContainer: (id: string, to: number) => void;
  findItemToContainer: (id: string) => { index: number };
}

interface Item {
  type: string;
  id: string;
  originalIndex: string;
}

export const DndItem: React.FC<DndItemProps> = ({
  id,
  text,
  checked,
  checkedItemToContainer,
  moveItemToContainer,
  findItemToContainer,
  type,
}) => {
  const originalIndex = findItemToContainer(id).index;
  const [{ opacity }, drag, preview] = useDrag({
    item: { type: type, id, originalIndex },
    collect: (monitor) => ({
      isDragging: monitor.isDragging(),
      opacity: monitor.isDragging() ? 0 : 1,
    }),
    // end: (dropResult, monitor) => {
    //     const { id: droppedId, originalIndex } = monitor.getItem()
    //     const didDrop = monitor.didDrop()
    //     if (!didDrop) {
    //         moveItemToContainer(droppedId, originalIndex)
    //     }
    // },
  });

  const [, drop] = useDrop({
    accept: type,
    canDrop: () => false,
    hover({ id: draggedId }: Item, monitor) {
      if (draggedId !== id) {
        const { index: overIndex } = findItemToContainer(id);
        moveItemToContainer(draggedId, overIndex);
      }
    },
  });

  const handleItemChecked = () => {
    checkedItemToContainer(id);
  };

  return (
    <div ref={preview} style={{ ...style, opacity }}>
      <Row>
        <Col span={4}>
          <span ref={(node) => drag(drop(node))} style={handleStyle}>
            <MenuOutlined />
          </span>
        </Col>
        <Col span={12}>{text}</Col>
        <Col span={4}>
          <span onClick={handleItemChecked}>
            {checked ? <XuanZeDanXuan /> : <DanSuanShiXiao />}

            {/* {checked ? '11' : '22'} */}
          </span>
        </Col>
      </Row>
    </div>
  );
};
