import React, { useEffect, useMemo, useState } from "react";
import style from "./index.module.scss";
import { Horn } from "../../../components/icon/icon";

const AudioType: React.FC<any> = (props) => {
  // const { src } = props.contentObject.data.content;
  const [duration, setDuration] = useState<number>(0);
  const [isPlaying, setIsPlaying] = useState<boolean>(false);
  const { msgItem } = props;
  const contentObject =
    typeof msgItem.content === "string"
      ? JSON.parse(msgItem.content)
      : msgItem.content;
  // const audioRef = useRef<any>();
  const audioElement = useMemo(() => {
    const audio = window.document.createElement("audio");
    audio.src = contentObject.data.content;
    return audio;
  }, [contentObject.data]);

  const startPlayAudio = () => {
    audioElement.play();
    setIsPlaying(true);
  };
  const endPlayAudio = () => {
    setIsPlaying(false);
  };
  useEffect(() => {
    audioElement.addEventListener("loaded", (e: any) => {
      setDuration(e.duration === 0 ? 1 : e.duration);
    });
    return () => {
      audioElement.removeEventListener("loaded", (e) => {});
    };
  }, [audioElement]);

  useEffect(() => {
    audioElement.addEventListener("ended", endPlayAudio);
    audioElement.removeEventListener("ended", (e) => {});
  }, [audioElement, isPlaying]);

  return (
    <div className={style["audio-container"]}>
      <div className={style["audio-top"]} onClick={startPlayAudio}>
        {
          <span>
            <b className={style["duration"]}>{`${duration} "`}</b>
          </span>
        }
        {isPlaying && <span className={style["aduio-playing"]}></span>}

        {!isPlaying && (
          <span className={style["r-agent"]}>
            {" "}
            <Horn />
          </span>
        )}
      </div>

      {contentObject.data.recognition && (
        <>
          <div className={style["audio-hr"]}></div>
          <div className={style["audio-text"]}>
            {contentObject.data.recognition}
          </div>
        </>
      )}
    </div>
  );
};
export default AudioType;
