import React, { useCallback, useEffect, useRef, useState } from "react";
import moment, { Moment } from "moment";
import { Button, DatePicker, Input, message, Spin } from "udesk-ui";
import * as speicalTypeItem from "./components";
import style from "./index.module.scss";
import {
  getJSON,
  imLogIsEvent,
  imLogIsMessage,
  postJSON,
  r,
} from "../../utils/index";
import { useTranslation } from "react-i18next";
import { CheckCircle } from "../../components/icon/icon";
import { CuoHao } from "../../components/icon/icon";
import { useDateRanges } from "../../utils/util";
import {csApiGetJSON} from "../../utils/csApi";

const { RangePicker } = DatePicker;
type MsgType =
  | "audio"
  | "video"
  | "file"
  | "image"
  | "location"
  | "rich"
  | "struct"
  | "message";

interface Msg {
  name: string;
  content: string;
  time: string;
  sessionId: string;

  [propName: string]: any;
}

interface MsgRquestRes {
  msgList: Msg[];
  lastId: string;
  nextId: string;
}

type Props = {
  chat: IMineChat;
  event: any;
  setShowRecord: (option: { show: boolean }) => void;
};
const ImMsgRecord: React.FC<Props> = (props) => {
  //多语言配置
  const { t } = useTranslation();
  //ember传进来的参数
  // const udeskWebappContext: any = useContext(UdeskContext)
  const { chat, event } = props;
  // const [udeskWebappContext, setUdeskWebappContext] = useState<any>(useContext(UdeskContext))
  const [msgList, setMsgList] = useState<Msg[]>([]); //消息列表 非查询
  const [searchMsgList, setSearchMsgList] = useState<Msg[]>([]); //消息列表 查询的结果
  const [nearbyMsgList, setNearbyMsgList] = useState<Msg[]>([]);
  // const [tabType, setType] = useState<string>('all')//tab切换状态  'all' 'flie' 'pic'
  const [lastId, setLastId] = useState<string>(""); //上一个对话的id
  const [nextId, setNextId] = useState<string>(""); //下一个对话的id
  const [topSpinning, setTopSpinning] = useState<boolean>(false); //头部loading加载状态是否显示
  const [bottomSpinning, setBottomSpinning] = useState<boolean>(false); //底部loading加载状态是否显示
  const [searchType, SetSearchType] = useState<string>("all"); //是否点击的查询   点击了以后 显示查询结果   否则显示全部
  const [searchValue, setSearchValue] = useState<string>(""); //输入的搜索关键词
  const [startDate, setStartDate] = useState<Moment>(
    moment().subtract(3, "month")
  ); //查询的开始时间
  const [endDate, setEndDate] = useState<Moment>(moment()); //查询的结束始时间
  const msgRef: any = useRef(null);
  const [currentLog, setCurrentLog] = useState<any>(); //翻页时的当前的最前的一条数据  用来请求数据回来 记忆滚动用
  const [searchPage, setSearchPage] = useState<number>(1); //当前请求的页数  搜索聊天记录用
  const [totalPage, setTotalPage] = useState<number>(0); //搜索数据的总页数
  const [totalCount, setTotalCount] = useState<number>(0); //搜索数据的总条数
  const [searchLastId, setSearchLastId] = useState<string>("");
  const [searchNextId, setSearchNextId] = useState<string>("");
  // 进入第一次  搜索当前回话的消息记录
  useEffect(() => {
    initData(chat.im_sub_session_id + "", msgList, -1).then((res) => {
      setLastId(res?.lastId);
      setNextId(res?.nextId);
      setMsgList((res?.msgList as Msg[]) || []);
    });
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  const _scrollToImLog = useCallback(
    (offset = 0) => {
      // 获取滚动元素
      let results: any = document.getElementById(`result`);
      //获取当前最新元素所在位置
      let imLogTop = currentLog
        ? document.getElementById(`data-im-log-${currentLog.id}`)?.offsetTop
        : 0;

      if (!imLogTop) {
        return;
      }
      // 外层元素滚动到 之前滚动的高度+新加进来的会话列表的高度+偏移量
      results.scrollTop = results.scrollTop + imLogTop + offset;
    },
    [currentLog]
  );

  //每次请求以后把请求前的第一或者最新的第一条复制以后  自动滚到该位置
  useEffect(() => {
    if (!currentLog) return;
    _scrollToImLog(-46);
  }, [_scrollToImLog, currentLog]);

  // 通过sessionId获取消息列表
  const getMsgList = useCallback(
    (id: string) => {
      return new Promise<MsgRquestRes>((resolve, reject) => {
        csApiGetJSON('/cs/im/agent/im/im_logs/agent_logs', {
          params: {
            im_session_id: id,
            customer_id: chat.customer_id,
            session_key: chat.session_key,
          }
        }).then(res=>res.data).then(res => {
          if (res.code === 1000 || res.code === "1000") {
            resolve({
              msgList: res.im_logs,
              nextId: res.next_im_session_id,
              lastId: res.last_im_session_id,
            });
          } else {
            reject(res);
          }
        });
      });
    },
    [chat.customer_id, chat.session_key]
  );

  //初始化请求数据
  //nextId 初始化都传-1  只将请求后的第一次的res.nextId复制给nextId  因为是向上查询  nextId只会是第一次的nextId
  const initData: (
    id: string,
    newMsgList: Msg[],
    nextId?: any
  ) => Promise<any> = useCallback(
    (id, newMsgList, nextId) => {
      return getMsgList(id)
        .then((res) => {
          // 将之前请求数据  与 此次请求数据合并
          const _newMsgList = [...res.msgList, ...newMsgList];
          //因为是向上查询  所以下一个的sessionId取第一次的值  一旦nextId有值 便不再重新赋值
          let currentNextId = nextId === -1 ? res.nextId : nextId;
          // 如果 聊天记录条数小于20  并且  还有上一条对话    继续请求 上一次对话  知道聊天记录多余20条
          if (_newMsgList.length < 20 && res.lastId) {
            return initData(res.lastId, _newMsgList, currentNextId);
          } else {
            return {
              msgList: _newMsgList,
              lastId: res.lastId.toString(),
              nextId: nextId.toString(),
            };
          }
        })
        .catch((e) => {
          message.error(
            e.code_message ||
              (e.exceptopn && e.exceptopn.message) ||
              t("page.im.record.requestError")
          );
          return {};
        });
    },
    [getMsgList, t]
  );

  let isBottom = false,
    isTop = false;
  //监听的滚动事件
  //如果是搜索事件  搜索只有触底翻页   即searchType是search时
  //查询全部和查询前后文是同一个接口 只是在处理返回数据会有区别  即searchType是all和nearby
  const handleMsgScroll = useCallback(
    (e) => {
      const ref = e.target;
      const _lastId = searchType === "nearby" ? searchLastId : lastId;
      const _nextId = searchType === "nearby" ? searchNextId : nextId;

      // 滚动到顶部加载前一个session
      if (ref.scrollTop < 1 && _lastId && !isTop && !isBottom) {
        // 如果是搜索时的吸顶 不进行操作
        if (searchType === "search") return;
        setTopSpinning(true);
        // eslint-disable-next-line react-hooks/exhaustive-deps
        isTop = true;
        getMsgList(_lastId)
          .then((res) => {
            setTopSpinning(false);
            searchType === "nearby"
              ? setSearchLastId(res.lastId)
              : setLastId(res.lastId);

            //根据全部 还是上下文  处理不一样的数据
            if (searchType === "all") {
              setMsgList([...res.msgList, ...msgList]);
              setCurrentLog(msgList[0]);
            } else if (searchType === "nearby") {
              setNearbyMsgList([...res.msgList, ...nearbyMsgList]);
              setCurrentLog(nearbyMsgList[0]);
            }
            isTop = false;
          })
          .catch((e) => {
            setTopSpinning(false);
            isTop = false;
          });
        return;
      }
      //滚动到底部加载下一个session
      if (
        ref.scrollTop === ref.scrollHeight - ref.clientHeight &&
        !isTop &&
        !isBottom
      ) {
        //搜索有触底
        if (searchType === "search") {
          if (searchPage < totalPage + 1) {
            setBottomSpinning(true);
            // eslint-disable-next-line react-hooks/exhaustive-deps
            isBottom = true;
            searchRecord(searchMsgList, searchPage);
          }
        } else {
          if (_nextId && _nextId !== "-1") {
            setBottomSpinning(true);
            isBottom = true;
            getMsgList(_nextId)
              .then((res) => {
                setBottomSpinning(false);
                if (searchType === "all") {
                  setMsgList([...res.msgList, ...msgList]);
                } else if (searchType === "nearby") {
                  setNearbyMsgList([...res.msgList, ...nearbyMsgList]);
                }
                searchType === "nearby"
                  ? setSearchNextId(res.lastId)
                  : setNextId(res.lastId);
                isBottom = false;
              })
              .catch((e) => {
                setBottomSpinning(false);
                isBottom = false;
              });
          }
        }
        return;
      }
    },
    [
      nextId,
      lastId,
      msgList,
      searchType,
      nearbyMsgList,
      totalPage,
      searchPage,
      searchLastId,
      searchNextId,
    ]
  );

  //改变事件范围选择
  const rangePickerChange = useCallback((dates: any, dateStrings: any) => {
    setStartDate(dates[0]);
    setEndDate(dates[1]);
  }, []);
  // 输入框改变
  const inputChange = useCallback((e) => {
    setSearchValue(e.target.value.trim());
  }, []);
  // 切换tab
  // const changeTab = useCallback((type: string) => {
  //     setType(type)

  // }, [])

  // 根据时间和关键字查询聊天记录
  const searchRecord = useCallback(
    (list, page) => {
      if (!searchValue) {
        message.warning("输入关键字查询");
        return;
      }
      postJSON("/im/im_logs/search", {
        customer_id: chat.customer_id,
        session_key: chat.session_key,
        page_size: 50,
        keyword: searchValue,
        page: page,
        start_date: startDate.format("YYYY-MM-DD"),
        end_date: endDate.format("YYYY-MM-DD"),
      }).then((res) => {
        if (res.code === 1000 || res.code === "1000") {
          // eslint-disable-next-line react-hooks/exhaustive-deps
          isBottom = false;
          setBottomSpinning(false);
          setTotalCount(res?.meta?.total_count);
          setTotalPage(res?.meta?.total_pages);
          setSearchPage(page + 1);
          setSearchMsgList([
            ...list,
            ...res.im_logs.map((item: any) => _matchedResultToImLog(item)),
          ]);
        } else {
          message.error(
            res.code_message ||
              (res.exceptopn && res.exceptopn.message) ||
              t("page.im.record.requestError")
          );
        }
      });
      SetSearchType("search");
    },
    [
      searchValue,
      chat.customer_id,
      chat.session_key,
      startDate,
      endDate,
      searchPage,
    ]
  );

  //点击了搜索  先将分页  数据置空
  const beforeSearchRecord = useCallback(() => {
    setSearchMsgList([]);
    setSearchPage(1);
    searchRecord([], 1);
  }, [searchRecord]);

  //查询结果中搜索上下文
  const searchNearbyById = useCallback(
    (sessionId: string, id: string) => {
      initData(sessionId, [], -1).then((res) => {
        setSearchLastId(res.lastId);
        setSearchNextId(res.nextId);
        SetSearchType("nearby");
        setNearbyMsgList(res.msgList as Msg[]);
        //滚动到查询的信息的位置
        document.getElementById(`data-im-log-${id}`)?.scrollIntoView();
      });
    },
    [initData]
  );

  // 处理是事件时的具体显示
  const handleEvent = useCallback(
    (item: any) => {
      const contentObj = JSON.parse(item.content);
      if (contentObj.data.timeout_freq === 1) {
        return t("page.im.record.tip.noAnswer");
      } else if (contentObj.data.timeout_freq === 2) {
        return t("page.im.record.tip.customerOvertime");
      } else if (contentObj.data.timeout_auto_msg) {
        return r(t("page.im.record.tip.agentOvertime"), item.agent_nick_name);
      } else if (contentObj.data.msg_close) {
        return r(t("page.im.record.tip.agentCloseChat"), item.agent_nick_name);
      } else if (contentObj.type === "transfer") {
        return contentObj.data.content || t("page.im.record.tip.chatTransfer");
      } else {
        return typeof contentObj.data.content === "string"
          ? contentObj.data.content
          : "";
      }
    },
    [t]
  );
  //搜索得到的数据和用sessionid查的数据格式不一样   将搜索得到的数据处理成一样的格式
  const _matchedResultToImLog = useCallback((matchedResult) => {
    return {
      id: matchedResult.id,
      im_session_id: matchedResult.im_session_id,
      agent_id: matchedResult.agent.id,
      customer_id: matchedResult.customer.id,
      im_sub_session_id: matchedResult.im_sub_session_id,
      sender: matchedResult.sender,
      user_nick_name:
        matchedResult.sender === "customer"
          ? matchedResult.customer.nick_name
          : matchedResult.agent.nick_name,
      content: matchedResult.content,
      agent_nick_name: matchedResult.agent.nick_name,
      created_at: matchedResult.created_at,
    };
  }, []);
  //关闭当前消息记录
  const closeRecord = useCallback(() => {
    props.setShowRecord({
      show: false,
    });
    event.updateChat &&
      event.updateChat({
        chat: chat,
        updateProps: {
          newRecord: {},
        },
      });
  }, [props, chat, event]);
  //根据消息类型渲染不同的消息组件
  const recordList = useCallback((item) => {
    let type: MsgType = JSON.parse(item.content).type;
    const SpeicalTypeItem = speicalTypeItem[type];
    return SpeicalTypeItem ? <SpeicalTypeItem msgItem={item} /> : null;
  }, []);
  // 消息记录的头部名称和时间
  const recordHeader = useCallback(
    (item) => {
      return (
        <div
          className={`${style["time-wrapper"]} ${
            style[`sender-is-${item.sender}`]
          }`}
        >
          {/* 名称 */}
          <div className={style["name"]}>{item.user_nick_name}</div>
          <div>{moment(item.created_at).format("YYYY/MM/DD hh:mm:ss")}</div>

          {/* 右侧查看前后文   只有搜索结果是显示  并且是hover时显示 */}
          {searchType === "search" && (
            <div
              className={style["search-nearby"]}
              onClick={() => searchNearbyById(item.im_session_id, item.id)}
            >
              {t("page.im.record.searchNearby")}
            </div>
          )}
        </div>
      );
    },
    [searchNearbyById, searchType, t]
  );

  const ranges = useDateRanges();
  return (
    <>
      <div className={style["record-wrapper"]}>
        {/* 头部标题  和返回按钮 */}
        <div className={style["header-wrapper"]}>
          <div className={style["header-title"]}>
            {t("page.im.record.imLogRecords")}
          </div>
          <CuoHao className={style["header-icons"]} onClick={closeRecord} />
        </div>

        {/* 头部tab切换 */}
        {/* <Row className={style['tab-wrapper']}>
                <Col span={8} className={`${style['tab-item-name']} ${tabType === 'all' ? style['tab-item-active'] : ''}`} onClick={() => changeTab('all')}>
                    全部
                </Col>
                <Col span={8} className={`${style['tab-item-name']} ${tabType === 'file' ? style['tab-item-active'] : ''}`} onClick={() => changeTab('file')}>
                    文件
                </Col>
                <Col span={8} className={`${style['tab-item-name']} ${tabType === 'img' ? style['tab-item-active'] : ''}`} onClick={() => changeTab('img')}>
                    图片/视频
                </Col>
            </Row> */}
        {/* 日期选择  输入框和确定按钮 */}
        <div className={style["picker-and-search"]} id="timer">
          <RangePicker
            defaultValue={[startDate, endDate]}
            ranges={ranges}
            onChange={rangePickerChange}
            allowClear={false}
            className={style["range-picker"]}
          />
          <Input
            className={style["input"]}
            onChange={inputChange}
            onPressEnter={beforeSearchRecord}
          />
          <Button
            className={style["btn"]}
            type="primary"
            onClick={beforeSearchRecord}
          >
            {t("confirm")}
          </Button>
        </div>
        {/* 搜索结果 总数提示 和 返回按钮 */}
        {searchType !== "all" && (
          <div className={style["search-result"]}>
            {/* 已搜到xxx条消息记录 已搜到<span>{totalCount}</span>条相关记录 */}
            {searchType === "search" && (
              <div
                dangerouslySetInnerHTML={{
                  __html: r(
                    t("page.im.record.searchRes"),
                    `<span>${totalCount}</span>`
                  ),
                }}
              />
            )}
            {/* 返回按钮 */}
            <div
              className={style["back-btn"]}
              onClick={() =>
                SetSearchType(searchType === "nearby" ? "search" : "all")
              }
            >
              {t("back")}
            </div>
          </div>
        )}
        <div
          className={style["msg-wrapper"]}
          ref={msgRef}
          onScroll={handleMsgScroll}
          id="result"
        >
          <div>
            {/* 头部的loading  触顶时加载 */}
            <div hidden={!topSpinning} className={style["spin-wrapper"]}>
              <Spin />
            </div>
            {/* 如果是搜索状态  显示的是搜索结果的消息列表   否则渲染全部消息 */}
            {(searchType === "all"
              ? msgList
              : searchType === "search"
              ? searchMsgList
              : nearbyMsgList
            ).map((item: Msg) => {
              return (
                // {/* 内容 ===================start */}
                <div className={style["message-content"]} key={item.id}>
                  {/* {JSON.parse(item.content).data.content} */}
                  {/* 是事件 */}
                  {imLogIsEvent(item) && (
                    <div
                      className={style["msg-item"]}
                      id={`data-im-log-${item.id}`}
                    >
                      <div>
                        {/* 头部姓名+时间 ==================start */}
                        {recordHeader(item)}
                        <div className={style["is-event"]}>
                          <span>{handleEvent(item)}</span>
                        </div>
                      </div>
                    </div>
                  )}
                  {/* 是消息 */}
                  {imLogIsMessage(item) && (
                    <div
                      className={style["msg-item"]}
                      id={`data-im-log-${item.id}`}
                    >
                      <div>
                        {/* 头部姓名+时间 ==================start */}
                        {recordHeader(item)}
                        <div className={style["is-message"]}>
                          {item.send_status === "rollback" && (
                            <div className={style["send-status"]}>
                              <span>
                                <CheckCircle />{" "}
                                {t("page.im.record.tip.withdrawed")}
                              </span>
                            </div>
                          )}
                          {item.send_status === "off_sending" && (
                            <div className={style["send-status"]}>
                              <img src="images/im/off-send.png" alt="" />
                            </div>
                          )}
                          {recordList(item)}
                        </div>
                      </div>
                    </div>
                  )}
                </div>
              );
            })}
            {/* 底部的loading  触底时加载 */}
            <div hidden={!bottomSpinning} className={style["spin-wrapper"]}>
              <Spin />
            </div>
          </div>
        </div>
      </div>
    </>
  );
};

export default ImMsgRecord;
