//搜索公司
import React, { useCallback, useContext, useState } from "react";
import { Select } from "udesk-ui";
import { TicketContext } from "../../../ticket/utilities";

const { Option } = Select;

interface OrgOption {
  value: string;
  label: string;
  key?: string;
}

function AgentGroupsField(props: TicketField & any) {
  const { changeAgentGroup } = useContext(TicketContext) as any;
  // const udeskWebappContext: any = useContext(UdeskContext);//ember传进来的参数
  let langList: any[] = props.agentGroups;
  const [value, setValue] = useState<OrgOption>({
    value: "",
    label: "--",
  }); //选择的选项
  // 处理客服组数据   转完格式以后在前边加上空选项
  const data = (() => {
    let list = langList.map((item) => {
      return {
        value: item.id,
        label: item.name,
      };
    });
    // 前边加空选项
    list.unshift({
      value: "",
      label: "--",
    });
    return list;
  })(); //搜索到的数据

  // 选项变化时回调
  const handleChange = useCallback(
    (value: OrgOption) => {
      setValue(value);
      changeAgentGroup(value ? value.value : "");
      props.onChange && props.onChange(value ? value.value : "");
    },
    [changeAgentGroup, props]
  );
  //筛选规则
  const handleFilter = useCallback((inputValue, option) => {
    if (option.children.includes(inputValue)) return true;
    else return false;
  }, []);
  return (
    <>
      {/* 带输入的下拉选择框 */}
      <Select
        showSearch //显示搜索
        showArrow={true} //显示下拉小箭头
        labelInValue={true} //是否把每个选项的 label 包装到 value 中
        value={value} //选择的选项
        placeholder={props.placeholder} //占位符
        onChange={handleChange} //选中 option
        style={{ width: "100%" }}
        filterOption={handleFilter}
        getPopupContainer={(triggerNode) => triggerNode} //菜单渲染父节点
      >
        {data.map((d) => (
          <Option key={d.value} value={d.value}>
            {d.label}
          </Option>
        ))}
      </Select>
    </>
  );
}

export default AgentGroupsField;
