//搜索公司
import React, { useCallback, useContext, useEffect, useState } from "react";
import { Select } from "udesk-ui";
import { csApiGetJSON } from "../../../../utils/csApi";
import { TicketContext } from "../../../../im-panel/ticket/utilities";

const { Option } = Select;

interface OrgOption {
  value: string;
  label: string;
  key?: string;
}

interface GropsUserRes {
  user_group: {
    users: {
      id: string;
      nick_name: string;
    }[];
  };
}

function AgentUserField(props: TicketField) {
  const { agentGroup } = useContext(TicketContext) as any;
  const [value, setValue] = useState<OrgOption>(); //选择的选项
  const [data, setData] = useState<OrgOption[]>([]);
  useEffect(() => {
    //负责组只要改变  负责人就置空  重新选择
    setValue({ value: "", label: "" });
    if (!agentGroup) {
      // 如果选负责组是空选项   负责人数据置空
      setData([]);
    } else {
      // 请求负责人数据
      csApiGetJSON("/cs/ts/api/user_groups/" + agentGroup).then(res=>res.data).then((res: GropsUserRes) => {
        let list = res.user_group.users.map((item) => {
          return {
            value: item.id,
            label: item.nick_name,
          } as OrgOption;
        });
        // 前边加空选项
        list.unshift({
          value: "",
          label: "--",
        });
        setData(list);
      });
    }
  }, [agentGroup]);
  // 选项变化时回调
  const handleChange = useCallback(
    (value: OrgOption) => {
      setValue(value);
      props.onChange && props.onChange(value ? value.value : "");
    },
    [props]
  );
  //筛选规则
  const handleFilter = useCallback((inputValue, option) => {
    if (option.children.includes(inputValue)) return true;
    else return false;
  }, []);
  return (
    <>
      {/* 带输入的下拉选择框 */}
      <Select
        showSearch //显示搜索
        showArrow={true} //显示下拉小箭头
        labelInValue={true} //是否把每个选项的 label 包装到 value 中
        value={value} //选择的选项
        placeholder={props.placeholder} //占位符
        onChange={handleChange} //选中 option
        style={{ width: "100%" }}
        filterOption={handleFilter}
        getPopupContainer={(triggerNode) => triggerNode} //菜单渲染父节点
      >
        {data.map((d) => (
          <Option key={d.value} value={d.value}>
            {d.label}
          </Option>
        ))}
      </Select>
    </>
  );
}

export default AgentUserField;
