/* eslint-disable jsx-a11y/anchor-is-valid */
import React, { useCallback, useEffect, useState } from "react";
import { Input, message } from "udesk-ui";
import style from "./index.module.scss";
import FieldWarning from "../../../../components/fieldFormItem/components/field_warning";
import { getJSON } from "../../../../utils/index";
import { useTranslation } from "react-i18next";
import { MinusOutlined, PlusOutlined } from "@ant-design/icons";

interface RepeatCell {
  id: number;
  name: string;
}

interface CellProps extends TicketField {
  udeskWebappContext: ImContextProps;
}

function CellphoneField(props: any) {
  const { event } = props;
  //多语言
  const { t } = useTranslation();
  const list = typeof props.value === "string" ? props.value.split(",") : [];
  const [emailList, setEmailList] = useState<string[]>(list);
  const [errMessage, SetErrMessage] = useState<string[]>([]);
  const [repeat, SetRepeat] = useState<RepeatCell[]>([]);
  //手机正则
  const mobilePtn = /^1[34578][0-9]{9}$/;
  //座机正则
  const landlinePtn = /^(0[0-9]{2,3}-)([2-9][0-9]{6,7})+(-[0-9]{1,4})?$/;
  // 进来第一次生成对应邮箱数量的报错信息
  useEffect(() => {
    SetErrMessage([...new Array(list.length)]);
    SetRepeat([...new Array(list.length)]);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  // 新加邮箱时   改变外部form数据
  const addEmail = useCallback(() => {
    // props.onChange && props.onChange(emailList.concat('').join(','))
    setEmailList([...emailList.concat("")]);
    SetErrMessage([...errMessage.concat("")]);
    SetRepeat([...repeat.concat({} as RepeatCell)]);
  }, [emailList, errMessage, repeat]);
  // 删除   删除邮箱  删除报错信息
  const deleteEmail = useCallback(
    (index) => {
      emailList.splice(index, 1);
      errMessage.splice(index, 1);
      setEmailList([...emailList]);
      SetErrMessage([...errMessage]);
      // 删除后  更新外部form
      props.onChange && props.onChange(emailList.join(","));
    },
    [emailList, errMessage, props]
  );
  // 输入值改变时  改变相对应的数据
  const emailInputChange = useCallback(
    (e, index) => {
      emailList[index] = e.target.value;
      setEmailList([...emailList]);
    },
    [emailList]
  );
  // 失焦触发  校验邮箱  更改外部form
  const emailOnBlur = useCallback(
    (index) => {
      const value = emailList[index];
      // 查看是否重复输入同一手机号
      let valueNum = 0;
      emailList.forEach((item) => {
        if (item === value) valueNum++;
      });
      // 如果有重复的数据  提示重复  并且将该值置空
      if (valueNum > 1) {
        message.warning("电话信息重复，请检查");
        emailList[index] = "";
        setEmailList([...emailList]);
        return;
      }

      //校验电话号码格式
      if (value && !mobilePtn.test(value) && !landlinePtn.test(value)) {
        errMessage[index] = "号码格式不正确";
        SetErrMessage([...errMessage]);
        return;
      } else {
        errMessage[index] = "";
        SetErrMessage([...errMessage]);
      }
      //查询是否有重复数据
      getJSON(
        "/customers/check_duplication?customer%5Bcellphone%5D=" + value
      ).then((res) => {
        if (res.duplications.cellphone) {
          repeat[index] = {
            id: res.duplications.cellphone.id,
            name: res.duplications.cellphone.name,
          };
          SetRepeat([...repeat]);
          // 有重复   将填写的置空
          emailList[index] = "";
          setEmailList([...emailList]);
        } else {
          repeat[index] = {} as RepeatCell;
          SetRepeat([...repeat]);
        }
      });
    },
    [emailList, errMessage, landlinePtn, mobilePtn, repeat]
  );
  useEffect(() => {
    props.onChange && props.onChange(emailList.join(","));
  }, [emailList, props]);
  //点击重复信息时   跳转
  const clickRepeatWarning = useCallback(
    (id: number) => {
      event.transitionToRoute &&
        event.transitionToRoute({
          route: "crm.customer.show",
          path: id,
        });
    },
    [event]
  );
  return (
    <>
      {emailList.map((item, index) => {
        return (
          <div key={index} className={style["email-item-wrapper"]}>
            <div className={style["email-item-box"]}>
              <Input
                style={{ flex: 1 }}
                placeholder={props.placeholder}
                value={item}
                onChange={(e) => emailInputChange(e, index)}
                onBlur={() => emailOnBlur(index)}
              />
              {(function () {
                if (index) {
                  return (
                    <div
                      className={style["add-icon"]}
                      onClick={() => deleteEmail(index)}
                    >
                      <MinusOutlined />
                      <div>{t("delete")}</div>
                    </div>
                  );
                } else {
                  return (
                    <div className={style["add-icon"]} onClick={addEmail}>
                      <PlusOutlined />
                      <div>{t("add")}</div>
                    </div>
                  );
                }
              })()}
            </div>
            {errMessage[index] && (
              <div style={{ marginTop: "5px" }}>
                <FieldWarning warningText={errMessage[index]} />
              </div>
            )}
            {repeat.length > 0 && repeat[index] && repeat[index].id && (
              <div style={{ marginTop: "5px" }}>
                <div className={style["repeat-warning"]}>
                  <div className={style["warning-img"]}></div>
                  <div>
                    此电话与已有客户
                    <a onClick={() => clickRepeatWarning(repeat[index].id)}>
                      {repeat[index].name}
                    </a>
                    重复
                  </div>
                </div>
              </div>
            )}
          </div>
        );
      })}
    </>
  );
}

export default CellphoneField;
