/* eslint-disable jsx-control-statements/jsx-use-if-tag */
import React, { useCallback, useContext, useEffect, useState } from "react";
import { Select, Spin } from "udesk-ui";
import { postJSON } from "../../../../utils/index";
import style from "./index.module.scss";
import SearchCustomer from "../search-customer";
import { PlusOutlined, SearchOutlined } from "@ant-design/icons";
import { TicketContext } from "../../../../im-panel/ticket/utilities";

const { Option } = Select;

function Customer(props: any) {
  const { changePageType, customer } = useContext(TicketContext) as any;
  const [value, setValue] = useState<AntdSelectOptions>(
    {} as AntdSelectOptions
  ); //选择的选项
  const [fetching, setFetching] = useState<boolean>(false); //无数据时展示
  const [data, setData] = useState<AntdSelectOptions[]>([]); //搜索到的数据
  const [lastFetchId, setLastFetchId] = useState<number>(0); //
  const [showSearchMoadl, setShowSearchMoadl] = useState<boolean>(false);
  const [page, setPage] = useState<number>(1);
  const [searchValue, setSearchValue] = useState<string>("");
  const [totalPages, setTotalPages] = useState<number>(0);

  //监听传进来的值变化  有变化就更新value
  useEffect(() => {
    if (customer.value) {
      setValue(customer as AntdSelectOptions);
      props.onChange && props.onChange(customer);
    } else {
      setValue(props.value);
    }
  }, [customer, props, props.value]);
  const beforeSearch = useCallback((searchValue) => {
    setSearchValue(searchValue);
    setData([]);
    setPage(1);
    fetchUser(searchValue);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  // 搜索用户
  const fetchUser = useCallback(
    (searchValue) => {
      setLastFetchId(() => lastFetchId + 1);
      const fetchId = lastFetchId;

      setFetching(true);
      postJSON("/customers/contain_search", {
        page: page,
        page_size: 20,
        prefix: searchValue,
      }).then((res) => {
        setPage(page + 1);
        setTotalPages(res.meta.total_pages);
        if (fetchId !== lastFetchId) {
          // for fetch callback order
          return;
        }
        const _data = res.customers.map((item: any) => {
          return {
            label: item.nick_name,
            value: item.id,
          };
        });
        setData([...data, ..._data]);
        setFetching(false);
      });
    },
    [lastFetchId, page, data]
  );
  //
  const handleChange = useCallback(
    (value: AntdSelectOptions) => {
      setValue(value);
      setFetching(false);
      props.onChange && props.onChange(value);
    },
    [props]
  );
  // 点击搜索按钮  打开搜索弹窗
  const searchUser = useCallback(() => {
    setShowSearchMoadl(true);
  }, []);
  // 关闭搜索弹窗
  const closeSearch = useCallback(() => {
    setShowSearchMoadl(false);
  }, []);
  const suffixIcon = (
    <div onClick={searchUser} className={style["search-icon"]}>
      <SearchOutlined />
    </div>
  );
  //处理滚动事假
  const handleScroll = useCallback(
    (e) => {
      e.stopPropagation();
      if (
        e.target.scrollTop ===
        e.target.scrollHeight - e.target.clientHeight
      ) {
        if (page < totalPages) {
          fetchUser(searchValue);
        }
      }
    },
    [page, totalPages, fetchUser, searchValue]
  );
  // 聚焦时  将外部表单取消滚动  防止滚动穿透
  const handleFocus = useCallback(() => {
    props.setIsScroll && props.setIsScroll(false);
  }, [props]);
  // 失焦时  将外部滚动复原
  const handleBlur = useCallback(() => {
    props.setIsScroll && props.setIsScroll(true);
  }, [props]);
  const selectCustomer = useCallback(
    (customer: any) => {
      handleChange(customer);
    },
    [handleChange]
  );
  return (
    <div style={{ display: "flex" }}>
      {/* 带输入的下拉选择框 */}
      <Select
        showSearch
        showArrow={true}
        labelInValue={true}
        value={value}
        placeholder=""
        notFoundContent={fetching ? <Spin size="small" /> : null}
        filterOption={false}
        onSearch={beforeSearch}
        onChange={handleChange}
        style={{ width: "100%" }}
        getPopupContainer={(triggerNode) => triggerNode}
        suffixIcon={suffixIcon}
        onPopupScroll={handleScroll}
        onFocus={handleFocus}
        onBlur={handleBlur}
      >
        {data.map((d) => (
          <Option key={d.value} value={d.value}>
            {d.label}
          </Option>
        ))}
      </Select>
      {/* 选择框后边的加号  点击添加新客户 */}
      <div
        className={style["input-after-icon"]}
        onClick={() => changePageType("customer")}
      >
        <PlusOutlined />
      </div>
      {/* 搜索客户弹窗 */}
      <SearchCustomer
        close={closeSearch}
        show={showSearchMoadl}
        selectCustomer={selectCustomer}
        ticketTab={props.ticketTab}
      />
    </div>
  );
}

export default Customer;
