import React, { useCallback, useEffect, useState } from "react";
import { Input } from "udesk-ui";
import style from "./index.module.scss";
import { useTranslation } from "react-i18next";
import FieldWarning from "../../../../components/fieldFormItem/components/field_warning";
import { MinusOutlined, PlusOutlined } from "@ant-design/icons";

function EmailField(props: TicketField) {
  //多语言
  const { t } = useTranslation();
  const list = props.value.length ? props.value : [""];
  const [emailList, setEmailList] = useState<string[]>(list);
  const [errMessage, SetErrMessage] = useState<string[]>([]);
  // 进来第一次生成对应邮箱数量的报错信息
  useEffect(() => {
    SetErrMessage([...new Array(list.length)]);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  // 新加邮箱时   改变外部form数据
  const addEmail = useCallback(() => {
    // props.onChange && props.onChange(emailList.concat('').join(','))
    setEmailList([...emailList.concat("")]);
    SetErrMessage([...errMessage.concat("")]);
  }, [emailList, errMessage]);
  // 删除   删除邮箱  删除报错信息
  const deleteEmail = useCallback(
    (index) => {
      emailList.splice(index, 1);
      errMessage.splice(index, 1);
      setEmailList([...emailList]);
      SetErrMessage([...errMessage]);
      // 删除后  更新外部form
      props.onChange && props.onChange(emailList);
    },
    [emailList, errMessage, props]
  );
  // 输入值改变时  改变相对应的数据
  const emailInputChange = useCallback(
    (e, index) => {
      emailList[index] = e.target.value;
      setEmailList([...emailList]);
    },
    [emailList]
  );
  // 失焦触发  校验邮箱  更改外部form
  const emailOnBlur = useCallback(
    (index) => {
      const value = emailList[index];
      if (
        value &&
        !/^[A-Za-z0-9\u4e00-\u9fa5]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+$/i.test(
          value
        )
      ) {
        errMessage[index] = "邮箱格式不正确";
        SetErrMessage([...errMessage]);
      } else {
        errMessage[index] = "";
        SetErrMessage([...errMessage]);
      }
      props.onChange && props.onChange(emailList);
    },
    [emailList, errMessage, props]
  );
  return (
    <>
      {emailList.map((item, index) => {
        return (
          <div key={index} className={style["email-item-wrapper"]}>
            <div className={style["email-item-box"]}>
              <Input
                style={{ flex: 1 }}
                placeholder={props.placeholder}
                value={item}
                onChange={(e) => emailInputChange(e, index)}
                onBlur={() => emailOnBlur(index)}
              />

              {!index ? (
                <div className={style["add-icon"]} onClick={addEmail}>
                  <PlusOutlined />
                  <div>{t("add")}</div>
                </div>
              ) : (
                <div
                  className={style["add-icon"]}
                  onClick={() => deleteEmail(index)}
                >
                  <MinusOutlined />
                  <div>{t("delete")}</div>
                </div>
              )}
            </div>
            <div style={{ marginTop: "5px" }}>
              <FieldWarning warningText={errMessage[index]} />
            </div>
          </div>
        );
      })}
    </>
  );
}

export default EmailField;
