import React, {
  useState,
  useCallback,
  useContext,
  useEffect,
  useMemo,
} from "react";
import { Select } from "udesk-ui";
import style from "./index.module.scss";
import { useTranslation } from "react-i18next";
import { StarFilled } from "@ant-design/icons";
import uniqBy from "lodash-es/uniqBy";
import { TicketContext } from "../../../../im-panel/ticket/utilities";

interface FolloweProps {
  onChange?: (value: number[]) => void;
  value?: number[];
}

function FollowerField(props: FolloweProps & any) {
  //多语言
  const { t } = useTranslation();
  const [value, setValue] = useState<number[]>([]); //选择的选项
  const [starState, setStarState] = useState<boolean>(false);
  const { agent } = useContext(TicketContext) as any; //客服组
  // const udeskWebappContext: any = useContext(UdeskContext);//ember传过来的数据
  // const currentUserId = udeskWebappContext.currentUser.id

  const { currentUser = {}, admin = {} } = props;
  const { id: currentUserId } = currentUser;
  useEffect(() => {
    setValue(props.value as number[]);
  }, [props.value]);
  // 处理选项  从客服组中去除每组中的客服  并且最后加上管理员
  const radio = useMemo(() => {
    let list: SelectOptions[] = [];
    agent.forEach((item: any) => {
      if (item.children.length) {
        list = [
          ...list,
          ...item.children.map((item: any) => {
            return {
              title: item.label,
              value: item.value,
            };
          }),
        ];
      }
    });
    list = uniqBy(list, "value");
    list.push({
      title: admin.nick_name,
      value: admin.id,
    });
    return list;
  }, [admin.id, admin.nick_name, agent]);

  useEffect(() => {
    //选项变化时  根据选项中有没有包含当前客服 改变星星状态
    setStarState(value.includes(currentUserId));
  }, [currentUserId, value]);
  // 点击星星事件
  const clickStart = useCallback(() => {
    // 当前 starState true  点击事件是为了  删除选择的当前的客服
    // 当前 starState false   点击事件是为了  选择的当前的客服
    if (starState) {
      const index = value.indexOf(currentUserId);
      setValue(() => {
        value.splice(index, 1);
        return [...value];
      });
      props.onChange && props.onChange([...value]);
    } else {
      setValue([...value, currentUserId]);
      props.onChange && props.onChange([...value, currentUserId]);
    }
    // 改变选择时会触发上边的useEffect处理星星状态  此处不进行处理
  }, [currentUserId, props, starState, value]);
  // 选项变化时回调
  const handleChange = useCallback(
    (value: number[]) => {
      setValue(value);
      props.onChange && props.onChange(value);
    },
    [props]
  );
  //筛选规则
  const handleFilter = useCallback((inputValue, option) => {
    return option.children.includes(inputValue);
  }, []);
  return (
    <div className={style["follower-wrapper"]}>
      {/* 带输入的下拉选择框 */}
      <Select
        mode="multiple"
        showSearch //显示搜索
        showArrow={true} //显示下拉小箭头
        value={value} //选择的选项
        placeholder={t("page.im.ticket.choiceFollower")} //占位符
        onChange={handleChange} //选中 option
        style={{ width: "100%" }}
        filterOption={handleFilter}
        getPopupContainer={(triggerNode) => triggerNode} //菜单渲染父节点
      >
        {radio.map((d) => (
          <Select.Option key={d.value} value={d.value}>
            {d.title}
          </Select.Option>
        ))}
      </Select>
      <div
        className={style["star-icon"]}
        onClick={clickStart}
        style={{ color: starState ? "#FDBD68" : "#333" }}
      >
        <StarFilled />
      </div>
    </div>
  );
}

export default FollowerField;
