import React, { useCallback, useEffect, useState } from "react";
import { Select, Spin } from "udesk-ui";
import { postJSON } from "../../../utils/index";

const { Option } = Select;

function MerchantField(props: any) {
  const [value, setValue] = useState<AntdSelectOptions>(
    {} as AntdSelectOptions
  ); //选择的选项
  const [fetching, setFetching] = useState<boolean>(false); //无数据时展示
  const [data, setData] = useState<AntdSelectOptions[]>([]); //搜索到的数据
  const [lastFetchId, setLastFetchId] = useState<number>(0); //
  const [page, setPage] = useState<number>(1);
  const [searchValue, setSearchValue] = useState<string>("");
  const [totalPages, setTotalPages] = useState<number>(0);

  const fetchMerchant = useCallback(
    (searchValue) => {
      setLastFetchId(() => lastFetchId + 1);
      const fetchId = lastFetchId;

      setFetching(true);
      postJSON("/mchat_merchants/query_merchants", {
        mchat_name: searchValue,
        page: page,
        per_page: 20,
      }).then((res) => {
        setPage(page + 1);
        setTotalPages(res.meta.total_pages);
        if (fetchId !== lastFetchId) {
          // for fetch callback order
          return;
        }
        const _data = res.customers.map((item: any) => {
          return {
            label: item.nick_name,
            value: item.id,
          };
        });
        setData([...data, ..._data]);
        setFetching(false);
      });
    },
    [lastFetchId, page, data]
  );
  const beforeSearch = useCallback(
    (searchValue) => {
      setSearchValue(searchValue);
      setData([]);
      setPage(1);
      fetchMerchant(searchValue);
    },
    [fetchMerchant]
  );
  useEffect(() => {
    setValue(props.value);
  }, [props.value]);

  function handleChange(value: AntdSelectOptions) {
    setValue(value);
    setFetching(false);
    props.onChange && props.onChange(value);
  }

  //处理滚动事假
  const handleScroll = useCallback(
    (e) => {
      e.stopPropagation();
      if (
        e.target.scrollTop ===
        e.target.scrollHeight - e.target.clientHeight
      ) {
        if (page < totalPages) {
          fetchMerchant(searchValue);
        }
      }
    },
    [page, totalPages, fetchMerchant, searchValue]
  );
  // 聚焦时  将外部表单取消滚动  防止滚动穿透
  const handleFocus = useCallback(() => {
    props.setIsScroll && props.setIsScroll(false);
  }, [props]);
  // 失焦时  将外部滚动复原
  const handleBlur = useCallback(() => {
    props.setIsScroll && props.setIsScroll(true);
  }, [props]);
  return (
    <Select
      showSearch
      showArrow={true}
      labelInValue={true}
      value={value}
      placeholder=""
      notFoundContent={fetching ? <Spin size="small" /> : null}
      filterOption={false}
      onSearch={beforeSearch}
      onChange={handleChange}
      style={{ width: "100%" }}
      getPopupContainer={(triggerNode) => triggerNode}
      onPopupScroll={handleScroll}
      onFocus={handleFocus}
      onBlur={handleBlur}
    >
      {data.map((d) => (
        <Option key={d.value} value={d.value}>
          {d.label}
        </Option>
      ))}
    </Select>
  );
}

export default MerchantField;
