import React, { useCallback, useContext, useEffect, useState } from "react";
import { Cascader } from "udesk-ui";
import style from "./index.module.scss";
import { TicketContext } from "../../../../im-panel/ticket/utilities";

interface Option {
  value: string | number;
  label?: React.ReactNode;
  disabled?: boolean;
  children?: Option[];
}

interface SelfProps {
  value?: string[];
  onChange?: (value: string[]) => void;
}

function RecevierField(props: SelfProps) {
  const { onChange } = props;
  const [value, setValue] = useState<any>();
  const [options, setOptions] = useState<Option[]>([]);
  const { agent } = useContext(TicketContext) as any;

  const cascaderOnChange = useCallback(
    (value, selectedOptions) => {
      setValue(value);
      onChange && onChange(value);
    },
    [onChange]
  );
  const filter = useCallback((inputValue: any, path: any) => {
    return path.some(
      (option: any) =>
        option.label.toLowerCase().indexOf(inputValue.toLowerCase()) > -1
    );
  }, []);
  useEffect(() => {
    setOptions(agent);
  }, [agent]);
  useEffect(() => {
    setValue(props.value);
  }, [props.value]);
  return (
    <div className={style["cascader-wrapper"]}>
      <Cascader
        value={value}
        options={options}
        onChange={cascaderOnChange}
        placeholder=""
        changeOnSelect
        showSearch={{ filter }}
      />
    </div>
  );
}

export default RecevierField;
