import React, {
  useCallback,
  useEffect,
  useState,
  useMemo,
  useRef,
} from "react";
import {
  Button,
  Checkbox,
  Col,
  Input,
  Modal,
  Popover,
  Row,
  Select,
  Table,
  message,
} from "udesk-ui";
import style from "./index.module.scss";
import { postJSON } from "../../../../utils/index";
import { FilterFilled } from "@ant-design/icons";
import { useTranslation } from "react-i18next";
import moment from "moment";
const { Option } = Select;

interface SearchCustomerModal {
  close: () => void;
  show: boolean;
  selectCustomer: (value: any) => void;
}

interface FilterOption {
  name: string;
  id: number;
  sort_by?: any[];
  all_conditions?: any[];
}

interface ColumsOption {
  label: string;
  value: string;
  display: boolean;
}

function SearchCustomer(props: SearchCustomerModal & any) {
  // const udeskWebappContext: any = useContext(UdeskContext);//ember传进来的参数
  const { t } = useTranslation();
  const columnsOption = useMemo(() => {
    if (props?.ticketTab?.tableColumns) {
      return props?.ticketTab?.tableColumns.map((item: any) => {
        return {
          value: item.value,
          display: item.display,
          label: t(`model.customer.${item.value}`, item.label),
        };
      });
    } else {
      return [];
    }
  }, [props?.ticketTab?.tableColumns, t]);
  const filter: FilterOption[] = props?.ticketTab?.customerFilters; //客户过滤器选项
  const [selectValue, setSelectValue] = useState<number>(filter[0].id); //选择的客户过滤器的id
  const [inputValue, setInputValue] = useState<string>(""); //输入的搜索内容
  const [btnLoading, setBtnLoading] = useState<boolean>(false); //搜索按钮的loading状态
  // const [columsOption, setColumsOption] = useState<any[]>(udeskWebappContext.tableColumns)//列表选项
  const [tableData, setTableData] = useState<any[]>([]);
  const [columns, setColumns] = useState<object[]>([]); //表格 表头数据
  const [filterSelect, setFilterSelect] = useState<string[]>([]); //列表选项的选择值
  const [currentCustomer, setCurrentCustomer] = useState<AntdSelectOptions>();
  const [currentPage, setCurrentPage] = useState<number>(1); //当前页数
  const [totalPage, setTotalPage] = useState<number>(0);
  // const [pageSize,setPageSize] = useState<number>(5)
  const pageSize = useRef<number>(5);
  //下拉多选变化时  更新表头
  useEffect(() => {
    setColumns(() => {
      return columnsOption
        .filter((item: any) => filterSelect.includes(item.value))
        .map((item: any) => {
          return {
            title: item.label,
            dataIndex: item.value,
            key: item.value,
            align: "left",
            width: 150,
            // responsive:['lg']
          };
        });
    });
    setTableData([...tableData]);
  }, [columnsOption, filterSelect]);

  useEffect(() => {
    setFilterSelect(() => {
      return columnsOption
        .filter((item: any) => item.display)
        .map((item: any) => {
          return item.value;
        });
    });
    searchCustomer();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  const handleOk = useCallback(() => {
    props.selectCustomer(currentCustomer);
    props.close();
  }, [currentCustomer, props]);
  // 点击取消 关闭弹窗
  const handleCancel = useCallback(() => {
    props.close();
  }, [props]);
  // 弹窗页脚
  const footer = (
    <div style={{ textAlign: "center" }}>
      <Button key="back" onClick={handleCancel}>
        取消
      </Button>
      <Button key="submit" type="primary" onClick={handleOk}>
        确定
      </Button>
    </div>
  );
  const searchCustomer = useCallback(
    (page?: number) => {
      setCurrentPage(page || 1);
      let _filter = filter.find((i) => i.id === selectValue);
      if (!_filter) {
        return message.error("没有权限或过滤器已删除!");
      }

      let params = {
        all_conditions: _filter.all_conditions || [],
        column: "",
        order: "",
        filter_id: selectValue,
        query: inputValue,
        page: page || currentPage,
        page_size: pageSize.current,
      };
      setBtnLoading(true);
      setTableData([]);
      postJSON("/customers/search", params)
        .then((res) => {
          setBtnLoading(false);
          if (res.code === 1000) {
            res.customers.forEach((item: any) => {
              item.updated_at = moment(
                new Date(item.updated_at || null)
              ).format("YYYY/MM/DD HH:mm");
              item.organizations = res.organizations
                .filter((_item: any) => item.organizations.includes(_item.id))
                .map((item: any) => item.name)
                .join(",");
              item.level =
                item.level === "normal"
                  ? t("enums.customerLevel.normal")
                  : "VIP";
            });
            setTableData(res.customers);
            setTotalPage(res.meta.total_pages);
          }
        })
        .catch((e) => {
          setBtnLoading(false);
        });
    },
    [filter, selectValue, inputValue, currentPage, t]
  );
  // 表格单选选择时触发
  const tableSelectOnChange = useCallback((selectedRows) => {
    setCurrentCustomer({
      label: selectedRows[0].nick_name,
      value: selectedRows[0].id,
    });
  }, []);
  // 列选项变化时触发
  const changeTableFilter = useCallback((checkedValue) => {
    setFilterSelect(checkedValue);
  }, []);
  //表格头部选择  气泡弹窗中的
  const tableFilterList = (
    <div className={style["table-filter-list"]}>
      <Checkbox.Group
        value={filterSelect}
        options={columnsOption}
        onChange={changeTableFilter}
      />
    </div>
  );
  const handleTableChange = useCallback(
    (pagination) => {
      let page = pagination.current;
      let size = pagination.pageSize;
      pageSize.current = size;
      searchCustomer(page);
    },
    [pageSize, searchCustomer]
  );
  return (
    <div>
      <Modal
        centered
        bodyStyle={{ padding: "0 0 20px 0" }}
        visible={props.show}
        title="选择客户"
        onOk={handleOk}
        onCancel={handleCancel}
        footer={footer}
        width={900}
      >
        <div className={style["modal-content"]}>
          {/* 顶部*/}
          <Row gutter={[16, 24]}>
            {/* 下拉选择框 客户过滤器 */}
            <Col>
              <Select
                value={selectValue}
                style={{ minWidth: "100px" }}
                onChange={(value) => setSelectValue(value)}
              >
                {filter.map((item) => {
                  return (
                    <Option key={item.id} value={item.id}>
                      {item.name}
                    </Option>
                  );
                })}
              </Select>
            </Col>
            {/* 输入框 */}
            <Col>
              <Input
                value={inputValue}
                onChange={(e) => setInputValue(e.target.value)}
                onPressEnter={() => searchCustomer()}
              />
            </Col>
            {/* 搜索按钮 */}
            <Col>
              <Button
                loading={btnLoading}
                type="primary"
                onClick={() => searchCustomer()}
              >
                搜索
              </Button>
            </Col>
          </Row>
          <Row>
            <div className={style["table-wrapper"]}>
              <Popover
                className={style["popover-wrapper"]}
                placement="bottomRight"
                title=""
                content={tableFilterList}
                trigger="click"
              >
                <div className={style["table-filter-wrapper"]}>
                  列<FilterFilled />
                </div>
              </Popover>
              <Table
                scroll={{
                  x: "100%",
                  scrollToFirstRowOnChange: true,
                  y: 320,
                }}
                rowKey="id"
                style={{ width: "100%", height: "320px" }}
                columns={columns}
                dataSource={tableData}
                rowSelection={{
                  type: "radio",
                  columnWidth: "30px",
                  onChange: (selectedRowKeys, selectedRows) =>
                    tableSelectOnChange(selectedRows),
                  // onSelect: (record, selected, selectedRows, nativeEvent) => selectCustomer()
                }}
                pagination={{
                  defaultPageSize: 5,
                  pageSizeOptions: ["5", "10", "20", "30", "50", "100"],
                  showQuickJumper: true,
                  defaultCurrent: 1,
                  total: totalPage,
                  showSizeChanger: true,
                  current: currentPage,
                }}
                onChange={(pagination: any) => handleTableChange(pagination)}
              />
            </div>
          </Row>
        </div>
      </Modal>
    </div>
  );
}

export default SearchCustomer;
