/* eslint-disable jsx-control-statements/jsx-use-if-tag */
//搜索公司
import React, {
  useCallback,
  useContext,
  useEffect,
  useRef,
  useState,
} from "react";
import { Select, Spin } from "udesk-ui";
import { csApiPostJSON } from "../../../../utils/csApi";
import style from "./index.module.scss";
import { useTranslation } from "react-i18next";
import { MinusOutlined, PlusOutlined } from "@ant-design/icons";
import { TicketContext } from "../../../../im-panel/ticket/utilities";

const { Option } = Select;

function SearchOrgField(props: any) {
  const { roleCan, company, event } = props;
  //多语言
  const { t } = useTranslation();
  const ref = useRef<HTMLDivElement>(null);
  //ember传进来的参数
  const canCreate: boolean = roleCan.add_new_organization;
  const canAdd: boolean = company.support_multi_organizations as false;
  const { changePageType, organization } = useContext(TicketContext) as any;
  const [fetching, setFetching] = useState<boolean>(false); //无数据时展示
  const [lastFetchId, setLastFetchId] = useState<number>(0); //
  const [data, setData] = useState<AntdSelectOptions[]>([]); //搜索到的数据
  const [valueList, setValueList] = useState<AntdSelectOptions[]>([]);
  const [filterList, setFilterList] = useState<number[]>([]);
  useEffect(() => {
    const list = [organization as AntdSelectOptions];
    setValueList([...list]);
    props.onChange && props.onChange(list.map((item) => item.value || ""));
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [organization]);

  // 搜索用户
  const fetchUser = useCallback(
    (searchValue: string) => {
      setLastFetchId(() => lastFetchId + 1);
      const fetchId = lastFetchId;
      setData([]);
      setFetching(true);
      csApiPostJSON("/cs/ts/api/organizations/search", {query: searchValue}).then(res=>res.data).then((res) => {
        if (fetchId !== lastFetchId) {
          // for fetch callback order
          return;
        }
        // 处理数据格式
        const data = res.organizations.map((item: any) => {
          return {
            value: item.id,
            label: item.name,
          };
        });
        setData(data);
        setFetching(false);
      });
    },
    [lastFetchId]
  );

  // 选项变化时回调
  function handleChange(value: AntdSelectOptions, index: number) {
    const list = valueList;
    //改变当前选择的公司的值
    list.splice(index, 1, value || ({} as AntdSelectOptions));
    // 重新赋值
    setValueList([...list]);
    setFetching(false);
    // 将选择的id数组更新   为了搜索数据之后筛选用
    setFilterList(list.map((item) => item.value) as number[]);
    // 改变外部form值
    props.onChange && props.onChange(list.map((item) => item.value || ""));
  }

  // 添加新的公司
  const addOrg = useCallback(() => {
    let canAdd = true;
    //如果有公司还没有选  禁止新加
    valueList.forEach((item) => {
      if (!item.value) {
        canAdd = false;
        return;
      }
    });
    if (!valueList[valueList.length - 1].value) {
      canAdd = false;
      return;
    }
    canAdd && setValueList([...valueList.concat({} as AntdSelectOptions)]);
  }, [valueList]);
  // 删除公司
  const deleteOrg = useCallback(
    (index: number) => {
      setValueList(() => {
        valueList.splice(index, 1);
        return [...valueList];
      });
    },
    [valueList]
  );
  // 点击已选择的公司  进入公司详情
  const gotoCompany = useCallback(
    (e) => {
      // 如果没有选择  不进行操作
      if (!valueList.length || !valueList[0].value) return;
      // 判断点击的是否是选择的字的地方  其他地方不触发
      if (e.target.className === "#{$prefix}-select-selection-item") {
        event.transitionToRoute &&
          event.transitionToRoute({
            route: "crm.organization.show",
            path: valueList[0].value,
          });
      }
    },
    [event, valueList]
  );
  const getPopupContainer = useCallback(() => ref.current || document.body, []);
  return (
    <div className={style["select-wrapper"]} ref={ref}>
      <div className={style["org-select-wrapper"]}>
        {/* 带输入的下拉选择框 */}
        {valueList.map((item, index) => {
          return (
            <div
              key={index}
              className={style["select-itme-wrapper"]}
              id="select"
            >
              <Select
                onClick={gotoCompany}
                showSearch //显示搜索
                showArrow={true} //显示下拉小箭头
                labelInValue={true} //是否把每个选项的 label 包装到 value 中
                value={valueList[index]} //选择的选项
                placeholder={props.placeholder} //占位符
                notFoundContent={fetching ? <Spin size="small" /> : null}
                filterOption={false} //是否根据输入项进行筛选
                onSearch={fetchUser} //文本框值变化时回调
                onChange={(value) => handleChange(value, index)} //选中 option
                style={{ width: "100%" }}
                getPopupContainer={getPopupContainer} //菜单渲染父节点
                allowClear={true} //支持清除
                disabled={index !== valueList.length - 1}
              >
                {data.map((d) => {
                  if (!filterList.includes(d.value as number))
                    return (
                      <Option key={d.value} value={d.value}>
                        {d.label}
                      </Option>
                    );
                  else return <></>;
                })}
              </Select>
              {/* 当前公司配置可多选 */}
              {(function () {
                if (canAdd) {
                  if (index) {
                    return (
                      <div
                        onClick={() => deleteOrg(index)}
                        className={style["add-org"]}
                      >
                        <MinusOutlined />
                      </div>
                    );
                  } else {
                    return (
                      <div className={style["add-org"]} onClick={addOrg}>
                        <PlusOutlined />
                      </div>
                    );
                  }
                }
              })()}
            </div>
          );
        })}
      </div>

      {/* 选择框后边的加号  点击添加新公司 */}
      {/* 如果该角色有新增权限 并且 公司条数少于20  显示新建按钮 */}
      {canCreate && valueList.length < 20 && (
        <div
          className={style["input-after-icon"]}
          onClick={() => changePageType("org")}
        >
          <PlusOutlined />
          {t("page.im.ticket.addCompany")}
        </div>
      )}
    </div>
  );
}

export default SearchOrgField;
