//标签选择  可分词
//由于需要回显  初始化时没有数据源  只用id取不到显示内容  所以原数据返回 存储 回填
//返回数据格式为 [{value:1,label:'xxxx'},...]
//如只需id 在外部提交时单独处理
import React, { useCallback, useEffect, useState } from "react";
import { Select, Spin, Tag } from "udesk-ui";
import { getJSON } from "../../../../utils/index";

const { Option } = Select;

interface TagOptions {
  value: string;
  label: string;
  color?: string;
  key?: string;
}

function TagField(props: TicketField) {
  const { trigger = "onChange" } = props;
  const [value, setValue] = useState<TagOptions[]>([]); //选择的选项
  const [fetching, setFetching] = useState<boolean>(false); //无数据时展示
  const [lastFetchId, setLastFetchId] = useState<number>(0); //
  const [data, setData] = useState<TagOptions[]>([]); //搜索到的数据
  const [hasFocus, setHasFocus] = useState(false);

  useEffect(() => {
    fetchUser();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  useEffect(() => {
    setValue(() => {
      if (Array.isArray(props.value) && props.value.length > 0) {
        return props.value.map((item: any) => {
          return {
            value: (item.value || item.id) as string,
            label: item.label || item.name,
            color: item.color,
          };
        });
      } else {
        return [];
      }
    });
  }, [props.value]);
  // 搜索用户
  const fetchUser = useCallback(
    (searchValue: string = "") => {
      setLastFetchId(() => lastFetchId + 1);
      const fetchId = lastFetchId;
      setData([]);
      setFetching(true);
      getJSON("/tags?query=" + searchValue).then((res) => {
        if (fetchId !== lastFetchId) {
          // for fetch callback order
          return;
        }
        const data: TagOptions[] = (res.tags || []).map((item: any) => {
          return {
            value: item.id as string,
            label: item.name,
            color: item.color,
          };
        });
        setData(data);
        setFetching(false);
      });
    },
    [lastFetchId]
  );

  const reSelect = useCallback(() => {
    getJSON("/tags?query=").then((res) => {
      let data: TagOptions[] = (res.tags || []).map((item: any) => {
        return {
          label: item.name,
          value: item.id as string,
          color: item.color,
        };
      });
      setData(data);
    });
  }, []);

  const selectOnFocus = useCallback(() => {
    setHasFocus(true);
    getJSON("/tags?query=").then((res) => {
      let data: TagOptions[] = (res.tags || []).map((item: any) => {
        return {
          label: item.name,
          value: item.id as string,
          color: item.color,
        };
      });
      setData(data);
    });
  }, []);

  // 选项变化时回调
  const handleChange = useCallback(
    (value: TagOptions[]) => {
      if (value.length && !value[0].color) value[0].color = "1";
      setValue(value);
      setFetching(false);

      if (trigger === "onChange") {
        props.onChange && props.onChange(value);
      }

      if (!hasFocus) {
        if (trigger === "onBlur") {
          props.onBlur && props.onBlur(value);
        }
      }

      // selectOnFocus()
      reSelect();
    },
    [hasFocus, props, reSelect, trigger]
  );

  const handleOnBlur = useCallback(() => {
    setHasFocus(false);
    if (value.length && !value[0].color) value[0].color = "1";

    if (trigger === "onBlur") {
      props.onBlur && props.onBlur(value);
    }
  }, [props, trigger, value]);

  const handleColor = useCallback(
    (value) => {
      let colorType = "1";
      //查询数据源中是否有选择的数据  如果有   颜色用数据源中此数据的颜色
      for (let key in data) {
        if (data[key].value === value) {
          colorType = data[key].color as string;
        }
      }
      switch (colorType) {
        case "1":
          return "green";
        case "2":
          return "lime";
        case "3":
          return "blue";
        case "4":
          return "geekblue";
        case "5":
          return "magenta";
        case "6":
          return "purple";
        case "7":
          return "red";
        case "8":
          return "orange";
        case "9":
          return "gold";
        case "10":
          return "#999999";
      }
    },
    [data]
  );
  const tagRender = useCallback(
    (props: any) => {
      const { label, value, closable, onClose } = props;
      const color = handleColor(value);
      return (
        <Tag
          color={color}
          closable={closable}
          onClose={onClose}
          style={{ marginRight: 3 }}
        >
          {label}
        </Tag>
      );
    },
    [handleColor]
  );

  return (
    <div>
      {/* 带输入的下拉选择框 */}
      <Select
        mode="tags"
        showSearch //显示搜索
        showArrow={true} //显示下拉小箭头
        labelInValue={true} //是否把每个选项的 label 包装到 value 中
        value={value} //选择的选项
        placeholder={props.placeholder} //占位符
        notFoundContent={fetching ? <Spin size="small" /> : null}
        filterOption={false} //是否根据输入项进行筛选
        onSearch={fetchUser} //文本框值变化时回调
        onChange={handleChange} //选中 option
        style={{ width: "100%" }}
        getPopupContainer={(triggerNode) => triggerNode} //菜单渲染父节点
        allowClear={true} //支持清除
        onFocus={selectOnFocus} //获取焦点触发
        onBlur={handleOnBlur}
        tagRender={tagRender}
        tokenSeparators={[","]}
      >
        {data.map(({ value, label }) => (
          <Option key={value} value={value}>
            {label}
          </Option>
        ))}
      </Select>
    </div>
  );
}

export default TagField;
