import React, {
  useCallback,
  useContext,
  useEffect,
  useMemo,
  useRef,
  useState,
} from "react";
import style from "../index.module.scss";
import { Button, Form, message } from "udesk-ui";
import { useTranslation } from "react-i18next";
import * as FieldComponent from "../components/index";
import { Store } from "antd/lib/form/interface";
import { formatCustomField, postJSON, r } from "../../../utils/index";
import { csApiPostJSON } from "../../../utils/csApi";
import CustomField from "../../../components/custom-field-container";
import cloneDeep from "lodash-es/cloneDeep";
import { TicketContext } from "../../../im-panel/ticket/utilities";

interface CreateCustomerFormdata {
  nick_name: string;
  level: string;
  email: string | string[];
  description: string;
  cellphones?: string;
  tags?: string[];
  lang: string;
  owner_group_id: string;
  owner_id: string;
  cellphones_attributes?: { content: string }[];
  customerListSelected?: boolean;
  merging?: boolean;
  default_organization_id?: null;
  organizations?: number[];

  [propName: string]: any;
}

interface SubmitParams {
  customer: any;
  tags?: string;
  other_emails?: (null | string)[][];
}
const langsDict = [
  { id: "", text: "默认语言", lang_code: "" },
  { id: "zh-cn", text: "Chinese/简体中文", lang_code: "zh-cn" },
  { id: "en-us", text: "English", lang_code: "en-us" },
  { id: "bn", text: "Bengali/বাংলা ভাষা", lang_code: "bn" },
  { id: "de", text: "German/Deutsch", lang_code: "de" },
  { id: "es", text: "Spanisch/Español", lang_code: "es" },
  { id: "hi", text: "Hindi/हिन्दी", lang_code: "hi" },
  { id: "id", text: "Indonesian/Bahasa indonesia", lang_code: "id" },
  { id: "it", text: "Italian/italiano", lang_code: "it" },
  { id: "ja", text: "Japanese/日本語", lang_code: "ja" },
  { id: "ko", text: "Korean/한국어", lang_code: "ko" },
  { id: "pt", text: "Portuguese/Português", lang_code: "pt" },
  { id: "ru", text: "Russian/русский", lang_code: "ru" },
  { id: "th", text: "Thai/ไทย", lang_code: "th" },
  { id: "vi", text: "Vietnamese/Người việt nam", lang_code: "vi" },
  { id: "zh-HK", text: "HK Traditional Chinese/香港繁体", lang_code: "zh-HK" },
  { id: "el", text: "Greek/Ελληνική γλώσσα", lang_code: "el" },
  { id: "tr", text: "Turk/Türk dili", lang_code: "tr" },
  { id: "fr", text: "French/Française", lang_code: "fr" },
];
function CreateCustomer(props: any) {
  const formRef: any = useRef(null);
  //多语言
  const { t } = useTranslation();
  //ember传进来的参数

  const {
    company,
    customerFieldsv2,
    customerFieldsAuthority,
    ticketTab,
    roleCan,
    agentGroups,
    event,
  } = props;
  const _customerFieldsv2 = useMemo(() => {
    return cloneDeep(customerFieldsv2);
  }, [customerFieldsv2]);
  //新建工单用的useContext
  const { changePageType, changeCustomer } = useContext(TicketContext) as any;
  //自定义字段数据 field
  const [customField, setCustomField] = useState<any[]>([]);
  //自定义字段的fromData
  const [customFormData, setCustomFormData] = useState<any>();
  // 首次进入 处理自定义字段
  useEffect(() => {
    //循环用户自定义字段   从中筛选出自定义字段和im显示的子弹 并且处理是否必填  agent_permission=1就是必填的
    let contextCustomerFields = _customerFieldsv2.filter((item: any) => {
      return item.agent_permission && item.im_show;
    });
    const customerFields = formatCustomField({
      fields: contextCustomerFields,
      trigger: "onBlur",
      fieldsAuthority: customerFieldsAuthority,
      fieldType: "customer",
      editType: "new",
    }); //自定义字段的formItem
    setCustomField(customerFields);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  // 提交时触发
  const handleSubmit = useCallback(
    (value: Store) => {
      let params: SubmitParams = { customer: {} };
      let data = value as CreateCustomerFormdata;
      // 处理cellphones数据格式
      data.cellphones_attributes = data.cellphones
        ? data.cellphones.split(",").map((item) => {
            return {
              content: item,
            };
          })
        : [];
      //处理完成后删除cellphones   没有必要传
      delete data.cellphones;
      // 默认值
      data.customerListSelected = false;
      data.merging = false;
      data.default_organization_id = null;
      // 如果选了tags  给params赋值tags
      if (data?.tags?.length) {
        params.tags = data?.tags.join(",");
      }
      // 删除formdata的tags
      delete data.tags;
      //处理邮箱数据格式
      if (data.email.length > 1) {
        let ohter_emails: string[] = data.email as string[];
        //email 是 填的email的第一个值  其他的都是ohter_emails
        data.email = ohter_emails[0];
        const new_email = [null, ...ohter_emails.slice(1)];
        params.other_emails = [new_email];
      }
      // 将完整form中的自定义字段拆出来
      let custom_fields: { [index: string]: any } = {};
      // 循环完整表单数据
      for (let key in data) {
        // 如果当前循环的数据key在自定义表单中也有  取出来  删除
        if (key in customFormData) {
          custom_fields[key] = data[key];
          delete data[key];
        }
      }
      data.custom_fields = custom_fields;
      //选择的公司和邮箱处理空数据
      data.organizations = (data.organizations as number[]).filter(
        (item) => item
      );
      params.customer = data;
      csApiPostJSON("/cs/ts/api/customers", params).then(res=>res.data)
        .then((res) => {
          // 开始提交请求
          if (res.code === 1000) {
            // 创建成功后将创建的用户更新   给新增工单页面回显用
            changeCustomer({
              label: res.customer.nick_name,
              value: res.customer.id,
            });
            //更改页面类型  返回新建工单
            changePageType("ticket");
          } else {
            message.error(
              res?.code_message || res?.exception?.message || t("saveError")
            );
            message.error(res.code_message);
          }
        })
        .catch((e) => {
          e = JSON.parse(e.message);
          message.error(e.data.code_message);
        });
    },
    [changeCustomer, changePageType, customFormData, t]
  );
  const handleInitialValue = useMemo(() => {
    let customerFormData: { [index: string]: any } = {}; //自定义字段的formData
    //循环用户自定义字段
    _customerFieldsv2
      .filter((item: any) => {
        return item.agent_permission && item.im_show;
      })
      .forEach((item: any) => {
        customerFormData[item.name] =
          item.name.split("_")[0] === "TextField" ? "" : [];
      });
    setCustomFormData(customerFormData);
    return Object.assign(
      {
        nick_name: "",
        level: "normal",
        email: [],
        description: "",
        organizations: [],
        cellphones: "",
        tags: [],
        lang: "",
        owner_group_id: "",
        owner_id: "",
      },
      customerFormData
    );
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [false]);
  return (
    <>
      <div className={style["ticket-form"]}>
        <Form
          ref={formRef}
          initialValues={handleInitialValue}
          onFinish={handleSubmit}
          layout="vertical"
        >
          {/* 客户名称 */}
          <Form.Item
            name="nick_name"
            label={t("page.im.ticket.nick_name")}
            required={true}
            rules={[
              {
                required: true,
                message: r(t("needWrite"), t("page.im.ticket.nick_name")),
              },
            ]}
          >
            <FieldComponent.text
              trigger="onChange"
              isRequired={true}
              id="nick_name"
              type="text"
            />
          </Form.Item>
          {/* 等级 */}
          <Form.Item name="level" label={t("page.im.ticket.level")}>
            <FieldComponent.droplist
              id="level"
              type="droplist"
              trigger="onChange"
              options={[
                {
                  value: "normal",
                  title: t("page.im.ticket.normal"),
                },
                {
                  value: "vip",
                  title: "VIP",
                },
              ]}
            />
          </Form.Item>
          {/* 邮箱 */}
          <Form.Item
            name="email"
            label={t("page.im.ticket.email")}
            rules={[
              {
                validator: (rule: any, value: string[]) => {
                  let error = 0;
                  value.forEach((item) => {
                    if (
                      item &&
                      !/^[A-Za-z0-9\u4e00-\u9fa5]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+$/i.test(
                        item
                      )
                    ) {
                      error++;
                    }
                  });
                  return error ? Promise.reject("") : Promise.resolve();
                },
              },
            ]}
          >
            <FieldComponent.email placeholder={t("page.im.ticket.email")} />
          </Form.Item>
          {/* '描述',*/}
          <Form.Item name="description" label={t("page.im.ticket.description")}>
            <FieldComponent.textarea
              trigger="onChange"
              placeholder={t("page.im.ticket.forCustomerDesc")}
              id="description"
              type="textarea"
            />
          </Form.Item>
          {/* '公司' */}
          <Form.Item
            name="organizations"
            label={t("page.im.ticket.organizations")}
          >
            <FieldComponent.org
              placeholder={t("page.im.ticket.keywordQueryCompany")}
              roleCan={roleCan}
              company={company}
              event={event}
            />
          </Form.Item>
          {/* 电话 */}
          <Form.Item
            name="cellphones"
            label={t("page.im.ticket.phone")}
            rules={[
              {
                validator: (rule: any, value: string) => {
                  const list = value.split(",");
                  let error = 0,
                    errorText = "";
                  list.forEach((item) => {
                    if (
                      !/^1[34578][0-9]{9}$/.test(item) &&
                      !/^(0[0-9]{2,3}-)([2-9][0-9]{6,7})+(-[0-9]{1,4})?$/
                    ) {
                      error++;
                      errorText = errorText || "";
                    }
                  });
                  return error ? Promise.reject(errorText) : Promise.resolve();
                },
              },
            ]}
          >
            <FieldComponent.cellphone
              placeholder={t("page.im.ticket.phone")}
              event={event}
            />
          </Form.Item>
          {/* 标签 */}
          <Form.Item name="tags" label={t("page.im.ticket.tags")}>
            <FieldComponent.tag />
          </Form.Item>
          {/* 语言 */}
          {company.is_support_lang && (
            <Form.Item name="lang" label={t("page.im.ticket.lang")}>
              <FieldComponent.selectSearch
                options={langsDict.map((item: { id: string; text: string }) => {
                  return {
                    value: item.id,
                    title: item.text,
                  };
                })}
              />
            </Form.Item>
          )}
          {/* 负责组 */}
          <Form.Item
            name="owner_group_id"
            label={t("page.im.ticket.owner_group_name")}
          >
            <FieldComponent.agentGroups agentGroups={agentGroups} />
          </Form.Item>
          {/* 负责人 */}
          <Form.Item name="owner_id" label={t("page.im.ticket.owner_name")}>
            <FieldComponent.agent />
          </Form.Item>
          {/* customField */}
          {customField.length > 1 && <CustomField fields={customField} />}
        </Form>
      </div>

      <div className={style["bottom-btn-list"]}>
        <Button
          style={{ marginRight: "10px" }}
          onClick={() => changePageType("ticket")}
        >
          {t("cancel")}
        </Button>
        <Button type="primary" onClick={() => formRef.current.submit()}>
          {t("submit")}
        </Button>
      </div>
    </>
  );
}

export default CreateCustomer;
