import React, {
  useCallback,
  useContext,
  useEffect,
  useMemo,
  useRef,
  useState,
} from "react";
import style from "../index.module.scss";
import { Button, Form, message } from "udesk-ui";
import { useTranslation } from "react-i18next";
import * as FieldComponent from "../components/index";
import { Store } from "antd/lib/form/interface";
import { formatCustomField, postJSON } from "../../../utils/index";
import CustomField from "../../../components/custom-field-container";
import cloneDeep from "lodash-es/cloneDeep";
import { TicketContext } from "../../../im-panel/ticket/utilities";

interface CreateOrgFormdata {
  name: string;
  level: string;
  description: string;
  org_email_suffix: string;
  domains: string;

  [propName: string]: any;
}

interface SubmitParams {
  organization: {
    name: string;
    description: string;
    domains: string;
    level: string;
    custom_fields: { [index: string]: any };
  };
}

function CreateOrg(props: any) {
  const formRef: any = useRef(null);
  //多语言
  const { t } = useTranslation();
  //ember传进来的参数
  // const udeskWebappContext: ImContextProps = useContext(UdeskContext);
  const { organizationFieldsv2 = [] } = props;
  //新建工单用的useContext
  const { changePageType, changeOrganization } = useContext(
    TicketContext
  ) as any;
  //自定义字段数据 field
  const [orgField, setOrgField] = useState<IFieldItem[]>([]);
  ////自定义字段的fromData
  const [customData, setCustomFormData] = useState<any>();
  // 首次进入 处理自定义字段

  const _organizationFieldsv2 = useMemo(() => {
    return cloneDeep(organizationFieldsv2);
  }, [organizationFieldsv2]);

  useEffect(() => {
    //循环用户自定义字段   从中筛选出自定义字段和im显示的子弹 并且处理是否必填  agent_permission=1就是必填的
    let contextCustomerFields = _organizationFieldsv2.filter((item: any) => {
      item.is_required = item.agent_permission === 1 ? true : false;
      return item.agent_permission && item.im_show;
    });
    const organizationFields = formatCustomField({
      fields: contextCustomerFields,
    }); //自定义字段的formItem
    //给form字段重新复制 合并
    setOrgField(organizationFields);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  // 提交时触发
  const handleSubmit = useCallback(
    (value: Store) => {
      let params = { organization: {} } as SubmitParams;
      let data = value as CreateOrgFormdata;
      // 将完整form中的自定义字段拆出来
      let org_fields: { [index: string]: any } = {};
      // 循环完整表单数据
      for (let key in data) {
        // 如果当前循环的数据key在自定义表单中也有  取出来  删除
        if (key in customData) {
          org_fields[key] = data[key];
          delete data[key];
        }
      }
      params.organization = Object.assign(
        {
          custom_fields: org_fields,
        },
        data
      );
      // 开始提交请求
      postJSON("/organizations", params)
        .then((res) => {
          if (res.code === 1000) {
            // 创建成功后将创建的用户更新   给新增工单页面回显用
            changeOrganization({
              label: res.organization.name,
              value: res.organization.id,
            });
            //更改页面类型  返回新建工单
            changePageType("customer");
          } else {
            message.error(
              res?.code_message || res?.exception?.message || t("saveError")
            );
          }
        })
        .catch((e) => {
          console.log(e);
          message.error(t("saveError"));
        });
    },
    [changeOrganization, changePageType, customData, t]
  );
  // 表单数据改变触发  开发时方便查看  提测注释掉
  const handleChangeValue = useCallback((changeValue: any, allValue: any) => {
    // setFormData(allValue);
  }, []);

  const handleInitialValue = useMemo(() => {
    let customerFormData: { [index: string]: any } = {}; //自定义字段的formData
    //循环用户自定义字段   将处理过后的数据和formField,formData合并
    _organizationFieldsv2
      .filter((item: any) => {
        return item.agent_permission && item.im_show;
      })
      .forEach((item: any) => {
        customerFormData[item.name] =
          item.name.split("_")[0] === "TextField" ? "" : [];
      });
    setCustomFormData(customerFormData);
    return Object.assign(
      {
        name: "",
        level: "vip",
        description: "",
        org_email_suffix: "",
        domains: "",
      },
      customerFormData
    );
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [false]);
  return (
    <>
      <div className={style["ticket-form"]}>
        <Form
          ref={formRef}
          initialValues={handleInitialValue}
          onFinish={handleSubmit}
          layout="vertical"
          onValuesChange={handleChangeValue}
        >
          {/* 公司名称 */}
          <Form.Item
            name="name"
            label={t("page.im.ticket.organization_name")}
            required={true}
            rules={[{ required: true, message: `请填写公司名称` }]}
          >
            <FieldComponent.text
              trigger="onChange"
              isRequired={true}
              id="name"
              type="text"
            />
          </Form.Item>
          {/* 等级 */}
          <Form.Item name="level" label={t("page.im.ticket.level")}>
            <FieldComponent.droplist
              id="level"
              type="droplist"
              trigger="onChange"
              options={[
                {
                  value: "normal",
                  title: t("page.im.ticket.normal"),
                },
                {
                  value: "vip",
                  title: "VIP",
                },
              ]}
            />
          </Form.Item>
          {/* '描述',*/}
          <Form.Item name="description" label={t("page.im.ticket.description")}>
            <FieldComponent.textarea
              trigger="onChange"
              placeholder={t("page.im.ticket.descTip")}
              id="description"
              type="textarea"
            />
          </Form.Item>
          {/* 公司邮箱后缀 */}
          <Form.Item
            name="org_email_suffix"
            label={t("page.im.ticket.org_email_suffix")}
          >
            <FieldComponent.textHasPoint
              remark={t("page.im.ticket.emailSuffixHelpText")}
            />
          </Form.Item>
          {/* 公司域名 */}
          <Form.Item name="domains" label={t("page.im.ticket.domains")}>
            <FieldComponent.textHasPoint
              remark={t("page.im.ticket.helpBlock")}
            />
          </Form.Item>
          {orgField.length > 1 && <CustomField fields={orgField} />}
        </Form>
      </div>
      <div className={style["bottom-btn-list"]}>
        <Button
          style={{ marginRight: "10px" }}
          onClick={() => changePageType("customer")}
        >
          {t("cancel")}
        </Button>
        <Button type="primary" onClick={() => formRef.current.submit()}>
          {t("save")}
        </Button>
      </div>
    </>
  );
}

export default CreateOrg;
