/* eslint-disable no-unreachable */
import React, {
  useCallback,
  useEffect,
  useMemo,
  useState,
  useRef,
} from "react";
import style from "../index.module.scss";
import { useTranslation } from "react-i18next";
import { Button, Form, message, Select } from "udesk-ui";
import FormTinymce from "../../../components/tinymce/tinymce";
import * as FieldComponent from "../components/index";
import { formatCustomField, postJSON, r, getJSON } from "../../../utils/index";
import { csApiPostJSON } from '../../../utils/csApi';
import CustomField from "../../../components/custom-field-container";
import isEmpty from "lodash-es/isEmpty";
import moment from "moment";
import useDefault from "react-use/esm/useDefault";

let { Item: FormItem } = Form;

interface SubmitParams {
  ticket: {
    im_sub_session_id?: number; //会话id
    subject: string; //主题
    content: string; //内容
    customer_id: number; //客户(必填) 提单人
    agent_group_id: number; //客服组 ID
    assignee_id: number; //客服 ID
    status_id: string; //状态
    priority_id: string; //优先级
    platform_name: string; //工单来源, 可以是 'im', 'call' 或 'manual_input'
    attachment_ids: number[]; //一个数组。七牛返回的 attachment_id
    template_id: number; //工单模板 ID
    follower_ids: number[]; //关注人
    custom_fields: { [key: string]: any }; //自定义字段
  };
  tags: string;
}

interface TicketProps {
  setShowTicket: React.Dispatch<any>;
  onSaveSuccess?: (ticket: ITicket) => void;
}
const ticketStatus = [
  { key: "open", id: 1, name: "开启" },
  { key: "solving", id: 4, name: "解决中" },
  { key: "resolved", id: 2, name: "已解决" },
  { key: "closed", id: 3, name: "已关闭" },
];
const ticketPriority = [
  { key: "urgency", id: 1, name: "紧急" },
  { key: "high", id: 2, name: "高" },
  { key: "medium", id: 3, name: "标准" },
  { key: "low", id: 4, name: "低" },
];
const templateSelectStyle = { width: "auto", minWidth: "100px" };
const cancelButtonStyle = { marginRight: "10px" };
function CreateTicket(props: TicketProps & any) {
  const {
    currentUser,
    chat = {},
    ticketFieldsAbilities,
    company,
    roleCan,
    ticketTab,
    admin,
    event,
    onSaveSuccess,
  } = props;
  const [formRef] = Form.useForm();

  //处理工单模板选项数据
  const ticketTemplates: AntdSelectOptions[] = useMemo(() => {
    return props.ticketTemplates.map((item: { id: number; name: string }) => {
      return {
        value: item.id,
        label: item.name,
      };
    });
  }, [props.ticketTemplates]);

  //是否已经成功提交表单
  const [isSuccess, setIsSuccess] = useState<boolean>(false);
  // 多语言
  const { t } = useTranslation();
  // 选择的模板id
  const [template_id, setTemplate_id] = useDefault<number>(
    ticketTemplates[0].value as number,
    ticketTemplates[0].value as number
  );
  // 自定义字段
  const [customField, setCustomField] = useState<any[]>([]);
  // 获取当前工单的缓存信息
  const getLocalStorage = useCallback(() => {
    let localStorageData: any = localStorage.getItem(
      `${props.currentUser.id}-ticket-form-stash-at-email-im-message-${chat?.im_sub_session_id}`
    );
    localStorageData && (localStorageData = JSON.parse(localStorageData));
    return localStorageData;
  }, [chat?.im_sub_session_id, props.currentUser.id]);
  const noteCustomFields = useRef<any>();
  // 处理表单初始信息
  const formData = useMemo(() => {
    //缓存的数据
    let localStorageData: any = getLocalStorage();
    return {
      subject: r(t("page.im.ticket.createTicketByIM"), currentUser.nick_name),
      content:
        localStorageData && !isEmpty(localStorageData.createTicketFormData)
          ? localStorageData.createTicketFormData.content
          : "",
      customer_id: {
        value: chat.customer_id,
        label: chat.nick_name,
      },
      agent: [],
      follower_ids: [],
      status_id: 1,
      priority_id: 3,
      tags: [],
      attachment_ids: [],
      merchant_sid: {},
    };
  }, [
    getLocalStorage,
    chat.customer_id,
    chat.nick_name,
    t,
    currentUser.nick_name,
  ]);

  //首次进入  查看是否有缓存的模板id  如果有  将模板id换成缓存的id
  useEffect(() => {
    let _willDeleteItems = localStorage.getItem("will-delete-items");
    if (_willDeleteItems) {
      let willDeleteItems = JSON.parse(_willDeleteItems);
      let newWillDeleteItems: { [key: string]: string } = {};
      for (let item in willDeleteItems) {
        if (localStorage.getItem(item)) {
          if (moment(willDeleteItems[item]).isBefore(moment())) {
            localStorage.removeItem(item);
          } else {
            newWillDeleteItems[item] = willDeleteItems[item];
          }
        }
      }
      localStorage.setItem(
        "will-delete-items",
        JSON.stringify(newWillDeleteItems)
      );
    }
    let localStorageData: any = getLocalStorage();
    if (localStorageData) {
      setTemplate_id(localStorageData.createTicketTemplateId);
    }

    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  //整个表单是否滚动   为了解决内部的下拉选择框的滚动穿透  聚焦下拉选择时  将overScroll变成hidden
  const [isScroll, setIsScroll] = useState<boolean>(true);

  //模板改变时   将对应模板id的模板筛选出来  渲染新的自定义表单
  useEffect(() => {
    let localStorageData: any = getLocalStorage();
    const field = formatCustomField({
      fields:
        (props.ticketTemplates.find(
          (item: any) => item.id === template_id
        ) as any)?.fields || [],
      trigger: "onChange",
      fieldsAuthority: ticketFieldsAbilities[template_id],
      fieldType: "ticket",
      editType: "new",
    });
    //处理自定义字段的formData 处理好以后与系统字段合并  给formData重置初始值
    // 如果当前会话id有缓存的模板信息  直接取出  如果没有  进行以上处理
    let customFormData: { [index: string]: any } = {}; //自定义字段的formData
    if (localStorageData && !isEmpty(localStorageData.createTicketFormData)) {
      customFormData = localStorageData.createTicketFormData;
    } else {
      //循环用户自定义字段   将自定义字段对应的key 和value取出  作为自定义字段的formData
      field.forEach((item: any) => {
        customFormData[item.field.id] =
          item.field.id.split("_")[0] === "TextField" ? "" : [];
      });
    }
    // 将自定义的formData和系统字段的formdata合并   然后form重置初始值
    formRef.setFieldsValue(Object.assign(formData, customFormData));
    //保存自定义字段  渲染自定义字段的组件使用
    setCustomField(field);
    //离开新建工单页面时  缓存工单模板id和表单数据
    return () => {
      let data = {
        createTicketTemplateId: template_id,
        createTicketFormData: isSuccess ? {} : formRef.getFieldsValue(),
      };
      let stashKey = `${currentUser.id}-ticket-form-stash-at-email-im-message-${chat.im_sub_session_id}`;
      localStorage.setItem(stashKey, JSON.stringify(data));
      //存时间
      if (!localStorage.getItem("will-delete-items")) {
        window.localStorage.setItem("will-delete-items", JSON.stringify({}));
      }
      let willDeleteItems: any = window.localStorage.getItem(
        "will-delete-items"
      );
      willDeleteItems = JSON.parse(willDeleteItems);
      willDeleteItems[stashKey] = moment().add(24, "hours").toJSON();
      localStorage.setItem(
        "will-delete-items",
        JSON.stringify(willDeleteItems)
      );
    };
  }, [
    template_id,
    isSuccess,
    getLocalStorage,
    props.ticketTemplates,
    ticketFieldsAbilities,
    currentUser.id,
    chat.im_sub_session_id,
    formRef,
    formData,
  ]);
  //点击公司  跳转到对应的公司详情
  const gotoTicket = useCallback(
    (id: number) => {
      event.transitionToRoute &&
        event.transitionToRoute({
          route: "ticket.show",
          path: id,
        });
    },
    [event]
  );
  // 关闭工单
  const closeTicket = useCallback(() => {
    props.setShowTicket({
      show: false,
    });
    event.updateChat &&
      event.updateChat({
        chat: chat,
        updateProps: {
          newTicketReact: {},
        },
      });
  }, [props, chat, event]);
  // 提交表单
  const handleSubmit = useCallback(
    (value: any) => {
      const params: SubmitParams = {
        ticket: {
          assignee_id: currentUser.id,
          platform_name: "im",
          template_id: template_id,
          im_sub_session_id: chat.im_sub_session_id as number,
        },
        tags: "",
      } as SubmitParams;
      // 如果选了受理人 拆成客服组id客服id
      if (value.agent.length) {
        params.ticket.agent_group_id = value.agent[0];
        params.ticket.assignee_id = value.agent[1] || "";
      }
      delete value.agent;

      // 处理自定义字段   将自定义字段从formData中筛选出来
      let custom_fields: { [objKey: string]: any } = {};
      customField.forEach((item) => {
        let key = item.field.id;
        if (value[key] !== undefined) {
          custom_fields[key] = Array.isArray(value[key])
            ? value[key].join(",")
            : value[key];
        }
        delete value[key];
      });
      params.ticket.custom_fields = custom_fields;
      //处理标签
      value.tags.length &&
        (params.tags = value.tags
          .map((item: AntdSelectOptions) => item.label)
          .join(","));
      delete value.tags;
      //处理客户id
      value.customer_id = value.customer_id.value;
      //处理附件id
      value.attachment_ids = value.attachment_ids
        ? value.attachment_ids.map((item: any) => item.uid)
        : [];
      //处理商户
      if (company.category === "mchat_primary")
        value.merchant_sid = value.merchant_sid.value;
      // 将剩下的字段和处理好的ticket合并
      params.ticket = Object.assign(params.ticket, value);
      csApiPostJSON("/cs/ts/api/tickets", params)
        .then(res=>res.data)
        .then((res) => {
          if (res.code === 1000) {
            setIsSuccess(true);
            let success = (
              // eslint-disable-next-line jsx-a11y/anchor-is-valid
              <span>
                工单
                <a onClick={() => gotoTicket(res.ticket.id)}>
                  {res.ticket.field_num}
                </a>
                创建成功
              </span>
            );
            message.success(success);
            setTimeout(() => {
              closeTicket();
            }, 1500);
            onSaveSuccess?.(res.ticket);
          } else {
            message.error(
              res?.code_message || res?.exception?.message || t("saveError")
            );
          }
        })
        .catch((e) => {
          message.error(e);
        });
    },
    [
      closeTicket,
      customField,
      gotoTicket,
      t,
      template_id,
      chat,
      company.category,
    ]
  );

  // 表单数据改变触发
  function handleChangeValue(changeValue: any, allValue: any) {
    // console.log(allValue)
  }

  // 改变模板  记录模板id  改变自定义字段
  const changeTemplate = useCallback((value: number) => {
    setTemplate_id(value);
  }, []);

  useEffect(() => {
    const handleFields = function () {
      customField.forEach((i) => {
        const defaults = i?.field?.options?.filter((j: any) => j.default) || [];
        if (!defaults.length) {
          return;
        }
        formRef.setFieldsValue({
          [i.field.id]: defaults.map((j: any) => j.value),
        });
      });
      for (let item in noteCustomFields.current) {
        let field = customField.find((_item) => _item.field.id === item);
        if (field) {
          formRef.setFieldsValue({ [item]: noteCustomFields.current[item] });
        }
      }
    };
    if (noteCustomFields.current) {
      handleFields();
    } else {
      getJSON("/notes/detail", {
        type: "im",
        ref_id: chat.im_sub_session_id,
        template_id: "",
      }).then((res) => {
        noteCustomFields.current = res.data.custom_fields;
        handleFields();
      });
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [customField, formRef]);

  return (
    <>
      {/* 新建工单内容 */}
      <div
        className={style["ticket-form"]}
        style={{ overflowY: isScroll ? "scroll" : "hidden" }}
      >
        <Form
          form={formRef}
          onFinish={handleSubmit}
          initialValues={formData}
          layout="vertical"
        >
          {/* 主题 */}
          <Form.Item label={t("page.im.ticket.subject")} name="subject">
            <FieldComponent.text
              id={"subject"}
              trigger="onChange"
              type="text"
            />
          </Form.Item>
          {/* 描述 */}
          <Form.Item label={t("page.im.ticket.content")} name="content">
            <FormTinymce
              autoresize_min_height={130}
              autoresize_max_height={200}
              statusbar={true}
              paste_data_images={true}
            />
          </Form.Item>
          {/* 上传附件 */}
          {roleCan.ticket_attachments && (
            <FormItem label="" name="attachment_ids">
              <FieldComponent.upload ticketTab={ticketTab} />
            </FormItem>
          )}

          {/* 客户 */}
          <Form.Item label={t("page.im.ticket.customer")} name="customer_id">
            <FieldComponent.customer
              setIsScroll={setIsScroll}
              ticketTab={ticketTab}
            />
          </Form.Item>
          {/* 受理人 */}
          <Form.Item label={t("page.im.ticket.receiver")} name="agent">
            <FieldComponent.receiver />
          </Form.Item>
          {/* 关注人 */}
          <Form.Item label={t("page.im.ticket.follower")} name="follower_ids">
            <FieldComponent.follower admin={admin} currentUser={currentUser} />
          </Form.Item>
          {/* 商户 */}
          {company.category === "mchat_primary" && (
            <FieldComponent.merchant setIsScroll={setIsScroll} />
          )}
          {/* 状态 */}
          <Form.Item label={t("page.im.ticket.status")} name="status_id">
            <FieldComponent.selectSearch
              options={ticketStatus.map((item: any) => {
                return {
                  value: item.id,
                  title: item.name,
                };
              })}
            />
          </Form.Item>
          {/* 优先级 */}
          <Form.Item label={t("page.im.ticket.priority")} name="priority_id">
            <FieldComponent.selectSearch
              options={ticketPriority.map((item: any) => {
                return {
                  value: item.id,
                  title: item.name,
                };
              })}
            />
          </Form.Item>
          {/* 标签 */}
          <Form.Item label={t("page.im.ticket.tags")} name="tags">
            <FieldComponent.tag />
          </Form.Item>
          {/* 切换工单模板 */}
          <Form.Item label="">
            <div className={style["select-template-wrapper"]}>
              <div className={style["template-title"]}>
                {t("page.im.ticket.ticketTemplate")}
              </div>
              <Select
                showSearch
                showArrow={true} //显示下拉小箭头
                optionFilterProp="label"
                onChange={changeTemplate} //选中 option
                style={templateSelectStyle}
                value={template_id}
                dropdownMatchSelectWidth={false}
                options={ticketTemplates}
              />
            </div>
          </Form.Item>
          {customField.length > 0 && <CustomField fields={customField} />}
        </Form>
      </div>
      <div className={style["bottom-btn-list"]}>
        <Button style={cancelButtonStyle} onClick={closeTicket}>
          {t("cancel")}
        </Button>
        <Button type="primary" onClick={() => formRef.submit()}>
          {t("save")}
        </Button>
      </div>
    </>
  );
}

export default CreateTicket;
