import React, { useEffect, useMemo, useState } from "react";
import style from "./index.module.scss";
import { useTranslation } from "react-i18next";
import CreateTicket from "./create-ticket-page";
import CreateCustomer from "./create-customer-page";
import CreateOrg from "./create-org-page";
import { getJSON } from "../../utils";
import { TicketContext } from "./utilities";

interface TicketProps {
  setShowTicket: React.Dispatch<any>;
  onCreateSuccess: (ticket: ITicket) => void;
}

function CreateTicketIndex({ onCreateSuccess, ...props }: TicketProps & any) {
  const [agent, setAgent] = useState<AntdSelectOptions[]>([]);
  useEffect(() => {
    getJSON("/user_groups/list_with_agents_status").then((res) => {
      let options = res.user_groups.map((item: any) => {
        return {
          label: res.ticket_setting_is_open
            ? `${item.name}  (${res.ticket_setting_online_title} ${item.agents_availability_count}/${item.agents_count})`
            : item.name,
          value: item.id,
          agents_availability_count: item.agents_availability_count,
          agents_count: item.agents_count,
          children: item.agents.map((item: any) => {
            return {
              label: res.ticket_setting_is_open
                ? `${item.nick_name}  (${
                    item.availability
                      ? res.ticket_setting_online_title
                      : res.ticket_setting_offline_title
                  })`
                : item.nick_name,
              value: item.id,
              availability: item.availability,
            };
          }),
        };
      });
      setAgent(options);
    });
  }, []);
  const { t } = useTranslation();
  const [pageType, setPageType] = useState("ticket"); //页面类型  用于切换界面   ticket 新建工单  customer 新建客户 org 新建公司
  const [agentGroup, setAgentGroup] = useState(""); //负责组   新增用户中的负责组 组件中改变 负责组选择  改变该值  在 负责人组件中 使用
  const [customer, setCustomer] = useState<AntdSelectOptions>(
    {} as AntdSelectOptions
  );
  const [organization, setOrganization] = useState<AntdSelectOptions>(
    {} as AntdSelectOptions
  );
  //根据pagetype不一样 改变头部标题
  const header =
    pageType === "ticket"
      ? t("page.im.ticket.createTicket")
      : pageType === "customer"
      ? t("page.im.ticket.createCustomer")
      : t("page.im.ticket.createOrg");
  const contextContent = useMemo(() => {
    return {
      changePageType: (type: string) => setPageType(type),
      agentGroup,
      changeAgentGroup: (type: string) => setAgentGroup(type),
      customer,
      changeCustomer: (object: AntdSelectOptions) => setCustomer(object),
      organization,
      changeOrganization: (object: AntdSelectOptions) =>
        setOrganization(object),
      agent,
    };
  }, [agent, agentGroup, customer, organization]);
  const content = useMemo(() => {
    switch (pageType) {
      case "ticket":
        return (
          <CreateTicket
            setShowTicket={props.setShowTicket}
            onSaveSuccess={onCreateSuccess}
            {...props}
          />
        );
      case "customer":
        return <CreateCustomer {...props} />;
      default:
        return <CreateOrg {...props} />;
    }
  }, [pageType, props]);
  return (
    <div className={style["ticket-wrapper"]}>
      {/* 头部标题  新建工单 */}
      <div className={style["ticket-header"]}>{header}</div>
      <TicketContext.Provider value={contextContent}>
        {content}
      </TicketContext.Provider>
    </div>
  );
}

export default CreateTicketIndex;
