import React, { useCallback } from "react";
import style from "./style.module.scss";
import { Button, DatePicker, Form, Input, message } from "udesk-ui";
import { CuoHao, DuiHao } from "../../components/icon/icon";
import isFunction from "lodash-es/isFunction";
import moment from "moment";
import { useTranslation } from "react-i18next";
import { postJSON, YYYY_MM_DD_HH_MM_SS } from "../../utils";

interface ImVisitProps {
  content?: string;
  showCancel?: boolean;
  setShowVisit: any;
  event?: any;
  chat: any;
}

const ImVisit: React.FC<ImVisitProps> = (props) => {
  const { t } = useTranslation();
  const [formRef] = Form.useForm();
  const { event, setShowVisit, chat } = props;

  const handleSave = useCallback(
    (allValue: any) => {
      let { schedule_time, remark } = allValue;

      if (!schedule_time) {
        message.warn(t("page.im.visit.choiceTime"));
        return;
      }
      if (moment(schedule_time).isBefore(moment())) {
        message.warn(t("page.im.visit.alertTimeTip"));
        return;
      }

      if (isFunction(setShowVisit)) {
        setShowVisit({
          show: false,
        });
      }
      event.updateChat &&
        event.updateChat({
          chat: chat,
          key: "newVisit",
          value: {},
        });
      postJSON("alarms", {
        alarm: {
          call_log_id: null,
          callout_task_id: null,
          callout_task_import_method: null,
          callout_task_name: null,
          callout_task_source: null,
          contact_list_id: null,
          contact_method: "im",
          customer_id: chat.customer_id,
          hide_number: false,
          im_sub_session_id: chat.im_sub_session_id,
          overdue: false,
          remark: remark || "",
          result: "unexecuted",
          schedule_time: moment(schedule_time).toDate(),
          ticket_id: null,
          user: null,
          user_id: null,
        },
      }).then((res) => {
        if (res.code === "1000" || res.code === 1000) {
          message.success("保存成功");
        } else {
          message.warning(
            res?.code_message ||
              res?.exception?.message ||
              t("page.im.imMessages.voiceOvertime")
          );
        }
      });
    },
    [chat, event, setShowVisit, t]
  );

  const handleClose = useCallback(() => {
    setShowVisit({
      show: false,
    });
    event.updateChat &&
      event.updateChat({
        chat: chat,
        key: "newRecord",
        value: {},
      });
  }, [chat, event, setShowVisit]);

  const save = useCallback(() => {
    formRef.submit();
  }, [formRef]);

  return (
    <div className={style["visit-container"]}>
      <div className={style["visit-header"]}>
        <div className={style["header-name"]}>{t("page.im.visit.title")}</div>
        <div className={style["header-icons"]}>
          <span onClick={save}>
            <DuiHao />
          </span>
          <span onClick={handleClose}>
            <CuoHao />
          </span>
        </div>
      </div>
      <div className={style["visit-body"]}>
        <Form
          form={formRef}
          layout={"vertical"}
          onFinish={handleSave}
          initialValues={{
            schedule_time: moment(),
          }}
        >
          <Form.Item label="计划时间" name="schedule_time">
            <DatePicker format={YYYY_MM_DD_HH_MM_SS} showTime />
          </Form.Item>

          <Form.Item label="备注" name="remark">
            <Input.TextArea />
          </Form.Item>

          <Form.Item>
            {props.showCancel && (
              <Button type="primary" onClick={handleClose}>
                {t("cancel")}
              </Button>
            )}
            <Button type="primary" htmlType="submit" size="small">
              {t("save")}
            </Button>
          </Form.Item>
        </Form>
      </div>
    </div>
  );
};

export default ImVisit;
