'use strict';

module.exports = logSdkWebpackPlugin;

function logSdkWebpackPlugin(props) {
    if (!props.projectName) {
        throw new Error('必须传 projectName');
    }
    this.projectName = props.projectName;
}

logSdkWebpackPlugin.prototype.apply = function(compiler) {
    // 指定要附加到的事件钩子函数
    compiler.hooks.done.tap(
        'LogPlugin',
        (stats) => {
            console.log(stats.toString({
                colors: true    // 在控制台展示颜色
            }));

            if (stats.hasErrors()) {
                return;
            }

            const project = this.projectName;
            let result = `\n//请将以下代码复制到 webapp 项目的 config/sdk.js 中的 ${project} 部分\n`; 

            const statsJson = stats.toJson({
                modules: false,
                chunkGroups: false,
                chunks: false,
                chunkOrigins: false,
                entrypoints: false
            });

            const assetsByChunkName = statsJson.assetsByChunkName;
            for (const chunkName in assetsByChunkName) {
                let chunkAssets = assetsByChunkName[chunkName];
                if (Array.isArray(chunkAssets)) {
                    chunkAssets = chunkAssets.filter(i => i.endsWith('.js') || i.endsWith('.css'));
                } else {
                    chunkAssets = [chunkAssets];
                }
                chunkAssets.forEach(i => {
                    let dir = i.substring(0, i.lastIndexOf('/'));
                    let ext = i.substring(i.lastIndexOf('.'));
                    result += `'/${project}${dir ? ('/' + dir) : ''}/${chunkName}${ext}':'/${project}/${i}',`;
                });
            }

            console.log(result + '\n\n');
        }
    );
};
