import React, { Component, Fragment }                  from 'react';
import PropTypes                                       from 'prop-types';
import { canShowCustomerPhoneNumber, maskPhoneNumber } from '@udesk-webapp/utils';

class Index extends Component {
    constructor(props) {
        super(props);
    }

    shouldMask() {
        let {isMask, customer, roleCan, currentUser} = this.props;
        if (typeof isMask === 'boolean') {
            return isMask;
        }
        try {
            if (typeof isMask === 'function') {
                return isMask();
            }
        } catch (e) {
            console.error(e);
        }

        return !canShowCustomerPhoneNumber(currentUser, customer, roleCan);
    }

    render() {
        let {phoneIcon, smsIcon, number, onPhoneClick, onSmsClick} = this.props;
        let finalNumber = this.shouldMask() ? maskPhoneNumber(number) : number;
        if (phoneIcon && smsIcon) {
            return <Fragment>
                <a href="javascript:" onClick={(e) => onPhoneClick(finalNumber, number, e)}>
                    {finalNumber}
                    <i className="icon icon-phone"/>
                </a>
                <i className="icon icon-comment" onClick={(e) => onSmsClick(finalNumber, number, e)}/>
            </Fragment>;
        } else if (phoneIcon) {
            return <a href="javascript:" onClick={(e) => onPhoneClick(finalNumber, number, e)}>
                {finalNumber} <i className="icon icon-phone"/>
            </a>;
        } else if (smsIcon) {
            return <a href="javascript:" onClick={(e) => onSmsClick(finalNumber, number, e)}>
                {finalNumber} <i className="icon icon-comment"/>
            </a>;
        } else {
            return (
                <Fragment>{finalNumber}</Fragment>
            );
        }
    }
}

//@formatter:off
Index.propTypes = {
    /**
     * 要显示的电话号码
     */
    number       : PropTypes.string,
    /**
     * 电话号码所属客户
     */
    customer     : PropTypes.shape({
                       owner_id       : PropTypes.number,
                       owner_group_id : PropTypes.number
                   }),
    phoneIcon    : PropTypes.bool,
    smsIcon      : PropTypes.bool,
    onPhoneClick : PropTypes.func,
    onSmsClick   : PropTypes.func,
    currentUser  : PropTypes.shape({
                       id          : PropTypes.number,
                       user_groups : PropTypes.arrayOf(PropTypes.shape({
                                         id : PropTypes.number
                                     }))
                   }),
    roleCan      : PropTypes.shape({
                       customer_show_cellphone_all      : PropTypes.bool,
                       customer_show_cellphone_group    : PropTypes.bool,
                       customer_show_cellphone_personal : PropTypes.bool
                   }),
    isMask       : PropTypes.oneOfType([PropTypes.bool, PropTypes.func])
};
//@formatter:on

export default Index;
