import React, { Component, createRef } from 'react';
import PropTypes                       from 'prop-types';
import CommonDropdown                  from './CommonDropdown';
import Control                         from './Control';

class SingleSelect extends Component {
    _ref = createRef();

    constructor(props) {
        super(props);
        this.state = {
            isOpen  : false,
            options : []
        };
    }

    render() {
        let {
                controlClassName,
                controlStyle,
                prefix         = '',
                options        = [],
                getOptionLabel = v => JSON.stringify(v),
                getOptionValue = v => v,
                onChange,
                value,
                placeholder    = '全部',
                t              = v => v,
                loadOptions,
                components,
                className      = ''
            } = this.props;

        return (
            <div className={'select-with-search ' + className} ref={this._ref}>
                <div className={'align-justify ' + controlClassName} style={controlStyle} onClick={this.toggleDropdown}>
                    <components.Control value={value} options={options} prefix={prefix}
                                        getOptionLabel={getOptionLabel} getOptionValue={getOptionValue}
                                        placeholder={placeholder}
                                        isOpen={this.state.isOpen}
                                        t={t}/>
                </div>
                {do {
                    if (this.state.isOpen) {
                        if (!loadOptions) {
                            <CommonDropdown value={value}
                                            onChange={this._onChange}
                                            getOptionValue={getOptionValue}
                                            getOptionLabel={getOptionLabel}
                                            t={t}
                                            options={options}/>;
                        }
                    }
                }}
            </div>
        );
    }

    _onChange = (value, option) => {
        this.setState({isOpen : false});
        this.props.onChange(value, option);
    };

    componentDidMount() {
        document.addEventListener('mousedown', this.handleClickOutside);
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClickOutside);
    }

    toggleDropdown = () => {
        this.setState({isOpen : !this.state.isOpen});
    };

    handleClickOutside = (event) => {
        if (this._ref.current && !this._ref.current.contains(event.target)) {
            this.setState({isOpen : false});
        }
    };

}

SingleSelect.propTypes = {
    controlClassName : PropTypes.string,
    controlStyle     : PropTypes.object,
    prefix           : PropTypes.string,
    value            : PropTypes.any,
    options          : PropTypes.arrayOf(PropTypes.any),
    loadOptions      : PropTypes.func,
    getOptionLabel   : PropTypes.func,
    getOptionValue   : PropTypes.func,
    onChange         : PropTypes.func,
    placeholder      : PropTypes.string,
    className        : PropTypes.string,
    components       : PropTypes.object
};

const defaultComponents = {
    Control : Control
};

SingleSelect.defaultProps = {
    components       : defaultComponents,
    controlClassName : 'form-control',
    className        : ''
};

export default SingleSelect;

export const components = defaultComponents;
