import chain                           from 'lodash-es/chain';
import filter                          from 'lodash-es/filter';
import find                            from 'lodash-es/find';
import forEach                         from 'lodash-es/forEach';
import includes                        from 'lodash-es/includes';
import isArray                         from 'lodash-es/isArray';
import isEmpty                         from 'lodash-es/isEmpty';
import map                             from 'lodash-es/map';
import some                            from 'lodash-es/some';
import startsWith                      from 'lodash-es/startsWith';
import uniq                            from 'lodash-es/uniq';
import PropTypes                       from 'prop-types';
import React, { Component, createRef } from 'react';
import Popover                         from '@udesk-webapp/popover';
import IndeterminateCheckbox           from './IndeterminateCheckbox';

class ChainedSelect extends Component {
    constructor(props) {
        super(props);
        this.state = {
            openedNode : []
        };
        this._openedNode = [];
        this._scrollTop = 0;
    }

    render() {
        let {t, value = [], ...props} = this.props;
        return (
            <Popover {...props} value={value} components={{
                Control    : ({prefix, value = [], controlClassName, isOpen, placeholder, popContentStyle, options, ...props}) => (
                    <div className={controlClassName} {...props}>
                        <span>{prefix} {do {
                            if (isArray(value) && !isEmpty(value)) {
                                t('label.selected');
                            } else {
                                placeholder;
                            }
                        }}
                        </span>
                        <i className={`icon icon-caret-${isOpen ? 'up' : 'down'}`}/>
                    </div>
                ),
                PopContent : ({options, ...props}) => {
                    return <NodeList
                        content={map(options, (option) => ({
                            id      : option.value, title : option.title, subs : option.subs,
                            checked : includes(value, option.value)
                        }))} {...props}
                        openedNode={this._openedNode}
                        onNodeOpen={node => this._openedNode.push(node)}
                        onNodeClose={node => this._openedNode = filter(this._openedNode, (i) => i !== node)}
                        style={{
                            'minWidth' : '100%', whiteSpace : 'nowrap', padding : '5px 10px 0', maxHeight : '330px',
                            overflow   : 'auto'
                        }}
                        value={value}
                        onChildNodeChange={this.onChildNodeChange}
                        onScroll={this.saveScrollTop}
                        scrollTop={this._scrollTop}
                    />;
                }
            }}/>
        );
    }

    onChildNodeChange = (childOptionValue, isChecked, checkedIds, uncheckedIds, originChangedNodeId) => {
        let {value = [], onChange} = this.props;
        let finalValue = [...value, ...checkedIds].filter(i => !includes(uncheckedIds, i))
                                                  .filter(i => i === originChangedNodeId ||
                                                               !startsWith(i, originChangedNodeId + ',')); //过滤掉当前节点的所有子节点
        finalValue = uniq(finalValue);

        onChange(finalValue);
    };

    saveScrollTop = (e) => {
        this._scrollTop = e.target.scrollTop;
    };
}

class Node extends Component {
    constructor(props) {
        super(props);
        let {id, value, checked} = props;
        this.state = {
            isOpen : includes(props.openedNode, id),
            subs   : map(props.subs, child => {
                let childId = id + ',' + child.value;
                return {
                    id      : childId,
                    title   : child.title,
                    checked : checked || includes(value, childId),
                    subs    : child.subs
                };
            })
        };
    }

    render() {
        let {checked, title, id, content, onChange, value, onNodeOpen, onNodeClose, openedNode, onChildNodeChange} = this.props;
        let {isOpen, subs} = this.state;
        let nodeStyle = {};
        if (isEmpty(subs)) {
            nodeStyle.paddingLeft = '13px';
        }
        return <div>
            <div style={nodeStyle}>
                {(() => {
                    if (!isEmpty(subs)) {
                        return <i className={`icon icon-caret-${isOpen ? 'down' : 'right'}`}
                                  onClick={() => this.toggleOpen()}/>;
                    }
                })()}
                <label>
                    <IndeterminateCheckbox className="mgr-5"
                                           indeterminate={some(value, (i) => startsWith(i, id + ',') && i !== id)}
                                           checked={checked}
                                           type='checkbox'
                                           value={id}
                                           onChange={(e) => {
                                               onChildNodeChange(
                                                   id,
                                                   e.target.checked,
                                                   e.target.checked ? [id] : [],
                                                   e.target.checked ? [] : [id],
                                                   id
                                               );
                                           }}/>
                    {title}
                </label>
            </div>
            {(() => {
                if (isOpen && !isEmpty(subs)) {
                    return <div style={{paddingLeft : '13px'}}>
                        <NodeList content={subs}
                                  parentOptionValue={id}
                                  value={value}
                                  onChange={onChange}
                                  onNodeOpen={onNodeOpen}
                                  onNodeClose={onNodeClose}
                                  openedNode={openedNode}
                                  parentNodeContent={content}
                                  onChildNodeChange={this.onChildNodeChange}
                        />
                    </div>;
                }
            })()}
        </div>;
    }

    onChildNodeChange = (childId, isChecked, checkedIds, uncheckedIds, originChangedNodeId) => {
        let {subs} = this.state;
        let {id} = this.props;
        find(subs, {id : childId}).checked = isChecked;

        let checkedSubs = [];
        let uncheckedSubs = [];

        forEach(subs, i => {
            if (i.checked) {
                checkedSubs.push(i.id);
            } else {
                uncheckedSubs.push(i.id);
            }
        });

        if (uncheckedSubs.length === 0) {
            //如果子节点全部选中，则选中当前节点，并删除所有子节点
            this.props.onChildNodeChange(id, true, [...checkedIds, id], [...uncheckedIds,
                                                                         ...checkedSubs], originChangedNodeId);
        } else if (checkedSubs.length === 0) {
            //如果子节点全部未选中，则删除当前节点
            this.props.onChildNodeChange(id, false, [...checkedIds], [...uncheckedIds, id], originChangedNodeId);
        } else {
            this.props.onChildNodeChange(id, false, [...checkedIds, ...checkedSubs], [...uncheckedIds, id,
                                                                                      ...uncheckedSubs], originChangedNodeId);
        }
    };

    toggleOpen() {
        let isOpen = !this.state.isOpen;
        this.setState({isOpen});
        if (isOpen) {
            this.props.onNodeOpen(this.props.id);
        } else {
            this.props.onNodeClose(this.props.id);
        }
    }
}

class NodeList extends Component {
    _ref = createRef();

    constructor(props) {
        super(props);
    }

    render() {
        let {content, style, ...props} = this.props;
        if (isArray(content) && !isEmpty(content)) {
            return <div style={style} ref={this._ref}>
                {map(content, (item) => (
                    <Node key={item.id} {...item} {...props}/>
                ))}
            </div>;
        }
    }

    componentDidMount() {
        let element = this._ref.current;
        if (this.props.scrollTop) {
            element.scrollTop = this.props.scrollTop;
        }
        if (this.props.onScroll) {
            element.addEventListener('scroll', (e) => {
                this.props.onScroll(e);
            });
        }
    }
}

ChainedSelect.propTypes = {
    id                : PropTypes.any,
    value             : PropTypes.any,
    checked           : PropTypes.any,
    openedNode        : PropTypes.any,
    subs              : PropTypes.any,
    title             : PropTypes.any,
    content           : PropTypes.any,
    onChange          : PropTypes.any,
    onNodeOpen        : PropTypes.any,
    onNodeClose       : PropTypes.any,
    onChildNodeChange : PropTypes.any,
    popContentStyle   : PropTypes.object
};

Node.propTypes = {
    id                : PropTypes.string,
    value             : PropTypes.array,
    checked           : PropTypes.bool,
    openedNode        : PropTypes.array,
    subs              : PropTypes.array,
    title             : PropTypes.string,
    content           : PropTypes.string,
    onChange          : PropTypes.func,
    onNodeOpen        : PropTypes.func,
    onNodeClose       : PropTypes.func,
    onChildNodeChange : PropTypes.func
};
Node.defaultProps = {};

ChainedSelect.defaultProps = {
    className        : 'select-with-search',
    controlClassName : 'align-justify form-control'
};
export default ChainedSelect;
