import React, { Component } from 'react';
import PropTypes            from 'prop-types';

class DateRangePicker extends Component {
    constructor(props) {
        super(props);
        this.dateRangePicker = null;
    }

    attachPickerEvents() {
        let self = this;
        this.dateRangePicker.off('apply.daterangepicker').on('apply.daterangepicker', (ev, picker) => {
            let {
                    applyAction
                } = self.props;
            let start = picker.startDate;
            let end = picker.endDate;

            if (applyAction != null) {
                if (typeof applyAction === 'function') {
                    applyAction(start.toDate(), end.toDate());
                }
            }
        });
        this.dateRangePicker.off('cancel.daterangepicker').on('cancel.daterangepicker', (ev, picker) => {
            let {
                    applyAction,
                    enableClear
                } = self.props;

            if (applyAction != null && enableClear) {
                if (typeof applyAction === 'function') {
                    applyAction(null, null);
                }
            }
        });
        let {onShowCalendar, onHideCalendar, onShow, onHide} = this.props;
        if ('function' === typeof onShowCalendar) {
            this.dateRangePicker.off('showCalendar.daterangepicker').on('showCalendar.daterangepicker', onShowCalendar);
        }
        if ('function' === typeof onHideCalendar) {
            this.dateRangePicker.off('hideCalendar.daterangepicker').on('hideCalendar.daterangepicker', onHideCalendar);
        }
        if ('function' === typeof onShow) {
            this.dateRangePicker.off('show.daterangepicker').on('show.daterangepicker', onShow);
        }
        if ('function' === typeof onHide) {
            this.dateRangePicker.off('hide.daterangepicker').on('hide.daterangepicker', onHide);
        }
    }

    clearValue() {
        let {applyAction} = this.props;
        if (typeof applyAction === 'function') {
            applyAction(null, null);
        }
    }

    setDatePickerData(props) {
        let {
                start,
                end
            } = props;
        this.dateRangePicker.data('daterangepicker').setStartDate(start);
        this.dateRangePicker.data('daterangepicker').setEndDate(end);
    }

    componentDidMount() {
        this.initDateRangePicker();
    }

    shouldComponentUpdate(nextProps, nextState) {
        this.setDatePickerData(nextProps);
        return true;
    }

    componentDidUpdate(prevProps, prevState) {
        if (prevProps.singleDatePicker !== this.props.singleDatePicker ||
            prevProps.timePicker !== this.props.timePicker) {
            this.removeDateRangePicker();
            this.initDateRangePicker();
        }
    }

    initDateRangePicker() {
        this.dateRangePicker = $(this.dateRangePickerElement);
        this.dateRangePicker.on('click', '.icon-close-circle', (e) => {
            e.stopPropagation();
            this.clearValue();
        });
        this.dateRangePicker.daterangepicker(this.getOptions(), this.handleCallback);
        this.attachPickerEvents();
        this.setDatePickerData(this.props);
        if (typeof this.props.onCreated === 'function') {
            this.props.onCreated();
        }
    }

    removeDateRangePicker() {
        let dateRangePicker = this.dateRangePicker.data('daterangepicker');
        if (dateRangePicker != null) {
            this.dateRangePicker.off('apply.daterangepicker');
            this.dateRangePicker.off('cancel.daterangepicker');
            this.dateRangePicker.off('showCalendar.daterangepicker');
            this.dateRangePicker.off('hideCalendar.daterangepicker');
            this.dateRangePicker.off('show.daterangepicker');
            this.dateRangePicker.off('hide.daterangepicker');

            if (typeof dateRangePicker.remove === 'function') {
                dateRangePicker.remove();
            }

        }
        this.dateRangePicker.off();
        this.dateRangePicker = null;
    }

    componentWillUnmount() {
        this.removeDateRangePicker();
        this.dateRangePickerElement = null;
    }

    getOptions() {
        let {
                start,
                end,
                serverFormat,
                minDate,
                maxDate,
                alwaysShowCalendars,
                autoUpdateInput,
                autoApply,
                applyLabel,
                cancelLabel,
                customRangeLabel,
                fromLabel,
                toLabel,
                format,
                firstDay,
                daysOfWeek,
                monthNames,
                timePicker,
                buttonClasses,
                applyClass,
                cancelClass,
                separator,
                singleDatePicker,
                drops,
                opens,
                timePicker24Hour,
                timePickerSeconds,
                timePickerIncrement,
                showWeekNumbers,
                showDropdowns,
                linkedCalendars,
                dateLimit,
                parentEl,
                enableClear,
                t = (resId) => resId
            } = this.props;

        if (!daysOfWeek || daysOfWeek.length < 1) {
            daysOfWeek = [
                t('date_range_picker.days_of_week.sun'),
                t('date_range_picker.days_of_week.mon'),
                t('date_range_picker.days_of_week.tu'),
                t('date_range_picker.days_of_week.we'),
                t('date_range_picker.days_of_week.th'),
                t('date_range_picker.days_of_week.fr'),
                t('date_range_picker.days_of_week.sa')
            ];
        }
        if (!monthNames || monthNames.length < 1) {
            monthNames = [
                t('date_range_picker.month_names.jan'),
                t('date_range_picker.month_names.feb'),
                t('date_range_picker.month_names.mar'),
                t('date_range_picker.month_names.apr'),
                t('date_range_picker.month_names.may'),
                t('date_range_picker.month_names.jun'),
                t('date_range_picker.month_names.jul'),
                t('date_range_picker.month_names.aug'),
                t('date_range_picker.month_names.sep'),
                t('date_range_picker.month_names.oct'),
                t('date_range_picker.month_names.nov'),
                t('date_range_picker.month_names.dec')
            ];
        }

        let options = {
            alwaysShowCalendars : alwaysShowCalendars,
            autoUpdateInput     : autoUpdateInput,
            autoApply           : autoApply,
            locale              : {
                applyLabel       : applyLabel || t('date_range_picker.apply_label'),
                cancelLabel      : cancelLabel ||
                                   (enableClear ? t('date_range_picker.empty_label') : t('date_range_picker.cancel_label')),
                customRangeLabel : customRangeLabel || t('date_range_picker.custom_range_label'),
                fromLabel        : fromLabel || t('date_range_picker.from_label'),
                toLabel          : toLabel || t('date_range_picker.to_label'),
                format           : format,
                firstDay         : firstDay,
                daysOfWeek       : daysOfWeek,
                monthNames       : monthNames
            },
            startDate           : start,
            endDate             : end,
            minDate             : minDate,
            maxDate             : maxDate,
            timePicker          : timePicker,
            buttonClasses       : buttonClasses,
            applyClass          : applyClass,
            cancelClass         : cancelClass,
            separator           : separator,
            singleDatePicker    : singleDatePicker,
            drops               : drops,
            opens               : opens,
            timePicker24Hour    : timePicker24Hour,
            timePickerSeconds   : timePickerSeconds,
            timePickerIncrement : timePickerIncrement,
            showWeekNumbers     : showWeekNumbers,
            showDropdowns       : showDropdowns,
            linkedCalendars     : linkedCalendars,
            dateLimit           : dateLimit,
            parentEl            : parentEl
        };

        if (!singleDatePicker) {
            let ranges = this.props.ranges;
            if (ranges === null) {
                ranges = {
                    [t('date_range_picker.ranges.today')]               : [moment().startOf('day'),
                                                                           moment().endOf('day')],
                    [t('date_range_picker.ranges.yesterday')]           : [moment().subtract(1, 'days').startOf('day'),
                                                                           moment().subtract(1, 'days').endOf('day')],
                    [t('date_range_picker.ranges.recently_seven_day')]  : [moment().subtract(6, 'days').startOf('day'),
                                                                           moment().endOf('day')],
                    [t('date_range_picker.ranges.recently_thirty_day')] : [moment().subtract(29, 'days').startOf('day'),
                                                                           moment().endOf('day')],
                    [t('date_range_picker.ranges.this_month')]          : [moment().startOf('month').startOf('day'),
                                                                           moment().endOf('month').endOf('day')],
                    [t('date_range_picker.ranges.last_month')]          : [moment().subtract(1, 'month')
                                                                                   .startOf('month')
                                                                                   .startOf('day'),
                                                                           moment().subtract(1, 'month').endOf('month')
                                                                                   .endOf('day')],
                    [t('date_range_picker.ranges.this_year')]           : [moment().startOf('year').startOf('day'),
                                                                           moment().endOf('year').endOf('day')],
                    [t('date_range_picker.ranges.last_year')]           : [moment().subtract(1, 'year').startOf('year')
                                                                                   .startOf('day'),
                                                                           moment().subtract(1, 'year').endOf('year')
                                                                                   .endOf('day')],
                    [t('date_range_picker.ranges.recently_year')]       : [moment().subtract(1, 'year'), moment()]
                };
            }
            options.ranges = ranges;
        }
        options = Object.assign({}, options);

        return options;
    }

    render() {
        let {
                className,
                format,
                start,
                end,
                separator,
                singleDatePicker,
                calendarIcon,
                enableClear
            } = this.props;

        let momentStartDate;
        let momentEndDate;
        let startDate = '';
        let endDate = '';

        if (start) {
            if (typeof start === 'string') {
                momentStartDate = moment(start, format);
            } else {
                momentStartDate = moment(start);
            }
            startDate = momentStartDate.isValid() ? momentStartDate.format(format) : undefined;
        }
        if (end) {
            if (typeof end === 'string') {
                momentEndDate = moment(end, format);
            } else {
                momentEndDate = moment(end);
            }
            endDate = momentEndDate.isValid() ? momentEndDate.format(format) : undefined;
        }

        let rangeText = '';
        if (singleDatePicker) {
            if (startDate) {
                rangeText = startDate;
            }
        } else {
            if (startDate || endDate) {
                rangeText = [startDate, endDate].join(separator);
            }
        }

        return (
            <div ref={(dateRangePickerElement) => this.dateRangePickerElement = dateRangePickerElement}
                 className={'date-range-picker flex-layout ' + className}>
                <span className='flex-1'>{rangeText}</span>
                {enableClear && rangeText &&
                 <i className='icon icon-close-circle' style={{marginLeft : '5px', cursor : 'pointer'}}/>}
                {calendarIcon && <i className="icon icon-calendar" style={{marginLeft : '5px'}}/>}
            </div>
        );
    }
}

DateRangePicker.propTypes = {
    t              : PropTypes.func,
    calendarIcon   : PropTypes.bool,
    start          : PropTypes.oneOfType([PropTypes.string, PropTypes.instanceOf(Date)]),
    end            : PropTypes.oneOfType([PropTypes.string, PropTypes.instanceOf(Date)]),
    onShowCalendar : PropTypes.func,
    onHideCalendar : PropTypes.func,
    onShow         : PropTypes.func,
    onHide         : PropTypes.func
};

DateRangePicker.defaultProps = {
    start                   : undefined,
    end                     : undefined,
    className               : 'form-control input-sm',
    minDate                 : undefined,
    maxDate                 : undefined,
    timePicker              : false,
    timePicker24Hour        : true,
    timePickerSeconds       : false,
    timePickerIncrement     : 1,
    showWeekNumbers         : false,
    showDropdowns           : false,
    linkedCalendars         : false,
    datelimit               : false,
    parentEl                : 'body',
    format                  : 'YYYY-MM-DD',
    serverFormat            : 'YYYY-MM-DD',
    separator               : ' - ',
    singleDatePicker        : false,
    placeholder             : null,
    buttonClasses           : ['btn', 'btn-sm'],
    applyClass              : 'btn-success',
    cancelClass             : 'btn-default',
    drops                   : 'down',
    opens                   : 'right',
    daysOfWeek              : [],
    monthNames              : [],
    removeDropdownOnDestroy : false,
    cancelLabel             : '',
    applyLabel              : '',
    customRangeLabel        : '',
    fromLabel               : '',
    toLabel                 : '',
    autoUpdateInput         : false,
    autoApply               : false,
    alwaysShowCalendars     : false,
    firstDay                : 0,
    ranges                  : null,
    applyAction             : null,
    languageCode            : 'zh-cn',
    enableClear             : false,
    onCreated               : undefined
};

export default DateRangePicker;
