import React, { Component, createRef } from 'react';
import PropTypes from 'prop-types';
import CommonDropdown from './CommonDropdown';
import Control from './Control';

class MultiSelect extends Component {
    _ref = createRef();

    constructor(props) {
        super(props);
        this.state = {
            isOpen: false,
            options: []
        };
    }

    render() {
        let {
            controlClassName,
            controlStyle,
            prefix = '',
            options = [],
            getOptionLabel = v => JSON.stringify(v),
            getOptionValue = v => v,
            onChange,
            value,
            placeholder = '全部',
            t = v => v,
            loadOptions,
            components,
            showAllOption
        } = this.props;
        return (
            <div className="select-with-search" ref={this._ref}>
                <div className={'align-justify ' + controlClassName} style={controlStyle} onClick={this.toggleDropdown}>
                    <components.Control value={value} options={options} prefix={prefix}
                        getOptionLabel={getOptionLabel} getOptionValue={getOptionValue}
                        showAllOption={showAllOption}
                        placeholder={placeholder}
                        isOpen={this.state.isOpen}
                        t={t} />
                </div>
                {do {
                    if (this.state.isOpen) {
                        if (!loadOptions) {
                    <CommonDropdown value={value}
                        onChange={onChange}
                        getOptionValue={getOptionValue}
                        getOptionLabel={getOptionLabel}
                        t={t}
                        showAllOption={showAllOption}
                        options={options} />;
                        }
                    }
                }}
            </div>
        );
    }

    componentDidMount() {
        document.addEventListener('mousedown', this.handleClickOutside);
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClickOutside);
    }

    toggleDropdown = () => {
        this.setState({ isOpen: !this.state.isOpen });
    };

    handleClickOutside = (event) => {
        if (this._ref.current && !this._ref.current.contains(event.target)) {
            this.setState({ isOpen: false });
        }
    };

}

MultiSelect.propTypes = {
    controlClassName: PropTypes.string,
    controlStyle: PropTypes.object,
    prefix: PropTypes.string,
    value: PropTypes.arrayOf(PropTypes.any),
    options: PropTypes.arrayOf(PropTypes.any),
    loadOptions: PropTypes.func,
    getOptionLabel: PropTypes.func,
    getOptionValue: PropTypes.func,
    onChange: PropTypes.func,
    placeholder: PropTypes.string,
    components: PropTypes.object
};

const defaultComponents = {
    Control: Control
};

MultiSelect.defaultProps = {
    components: defaultComponents,
    controlClassName: 'form-control'
};

export default MultiSelect;

export const components = defaultComponents;
