/*eslint no-unused-vars: ["error", { "argsIgnorePattern": "popContentStyle" }]*/
import React, { Component, createRef } from 'react';
import PropTypes from 'prop-types';
import Popover from '@udesk-webapp/popover';

class TextDropdown extends Component {
    constructor(props) {
        super(props);
    }

    render() {
        let { onChange, t, ...props } = this.props;
        return (
            <Popover
                {...props}
                components={{
                    Control: ({ 
                        value, 
                        prefix, 
                        controlClassName,
                        controlStyle,
                        placeholder, 
                        isOpen, 
                        popContentStyle, 
                        isBlank, 
                        allowBlank,
                        isNotBlank,
                        allowNotBlank,
                        ...props 
                    }) => {
                        let title = value || placeholder;
                        if((allowBlank && isBlank) && (allowNotBlank && isNotBlank)) {
                            title = placeholder;
                        }
                        if(allowBlank && isBlank && !isNotBlank) {
                            title = t('filter.operator.blank');
                        }
                        if(allowNotBlank && isNotBlank && !isBlank) {
                            title = t('filter.operator.present');
                        }
                        return (
                            <div
                                className={controlClassName}
                                style={controlStyle}
                                {...props}
                            >
                                <span>
                                    {prefix}{` ${title}`}
                                </span>
                                <i className={`icon icon-caret-${isOpen ? 'up' : 'down'}`} />
                            </div>
                        );
                    },
                    PopContent: ({ closePopContent, ...props }) => {
                        return (
                            <PopContent
                                {...props}
                                t={t}
                                onChange={(v, isBlank, isNotBlank) => {
                                    onChange(v, isBlank, isNotBlank);
                                    closePopContent();
                                }}
                            />
                        );
                    }
                }}
            />
        );
    }
}

class PopContent extends Component {
    _ref = createRef();

    state = {
        value: this.props.value,
        isBlank: this.props.isBlank,
        isNotBlank: this.props.isNotBlank,
        changeFromState: false
    };

    static getDerivedStateFromProps(props, state) {
        if (state.changeFromState) {
            return {
                value: state.value,
                isBlank: state.isBlank,
                isNotBlank: state.isNotBlank,
                changeFromState: false
            };
        } else if (props.value !== state.value || props.isBlank !== state.isBlank || props.isNotBlank !== state.isNotBlank) {
            return {
                value: props.value,
                isBlank: props.isBlank,
                isNotBlank: props.isNotBlank,
                changeFromState: false
            };
        }

        return null;
    }

    render() {
        let { onChange, allowBlank, allowNotBlank, t, ...props } = this.props;
        let { isBlank, isNotBlank, value } = this.state;
        return <div style={props.popContentStyle}>
            <div className="form-group">
                <input className="form-control" value={value} ref={this._ref}
                    onKeyDown={this.onInputKeyDown}
                    disabled={
                        ((allowBlank && isBlank) || (allowNotBlank && isNotBlank)) 
                            ? true
                            : false
                    }
                    onChange={(e) => {
                        // if (allowBlank) {
                        //     this.setState({ value: e.target.value, isBlank: !!isBlank, changeFromState: true });
                        // } else {
                        //     this.setState({ value: e.target.value, changeFromState: true });
                        // }
                        if (allowBlank && allowNotBlank) {
                            this.setState({ 
                                value: e.target.value, 
                                isBlank: !!isBlank,
                                isNotBlank: !!isNotBlank,
                                changeFromState: true 
                            });
                        } else if(allowBlank && !allowNotBlank) {
                            this.setState({ 
                                value: e.target.value, 
                                isBlank: !!isBlank,
                                changeFromState: true 
                            });
                        } else if(allowNotBlank && !allowBlank) {
                            this.setState({ 
                                value: e.target.value, 
                                isNotBlank: !!isNotBlank,
                                changeFromState: true 
                            });
                        } else {
                            this.setState({ 
                                value: e.target.value,
                                changeFromState: true 
                            });
                        }
                    }} />
            </div>
            {do {
                if (allowBlank) {
                <div className='checkbox'>
                    <label>
                        <input type='checkbox' checked={isBlank}
                            onChange={(e) => this.setState({
                                isBlank: e.target.checked, 
                                changeFromState: true
                            })} /> {t('filter.operator.blank')}
                    </label>
                </div>;
                }
            }}
            {do {
                if (allowNotBlank) {
                    <div className='checkbox'>
                        <label>
                            <input 
                                type='checkbox' 
                                checked={isNotBlank}
                                onChange={(e) => this.setState({
                                    isNotBlank: e.target.checked, 
                                    changeFromState: true
                                })} 
                            /> 
                            {t('filter.operator.present')}
                        </label>
                    </div>;
                }
            }}
            <button 
                type='button' 
                onClick={() => onChange(value, isBlank, isNotBlank)}
                className='btn btn-default'
            >
                {t('label.confirm')}
            </button>
            <button type='button' className='btn btn-link' onClick={() => {
                // if (allowBlank) {
                //     this.setState({ 
                //         value: '', 
                //         isBlank: false, 
                //         changeFromState: true 
                //     });
                //     onChange('', false);
                // } else {
                //     this.setState({ value: '', changeFromState: true });
                //     onChange('');
                // }
                if(allowBlank && allowNotBlank) {
                    this.setState({ 
                        value: '', 
                        isBlank: false,
                        isNotBlank: false,
                        changeFromState: true 
                    });
                    onChange('', false, false);
                    return;
                }
                if(allowBlank) {
                    this.setState({ 
                        value: '', 
                        isBlank: false, 
                        changeFromState: true 
                    });
                    onChange('', false);
                    return;
                }
                if(allowNotBlank) {
                    this.setState({ 
                        value: '', 
                        isNotBlank: false, 
                        changeFromState: true 
                    });
                    onChange('', undefined, false);
                    return;
                }
                this.setState({
                    value: '', 
                    changeFromState: true 
                });
                onChange('');
            }}>
                {t('label.clear')}
            </button>
        </div>;
    }

    componentDidMount() {
        this._ref.current.focus();
    }

    onInputKeyDown = (e) => {
        if (e.key === 'Enter') {
            this.props.onChange(e.target.value);
        }
    };
}

PopContent.propTypes = {
    value: PropTypes.string,
    onChange: PropTypes.func
};

TextDropdown.propTypes = {
    t: PropTypes.func,
    controlClassName: PropTypes.string,
    onChange: PropTypes.func,
    value: PropTypes.string,
    /**
     * 是否显示“为空“选框
     */
    allowBlank: PropTypes.bool,
    /**
     * “为空“选框的选中状态
     */
    isBlank: PropTypes.bool,
    allowNotBlank: PropTypes.bool,//是否显示“不为空“选框
    isNotBlank: PropTypes.bool,//"不为空"选框的选中状态
};

TextDropdown.defaultProps = {
    className: 'select-with-search',
    controlClassName: 'align-justify form-control',
    value: '',
    popContentStyle: { padding: '15px', minWidth: '200px' }
};

export default TextDropdown;
