import React, { Component } from 'react';
import PropTypes            from 'prop-types';

class Popover extends Component {
    _ref = React.createRef();
    _doNotClosePopover = false;

    constructor(props) {
        super(props);
        this.state = {
            isOpen : false
        };
    }

    render() {
        let {components, style, className, ...props} = this.props;

        if (!components.Control) {
            components.Control = DefaultControl;
        }
        if (!components.PopContent) {
            components.PopContent = DefaultPopContent;
        }

        return (
            <div style={style} className={className} ref={this._ref}>
                <components.Control {...props} isOpen={this.state.isOpen} onClick={() => this.toggleOpen()}/>
                {this.state.isOpen && <components.PopContent {...props} isOpen={this.state.isOpen}
                                                             closePopContent={() => this.setState({isOpen : false})}/>}
            </div>
        );
    }

    componentDidMount() {
        this._ref.current.addEventListener('click', () => {
            this._doNotClosePopover = true;
        });
        document.body.addEventListener('click', this._bodyEvent = () => {
            if (this._doNotClosePopover) {
                this._doNotClosePopover = false;
                return;
            }
            this.setState({isOpen : false});
        });
    }

    componentWillUnmount() {
        document.body.removeEventListener('click', this._bodyEvent);
    }

    toggleOpen() {
        this.setState({
            isOpen : !this.state.isOpen
        });
    }
}

Popover.propTypes = {
    components : PropTypes.object
};

const DefaultControl = (props) => (<button type='button' {...props}>Default Control</button>);
const DefaultPopContent = (props) => (<div {...props}>Default Pop Content</div>);
Popover.defaultProps = {
    components : {
        Control    : DefaultControl,
        PopContent : DefaultPopContent
    }
};
export default Popover;
