import React, { Component }           from 'react';
import { Manager, Popper, Reference } from 'react-popper';
import ColumnControl                  from '@udesk-webapp/column-control';
import PropTypes                      from 'prop-types';
import assign from 'lodash-es/assign';

class ColumnControlPopover extends Component {
    constructor(props) {
        super(props);
        this.state = {isOpen : false};
        this._ref = React.createRef();
        this._doNotClosePopover = false;
    }

    render() {
        let managerClassNames = ['dropdown'];
        if (this.state.isOpen) {
            managerClassNames.push('open');
        }
        let {
                columns, cols = 1, t, onChange, className = '',
                showSearch,
                classNameOfSearchInput,
                classNameOfSearchInputWrapper,
                classNameOfSearchInputIcon,
                classNameOfRecoverDefaultWrapper,
                classNameOfColumnsWrapper,
                showRecoverDefault,
                onRecoverDefaultClick,
                styleOfColumnsWrapper,
                searchPlaceholder
            } = this.props;

        return (
            <div className={managerClassNames.join(' ') + ' ' + className} ref={this._ref}>
                <Manager>
                    <Reference>
                        {({ref}) => (
                            //@formatter:off
                            <button type      = "button"
                                    className = "btn btn-default"
                                    ref       = {ref}
                                    onClick   = {() => this._toggle()}
                            >
                                {t('label.column')} <i className="icon icon-caret-down"/>
                            </button>
                            //@formatter:on
                        )}
                    </Reference>
                    {this.state.isOpen && <Popper placement="bottom-end">
                        {({ref, style, placement}) => (
                            <div ref={ref} style={assign({zIndex : 1000000}, style)} data-placement={placement}>
                                {/*@formatter:off*/}
                                <ColumnControl
                                    style                                    = {{
                                                                                 backgroundColor : '#fff',
                                                                                 border          : '1px solid #e1e1e1',
                                                                                 width           : cols * 165 + 20
                                                                               }}
                                    columns                                  = {columns}
                                    cols                                     = {cols}
                                    onChange                                 = {onChange}
                                    t                                        = {t}
                                    showSearch                               = {showSearch}
                                    classNameOfSearchInput                   = {classNameOfSearchInput}
                                    classNameOfSearchInputWrapper            = {classNameOfSearchInputWrapper}
                                    classNameOfSearchInputIcon               = {classNameOfSearchInputIcon}
                                    classNameOfRecoverDefaultWrapper         = {classNameOfRecoverDefaultWrapper}
                                    classNameOfColumnsWrapper                = {classNameOfColumnsWrapper}
                                    showRecoverDefault                       = {showRecoverDefault}
                                    onRecoverDefaultClick                    = {onRecoverDefaultClick}
                                    styleOfColumnsWrapper                    = {styleOfColumnsWrapper}
                                    searchPlaceholder                        = {searchPlaceholder}
                                />
                                {/*@formatter:on*/}
                            </div>
                        )}
                    </Popper>}
                </Manager>
            </div>
        );
    }

    componentDidMount() {
        this._ref.current.addEventListener('click', () => {
            this._doNotClosePopover = true;
        });
        document.addEventListener('click', this._bodyEvent = () => {
            if (this._doNotClosePopover) {
                this._doNotClosePopover = false;
                return;
            }
            this._close();
        });
    }

    componentWillUnmount() {
        document.removeEventListener('click', this._bodyEvent);
    }

    _close() {
        if (this.state.isOpen) {
            let {onClose} = this.props;
            onClose && onClose();
            this.setState({isOpen : false});
        }
    }

    _open() {
        let {onOpen} = this.props;
        onOpen && onOpen();
        this.setState({isOpen : true});
    }

    _toggle() {
        if (this.state.isOpen) {
            this._close();
        } else {
            this._open();
        }
    }
}

ColumnControlPopover.propTypes = {
    columns                          : PropTypes.array,
    cols                             : PropTypes.number,
    onChange                         : PropTypes.func,
    className                        : PropTypes.string,
    t                                : PropTypes.func,
    showSearch                       : PropTypes.bool,
    classNameOfSearchInput           : PropTypes.string,
    classNameOfSearchInputWrapper    : PropTypes.string,
    classNameOfSearchInputIcon       : PropTypes.string,
    classNameOfRecoverDefaultWrapper : PropTypes.string,
    classNameOfColumnsWrapper        : PropTypes.string,
    showRecoverDefault               : PropTypes.bool,
    onRecoverDefaultClick            : PropTypes.func,
    styleOfColumnsWrapper            : PropTypes.object,
    onOpen                           : PropTypes.func,
    onClose                          : PropTypes.func,
    searchPlaceholder                : PropTypes.string
};

export default ColumnControlPopover;
