import React, {
    ElementRef,
    useCallback,
    useEffect,
    // useRef,
    useMemo,
} from 'react';
import { Provider } from 'react-redux';
import useLocalStorage from 'react-use/esm/useLocalStorage';
import { UdeskContext, useDispatch } from './store/store-hook';
import { store } from './store/index';
import { formatProps, staticData } from './utils/index';
import { setRUrls } from './store/urls';
import { setCompany } from './store/company';
import { setRoleCan } from './store/role-can';
import { setInitData } from './store/init-data';
import { setCurrentUser } from './store/current-user';
import { setAgents } from './store/agents';
import { setGroups } from './store/groups';
import { setFields } from './store/fields';
import { setNoteTemplateConfig } from './store/note-templates';
import { setPlatform } from './store/platform';
import { getAgentConfig } from './store/agent-config';
import ImPanel from './im-panel/im-entry';
import './index.module.scss';
import { ConfigProvider } from 'antd';
// const MESSAGES_MIN_WIDTH = 395;

const SessionAssist = React.forwardRef<ElementRef<typeof ImPanel>, ImContextProps>((props: ImContextProps, ref) => {
    // const imPanelEntryRef = useRef<ElementRef<typeof ImPanel>>(null);
    // const imMessageRef = useRef<ElementRef<"div">>(null);
    // const rightInnerRef = useRef<ElementRef<"div">>(null);

    const dispatch = useDispatch();
    const [rightWidth = 400, setRightWidth] = useLocalStorage('stretchable-box-right-panel-width', 400, {
        raw: false,
        serializer: (value) => value + '',
        deserializer: (value: string) => parseInt(value) || 370,
    });

    const initData = useMemo(() => {
        console.log('初始化开始');
        staticData.changeRobotUrl(props.kmrobot_url);
        staticData.changePreviewUrl(props.previewUrl);

        const _props = formatProps(props);

        dispatch(setCompany(_props.company));
        dispatch(setRoleCan(_props.roleCan));
        dispatch(setCurrentUser(_props.user));
        dispatch(setAgents(_props.agents));
        dispatch(setGroups(_props.agentGroups));
        dispatch(setInitData(_props)); //props所有数据
        dispatch(
            setRUrls({
                apiHost: _props.apiHost,
                vistor_server: _props.vistor_server,
            })
        );
        dispatch(
            setFields({
                customerFields: _props.customerFields,
                customerFieldsv2: _props.customerFieldsv2,
                organizationFields: _props.organizationFields,
                organizationFieldsv2: _props.organizationFieldsv2,
                noteFields: _props.noteFields,
            })
        );
        dispatch(
            setNoteTemplateConfig({
                noteTemplates: _props.noteTemplates || [],
                imAgentNoteTemplatesSort: _props.imAgentNoteTemplatesSort || [],
            })
        );
        dispatch(setPlatform(_props.platforms));
        dispatch(getAgentConfig() as any);
        return;
    }, []);

    useEffect(() => {
        const _props = formatProps(props);
        dispatch(setInitData(_props));
    }, [props]);
    const onRightWidthChange = useCallback(
        (changedWidth) => {
            // const messageWidth = imMessageRef?.current?.getWidth?.();
            // if (
            //   messageWidth &&
            //   messageWidth < MESSAGES_MIN_WIDTH &&
            //   changedWidth > rightWidth
            // ) {
            //   const rightInnerWidth = rightInnerRef.current!.offsetWidth;
            //   return setRightWidth(rightInnerWidth - MESSAGES_MIN_WIDTH);
            // } else {
            props.event.onWidthChange(changedWidth);
            return setRightWidth(changedWidth);
            // }
        },
        [rightWidth, setRightWidth]
    );

    return (
        <ImPanel
            ref={ref}
            width={rightWidth}
            cssMaxWidth={`calc(100% - ${props.tabsWidth}px)`}
            onWidthChange={onRightWidthChange}
        />
    );
});
const Index = React.forwardRef<ElementRef<typeof SessionAssist>, ImContextProps>((props: ImContextProps, ref) => {
    return (
        <ConfigProvider prefixCls='udesk-avs-web-prefix'>
            <Provider context={UdeskContext} store={store}>
                <SessionAssist {...props} ref={ref} />
            </Provider>
        </ConfigProvider>
       
    );
});
Index.displayName = 'Index';
export default Index;
