// @ts-nocheck
/** @jsx jsx */
import {jsx} from '@emotion/react';
import {Component, createRef, CSSProperties, ElementRef} from 'react';
import {Manager, Popper, Reference} from 'react-popper';
import assign from 'lodash-es/assign';
import {popper} from './style';
import {Column, ValueType} from "./types";
import CheckList from './CheckList';
import contentStyle from './style';

type Props =
    {
        className: string
        columnButtonClassName: string
        columnButtonStyle: CSSProperties
        columns: Column[]
        value?: ValueType
        t: (string) => string
        onClose: () => void
        onOpen: () => void
        onChange?: (value: ValueType) => void
    }

interface State {
    isOpen: boolean,
    checkedColumns: Array<string>;
}

class ColumnControlPopover extends Component<Props, State> {
    private _ref = createRef<ElementRef<'div'>>();
    private _doNotClosePopover = false;

    constructor(props) {
        super(props);
        this.state = {
            isOpen: false,
            checkedColumns: this.props.value || []
        };
    }

    handleCheckChange(checked, column) {
        console.log(checked, column);
    }

    render() {
        let managerClassNames = ['dropdown'];
        if (this.state.isOpen) {
            managerClassNames.push('open');
        }
        let {
            columns, t, className = '',
            value=[],
            columnButtonClassName,
            columnButtonStyle,
        } = this.props;

        const defaultBtnStyle =  {};  

        return (
            <div className={managerClassNames.join(' ') + ' ' + className} ref={this._ref}>
                <Manager>
                    <Reference>
                        {({ref}) => (
                            <button type="button"
                                    className={`${columnButtonClassName} table-column-control-btn`}
                                    style={Object.assign(defaultBtnStyle, columnButtonStyle)}
                                    ref = {ref}
                                    onClick= {() => this._toggle()}>
                                {t('label.column')} <i className="icon icon-caret-down"/>
                            </button>
                        )}
                    </Reference>
                    {this.state.isOpen && <Popper placement="bottom-end">
                        {({ref, style, placement}) => (
                            <div 
                                ref={ref} 
                                style={assign({zIndex: 1000000}, style)}
                                css={popper}
                                data-placement={placement}
                            >
                                <div css={contentStyle}>
                                    <CheckList
                                        columns = {columns}
                                        value = {value}
                                        onChange={this._onChange}
                                    >

                                    </CheckList>
                                </div>
                            </div>
                        )}
                    </Popper>}
                </Manager>
            </div>
        );
    }

    private _bodyEvent = () => {
        if (this._doNotClosePopover) {
            this._doNotClosePopover = false;
            return;
        }
        this._close();
    }

    componentDidMount() {
        if (this._ref.current) {
            this._ref.current.addEventListener('click', () => {
                this._doNotClosePopover = true;
            });
            document.addEventListener('click', this._bodyEvent);
        }
    }

    componentWillUnmount() {
        document.removeEventListener('click', this._bodyEvent);
    }

    _close() {
        if (this.state.isOpen) {
            let {onClose} = this.props;
            onClose && onClose();
            this.setState({isOpen: false});
        }
    }

    _open() {
        let {onOpen} = this.props;
        onOpen && onOpen();
        this.setState({isOpen: true});
    }

    _toggle() {
        if (this.state.isOpen) {
            this._close();
        } else {
            this._open();
        }
    }

    _onChange = (value) => {
        let {onChange} = this.props;
        if (typeof onChange === 'function') {
            onChange(value);
        }
    };

    static defaultProps = {
        columnButtonClassName: 'btn btn-default',
    };
}

export default ColumnControlPopover;
