import React, {ComponentProps, FC, useMemo} from "react";
import Select from "udesk-ui/es/select";

interface Staff {
    id: number,
    nick_name: string,
    agent_name_pinyin?: string
}

type Props = { staffs: Staff[] } & ComponentProps<typeof Select>;

type OptionType = (Staff & { value: Staff['id'], label: Staff['nick_name'] });

const filterOption: ComponentProps<typeof Select>['filterOption'] = (inputValue, option) => option?.nick_name.indexOf(inputValue) > -1 || option?.agent_name_pinyin.indexOf(inputValue) > -1;

const StaffSelect: FC<Props> = ({staffs, ...props}) => {
    const options = useMemo<OptionType[]>(() => staffs?.map(i => ({
        id: i.id,
        nick_name: i.nick_name,
        agent_name_pinyin: i.agent_name_pinyin,
        value: i.id,
        label: i.nick_name
    })) ?? [], [staffs]);

    return <Select {...props}
                   filterOption={filterOption}
                   options={options}/>
};

StaffSelect.defaultProps = {
    showSearch: true,
    allowClear: true,
    dropdownMatchSelectWidth: false
};

export default StaffSelect;
