import * as React from 'react';
import { useEffect, useState } from 'react';
import { Checkbox, Button } from 'udesk-ui';
import {
  HeaderWrap,
  HeaderWrapAllCheck,
  HeaderWrapExtraButton,
  HeaderWrapCheckAllText,
  HeaderWrapAllCheckInfo,
  HeaderWrapAllCheckInfoNumber,
} from './components/HeaderWrap';
import { useLocaleReceiver } from '../../../locale-provider/LocaleReceiver';

interface checkAllComponentTypes {
  checkedList: any[];
  handleCheckAllChange: (item: any) => void;
  handleCancelSelect: () => void;
  dataSource: any[];
  itemDisable?: (item: any) => boolean;
}

const CheckAllComponent = (props: checkAllComponentTypes) => {
  const { checkedList, handleCheckAllChange, dataSource, handleCancelSelect, itemDisable } = props;
  const [isIndeterminate, setIsIndeterminate] = useState(false);
  const [isCheckAll, setIsCheckAll] = useState(false);
  const [contextLocale] = useLocaleReceiver('ListCheckboxHeader');

  useEffect(() => {
    if (!checkedList.length) {
      setIsIndeterminate(false);
      setIsCheckAll(false);
      return;
    }
    let checkedIds = checkedList?.map((el: any) => el.id);
    let len = 0;
    let disableLen = 0;
    dataSource.forEach((el: any) => {
      if (checkedIds.includes(el.id)) {
        if (typeof itemDisable === 'function' && !itemDisable?.(el)) {
          disableLen += 1;
        }
        len += 1;
      }
    });
    if (len === 0) {
      setIsIndeterminate(false);
      setIsCheckAll(false);
    } else if (len === dataSource.length) {
      setIsIndeterminate(false);
      setIsCheckAll(true);
    } else {
      if (typeof itemDisable === 'function') {
        let currDisableLen = dataSource.filter((el: any) => !itemDisable?.(el))?.length;
        //len 判断了0的情况 这里就无须再次判断
        if (disableLen === currDisableLen) {
          setIsIndeterminate(false);
          setIsCheckAll(true);
        } else {
          setIsIndeterminate(true);
          setIsCheckAll(false);
        }
      } else {
        setIsIndeterminate(true);
        setIsCheckAll(false);
      }
    }
  }, [dataSource, checkedList]);

  return (
    <HeaderWrapAllCheck>
      <Checkbox
        disabled={dataSource?.every((el: any) => itemDisable?.(el))}
        indeterminate={isIndeterminate}
        onChange={handleCheckAllChange}
        checked={isCheckAll}
      />
      <HeaderWrapCheckAllText>{contextLocale.lang.selectAll}</HeaderWrapCheckAllText>
      {checkedList.length ? (
        <>
          <HeaderWrapAllCheckInfo>
            {contextLocale.lang.selected}
            <HeaderWrapAllCheckInfoNumber>{checkedList.length}</HeaderWrapAllCheckInfoNumber>
            {contextLocale.lang.term}
          </HeaderWrapAllCheckInfo>
          <Button className="allCheck-cancel" type="link" onClick={handleCancelSelect}>
            {contextLocale.lang.deselect}
          </Button>
        </>
      ) : null}
    </HeaderWrapAllCheck>
  );
};

const CheckAllComponentMemo = React.memo(CheckAllComponent);

interface ListHeaderProps {
  dataSource: any[];
  checkedList: any[];
  extraButton: React.ReactNode;
  handleCheckAllChange: (item: any) => void;
  handleCancelSelect: () => void;
  itemDisable?: (item: any) => boolean;
}

const ListHeader = (props: ListHeaderProps) => {
  const {
    checkedList,
    handleCheckAllChange,
    handleCancelSelect,
    itemDisable,
    dataSource,
    extraButton,
  } = props;

  return (
    <>
      <HeaderWrap checked={!!checkedList.length}>
        <CheckAllComponentMemo
          checkedList={checkedList}
          handleCheckAllChange={handleCheckAllChange}
          dataSource={dataSource}
          handleCancelSelect={handleCancelSelect}
          itemDisable={itemDisable}
        />
        <HeaderWrapExtraButton>{extraButton}</HeaderWrapExtraButton>
      </HeaderWrap>
    </>
  );
};

export default ListHeader;
