## ListCheckbox

#### 基础用法

Demo:

```tsx
import React, { useState, useEffect } from 'react';
import { ListCheckbox } from 'udesk_gm_ui';

const Demo = () => {
  const [currSelect, setCurrSelect] = useState<any[]>([]);

  let listData: any[] = [];

  for (let i = 0; i < 6; i++) {
    listData.push({
      id: i,
      title: `udesk-ui part ${i}`,
      description: 'udesk-ui, a design language for background applications, is refined by cheng.',
    });
  }

  const handleRenderItem = (item: any) => {
    return (
      <>
        {item.title}
        <p> {item.description}</p>
      </>
    );
  };

  const handleOnChange = (checkedList: any[]) => {
    console.log('checkedList', checkedList);
  };

  const handleItemDisable = (item: any) => {
    return item?.id === 2;
  };

  useEffect(() => {
    setCurrSelect(listData?.slice(0, 2));
  }, []);

  return (
    <div style={{ background: '#fff', borderRadius: '4px', padding: 12 }}>
      <ListCheckbox
        onChange={handleOnChange}
        dataSource={listData}
        value={currSelect}
        itemDisable={handleItemDisable}
        renderItem={handleRenderItem}
      />
    </div>
  );
};

export default Demo;
```

#### 使用 ref,自定义文字图标以及 extraButton

```tsx
import React, { useRef, useEffect } from 'react';
import { ListCheckbox } from 'udesk_gm_ui';

const Demo = () => {
  let listRef = useRef<any>({});

  let listData: any[] = [];

  for (let i = 0; i < 6; i++) {
    listData.push({
      id: i,
      title: `udesk-ui part ${i}`,
      description: 'udesk-ui, a design language for background applications, is refined by cheng.',
    });
  }

  const handleRenderItem = (item: any) => {
    return (
      <>
        {item.title}
        <p> {item.description}</p>
      </>
    );
  };

  const handleBtnClick = () => {
    listRef?.current?.resetSelect?.();
  };
  const handleOnChange = (checkedList: any[]) => {
    console.log('checkedList', checkedList);
  };

  useEffect(() => {
    console.log('listRef', listRef);
  }, []);

  const ExtraButton = () => (
    <>
      <span>DIY SPACE</span>
    </>
  );

  return (
    <div style={{ background: '#fff', borderRadius: '4px', padding: 12 }}>
      <span
        style={{
          display: 'inline-block',
          padding: '6px 12px',
          cursor: 'pointer',
          background: '#1A6EFF',
        }}
        onClick={handleBtnClick}
      >
        reset button
      </span>
      <ListCheckbox
        ref={listRef}
        onChange={handleOnChange}
        dataSource={listData}
        extraButton={<ExtraButton />}
        renderItem={handleRenderItem}
      />
    </div>
  );
};

export default Demo;
```

## API

| 参数        | 说明                       | 类型                    | 默认值 | 版本 |
| ----------- | -------------------------- | ----------------------- | ------ | ---- |
| value       | 选中项                     | item[]                  | -      |      |
| onChange    | 返回选中项                 | function(checkedList){} | -      |      |
| extraButton | 自定义右上角组件           | ReactNode               | []     |      |
| itemDisable | 函数返回 true 则对应项禁用 | (item)=>boolean         | false  |      |

其余 api 与`Udesk-ui`的`List`组件保持一致

## ListCheckboxInstance

| 参数        | 说明       | 类型       | 默认值 | 版本 |
| ----------- | ---------- | ---------- | ------ | ---- |
| resetSelect | 重置选中项 | () => void | -      |      |
