/** @jsx jsx */
import {jsx} from '@emotion/react';
import {Component, ComponentProps, createRef, ElementRef, forwardRef, HTMLProps} from 'react';
import {Column, ValueType} from "./types";

type Props = {
    content: Column[],
    className: string,
    onChange: (value: string[]) => void,
    value: ValueType,
    t: (value: string) => string,
    scrollbarFollow?: boolean
}

type State = {
    selected: Column | null
}

class List extends Component<Props, State> {
    ref = createRef<ElementRef<'select'>>();

    constructor(props: Props) {
        super(props);
        this.state = {
            selected: null
        };
    }

    render() {
        let {
            content,
            className,
            value,
            t
        } = this.props;
        return (
            <select ref={this.ref} multiple={true} className={className || ''} value={value as any}
                    onChange={this._onChange}>
                {content.map((item, index) => (
                        <option
                            value={item.id}
                            key={index}>
                            {item.name || t(item.id + '')}
                        </option>
                ))}
            </select>
        );
    }

    _onChange: HTMLProps<HTMLSelectElement>['onChange'] = (e) => {
        let element = e.currentTarget;
        let selectedOptions = element.selectedOptions;
        let selectedValues: string[] = [];
        for (let i = 0, len = selectedOptions.length; i < len; i++) {
            selectedValues.push(selectedOptions[i].value);
        }
        this.props.onChange(selectedValues);
    };

    scrollToSelectedItem() {
        if (this.ref?.current?.selectedOptions && this.ref.current.selectedOptions?.length > 0) {
            const middleOptionOffsetTop = this.ref.current.selectedOptions[Math.floor(this.ref.current.selectedOptions.length / 2)].offsetTop;

            this.ref.current.scrollTo(0, middleOptionOffsetTop - this.ref.current.clientHeight / 2);
        }
    }

    componentDidUpdate(prevProps) {
        if (this.props.scrollbarFollow &&
            this.props.value.length !== 0 &&
            this.props.value.length === prevProps.value.length &&
            this.props.value[0] === prevProps.value[0]) {
            if (this.props.content.findIndex(i => i.id === this.props.value[0]) !==
                prevProps.content.findIndex(i => i.id === prevProps.value[0])) {
                setTimeout(() => {
                    this.scrollToSelectedItem();
                }, 1);
            }
        }
    }

}

const ListForwardRef = forwardRef<ElementRef<typeof List>, ComponentProps<typeof List>>((props, ref) => (
    <List ref={ref} {...props}/>
));

export default List;
