const exec = require('child_process').exec;
const path = require('path');
const { requireOptions } = require('../require-options');
const executor = require('../executor');
const utils = require('../utils/option-utils');
const log = require('npmlog');

async function execGitDiff(commitHash, options) {
    const localesFilePath = options.localesFilePath || [];
    const showLog = options.showLog;

    exec(`git diff --name-status ${commitHash}`, 'utf8', function (err, changedFileList) {
        if (!err) {
            if (showLog) {
                log.info(commitHash);
                log.info(changedFileList);
            }
            // 修改类型分为三种['M', 'D', 'A']，其中D是删除，不处理
            const typeList = ['M', 'A'];
            const dictList = {
                M: '修改',
                D: '删除',
                A: '新增',
            };
            const typeListMap = new Map(typeList.map((t) => [t, t]));
            const typeListIter = typeListMap.entries();
            co(typeListIter);

            async function co(typeListIter) {
                const iter = typeListIter.next().value;
                if (iter) {
                    await run(iter[1], options);
                } else {
                    if (showLog) {
                        log.info(`================== all finish =================`);
                    }
                    return;
                }
                await co(typeListIter);
            }

            async function run(type, options) {
                const filePathList = getFilePathList(changedFileList, type).filter((p) => {
                    if (
                        localesFilePath &&
                        localesFilePath[0] &&
                        typeof localesFilePath[0] !== 'string'
                    ) {
                        return localesFilePath.every((l) => i.filePath !== p);
                    }
                    return !localesFilePath.includes(p);
                });
                if (showLog) {
                    log.info(`================== ${dictList[type]}文件 start =================`);
                    log.info(filePathList.join('\n'));
                }
                const list = new Map();
                filePathList.forEach((p, i) => list.set(i, p));
                const listIter = list.entries();
                await co(listIter);

                async function co(listIter) {
                    const iter = listIter.next().value;
                    if (iter) {
                        await executor.run(iter[1], options);
                    } else {
                        if (showLog) {
                            log.info(
                                `================== ${dictList[type]}文件 finish =================`
                            );
                        }
                        return;
                    }
                    await co(listIter);
                }
            }
        }
    });
}
async function main(commitHash, noEmit, showLog = true) {
    const options = Object.assign({ noEmit, showLog }, utils.formatOptions(requireOptions() || {}));

    if (!commitHash) {
        if (options && options.gitDiffBrancheName) {
            exec(
                `git log ${options.gitDiffBrancheName} -n 1 --pretty=format:"%H"
            `,
                'utf8',
                function (err, commitId) {
                    execGitDiff(commitId, options);
                }
            );
        } else {
            log.error(`请配置分支名称:gitDiffBrancheName或者提供commit id`);
        }
    } else {
        execGitDiff(commitHash, options);
    }
}

module.exports = main;

// 获取git改动清单
function getFilePathList(str, type) {
    const arr = str.split('\n');
    const replaceReg = new RegExp(`^[${type}]\t`);
    const regex = new RegExp(`^[${type}].*`);
    return arr
        .filter((item) => regex.test(item) && item !== undefined)
        .map((i) => i.replace(replaceReg, ''))
        .filter((i) => ['.ts', '.tsx', '.js', '.jsx'].includes(path.extname(i)));
}
