const path = require('path');
const utils = require('../utils/utils');
const readLocalesFile = require('../read-locales-file');
const core = require('../core');
const fs = require('fs-extra');
const merge = require('lodash.merge');

async function main(
    templateFilePath,
    outputFilePath,
    targetLanguage,
    translateVariableTemplate = false
) {
    if (!targetLanguage) {
        console.log('Error: 请输入目标语言');
        console.log('Error: 请输入目标语言');
        console.log('Error: 请输入目标语言');
        return;
    }
    if (!outputFilePath) {
        console.log('Error: 请输入输出文件路径');
        console.log('Error: 请输入输出文件路径');
        console.log('Error: 请输入输出文件路径');
        return;
    }
    const fileType = path.extname(templateFilePath).replace('.', '');
    const templateFileName = path.basename(templateFilePath);
    const outputFileName = path.basename(outputFilePath);
    const template = readLocalesFile({
        localesFilePath: [templateFilePath, outputFilePath],
        fileType,
    });
    let ret = merge({}, template[templateFileName], template[outputFileName]);
    let keysNeedManual = [];

    await walk(ret, targetLanguage, keysNeedManual, translateVariableTemplate);
    if (fileType === 'json') {
        fs.writeFileSync(outputFilePath, utils.formatCodeJSON(JSON.stringify(ret.locales)));
    } else {
        fs.writeFileSync(
            outputFilePath,
            utils.formatCode(`export default ${JSON.stringify(ret.locales)}`)
        );
    }
    console.log('完成');
    console.log('\n');
    if (keysNeedManual.length) {
        if (translateVariableTemplate) {
            console.log('已翻译，需要review↓');
            console.log('已翻译，需要review↓');
            console.log('已翻译，需要review↓');
        } else {
            console.log('需要手动翻译↓');
            console.log('需要手动翻译↓');
            console.log('需要手动翻译↓');
        }
        console.log(keysNeedManual.join('\n'));
    }
}

async function walk(obj, targetLanguage, keysNeedManual, translateVariableTemplate) {
    let stack = [obj];
    let requstQueue = [];
    while (stack.length) {
        let current = stack.shift();
        for (key of Object.keys(current)) {
            if (typeof current[key] !== 'string') {
                stack.push(current[key]);
            } else {
                if (
                    current[key] &&
                    core.chineseFindReg.test(current[key]) &&
                    !current[key].match(/\{/g)
                ) {
                    requstQueue.push(
                        (function (current, key) {
                            return async () => {
                                return core.bdTran(current[key], targetLanguage).then((resp) => {
                                    current[key] = resp;
                                });
                            };
                        })(current, key)
                    );
                } else if (current[key].match(/\{/g)) {
                    if (translateVariableTemplate) {
                        requstQueue.push(
                            (function (current, key) {
                                return async () => {
                                    return core
                                        .bdTran(current[key], targetLanguage)
                                        .then((resp) => {
                                            current[key] = resp;
                                        });
                                };
                            })(current, key)
                        );
                    }
                    keysNeedManual.push(`${key}: ${current[key]}`);
                }
            }
        }
    }
    function branchRequest(requstQueue) {
        if (!requstQueue.length) return;
        let current = requstQueue.splice(0, Math.min(9, requstQueue.length));
        const step = new Promise((res) => {
            setTimeout(res, 1000);
        });
        return Promise.all([...current.map((i) => i()), step]).then(async () => {
            await branchRequest(requstQueue);
        });
    }
    await branchRequest(requstQueue);
}

module.exports = main;

// 获取git改动清单
