## Demo

```tsx
/**
 * title: 基础使用
 * desc: 自适应大小的表格，默认在容器内滚动，可以设置autoSize字段关闭
 * background: 'rgb(240, 242, 245)'
 */

import React from 'react';
import { MemoTable } from 'udesk_gm_ui';
import { Tag, Space, Button } from 'udesk-ui';

const data = [];
for (let i = 1; i <= 100; i++) {
  if (i % 2) {
    data.push({
      key: i,
      name: 'John Brown',
      age: 32,
      address: 'New York No. 1 Lake Park',
      tags: ['nice', 'developer'],
    });
  } else {
    data.push({
      key: i,
      name: 'Joe Black',
      age: 32,
      address: 'Sidney No. 1 Lake Park',
      tags: ['cool', 'teacher'],
    });
  }
}

const originColumns = [
  {
    title: 'Name',
    dataIndex: 'name',
    key: 'name',
    render: (text) => <a>{text}</a>,
  },
  {
    title: 'Age',
    dataIndex: 'age',
    key: 'age',
  },
  {
    title: 'Address',
    dataIndex: 'address',
    key: 'address',
    width: 100,
    ellipsis: true,
  },
  {
    title: 'Tags',
    key: 'tags',
    dataIndex: 'tags',
    render: (tags) => (
      <>
        {tags.map((tag) => {
          let color = tag.length > 5 ? 'geekblue' : 'green';
          if (tag === 'loser') {
            color = 'volcano';
          }
          return (
            <Tag color={color} key={tag}>
              {tag.toUpperCase()}
            </Tag>
          );
        })}
      </>
    ),
  },
];

export default () => {
  const [dataSource, setDataSource] = React.useState([]);
  const columns = React.useMemo(() => {
    const res = [...originColumns];
    for (let i = 1; i < 50; i++) {
      res.push({
        title: `anyThinganyThinganyThinganyThing${i}`,
        dataIndex: `anyThing${i}`,
        key: `anyThing${i}`,
      });
    }
    res.push({
      title: 'Action',
      key: 'action',
      fixed: 'right',
      render: (text, record) => (
        <Space size="middle">
          <a>Invite {record.name}</a>
          <a>Delete</a>
        </Space>
      ),
    });
    return res;
  }, []);
  React.useEffect(() => {
    setTimeout(() => {
      setDataSource(data);
    }, 2000);
  }, []);

  const tableRef = React.useRef();
  const tableRef2 = React.useRef();
  return (
    <>
      <div style={{ color: '#000' }}>自动计算pagination高度</div>
      <Space>
        <Button
          style={{ marginTop: 16 }}
          onClick={() => tableRef2.current?.clearScrollPosition?.()}
        >
          清除滚动条位置数据
        </Button>
        <Button
          style={{ marginTop: 16 }}
          onClick={() => tableRef2.current?.scrollTo?.(1500, 500)}
        >{`手动滚动到{x: 1500, y: 500}位置`}</Button>
      </Space>
      <div style={{ height: 300, backgroundColor: '#f6ffed', marginTop: 8 }}>
        <MemoTable ref={tableRef2} columns={columns} dataSource={dataSource} cacheId="xxxxxxxx1" />
      </div>
      <Button style={{ marginTop: 16 }} onClick={() => tableRef.current?.clearScrollPosition?.()}>
        清除滚动条位置数据
      </Button>
      <div style={{ height: 300, backgroundColor: '#f6ffed', marginTop: 8 }}>
        <MemoTable
          ref={tableRef}
          columns={columns}
          dataSource={dataSource}
          pagination={false}
          cacheId="xxxxxxxx"
        />
      </div>
    </>
  );
};
```

```tsx
/**
 * title: 行多选
 * desc: rowSelectionEnhance字段对rowSelection字段进行了扩展，提供更便捷的行多选能力，注意：rowSelection具有更高的优先级，请不要一起使用
 * background: 'rgb(240, 242, 245)'
 */

import React from 'react';
import { MemoTable } from 'udesk_gm_ui';
import { Tag, Space, Button } from 'udesk-ui';

const originColumns = [
  {
    title: 'Name',
    dataIndex: 'name',
    key: 'name',
    render: (text) => <a>{text}</a>,
  },
  {
    title: 'Age',
    dataIndex: 'age',
    key: 'age',
  },
  {
    title: 'Address',
    dataIndex: 'address',
    key: 'address',
    width: 200,
  },
  {
    title: 'Tags',
    key: 'tags',
    dataIndex: 'tags',
    render: (tags) => (
      <>
        {tags.map((tag) => {
          let color = tag.length > 5 ? 'geekblue' : 'green';
          if (tag === 'loser') {
            color = 'volcano';
          }
          return (
            <Tag color={color} key={tag}>
              {tag.toUpperCase()}
            </Tag>
          );
        })}
      </>
    ),
  },
];

export default () => {
  const [dataSource, setDataSource] = React.useState([]);
  const [selectedRowKeys, setSelectedRowKeys] = React.useState([]);
  const [pagination, setPagination] = React.useState({
    pageSize: 10,
    current: 1,
  });
  const columns = React.useMemo(() => {
    const res = [...originColumns];
    res.push({
      title: 'Action',
      key: 'action',
      fixed: 'right',
      render: (text, record) => (
        <Space size="middle">
          <a>Invite {record.name}</a>
          <a>Delete</a>
        </Space>
      ),
    });
    return res;
  }, []);
  React.useEffect(() => {
    const data = [];
    for (let i = 1; i <= pagination.pageSize; i++) {
      const uid = `${(pagination.current - 1) * pagination.pageSize + i}`;
      if (i % 2) {
        data.push({
          key: uid,
          id: uid,
          name: uid,
          age: 32,
          address: 'New York No. 1 Lake Park',
          tags: ['nice', 'developer'],
        });
      } else {
        data.push({
          key: uid,
          id: uid,
          name: uid,
          age: 32,
          address: 'Sidney No. 1 Lake Park',
          tags: ['cool', 'teacher'],
        });
      }
    }
    setTimeout(() => {
      setDataSource(data);
    }, 500);
  }, [pagination]);

  return (
    <>
      <Space>
        <Button onClick={() => setSelectedRowKeys([])}>清除选中数据</Button>
        <span style={{ color: 'blue' }}>已选中：{selectedRowKeys.join(',')}</span>
      </Space>
      <div style={{ height: 300, backgroundColor: '#f6ffed', marginTop: 8 }}>
        <MemoTable
          columns={columns}
          dataSource={dataSource}
          pagination={{
            ...pagination,
            onChange: (current, pageSize) => {
              setPagination({
                current,
                pageSize,
              });
            },
            total: 200,
          }}
          rowSelectionEnhance={{
            selectedRowKeys, // 需要清空选中的场景可以传入selectedRowKeys使组件受控
            onSelectedRowKeysChange: (keys: string[], items: any[]) => {
              console.log(keys, items);
              setSelectedRowKeys(keys);
            },
          }}
        />
      </div>
    </>
  );
};
```

## API

| 参数 | 说明 | 类型 | 默认值 | 版本 |
| --- | --- | --- | --- | --- |
| autoSize | 是否启用自适应容器大小 | boolean | true |  |
| cacheId | 持久化 table 滚动数据的 id，需要全局唯一 | string | - |  |
| rowSelectionEnhance | `antd Table` 组件 `rowSelection` 字段的加强版，修改了部分属性，支持跨页多选，配置项见下 | object | - |  |

其余 api 与`Udesk-ui`的`Table`组件保持一致

### rowSelectionEnhance

选择功能的增强配置。删减了antd中 `rowSelection` 字段的部分属性，其余属性保持不变，删减属性名如下：

`onChange`, `onSelect`, `onSelectMultiple`, `onSelectAll`, `onSelectInvert`, `onSelectNone`, `preserveSelectedRowKeys`

增强、新加属性如下：

| 参数 | 说明 | 类型 | 默认值 | 版本 |
| --- | --- | --- | --- | --- |
| defaultSelectedRowKeys | 默认选中行的 key 的合集，只在初始化时生效 | string[] \| number[] | - |  |  
| selectedRowKeys | 选中行的 key 的合集，传入次字段时，变为受控组件 | string[] \| number[] | - |  |  
| rowKeysFieldName | `selectedRowKeys`以及`onSelectedRowKeysChange`第一个形参中元素对应到`dataSource`中的字段的字段名，默认为`id` | string | id |  |  
| onSelectedRowKeysChange | 选中行变化时触发的回调方法，手动设置`selectedRowKeys`不触发 | (keys: string[], items: any[]) => void | - |  |  

## Instance

| 参数                | 说明                     | 类型       | 默认值 | 版本 |
| ------------------- | ------------------------ | ---------- | ------ | ---- |
| clearScrollPosition | 清空浏览器缓存的位置信息 | () => void | -      |      |
| scrollTo            | 主动控制滚动条跳转位置   | () => void | -      |      |
