import React from 'react';
import { useEffect, useState } from 'react';
import { TableProps } from '../..';

const canvas = document.createElement('canvas');

interface UseColumnsFormatType<T> {
  (propsColumns: T): T;
}

const useColumnsFormat: UseColumnsFormatType<TableProps['columns']> = (propsColumns) => {
  const [columns, setColumns] = useState<TableProps['columns']>([]);
  useEffect(() => {
    setColumns(
      propsColumns?.map((column, index) => {
        const { title, sorter } = column;

        if (typeof title === 'string') {
          let res = {
            width: (getTextWidth(title) >>> 0) + 35 + (sorter ? 16 : 0),
            ...column,
          };

          if (column.ellipsis) {
            res.className = 'gm-ui-table-col-max-width-' + column.key
          }
          return res;
        }
        return column;
      }),
    );
  }, [propsColumns]);
  return columns;
};

export default useColumnsFormat;

function getTextWidth(text: string, font = '14px Microsoft YaHei') {
  let context: any = canvas.getContext('2d');
  context.font = font;
  let textMetrics = context.measureText(text);
  return textMetrics.width;
}
