import { useEffect } from 'react';
import { useStorage } from '../../../utils';
import { StorageManager } from '../../../utils/hooks/useStorage';

const MAX_LIMIT = 50;

type UseMemoScrollPositionArg = {
  cacheId: string;
  tableBodyPrefixCls: string;
  wrapRef: React.RefObject<HTMLDivElement>;
  dataSource: readonly any[] | undefined;
  columns: readonly any[] | undefined;
};
type UseMemoScrollPosition = (props: UseMemoScrollPositionArg) => {
  tryToSavePrevScrollPosition: () => void;
  tryToResetPrevScrollPosition: (x: number, y: number) => void;
  storage: StorageManager;
};

const useMemoScrollPosition: UseMemoScrollPosition = ({
  cacheId,
  tableBodyPrefixCls,
  wrapRef,
  dataSource,
  columns,
}) => {
  const storage = useStorage(cacheId, 'session');
  let limitCount = 0;

  const tryToSavePrevScrollPosition = () => {
    const dom = wrapRef.current?.querySelector(`.${tableBodyPrefixCls}`);
    const scrollTop = dom?.scrollTop;
    const scrollLeft = dom?.scrollLeft;
    storage.save({ scrollTop, scrollLeft });
  };
  const tryToResetPrevScrollPosition = (x: number, y: number) => {
    const dom = wrapRef.current?.querySelector(`.${tableBodyPrefixCls}`);
    if (!dom) {
      limitCount++;
      if (limitCount < MAX_LIMIT) {
        requestAnimationFrame(tryToResetPrevScrollPosition.bind(null, x, y));
      } else {
        limitCount = 0;
      }
    } else {
      limitCount = 0;
      dom.scrollTo(x, y);
    }
  };

  useEffect(() => {
    // ant-table-body
    const storagesValue = storage.read();
    if (storagesValue && dataSource?.length) {
      requestAnimationFrame(
        tryToResetPrevScrollPosition.bind(null, storagesValue.scrollLeft, storagesValue.scrollTop),
      );
    }
  }, [columns, dataSource]);

  return {
    tryToSavePrevScrollPosition,
    tryToResetPrevScrollPosition,
    storage,
  };
};

export default useMemoScrollPosition;
