import { useLayoutEffect, useState, useEffect } from 'react';
import useResizeObserver from 'use-resize-observer';
import { TableProps } from '../..';

interface UseScrollArg {
  wrapRef: React.RefObject<HTMLDivElement>;
  propsScroll: TableProps['scroll'];
  prefixCls: string;
  dataSource: TableProps['dataSource'];
}

interface UseScroll {
  (
    { wrapRef, propsScroll, prefixCls, dataSource }: UseScrollArg,
    deps: any[],
  ): TableProps['scroll'];
}

const useScroll: UseScroll = ({ wrapRef, propsScroll, prefixCls, dataSource }, deps) => {
  const { width = 1, height: h = 1 } = useResizeObserver<HTMLDivElement>({
    ref: wrapRef,
  });
  const [scroll, setScroll] = useState<TableProps['scroll']>({ ...propsScroll });

  useLayoutEffect(() => {
    const paginationDomInstance = wrapRef.current?.querySelector(`.${prefixCls}-pagination`);
    const tableHeadDomInstance = wrapRef.current?.querySelector(`.${prefixCls}-table-thead`);
    const tableHeight = getTableScrollY(paginationDomInstance, tableHeadDomInstance, h);
    setScroll(
      Object.assign(
        {
          scrollToFirstRowOnChange: true,
          x: 'max-content',
          y: dataSource?.length ? tableHeight : undefined,
        },
        propsScroll,
      ),
    );
  }, [...deps, width, h, dataSource]);

  return scroll;
};

export default useScroll;

const heightProps = [
  'height',
  'marginTop',
  'marginBottom',
  'paddingTop',
  'paddingBottom',
  'borderBottomWidth',
  'borderTopWidth',
];

export function getTableScrollY(
  paginationDomInstance: Element | undefined | null,
  tableHeadDomInstance: Element | undefined | null,
  wrapHeight: number,
) {
  const tableHeadDomHeight = getComputedStyleTotalHeight(tableHeadDomInstance),
    paginationDomHeight = getComputedStyleTotalHeight(paginationDomInstance);
  return Math.max(wrapHeight - tableHeadDomHeight - paginationDomHeight, 0);
}

function getComputedStyleTotalHeight(domInstance: Element | undefined | null) {
  if (!domInstance) return 0;

  const computedStyle = window.getComputedStyle(domInstance);
  return heightProps.reduce((prev, cur) => prev + parseInt(computedStyle[cur as any], 10), 0);
}
