const path = require('path');
const fs = require('fs-extra');
const prettier = require('prettier');
const { default: traverse } = require('@babel/traverse');
const MagicString = require('magic-string');

function readLocalesFile({ localesFilePath, fileType }) {
    const cwd = process.cwd();
    // console.log('localesFilePath ::', localesFilePath);

    const localesFilePathMap = {};
    const getPath = (p) => (typeof p === 'string' ? p : p.outputPath);
    localesFilePath.forEach((p) => {
        let locales = '';
        try {
            locales = fs.readFileSync(path.resolve(cwd, getPath(p)), 'utf-8');
        } catch (e) {
            locales = fileType === 'json' ? '{}' : 'export default {}';
        }
        const s = new MagicString(locales);
        let objectContent = '';
        const fileName = path.basename(getPath(p));

        if (fileType === 'json') {
            localesFilePathMap[fileName] = { locales: JSON.parse(locales.toString()) };
            return;
        }
        prettier.format(locales, {
            parser(text, parsers) {
                let ast = parsers['babel-ts'](text);
                traverse(ast, {
                    VariableDeclarator(path) {
                        if (path.node.init && path.node.init.type === 'ObjectExpression') {
                            objectContent = s.slice(path.node.init.start, path.node.init.end);
                        }
                    },
                    ObjectExpression(path) {
                        if (path.parentPath.type === 'ExportDefaultDeclaration') {
                            objectContent = s.slice(path.node.start, path.node.end);
                        }
                    },
                });
                return ast;
            },
        });
        if (objectContent) {
            const tempPath = path.resolve(cwd, 'node_modules/.usedk_locales_temp');
            fs.ensureDirSync(tempPath);
            const cacheFilePath = path.resolve(tempPath, fileName);
            fs.writeFileSync(cacheFilePath, `module.exports = ${objectContent}`);
            localesFilePathMap[fileName] = { locales: require(cacheFilePath) };
        }
    });
    return localesFilePathMap;
}

module.exports = readLocalesFile;
