import * as React from 'react';
import { useState, useEffect } from 'react';
import { Input } from 'udesk-ui';
import { useLocaleReceiver } from '../../../locale-provider/LocaleReceiver';

type CustomLogicInputProps = {
  value?: string;
  onChange?: (value: string) => void;
};

function CustomLogicInput({ value, onChange }: CustomLogicInputProps) {
  const [inputValue, setInputValue] = useState('');
  const [contextLocale] = useLocaleReceiver('RulesConfig');

  const onInputChange = (e: any) => {
    let currentValue = e.target.value;
    let newValue;
    let letterReg = /^[andor]*$/;
    newValue = currentValue.replace(
      /[a-zA-Z]+|\d+|\(|\)|[ +]/g,
      function (word: string, index: number, str: any[]) {
        if (letterReg.test(word)) {
          if (!letterReg.test(str[index - 1]) && str[index - 1] !== ' ') {
            return ' ' + word;
          }
          let nextIndex = index + word.length;
          if (str[nextIndex] && !letterReg.test(str[nextIndex]) && str[nextIndex] !== ' ') {
            return word + ' ';
          }
        }
        return word;
      },
    );
    onChange?.(newValue);
  };
  useEffect(() => {
    setInputValue(value || '');
  }, [value]);
  return (
    <Input
      placeholder={contextLocale.lang.judgmentLogicTypes.pleaseInputCustomLogic}
      value={inputValue}
      onChange={onInputChange}
    />
  );
}

export default CustomLogicInput;
