import { message } from 'udesk-ui';

const TagValueMaxLength = 8;

export const validateValues = (
  values: any,
  configRuleTypes: any,
  fieldOperators: any,
  judgmentLogicTypes: any,
  contextLocale: any,
) => {
  if (!values.name || !values.name.trim()) {
    message.error(contextLocale.lang.validate.inputName);
    return false;
  }
  let tagValueLengthError = false;
  let isExistNullTagValue;
  if (Array.isArray(values.tagValueLists) && values?.tagValueLists?.length) {
    if (values.tagValueLists.length > TagValueMaxLength) {
      tagValueLengthError = true;
    }
    for (let index = 0; index < values?.tagValueLists?.length; index++) {
      let tagValueItem = values.tagValueLists[index];
      let field = tagValueItem.field;
      let operator = tagValueItem.operator;
      let value = tagValueItem.value;

      if (field === '' || field == null) {
        isExistNullTagValue = true;
        break;
      } else {
        if (field == configRuleTypes.platform.id) {
          if (!Array.isArray(value) || !value.length) {
            isExistNullTagValue = true;
            break;
          }
        } else if (
          field == configRuleTypes.dialogueDesc.id &&
          (operator == fieldOperators.exist.id || operator == fieldOperators.not_exist.id)
        ) {
          continue;
        } else if (
          field == configRuleTypes.tags.id &&
          (operator == fieldOperators.exist.id || operator == fieldOperators.not_exist.id)
        ) {
          continue;
        } else if (
          field == configRuleTypes.queryContent.id &&
          (operator == fieldOperators.containsSensitive.id ||
            operator == fieldOperators.contains.id)
        ) {
          continue;
        } else if (field == configRuleTypes.orderReply.id) {
          continue;
        } else {
          if (value === '' || value == null) {
            isExistNullTagValue = true;
            break;
          }
        }
      }
    }
  }
  if (tagValueLengthError) {
    message.error(contextLocale.lang.validate.lengthError8);
    return;
  }
  if (isExistNullTagValue) {
    message.error(contextLocale.lang.validate.rulesConfigcanNotBeEmpty);
    return;
  }
  if (
    values.judgmentLogic === judgmentLogicTypes.custom.id &&
    (!values.tag || !values.tag.trim())
  ) {
    message.error(contextLocale.lang.validate.inputCustomName);
    return;
  }
  values.name = values.name.trim();
  if (values.judgmentLogic === judgmentLogicTypes.custom.id) {
    values.tag = values.tag.trim();
  }
  let reg = /^[() andor\d]*$/;
  if (values.tag && !reg.test(values.tag)) {
    message.error(contextLocale.lang.validate.customTagDataError);
    return;
  }

  return true;
};

// 从km复制来
export const formatData = (
  values: any,
  tagValueListsKey: string,
  judgmentLogicTypes: any,
  configRuleTypes: any,
) => {
  if (values.judgmentLogic == judgmentLogicTypes.meetAll.id) {
    values.tag = getLogicText(' and ', values?.[tagValueListsKey]);
  } else if (values.judgmentLogic == judgmentLogicTypes.meetAny.id) {
    values.tag = getLogicText(' or ', values?.[tagValueListsKey]);
  }
  // 格式化数据后返回
  Array.isArray(values?.[tagValueListsKey]) &&
    values?.[tagValueListsKey].forEach((item: any) => {
      if (item.field == configRuleTypes.platform.id) {
        let formatValue = item?.value?.join(',');
        item.value = formatValue;
      } else if (item.field == configRuleTypes.currentTime.id) {
        // 无需处理
      } else if (item.field == configRuleTypes.orderReply.id) {
        item.operator = 'is';
        item.value = 'true';
      } else if (item.value) {
        let formatValue = item.value.replace(/，/g, ',');
        item.value = formatValue;
      }
    });
  return values;
};
const getLogicText = (separator: string, tagValueLists: any[]) => {
  if (Array.isArray(tagValueLists)) {
    let resultList = [];
    let tagValueLength = tagValueLists.length;
    for (let index = 1; index <= tagValueLength; index++) {
      resultList.push(index);
    }
    return '(' + resultList.join(separator) + ')';
  }
};

// 从km复制来
export const formatToComponentData = (
  tagValueLists: any,
  configRuleTypes: any,
  manyAnswerChannelTypes: any,
) => {
  // 处理tagValueLists字段，封装到过滤器组件中会有bug，暂时只能放到这里处理好数据给封装的过滤器组件
  // 格式化数据后传递
  tagValueLists = tagValueLists?.map((item: any) => {
    let tagItem = { ...item };
    if (tagItem.field == configRuleTypes.platform.id && typeof tagItem.value === 'string') {
      let formatValue = tagItem.value;
      let weChatValue = manyAnswerChannelTypes.WEChat.id; // "wechat,微信" 特殊处理一下
      if (formatValue.includes(weChatValue)) {
        formatValue = formatValue.replace(weChatValue, '#####'); // "#####" 临时占位用
      }
      let formatValueList = formatValue.split(',');
      let findIndex = formatValueList.findIndex((val: any) => {
        return val === '#####';
      });
      if (findIndex > -1) {
        formatValueList[findIndex] = weChatValue;
      }
      tagItem.value = formatValueList;
    }
    return tagItem;
  });

  return tagValueLists;
};
