import React, { useEffect, useMemo, useRef, useState } from 'react';
import { MenuOutlined, SettingsOutlined } from '@udesk/icons';
import { Button, Checkbox, Popover, Space, Transfer, Tree } from 'udesk-ui';
import { TransferProps } from 'udesk-ui/es/transfer';
import { TableProps } from '../../memo-table';
import { SortableContainer, SortableElement, SortableHandle } from 'react-sortable-hoc';
import { arrayMoveImmutable } from '../../utils/array/array-move';
import { StandardizeTableTableProps } from '../types';
import styled from 'styled-components';


export interface ColumnsControlProps {
  columns: StandardizeTableTableProps['columns'];
  showColumnKeys: ColumnsControlTransferProps['showColumnKeys'];
  onChange: ColumnsControlTransferProps['onChange'];
}

interface ColumnsControlTransferProps extends TransferProps<any> {
  showColumnKeys: string[];
  onChange: (showKeys: string[]) => void;
  getContainer: () => any;
}

interface SortableItemProps {
  selectedKeys: string[];
  item: any;
  onItemSelect: (key: string, check: boolean) => void;
  index: number;
}
const DragHandle = SortableHandle(() => (
  <div className="icon-unordered-list">
    <MenuOutlined />
  </div>
));
const SortableItem: any = SortableElement(({ selectedKeys, item, onItemSelect }: any) => {
  return (
    <div style={{ padding: '6px 12px', lineHeight: '20px', transition: 'all 0.3s', zIndex: 1031 }}>
      <div style={{display: 'flex', alignItems: 'center'}}>
        <Checkbox
          style={{flex: 1}}
          onChange={(e) => {
            onItemSelect(item.key, e.target.checked);
          }}
          checked={selectedKeys.includes(item.key)}
        >
          {item.title}
        </Checkbox>
        {/* <div key={item.key}></div> */}
        <DragHandle />
      </div>
    </div>
  );
});
const SortableList = SortableContainer(({ list, selectedKeys, onItemSelect }: any) => {
  return (
    <ul>
      {list.map((item: any, index: number) => (
        <SortableItem
          key={item.key}
          index={index}
          item={item}
          selectedKeys={selectedKeys}
          onItemSelect={onItemSelect}
        />
      ))}
    </ul>
  );
});

const ColumnsControlTransfer: React.FC<ColumnsControlTransferProps> = (props) => {
  const { dataSource, showColumnKeys, onChange: onChangeProps, getContainer } = props;
  const onChange = (targetKeys: any) => {
    onChangeProps?.(targetKeys);
  };

  const onSortChange = (filteredItems: any[]) => {
    onChange(filteredItems.map((i) => i.key));
  };

  return (
    <Transfer
      dataSource={dataSource}
      targetKeys={showColumnKeys}
      onChange={onChange}
      render={(item: any) => item.title}
      showSearch
      filterOption={(inputValue: string, option: any) => {
        return option?.title?.includes(inputValue);
      }}
    >
      {({ direction, onItemSelect, selectedKeys, filteredItems }) => {
        if (direction === 'right') {
          return (
            <SortableList
              list={filteredItems}
              onSortChange={onSortChange}
              selectedKeys={selectedKeys}
              onItemSelect={onItemSelect}
              onSortEnd={({ oldIndex, newIndex }: any) => {
                let res = arrayMoveImmutable(showColumnKeys, oldIndex, newIndex);
                onChange(res);
              }}
              getContainer={getContainer}
              useDragHandle={true}
              // style={{ display: 'flex', flexDirection: 'column' }}
            ></SortableList>
          );
        }
      }}
    </Transfer>
  );
};

const ColumnsControlTransferWrap = styled.div`
  ul {
    padding: 0;
    margin: 0;
  }
`;

const ColumnsControl: React.FC<ColumnsControlProps> = (props) => {
  const { columns, showColumnKeys, onChange } = props;
  const dataSource = useMemo(() => {
    return columns?.map((i) => ({ title: i.title, key: i.key })) ?? [];
  }, [columns]);
  const ref = useRef<HTMLDivElement>(null);
  return (
    <Popover
      placement="bottomRight"
      //   visible={true}
      content={
        <ColumnsControlTransferWrap ref={ref}>
          <ColumnsControlTransfer
            dataSource={dataSource}
            showColumnKeys={showColumnKeys}
            onChange={onChange}
            getContainer={() => ref.current || document.body}
          />
        </ColumnsControlTransferWrap>
      }
      trigger="click"
      arrowPointAtCenter
    >
      <Button icon={<SettingsOutlined style={{ marginRight: 4, fontSize: '16px' }} />}>列</Button>
    </Popover>
  );
};

export type ColumnsControlType = typeof ColumnsControl;

export default ColumnsControl;
