import React, { useEffect } from 'react';
// @ts-ignore
import { StandardizeTable, CommonPage } from 'udesk_gm_ui';
import { Button, Badge, Switch } from 'udesk-ui';
import styled from 'styled-components';
// @ts-ignore
import { StandardizeTableProps, StandardizeTableTableProps } from 'udesk_gm_ui/es/standardize-table/types/index';

const DemoWrap = styled.div`
  color: rgba(0, 0, 0, 0.85);
  height: 700px;
  .xxx-page {
    overflow-y: hidden;
  }
`;

const { RequestTable } = StandardizeTable;

const treeDataOrigin = [
  {
    title: '不可删除节点',
    key: 1,
    id: 1,
    default: 1,
  },
  {
    title: '我超级长我超级长我超级长我超级长我超级长我超级长',
    key: 123,
    id: 123,
    children: [
      {
        title: '子节点127',
        key: 127,
        id: 127,
        children: [
          {
            title: '子节点128',
            key: 128,
            id: 128,
          },
        ],
      },
    ],
  },
  {
    title: '啦啦啦啦',
    key: 124,
    id: 124,
  },
];
const treeNodeActionMenu = [
  {
    label: '编辑',
    onClick: (item: any) => console.log(item),
  },
  {
    label: '删除',
    danger: true,
    onClick: (item: any) => console.log(item),
    disabled: (item: any) => Boolean(item.default),
  },
];

const originColumns: StandardizeTableTableProps['columns'] = [
  {
    key: 'taskName',
    dataIndex: 'taskName',
    title: '质检任务',
    // width: '10%',
    ellipsis: true,
  },
  {
    key: 'createUserUsername',
    dataIndex: 'createUserUsername',
    title: '操作人',
  },
  {
    key: 'createTime',
    dataIndex: 'createTime',
    title: '上传时间',
    ellipsis: true,
    // width: '15%',
  },
  {
    key: 'endTime',
    dataIndex: 'endTime',
    title: '结束时间',
    // width: '15%',
    ellipsis: true,
  },
  {
    key: 'total',
    dataIndex: 'total',
    title: '上传总量',
    sorter: true,
    // width: '10%',
  },
  {
    key: 'success',
    dataIndex: 'success',
    title: '质检成功',
    // width: '10%',
  },
  {
    key: 'fail',
    dataIndex: 'fail',
    title: '质检失败',
    // width: '10%',
  },
  {
    key: 'status',
    dataIndex: 'status',
    title: '质检进度',
    // width: '10%',
    render: (text: any, record: any, index: any) => (
      <React.Fragment>
        <Badge status={record.status === 0 && record.total ? 'processing' : 'success'} />
        <span>{record.status === 0 && record.total ? '进行中' : '已完成'}</span>
      </React.Fragment>
    ),
  },
  {
    key: 'actions',
    dataIndex: 'actions',
    title: '操作',
    renderType: 'actions',
    buttons: [
      {
        label: '编辑',
        onClick: (item: any) => {
          console.log('item:: ', item)
        }
      },
      {
        label: '查看',
        onClick: (item: any) => {
          console.log('item:: ', item)
        }
      },
      {
        label: '更多操作',
        onClick: (item: any) => {
          console.log('item:: ', item)
        }
      },
      {
        label: '删除',
        danger: true,
        onClick: (item: any) => {
          console.log('item:: ', item)
        }
      },
    ]
  },
];

const filter: StandardizeTableProps['filter'] = {
  fields: [
    {
      type: 'RangePicker',
      label: '上传时间',
      name: 'time',
      isDefault: true,
    },
    {
      type: 'Input',
      label: '任务名称',
      name: 'taskName',
      isDefault: true,
    },
    {
      type: 'Select',
      label: '操作人',
      name: 'createUserId',
      isDefault: true,
      props: {
        optionFilterProp: 'label',
        showSearch: true,
        style: {
          width: 125,
        },
      },
    },
    {
      type: 'Select',
      label: '质检进度',
      isDefault: true,
      name: 'status',
      props: {
        options: [
          {
            label: '全部',
            value: null,
          },
          {
            label: '进行中',
            value: 0,
          },
          {
            label: '已完成',
            value: 1,
          },
        ],
      },
    },
    {
      type: 'Select',
      name: 'type1',
      label: '默认隐藏1',
    },
    {
      type: 'Input',
      name: 'type3',
      label: '默认隐藏2',
    },
    {
      type: 'Input',
      name: 'type4',
      label: '默认隐藏4',
    },
  ],
  initialValues: {
    taskName: '上传',
  },
  //   onSearch: (v, o) => {
  //     console.log('v, o', v, o);
  //   },
};

const ClassificationTreeTitle = () => {
  return (
    <div style={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center' }}>
      <div>标题</div>
      <Button size="small" type="link">
        新建
      </Button>
    </div>
  );
};
export default () => {
  const [setting, setSetting] = React.useState<any>({
    classification: false,
    filter: false,
    key: 'table',
  });
  const [dataSource, setDataSource] = React.useState<any[]>([]);
  const [treeData, setTreeData] = React.useState<any[]>([]);
  const [tableLoading, setTableLoading] = React.useState(false);
  const [treeLoading, setTreeLoading] = React.useState(false);
  const [selectedClassificationId, setSelectedClassificationId] = React.useState<any>(null);
  const [pagination, setPagination] = React.useState({
    current: 2,
    pageSize: 100,
    total: 3333,
  });
  const onPaginationChange = (current: number, pageSize: number) => {
    setPagination({
      ...pagination,
      current,
      pageSize,
    });
  };

  const classification = {
    title: <ClassificationTreeTitle />,
    treeData,
    loading: treeLoading,
    treeNodeActionMenu,
    defaultSelectedKeys: [selectedClassificationId],
    // selectedKeys: [selectedClassificationId],
    onSelect: (selectedKey: any, params: any) => {
      console.log('selectedKey, params:: ', selectedKey, params);
      //   setSelectedClassificationId(selectedKey);
      setPagination((prev) => {
        return {
          ...prev,
          ...params.pagination,
        };
      });
    },
  };

  const table = {
    loading: tableLoading,
    columns: originColumns,
    dataSource,
    rowSelectionEnhance: {
      onSelectedRowKeysChange: (keys: any[], items: any[]) => {
        console.log('keys, items', keys, items);
      },
    },
    pagination: {
      ...pagination,
      onChange: onPaginationChange,
    },
  };

  useEffect(() => {
    setTreeLoading(true);
    setTimeout(() => {
      setTreeData(treeDataOrigin);
      setSelectedClassificationId(treeDataOrigin[2].id);
      setTreeLoading(false);
    }, 1000);
  }, []);
  return (
    <React.Fragment>
      <div style={{marginBottom: 8, color: '#000'}}>
        分类：<Switch checked={setting.classification} onChange={(checked: boolean) => {
          setSetting((prev: any) => ({
            ...prev,
            classification: checked,
            key: 'table' + Math.random()
          }))
        }}></Switch>
        过滤器：<Switch checked={setting.filter} onChange={(checked: boolean) => {
          setSetting((prev: any) => ({
            ...prev,
            filter: checked,
            key: 'table' + Math.random()
          }))
        }}></Switch>
      </div>
      <DemoWrap>
        <RequestTable
          key={setting.key}
          onRequest={(params: any) => {
            console.log('params:: ', params);
            setTableLoading(true);
            setTimeout(() => {
                const { pageSize, current } = params.pagination;
                setDataSource(
                  new Array(pageSize).fill(1).map((_, i) => ({
                    taskName: '本地录音上传质检' + i,
                    createUserUsername: 'linapp',
                    createTime: '2022-05-27 17:57:17',
                    endTime: '2022-05-27 18:00:28',
                    total: 17,
                    id: `${pagination.current}-${i}`, // 假数据，会影响跨页多选
                  })),
                );
                setTableLoading(false);
            }, 1000);
          }}
          table={table}
          filter={setting.filter ? filter : undefined}
          classification={setting.classification ? classification : undefined}
          toolBarRightButtons={[
            {
              label: '主按钮',
              mainBtn: true,
              onClick: () => {
                console.log('主按钮 onClick');
              },
            },
            {
              label: '更多操作1',
              onClick: () => {
                console.log('更多操作1 onClick');
              },
            },
            {
              label: '更多操作2',
              confirm: {
                title: '自定义导出弹窗文案',
                okText: '自定义按钮文案',
              },
              onClick: () => {
                console.log('自定义导出弹窗文案 onClick');
              },
            },
            {
              label: '更多操作3',
              onClick: () => {
                console.log('更多操作3 onClick');
              },
            },
          ]}
          exportBtn={{
            onClick: (type: any) => {
              console.log('exportBtn onClick ==> type::', type);
              return new Promise((res) => {
                setTimeout(() => {
                  res(type);
                }, 2000);
              });
            },
            dropdown: [
              {
                label: 'excel',
                type: 'excel',
              },
              {
                label: 'word',
                type: 'word',
                confirm: {
                  title: '自定义导出弹窗文案',
                },
              },
            ],
          }}
          importBtn={{
            onClick: () => {
              console.log('import');
              return Promise.resolve();
            },
          }}
          batchedActions={[
            {
              label: '批量编辑',
              onClick: (item: any) =>
                Promise.resolve(item).then((item) => {
                  console.log(item);
                }),
            },
            {
              label: '批量删除',
              danger: true,
              onClick: (item: any) =>
                Promise.resolve(item).then((item) => {
                  console.log(item);
                }),
            },
          ]}
          // columnsControl={{}}
          showFullscreen={true}
        ></RequestTable>
      </DemoWrap>
    </React.Fragment>
  );
};
