## Demo

<code src="./demo/request-table.tsx" background='rgb(240, 242, 245)' title='接管请求发送的RequestTable' compact='true'></code>

```tsx
/**
 * title: 基础使用
 * desc: CommonPage为标准右侧内容区域布局组件(即红框内区域) 内置了PageHeader组件，除API中展示的props外，均透传至PageHeader
 * background: 'rgb(240, 242, 245)'
 * compact: true
 */

import React from 'react';
import { StandardizeTable, CommonPage } from 'udesk_gm_ui';
import { Button, Badge } from 'udesk-ui';
import zhCN from 'udesk_gm_ui/es/locale/zh_CN';
import styled from 'styled-components';

const DemoWrap = styled.div`
  color: rgba(0, 0, 0, 0.85);
  height: 700px;
  .xxx-page {
    overflow-y: hidden;
  }
`;

const treeDataOrigin = [
  {
    title: '不可删除节点',
    key: 1,
    id: 1,
    default: 1,
  },
  {
    title: '我超级长我超级长我超级长我超级长我超级长我超级长',
    key: 123,
    id: 123,
    children: [
      {
        title: '子节点127',
        key: 127,
        id: 127,
        children: [
          {
            title: '子节点128',
            key: 128,
            id: 128,
          },
        ],
      },
    ],
  },
  {
    title: '啦啦啦啦',
    key: 124,
    id: 124,
  },
];
const treeNodeActionMenu = [
  {
    label: '编辑',
    onClick: (item) => console.log(item),
  },
  {
    label: '删除',
    danger: true,
    onClick: (item) => console.log(item),
    disabled: (item) => Boolean(item.default),
  },
];

const data = [];
for (let i = 1; i <= 100; i++) {
  data.push({
    taskName: '本地录音上传质检' + i,
    createUserUsername: 'linapp',
    createTime: '2022-05-27 17:57:17',
    endTime: '2022-05-27 18:00:28',
    total: 17,
  });
}
const originColumns = [
  {
    key: 'taskName',
    dataIndex: 'taskName',
    title: '质检任务',
    // width: '10%',
    ellipsis: true,
  },
  {
    key: 'createUserUsername',
    dataIndex: 'createUserUsername',
    title: '操作人',
  },
  {
    key: 'createTime',
    dataIndex: 'createTime',
    title: '上传时间',
    ellipsis: true,
    // width: '15%',
  },
  {
    key: 'endTime',
    dataIndex: 'endTime',
    title: '结束时间',
    // width: '15%',
    ellipsis: true,
  },
  {
    key: 'total',
    dataIndex: 'total',
    title: '上传总量',
    sorter: true,
    // width: '10%',
  },
  {
    key: 'success',
    dataIndex: 'success',
    title: '质检成功',
    // width: '10%',
  },
  {
    key: 'fail',
    dataIndex: 'fail',
    title: '质检失败',
    // width: '10%',
  },
  {
    key: 'status',
    dataIndex: 'status',
    title: '质检进度',
    // width: '10%',
    render: (text, record, index) => (
      <React.Fragment>
        <Badge status={record.status === 0 && record.total ? 'processing' : 'success'} />
        <span>{record.status === 0 && record.total ? '进行中' : '已完成'}</span>
      </React.Fragment>
    ),
  },
  {
    key: 'actions',
    dataIndex: 'actions',
    title: '操作',
    // width: '10%',
    render: (text, record, index) => (
      <Button
        type="link"
        size="small"
        disabled={record.status === 0 && record.total}
        onClick={(e) => {
          e.stopPropagation();
        }}
      >
        下载
      </Button>
    ),
  },
];
const filterFields = [
  {
    type: 'RangePicker',
    label: '上传时间',
    name: 'time',
    isDefault: true,
  },
  {
    type: 'Input',
    label: '任务名称',
    name: 'taskName',
    isDefault: true,
  },
  {
    type: 'Select',
    label: '操作人',
    name: 'createUserId',
    isDefault: true,
    props: {
      optionFilterProp: 'label',
      showSearch: true,
      style: {
        width: 125,
      },
    },
  },
  {
    type: 'Select',
    label: '质检进度',
    isDefault: true,
    name: 'status',
    props: {
      options: [
        {
          label: '全部',
          value: null,
        },
        {
          label: '进行中',
          value: 0,
        },
        {
          label: '已完成',
          value: 1,
        },
      ],
    },
  },
  {
    type: 'Select',
    name: 'type1',
    label: '默认隐藏1',
  },
  {
    type: 'Input',
    name: 'type3',
    label: '默认隐藏2',
  },
  {
    type: 'Input',
    name: 'type4',
    label: '默认隐藏4',
  },
];
export default () => {
  const [dataSource, setDataSource] = React.useState([]);
  const [treeData, setTreeData] = React.useState([]);
  const [tableLoading, setTableLoading] = React.useState(false);
  const [selectedClassificationId, setSelectedClassificationId] = React.useState(null);
  const [pagination, setPagination] = React.useState({
    current: 2,
    pageSize: 100,
    total: 3333,
  });
  const onPaginationChange = (current, pageSize) => {
    setPagination({
      ...pagination,
      current,
      pageSize,
    });
  };

  React.useEffect(() => {
    setTableLoading(true);
    setTimeout(() => {
      setTreeData([...treeDataOrigin]);
      setSelectedClassificationId(treeDataOrigin[0].id);
    }, 1000);
  }, []);

  React.useEffect(() => {
    setTableLoading(true);
    if (!selectedClassificationId) return;
    setTimeout(() => {
      setDataSource(
        data.map((i, index) => {
          // 注意 这个地方其实是有问题的，不能用index做id，会变，此处仅演示
          const id = `${pagination.current}-${index}`;
          return {
            ...i,
            id,
            // key: id,
          };
        }),
      );
      setTableLoading(false);
    }, 1000);
  }, [selectedClassificationId, pagination]);

  return (
    <DemoWrap>
      <CommonPage
        title="典型列表页面示例"
        pageBodyClassName="xxx-page"
        padding={0}
        ghostPage={true}
      >
        <StandardizeTable
          table={{
            loading: tableLoading,
            columns: originColumns,
            dataSource,
            rowSelectionEnhance: {
              onSelectedRowKeysChange: (keys, items) => {
                console.log('keys, items', keys, items);
              },
            },
            pagination: {
              ...pagination,
              onChange: onPaginationChange,
            },
          }}
          filter={{
            fields: filterFields,
            onSearch: (v, o) => {
              console.log('v, o', v, o);
            },
          }}
          classification={{
            title: (
              <div
                style={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center' }}
              >
                <div>标题</div>
                <Button size="small" type="link">
                  新建
                </Button>
              </div>
            ),
            treeData,
            loading: tableLoading,
            treeNodeActionMenu,
            selectedKeys: [selectedClassificationId],
            onSelect: (selectedKey, params) => {
              console.log('selectedKey, params:: ', selectedKey, params);
              setSelectedClassificationId(selectedKey);
              setPagination((prev) => {
                return {
                  ...prev,
                  ...params.pagination,
                };
              });
            },
          }}
          toolBarRightButtons={[
            {
              label: '主按钮',
              mainBtn: true,
              onClick: () => {
                console.log('主按钮 onClick');
              },
            },
            {
              label: '更多操作1',
              onClick: () => {
                console.log('更多操作1 onClick');
              },
            },
            {
              label: '更多操作2',
              confirm: {
                title: '自定义导出弹窗文案',
                okText: '自定义按钮文案',
              },
              onClick: () => {
                console.log('自定义导出弹窗文案 onClick');
              },
            },
            {
              label: '更多操作3',
              onClick: () => {
                console.log('更多操作3 onClick');
              },
            },
          ]}
          exportBtn={{
            onClick: (type) => {
              console.log('exportBtn onClick ==> type::', type);
              return new Promise((res) => {
                setTimeout(() => {
                  res();
                }, 2000);
              });
            },
            dropdown: [
              {
                label: 'excel',
                type: 'excel',
              },
              {
                label: 'word',
                type: 'word',
                confirm: {
                  title: '自定义导出弹窗文案',
                },
              },
            ],
          }}
          importBtn={{
            onClick: () => {
              console.log('import');
            },
          }}
          batchedActions={[
            {
              label: '批量编辑',
              onClick: (item) =>
                Promise.resolve(item).then((item) => {
                  console.log(item);
                }),
            },
            {
              label: '批量删除',
              danger: true,
              onClick: (item) =>
                Promise.resolve(item).then((item) => {
                  console.log(item);
                }),
            },
          ]}
          columnsControl={{}}
          showFullscreen={true}
        ></StandardizeTable>
      </CommonPage>
    </DemoWrap>
  );
};
```


## API

| 参数 | 说明 | 类型 | 默认值 | 版本 |
| --- | --- | --- | --- | --- |
| pageBodyClassName | content 区域的 class 名，必传参数 | string |  |  |
| loading | 控制 content 区域的 loading | boolean | false |  |
| ghostPage | content 区域背景颜色是否为透明色 | boolean | false |  |
| padding | content 区域的 padding 值，默认为 16px | number | 16 |  |
| footBar | 渲染固定页脚 | ReactNode |  |  |
| navigationTabs | 用于渲染 PageHeader 下方的路由导航用的 Tabs | NavigationTabItem[]（见 Interface 部分） |  |  |
| currentNavigationPath | 当前所在路由的 url 路径，一般从`history`对象获取，详细获取方法见`路由导航Demo` | string |  |  |
| onNavigationTabsChange | Tabs 变化时的回调函数 | (tab: NavigationTabItem) => void（见 Interface 部分） |  |  |

其余 api 与`Udesk-ui`的`PageHeader`组件保持一致

## 静态方法

| 参数 | 说明 | 类型 | 默认值 | 版本 |
| --- | --- | --- | --- | --- |
| PageInfoMemo | 储存缓存的数据的组件，使用此组件包裹后即可使用 CommonPage 的数据缓存能力 | ReactNode | - |  |
| usePageStateMemo | 自定义 hook，用于储存、读取缓存的数据 | () => ({pageInfo, storePageInfo, clearPageInfo}) | - |  |

## PageInfoMemo 的实例方法

| 参数          | 说明               | 类型                                        | 默认值 | 版本 |
| ------------- | ------------------ | ------------------------------------------- | ------ | ---- |
| pageInfo      | 缓存的数据         | {[key: string]: any}                        | -      |      |
| storePageInfo | 更新缓存数据的方法 | (newPageInfo: {[key: string]: any}) => void | -      |      |
| clearPageInfo | 清除缓存数据的方法 | () => void                                  | -      |      |

## Interface

```ts
interface NavigationTabItem {
  path: string;
  text: string;
  hidden?: boolean;
  pathName?: string;
}
```
