import React, { Key } from 'react';
import type { FormProps, FormInstance, FormFieldConfig } from 'udesk-ui/es/form-builder/types/form';
import { ColumnGroupType, ColumnType } from 'udesk-ui/lib/table';
import { ActionMenuItem } from '../../batched-actions';
import { CommonFilterInstance, CommonFilterProps } from '../../common-filter';
import { ExportButtonProps } from '../../export-button';
import type { TableInstance, TableProps } from '../../memo-table';
import type { TreeMenuProps } from '../../tree-menu/types';
import type { ColumnsControlProps, ColumnsControlType } from '../components/columns-control-btn';

export type ToolbarButtonsType = React.ReactNode;

export type ClassificationOnSelectOptions = {
  filter: any;
  pagination: TableProps['pagination'];
};

export interface ClassificationProps extends TreeMenuProps {
  title?: any;
}

export interface BatchedActionsType extends Omit<ActionMenuItem, 'onClick'> {
  onClick: (item?: any[]) => Promise<any> | void;
}
export interface ActionsButtonType extends Omit<ActionMenuItem, 'onClick'> {
  onClick: (item: any) => Promise<any> | void;
}

export interface StandardizeTableProps
  extends Omit<StandardizeTableViewProps, 'classification' | 'batchedActions'> {
  classification?: Omit<ClassificationProps, 'onSelect'> & {
    onSelect: (key: Key | undefined, options: ClassificationOnSelectOptions) => void;
  };
  batchedActions?: BatchedActionsType[];
  // autoTriggerOnSearchOnMount?: boolean;
}

export interface StandardizeTableInstance {
  pageInstance: HTMLDivElement;
  tableInstance: TableInstance;
  filterInstance: CommonFilterInstance;
}

type Unpacked<T> = T extends (infer U)[]
  ? U
  : T extends (...args: any[]) => infer U
  ? U
  : T extends Promise<infer U>
  ? U
  : T;

export interface StandardizeTableTableProps extends Omit<TableProps, 'columns'> {
  columns?: (Unpacked<TableProps['columns']> & {
    onChange?: (value: any, record: any, index: number) => void;
  } & (
      | {
          renderType?: 'actions';
          buttons: ActionsButtonType[];
        }
      | {
          renderType?: 'switch';
        }
    ))[];
}

export interface StandardizeTableViewProps {
  table: StandardizeTableTableProps;
  prefixCls?: string;
  className?: string;
  filter?: CommonFilterProps;
  classification?: ClassificationProps;
  toolBarRightButtons?: (ActionMenuItem & { mainBtn?: boolean })[];
  columnsControl?: ColumnsControlProps;
  exportBtn?: Omit<ExportButtonProps, 'exportDataCount'>;
  importBtn?: Omit<ExportButtonProps, 'exportDataCount'>;
  batchedActions?: ActionMenuItem[];
  showFullscreen?: boolean;
}

export interface TableWrapProps {
  prefixCls: string;
  padding?: number | string;
}

export interface ToolbarProps {
  visible: boolean;
  prefixCls: string;
}
