import React                from 'react';
import { configure, mount } from 'enzyme';
import TextDropdown         from './index';
import Adapter              from 'enzyme-adapter-react-16';

configure({adapter : new Adapter()});

test('Popover test', () => {
    let wrapper = mount(<TextDropdown
        value=''
        prefix='前缀:'
        placeholder='全部'
        controlClassName='control'
        t={v => v}
        onChange={(value, isBlank, isNotBlank) => wrapper.setProps({value, isBlank, isNotBlank})}
    />);

    function open() {
        wrapper.find('.control').simulate('click');
    }
    function clickClearBtn() {
        // expect(wrapper.find('button').at(1).text()).toBe('');
        wrapper.find('button').at(1).simulate('click');
    }
    function clickConformBtn() {
        wrapper.find('button').at(0).simulate('click');
    }

    expect(wrapper.find('.control').text()).toBe('前缀: 全部');

    wrapper.find('.control').simulate('click');

    expect(wrapper.find('.form-control').length).toBe(1);

    expect(wrapper.find('button').at(0).text()).toBe('label.confirm');

    wrapper.find('.form-control').simulate('change', {target : {value : '123'}});
    expect(wrapper.find('.form-control').prop('value')).toBe('123');

    wrapper.find('button').at(0).simulate('click');
    expect(wrapper.prop('value')).toBe('123');

    wrapper.find('.control').simulate('click');
    wrapper.find('button').at(1).simulate('click');
    expect(wrapper.prop('value')).toBe('');

    wrapper.find('.control').simulate('click');
    wrapper.find('input').simulate('change', {target : {value : 'abc'}});
    wrapper.find('input').simulate('keyDown', {key : 'Enter', which : 13, keyCode : 13});
    expect(wrapper.find('input').length).toBe(0);
    expect(wrapper.prop('value')).toBe('abc');
    //只有“为空”按钮
    wrapper.setProps({allowBlank : true});
    wrapper.find('.control').simulate('click');
    expect(wrapper.find('[type="checkbox"]').length).toBe(1);

    wrapper.setProps({value : '321'});
    expect(wrapper.find('.form-control').prop('value')).toBe('321');

    wrapper.setProps({isBlank : true});
    expect(wrapper.find('.form-control').prop('disabled')).toBe(true);
    expect(wrapper.find('[type="checkbox"]').prop('checked')).toBe(true);

    wrapper.find('[type="checkbox"]').simulate('change', {target : {checked : false}});
    expect(wrapper.find('[type="checkbox"]').prop('checked')).toBe(false);
    expect(wrapper.find('.form-control').prop('disabled')).toBe(false);

    wrapper.find('[type="checkbox"]').simulate('change', {target : {checked : true}});
    expect(wrapper.find('[type="checkbox"]').prop('checked')).toBe(true);
    wrapper.find('button').at(0).simulate('click');
    expect(wrapper.prop('isBlank')).toBe(true);
    expect(wrapper.find('.control').text()).toBe('前缀: filter.operator.blank');
   
    //测试allowBlank为true时，清空按钮
    open();
    clickClearBtn();
    expect(wrapper.prop('value')).toBe('');
    expect(wrapper.prop('isBlank')).toBe(false);
    open();
    expect(wrapper.find('[type="checkbox"]').prop('checked')).toBe(false);
    expect(wrapper.find('.form-control').prop('value')).toBe('');
    clickClearBtn();

     //只有“不为空”按钮

    
    //同时有“为空”，“不为空”按钮
    wrapper.setProps({
        allowBlank: true,
        allowNotBlank: true,
    });
    open();
    expect(wrapper.find('[type="checkbox"]').length).toBe(2);

    wrapper.setProps({value : '321'});
    expect(wrapper.find('.form-control').prop('value')).toBe('321');

    wrapper.setProps({isBlank : true});
    expect(wrapper.find('.form-control').prop('disabled')).toBe(true);
    expect(wrapper.find('[type="checkbox"]').at(0).prop('checked')).toBe(true);
    expect(wrapper.find('[type="checkbox"]').at(1).prop('checked')).toBeUndefined();

    wrapper.find('[type="checkbox"]').at(0).simulate('change', {target : {checked : false}});
    expect(wrapper.find('[type="checkbox"]').at(0).prop('checked')).toBe(false);
    expect(wrapper.find('[type="checkbox"]').at(1).prop('checked')).toBeUndefined();
    expect(wrapper.find('.form-control').prop('disabled')).toBe(false);
    clickClearBtn();

    open();
    wrapper.setProps({isNotBlank : true});
    expect(wrapper.find('.form-control').prop('disabled')).toBe(true);
    expect(wrapper.find('[type="checkbox"]').at(0).prop('checked')).toBe(false);
    expect(wrapper.find('[type="checkbox"]').at(1).prop('checked')).toBe(true);

    wrapper.find('[type="checkbox"]').at(1).simulate('change', {target : {checked : false}});
    expect(wrapper.find('[type="checkbox"]').at(0).prop('checked')).toBe(false);
    expect(wrapper.find('[type="checkbox"]').at(1).prop('checked')).toBe(false);
    expect(wrapper.find('.form-control').prop('disabled')).toBe(false);
    clickClearBtn();

    open();
    wrapper.setProps({isBlank : true, isNotBlank: true});
    expect(wrapper.find('.form-control').prop('disabled')).toBe(true);
    expect(wrapper.find('[type="checkbox"]').at(0).prop('checked')).toBe(true);
    expect(wrapper.find('[type="checkbox"]').at(1).prop('checked')).toBe(true);

    wrapper.find('[type="checkbox"]').at(1).simulate('change', {target : {checked : false}});
    expect(wrapper.find('[type="checkbox"]').at(1).prop('checked')).toBe(false);
    expect(wrapper.find('.form-control').prop('disabled')).toBe(true);

    wrapper.find('[type="checkbox"]').at(1).simulate('change', {target : {checked : true}});
    expect(wrapper.find('[type="checkbox"]').at(1).prop('checked')).toBe(true);
    expect(wrapper.find('.form-control').prop('disabled')).toBe(true);
    //wrapper.find('button').at(0).simulate('click');
    clickConformBtn();
    expect(wrapper.prop('isBlank')).toBe(true);
    expect(wrapper.prop('isNotBlank')).toBe(true);
    expect(wrapper.find('.control').text()).toBe('前缀: 全部');

    //测试allowBlank为true和allowNotBlank为true时，清空按钮
    open();
    expect(wrapper.prop('value')).toBe('');
    expect(wrapper.prop('isBlank')).toBe(true);
    expect(wrapper.prop('isNotBlank')).toBe(true);
    expect(wrapper.prop('allowBlank')).toBe(true);
    expect(wrapper.prop('allowNotBlank')).toBe(true);
    expect(wrapper.find('button')).toHaveLength(2);
    clickClearBtn();
    expect(wrapper.prop('value')).toBe('');
    expect(wrapper.prop('isBlank')).toBe(false);
    expect(wrapper.prop('isNotBlank')).toBe(false);
    open();
    expect(wrapper.find('[type="checkbox"]').at(0).prop('checked')).toBe(false);
    expect(wrapper.find('[type="checkbox"]').at(1).prop('checked')).toBe(false);
    expect(wrapper.find('.form-control').prop('value')).toBe('');
    clickConformBtn();
});