//@flow

import type { ElementRef, Node }      from 'react';
import React, { Component, Fragment } from 'react';

type Props = {
    children: Node | string,
    dnd?: boolean,
    columnIndex: number,
    onDndMouseDown: (SyntheticMouseEvent<HTMLTableCellElement>)=>void,
    sortDirection?: 'asc' | 'desc',
    sortable?: boolean,
    onSort?: ('asc' | 'desc')=>void
}

class Th extends Component<Props> {
    // _dndMouseDown: boolean;
    _ref = React.createRef<ElementRef<'th'>>();

    render() {
        let {
                children,
                dnd,
                columnIndex,
                onDndMouseDown,
                sortDirection,
                sortable,
                onSort
            } = this.props;

        let thProps = {};

        let dndNode = null;
        if (dnd) {
            dndNode = (
                <Fragment>
                    <span className="dnd-handler" >
                        {/* onMouseDown={() => this._dndMouseDown = true} */}
                        <i className="icon icon-tuodong"/> 
                    </span>
                    <div className='dnd-mask'/>
                </Fragment>
            );
            //thProps.onMouseDown = (e) => this._dndMouseDown && onDndMouseDown(e);
            thProps.onMouseDown = (e) => {
                onDndMouseDown(e);
            };
            thProps['data-column-index'] = columnIndex;
        }

        let sortNode = null;
        if (sortable) {
            // if (sortDirection === 'desc' || sortDirection === 'asc') {
            //     sortNode = (
            //         <span>
            //             <i className={`text-success icon icon-caret-${sortDirection === 'asc' ? 'up' : 'down'}`}/>
            //         </span>
            //     );
            // }
            sortNode = (
                <span className='sort-btn-wrapped'>
                    <i className={`icon icon-caret-up ${sortDirection === 'asc' ? 'text-success active' : ''}`}/>
                    <i className={`icon icon-caret-down ${sortDirection === 'desc' ? 'text-success active' : ''}`}/>
                </span>
            );
            thProps.onClick = () => {
                console.log('sort-btn-click');
                (typeof onSort === 'function') && onSort(sortDirection === 'asc' ? 'desc' : 'asc');
            };
        }

        return (
            <th 
                ref={this._ref}
                className={`${dnd ? 'dnd' : ''} ${sortable ? 'sortable' : ''}`}
                {...thProps}
            >
                {dndNode}
                {children}
                {sortNode}
            </th>
        );
    }

    // _onMouseUp = () => {
    //     this._dndMouseDown = false;
    // };

    // componentDidMount() {
    //     document.addEventListener("mouseup", this._onMouseUp);
    // }

    // componentWillUnmount() {
    //     document.removeEventListener("mouseup", this._onMouseUp);
    // }
}

export default Th;