//@flow

import type { ElementRef }  from 'react';
import React, { Component } from 'react';
import Th                   from './th';
import throttle from 'lodash-es/throttle';

type Props<U> = {
    columns: Column[],
    dnd?: boolean,
    onCheckboxChange?: (SyntheticInputEvent<HTMLInputElement>)=>void,
    checkboxChecked?: boolean,
    sortingColumns?: Object,
    checkbox?: boolean,
    onSort?: (Column, 'asc' | 'desc', number)=>void,
    onColumnPlaceChange?: (Column, number, number)=>void,
    t: (string)=>string
};
type State = {};

function getScrollParent(node: ?HTMLElement): ?HTMLElement {
    const isElement = node instanceof HTMLElement;
    const overflowX = isElement && window.getComputedStyle(node).overflowX;
    const isScrollable = overflowX !== 'visible' && overflowX !== 'hidden';

    if (!node) {
        return null;
    } else if (isScrollable && node.scrollWidth >= node.clientWidth) {
        return node;
    }

    return getScrollParent(node.parentNode) || document.body;
}

class Thead<T> extends Component<Props<T>, State> {
    _onDndMouseDown: (SyntheticMouseEvent<HTMLTableCellElement>, Column)=>void;
    _ref = React.createRef<ElementRef<'thead'>>();
    _thWrapper = React.createRef<ElementRef<'tr'>>();
    _thWrapperWidth = 0;

    _draggingTh: ?HTMLTableCellElement = null;
    _draggingThLeft = 0;
    _draggingThRect = {};

    _nextTh = null;
    _nextThRect = {};

    _prevTh = null;
    _prevThRect = {};

    _sortingTh = null;
    _dragTargetTh = null

    _dragTargetColumn = null;
    _beforeMouseClientX = 0;

    _scrollBox = null;
    _scrollBoxRect = {};

    _isMouseDown = false;

    render() {
        let {
                checkbox,
                checkboxChecked,
                onCheckboxChange,
                columns,
                dnd,
                sortingColumns,
                onSort,
                t
            } = this.props;
        return (
            <thead ref={this._ref}>
            <tr ref={this._thWrapper}>
                {checkbox && <th><input type='checkbox' checked={checkboxChecked} onChange={onCheckboxChange}/></th>}
                {columns.map((i, idx) => {
                    return (<Th 
                                key={i.id} 
                                dnd={dnd} 
                                columnIndex={idx} 
                                sortable={i.sortable}
                                sortDirection={sortingColumns && sortingColumns[i.id]}
                                onSort={(direction) => {
                                    if (typeof onSort === 'function') {
                                        console.log('onSort');
                                        return onSort(i, direction, idx);
                                    }
                                }}
                                onDndMouseDown={(e) => this._onDndMouseDown(e, i)}
                            >
                                {i.name || t(i.id)}
                            </Th>);
                })}
            </tr>
            </thead>
        );
    }

    //计算排序节点的前一个和后一个节点，随着排序节点位置的变化，其前后节点也随之改变
    _resetDraggingThSiblings() {
        console.log('_resetDraggingThSiblings', this._dragTargetTh)
        if (
            this._dragTargetTh 
            && this._dragTargetTh.previousElementSibling 
            && this._dragTargetTh.previousElementSibling.hasAttribute('data-column-index')
            ) {
            this._prevTh = this._dragTargetTh.previousElementSibling;
        } else {
            this._prevTh = null;
        }
        this._prevThRect = this._prevTh ? this._prevTh.getBoundingClientRect() : {};
        if (this._dragTargetTh &&
            this._dragTargetTh.nextElementSibling &&
            this._dragTargetTh.nextElementSibling.hasAttribute('data-column-index')) {
            this._nextTh = this._dragTargetTh.nextElementSibling;
        } else {
            this._nextTh = null;
        }
        this._nextThRect = this._nextTh ? this._nextTh.getBoundingClientRect() : {};
    }

    _onDndMouseDown = (e: SyntheticMouseEvent<HTMLTableCellElement>, dragTargetColumn: Column) => {
        if (!this._thWrapper.current) {
            return;
        }
        this._isMouseDown = true;
        // 记录正在拖拽的元素
        
        // 记录当前鼠标位置
        this._beforeMouseClientX = e.clientX;
        this._dragTargetTh = e.currentTarget;
        this._dragTargetColumn = dragTargetColumn;
        console.log('_onDndMouseDown')
    };
    
    _mouseMove = (e: SyntheticMouseEvent<HTMLElement>) => {
        if (!this._isMouseDown) {
            return;
        }

        let offset = e.clientX - this._beforeMouseClientX;
        console.log('_mouseMove', offset);
        //点击时可能会抖动 水平拖拽超过1认为是拖拽
        if (Math.abs(offset) <= 1) {
            return;
        }
        this._createDraggingTh();

        this._draggingThLeft += offset;
        this._draggingThRect.left += offset;

        this._updateDraggingThPosition(this._draggingThRect.left);

        this._beforeMouseClientX = e.clientX;

        this._scroll(offset);
        this._moveTh();
    };

    _mouseUp = () => {
        console.log('_mouseUp');
        this._isMouseDown = false;

        if (!this._dragTargetTh) {
            return;
        }

        let {onColumnPlaceChange} = this.props;
        console.log('this._dragTargetColumn', this._dragTargetColumn, this._dragTargetColumn && typeof onColumnPlaceChange === 'function')
        if (this._dragTargetColumn && typeof onColumnPlaceChange === 'function' && (this._prevTh || this._nextTh)) {
            let sortingThIndex = parseInt(this._dragTargetTh.dataset.columnIndex, 10);
            let newIndex = sortingThIndex;
            console.log('sortingThIndex', sortingThIndex, newIndex);
            if (!this._prevTh) {
                newIndex = 0;
            } else if (!this._nextTh) {
                newIndex = this.props.columns.length - 1;
            } else {
                let prevThIndex = parseInt(this._prevTh.dataset.columnIndex, 10);
                //$FlowFixMe
                let nextThIndex = parseInt(this._nextTh.dataset.columnIndex, 10);
                if (sortingThIndex < prevThIndex) {
                    newIndex = prevThIndex;
                } else if (sortingThIndex > nextThIndex) {
                    newIndex = nextThIndex;
                }
            }
            console.log('newIndex', sortingThIndex, newIndex);
            if (sortingThIndex !== newIndex) {
                //$FlowFixMe
                onColumnPlaceChange(this._dragTargetColumn, sortingThIndex, newIndex);
            }
        }

        if (this._draggingTh) {
            this._draggingTh.remove();
        }
        if (this._dragTargetTh) {
            //$FlowFixMe
            this._dragTargetTh.className = this._dragTargetTh.className.replace(/ ?drag-target-element/g, '');
        }
        if (this._ref.current) {
            //$FlowFixMe
            this._ref.current.className = this._ref.current.className.replace(/ ?dragging/g, '');
        }
        this._draggingTh = null;
        this._draggingThRect = {};
        this._draggingThLeft = 0;
        this._nextTh = null;
        this._nextThRect = {};
        this._prevTh = null;
        this._prevThRect = {};
        this._scrollBox = null;
        this._scrollBoxRect = {};
        this._dragTargetTh = null;//销毁后不会触发click事件
        this._dragTargetColumn = null;
    };
    
    _createDraggingTh(){
        if(this._draggingTh || !this._dragTargetTh){
            return;
        }
        let thWrapperLeft = this._thWrapper.current.getBoundingClientRect().left;
        let dragTargetThRect = this._dragTargetTh.getBoundingClientRect();
        let {left : draggingThLeft, height, width} = this._draggingThRect = {
            left : dragTargetThRect.left, 
            width : dragTargetThRect.width, 
            height : dragTargetThRect.height
        };

        // 记录前一个和后一个节点
        this._resetDraggingThSiblings();

        /**
         * 复制当前准备拖动的元素，
         * 并将复制后的元素的外观位置和被复制元素保持一致
         */
        let draggingThClone = this._draggingTh = this._dragTargetTh.cloneNode(true);
        draggingThClone.className += ' dragging-element';
        this._draggingThLeft = draggingThLeft - thWrapperLeft;
        this._updateDraggingThPosition(draggingThLeft);
        draggingThClone.style.width = width + 'px';
        draggingThClone.style.height = height + 'px';
        draggingThClone.style.top = dragTargetThRect.top + 'px';

        // 原始元素被一层白色div遮罩
        this._dragTargetTh.className += ' drag-target-element';

        // 给当前thead加一个dragging的类
        if (this._ref.current) {
            this._ref.current.className += ' dragging';
        }

        // 将复制的元素加入到tr里
        if (this._thWrapper.current) {
            this._thWrapperWidth = this._thWrapper.current.offsetWidth;
            this._thWrapper.current.appendChild(this._draggingTh);
        }

        

        // 获取最近的滚动容器
        this._scrollBox = getScrollParent(this._ref.current);
        if (this._scrollBox) {
            this._scrollBoxRect = this._scrollBox.getBoundingClientRect();
        }
    }
    
    _updateDraggingThPosition(newPosition: number) {
        if (this._draggingTh) {
            this._draggingTh.style.left = `${newPosition}px`;
        }
    }

    _scroll(offset: number) {
        if (
            (offset > 0 && this._draggingThRect.left > this._scrollBoxRect.left + this._scrollBoxRect.width - 200) ||
            (offset < 0 && this._draggingThRect.left < this._scrollBoxRect.left + 200)
        ) {
            let scrollWidth = offset > 0 ? offset + 10 : offset - 10;
            this._draggingThLeft += scrollWidth;
            if (this._scrollBox) {
                this._scrollBox.scrollLeft += scrollWidth;
            }

            if (this._nextTh) {
                this._nextThRect = this._nextTh.getBoundingClientRect();
            }

            if (this._prevTh) {
                this._prevThRect = this._prevTh.getBoundingClientRect();
            }
        }
    }

    _moveTh() {
        if (
            this._thWrapper.current &&
            this._nextTh &&
            this._draggingThRect.left > (this._nextThRect.left + this._nextThRect.width / 2)
        ) {
            this._thWrapper.current.insertBefore(this._dragTargetTh, this._nextTh.nextElementSibling);
            this._resetDraggingThSiblings();
        } else if (
            this._thWrapper.current &&
            this._draggingThRect.left < (this._prevThRect.left + this._prevThRect.width / 2)
        ) {
            this._thWrapper.current.insertBefore(this._dragTargetTh, this._prevTh);
            this._resetDraggingThSiblings();
        }
    }

   
    _throttleMouseMove = throttle(this._mouseMove, 50)

    componentDidMount() {
        let {dnd} = this.props;
        if (dnd) {
            document.addEventListener('mouseup', this._mouseUp);
            document.addEventListener('mousemove', throttle(this._throttleMouseMove));
            //document.addEventListener('mousemove', this._mouseMove);
        }
    }

    

    componentWillMount() {
        document.removeEventListener('mouseup', this._mouseUp);
        document.removeEventListener('mousemove', this._throttleMouseMove);
    }

    static defaultProps = {
        t : (key: string) => key
    };
}

export default Thead;