import React, { useEffect, useMemo, useRef } from 'react';
import { Button, Icon, Modal, Dropdown, Menu } from 'udesk-ui';
import styled from 'styled-components';
import { highlight } from '../../utils';
import { MoreOutlined } from '@udesk/icons';
import BatchedActions from '../../batched-actions';

const { highlightText, clearHighlight } = highlight;

const HIGHLIGHT_CLASS_NAME = 'udsek-gm-ui-tree-menu__tree-node--highlight';

const TreeNodeTitle = styled.div`
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
  padding-right: 16px;
  flex: 1;
`;

const TreeNodeWrap = styled.div`
  display: flex;
  justify-content: space-between;
  .${HIGHLIGHT_CLASS_NAME} {
    color: red;
  }
  .batched-actions-dropdown {
    opacity: 0;
  }
  &:hover {
    .batched-actions-dropdown {
      opacity: 1;
    }
  }
`;

const DEFAULT_FIELD_NAMES = {
  title: 'title',
  key: 'key',
  children: 'children',
};

const defaultConfirm = {
  icon: <Icon type="ExclamationCircleOutlined" antdIcon={true} />,
};

const TreeNode = (props: any) => {
  const {
    menu: menuProps = [],
    switcherIconPosition,
    switcherIcon,
    fieldNames: customizeFieldNames,
    titleRender,
    treeNodeActionMenuTrigger,
    highlightKeyword,
    getPopupContainer,
    ...node
  } = props;

  const treeNodeInstance = useRef<HTMLDivElement>(null);

  const hiddenExtra = useMemo(
    () =>
      menuProps.every((item: any) =>
        typeof item.disabled === 'function' ? item.disabled(node) : item.disabled,
      ),
    [menuProps, node],
  );
  const fieldNames: typeof DEFAULT_FIELD_NAMES = useMemo(
    () => Object.assign({}, DEFAULT_FIELD_NAMES, customizeFieldNames),
    [customizeFieldNames],
  );
  const isLeaf = useMemo(
    () => !hasChildren(node, fieldNames.children),
    [node, fieldNames.children],
  );
  useEffect(() => {
    treeNodeInstance.current && clearHighlight(treeNodeInstance.current);
    if (highlightKeyword) {
      treeNodeInstance.current &&
        highlightText(treeNodeInstance.current, highlightKeyword, {
          className: HIGHLIGHT_CLASS_NAME,
        });
    }
  }, [highlightKeyword]);

  return (
    <TreeNodeWrap ref={treeNodeInstance}>
      <TreeNodeTitle
        className="tree-node-title"
        title={typeof node[fieldNames.title] === 'string' ? node[fieldNames.title] : undefined}
      >
        {typeof titleRender === 'function' ? titleRender(node) : node[fieldNames.title]}
      </TreeNodeTitle>
      <div onClick={(e) => e.stopPropagation()}>
        {hiddenExtra || (
          <BatchedActions
            menu={menuProps}
            trigger={treeNodeActionMenuTrigger}
            getPopupContainer={getPopupContainer}
            {...node}
          >
            <Button size="small" type="text" icon={<MoreOutlined />} />
          </BatchedActions>
        )}
      </div>
      {switcherIconPosition === 'right' && (
        <div className="switcher-icon-right">
          {!isLeaf && (switcherIcon || <Icon type="DownOutlined" antdIcon={true} />)}
        </div>
      )}
    </TreeNodeWrap>
  );
};

export { TreeNode };

function hasChildren(node: any, childrenPropsName: string): boolean {
  return Array.isArray(node[childrenPropsName]) && node[childrenPropsName].length > 0;
}
