## Demo

```tsx
/**
 * title: 基础使用
 * desc: treeData字段与Udesk-ui保持一致，扩展了treeNodeActionMenu字段以快捷的创建操作按钮及其交互
 * background: 'rgb(240, 242, 245)'
 */

import React from 'react';
import { TreeMenu } from 'udesk_gm_ui';
import { Input } from 'udesk-ui';

export default () => {
  const treeData = [
    {
      title: '不可删除节点',
      key: 1,
      id: 1,
      default: 1,
    },
    {
      title: '我超级长我超级长我超级长我超级长我超级长我超级长',
      key: 123,
      id: 123,
      children: [
        {
          title: '子节点127',
          key: 127,
          id: 127,
          children: [
            {
              title: '子节点128',
              key: 128,
              id: 128,
            },
          ],
        },
      ],
    },
    {
      title: '啦啦啦啦',
      key: 124,
      id: 124,
    },
  ];
  const treeNodeActionMenu = [
    {
      label: '编辑',
      onClick: (item) => console.log(item),
    },
    {
      label: '删除',
      danger: true,
      onClick: (item) => console.log(item),
      disabled: (item) => Boolean(item.default),
    },
  ];
  const [keyword, setKeyword] = React.useState('');
  return (
    <div style={{ display: 'flex', background: '#fff', color: '#000' }}>
      <div style={{ width: 300 }}>
        <Input.Search onSearch={(value) => setKeyword(value)} />
        <TreeMenu
          // titleRender={(item) => <div style={{ color: 'blue' }}>{item.title}</div>}
          highlightKeyword={keyword}
          treeData={treeData}
          treeNodeActionMenu={treeNodeActionMenu}
          treeNodeActionMenuTrigger={['click']}
        />
      </div>
      <div
        style={{
          flex: 1,
          borderLeft: '1px solid rgb(240, 242, 245)',
          padding: 16,
          overflow: 'auto',
        }}
      >
        <p>关于treeNodeActionMenu属性：</p>
        <p>1. disabled字段可以编写函数来动态置灰操作按钮</p>
        <p>2. danger字段为true时会触发 Modal.confirm 以进行二次确认，onClick会在二次确认后触发</p>
        <p>3. confirm字段可以重写弹窗中的内容</p>
      </div>
    </div>
  );
};
```

```tsx
/**
 * title: 格式化扁平数据
 * desc: TreeMenu组件上挂有专门用于格式化扁平数据到treeData数据类型的静态方法：formatTreeDataFromArrayToTree
 * background: 'rgb(240, 242, 245)'
 */

import React, { useState, useEffect } from 'react';
import { TreeMenu } from 'udesk_gm_ui';

const originData = [
  {
    title: '根节点1',
    key: 1,
    id: 1,
    default: 1,
    parentId: null,
  },
  {
    title: '子节点1-1',
    key: 4,
    id: 4,
    parentId: 2,
  },
  {
    title: '子节点1',
    key: 2,
    id: 2,
    parentId: 1,
  },
  {
    title: '子节点2',
    key: 3,
    id: 3,
    parentId: 1,
  },
  {
    title: '超长子节点3超长子节点3超长子节点3超长子节点3超长子节点3',
    key: 6,
    id: 6,
    parentId: 1,
  },
  {
    title: '根节点2',
    key: 5,
    id: 5,
    parentId: null,
  },
];

export default () => {
  const [treeData, setTreeData] = useState([]);
  useEffect(() => {
    setTimeout(() => {
      setTreeData(TreeMenu.formatTreeDataFromArrayToTree(originData, null, 'parentId'));
    }, 1000);
  }, []);
  const treeNodeActionMenu = [
    {
      label: '编辑',
      onClick: (item) => console.log(item),
    },
    {
      label: '删除',
      danger: true,
      onClick: (item) => console.log(item),
      disabled: (item) => Boolean(item.default),
    },
  ];
  return (
    <div style={{ display: 'flex', background: '#fff', color: '#000', height: 150 }}>
      <div style={{ width: 300 }}>
        <TreeMenu
          treeData={treeData}
          onSelect={(...args) => console.log(args)}
          treeNodeActionMenu={treeNodeActionMenu}
          defaultExpandAll
        />
      </div>
      <div
        style={{
          flex: 1,
          borderLeft: '1px solid rgb(240, 242, 245)',
          padding: 16,
          overflow: 'auto',
        }}
      >
        <p>TreeMenu高度默认为父容器高度的100%，如果父容器高度有限制，超出部分会滚动</p>
        <p>defaultExpandAll属性在异步加载数据的情况下可以默认展开全部了</p>
        <p>
          TreeMenu.formatTreeDataFromArrayToTree方法可以把扁平数据转成树，且兼容单根和多根的情况
        </p>
      </div>
    </div>
  );
};
```

```tsx
/**
 * title: 展开、收起图标显示到右侧
 * desc: 展示一下switcherIconPosition字段为‘right’时的效果，以及indentUnitWidth字段的使用
 * background: 'rgb(240, 242, 245)'
 */

import React, { useState, useEffect } from 'react';
import { TreeMenu } from 'udesk_gm_ui';

const originData = [
  {
    title: '根节点1',
    key: 1,
    id: 1,
    default: 1,
    parentId: null,
  },
  {
    title: '子节点1-1',
    key: 4,
    id: 4,
    parentId: 2,
  },
  {
    title: '超长子节点1超长子节点1超长子节点1超长子节点1超长子节点1',
    key: 2,
    id: 2,
    parentId: 1,
  },
  {
    title: '子节点2',
    key: 3,
    id: 3,
    parentId: 1,
  },
  {
    title: '超长子节点3超长子节点3超长子节点3超长子节点3超长子节点3',
    key: 6,
    id: 6,
    parentId: 1,
  },
  {
    title: '根节点2',
    key: 5,
    id: 5,
    parentId: null,
  },
];

export default () => {
  const [treeData, setTreeData] = useState([]);
  useEffect(() => {
    setTimeout(() => {
      setTreeData(TreeMenu.formatTreeDataFromArrayToTree(originData, null, 'parentId'));
    }, 1000);
  }, []);
  const treeNodeActionMenu = [
    {
      label: '编辑',
      onClick: (item) => console.log(item),
    },
    {
      label: '删除',
      danger: true,
      onClick: (item) => console.log(item),
      disabled: (item) => Boolean(item.default),
    },
  ];
  return (
    <div style={{ display: 'flex', background: '#fff', color: '#000', height: 150 }}>
      <div style={{ width: 300 }}>
        <TreeMenu
          checkable={true}
          switcherIconPosition="right"
          indentUnitWidth={14}
          treeData={treeData}
          onSelect={(...args) => console.log(args)}
          treeNodeActionMenu={treeNodeActionMenu}
          defaultExpandAll
        />
      </div>
      <div
        style={{
          flex: 1,
          borderLeft: '1px solid rgb(240, 242, 245)',
          padding: 16,
          overflow: 'auto',
        }}
      >
        <p>Tips：</p>
        <p>
          1.
          如果写的是一棵组织架构树（包含部门和员工），且同一员工在不同部门都出现了，想要勾选其中一个部门的员工A就达到选中所有部门下的员工A的话，不要使用‘部门id+员工id’的形式拼接员工节点的key。
        </p>
        <p>
          2.
          因为部门id和员工id可能出现重复的情况，只要在对应的key前加入标识进行区分即可，比如：`user-`
          + userId。
        </p>
      </div>
    </div>
  );
};
```

```tsx
/**
 * title: fieldNames & highlightKeyword
 * desc: fieldNames用于动态设置treeData中的取的字段名，与Udesk-ui保持一致
 * background: 'rgb(240, 242, 245)'
 */

import React from 'react';
import { TreeMenu } from 'udesk_gm_ui';
import { Icon, Input } from 'udesk-ui';

export default () => {
  const treeData = [
    {
      name: '不可删除节点',
      key: 1,
      id: 1,
      default: 1,
    },
    {
      name: '我超级长我超级长我超级长我超级长我超级长我超级长',
      key: 123,
      id: 123,
      childItems: [
        {
          name: '子节点127',
          key: 127,
          id: 127,
          // isLeaf: false,
          childItems: [
            {
              name: '子节点128',
              key: 128,
              id: 128,
            },
          ],
        },
      ],
    },
    {
      name: 'xxxxxxx',
      key: 124,
      id: 124,
      // isLeaf: true
    },
  ];
  const treeNodeActionMenu = [
    {
      label: '编辑',
      onClick: (item) => console.log(item),
    },
    {
      label: '删除',
      danger: true,
      onClick: (item) => console.log(item),
      disabled: (item) => Boolean(item.default),
    },
  ];
  const [keyword, setKeyword] = React.useState('');
  return (
    <div style={{ display: 'flex', background: '#fff', color: '#000' }}>
      <div style={{ width: 250 }}>
        <Input.Search onSearch={(value) => setKeyword(value)} />

        <TreeMenu
          treeData={treeData}
          treeNodeActionMenu={treeNodeActionMenu}
          fieldNames={{ children: 'childItems', title: 'name' }}
          highlightKeyword={keyword}
          icon={(props) => {
            if (props.childItems) {
              if (props.expanded) return <Icon type="AlertOutlined" antdIcon={true} />;
              return <Icon type="AimOutlined" antdIcon={true} />;
            }
          }}
        />
      </div>
      <div style={{ width: 250 }}>
        <TreeMenu
          treeData={treeData}
          treeNodeActionMenu={treeNodeActionMenu}
          fieldNames={{ children: 'childItems', title: 'name' }}
          switcherIconPosition="right"
          titleRender={(node) => {
            return <div style={{ color: 'red' }}>{node.name}</div>;
          }}
        />
      </div>
      <div
        style={{
          flex: 1,
          borderLeft: '1px solid rgb(240, 242, 245)',
          padding: 16,
          overflow: 'auto',
        }}
      >
        <p>设置fieldNames字段后需注意：</p>
        <p>1. 未设置switcherIconPosition时，是不是叶子节点由udesk-ui判断</p>
        <p>2. 设置switcherIconPosition后，是不是叶子节点由gm-ui判断</p>
        <p>3. 对于2的情况，有bug随时反馈</p>
      </div>
    </div>
  );
};
```

## API

| 参数 | 说明 | 类型 | 默认值 | 版本 |
| --- | --- | --- | --- | --- |
| treeData | 树渲染时使用的源数据 | TreeNode[]（见 Interface 部分） | - |  |
| treeNodeActionMenu | 渲染树节点后面的操作项的源数据，其中`danger`字段为`true`时会触发 Modal.confirm 以进行二次确认 | ActionMenuItem[]（见 Interface 部分） | - |  |
| treeNodeActionMenuTrigger | 渲染树节点后面的操作项浮层的触发方式 | Array<'click' \| 'hover' \| 'contextMenu'> | ['hover'] |  |
| switcherIconPosition | 树节点的展开图标位置 | 'left' \| 'right' | 'left' |  |
| indentUnitWidth | 树节点缩进单元的大小 | number | 24 |  |
| highlightKeyword | 搜索关键字，会进行树节点的筛选和高亮 | string | - |  |
| expandAction | 目录展开逻辑，可选：false \| click \| doubleClick | string \| boolean | click |  |

其余 api 与`Udesk-ui`的`Tree`组件保持一致

## 静态方法

| 参数 | 说明 | 类型 | 默认值 | 版本 |
| --- | --- | --- | --- | --- |
| formatTreeDataFromArrayToTree | 用于格式化扁平数据到 treeData 数据类型 | FormatTreeDataFromArrayToTree（见 Interface 部分） | - |  |

## Interface

```ts
type TreeNode = {
  key: string | number;
  title: ReactNode;
  children?: TreeNode[];
  disabled?: boolean;
  selectable?: boolean;
  [key: string]: any;
};

interface ActionMenuItem {
  label: string;
  onClick: (item: TreeNode) => void;
  danger?: boolean;
  disabled?: boolean | ((item: TreeNode) => boolean);
  confirm?: Omit<ModalFuncProps, 'onOk'>; //与udesk-ui中Modal.confirm除onOk外的api保持一致
}

type FormatTreeDataFromArrayToTree = (
  data: TreeNode[] = [], // 后端接口返回的扁平化数据
  treeRootParentId: string | number | null, // 根节点的parentId的值，看接口返回的具体值
  parentIdPropsName: string = 'pid', // 表示节点的parentId的字段名
  idPropsName: string = 'id', // 表示节点的id的字段名
): TreeMenuProps['treeData']
```
