import { TreeMenuProps, TreeNode } from '../types';

/**
 * 用于格式化扁平数据到 treeData 数据类型
 *
 * @param {TreeNode[]} [data=[]] 后端接口返回的扁平化数据
 * @param {(string | number | null)} treeRootParentId 根节点的parentId的值，看接口返回的具体值
 * @param {string} [parentIdPropsName='pid'] 表示节点的parentId的字段名
 * @param {string} [idPropsName='id'] 表示节点的id的字段名
 * @return {*}  {TreeMenuProps['treeData']} TreeMenu组件能接收的treeData值
 */
export default function formatTreeDataFromArrayToTree(
  data: TreeNode[] = [],
  treeRootParentId: string | number | null,
  parentIdPropsName: string = 'pid',
  idPropsName: string = 'id',
): TreeMenuProps['treeData'] {
  const idMapping = new Map();
  data.forEach((el, i) => {
    el.isLeaf = true;
    idMapping.set(el[idPropsName], i);
  });
  let root: { children: TreeMenuProps['treeData'] } = { children: [] };
  data.forEach((el) => {
    // 判断根节点
    if (el[parentIdPropsName] === treeRootParentId) {
      root.children!.push(el);
    }
    // 用映射表找到父元素
    const parentEl = data[idMapping.get(el[parentIdPropsName])];
    // 把当前元素添加到父元素的`children`数组中
    if (parentEl) {
      parentEl.children = [...(parentEl.children || []), el];
      parentEl.isLeaf = false;
    }
  });
  return root.children;
}
