import {
    SparkMD5
} from '../../../udesk/ref-npm-modules/plugins/utils.crypto.md5';
import { installPlugin, globalInstall } from '../../plugin-base';

const HASHER_FIELD_NAME_SYMBOL = Symbol("_hasher")

const PLUGIN_PATH = "utils.crypto.md5";
const PLUGIN = {
    hash,
    getHasher
};

export default function install(udesk) {
    installPlugin(udesk, PLUGIN_PATH, PLUGIN);
}
globalInstall(PLUGIN_PATH, PLUGIN);


function hash(content) {
    let hasher = this.getHasher();
    let hash = hasher.update(content).digest();
    hasher.destroy();
    return hash;
}

function getHasher() {
    return new Md5Hasher(new SparkMD5());
}

class Md5Hasher {
    constructor(innerHasher) {
        this[HASHER_FIELD_NAME_SYMBOL] = innerHasher;
    }

    update(content) {
        content = stringify(content);
        this[HASHER_FIELD_NAME_SYMBOL].append(content);
        return this;
    }

    digest() {
        return this[HASHER_FIELD_NAME_SYMBOL].end();
    }

    reset() {
        this[HASHER_FIELD_NAME_SYMBOL].reset();
        return this;
    }

    destroy() {
        return this[HASHER_FIELD_NAME_SYMBOL].destroy();
    }
}

function stringify(content) {
    if (content == null || (typeof content === "number" && isNaN(content))) {
        content = "";
    }
    if (typeof content !== "string") {
        if (typeof content === "object" || Array.isArray(content)) {
            content = JSON.stringify(content);
        } else {
            content = content.toString();
        }
    }
    return content;
}
