const HASHER_FIELD_NAME_SYMBOL = Symbol("_hasher")

export function hash(content) {
    let hasher = this.getHasher();
    let hash = hasher.update(content).digest();
    hasher.destroy();
    return hash;
}

export function hmacHash(content, key) {
    let hasher = this.getHasher(key);
    let hash = hasher.update(content).digest();
    hasher.destroy();
    return hash;
}

export class ShaHasher {
    constructor(innerHasher) {
        this[HASHER_FIELD_NAME_SYMBOL] = innerHasher;
    }

    update(content) {
        content = stringify(content);
        this[HASHER_FIELD_NAME_SYMBOL].update(content);
        return this;
    }

    digest() {
        return this[HASHER_FIELD_NAME_SYMBOL].hex();
    }

    reset() {
        return this;
    }

    destroy() {

    }
}

export function stringify(content) {
    if (content == null || (typeof content === "number" && isNaN(content))) {
        content = "";
    }
    if (typeof content !== "string") {
        if (typeof content === "object" || Array.isArray(content)) {
            content = JSON.stringify(content);
        } else {
            content = content.toString();
        }
    }
    return content;
}
