import {
    imurmurhash as Imurmurhash
} from '../../../udesk/ref-npm-modules/plugins/utils.object.hash';
import { installPlugin, globalInstall } from '../../plugin-base';

const IMURMURHASH_FIELD_NAME_SYMBOL = Symbol("_imurmurhash");
const PLUGIN_PATH = "utils.object";
const PLUGIN = {
    hash,
    getHasher
};
export default function install(udesk) {
    installPlugin(udesk, PLUGIN_PATH, PLUGIN);
}
globalInstall(PLUGIN_PATH, PLUGIN);


function hash(content) {
    let hasher = this.getHasher();
    let hash = hasher.update(content).digest();
    hasher.destroy();
    return hash;
}

function getHasher() {
    return new ObjectHasher(new Imurmurhash());
}

class ObjectHasher {
    constructor(innerHasher) {
        this[IMURMURHASH_FIELD_NAME_SYMBOL] = innerHasher;
    }

    update(content) {
        content = stringify(content);
        this[IMURMURHASH_FIELD_NAME_SYMBOL].hash(content);
        return this;
    }

    digest() {
        return this[IMURMURHASH_FIELD_NAME_SYMBOL].result();
    }

    reset(seed) {
        this[IMURMURHASH_FIELD_NAME_SYMBOL].reset(seed);
        return this;
    }

    destroy() {

    }
}

function stringify(content) {
    if (content == null || (typeof content === "number" && isNaN(content))) {
        content = "";
    }
    if (typeof content !== "string") {
        if (typeof content === "object" || Array.isArray(content)) {
            content = JSON.stringify(content);
        } else {
            content = content.toString();
        }
    }
    return content;
}
