import React from 'react';
import PropTypes from 'prop-types';
import UdeskReact from '../react';
import {
    NotImplementedError
} from 'udesk-web-toolkits/src/udesk/error';

class EmbedOuterComponent extends React.Component {
    static defaultProps = {
        component: null,
        options: null,
    }

    static propTypes = {
        options: PropTypes.object
    }

    privates = {
        componentInstance: null
    }

    init() {
        let options = this.props.options;
        this.privates.options = options;
    }

    updateProps(options) {
        this.privates.options = options;
        this.actions.update();
    }

    callComponentApi(name, ...args) {
        if (this.privates.componentInstance) {
            if (Object.prototype.hasOwnProperty.call(this.privates.componentInstance,name) || typeof this.privates.componentInstance[name] === "function") {
                return this.privates.componentInstance[name].call(this.privates.componentInstance, ...args);
            } else {
                throw new NotImplementedError(name);
            }
        }
    }

    actions = {
        onReactComponentCreated(instance) {
            this.privates.componentInstance = instance;
        },
        onReactComponentDestroyed() {
            this.privates.componentInstance = null;
        }
    }

    render() {
        let {
            actions,
            privates
        } = this;
        let {
            component: Component,
        } = this.props;
        let options = privates.options;
        if (Component == null) {
            return "";
        }
        if (options == null) {
            options = {};
        }
        return (<Component {...options} onCreated={actions.onReactComponentCreated} onDestroyed={actions.onReactComponentDestroyed} />);
    }
}

export default UdeskReact.udeskify(EmbedOuterComponent);