
import React from 'react';
import { render, unmountComponentAtNode } from 'react-dom';
import {
    NotImplementedError
} from 'udesk-web-toolkits/src/udesk/error';
import ReactComponentProxy from "./react-component-proxy";

const SDK_COMPONENT_GLOBAL_NAMESPACE = "__SDK_COMPONENT_GLOBAL_NAMESPACE__";

class ReactComponentSdkBaseClass {
    static renderComponent(componentName, container, options = {}) {
        if (this.ComponentsMapping == null) {
            throw new NotImplementedError("ComponentsMapping");
        }
        let component = getComponent(this.ComponentsMapping, componentName);
        if (component) {
            if (container == null) {
                container = document.body;
            }

            let outerInstanceInstance = null;
            let onReactComponentProxyCreated = (instance) => {
                outerInstanceInstance = instance;
            };
            let onReactComponentProxyDestroyed = () => {
                outerInstanceInstance = null;
            };
            render(<ReactComponentProxy component={component} options={options} onCreated={onReactComponentProxyCreated} onDestroyed={onReactComponentProxyDestroyed} />, container);
            return {
                updateOptions: function (newOptions) {
                    if (outerInstanceInstance != null && typeof outerInstanceInstance.updateProps === "function") { 
                        outerInstanceInstance.updateProps(newOptions);
                    }
                },
                call(name, ...args) {
                    if (outerInstanceInstance) {
                        return outerInstanceInstance.callComponentApi(...arguments);
                    }
                }
            };
        } else {
            throw new NotImplementedError(componentName);
        }
    }

    static destroyComponent(container) {
        if (container == null) {
            return;
        }
        unmountComponentAtNode(container);
    }

    static get initGlobalNameSpace() {
        return function (sdkClass, options) {
            if (sdkClass == null) {
                throw new ArgumentNullError(`initGlobalNameSpace.sdkClass`);
            }
            if (typeof options !== "object") {
                options = {
                    defaultNamespace: null
                };
            }
            let globalNamespace = window[SDK_COMPONENT_GLOBAL_NAMESPACE];

            if (typeof globalNamespace === "function") {
                globalNamespace = globalNamespace();
            }
            if (globalNamespace == null) {
                if (options.defaultNamespace) {
                    globalNamespace = options.defaultNamespace;
                }
                else {
                    globalNamespace = "";
                }
            }

            if (globalNamespace !== "") {
                window[globalNamespace] = sdkClass;
            }

            if (typeof window.__SDK_LOADED_CALLBACK__ === "function") {
                window.__SDK_LOADED_CALLBACK__(sdkClass);
            }
        };
    }
}

function getComponent(mappings, key) {
    if (mappings == null || typeof mappings !== "object") {
        return mappings;
    }
    if (key == null || key === "") {
        return null;
    }
    if (Object.prototype.hasOwnProperty.call(mappings,key)) {
        return mappings[key];
    }
    let keys = key.split(".");
    if (keys.length > 0) {
        if (Object.prototype.hasOwnProperty.call(mappings,keys[0])) {
            return getComponent(mappings[keys[0]],keys.slice(1).join("."));
        }
    }
    return null;
}

export default ReactComponentSdkBaseClass;

export {
    ReactComponentSdkBaseClass
};