import React, { useCallback, useState, useMemo, useRef } from 'react';
import { Button, Col, Modal, Row } from 'udesk-ui';

import { useTranslation } from 'react-i18next';

import UploadBox from '@udesk-webapp/upload-box';
import css from './index.module.less';
import CheckOutlined from '@ant-design/icons/CheckOutlined';
import CloseCircleFilled from '@ant-design/icons/CloseCircleFilled';

type UploadImgList = {
    src: string;
    isSelect: boolean;
};

interface ModalProps {
    show: boolean;
    close: () => void;
    addImg: (imgList: UploadImgList[]) => void;
    marking?: string;
}

function UploadModal({ marking, ...props }: ModalProps) {
    const uploadRef = useRef<any>();
    // 上传的图片列表
    const [imageFiles, setImageFiles] = useState<any[]>([]);
    const [selectedImages, setSelectedImages] = useState<any[]>([]);

    //多语言配置
    const { t } = useTranslation();

    // 点击插入  筛选出被选择的图片   将选择的图片 插入富文本编辑
    function handleOk() {
        // props.addImg(imgList.filter(item => item.isSelect));
        if (selectedImages) {
            props.addImg(
                selectedImages
                    ?.filter((i) => i.url)
                    .map((i) => ({ src: i.url!, isSelect: true }))
            );
            // 插入后清空
            uploadRef.current.deleteAll();
            setSelectedImages([]);
            setImageFiles([]);
        }
    }

    // 点击取消 关闭弹窗
    function handleCancel() {
        props.close();
    }

    // 删除图片
    function deleteImg(e: any, index: number) {
        e.stopPropagation();
        const delItem = imageFiles[index];
        setImageFiles((v) => {
            if (!v) {
                return v;
            }
            const result = [...v];
            result.splice(index, 1);
            return result;
        });
        //同时删除antdUpload中的数据，
        imageFiles && uploadRef.current.deleteItem(delItem);
        // 删除select
        setSelectedImages((v = []) => {
            if (v.includes(delItem)) {
                return v.filter((i) => i !== delItem);
            } else {
                return v;
            }
        });
    }

    // 选择图片  显示右下角对勾
    function selectImg(imageFile: any) {
        setSelectedImages((v = []) => {
            if (v.includes(imageFile)) {
                return v.filter((i) => i !== imageFile);
            } else {
                return [...v, imageFile];
            }
        });
    }

    const onChange: React.ComponentProps<
        typeof UploadBox
    >['onChange'] = useCallback((fileList) => {
        setImageFiles(fileList);
        setSelectedImages(fileList);
    }, []);

    // 弹窗页脚
    const footer = useMemo(() => {
        return (
            <div className={css['modal-btn']}>
                <UploadBox
                    className={css['upload-btn']}
                    {...props}
                    value={imageFiles}
                    marking={marking || 'web.ts.ticket_reply'}
                    onChange={onChange}
                    noAttachment={false}
                    showUploadList={false}
                    accept='image/jpeg,image/jpg,image/png'
                    ref={uploadRef}
                >
                    <Button>{t('uploader.uploadImg')}</Button>
                </UploadBox>

                <Button key='back' onClick={handleCancel}>
                    {t('cancel')}
                </Button>
                <Button key='submit' type='primary' onClick={handleOk}>
                    {t('uploader.insertImg')}
                </Button>
            </div>
        );
    }, [imageFiles, selectedImages]);

    return (
        <div>
            <Modal
                visible={props.show}
                title={t('uploader.uploadImg')}
                onOk={handleOk}
                onCancel={handleCancel}
                footer={footer}
                width={600}
                zIndex={9999}
            >
                <div className={css['modal-content']}>
                    <Row gutter={[16, 24]}>
                        {imageFiles?.map((item, index) => {
                            return (
                                <Col span={8} key={item.url}>
                                    <div
                                        className={css['img-wrapper']}
                                        onClick={() => selectImg(item)}
                                    >
                                        <img src={item.url} alt={item.url} />

                                        <div
                                            className={css['delete-icon']}
                                            onClick={(e) => deleteImg(e, index)}
                                        >
                                            <CloseCircleFilled />
                                        </div>

                                        {selectedImages?.includes(item) && (
                                            <div className={css['select-icon']}>
                                                <CheckOutlined />
                                            </div>
                                        )}
                                    </div>
                                </Col>
                            );
                        })}
                    </Row>
                </div>
            </Modal>
        </div>
    );
}

export default UploadModal;
