import {extend, isEmpty, isObject, pick} from "lodash";
import * as imLogUtils from '@udesk-webapp/im-log/es/utilities'
const {replaceWechatEmotions} = imLogUtils;

export function handleMessageData (msg: any, dataType = '', wechatSuffix)  {
    const _list: any[] = []
    if (dataType === 'robot') {
        msg.isRobot = true
    }
    _list.push(msgReceiveHandle(msg, true, wechatSuffix))
    //由于无用消息要单独作为一条消息展示，所以将评价无用的消息复制一条出来作为无用消息，新建类型为unsatisfied。即后端一条数据前端要渲染成两条消息的效果
    if (msg?.content?.data?.unsatisfied_tags_result) {
        const _msg = {...msg}
        _msg.id = _msg.id + '_unsatisfied'
        _msg.message_id = _msg.message_id + '_unsatisfied'
        _msg.type = "unsatisfied"

        _list.push(msgReceiveHandle(_msg, true))
    }
    return _list
}

function msgReceiveHandle(_msgObj: any, isLog?: boolean, wechatSuffix?: boolean) {
    let msgObj = Object.assign({..._msgObj}, {
        richContent: _msgObj.content
    })
    if (typeof msgObj.is_receive === 'undefined') {
        msgObj.is_receive = msgObj.sender === 'customer';
    }


    // log | xmpp | send
    // log返回的content是一个字符串,将字符串序列化之后的属性, 附加在msgObj身上
    if (isLog) {
        let content = msgObj.content;
        if (typeof content === 'string') {
            try {
                content = JSON.parse(content);
            } catch (e) {
                console.log(e)
            }
        }
        if (isObject(content)) {
            if (msgObj.log_type === 'to_customer') {
                (content as any).read_status = msgObj.read_status
            }
            msgObj = extend(msgObj, content);
            const data = msgObj.data;

            if (isObject(data)) {
                msgObj = extend(msgObj, data);
            }
        }
    } else {
        const data = msgObj.data;

        if (isObject(data)) {
            msgObj = extend(msgObj, data);
        }
    }
    if (_msgObj.type === 'unsatisfied') {
        msgObj.type = "unsatisfied"
    }
    // ["content", "filename", "filesize", "file_policy", "duration", "recognition", "timeout_freq", "timeout_auto_msg", "msg_close"].forEach(function(key) {
    //     !data[key] && msgObj[key] && (data[key] = msgObj[key]);
    // });

    // msgObj = extend({}, msgObj, data);
    // 机器人消息特殊处理
    if (msgObj.isRobot) {
        let logContent;
        msgObj.agent_id = 'robot';
        msgObj.id = 'robot' + msgObj.id;

        if (!(['image', 'link', 'audio', 'rich', 'custom_card', 'unsatisfied', 'task_button', 'map','video','miniprogrampage'].includes(msgObj.type))) {
            msgObj.type = 'message';
            msgObj.content = msgObj.content + '';
        }

        if (['link', 'custom_card'].includes(msgObj.type)) {
            try {
                logContent = JSON.parse(msgObj.content);
            } catch (e) {
                logContent = msgObj.content;
            }
            msgObj.faviconUrl = logContent.faviconUrl;
            msgObj.title = logContent.title;
            msgObj.answerUrl = logContent.answerUrl;
            msgObj.cardList = logContent.cardList;
            msgObj.cardListType = logContent.cardListType;
        }
        msgObj.content = msgObj.content || '';
        try {
            const content_raw = JSON.parse(msgObj.content_raw)
            if (content_raw.robotAnswer[0].msgType === 'iframe_link') {
                msgObj.type = 'text'
                msgObj.content = '点击提交工单'
            }else if (content_raw.robotAnswer[0].msgType === 'order_list'){
                msgObj.type = 'text'
                msgObj.content = '点击进入工单查询'
            }
        }catch(e) {
            console.log('content_raw反序列化失败')
        }


    }
    //机器人消息全部进行表情替换，人工消息只进行富文本的表情替换，文本的在文本渲染组件中处理了
    if ((msgObj.isRobot || msgObj.type === 'rich') && typeof msgObj.content === 'string' && msgObj.content.search('/:') > -1) {
        msgObj.content = replaceWechatEmotions(msgObj.content);
    }

    if (isEmpty(msgObj.content) && msgObj.type === 'image') {
        msgObj.content = '';
    }

    const senderName = calculateSenderName(msgObj, wechatSuffix);
    const now = + new Date()


    const _id = msgObj.id || msgObj.message_id
    return {
        ...pick(msgObj,
            'content',
            'platform',
            'date',
            'is_receive',
            'auto',
            'sender',
            'agent_id',
            'customer_id',
            'agent_nick_name',
            'user_nick_name',
            'created_time',
            'version',
            'monitor',
            'inviter',
            'send_status',
            'rollback_at',
            'timeout_freq',
            'respone',
            'pic_url',
            'desc',
            'origin_url',
            'name',
            'url',
            'faviconUrl',
            'title',
            'answerUrl',
            'cardList',
            'is_welcome',
            'richContent',
            'cardListType'
        ),
        type: msgObj.type || 'message',
        id: _id,
        msg_id: msgObj.message_id,
        responseId:msgObj.log_id || msgObj.id || '',
        message_display_status: msgObj.message_display_status,
        data: msgObj.data || {},
        sender_name: senderName,
        is_save: msgObj.isSave,
        im_sub_session_id: parseInt(msgObj.im_sub_session_id, 10),
        created_at: msgObj.created_at || now,
        updated_at: msgObj.updated_at || now,
        isRobot: msgObj.isRobot,
        // 文件
        filename: msgObj.filename || '',
        filesize: msgObj.filesize || '',
        filePreviewUrl:msgObj.previewUrl || '',
        fileext: msgObj.fileext || '',
        file_policy: msgObj.file_policy || '',
        filetype: msgObj.filetype || '',

        invitation: msgObj.invitation || '',
        push_type: msgObj.push_type,
        status: msgObj.status || '',
        respone: msgObj.respone || '',
        recognition: msgObj.recognition || '',
        font: msgObj.font || '',
        log_type: msgObj.log_type || '',
        active_guest: msgObj.active_guest || '',
        read_status: msgObj.read_status === undefined ? false : msgObj.read_status,
        translation: msgObj.translation || '',
        form_id: msgObj.form_id || '',
        timeout_auto_msg: msgObj.timeout_auto_msg || false,
        msg_close: msgObj.msg_close || false,
        customer_close: msgObj.customer_close || false,
        fileId: msgObj.fileId || '',
        open_robot_recommends: msgObj.open_robot_recommends || false,
        event_text: msgObj.event_text || '',
        notShow: msgObj.notShow || false,
        role_hidden: msgObj.role_hidden,
        content_raw: msgObj.content_raw,
        is_wechat_fail:(msgObj.send_status === 'fail'&& msgObj.respone && msgObj.platform === 'wechat') ? true:false,
        hideMsg:msgObj.hideMsg || false,
        transactionId:msgObj.transactionId || '',
        quote_message: msgObj.quote_message || '',
        quote_message_id: msgObj.quote_message_id || '',
        quote_message_hide_status: msgObj.quote_message_hide_status || false,
        quote_message_rollback_status: msgObj.quote_message_rollback_status || false,
        quote_message_from_colleague: msgObj.quote_message_from_colleague || false,
        customer_nick_name: msgObj.customer_nick_name || false,
        atList: msgObj.atList,
        subject: msgObj?.subject || '',
        smtpSettingId: msgObj?.smtpSettingId || '',
        thumbnail: msgObj?.thumbnail || '',
        from: msgObj?.from || '',
        to: msgObj?.to || '',
        cc: msgObj?.cc || ''
    } as IMessage
}

function calculateSenderName(msgObj: any, wechatSuffix?: boolean) {
    let senderName = '';
    if (!msgObj.is_receive) {
        if (msgObj.inviter) {
            senderName = msgObj.inviter.nick_name;
        } else if (msgObj.isRobot) {
            senderName = '机器人客服'
        }else {
            senderName = msgObj.nick_name || msgObj.agent_nick_name || msgObj.user_nick_name || msgObj?.chat?.agent?.name || msgObj.from_user_name
        }
    } else {
        // if (wechatSuffix) { // 带有@微信标识需要显示昵称
        senderName = msgObj.from_user_name || '';
        // }
    }
    if (msgObj.auto) {
        senderName = '系统';
    }

    return senderName;
}