import axios, { AxiosRequestConfig, Method } from 'axios';
import Cookies from 'js-cookie';
import merge from 'lodash-es/merge';
import { store } from "../store";
import { isSameOrigin } from '@udesk-webapp/utils';
// axios.defaults.baseURL = 'http://reocar.b3.tryudesk.com/spa1';
// axios.defaults.baseURL = 'http://reocar.ud.com/spa1';
axios.defaults.baseURL = '/spa1';

interface IHeader {
    'X-CSRF-Token'?: string;
    'X-Udesk-Tid'?: string;
    'X-Udesk-Ifc'?: string;
    Authorization?: string;
}

function ajaxBefore(url: string, method: string, params?: any, options?: AxiosRequestConfig) {
    const headers: IHeader = {};
    // 是否开启了CXRFTOKEN
    const _isSameOrigin = isSameOrigin(url);
    const csrfToken = getCsrfToken();
    console.log('store',store.getState().initData);
    if (_isSameOrigin && csrfToken) {
        headers['X-CSRF-Token'] = csrfToken;
    }
    // 是否使用了SDK

    if (_isSameOrigin && csrfToken) {
        headers['X-CSRF-Token'] = csrfToken;
    }
    //如果启用了灰度，则读取 cookie 里的 tid 和 ifc，并把值加到 header 里
    const tid = Cookies.get('tid');
    const ifc = Cookies.get('ifc');
    const token = store.getState().initData?.token;
    const credentials = store.getState().initData?.credentials;
    const sdk = store.getState().initData?.sdk;
    if (tid) {
        headers['X-Udesk-Tid'] = tid;
    }
    if (ifc) {
        headers['X-Udesk-Ifc'] = ifc;
    }
    if (sdk && token) {
        headers['Authorization'] = `Bearer ${token}`;
        axios.defaults.baseURL = store.getState().initData?.spa1Path;
    }
    if(!credentials){
        axios.defaults.withCredentials = false;
    }
    const setHeaderOptions: any = merge({ headers: headers }, { ...options });

    const requestPptions = {
        url: url,
        method: method,
        data: params,
        ...setHeaderOptions,
    };
    if (method.toUpperCase() === 'GET') {
        requestPptions.params = params;
    }
    return axios(requestPptions).then((res) => {
        const code = res?.data?.code;
        if (code === 1000 || code === '1000') {
            return res.data;
        } else {
            if (res.status >= 200 && res.status < 300) {
                return res.data;
            } else {
                return Promise.reject(res.data);
            }
        }
    });
}

export function request(
    options: Omit<AxiosRequestConfig, 'url' | 'method'> & { url: string; method: Method }
) {
    const { url, method, data, ..._options } = options;
    return ajaxBefore(url, method, data, _options);
}

export function getJSON(url: string, params?: any, options?: AxiosRequestConfig) {
    const _url: string = url;
    // if (!isEmpty(params)) {
    //     _url += ('?' + Qs.stringify(params))
    // }
    return ajaxBefore(_url, 'GET', params, options);
}

export function postJSON(url: string, params?: any, options?: AxiosRequestConfig) {
    return ajaxBefore(url, 'post', params, options);
}

export function post(url: string, params?: any, options?: AxiosRequestConfig) {
    const searchParams = new URLSearchParams();
    for (const i in params) {
        if (Object.prototype.hasOwnProperty.call(params, i) && params[i] !== undefined) {
            searchParams.append(i, params[i]);
        }
    }
    return ajaxBefore(url, 'post', searchParams, options);
}

export function putJSON(url: string, params?: any, options?: AxiosRequestConfig) {
    return ajaxBefore(url, 'put', params, options);
}

export function deleteJSON(url: string, params?: any, options?: AxiosRequestConfig) {
    return ajaxBefore(url, 'delete', params, options);
}

export function getCsrfToken() {
    const UDD = (window as any).UDD || {};
    const user = UDD.user || {};

    if (user.csrf_token) {
        return user.csrf_token;
    }
}
