import {default as originAxios, AxiosRequestConfig } from 'axios';
import Cookies from 'js-cookie';
import merge from 'lodash-es/merge';

// axios.defaults.baseURL = 'http://reocar.b3.tryudesk.com/spa1';
// axios.defaults.baseURL = 'http://reocar.ud.com/spa1';
const axios = originAxios.create();
axios.defaults.baseURL = '/spa1';

interface IHeader {
    'X-CSRF-Token'?: string,
    'X-Udesk-Tid'?: string,
    'X-Udesk-Ifc'?: string
}
function isSameOrigin(url: string) {
    let loc = window.location;
    let a = document.createElement('a');
    a.href = url;
    return a.hostname === loc.hostname &&
        a.port === loc.port &&
        a.protocol === loc.protocol;
}

function getCsrfToken() {
    const UDD = (window as any).UDD || {};
    const user = UDD.user || {};

    if (user.csrf_token) {
        return user.csrf_token;
    }
}


function ajaxBefore(url: string, method: string, params?: any, options?: AxiosRequestConfig) {
    let headers: IHeader = {};
    // 是否开启了CXRFTOKEN
    const _isSameOrigin = isSameOrigin(url);
    const csrfToken = getCsrfToken();
    if (_isSameOrigin && csrfToken) {
        headers['X-CSRF-Token'] = csrfToken;
    }
    //如果启用了灰度，则读取 cookie 里的 tid 和 ifc，并把值加到 header 里
    const tid = Cookies.get('tid');
    const ifc = Cookies.get('ifc');

    if (tid) {
        headers['X-Udesk-Tid'] = tid;
    }
    if (ifc) {
        headers['X-Udesk-Ifc'] = ifc;
    }

    let setHeaderOptions: any = merge({ headers: headers }, { ...options });

    let requestPptions = {
        url: url,
        method: method,
        data: params,
        ...setHeaderOptions
    };
    if (method.toUpperCase() === 'GET') {
        requestPptions.params = params;
    }
    return axios(requestPptions).then(res => {
        const code = res?.data?.code;
        if (code === 1000 || code === '1000') {
            return res.data;
        } else {
            if (res.status >= 200 && res.status < 300) {
                return res.data;
            } else {
                return Promise.reject(res.data);
            }
        }
    });
}

export function getJSON(url: string, params?: any, options?: AxiosRequestConfig) {
    let _url: string = url;
    // if (!isEmpty(params)) {
    //     _url += ('?' + Qs.stringify(params))
    // }
    return ajaxBefore(_url, 'GET', params, options);
}

export function postJSON(url: string, params?: any, options?: AxiosRequestConfig) {
    return ajaxBefore(url, 'post', params, options);
}
