import { getJSON, postJSON, putJSON } from "../../utils";
import { message } from "udesk-ui";
import i18n from "../../i18n";
import {csApiGetJSON} from "../csApi";
import { AxiosRequestConfig } from 'axios';

interface Option {
  noCode?: boolean;
  params?: any;
  options?: AxiosRequestConfig;
  errMsg?: any;
  success?: (response: any) => void;
  fail?: (reason: any) => void;
  finally?: () => void;
}

/**
 *
 * @param response
 * @param {object} options
 * @param {boolean} options.noCode - 界面上是否显示报错，true 为不显示
 */
function success(response: any, options?: Pick<Option, "noCode" | "success">) {
  options?.success?.(response);
  // eslint-disable-next-line eqeqeq
  if (!options?.noCode && response?.code != 1000) {
    if (
      response?.code_message ||
      response?.exception?.message ||
      i18n.t("tip.error.requestError")
    ) {
      console.log(response, options);
      message.error(
        response?.code_message ||
          response?.exception?.message ||
          i18n.t("tip.error.requestError")
      );
    }
  }
  return response;
}

export function getJson(url: string, options?: Option) {
  return getJSON(url, options?.params, options?.options)
    .then(
      (response: any) => {
        return success(response, options);
      },
      (reason: any) => {
        options?.fail?.(reason);
        !options?.noCode &&
          message.error(options?.errMsg || i18n.t("tip.error.requestError"));
      }
    )
    .finally((...argus) => {
      options?.finally?.(...argus);
    })
    .catch((e) => {
      console.error(e);
    });
}

export function csApiGetJson(url: string, options?: Option) {
  return csApiGetJSON(url, {params:options?.params, ...options?.options}).then(res=>res.data).then((response: any) => {
    return success(response, options);
  }, (reason: any) => {
    options?.fail?.(reason);
    !options?.noCode && message.error(options?.errMsg || i18n.t('tip.error.requestError'));
  }).finally(() => {
    options?.finally?.();
  }).catch(e => {
    console.error(e);
  });
}

export function postJson(url: string, options?: Option) {
  return postJSON(url, options?.params, options?.options)
    .then(
      (response: any) => {
        return success(response, options);
      },
      (reason: any) => {
        options?.fail?.(reason);
        message.error(options?.errMsg || i18n.t("tip.error.requestError"));
      }
    )
    .finally((...argus) => {
      options?.finally?.(...argus);
    })
    .catch((e) => {
      console.error(e);
    });
}

export function putJson(url: string, options?: Option) {
  return putJSON(url, options?.params, options?.options)
    .then(
      (response: any) => {
        return success(response, options);
      },
      (reason: any) => {
        options?.fail?.(reason);
        message.error(options?.errMsg || i18n.t("tip.error.requestError"));
      }
    )
    .finally((...argus) => {
      options?.finally?.(...argus);
    })
    .catch((e) => {
      console.error(e);
    });
}
