import apis from "../../common/config/apis";
import {csApiGetJson, getJson, postJson} from "./common";
import { getJSON,  } from "../../utils";
import { csApiPutJSON } from "../../utils/csApi";
import qs from "qs";

interface ISuccessCallback {
  (resp: any): void;
}

export interface ICustomerOptions {
  url?: string;
  errMsg?: string;
  params?: ObjectWithIndex;
  options?: ObjectWithIndex;
  success?: ISuccessCallback;
}

function postUser(options?: ICustomerOptions): Promise<any> {
  return postJson(options?.url || apis.postUser, options || {});
}

function getMonitorUserList(): Promise<any> {
  return csApiGetJson(apis.getMonitorUserList);
}

/**
 * 检查电话号和邮箱是否已经被使用
 * @param {number} customerId - 客户 id
 * @param {string} cellphone - 被检验的电话号码
 * @param {string} email - 被检验的邮箱
 */
export async function checkDuplicate(
  customerId: number,
  { cellphone, email }: { cellphone?: string; email?: string }
): Promise<{
  code: number;
  duplications: Record<"cellphone" | "email", { name: string; id: number }>;
}> {
  if (!customerId) {
    throw new Error("customerId is required");
  }
  if (!cellphone && !email) {
    throw new Error("cellphone or email is required");
  }
  let resp;
  const url = "/customers/check_duplication";
  let params;
  if (cellphone) {
    params = { customer: { id: customerId, cellphone } };
  } else if (email) {
    params = { customer: { id: customerId, email } };
  }
  resp = await getJSON(url, params, {
    paramsSerializer: function (params) {
      return qs.stringify(params, { arrayFormat: "brackets" });
    },
  });

  if (resp.code !== 1000) {
    throw new Error(resp?.exception?.message);
  }
  return resp;
}

/**
 * 修改客户
 * @param customer
 */
export async function putCustomer(
  customer: { id: Customer["id"] } & Partial<Customer>
) {
  if (!customer.id) {
    throw new Error("customer.id is required");
  }
  const resp = await csApiPutJSON(`/cs/ts/api/customers/${customer.id}`, { customer }).then(res => res.data);
  if (resp.code === 1000) {
    return resp;
  }
  throw new Error(resp?.exception?.message);
}

export default {
  postUser,
  getMonitorUserList,
};
