import i18n from "../i18n";
import { getJSON } from "./ajax";
import { store } from "../store";

let loadFiles: any[] = [];
let state: any = {};

export function loadJS(url: string, cbFn: () => void, isNotReplace: boolean) {
  if (!url) return;

  switch (url) {
    case "lightbox":
      url = "libs/lightbox.js";
      break;
  }

  if (!isNotReplace) {
    if (loadFiles.includes(url)) {
      return cbFn && cbFn();
    }
    loadFiles.push(url);
  }

  getScript(url, cbFn);
}

function getScript(source: string, callback: () => void) {
  var script: any = document.createElement("script");
  var prior: any = document.getElementsByTagName("script")[0];
  script.async = true;

  script.onload = script.onreadystatechange = function (
    _: any,
    isAbort: boolean
  ) {
    if (
      isAbort ||
      !script.readyState ||
      /loaded|complete/.test(script.readyState)
    ) {
      script.onload = script.onreadystatechange = null;
      script = undefined;

      if (!isAbort && callback) setTimeout(callback, 0);
    }
  };

  script.src = source;
  prior.parentNode.insertBefore(script, prior);
}

export function getEnumText(enumName: string, key: string) {
  if (!enumName || !key) {
    return "";
  }
  return i18n.t(`enums.${enumName}.${key}`);
}

export function hasImNoteRequest(id: number) {
  var imNoteFields =
    (
      store
        .getState()
        .noteTemplateConfig.noteTemplates.find(
          (item) => item.id === id && item.category === "im"
        ) || {}
    ).fields || [];
  if (imNoteFields.length === 0) {
    return false;
  }

  var hasImNoteRequest = imNoteFields.some(function (field) {
    return field.is_required;
  });
  return hasImNoteRequest;
}

export function getImNoteReqSwitch() {
  let data = state.imNotReqSwitchConfig;
  if (data) {
    return new Promise((resolve, reject) => {
      resolve(data);
    });
  }
  return getJSON(`/im/im_setting/agent_note_fill`).then((res = {}) => {
    if (res.code === 1000) {
      state.imNotReqSwitchConfig = res;
    }
    return res;
  });
}

/* 桌面通知
   title:通知标题
   msg：通知具体内容
   options:{
       click():点击桌面通知事件
   }
*/
let notificationQueues: any[] = [];

export function notification(title: string, msg: string, options: any) {
  title = title || i18n.t("page.im.imMessages.systemNotice");
  options = options || {};

  if (!window.Notification) {
    console.log("Notification Not supported！");
    return;
  }

  if (Notification.permission === "granted") {
    Notification.requestPermission(function (permission) {});
    var instance = new Notification(title, {
      body: msg,
      icon: options.img || "./notification24.png",
    });
    options.url && window.open(options.url);
    instance.onclick = function (ev) {
      options.click && options.click(ev);
      instance.close();
    };
    options.count_down &&
      setTimeout(function () {
        instance.close();
      }, options.count_down);
    notificationQueues.push(instance);
    if (notificationQueues.length > 1) {
      var instance1 = notificationQueues.shift();
      instance1.close();
    }
  } else {
    try {
      return Notification.requestPermission();
    } catch (e) {
      console.log(e);
    }
  }
}

export function getMonitorConfig() {
  if (state.monitorConfig) {
    return new Promise((resolve, reject) => {
      resolve(state.monitorConfig);
    });
  }
  return getJSON("/im/im_monitor_configs.json").then((res) => {
    state.monitorConfig = res;
    return res;
  });
}

export function dataURItoBlob(dataURI: string) {
  // convert base64 to raw binary data held in a string
  // doesn't handle URLEncoded DataURIs - see SO answer #6850276 for code that does this
  let byteString = atob(dataURI.split(",")[1]);

  // separate out the mime component
  let mimeString = dataURI.split(",")[0].split(":")[1].split(";")[0];

  // write the bytes of the string to an ArrayBuffer
  let ab = new ArrayBuffer(byteString.length);
  let ia = new Uint8Array(ab);
  for (let i = 0; i < byteString.length; i++) {
    ia[i] = byteString.charCodeAt(i);
  }

  return new Blob([ab], { type: mimeString });
}
