import find from "lodash-es/find";
import isArray from "lodash-es/isArray";
import isNil from "lodash-es/isNil";
import isString from "lodash-es/isString";
import reduce from "lodash-es/reduce";
import numeral from "numeral";

/**
 * 将自定字段的值转换为文字，例如多选字段'1,2'转换为'选项1,选项2'
 * @param {CustomField} field - 自定义字段
 * @param {string|string[]} value - 值
 */
export function customFieldValueToText(
  field: CustomField,
  value?: string | string[]
) {
  if (!value) {
    return "";
  }

  const type = field.content_type;
  let valueString = isString(value) ? value : value?.join(",");
  let valueAry = isArray(value) ? value : value?.split(",");
  let options = field.select_options;

  switch (type) {
    case "droplist":
    case "checkbox":
    case "radio":
      return options
        .filter((i) => valueAry.includes(`${i.value}`))
        .map((i) => i.title)
        .join(",");
    case "chained_droplist": {
      if (!valueString) {
        return "";
      }
      let resultAry: string[] = [];
      reduce(
        valueAry,
        (preValue, item) => {
          let option = find(preValue, ["value", item]);
          if (option) {
            resultAry.push(option.title);
          }
          return option?.subs ?? [];
        },
        options
      );
      return resultAry.join("/");
    }
    case "calculate":
      switch (field.calculate_result_type) {
        case "duration":
          return isNil(valueString)
            ? ""
            : numeral(valueString).format("00:00:00");
        case "percentage":
          return isNil(valueString) ? "" : numeral(valueString).format("0.00%");
        case "numeric":
          return isNil(valueString) ? "" : Number(valueString);
        default:
          return valueString;
      }
    default:
      return valueString;
  }
}
