import { csApiPutJSON } from "../utils/csApi";

/**
 * 是否有权限查看编辑客户电话
 * @param customer 必须，客户对象
 * @param customer.owner_group_id - 客户所属客服组
 * @param customer.owner_id - 客户所属客服
 * @param {object} options
 * @return {boolean}
 */
export function roleCanCustomerPhone(
  customer: Pick<Customer, "owner_group_id" | "owner_id">,
  options: {
    role_can: Pick<
      IRoleCan,
      | "customer_show_cellphone_all"
      | "customer_show_cellphone_group"
      | "customer_show_cellphone_personal"
    >;
    agent: Pick<IUser, "user_groups" | "id" | "role">;
  }
) {
  if (!customer) {
    return false;
  }
  let { agent, role_can } = options;
  if (agent.role === "admin" || role_can.customer_show_cellphone_all) {
    return true;
  }

  let user_groups = agent.user_groups || [];
  let ownerGroupId = customer.owner_group_id;
  if (
    role_can.customer_show_cellphone_group &&
    user_groups.some((i) => i.id === ownerGroupId)
  ) {
    return true;
  }

  let ownerId = customer.owner_id;
  return !!(
    role_can.customer_show_cellphone_personal &&
    ownerId &&
    `${ownerId}` === `${agent.id}`
  );
}

export async function saveCustomer(
  customer: Customer,
  options: {
    role_can: Parameters<typeof roleCanCustomerPhone>["1"]["role_can"] &
      Pick<IRoleCan, "set_customer_level">;
    company: Pick<Company, "is_support_lang">;
    agent: Pick<IUser, "id" | "role" | "user_groups">;
  }
) {
  let { role_can, company, agent } = options;
  if (!role_can.set_customer_level) {
    delete customer.level;
  }
  if (!company.is_support_lang) {
    delete customer.lang;
  }
  if (!roleCanCustomerPhone(customer, { role_can, agent })) {
    delete customer.cellphones;
    delete customer.cellphones_attributes;
  }
  const resp = await csApiPutJSON("/cs/ts/api/customers/" + customer.id, {
    customer,
    tags: customer.tags,
  }).then(res => res.data);
  if (resp?.code === 1000) {
    resp.customer.organizations = resp.organizations;
    return resp.customer;
  }
  throw new Error(resp.exception?.message);
}
