export default {
  highlightText,
  clearHighlight,
};

interface HighlightTextOptions {
  tagName?: string;
  className?: string;
}

const HIGHLIGHT_ATTRIBUTE_NAME = '_udeskGmUiHighlight';

function highlightText(elem: Node, keyword: string, options?: HighlightTextOptions) {
  if (!keyword) return;
  let textNodes = getTextNodesIn(elem, (n) => {
    let keywordRegExp = new RegExp(keyword);
    return !/^\n\s*$/.test(n.nodeValue ?? '') && keywordRegExp.test(n.nodeValue ?? '');
  });
  let splitRegExp = new RegExp(`(${keyword})`);
  textNodes.forEach((node) => {
    let txtArr = (node.nodeValue ?? '').split(splitRegExp).filter((txt) => txt !== '');
    txtArr.forEach((txt) => {
      let newNode;
      if (txt !== keyword) {
        newNode = document.createTextNode(txt);
      } else {
        let tagName = (options && options.tagName) || 'em';
        newNode = document.createElement(tagName);
        newNode.setAttribute(HIGHLIGHT_ATTRIBUTE_NAME, '');
        if (options && options.className) {
          newNode.className = options.className;
        }
        newNode.appendChild(document.createTextNode(txt));
      }
      node.parentNode && node.parentNode.insertBefore(newNode, node);
    });
    node.parentNode && node.parentNode.removeChild(node);
  });
}

function clearHighlight(elem: ParentNode) {
  let emElems = getElementsByAttributeName(elem, HIGHLIGHT_ATTRIBUTE_NAME);

  for (let i = emElems.length - 1; i >= 0; i--) {
    let emNode = emElems[i];
    let parentNode = emNode.parentNode;
    if (parentNode && emNode.textContent) {
      let newNode = document.createTextNode(emNode.textContent);
      parentNode.insertBefore(newNode, emNode);
      parentNode.removeChild(emNode);
      parentNode.normalize();
    }
  }
}

function getTextNodesIn(elem: Node, opt_fnFilter?: (n: Node, elem: Node) => boolean) {
  let textNodes: Node[] = [];
  if (elem) {
    for (let nodes = elem.childNodes, i = nodes.length; i--; ) {
      let node = nodes[i],
        nodeType = node.nodeType;
      if (nodeType === 3) {
        if (!opt_fnFilter || opt_fnFilter(node, elem)) {
          textNodes.push(node);
        }
      } else if (nodeType === 1 || nodeType === 9 || nodeType === 11) {
        textNodes = textNodes.concat(getTextNodesIn(node, opt_fnFilter));
      }
    }
  }
  return textNodes;
}

function getElementsByAttributeName(elem: ParentNode, attributeName: string) {
  let matchNodes: Node[] = [];
  if (elem) {
    let nodes = elem.children;
    for (let i = 0; i < nodes.length; i++) {
      let node = nodes[i];
      if (node.hasAttribute(attributeName)) {
        matchNodes.push(node);
      } else {
        matchNodes = matchNodes.concat(getElementsByAttributeName(node, attributeName));
      }
    }
  }
  return matchNodes;
}
