import { useState } from 'react';

export type CacheMode = 'session' | 'local';

export default function useStorage(
  cacheId: string,
  cacheMode: CacheMode = 'session',
): StorageManager {
  const [storage] = useState(new StorageManager(cacheMode, cacheId));
  return storage;
}

export class StorageManager {
  private cacheMode: CacheMode;
  private cacheId: string;
  private storage: Storage;

  constructor(cacheMode: CacheMode, cacheId: string) {
    this.cacheMode = cacheMode;
    this.cacheId = cacheId;
    this.storage = sessionStorage;

    if (cacheMode === 'local') {
      this.storage = localStorage;
    }
  }

  save(value: any) {
    if (!this.cacheId) return;
    try {
      this.storage.setItem(this.cacheId, JSON.stringify(value));
    } catch (e) {
      console.log('save failed :: ', e);
    }
  }
  read() {
    if (!this.cacheId) return;
    try {
      let value = this.storage.getItem(this.cacheId);
      return value ? JSON.parse(value) : value;
    } catch (e) {
      console.log('read failed :: ', e);
    }
  }
  delete() {
    if (!this.cacheId) return;
    try {
      this.storage.removeItem(this.cacheId);
    } catch (e) {
      console.log('delete failed :: ', e);
    }
  }
}
