import { getJSON, postJSON, putJSON } from './ajax'
import { message } from 'udesk-ui'

const getToken = (fileName?: string, marking?: string) => {
    fileName = fileName || 'image.jpg';
    marking = marking || 'web.cc.qiniu';
    return new Promise((resolve, reject) => {
        getJSON('/file_upload_policy', {
            marking,
            key: fileName
        }).then(res => {
            if (res.code === 1000) {
                resolve(res);
            } else {
                message.error('上传时获取token失败');
                reject(res);
            }
        })
    })
}

interface ISubmit {
    config: any
    _data: string
    data: string
    success?: (value: any) => void
    complete?: () => void
    random_filename?: string
    key?: string
    url?: string
    error?: () => void
    noTip?: string
}

const sendImgByBase64 = (cfg: ISubmit) => {
    cfg = cfg || {};
    var config = cfg.config || {};
    switch (config.storage_policy) {
        case 'ali':
            sendAliBase64(cfg);
            break;
        case 'minio':
            sendMinioBase64(cfg);
            break;
    }
};

function getExt(urlData: any) {
    var arr = urlData.split(',');
    var mime = arr[0].match(/:(.*?);/)[1];
    return mime.replace("image/", "");
}

export function dataUrlToFile(dataUrl: string, filename: string) {
     const arr = dataUrl.split(',');
     const mime = (arr[0] || '').match(/:(.*?);/)?.[1];
     const bytes = window.atob(arr[1]);

    const ab = new ArrayBuffer(bytes.length);

    const u8arr = new Uint8Array(ab);
    for (let i = 0; i < bytes.length; i++) {
        u8arr[i] = bytes.charCodeAt(i);
    }

    return new File([u8arr], filename, {type: mime});
}

function addToken(fileObj: any, cfg: any) {
    var config = cfg.config || {};
    var formObj = new FormData();

    formObj.append('name', fileObj.name);
    formObj.append('type', fileObj.type);
    formObj.append('lastModifiedDate', fileObj.lastModifiedDate);
    formObj.append('size', fileObj.size);
    formObj.append('policy', config.policy);
    formObj.append('OSSAccessKeyId', config.accessid);
    formObj.append('Signature', config.signature);
    formObj.append('key', cfg.key);
    formObj.append('file', fileObj);
    return formObj;
}

function sendAliBase64(cfg: ISubmit) {
    var config = cfg.config || {};
    var ext = getExt(cfg._data);
    var imageName = 'image_' + (+new Date()) + '_' + Math.random().toString(32).slice(-5) + '.' + ext;
    cfg.random_filename = imageName;
    cfg.key = config.dir + imageName;
    cfg.url = `${config.host}/${cfg.key}`;

    var fileObj = dataUrlToFile(cfg._data, imageName);
    var formDateObj = addToken(fileObj, cfg);

    postJSON(config.host, formDateObj).then(res => {
        if (res.status === 204) {
            cfg.success && cfg.success({src: cfg.url});
        } else {
            cfg.error && cfg.error();
            !cfg.noTip && message.error("上传失败，请重新上传");
        }
    })
}

function sendMinioBase64(cfg: ISubmit) {
    var config = cfg.config || {};
    var fields = config.fields || {};
    putJSON(config.host, cfg.data, {
        headers: {
            "Content-Type": "image/jpg",
            'X-Udesk-Minio-policy': fields.policy,
            'X-Udesk-Minio-x-amz-algorithm': fields['x-amz-algorithm'],
            'X-Udesk-Minio-x-amz-credential': fields['x-amz-credential'],
            'X-Udesk-Minio-x-amz-date': fields['x-amz-date'],
            'X-Udesk-Minio-x-amz-signature': fields['x-amz-signature'],
            'X-Udesk-Minio-key': fields.key,
            'X-Amz-Meta-Content-Encoding': 'base64,='
        }
    }).then(res => {
        if (res.code === 1000 || res.code === '1000') {
            cfg.success && cfg.success({src: config.host + '/' + config.file_path});
        } else {
            cfg.error && cfg.error();
            !cfg.noTip && message.error("上传失败，请重新上传");
        }
    })
}

export default {
    getToken,
    sendImgByBase64,
    dataUrlToFile
}
