import isFunction from 'lodash-es/isFunction';
import assign from 'lodash-es/assign';
import includes from 'lodash-es/includes';

export function removeHTMLTag(str: string) {
    if (!str) {
        return '';
    }
    str = str.replace(/<\/?[^>]*>/g, ''); //去除HTML tag
    // str = str.replace(/[ | ]*\n/g, '\n'); //去除行尾空白
    //str = str.replace(/\n[\s| | ]*\r/g,'\n'); //去除多余空行
    str = str.replace(/&nbsp;/gi, ' '); //去掉&nbsp;
    return str;
}

export function getId() {
    return Date.parse(new Date().toString()) + Math.floor(Math.random() * 1000000);
}

// 动态获取字数长度限制的警告信息
export function r(template: string, ...args: any) {
    for (let i = 0; i < args.length; i++) {
        template = template.replace(/{[^}]+}/, args[i]);
    }
    return template;
}

export function getFileTypeByContentType(contentType: string) {
    switch (contentType) {
        case 'link':
        case 'datetime':
        case 'text':
        case 'date':
        case 'time':
        case 'numeric':
        case 'calculate':
        case 'area_text':
        case 'number':
        case 'location':
            return 'TextField';
        case 'radio':
        case 'checkbox':
        case 'radio_droplist':
        case 'checkbox_droplist':
        case 'chained_droplist':
        case 'tag_tree':
        default:
            return 'SelectField';
    }
}

function textValidFn(is_required: boolean, _value: string = '', field: any) {
    let validTexts: string[] = [];
    let setting = field.setting || {};
    // 检查必填
    if (is_required && !_value) {
        validTexts.push('该字段为必填项');
    }

    // 长度校验
    if (setting.max_length && _value.length > setting.max_length) {
        validTexts.push(`超过最大长度限制(${setting.max_length})`);
    }
    // if (_value.length > 10) {
    //     validTexts.push(`超过最大长度限制(${10})`);
    // }
    // 正则校验
    if (setting.valid_reg) {
        const reg = new RegExp(setting.valid_reg, 'ig');
        if (!reg.test(_value)) {
            validTexts.push(`内容格式校验失败`);
        }
    }

    if (validTexts.length > 0) {
        const errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    } else {
        return Promise.resolve();
    }
}

function textareaValidFn(is_required: boolean, _value: string = '', field: any) {
    let validTexts: string[] = [];
    let setting = field.setting || {};
    // 检查必填
    if (is_required && !_value) {
        validTexts.push('该字段为必填项');
    }

    // 长度校验
    if (setting.max_length && _value.length > setting.max_length) {
        validTexts.push(`超过最大长度限制(${setting.max_length})`);
    }

    if (validTexts.length > 0) {
        const errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    } else {
        return Promise.resolve();
    }
}

function linkValidFn(is_required: boolean, _value: string = '', field: any) {
    let validTexts: string[] = [];
    // 检查必填
    if (is_required && !_value) {
        validTexts.push('该字段为必填项');
    }

    if (validTexts.length > 0) {
        const errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    } else {
        return Promise.resolve();
    }
}

function numberValidFn(is_required: boolean, _value: string = '', field: any) {
    let validTexts: string[] = [];
    // 检查必填
    if (is_required && !_value) {
        validTexts.push('该字段为必填项');
    }
    if (validTexts.length > 0) {
        const errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    } else {
        return Promise.resolve();
    }
}

function commonValidFn(rule: any, _value: string = '', field: any) {
    let validTexts: string[] = [];
    // 检查必填
    if (field.isRequired && (!_value || !_value.length)) {
        validTexts.push('该字段为必填项');
    }
    if (validTexts.length > 0) {
        const errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    } else {
        return Promise.resolve();
    }
}

function chainedValidFn(rule: any, _value: any[] = [], field: any) {
    let validTexts: string[] = [];
    // 检查必填
    if (field.is_required && (!_value || !_value.length)) {
        validTexts.push('该字段为必填项');
    }
    /*
     *如果开启了选择最后一级的开关
     *如果填过这个字段
     *并且此字段是可编辑的
     *进行是否填写到最后一级的判断
     */

    if (_value.length && field.full_select && !field.disabled) {
        /*
         * 最后一级的数据
         * 根据选择的数据（数组），循环查找当前对应的已经选择的数据，将该层级数据的subs赋值给lastOption
         * 直到循环完毕，lastOption为下一次应该选择的数据，如果没有数据，说明最后一级已经选完了。
         */
        let lastOption = field.select_options;

        _value.forEach((item: any) => {
            lastOption.forEach((_item: any) => {
                if (_item.value === item) {
                    lastOption = _item.subs || null;
                }
            });
        });

        if (lastOption) {
            validTexts.push('必须选择最末级');
        }
    }
    if (validTexts.length > 0) {
        const errMsg = validTexts.join(',');
        return Promise.reject(errMsg);
    } else {
        return Promise.resolve();
    }
}

const sysFields = [
    'name',
    'nick_name',
    'email',
    'description',
    'cellphone',
    'organizations',
    'organization',
    'tags',
    'cellphones',
    'domains',
    'org_email_suffix',
];

export function isSysField(type: string) {
    return sysFields.includes(type);
}

// 遍历级联字段
export function mapCascader(options: any) {
    let result: any = [];
    options.forEach((item: any) => {
        let obj: any = {
            label: item.title,
            value: item.value,
        };

        if (item.subs && item.subs.length > 0) {
            obj['children'] = mapCascader(item.subs);
        }
        result.push(obj);
    });
    return result;
}

/**
 * @param {string} fieldType  那种字段?   customer | ticket
 * @param {string} editType   什么操作?   edit | new | show
 * @param {object} fieldsAuthority    字段全选hash表
 */
export function formatCustomField({
    fields,
    fieldsAuthority,
    trigger = 'onChange',
    fieldType,
    editType,
}: any): IFieldItem[] {
    if (!Array.isArray(fields)) {
        return [fields];
    }

    return fields
        .map((field: any) => {
            if (includes(sysFields, field.name)) {
                return { ...field, trigger };
            }

            // authKey 可能的值: editable 可编辑 readable 只读 invisible 不可见 must_fill_in 必填
            let authKey: string = '';
            // 处理字段权限相关
            if (fieldType && editType && fieldsAuthority) {
                switch (fieldType) {
                    case 'customer':
                        authKey = fieldsAuthority[field.name] ? fieldsAuthority[field.name][editType] || '' : '';
                        break;
                    case 'ticket':
                        authKey = fieldsAuthority[field.name] ? fieldsAuthority[field.name][editType] || '' : '';
                        break;
                }
            }
            // 字段不可见时返回空
            if (authKey === 'invisible') {
                return '';
            }

            let setting = field.setting || {};
            const tooltipObj = setting.tooltipObj || {};
            const explainObj = setting.explain || {};
            let fieldItem: IFieldItem = {
                label: field.title,
                layout: field.layout || '',
                tooltip: field.tooltip || '',
                tooltipShow: !!(tooltipObj.show && tooltipObj.content),
                lines: setting.lines || '',
                isRequired: field.is_required || false,
                // isRequired: authKey === 'must_fill_in' || field.is_required || false,
                field: {
                    trigger: field.content_type === 'staff' ? 'onChange' : trigger,
                    id: field.name,
                    type: field.content_type,
                    remark: field.comment || '',
                    explain: explainObj.content,
                    explainShow: !!(explainObj.show || explainObj.content),
                    disabled: authKey === 'readable',
                    isRequired: authKey === 'must_fill_in',
                    options: [],
                    rowsLine: 4,
                },
            };
            switch (field.content_type) {
                case 'text':
                    fieldItem = Object.assign(fieldItem, {
                        field: {
                            ...fieldItem.field,
                            // defaultValue: field.default_value || '',
                            placeholder: field.palceholder || '',
                            maxLength: field.max_length || null,
                            validReg: field.valid_reg || null,
                            customValidFn: isFunction(field.customValidFn)
                                ? field.customValidFn
                                : function (rule: any, value: string) {
                                      return textValidFn(fieldItem.field.isRequired, value, field);
                                  },
                        },
                    });
                    break;
                case 'area_text':
                    fieldItem = Object.assign(fieldItem, {
                        field: {
                            ...fieldItem.field,
                            type: 'textarea',
                            placeholder: field.palceholder || '',
                            // defaultValue: field.default_value || '',
                            maxLength: field.max_length || null,
                            customValidFn: isFunction(field.customValidFn)
                                ? field.customValidFn
                                : function (rule: any, value: string) {
                                      return textareaValidFn(fieldItem.field.isRequired, value, field);
                                  },
                        },
                    });
                    break;
                case 'link':
                    fieldItem = Object.assign(fieldItem, {
                        field: {
                            ...fieldItem.field,
                            // defaultValue: field.default_value || '',
                            placeholder: field.palceholder || '',
                            regLink: true,
                            customValidFn: isFunction(field.customValidFn)
                                ? field.customValidFn
                                : function (rule: any, value: string) {
                                      return linkValidFn(fieldItem.field.isRequired, value, field);
                                  },
                        },
                    });
                    break;
                case 'date':
                case 'time':
                case 'datetime':
                case 'location':
                    fieldItem = Object.assign(fieldItem, {
                        field: {
                            ...fieldItem.field,
                            customValidFn: isFunction(field.customValidFn)
                                ? field.customValidFn
                                : function (rule: any, value: string) {
                                      return commonValidFn(rule, value, fieldItem.field);
                                  },
                        },
                    });
                    break;
                case 'calculate':
                    fieldItem = Object.assign(fieldItem, {
                        field: {
                            ...fieldItem.field,
                            customValidFn: isFunction(field.customValidFn)
                                ? field.customValidFn
                                : function (rule: any, value: string) {
                                      return commonValidFn(rule, value, fieldItem.field);
                                  },
                        },
                    });

                    break;
                case 'number':
                case 'numeric':
                    fieldItem = Object.assign(fieldItem, {
                        field: {
                            ...fieldItem.field,
                            // defaultValue: field.default_value || '',
                            placeholder: field.palceholder || '',
                            integerDigits: field.integer_digits || '',
                            decimalDigits: field.decimal_digits || '',
                            addonBefore: field.addon_before || '',
                            addonAfter: field.addon_after || '',
                            prefix: field.prefix || '',
                            suffix: field.suffix || '',
                            customValidFn: isFunction(field.customValidFn)
                                ? field.customValidFn
                                : function (rule: any, value: string) {
                                      return numberValidFn(fieldItem.field.isRequired, value, field);
                                  },
                        },
                    });
                    break;
                case 'droplist':
                case 'droplist_multi':
                    fieldItem = Object.assign(fieldItem, {
                        field: {
                            ...fieldItem.field,
                            trigger: 'onChange',
                            options: field.select_options.map((option: any) => {
                                return {
                                    icon: option.icon || '',
                                    title: option.title,
                                    bgColor: option.bg_clor || '',
                                    default: option.default || false,
                                    value: option.value,
                                };
                            }),
                            customValidFn: isFunction(field.customValidFn)
                                ? field.customValidFn
                                : function (rule: any, value: string) {
                                      return commonValidFn(rule, value, fieldItem.field);
                                  },
                        },
                    });
                    break;
                case 'radio':
                case 'checkbox':
                    fieldItem = Object.assign(fieldItem, {
                        field: {
                            ...fieldItem.field,
                            arrangement: field.arrangement || 'auto',
                            trigger: 'onChange',
                            options: field.select_options.map((option: any) => {
                                return {
                                    icon: option.icon || '',
                                    title: option.title,
                                    bgColor: option.bg_clor || '',
                                    default: option.default || false,
                                    value: option.value,
                                };
                            }),
                            customValidFn: isFunction(field.customValidFn)
                                ? field.customValidFn
                                : function (rule: any, value: string) {
                                      return commonValidFn(rule, value, fieldItem.field);
                                  },
                        },
                    });
                    break;
                case 'chained_droplist':
                    fieldItem = Object.assign(fieldItem, {
                        field: {
                            ...fieldItem.field,
                            type: 'cascade',
                            trigger: 'onChange',
                            arrangement: field.arrangement || 'auto',
                            viewType: field.view_type,
                            options: mapCascader(field.select_options),
                            customValidFn: isFunction(field.customValidFn)
                                ? field.customValidFn
                                : function (rule: any, value: any[]) {
                                      return chainedValidFn(
                                          rule,
                                          value,
                                          Object.assign(field, {
                                              disabled: fieldItem?.field?.disabled,
                                          })
                                      );
                                  },
                        },
                    });
                    break;
            }

            fieldItem = Object.assign(fieldItem, isFunction(field.convertProps) ? field.convertProps(field) : {});
            return fieldItem;
        })
        .filter((i) => i);
}

export function getFieldsDefaultValue({ fields = [], values = {} }: any) {
    let defaultValues: any = {};

    fields.forEach((field: any) => {
        const setting = field.setting || {};

        let selectOption: any = field.select_options || [];
        let selectOptionDefault: any = [];
        switch (field.content_type) {
            case 'text':
            case 'area_text':
            case 'link':
            case 'date':
            case 'time':
            case 'datetime':
            case 'location':
            case 'calculate':
            case 'number':
            case 'numeric':
                if (setting.default_value) {
                    defaultValues[field.name] = setting.default_value;
                } else {
                    defaultValues[field.name] = '';
                }
                break;
            case 'droplist':
            case 'droplist_multi':
            case 'radio':
            case 'checkbox':
                selectOptionDefault = selectOption.filter((option: any) => {
                    return option.default;
                });
                if (selectOptionDefault && selectOptionDefault.length > 0) {
                    defaultValues[field.name] = selectOptionDefault.map((option: any) => option.value);
                } else {
                    defaultValues[field.name] = '';
                }
                break;
            case 'chained_droplist':
                defaultValues[field.name] = '';
                break;
        }
    });

    return Object.assign({}, defaultValues, values);
}

export function formatProps(props: any) {
    let pageProps: any = {};
    const page = props.page || 'im';
    const commonProps = {
        page: props.page,
        lang: props.lang,
        company: props.company,
        roleCan: props.roleCan,
        isAdmin: props.isAdmin,
        user: props.user, // 当前客服
        admin: props.admin, // 系统管理员
        agents: props.agents || [], // 客服组
        agentGroups: props.agentGroups, //负责组
        customerFieldsv2: props.customerFieldsv2,
        organizationFieldsv2: props.organizationFieldsv2, //公司自定义字段
        noteTemplates: props.noteTemplates || [],
        imAgentNoteTemplatesSort: props.im_agent_note_templates_sort || [],
        ticketTemplates: props.ticketTemplates || [],
        customerFieldsAuthority: props.customFieldsAuthority?.customer_fields_abilities || {},
        ticketFieldsAbilities: props.customFieldsAuthority?.ticket_fields_abilities || {},
        tabPlugins: props.tabPlugins,
        tabsWidth: props.tabsWidth,
        getTencentMap: props.getTencentMap,
        companyConfig: props.companyConfig,
        previewUrl: props.previewUrl,
        currentTab: props.currentTab,
        agentStatus: props.agentStatus,
        pluginFilter: props.pluginFilter || [],
        customerFields: props.customerFields, //客户自定义字段
        organizationFields: props.organizationFields, //公司自定义字段
        noteFields: props.noteFields || [],
        ticketTab: props.ticketTab,
        currentUser: props.currentUser,
        smsapp: props.smsapp,
        traceOpen: props.traceOpen,
        apiHost: props.apiHost,
        kmrobot_url: props.kmrobot_url,
        vistor_server: props.vistor_server,
        fieldHooks: props.fieldHooks,
        getCompanyCategoryConfig: props.getCompanyCategoryConfig,
        platforms: props.platforms || [],
        mchatConfig: props.mchatConfig,
        modals: props.modals,
        sdk: props?.sdk,
        token: props?.token,
        spa1Path: props?.spa1Path,
        credentials: props?.credentials,
        template: props?.template,
    };
    switch (page) {
        case 'im':
            pageProps = {
                accessInfo: props.chat?.accessInfo,
                agent: props.chat?.agent || {},
                customerId: props.chat?.customer_id,
                sessionKey: props.chat?.session_key,
                imSubSessionId: props.chat?.im_sub_session_id,
                customer: props.chat?.customer || {},
                organizations: props.chat?.organizations || [],
                agentConfig: props.agentConfig,
                agentTabConfig: props?.agentConfig?.agent_tab_config || {},
                newVisit: props.chat?.newVisit || {},
                newTicketReact: props.chat?.newTicketReact || {},
                newRecord: props.chat?.newRecord || {},
                imNewPlatforms: props.imNewPlatforms,
                chat: props.chat,
                platform: props.chat?.platform || '',
                plugTab: props.chat?.plugTab || {},
                event: props.event || {},
                ImNoteTemplates: props.ImNoteTemplates || [],
                companyCategoryConfig: props.companyCategoryConfig,
            };
            break;
        case 'mpv':
            pageProps = {
                accessInfo: props.chat?.accessInfo,
                agent: props.chat?.agent || {},
                customerId: props.chat?.customer_id,
                sessionKey: props.chat?.session_key,
                imSubSessionId: props.chat?.im_sub_session_id,
                customer: props.chat?.customer || {},
                organizations: props.chat?.organizations || [],
                agentConfig: props.agentConfig,
                agentTabConfig: props?.agentConfig?.agent_tab_config || {},
                newVisit: props.chat?.newVisit || {},
                newTicketReact: props.chat?.newTicketReact || {},
                newRecord: props.chat?.newRecord || {},
                imNewPlatforms: props.imNewPlatforms,
                chat: props.chat,
                platform: props.chat?.platform || '',
                plugTab: props.chat?.plugTab || {},
                event: props.event || {},
                ImNoteTemplates: props.ImNoteTemplates || [],
                companyCategoryConfig: props.companyCategoryConfig,
            };
            break;
    }
    return assign(commonProps, pageProps);
}

export function stringToHex(str: string): string {
    const hexResult:string[] = [];
    for (let i = 0, len = str.length; i < len; i++) {
        const hex = str.charCodeAt(i).toString(16);
        hexResult.push(hex);
    }
    return hexResult.join('');
}

export function getPreviewUrl(url: string, options: { needEncode?: boolean; previewUrl: string }): string {
    if (!url) {
        return '';
    }
    const { needEncode = false, previewUrl } = options || {};
    if (needEncode) {
        url = encodeURIComponent(decodeURIComponent(url));
        url = url.replace(/%2F/g, '/');
    }
    let hexUrl;
    try {
        hexUrl = stringToHex(url);
    } catch (e) {
        console.error('preview-url-helper', e);
        return url;
    }

    return `${previewUrl}/wopi/files/${hexUrl}/documents`;
}

/**
 * 通过文件名判断是否可以预览
 * @param {string} fileName
 * @returns {boolean}
 */
export function canPreviewByFileName(fileName: string): boolean {
    return /\.(doc|docx|odt|rtf|wps|ppt|pptx|odp|dp|xls|xlsx|ods|csv|et|pdf)$/.test(fileName);
}

export function replaceLinks(text: any) {
    return (text + '').replace(
        /((http|https):\/\/([\w-]+\.)+[\w-:]+(\/[\w-./?;%&=#：:]*)?)/g,
        '<a href="$1" target="_blank">$1</a>'
    );
}
import { useMemo } from 'react';
import moment, { Moment } from 'moment';
import { useTranslation } from 'react-i18next';

export function useDateRanges() {
    const { t } = useTranslation();
    return useMemo<{ [key: string]: [Moment, Moment] }>(
        () => ({
            [t('page.im.record.today')]: [moment(), moment()],
            [t('page.im.record.yesterday')]: [moment().subtract(1, 'days'), moment().subtract(1, 'days')],
            [t('page.im.record.recentSevenDays')]: [moment().subtract(6, 'days'), moment()],
            [t('page.im.record.recentThirtyDays')]: [moment().subtract(29, 'days'), moment()],
            [t('page.im.record.currentMonth')]: [moment().startOf('month'), moment().endOf('month')],
            [t('page.im.record.lastMonth')]: [
                moment().subtract(1, 'month').startOf('month'),
                moment().subtract(1, 'month').endOf('month'),
            ],
            [t('page.im.record.currentYear')]: [moment().startOf('year').startOf('day'), moment()],
            [t('page.im.record.lastYear')]: [
                moment().subtract(1, 'year').startOf('year').startOf('day'),
                moment().subtract(1, 'year').endOf('year').endOf('day'),
            ],
            [t('page.im.record.recentOneYear')]: [moment().subtract(1, 'year'), moment()],
        }),
        [t]
    );
}
