const path = require('path');
const fs = require('fs-extra');
const prettier = require('prettier');
const log = require('../log');

function getKeyPath(generatePathMap, codeFilePath) {
    let iterator = generatePathMap.entries();
    let keyPath = [];
    let reg = iterator.next().value;

    while (!keyPath.length && reg) {
        if (`${path.dirname(codeFilePath)}/`.match(reg[0])) {
            let extension = path.extname(codeFilePath);

            let fileName = path.basename(codeFilePath, extension);

            keyPath = `${path.dirname(codeFilePath)}/${fileName}`
                .replace(reg[0], '')
                .split('/')
                .map((i) => camelCase(i));
        } else {
            reg = iterator.next().value;
        }
    }
    if (!reg) {
        log.warn(`${codeFilePath}，未匹配到合法路径，请修改配置项`);
        return [];
    }
    // console.log('keyPath ::',  keyPath);

    return [keyPath, reg];
}

let prettierrc = {};

const formatCodeCore = (code, parser) => {
    return prettier.format(code, {
        ...prettierrc,
        parser,
    });
};

const formatCode = (code) => formatCodeCore(code, 'typescript');
const formatCodeJS = (code) => formatCodeCore(code, 'babel');
const formatCodeJSON = (code) => formatCodeCore(code, 'json');

function tryRequireFile(fileName) {
    try {
        const prettierrc = require(path.resolve(process.cwd(), fileName));
        log.info(`业务项目中存在${fileName}文件，使用该配置进行格式化`);
        return prettierrc;
    } catch (err) {
        // console.log(`业务项目中没有${fileName}文件`);
    }
}

function getPrettierrc() {
    const packageJson = require(path.resolve(process.cwd(), 'package.json'));
    const { prettier: prettierrcBusiness } = packageJson;

    if (Boolean(prettierrcBusiness)) {
        if (typeof prettierrcBusiness === 'object') {
            prettierrc = prettierrcBusiness;
        } else if (typeof prettierrcBusiness === 'string') {
            prettierrc = require(prettierrcBusiness);
        }
        log.info(`使用package.json中的prettier配置进行格式化`);
    } else {
        const configFiles = [
            tryRequireFile('.prettierrc.json'),
            tryRequireFile('.prettierrc.js'),
            tryRequireFile('prettier.config.js'),
        ];
        configFiles.some((item) => {
            prettierrc = item;
            return Boolean(item);
        });
    }
    if (!prettierrc) {
        log.info(`业务项目中没有可用的prettier配置文件，使用@udesk/prettier-config提供的默认配置`);
        prettierrc = require('@udesk/prettier-config');
    }
    log.info('配置如下：');
    log.info(prettierrc);
}

/** 将字符串转化为驼峰写法 */
const camelCase = (str, firstLetterUpperCase = false) => {
    return str
        .split('-')
        .map((_, index) =>
            _.replace(/^\w/, (item) =>
                firstLetterUpperCase
                    ? item.toUpperCase()
                    : index === 0
                    ? item.toLowerCase()
                    : item.toUpperCase()
            )
        )
        .join('');
};

// const walk = (codeFilePath) => {
//     let ret = [];
//     const stat = fs.statSync(codeFilePath);
//     if(!stat){
//         console.log('文件错误');
//         return [];
//     }
//     if (stat.isDirectory()) {
//         const list = fs.readdirSync(codeFilePath);
//         console.log('list ::',  list);
//         list.forEach(item => {
//             walk(`${codeFilePath}/${item}`);
//         })
//         return;
//     }else{
//         if(['.ts', '.tsx', '.js', 'jsx'].includes(path.extname(codeFilePath))){
//             ret = [...ret, ]
//         }
//     }
//     console.log('codeFilePath ::',  codeFilePath);
// }
function walk(dir) {
    if (
        !fs.statSync(dir).isDirectory() &&
        ['.ts', '.tsx', '.js', '.jsx'].includes(path.extname(dir))
    ) {
        return [dir];
    }
    var results = [];
    var list = fs.readdirSync(dir);
    list.forEach((file) => {
        file = dir + '/' + file;
        var stat = fs.statSync(file);
        if (stat && stat.isDirectory()) {
            results = results.concat(walk(file));
        } else {
            if (['.ts', '.tsx', '.js', '.jsx'].includes(path.extname(file))) {
                results.push(file);
            }
        }
    });
    return results;
}

module.exports = {
    getKeyPath,
    formatCode,
    formatCodeJS,
    formatCodeJSON,
    camelCase,
    walk,
    getPrettierrc,
};
