const nodePathModule = require('path');
const fs = require('fs-extra');
const prettier = require('prettier');
const { default: traverse } = require('@babel/traverse');
const MagicString = require('magic-string');
const utils = require('./utils/utils');

function writeLocalesFile({ localesFilePath, fileType }, localesFilePathMap) {
    const cwd = process.cwd();
    // console.log('cwd ::',  cwd);
    localesFilePath.forEach((p) => {
        const fileName = nodePathModule.basename(p);
        const locales = fs.readFileSync(nodePathModule.resolve(cwd, p), 'utf-8');
        // console.log('locales: ', locales);
        const s = new MagicString(locales);
        if (fileType === 'json') {
            fs.writeFileSync(
                nodePathModule.resolve(cwd, p),
                utils.formatCodeJSON(JSON.stringify(localesFilePathMap[fileName].locales))
            );
            return;
        }
        prettier.format(locales, {
            parser(text, parsers) {
                let ast = parsers['babel-ts'](text);
                // console.log('localesFilePathMap ::',  localesFilePathMap);
                traverse(ast, {
                    VariableDeclarator(path) {
                        if (path.node.init && path.node.init.type === 'ObjectExpression') {
                            s.overwrite(
                                path.node.init.start,
                                path.node.init.end,
                                JSON.stringify(localesFilePathMap[fileName].locales)
                            );
                            fs.writeFileSync(
                                nodePathModule.resolve(cwd, p),
                                utils.formatCode(s.toString())
                            );
                        }
                    },
                    ObjectExpression(path) {
                        if (
                            path.parentPath &&
                            path.parentPath.type === 'ExportDefaultDeclaration'
                        ) {
                            s.overwrite(
                                path.node.start,
                                path.node.end,
                                JSON.stringify(localesFilePathMap[fileName].locales)
                            );
                            fs.writeFileSync(
                                nodePathModule.resolve(cwd, p),
                                utils.formatCode(s.toString())
                            );
                        }
                    },
                });
                return ast;
            },
        });
    });
}

module.exports = writeLocalesFile;
